// FinalBurn NEO ZX Spectrum driver.  NEO edition!
// media: .tap, .z80 snapshots
// input: kbd, kempston: joy1, sinclair intf.2: joy1 & joy2

#include "tiles_generic.h"
#include "z80_intf.h"
#include "ay8910.h"
#include <math.h>
#include "biquad.h"

#if defined (_MSC_VER)
#define strcasecmp stricmp
#endif

static INT32 SpecMode = 0;
#define SPEC_TAP	(1 << 0)
#define SPEC_Z80	(1 << 1)
#define SPEC_128K   (1 << 2)
#define SPEC_PLUS2  (1 << 3) // +2a (Amstrad model)
#define SPEC_INVES	(1 << 4) // Spanish clone (non-contended ula)
#define SPEC_AY8910	(1 << 5)

static UINT8 SpecInputKbd[0x10][0x05] = {
	{ 0, 0, 0, 0, 0 }, // Shift, Z, X, C, V
	{ 0, 0, 0, 0, 0 }, // A, S, D, F, G
	{ 0, 0, 0, 0, 0 }, // Q, W, E, R, T
	{ 0, 0, 0, 0, 0 }, // 1, 2, 3, 4, 5
	{ 0, 0, 0, 0, 0 }, // 0, 9, 8, 7, 6
	{ 0, 0, 0, 0, 0 }, // P, O, I, U, Y
	{ 0, 0, 0, 0, 0 }, // Enter, L, K, J, H
	{ 0, 0, 0, 0, 0 }, // Space, Sym, M, N, B
	{ 0, 0, 0, 0, 0 }, // Kempston joy1
	{ 0, 0, 0, 0, 0 }, // Sinclair intf.2 joy1
	{ 0, 0, 0, 0, 0 }, // Sinclair intf.2 joy2
	// 128K+ Caps Shift (Secondary Membrane)
	{ 0, 0, 0, 0, 0 }, // Edit, CapsLock, True Vid, Inv Vid, Cursor Left
	{ 0, 0, 0, 0, 0 }, // Del, Graph, Cursor Right, Cursor Up, Cursor Down
	{ 0, 0, 0, 0, 0 }, // Break, Ext Mode
	// 128K+ Symbol Shift (Secondary Membrane)
	{ 0, 0, 0, 0, 0 }, // Quote, Semicolon
	{ 0, 0, 0, 0, 0 }  // Period, Comma
};

static UINT8 SpecDips[1];
static UINT8 SpecInput[0x10];
static UINT8 SpecReset;

static UINT8 *AllMem;
static UINT8 *MemEnd;
static UINT8 *RamStart;
static UINT8 *RamEnd;
static UINT8 *SpecZ80Rom;
static UINT8 *SpecZ80Ram;
static UINT8 *SpecVideoRam;
static UINT8 *SpecSnapshotData;
static INT32  SpecSnapshotDataLen;
static UINT8 *SpecTAP;
static INT32  SpecTAPLen;

static UINT32 *SpecPalette;
static UINT8 SpecRecalc;

static UINT8 ula_attr;
static UINT8 ula_scr;
static UINT8 ula_byte;
static UINT8 ula_border;
static UINT8 ula_flash;
static INT32 ula_last_cyc;
static INT32 nExtraCycles;

static INT16 *Buzzer;

static INT32 SpecRamPage; // calculated from Spec128kMapper*
static INT32 SpecRomPage; // calculated from Spec128kMapper*
static INT32 Spec128kMapper;
static INT32 Spec128kMapper2;

static INT32 SpecScanlines;
static INT32 SpecCylesPerScanline;
static INT32 SpecContention;

static INT16 *dacbuf; // for dc offset filter
static INT16 dac_lastin;
static INT16 dac_lastout;

static struct BurnInputInfo SpecInputList[] =
{
	{"P1 Up",			BIT_DIGITAL,	&SpecInputKbd[8][3],	"p1 up"				},
	{"P1 Down",			BIT_DIGITAL,	&SpecInputKbd[8][2],	"p1 down"			},
	{"P1 Left",			BIT_DIGITAL,	&SpecInputKbd[8][1],	"p1 left"			},
	{"P1 Right",		BIT_DIGITAL,	&SpecInputKbd[8][0],	"p1 right"			},
	{"P1 Fire",			BIT_DIGITAL,	&SpecInputKbd[8][4],	"p1 fire 1"			},

	{"P2 Up",			BIT_DIGITAL,	&SpecInputKbd[10][3],	"p2 up"				},
	{"P2 Down",			BIT_DIGITAL,	&SpecInputKbd[10][2],	"p2 down"			},
	{"P2 Left",			BIT_DIGITAL,	&SpecInputKbd[10][0],	"p2 left"			},
	{"P2 Right",		BIT_DIGITAL,	&SpecInputKbd[10][1],	"p2 right"			},
	{"P2 Fire",			BIT_DIGITAL,	&SpecInputKbd[10][4],	"p2 fire 1"			},

	{"ENTER",			BIT_DIGITAL,	&SpecInputKbd[6][0],	"keyb_enter"		},
	{"SPACE",			BIT_DIGITAL,	&SpecInputKbd[7][0],	"keyb_space"		},
	{"CAPS SHIFT",		BIT_DIGITAL,	&SpecInputKbd[0][0],	"keyb_left_shift"	},
	{"SYMBOL SHIFT",	BIT_DIGITAL,	&SpecInputKbd[7][1],	"keyb_right_shift"	},
	{"Q",				BIT_DIGITAL,	&SpecInputKbd[2][0],	"keyb_Q"			},
	{"W",				BIT_DIGITAL,	&SpecInputKbd[2][1],	"keyb_W"			},
	{"E",				BIT_DIGITAL,	&SpecInputKbd[2][2],	"keyb_E"			},
	{"R",				BIT_DIGITAL,	&SpecInputKbd[2][3],	"keyb_R"			},
	{"T",				BIT_DIGITAL,	&SpecInputKbd[2][4],	"keyb_T"			},
	{"Y",				BIT_DIGITAL,	&SpecInputKbd[5][4],	"keyb_Y"			},
	{"U",				BIT_DIGITAL,	&SpecInputKbd[5][3],	"keyb_U"			},
	{"I",				BIT_DIGITAL,	&SpecInputKbd[5][2],	"keyb_I"			},
	{"O",				BIT_DIGITAL,	&SpecInputKbd[5][1],	"keyb_O"			},
	{"P",				BIT_DIGITAL,	&SpecInputKbd[5][0],	"keyb_P"			},
	{"A",				BIT_DIGITAL,	&SpecInputKbd[1][0],	"keyb_A"			},
	{"S",				BIT_DIGITAL,	&SpecInputKbd[1][1],	"keyb_S"			},
	{"D",				BIT_DIGITAL,	&SpecInputKbd[1][2],	"keyb_D"			},
	{"F",				BIT_DIGITAL,	&SpecInputKbd[1][3],	"keyb_F"			},
	{"G",				BIT_DIGITAL,	&SpecInputKbd[1][4],	"keyb_G"			},
	{"H",				BIT_DIGITAL,	&SpecInputKbd[6][4],	"keyb_H"			},
	{"J",				BIT_DIGITAL,	&SpecInputKbd[6][3],	"keyb_J"			},
	{"K",				BIT_DIGITAL,	&SpecInputKbd[6][2],	"keyb_K"			},
	{"L",				BIT_DIGITAL,	&SpecInputKbd[6][1],	"keyb_L"			},
	{"Z",				BIT_DIGITAL,	&SpecInputKbd[0][1],	"keyb_Z"			},
	{"X",				BIT_DIGITAL,	&SpecInputKbd[0][2],	"keyb_X"			},
	{"C",				BIT_DIGITAL,	&SpecInputKbd[0][3],	"keyb_C"			},
	{"V",				BIT_DIGITAL,	&SpecInputKbd[0][4],	"keyb_V"			},
	{"B",				BIT_DIGITAL,	&SpecInputKbd[7][4],	"keyb_B"			},
	{"N",				BIT_DIGITAL,	&SpecInputKbd[7][3],	"keyb_N"			},
	{"M",				BIT_DIGITAL,	&SpecInputKbd[7][2],	"keyb_M"			},
	{"1",				BIT_DIGITAL,	&SpecInputKbd[3][0],	"keyb_1"			},
	{"2",				BIT_DIGITAL,	&SpecInputKbd[3][1],	"keyb_2"			},
	{"3",				BIT_DIGITAL,	&SpecInputKbd[3][2],	"keyb_3"			},
	{"4",				BIT_DIGITAL,	&SpecInputKbd[3][3],	"keyb_4"			},
	{"5",				BIT_DIGITAL,	&SpecInputKbd[3][4],	"keyb_5"			},
	{"6",				BIT_DIGITAL,	&SpecInputKbd[4][4],	"keyb_6"			},
	{"7",				BIT_DIGITAL,	&SpecInputKbd[4][3],	"keyb_7"			},
	{"8",				BIT_DIGITAL,	&SpecInputKbd[4][2],	"keyb_8"			},
	{"9",				BIT_DIGITAL,	&SpecInputKbd[4][1],	"keyb_9"			},
	{"0",				BIT_DIGITAL,	&SpecInputKbd[4][0],	"keyb_0"			},

	{"EDIT",			BIT_DIGITAL,	&SpecInputKbd[11][0],	"keyb_insert"		},
	{"CAPS LOCK",		BIT_DIGITAL,	&SpecInputKbd[11][1],	"keyb_caps_lock"	},
	{"TRUE VID",		BIT_DIGITAL,	&SpecInputKbd[11][2],	"keyb_home"			},
	{"INV VID",			BIT_DIGITAL,	&SpecInputKbd[11][3],	"keyb_end"			},

	{"DEL",				BIT_DIGITAL,	&SpecInputKbd[12][0],	"keyb_backspace"	},
	{"GRAPH",			BIT_DIGITAL,	&SpecInputKbd[12][1],	"keyb_left_alt"		},

	{"Cursor Up",		BIT_DIGITAL,	&SpecInputKbd[12][3],	"keyb_cursor_up"	}, // don't auto-map these cursors by default
	{"Cursor Down",		BIT_DIGITAL,	&SpecInputKbd[12][4],	"keyb_cursor_down"	}, // causes trouble w/games when using cursors
	{"Cursor Left",		BIT_DIGITAL,	&SpecInputKbd[11][4],	"keyb_cursor_left"	}, // as joystick! (yie ar kungfu, ...)
	{"Cursor Right",	BIT_DIGITAL,	&SpecInputKbd[12][2],	"keyb_cursor_right"	},

	{"BREAK",			BIT_DIGITAL,	&SpecInputKbd[13][0],	"keyb_pause"		},
	{"EXT MODE",		BIT_DIGITAL,	&SpecInputKbd[13][1],	"keyb_left_ctrl"	},

	{"\"",				BIT_DIGITAL,	&SpecInputKbd[14][0],	"keyb_apost"		},
	{";",				BIT_DIGITAL,	&SpecInputKbd[14][1],	"keyb_colon"		},

	{".",				BIT_DIGITAL,	&SpecInputKbd[15][2],	"keyb_stop"			},
	{",",				BIT_DIGITAL,	&SpecInputKbd[15][3],	"keyb_comma"		},

	{"Reset",			BIT_DIGITAL,	&SpecReset,				"reset"				},
	{"Dip A",			BIT_DIPSWITCH,	SpecDips + 0,			"dip"				},
};

STDINPUTINFO(Spec)

static struct BurnDIPInfo SpecDIPList[]=
{
	DIP_OFFSET(0x43)

	{0, 0xfe, 0   , 2   , "Hardware Version (48K)"	},
	{0, 0x01, 0x80, 0x00, "Issue 2"					},
	{0, 0x01, 0x80, 0x80, "Issue 3"					},

	{0, 0xfe, 0   , 7   , "Joystick Config"			},
	{0, 0x01, 0x5f, 0x00, "Kempston"				},
	{0, 0x01, 0x5f, 0x01, "Sinclair Interface 2"	},
	{0, 0x01, 0x5f, 0x02, "QAOPM"					},
	{0, 0x01, 0x5f, 0x04, "QAOP Space"				},
	{0, 0x01, 0x5f, 0x08, "Cursor Keys"				},
	{0, 0x01, 0x5f, 0x40, "Jumping Jack"			},
	{0, 0x01, 0x5f, 0x10, "Disabled"				},

	{0, 0xfe, 0   , 2   , "Interface 2 Joyport"		},
	{0, 0x01, 0x20, 0x00, "Normal"					},
	{0, 0x01, 0x20, 0x20, "Swapped: Joy1 <-> Joy2"	},
};

static struct BurnDIPInfo SpecDefaultDIPList[]=
{
	{0, 0xff, 0xff, 0x80, NULL						}, // Issue 3 + Kempston (Blinky's Scary School requires issue 3)
};

static struct BurnDIPInfo SpecIssue2DIPList[]=
{
	{0, 0xff, 0xff, 0x00, NULL						}, // Issue 2 + Kempston (Abu Simbel (English/Gremlin) requires issue 2)
};

static struct BurnDIPInfo SpecIntf2DIPList[]=
{
	{0, 0xff, 0xff, 0x81, NULL						}, // Sinclair Interface 2 (2 Joysticks)
};

static struct BurnDIPInfo SpecIntf2SwappedDIPList[]=
{
	{0, 0xff, 0xff, 0xa1, NULL						}, // Sinclair Interface 2 (2 Joysticks), Swapped (Joy1 <-> Joy2)
};

static struct BurnDIPInfo SpecQAOPMDIPList[]=
{
	{0, 0xff, 0xff, 0x82, NULL						}, // Kempston mapped to QAOPM (moon ranger)
};

static struct BurnDIPInfo SpecQAOPSpaceDIPList[]=
{
	{0, 0xff, 0xff, 0x84, NULL						}, // Kempston mapped to QAOPSpace (jet pack bob)
};

static struct BurnDIPInfo SpecJJackDIPList[]=
{
	{0, 0xff, 0xff, 0xc0, NULL						}, // Kempston mapped to space, symbol shift, caps shift (jumping jack)
};

static struct BurnDIPInfo SpecCursorKeysDIPList[]=
{
	{0, 0xff, 0xff, 0x88, NULL						}, // Kempston mapped to Cursor Keys
};

STDDIPINFOEXT(Spec, SpecDefault, Spec)
STDDIPINFOEXT(SpecIssue2, SpecIssue2, Spec)
STDDIPINFOEXT(SpecIntf2, SpecIntf2, Spec)
STDDIPINFOEXT(SpecIntf2Swap, SpecIntf2Swapped, Spec)
STDDIPINFOEXT(SpecQAOPM, SpecQAOPM, Spec)
STDDIPINFOEXT(SpecQAOPSpace, SpecQAOPSpace, Spec)
STDDIPINFOEXT(SpecJJack, SpecJJack, Spec)
STDDIPINFOEXT(SpecCursorKeys, SpecCursorKeys, Spec)

static void spectrum128_bank(); // forwards
static void spectrum_loadz80();
static void ula_init(INT32 scanlines, INT32 cyc_scanline, INT32 contention);
static void update_ula(INT32 cycle);

// Spectrum 48k tap-loading robot -dink
static INT32 CASFrameCounter = 0; // for autoloading
static INT32 CASAutoLoadPos = 0;
static INT32 CASAutoLoadTicker = 0;
static void SpecTAPReset(); // forward

static void SpecLoadTAP()
{
	CASAutoLoadPos = 0;
	CASAutoLoadTicker = 0;
	CASFrameCounter = 0;
}

static UINT8* FindInput(char *str)
{
	for (INT32 i = 0; SpecInputList[i].szName != NULL; i++) {
		if (!strcmp(str, SpecInputList[i].szName)) {
			return SpecInputList[i].pVal;
		}
	}
	return NULL;
}

static void SetInput(char *str, INT32 data)
{
	UINT8 *x = FindInput(str);
	if (x) {
		x[0] = data;
	}
}

static void TAPAutoLoadTick()
{
	const UINT8 TAPLoader[2][10] = { { "J\"\"\n\0" }, { "\n\0" } }; // 48k, 128k
	const INT32 KeyDelay = 12; // frames 0-4: press key, 5-11: delay, 11: next character.

	if (CASAutoLoadPos == 0xff) return;

	UINT8 c = TAPLoader[((SpecMode & SPEC_128K) ? 1 : 0)][CASAutoLoadPos];
	if (!c) {
		CASAutoLoadPos = 0xff;
		return;
	}

	if ((CASAutoLoadTicker % KeyDelay) < 5) {
		switch (c) {
			case '\"': {
				SetInput("SYMBOL SHIFT", 1);
				SetInput("P", 1);
				break;
			}
			case 'J': {
				SetInput("J", 1);
				break;
			}
			case '\n': {
				SetInput("ENTER", 1);
				break;
			}
		}
	}

	if ((CASAutoLoadTicker % KeyDelay) == KeyDelay - 1) CASAutoLoadPos++;
	CASAutoLoadTicker++;
}

static void TAPAutoLoadRobot()
{
	if (SpecMode & SPEC_TAP && CASFrameCounter > 90) {
		TAPAutoLoadTick();
	}
	CASFrameCounter++;
}
// End TAP Robot

// Oversampling Buzzer-DAC
static INT32 buzzer_last_update;
static INT32 buzzer_last_data;
static INT32 buzzer_data_len;
static INT32 buzzer_data_frame;
static INT32 buzzer_data_frame_minute;

static const INT32 buzzer_oversample = 3000;

static BIQ biquad[2]; // snd/biquad.h

static void BuzzerInit() // keep in DoReset()!
{
	biquad[0].init(FILT_LOWPASS, nBurnSoundRate, 7000, 0.554, 0.0);
	biquad[1].init(FILT_LOWPASS, nBurnSoundRate, 8000, 0.554, 0.0);

	buzzer_data_frame_minute = (SpecCylesPerScanline * SpecScanlines * 50.00);
	buzzer_data_frame = ((double)(SpecCylesPerScanline * SpecScanlines) * nBurnSoundRate * buzzer_oversample) / buzzer_data_frame_minute;
}

static void BuzzerExit()
{
	biquad[0].exit();
	biquad[1].exit();
}

static void BuzzerAdd(INT16 data)
{
	data *= (1 << 12);

	if (data != buzzer_last_data) {
		INT32 len = ((double)(ZetTotalCycles() - buzzer_last_update) * nBurnSoundRate * buzzer_oversample) / buzzer_data_frame_minute;
		if (len > 0)
		{
			for (INT32 i = buzzer_data_len; i < buzzer_data_len+len; i++) {
				// if len goes over buzzer_data_frame, wrap around to the beginning.
				Buzzer[i % buzzer_data_frame] = buzzer_last_data;
			}
			buzzer_data_len += len;
		}

		buzzer_last_data = data;
		buzzer_last_update = ZetTotalCycles();
	}
}

static void BuzzerRender(INT16 *dest)
{
	INT32 buzzer_data_pos = 0;

	// fill buffer (if needed)
	if (buzzer_data_len < buzzer_data_frame) {
		for (INT32 i = buzzer_data_len; i < buzzer_data_frame; i++) {
			Buzzer[i] = buzzer_last_data;
		}
		buzzer_data_len = buzzer_data_frame;
	}

	// average + mixdown
	for (INT32 i = 0; i < nBurnSoundLen; i++) {
		INT32 sample = 0;
		for (INT32 j = 0; j < buzzer_oversample; j++) {
			sample += Buzzer[buzzer_data_pos++];
		}
		sample = (INT32)(biquad[1].filter(biquad[0].filter((double)sample / buzzer_oversample)));
		dest[0] = BURN_SND_CLIP(sample);
		dest[1] = BURN_SND_CLIP(sample);
		dest += 2;
	}

	buzzer_data_len = 0;
	buzzer_last_update = 0;
}

// end Oversampling Buzzer-DAC

static INT32 MemIndex()
{
	UINT8 *Next; Next = AllMem;

	SpecZ80Rom              = Next; Next += 0x04000 * 4;
	SpecSnapshotData        = Next; Next += 0x20000;
	SpecTAP                 = Next; Next += 0x800000;

	RamStart                = Next;
	SpecZ80Ram              = Next; Next += 0x20000;
	RamEnd                  = Next;

	SpecPalette             = (UINT32*)Next; Next += 0x00010 * sizeof(UINT32);
	dacbuf                  = (INT16*)Next; Next += 0x800 * 2 * sizeof(INT16);

	Buzzer                  = (INT16*)Next; Next += 1000 * buzzer_oversample * sizeof(INT16);

	MemEnd                  = Next;

	return 0;
}

static INT32 SpecDoReset()
{
	ZetOpen(0);
	ZetReset();
	if (SpecMode & SPEC_AY8910) {
		AY8910Reset(0);
	}
	ZetClose();

	BuzzerInit();

	SpecVideoRam = SpecZ80Ram;
	Spec128kMapper = 0;
	Spec128kMapper2 = 0;
	ula_border = 0;

	if (SpecMode & SPEC_128K) {
		ZetOpen(0);
		spectrum128_bank();
		ZetClose();
	}

	if (SpecMode & SPEC_Z80) {
		spectrum_loadz80();
	}

	if (SpecMode & SPEC_TAP) {
		SpecLoadTAP(); // reset robot
		SpecTAPReset(); // reset tap engine
	}

	dac_lastin = 0;
	dac_lastout = 0;

	ula_last_cyc = 0;
	ula_byte = 0xff;
	ula_attr = 0x00;
	ula_scr = 0x00;

	nExtraCycles = 0;

	return 0;
}

static UINT8 __fastcall SpecZ80Read(UINT16 address)
{
	if (address < 0x4000) {
		return SpecZ80Rom[address];
	}

	if (address >= 0x4000 && address <= 0x7fff) {
		return SpecZ80Ram[address & 0x3fff];
	}

	if (address >= 0x8000 && address <= 0xffff) {
		return SpecZ80Ram[0x4000 + (address & 0x7fff)];
	}

	bprintf(0, _T("mr %x\n"), address);

	return 0;
}

static void __fastcall SpecZ80Write(UINT16 address, UINT8 data)
{
	if (address < 0x4000) return;

	if (address >= 0x4000 && address <= 0x7fff) {
		SpecZ80Ram[address & 0x3fff] = data;
		return;
	}

	if (address >= 0x8000 && address <= 0xffff) {
		SpecZ80Ram[0x4000 + (address & 0x7fff)] = data;
		return;
	}

	bprintf(0, _T("mw %x %x\n"), address, data);
}

// the 128K has a secondary kbd matrix that added more keys to the keyboard
// which were normally accessed by pressing Caps Shift / Symbol Shift.
// When one of those keys are pressed - it basically uses a little hw hack
// to hold the caps/symbol shift button. -dink
static INT32 check_caps_shift()
{
	INT32 ret = 0;
	for (INT32 i = 11; i <= 13; i++) {
		// check if a 2ndary matrix button that needs CAPS Shift has been pressed
		if (SpecInput[i] != 0x1f) ret = 1;
	}
	return ret;
}

static INT32 check_symbol_shift()
{
	INT32 ret = 0;
	for (INT32 i = 14; i <= 15; i++) {
		// check if a 2ndary matrix button that needs Symbol Shift has been pressed
		if (SpecInput[i] != 0x1f) ret = 1;
	}
	return ret;
}

static UINT8 read_keyboard(UINT16 address)
{
	UINT8 keytmp = 0x1f;

	for (INT32 i = 0; i < 8; i++) { // process all kbd rows
		if (~address & (1 << (i + 8))) {
			switch (i) {
				case 0:
					keytmp &= SpecInput[i];
					if (check_caps_shift()) keytmp &= ~(1 << 0);
					break;

				case 3:
					keytmp &= SpecInput[i] & SpecInput[11] & SpecInput[10]; // caps shift0, intf2 joy2
					break;

				case 4:
					keytmp &= SpecInput[i] & SpecInput[12] & SpecInput[9]; // caps shift1, intf2 joy1
					break;

				case 5:
					keytmp &= SpecInput[i] & SpecInput[14]; // symbol shift0
					break;

				case 7:
					keytmp &= SpecInput[i] & SpecInput[13] & SpecInput[15]; // caps shift2, symbol shift1
					if (check_symbol_shift()) keytmp &= ~(1 << 1);
					break;

				default:
					keytmp &= SpecInput[i];
					break;
			}
		}
	}

	if (SpecMode & SPEC_128K || SpecDips[0] & 0x80) {	// 128K or Issue 3
		keytmp |= (ula_border & 0x10) ? 0xe0 : 0xa0;
	} else {											// Issue 2
		keytmp |= (ula_border & 0x18) ? 0xe0 : 0xa0;
	}

	return keytmp;
}

static UINT8 __fastcall SpecZ80PortRead(UINT16 address)
{
	if (~address & 0x0001) { // keyboard
		return read_keyboard(address);
	}

	if ((address & 0x1f) == 0x1f && (address & 0x20) == 0) {
		// Kempston only pulls A5 low - Kempston RE: https://www.youtube.com/watch?v=4e1MlxPVyD4
		return SpecInput[8]; // kempston (returns 0xff when disabled)
	}

	if ((address & 0xc002) == 0xc000 && (SpecMode & SPEC_AY8910)) {
		return AY8910Read(0);
	}

	update_ula(ZetTotalCycles()); // get up-to-date ula_byte!

	return ula_byte; // Floating Memory
}

static void __fastcall SpecZ80PortWrite(UINT16 address, UINT8 data)
{
	if (~address & 0x0001) {
		BuzzerAdd((data & 0x10) >> 4);

		ula_border = data;
		return;
	}

	if (SpecMode & SPEC_AY8910) {
		switch (address & 0xc002) {
			case 0x8000: AY8910Write(0, 1, data); return;
			case 0xc000: AY8910Write(0, 0, data); return;
		}
	}

	if ((address & 0xff) == 0xfd) return; // Ignore (Jetpac writes here due to a bug in the game code, and it's the reason it won't work on 128k)

	//bprintf(0, _T("pw %x %x\n"), address, data);
}

static void spectrum128_bank()
{
	SpecVideoRam = SpecZ80Ram + ((5 + ((Spec128kMapper & 0x08) >> 2)) << 14);

	SpecRamPage = Spec128kMapper & 0x07;
	SpecRomPage = ((Spec128kMapper & 0x10) >> 4) * 0x4000;

	if (SpecMode & SPEC_PLUS2) {
		SpecRomPage += ((Spec128kMapper2 & 4) >> 1) * 0x4000;

		if (Spec128kMapper2 & 0x01) { // +2a/+3 "Special" paging mode
			const INT32 special_map[4][4] = { { 0, 1, 2, 3 },  { 4, 5, 6, 7 },  { 4, 5, 6, 3 },  { 4, 7, 6, 3 } };
			const INT32 special_sel = (Spec128kMapper2 >> 1) & 3;

			ZetMapMemory(SpecZ80Ram + (special_map[special_sel][0] << 14), 0x0000, 0x3fff, MAP_RAM);
			ZetMapMemory(SpecZ80Ram + (special_map[special_sel][1] << 14), 0x4000, 0x7fff, MAP_RAM);
			ZetMapMemory(SpecZ80Ram + (special_map[special_sel][2] << 14), 0x8000, 0xbfff, MAP_RAM);
			ZetMapMemory(SpecZ80Ram + (special_map[special_sel][3] << 14), 0xc000, 0xffff, MAP_RAM);
		} else {
			ZetUnmapMemory(0x0000, 0xffff, MAP_RAM);
		}
	}

	Z80Contention_set_bank(SpecRamPage);
}

static UINT8 __fastcall SpecSpec128Z80Read(UINT16 address)
{
	if (address < 0x4000) {
		return SpecZ80Rom[SpecRomPage + address];
	}

	if (address >= 0x4000 && address <= 0x7fff) {
		return SpecZ80Ram[(5 << 14) + (address & 0x3fff)];
	}

	if (address >= 0x8000 && address <= 0xbfff) {
		return SpecZ80Ram[(2 << 14) + (address & 0x3fff)];
	}

	if (address >= 0xc000 && address <= 0xffff) {
		return SpecZ80Ram[(SpecRamPage << 14) + (address & 0x3fff)];
	}

	bprintf(0, _T("mr %x\n"), address);

	return 0;
}

static void __fastcall SpecSpec128Z80Write(UINT16 address, UINT8 data)
{
	if (address < 0x4000) return; // ROM

	if (address >= 0x4000 && address <= 0x7fff) {
		SpecZ80Ram[(5 << 14) + (address & 0x3fff)] = data;
		return;
	}

	if (address >= 0x8000 && address <= 0xbfff) {
		SpecZ80Ram[(2 << 14) + (address & 0x3fff)] = data;
		return;
	}

	if (address >= 0xc000 && address <= 0xffff) {
		SpecZ80Ram[(SpecRamPage << 14) + (address & 0x3fff)] = data;
		return;
	}

	bprintf(0, _T("mw %x %x\n"), address, data);
}

static UINT8 __fastcall SpecSpec128Z80PortRead(UINT16 address)
{
	if (~address & 0x0001) { // keyboard
		return read_keyboard(address);
	}

	if ((address & 0x1f) == 0x1f && (address & 0x20) == 0) {
		// Kempston only pulls A5 low - Kempston RE: https://www.youtube.com/watch?v=4e1MlxPVyD4
		return SpecInput[8]; // kempston (returns 0xff when disabled)
	}

	if ((address & 0xc002) == 0xc000 && (SpecMode & SPEC_AY8910)) {
		return AY8910Read(0);
	}

	if ((address & 0x8002) == 0x0000) {
		// todo: figure out what 0x7ffd / 3ffd read does
		//bprintf(0, _T("reading %x (%x)\n"), address, Spec128kMapper);
	}

	update_ula(ZetTotalCycles()); // get up-to-date ula_byte!

	return ula_byte; // Floating Memory
}

static void __fastcall SpecSpec128Z80PortWrite(UINT16 address, UINT8 data)
{
	if (~address & 0x0001) {
		BuzzerAdd((data & 0x10) >> 4);

		ula_border = data;
		// needs to fall through!!
	}

	if (SpecMode & SPEC_PLUS2) { // 128k +2a / +3
		if ((address & 0xc002) == 0x4000) { // 7ffd
			if (Spec128kMapper & 0x20) return; // memory lock-out latch

			Spec128kMapper = data;
			//bprintf(0, _T("7ffd: %x\t%x\n"), address, data);
			spectrum128_bank();
			return;
		}

		if ((address & 0xf002) == 0x1000) { // 1ffd
			if (Spec128kMapper & 0x20) return; // memory lock-out latch

			//bprintf(0, _T("1ffd: %x\t%x\n"), address, data);
			Spec128kMapper2 = data;
			spectrum128_bank();
			return;
		}
	} else { // standard 128k
		if ((address & 0x8002) == 0x0000) { // 7ffd
			if (Spec128kMapper & 0x20) return; // memory lock-out latch

			Spec128kMapper = data;
			//bprintf(0, _T("7ffd: %x\t%x\n"), address, data);
			spectrum128_bank();
			return;
		}
	}

	if (SpecMode & SPEC_AY8910) {
		switch (address & 0xc002) {
			case 0x8000: AY8910Write(0, 1, data); return;
			case 0xc000: AY8910Write(0, 0, data); return;
		}
	}

	if (address == 0xff3b || address == 0xbf3b) return; // ignore (some games check for "ula plus" here)

	//bprintf(0, _T("pw %x %x\n"), address, data);
}

// Spectrum TAP loader (c) 2020 dink
#define DEBUG_TAP 0
#define BLKNUM 0x200
static UINT8 *SpecTAPBlock[BLKNUM];
static INT32 SpecTAPBlockLen[BLKNUM];

static INT32 SpecTAPBlocks = 0; // 1-based
static INT32 SpecTAPBlocknum = 0; // 0-based
static INT32 SpecTAPPos = 0; // 0-based
static INT32 SpecTAPLoading = 0;

static void SpecTAPReset()
{
	SpecTAPBlocknum = 0;
	SpecTAPPos = 0;
	SpecTAPLoading = 0;
}

static void SpecTAPInit()
{
	for (INT32 i = 0; i < BLKNUM; i++) {
		SpecTAPBlock[i] = NULL;
		SpecTAPBlockLen[i] = 0;
	}
	SpecTAPBlocks = 0;
	SpecTAPBlocknum = 0;
	if (DEBUG_TAP) {
		bprintf(0, _T("**  - Spectrum TAP Loader -\n"));
		bprintf(0, _T("Block#\tLength\tOffset\n"));
	}
	for (INT32 i = 0; i < SpecTAPLen;) {
		INT32 block_size = SpecTAP[i+0] | (SpecTAP[i+1] << 8);

		if (block_size) {
			if (DEBUG_TAP) {
				bprintf(0, _T("%x\t%d\t%x\n"), SpecTAPBlocks, block_size, i+2);
			}

			SpecTAPBlock[SpecTAPBlocks] = &SpecTAP[i+2];
			SpecTAPBlockLen[SpecTAPBlocks] = block_size-2;
			SpecTAPBlocks++;
			if (SpecTAPBlocks >= BLKNUM) {
				bprintf(PRINT_ERROR, _T(".TAP Loader: Tape blocks exceeded.\n"));
				break;
			}
		}

		i += block_size + 2;
	}
}

static INT32 SpecTAPDMACallback()
{
	if (~SpecMode & SPEC_TAP || SpecTAPBlocks == 0) return 0;

	UINT8 *data = SpecTAPBlock[SpecTAPBlocknum];

	INT32 transfer_ok = 0;
	INT32 carry_val = 0;
	INT32 checksum = 0;
	INT32 offset = 0;
	UINT8 byte = 0;

	ActiveZ80EXAF();
	INT32 tap_block = data[0];
	INT32 cpu_block = ActiveZ80GetAF() >> 8;
	INT32 address = ActiveZ80GetIX();
	INT32 length = ActiveZ80GetDE();
	INT32 length_unadjusted = length;

	// If anything is changed here, try the following testcases:
	// Chickin Chase, Alter Ego, V

	if (DEBUG_TAP) {
		bprintf(0, _T("TAP blocknum %d\n"), SpecTAPBlocknum);
		bprintf(0, _T("TAP blocklen %d\n"), SpecTAPBlockLen[SpecTAPBlocknum]);
		bprintf(0, _T("TAP blocktype %x\n"), tap_block);
		bprintf(0, _T("CPU blocktype %x\n"), cpu_block);
		bprintf(0, _T("CPU address %x\n"), address);
		bprintf(0, _T("CPU length %x\n"), length);
	}
	if (length > SpecTAPBlockLen[SpecTAPBlocknum]) {
		bprintf(0, _T("CPU Requested length %x > tape block length %x, adjusting.\n"), length, SpecTAPBlockLen[SpecTAPBlocknum]);
		length = SpecTAPBlockLen[SpecTAPBlocknum];
	}
	if (cpu_block == tap_block) { // we found our block! :)
		if (ActiveZ80GetCarry()) {
			if (DEBUG_TAP) {
				bprintf(0, _T("loading data\n"));
			}
			// load
			offset = 0;
			checksum = tap_block;
			while (offset < length) {
				if (offset+1 > SpecTAPBlockLen[SpecTAPBlocknum]) {
					bprintf(0, _T(".TAP Loader: trying to read past block.  offset %x  blocklen %x\n"), offset, SpecTAPBlockLen[SpecTAPBlocknum]);
					carry_val = 0;
					break;
				}
				byte = data[offset+1];
				ZetWriteByte((address + offset) & 0xffff, data[offset+1]);
				checksum ^= data[offset+1];
				offset++;
			}
			if (DEBUG_TAP) {
				bprintf(0, _T("end dma, checksum %x  tap checksum %x\n"), checksum, data[offset+1]);
			}
			carry_val = (checksum == data[offset+1]);
			transfer_ok = 1;
		}
	}

	ActiveZ80SetCarry(carry_val);
	ActiveZ80SetIX((address + offset) & 0xffff);
	if (transfer_ok) ActiveZ80SetDE(length_unadjusted - SpecTAPBlockLen[SpecTAPBlocknum]);
	ActiveZ80SetHL((checksum << 8) | byte);
	ActiveZ80SetA(0);
	ActiveZ80SetPC(0x05e2);

	SpecTAPBlocknum = (SpecTAPBlocknum + 1) % SpecTAPBlocks;

	return 0;
}

static void snapshot_write_ram(UINT32 address, UINT8 data)
{
	if (address >= 0x4000 && address < 0x24000) { // ignore writes to bios area (snapshot)
		SpecZ80Ram[address - 0x4000] = data;
	} else {
		bprintf(PRINT_ERROR, _T(".z80, snapshot_write_ram(%x, %x).\n"));
	}
}

static void z80_rle_decompress(UINT8 *source, UINT32 dest, UINT16 size)
{
	while (size > 0) {
		if (size > 2 && source[0] == 0xed && source[1] == 0xed) {
			UINT8 count = source[2];
			UINT8 data = source[3];

			if (count == 0)	{
				bprintf(PRINT_ERROR, _T(".z80 rle_decompress: zero length rle-block? eek. (bad .z80 file)\n"));
				return;
			}

			if (count > size) {
				bprintf(PRINT_ERROR, _T(".z80 rle_decompress: count > size, eek. (bad .z80 file)\n"));
				count = size;
			}

			for (INT32 i = 0; i < count; i++) {
				snapshot_write_ram(dest, data);
				dest++;
			}

			source += 4;
			size -= count;
		} else {
			snapshot_write_ram(dest, source[0]);

			dest++;
			source++;
			size--;
		}
	}
}

static UINT16 mem2uint16(INT32 posi, INT32 bigendian)
{
	UINT8 *umem = (UINT8*)&SpecSnapshotData[posi];

	if (bigendian) {
		return (umem[0] << 8) + umem[1];
	} else {
		return (umem[1] << 8) + umem[0];
	}
}

static UINT32 page_to_mem(INT32 page, INT32 is128k)
{
	if (is128k) {
		switch (page) {
			case 3: return 0x4000;
			case 4: return 0x8000;
			case 5: return 0xc000;
			case 6: return 0x10000;
			case 7: return 0x14000;
			case 8: return 0x18000;
			case 9: return 0x1c000;
			case 10: return 0x20000; // ram ends at 0x24000 (0-4000 = rom)
			default: return 0x0000;
		}
	} else {
		switch (page) {
			case 4: return 0x8000;
			case 5: return 0xc000;
			case 8: return 0x4000;
			default: return 0x0000;
		}
	}

	return 0x0000;
}

static void spectrum_loadz80()
{
	ZetSetAF(0, mem2uint16(0, 1));
	ZetSetBC(0, mem2uint16(2, 0));
	ZetSetHL(0, mem2uint16(4, 0));
	UINT16 v1PC = mem2uint16(6, 0);
	ZetSetSP(0, mem2uint16(8, 0));
	ZetSetI(0, (SpecSnapshotData[10] & 0xff));
	ZetSetR(0, (SpecSnapshotData[11] & 0x7f) | ((SpecSnapshotData[12] & 0x01) << 7));
	ula_border = (ula_border & 0xf8) | ((SpecSnapshotData[12] & 0x0e) >> 1);
	ZetSetDE(0, mem2uint16(13, 0));
	ZetSetBC2(0, mem2uint16(15, 0));
	ZetSetDE2(0, mem2uint16(17, 0));
	ZetSetHL2(0, mem2uint16(19, 0));
	ZetSetAF2(0, mem2uint16(21, 1));
	ZetSetIY(0, mem2uint16(23, 0));
	ZetSetIX(0, mem2uint16(25, 0));
	ZetSetIFF1(0, (SpecSnapshotData[27]) ? 1 : 0);
	ZetSetIFF2(0, (SpecSnapshotData[28]) ? 1 : 0);
	ZetSetIM(0, (SpecSnapshotData[29] & 0x03));

	if (v1PC != 0) { // version 1 (48k) snapshot
		bprintf(0, _T(".z80 version 1 - 48k snapshot - "));

		ZetSetPC(0, v1PC);

		if (SpecSnapshotData[12] & 0x20) {
			bprintf(0, _T(".z80: rle-compressed\n"));
			z80_rle_decompress(SpecSnapshotData + 30, 0x4000, 0xc000);
		} else {
			bprintf(0, _T(".z80: un-compressed\n")); // testcase: Horace Goes Skiing
			for (INT32 i = 0x4000; i < 0xc000; i++) {
				snapshot_write_ram(i, SpecSnapshotData[30 + (i - 0x4000)]);
			}
		}
	} else {
		INT32 v2_v3_headerlen = mem2uint16(30, 0);
		INT32 hwmode = SpecSnapshotData[34];
		INT32 v2 = (v2_v3_headerlen == 23);
		INT32 is_128k = ((v2) ? (hwmode > 2) : (hwmode > 3));

		bprintf(0, _T(".z80 version %d - "), (v2) ? 2 : 3);
		if (is_128k) {
			bprintf(0, _T("128k\n"));
			if (~SpecMode & SPEC_128K) {
				bprintf(PRINT_ERROR, _T(".z80 Error: loading 128k snapshot on 48k hw!\n"));
				return;
			}
		} else {
			bprintf(0, _T("48k\n"));
		}

		ZetSetPC(0, mem2uint16(32, 0));

		if (SpecMode & SPEC_AY8910 && SpecSnapshotData[37] & (1<<2)) { // AY8910
			bprintf(0, _T(".z80 contains AY8910 registers\n"));
			ZetOpen(0);
			for (INT32 i = 0; i < 0x10; i++) { // write regs
				AY8910Write(0, 0, i);
				AY8910Write(0, 1, SpecSnapshotData[39 + i]);
			}
			AY8910Write(0, 0, SpecSnapshotData[38]); // write latch
			ZetClose();
		}

		INT32 offset = v2_v3_headerlen + 32;
		while (offset < SpecSnapshotDataLen)
		{
			UINT16 length = mem2uint16(offset, 0);
			UINT32 dest = page_to_mem(SpecSnapshotData[offset + 2], is_128k);

			if (length == 0xffff) {
				length = 0x4000;
				bprintf(0, _T(".z80: copying $%x uncompressed bytes to %x\n"), length, dest);
				for (INT32 i = 0; i < length; i++)
					snapshot_write_ram(dest + i, SpecSnapshotData[offset + 3 + i]);
			} else {
				bprintf(0, _T(".z80: decompressing $%x bytes to %x\n"), length, dest);
				z80_rle_decompress(&SpecSnapshotData[offset + 3], dest, 0x4000);
			}

			offset += 3 + length;
		}

		if (is_128k) {
			Spec128kMapper = (SpecSnapshotData[35] & 0xff);
			ZetOpen(0);
			spectrum128_bank();
			ZetClose();
		}
	}
}

static INT32 BurnGetLength(INT32 rom_index)
{
	struct BurnRomInfo ri = { "", 0, 0, 0 };
	BurnDrvGetRomInfo(&ri, 0);
	return ri.nLen;
}

struct s_modes {
	INT32 mode;
	TCHAR text[40];
};

static s_modes speccy_modes[] = {
	{ SPEC_TAP,		_T(".tap file")						},
	{ SPEC_Z80,		_T(".z80 file")						},
	{ SPEC_128K,	_T("128K")							},
	{ SPEC_PLUS2,	_T("+2a")							},
	{ SPEC_INVES,	_T("Investronica (un-contended)")	},
	{ SPEC_AY8910,	_T("AY-8910 PSG")					},
	{ -1,			_T("")								}
};

static void PrintSpeccyMode()
{
	bprintf(0, _T("Speccy Init w/ "));

	for (INT32 i = 0; speccy_modes[i].mode != -1; i++) {
		if (SpecMode & speccy_modes[i].mode) {
			bprintf(0, _T("%s, "), speccy_modes[i].text);
		}
	}

	bprintf(0, _T("...\n"));
}


static INT32 SpectrumInit(INT32 Mode)
{
	SpecMode = Mode;

	SpecMode |= SPEC_AY8910; // Add an AY-8910!

	PrintSpeccyMode();

	BurnSetRefreshRate(50.0);

	BurnAllocMemIndex();

	INT32 nRet = 0;

	if (SpecMode & SPEC_Z80) {
		// Snapshot
		SpecSnapshotDataLen = BurnGetLength(0);
		nRet = BurnLoadRom(SpecSnapshotData + 0x00000, 0, 1); if (nRet != 0) return 1;
		nRet = BurnLoadRom(SpecZ80Rom + 0x00000, 0x80, 1); if (nRet != 0) return 1;
	}
	else if (SpecMode & SPEC_TAP) {
		// TAP
		SpecTAPLen = BurnGetLength(0);
		nRet = BurnLoadRom(SpecTAP + 0x00000, 0, 1); if (nRet != 0) return 1;
		nRet = BurnLoadRom(SpecZ80Rom + 0x00000, 0x80, 1); if (nRet != 0) return 1;

		SpecTAPInit();
	} else {
		// System
		nRet = BurnLoadRom(SpecZ80Rom + 0x00000, 0, 1); if (nRet != 0) return 1;
	}

	ZetInit(0);
	ZetOpen(0);
	ZetSetReadHandler(SpecZ80Read);
	ZetSetWriteHandler(SpecZ80Write);
	ZetSetInHandler(SpecZ80PortRead);
	ZetSetOutHandler(SpecZ80PortWrite);
	if (SpecMode & SPEC_TAP) {
		bprintf(0, _T("**  Spectrum: Using TAP file (len 0x%x) - DMA Loader\n"), SpecTAPLen);
		z80_set_spectrum_tape_callback(SpecTAPDMACallback);
	}
	if (~SpecMode & SPEC_INVES) {
		Z80InitContention(48, &update_ula);
	}
	ZetClose();

	AY8910Init(0, 17734475 / 10, 0);
	AY8910SetAllRoutes(0, 0.20, BURN_SND_ROUTE_BOTH);
	AY8910SetBuffered(ZetTotalCycles, 224*312*50);

	// Init Buzzer (in DoReset!)

	GenericTilesInit();

	ula_init(312, 224, 14335);

	SpecDoReset();

	return 0;
}

static INT32 Spectrum128Init(INT32 Mode)
{
	SpecMode = Mode;

	SpecMode |= SPEC_AY8910; // Add an AY-8910! (always on-board w/128K)

	BurnSetRefreshRate(50.0);

	BurnAllocMemIndex();

	PrintSpeccyMode();

	INT32 nRet = 0;

	if (SpecMode & SPEC_Z80) {
		// Snapshot
		SpecSnapshotDataLen = BurnGetLength(0);
		nRet = BurnLoadRom(SpecSnapshotData + 0x00000, 0, 1); if (nRet != 0) return 1;
		nRet = BurnLoadRom(SpecZ80Rom + 0x00000, 0x80, 1); if (nRet != 0) return 1;
		nRet = BurnLoadRom(SpecZ80Rom + 0x04000, 0x81, 1); if (nRet != 0) return 1;
		if (SpecMode & SPEC_PLUS2) {
			nRet = BurnLoadRom(SpecZ80Rom + 0x08000, 0x82, 1); if (nRet != 0) return 1;
			nRet = BurnLoadRom(SpecZ80Rom + 0x0c000, 0x83, 1); if (nRet != 0) return 1;
		}
	}
	else if (SpecMode & SPEC_TAP) {
		// TAP
		SpecTAPLen = BurnGetLength(0);
		nRet = BurnLoadRom(SpecTAP + 0x00000, 0, 1); if (nRet != 0) return 1;
		nRet = BurnLoadRom(SpecZ80Rom + 0x00000, 0x80, 1); if (nRet != 0) return 1;
		nRet = BurnLoadRom(SpecZ80Rom + 0x04000, 0x81, 1); if (nRet != 0) return 1;
		if (SpecMode & SPEC_PLUS2) {
			nRet = BurnLoadRom(SpecZ80Rom + 0x08000, 0x82, 1); if (nRet != 0) return 1;
			nRet = BurnLoadRom(SpecZ80Rom + 0x0c000, 0x83, 1); if (nRet != 0) return 1;
		}

		SpecTAPInit();
	} else {
		// System
		nRet = BurnLoadRom(SpecZ80Rom + 0x00000, 0, 1); if (nRet != 0) return 1;
		nRet = BurnLoadRom(SpecZ80Rom + 0x04000, 1, 1); if (nRet != 0) return 1;
		if (SpecMode & SPEC_PLUS2) {
			nRet = BurnLoadRom(SpecZ80Rom + 0x08000, 2, 1); if (nRet != 0) return 1;
			nRet = BurnLoadRom(SpecZ80Rom + 0x0c000, 3, 1); if (nRet != 0) return 1;
		}
	}

	ZetInit(0);
	ZetOpen(0);
	ZetSetReadHandler(SpecSpec128Z80Read);
	ZetSetWriteHandler(SpecSpec128Z80Write);
	ZetSetInHandler(SpecSpec128Z80PortRead);
	ZetSetOutHandler(SpecSpec128Z80PortWrite);
	if (SpecMode & SPEC_TAP) {
		bprintf(0, _T("**  Spectrum 128k: Using TAP file (len 0x%x) - DMA Loader\n"), SpecTAPLen);
		z80_set_spectrum_tape_callback(SpecTAPDMACallback);
	}
	if (~SpecMode & SPEC_INVES) {
		Z80InitContention((SpecMode & SPEC_PLUS2) ? 1282 : 128, &update_ula);
	}
	ZetClose();

	AY8910Init(0, 17734475 / 10, 0);
	AY8910SetAllRoutes(0, 0.20, BURN_SND_ROUTE_BOTH);
	AY8910SetBuffered(ZetTotalCycles, 228*311*50);

	// Init Buzzer (in DoReset!)

	GenericTilesInit();

	ula_init(311, 228, 14361);

	SpecDoReset();

	return 0;
}

static INT32 get_type()
{
	char *rn = NULL;

	if (!BurnDrvGetRomName(&rn, 0, 0))
	{
		INT32 len = strlen(rn);

		if (len > 4) {
			if (!strcasecmp(".z80", rn + (len-4))) {
				return SPEC_Z80;
			}
			if (!strcasecmp(".tap", rn + (len-4))) {
				return SPEC_TAP;
			}
		}
	}

	return 0;
}

static INT32 SpecInit()
{
	return SpectrumInit(get_type());
}

static INT32 Spec128KInit()
{
	return Spectrum128Init(SPEC_128K | get_type());
}

static INT32 Spec128KPlus2Init()
{
	return Spectrum128Init(SPEC_128K | SPEC_PLUS2 | get_type());
}

static INT32 Spec128KInvesInit()
{
	return Spectrum128Init(SPEC_128K | SPEC_INVES | get_type());
}

static INT32 SpecExit()
{
	ZetExit();

	if (SpecMode & SPEC_AY8910) AY8910Exit(0);

	BuzzerExit();

	GenericTilesExit();

	BurnFreeMemIndex();

	return 0;
}

static void SpecCalcPalette()
{
	UINT32 spec_pal[0x10] = {
		0x000000, 0x0000bf, 0xbf0000, 0xbf00bf,
		0x00bf00, 0x00bfbf, 0xbfbf00, 0xbfbfbf,
		0x000000, 0x0000ff, 0xff0000, 0xff00ff,
		0x00ff00, 0x00ffff, 0xffff00, 0xffffff
	};

	for (INT32 i = 0; i < 0x10; i++) {
		SpecPalette[i] = BurnHighCol((spec_pal[i] >> 16) & 0xff, (spec_pal[i] >> 8) & 0xff, spec_pal[i] & 0xff, 0);
	}
}

static INT32 SpecDraw()
{
	if (SpecRecalc) {
		SpecCalcPalette();
		SpecRecalc = 0;
	}

	BurnTransferCopy(SpecPalette);

	return 0;
}

// dink's ULA simulator 2000 SSEI (super special edition intense)
static INT32 CONT_START;
static INT32 CONT_END;
static INT32 BORDER_START;
static INT32 BORDER_END;

static void ula_init(INT32 scanlines, INT32 cyc_scanline, INT32 contention)
{
	SpecScanlines = scanlines;
	SpecCylesPerScanline = cyc_scanline;
	SpecContention = contention;
	CONT_START  = SpecContention; // 48k; 14335 / 128k: 14361
	CONT_END    = (CONT_START + 192 * SpecCylesPerScanline);

	BORDER_START= SpecContention + 3; // start at first vis. pixel (top-left)
	BORDER_START-=(SpecCylesPerScanline * 16) + 8; // go back 16 lines
	BORDER_END  = SpecCylesPerScanline * (16+256+16);
}

#if 0
static UINT8 ula_snow()
{
	extern int counter;
	return (ActiveZ80GetI() >= 0x40 && ActiveZ80GetI() <= 0x7f && (ActiveZ80GetLastOp() & counter) == 0/* && (rand() & 0x187)==0x181*/) ? (ActiveZ80GetR() & 0x7f) : 0x00;
}
#endif

static void ula_run_cyc(INT32 cyc, INT32 draw_screen)
{
	ula_byte = 0xff; // open-bus byte 0xff unless ula fetches screen data

	// borders (top + sides + bottom) - note: only drawing 16px borders here!
	if (cyc >= BORDER_START && cyc <= BORDER_END) {
		INT32 offset = cyc - BORDER_START;
		INT32 x = ((offset) % SpecCylesPerScanline) * 2;
		INT32 y =  (offset) / SpecCylesPerScanline;
		INT32 border = ula_border & 0x07;

		INT32 draw = 0;

		// top border
		if (y >= 0 && y < 16 && x >= 0 && x < nScreenWidth) {
			draw = 1;
		}

		// side borders
		if (y >= 16 && y < 16+192+16 && ((x >= 0 && x < 16) || (x >= 16+256 && x < 16+256+16)) && x < nScreenWidth) {
			draw = 1;
		}

		// bottom border
		if (y >= 16 + 192 && y < 16+192+16 && x >= 0 && x < nScreenWidth) {
			draw = 1;
		}

		if (draw) {
			if (draw_screen && ((x & 7) == 0) && x <= nScreenWidth - 8) {
				for (INT32 xx = x; xx < x + 8; xx++) {
					pTransDraw[y * nScreenWidth + xx] = border;
				}
			}
		}
	}

	// active area
	if (cyc > CONT_START && cyc <= CONT_END) {
		INT32 offset = cyc - (CONT_START+1); // get to start of contention pattern
		INT32 x = (offset) % SpecCylesPerScanline;
		INT32 y = (offset) / SpecCylesPerScanline;

		if (x < 128) {
			switch (x & 7) {
				default: // 0, 1, 6, 7
					ula_byte = 0xff;
					break;

				case 2:
				case 4:
					ula_scr = SpecVideoRam[((y & 0xc0) << 5) | ((y & 0x38) << 2) | ((y & 0x07) << 8) | (x >> 2)];// | ula_snow()<<1];
					ula_byte = ula_scr;
					break;

				case 3:
				case 5:
					ula_attr = SpecVideoRam[0x1800 | ((y & 0xf8) << 2) | (x >> 2)];
					ula_byte = ula_attr;

					UINT16 *dst = pTransDraw + ((y + 16) * nScreenWidth) + (((x << 1) + 16) & ~7);
					UINT8 ink = (ula_attr & 0x07) + ((ula_attr >> 3) & 0x08);
					UINT8 pap = (ula_attr >> 3) & 0x0f;

					if (ula_flash & 0x10 && ula_attr & 0x80) ula_scr = ~ula_scr;

					if (draw_screen) {
						*dst++ = (ula_scr & (1 << 7)) ? ink : pap;
						*dst++ = (ula_scr & (1 << 6)) ? ink : pap;
						*dst++ = (ula_scr & (1 << 5)) ? ink : pap;
						*dst++ = (ula_scr & (1 << 4)) ? ink : pap;
						*dst++ = (ula_scr & (1 << 3)) ? ink : pap;
						*dst++ = (ula_scr & (1 << 2)) ? ink : pap;
						*dst++ = (ula_scr & (1 << 1)) ? ink : pap;
						*dst++ = (ula_scr & (1 << 0)) ? ink : pap;
					}
					break;
			}
		}
	}
}

static void update_ula(INT32 cycle)
{
	//bprintf(0, _T("update_ula:  %d   last %d\n"), cycle, ula_last_cyc);
	if (cycle == -1) {
		// next frame!
		ula_last_cyc = 0;
		return;
	}

	for (INT32 i = ula_last_cyc+1; i <= cycle; i++) {
		ula_run_cyc(i, 1);
	}

	ula_last_cyc = cycle;
}

static void mix_dcblock(INT16 *inbuf, INT16 *outbuf, INT32 sample_nums)
{
	INT16 out;

	for (INT32 sample = 0; sample < sample_nums; sample++)
	{
		INT16 result = inbuf[sample * 2 + 0]; // source sample

		// dc block
		out = result - dac_lastin + 0.998 * dac_lastout;
		dac_lastin = result;
		dac_lastout = out;

		out *= 2.5;

		// add to stream (+include ay if Spec128)
		if (SpecMode & SPEC_AY8910) {
			outbuf[sample * 2 + 0] = BURN_SND_CLIP(outbuf[sample * 2 + 0] + out);
			outbuf[sample * 2 + 1] = BURN_SND_CLIP(outbuf[sample * 2 + 1] + out);
		} else {
			outbuf[sample * 2 + 0] = BURN_SND_CLIP(out);
			outbuf[sample * 2 + 1] = BURN_SND_CLIP(out);
		}
	}
}

static void SwapByte(UINT8 &a, UINT8 &b)
{ // a <-> b, using xor
	a = a ^ b;
	b = a ^ b;
	a = a ^ b;
}

static INT32 SpecFrame()
{
	if (SpecReset) SpecDoReset();

	if (SpecMode & SPEC_TAP) TAPAutoLoadRobot();

	{
		// Init keyboard matrix & 128k secondary matrix (caps/shifts lines) active-low (0x1f)
		SpecInput[0] = SpecInput[1] = SpecInput[2] = SpecInput[3] = SpecInput[4] = SpecInput[5] = SpecInput[6] = SpecInput[7] = 0x1f;
		SpecInput[11] = SpecInput[12] = SpecInput[13] = SpecInput[14] = SpecInput[15] = 0x1f;

		SpecInput[8] = 0x00; // kempston joy (active high)
		SpecInput[9] = SpecInput[10] = 0x1f; // intf2 joy (active low)

		if (SpecDips[0] & 0x01) { // use intf2: map kempston joy1 to intf2 joy1
			SpecInputKbd[9][1] = SpecInputKbd[8][3];
			SpecInputKbd[9][2] = SpecInputKbd[8][2];
			SpecInputKbd[9][4] = SpecInputKbd[8][1];
			SpecInputKbd[9][3] = SpecInputKbd[8][0];
			SpecInputKbd[9][0] = SpecInputKbd[8][4];

			if (SpecDips[0] & 0x20) { // swap intf2 joy1 <-> joy2
				SwapByte(SpecInputKbd[9][1], SpecInputKbd[10][3]);
				SwapByte(SpecInputKbd[9][2], SpecInputKbd[10][2]);
				SwapByte(SpecInputKbd[9][4], SpecInputKbd[10][0]);
				SwapByte(SpecInputKbd[9][3], SpecInputKbd[10][1]);
				SwapByte(SpecInputKbd[9][0], SpecInputKbd[10][4]);
			}
		}

		if (SpecDips[0] & (0x02|0x04)) { // map Kempston to QAOPM/QAOP SPACE
			SpecInputKbd[2][0] |= SpecInputKbd[8][3]; // Up -> Q
			SpecInputKbd[1][0] |= SpecInputKbd[8][2]; // Down -> A
			SpecInputKbd[5][1] |= SpecInputKbd[8][1]; // Left -> O
			SpecInputKbd[5][0] |= SpecInputKbd[8][0]; // Right -> P

			switch (SpecDips[0] & (0x02|0x04)) {
				case 0x02:
					SpecInputKbd[7][2] |= SpecInputKbd[8][4]; // Fire -> M
					break;
				case 0x04:
					SpecInputKbd[7][0] |= SpecInputKbd[8][4]; // Fire -> Space
					break;
			}
		}

		if (SpecDips[0] & 0x08) { // map kempston joy1 to Cursor Keys
			SpecInputKbd[4][3] |= SpecInputKbd[8][3]; // Up -> 7
			SpecInputKbd[4][4] |= SpecInputKbd[8][2]; // Down -> 6
			SpecInputKbd[3][4] |= SpecInputKbd[8][1]; // Left -> 5
			SpecInputKbd[4][2] |= SpecInputKbd[8][0]; // Right -> 8
			SpecInputKbd[4][0] |= SpecInputKbd[8][4]; // Fire -> 0
		}

		if (SpecDips[0] & 0x40) { // map kempston joy1 to Jumping Jack keys
			SpecInputKbd[7][1] |= SpecInputKbd[8][1]; // Left -> Symbol Shift
			SpecInputKbd[7][0] |= SpecInputKbd[8][0]; // Right -> Space
			SpecInputKbd[0][0] |= SpecInputKbd[8][4]; // Fire -> Caps Shift
		}

		// Compile the input matrix
		for (INT32 i = 0; i < 5; i++) {
			for (INT32 j = 0; j < 0x10; j++) { // 8x5 keyboard matrix
				SpecInput[j] ^= (SpecInputKbd[j][i] & 1) << i;
			}
		}

		// Disable inactive hw
		if ((SpecDips[0] & 0x1f) != 0x00) { // kempston not selected
			SpecInput[8] = 0xff; // kempston joy (active high, though none present returns 0xff)
		}
		if ((SpecDips[0] & 0x1f) != 0x01) { // intf2 not selected
			SpecInput[9] = SpecInput[10] = 0x1f; // intf2 joy (active low)
		}
	}

	INT32 nCyclesDo = 0;

	ZetNewFrame();
	ZetOpen(0);
	ZetIdle(nExtraCycles);
	nExtraCycles = 0;

	const INT32 IRQ_LENGTH = 38; // 48k 32, 128k 36 (z80 core takes care of this - only allowing irq for proper length of machine, so value here is OK)

	for (INT32 i = 0; i < SpecScanlines; i++) {
		if (i == 0) {
			ZetSetIRQLine(0, CPU_IRQSTATUS_ACK);
			ZetRun(IRQ_LENGTH);
			ZetSetIRQLine(0, CPU_IRQSTATUS_NONE);
			ula_flash = (ula_flash + 1) & 0x1f;
		}

		nCyclesDo += SpecCylesPerScanline;
		ZetRun(nCyclesDo - ZetTotalCycles());

		if (SpecMode & SPEC_INVES) {
			update_ula(ZetTotalCycles());
		}
	}

	if (SpecMode & SPEC_INVES) {
		update_ula(-1); // notify start next frame
	}

	if (pBurnDraw) {
		SpecDraw();
	}

	if (pBurnSoundOut) {
		if (SpecMode & SPEC_AY8910) {
			AY8910Render(pBurnSoundOut, nBurnSoundLen);
		}

		BuzzerRender(dacbuf);
	    mix_dcblock(dacbuf, pBurnSoundOut, nBurnSoundLen);
	}

	INT32 tot_frame = SpecScanlines * SpecCylesPerScanline;
	nExtraCycles = ZetTotalCycles() - tot_frame;

	ZetClose();

	return 0;
}

static INT32 SpecScan(INT32 nAction, INT32* pnMin)
{
	if (pnMin != NULL) {
		*pnMin = 0x029744;
	}

	if (nAction & ACB_MEMORY_RAM) {
		ScanVar(RamStart, RamEnd-RamStart, "All RAM");
	}

	if (nAction & ACB_DRIVER_DATA) {
		ZetScan(nAction);

		if (SpecMode & SPEC_AY8910) {
			AY8910Scan(nAction, pnMin);
		}

		SCAN_VAR(ula_attr);
		SCAN_VAR(ula_scr);
		SCAN_VAR(ula_byte);
		SCAN_VAR(ula_border);
		SCAN_VAR(ula_flash);
		SCAN_VAR(ula_last_cyc);

		SCAN_VAR(Spec128kMapper);
		SCAN_VAR(Spec128kMapper2);

		SCAN_VAR(nExtraCycles);

		if (SpecMode & SPEC_TAP) {
			// .TAP
			SCAN_VAR(SpecTAPBlocknum);
			// .TAP Robot
			SCAN_VAR(CASAutoLoadPos);
			SCAN_VAR(CASAutoLoadTicker);
			SCAN_VAR(CASFrameCounter);
		}
	}

	if (nAction & ACB_WRITE && SpecMode & SPEC_128K) {
		ZetOpen(0);
		spectrum128_bank();
		ZetClose();
	}

	return 0;
}

static INT32 SpectrumGetZipName(char** pszName, UINT32 i)
{
	static char szFilename[MAX_PATH];
	char* pszGameName = NULL;

	if (pszName == NULL) {
		return 1;
	}

	if (i == 0) {
		pszGameName = BurnDrvGetTextA(DRV_NAME);
	} else {
		if (i == 1 && BurnDrvGetTextA(DRV_BOARDROM)) {
			pszGameName = BurnDrvGetTextA(DRV_BOARDROM);
		} else {
			pszGameName = BurnDrvGetTextA(DRV_PARENT);
		}
	}

	if (pszGameName == NULL || i > 2) {
		*pszName = NULL;
		return 1;
	}

	// remove leader
	memset(szFilename, 0, MAX_PATH);
	for (UINT32 j = 0; j < (strlen(pszGameName) - 5); j++) {
		szFilename[j] = pszGameName[j + 5];
	}

	*pszName = szFilename;

	return 0;
}

// BIOS Handling
static struct BurnRomInfo emptyRomDesc[] = {
	{ "", 0, 0, 0 },
};

static struct BurnRomInfo SpectrumRomDesc[] = {
	{ "spectrum.rom",  0x04000, 0xddee531f, BRF_BIOS },
};

STD_ROM_PICK(Spectrum)
STD_ROM_FN(Spectrum)

static struct BurnRomInfo Spec128RomDesc[] = {
	{ "zx128_0.rom",   0x04000, 0xe76799d2, BRF_BIOS },
	{ "zx128_1.rom",   0x04000, 0xb96a36be, BRF_BIOS },
};

STD_ROM_PICK(Spec128)
STD_ROM_FN(Spec128)

static struct BurnRomInfo Spec1282aRomDesc[] = {
	{ "zx128_2a_0.rom",   0x04000, 0x30c9f490, BRF_BIOS },
	{ "zx128_2a_1.rom",   0x04000, 0xa7916b3f, BRF_BIOS },
	{ "zx128_2a_2.rom",   0x04000, 0xc9a0b748, BRF_BIOS },
	{ "zx128_2a_3.rom",   0x04000, 0xb88fd6e3, BRF_BIOS },
};

STD_ROM_PICK(Spec1282a)
STD_ROM_FN(Spec1282a)

struct BurnDriver BurnSpecSpectrumBIOS = {
	"spec_spectrum", NULL, NULL, NULL, "1982",
	"ZX Spectrum BIOS\0", "BIOS Only", "Sinclair Research Limited", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_BOARDROM, 0, HARDWARE_SPECTRUM, GBF_BIOS, 0,
	SpectrumGetZipName, SpectrumRomInfo, SpectrumRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

struct BurnDriver BurnSpecSpectrum = {
	"spec_spec48k", NULL, NULL, NULL, "1982",
	"ZX Spectrum 16K/48K (OS)\0", NULL, "Sinclair Research Limited", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_BIOS, 0,
	SpectrumGetZipName, SpectrumRomInfo, SpectrumRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

struct BurnDriver BurnSpecSpec128BIOS = {
	"spec_spec128", NULL, NULL, NULL, "1986",
	"ZX Spectrum 128 BIOS\0", "BIOS Only", "Sinclair Research Limited", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_BOARDROM, 0, HARDWARE_SPECTRUM, GBF_BIOS, 0,
	SpectrumGetZipName, Spec128RomInfo, Spec128RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

struct BurnDriver BurnSpecSpec1282aBIOS = {
	"spec_spec1282a", NULL, NULL, NULL, "1987",
	"ZX Spectrum 128 +2a BIOS\0", "BIOS Only", "Amstrad Plc.", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_BOARDROM, 0, HARDWARE_SPECTRUM, GBF_BIOS, 0,
	SpectrumGetZipName, Spec1282aRomInfo, Spec1282aRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KPlus2Init, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

struct BurnDriver BurnSpecSpec128 = {
	"spec_spec128k", NULL, NULL, NULL, "1986",
	"ZX Spectrum 128K (OS)\0", NULL, "Sinclair Research Limited", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_BIOS, 0,
	SpectrumGetZipName, Spec128RomInfo, Spec128RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

struct BurnDriver BurnSpecSpec1282a = {
	"spec_spec128k2a", NULL, NULL, NULL, "1987",
	"ZX Spectrum 128K +2a (OS)\0", NULL, "Amstrad Plc.", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_BIOS, 0,
	SpectrumGetZipName, Spec1282aRomInfo, Spec1282aRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KPlus2Init, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};


// -------------------------------------------------------------
// Games (Classics) - Memory Snapshots 'Z80' & Tape Images 'TAP'
// -------------------------------------------------------------


// 180 (128K)

static struct BurnRomInfo Spec180RomDesc[] = {
	{ "180 (1986)(Mastertronic Added Dimension)[128K].tap", 47982, 0xccbd1a31, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Spec180, Spec180, Spec128)
STD_ROM_FN(Spec180)

struct BurnDriver BurnSpec180 = {
	"spec_180", NULL, "spec_spec128", NULL, "1986",
	"180 (128K)\0", NULL, "Mastertronic Added Dimension", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 2, HARDWARE_SPECTRUM, GBF_SPORTSMISC, 0,
	SpectrumGetZipName, Spec180RomInfo, Spec180RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// 1942 (48K)

static struct BurnRomInfo Spec1942RomDesc[] = {
	{ "1942 (1986)(Elite Systems).z80", 0x08a78, 0x82b77807, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Spec1942, Spec1942, Spectrum)
STD_ROM_FN(Spec1942)

struct BurnDriver BurnSpec1942 = {
	"spec_1942", NULL, "spec_spectrum", NULL, "1986",
	"1942 (48K)\0", NULL, "Elite Systems", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 2, HARDWARE_SPECTRUM, GBF_VERSHOOT, 0,
	SpectrumGetZipName, Spec1942RomInfo, Spec1942RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// 1943 - The Battle of Midway (48K)

static struct BurnRomInfo Spec1943RomDesc[] = {
	{ "1943 - The Battle of Midway (1988)(Go!).z80", 0x0996f, 0xc97f9144, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Spec1943, Spec1943, Spectrum)
STD_ROM_FN(Spec1943)

struct BurnDriver BurnSpec1943 = {
	"spec_1943", NULL, "spec_spectrum", NULL, "1988",
	"1943 - The Battle of Midway (48K)\0", NULL, "Go!", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 2, HARDWARE_SPECTRUM, GBF_VERSHOOT, 0,
	SpectrumGetZipName, Spec1943RomInfo, Spec1943RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// 720 Degrees (48K)

static struct BurnRomInfo Spec720degRomDesc[] = {
	{ "720 Degrees (1987)(U.S. Gold).tap", 50644, 0x842e1279, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Spec720deg, Spec720deg, Spectrum)
STD_ROM_FN(Spec720deg)

struct BurnDriver BurnSpec720deg = {
	"spec_720deg", NULL, "spec_spectrum", NULL, "1986",
	"720 Degrees (48K)\0", NULL, "U.S. Gold", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_SPORTSMISC, 0,
	SpectrumGetZipName, Spec720degRomInfo, Spec720degRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Abu Simbel Profanation (Spanish) (48K)

static struct BurnRomInfo SpecabusimprdRomDesc[] = {
	{ "Abu Simbel Profanation (1985)(Dinamic Software).tap", 46362, 0x7849893d, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specabusimprd, Specabusimprd, Spectrum)
STD_ROM_FN(Specabusimprd)

struct BurnDriver BurnSpecabusimprd = {
	"spec_abusimprd", "spec_abusimpr", "spec_spectrum", NULL, "1985",
	"Abu Simbel Profanation (Spanish) (48K)\0", NULL, "Dinamic Software", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecabusimprdRomInfo, SpecabusimprdRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Abu Simbel Profanation (English) (48K)

static struct BurnRomInfo SpecabusimprRomDesc[] = {
	{ "Abu Simbel Profanation (1987)(Gremlin Graphics Software)[re-release].tap", 47312, 0x0635946a, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specabusimpr, Specabusimpr, Spectrum)
STD_ROM_FN(Specabusimpr)

struct BurnDriver BurnSpecabusimpr = {
	"spec_abusimpr", NULL, "spec_spectrum", NULL, "1987",
	"Abu Simbel Profanation (English) (48K)\0", NULL, "Gremlin Graphics", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecabusimprRomInfo, SpecabusimprRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecIssue2DIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Ace of Aces (48K)

static struct BurnRomInfo SpecaceofacesRomDesc[] = {
	{ "Ace of Aces (1986)(U.S. Gold).tap", 89410, 0xd8b0115b, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specaceofaces, Specaceofaces, Spectrum)
STD_ROM_FN(Specaceofaces)

struct BurnDriver BurnSpecaceofaces = {
	"spec_aceofaces", NULL, "spec_spectrum", NULL, "1986",
	"Ace of Aces (48K)\0", "Keys from 1 to 4 to change view - 'M' to display map", "U.S. Gold", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_SHOOT | GBF_SIM, 0,
	SpectrumGetZipName, SpecaceofacesRomInfo, SpecaceofacesRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Action Fighter (128K)

static struct BurnRomInfo SpecafighterRomDesc[] = {
	{ "Action Fighter (1989)(Firebird Software)[128K].z80", 0x0a81a, 0x55f30b2a, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specafighter, Specafighter, Spec128)
STD_ROM_FN(Specafighter)

struct BurnDriver BurnSpecafighter = {
	"spec_afighter", NULL, "spec_spec128", NULL, "1989",
	"Action Fighter (128K)\0", NULL, "Firebird Software", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_VERSHOOT, 0,
	SpectrumGetZipName, SpecafighterRomInfo, SpecafighterRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Addams Family, The (128K)

static struct BurnRomInfo SpecaddfamthRomDesc[] = {
	{ "Addams Family, The 128K (1991)(Ocean).tap", 131557, 0x10ca8414, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specaddfamth, Specaddfamth, Spec128)
STD_ROM_FN(Specaddfamth)

struct BurnDriver BurnSpecaddfamth = {
	"spec_addfamth", NULL, "spec_spec128", NULL, "1991",
	"Addams Family, The (128K)\0", NULL, "Ocean", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecaddfamthRomInfo, SpecaddfamthRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Adidas Championship Football (128K)

static struct BurnRomInfo SpecadichafoRomDesc[] = {
	{ "Adidas Championship Football 128K (1990)(Ocean).tap", 106547, 0x2b13ba5a, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specadichafo, Specadichafo, Spec128)
STD_ROM_FN(Specadichafo)

struct BurnDriver BurnSpecadichafo = {
	"spec_adichafo", NULL, "spec_spec128", NULL, "1990",
	"Adidas Championship Football (128K)\0", "Set '2. Sinclair' to use controller", "Ocean", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 2, HARDWARE_SPECTRUM, GBF_SPORTSFOOTBALL, 0,
	SpectrumGetZipName, SpecadichafoRomInfo, SpecadichafoRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecIntf2DIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Advanced Lawnmower Simulator II (128K)

static struct BurnRomInfo Specadvlawn2RomDesc[] = {
	{ "Advanced Lawnmower Simulator II 128K (1990)(J. Aron).tap", 7424, 0x20f80b05, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specadvlawn2, Specadvlawn2, Spec128)
STD_ROM_FN(Specadvlawn2)

struct BurnDriver BurnSpecadvlawn2 = {
	"spec_advlawn2", NULL, "spec_spec128", NULL, "1990",
	"Advanced Lawnmower Simulator II (128K)\0", NULL, "J. Aron Software", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_ACTION, 0,
	SpectrumGetZipName, Specadvlawn2RomInfo, Specadvlawn2RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Adventures of Buratino, The (Russian) (128K)

static struct BurnRomInfo SpecburatinoRomDesc[] = {
	{ "Adventures of Buratino, The 128K (1993)(Copper Feet).z80", 0x096ec, 0x3a0cb189, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specburatino, Specburatino, Spec128)
STD_ROM_FN(Specburatino)

struct BurnDriver BurnSpecburatino = {
	"spec_buratino", NULL, "spec_spec128", NULL, "1993",
	"Adventures of Buratino, The (Russian) (128K)\0", "Choose '4' to set Kempston controller", "Copper Feet", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_ADV, 0,
	SpectrumGetZipName, SpecburatinoRomInfo, SpecburatinoRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Alien Syndrome (48K)

static struct BurnRomInfo SpecaliensynRomDesc[] = {
	{ "Alien Syndrome 48K (1988)(ACE).tap", 74240, 0xed57e69d, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specaliensyn, Specaliensyn, Spectrum)
STD_ROM_FN(Specaliensyn)

struct BurnDriver BurnSpecaliensyn = {
	"spec_aliensyn", NULL, "spec_spectrum", NULL, "1987-88",
	"Alien Syndrome (48K)\0", NULL, "ACE Software", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 2, HARDWARE_SPECTRUM, GBF_RUNGUN, 0,
	SpectrumGetZipName, SpecaliensynRomInfo, SpecaliensynRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Altered Beast (128K)

static struct BurnRomInfo SpecaltbeastRomDesc[] = {
	{ "Altered Beast 128K (1989)(Activision).tap", 191417, 0xf4ce807b, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specaltbeast, Specaltbeast, Spec128)
STD_ROM_FN(Specaltbeast)

struct BurnDriver BurnSpecaltbeast = {
	"spec_altbeast", NULL, "spec_spec128", NULL, "1989",
	"Altered Beast (128K)\0", "Set 'E:Sinclair 1' (P1) and 'I:Sinclair 2' (P2) to use controllers", "Activision", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 2, HARDWARE_SPECTRUM, GBF_SCRFIGHT, 0,
	SpectrumGetZipName, SpecaltbeastRomInfo, SpecaltbeastRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecIntf2DIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Andy Capp (128K)

static struct BurnRomInfo SpecandycappRomDesc[] = {
	{ "Andy Capp 128K (1988)(Mirrorsoft).tap", 54909, 0x7c81909f, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specandycapp, Specandycapp, Spec128)
STD_ROM_FN(Specandycapp)

struct BurnDriver BurnSpecandycapp = {
	"spec_andycapp", NULL, "spec_spec128", NULL, "1988",
	"Andy Capp (128K)\0", NULL, "Mirrorsoft", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_ADV, 0,
	SpectrumGetZipName, SpecandycappRomInfo, SpecandycappRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Angel Nieto Pole 500cc (Spanish) (128K)

static struct BurnRomInfo Specanpole128RomDesc[] = {
	{ "Angel Nieto Pole 500cc ES 128K (1990)(Opera Soft).tap", 68099, 0x010c9f97, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specanpole128, Specanpole128, Spec1282a)
STD_ROM_FN(Specanpole128)

struct BurnDriver BurnSpecanpole128 = {
	"spec_anpole128", NULL, "spec_spec1282a", NULL, "1990",
	"Angel Nieto Pole 500cc (Spanish) (128K)\0", "Press '6', '7' and ENTER to choose and set options", "Opera Soft", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 2, HARDWARE_SPECTRUM, GBF_RACING, 0,
	SpectrumGetZipName, Specanpole128RomInfo, Specanpole128RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecIntf2DIPInfo,
	Spec128KPlus2Init, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Angel Nieto Pole 500cc (Spanish) (48K)

static struct BurnRomInfo Specanpole48RomDesc[] = {
	{ "Angel Nieto Pole 500cc ES 48K (1990)(Opera Soft).tap", 68296, 0xf3e448a8, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specanpole48, Specanpole48, Spectrum)
STD_ROM_FN(Specanpole48)

struct BurnDriver BurnSpecanpole48 = {
	"spec_anpole48", "spec_anpole128", "spec_spectrum", NULL, "1990",
	"Angel Nieto Pole 500cc (Spanish) (48K)\0", "Press '6', '7' and ENTER to choose and set options", "Opera Soft", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE, 2, HARDWARE_SPECTRUM, GBF_RACING, 0,
	SpectrumGetZipName, Specanpole48RomInfo, Specanpole48RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecIntf2DIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// APB - All Points Bulletin (128K)

static struct BurnRomInfo SpecapbRomDesc[] = {
	{ "APB - All Points Bulletin 128K (1989)(Domark).tap", 75630, 0x1d6e4ca9, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specapb, Specapb, Spec128)
STD_ROM_FN(Specapb)

struct BurnDriver BurnSpecapb = {
	"spec_apb", NULL, "spec_spec128", NULL, "1989",
	"APB - All Points Bulletin (128K)\0", NULL, "Domark", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_ACTION, 0,
	SpectrumGetZipName, SpecapbRomInfo, SpecapbRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Arkanoid (48K)

static struct BurnRomInfo SpecarkanoidRomDesc[] = {
	{ "Arkanoid (1987)(Imagine Software).z80", 0x08ad3, 0x6fa4f00f, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specarkanoid, Specarkanoid, Spectrum)
STD_ROM_FN(Specarkanoid)

struct BurnDriver BurnSpecarkanoid = {
	"spec_arkanoid", NULL, "spec_spectrum", NULL, "1987",
	"Arkanoid (48K)\0", NULL, "Imagine Software", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_BREAKOUT, 0,
	SpectrumGetZipName, SpecarkanoidRomInfo, SpecarkanoidRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Arkanoid II - Revenge of Doh (48K)

static struct BurnRomInfo Specarkanoid2_48RomDesc[] = {
	{ "Arkanoid - Revenge of Doh 48K (1988)(Imagine).tap", 40118, 0xcb8aa807, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specarkanoid2_48, Specarkanoid2_48, Spectrum)
STD_ROM_FN(Specarkanoid2_48)

struct BurnDriver BurnSpecarkanoid2_48 = {
	"spec_arkanoid2_48", "spec_arkanoid2", "spec_spectrum", NULL, "1988",
	"Arkanoid II - Revenge of Doh (48K)\0", "Z: left - M: right - ENTER: fire and start", "Imagine Software", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE, 1, HARDWARE_SPECTRUM, GBF_BREAKOUT, 0,
	SpectrumGetZipName, Specarkanoid2_48RomInfo, Specarkanoid2_48RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Arkanoid II - Revenge of Doh (128K)

static struct BurnRomInfo Specarkanoid2RomDesc[] = {
	{ "Arkanoid - Revenge of Doh 128K (1988)(Imagine).tap", 43938, 0xfd20d4e8, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specarkanoid2, Specarkanoid2, Spec128)
STD_ROM_FN(Specarkanoid2)

struct BurnDriver BurnSpecarkanoid2 = {
	"spec_arkanoid2", NULL, "spec_spec128", NULL, "1988",
	"Arkanoid II - Revenge of Doh (128K)\0", "Z: left - M: right - ENTER: fire and start", "Imagine Software", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_BREAKOUT, 0,
	SpectrumGetZipName, Specarkanoid2RomInfo, Specarkanoid2RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Army Moves (128K)

static struct BurnRomInfo Specarmymvs128RomDesc[] = {
	{ "Army Moves 128K (1987)(Imagine).tap", 68998, 0xdd386d25, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specarmymvs128, Specarmymvs128, Spec128)
STD_ROM_FN(Specarmymvs128)

struct BurnDriver BurnSpecarmymvs128 = {
	"spec_armymvs128", NULL, "spec_spec128", NULL, "1986-87",
	"Army Moves (128K)\0", "Password Part 2: 27351", "Imagine & Dinamic Software", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_HORSHOOT | GBF_PLATFORM, 0,
	SpectrumGetZipName, Specarmymvs128RomInfo, Specarmymvs128RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Army Moves (48K)

static struct BurnRomInfo Specarmymvs48RomDesc[] = {
	{ "Army Moves 48K (1987)(Imagine).tap", 77664, 0x7f80f524, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specarmymvs48, Specarmymvs48, Spectrum)
STD_ROM_FN(Specarmymvs48)

struct BurnDriver BurnSpecarmymvs48 = {
	"spec_armymvs48", "spec_armymvs128", "spec_spectrum", NULL, "1986-87",
	"Army Moves (48K)\0", "Password Part 2: 27351", "Imagine & Dinamic Software", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE, 1, HARDWARE_SPECTRUM, GBF_HORSHOOT | GBF_PLATFORM, 0,
	SpectrumGetZipName, Specarmymvs48RomInfo, Specarmymvs48RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Aspar GP Master (Spanish) (48K)

static struct BurnRomInfo SpecaspargpmasRomDesc[] = {
	{ "Aspar GP Master ES (1988)(Dinamic).tap", 43743, 0x2a967d97, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specaspargpmas, Specaspargpmas, Spectrum)
STD_ROM_FN(Specaspargpmas)

struct BurnDriver BurnSpecaspargpmas = {
	"spec_aspargpmas", NULL, "spec_spectrum", NULL, "1988",
	"Aspar GP Master (Spanish) (48K)\0", "Set '3. Teclado' to use controller", "Dinamic Software", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_RACING, 0,
	SpectrumGetZipName, SpecaspargpmasRomInfo, SpecaspargpmasRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecQAOPSpaceDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Atic Atac (48K)

static struct BurnRomInfo SpecaticatacRomDesc[] = {
	{ "Atic Atac (1983)(Ultimate Play The Game).z80", 0x08994, 0x56520bdf, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specaticatac, Specaticatac, Spectrum)
STD_ROM_FN(Specaticatac)

struct BurnDriver BurnSpecaticatac = {
	"spec_aticatac", NULL, "spec_spectrum", NULL, "1983",
	"Atic Atac (48K)\0", NULL, "Ultimate Play The Game", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_ADV | GBF_MAZE, 0,
	SpectrumGetZipName, SpecaticatacRomInfo, SpecaticatacRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// ATV Simulator - All Terrain Vehicle (48K)

static struct BurnRomInfo SpecatvsimRomDesc[] = {
	{ "ATV Simulator (1987)(Codemasters).tap", 47048, 0x373a50ad, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specatvsim, Specatvsim, Spectrum)
STD_ROM_FN(Specatvsim)

struct BurnDriver BurnSpecatvsim = {
	"spec_atvsim", NULL, "spec_spectrum", NULL, "1987",
	"ATV Simulator - All Terrain Vehicle (48K)\0", NULL, "Codemasters", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 2, HARDWARE_SPECTRUM, GBF_RACING, 0,
	SpectrumGetZipName, SpecatvsimRomInfo, SpecatvsimRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Auf Wiedersehen Monty (128K)

static struct BurnRomInfo SpecaufwiemoRomDesc[] = {
	{ "Auf Wiedersehen Monty 128K (1987)(Gremlin Graphics).tap", 70498, 0x52564ec0, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specaufwiemo, Specaufwiemo, Spec128)
STD_ROM_FN(Specaufwiemo)

struct BurnDriver BurnSpecaufwiemo = {
	"spec_aufwiemo", NULL, "spec_spec128", NULL, "1987",
	"Auf Wiedersehen Monty (128K)\0", NULL, "Gremlin Graphics", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecaufwiemoRomInfo, SpecaufwiemoRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Avalon (48K)

static struct BurnRomInfo SpecavalonRomDesc[] = {
	{ "Avalon (1984)(Hewson Consultants).z80", 0x083f2, 0xe65ee95c, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specavalon, Specavalon, Spectrum)
STD_ROM_FN(Specavalon)

struct BurnDriver BurnSpecavalon = {
	"spec_avalon", NULL, "spec_spectrum", NULL, "1984",
	"Avalon (48K)\0", NULL, "Hewson Consultants", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_ADV, 0,
	SpectrumGetZipName, SpecavalonRomInfo, SpecavalonRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Avenger (128K)

static struct BurnRomInfo SpecavengerRomDesc[] = {
	{ "Avenger 128K (1986)(Gremlin Graphics).tap", 54471, 0x9d484c92, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specavenger, Specavenger, Spec128)
STD_ROM_FN(Specavenger)

struct BurnDriver BurnSpecavenger = {
	"spec_avenger", NULL, "spec_spec128", NULL, "1986",
	"Avenger (128K)\0", NULL, "Gremlin Graphics", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_RUNGUN | GBF_MAZE, 0,
	SpectrumGetZipName, SpecavengerRomInfo, SpecavengerRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Basil the Great Mouse Detective (48K)

static struct BurnRomInfo SpecbasilmouseRomDesc[] = {
	{ "Basil the Great Mouse Detective (1987)(Gremlin Graphics).tap", 49699, 0xcf9534f4, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specbasilmouse, Specbasilmouse, Spectrum)
STD_ROM_FN(Specbasilmouse)

struct BurnDriver BurnSpecbasilmouse = {
	"spec_basilmouse", NULL, "spec_spectrum", NULL, "1987",
	"Basil the Great Mouse Detective (48K)\0", NULL, "Gremlin Graphics", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecbasilmouseRomInfo, SpecbasilmouseRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Beach Buggy Simulator (48K)

static struct BurnRomInfo SpecbeabugsiRomDesc[] = {
	{ "Beach Buggy Simulator 48K (1988)(Silverbird).tap", 48064, 0x4f4f62f2, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specbeabugsi, Specbeabugsi, Spectrum)
STD_ROM_FN(Specbeabugsi)

struct BurnDriver BurnSpecbeabugsi = {
	"spec_beabugsi", NULL, "spec_spectrum", NULL, "1988",
	"Beach Buggy Simulator (48K)\0", NULL, "Silverbird Software", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 2, HARDWARE_SPECTRUM, GBF_ACTION, 0,
	SpectrumGetZipName, SpecbeabugsiRomInfo, SpecbeabugsiRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Beyond the Ice Palace (128K)

static struct BurnRomInfo SpecbeyicepaRomDesc[] = {
	{ "Beyond the Ice Palace (1988)(Elite Systems).tap", 46226, 0x565cc58c, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specbeyicepa, Specbeyicepa, Spec128)
STD_ROM_FN(Specbeyicepa)

struct BurnDriver BurnSpecbeyicepa = {
	"spec_beyicepa", NULL, "spec_spec128", NULL, "1988",
	"Beyond the Ice Palace (128K)\0", NULL, "Elite Systems", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecbeyicepaRomInfo, SpecbeyicepaRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Bionic Commando (48K)

static struct BurnRomInfo Specbionicc_48RomDesc[] = {
	{ "Bionic Commandos (1988)(Go!).tap", 97616, 0x9093dd3d, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specbionicc_48, Specbionicc_48, Spectrum)
STD_ROM_FN(Specbionicc_48)

struct BurnDriver BurnSpecbionicc_48 = {
	"spec_bionicc_48", "spec_bionicc", "spec_spectrum", NULL, "1988",
	"Bionic Commando (48K)\0", NULL, "Go!", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE, 1, HARDWARE_SPECTRUM, GBF_RUNGUN | GBF_PLATFORM, 0,
	SpectrumGetZipName, Specbionicc_48RomInfo, Specbionicc_48RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Bionic Commando (128K)

static struct BurnRomInfo SpecbioniccRomDesc[] = {
	{ "Bionic Commando (1988)(Go!)[128K].z80", 0x166f5, 0x8eb507eb, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specbionicc, Specbionicc, Spec128)
STD_ROM_FN(Specbionicc)

struct BurnDriver BurnSpecbionicc = {
	"spec_bionicc", NULL, "spec_spec128", NULL, "1988",
	"Bionic Commando (128K)\0", NULL, "Go!", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_RUNGUN | GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecbioniccRomInfo, SpecbioniccRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Black Tiger (128K)

static struct BurnRomInfo SpecblktigerRomDesc[] = {
	{ "Black Tiger (1989)(U.S. Gold).tap", 122820, 0x12912cc8, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specblktiger, Specblktiger, Spec128)
STD_ROM_FN(Specblktiger)

struct BurnDriver BurnSpecblktiger = {
	"spec_blktiger", NULL, "spec_spec128", NULL, "1989",
	"Black Tiger (128K)\0", NULL, "U.S. Gold", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_RUNGUN | GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecblktigerRomInfo, SpecblktigerRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Blinky's Scary School (128K)

static struct BurnRomInfo SpecblinkysRomDesc[] = {
	{ "Blinky's Scary School (1990)(Zeppelin Games).tap", 53665, 0x6fcf88b5, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specblinkys, Specblinkys, Spec128)
STD_ROM_FN(Specblinkys)

struct BurnDriver BurnSpecblinkys = {
	"spec_blinkys", NULL, "spec_spec128", NULL, "1990",
	"Blinky's Scary School (128K)\0", NULL, "Zeppelin Games", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_PLATFORM | GBF_ADV, 0,
	SpectrumGetZipName, SpecblinkysRomInfo, SpecblinkysRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// BMX Freestyle Simulator (48K)

static struct BurnRomInfo Specbmxfrees48RomDesc[] = {
	{ "BMX Freestyle Simulator (1989)(Codemasters).tap", 41824, 0x711d4d6a, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specbmxfrees48, Specbmxfrees48, Spectrum)
STD_ROM_FN(Specbmxfrees48)

struct BurnDriver BurnSpecbmxfrees48 = {
	"spec_bmxfrees48", "spec_bmxfrees", "spec_spectrum", NULL, "1989",
	"BMX Freestyle Simulator (48K)\0", NULL, "Codemasters", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE, 2, HARDWARE_SPECTRUM, GBF_RACING, 0,
	SpectrumGetZipName, Specbmxfrees48RomInfo, Specbmxfrees48RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// BMX Freestyle Simulator (128K)

static struct BurnRomInfo SpecbmxfreesRomDesc[] = {
	{ "BMX Freestyle Simulator (1989)(Codemasters).tap", 41824, 0x711d4d6a, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specbmxfrees, Specbmxfrees, Spec128)
STD_ROM_FN(Specbmxfrees)

struct BurnDriver BurnSpecbmxfrees = {
	"spec_bmxfrees", NULL, "spec_spec128", NULL, "1989",
	"BMX Freestyle Simulator (128K)\0", NULL, "Codemasters", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 2, HARDWARE_SPECTRUM, GBF_RACING, 0,
	SpectrumGetZipName, SpecbmxfreesRomInfo, SpecbmxfreesRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Bomb Jack (48K)

static struct BurnRomInfo Specbombjack_48RomDesc[] = {
	{ "Bomb Jack (1986)(Elite Systems).z80", 0x0a30a, 0x00e95211, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specbombjack_48, Specbombjack_48, Spectrum)
STD_ROM_FN(Specbombjack_48)

struct BurnDriver BurnSpecbombjack_48 = {
	"spec_bombjack_48", NULL, "spec_spectrum", NULL, "1986",
	"Bomb Jack (48K)\0", NULL, "Elite Systems", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 2, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, Specbombjack_48RomInfo, Specbombjack_48RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Bomb Jack II (48K)

static struct BurnRomInfo Specbmbjack2RomDesc[] = {
	{ "Bomb Jack II (1987)(Elite Systems).z80", 0x0b1b2, 0x6327f471, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specbmbjack2, Specbmbjack2, Spectrum)
STD_ROM_FN(Specbmbjack2)

struct BurnDriver BurnSpecbmbjack2 = {
	"spec_bmbjack2", NULL, "spec_spectrum", NULL, "1987",
	"Bomb Jack II (48K)\0", NULL, "Elite Systems", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 2, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, Specbmbjack2RomInfo, Specbmbjack2RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Boulder Dash (48K)

static struct BurnRomInfo SpecbouldashRomDesc[] = {
	{ "Boulder Dash (1984)(Front Runner).z80", 0x04849, 0x5d71133f, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specbouldash, Specbouldash, Spectrum)
STD_ROM_FN(Specbouldash)

struct BurnDriver BurnSpecbouldash = {
	"spec_bouldash", NULL, "spec_spectrum", NULL, "1984",
	"Boulder Dash (48K)\0", NULL, "Front Runner", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 2, HARDWARE_SPECTRUM, GBF_ACTION, 0,
	SpectrumGetZipName, SpecbouldashRomInfo, SpecbouldashRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Boulder Dash II - Rockford's Riot (48K)

static struct BurnRomInfo Specbouldsh2RomDesc[] = {
	{ "Boulder Dash II - Rockford's Riot (1985)(Prism Leisure).z80", 0x057a4, 0xbcab1101, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specbouldsh2, Specbouldsh2, Spectrum)
STD_ROM_FN(Specbouldsh2)

struct BurnDriver BurnSpecbouldsh2 = {
	"spec_bouldsh2", NULL, "spec_spectrum", NULL, "1985",
	"Boulder Dash II - Rockford's Riot (48K)\0", NULL, "Prism Leisure", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 2, HARDWARE_SPECTRUM, GBF_ACTION, 0,
	SpectrumGetZipName, Specbouldsh2RomInfo, Specbouldsh2RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Boulder Dash III (48K)

static struct BurnRomInfo Specbouldsh3RomDesc[] = {
	{ "Boulder Dash III (1986)(Prism Leisure).z80", 0x04982, 0xb61f2fae, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specbouldsh3, Specbouldsh3, Spectrum)
STD_ROM_FN(Specbouldsh3)

struct BurnDriver BurnSpecbouldsh3 = {
	"spec_bouldsh3", NULL, "spec_spectrum", NULL, "1986",
	"Boulder Dash III (48K)\0", NULL, "Prism Leisure", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 2, HARDWARE_SPECTRUM, GBF_ACTION, 0,
	SpectrumGetZipName, Specbouldsh3RomInfo, Specbouldsh3RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Boulder Dash V (48K)

static struct BurnRomInfo Specbouldsh4RomDesc[] = {
	{ "Boulder Dash V (1992)(Too Trek Moscow S.N.G.).z80", 0x08721, 0x6e9b68bf, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specbouldsh4, Specbouldsh4, Spectrum)
STD_ROM_FN(Specbouldsh4)

struct BurnDriver BurnSpecbouldsh4 = {
	"spec_bouldsh4", NULL, "spec_spectrum", NULL, "1992",
	"Boulder Dash V (48K)\0", NULL, "Too Trek Moscow S.N.G.", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 2, HARDWARE_SPECTRUM, GBF_ACTION, 0,
	SpectrumGetZipName, Specbouldsh4RomInfo, Specbouldsh4RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Boulder Dash VI (48K)

static struct BurnRomInfo Specbouldsh5RomDesc[] = {
	{ "Boulder Dash VI (1992)(Too Trek Moscow S.N.G.).z80", 0x08792, 0x6abfc981, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specbouldsh5, Specbouldsh5, Spectrum)
STD_ROM_FN(Specbouldsh5)

struct BurnDriver BurnSpecbouldsh5 = {
	"spec_bouldsh5", NULL, "spec_spectrum", NULL, "1992",
	"Boulder Dash VI (48K)\0", NULL, "Too Trek Moscow S.N.G.", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 2, HARDWARE_SPECTRUM, GBF_ACTION, 0,
	SpectrumGetZipName, Specbouldsh5RomInfo, Specbouldsh5RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Bounder (48K)

static struct BurnRomInfo SpecbounderRomDesc[] = {
	{ "Bounder 48K (1986)(Gremlin Graphics).tap", 56669, 0x9308e223, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specbounder, Specbounder, Spectrum)
STD_ROM_FN(Specbounder)

struct BurnDriver BurnSpecbounder = {
	"spec_bounder", NULL, "spec_spectrum", NULL, "1986",
	"Bounder (48K)\0", NULL, "Gremlin Graphics", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_ACTION, 0,
	SpectrumGetZipName, SpecbounderRomInfo, SpecbounderRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Bruce Lee (48K)

static struct BurnRomInfo SpecbruceleeRomDesc[] = {
	{ "Bruce Lee (1984)(U.S. Gold).z80", 0x08ceb, 0x8298df22, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specbrucelee, Specbrucelee, Spectrum)
STD_ROM_FN(Specbrucelee)

struct BurnDriver BurnSpecbrucelee = {
	"spec_brucelee", NULL, "spec_spectrum", NULL, "1984",
	"Bruce Lee (48K)\0", NULL, "U.S. Gold", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_ACTION, 0,
	SpectrumGetZipName, SpecbruceleeRomInfo, SpecbruceleeRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Boss, The (48K)

static struct BurnRomInfo SpecbosstheRomDesc[] = {
	{ "Boss, The (1984)(Peaksoft).z80", 0x09035, 0x9b7242b2, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specbossthe, Specbossthe, Spectrum)
STD_ROM_FN(Specbossthe)

struct BurnDriver BurnSpecbossthe = {
	"spec_bossthe", NULL, "spec_spectrum", NULL, "1984",
	"Boss, The (48K)\0", NULL, "Peaksoft", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_STRATEGY, 0,
	SpectrumGetZipName, SpecbosstheRomInfo, SpecbosstheRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Bubble Bobble - The Adventure (128K)

static struct BurnRomInfo SpecbubothadRomDesc[] = {
	{ "Bubble Bobble - The Adventure 128K (1993)(AP's Adventures).tap", 73681, 0x856eb771, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specbubothad, Specbubothad, Spec128)
STD_ROM_FN(Specbubothad)

struct BurnDriver BurnSpecbubothad = {
	"spec_bubothad", NULL, "spec_spec128", NULL, "1993",
	"Bubble Bobble - The Adventure (128K)\0", NULL, "AP's Adventures", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_ADV, 0,
	SpectrumGetZipName, SpecbubothadRomInfo, SpecbubothadRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Bubble Bobble (128K)

static struct BurnRomInfo SpecbublboblRomDesc[] = {
	{ "Bubble Bobble (1987)(Firebird).tap", 53224, 0xdce45529, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specbublbobl, Specbublbobl, Spec128)
STD_ROM_FN(Specbublbobl)

struct BurnDriver BurnSpecbublbobl = {
	"spec_bublbobl", NULL, "spec_spec128", NULL, "1987",
	"Bubble Bobble (128K)\0", NULL, "Firebird Software", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 2, HARDWARE_SPECTRUM, GBF_ACTION | GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecbublboblRomInfo, SpecbublboblRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Bubble Dizzy (48K)

static struct BurnRomInfo Specbubbdizz48RomDesc[] = {
	{ "Bubble Dizzy (1991)(Code Masters)(48K-128K).tap", 50433, 0x8e6625af, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specbubbdizz48, Specbubbdizz48, Spectrum)
STD_ROM_FN(Specbubbdizz48)

struct BurnDriver BurnSpecbubbdizz_48 = {
	"spec_bubbdizz48", "spec_bubbdizz128", "spec_spectrum", NULL, "1991",
	"Bubble Dizzy (48K)\0", NULL, "Codemasters", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE, 1, HARDWARE_SPECTRUM, GBF_ACTION, 0,
	SpectrumGetZipName, Specbubbdizz48RomInfo, Specbubbdizz48RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Bubble Dizzy (128K)

static struct BurnRomInfo Specbubbdizz128RomDesc[] = {
	{ "Bubble Dizzy (1991)(Code Masters)(48K-128K).tap", 50433, 0x8e6625af, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specbubbdizz128, Specbubbdizz128, Spec128)
STD_ROM_FN(Specbubbdizz128)

struct BurnDriver BurnSpecbubbdizz128 = {
	"spec_bubbdizz128", NULL, "spec_spec128", NULL, "1991",
	"Bubble Dizzy (128K)\0", NULL, "Codemasters", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_ACTION, 0,
	SpectrumGetZipName, Specbubbdizz128RomInfo, Specbubbdizz128RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Buck Rogers - Planet of Zoon (48K)

static struct BurnRomInfo SpecbuckrogersRomDesc[] = {
	{ "Buck Rogers - Planet of Zoom 48K (1985)(U.S. Gold).z80", 30201, 0xe8b65171, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specbuckrogers, Specbuckrogers, Spectrum)
STD_ROM_FN(Specbuckrogers)

struct BurnDriver BurnSpecbuckrogers = {
	"spec_buckrogers", NULL, "spec_spectrum", NULL, "1984",
	"Buck Rogers - Planet of Zoom (48K)\0", NULL, "U.S. Gold", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_SHOOT, 0,
	SpectrumGetZipName, SpecbuckrogersRomInfo, SpecbuckrogersRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Bullseye (Macsen) (48K)

static struct BurnRomInfo SpecbulleyeRomDesc[] = {
	{ "Bulls Eye (1984)(Macsen Software).z80", 0x09944, 0xce764dd1, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specbulleye, Specbulleye, Spectrum)
STD_ROM_FN(Specbulleye)

struct BurnDriver BurnSpecbulleye = {
	"spec_bulleye", NULL, "spec_spectrum", NULL, "1984",
	"Bullseye (Macsen) (48K)\0", NULL, "Macsen Software", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 2, HARDWARE_SPECTRUM, GBF_SPORTSMISC, 0,
	SpectrumGetZipName, SpecbulleyeRomInfo, SpecbulleyeRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Bullseye (Mastertronic) (48K)

static struct BurnRomInfo SpecbullseymRomDesc[] = {
	{ "Bullseye (1982)(Mastertronic)[aka Darts].z80", 0x0935f, 0x1caf68ca, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specbullseym, Specbullseym, Spectrum)
STD_ROM_FN(Specbullseym)

struct BurnDriver BurnSpecbullseym = {
	"spec_bullseym", NULL, "spec_spectrum", NULL, "1982",
	"Bullseye (Mastertronic) (48K)\0", NULL, "Mastertronic", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 2, HARDWARE_SPECTRUM, GBF_SPORTSMISC, 0,
	SpectrumGetZipName, SpecbullseymRomInfo, SpecbullseymRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Cabal (128K)

static struct BurnRomInfo SpeccabalRomDesc[] = {
	{ "Cabal 128K (1989)(Ocean).tap", 111703, 0x798f3c55, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Speccabal, Speccabal, Spec128)
STD_ROM_FN(Speccabal)

struct BurnDriver BurnSpeccabal = {
	"spec_cabal", NULL, "spec_spec128", NULL, "1988",
	"Cabal (128K)\0", NULL, "Ocean", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 2, HARDWARE_SPECTRUM, GBF_SHOOT, 0,
	SpectrumGetZipName, SpeccabalRomInfo, SpeccabalRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Camelot Warriors (48K)

static struct BurnRomInfo SpeccamelotwRomDesc[] = {
	{ "Camelot Warriors (1986)(Dinamic).tap", 41677, 0x27fdde63, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Speccamelotw, Speccamelotw, Spectrum)
STD_ROM_FN(Speccamelotw)

struct BurnDriver BurnSpeccamelotw = {
	"spec_camelotw", NULL, "spec_spectrum", NULL, "1986",
	"Camelot Warriors (48K)\0", NULL, "Dinamic Software", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpeccamelotwRomInfo, SpeccamelotwRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecQAOPMDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Capitan Trueno, El - Part 1 (Spanish) (48K)

static struct BurnRomInfo Speccaptrueno1RomDesc[] = {
	{ "Capitan Trueno, El - Part 1 ES (1989)(Dinamic).tap", 48603, 0x8e123899, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Speccaptrueno1, Speccaptrueno1, Spectrum)
STD_ROM_FN(Speccaptrueno1)

struct BurnDriver BurnSpeccaptrueno1 = {
	"spec_captrueno1", NULL, "spec_spectrum", NULL, "1989",
	"Capitan Trueno, El - Part 1 (Spanish) (48K)\0", NULL, "Dinamic Software", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_SCRFIGHT | GBF_PLATFORM, 0,
	SpectrumGetZipName, Speccaptrueno1RomInfo, Speccaptrueno1RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Capitan Trueno, El - Part 2 (Spanish) (48K)

static struct BurnRomInfo Speccaptrueno2RomDesc[] = {
	{ "Capitan Trueno, El - Part 2 ES (1989)(Dinamic).tap", 46209, 0x61b80ea8, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Speccaptrueno2, Speccaptrueno2, Spectrum)
STD_ROM_FN(Speccaptrueno2)

struct BurnDriver BurnSpeccaptrueno2 = {
	"spec_captrueno2", "spec_captrueno1", "spec_spectrum", NULL, "1989",
	"Capitan Trueno, El - Part 2 (Spanish) (48K)\0", "Password: 270653", "Dinamic Software", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE, 1, HARDWARE_SPECTRUM, GBF_SCRFIGHT | GBF_PLATFORM, 0,
	SpectrumGetZipName, Speccaptrueno2RomInfo, Speccaptrueno2RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Carlos Sainz (Spanish) (128K)

static struct BurnRomInfo SpeccarlossainzRomDesc[] = {
	{ "Carlos Sainz 128K (1990)(Zigurat).tap", 48193, 0x3974f703, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Speccarlossainz, Speccarlossainz, Spec128)
STD_ROM_FN(Speccarlossainz)

struct BurnDriver BurnSpeccarlossainz = {
	"spec_carlossainz", NULL, "spec_spec128", NULL, "1990",
	"Carlos Sainz (Spanish) (128K)\0", NULL, "Zigurat Software", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_RACING, 0,
	SpectrumGetZipName, SpeccarlossainzRomInfo, SpeccarlossainzRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Cashdash (48K)

static struct BurnRomInfo SpeccashdashRomDesc[] = {
	{ "Cash Dash (19xx)(Tynesoft).tap", 47712, 0xa278579f, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Speccashdash, Speccashdash, Spectrum)
STD_ROM_FN(Speccashdash)

struct BurnDriver BurnSpeccashdash = {
	"spec_cashdash", NULL, "spec_spectrum", NULL, "19xx",
	"Cashdash (48K)\0", NULL, "Tynesoft", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_ACTION, 0,
	SpectrumGetZipName, SpeccashdashRomInfo, SpeccashdashRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecIntf2DIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Cauldron (48K)

static struct BurnRomInfo SpeccauldronRomDesc[] = {
	{ "Cauldron (1985)(Palace).z80", 0x09f7b, 0xadea0ad1, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Speccauldron, Speccauldron, Spectrum)
STD_ROM_FN(Speccauldron)

struct BurnDriver BurnSpeccauldron = {
	"spec_cauldron", NULL, "spec_spectrum", NULL, "1985",
	"Cauldron (48K)\0", NULL, "Palace Software", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_HORSHOOT | GBF_ADV, 0,
	SpectrumGetZipName, SpeccauldronRomInfo, SpeccauldronRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Cauldron II - The Pumpkin Strikes Back (48K)

static struct BurnRomInfo Speccauldrn2RomDesc[] = {
	{ "Cauldron II (1986)(Palace).z80", 0x09349, 0xc73307f0, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Speccauldrn2, Speccauldrn2, Spectrum)
STD_ROM_FN(Speccauldrn2)

struct BurnDriver BurnSpeccauldrn2 = {
	"spec_cauldrn2", NULL, "spec_spectrum", NULL, "1986",
	"Cauldron II - The Pumpkin Strikes Back (48K)\0", NULL, "Palace Software", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_PLATFORM | GBF_ADV, 0,
	SpectrumGetZipName, Speccauldrn2RomInfo, Speccauldrn2RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Canyon Warrior (128K)

static struct BurnRomInfo SpeccanywarrRomDesc[] = {
	{ "Canyon Warrior (1989)(Mastertronic Plus).tap", 52451, 0x5574dd40, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Speccanywarr, Speccanywarr, Spec128)
STD_ROM_FN(Speccanywarr)

struct BurnDriver BurnSpeccanywarr = {
	"spec_canywarr", NULL, "spec_spec128", NULL, "1989",
	"Canyon Warrior (128K)\0", NULL, "Mastertronic Plus", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_VERSHOOT, 0,
	SpectrumGetZipName, SpeccanywarrRomInfo, SpeccanywarrRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Championship Jet Ski Simulator - Easy (48K)

static struct BurnRomInfo SpecchjesksiRomDesc[] = {
	{ "Championship Jet Ski Simulator - Easy (1989)(Codemasters).z80", 0x08e58, 0xff247fe4, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specchjesksi, Specchjesksi, Spectrum)
STD_ROM_FN(Specchjesksi)

struct BurnDriver BurnSpecchjesksi = {
	"spec_chjesksi", NULL, "spec_spectrum", NULL, "1989",
	"Championship Jet Ski Simulator - Easy (48K)\0", NULL, "Codemasters", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 2, HARDWARE_SPECTRUM, GBF_RACING, 0,
	SpectrumGetZipName, SpecchjesksiRomInfo, SpecchjesksiRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Championship Jet Ski Simulator - Hard (48K)

static struct BurnRomInfo SpecchjesksihRomDesc[] = {
	{ "Championship Jet Ski Simulator - Hard (1989)(Codemasters).z80", 0x08f27, 0xe54f4b7a, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specchjesksih, Specchjesksih, Spectrum)
STD_ROM_FN(Specchjesksih)

struct BurnDriver BurnSpecchjesksih = {
	"spec_chjesksih", "spec_chjesksi", "spec_spectrum", NULL, "1989",
	"Championship Jet Ski Simulator - Hard (48K)\0", NULL, "Codemasters", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE, 2, HARDWARE_SPECTRUM, GBF_RACING, 0,
	SpectrumGetZipName, SpecchjesksihRomInfo, SpecchjesksihRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Championship Sprint (48K)

static struct BurnRomInfo SpecchamspriRomDesc[] = {
	{ "Championship Sprint (1988)(Electric Dreams Software).z80", 0x07073, 0xf557d7f7, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specchamspri, Specchamspri, Spectrum)
STD_ROM_FN(Specchamspri)

struct BurnDriver BurnSpecchamspri = {
	"spec_chamspri", NULL, "spec_spectrum", NULL, "1988",
	"Championship Sprint (48K)\0", NULL, "Electric Dreams", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 2, HARDWARE_SPECTRUM, GBF_RACING, 0,
	SpectrumGetZipName, SpecchamspriRomInfo, SpecchamspriRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Chase H.Q. (48K)

static struct BurnRomInfo Specchasehq48RomDesc[] = {
	{ "Chase H.Q. (1989)(Ocean).tap", 118340, 0x236a9884, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specchasehq48, Specchasehq48, Spectrum)
STD_ROM_FN(Specchasehq48)

struct BurnDriver BurnSpecchasehq48 = {
	"spec_chasehq48", "spec_chasehq128", "spec_spectrum", NULL, "1989",
	"Chase H.Q. (48K)\0", NULL, "Ocean", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE, 1, HARDWARE_SPECTRUM, GBF_RACING, 0,
	SpectrumGetZipName, Specchasehq48RomInfo, Specchasehq48RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Chase H.Q. (128K)

static struct BurnRomInfo Specchasehq128RomDesc[] = {
	{ "Chase H.Q. (1989)(Ocean).tap", 118340, 0x236a9884, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specchasehq128, Specchasehq128, Spec128)
STD_ROM_FN(Specchasehq128)

struct BurnDriver BurnSpecchasehq128 = {
	"spec_chasehq128", NULL, "spec_spec128", NULL, "1989",
	"Chase H.Q. (128K)\0", NULL, "Ocean", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_RACING, 0,
	SpectrumGetZipName, Specchasehq128RomInfo, Specchasehq128RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Chase H.Q. II - Special Criminal Investigations (128K)

static struct BurnRomInfo Specchasehq2RomDesc[] = {
	{ "Chase H.Q. II - Special Criminal Investigations 128K (1990)(Ocean).tap", 122661, 0xfd47cf79, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specchasehq2, Specchasehq2, Spec128)
STD_ROM_FN(Specchasehq2)

struct BurnDriver BurnSpecchasehq2 = {
	"spec_chasehq2", NULL, "spec_spec128", NULL, "1990",
	"Chase H.Q. II - Special Criminal Investigations (128K)\0", NULL, "Ocean", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_RACING | GBF_SHOOT, 0,
	SpectrumGetZipName, Specchasehq2RomInfo, Specchasehq2RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Chequered Flag (48K)

static struct BurnRomInfo SpeccheqflagRomDesc[] = {
	{ "Chequered Flag (1982)(Sinclair Research).z80", 0x08e6f, 0xbe6e657f, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Speccheqflag, Speccheqflag, Spectrum)
STD_ROM_FN(Speccheqflag)

struct BurnDriver BurnSpeccheqflag = {
	"spec_cheqflag", NULL, "spec_spectrum", NULL, "1982",
	"Chequered Flag (48K)\0", "'A-F' to steer, '0' to accelerate, 'I' to brake and 'M-N' to shift gears", "Sinclair Research", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_RACING, 0,
	SpectrumGetZipName, SpeccheqflagRomInfo, SpeccheqflagRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Chicago 30's (Spanish) (48K)

static struct BurnRomInfo Specchicago30RomDesc[] = {
	{ "Chicago 30's ES 48K (1988)(Topo Soft).tap", 48259, 0xfbcd39d9, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specchicago30, Specchicago30, Spectrum)
STD_ROM_FN(Specchicago30)

struct BurnDriver BurnSpecchicago30 = {
	"spec_chicago30", NULL, "spec_spectrum", NULL, "1988",
	"Chicago 30's (Spanish) (48K)\0", NULL, "Topo Soft", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_RUNGUN, 0,
	SpectrumGetZipName, Specchicago30RomInfo, Specchicago30RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Chronos - A Tapestry of Time (48K)

static struct BurnRomInfo SpecchronosRomDesc[] = {
	{ "Chronos - A Tapestry of Time 48K (1987)(Mastertronic).z80", 42112, 0x9a40d9c4, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specchronos, Specchronos, Spectrum)
STD_ROM_FN(Specchronos)

struct BurnDriver BurnSpecchronos = {
	"spec_chronos", NULL, "spec_spectrum", NULL, "1987",
	"Chronos - A Tapestry of Time (48K)\0", NULL, "Mastertronic", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_HORSHOOT, 0,
	SpectrumGetZipName, SpecchronosRomInfo, SpecchronosRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Chuckie Egg (48K)

static struct BurnRomInfo SpecchuckeggRomDesc[] = {
	{ "Chuckie Egg 48K (1983)(A'n'F Software).tap", 20645, 0x15adf085, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specchuckegg, Specchuckegg, Spectrum)
STD_ROM_FN(Specchuckegg)

struct BurnDriver BurnSpecchuckegg = {
	"spec_chuckegg", NULL, "spec_spectrum", NULL, "1983",
	"Chuckie Egg (48K)\0", "Set '3' key type in Instructions 'I' menu to use controller", "A'n'F Software", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 4, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecchuckeggRomInfo, SpecchuckeggRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecQAOPMDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Chuckie Egg 2 (48K)

static struct BurnRomInfo Specchuckeg2RomDesc[] = {
	{ "Chuckie Egg 2 48K (1985)(A'n'F Software).tap", 49012, 0x7917ac74, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specchuckeg2, Specchuckeg2, Spectrum)
STD_ROM_FN(Specchuckeg2)

struct BurnDriver BurnSpecchuckeg2 = {
	"spec_chuckeg2", NULL, "spec_spectrum", NULL, "1985",
	"Chuckie Egg 2 (48K)\0", "At title press 'Space' and wait for loading time, then press 'Space' again", "A'n'F Software", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, Specchuckeg2RomInfo, Specchuckeg2RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// CJ's Elephant Antics (128K)

static struct BurnRomInfo Speccjseleph128RomDesc[] = {
	{ "CJ's Elephant Antics 128K (1991)(Codemasters).tap", 60962, 0xa44b3da6, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Speccjseleph128, Speccjseleph128, Spec128)
STD_ROM_FN(Speccjseleph128)

struct BurnDriver BurnSpeccjseleph128 = {
	"spec_cjseleph128", NULL, "spec_spec128", NULL, "1991",
	"CJ's Elephant Antics (128K)\0", "Press '3' to use controller", "Codemasters", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 2, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, Speccjseleph128RomInfo, Speccjseleph128RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// CJ In the USA (128K)

static struct BurnRomInfo SpeccjiiiusaRomDesc[] = {
	{ "CJ In the USA (1991)(Codemasters)(128k).tap", 64000, 0x455F660B, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Speccjiiiusa, Speccjiiiusa, Spec128)
STD_ROM_FN(Speccjiiiusa)

struct BurnDriver BurnSpeccjiiiusa = {
	"spec_cjiiiusa", NULL, "spec_spec128", NULL, "1991",
	"CJ In the USA (128K)\0", "In main menu: SPACE to choose, ENTER to select", "Codemasters", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 2, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpeccjiiiusaRomInfo, SpeccjiiiusaRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Coliseum (Spanish) (48K)

static struct BurnRomInfo SpeccoliseumRomDesc[] = {
	{ "Coliseum 48K ES (1988)(Topo Soft).tap", 52375, 0x50c20efc, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Speccoliseum, Speccoliseum, Spectrum)
STD_ROM_FN(Speccoliseum)

struct BurnDriver BurnSpeccoliseum = {
	"spec_coliseum", NULL, "spec_spectrum", NULL, "1988",
	"Coliseum (Spanish) (48K)\0", NULL, "Topo Soft", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_ACTION, 0,
	SpectrumGetZipName, SpeccoliseumRomInfo, SpeccoliseumRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Combat School (48K)

static struct BurnRomInfo Speccschool48RomDesc[] = {
	{ "Combat School 48K (1987)(Ocean).tap", 109676, 0x14c7b9ea, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Speccschool48, Speccschool48, Spectrum)
STD_ROM_FN(Speccschool48)

struct BurnDriver BurnSpeccschool48 = {
	"spec_cschool48", "spec_cschool128", "spec_spectrum", NULL, "1987",
	"Combat School (48K)\0", NULL, "Ocean", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE, 2, HARDWARE_SPECTRUM, GBF_RUNGUN | GBF_SPORTSMISC, 0,
	SpectrumGetZipName, Speccschool48RomInfo, Speccschool48RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Combat School (128K)

static struct BurnRomInfo Speccschool128RomDesc[] = {
	{ "Combat School 128K (1987)(Ocean).tap", 118171, 0xa45d89c2, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Speccschool128, Speccschool128, Spec128)
STD_ROM_FN(Speccschool128)

struct BurnDriver BurnSpeccschool128 = {
	"spec_cschool128", NULL, "spec_spec128", NULL, "1987",
	"Combat School (128K)\0", NULL, "Ocean", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 2, HARDWARE_SPECTRUM, GBF_RUNGUN | GBF_SPORTSMISC, 0,
	SpectrumGetZipName, Speccschool128RomInfo, Speccschool128RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Commando (French) (48K)

static struct BurnRomInfo SpeccommandofRomDesc[] = {
	{ "Commando (1984)(Loriciels)(fr).z80", 0x0a352, 0xc2b0cbfe, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Speccommandof, Speccommandof, Spectrum)
STD_ROM_FN(Speccommandof)

struct BurnDriver BurnSpeccommandof = {
	"spec_commandof", NULL, "spec_spectrum", NULL, "1984",
	"Commando (French) (48K)\0", NULL, "Loriciels", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_ADV, 0,
	SpectrumGetZipName, SpeccommandofRomInfo, SpeccommandofRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Commando (Pocket Money Software) (48K)

static struct BurnRomInfo SpeccommandopRomDesc[] = {
	{ "Commando (1984)(Pocket Money Software).z80", 0x04924, 0xeaac033a, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Speccommandop, Speccommandop, Spectrum)
STD_ROM_FN(Speccommandop)

struct BurnDriver BurnSpeccommandop = {
	"spec_commandop", NULL, "spec_spectrum", NULL, "1984",
	"Commando (Pocket Money Software) (48K)\0", NULL, "Pocket Money Software", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_ADV, 0,
	SpectrumGetZipName, SpeccommandopRomInfo, SpeccommandopRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Commando (Elite Systems) (48K)

static struct BurnRomInfo SpeccommandoRomDesc[] = {
	{ "Commando (1985)(Elite Systems).tap", 48833, 0x90a44ff9, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Speccommando, Speccommando, Spectrum)
STD_ROM_FN(Speccommando)

struct BurnDriver BurnSpeccommando = {
	"spec_commando", NULL, "spec_spectrum", NULL, "1985",
	"Commando (Elite Systems) (48K)\0", NULL, "Elite Systems", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_RUNGUN, 0,
	SpectrumGetZipName, SpeccommandoRomInfo, SpeccommandoRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Continental Circus (128K)

static struct BurnRomInfo SpeccontcircRomDesc[] = {
	{ "Continental Circus (1989)(Virgin Games).tap", 127564, 0xbe5c0729, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Speccontcirc, Speccontcirc, Spec128)
STD_ROM_FN(Speccontcirc)

struct BurnDriver BurnSpeccontcirc = {
	"spec_contcirc", NULL, "spec_spec128", NULL, "1989",
	"Continental Circus (128K)\0", NULL, "Virgin Mastertronic", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_RACING, 0,
	SpectrumGetZipName, SpeccontcircRomInfo, SpeccontcircRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Corsarios (Part 1 of 2) (128K)

static struct BurnRomInfo Speccorsarios1RomDesc[] = {
	{ "Corsarios (Part 1 of 2) (1989)(Opera Soft)(128k).z80", 0x09dae, 0xb7d7624e, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Speccorsarios1, Speccorsarios1, Spec128)
STD_ROM_FN(Speccorsarios1)

struct BurnDriver BurnSpeccorsarios1 = {
	"spec_corsarios1", NULL, "spec_spec128", NULL, "1989",
	"Corsarios (Part 1 of 2) (128K)\0", NULL, "Opera Soft", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_SCRFIGHT, 0,
	SpectrumGetZipName, Speccorsarios1RomInfo, Speccorsarios1RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecQAOPSpaceDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Corsarios (Part 2 of 2) (128K)

static struct BurnRomInfo Speccorsarios2RomDesc[] = {
	{ "Corsarios (Part 2 of 2) (1989)(Opera Soft)(128k).z80", 0x09fa3, 0xceb25c2a, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Speccorsarios2, Speccorsarios2, Spec128)
STD_ROM_FN(Speccorsarios2)

struct BurnDriver BurnSpeccorsarios2 = {
	"spec_corsarios2", "spec_corsarios1", "spec_spec128", NULL, "1989",
	"Corsarios (Part 2 of 2) (128K)\0", NULL, "Opera Soft", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE, 1, HARDWARE_SPECTRUM, GBF_SCRFIGHT, 0,
	SpectrumGetZipName, Speccorsarios2RomInfo, Speccorsarios2RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecQAOPSpaceDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Crystal Castles Diamond Plateaus in Space (48K)

static struct BurnRomInfo SpeccryscastRomDesc[] = {
	{ "Crystal Castles Diamond Plateaus in Space (1986)(U.S. Gold).tap", 46669, 0xf6e3d62a, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Speccryscast, Speccryscast, Spectrum)
STD_ROM_FN(Speccryscast)

struct BurnDriver BurnSpeccryscast = {
	"spec_cryscast", NULL, "spec_spectrum", NULL, "1986",
	"Crystal Castles Diamond Plateaus in Space (48K)\0", NULL, "U.S. Gold", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_ACTION, 0,
	SpectrumGetZipName, SpeccryscastRomInfo, SpeccryscastRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Cybernoid - The Fighting Machine (48K)

static struct BurnRomInfo SpecCybernoid48RomDesc[] = {
	{ "Cybernoid - The Fighting Machine (1988)(Hewson Consultants).tap", 46731, 0x965806fc, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecCybernoid48, SpecCybernoid48, Spectrum)
STD_ROM_FN(SpecCybernoid48)

struct BurnDriver BurnSpecCybernoid48 = {
	"spec_cybernoid48", "spec_cybernoid128", "spec_spectrum", NULL, "1988",
	"Cybernoid - The Fighting Machine (48K)\0", NULL, "Hewson Consultants", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE, 1, HARDWARE_SPECTRUM, GBF_SHOOT, 0,
	SpectrumGetZipName, SpecCybernoid48RomInfo, SpecCybernoid48RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Cybernoid - The Fighting Machine (128K)

static struct BurnRomInfo SpecCybernoid128RomDesc[] = {
	{ "Cybernoid - The Fighting Machine (1988)(Hewson Consultants).tap", 46731, 0x965806fc, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecCybernoid128, SpecCybernoid128, Spec128)
STD_ROM_FN(SpecCybernoid128)

struct BurnDriver BurnSpecCybernoid128 = {
	"spec_cybernoid128", NULL, "spec_spec128", NULL, "1988",
	"Cybernoid - The Fighting Machine (128K)\0", NULL, "Hewson Consultants", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_SHOOT, 0,
	SpectrumGetZipName, SpecCybernoid128RomInfo, SpecCybernoid128RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Cybernoid II - The Revenge (48K)

static struct BurnRomInfo SpecCybernoidii48RomDesc[] = {
	{ "Cybernoid II - The Revenge 48K (1988)(Hewson Consultants).tap", 48032, 0xa66dd375, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecCybernoidii48, SpecCybernoidii48, Spectrum)
STD_ROM_FN(SpecCybernoidii48)

struct BurnDriver BurnSpecCybernoidii48 = {
	"spec_cybernoidii48", "spec_cybernoidii128", "spec_spectrum", NULL, "1988",
	"Cybernoid II - The Revenge (48K)\0", NULL, "Hewson Consultants", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE, 1, HARDWARE_SPECTRUM, GBF_SHOOT, 0,
	SpectrumGetZipName, SpecCybernoidii48RomInfo, SpecCybernoidii48RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Cybernoid II - The Revenge (128K)

static struct BurnRomInfo SpecCybernoidii128RomDesc[] = {
	{ "Cybernoid II - The Revenge 128K (1988)(Hewson Consultants).tap", 48033, 0x5d933be6, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecCybernoidii128, SpecCybernoidii128, Spec128)
STD_ROM_FN(SpecCybernoidii128)

struct BurnDriver BurnSpecCybernoidii128 = {
	"spec_cybernoidii128", NULL, "spec_spec128", NULL, "1988",
	"Cybernoid II - The Revenge (128K)\0", NULL, "Hewson Consultants", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_SHOOT, 0,
	SpectrumGetZipName, SpecCybernoidii128RomInfo, SpecCybernoidii128RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Daley Thompson's Decathlon - Day 1 (48K)

static struct BurnRomInfo SpecdtdecthnRomDesc[] = {
	{ "Daley Thompson's Decathlon - Day 1 (1984)(Ocean).z80", 0x08bbe, 0xf31094d1, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specdtdecthn, Specdtdecthn, Spectrum)
STD_ROM_FN(Specdtdecthn)

struct BurnDriver BurnSpecdtdecthn = {
	"spec_dtdecthn", NULL, "spec_spectrum", NULL, "1984",
	"Daley Thompson's Decathlon - Day 1 (48K)\0", NULL, "Ocean", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_SPORTSMISC, 0,
	SpectrumGetZipName, SpecdtdecthnRomInfo, SpecdtdecthnRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Daley Thompson's Decathlon - Day 2 (48K)

static struct BurnRomInfo Specdtdecthn2RomDesc[] = {
	{ "Daley Thompson's Decathlon - Day 2 (1984)(Ocean).z80", 0x08a49, 0x500ca1a5, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specdtdecthn2, Specdtdecthn2, Spectrum)
STD_ROM_FN(Specdtdecthn2)

struct BurnDriver BurnSpecdtdecthn2 = {
	"spec_dtdecthn2", "spec_dtdecthn", "spec_spectrum", NULL, "1984",
	"Daley Thompson's Decathlon - Day 2 (48K)\0", NULL, "Ocean", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE, 1, HARDWARE_SPECTRUM, GBF_SPORTSMISC, 0,
	SpectrumGetZipName, Specdtdecthn2RomInfo, Specdtdecthn2RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Daley Thompson's Olympic Challenge (128K)

static struct BurnRomInfo SpecdatholchRomDesc[] = {
	{ "Daley Thompson's Olympic Challenge (1988)(Ocean)[128K].z80", 0x1d5dc, 0xfc2d513f, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specdatholch, Specdatholch, Spec128)
STD_ROM_FN(Specdatholch)

struct BurnDriver BurnSpecdatholch = {
	"spec_datholch", NULL, "spec_spec128", NULL, "1988",
	"Daley Thompson's Olympic Challenge (128K)\0", NULL, "Ocean", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_SPORTSMISC, 0,
	SpectrumGetZipName, SpecdatholchRomInfo, SpecdatholchRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Daley Thompson's Super-Test (48K)

static struct BurnRomInfo SpecDtsupertest48RomDesc[] = {
	{ "Daley Thompson's Super-Test 48K (1985)(Ocean).tap", 49345, 0x3c3d0d1e, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecDtsupertest48, SpecDtsupertest48, Spectrum)
STD_ROM_FN(SpecDtsupertest48)

struct BurnDriver BurnSpecDtsupertest48 = {
	"spec_dtsupertest48", "spec_dtsupertest128", "spec_spectrum", NULL, "1985",
	"Daley Thompson's Super-Test (48K)\0", NULL, "Ocean", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE, 1, HARDWARE_SPECTRUM, GBF_SPORTSMISC, 0,
	SpectrumGetZipName, SpecDtsupertest48RomInfo, SpecDtsupertest48RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Daley Thompson's Super-Test (128K)

static struct BurnRomInfo SpecDtsupertest128RomDesc[] = {
	{ "Daley Thompson's Super-Test 128K (1985)(Ocean).tap", 128381, 0xe8ab5d26, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecDtsupertest128, SpecDtsupertest128, Spec128)
STD_ROM_FN(SpecDtsupertest128)

struct BurnDriver BurnSpecDtsupertest128 = {
	"spec_dtsupertest128", NULL, "spec_spec128", NULL, "1985",
	"Daley Thompson's Super-Test (128K)\0", NULL, "Ocean", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_SPORTSMISC, 0,
	SpectrumGetZipName, SpecDtsupertest128RomInfo, SpecDtsupertest128RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Dan Dare - Pilot of the Future (48K)

static struct BurnRomInfo SpecdandareRomDesc[] = {
	{ "Dan Dare - Pilot of the Future (1986)(Virgin Games).z80", 42412, 0x9378e2c3, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specdandare, Specdandare, Spectrum)
STD_ROM_FN(Specdandare)

struct BurnDriver BurnSpecdandare = {
	"spec_dandare", NULL, "spec_spectrum", NULL, "1986",
	"Dan Dare - Pilot of the Future (48K)\0", NULL, "Virgin Games", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_RUNGUN | GBF_ADV, 0,
	SpectrumGetZipName, SpecdandareRomInfo, SpecdandareRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Dan Dare II - Mekon's Revenge (48K)

static struct BurnRomInfo Specdandare2RomDesc[] = {
	{ "Dan Dare II - Mekon's Revenge (1988)(Virgin Games).z80", 44497, 0x03159016, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specdandare2, Specdandare2, Spectrum)
STD_ROM_FN(Specdandare2)

struct BurnDriver BurnSpecdandare2 = {
	"spec_dandare2", NULL, "spec_spectrum", NULL, "1988",
	"Dan Dare II - Mekon's Revenge (48K)\0", NULL, "Virgin Games", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_SHOOT | GBF_ADV, 0,
	SpectrumGetZipName, Specdandare2RomInfo, Specdandare2RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Dan Dare III - The Escape (48K)

static struct BurnRomInfo Specdandare3RomDesc[] = {
	{ "Dan Dare III - The Escape (1990)(Virgin Games).z80", 42660, 0xad0767cb, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specdandare3, Specdandare3, Spectrum)
STD_ROM_FN(Specdandare3)

struct BurnDriver BurnSpecdandare3 = {
	"spec_dandare3", NULL, "spec_spectrum", NULL, "1990",
	"Dan Dare III - The Escape (48K)\0", NULL, "Virgin Games", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_RUNGUN | GBF_ADV, 0,
	SpectrumGetZipName, Specdandare3RomInfo, Specdandare3RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Darius+ (128K)

static struct BurnRomInfo SpecdariusRomDesc[] = {
	{ "Darius+ (1990)(The Edge Software).tap", 0x35789, 0xe47f46f1, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specdarius, Specdarius, Spec128)
STD_ROM_FN(Specdarius)

struct BurnDriver BurnSpecdarius = {
	"spec_darius", NULL, "spec_spec128", NULL, "1990",
	"Darius+ (128K)\0", NULL, "The Edge", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_HORSHOOT, 0,
	SpectrumGetZipName, SpecdariusRomInfo, SpecdariusRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Darkman (128K)

static struct BurnRomInfo SpecdarkmanRomDesc[] = {
	{ "Darkman 128K (1991)(Ocean).tap", 181471, 0x697be425, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specdarkman, Specdarkman, Spec128)
STD_ROM_FN(Specdarkman)

struct BurnDriver BurnSpecdarkman = {
	"spec_darkman", NULL, "spec_spec128", NULL, "1991",
	"Darkman (128K)\0", NULL, "Ocean", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_SCRFIGHT | GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecdarkmanRomInfo, SpecdarkmanRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Deathchase (48K)

static struct BurnRomInfo SpecdeathchaRomDesc[] = {
	{ "Deathchase (1983)(Micromega).z80", 0x02bed, 0x040769a4, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specdeathcha, Specdeathcha, Spectrum)
STD_ROM_FN(Specdeathcha)

struct BurnDriver BurnSpecdeathcha = {
	"spec_deathcha", NULL, "spec_spectrum", NULL, "1983",
	"Deathchase (48K)\0", NULL, "Micromega", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_SHOOT, 0,
	SpectrumGetZipName, SpecdeathchaRomInfo, SpecdeathchaRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Dizzy - Down the Rapids (128K)

static struct BurnRomInfo SpecdizdowraRomDesc[] = {
	{ "Dizzy - Down the Rapids (1991)(Codemasters)[128K].z80", 0x0a17e, 0x426abaa3, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specdizdowra, Specdizdowra, Spec128)
STD_ROM_FN(Specdizdowra)

struct BurnDriver BurnSpecdizdowra = {
	"spec_dizdowra", NULL, "spec_spec128", NULL, "1991",
	"Dizzy - Down the Rapids (128K)\0", NULL, "Codemasters", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 2, HARDWARE_SPECTRUM, GBF_ACTION, 0,
	SpectrumGetZipName, SpecdizdowraRomInfo, SpecdizdowraRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Dizzy - The Ultimate Cartoon Adventure (Russian) (48K) (Hack)

static struct BurnRomInfo Specdizzy1ruRomDesc[] = {
	{ "Dizzy - The Ultimate Cartoon Adventure (1987)(Codemasters)[ru].tap", 48813, 0x85067eff, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specdizzy1ru, Specdizzy1ru, Spectrum)
STD_ROM_FN(Specdizzy1ru)

struct BurnDriver BurnSpecdizzy1ru = {
	"spec_dizzy1ru", "spec_dizzy1", "spec_spectrum", NULL, "199?",
	"Dizzy - The Ultimate Cartoon Adventure (Russian) (48K) (Hack)\0", NULL, "Codemasters", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE | BDF_HACK, 1, HARDWARE_SPECTRUM, GBF_PLATFORM | GBF_ADV, 0,
	SpectrumGetZipName, Specdizzy1ruRomInfo, Specdizzy1ruRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Dizzy - The Ultimate Cartoon Adventure (48K)

static struct BurnRomInfo Specdizzy1RomDesc[] = {
	{ "Dizzy - The Ultimate Cartoon Adventure (1987)(Codemasters).tap", 48901, 0xda1ce6bf, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specdizzy1, Specdizzy1, Spectrum)
STD_ROM_FN(Specdizzy1)

struct BurnDriver BurnSpecdizzy1 = {
	"spec_dizzy1", NULL, "spec_spectrum", NULL, "1987",
	"Dizzy - The Ultimate Cartoon Adventure (48K)\0", NULL, "Codemasters", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_PLATFORM | GBF_ADV, 0,
	SpectrumGetZipName, Specdizzy1RomInfo, Specdizzy1RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Dizzy - The Ultimate Cartoon Adventure - Crash Special Edition (48K)

static struct BurnRomInfo Specdizzy1cseRomDesc[] = {
	{ "Dizzy - The Ultimate Cartoon Adventure - Crash Special Edition (1987)(Codemasters).tap", 48581, 0x83d585be, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specdizzy1cse, Specdizzy1cse, Spectrum)
STD_ROM_FN(Specdizzy1cse)

struct BurnDriver BurnSpecdizzy1cse = {
	"spec_dizzy1cse", "spec_dizzy1", "spec_spectrum", NULL, "1987",
	"Dizzy - The Ultimate Cartoon Adventure - Crash Special Edition (48K)\0", "Z: left - X: right - SPACE: jump - ENTER: action", "Codemasters", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE | BDF_HACK, 1, HARDWARE_SPECTRUM, GBF_PLATFORM | GBF_ADV, 0,
	SpectrumGetZipName, Specdizzy1cseRomInfo, Specdizzy1cseRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Dizzy II - Treasure Island Dizzy (Russian) (128K) (Hack)

static struct BurnRomInfo Specdizzy2ruRomDesc[] = {
	{ "Dizzy 2 - Treasure Island Dizzy 128K (1988)(Codemasters)[ru].z80", 57508, 0xccc7f01b, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specdizzy2ru, Specdizzy2ru, Spec128)
STD_ROM_FN(Specdizzy2ru)

struct BurnDriver BurnSpecdizzy2ru = {
	"spec_dizzy2ru", "spec_dizzy2ay", "spec_spec128", NULL, "199?",
	"Dizzy II - Treasure Island Dizzy (Russian) (128K) (Hack)\0", NULL, "unknown", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE | BDF_HACK, 1, HARDWARE_SPECTRUM, GBF_PLATFORM | GBF_ADV, 0,
	SpectrumGetZipName, Specdizzy2ruRomInfo, Specdizzy2ruRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Dizzy II - Treasure Island Dizzy (48K)

static struct BurnRomInfo Specdizzy2RomDesc[] = {
	{ "Dizzy 2 - Treasure Island Dizzy 48K (1988)(Codemasters).tap", 44195, 0x3c680bbd, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specdizzy2, Specdizzy2, Spectrum)
STD_ROM_FN(Specdizzy2)

struct BurnDriver BurnSpecdizzy2 = {
	"spec_dizzy2", "spec_dizzy2ay", "spec_spectrum", NULL, "1988",
	"Dizzy II - Treasure Island Dizzy (48K)\0", NULL, "Codemasters", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE, 1, HARDWARE_SPECTRUM, GBF_PLATFORM | GBF_ADV, 0,
	SpectrumGetZipName, Specdizzy2RomInfo, Specdizzy2RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Dizzy II - Treasure Island Dizzy (128K)

static struct BurnRomInfo Specdizzy2ayRomDesc[] = {
	{ "Dizzy 2 - Treasure Island Dizzy 128K (1988)(Codemasters).tap", 57678, 0x0fd65ba2, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specdizzy2ay, Specdizzy2ay, Spec128)
STD_ROM_FN(Specdizzy2ay)

struct BurnDriver BurnSpecdizzy2ay = {
	"spec_dizzy2ay", NULL, "spec_spec128", NULL, "1988",
	"Dizzy II - Treasure Island Dizzy (128K)\0", NULL, "Codemasters", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_PLATFORM | GBF_ADV, 0,
	SpectrumGetZipName, Specdizzy2ayRomInfo, Specdizzy2ayRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Dizzy III - Fantasy World Dizzy (Russian) (128K) (Hack)

static struct BurnRomInfo Specdizzy3ruRomDesc[] = {
	{ "Dizzy 3 - Fantasy World Dizzy (1989)(Codemasters)[ru].tap", 64532, 0x2a83e0d8, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specdizzy3ru, Specdizzy3ru, Spec128)
STD_ROM_FN(Specdizzy3ru)

struct BurnDriver BurnSpecdizzy3ru = {
	"spec_dizzy3ru", "spec_dizzy3", "spec_spec128", NULL, "199?",
	"Dizzy III - Fantasy World Dizzy (Russian) (128K) (Hack)\0", NULL, "unknown", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE | BDF_HACK, 1, HARDWARE_SPECTRUM, GBF_PLATFORM | GBF_ADV, 0,
	SpectrumGetZipName, Specdizzy3ruRomInfo, Specdizzy3ruRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Dizzy III - Fantasy World Dizzy (128K)

static struct BurnRomInfo Specdizzy3RomDesc[] = {
	{ "Dizzy 3 - Fantasy World Dizzy (1989)(Codemasters).tap", 62876, 0x2080a324, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specdizzy3, Specdizzy3, Spec128)
STD_ROM_FN(Specdizzy3)

struct BurnDriver BurnSpecdizzy3 = {
	"spec_dizzy3", NULL, "spec_spec128", NULL, "1989",
	"Dizzy III - Fantasy World Dizzy (128K)\0", NULL, "Codemasters", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_PLATFORM | GBF_ADV, 0,
	SpectrumGetZipName, Specdizzy3RomInfo, Specdizzy3RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Dizzy IV - Magicland Dizzy (48K)

static struct BurnRomInfo Specdizzy4_48RomDesc[] = {
	{ "Dizzy 4 - Magicland Dizzy 48K (1990)(Codemasters).tap", 56260, 0xbd5a1fca, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specdizzy4_48, Specdizzy4_48, Spectrum)
STD_ROM_FN(Specdizzy4_48)

struct BurnDriver BurnSpecdizzy4_48 = {
	"spec_dizzy4_48", "spec_dizzy4", "spec_spectrum", NULL, "1989",
	"Dizzy IV - Magicland Dizzy (48K)\0", NULL, "Codemasters", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE, 1, HARDWARE_SPECTRUM, GBF_PLATFORM | GBF_ADV, 0,
	SpectrumGetZipName, Specdizzy4_48RomInfo, Specdizzy4_48RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Dizzy IV - Magicland Dizzy (128K)

static struct BurnRomInfo Specdizzy4RomDesc[] = {
	{ "Dizzy 4 - Magicland Dizzy 128K (1990)(Codemasters).tap", 64959, 0x52c74596, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specdizzy4, Specdizzy4, Spec128)
STD_ROM_FN(Specdizzy4)

struct BurnDriver BurnSpecdizzy4 = {
	"spec_dizzy4", NULL, "spec_spec128", NULL, "1990",
	"Dizzy IV - Magicland Dizzy (128K)\0", NULL, "Codemasters", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_PLATFORM | GBF_ADV, 0,
	SpectrumGetZipName, Specdizzy4RomInfo, Specdizzy4RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Dizzy IV - Magicland Dizzy (Russian) (128K) (Hack)

static struct BurnRomInfo Specdizzy4ruRomDesc[] = {
	{ "Dizzy 4 - Magicland Dizzy 128K (1990)(Codemasters)[ru].tap", 58305, 0x328a1777, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specdizzy4ru, Specdizzy4ru, Spec128)
STD_ROM_FN(Specdizzy4ru)

struct BurnDriver BurnSpecdizzy4ru = {
	"spec_dizzy4ru", "spec_dizzy4", "spec_spec128", NULL, "199?",
	"Dizzy IV - Magicland Dizzy (Russian) (128K) (Hack)\0", NULL, "Codemasters", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE | BDF_HACK, 1, HARDWARE_SPECTRUM, GBF_PLATFORM | GBF_ADV, 0,
	SpectrumGetZipName, Specdizzy4ruRomInfo, Specdizzy4ruRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Dizzy V - Spellbound Dizzy (128K)

static struct BurnRomInfo Specdizzy5RomDesc[] = {
	{ "Dizzy 5 - Spellbound Dizzy 128K (1991)(Codemasters).tap", 96808, 0xb759620a, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specdizzy5, Specdizzy5, Spec128)
STD_ROM_FN(Specdizzy5)

struct BurnDriver BurnSpecdizzy5 = {
	"spec_dizzy5", NULL, "spec_spec128", NULL, "1991",
	"Dizzy V - Spellbound Dizzy (128K)\0", NULL, "Codemasters", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_PLATFORM | GBF_ADV, 0,
	SpectrumGetZipName, Specdizzy5RomInfo, Specdizzy5RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Dizzy VI - Prince of the Yolkfolk (Russian) (48K) (Hack)

static struct BurnRomInfo Specdizzy6ruRomDesc[] = {
	{ "Dizzy 6 - Dizzy, Prince of the YolkFolk (1991)(Codemasters)[ru].tap", 41101, 0xc1462db4, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specdizzy6ru, Specdizzy6ru, Spectrum)
STD_ROM_FN(Specdizzy6ru)

struct BurnDriver BurnSpecdizzy6ru = {
	"spec_dizzy6ru", "spec_dizzy6", "spec_spectrum", NULL, "199?",
	"Dizzy VI - Prince of the Yolkfolk (Russian) (48K) (Hack)\0", NULL, "Codemasters", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE | BDF_HACK, 1, HARDWARE_SPECTRUM, GBF_PLATFORM | GBF_ADV, 0,
	SpectrumGetZipName, Specdizzy6ruRomInfo, Specdizzy6ruRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Dizzy VI - Prince of the Yolkfolk (128K)

static struct BurnRomInfo Specdizzy6RomDesc[] = {
	{ "Dizzy 6 - Dizzy, Prince of the YolkFolk 128K (1991)(Codemasters).tap", 47088, 0xc46222fc, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specdizzy6, Specdizzy6, Spec128)
STD_ROM_FN(Specdizzy6)

struct BurnDriver BurnSpecdizzy6 = {
	"spec_dizzy6", NULL, "spec_spec128", NULL, "1991",
	"Dizzy VI - Prince of the Yolkfolk (128K)\0", NULL, "Codemasters", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_PLATFORM | GBF_ADV, 0,
	SpectrumGetZipName, Specdizzy6RomInfo, Specdizzy6RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Dizzy VII - Crystal Kingdom Dizzy (128K)

static struct BurnRomInfo Specdizzy7RomDesc[] = {
	{ "Dizzy 7 - Crystal Kingdom Dizzy 128K (1992)(Codemasters).z80", 47389, 0x16fb82f0, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specdizzy7, Specdizzy7, Spec128)
STD_ROM_FN(Specdizzy7)

struct BurnDriver BurnSpecdizzy7 = {
	"spec_dizzy7", NULL, "spec_spec128", NULL, "1992",
	"Dizzy VII - Crystal Kingdom Dizzy (128K)\0", NULL, "Codemasters", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_PLATFORM | GBF_ADV, 0,
	SpectrumGetZipName, Specdizzy7RomInfo, Specdizzy7RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Dizzy VII - Crystal Kingdom Dizzy (Russian) (128K) (Hack)

static struct BurnRomInfo Specdizzy7ruRomDesc[] = {
	{ "Dizzy 7 - Crystal Kingdom Dizzy 128K (1992)(Codemasters)[ru].z80", 61746, 0xd6b0801d, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specdizzy7ru, Specdizzy7ru, Spec128)
STD_ROM_FN(Specdizzy7ru)

struct BurnDriver BurnSpecdizzy7ru = {
	"spec_dizzy7ru", "spec_dizzy7", "spec_spec128", NULL, "1993",
	"Dizzy VII - Crystal Kingdom Dizzy (Russian) (128K) (Hack)\0", NULL, "Codemasters", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE | BDF_HACK, 1, HARDWARE_SPECTRUM, GBF_PLATFORM | GBF_ADV, 0,
	SpectrumGetZipName, Specdizzy7ruRomInfo, Specdizzy7ruRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Donkey Kong (48K)

static struct BurnRomInfo SpecdkongRomDesc[] = {
	{ "Donkey Kong (1986)(Ocean).tap", 57131, 0xf54f6058, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specdkong, Specdkong, Spectrum)
STD_ROM_FN(Specdkong)

struct BurnDriver BurnSpecdkong = {
	"spec_dkong", NULL, "spec_spectrum", NULL, "1986",
	"Donkey Kong (48K)\0", NULL, "Ocean", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 2, HARDWARE_SPECTRUM, GBF_ACTION, 0,
	SpectrumGetZipName, SpecdkongRomInfo, SpecdkongRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Donkey Kong Jr. (48K)

static struct BurnRomInfo SpecdkongjrRomDesc[] = {
	{ "Donkey Kong Jr. (19xx)(Sir Clive and Mr ZX).z80", 0x02a1d, 0x91569bef, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specdkongjr, Specdkongjr, Spectrum)
STD_ROM_FN(Specdkongjr)

struct BurnDriver BurnSpecdkongjr = {
	"spec_dkongjr", NULL, "spec_spectrum", NULL, "19xx",
	"Donkey Kong Jr. (48K)\0", NULL, "Sir Clive and Mr ZX", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_ACTION, 0,
	SpectrumGetZipName, SpecdkongjrRomInfo, SpecdkongjrRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Doom (Demo) (128K)

static struct BurnRomInfo SpecdoomdemoRomDesc[] = {
	{ "Doom (demo) (1996)(Digital Reality)[128K].z80", 0x11d9a, 0xb310b6f1, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specdoomdemo, Specdoomdemo, Spec128)
STD_ROM_FN(Specdoomdemo)

struct BurnDriver BurnSpecdoomdemo = {
	"spec_doomdemo", NULL, "spec_spec128", NULL, "1996",
	"Doom (Demo) (128K)\0", NULL, "Digital Reality", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_SHOOT, 0,
	SpectrumGetZipName, SpecdoomdemoRomInfo, SpecdoomdemoRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Double Dragon III - The Rosetta Stone (128K)

static struct BurnRomInfo Specddragon3RomDesc[] = {
	{ "Double Dragon III - The Rosetta Stone 128K (1991)(Storm).tap", 312619, 0x09d741f5, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specddragon3, Specddragon3, Spec128)
STD_ROM_FN(Specddragon3)

struct BurnDriver BurnSpecddragon3 = {
	"spec_ddragon3", NULL, "spec_spec128", NULL, "1991",
	"Double Dragon III - The Rosetta Stone (128K)\0", NULL, "Storm Software", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 2, HARDWARE_SPECTRUM, GBF_SCRFIGHT, 0,
	SpectrumGetZipName, Specddragon3RomInfo, Specddragon3RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecIntf2DIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Draconus (48K)

static struct BurnRomInfo SpecDraconusRomDesc[] = {
	{ "Draconus 48K (1988)(Zeppelin Games).tap", 39793, 0x6aeb0e66, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecDraconus, SpecDraconus, Spectrum)
STD_ROM_FN(SpecDraconus)

struct BurnDriver BurnSpecDraconus = {
	"spec_draconus", NULL, "spec_spectrum", NULL, "1988",
	"Draconus (48K)\0", "Press ENTER for game menu", "Zeppelin Games", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecDraconusRomInfo, SpecDraconusRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Dragon Breed (48K)

static struct BurnRomInfo SpecdrgbreedRomDesc[] = {
	{ "Dragon Breed (1990)(Activision).tap", 126452, 0xc7158fdf, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specdrgbreed, Specdrgbreed, Spectrum)
STD_ROM_FN(Specdrgbreed)

struct BurnDriver BurnSpecdrgbreed = {
	"spec_drgbreed", NULL, "spec_spectrum", NULL, "1989",
	"Dragon Breed (48K)\0", NULL, "Activision", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_HORSHOOT, 0,
	SpectrumGetZipName, SpecdrgbreedRomInfo, SpecdrgbreedRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Dragon Ninja (48K)

static struct BurnRomInfo Specdninja48RomDesc[] = {
	{ "Dragon Ninja 48K (1988)(Imagine).tap", 95634, 0x5ba91604, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specdninja48, Specdninja48, Spectrum)
STD_ROM_FN(Specdninja48)

struct BurnDriver BurnSpecdninja48 = {
	"spec_dninja48", "spec_dninja128", "spec_spectrum", NULL, "1988",
	"Dragon Ninja (48K)\0", NULL, "Imagine Software", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE, 1, HARDWARE_SPECTRUM, GBF_SCRFIGHT, 0,
	SpectrumGetZipName, Specdninja48RomInfo, Specdninja48RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Dragon Ninja (128K)

static struct BurnRomInfo Specdninja128RomDesc[] = {
	{ "Dragon Ninja 128K (1988)(Imagine).tap", 118559, 0x636223a9, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specdninja128, Specdninja128, Spec128)
STD_ROM_FN(Specdninja128)

struct BurnDriver BurnSpecdninja128 = {
	"spec_dninja128", NULL, "spec_spec128", NULL, "1988",
	"Dragon Ninja (128K)\0", NULL, "Imagine Software", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_SCRFIGHT, 0,
	SpectrumGetZipName, Specdninja128RomInfo, Specdninja128RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Dragontorc (48K)

static struct BurnRomInfo SpecdragontoRomDesc[] = {
	{ "Dragontorc (1985)(Hewson Consultants).z80", 0x09d26, 0xc2e5c32e, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specdragonto, Specdragonto, Spectrum)
STD_ROM_FN(Specdragonto)

struct BurnDriver BurnSpecdragonto = {
	"spec_dragonto", NULL, "spec_spectrum", NULL, "1985",
	"Dragontorc (48K)\0", "At title-screen press 'S' to start game", "Hewson Consultants", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_ADV, 0,
	SpectrumGetZipName, SpecdragontoRomInfo, SpecdragontoRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Driller (48K)

static struct BurnRomInfo SpecdrillerRomDesc[] = {
	{ "Driller (1987)(Incentive).tap", 55483, 0x0387f1c0, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specdriller, Specdriller, Spectrum)
STD_ROM_FN(Specdriller)

struct BurnDriver BurnSpecdriller = {
	"spec_driller", NULL, "spec_spectrum", NULL, "1987",
	"Driller (48K)\0", NULL, "Incentive", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_SHOOT | GBF_ADV, 0,
	SpectrumGetZipName, SpecdrillerRomInfo, SpecdrillerRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Druid (48K)

static struct BurnRomInfo SpecDruidRomDesc[] = {
	{ "Druid (1986)(Firebird).tap", 45791, 0x19fb94a7, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecDruid, SpecDruid, Spectrum)
STD_ROM_FN(SpecDruid)

struct BurnDriver BurnSpecDruid = {
	"spec_druid", NULL, "spec_spectrum", NULL, "1986",
	"Druid (48K)\0", "Press '2' to set Kempston Joystick", "Firebird Software", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_RUNGUN | GBF_MAZE, 0,
	SpectrumGetZipName, SpecDruidRomInfo, SpecDruidRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Druid II - Enlightenment (48K)

static struct BurnRomInfo SpecDruid2RomDesc[] = {
	{ "Druid II - Enlightenment (1987)(Firebird).tap", 58884, 0x8a5474aa, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecDruid2, SpecDruid2, Spectrum)
STD_ROM_FN(SpecDruid2)

struct BurnDriver BurnSpecDruid2 = {
	"spec_druid2", NULL, "spec_spectrum", NULL, "1987",
	"Druid II - Enlightenment (48K)\0", "Press 'J' to set Kempston Joystick", "Firebird Software", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_RUNGUN | GBF_MAZE, 0,
	SpectrumGetZipName, SpecDruid2RomInfo, SpecDruid2RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Dynamite Dan (48K)

static struct BurnRomInfo SpecdynadanRomDesc[] = {
	{ "Dynamite Dan (1985)(Mirrorsoft).z80", 0x0a6a8, 0x218460b1, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specdynadan, Specdynadan, Spectrum)
STD_ROM_FN(Specdynadan)

struct BurnDriver BurnSpecdynadan = {
	"spec_dynadan", NULL, "spec_spectrum", NULL, "1985",
	"Dynamite Dan (48K)\0", NULL, "Mirrorsoft", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecdynadanRomInfo, SpecdynadanRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Dynamite Dan II - Dr. Blitzen and the Islands of Arcanum (48K)

static struct BurnRomInfo Specdynadan2RomDesc[] = {
	{ "Dynamite Dan II - Dr Blitzen And The Islands Of Arcanum (1985)(Mirrorsoft).z80", 0x0aaee, 0xdf00027d, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specdynadan2, Specdynadan2, Spectrum)
STD_ROM_FN(Specdynadan2)

struct BurnDriver BurnSpecdynadan2 = {
	"spec_dynadan2", NULL, "spec_spectrum", NULL, "1986",
	"Dynamite Dan II - Dr. Blitzen and the Islands of Arcanum (48K)\0", NULL, "Mirrorsoft", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, Specdynadan2RomInfo, Specdynadan2RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Elite (48K-128K)

static struct BurnRomInfo SpeceliteRomDesc[] = {
	{ "Elite (1985)(Firebird Software)(128k).z80", 0x0a8f4, 0xd914f0df, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specelite, Specelite, Spec128)
STD_ROM_FN(Specelite)

struct BurnDriver BurnSpecelite = {
	"spec_elite", NULL, "spec_spec128", NULL, "1985",
	"Elite (48K-128K)\0", NULL, "Firebird Software", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_SIM | GBF_STRATEGY, 0,
	SpectrumGetZipName, SpeceliteRomInfo, SpeceliteRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Elven Warrior (128K)

static struct BurnRomInfo SpecelvewarrRomDesc[] = {
	{ "Elven Warrior (1989)(Players Premier).tap", 51605, 0x10bcb8c8, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specelvewarr, Specelvewarr, Spec128)
STD_ROM_FN(Specelvewarr)

struct BurnDriver BurnSpecelvewarr = {
	"spec_elvewarr", NULL, "spec_spec128", NULL, "1989",
	"Elven Warrior (128K)\0", NULL, "Players Premier", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecelvewarrRomInfo, SpecelvewarrRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Emilio Butragueno Futbol (Spanish) (48K)

static struct BurnRomInfo SpecemilbutrafutbolRomDesc[] = {
	{ "Emilio Butragueno Futbol (1988)(Ocean Software - Topo Soft).tap", 35014, 0x99569c38, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specemilbutrafutbol, Specemilbutrafutbol, Spectrum)
STD_ROM_FN(Specemilbutrafutbol)

struct BurnDriver BurnSpecemilbutrafutbol = {
	"spec_emilbutrafutbol", NULL, "spec_spectrum", NULL, "1988",
	"Emilio Butragueno Futbol (Spanish) (48K)\0", NULL, "Topo Soft", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 2, HARDWARE_SPECTRUM, GBF_SPORTSFOOTBALL, 0,
	SpectrumGetZipName, SpecemilbutrafutbolRomInfo, SpecemilbutrafutbolRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Emilio Sanchez Vicario Grand Slam (Spanish) (48K)

static struct BurnRomInfo SpecemilsanchgslamRomDesc[] = {
	{ "Emilio Sanchez Vicario Grand Slam ES (1990)(Zigurat).tap", 46879, 0x22f0f155, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specemilsanchgslam, Specemilsanchgslam, Spectrum)
STD_ROM_FN(Specemilsanchgslam)

struct BurnDriver BurnSpecemilsanchgslam = {
	"spec_emilsanchgslam", NULL, "spec_spectrum", NULL, "1989",
	"Emilio Sanchez Vicario Grand Slam (Spanish) (48K)\0", NULL, "Zigurat Software", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 2, HARDWARE_SPECTRUM, GBF_SPORTSMISC, 0,
	SpectrumGetZipName, SpecemilsanchgslamRomInfo, SpecemilsanchgslamRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// E-motion (128K)

static struct BurnRomInfo SpecEmotionRomDesc[] = {
	{ "E-motion 128K (1990)(U.S. Gold).z80", 44420, 0xcc29fe26, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecEmotion, SpecEmotion, Spec128)
STD_ROM_FN(SpecEmotion)

struct BurnDriver BurnSpecEmotion = {
	"spec_emotion", NULL, "spec_spec128", NULL, "1990",
	"E-motion (128K)\0", NULL, "U.S. Gold", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 2, HARDWARE_SPECTRUM, GBF_ACTION | GBF_PUZZLE, 0,
	SpectrumGetZipName, SpecEmotionRomInfo, SpecEmotionRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Enduro Racer (128K)

static struct BurnRomInfo SpecenduroRomDesc[] = {
	{ "Enduro Racer (1987)(Activision).tap", 49670, 0x0e34f8bf, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specenduro, Specenduro, Spec128)
STD_ROM_FN(Specenduro)

struct BurnDriver BurnSpecenduro = {
	"spec_enduro", NULL, "spec_spec128", NULL, "1987",
	"Enduro Racer (128K)\0", NULL, "Activision", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 2, HARDWARE_SPECTRUM, GBF_RACING, 0,
	SpectrumGetZipName, SpecenduroRomInfo, SpecenduroRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Espada Sagrada, La (Spanish) (128K)

static struct BurnRomInfo SpecespadasagradaRomDesc[] = {
	{ "Espada Sagrada, La 128K (1990)(Topo Soft).tap", 93687, 0x60affc15, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specespadasagrada, Specespadasagrada, Spec128)
STD_ROM_FN(Specespadasagrada)

struct BurnDriver BurnSpecespadasagrada = {
	"spec_espadasagrada", NULL, "spec_spec128", NULL, "1990",
	"Espada Sagrada, La (Spanish) (128K)\0", NULL, "Topo Soft", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecespadasagradaRomInfo, SpecespadasagradaRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Exolon (48K)

static struct BurnRomInfo Specexolon48RomDesc[] = {
	{ "Exolon 48K (1987)(Hewson Consultants).tap", 44921, 0xedd68b67, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specexolon48, Specexolon48, Spectrum)
STD_ROM_FN(Specexolon48)

struct BurnDriver BurnSpecexolon48 = {
	"spec_exolon48", "spec_exolon128", "spec_spectrum", NULL, "1987",
	"Exolon (48K)\0", NULL, "Hewson Consultants", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE, 1, HARDWARE_SPECTRUM, GBF_RUNGUN, 0,
	SpectrumGetZipName, Specexolon48RomInfo, Specexolon48RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Exolon (128K)

static struct BurnRomInfo Specexolon128RomDesc[] = {
	{ "Exolon 128K (1987)(Hewson Consultants).tap", 42957, 0x7094b901, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specexolon128, Specexolon128, Spec128)
STD_ROM_FN(Specexolon128)

struct BurnDriver BurnSpecexolon128 = {
	"spec_exolon128", NULL, "spec_spec128", NULL, "1987",
	"Exolon (128K)\0", NULL, "Hewson Consultants", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_RUNGUN, 0,
	SpectrumGetZipName, Specexolon128RomInfo, Specexolon128RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Fantasy Zone, The (128K)

static struct BurnRomInfo SpecfntzoneRomDesc[] = {
	{ "Fantasy Zone, The (19xx)(-)[128K].z80", 0x05c6c, 0xe09d79d8, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specfntzone, Specfntzone, Spec128)
STD_ROM_FN(Specfntzone)

struct BurnDriver BurnSpecfntzone = {
	"spec_fntzone", NULL, "spec_spec128", NULL, "19xx",
	"Fantasy Zone, The (128K)\0", "Adult game", "Unknown", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_MISC, 0,
	SpectrumGetZipName, SpecfntzoneRomInfo, SpecfntzoneRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Fast Food Dizzy (48K)

static struct BurnRomInfo Specffdizzy48RomDesc[] = {
	{ "Fast Food Dizzy 48K (1989)(Codemasters).tap", 42067, 0x9c820493, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specffdizzy48, Specffdizzy48, Spectrum)
STD_ROM_FN(Specffdizzy48)

struct BurnDriver BurnSpecffdizzy48 = {
	"spec_ffdizzy48", "spec_ffdizzy128", "spec_spectrum", NULL, "1989",
	"Fast Food Dizzy (48K)\0", NULL, "Codemasters", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE, 1, HARDWARE_SPECTRUM, GBF_MAZE | GBF_ACTION, 0,
	SpectrumGetZipName, Specffdizzy48RomInfo, Specffdizzy48RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Fast Food Dizzy (128K)

static struct BurnRomInfo Specffdizzy128RomDesc[] = {
	{ "Fast Food Dizzy 128K (1989)(Codemasters).tap", 53170, 0x025c92cb, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specffdizzy128, Specffdizzy128, Spec128)
STD_ROM_FN(Specffdizzy128)

struct BurnDriver BurnSpecffdizzy128 = {
	"spec_ffdizzy128", NULL, "spec_spec128", NULL, "1989",
	"Fast Food Dizzy (128K)\0", NULL, "Codemasters", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_MAZE | GBF_ACTION, 0,
	SpectrumGetZipName, Specffdizzy128RomInfo, Specffdizzy128RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Fernando Martin Basket Master (Spanish) (48K)

static struct BurnRomInfo SpecfernmartbasketmRomDesc[] = {
	{ "Fernando Martin Basket Master ES 48K (1987)(Dinamic).tap", 47196, 0xf802ceb0, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specfernmartbasketm, Specfernmartbasketm, Spectrum)
STD_ROM_FN(Specfernmartbasketm)

struct BurnDriver BurnSpecfernmartbasketm = {
	"spec_fernmartbasketm", NULL, "spec_spectrum", NULL, "1987",
	"Fernando Martin Basket Master (Spanish) (48K)\0", NULL, "Dinamic Software", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 2, HARDWARE_SPECTRUM, GBF_SPORTSMISC, 0,
	SpectrumGetZipName, SpecfernmartbasketmRomInfo, SpecfernmartbasketmRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Feud (48K)

static struct BurnRomInfo SpecfeudRomDesc[] = {
	{ "Feud (1987)(Bulldog).tap", 47901, 0xf69e87b4, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specfeud, Specfeud, Spectrum)
STD_ROM_FN(Specfeud)

struct BurnDriver BurnSpecfeud = {
	"spec_feud", NULL, "spec_spectrum", NULL, "1987",
	"Feud (48K)\0", NULL, "Bulldog", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_ACTION | GBF_ADV, 0,
	SpectrumGetZipName, SpecfeudRomInfo, SpecfeudRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Final Matrix, The (48K)

static struct BurnRomInfo SpecfinalmatrixRomDesc[] = {
	{ "Final Matrix, The (1987)(Gremlin Graphics).tap", 67727, 0xdb92a506, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specfinalmatrix, Specfinalmatrix, Spectrum)
STD_ROM_FN(Specfinalmatrix)

struct BurnDriver BurnSpecfinalmatrix = {
	"spec_finalmatrix", NULL, "spec_spectrum", NULL, "1987",
	"Final Matrix, The (48K)\0", NULL, "Gremlin Graphics", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_MAZE | GBF_ACTION, 0,
	SpectrumGetZipName, SpecfinalmatrixRomInfo, SpecfinalmatrixRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Firelord (48K)

static struct BurnRomInfo SpecFirelordRomDesc[] = {
	{ "Firelord (1986)(Hewson Consultants).tap", 48559, 0xe586189f, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecFirelord, SpecFirelord, Spectrum)
STD_ROM_FN(SpecFirelord)

struct BurnDriver BurnSpecFirelord = {
	"spec_firelord", NULL, "spec_spectrum", NULL, "1986",
	"Firelord (48K)\0", NULL, "Hewson Consultants", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_ADV | GBF_ACTION, 0,
	SpectrumGetZipName, SpecFirelordRomInfo, SpecFirelordRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Flying Shark (128K)

static struct BurnRomInfo SpecfsharkRomDesc[] = {
	{ "Flying Shark 128K (1987)(Firebird).tap", 36523, 0x634f50aa, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specfshark, Specfshark, Spec128)
STD_ROM_FN(Specfshark)

struct BurnDriver BurnSpecfshark = {
	"spec_fshark", NULL, "spec_spec128", NULL, "1987",
	"Flying Shark (128K)\0", NULL, "Firebird Software", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_VERSHOOT, 0,
	SpectrumGetZipName, SpecfsharkRomInfo, SpecfsharkRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Football Director (48K)

static struct BurnRomInfo SpecfootdireRomDesc[] = {
	{ "Football Director (1986)(D&H Games).tap", 47945, 0x71643b24, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specfootdire, Specfootdire, Spectrum)
STD_ROM_FN(Specfootdire)

struct BurnDriver BurnSpecfootdire = {
	"spec_footdire", NULL, "spec_spectrum", NULL, "1986",
	"Football Director (48K)\0", NULL, "D&H Games", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_SPORTSFOOTBALL | GBF_STRATEGY, 0,
	SpectrumGetZipName, SpecfootdireRomInfo, SpecfootdireRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Football Manager - World Cup Edition (48K)

static struct BurnRomInfo SpecftmanwcRomDesc[] = {
	{ "Football Manager - World Cup Edition (1990)(Addictive Games).tap", 41462, 0x52f22bdb, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specftmanwc, Specftmanwc, Spectrum)
STD_ROM_FN(Specftmanwc)

struct BurnDriver BurnSpecftmanwc = {
	"spec_ftmanwc", NULL, "spec_spectrum", NULL, "1990",
	"Football Manager - World Cup Edition (48K)\0", NULL, "Addictive Games", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_SPORTSFOOTBALL | GBF_STRATEGY, 0,
	SpectrumGetZipName, SpecftmanwcRomInfo, SpecftmanwcRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Football Manager (48K)

static struct BurnRomInfo SpecftmanRomDesc[] = {
	{ "Football Manager (1982)(Addictive Games).tap", 35582, 0x9ff2f05d, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specftman, Specftman, Spectrum)
STD_ROM_FN(Specftman)

struct BurnDriver BurnSpecftman = {
	"spec_ftman", NULL, "spec_spectrum", NULL, "1982",
	"Football Manager (48K)\0", NULL, "Addictive Games", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_SPORTSFOOTBALL | GBF_STRATEGY, 0,
	SpectrumGetZipName, SpecftmanRomInfo, SpecftmanRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Football Manager 2 (48K)

static struct BurnRomInfo Specfootman2RomDesc[] = {
	{ "Football Manager 2 (1988)(Addictive Games).tap", 50435, 0x060ddf75, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specfootman2, Specfootman2, Spectrum)
STD_ROM_FN(Specfootman2)

struct BurnDriver BurnSpecfootman2 = {
	"spec_footman2", NULL, "spec_spectrum", NULL, "1988",
	"Football Manager 2 (48K)\0", NULL, "Addictive Games", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_SPORTSFOOTBALL | GBF_STRATEGY, 0,
	SpectrumGetZipName, Specfootman2RomInfo, Specfootman2RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Football Manager 3 (48K)

static struct BurnRomInfo Specfootman3RomDesc[] = {
	{ "Football Manager 3 (1991)(Addictive Games).tap", 46747, 0x5e1a8c16, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specfootman3, Specfootman3, Spectrum)
STD_ROM_FN(Specfootman3)

struct BurnDriver BurnSpecfootman3 = {
	"spec_footman3", NULL, "spec_spectrum", NULL, "1991",
	"Football Manager 3 (48K)\0", NULL, "Addictive Games", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_SPORTSFOOTBALL | GBF_STRATEGY, 0,
	SpectrumGetZipName, Specfootman3RomInfo, Specfootman3RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecQAOPSpaceDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Footballer of the Year (48K)

static struct BurnRomInfo SpecfotyRomDesc[] = {
	{ "Footballer of the Year (1986)(Gremlin Graphics Software).z80", 0x08137, 0x2af522d0, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specfoty, Specfoty, Spectrum)
STD_ROM_FN(Specfoty)

struct BurnDriver BurnSpecfoty = {
	"spec_foty", NULL, "spec_spectrum", NULL, "1986",
	"Footballer of the Year (48K)\0", NULL, "Gremlin Graphics", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_SPORTSFOOTBALL | GBF_STRATEGY, 0,
	SpectrumGetZipName, SpecfotyRomInfo, SpecfotyRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Footballer of the Year 2 (48K)

static struct BurnRomInfo Specfoty2RomDesc[] = {
	{ "Footballer of the Year 2 (1987)(Gremlin Graphics Software).z80", 0x08d8f, 0x3722f534, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specfoty2, Specfoty2, Spectrum)
STD_ROM_FN(Specfoty2)

struct BurnDriver BurnSpecfoty2 = {
	"spec_foty2", NULL, "spec_spectrum", NULL, "1987",
	"Footballer of the Year 2 (48K)\0", NULL, "Gremlin Graphics", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_SPORTSFOOTBALL | GBF_STRATEGY, 0,
	SpectrumGetZipName, Specfoty2RomInfo, Specfoty2RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Forgotten Worlds (128K)

static struct BurnRomInfo SpecforgottnRomDesc[] = {
	{ "Forgotten Worlds (1989)(U.S. Gold)[128K].z80", 0x0e045, 0x33ef767e, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specforgottn, Specforgottn, Spec128)
STD_ROM_FN(Specforgottn)

struct BurnDriver BurnSpecforgottn = {
	"spec_forgottn", NULL, "spec_spec128", NULL, "1989",
	"Forgotten Worlds (128K)\0", NULL, "U.S. Gold", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 2, HARDWARE_SPECTRUM, GBF_HORSHOOT, 0,
	SpectrumGetZipName, SpecforgottnRomInfo, SpecforgottnRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Fred (48K)

static struct BurnRomInfo SpecfredRomDesc[] = {
	{ "Fred (1984)(Quicksilva).tap", 31242, 0x8c4bd9a9, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specfred, Specfred, Spectrum)
STD_ROM_FN(Specfred)

struct BurnDriver BurnSpecfred = {
	"spec_fred", NULL, "spec_spectrum", NULL, "1984",
	"Fred (48K)\0", NULL, "Quicksilva", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecfredRomInfo, SpecfredRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Freddy Hardest (Spanish) (48K)

static struct BurnRomInfo SpecfredhardRomDesc[] = {
	{ "Freddy Hardest ES (1987)(Dinamic).tap", 95231, 0xff6d65f0, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specfredhard, Specfredhard, Spectrum)
STD_ROM_FN(Specfredhard)

struct BurnDriver BurnSpecfredhard = {
	"spec_fredhard", NULL, "spec_spectrum", NULL, "1987",
	"Freddy Hardest (Spanish) (48K)\0", NULL, "Dinamic Software", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_RUNGUN, 0,
	SpectrumGetZipName, SpecfredhardRomInfo, SpecfredhardRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Frogger (48K)

static struct BurnRomInfo SpecfroggerRomDesc[] = {
	{ "Frogger 48K (1983)(A'n'F Software).tap", 16497, 0xab533972, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specfrogger, Specfrogger, Spectrum)
STD_ROM_FN(Specfrogger)

struct BurnDriver BurnSpecfrogger = {
	"spec_frogger", NULL, "spec_spectrum", NULL, "1983",
	"Frogger (48K)\0", NULL, "A'n'F Software", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_ACTION, 0,
	SpectrumGetZipName, SpecfroggerRomInfo, SpecfroggerRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecQAOPSpaceDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Ghostbusters (48K)

static struct BurnRomInfo Specghostb_48RomDesc[] = {
	{ "Ghostbusters 48K (1984)(Activision).tap", 39790, 0x25deb7c0, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specghostb_48, Specghostb_48, Spectrum)
STD_ROM_FN(Specghostb_48)

struct BurnDriver BurnSpecghostb_48 = {
	"spec_ghostb_48", "spec_ghostb", "spec_spectrum", NULL, "1984",
	"Ghostbusters (48K)\0", NULL, "Activision", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE, 1, HARDWARE_SPECTRUM, GBF_ACTION, 0,
	SpectrumGetZipName, Specghostb_48RomInfo, Specghostb_48RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Ghostbusters (128K)

static struct BurnRomInfo SpecghostbRomDesc[] = {
	{ "Ghostbusters 128K (1984)(Activision).tap", 69552, 0xc4c790a1, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specghostb, Specghostb, Spec128)
STD_ROM_FN(Specghostb)

struct BurnDriver BurnSpecghostb = {
	"spec_ghostb", NULL, "spec_spec128", NULL, "1984",
	"Ghostbusters (128K)\0", NULL, "Activision", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_ACTION, 0,
	SpectrumGetZipName, SpecghostbRomInfo, SpecghostbRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Ghostbusters II (128K) (Trainer)

static struct BurnRomInfo Specghostb2RomDesc[] = {
	{ "Ghostbusters II (1989)(Activision)[t].tap", 174896, 0x1594430c, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specghostb2, Specghostb2, Spec128)
STD_ROM_FN(Specghostb2)

struct BurnDriver BurnSpecghostb2 = {
	"spec_ghostb2", NULL, "spec_spec128", NULL, "1989",
	"Ghostbusters II (128K) (Trainer)\0", NULL, "Activision", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_ACTION, 0,
	SpectrumGetZipName, Specghostb2RomInfo, Specghostb2RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Ghouls 'n' Ghosts (128K)

static struct BurnRomInfo SpecgngRomDesc[] = {
	{ "Ghouls 'n' Ghosts (1989)(U.S. Gold)[128K].z80", 0x1a8d2, 0x1b626fe8, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specgng, Specgng, Spec128)
STD_ROM_FN(Specgng)

struct BurnDriver BurnSpecgng = {
	"spec_gng", NULL, "spec_spec128", NULL, "1989",
	"Ghouls 'n' Ghosts (128K)\0", NULL, "U.S. Gold", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_RUNGUN | GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecgngRomInfo, SpecgngRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// G-LOC (128K)

static struct BurnRomInfo SpecglocRomDesc[] = {
	{ "G-LOC (1992)(U.S. Gold)[128K].z80", 0x16f39, 0xeeae7278, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specgloc, Specgloc, Spec128)
STD_ROM_FN(Specgloc)

struct BurnDriver BurnSpecgloc = {
	"spec_gloc", NULL, "spec_spec128", NULL, "1992",
	"G-LOC (128K)\0", NULL, "U.S. Gold", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_SHOOT, 0,
	SpectrumGetZipName, SpecglocRomInfo, SpecglocRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Great Escape, The (48K)

static struct BurnRomInfo SpecgreatescRomDesc[] = {
	{ "Great Escape, The (1986)(Ocean).tap", 50366, 0x0d3be279, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specgreatesc, Specgreatesc, Spectrum)
STD_ROM_FN(Specgreatesc)

struct BurnDriver BurnSpecgreatesc = {
	"spec_greatesc", NULL, "spec_spectrum", NULL, "1986",
	"Great Escape, The (48K)\0", NULL, "Ocean", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_ADV, 0,
	SpectrumGetZipName, SpecgreatescRomInfo, SpecgreatescRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Green Beret (48K)

static struct BurnRomInfo Specgberet_48RomDesc[] = {
	{ "Green Beret (1986)(Imagine Software).z80", 0x0ad53, 0x55f36544, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specgberet_48, Specgberet_48, Spectrum)
STD_ROM_FN(Specgberet_48)

struct BurnDriver BurnSpecgberet_48 = {
	"spec_gberet_48", NULL, "spec_spectrum", NULL, "1986",
	"Green Beret (48K)\0", NULL, "Imagine Software", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_RUNGUN, 0,
	SpectrumGetZipName, Specgberet_48RomInfo, Specgberet_48RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Gryzor (48K)

static struct BurnRomInfo Specgryzor48RomDesc[] = {
	{ "Gryzor 48K (1987)(Ocean).tap", 49774, 0x1b11c206, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specgryzor48, Specgryzor48, Spectrum)
STD_ROM_FN(Specgryzor48)

struct BurnDriver BurnSpecgryzor48 = {
	"spec_gryzor48", "spec_gryzor128", "spec_spectrum", NULL, "1987",
	"Gryzor (48K)\0", "SPACE: jump - SPACE+down: climb down", "Ocean", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE, 1, HARDWARE_SPECTRUM, GBF_RUNGUN | GBF_PLATFORM, 0,
	SpectrumGetZipName, Specgryzor48RomInfo, Specgryzor48RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Gryzor (128K)

static struct BurnRomInfo Specgryzor128RomDesc[] = {
	{ "Gryzor 128K (1987)(Ocean).tap", 115219, 0x941d84c9, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specgryzor128, Specgryzor128, Spec128)
STD_ROM_FN(Specgryzor128)

struct BurnDriver BurnSpecgryzor128 = {
	"spec_gryzor128", NULL, "spec_spec128", NULL, "1987",
	"Gryzor (128K)\0", "SPACE: jump - SPACE+down: climb down", "Ocean", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_RUNGUN | GBF_PLATFORM, 0,
	SpectrumGetZipName, Specgryzor128RomInfo, Specgryzor128RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Hammer Boy (Part 1) (128K)

static struct BurnRomInfo Spechammerboy1RomDesc[] = {
	{ "Hammer Boy (1991)(Dinamic Software)(Part 1 of 2)(128k).tap", 48943, 0x1770245b, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Spechammerboy1, Spechammerboy1, Spec128)
STD_ROM_FN(Spechammerboy1)

struct BurnDriver BurnSpechammerboy1 = {
	"spec_hammerboy1", NULL, "spec_spec128", NULL, "1991",
	"Hammer Boy (Part 1) (128K)\0", NULL, "Dinamic Software", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_ACTION, 0,
	SpectrumGetZipName, Spechammerboy1RomInfo, Spechammerboy1RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Hammer Boy (Part 2) (128K)

static struct BurnRomInfo Spechammerboy2RomDesc[] = {
	{ "Hammer Boy (1991)(Dinamic Software)(Part 2 of 2)(128k).tap", 48943, 0x858a0af4, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Spechammerboy2, Spechammerboy2, Spec128)
STD_ROM_FN(Spechammerboy2)

struct BurnDriver BurnSpechammerboy2 = {
	"spec_hammerboy2", "spec_hammerboy1", "spec_spec128", NULL, "1991",
	"Hammer Boy (Part 2) (128K)\0", NULL, "Dinamic Software", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE, 1, HARDWARE_SPECTRUM, GBF_ACTION, 0,
	SpectrumGetZipName, Spechammerboy2RomInfo, Spechammerboy2RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// H.A.T.E. - Hostile All Terrain Encounter (48K)

static struct BurnRomInfo SpechatehateRomDesc[] = {
	{ "H.A.T.E. - Hostile All Terrain Encounter (1989)(Vortex Software).tap", 42128, 0x0fcfc02e, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Spechatehate, Spechatehate, Spectrum)
STD_ROM_FN(Spechatehate)

struct BurnDriver BurnSpechatehate = {
	"spec_hatehate", NULL, "spec_spectrum", NULL, "1989",
	"H.A.T.E. - Hostile All Terrain Encounter (48K)\0", NULL, "Gremlin Graphics", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_SHOOT, 0,
	SpectrumGetZipName, SpechatehateRomInfo, SpechatehateRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Hard Drivin' (128K)

static struct BurnRomInfo SpecharddrivRomDesc[] = {
	{ "Hard Drivin' 128K (1989)(Domark).tap", 47211, 0xf55dc984, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specharddriv, Specharddriv, Spec128)
STD_ROM_FN(Specharddriv)

struct BurnDriver BurnSpecharddriv = {
	"spec_harddriv", NULL, "spec_spec128", NULL, "1989",
	"Hard Drivin' (128K)\0", NULL, "Domark", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_RACING, 0,
	SpectrumGetZipName, SpecharddrivRomInfo, SpecharddrivRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Head over Heels (128K)

static struct BurnRomInfo SpecheadheelRomDesc[] = {
	{ "Head over Heels (1987)(Ocean)[128K].z80", 0x0c5a5, 0x9f148037, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specheadheel, Specheadheel, Spec128)
STD_ROM_FN(Specheadheel)

struct BurnDriver BurnSpecheadheel = {
	"spec_headheel", NULL, "spec_spec128", NULL, "1987",
	"Head over Heels (128K)\0", "Carry: ENTER - Fire: X - Swop: S", "Ocean", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_ADV, 0,
	SpectrumGetZipName, SpecheadheelRomInfo, SpecheadheelRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Heart Broken (48K)

static struct BurnRomInfo SpechrtbbrknRomDesc[] = {
	{ "Heart Broken 48K (1989)(Atlantis Software).tap", 54856, 0x34ed13a7, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Spechrtbbrkn, Spechrtbbrkn, Spectrum)
STD_ROM_FN(Spechrtbbrkn)

struct BurnDriver BurnSpechrtbbrkn = {
	"spec_hrtbbrkn", NULL, "spec_spectrum", NULL, "1989",
	"Heart Broken (48K)\0", NULL, "Atlantis Software", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_PLATFORM | GBF_ADV, 0,
	SpectrumGetZipName, SpechrtbbrknRomInfo, SpechrtbbrknRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Heartland (48K)

static struct BurnRomInfo SpecheartlanRomDesc[] = {
	{ "Heartland (1986)(Odin Computer Graphics).tap", 42923, 0xd756125c, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specheartlan, Specheartlan, Spectrum)
STD_ROM_FN(Specheartlan)

struct BurnDriver BurnSpecheartlan = {
	"spec_heartlan", NULL, "spec_spectrum", NULL, "1986",
	"Heartland (48K)\0", NULL, "Odin Computer Graphics", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_ACTION | GBF_ADV, 0,
	SpectrumGetZipName, SpecheartlanRomInfo, SpecheartlanRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Highway Encounter (48K)

static struct BurnRomInfo SpechighencoRomDesc[] = {
	{ "Highway Encounter (1985)(Vortex Software).z80", 0x0a36c, 0x7946eec7, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Spechighenco, Spechighenco, Spectrum)
STD_ROM_FN(Spechighenco)

struct BurnDriver BurnSpechighenco = {
	"spec_highenco", NULL, "spec_spectrum", NULL, "1985",
	"Highway Encounter (48K)\0", NULL, "Vortex Software", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_SHOOT, 0,
	SpectrumGetZipName, SpechighencoRomInfo, SpechighencoRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Hobbit, The v1.0 (48K)

static struct BurnRomInfo Spechobbit2RomDesc[] = {
	{ "Hobbit, The v1.0 (1982)(Melbourne House).z80", 0x0a4c3, 0x46c20d35, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Spechobbit2, Spechobbit2, Spectrum)
STD_ROM_FN(Spechobbit2)

struct BurnDriver BurnSpechobbit2 = {
	"spec_hobbit2", "spec_hobbit", "spec_spectrum", NULL, "1982",
	"Hobbit, The v1.0 (48K)\0", NULL, "Melbourne House", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE, 1, HARDWARE_SPECTRUM, GBF_ADV, 0,
	SpectrumGetZipName, Spechobbit2RomInfo, Spechobbit2RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Hobbit, The v1.2 (48K)

static struct BurnRomInfo SpechobbitRomDesc[] = {
	{ "Hobbit, The v1.2 (1982)(Melbourne House).z80", 0x0b1ee, 0x10231c84, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Spechobbit, Spechobbit, Spectrum)
STD_ROM_FN(Spechobbit)

struct BurnDriver BurnSpechobbit = {
	"spec_hobbit", NULL, "spec_spectrum", NULL, "1982",
	"Hobbit, The v1.2 (48K)\0", NULL, "Melbourne House", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_ADV, 0,
	SpectrumGetZipName, SpechobbitRomInfo, SpechobbitRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Horace Goes Skiing (48K)

static struct BurnRomInfo SpechoraceskRomDesc[] = {
	{ "Horace Goes Skiing (1982)(Sinclair Research).z80", 0x0c01e, 0x02cd124b, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Spechoracesk, Spechoracesk, Spectrum)
STD_ROM_FN(Spechoracesk)

struct BurnDriver BurnSpechoracesk = {
	"spec_horacesk", NULL, "spec_spectrum", NULL, "1982",
	"Horace Goes Skiing (48K)\0", NULL, "Sinclair Research", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_ACTION, 0,
	SpectrumGetZipName, SpechoraceskRomInfo, SpechoraceskRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecIntf2DIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// How to be a Complete Bastard (128K)

static struct BurnRomInfo SpechowbastRomDesc[] = {
	{ "How to be a Complete Bastard (1987)(Virgin Games)[128K].z80", 0x0e728, 0x7460da43, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Spechowbast, Spechowbast, Spec128)
STD_ROM_FN(Spechowbast)

struct BurnDriver BurnSpechowbast = {
	"spec_howbast", NULL, "spec_spec128", NULL, "1987",
	"How to be a Complete Bastard (128K)\0", NULL, "Virgin Games", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_ADV, 0,
	SpectrumGetZipName, SpechowbastRomInfo, SpechowbastRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Hudson Hawk (128K)

static struct BurnRomInfo SpechudshawkRomDesc[] = {
	{ "Hudson Hawk 128K (1991)(Ocean).tap", 135956, 0xbccb6e2a, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Spechudshawk, Spechudshawk, Spec128)
STD_ROM_FN(Spechudshawk)

struct BurnDriver BurnSpechudshawk = {
	"spec_hudshawk", NULL, "spec_spec128", NULL, "1991",
	"Hudson Hawk (128K)\0", NULL, "Ocean", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpechudshawkRomInfo, SpechudshawkRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecIntf2DIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Jack the Nipper (48K)

static struct BurnRomInfo SpecjacknipperRomDesc[] = {
	{ "Jack the Nipper 48K (1986)(Gremlin Graphics).tap", 46966, 0x729f41ab, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specjacknipper, Specjacknipper, Spectrum)
STD_ROM_FN(Specjacknipper)

struct BurnDriver BurnSpecjacknipper = {
	"spec_jacknipper", NULL, "spec_spectrum", NULL, "1986",
	"Jack the Nipper (48K)\0", NULL, "Gremlin Graphics", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_ADV, 0,
	SpectrumGetZipName, SpecjacknipperRomInfo, SpecjacknipperRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Jack the Nipper II - In Coconut Capers (48K)

static struct BurnRomInfo Specjacknipper2RomDesc[] = {
	{ "Jack the Nipper II - In Coconut Capers 48K (1987)(Gremlin Graphics).z80", 43126, 0xdc5995ff, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specjacknipper2, Specjacknipper2, Spectrum)
STD_ROM_FN(Specjacknipper2)

struct BurnDriver BurnSpecjacknipper2 = {
	"spec_jacknipper2", NULL, "spec_spectrum", NULL, "1987",
	"Jack the Nipper II - In Coconut Capers (48K)\0", "Set '4 Interface II' to use controller", "Gremlin Graphics", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, Specjacknipper2RomInfo, Specjacknipper2RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecIntf2DIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Jet Set Willy (48K)

static struct BurnRomInfo SpecjswillyRomDesc[] = {
	{ "Jet Set Willy 48K (1984)(Software Projects).tap", 33280, 0x17ec8978, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specjswilly, Specjswilly, Spectrum)
STD_ROM_FN(Specjswilly)

struct BurnDriver BurnSpecjswilly = {
	"spec_jswilly", NULL, "spec_spectrum", NULL, "1984",
	"Jet Set Willy (48K)\0", NULL, "Software Projects", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecjswillyRomInfo, SpecjswillyRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecCursorKeysDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Jet Set Willy II - The Final Frontier (48K)

static struct BurnRomInfo Specjswilly2RomDesc[] = {
	{ "Jet Set Willy II - The Final Frontier 48K (1985)(Software Projects).tap", 49881, 0x85a4ac92, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specjswilly2, Specjswilly2, Spectrum)
STD_ROM_FN(Specjswilly2)

struct BurnDriver BurnSpecjswilly2 = {
	"spec_jswilly2", NULL, "spec_spectrum", NULL, "1985",
	"Jet Set Willy II - The Final Frontier (48K)\0", NULL, "Software Projects", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, Specjswilly2RomInfo, Specjswilly2RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecQAOPMDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Jet Set Willy III (48K)

static struct BurnRomInfo Specjswilly3RomDesc[] = {
	{ "Jet Set Willy III (1985)(MB - APG Software).z80", 0x08061, 0xbde7b5ae, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specjswilly3, Specjswilly3, Spectrum)
STD_ROM_FN(Specjswilly3)

struct BurnDriver BurnSpecjswilly3 = {
	"spec_jswilly3", NULL, "spec_spectrum", NULL, "1985",
	"Jet Set Willy III (48K)\0", NULL, "MB - APG Software", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, Specjswilly3RomInfo, Specjswilly3RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Jet Pac (16K)

static struct BurnRomInfo SpecjetpacRomDesc[] = {
	{ "Jet Pac 16K (1983)(Ultimate Play The Game).tap",     15366, 0xeb8ce1c8, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specjetpac, Specjetpac, Spectrum)
STD_ROM_FN(Specjetpac)

struct BurnDriver BurnSpecjetpac = {
	"spec_jetpac", NULL, "spec_spectrum", NULL, "1983",
	"Jet Pac (16K)\0", NULL, "Ultimate Play The Game", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 2, HARDWARE_SPECTRUM, GBF_ACTION, 0,
	SpectrumGetZipName, SpecjetpacRomInfo, SpecjetpacRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Jetsons The Computer Game (128K)

static struct BurnRomInfo SpecJetsonsRomDesc[] = {
	{ "Jetsons - The Computer Game 128K (1992)(Hi-Tec).tap", 117628, 0xdf6fe05a, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecJetsons, SpecJetsons, Spec128)
STD_ROM_FN(SpecJetsons)

struct BurnDriver BurnSpecJetsons = {
	"spec_jetsons", NULL, "spec_spec128", NULL, "1992",
	"Jetsons The Computer Game (128K)\0", "Mega Trainer version", "Hi-Tec Software", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_ADV, 0,
	SpectrumGetZipName, SpecJetsonsRomInfo, SpecJetsonsRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Joe Blade (128K)

static struct BurnRomInfo Specjoebld128RomDesc[] = {
	{ "Joe Blade (1987)(Players)(48K-128K).tap", 50660, 0x27e8fa83, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specjoebld128, Specjoebld128, Spec128)
STD_ROM_FN(Specjoebld128)

struct BurnDriver BurnSpecjoebld128 = {
	"spec_joebld128", NULL, "spec_spec128", NULL, "1987",
	"Joe Blade (128K)\0", NULL, "Players Premier", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_RUNGUN | GBF_ADV, 0,
	SpectrumGetZipName, Specjoebld128RomInfo, Specjoebld128RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Joe Blade (48K)

static struct BurnRomInfo Specjoebld48RomDesc[] = {
	{ "Joe Blade (1987)(Players)(48K-128K).tap", 50660, 0x27e8fa83, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specjoebld48, Specjoebld48, Spectrum)
STD_ROM_FN(Specjoebld48)

struct BurnDriver BurnSpecjoebld48 = {
	"spec_joebld48", "spec_joebld128", "spec_spectrum", NULL, "1987",
	"Joe Blade (48K)\0", NULL, "Players Premier", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE, 1, HARDWARE_SPECTRUM, GBF_RUNGUN | GBF_ADV, 0,
	SpectrumGetZipName, Specjoebld48RomInfo, Specjoebld48RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Joe Blade II (128K)

static struct BurnRomInfo Specjoebld2RomDesc[] = {
	{ "Joe Blade 2 (1988)(Players Premier)(128k).z80", 0x0bb13, 0x20a19599, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specjoebld2, Specjoebld2, Spec128)
STD_ROM_FN(Specjoebld2)

struct BurnDriver BurnSpecjoebld2 = {
	"spec_joebld2", NULL, "spec_spec128", NULL, "1988",
	"Joe Blade II (128K)\0", NULL, "Players Premier", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_RUNGUN | GBF_ADV, 0,
	SpectrumGetZipName, Specjoebld2RomInfo, Specjoebld2RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Joe Blade III (128K)

static struct BurnRomInfo Specjoebld3RomDesc[] = {
	{ "Joe Blade 3 (1989)(Players Premier)(128k).z80", 0x0b400, 0x422359d9, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specjoebld3, Specjoebld3, Spec128)
STD_ROM_FN(Specjoebld3)

struct BurnDriver BurnSpecjoebld3 = {
	"spec_joebld3", NULL, "spec_spec128", NULL, "1989",
	"Joe Blade III (128K)\0", NULL, "Players Premier", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_RUNGUN | GBF_ADV, 0,
	SpectrumGetZipName, Specjoebld3RomInfo, Specjoebld3RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Kick Off (48K)

static struct BurnRomInfo SpeckickoffRomDesc[] = {
	{ "Kick Off 48K (1989)(Anco).tap", 47041, 0xf54acc07, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Speckickoff, Speckickoff, Spectrum)
STD_ROM_FN(Speckickoff)

struct BurnDriver BurnSpeckickoff = {
	"spec_kickoff", NULL, "spec_spectrum", NULL, "1989",
	"Kick Off (48K)\0", NULL, "Anco", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 2, HARDWARE_SPECTRUM, GBF_SPORTSFOOTBALL, 0,
	SpectrumGetZipName, SpeckickoffRomInfo, SpeckickoffRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Kick Off 2 (128K)

static struct BurnRomInfo SpecKickoff2RomDesc[] = {
	{ "Kick Off 2 (1990)(Anco).tap", 77052, 0x74a921aa, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecKickoff2, SpecKickoff2, Spec128)
STD_ROM_FN(SpecKickoff2)

struct BurnDriver BurnSpecKickoff2 = {
	"spec_kickoff2", NULL, "spec_spec128", NULL, "1990",
	"Kick Off 2 (128K)\0", "Use Q, A and C to choose and select options", "Anco", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 2, HARDWARE_SPECTRUM, GBF_SPORTSFOOTBALL, 0,
	SpectrumGetZipName, SpecKickoff2RomInfo, SpecKickoff2RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Kick Off World Cup Edition (128K)

static struct BurnRomInfo SpeckickoffwRomDesc[] = {
	{ "Kick Off World Cup Edition (1990)(Anco Software)[128K].z80", 0x09ba6, 0x2a01e70e, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Speckickoffw, Speckickoffw, Spec128)
STD_ROM_FN(Speckickoffw)

struct BurnDriver BurnSpeckickoffw = {
	"spec_kickoffw", NULL, "spec_spec128", NULL, "1990",
	"Kick Off World Cup Edition (128K)\0", NULL, "Anco", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 2, HARDWARE_SPECTRUM, GBF_SPORTSFOOTBALL, 0,
	SpectrumGetZipName, SpeckickoffwRomInfo, SpeckickoffwRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Kikstart 2 + Course Designer (48K)

static struct BurnRomInfo Speckikstrt2RomDesc[] = {
	{ "Kikstart 2 + Course Designer (1988)(Mastertronic).tap", 35230, 0xf4beb546, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Speckikstrt2, Speckikstrt2, Spectrum)
STD_ROM_FN(Speckikstrt2)

struct BurnDriver BurnSpeckikstrt2 = {
	"spec_kikstrt2", NULL, "spec_spectrum", NULL, "1988",
	"Kikstart 2 + Course Designer (48K)\0", "Set '4. CURSOR' to use controller", "Mastertronic", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 2, HARDWARE_SPECTRUM, GBF_ACTION, 0,
	SpectrumGetZipName, Speckikstrt2RomInfo, Speckikstrt2RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecCursorKeysDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Knight Lore (48K)

static struct BurnRomInfo SpecknigloreRomDesc[] = {
	{ "Knight Lore (1984)(Ultimate Play The Game).z80", 0x08c4c, 0x137ffdb2, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Speckniglore, Speckniglore, Spectrum)
STD_ROM_FN(Speckniglore)

struct BurnDriver BurnSpeckniglore = {
	"spec_kniglore", NULL, "spec_spectrum", NULL, "1984",
	"Knight Lore (48K)\0", NULL, "Ultimate Play The Game", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_ADV, 0,
	SpectrumGetZipName, SpecknigloreRomInfo, SpecknigloreRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Kong (Ocean) (48K)

static struct BurnRomInfo SpeckongRomDesc[] = {
	{ "Kong 48K (1983)(Ocean).tap", 45603, 0xd5a04266, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Speckong, Speckong, Spectrum)
STD_ROM_FN(Speckong)

struct BurnDriver BurnSpeckong = {
	"spec_kong", NULL, "spec_spectrum", NULL, "1984",
	"Kong (Ocean) (48K)\0", NULL, "Ocean", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_ACTION, 0,
	SpectrumGetZipName, SpeckongRomInfo, SpeckongRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Kong 2 - Kong Strikes Back (48K)

static struct BurnRomInfo Speckong2RomDesc[] = {
	{ "Kong Strikes Back! 48K (1984)(Ocean).tap", 32685, 0x134e89ea, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Speckong2, Speckong2, Spectrum)
STD_ROM_FN(Speckong2)

struct BurnDriver BurnSpeckong2 = {
	"spec_kong2", NULL, "spec_spectrum", NULL, "1985",
	"Kong 2 - Kong Strikes Back (48K)\0", "Press 'J' for controller menu", "Ocean", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_ACTION, 0,
	SpectrumGetZipName, Speckong2RomInfo, Speckong2RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Kung-Fu Master (48K)

static struct BurnRomInfo SpeckungfumRomDesc[] = {
	{ "Kung-Fu Master (1986)(U.S. Gold).z80", 0x0a7a1, 0x7a375d9e, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Speckungfum, Speckungfum, Spectrum)
STD_ROM_FN(Speckungfum)

struct BurnDriver BurnSpeckungfum = {
	"spec_kungfum", NULL, "spec_spectrum", NULL, "1986",
	"Kung-Fu Master (48K)\0", NULL, "U.S. Gold", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 2, HARDWARE_SPECTRUM, GBF_SCRFIGHT, 0,
	SpectrumGetZipName, SpeckungfumRomInfo, SpeckungfumRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Kwik Snax Dizzy (128K)

static struct BurnRomInfo SpecksdizzyRomDesc[] = {
	{ "Kwik Snax Dizzy 128K (1990)(Codemasters).tap", 100738, 0x0b234319, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specksdizzy, Specksdizzy, Spec128)
STD_ROM_FN(Specksdizzy)

struct BurnDriver BurnSpecksdizzy = {
	"spec_ksdizzy", NULL, "spec_spec128", NULL, "1990",
	"Kwik Snax Dizzy (128K)\0", NULL, "Codemasters", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_MAZE | GBF_ACTION, 0,
	SpectrumGetZipName, SpecksdizzyRomInfo, SpecksdizzyRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Last Ninja 2 - Back with a Vengeance (128K)

static struct BurnRomInfo Speclninja2RomDesc[] = {
	{ "Last Ninja 2 - Back with a Vengeance (1988)(System 3).tap", 233900, 0xa6be2baa, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Speclninja2, Speclninja2, Spec128)
STD_ROM_FN(Speclninja2)

struct BurnDriver BurnSpeclninja2 = {
	"spec_lninja2", NULL, "spec_spec128", NULL, "1988",
	"Last Ninja 2 - Back with a Vengeance (128K)\0", NULL, "System 3", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_ACTION | GBF_ADV, 0,
	SpectrumGetZipName, Speclninja2RomInfo, Speclninja2RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Line of Fire (128K)

static struct BurnRomInfo SpeclinefireRomDesc[] = {
	{ "Line of Fire 128K (1990)(U.S. Gold).tap", 165694, 0x4f1c0e0e, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Speclinefire, Speclinefire, Spec128)
STD_ROM_FN(Speclinefire)

struct BurnDriver BurnSpeclinefire = {
	"spec_linefire", NULL, "spec_spec128", NULL, "1990",
	"Line of Fire (128K)\0", NULL, "U.S. Gold", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 2, HARDWARE_SPECTRUM, GBF_SHOOT, 0,
	SpectrumGetZipName, SpeclinefireRomInfo, SpeclinefireRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Livingstone, I Presume (128K)

static struct BurnRomInfo SpeclivipresRomDesc[] = {
	{ "Livingstone, I Presume (1987)(Opera Soft)(128k).z80", 0x0aff2, 0x50f57c95, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Speclivipres, Speclivipres, Spec128)
STD_ROM_FN(Speclivipres)

struct BurnDriver BurnSpeclivipres = {
	"spec_livipres", NULL, "spec_spec128", NULL, "1987",
	"Livingstone, I Presume (128K)\0", NULL, "Opera Soft", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpeclivipresRomInfo, SpeclivipresRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecQAOPSpaceDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Livingstone Supongo II (I Premuse II) - Part 1 (48K)

static struct BurnRomInfo Speclivipres21RomDesc[] = {
	{ "Livingstone Supongo II - part 1 48K (1989)(Opera Soft).z80", 42326, 0x995fb9a8, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Speclivipres21, Speclivipres21, Spectrum)
STD_ROM_FN(Speclivipres21)

struct BurnDriver BurnSpeclivipres21 = {
	"spec_livipres21", NULL, "spec_spectrum", NULL, "1989",
	"Livingstone Supongo II (I Premuse II) - Part 1 (48K)\0", NULL, "Opera Soft", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, Speclivipres21RomInfo, Speclivipres21RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecQAOPSpaceDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Livingstone Supongo II (I Premuse II) - Part 2 (48K)

static struct BurnRomInfo Speclivipres22RomDesc[] = {
	{ "Livingstone Supongo II - part 2 48K (1989)(Opera Soft).z80", 36481, 0x851ca3c8, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Speclivipres22, Speclivipres22, Spectrum)
STD_ROM_FN(Speclivipres22)

struct BurnDriver BurnSpeclivipres22 = {
	"spec_livipres22", "spec_livipres21", "spec_spectrum", NULL, "1989",
	"Livingstone Supongo II (I Premuse II) - Part 2 (48K)\0", "Password: 15215", "Opera Soft", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, Speclivipres22RomInfo, Speclivipres22RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecQAOPSpaceDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Mad Mix Game (Spanish) (128K)

static struct BurnRomInfo SpecmadmixgameRomDesc[] = {
	{ "Mad Mix Game (1988)(Topo Soft)(es)(128k).z80", 0x0a9ad, 0xc35eb329, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specmadmixgame, Specmadmixgame, Spec128)
STD_ROM_FN(Specmadmixgame)

struct BurnDriver BurnSpecmadmixgame = {
	"spec_madmixgame", NULL, "spec_spec128", NULL, "1988",
	"Mad Mix Game (Spanish) (128K)\0", NULL, "Topo Soft", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_MAZE | GBF_ACTION, 0,
	SpectrumGetZipName, SpecmadmixgameRomInfo, SpecmadmixgameRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Manchester United Europe (128K)

static struct BurnRomInfo SpecmanutdeuRomDesc[] = {
	{ "Manchester United Europe (1991)(Krisalis).tap", 79751, 0xc8b8d2aa, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specmanutdeu, Specmanutdeu, Spec128)
STD_ROM_FN(Specmanutdeu)

struct BurnDriver BurnSpecmanutdeu = {
	"spec_manutdeu", NULL, "spec_spec128", NULL, "1991",
	"Manchester United Europe (128K)\0", NULL, "Krisalis Software", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 2, HARDWARE_SPECTRUM, GBF_SPORTSFOOTBALL, 0,
	SpectrumGetZipName, SpecmanutdeuRomInfo, SpecmanutdeuRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecIntf2DIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Manic Miner (Bug-Byte Software) (48K)

static struct BurnRomInfo SpecmminerRomDesc[] = {
	{ "Manic Miner (1983)(Bug-Byte Software).z80", 0x06834, 0x024b1971, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specmminer, Specmminer, Spectrum)
STD_ROM_FN(Specmminer)

struct BurnDriver BurnSpecmminer = {
	"spec_mminer", NULL, "spec_spectrum", NULL, "1983",
	"Manic Miner (Bug-Byte Software) (48K)\0", NULL, "Bug-Byte Software", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecmminerRomInfo, SpecmminerRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Manic Miner (Software Projects) (48K)

static struct BurnRomInfo SpecmminerspRomDesc[] = {
	{ "Manic Miner (1983)(Software Projects).z80", 0x05ff0, 0x2187b9dd, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specmminersp, Specmminersp, Spectrum)
STD_ROM_FN(Specmminersp)

struct BurnDriver BurnSpecmminersp = {
	"spec_mminersp", NULL, "spec_spectrum", NULL, "1983",
	"Manic Miner (Software Projects) (48K)\0", NULL, "Software Projects", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecmminerspRomInfo, SpecmminerspRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Marble Madness - Deluxe Edition (48K)

static struct BurnRomInfo SpecmarblemRomDesc[] = {
	{ "Marble Madness - Deluxe Edition (1987)(Melbourne House).z80", 0x0793d, 0x3d08e9ec, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specmarblem, Specmarblem, Spectrum)
STD_ROM_FN(Specmarblem)

struct BurnDriver BurnSpecmarblem = {
	"spec_marblem", NULL, "spec_spectrum", NULL, "1987",
	"Marble Madness - Deluxe Edition (48K)\0", NULL, "Melbourne House", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_ACTION, 0,
	SpectrumGetZipName, SpecmarblemRomInfo, SpecmarblemRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Mario Bros (48K)

static struct BurnRomInfo SpecmaribrosRomDesc[] = {
	{ "Mario Bros (1987)(Ocean).z80", 0x08f38, 0xe42c245c, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specmaribros, Specmaribros, Spectrum)
STD_ROM_FN(Specmaribros)

struct BurnDriver BurnSpecmaribros = {
	"spec_maribros", NULL, "spec_spectrum", NULL, "1987",
	"Mario Bros (48K)\0", NULL, "Ocean", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 2, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecmaribrosRomInfo, SpecmaribrosRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecQAOPMDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Match Day (48K)

static struct BurnRomInfo SpecmatchdayRomDesc[] = {
	{ "Match Day (1985)(Ocean).z80", 0x0a809, 0x59d3bc21, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specmatchday, Specmatchday, Spectrum)
STD_ROM_FN(Specmatchday)

struct BurnDriver BurnSpecmatchday = {
	"spec_matchday", NULL, "spec_spectrum", NULL, "1985",
	"Match Day (48K)\0", NULL, "Ocean", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 2, HARDWARE_SPECTRUM, GBF_SPORTSFOOTBALL, 0,
	SpectrumGetZipName, SpecmatchdayRomInfo, SpecmatchdayRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Match Day II (48K)

static struct BurnRomInfo Specmatchdy2RomDesc[] = {
	{ "Match Day II (1987)(Ocean).z80", 0x09469, 0x910131c2, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specmatchdy2, Specmatchdy2, Spectrum)
STD_ROM_FN(Specmatchdy2)

struct BurnDriver BurnSpecmatchdy2 = {
	"spec_matchdy2", NULL, "spec_spectrum", NULL, "1987",
	"Match Day II (48K)\0", NULL, "Ocean", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 2, HARDWARE_SPECTRUM, GBF_SPORTSFOOTBALL, 0,
	SpectrumGetZipName, Specmatchdy2RomInfo, Specmatchdy2RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Match of the Day (128K)

static struct BurnRomInfo SpecmtchotdRomDesc[] = {
	{ "Match of the Day (1992)(Zeppelin Games)[128K].z80", 0x0cd0a, 0xeb11c05c, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specmtchotd, Specmtchotd, Spec128)
STD_ROM_FN(Specmtchotd)

struct BurnDriver BurnSpecmtchotd = {
	"spec_mtchotd", NULL, "spec_spec128", NULL, "1992",
	"Match of the Day (128K)\0", NULL, "Zeppelin Games", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_STRATEGY, 0,
	SpectrumGetZipName, SpecmtchotdRomInfo, SpecmtchotdRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Merlin (48K)

static struct BurnRomInfo SpecmerlinRomDesc[] = {
	{ "Merlin 48K (1988)(Firebird).z80", 41557, 0xa37c1a27, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specmerlin, Specmerlin, Spectrum)
STD_ROM_FN(Specmerlin)

struct BurnDriver BurnSpecmerlin = {
	"spec_merlin", NULL, "spec_spectrum", NULL, "1987",
	"Merlin (48K)\0", NULL, "Firebird Software", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_ADV, 0,
	SpectrumGetZipName, SpecmerlinRomInfo, SpecmerlinRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Metal Army (48K)

static struct BurnRomInfo SpecmetaarmyRomDesc[] = {
	{ "Metal Army (1988)(Players).tap", 40251, 0xdbced42f, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specmetaarmy, Specmetaarmy, Spectrum)
STD_ROM_FN(Specmetaarmy)

struct BurnDriver BurnSpecmetaarmy = {
	"spec_metaarmy", NULL, "spec_spectrum", NULL, "1988",
	"Metal Army (48K)\0", NULL, "Players Premier", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_RUNGUN | GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecmetaarmyRomInfo, SpecmetaarmyRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Midnight Resistance (48K)

static struct BurnRomInfo Specmresist_48RomDesc[] = {
	{ "Midnight Resistance (1990)(Ocean).z80", 0x0a2e3, 0x70b26b8c, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specmresist_48, Specmresist_48, Spectrum)
STD_ROM_FN(Specmresist_48)

struct BurnDriver BurnSpecmresist_48 = {
	"spec_mresist_48", "spec_mresist", "spec_spectrum", NULL, "1990",
	"Midnight Resistance (48K)\0", NULL, "Ocean", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE, 1, HARDWARE_SPECTRUM, GBF_RUNGUN, 0,
	SpectrumGetZipName, Specmresist_48RomInfo, Specmresist_48RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Midnight Resistance (128K)

static struct BurnRomInfo SpecmresistRomDesc[] = {
	{ "Midnight Resistance (1990)(Ocean)[128K].z80", 0x1b641, 0x614f8f38, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specmresist, Specmresist, Spec128)
STD_ROM_FN(Specmresist)

struct BurnDriver BurnSpecmresist = {
	"spec_mresist", NULL, "spec_spec128", NULL, "1990",
	"Midnight Resistance (128K)\0", NULL, "Ocean", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_RUNGUN, 0,
	SpectrumGetZipName, SpecmresistRomInfo, SpecmresistRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// MoonTorc (48K)

static struct BurnRomInfo SpecmoontorcRomDesc[] = {
	{ "Moontorc (1991)(Atlantis).tap", 58279, 0x0fe238a1, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specmoontorc, Specmoontorc, Spectrum)
STD_ROM_FN(Specmoontorc)

struct BurnDriver BurnSpecmoontorc = {
	"spec_moontorc", NULL, "spec_spectrum", NULL, "1991",
	"MoonTorc (48K)\0", NULL, "Atlantis Software", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_PLATFORM | GBF_ADV, 0,
	SpectrumGetZipName, SpecmoontorcRomInfo, SpecmoontorcRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecQAOPMDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Moonwalker - The Computer Game (128K)

static struct BurnRomInfo SpecmoonwalkRomDesc[] = {
	{ "Moonwalker - The Computer Game (1989)(U.S. Gold).tap", 170496, 0x0c84679e, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specmoonwalk, Specmoonwalk, Spec128)
STD_ROM_FN(Specmoonwalk)

struct BurnDriver BurnSpecmoonwalk = {
	"spec_moonwalk", NULL, "spec_spec128", NULL, "1989",
	"Moonwalker - The Computer Game (128K)\0", NULL, "U.S. Gold", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_ACTION, 0,
	SpectrumGetZipName, SpecmoonwalkRomInfo, SpecmoonwalkRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Mot - Part 1 (Spanish) (48K)

static struct BurnRomInfo Specmotopera1RomDesc[] = {
	{ "Mot (1989)(Opera Soft)(Sp)(48k)[Part 1 of 3].tap", 49084, 0x66889cb5, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specmotopera1, Specmotopera1, Spectrum)
STD_ROM_FN(Specmotopera1)

struct BurnDriver BurnSpecmotopera1 = {
	"spec_motopera1", NULL, "spec_spectrum", NULL, "1989",
	"Mot - Part 1 (Spanish) (48K)\0", NULL, "Opera Soft", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_ACTION, 0,
	SpectrumGetZipName, Specmotopera1RomInfo, Specmotopera1RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecIntf2DIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Mot - Part 2 (Spanish) (48K)

static struct BurnRomInfo Specmotopera2RomDesc[] = {
	{ "Mot (1989)(Opera Soft)(Sp)(48k)[Part 2 of 3].tap", 49084, 0x9b20327d, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specmotopera2, Specmotopera2, Spectrum)
STD_ROM_FN(Specmotopera2)

struct BurnDriver BurnSpecmotopera2 = {
	"spec_motopera2", "spec_motopera1", "spec_spectrum", NULL, "1989",
	"Mot - Part 2 (Spanish) (48K)\0", NULL, "Opera Soft", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE, 1, HARDWARE_SPECTRUM, GBF_ACTION, 0,
	SpectrumGetZipName, Specmotopera2RomInfo, Specmotopera2RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecIntf2DIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Mot - Part 3 (Spanish) (48K)

static struct BurnRomInfo Specmotopera3RomDesc[] = {
	{ "Mot (1989)(Opera Soft)(Sp)(48k)[Part 3 of 3].tap", 49084, 0x0b659f33, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specmotopera3, Specmotopera3, Spectrum)
STD_ROM_FN(Specmotopera3)

struct BurnDriver BurnSpecmotopera3 = {
	"spec_motopera3", "spec_motopera1", "spec_spectrum", NULL, "1989",
	"Mot - Part 3 (Spanish) (48K)\0", NULL, "Opera Soft", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE, 1, HARDWARE_SPECTRUM, GBF_ACTION, 0,
	SpectrumGetZipName, Specmotopera3RomInfo, Specmotopera3RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecIntf2DIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Ms. Pac-Man (48K)

static struct BurnRomInfo SpecmspacmanRomDesc[] = {
	{ "Ms. Pac-Man (1984)(Atarisoft).z80", 0x08e51, 0x168677eb, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specmspacman, Specmspacman, Spectrum)
STD_ROM_FN(Specmspacman)

struct BurnDriver BurnSpecmspacman = {
	"spec_mspacman", NULL, "spec_spectrum", NULL, "1984",
	"Ms. Pac-Man (48K)\0", NULL, "Atarisoft", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 2, HARDWARE_SPECTRUM, GBF_MAZE | GBF_ACTION, 0,
	SpectrumGetZipName, SpecmspacmanRomInfo, SpecmspacmanRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Mundial de Futbol (Spanish) (128K)

static struct BurnRomInfo SpecmundialfutbolRomDesc[] = {
	{ "Mundial de Futbol ES (1990)(Opera Soft).tap", 98168, 0xe41fa6c3, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specmundialfutbol, Specmundialfutbol, Spec128)
STD_ROM_FN(Specmundialfutbol)

struct BurnDriver BurnSpecmundialfutbol = {
	"spec_mundialfutbol", NULL, "spec_spec128", NULL, "1990",
	"Mundial de Futbol (Spanish) (128K)\0", NULL, "Opera Soft", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 2, HARDWARE_SPECTRUM, GBF_SPORTSFOOTBALL, 0,
	SpectrumGetZipName, SpecmundialfutbolRomInfo, SpecmundialfutbolRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Mutan Zone (Spanish) (Part 1 of 2) (128K)

static struct BurnRomInfo SpecmutazoneRomDesc[] = {
	{ "Mutan Zone (1989)(Opera Soft)(es)(Part 1 of 2)(128k).z80", 0x0ae17, 0xc83a4c4d, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specmutazone, Specmutazone, Spec128)
STD_ROM_FN(Specmutazone)

struct BurnDriver BurnSpecmutazone = {
	"spec_mutazone", NULL, "spec_spec128", NULL, "1989",
	"Mutan Zone (Spanish) (Part 1 of 2) (128K)\0", NULL, "Opera Soft", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_RUNGUN, 0,
	SpectrumGetZipName, SpecmutazoneRomInfo, SpecmutazoneRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecQAOPSpaceDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Mutan Zone (Spanish) (Part 2 of 2) (128K)

static struct BurnRomInfo Specmutazone2RomDesc[] = {
	{ "Mutan Zone (1989)(Opera Soft)(es)(Part 2 of 2)(128k).z80", 0x0adf1, 0x24ee7c22, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specmutazone2, Specmutazone2, Spec128)
STD_ROM_FN(Specmutazone2)

struct BurnDriver BurnSpecmutazone2 = {
	"spec_mutazone2", "spec_mutazone", "spec_spec128", NULL, "1989",
	"Mutan Zone (Spanish) (Part 2 of 2) (128K)\0", NULL, "Opera Soft", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE, 1, HARDWARE_SPECTRUM, GBF_RUNGUN, 0,
	SpectrumGetZipName, Specmutazone2RomInfo, Specmutazone2RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecQAOPSpaceDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Navy Moves - Part 1 (Spanish) (48K)

static struct BurnRomInfo Specnavymoves1RomDesc[] = {
	{ "Navy Moves ES - Part 1 (1988)(Dinamic).tap", 47056, 0xb5090163, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specnavymoves1, Specnavymoves1, Spectrum)
STD_ROM_FN(Specnavymoves1)

struct BurnDriver BurnSpecnavymoves1 = {
	"spec_navymoves1", NULL, "spec_spectrum", NULL, "1988",
	"Navy Moves - Part 1 (Spanish) (48K)\0", "Set '3-Sinclair' to use controller", "Dinamic Software", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_ACTION, 0,
	SpectrumGetZipName, Specnavymoves1RomInfo, Specnavymoves1RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecIntf2DIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Navy Moves - Part 2 (Spanish) (48K)

static struct BurnRomInfo Specnavymoves2RomDesc[] = {
	{ "Navy Moves ES - Part 2 (1988)(Dinamic).tap", 46510, 0xc89abc9d, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specnavymoves2, Specnavymoves2, Spectrum)
STD_ROM_FN(Specnavymoves2)

struct BurnDriver BurnSpecnavymoves2 = {
	"spec_navymoves2", "spec_navymoves1", "spec_spectrum", NULL, "1988",
	"Navy Moves - Part 2 (Spanish) (48K)\0", "Password: 63723", "Dinamic Software", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE, 1, HARDWARE_SPECTRUM, GBF_RUNGUN | GBF_PLATFORM, 0,
	SpectrumGetZipName, Specnavymoves2RomInfo, Specnavymoves2RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecIntf2DIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Navy SEALs - Part 1 (128K)

static struct BurnRomInfo SpecnvysealsRomDesc[] = {
	{ "Navy SEALs 128K (1991)(Ocean)(Part1).tap", 136806, 0x01b1a341, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specnvyseals, Specnvyseals, Spec128)
STD_ROM_FN(Specnvyseals)

struct BurnDriver BurnSpecnvyseals = {
	"spec_nvyseals", NULL, "spec_spec128", NULL, "1991",
	"Navy SEALs - Part 1 (128K)\0", NULL, "Ocean", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecnvysealsRomInfo, SpecnvysealsRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Navy SEALs - Part 2 (128K)

static struct BurnRomInfo Specnvyseals2RomDesc[] = {
	{ "Navy SEALs 128K (1991)(Ocean)(Part2).tap", 136805, 0x6b7f3a21, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specnvyseals2, Specnvyseals2, Spec128)
STD_ROM_FN(Specnvyseals2)

struct BurnDriver BurnSpecnvyseals2 = {
	"spec_nvyseals2", "spec_nvyseals", "spec_spec128", NULL, "1991",
	"Navy SEALs - Part 2 (128K)\0", NULL, "Ocean", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE, 1, HARDWARE_SPECTRUM, GBF_RUNGUN, 0,
	SpectrumGetZipName, Specnvyseals2RomInfo, Specnvyseals2RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Nebulus (48K)

static struct BurnRomInfo SpecnebulusRomDesc[] = {
	{ "Nebulus 48K (1987)(Hewson Consultants).tap", 44437, 0x46353552, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specnebulus, Specnebulus, Spectrum)
STD_ROM_FN(Specnebulus)

struct BurnDriver BurnSpecnebulus = {
	"spec_nebulus", NULL, "spec_spectrum", NULL, "1987",
	"Nebulus (48K)\0", NULL, "Hewson Consultants", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 2, HARDWARE_SPECTRUM, GBF_PLATFORM | GBF_ACTION, 0,
	SpectrumGetZipName, SpecnebulusRomInfo, SpecnebulusRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Nemesis - The Final Challenge (48K)

static struct BurnRomInfo SpecnemesisRomDesc[] = {
	{ "Nemesis (1987)(The Hit Squad)[re-release].z80", 0x0a71d, 0xad42684d, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specnemesis, Specnemesis, Spectrum)
STD_ROM_FN(Specnemesis)

struct BurnDriver BurnSpecnemesis = {
	"spec_nemesis", NULL, "spec_spectrum", NULL, "1987",
	"Nemesis - The Final Challenge (48K)\0", NULL, "Konami - The Hit Squad", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 2, HARDWARE_SPECTRUM, GBF_HORSHOOT, 0,
	SpectrumGetZipName, SpecnemesisRomInfo, SpecnemesisRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Olli & Lissa - The Ghost of Shilmoore Castle (48K)

static struct BurnRomInfo SpecollilsaRomDesc[] = {
	{ "Olli & Lissa - The Ghost Of Shilmoore Castle (1986)(Firebird).z80", 0x09f58, 0xed38d8ad, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specollilsa, Specollilsa, Spectrum)
STD_ROM_FN(Specollilsa)

struct BurnDriver BurnSpecollilsa = {
	"spec_ollilsa", NULL, "spec_spectrum", NULL, "1986",
	"Olli & Lissa - The Ghost of Shilmoore Castle (48K)\0", NULL, "Firebird Software", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecollilsaRomInfo, SpecollilsaRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Olli & Lissa II - Halloween (48K)

static struct BurnRomInfo Specollilsa2RomDesc[] = {
	{ "Olli & Lissa II - Halloween (1987)(Firebird).z80", 0x0a6b1, 0x99e59116, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specollilsa2, Specollilsa2, Spectrum)
STD_ROM_FN(Specollilsa2)

struct BurnDriver BurnSpecollilsa2 = {
	"spec_ollilsa2", NULL, "spec_spectrum", NULL, "1987",
	"Olli & Lissa II - Halloween (48K)\0", NULL, "Silverbird Software", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, Specollilsa2RomInfo, Specollilsa2RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Olli & Lissa III - The Candlelight Adventure (48K)

static struct BurnRomInfo Specollilsa3RomDesc[] = {
	{ "Olli & Lissa 3 (1989)(Cartoon Time).tap", 44893, 0xc5703df4, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specollilsa3, Specollilsa3, Spectrum)
STD_ROM_FN(Specollilsa3)

struct BurnDriver BurnSpecollilsa3 = {
	"spec_ollilsa3", NULL, "spec_spectrum", NULL, "1989",
	"Olli & Lissa III - The Candlelight Adventure (48K)\0", NULL, "Codemasters", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, Specollilsa3RomInfo, Specollilsa3RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Operation Thunderbolt (128K)

static struct BurnRomInfo SpecothunderRomDesc[] = {
	{ "Operation Thunderbolt 128K (1989)(Ocean).tap", 138426, 0x16257514, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specothunder, Specothunder, Spec128)
STD_ROM_FN(Specothunder)

struct BurnDriver BurnSpecothunder = {
	"spec_othunder", NULL, "spec_spec128", NULL, "1989",
	"Operation Thunderbolt (128K)\0", NULL, "Ocean", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 2, HARDWARE_SPECTRUM, GBF_SHOOT, 0,
	SpectrumGetZipName, SpecothunderRomInfo, SpecothunderRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Operation Wolf (128K)

static struct BurnRomInfo SpecopwolfRomDesc[] = {
	{ "Operation Wolf (1988)(Ocean)[128K].z80", 0x1da84, 0xa7cd683f, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specopwolf, Specopwolf, Spec128)
STD_ROM_FN(Specopwolf)

struct BurnDriver BurnSpecopwolf = {
	"spec_opwolf", NULL, "spec_spec128", NULL, "1988",
	"Operation Wolf (128K)\0", "Set '2. Sinclair 1' to use controller - Press SPACE for grenades", "Ocean", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_SHOOT, 0,
	SpectrumGetZipName, SpecopwolfRomInfo, SpecopwolfRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecIntf2DIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Pac-Land (128K)

static struct BurnRomInfo SpecpaclandRomDesc[] = {
	{ "Pac-Land (1989)(Grandslam Entertainments)[128K].z80", 0x0b9d7, 0x78fe5d59, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specpacland, Specpacland, Spec128)
STD_ROM_FN(Specpacland)

struct BurnDriver BurnSpecpacland = {
	"spec_pacland", NULL, "spec_spec128", NULL, "1989",
	"Pac-Land (128K)\0", "ENTER to add coins, 1 or 2 to play", "Grandslam Entertainments", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecpaclandRomInfo, SpecpaclandRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Pac-Man (Atarisoft) (48K)

static struct BurnRomInfo SpecpacmanRomDesc[] = {
	{ "Pac-Man (1983)(Atarisoft).z80", 0x035fc, 0xe3c56f6b, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specpacman, Specpacman, Spectrum)
STD_ROM_FN(Specpacman)

struct BurnDriver BurnSpecpacman = {
	"spec_pacman", NULL, "spec_spectrum", NULL, "1983",
	"Pac-Man (Atarisoft) (48K)\0", NULL, "Atarisoft", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_MAZE | GBF_ACTION, 0,
	SpectrumGetZipName, SpecpacmanRomInfo, SpecpacmanRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecQAOPSpaceDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Pac-Mania (128K)

static struct BurnRomInfo SpecpacmaniaRomDesc[] = {
	{ "Pac-Mania (1988)(Grandslam Entertainments)[128K].z80", 0x0a813, 0x938c60ff, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specpacmania, Specpacmania, Spec128)
STD_ROM_FN(Specpacmania)

struct BurnDriver BurnSpecpacmania = {
	"spec_pacmania", NULL, "spec_spec128", NULL, "1988",
	"Pac-Mania (128K)\0", NULL, "Grandslam Entertainments", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_MAZE | GBF_ACTION, 0,
	SpectrumGetZipName, SpecpacmaniaRomInfo, SpecpacmaniaRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Pang (128K)

static struct BurnRomInfo SpecpangRomDesc[] = {
	{ "Pang 128K (1990)(Ocean).tap", 100344, 0x06ce4e01, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specpang, Specpang, Spec128)
STD_ROM_FN(Specpang)

struct BurnDriver BurnSpecpang = {
	"spec_pang", NULL, "spec_spec128", NULL, "1990",
	"Pang (128K)\0", NULL, "Ocean", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 2, HARDWARE_SPECTRUM, GBF_ACTION, 0,
	SpectrumGetZipName, SpecpangRomInfo, SpecpangRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Panic Dizzy (128K)

static struct BurnRomInfo SpecpandizzyRomDesc[] = {
	{ "Panic Dizzy (1991)(Codemasters).tap", 48017, 0xdbc18295, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specpandizzy, Specpandizzy, Spec128)
STD_ROM_FN(Specpandizzy)

struct BurnDriver BurnSpecpandizzy = {
	"spec_pandizzy", NULL, "spec_spec128", NULL, "1991",
	"Panic Dizzy (128K)\0", NULL, "Codemasters", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_PUZZLE, 0,
	SpectrumGetZipName, SpecpandizzyRomInfo, SpecpandizzyRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Paperboy (48K)

static struct BurnRomInfo SpecpaperboyRomDesc[] = {
	{ "Paperboy (1986)(Elite Systems).tap", 51430, 0xac1c032c, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specpaperboy, Specpaperboy, Spectrum)
STD_ROM_FN(Specpaperboy)

struct BurnDriver BurnSpecpaperboy = {
	"spec_paperboy", NULL, "spec_spectrum", NULL, "1986",
	"Paperboy (48K)\0", NULL, "Elite Systems", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_ACTION, 0,
	SpectrumGetZipName, SpecpaperboyRomInfo, SpecpaperboyRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Paperboy 2 (48K)

static struct BurnRomInfo Specpaperby2RomDesc[] = {
	{ "Paperboy 2 (1992)(Mindscape International).z80", 39870, 0x523b2b3b, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specpaperby2, Specpaperby2, Spectrum)
STD_ROM_FN(Specpaperby2)

struct BurnDriver BurnSpecpaperby2 = {
	"spec_paperby2", NULL, "spec_spectrum", NULL, "1992",
	"Paperboy 2 (48K)\0", NULL, "Mindscape International", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_ACTION, 0,
	SpectrumGetZipName, Specpaperby2RomInfo, Specpaperby2RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Passing Shot (128K)

static struct BurnRomInfo SpecpassshotRomDesc[] = {
	{ "Passing Shot (1989)(Image Works).tap", 40394, 0x574083d5, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specpassshot, Specpassshot, Spec128)
STD_ROM_FN(Specpassshot)

struct BurnDriver BurnSpecpassshot = {
	"spec_passshot", NULL, "spec_spec128", NULL, "1989",
	"Passing Shot (128K)\0", "Press '1' to music and '2' to sound effects", "Image Works", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 2, HARDWARE_SPECTRUM, GBF_SPORTSMISC, 0,
	SpectrumGetZipName, SpecpassshotRomInfo, SpecpassshotRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Peter Shilton's Handball Maradona (48K)

static struct BurnRomInfo SpecpshandmRomDesc[] = {
	{ "Peter Shilton's Handball Maradona (1986)(Grandslam Entertainments).z80", 40447, 0x5d4a8e4d, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specpshandm, Specpshandm, Spectrum)
STD_ROM_FN(Specpshandm)

struct BurnDriver BurnSpecpshandm = {
	"spec_pshandm", NULL, "spec_spectrum", NULL, "1986",
	"Peter Shilton's Handball Maradona (48K)\0", NULL, "Grandslam Entertainments", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 2, HARDWARE_SPECTRUM, GBF_SPORTSFOOTBALL, 0,
	SpectrumGetZipName, SpecpshandmRomInfo, SpecpshandmRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Game Over II - Part 1 (aka Phantis) (48K)

static struct BurnRomInfo Specgameover21RomDesc[] = {
	{ "Game Over 2 Part 1 (1988)(Dinamic)[aka Phantis].tap", 48141, 0x8420b778, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specgameover21, Specgameover21, Spectrum)
STD_ROM_FN(Specgameover21)

struct BurnDriver BurnSpecgameover21 = {
	"spec_gameover21", NULL, "spec_spectrum", NULL, "1988",
	"Game Over II - Part 1 (aka Phantis) (48K)\0", NULL, "Dinamic Software", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_HORSHOOT, 0,
	SpectrumGetZipName, Specgameover21RomInfo, Specgameover21RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Game Over II - Part 2 (aka Phantis) (48K)

static struct BurnRomInfo Specgameover22RomDesc[] = {
	{ "Game Over 2 Part 2 (1988)(Dinamic)[aka Phantis].tap", 48155, 0x223202b2, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specgameover22, Specgameover22, Spectrum)
STD_ROM_FN(Specgameover22)

struct BurnDriver BurnSpecgameover22 = {
	"spec_gameover22", "spec_gameover21", "spec_spectrum", NULL, "1988",
	"Game Over II - Part 2 (aka Phantis) (48K)\0", NULL, "Dinamic Software", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE, 1, HARDWARE_SPECTRUM, GBF_ACTION, 0,
	SpectrumGetZipName, Specgameover22RomInfo, Specgameover22RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Phantis - Part 1 (Spanish) (48K)

static struct BurnRomInfo Specphantis1RomDesc[] = {
	{ "Phantis - Part 1 ES (1987)(Dinamic).tap", 48529, 0xca456c10, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specphantis1, Specphantis1, Spectrum)
STD_ROM_FN(Specphantis1)

struct BurnDriver BurnSpecphantis1 = {
	"spec_phantis1", NULL, "spec_spectrum", NULL, "1987",
	"Phantis - Part 1 (Spanish) (48K)\0", NULL, "Dinamic Software", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_HORSHOOT, 0,
	SpectrumGetZipName, Specphantis1RomInfo, Specphantis1RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Phantis - Part 2 (Spanish) (48K)

static struct BurnRomInfo Specphantis2RomDesc[] = {
	{ "Phantis - Part 2 ES (1987)(Dinamic).tap", 48529, 0xb9045126, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specphantis2, Specphantis2, Spectrum)
STD_ROM_FN(Specphantis2)

struct BurnDriver BurnSpecphantis2 = {
	"spec_phantis2", "spec_phantis1", "spec_spectrum", NULL, "1987",
	"Phantis - Part 2 (Spanish) (48K)\0", "Password: 18757", "Dinamic Software", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE, 1, HARDWARE_SPECTRUM, GBF_ACTION, 0,
	SpectrumGetZipName, Specphantis2RomInfo, Specphantis2RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Phoenix - Text Adventure (48K)

static struct BurnRomInfo SpecphoenixRomDesc[] = {
	{ "Phoenix (1991)(Zenobi).tap", 37722, 0x8cf4e0e2, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specphoenix, Specphoenix, Spectrum)
STD_ROM_FN(Specphoenix)

struct BurnDriver BurnSpecphoenix = {
	"spec_phoenix", NULL, "spec_spectrum", NULL, "1991",
	"Phoenix - Text Adventure (48K)\0", NULL, "Zenobi Software", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_ADV, 0,
	SpectrumGetZipName, SpecphoenixRomInfo, SpecphoenixRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Pippo (48K)

static struct BurnRomInfo SpecpippoRomDesc[] = {
	{ "Pippo 48K (1986)(Mastertronic).tap", 42906, 0xbf464d08, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specpippo, Specpippo, Spectrum)
STD_ROM_FN(Specpippo)

struct BurnDriver BurnSpecpippo = {
	"spec_pippo", NULL, "spec_spectrum", NULL, "1986",
	"Pippo (48K)\0", NULL, "Mastertronic", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_ACTION, 0,
	SpectrumGetZipName, SpecpippoRomInfo, SpecpippoRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Pixy the Microdot 2 (48K)

static struct BurnRomInfo Specpixymicrod2RomDesc[] = {
	{ "Pixy the Microdot 2 (1992)(Your Sinclair).z80", 40323, 0x9de92490, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specpixymicrod2, Specpixymicrod2, Spectrum)
STD_ROM_FN(Specpixymicrod2)

struct BurnDriver BurnSpecpixymicrod2 = {
	"spec_pixymicrod2", NULL, "spec_spectrum", NULL, "1992",
	"Pixy the Microdot 2 (48K)\0", NULL, "Your Sinclair", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_ACTION, 0,
	SpectrumGetZipName, Specpixymicrod2RomInfo, Specpixymicrod2RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecQAOPSpaceDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Platoon (128K)

static struct BurnRomInfo SpecplatoonRomDesc[] = {
	{ "Platoon (1988)(Ocean)[128K].z80", 106785, 0xaa4d4d13, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specplatoon, Specplatoon, Spec128)
STD_ROM_FN(Specplatoon)

struct BurnDriver BurnSpecplatoon = {
	"spec_platoon", NULL, "spec_spec128", NULL, "1988",
	"Platoon (128K)\0", NULL, "Ocean", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_SHOOT, 0,
	SpectrumGetZipName, SpecplatoonRomInfo, SpecplatoonRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Plotting (128K)

static struct BurnRomInfo SpecplottingRomDesc[] = {
	{ "Plotting (1990)(Ocean)[128K].z80", 40883, 0xdd65a0b5, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specplotting, Specplotting, Spec128)
STD_ROM_FN(Specplotting)

struct BurnDriver BurnSpecplotting = {
	"spec_plotting", NULL, "spec_spec128", NULL, "1990",
	"Plotting (128K)\0", NULL, "Ocean", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_PUZZLE, 0,
	SpectrumGetZipName, SpecplottingRomInfo, SpecplottingRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Pole Position (Atarisoft) (48K)

static struct BurnRomInfo SpecpoleposaRomDesc[] = {
	{ "Pole Position (1984)(Atarisoft).tap", 47760, 0xfc80c755, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specpoleposa, Specpoleposa, Spectrum)
STD_ROM_FN(Specpoleposa)

struct BurnDriver BurnSpecpoleposa = {
	"spec_poleposa", NULL, "spec_spectrum", NULL, "1984",
	"Pole Position (48K)\0", "Up or down to shift gear, fire button to brake", "Atarisoft", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_RACING, 0,
	SpectrumGetZipName, SpecpoleposaRomInfo, SpecpoleposaRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Poli Diaz (Spanish) (128K)

static struct BurnRomInfo SpecpolidiazRomDesc[] = {
	{ "Poli Diaz (1990)(Opera Soft)(es)(128k).z80", 0x0b232, 0x635c6283, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specpolidiaz, Specpolidiaz, Spec128)
STD_ROM_FN(Specpolidiaz)

struct BurnDriver BurnSpecpolidiaz = {
	"spec_polidiaz", NULL, "spec_spec128", NULL, "1990",
	"Poli Diaz (Spanish) (128K)\0", NULL, "Opera Soft", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 2, HARDWARE_SPECTRUM, GBF_VSFIGHT, 0,
	SpectrumGetZipName, SpecpolidiazRomInfo, SpecpolidiazRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Power Drift (128K)

static struct BurnRomInfo SpecpdriftRomDesc[] = {
	{ "Power Drift (1989)(Activision)[128K].z80", 0x160f6, 0x34a7f74a, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specpdrift, Specpdrift, Spec128)
STD_ROM_FN(Specpdrift)

struct BurnDriver BurnSpecpdrift = {
	"spec_pdrift", NULL, "spec_spec128", NULL, "1989",
	"Power Drift (128K)\0", NULL, "Activision", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_RACING, 0,
	SpectrumGetZipName, SpecpdriftRomInfo, SpecpdriftRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Puzznic (128K)

static struct BurnRomInfo SpecpuzznicRomDesc[] = {
	{ "Puzznic (1990)(Ocean).tap", 33679, 0xeceabdd0, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specpuzznic, Specpuzznic, Spec128)
STD_ROM_FN(Specpuzznic)

struct BurnDriver BurnSpecpuzznic = {
	"spec_puzznic", NULL, "spec_spec128", NULL, "1990",
	"Puzznic (128K)\0", NULL, "Ocean", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_PUZZLE, 0,
	SpectrumGetZipName, SpecpuzznicRomInfo, SpecpuzznicRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecQAOPSpaceDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Quartet (48K)

static struct BurnRomInfo SpecquartetRomDesc[] = {
	{ "Quartet (1987)(Activision).z80", 0x08a9e, 0x45711e73, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specquartet, Specquartet, Spectrum)
STD_ROM_FN(Specquartet)

struct BurnDriver BurnSpecquartet = {
	"spec_quartet", NULL, "spec_spectrum", NULL, "1987",
	"Quartet (48K)\0", "To start a game session press 'S' and after ENTER", "Activision", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 4, HARDWARE_SPECTRUM, GBF_RUNGUN, 0,
	SpectrumGetZipName, SpecquartetRomInfo, SpecquartetRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Quazatron (48K)

static struct BurnRomInfo SpecquazatroRomDesc[] = {
	{ "Quazatron (1986)(Hewson Consultants).z80", 0x07e39, 0xdf931658, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specquazatro, Specquazatro, Spectrum)
STD_ROM_FN(Specquazatro)

struct BurnDriver BurnSpecquazatro = {
	"spec_quazatro", NULL, "spec_spectrum", NULL, "1986",
	"Quazatron (48K)\0", NULL, "Hewson Consultants", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_ACTION, 0,
	SpectrumGetZipName, SpecquazatroRomInfo, SpecquazatroRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Rainbow Islands - The Story of Bubble Bobble 2 (48K)

static struct BurnRomInfo Specrbisland_48RomDesc[] = {
	{ "Rainbow Islands - The Story of Bubble Bobble 2 (1990)(Ocean).z80", 0x07da0, 0xad2c841f, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specrbisland_48, Specrbisland_48, Spectrum)
STD_ROM_FN(Specrbisland_48)

struct BurnDriver BurnSpecrbisland_48 = {
	"spec_rbisland_48", "spec_rbisland", "spec_spectrum", NULL, "1990",
	"Rainbow Islands - The Story of Bubble Bobble 2 (48K)\0", "Press ENTER to add 'coins'", "Ocean", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, Specrbisland_48RomInfo, Specrbisland_48RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Rainbow Islands - The Story of Bubble Bobble 2 (128K)

static struct BurnRomInfo SpecrbislandRomDesc[] = {
	{ "Rainbow Islands - The Story of Bubble Bobble 2 (1990)(Ocean)[128K].z80", 0x161bb, 0x0211cd1d, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specrbisland, Specrbisland, Spec128)
STD_ROM_FN(Specrbisland)

struct BurnDriver BurnSpecrbisland = {
	"spec_rbisland", NULL, "spec_spec128", NULL, "1990",
	"Rainbow Islands - The Story of Bubble Bobble 2 (128K)\0", "Press ENTER to add 'coins'", "Ocean", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecrbislandRomInfo, SpecrbislandRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// R.A.M. (Spanish) (48K)

static struct BurnRomInfo SpecramtopoRomDesc[] = {
	{ "R.A.M. ES (1990)(Topo Soft).tap", 62593, 0xeaaf3bd1, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specramtopo, Specramtopo, Spectrum)
STD_ROM_FN(Specramtopo)

struct BurnDriver BurnSpecramtopo = {
	"spec_ramtopo", NULL, "spec_spectrum", NULL, "1990",
	"R.A.M. (Spanish) (48K)\0", NULL, "Topo Soft", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_RUNGUN, 0,
	SpectrumGetZipName, SpecramtopoRomInfo, SpecramtopoRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Renegade (48K)

static struct BurnRomInfo Specrenegade48RomDesc[] = {
	{ "Renegade 48K (1987)(Imagine).tap", 48977, 0x0cd81e69, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specrenegade48, Specrenegade48, Spectrum)
STD_ROM_FN(Specrenegade48)

struct BurnDriver BurnSpecrenegade48 = {
	"spec_renegade48", "spec_renegade128", "spec_spectrum", NULL, "1987",
	"Renegade (48K)\0", NULL, "Imagine Software", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE, 1, HARDWARE_SPECTRUM, GBF_SCRFIGHT, 0,
	SpectrumGetZipName, Specrenegade48RomInfo, Specrenegade48RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Renegade (128K)

static struct BurnRomInfo Specrenegade128RomDesc[] = {
	{ "Renegade 128K (1987)(Imagine).z80", 93965, 0xcd930d9a, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specrenegade128, Specrenegade128, Spec128)
STD_ROM_FN(Specrenegade128)

struct BurnDriver BurnSpecrenegade128 = {
	"spec_renegade128", NULL, "spec_spec128", NULL, "1987",
	"Renegade (128K)\0", NULL, "Imagine Software", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_SCRFIGHT, 0,
	SpectrumGetZipName, Specrenegade128RomInfo, Specrenegade128RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Renegade II - Target Renegade (128K)

static struct BurnRomInfo Specrenegad2RomDesc[] = {
	{ "Renegade II - Target Renegade (1988)(Imagine Software)[128K].z80", 108880, 0x25d57e2c, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specrenegad2, Specrenegad2, Spec128)
STD_ROM_FN(Specrenegad2)

struct BurnDriver BurnSpecrenegad2 = {
	"spec_renegad2", NULL, "spec_spec128", NULL, "1988",
	"Renegade II - Target Renegade (128K)\0", NULL, "Imagine Software", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 2, HARDWARE_SPECTRUM, GBF_SCRFIGHT, 0,
	SpectrumGetZipName, Specrenegad2RomInfo, Specrenegad2RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Renegade III - The Final Chapter (128K)

static struct BurnRomInfo Specrenegad3RomDesc[] = {
	{ "Renegade III - The Final Chapter 128K (1989)(Imagine).tap", 79244, 0xd0c2761b, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specrenegad3, Specrenegad3, Spec128)
STD_ROM_FN(Specrenegad3)

struct BurnDriver BurnSpecrenegad3 = {
	"spec_renegad3", NULL, "spec_spec128", NULL, "1989",
	"Renegade III - The Final Chapter (128K)\0", NULL, "Imagine Software", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_SCRFIGHT, 0,
	SpectrumGetZipName, Specrenegad3RomInfo, Specrenegad3RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Rex (aka Zenith) - Part 1 (128K)

static struct BurnRomInfo SpecrexRomDesc[] = {
	{ "Rex (aka Zenith) - Part 1 (1988)(Martech Games).tap", 48077, 0x7284eb38, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specrex, Specrex, Spec128)
STD_ROM_FN(Specrex)

struct BurnDriver BurnSpecrex = {
	"spec_rex", NULL, "spec_spec128", NULL, "1988",
	"Rex (aka Zenith) - Part 1 (128K)\0", NULL, "Martech Games", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_RUNGUN | GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecrexRomInfo, SpecrexRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Rex (aka Zenith) - Part 2 (128K)

static struct BurnRomInfo Specrex2RomDesc[] = {
	{ "Rex (aka Zenith) - Part 2 (1988)(Martech Games).tap", 47681, 0x7a54760c, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specrex2, Specrex2, Spec128)
STD_ROM_FN(Specrex2)

struct BurnDriver BurnSpecrex2 = {
	"spec_rex2", "spec_rex", "spec_spec128", NULL, "1988",
	"Rex (aka Zenith) - Part 2 (128K)\0", "Password: 8881808384829507", "Martech Games", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE, 1, HARDWARE_SPECTRUM, GBF_RUNGUN | GBF_PLATFORM, 0,
	SpectrumGetZipName, Specrex2RomInfo, Specrex2RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Rick Dangerous (128K)

static struct BurnRomInfo SpecrickdangRomDesc[] = {
	{ "Rick Dangerous 128K (1989)(Firebird).tap", 48387, 0x48b1600d, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specrickdang, Specrickdang, Spec128)
STD_ROM_FN(Specrickdang)

struct BurnDriver BurnSpecrickdang = {
	"spec_rickdang", NULL, "spec_spec128", NULL, "1989",
	"Rick Dangerous (128K)\0", NULL, "Firebird Software", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecrickdangRomInfo, SpecrickdangRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Rick Dangerous 2 (128K)

static struct BurnRomInfo Specrickdang2RomDesc[] = {
	{ "Rick Dangerous 2 (1990)(Micro Style).tap", 0x26599, 0x5cf79480, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specrickdang2, Specrickdang2, Spec128)
STD_ROM_FN(Specrickdang2)

struct BurnDriver BurnSpecrickdang2 = {
	"spec_rickdang2", NULL, "spec_spec128", NULL, "1990",
	"Rick Dangerous 2 (128K)\0", NULL, "Micro Style", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, Specrickdang2RomInfo, Specrickdang2RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Robocop (128K)

static struct BurnRomInfo SpecrobocopRomDesc[] = {
	{ "Robocop (1988)(Ocean)[128K].z80", 0x1cbf8, 0xdcc4bf16, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specrobocop, Specrobocop, Spec128)
STD_ROM_FN(Specrobocop)

struct BurnDriver BurnSpecrobocop = {
	"spec_robocop", NULL, "spec_spec128", NULL, "1988",
	"Robocop (128K)\0", NULL, "Ocean", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_RUNGUN, 0,
	SpectrumGetZipName, SpecrobocopRomInfo, SpecrobocopRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Robocop 2 (128K)

static struct BurnRomInfo Specrobocop2RomDesc[] = {
	{ "Robocop 2 (1990)(Ocean)[128K].z80", 0x1c73e, 0xe9b44bc7, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specrobocop2, Specrobocop2, Spec128)
STD_ROM_FN(Specrobocop2)

struct BurnDriver BurnSpecrobocop2 = {
	"spec_robocop2", NULL, "spec_spec128", NULL, "1990",
	"Robocop 2 (128K)\0", NULL, "Ocean", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_RUNGUN | GBF_PLATFORM, 0,
	SpectrumGetZipName, Specrobocop2RomInfo, Specrobocop2RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Robocop 3 (128K)

static struct BurnRomInfo Specrobocop3RomDesc[] = {
	{ "Robocop 3 (1992)(Ocean)[128K].z80", 0x1ac8a, 0x21b5c6b7, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specrobocop3, Specrobocop3, Spec128)
STD_ROM_FN(Specrobocop3)

struct BurnDriver BurnSpecrobocop3 = {
	"spec_robocop3", NULL, "spec_spec128", NULL, "1992",
	"Robocop 3 (128K)\0", NULL, "Ocean", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_SHOOT, 0,
	SpectrumGetZipName, Specrobocop3RomInfo, Specrobocop3RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecIntf2DIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Rock 'n Roller (Spanish) (48K)

static struct BurnRomInfo SpecrocknrollerRomDesc[] = {
	{ "Rock'n Roller ES (1988)(Topo Soft).z80", 43965, 0x16da9309, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specrocknroller, Specrocknroller, Spectrum)
STD_ROM_FN(Specrocknroller)

struct BurnDriver BurnSpecrocknroller = {
	"spec_rocknroller", NULL, "spec_spectrum", NULL, "1988",
	"Rock 'n Roller (Spanish) (48K)\0", NULL, "Topo Soft", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_ACTION, 0,
	SpectrumGetZipName, SpecrocknrollerRomInfo, SpecrocknrollerRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Rock Star Ate my Hamster (128K)

static struct BurnRomInfo SpecrockshamRomDesc[] = {
	{ "Rock Star Ate My Hamster! 128K (1989)(Codemasters).tap", 67285, 0xba0efbf5, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specrocksham, Specrocksham, Spec128)
STD_ROM_FN(Specrocksham)

struct BurnDriver BurnSpecrocksham = {
	"spec_rocksham", NULL, "spec_spec128", NULL, "1989",
	"Rock Star Ate my Hamster (128K)\0", NULL, "Codemasters", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_STRATEGY, 0,
	SpectrumGetZipName, SpecrockshamRomInfo, SpecrockshamRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Rolling Thunder (48K)

static struct BurnRomInfo SpecrollthunRomDesc[] = {
	{ "Rolling Thunder 48K (1988)(U.S. Gold).tap", 42913, 0x796b68c6, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specrollthun, Specrollthun, Spectrum)
STD_ROM_FN(Specrollthun)

struct BurnDriver BurnSpecrollthun = {
	"spec_rollthun", NULL, "spec_spectrum", NULL, "1988",
	"Rolling Thunder (48K)\0", NULL, "U.S. Gold", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_RUNGUN | GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecrollthunRomInfo, SpecrollthunRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Satan - Part 1 (48K)

static struct BurnRomInfo SpecsatanRomDesc[] = {
	{ "Satan Part 1 (1990)(Dinamic).tap", 48395, 0x2098b208, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specsatan, Specsatan, Spectrum)
STD_ROM_FN(Specsatan)

struct BurnDriver BurnSpecsatan = {
	"spec_satan", NULL, "spec_spectrum", NULL, "1990",
	"Satan - Part 1 (48K)\0", NULL, "Dinamic Software", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecsatanRomInfo, SpecsatanRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Satan - Part 2 (48K)

static struct BurnRomInfo Specsatan2RomDesc[] = {
	{ "Satan Part 2 (1990)(Dinamic).tap", 47945, 0xb6586a99, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specsatan2, Specsatan2, Spectrum)
STD_ROM_FN(Specsatan2)

struct BurnDriver BurnSpecsatan2 = {
	"spec_satan2", "spec_satan", "spec_spectrum", NULL, "1990",
	"Satan - Part 2 (48K)\0", "Password: 01020304", "Dinamic Software", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, Specsatan2RomInfo, Specsatan2RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// S.D.I. - Strategic Defence Initiative (48K)

static struct BurnRomInfo SpecsdisdiRomDesc[] = {
	{ "S.D.I. -  Strategic Defense Initiative 48K (1988)(Activision).tap", 48053, 0xaedbad94, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specsdisdi, Specsdisdi, Spectrum)
STD_ROM_FN(Specsdisdi)

struct BurnDriver BurnSpecsdisdi = {
	"spec_sdisdi", NULL, "spec_spectrum", NULL, "1988",
	"S.D.I. - Strategic Defence Initiative (48K)\0", NULL, "Activision", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 2, HARDWARE_SPECTRUM, GBF_HORSHOOT, 0,
	SpectrumGetZipName, SpecsdisdiRomInfo, SpecsdisdiRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Saboteur! (48K)

static struct BurnRomInfo Specsabot_48RomDesc[] = {
	{ "Saboteur 48K (1985)(Durell).z80", 39664, 0xa7a89067, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specsabot_48, Specsabot_48, Spectrum)
STD_ROM_FN(Specsabot_48)

struct BurnDriver BurnSpecsabot_48 = {
	"spec_sabot_48", NULL, "spec_spectrum", NULL, "1985",
	"Saboteur! (48K)\0", NULL, "Durell Software", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_ACTION, 0,
	SpectrumGetZipName, Specsabot_48RomInfo, Specsabot_48RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Saboteur 2: Avenging Angel (48K)

static struct BurnRomInfo Specsabotur2std_48RomDesc[] = {
	{ "Saboteur 2 - Avenging Angel 48K (1987)(Durell).z80", 0x09ab5, 0x8904b9ba, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specsabotur2std_48, Specsabotur2std_48, Spectrum)
STD_ROM_FN(Specsabotur2std_48)

struct BurnDriver BurnSpecsabotur2std_48 = {
	"spec_sabotur2std_48", "spec_sabotur2", "spec_spectrum", NULL, "1987",
	"Saboteur 2: Avenging Angel (48K)\0", NULL, "Durell Software", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE, 1, HARDWARE_SPECTRUM, GBF_ACTION, 0,
	SpectrumGetZipName, Specsabotur2std_48RomInfo, Specsabotur2std_48RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Saboteur 2: Avenging Angel (128K)

static struct BurnRomInfo Specsabotur2RomDesc[] = {
	{ "Saboteur 2 - Avenging Angel 128K (1987)(Durell).z80", 0x0b790, 0x7aad77db, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specsabotur2, Specsabotur2, Spec128)
STD_ROM_FN(Specsabotur2)

struct BurnDriver BurnSpecsabotur2 = {
	"spec_sabotur2", NULL, "spec_spec128", NULL, "1987",
	"Saboteur 2: Avenging Angel (128K)\0", NULL, "Durell Software", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_ACTION, 0,
	SpectrumGetZipName, Specsabotur2RomInfo, Specsabotur2RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Salamander (48K)

static struct BurnRomInfo SpecsalamandRomDesc[] = {
	{ "Salamander (1987)(Imagine Software).z80", 0x0a680, 0x5ae35d91, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specsalamand, Specsalamand, Spectrum)
STD_ROM_FN(Specsalamand)

struct BurnDriver BurnSpecsalamand = {
	"spec_salamand", NULL, "spec_spectrum", NULL, "1987",
	"Salamander (48K)\0", NULL, "Imagine Software", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 2, HARDWARE_SPECTRUM, GBF_HORSHOOT | GBF_VERSHOOT, 0,
	SpectrumGetZipName, SpecsalamandRomInfo, SpecsalamandRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Savage - Intro (48K)

static struct BurnRomInfo SpecsavageiRomDesc[] = {
	{ "Savage, The (1988)(Firebird)[intro].z80", 31023, 0xe606ebf0, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specsavagei, Specsavagei, Spectrum)
STD_ROM_FN(Specsavagei)

struct BurnDriver BurnSpecsavagei = {
	"spec_savagei", NULL, "spec_spectrum", NULL, "1988",
	"Savage - Intro (48K)\0", NULL, "Firebird Software", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_MISC, 0,
	SpectrumGetZipName, SpecsavageiRomInfo, SpecsavageiRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Savage - Part 1 (48K)

static struct BurnRomInfo Specsavage1RomDesc[] = {
	{ "Savage, The (1988)(Firebird)[Part 1 of 3].z80", 37764, 0x3db88855, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specsavage1, Specsavage1, Spectrum)
STD_ROM_FN(Specsavage1)

struct BurnDriver BurnSpecsavage1 = {
	"spec_savage1", "spec_savagei", "spec_spectrum", NULL, "1988",
	"Savage - Part 1 (48K)\0", NULL, "Firebird Software", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE, 1, HARDWARE_SPECTRUM, GBF_RUNGUN, 0,
	SpectrumGetZipName, Specsavage1RomInfo, Specsavage1RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Savage - Part 2 (48K)

static struct BurnRomInfo Specsavage2RomDesc[] = {
	{ "Savage, The (1988)(Firebird)[Part 2 of 3].z80", 0x09a20, 0x4f8ddec1, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specsavage2, Specsavage2, Spectrum)
STD_ROM_FN(Specsavage2)

struct BurnDriver BurnSpecsavage2 = {
	"spec_savage2", "spec_savagei", "spec_spectrum", NULL, "1988",
	"Savage - Part 2 (48K)\0", "Level code: SABATTA", "Firebird Software", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE, 1, HARDWARE_SPECTRUM, GBF_SHOOT, 0,
	SpectrumGetZipName, Specsavage2RomInfo, Specsavage2RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Savage - Part 3  (48K)

static struct BurnRomInfo Specsavage3RomDesc[] = {
	{ "Savage, The (1988)(Firebird)[Part 3 of 3].z80", 0x08475, 0xe994f627, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specsavage3, Specsavage3, Spectrum)
STD_ROM_FN(Specsavage3)

struct BurnDriver BurnSpecsavage3 = {
	"spec_savage3", "spec_savagei", "spec_spectrum", NULL, "1988",
	"Savage - Part 3 (48K)\0", "Level code: FERGUS", "Firebird Software", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE, 1, HARDWARE_SPECTRUM, GBF_RUNGUN, 0,
	SpectrumGetZipName, Specsavage3RomInfo, Specsavage3RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Scramble Spirits (128K)

static struct BurnRomInfo SpecscrspirtRomDesc[] = {
	{ "Scramble Spirits 128K (1990)(Grandslam).tap", 105793, 0x05c9e40e, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specscrspirt, Specscrspirt, Spec128)
STD_ROM_FN(Specscrspirt)

struct BurnDriver BurnSpecscrspirt = {
	"spec_scrspirt", NULL, "spec_spec128", NULL, "1990",
	"Scramble Spirits (128K)\0", "NB: in Kempston controller the x-asis is reversed", "Grandslam Entertainments", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 2, HARDWARE_SPECTRUM, GBF_VERSHOOT, 0,
	SpectrumGetZipName, SpecscrspirtRomInfo, SpecscrspirtRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Seymour At The Movies (48K)

static struct BurnRomInfo Specseymmovi_48RomDesc[] = {
	{ "Seymour At The Movies (1991)(Codemasters)(48K).z80", 0x0b3c7, 0xc91a5fa5, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specseymmovi_48, Specseymmovi_48, Spectrum)
STD_ROM_FN(Specseymmovi_48)

struct BurnDriver BurnSpecseymmovi = {
	"spec_seymmovi_48", "spec_seymmovi", "spec_spectrum", NULL, "1991",
	"Seymour at the Movies (48K)\0", NULL, "Codemasters", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE, 1, HARDWARE_SPECTRUM, GBF_ADV, 0,
	SpectrumGetZipName, Specseymmovi_48RomInfo, Specseymmovi_48RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Seymour At The Movies (128K)

static struct BurnRomInfo SpecSeymmoviRomDesc[] = {
	{ "Seymour At the Movies (1991)(Codemasters)(128K).z80", 79527, 0xd1a72ae0, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecSeymmovi, SpecSeymmovi, Spec128)
STD_ROM_FN(SpecSeymmovi)

struct BurnDriver BurnSpecSeymmovi = {
	"spec_seymmovi", NULL, "spec_spec128", NULL, "1991",
	"Seymour At The Movies (128K)\0", NULL, "Codemasters", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_ADV, 0,
	SpectrumGetZipName, SpecSeymmoviRomInfo, SpecSeymmoviRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Seymour Take One (48K)

static struct BurnRomInfo SpecseytakonRomDesc[] = {
	{ "Seymour Take One (1991)(Codemasters).z80", 0x0858a, 0x21645f38, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specseytakon, Specseytakon, Spectrum)
STD_ROM_FN(Specseytakon)

struct BurnDriver BurnSpecseytakon = {
	"spec_seytakon", NULL, "spec_spectrum", NULL, "1991",
	"Seymour Take One (48K)\0", NULL, "Codemasters", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_ADV, 0,
	SpectrumGetZipName, SpecseytakonRomInfo, SpecseytakonRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecIntf2DIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Shadow Warriors (128K)

static struct BurnRomInfo SpecshadwarrRomDesc[] = {
	{ "Shadow Warriors (1990)(Ocean).tap", 130491, 0xa9adbe6d, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specshadwarr, Specshadwarr, Spec128)
STD_ROM_FN(Specshadwarr)

struct BurnDriver BurnSpecshadwarr = {
	"spec_shadwarr", NULL, "spec_spec128", NULL, "1990",
	"Shadow Warriors (128K)\0", NULL, "Ocean", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_SCRFIGHT, 0,
	SpectrumGetZipName, SpecshadwarrRomInfo, SpecshadwarrRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecIntf2DIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Shinobi (128K)

static struct BurnRomInfo SpecshinobiRomDesc[] = {
	{ "Shinobi (1989)(Mastertronic Plus).tap", 231589, 0x195feaad, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specshinobi, Specshinobi, Spec128)
STD_ROM_FN(Specshinobi)

struct BurnDriver BurnSpecshinobi = {
	"spec_shinobi", NULL, "spec_spec128", NULL, "1989",
	"Shinobi (128K)\0", NULL, "Mastertronic Plus", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_SCRFIGHT | GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecshinobiRomInfo, SpecshinobiRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Silkworm (128K)

static struct BurnRomInfo SpecsilkwormRomDesc[] = {
	{ "Silkworm 128K (1989)(Mastertronic Plus).tap", 44612, 0x1980dccc, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specsilkworm, Specsilkworm, Spec128)
STD_ROM_FN(Specsilkworm)

struct BurnDriver BurnSpecsilkworm = {
	"spec_silkworm", NULL, "spec_spec128", NULL, "1989",
	"Silkworm (128K)\0", NULL, "Mastertronic Plus", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 2, HARDWARE_SPECTRUM, GBF_HORSHOOT, 0,
	SpectrumGetZipName, SpecsilkwormRomInfo, SpecsilkwormRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Sim City (48K)

static struct BurnRomInfo SpecsimcityRomDesc[] = {
	{ "Sim City (1989)(Infogrames).z80", 34657, 0xd7281b2e, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specsimcity, Specsimcity, Spectrum)
STD_ROM_FN(Specsimcity)

struct BurnDriver BurnSpecsimcity = {
	"spec_simcity", NULL, "spec_spectrum", NULL, "1989",
	"Sim City (48K)\0", NULL, "Infogrames", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_STRATEGY, 0,
	SpectrumGetZipName, SpecsimcityRomInfo, SpecsimcityRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecQAOPSpaceDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Simulador Profesional de Tenis (Spanish) (48K-128K)

static struct BurnRomInfo SpecsimulprotenisRomDesc[] = {
	{ "Simulador Profesional de Tenis ES 128K (1990)(Dinamic).tap", 53988, 0x2a031f63, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specsimulprotenis, Specsimulprotenis, Spec128)
STD_ROM_FN(Specsimulprotenis)

struct BurnDriver BurnSpecsimulprotenis = {
	"spec_simulprotenis", NULL, "spec_spec128", NULL, "1990",
	"Simulador Profesional de Tenis (Spanish) (48K-128K)\0", NULL, "Dinamic Software", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 2, HARDWARE_SPECTRUM, GBF_SPORTSMISC, 0,
	SpectrumGetZipName, SpecsimulprotenisRomInfo, SpecsimulprotenisRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Sito Pons 500cc Grand Prix (Spanish) (48K)

static struct BurnRomInfo SpecsitoponsgpRomDesc[] = {
	{ "Sito Pons 500cc Grand Prix ES 48K (1990)(Zigurat).tap", 49795, 0x1afbe22f, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specsitoponsgp, Specsitoponsgp, Spectrum)
STD_ROM_FN(Specsitoponsgp)

struct BurnDriver BurnSpecsitoponsgp = {
	"spec_sitoponsgp", NULL, "spec_spectrum", NULL, "1990",
	"Sito Pons 500cc Grand Prix (Spanish) (48K)\0", "Press 'Z' to quit the game session", "Zigurat Software", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_RACING, 0,
	SpectrumGetZipName, SpecsitoponsgpRomInfo, SpecsitoponsgpRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Skool Daze (48K)

static struct BurnRomInfo SpecskoldazeRomDesc[] = {
	{ "Skool Daze (1985)(Microsphere).z80", 0x0a5f5, 0x4034c78b, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specskoldaze, Specskoldaze, Spectrum)
STD_ROM_FN(Specskoldaze)

struct BurnDriver BurnSpecskoldaze = {
	"spec_skoldaze", NULL, "spec_spectrum", NULL, "1985",
	"Skool Daze (48K)\0", NULL, "Microsphere", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_ADV, 0,
	SpectrumGetZipName, SpecskoldazeRomInfo, SpecskoldazeRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecCursorKeysDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Slap Fight (48K)

static struct BurnRomInfo SpecslapfighRomDesc[] = {
	{ "Slap Fight (1987)(Imagine).tap", 47183, 0xda714b33, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specslapfigh, Specslapfigh, Spectrum)
STD_ROM_FN(Specslapfigh)

struct BurnDriver BurnSpecslapfigh = {
	"spec_slapfigh", NULL, "spec_spectrum", NULL, "1987",
	"Slap Fight (48K)\0", NULL, "Imagine Software", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 2, HARDWARE_SPECTRUM, GBF_VERSHOOT, 0,
	SpectrumGetZipName, SpecslapfighRomInfo, SpecslapfighRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Slightly Magic (128K)

static struct BurnRomInfo SpecslightmgRomDesc[] = {
	{ "Slightly Magic (1990)(Codemasters)[128K].z80", 0x11bed, 0x12aaa197, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specslightmg, Specslightmg, Spec128)
STD_ROM_FN(Specslightmg)

struct BurnDriver BurnSpecslightmg = {
	"spec_slightmg", NULL, "spec_spec128", NULL, "1990",
	"Slightly Magic (128K)\0", NULL, "Codemasters", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_PLATFORM | GBF_ADV, 0,
	SpectrumGetZipName, SpecslightmgRomInfo, SpecslightmgRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Smash TV (128K)

static struct BurnRomInfo SpecsmashtvRomDesc[] = {
	{ "Smash TV (1991)(Ocean)[128K].z80", 0x0e6c1, 0x2f90973d, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specsmashtv, Specsmashtv, Spec128)
STD_ROM_FN(Specsmashtv)

struct BurnDriver BurnSpecsmashtv = {
	"spec_smashtv", NULL, "spec_spec128", NULL, "1991",
	"Smash TV (128K)\0", NULL, "Ocean", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_RUNGUN, 0,
	SpectrumGetZipName, SpecsmashtvRomInfo, SpecsmashtvRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Soldier of Fortune (48K)

static struct BurnRomInfo SpecsoldfortRomDesc[] = {
	{ "Soldier Of Fortune (1988)(Firebird).z80", 0x0adff, 0xc3dc26df, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specsoldfort, Specsoldfort, Spectrum)
STD_ROM_FN(Specsoldfort)

struct BurnDriver BurnSpecsoldfort = {
	"spec_soldfort", NULL, "spec_spectrum", NULL, "1988",
	"Soldier of Fortune (48K)\0", NULL, "Firebird Software", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecsoldfortRomInfo, SpecsoldfortRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Spy Hunter (48K)

static struct BurnRomInfo SpecspyhuntRomDesc[] = {
	{ "Spy Hunter (1985)(U.S. Gold).tap", 32547, 0x3febdfd4, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specspyhunt, Specspyhunt, Spectrum)
STD_ROM_FN(Specspyhunt)

struct BurnDriver BurnSpecspyhunt = {
	"spec_spyhunt", NULL, "spec_spectrum", NULL, "1985",
	"Spy Hunter (48K)\0", NULL, "U.S. Gold", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_VERSHOOT, 0,
	SpectrumGetZipName, SpecspyhuntRomInfo, SpecspyhuntRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Spy vs Spy (48K)

static struct BurnRomInfo SpecspyvspyRomDesc[] = {
	{ "Spy vs Spy (1985)(Beyond Software).z80", 40858, 0xa5fc636b, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specspyvspy, Specspyvspy, Spectrum)
STD_ROM_FN(Specspyvspy)

struct BurnDriver BurnSpecspyvspy = {
	"spec_spyvspy", NULL, "spec_spectrum", NULL, "1985",
	"Spy vs Spy (48K)\0", NULL, "Beyond Software", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 2, HARDWARE_SPECTRUM, GBF_ACTION | GBF_STRATEGY, 0,
	SpectrumGetZipName, SpecspyvspyRomInfo, SpecspyvspyRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecCursorKeysDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Spy vs Spy II - The Island Caper (48K)

static struct BurnRomInfo Specspyvspy2RomDesc[] = {
	{ "Spy vs Spy II - The Island Caper (1987)(Databyte).z80", 0x09350, 0xe5133176, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specspyvspy2, Specspyvspy2, Spectrum)
STD_ROM_FN(Specspyvspy2)

struct BurnDriver BurnSpecspyvspy2 = {
	"spec_spyvspy2", NULL, "spec_spectrum", NULL, "1987",
	"Spy vs Spy II - The Island Caper (48K)\0", NULL, "Databyte", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 2, HARDWARE_SPECTRUM, GBF_ACTION | GBF_STRATEGY, 0,
	SpectrumGetZipName, Specspyvspy2RomInfo, Specspyvspy2RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Spy vs Spy III - Arctic Antics (48K)

static struct BurnRomInfo Specspyvspy3RomDesc[] = {
	{ "Spy vs Spy III - Arctic Antics (1988)(Databyte).z80", 0x07a46, 0x9bf7db2a, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specspyvspy3, Specspyvspy3, Spectrum)
STD_ROM_FN(Specspyvspy3)

struct BurnDriver BurnSpecspyvspy3 = {
	"spec_spyvspy3", NULL, "spec_spectrum", NULL, "1988",
	"Spy vs Spy III - Arctic Antics (48K)\0", NULL, "Databyte", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 2, HARDWARE_SPECTRUM, GBF_ACTION | GBF_STRATEGY, 0,
	SpectrumGetZipName, Specspyvspy3RomInfo, Specspyvspy3RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Star Farce (48K)

static struct BurnRomInfo SpecstarfarcRomDesc[] = {
	{ "Star Farce (1988)(Mastertronic).tap", 44969, 0xb7915492, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specstarfarc, Specstarfarc, Spectrum)
STD_ROM_FN(Specstarfarc)

struct BurnDriver BurnSpecstarfarc = {
	"spec_starfarc", NULL, "spec_spectrum", NULL, "1988",
	"Star Farce (48K)\0", NULL, "Mastertronic", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_VERSHOOT, 0,
	SpectrumGetZipName, SpecstarfarcRomInfo, SpecstarfarcRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Starquake (48K)

static struct BurnRomInfo SpecstarquakRomDesc[] = {
	{ "Starquake (1985)(Bubble Bus).tap", 56203, 0x16e3e1b2, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specstarquak, Specstarquak, Spectrum)
STD_ROM_FN(Specstarquak)

struct BurnDriver BurnSpecstarquak = {
	"spec_starquak", NULL, "spec_spectrum", NULL, "1985",
	"Starquake (48K)\0", NULL, "Bubblebus Software", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecstarquakRomInfo, SpecstarquakRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Stop the Express (48K)

static struct BurnRomInfo SpecstopexprRomDesc[] = {
	{ "Stop The Express (1983)(Hudson Soft).tap", 22521, 0x413eeba8, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specstopexpr, Specstopexpr, Spectrum)
STD_ROM_FN(Specstopexpr)

struct BurnDriver BurnSpecstopexpr = {
	"spec_stopexpr", NULL, "spec_spectrum", NULL, "1983",
	"Stop the Express (48K)\0", NULL, "Sinclair Research", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_ACTION, 0,
	SpectrumGetZipName, SpecstopexprRomInfo, SpecstopexprRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecIntf2DIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Stormlord (48K)

static struct BurnRomInfo SpecstormlorRomDesc[] = {
	{ "Stormlord 48K (1989)(Hewson Consultants).tap", 46265, 0x6bef026d, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specstormlor, Specstormlor, Spectrum)
STD_ROM_FN(Specstormlor)

struct BurnDriver BurnSpecstormlor = {
	"spec_stormlor", "spec_stormlor128", "spec_spectrum", NULL, "1989",
	"Stormlord (48K)\0", NULL, "Hewson Consultants", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecstormlorRomInfo, SpecstormlorRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Stormlord (128K)

static struct BurnRomInfo Specstormlor128RomDesc[] = {
	{ "Stormlord 128K (1989)(Hewson Consultants).tap", 46266, 0xc91e464f, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specstormlor128, Specstormlor128, Spec128)
STD_ROM_FN(Specstormlor128)

struct BurnDriver BurnSpecstormlor128 = {
	"spec_stormlor128", NULL, "spec_spec128", NULL, "1989",
	"Stormlord (128K)\0", NULL, "Hewson Consultants", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, Specstormlor128RomInfo, Specstormlor128RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Street Fighter (48K)

static struct BurnRomInfo SpecsfightRomDesc[] = {
	{ "Street Fighter 48K (1988)(Go!).tap", 91844, 0x7d79c29b, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specsfight, Specsfight, Spectrum)
STD_ROM_FN(Specsfight)

struct BurnDriver BurnSpecsfight = {
	"spec_sfight", NULL, "spec_spectrum", NULL, "1988",
	"Street Fighter (48K)\0", NULL, "Go!", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 2, HARDWARE_SPECTRUM, GBF_VSFIGHT, 0,
	SpectrumGetZipName, SpecsfightRomInfo, SpecsfightRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Strider (128K)

static struct BurnRomInfo SpecstriderRomDesc[] = {
	{ "Strider (1989)(U.S. Gold).tap", 121097, 0x4e28a067, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specstrider, Specstrider, Spec128)
STD_ROM_FN(Specstrider)

struct BurnDriver BurnSpecstrider = {
	"spec_strider", NULL, "spec_spec128", NULL, "1989",
	"Strider (128K)\0", NULL, "U.S. Gold", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_RUNGUN | GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecstriderRomInfo, SpecstriderRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Strider II (128K)

static struct BurnRomInfo Specstrider2RomDesc[] = {
	{ "Strider II (1990)(U.S. Gold).tap", 0x21e24, 0xd1b065fc, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specstrider2, Specstrider2, Spec128)
STD_ROM_FN(Specstrider2)

struct BurnDriver BurnSpecstrider2 = {
	"spec_strider2", NULL, "spec_spec128", NULL, "1990",
	"Strider II (128K)\0", NULL, "U.S. Gold", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_RUNGUN | GBF_PLATFORM, 0,
	SpectrumGetZipName, Specstrider2RomInfo, Specstrider2RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Striker (128K)

static struct BurnRomInfo SpecstrikerRomDesc[] = {
	{ "Striker (1989)(Cult Games)[128K].z80", 0x090e9, 0xf7144c4d, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specstriker, Specstriker, Spec128)
STD_ROM_FN(Specstriker)

struct BurnDriver BurnSpecstriker = {
	"spec_striker", NULL, "spec_spec128", NULL, "1989",
	"Striker (128K)\0", NULL, "Cult Games", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_STRATEGY | GBF_SPORTSFOOTBALL, 0,
	SpectrumGetZipName, SpecstrikerRomInfo, SpecstrikerRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Stunt Car Racer (128K)

static struct BurnRomInfo SpecstuntcarRomDesc[] = {
	{ "Stunt Car Racer 128K (1989)(Micro Style).tap", 55835, 0xfdd995ec, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specstuntcar, Specstuntcar, Spec128)
STD_ROM_FN(Specstuntcar)

struct BurnDriver BurnSpecstuntcar = {
	"spec_stuntcar", NULL, "spec_spec128", NULL, "1989",
	"Stunt Car Racer (128K)\0", NULL, "Micro Style", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_RACING, 0,
	SpectrumGetZipName, SpecstuntcarRomInfo, SpecstuntcarRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Subbuteo - The Computer Game (128K)

static struct BurnRomInfo SpecsubbuteoRomDesc[] = {
	{ "Subbuteo - The Computer Game 128K (1990)(Electronic Zoo).tap", 76987, 0x9a6211f8, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specsubbuteo, Specsubbuteo, Spec128)
STD_ROM_FN(Specsubbuteo)

struct BurnDriver BurnSpecsubbuteo = {
	"spec_subbuteo", NULL, "spec_spec128", NULL, "1990",
	"Subbuteo - The Computer Game (128K)\0", NULL, "Electronic Zoo", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 2, HARDWARE_SPECTRUM, GBF_SPORTSFOOTBALL | GBF_STRATEGY, 0,
	SpectrumGetZipName, SpecsubbuteoRomInfo, SpecsubbuteoRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Super Hang-On - Part 1 - Beginner (48K)

static struct BurnRomInfo SpecshangonRomDesc[] = {
	{ "Super Hang-On - Part 1 - Beginner (1986)(Electric Dreams Software).z80", 0x08818, 0x6021b420, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specshangon, Specshangon, Spectrum)
STD_ROM_FN(Specshangon)

struct BurnDriver BurnSpecshangon = {
	"spec_shangon", NULL, "spec_spectrum", NULL, "1986",
	"Super Hang-On - Part 1 - Beginner (48K)\0", NULL, "Electric Dreams", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_RACING, 0,
	SpectrumGetZipName, SpecshangonRomInfo, SpecshangonRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Super Hang-On - Part 2 - Junior (48K)

static struct BurnRomInfo Specshangon2RomDesc[] = {
	{ "Super Hang-On - Part 2 - Junior (1986)(Electric Dreams Software).z80", 0x08690, 0xd8180d70, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specshangon2, Specshangon2, Spectrum)
STD_ROM_FN(Specshangon2)

struct BurnDriver BurnSpecshangon2 = {
	"spec_shangon2", "spec_shangon", "spec_spectrum", NULL, "1986",
	"Super Hang-On - Part 2 - Junior (48K)\0", NULL, "Electric Dreams", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE, 1, HARDWARE_SPECTRUM, GBF_RACING, 0,
	SpectrumGetZipName, Specshangon2RomInfo, Specshangon2RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Super Hang-On - Part 3 - Senior (48K)

static struct BurnRomInfo Specshangon3RomDesc[] = {
	{ "Super Hang-On - Part 3 - Senior (1986)(Electric Dreams Software).z80", 0x08adc, 0x363567ec, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specshangon3, Specshangon3, Spectrum)
STD_ROM_FN(Specshangon3)

struct BurnDriver BurnSpecshangon3 = {
	"spec_shangon3", "spec_shangon", "spec_spectrum", NULL, "1986",
	"Super Hang-On - Part 3 - Senior (48K)\0", NULL, "Electric Dreams", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE, 1, HARDWARE_SPECTRUM, GBF_RACING, 0,
	SpectrumGetZipName, Specshangon3RomInfo, Specshangon3RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Super Hang-On - Part 4 - Expert (48K)

static struct BurnRomInfo Specshangon4RomDesc[] = {
	{ "Super Hang-On - Part 4 - Expert (1986)(Electric Dreams Software).z80", 0x0868a, 0x8cb2ac52, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specshangon4, Specshangon4, Spectrum)
STD_ROM_FN(Specshangon4)

struct BurnDriver BurnSpecshangon4 = {
	"spec_shangon4", "spec_shangon", "spec_spectrum", NULL, "1986",
	"Super Hang-On - Part 4 - Expert (48K)\0", NULL, "Electric Dreams", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE, 1, HARDWARE_SPECTRUM, GBF_RACING, 0,
	SpectrumGetZipName, Specshangon4RomInfo, Specshangon4RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Super Monaco GP (128K)

static struct BurnRomInfo SpecsmgpRomDesc[] = {
	{ "Super Monaco GP 128K (1991)(U.S. Gold).tap", 123409, 0x14cba3ef, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specsmgp, Specsmgp, Spec128)
STD_ROM_FN(Specsmgp)

struct BurnDriver BurnSpecsmgp = {
	"spec_smgp", NULL, "spec_spec128", NULL, "1991",
	"Super Monaco GP (128K)\0", NULL, "U.S. Gold", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_RACING, 0,
	SpectrumGetZipName, SpecsmgpRomInfo, SpecsmgpRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Super Sprint (48K)

static struct BurnRomInfo SpecsupespriRomDesc[] = {
	{ "Super Sprint (1987)(Activision).z80", 0x07a7e, 0x52ee2754, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specsupespri, Specsupespri, Spectrum)
STD_ROM_FN(Specsupespri)

struct BurnDriver BurnSpecsupespri = {
	"spec_supespri", NULL, "spec_spectrum", NULL, "1987",
	"Super Sprint (48K)\0", NULL, "Activision", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 2, HARDWARE_SPECTRUM, GBF_RACING, 0,
	SpectrumGetZipName, SpecsupespriRomInfo, SpecsupespriRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// SWIV (128K)

static struct BurnRomInfo SpecswivRomDesc[] = {
	{ "SWIV (1991)(Storm Software).tap", 0x2378e, 0xbaf9be65, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specswiv, Specswiv, Spec128)
STD_ROM_FN(Specswiv)

struct BurnDriver BurnSpecswiv = {
	"spec_swiv", NULL, "spec_spec128", NULL, "1991",
	"SWIV (128K)\0", NULL, "Storm Software", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 2, HARDWARE_SPECTRUM, GBF_VERSHOOT, 0,
	SpectrumGetZipName, SpecswivRomInfo, SpecswivRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecIntf2DIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Technician Ted (48K)

static struct BurnRomInfo Spectechted_48RomDesc[] = {
	{ "Technician Ted (1984)(Hewson Consultants).z80", 0x0a2af, 0x90e4eaee, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Spectechted_48, Spectechted_48, Spectrum)
STD_ROM_FN(Spectechted_48)

struct BurnDriver BurnSpectechted_48 = {
	"spec_techted_48", NULL, "spec_spectrum", NULL, "1984",
	"Technician Ted (48K)\0", NULL, "Hewson Consultants", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, Spectechted_48RomInfo, Spectechted_48RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecQAOPSpaceDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Teenage Mutant Hero Turtles - The Coin-Op (128K)

static struct BurnRomInfo SpectmhtarcRomDesc[] = {
	{ "Teenage Mutant Hero Turtles - The Coin-Op (1991)(Image Works).tap", 155263, 0xa1ea1547, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Spectmhtarc, Spectmhtarc, Spec128)
STD_ROM_FN(Spectmhtarc)

struct BurnDriver BurnSpectmhtarc = {
	"spec_tmhtarc", NULL, "spec_spec128", NULL, "1991",
	"Teenage Mutant Hero Turtles - The Coin-Op (128K)\0", NULL, "Image Works", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 2, HARDWARE_SPECTRUM, GBF_SCRFIGHT, 0,
	SpectrumGetZipName, SpectmhtarcRomInfo, SpectmhtarcRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Teenage Mutant Hero Turtles (48K)

static struct BurnRomInfo SpectmhtRomDesc[] = {
	{ "Teenage Mutant Hero Turtles (1990)(Image Works).tap", 43029, 0xd96767fb, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Spectmht, Spectmht, Spectrum)
STD_ROM_FN(Spectmht)

struct BurnDriver BurnSpectmht = {
	"spec_tmht", NULL, "spec_spectrum", NULL, "1990",
	"Teenage Mutant Hero Turtles (48K)\0", NULL, "Image Works", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_SCRFIGHT | GBF_PLATFORM, 0,
	SpectrumGetZipName, SpectmhtRomInfo, SpectmhtRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Terra Cresta (48K)

static struct BurnRomInfo SpecterracrestaRomDesc[] = {
	{ "Terra Cresta (1986)(Imagine).tap", 48178, 0x55a50359, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specterracresta, Specterracresta, Spectrum)
STD_ROM_FN(Specterracresta)

struct BurnDriver BurnSpecterracresta = {
	"spec_terracresta", NULL, "spec_spectrum", NULL, "1986",
	"Terra Cresta (48K)\0", NULL, "Imagine Software", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_VERSHOOT, 0,
	SpectrumGetZipName, SpecterracrestaRomInfo, SpecterracrestaRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Tetris (Mirrorsoft) (128K)

static struct BurnRomInfo SpectetrisRomDesc[] = {
	{ "Tetris 128K (1988)(Mirrorsoft).tap", 38370, 0x6e160a6c, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Spectetris, Spectetris, Spec128)
STD_ROM_FN(Spectetris)

struct BurnDriver BurnSpectetris = {
	"spec_tetris", NULL, "spec_spec128", NULL, "1987-88",
	"Tetris (Mirrorsoft) (128K)\0", "Press 'O' for options", "Mirrorsoft", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_PUZZLE, 0,
	SpectrumGetZipName, SpectetrisRomInfo, SpectetrisRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Three Weeks in Paradise (48K)

static struct BurnRomInfo Spec3weekspr_48RomDesc[] = {
	{ "Three Weeks In Paradise (1986)(Mikro-Gen).z80", 0x0a49e, 0x3c7ac0a9, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Spec3weekspr_48, Spec3weekspr_48, Spectrum)
STD_ROM_FN(Spec3weekspr_48)

struct BurnDriver BurnSpec3weekspr_48 = {
	"spec_3weekspr_48", "spec_3weekspr", "spec_spectrum", NULL, "1986",
	"Three Weeks in Paradise (48K)\0", NULL, "Mikro-Gen", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE, 1, HARDWARE_SPECTRUM, GBF_PLATFORM | GBF_ADV, 0,
	SpectrumGetZipName, Spec3weekspr_48RomInfo, Spec3weekspr_48RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Three Weeks in Paradise (128K)

static struct BurnRomInfo Spec3weeksprRomDesc[] = {
	{ "Three Weeks In Paradise (1986)(Mikro-Gen)(128k).z80", 0x0e06c, 0xf21d8b5d, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Spec3weekspr, Spec3weekspr, Spec128)
STD_ROM_FN(Spec3weekspr)

struct BurnDriver BurnSpec3weekspr = {
	"spec_3weekspr", NULL, "spec_spec128", NULL, "1986",
	"Three Weeks in Paradise (128K)\0", NULL, "Mikro-Gen", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_PLATFORM | GBF_ADV, 0,
	SpectrumGetZipName, Spec3weeksprRomInfo, Spec3weeksprRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Thunder Blade (128K)

static struct BurnRomInfo SpecthndrbldRomDesc[] = {
	{ "Thunder Blade (1988)(U.S. Gold).tap", 114186, 0xa56877b6, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specthndrbld, Specthndrbld, Spec128)
STD_ROM_FN(Specthndrbld)

struct BurnDriver BurnSpecthndrbld = {
	"spec_thndrbld", NULL, "spec_spec128", NULL, "1988",
	"Thunder Blade (128K)\0", NULL, "U.S. Gold", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_SHOOT, 0,
	SpectrumGetZipName, SpecthndrbldRomInfo, SpecthndrbldRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Tiger Road (128K)

static struct BurnRomInfo SpectigeroadRomDesc[] = {
	{ "Tiger Road (1988)(Go!)[128K].z80", 0x0b726, 0xede04afd, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Spectigeroad, Spectigeroad, Spec128)
STD_ROM_FN(Spectigeroad)

struct BurnDriver BurnSpectigeroad = {
	"spec_tigeroad", NULL, "spec_spec128", NULL, "1988",
	"Tiger Road (128K)\0", "Press '4' to use controller", "Go!", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_SCRFIGHT, 0,
	SpectrumGetZipName, SpectigeroadRomInfo, SpectigeroadRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Time Scanner (128K)

static struct BurnRomInfo SpectimescanRomDesc[] = {
	{ "Time Scanner 128K (1989)(Activision).tap", 113308, 0x3698495a, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Spectimescan, Spectimescan, Spec128)
STD_ROM_FN(Spectimescan)

struct BurnDriver BurnSpectimescan = {
	"spec_timescan", NULL, "spec_spec128", NULL, "1989",
	"Time Scanner (128K)\0", "Flippers: W and O - Tilt: - Q, P and SPACE", "Activision", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_PINBALL, 0,
	SpectrumGetZipName, SpectimescanRomInfo, SpectimescanRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Titanic - Part 1 (Spanish) (48K)

static struct BurnRomInfo Spectitanictopo1RomDesc[] = {
	{ "Titanic - part 1 ES (1988)(Topo Soft).tap", 50546, 0x797a006e, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Spectitanictopo1, Spectitanictopo1, Spectrum)
STD_ROM_FN(Spectitanictopo1)

struct BurnDriver BurnSpectitanictopo1 = {
	"spec_titanictopo1", NULL, "spec_spectrum", NULL, "1988",
	"Titanic - Part 1 (Spanish) (48K)\0", NULL, "Topo Soft", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_ACTION, 0,
	SpectrumGetZipName, Spectitanictopo1RomInfo, Spectitanictopo1RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Titanic - Part 2 (Spanish) (48K)

static struct BurnRomInfo Spectitanictopo2RomDesc[] = {
	{ "Titanic - part 2 ES (1988)(Topo Soft).tap", 46204, 0x547d088f, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Spectitanictopo2, Spectitanictopo2, Spectrum)
STD_ROM_FN(Spectitanictopo2)

struct BurnDriver BurnSpectitanictopo2 = {
	"spec_titanictopo2", "spec_titanictopo1", "spec_spectrum", NULL, "1988",
	"Titanic - Part 2 (Spanish) (48K)\0", "Password: SUSIE", "Topo Soft", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE, 1, HARDWARE_SPECTRUM, GBF_ACTION, 0,
	SpectrumGetZipName, Spectitanictopo2RomInfo, Spectitanictopo2RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Total Recall (128K)

static struct BurnRomInfo SpectotrcallRomDesc[] = {
	{ "Total Recall (1991)(Ocean)[128K].z80", 0x17197, 0xab3503be, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Spectotrcall, Spectotrcall, Spec128)
STD_ROM_FN(Spectotrcall)

struct BurnDriver BurnSpectotrcall = {
	"spec_totrcall", NULL, "spec_spec128", NULL, "1991",
	"Total Recall (128K)\0", NULL, "Ocean", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_SCRFIGHT | GBF_PLATFORM, 0,
	SpectrumGetZipName, SpectotrcallRomInfo, SpectotrcallRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Track and Field (48K)

static struct BurnRomInfo SpectracfielRomDesc[] = {
	{ "Track and Field (1988)(Ocean).z80", 0x05787, 0x7ddee010, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Spectracfiel, Spectracfiel, Spectrum)
STD_ROM_FN(Spectracfiel)

struct BurnDriver BurnSpectracfiel = {
	"spec_tracfiel", NULL, "spec_spectrum", NULL, "1988",
	"Track and Field (48K)\0", NULL, "Ocean", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 2, HARDWARE_SPECTRUM, GBF_SPORTSMISC, 0,
	SpectrumGetZipName, SpectracfielRomInfo, SpectracfielRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecIntf2DIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Trantor - The Last Stormtrooper (48K)

static struct BurnRomInfo SpectrantorRomDesc[] = {
	{ "Trantor - The Last Stormtrooper 48K (1987)(Go!).tap", 47841, 0xd385ae7e, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Spectrantor, Spectrantor, Spectrum)
STD_ROM_FN(Spectrantor)

struct BurnDriver BurnSpectrantor = {
	"spec_trantor", NULL, "spec_spectrum", NULL, "1987",
	"Trantor - The Last Stormtrooper (48K)\0", NULL, "Go!", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_RUNGUN, 0,
	SpectrumGetZipName, SpectrantorRomInfo, SpectrantorRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Trap Door, The (48K)

static struct BurnRomInfo SpectradoothRomDesc[] = {
	{ "Trap Door, The 48K (1986)(Piranha).tap", 42677, 0x0d070aa6, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Spectradooth, Spectradooth, Spectrum)
STD_ROM_FN(Spectradooth)

struct BurnDriver BurnSpectradooth = {
	"spec_tradooth", NULL, "spec_spectrum", NULL, "1986",
	"Trap Door, The (48K)\0", NULL, "Piranha", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_ADV, 0,
	SpectrumGetZipName, SpectradoothRomInfo, SpectradoothRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Underwurlde (48K)

static struct BurnRomInfo SpecunderwurRomDesc[] = {
	{ "Underwurlde (1984)(Ultimate Play The Game).tap", 46026, 0x0a46e7cf, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specunderwur, Specunderwur, Spectrum)
STD_ROM_FN(Specunderwur)

struct BurnDriver BurnSpecunderwur = {
	"spec_underwur", NULL, "spec_spectrum", NULL, "1984",
	"Underwurlde (48K)\0", NULL, "Ultimate Play The Game", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_PLATFORM | GBF_ADV, 0,
	SpectrumGetZipName, SpecunderwurRomInfo, SpecunderwurRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Untouchables, The (128K)

static struct BurnRomInfo SpecuntouchbRomDesc[] = {
	{ "Untouchables, The (1989)(Ocean).tap", 182932, 0x81349cbf, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specuntouchb, Specuntouchb, Spec128)
STD_ROM_FN(Specuntouchb)

struct BurnDriver BurnSpecuntouchb = {
	"spec_untouchb", NULL, "spec_spec128", NULL, "1989",
	"Untouchables, The (128K)\0", NULL, "Ocean", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_SHOOT | GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecuntouchbRomInfo, SpecuntouchbRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Vindicator, The (128K)

static struct BurnRomInfo SpecvindtheRomDesc[] = {
	{ "Vindicator, The (1988)(Imagine Software)[128K].z80", 0x1b692, 0x57c8a81d, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specvindthe, Specvindthe, Spec128)
STD_ROM_FN(Specvindthe)

struct BurnDriver BurnSpecvindthe = {
	"spec_vindthe", NULL, "spec_spec128", NULL, "1988",
	"Vindicator, The (128K)\0", NULL, "Imagine Software", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_ACTION | GBF_SHOOT, 0,
	SpectrumGetZipName, SpecvindtheRomInfo, SpecvindtheRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Vindicators (128K)

static struct BurnRomInfo SpecvindicatRomDesc[] = {
	{ "Vindicators (1989)(Domark).tap", 0x0ffc4, 0xc4b6502e, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specvindicat, Specvindicat, Spec128)
STD_ROM_FN(Specvindicat)

struct BurnDriver BurnSpecvindicat = {
	"spec_vindicat", NULL, "spec_spec128", NULL, "1989",
	"Vindicators (128K)\0", NULL, "Domark", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 2, HARDWARE_SPECTRUM, GBF_SHOOT, 0,
	SpectrumGetZipName, SpecvindicatRomInfo, SpecvindicatRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecIntf2DIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Wacky Darts (48K)

static struct BurnRomInfo Specwackdarts_48RomDesc[] = {
	{ "Wacky Darts (1991)(Codemasters)(48K).z80", 0x0ae9d, 0x6214a4ce, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specwackdarts_48, Specwackdarts_48, Spectrum)
STD_ROM_FN(Specwackdarts_48)

struct BurnDriver BurnSpecwackdart_48 = {
	"spec_wackdarts_48", "spec_wackydarts", "spec_spectrum", NULL, "1991",
	"Wacky Darts (48K)\0", "In options menu: keys '6', '7' and ENTER to choose and set", "Codemasters", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE, 8, HARDWARE_SPECTRUM, GBF_SPORTSMISC, 0,
	SpectrumGetZipName, Specwackdarts_48RomInfo, Specwackdarts_48RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecIntf2DIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Wacky Races (128K)

static struct BurnRomInfo SpecwackraceRomDesc[] = {
	{ "Wacky Races 128K (1992)(Hi-Tec).tap", 149897, 0xc46f4db9, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specwackrace, Specwackrace, Spec128)
STD_ROM_FN(Specwackrace)

struct BurnDriver BurnSpecwackrace = {
	"spec_wackrace", NULL, "spec_spec128", NULL, "1992",
	"Wacky Races (128K)\0", NULL, "Hi-Tec Software", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecwackraceRomInfo, SpecwackraceRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecIntf2DIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Wanted! Monty Mole (48K)

static struct BurnRomInfo SpecwanmonmoRomDesc[] = {
	{ "Wanted - Monty Mole (1984)(Gremlin Graphics).tap", 49411, 0xa50a88cc, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specwanmonmo, Specwanmonmo, Spectrum)
STD_ROM_FN(Specwanmonmo)

struct BurnDriver BurnSpecwanmonmo = {
	"spec_wanmonmo", NULL, "spec_spectrum", NULL, "1984",
	"Wanted! Monty Mole (48K)\0", NULL, "Gremlin Graphics", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecwanmonmoRomInfo, SpecwanmonmoRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Way of the Exploding Fist, The (48K)

static struct BurnRomInfo SpecwayexplfRomDesc[] = {
	{ "Way of the Exploding Fist, The (1985)(Melbourne House).tap", 45962, 0x7e81d64e, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specwayexplf, Specwayexplf, Spectrum)
STD_ROM_FN(Specwayexplf)

struct BurnDriver BurnSpecwayexplf = {
	"spec_wayexplf", NULL, "spec_spectrum", NULL, "1985",
	"Way of the Exploding Fist, The (48K)\0", "Press '1' or '2' to one-two players mode and '0' to redefine controls", "Melbourne House", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 2, HARDWARE_SPECTRUM, GBF_VSFIGHT, 0,
	SpectrumGetZipName, SpecwayexplfRomInfo, SpecwayexplfRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// WEC Le Mans (128K)

static struct BurnRomInfo SpecweclemansRomDesc[] = {
	{ "WEC Le Mans (1989)(Imagine)(48K-128K).tap", 112833, 0x835ac96a, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specweclemans, Specweclemans, Spec128)
STD_ROM_FN(Specweclemans)

struct BurnDriver BurnSpecweclemans = {
	"spec_weclemans", NULL, "spec_spec128", NULL, "1988-89",
	"WEC Le Mans (128K)\0", NULL, "Imagine Software", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_RACING, 0,
	SpectrumGetZipName, SpecweclemansRomInfo, SpecweclemansRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Wild West Seymour (48K)

static struct BurnRomInfo Specwwseymr_48RomDesc[] = {
	{ "Wild West Seymour (1992)(Codemasters).z80", 0x0b4ab, 0xbf324d91, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specwwseymr_48, Specwwseymr_48, Spectrum)
STD_ROM_FN(Specwwseymr_48)

struct BurnDriver BurnSpecwwseymr_48 = {
	"spec_wwseymr_48", "spec_wwseymr", "spec_spectrum", NULL, "1992",
	"Wild West Seymour (48K)\0", NULL, "Codemasters", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE, 1, HARDWARE_SPECTRUM, GBF_ADV, 0,
	SpectrumGetZipName, Specwwseymr_48RomInfo, Specwwseymr_48RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Wild West Seymour (128K)

static struct BurnRomInfo SpecwwseymrRomDesc[] = {
	{ "Wild West Seymour (1992)(Codemasters)(128k).z80", 0x141c6, 0x01dc3cee, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specwwseymr, Specwwseymr, Spec128)
STD_ROM_FN(Specwwseymr)

struct BurnDriver BurnSpecwwseymr = {
	"spec_wwseymr", NULL, "spec_spec128", NULL, "1992",
	"Wild West Seymour (128K)\0", NULL, "Codemasters", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_ADV, 0,
	SpectrumGetZipName, SpecwwseymrRomInfo, SpecwwseymrRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Wonder Boy (128K)

static struct BurnRomInfo SpecwboyRomDesc[] = {
	{ "Wonder Boy (1987)(Activision)[128K].z80", 0x11193, 0xb492a055, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specwboy, Specwboy, Spec128)
STD_ROM_FN(Specwboy)

struct BurnDriver BurnSpecwboy = {
	"spec_wboy", NULL, "spec_spec128", NULL, "1987",
	"Wonder Boy (128K)\0", NULL, "Activision", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 2, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecwboyRomInfo, SpecwboyRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Xenon (128K)

static struct BurnRomInfo SpecxenonRomDesc[] = {
	{ "Xenon (1989)(Melbourne House).tap", 69556, 0xb9201d1c, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specxenon, Specxenon, Spec128)
STD_ROM_FN(Specxenon)

struct BurnDriver BurnSpecxenon = {
	"spec_xenon", NULL, "spec_spec128", NULL, "1989",
	"Xenon (128K)\0", NULL, "Melbourne House", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_VERSHOOT, 0,
	SpectrumGetZipName, SpecxenonRomInfo, SpecxenonRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Xevious (48K)

static struct BurnRomInfo SpecxeviousRomDesc[] = {
	{ "Xevious (1986)(U.S. Gold).tap", 44672, 0xf40c7c95, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specxevious, Specxevious, Spectrum)
STD_ROM_FN(Specxevious)

struct BurnDriver BurnSpecxevious = {
	"spec_xevious", NULL, "spec_spectrum", NULL, "1986",
	"Xevious (48K)\0", NULL, "U.S. Gold", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 2, HARDWARE_SPECTRUM, GBF_VERSHOOT, 0,
	SpectrumGetZipName, SpecxeviousRomInfo, SpecxeviousRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Zaxxon (48K)

static struct BurnRomInfo SpeczaxxonRomDesc[] = {
	{ "Zaxxon (1985)(U.S. Gold).tap", 49889, 0x6d169dbf, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Speczaxxon, Speczaxxon, Spectrum)
STD_ROM_FN(Speczaxxon)

struct BurnDriver BurnSpeczaxxon = {
	"spec_zaxxon", NULL, "spec_spectrum", NULL, "1985",
	"Zaxxon (48K)\0", NULL, "U.S. Gold", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 2, HARDWARE_SPECTRUM, GBF_SHOOT, 0,
	SpectrumGetZipName, SpeczaxxonRomInfo, SpeczaxxonRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Lorna (128K)

static struct BurnRomInfo SpecLornaRomDesc[] = {
	{ "Lorna (1990)(Topo Soft)(Sp).tap", 0x1cf8a, 0xa09afe9a, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecLorna, SpecLorna, Spec128)
STD_ROM_FN(SpecLorna)

struct BurnDriver BurnSpecLorna = {
	"spec_lorna", NULL, "spec_spec128", NULL, "1990",
	"Lorna (128K)\0", NULL, "Topo Soft", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_RUNGUN, 0,
	SpectrumGetZipName, SpecLornaRomInfo, SpecLornaRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInvesInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Myth - History In The Making (128K)

static struct BurnRomInfo SpecMythRomDesc[] = {
	{ "Myth - History In The Making (1989)(System 3 Software).tap", 0x2f76e, 0xbc8f2fe4, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecMyth, SpecMyth, Spec128)
STD_ROM_FN(SpecMyth)

struct BurnDriver BurnSpecMyth = {
	"spec_myth", NULL, "spec_spec128", NULL, "1989",
	"Myth - History In The Making (128K)\0", NULL, "System 3", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecMythRomInfo, SpecMythRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Narco Police (128K)

static struct BurnRomInfo SpecNarcopolRomDesc[] = {
	{ "Narco Police (1991)(Rajsoft).tap", 0x1cf59, 0xd47256a3, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecNarcopol, SpecNarcopol, Spec128)
STD_ROM_FN(SpecNarcopol)

struct BurnDriver BurnSpecNarcopol = {
	"spec_narcopol", NULL, "spec_spec128", NULL, "1991",
	"Narco Police (128K)\0", NULL, "Rajsoft", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_SHOOT, 0,
	SpectrumGetZipName, SpecNarcopolRomInfo, SpecNarcopolRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecIntf2DIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// New Zealand Story, The (128K)

static struct BurnRomInfo SpecTnzsRomDesc[] = {
	{ "New Zealand Story, The (1989)(Ocean Software).tap", 0x2b800, 0xe6f32d53, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecTnzs, SpecTnzs, Spec128)
STD_ROM_FN(SpecTnzs)

struct BurnDriver BurnSpecTnzs = {
	"spec_tnzs", NULL, "spec_spec128", NULL, "1989",
	"New Zealand Story, The (128K)\0", NULL, "Ocean", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecTnzsRomInfo, SpecTnzsRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecIntf2DIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Turbo Outrun (128K)

static struct BurnRomInfo SpecToutrunRomDesc[] = {
	{ "Turbo Outrun (1990)(US Gold).tap", 0x5a75a, 0x80a6916d, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecToutrun, SpecToutrun, Spec128)
STD_ROM_FN(SpecToutrun)

struct BurnDriver BurnSpecToutrun = {
	"spec_toutrun", NULL, "spec_spec128", NULL, "1990",
	"Turbo Outrun (128K)\0", NULL, "U.S. Gold", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_RACING, 0,
	SpectrumGetZipName, SpecToutrunRomInfo, SpecToutrunRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Astro Marine Corps - Part 1 (Spanish) (128K)

static struct BurnRomInfo SpecastmarcsRomDesc[] = {
	{ "Astro Marine Corps (1989)(Dinamic Software)(es)(Part 1 of 2).z80", 0x0c38f, 0x643f916a, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specastmarcs, Specastmarcs, Spec128)
STD_ROM_FN(Specastmarcs)

struct BurnDriver BurnSpecastmarcs = {
	"spec_astmarcs", NULL, "spec_spec128", NULL, "1989",
	"Astro Marine Corps - Part 1 (Spanish) (128K)\0", NULL, "Dinamic Software", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_RUNGUN, 0,
	SpectrumGetZipName, SpecastmarcsRomInfo, SpecastmarcsRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Astro Marine Corps - Part 2 (Spanish) (128K)

static struct BurnRomInfo Specastmarcs2RomDesc[] = {
	{ "Astro Marine Corps (1989)(Dinamic Software)(es)(Part 2 of 2).z80", 0x0bcce, 0x295074fd, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specastmarcs2, Specastmarcs2, Spec128)
STD_ROM_FN(Specastmarcs2)

struct BurnDriver BurnSpecastmarcs2 = {
	"spec_astmarcs2", "spec_astmarcs", "spec_spec128", NULL, "1989",
	"Astro Marine Corps - Part 2 (Spanish) (128K)\0", NULL, "Dinamic Software", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE, 1, HARDWARE_SPECTRUM, GBF_RUNGUN, 0,
	SpectrumGetZipName, Specastmarcs2RomInfo, Specastmarcs2RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Choy-Lee-Fut Kung-Fu Warrior (Spanish) (48K)

static struct BurnRomInfo SpecclfkfwRomDesc[] = {
	{ "Choy-Lee-Fut Kung-Fu Warrior (1990)(Positive)(es).tap", 0x0bf47, 0x4af1bacf, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specclfkfw, Specclfkfw, Spectrum)
STD_ROM_FN(Specclfkfw)

struct BurnDriver BurnSpecclfkfw = {
	"spec_clfkfw", NULL, "spec_spectrum", NULL, "1990",
	"Choy-Lee-Fut Kung-Fu Warrior (Spanish) (48K)\0", NULL, "Positive", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_VSFIGHT, 0,
	SpectrumGetZipName, SpecclfkfwRomInfo, SpecclfkfwRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Crosswize (48K)

static struct BurnRomInfo SpeccrosswizeRomDesc[] = {
	{ "Crosswize (1988)(Firebird Software).tap", 0x0c4e7, 0x8a6d909b, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Speccrosswize, Speccrosswize, Spectrum)
STD_ROM_FN(Speccrosswize)

struct BurnDriver BurnSpeccrosswize = {
	"spec_crosswize", NULL, "spec_spectrum", NULL, "1988",
	"Crosswize (48K)\0", NULL, "Firebird Software", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_HORSHOOT, 0,
	SpectrumGetZipName, SpeccrosswizeRomInfo, SpeccrosswizeRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Goody (Spanish) (128K)

static struct BurnRomInfo SpecgoodyesRomDesc[] = {
	{ "Goody (1987)(Opera Soft)(es).tap", 0x0bdae, 0x2fd4f11b, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specgoodyes, Specgoodyes, Spec128)
STD_ROM_FN(Specgoodyes)

struct BurnDriver BurnSpecgoodyes = {
	"spec_goodyes", NULL, "spec_spec128", NULL, "1987",
	"Goody (Spanish) (128K)\0", NULL, "Opera Soft", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecgoodyesRomInfo, SpecgoodyesRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecQAOPSpaceDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Ice Breaker (Spanish) (128K)

static struct BurnRomInfo SpecicebreakerRomDesc[] = {
	{ "Ice Breaker (1990)(Topo Soft)(es).tap", 0x127d5, 0x62ce5b80, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specicebreaker, Specicebreaker, Spec128)
STD_ROM_FN(Specicebreaker)

struct BurnDriver BurnSpecicebreaker = {
	"spec_icebreaker", NULL, "spec_spec128", NULL, "1990",
	"Ice Breaker (Spanish) (128K)\0", NULL, "Topo Soft", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_VERSHOOT, 0,
	SpectrumGetZipName, SpecicebreakerRomInfo, SpecicebreakerRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Robin of the Wood (128K)

static struct BurnRomInfo SpecrobinofwoodRomDesc[] = {
	{ "Robin of the Wood (1985)(Odin Computer Graphics)(128k).tap", 0x0f37e, 0xce069366, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specrobinofwood, Specrobinofwood, Spec128)
STD_ROM_FN(Specrobinofwood)

struct BurnDriver BurnSpecrobinofwood = {
	"spec_robinofwood", NULL, "spec_spec128", NULL, "1985",
	"Robin of the Wood (128K)\0", NULL, "Odin Computer Graphics", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_ACTION | GBF_ADV, 0,
	SpectrumGetZipName, SpecrobinofwoodRomInfo, SpecrobinofwoodRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Sidewize (48K)

static struct BurnRomInfo SpecsidewizeRomDesc[] = {
	{ "Sidewize (1987)(Firebird Software).tap", 0x0a2c8, 0xdb066855, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specsidewize, Specsidewize, Spectrum)
STD_ROM_FN(Specsidewize)

struct BurnDriver BurnSpecsidewize = {
	"spec_sidewize", NULL, "spec_spectrum", NULL, "1987",
	"Sidewize (48K)\0", NULL, "Firebird Software", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 2, HARDWARE_SPECTRUM, GBF_HORSHOOT, 0,
	SpectrumGetZipName, SpecsidewizeRomInfo, SpecsidewizeRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Plot, The (48K)

static struct BurnRomInfo SpecplottheRomDesc[] = {
	{ "Plot, The (1988)(Firebird Software).tap", 0x0af68, 0x1adf1093, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specplotthe, Specplotthe, Spectrum)
STD_ROM_FN(Specplotthe)

struct BurnDriver BurnSpecplotthe = {
	"spec_plotthe", NULL, "spec_spectrum", NULL, "1988",
	"Plot, The (48K)\0", NULL, "Firebird Software", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecplottheRomInfo, SpecplottheRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Tintin on the Moon (48K)

static struct BurnRomInfo SpectintmoonRomDesc[] = {
	{ "Tintin on the Moon (1989)(Infogrames).tap", 0x09ca6, 0x1959b7a0, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Spectintmoon, Spectintmoon, Spectrum)
STD_ROM_FN(Spectintmoon)

struct BurnDriver BurnSpectintmoon = {
	"spec_tintmoon", NULL, "spec_spectrum", NULL, "1989",
	"Tintin on the Moon (48K)\0", NULL, "Infogrames", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_ACTION | GBF_PLATFORM, 0,
	SpectrumGetZipName, SpectintmoonRomInfo, SpectintmoonRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Turbo Cup Challenge (128K)

static struct BurnRomInfo SpecturbocupchRomDesc[] = {
	{ "Turbo Cup Challenge (1989)(Loriciels).tap", 0x0b19c, 0xb81e5707, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specturbocupch, Specturbocupch, Spec128)
STD_ROM_FN(Specturbocupch)

struct BurnDriver BurnSpecturbocupch = {
	"spec_turbocupch", NULL, "spec_spec128", NULL, "1989",
	"Turbo Cup Challenge (128K)\0", NULL, "Loriciels", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_RACING, 0,
	SpectrumGetZipName, SpecturbocupchRomInfo, SpecturbocupchRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecQAOPSpaceDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Vigilante (48K)

static struct BurnRomInfo SpecvigilanteRomDesc[] = {
	{ "Vigilante (1989)(U.S. Gold).tap", 0x1b03a, 0x372bf21f, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specvigilante, Specvigilante, Spectrum)
STD_ROM_FN(Specvigilante)

struct BurnDriver BurnSpecvigilante = {
	"spec_vigilante", NULL, "spec_spectrum", NULL, "1989",
	"Vigilante (48K)\0", NULL, "U.S. Gold", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_SCRFIGHT, 0,
	SpectrumGetZipName, SpecvigilanteRomInfo, SpecvigilanteRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Impossamole (128K)

static struct BurnRomInfo SpecimpossamoleRomDesc[] = {
	{ "Impossamole (1990)(Gremlin Graphics Software).tap", 0x25438, 0xf1c505c7, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specimpossamole, Specimpossamole, Spec128)
STD_ROM_FN(Specimpossamole)

struct BurnDriver BurnSpecimpossamole = {
	"spec_impossamole", NULL, "spec_spec128", NULL, "1990",
	"Impossamole (128K)\0", NULL, "The Edge", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecimpossamoleRomInfo, SpecimpossamoleRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Saint Dragon (128K)

static struct BurnRomInfo SpecstdragonRomDesc[] = {
	{ "St. Dragon (1990)(Storm Software).tap", 0x2e158, 0x6bb077c8, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specstdragon, Specstdragon, Spec128)
STD_ROM_FN(Specstdragon)

struct BurnDriver BurnSpecstdragon = {
	"spec_stdragon", NULL, "spec_spec128", NULL, "1990",
	"Saint Dragon (128K)\0", NULL, "Storm Software", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_HORSHOOT, 0,
	SpectrumGetZipName, SpecstdragonRomInfo, SpecstdragonRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecQAOPMDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Stormlord II: Deliverance (128K)

static struct BurnRomInfo Specstormlord2RomDesc[] = {
	{ "Stormlord II - Deliverance (1990)(Hewson Consultants).tap", 128474, 0x2e4e34c6, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specstormlord2, Specstormlord2, Spec128)
STD_ROM_FN(Specstormlord2)

struct BurnDriver BurnSpecstormlord2 = {
	"spec_stormlord2", NULL, "spec_spec128", NULL, "1990",
	"Stormlord II: Deliverance (128K)\0", NULL, "Hewson Consultants", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, Specstormlord2RomInfo, Specstormlord2RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Desperado 2 (Spanish) (128K)

static struct BurnRomInfo Specdesperd2RomDesc[] = {
	{ "Desperado 2 (1991)(Topo Soft)(es).tap", 0x18bc6, 0xf1aca40a, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specdesperd2, Specdesperd2, Spec128)
STD_ROM_FN(Specdesperd2)

struct BurnDriver BurnSpecdesperd2 = {
	"spec_desperd2", NULL, "spec_spec128", NULL, "1991",
	"Desperado 2 (Spanish) (128K)\0", "Password Zona 2: CLAVE", "Topo Soft", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_RUNGUN | GBF_SHOOT, 0,
	SpectrumGetZipName, Specdesperd2RomInfo, Specdesperd2RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecQAOPSpaceDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Mad Mix 2 - En el Castillo de los Fantasmas (Spanish) (128K)

static struct BurnRomInfo SpecmamiencaRomDesc[] = {
	{ "Mad Mix 2 - En el Castillo de los Fantasmas (1990)(Topo Soft)(es).tap", 0x13ae2, 0x2e343a0c, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specmamienca, Specmamienca, Spec128)
STD_ROM_FN(Specmamienca)

struct BurnDriver BurnSpecmamienca = {
	"spec_mamienca", NULL, "spec_spec128", NULL, "1990",
	"Mad Mix 2 - En el Castillo de los Fantasmas (Spanish) (128K)\0", NULL, "Topo Soft", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_MAZE | GBF_ACTION, 0,
	SpectrumGetZipName, SpecmamiencaRomInfo, SpecmamiencaRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Paris-Dakar (Spanish) (48K)

static struct BurnRomInfo SpecparisdakaresRomDesc[] = {
	{ "Paris-Dakar (1988)(Zigurat Software)(es).tap", 0x0f960, 0x15728fdf, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specparisdakares, Specparisdakares, Spectrum)
STD_ROM_FN(Specparisdakares)

struct BurnDriver BurnSpecparisdakares = {
	"spec_parisdakares", NULL, "spec_spectrum", NULL, "1988",
	"Paris-Dakar (Spanish) (48K)\0", NULL, "Zigurat Software", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_RACING, 0,
	SpectrumGetZipName, SpecparisdakaresRomInfo, SpecparisdakaresRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Perico Delgado Maillot Amarillo (Spanish) (48K)

static struct BurnRomInfo SpecpericodelgadoRomDesc[] = {
	{ "Perico Delgado Maillot Amarillo (1989)(Topo Soft)(es).tap", 0x2497f, 0x284b616d, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specpericodelgado, Specpericodelgado, Spectrum)
STD_ROM_FN(Specpericodelgado)

struct BurnDriver BurnSpecpericodelgado = {
	"spec_pericodelgado", NULL, "spec_spectrum", NULL, "1989",
	"Perico Delgado Maillot Amarillo (Spanish) (48K)\0", NULL, "Topo Soft", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_RACING, 0,
	SpectrumGetZipName, SpecpericodelgadoRomInfo, SpecpericodelgadoRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Soviet (Part 1 of 2) (128K)

static struct BurnRomInfo Specsovietpart1RomDesc[] = {
	{ "Soviet (1990)(Opera Soft)(Part 1 of 2).tap", 0x15ec0, 0x407510c3, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specsovietpart1, Specsovietpart1, Spec128)
STD_ROM_FN(Specsovietpart1)

struct BurnDriver BurnSpecsovietpart1 = {
	"spec_sovietpart1", NULL, "spec_spec128", NULL, "1990",
	"Soviet (Part 1 of 2) (128K)\0", "Set '1 keyboard' to use controller", "Opera Soft", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_RUNGUN, 0,
	SpectrumGetZipName, Specsovietpart1RomInfo, Specsovietpart1RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecQAOPSpaceDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Soviet (Part 2 of 2) (128K)

static struct BurnRomInfo Specsovietpart2RomDesc[] = {
	{ "Soviet (1990)(Opera Soft)(Part 2 of 2).tap", 0x186c4, 0x7db72ee5, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specsovietpart2, Specsovietpart2, Spec128)
STD_ROM_FN(Specsovietpart2)

struct BurnDriver BurnSpecsovietpart2 = {
	"spec_sovietpart2", "spec_sovietpart1", "spec_spec128", NULL, "1990",
	"Soviet (Part 2 of 2) (128K)\0", "Set '1 keyboard' to use controller", "Opera Soft", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE, 1, HARDWARE_SPECTRUM, GBF_RUNGUN, 0,
	SpectrumGetZipName, Specsovietpart2RomInfo, Specsovietpart2RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecQAOPSpaceDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Tour 91 (Spanish) (128K)

static struct BurnRomInfo Spectour91RomDesc[] = {
	{ "Tour 91 (1991)(Topo Soft)(es).tap", 0x26f73, 0xa27369fc, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Spectour91, Spectour91, Spec128)
STD_ROM_FN(Spectour91)

struct BurnDriver BurnSpectour91 = {
	"spec_tour91", NULL, "spec_spec128", NULL, "1991",
	"Tour 91 (Spanish) (128K)\0", NULL, "Topo Soft", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_SPORTSMISC, 0,
	SpectrumGetZipName, Spectour91RomInfo, Spectour91RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Viaje al Centro de la Tierra (Spanish) (128K)

static struct BurnRomInfo SpecviajecentterraRomDesc[] = {
	{ "Viaje al Centro de la Tierra (1989)(Topo Soft).tap", 0x1eaf1, 0xdbc5e51b, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specviajecentterra, Specviajecentterra, Spec128)
STD_ROM_FN(Specviajecentterra)

struct BurnDriver BurnSpecviajecentterra = {
	"spec_viajecentterra", NULL, "spec_spec128", NULL, "1989",
	"Viaje al Centro de la Tierra (Spanish) (128K)\0", NULL, "Topo Soft", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_ADV, 0,
	SpectrumGetZipName, SpecviajecentterraRomInfo, SpecviajecentterraRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// WWF WrestleMania (128K)

static struct BurnRomInfo SpecwwfwresmaniaRomDesc[] = {
	{ "WWF WrestleMania (1991)(Ocean Software).tap", 0x331b1, 0xb377350f, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specwwfwresmania, Specwwfwresmania, Spec128)
STD_ROM_FN(Specwwfwresmania)

struct BurnDriver BurnSpecwwfwresmania = {
	"spec_wwfwresmania", NULL, "spec_spec128", NULL, "1991",
	"WWF WrestleMania (128K)\0", NULL, "Ocean", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 2, HARDWARE_SPECTRUM, GBF_VSFIGHT, 0,
	SpectrumGetZipName, SpecwwfwresmaniaRomInfo, SpecwwfwresmaniaRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Zona 0 (Spanish) (128K)

static struct BurnRomInfo Speczona0RomDesc[] = {
	{ "Zona 0 (1991)(Topo Soft)(es).tap", 0x12b84, 0xdf7e9568, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Speczona0, Speczona0, Spec128)
STD_ROM_FN(Speczona0)

struct BurnDriver BurnSpeczona0 = {
	"spec_zona0", NULL, "spec_spec128", NULL, "1991",
	"Zona 0 (Spanish) (128K)\0", NULL, "Topo Soft", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_ACTION, 0,
	SpectrumGetZipName, Speczona0RomInfo, Speczona0RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Double Dragon (48K)

static struct BurnRomInfo Specddragon1RomDesc[] = {
	{ "Double Dragon (1989)(Melbourne House).tap", 0x29ffb, 0xa38e0c54, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specddragon1, Specddragon1, Spec128)
STD_ROM_FN(Specddragon1)

struct BurnDriver BurnSpecddragon1 = {
	"spec_ddragon1", NULL, "spec_spec128", NULL, "1989",
	"Double Dragon (48K)\0", NULL, "Melbourne House", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 2, HARDWARE_SPECTRUM, GBF_SCRFIGHT, 0,
	SpectrumGetZipName, Specddragon1RomInfo, Specddragon1RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Double Dragon II: The Revenge (128K)

static struct BurnRomInfo Specddragon2RomDesc[] = {
	{ "Double Dragon II - The Revenge (1989)(Virgin Games).tap", 0x2d64f, 0xaf872e35, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specddragon2, Specddragon2, Spec128)
STD_ROM_FN(Specddragon2)

struct BurnDriver BurnSpecddragon2 = {
	"spec_ddragon2", NULL, "spec_spec128", NULL, "1989",
	"Double Dragon II: The Revenge (128K)\0", NULL, "Virgin Games", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 2, HARDWARE_SPECTRUM, GBF_SCRFIGHT, 0,
	SpectrumGetZipName, Specddragon2RomInfo, Specddragon2RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Mortadelo y Filemon II (Part 1 of 2) (Spanish)

static struct BurnRomInfo Specmortfilemon21RomDesc[] = {
	{ "Mortadelo y Filemon II (1990)(Dro Soft)(Part 1 of 2)(es).tap", 0x0bba9, 0x3735008e, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specmortfilemon21, Specmortfilemon21, Spec128)
STD_ROM_FN(Specmortfilemon21)

struct BurnDriver BurnSpecmortfilemon21 = {
	"spec_mortfilemon21", NULL, "spec_spec128", NULL, "1990",
	"Mortadelo y Filemon II (Part 1 of 2) (Spanish)\0", NULL, "Dro Soft", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, Specmortfilemon21RomInfo, Specmortfilemon21RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Mortadelo y Filemon II (Part 2 of 2) (Spanish)

static struct BurnRomInfo Specmortfilemon22RomDesc[] = {
	{ "Mortadelo y Filemon II (1990)(Dro Soft)(Part 2 of 2)(es).tap", 0x0b6a3, 0x28472d0e, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specmortfilemon22, Specmortfilemon22, Spec128)
STD_ROM_FN(Specmortfilemon22)

struct BurnDriver BurnSpecmortfilemon22 = {
	"spec_mortfilemon22", "spec_mortfilemon21", "spec_spec128", NULL, "1990",
	"Mortadelo y Filemon II (Part 2 of 2) (Spanish)\0", NULL, "Dro Soft", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, Specmortfilemon22RomInfo, Specmortfilemon22RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Desperado (48K)

static struct BurnRomInfo SpecDesperadoRomDesc[] = {
	{ "Desperado (1986)(Central Solutions).tap", 0x9a0a, 0x488f2cd5, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecDesperado, SpecDesperado, Spectrum)
STD_ROM_FN(SpecDesperado)

struct BurnDriver BurnSpecDesperado = {
	"spec_desperado", NULL, "spec_spectrum", NULL, "1986",
	"Desperado (48K)\0", NULL, "Central Solutions", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_ADV, 0,
	SpectrumGetZipName, SpecDesperadoRomInfo, SpecDesperadoRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Mikie (48K)

static struct BurnRomInfo SpecMikieRomDesc[] = {
	{ "Mikie (1985)(Imagine Software).tap", 0xbc79, 0x45203b87, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecMikie, SpecMikie, Spectrum)
STD_ROM_FN(SpecMikie)

struct BurnDriver BurnSpecMikie = {
	"spec_mikie", NULL, "spec_spectrum", NULL, "1985",
	"Mikie (48K)\0", NULL, "Imagine Software", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_ACTION, 0,
	SpectrumGetZipName, SpecMikieRomInfo, SpecMikieRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Super Trux (128K)

static struct BurnRomInfo SpecsupertruxRomDesc[] = {
	{ "Super Trux (1988)(Elite Systems).tap", 0xdd35, 0x65fcf302, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specsupertrux, Specsupertrux, Spec128)
STD_ROM_FN(Specsupertrux)

struct BurnDriver BurnSpecsupertrux = {
	"spec_supertrux", NULL, "spec_spec128", NULL, "1988",
	"Super Trux (128K)\0", "Set 'S SINCLAIR' to use controller", "Elite Systems", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_RACING, 0,
	SpectrumGetZipName, SpecsupertruxRomInfo, SpecsupertruxRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecIntf2DIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Glider Rider (128K)

static struct BurnRomInfo SpecGliderrRomDesc[] = {
	{ "Glider Rider (1987)(Quicksilva).tap", 0xfe51, 0x2789b5bb, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecGliderr, SpecGliderr, Spec128)
STD_ROM_FN(SpecGliderr)

struct BurnDriver BurnSpecGliderr = {
	"spec_gliderr", NULL, "spec_spec128", NULL, "1987",
	"Glider Rider (128K)\0", NULL, "Quicksilva", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_ACTION, 0,
	SpectrumGetZipName, SpecGliderrRomInfo, SpecGliderrRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// North & South (128K)

static struct BurnRomInfo SpecnorthnsouthRomDesc[] = {
	{ "North & South (1991)(Infogrames)(128k).z80", 0x1bf6b, 0x8ca87f0d, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specnorthnsouth, Specnorthnsouth, Spec128)
STD_ROM_FN(Specnorthnsouth)

struct BurnDriver BurnSpecnorthnsouth = {
	"spec_northnsouth", NULL, "spec_spec128", NULL, "1991",
	"North & South (128K)\0", "Use 'O', 'P' and SPACE for set language", "Infogrames", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 2, HARDWARE_SPECTRUM, GBF_STRATEGY, 0,
	SpectrumGetZipName, SpecnorthnsouthRomInfo, SpecnorthnsouthRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Afteroids (Spanish) (48K)

static struct BurnRomInfo SpecafteroidsRomDesc[] = {
	{ "Afteroids (1988)(Zigurat Software)(es).tap", 0x0c30b, 0x53f59e66, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specafteroids, Specafteroids, Spectrum)
STD_ROM_FN(Specafteroids)

struct BurnDriver BurnSpecafteroids = {
	"spec_afteroids", NULL, "spec_spectrum", NULL, "1988",
	"Afteroids (Spanish) (48K)\0", NULL, "Zigurat Software", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_SHOOT, 0,
	SpectrumGetZipName, SpecafteroidsRomInfo, SpecafteroidsRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Humphrey (Spanish) (48K)

static struct BurnRomInfo SpechumphreyRomDesc[] = {
	{ "Humphrey (1988)(Zigurat Software)(es).tap", 0x0c30b, 0xdd0455ba, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Spechumphrey, Spechumphrey, Spectrum)
STD_ROM_FN(Spechumphrey)

struct BurnDriver BurnSpechumphrey = {
	"spec_humphrey", NULL, "spec_spectrum", NULL, "1988",
	"Humphrey (Spanish) (48K)\0", NULL, "Zigurat Software", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_ACTION, 0,
	SpectrumGetZipName, SpechumphreyRomInfo, SpechumphreyRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Shadow Dancer (128K)

static struct BurnRomInfo SpecshaddancerRomDesc[] = {
	{ "Shadow Dancer (1991)(U.S. Gold).tap", 0x239f7, 0xfaa90676, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specshaddancer, Specshaddancer, Spec128)
STD_ROM_FN(Specshaddancer)

struct BurnDriver BurnSpecshaddancer = {
	"spec_shaddancer", NULL, "spec_spec128", NULL, "1991",
	"Shadow Dancer (128K)\0", NULL, "U.S. Gold", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_RUNGUN | GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecshaddancerRomInfo, SpecshaddancerRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Airborne Ranger (48K)

static struct BurnRomInfo SpecairbornerangerRomDesc[] = {
	{ "Airborne Ranger (1988)(MicroProse Software).tap", 0x40e88, 0xa82ed5c5, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specairborneranger, Specairborneranger, Spectrum)
STD_ROM_FN(Specairborneranger)

struct BurnDriver BurnSpecairborneranger = {
	"spec_airborneranger", NULL, "spec_spectrum", NULL, "1988",
	"Airborne Ranger (48K)\0", NULL, "MicroProse Software", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_RUNGUN | GBF_STRATEGY, 0,
	SpectrumGetZipName, SpecairbornerangerRomInfo, SpecairbornerangerRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Project Stealth Fighter (128K)

static struct BurnRomInfo SpecprojstealthfighterRomDesc[] = {
	{ "Project Stealth Fighter (1990)(MicroProse Software).tap", 0x2bde9, 0xf0b7dc66, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specprojstealthfighter, Specprojstealthfighter, Spec128)
STD_ROM_FN(Specprojstealthfighter)

struct BurnDriver BurnSpecprojstealthfighter = {
	"spec_projstealthfighter", NULL, "spec_spec128", NULL, "1990",
	"Project Stealth Fighter (128K)\0", NULL, "MicroProse Software", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_SIM, 0,
	SpectrumGetZipName, SpecprojstealthfighterRomInfo, SpecprojstealthfighterRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Desperado - Gunsmoke (Trainer) (48K)

static struct BurnRomInfo Specdesperado1trnRomDesc[] = {
	{ "Desperado (1987)(Topo Soft)(Trainer).tap", 149093, 0xff12d868, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specdesperado1trn, Specdesperado1trn, Spectrum)
STD_ROM_FN(Specdesperado1trn)

struct BurnDriver BurnSpecdesperado1trn = {
	"spec_desperado1trn", NULL, "spec_spectrum", NULL, "1987",
	"Desperado - Gunsmoke (Trainer) (48K)\0", NULL, "Topo Soft", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_RUNGUN, 0,
	SpectrumGetZipName, Specdesperado1trnRomInfo, Specdesperado1trnRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Hostages (128K)

static struct BurnRomInfo SpechostagesRomDesc[] = {
	{ "Hostages (1990)(Infogrames).tap", 222554, 0x171b72b5, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Spechostages, Spechostages, Spec128)
STD_ROM_FN(Spechostages)

struct BurnDriver BurnSpechostages = {
	"spec_hostages", NULL, "spec_spec128", NULL, "1990",
	"Hostages (128K)\0", NULL, "Infogrames", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_MISC | GBF_SHOOT, 0,
	SpectrumGetZipName, SpechostagesRomInfo, SpechostagesRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Ranarama (48K)

static struct BurnRomInfo SpecranaramaRomDesc[] = {
	{ "Ranarama (1987)(Hewson Consultants).z80", 38541, 0x4894e422, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specranarama, Specranarama, Spectrum)
STD_ROM_FN(Specranarama)

struct BurnDriver BurnSpecranarama = {
	"spec_ranarama", NULL, "spec_spectrum", NULL, "1987",
	"Ranarama (48K)\0", NULL, "Hewson Consultants", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_MAZE | GBF_RUNGUN, 0,
	SpectrumGetZipName, SpecranaramaRomInfo, SpecranaramaRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Zynaps (48K)

static struct BurnRomInfo SpeczynapsRomDesc[] = {
	{ "Zynaps (1987)(Hewson Consultants).tap", 42473, 0x2e272efc, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Speczynaps, Speczynaps, Spectrum)
STD_ROM_FN(Speczynaps)

struct BurnDriver BurnSpeczynaps = {
	"spec_zynaps", NULL, "spec_spectrum", NULL, "1987",
	"Zynaps (48K)\0", NULL, "Hewson Consultants", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 2, HARDWARE_SPECTRUM, GBF_HORSHOOT, 0,
	SpectrumGetZipName, SpeczynapsRomInfo, SpeczynapsRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Uridium (48K)

static struct BurnRomInfo SpecuridiumRomDesc[] = {
	{ "Uridium (1986)(Hewson Consultants).tap", 27747, 0x28bdbbfc, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specuridium, Specuridium, Spectrum)
STD_ROM_FN(Specuridium)

struct BurnDriver BurnSpecuridium = {
	"spec_uridium", NULL, "spec_spectrum", NULL, "1986",
	"Uridium (48K)\0", NULL, "Hewson Consultants", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 2, HARDWARE_SPECTRUM, GBF_HORSHOOT, 0,
	SpectrumGetZipName, SpecuridiumRomInfo, SpecuridiumRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Netherworld (128K)

static struct BurnRomInfo SpecnetherworldRomDesc[] = {
	{ "Netherworld (1988)(Hewson Consultants).tap", 44343, 0x48db7496, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specnetherworld, Specnetherworld, Spec128)
STD_ROM_FN(Specnetherworld)

struct BurnDriver BurnSpecnetherworld = {
	"spec_netherworld", NULL, "spec_spec128", NULL, "1988",
	"Netherworld (128K)\0", NULL, "Hewson Consultants", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_SHOOT, 0,
	SpectrumGetZipName, SpecnetherworldRomInfo, SpecnetherworldRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecQAOPSpaceDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Marauder (128K)

static struct BurnRomInfo SpecmarauderRomDesc[] = {
	{ "Marauder (1988)(Hewson Consultants).tap", 54339, 0xcbd7f723, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specmarauder, Specmarauder, Spec128)
STD_ROM_FN(Specmarauder)

struct BurnDriver BurnSpecmarauder = {
	"spec_marauder", NULL, "spec_spec128", NULL, "1988",
	"Marauder (128K)\0", NULL, "Hewson Consultants", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_VERSHOOT, 0,
	SpectrumGetZipName, SpecmarauderRomInfo, SpecmarauderRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Sentinel, The (48K)

static struct BurnRomInfo SpecthesentinelRomDesc[] = {
	{ "Sentinel, The (1987)(Firebird).tap", 45932, 0xc6957a83, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specthesentinel, Specthesentinel, Spectrum)
STD_ROM_FN(Specthesentinel)

struct BurnDriver BurnSpecthesentinel = {
	"spec_thesentinel", NULL, "spec_spectrum", NULL, "1987",
	"Sentinel, The (48K)\0", NULL, "Firebird Software", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_PUZZLE, 0,
	SpectrumGetZipName, SpecthesentinelRomInfo, SpecthesentinelRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Spindizzy (48K)

static struct BurnRomInfo SpecspindizzyRomDesc[] = {
	{ "Spindizzy (1986)(Electric Dreams).tap", 47877, 0x568a095a, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specspindizzy, Specspindizzy, Spectrum)
STD_ROM_FN(Specspindizzy)

struct BurnDriver BurnSpecspindizzy = {
	"spec_spindizzy", NULL, "spec_spectrum", NULL, "1986",
	"Spindizzy (48K)\0", NULL, "Electric Dreams", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_ACTION, 0,
	SpectrumGetZipName, SpecspindizzyRomInfo, SpecspindizzyRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Hopping Mad (128K)

static struct BurnRomInfo SpechoppingmadRomDesc[] = {
	{ "Hopping Mad (1988)(Elite Systems).tap", 40347, 0x620ba84c, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Spechoppingmad, Spechoppingmad, Spec128)
STD_ROM_FN(Spechoppingmad)

struct BurnDriver BurnSpechoppingmad = {
	"spec_hoppingmad", NULL, "spec_spec128", NULL, "1988",
	"Hopping Mad (128K)\0", NULL, "Elite Systems", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_ACTION, 0,
	SpectrumGetZipName, SpechoppingmadRomInfo, SpechoppingmadRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Black Lamp (128K)

static struct BurnRomInfo SpecblacklampRomDesc[] = {
	{ "Black Lamp (1988)(Firebird Software).tap", 47656, 0xaffec634, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specblacklamp, Specblacklamp, Spec128)
STD_ROM_FN(Specblacklamp)

struct BurnDriver BurnSpecblacklamp = {
	"spec_blacklamp", NULL, "spec_spec128", NULL, "1988",
	"Black Lamp (128K)\0", NULL, "Firebird Software", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecblacklampRomInfo, SpecblacklampRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Dark Fusion (128K)

static struct BurnRomInfo SpecdarkfusionRomDesc[] = {
	{ "Dark Fusion (1988)(Gremlin Graphics).tap", 65624, 0x87c7d4d6, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specdarkfusion, Specdarkfusion, Spec128)
STD_ROM_FN(Specdarkfusion)

struct BurnDriver BurnSpecdarkfusion = {
	"spec_darkfusion", NULL, "spec_spec128", NULL, "1988",
	"Dark Fusion (128K)\0", NULL, "Gremlin Graphics", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_RUNGUN, 0,
	SpectrumGetZipName, SpecdarkfusionRomInfo, SpecdarkfusionRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Switchblade (128K)

static struct BurnRomInfo SpecswitchbladeRomDesc[] = {
	{ "Switchblade (1991)(Gremlin Graphics).tap", 52937, 0xa050f9d6, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specswitchblade, Specswitchblade, Spec128)
STD_ROM_FN(Specswitchblade)

struct BurnDriver BurnSpecswitchblade = {
	"spec_switchblade", NULL, "spec_spec128", NULL, "1991",
	"Switchblade (128K)\0", NULL, "Gremlin Graphics", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecswitchbladeRomInfo, SpecswitchbladeRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// E-SWAT (128K)

static struct BurnRomInfo SpeceswatRomDesc[] = {
	{ "E-SWAT (1990)(U.S. Gold).tap", 213337, 0x721ab30f, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Speceswat, Speceswat, Spec128)
STD_ROM_FN(Speceswat)

struct BurnDriver BurnSpeceswat = {
	"spec_eswat", NULL, "spec_spec128", NULL, "1990",
	"E-SWAT (128K)\0", NULL, "U.S. Gold", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 2, HARDWARE_SPECTRUM, GBF_RUNGUN, 0,
	SpectrumGetZipName, SpeceswatRomInfo, SpeceswatRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecIntf2DIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Final Fight (128K)

static struct BurnRomInfo SpecfinalfightRomDesc[] = {
	{ "Final Fight (1991)(U.S. Gold).tap", 199327, 0x8993d2c6, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specfinalfight, Specfinalfight, Spec128)
STD_ROM_FN(Specfinalfight)

struct BurnDriver BurnSpecfinalfight = {
	"spec_finalfight", NULL, "spec_spec128", NULL, "1991",
	"Final Fight (128K)\0", "Press 'A' to use controller", "U.S. Gold", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_SCRFIGHT, 0,
	SpectrumGetZipName, SpecfinalfightRomInfo, SpecfinalfightRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecIntf2DIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Ikari Warriors (128K)

static struct BurnRomInfo SpecikariwarriorsRomDesc[] = {
	{ "Ikari Warriors (1988)(Elite Systems).tap", 47889, 0xf3fac369, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specikariwarriors, Specikariwarriors, Spec128)
STD_ROM_FN(Specikariwarriors)

struct BurnDriver BurnSpecikariwarriors = {
	"spec_ikariwarriors", NULL, "spec_spec128", NULL, "1988",
	"Ikari Warriors (128K)\0", NULL, "Elite Systems", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 2, HARDWARE_SPECTRUM, GBF_RUNGUN, 0,
	SpectrumGetZipName, SpecikariwarriorsRomInfo, SpecikariwarriorsRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Mercs (128K)

static struct BurnRomInfo SpecmercsRomDesc[] = {
	{ "Mercs (1991)(U.S. Gold).tap", 191310, 0x8b4f3dc9, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specmercs, Specmercs, Spec128)
STD_ROM_FN(Specmercs)

struct BurnDriver BurnSpecmercs = {
	"spec_mercs", NULL, "spec_spec128", NULL, "1991",
	"Mercs (128K)\0", NULL, "U.S. Gold", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 2, HARDWARE_SPECTRUM, GBF_RUNGUN, 0,
	SpectrumGetZipName, SpecmercsRomInfo, SpecmercsRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Turrican (48K)

static struct BurnRomInfo Specturrican1RomDesc[] = {
	{ "Turrican (1990)(Rainbow Arts).tap", 266448, 0xc5cfad1a, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specturrican1, Specturrican1, Spectrum)
STD_ROM_FN(Specturrican1)

struct BurnDriver BurnSpecturrican1 = {
	"spec_turrican1", NULL, "spec_spectrum", NULL, "1990",
	"Turrican (48K)\0", NULL, "Rainbow Arts", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_PLATFORM | GBF_RUNGUN, 0,
	SpectrumGetZipName, Specturrican1RomInfo, Specturrican1RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Turrican II (128K)

static struct BurnRomInfo Specturrican2RomDesc[] = {
	{ "Turrican II (1991)(Rainbow Arts).tap", 357318, 0x48f546ce, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specturrican2, Specturrican2, Spec128)
STD_ROM_FN(Specturrican2)

struct BurnDriver BurnSpecturrican2 = {
	"spec_turrican2", NULL, "spec_spec128", NULL, "1991",
	"Turrican II (128K)\0", NULL, "Rainbow Arts", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_PLATFORM | GBF_RUNGUN, 0,
	SpectrumGetZipName, Specturrican2RomInfo, Specturrican2RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Tetris 2 (128K)

static struct BurnRomInfo Spectetris2RomDesc[] = {
	{ "Tetris 2 (1990)(Fuxoft).tap", 24510, 0xeb9220fd, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Spectetris2, Spectetris2, Spec128)
STD_ROM_FN(Spectetris2)

struct BurnDriver BurnSpectetris2 = {
	"spec_tetris2", NULL, "spec_spec128", NULL, "1990",
	"Tetris 2 (128K)\0", NULL, "Fuxoft", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 2, HARDWARE_SPECTRUM, GBF_PUZZLE, 0,
	SpectrumGetZipName, Spectetris2RomInfo, Spectetris2RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecIntf2DIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Finders Keepers (48K)

static struct BurnRomInfo SpecfinderskeepersRomDesc[] = {
	{ "Finders Keepers (1985)(Mastertronic).tap", 44174, 0x06f43742, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specfinderskeepers, Specfinderskeepers, Spectrum)
STD_ROM_FN(Specfinderskeepers)

struct BurnDriver BurnSpecfinderskeepers = {
	"spec_finderskeepers", NULL, "spec_spectrum", NULL, "1985",
	"Finders Keepers (48K)\0", NULL, "Mastertronic", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecfinderskeepersRomInfo, SpecfinderskeepersRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Spellbound (128K)

static struct BurnRomInfo SpecspellboundRomDesc[] = {
	{ "Spellbound (1985)(Mastertronic).tap", 112432, 0x275e1ec1, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specspellbound, Specspellbound, Spec128)
STD_ROM_FN(Specspellbound)

struct BurnDriver BurnSpecspellbound = {
	"spec_spellbound", NULL, "spec_spec128", NULL, "1985",
	"Spellbound (128K)\0", "A, Z and SPACE to choose and set main options", "Mastertronic Added Dimension", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_ADV, 0,
	SpectrumGetZipName, SpecspellboundRomInfo, SpecspellboundRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Knight Tyme (128K)

static struct BurnRomInfo SpecknighttymeRomDesc[] = {
	{ "Knight Tyme (1986)(Mastertronic).tap", 119348, 0x79c80fac, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specknighttyme, Specknighttyme, Spec128)
STD_ROM_FN(Specknighttyme)

struct BurnDriver BurnSpecknighttyme = {
	"spec_knighttyme", NULL, "spec_spec128", NULL, "1986",
	"Knight Tyme (128K)\0", "Options menu = A, Z, Space", "Mastertronic Added Dimension", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_ADV, 0,
	SpectrumGetZipName, SpecknighttymeRomInfo, SpecknighttymeRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Stormbringer (128K)

static struct BurnRomInfo SpecstormbringerRomDesc[] = {
	{ "Stormbringer (1987)(Mastertronic).tap", 102898, 0x8819d367, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specstormbringer, Specstormbringer, Spec128)
STD_ROM_FN(Specstormbringer)

struct BurnDriver BurnSpecstormbringer = {
	"spec_stormbringer", NULL, "spec_spec128", NULL, "1987",
	"Stormbringer (128K)\0", NULL, "Mastertronic", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecstormbringerRomInfo, SpecstormbringerRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Hades Nebula (48K)

static struct BurnRomInfo SpechadesnebulaRomDesc[] = {
	{ "Hades Nebula (1987)(Nexus Productions).tap", 43264, 0xbeac467c, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Spechadesnebula, Spechadesnebula, Spectrum)
STD_ROM_FN(Spechadesnebula)

struct BurnDriver BurnSpechadesnebula = {
	"spec_hadesnebula", NULL, "spec_spectrum", NULL, "1987",
	"Hades Nebula (48K)\0", NULL, "Nexus Productions", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_VERSHOOT, 0,
	SpectrumGetZipName, SpechadesnebulaRomInfo, SpechadesnebulaRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Arc of Yesod, The (128K)

static struct BurnRomInfo SpecarcofyesodRomDesc[] = {
	{ "Arc of Yesod, The (1985)(Thor Computer Software).tap", 74016, 0xb4c5351b, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specarcofyesod, Specarcofyesod, Spec128)
STD_ROM_FN(Specarcofyesod)

struct BurnDriver BurnSpecarcofyesod = {
	"spec_arcofyesod", NULL, "spec_spec128", NULL, "1985",
	"Arc of Yesod, The (128K)\0", NULL, "Thor Computer Software", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_PLATFORM | GBF_ADV, 0,
	SpectrumGetZipName, SpecarcofyesodRomInfo, SpecarcofyesodRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Nodes of Yesod (128K)

static struct BurnRomInfo SpecNoyesod128RomDesc[] = {
	{ "Nodes of Yesod 128K (1985)(Odin Computer Graphics).tap", 83920, 0xdea25751, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecNoyesod128, SpecNoyesod128, Spec128)
STD_ROM_FN(SpecNoyesod128)

struct BurnDriver BurnSpecNoyesod128 = {
	"spec_noyesod128", NULL, "spec_spec128", NULL, "1985",
	"Nodes of Yesod (128K)\0", NULL, "Odin Computer Graphics", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_PLATFORM | GBF_ADV, 0,
	SpectrumGetZipName, SpecNoyesod128RomInfo, SpecNoyesod128RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Nodes of Yesod (48K)

static struct BurnRomInfo SpecNoyesod48RomDesc[] = {
	{ "Nodes of Yesod 48K (1985)(Odin Computer Graphics).tap", 36898, 0x1b5fe585, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecNoyesod48, SpecNoyesod48, Spectrum)
STD_ROM_FN(SpecNoyesod48)

struct BurnDriver BurnSpecNoyesod48 = {
	"spec_noyesod48", "spec_noyesod128", "spec_spectrum", NULL, "1985",
	"Nodes of Yesod (48K)\0", NULL, "Odin Computer Graphics", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE, 1, HARDWARE_SPECTRUM, GBF_PLATFORM | GBF_ADV, 0,
	SpectrumGetZipName, SpecNoyesod48RomInfo, SpecNoyesod48RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Blood Brothers (Module 1) (128K)

static struct BurnRomInfo Specbloodbrothers1RomDesc[] = {
	{ "Blood Brothers (Module 1)(1988)(Gremlin Graphics).z80", 42931, 0xb32fcea7, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specbloodbrothers1, Specbloodbrothers1, Spec128)
STD_ROM_FN(Specbloodbrothers1)

struct BurnDriver BurnSpecbloodbrothers1 = {
	"spec_bloodbrothers1", NULL, "spec_spec128", NULL, "1988",
	"Blood Brothers (Module 1) (128K)\0", NULL, "Gremlin Graphics", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 2, HARDWARE_SPECTRUM, GBF_SHOOT | GBF_PLATFORM, 0,
	SpectrumGetZipName, Specbloodbrothers1RomInfo, Specbloodbrothers1RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecQAOPSpaceDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Blood Brothers (Module 2) (128K)

static struct BurnRomInfo Specbloodbrothers2RomDesc[] = {
	{ "Blood Brothers (Module 2)(1988)(Gremlin Graphics).z80", 42790, 0x3fc07585, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specbloodbrothers2, Specbloodbrothers2, Spec128)
STD_ROM_FN(Specbloodbrothers2)

struct BurnDriver BurnSpecbloodbrothers2 = {
	"spec_bloodbrothers2", "spec_bloodbrothers1", "spec_spec128", NULL, "1988",
	"Blood Brothers (Module 2) (128K)\0", NULL, "Gremlin Graphics", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE, 2, HARDWARE_SPECTRUM, GBF_SHOOT | GBF_PLATFORM, 0,
	SpectrumGetZipName, Specbloodbrothers2RomInfo, Specbloodbrothers2RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecQAOPSpaceDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Blood Brothers (Module 3) (128K)

static struct BurnRomInfo Specbloodbrothers3RomDesc[] = {
	{ "Blood Brothers (Module 3)(1988)(Gremlin Graphics).z80", 42935, 0x735098ec, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specbloodbrothers3, Specbloodbrothers3, Spec128)
STD_ROM_FN(Specbloodbrothers3)

struct BurnDriver BurnSpecbloodbrothers3 = {
	"spec_bloodbrothers3", "spec_bloodbrothers1", "spec_spec128", NULL, "1988",
	"Blood Brothers (Module 3) (128K)\0", NULL, "Gremlin Graphics", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE, 2, HARDWARE_SPECTRUM, GBF_SHOOT | GBF_PLATFORM, 0,
	SpectrumGetZipName, Specbloodbrothers3RomInfo, Specbloodbrothers3RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecQAOPSpaceDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Destiny Mission (128K)

static struct BurnRomInfo SpecdestinymissionRomDesc[] = {
	{ "Destiny Mission (1990)(Williams Technology).tap", 42759, 0x71528fa3, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specdestinymission, Specdestinymission, Spec128)
STD_ROM_FN(Specdestinymission)

struct BurnDriver BurnSpecdestinymission = {
	"spec_destinymission", NULL, "spec_spec128", NULL, "1990",
	"Destiny Mission (128K)\0", NULL, "Williams Technology", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_HORSHOOT, 0,
	SpectrumGetZipName, SpecdestinymissionRomInfo, SpecdestinymissionRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Frost Byte (48K)

static struct BurnRomInfo SpecfrostbyteRomDesc[] = {
	{ "Frost Byte (1986)(Mikro-Gen).tap", 49465, 0x895fc6d8, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specfrostbyte, Specfrostbyte, Spectrum)
STD_ROM_FN(Specfrostbyte)

struct BurnDriver BurnSpecfrostbyte = {
	"spec_frostbyte", NULL, "spec_spectrum", NULL, "1986",
	"Frost Byte (48K)\0", NULL, "Mikro-Gen", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_PLATFORM | GBF_MAZE, 0,
	SpectrumGetZipName, SpecfrostbyteRomInfo, SpecfrostbyteRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Gauntlet III - The Final Quest (128K)

static struct BurnRomInfo Specgauntlet3RomDesc[] = {
	{ "Gauntlet III - The Final Quest 128K (1991)(U.S. Gold).tap", 352550, 0x9dd3f5fc, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specgauntlet3, Specgauntlet3, Spec128)
STD_ROM_FN(Specgauntlet3)

struct BurnDriver BurnSpecgauntlet3 = {
	"spec_gauntlet3", NULL, "spec_spec128", NULL, "1991",
	"Gauntlet III - The Final Quest (128K)\0", "Set 'Interface 2' to use controller", "U.S. Gold", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 2, HARDWARE_SPECTRUM, GBF_MAZE | GBF_RUNGUN, 0,
	SpectrumGetZipName, Specgauntlet3RomInfo, Specgauntlet3RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecIntf2DIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Hydrofool (128K)

static struct BurnRomInfo SpechydrofoolRomDesc[] = {
	{ "Hydrofool (1987)(Faster Than Light).tap", 45889, 0x82e3db69, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Spechydrofool, Spechydrofool, Spec128)
STD_ROM_FN(Spechydrofool)

struct BurnDriver BurnSpechydrofool = {
	"spec_hydrofool", NULL, "spec_spec128", NULL, "1987",
	"Hydrofool (128K)\0", NULL, "Faster Than Light", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_ACTION, 0,
	SpectrumGetZipName, SpechydrofoolRomInfo, SpechydrofoolRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Kong's Revenge - Part 1 (Spanish) (128K)

static struct BurnRomInfo Speckongsrevenge1RomDesc[] = {
	{ "Kong's Revenge (Part 1)(1991)(Zigurat Software).tap", 56868, 0x20a7e7ce, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Speckongsrevenge1, Speckongsrevenge1, Spec128)
STD_ROM_FN(Speckongsrevenge1)

struct BurnDriver BurnSpeckongsrevenge1 = {
	"spec_kongsrevenge1", NULL, "spec_spec128", NULL, "1991",
	"Kong's Revenge - Part 1 (Spanish) (128K)\0", NULL, "Zigurat Software", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, Speckongsrevenge1RomInfo, Speckongsrevenge1RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Kong's Revenge - Part 2 (Spanish) (128K)

static struct BurnRomInfo Speckongsrevenge2RomDesc[] = {
	{ "Kong's Revenge (Part 2)(1991)(Zigurat Software).tap", 56868, 0xdf77b41f, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Speckongsrevenge2, Speckongsrevenge2, Spec128)
STD_ROM_FN(Speckongsrevenge2)

struct BurnDriver BurnSpeckongsrevenge2 = {
	"spec_kongsrevenge2", "spec_kongsrevenge1", "spec_spec128", NULL, "1991",
	"Kong's Revenge - Part 2 (Spanish) (128K)\0", "Password: CHIRIPA - use num-keys from 1 to 7 to set password", "Zigurat Software", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE, 1, HARDWARE_SPECTRUM, GBF_RUNGUN, 0,
	SpectrumGetZipName, Speckongsrevenge2RomInfo, Speckongsrevenge2RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// North Star (48K)

static struct BurnRomInfo SpecnorthstarRomDesc[] = {
	{ "North Star (1988)(Gremlin Graphics).tap", 49369, 0x61d487db, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specnorthstar, Specnorthstar, Spectrum)
STD_ROM_FN(Specnorthstar)

struct BurnDriver BurnSpecnorthstar = {
	"spec_northstar", NULL, "spec_spectrum", NULL, "1988",
	"North Star (48K)\0", NULL, "Gremlin Graphics", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecnorthstarRomInfo, SpecnorthstarRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Athena (48K)

static struct BurnRomInfo SpecathenaRomDesc[] = {
	{ "Athena (1987)(Imagine).tap", 102859, 0xb457ae80, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specathena, Specathena, Spectrum)
STD_ROM_FN(Specathena)

struct BurnDriver BurnSpecathena = {
	"spec_athena", NULL, "spec_spectrum", NULL, "1987",
	"Athena (48K)\0", NULL, "Imagine Software", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecathenaRomInfo, SpecathenaRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Psycho Hopper (48K)

static struct BurnRomInfo SpecpsychohopperRomDesc[] = {
	{ "Psycho Hopper (1990)(Mastertronic Plus).tap", 50178, 0xbeda1169, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specpsychohopper, Specpsychohopper, Spectrum)
STD_ROM_FN(Specpsychohopper)

struct BurnDriver BurnSpecpsychohopper = {
	"spec_psychohopper", NULL, "spec_spectrum", NULL, "1989-90",
	"Psycho Hopper (48K)\0", "Set '3 Sinclair' to use controller", "Mastertronic Plus", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_RUNGUN | GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecpsychohopperRomInfo, SpecpsychohopperRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecIntf2DIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Psycho Pigs U.X.B. (128K)

static struct BurnRomInfo SpecpsychopigsRomDesc[] = {
	{ "Psycho Pigs U.X.B. (1988)(U.S. Gold).tap", 50190, 0x3632f18a, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specpsychopigs, Specpsychopigs, Spec128)
STD_ROM_FN(Specpsychopigs)

struct BurnDriver BurnSpecpsychopigs = {
	"spec_psychopigs", NULL, "spec_spec128", NULL, "1988",
	"Psycho Pigs U.X.B. (128K)\0", NULL, "U.S. Gold", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 2, HARDWARE_SPECTRUM, GBF_ACTION, 0,
	SpectrumGetZipName, SpecpsychopigsRomInfo, SpecpsychopigsRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Snoopy (48K)

static struct BurnRomInfo SpecsnoopyRomDesc[] = {
	{ "Snoopy (1990)(The Edge).tap", 40149, 0xa24e7539, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specsnoopy, Specsnoopy, Spectrum)
STD_ROM_FN(Specsnoopy)

struct BurnDriver BurnSpecsnoopy = {
	"spec_snoopy", NULL, "spec_spectrum", NULL, "1990",
	"Snoopy (48K)\0", NULL, "The Edge", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_ADV, 0,
	SpectrumGetZipName, SpecsnoopyRomInfo, SpecsnoopyRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Stainless Steel (48K)

static struct BurnRomInfo SpecstainlesssteelRomDesc[] = {
	{ "Stainless Steel (1986)(Mikro-Gen).tap", 44663, 0xc01c4113, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specstainlesssteel, Specstainlesssteel, Spectrum)
STD_ROM_FN(Specstainlesssteel)

struct BurnDriver BurnSpecstainlesssteel = {
	"spec_stainlesssteel", NULL, "spec_spectrum", NULL, "1986",
	"Stainless Steel (48K)\0", NULL, "Mikro-Gen", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_RUNGUN, 0,
	SpectrumGetZipName, SpecstainlesssteelRomInfo, SpecstainlesssteelRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Tai Chi Tortoise (128K)

static struct BurnRomInfo SpectaichitortoiseRomDesc[] = {
	{ "Tai Chi Tortoise (1991)(Zeppelin Games).tap", 29553, 0x5a60396a, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Spectaichitortoise, Spectaichitortoise, Spec128)
STD_ROM_FN(Spectaichitortoise)

struct BurnDriver BurnSpectaichitortoise = {
	"spec_taichitortoise", NULL, "spec_spec128", NULL, "1991",
	"Tai Chi Tortoise (128K)\0", NULL, "Zeppelin Games", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpectaichitortoiseRomInfo, SpectaichitortoiseRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Towdie (English) (128K)

static struct BurnRomInfo SpectowdieenRomDesc[] = {
	{ "Towdie (English)(1994)(Ultrasoft).tap", 55179, 0x20bd2e70, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Spectowdieen, Spectowdieen, Spec128)
STD_ROM_FN(Spectowdieen)

struct BurnDriver BurnSpectowdieen = {
	"spec_towdieen", NULL, "spec_spec128", NULL, "1994-2011",
	"Towdie (English) (128K)\0", NULL, "Ultrasoft - Softhouse", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_PLATFORM | GBF_ADV, 0,
	SpectrumGetZipName, SpectowdieenRomInfo, SpectowdieenRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecQAOPMDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Towdie (Slovak) (128K)

static struct BurnRomInfo SpectowdieskRomDesc[] = {
	{ "Towdie (Slovak)(1994)(Ultrasoft).tap", 55187, 0x04ba91b1, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Spectowdiesk, Spectowdiesk, Spec128)
STD_ROM_FN(Spectowdiesk)

struct BurnDriver BurnSpectowdiesk = {
	"spec_towdiesk", "spec_towdieen", "spec_spec128", NULL, "1994",
	"Towdie (Slovak) (128K)\0", NULL, "Ultrasoft", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE, 1, HARDWARE_SPECTRUM, GBF_PLATFORM | GBF_ADV, 0,
	SpectrumGetZipName, SpectowdieskRomInfo, SpectowdieskRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecQAOPMDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Typhoon (128K)

static struct BurnRomInfo SpectyphoonRomDesc[] = {
	{ "Typhoon (1988)(Imagine Software).tap", 130475, 0x1ab3aa4a, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Spectyphoon, Spectyphoon, Spec128)
STD_ROM_FN(Spectyphoon)

struct BurnDriver BurnSpectyphoon = {
	"spec_typhoon", NULL, "spec_spec128", NULL, "1988",
	"Typhoon (128K)\0", NULL, "Imagine Software", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_SHOOT, 0,
	SpectrumGetZipName, SpectyphoonRomInfo, SpectyphoonRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Silent Shadow (48K)

static struct BurnRomInfo SpecsilentshadowRomDesc[] = {
	{ "Silent Shadow (1988)(Topo Soft).tap", 138528, 0x76e0c5b8, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specsilentshadow, Specsilentshadow, Spectrum)
STD_ROM_FN(Specsilentshadow)

struct BurnDriver BurnSpecsilentshadow = {
	"spec_silentshadow", NULL, "spec_spectrum", NULL, "1988",
	"Silent Shadow (48K)\0", "Set '1. KEYS' to use controller", "Topo Soft", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 2, HARDWARE_SPECTRUM, GBF_VERSHOOT, 0,
	SpectrumGetZipName, SpecsilentshadowRomInfo, SpecsilentshadowRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecQAOPSpaceDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Night Breed (128K)

static struct BurnRomInfo SpecnbreedRomDesc[] = {
	{ "Night Breed (1990)(Ocean).tap", 188653, 0x5701dcaf, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specnbreed, Specnbreed, Spec128)
STD_ROM_FN(Specnbreed)

struct BurnDriver BurnSpecnbreed = {
	"spec_nbreed", NULL, "spec_spec128", NULL, "1990",
	"Night Breed (128K)\0", NULL, "Ocean", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_ACTION, 0,
	SpectrumGetZipName, SpecnbreedRomInfo, SpecnbreedRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Bobby Bearing (48K)

static struct BurnRomInfo SpecbobbybearingRomDesc[] = {
	{ "Bobby Bearing (1986)(The Edge).tap", 27778, 0x6b6895d2, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specbobbybearing, Specbobbybearing, Spectrum)
STD_ROM_FN(Specbobbybearing)

struct BurnDriver BurnSpecbobbybearing = {
	"spec_bobbybearing", NULL, "spec_spectrum", NULL, "1986",
	"Bobby Bearing (48K)\0", "6, 7 and 0 to select control type", "The Edge", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_MAZE, 0,
	SpectrumGetZipName, SpecbobbybearingRomInfo, SpecbobbybearingRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Incredible Shrinking Sphere (48K)

static struct BurnRomInfo SpecincshrinksphereRomDesc[] = {
	{ "Incredible Shrinking Sphere (1989)(Electric Dreams).tap", 42625, 0xf89b035d, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specincshrinksphere, Specincshrinksphere, Spectrum)
STD_ROM_FN(Specincshrinksphere)

struct BurnDriver BurnSpecincshrinksphere = {
	"spec_incshrinksphere", NULL, "spec_spectrum", NULL, "1989",
	"Incredible Shrinking Sphere (48K)\0", NULL, "Electric Dreams", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_MAZE, 0,
	SpectrumGetZipName, SpecincshrinksphereRomInfo, SpecincshrinksphereRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Alien Storm (128K)

static struct BurnRomInfo SpecAlienstormRomDesc[] = {
	{ "Alien Storm (1991)(U.S. Gold).tap", 252760, 0x8aa6b74d, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecAlienstorm, SpecAlienstorm, Spec128)
STD_ROM_FN(SpecAlienstorm)

struct BurnDriver BurnSpecAlienstorm = {
	"spec_alienstorm", NULL, "spec_spec128", NULL, "1991",
	"Alien Storm (128K)\0", NULL, "U.S. Gold", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 2, HARDWARE_SPECTRUM, GBF_SCRFIGHT | GBF_SHOOT, 0,
	SpectrumGetZipName, SpecAlienstormRomInfo, SpecAlienstormRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Guerrilla War (128K)

static struct BurnRomInfo SpecGuerrillawarRomDesc[] = {
	{ "Guerrilla War (1988)(Imagine Software).z80", 91016, 0xaf3aabbe, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecGuerrillawar, SpecGuerrillawar, Spec128)
STD_ROM_FN(SpecGuerrillawar)

struct BurnDriver BurnSpecGuerrillawar = {
	"spec_guerrillawar", NULL, "spec_spec128", NULL, "1988",
	"Guerrilla War (128K)\0", NULL, "Imagine Software", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 2, HARDWARE_SPECTRUM, GBF_RUNGUN, 0,
	SpectrumGetZipName, SpecGuerrillawarRomInfo, SpecGuerrillawarRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Predator (Trainer) (128K)

static struct BurnRomInfo SpecPredatorRomDesc[] = {
	{ "Predator 128K (1987)(Activision)[t +2].tap", 196738, 0xb7916d8c, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecPredator, SpecPredator, Spec128)
STD_ROM_FN(SpecPredator)

struct BurnDriver BurnSpecPredator = {
	"spec_predator", NULL, "spec_spec128", NULL, "1987",
	"Predator (Trainer) (128K)\0", NULL, "Activision", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_RUNGUN, 0,
	SpectrumGetZipName, SpecPredatorRomInfo, SpecPredatorRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Predator 2 (128K)

static struct BurnRomInfo SpecPredator2RomDesc[] = {
	{ "Predator 2 (1991)(Image Works).tap", 120822, 0xa287648e, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecPredator2, SpecPredator2, Spec128)
STD_ROM_FN(SpecPredator2)

struct BurnDriver BurnSpecPredator2 = {
	"spec_predator2", NULL, "spec_spec128", NULL, "1991",
	"Predator 2 (128K)\0", NULL, "Image Works", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_SHOOT, 0,
	SpectrumGetZipName, SpecPredator2RomInfo, SpecPredator2RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecQAOPSpaceDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Street Fighter II (128K)

static struct BurnRomInfo SpecStreetfighter2RomDesc[] = {
	{ "Street Fighter II (1993)(Go).tap", 519072, 0x052f9f1f, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecStreetfighter2, SpecStreetfighter2, Spec128)
STD_ROM_FN(SpecStreetfighter2)

struct BurnDriver BurnSpecStreetfighter2 = {
	"spec_streetfighter2", NULL, "spec_spec128", NULL, "1993",
	"Street Fighter II (128K)\0", NULL, "Go", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 2, HARDWARE_SPECTRUM, GBF_VSFIGHT, 0,
	SpectrumGetZipName, SpecStreetfighter2RomInfo, SpecStreetfighter2RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Terminator 2 - Judgement Day (128K)

static struct BurnRomInfo SpecTerminator2RomDesc[] = {
	{ "Terminator 2 - Judgement Day (1991)(Ocean Software).tap", 105018, 0x23bc34fc, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecTerminator2, SpecTerminator2, Spec128)
STD_ROM_FN(SpecTerminator2)

struct BurnDriver BurnSpecTerminator2 = {
	"spec_terminator2", NULL, "spec_spec128", NULL, "1991",
	"Terminator 2 - Judgement Day (128K)\0", NULL, "Ocean Software", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_ACTION | GBF_VSFIGHT, 0,
	SpectrumGetZipName, SpecTerminator2RomInfo, SpecTerminator2RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Karateka (Spanish) (128K)

static struct BurnRomInfo SpeckaratekaRomDesc[] = {
	{ "Karateka (1990)(Dro Soft).tap", 67058, 0x61d5c8ac, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Speckarateka, Speckarateka, Spec128)
STD_ROM_FN(Speckarateka)

struct BurnDriver BurnSpeckarateka = {
	"spec_karateka", NULL, "spec_spec128", NULL, "1990",
	"Karateka (Spanish) (128K)\0", NULL, "Dro Soft", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_VSFIGHT, 0,
	SpectrumGetZipName, SpeckaratekaRomInfo, SpeckaratekaRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Gunfright (48K)

static struct BurnRomInfo SpecGunfrightRomDesc[] = {
	{ "Gunfright (1986)(Ultimate Play The Game).tap", 43004, 0x2fe5588c, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecGunfright, SpecGunfright, Spectrum)
STD_ROM_FN(SpecGunfright)

struct BurnDriver BurnSpecGunfright = {
	"spec_gunfright", NULL, "spec_spectrum", NULL, "1986",
	"Gunfright (48K)\0", NULL, "Ultimate Play The Game", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_ADV, 0,
	SpectrumGetZipName, SpecGunfrightRomInfo, SpecGunfrightRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Arcadia (16K)

static struct BurnRomInfo SpecArcadiaRomDesc[] = {
	{ "Arcadia (1982)(Imagine)(16K).tap", 8522, 0x63ef88a9, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecArcadia, SpecArcadia, Spectrum)
STD_ROM_FN(SpecArcadia)

struct BurnDriver BurnSpecArcadia = {
	"spec_arcadia", NULL, "spec_spectrum", NULL, "1982",
	"Arcadia (16K)\0", NULL, "Imagine Software", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_SHOOT, 0,
	SpectrumGetZipName, SpecArcadiaRomInfo, SpecArcadiaRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Batty (48K)

static struct BurnRomInfo SpecBattyRomDesc[] = {
	{ "Batty (1987)(Hit-Pak).tap", 30142, 0x83ed1c93, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecBatty, SpecBatty, Spectrum)
STD_ROM_FN(SpecBatty)

struct BurnDriver BurnSpecBatty = {
	"spec_batty", NULL, "spec_spectrum", NULL, "1987",
	"Batty (48K)\0", "Press 'A' for 1P options and 'B' for 2P options", "Hit-Pak", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 2, HARDWARE_SPECTRUM, GBF_BREAKOUT, 0,
	SpectrumGetZipName, SpecBattyRomInfo, SpecBattyRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Cookie (16K)

static struct BurnRomInfo SpecCookieRomDesc[] = {
	{ "Cookie (1983)(Ultimate Play The Game)(16K).tap", 15646, 0xfffe62ca, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecCookie, SpecCookie, Spec128)
STD_ROM_FN(SpecCookie)

struct BurnDriver BurnSpecCookie = {
	"spec_cookie", NULL, "spec_spec128", NULL, "1983",
	"Cookie (16K)\0", NULL, "Ultimate Play The Game", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 2, HARDWARE_SPECTRUM, GBF_ACTION, 0,
	SpectrumGetZipName, SpecCookieRomInfo, SpecCookieRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Day in the Life, A (48K)

static struct BurnRomInfo SpecDayinthelifeRomDesc[] = {
	{ "Day in the Life, A (1985)(Micromega).tap", 50099, 0x80e1e221, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecDayinthelife, SpecDayinthelife, Spectrum)
STD_ROM_FN(SpecDayinthelife)

struct BurnDriver BurnSpecDayinthelife = {
	"spec_dayinthelife", NULL, "spec_spectrum", NULL, "1985",
	"Day in the Life, A (48K)\0", NULL, "Micromega", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_ACTION, 0,
	SpectrumGetZipName, SpecDayinthelifeRomInfo, SpecDayinthelifeRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Firefly (128K)

static struct BurnRomInfo SpecFireflyRomDesc[] = {
	{ "Firefly (1988)(Ocean).tap", 45885, 0x4c890b6f, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecFirefly, SpecFirefly, Spec128)
STD_ROM_FN(SpecFirefly)

struct BurnDriver BurnSpecFirefly = {
	"spec_firefly", NULL, "spec_spec128", NULL, "1988",
	"Firefly (128K)\0", NULL, "Ocean", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_SHOOT | GBF_STRATEGY, 0,
	SpectrumGetZipName, SpecFireflyRomInfo, SpecFireflyRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Gauntlet (128K)

static struct BurnRomInfo SpecGauntletRomDesc[] = {
	{ "Gauntlet (1987)(U.S. Gold)(48K-128K).tap", 150888, 0x173177cc, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecGauntlet, SpecGauntlet, Spec128)
STD_ROM_FN(SpecGauntlet)

struct BurnDriver BurnSpecGauntlet = {
	"spec_gauntlet", NULL, "spec_spec128", NULL, "1986",
	"Gauntlet (128K)\0", "In menu use Cursor Keys 5-6-7-8 + SPACE", "U.S. Gold", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 2, HARDWARE_SPECTRUM, GBF_MAZE | GBF_RUNGUN, 0,
	SpectrumGetZipName, SpecGauntletRomInfo, SpecGauntletRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Hyper Active (48K)

static struct BurnRomInfo SpecHyperactiveRomDesc[] = {
	{ "Hyper Active (1988)(Sinclair User).tap", 25305, 0xb7aa30e8, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecHyperactive, SpecHyperactive, Spectrum)
STD_ROM_FN(SpecHyperactive)

struct BurnDriver BurnSpecHyperactive = {
	"spec_hyperactive", NULL, "spec_spectrum", NULL, "1988",
	"Hyper Active (48K)\0", NULL, "Sinclair User", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_HORSHOOT, 0,
	SpectrumGetZipName, SpecHyperactiveRomInfo, SpecHyperactiveRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Sabre Wulf (48K)

static struct BurnRomInfo SpecSabrewulfRomDesc[] = {
	{ "Sabre Wulf (1984)(Ultimate Play The Game).z80", 39457, 0x75fe1462, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecSabrewulf, SpecSabrewulf, Spectrum)
STD_ROM_FN(SpecSabrewulf)

struct BurnDriver BurnSpecSabrewulf = {
	"spec_sabrewulf", NULL, "spec_spectrum", NULL, "1984",
	"Sabre Wulf (48K)\0", NULL, "Ultimate Play The Game", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 2, HARDWARE_SPECTRUM, GBF_ACTION, 0,
	SpectrumGetZipName, SpecSabrewulfRomInfo, SpecSabrewulfRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// The Biz (48K)

static struct BurnRomInfo SpecThebizRomDesc[] = {
	{ "The Biz (1984)(Virgin Games).tap", 47268, 0x6773d204, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecThebiz, SpecThebiz, Spectrum)
STD_ROM_FN(SpecThebiz)

struct BurnDriver BurnSpecThebiz = {
	"spec_thebiz", NULL, "spec_spectrum", NULL, "1984",
	"The Biz (48K)\0", NULL, "Virgin Games", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_STRATEGY, 0,
	SpectrumGetZipName, SpecThebizRomInfo, SpecThebizRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Trashman (48K)

static struct BurnRomInfo SpecTrashmanRomDesc[] = {
	{ "Trashman (1984)(New Generation).tap", 33845, 0x79aac256, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecTrashman, SpecTrashman, Spectrum)
STD_ROM_FN(SpecTrashman)

struct BurnDriver BurnSpecTrashman = {
	"spec_trashman", NULL, "spec_spectrum", NULL, "1984",
	"Trashman (48K)\0", NULL, "New Generation", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_ACTION, 0,
	SpectrumGetZipName, SpecTrashmanRomInfo, SpecTrashmanRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Aliens (UK Version) (48K)

static struct BurnRomInfo SpecAliensukRomDesc[] = {
	{ "Aliens UK (1986)(Electric Dreams).tap", 48291, 0xdb83e425, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecAliensuk, SpecAliensuk, Spectrum)
STD_ROM_FN(SpecAliensuk)

struct BurnDriver BurnSpecAliensuk = {
	"spec_aliensuk", NULL, "spec_spectrum", NULL, "1986",
	"Aliens (UK Version) (48K)\0", "Press 'Space' to walk through doors", "Electric Dreams", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_ADV | GBF_SHOOT, 0,
	SpectrumGetZipName, SpecAliensukRomInfo, SpecAliensukRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecCursorKeysDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Aliens (US Version) (48K)

static struct BurnRomInfo SpecAliensusRomDesc[] = {
	{ "Aliens US (1987)(Electric Dreams).tap", 192788, 0xd7c22d0d, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecAliensus, SpecAliensus, Spectrum)
STD_ROM_FN(SpecAliensus)

struct BurnDriver BurnSpecAliensus = {
	"spec_aliensus", NULL, "spec_spectrum", NULL, "1987",
	"Aliens (US Version) (48K)\0", NULL, "Electric Dreams", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_ADV, 0,
	SpectrumGetZipName, SpecAliensusRomInfo, SpecAliensusRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Casanova (48K)

static struct BurnRomInfo SpecCasanovaRomDesc[] = {
	{ "Casanova (1989)(Iber Software).tap", 45005, 0x1c66d96d, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecCasanova, SpecCasanova, Spectrum)
STD_ROM_FN(SpecCasanova)

struct BurnDriver BurnSpecCasanova = {
	"spec_casanova", NULL, "spec_spectrum", NULL, "1989",
	"Casanova (48K)\0", NULL, "Iber Software", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecCasanovaRomInfo, SpecCasanovaRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Human Killing Machine (48K)

static struct BurnRomInfo SpecHumankillmachineRomDesc[] = {
	{ "Human Killing Machine (1989)(U.S. Gold).tap", 83940, 0xfac2e471, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecHumankillmachine, SpecHumankillmachine, Spectrum)
STD_ROM_FN(SpecHumankillmachine)

struct BurnDriver BurnSpecHumankillmachine = {
	"spec_humankillmachine", NULL, "spec_spectrum", NULL, "1988",
	"Human Killing Machine (48K)\0", NULL, "U.S. Gold", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_VSFIGHT, 0,
	SpectrumGetZipName, SpecHumankillmachineRomInfo, SpecHumankillmachineRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Hypsys - Part 1 (48K)

static struct BurnRomInfo SpecHypsys1RomDesc[] = {
	{ "Hypsys - Part 1 (1989)(Dro Soft).tap", 49162, 0x0d76e3f1, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecHypsys1, SpecHypsys1, Spectrum)
STD_ROM_FN(SpecHypsys1)

struct BurnDriver BurnSpecHypsys1 = {
	"spec_hypsys1", NULL, "spec_spectrum", NULL, "1989",
	"Hypsys - Part 1 (48K)\0", NULL, "Dro Soft", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_VERSHOOT, 0,
	SpectrumGetZipName, SpecHypsys1RomInfo, SpecHypsys1RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Hypsys - Part 2 (48K)

static struct BurnRomInfo SpecHypsys2RomDesc[] = {
	{ "Hypsys - Part 2 (1989)(Dro Soft).tap", 49809, 0x97b3d2d9, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecHypsys2, SpecHypsys2, Spectrum)
STD_ROM_FN(SpecHypsys2)

struct BurnDriver BurnSpecHypsys2 = {
	"spec_hypsys2", "spec_hypsys1", "spec_spectrum", NULL, "1989",
	"Hypsys - Part 2 (48K)\0", NULL, "Dro Soft", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE, 1, HARDWARE_SPECTRUM, GBF_VERSHOOT, 0,
	SpectrumGetZipName, SpecHypsys2RomInfo, SpecHypsys2RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Karnov (128K)

static struct BurnRomInfo SpecKarnovRomDesc[] = {
	{ "Karnov (1988)(Electric Dreams).tap", 234536, 0x95d92161, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecKarnov, SpecKarnov, Spec128)
STD_ROM_FN(SpecKarnov)

struct BurnDriver BurnSpecKarnov = {
	"spec_karnov", NULL, "spec_spec128", NULL, "1988",
	"Karnov (128K)\0", NULL, "Electric Dreams", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_PLATFORM | GBF_RUNGUN, 0,
	SpectrumGetZipName, SpecKarnovRomInfo, SpecKarnovRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Kendo Warrior (128K)

static struct BurnRomInfo SpecKendowarriorRomDesc[] = {
	{ "Kendo Warrior (1989)(Byte Back).tap", 48724, 0xb4d3cb13, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecKendowarrior, SpecKendowarrior, Spec128)
STD_ROM_FN(SpecKendowarrior)

struct BurnDriver BurnSpecKendowarrior = {
	"spec_kendowarrior", NULL, "spec_spec128", NULL, "1989",
	"Kendo Warrior (128K)\0", NULL, "Byte Back", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_SCRFIGHT, 0,
	SpectrumGetZipName, SpecKendowarriorRomInfo, SpecKendowarriorRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Paradise Cafe (Portuguese) (48K)

static struct BurnRomInfo SpecparadisecafeRomDesc[] = {
	{ "Paradise Cafe PT (1985)(Damatta)[adult].z80", 36975, 0xd0016cf2, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specparadisecafe, Specparadisecafe, Spectrum)
STD_ROM_FN(Specparadisecafe)

struct BurnDriver BurnSpecparadisecafe = {
	"spec_paradisecafe", NULL, "spec_spectrum", NULL, "1985",
	"Paradise Cafe (Portuguese) (48K)\0", "Adult game", "Damatta", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_MISC, 0,
	SpectrumGetZipName, SpecparadisecafeRomInfo, SpecparadisecafeRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Xecutor (48K)

static struct BurnRomInfo SpecxecutorRomDesc[] = {
	{ "Xecutor (1987)(ACE Software).tap", 42063, 0xd1047062, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specxecutor, Specxecutor, Spectrum)
STD_ROM_FN(Specxecutor)

struct BurnDriver BurnSpecxecutor = {
	"spec_xecutor", "spec_xecutorremix", "spec_spectrum", NULL, "1987",
	"Xecutor (48K)\0", NULL, "ACE Software", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE, 2, HARDWARE_SPECTRUM, GBF_VERSHOOT, 0,
	SpectrumGetZipName, SpecxecutorRomInfo, SpecxecutorRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Dominator (48K)

static struct BurnRomInfo SpecdominatorRomDesc[] = {
	{ "Dominator (1989)(System 3).tap", 67437, 0x3af9c9f7, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specdominator, Specdominator, Spectrum)
STD_ROM_FN(Specdominator)

struct BurnDriver BurnSpecdominator = {
	"spec_dominator", NULL, "spec_spectrum", NULL, "1987",
	"Dominator (48K)\0", NULL, "System 3", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_VERSHOOT | GBF_HORSHOOT, 0,
	SpectrumGetZipName, SpecdominatorRomInfo, SpecdominatorRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Sword of the Samurai (48K)

static struct BurnRomInfo SpecSwordsamuraiRomDesc[] = {
	{ "Sword of the Samurai (1992)(Zeppelin Games Ltd).tap", 41439, 0x42e23e9f, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecSwordsamurai, SpecSwordsamurai, Spectrum)
STD_ROM_FN(SpecSwordsamurai)

struct BurnDriver BurnSpecSwordsamurai = {
	"spec_swordsamurai", NULL, "spec_spectrum", NULL, "1992",
	"Sword of the Samurai (48K)\0", NULL, "Zeppelin Games", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_RUNGUN | GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecSwordsamuraiRomInfo, SpecSwordsamuraiRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// California Games (48K)

static struct BurnRomInfo SpecCaliforniagamesRomDesc[] = {
	{ "California Games (1987)(U.S. Gold).tap", 168996, 0x52f1cd37, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecCaliforniagames, SpecCaliforniagames, Spectrum)
STD_ROM_FN(SpecCaliforniagames)

struct BurnDriver BurnSpecCaliforniagames = {
	"spec_californiagames", NULL, "spec_spectrum", NULL, "1987",
	"California Games (48K)\0", NULL, "U.S. Gold", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 4, HARDWARE_SPECTRUM, GBF_SPORTSMISC, 0,
	SpectrumGetZipName, SpecCaliforniagamesRomInfo, SpecCaliforniagamesRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecQAOPSpaceDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Deflektor (48K)

static struct BurnRomInfo SpecDeflektorRomDesc[] = {
	{ "Deflektor (1987)(Gremlin Graphics).tap", 28048, 0x04bf9aff, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecDeflektor, SpecDeflektor, Spectrum)
STD_ROM_FN(SpecDeflektor)

struct BurnDriver BurnSpecDeflektor = {
	"spec_deflektor", NULL, "spec_spectrum", NULL, "1987",
	"Deflektor (48K)\0", NULL, "Gremlin Graphics", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_PUZZLE, 0,
	SpectrumGetZipName, SpecDeflektorRomInfo, SpecDeflektorRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Games - Summer Edition, The (128K)

static struct BurnRomInfo SpecGamessummeredRomDesc[] = {
	{ "Games - Summer Edition, The (1989)(U.S. Gold).tap", 354208, 0xe8b9a3d0, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecGamessummered, SpecGamessummered, Spec128)
STD_ROM_FN(SpecGamessummered)

struct BurnDriver BurnSpecGamessummered = {
	"spec_gamessummered", NULL, "spec_spec128", NULL, "1989",
	"Games - Summer Edition, The (128K)\0", NULL, "U.S. Gold", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 2, HARDWARE_SPECTRUM, GBF_SPORTSMISC, 0,
	SpectrumGetZipName, SpecGamessummeredRomInfo, SpecGamessummeredRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Games - Winter Edition, The (48K)

static struct BurnRomInfo SpecGameswinteredRomDesc[] = {
	{ "Games - Winter Edition, The (1988)(U.S. Gold).tap", 146925, 0x15935233, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecGameswintered, SpecGameswintered, Spectrum)
STD_ROM_FN(SpecGameswintered)

struct BurnDriver BurnSpecGameswintered = {
	"spec_gameswintered", NULL, "spec_spectrum", NULL, "1988",
	"Games - Winter Edition, The (48K)\0", NULL, "U.S. Gold", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 2, HARDWARE_SPECTRUM, GBF_SPORTSMISC, 0,
	SpectrumGetZipName, SpecGameswinteredRomInfo, SpecGameswinteredRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Goonies, The (48K)

static struct BurnRomInfo SpecGooniesRomDesc[] = {
	{ "Goonies, The (1986)(U.S. Gold).tap", 48377, 0x9ed8efdf, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecGoonies, SpecGoonies, Spectrum)
STD_ROM_FN(SpecGoonies)

struct BurnDriver BurnSpecGoonies = {
	"spec_goonies", NULL, "spec_spectrum", NULL, "1986",
	"Goonies, The (48K)\0", "It's not a malfunction, the bg-music is horrible!", "U.S. Gold", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 2, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecGooniesRomInfo, SpecGooniesRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Hyper Sports (48K)

static struct BurnRomInfo SpecHypersportsRomDesc[] = {
	{ "Hyper Sports (1985)(Imagine).tap", 48751, 0xe3a072a5, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecHypersports, SpecHypersports, Spectrum)
STD_ROM_FN(SpecHypersports)

struct BurnDriver BurnSpecHypersports = {
	"spec_hypersports", NULL, "spec_spectrum", NULL, "1985",
	"Hyper Sports (48K)\0", NULL, "Imagine Software", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_SPORTSMISC, 0,
	SpectrumGetZipName, SpecHypersportsRomInfo, SpecHypersportsRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Lemmings (48K)

static struct BurnRomInfo SpecLemmingsRomDesc[] = {
	{ "Lemmings (1991)(Psygnosis).tap", 544963, 0x3a5fa27a, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecLemmings, SpecLemmings, Spectrum)
STD_ROM_FN(SpecLemmings)

struct BurnDriver BurnSpecLemmings = {
	"spec_lemmings", NULL, "spec_spectrum", NULL, "1991",
	"Lemmings (48K)\0", NULL, "Psygnosis", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_PUZZLE, 0,
	SpectrumGetZipName, SpecLemmingsRomInfo, SpecLemmingsRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// P-47 Thunderbolt (128K)

static struct BurnRomInfo SpecP47thunderboltRomDesc[] = {
	{ "P-47 Thunderbolt (1990)(Firebird).z80", 72894, 0x9efaf7fc, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecP47thunderbolt, SpecP47thunderbolt, Spec128)
STD_ROM_FN(SpecP47thunderbolt)

struct BurnDriver BurnSpecP47thunderbolt = {
	"spec_p47thunderbolt", NULL, "spec_spec128", NULL, "1990",
	"P-47 Thunderbolt (128K)\0", NULL, "Firebird Software", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_HORSHOOT, 0,
	SpectrumGetZipName, SpecP47thunderboltRomInfo, SpecP47thunderboltRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Summer Games (48K)

static struct BurnRomInfo SpecSummergamesRomDesc[] = {
	{ "Summer Games (1988)(U.S. Gold).tap", 103495, 0xa03cb5cb, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecSummergames, SpecSummergames, Spectrum)
STD_ROM_FN(SpecSummergames)

struct BurnDriver BurnSpecSummergames = {
	"spec_summergames", NULL, "spec_spectrum", NULL, "1988",
	"Summer Games (48K)\0", NULL, "U.S. Gold", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 4, HARDWARE_SPECTRUM, GBF_SPORTSMISC, 0,
	SpectrumGetZipName, SpecSummergamesRomInfo, SpecSummergamesRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Summer Games II (48K)

static struct BurnRomInfo SpecSummergames2RomDesc[] = {
	{ "Summer Games II (1988)(U.S. Gold).tap", 189121, 0xdbe0691e, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecSummergames2, SpecSummergames2, Spectrum)
STD_ROM_FN(SpecSummergames2)

struct BurnDriver BurnSpecSummergames2 = {
	"spec_summergames2", NULL, "spec_spectrum", NULL, "1988",
	"Summer Games II (48K)\0", NULL, "U.S. Gold", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 2, HARDWARE_SPECTRUM, GBF_SPORTSMISC, 0,
	SpectrumGetZipName, SpecSummergames2RomInfo, SpecSummergames2RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Super Wonder Boy (48K)

static struct BurnRomInfo SpecSuperwonderboyRomDesc[] = {
	{ "Super Wonder Boy (1989)(Activision).tap", 169846, 0x179244d3, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecSuperwonderboy, SpecSuperwonderboy, Spectrum)
STD_ROM_FN(SpecSuperwonderboy)

struct BurnDriver BurnSpecSuperwonderboy = {
	"spec_superwonderboy", NULL, "spec_spectrum", NULL, "1989",
	"Super Wonder Boy (48K)\0", NULL, "Activision", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_PLATFORM | GBF_ADV, 0,
	SpectrumGetZipName, SpecSuperwonderboyRomInfo, SpecSuperwonderboyRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Tai-Pan (128K)

static struct BurnRomInfo SpecTaipanRomDesc[] = {
	{ "Tai-Pan 128K (1987)(Ocean).z80", 82362, 0x3a80cd98, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecTaipan, SpecTaipan, Spec128)
STD_ROM_FN(SpecTaipan)

struct BurnDriver BurnSpecTaipan = {
	"spec_taipan", NULL, "spec_spec128", NULL, "1987",
	"Tai-Pan (128K)\0", NULL, "Ocean", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_ADV, 0,
	SpectrumGetZipName, SpecTaipanRomInfo, SpecTaipanRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// U.N. Squadron (128K)

static struct BurnRomInfo SpecUnsquadronRomDesc[] = {
	{ "UN Squadron (1990)(U.S Gold).tap", 303274, 0x12571e3a, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecUnsquadron, SpecUnsquadron, Spec128)
STD_ROM_FN(SpecUnsquadron)

struct BurnDriver BurnSpecUnsquadron = {
	"spec_unsquadron", NULL, "spec_spec128", NULL, "1990",
	"U.N. Squadron (128K)\0", NULL, "U.S. Gold", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 2, HARDWARE_SPECTRUM, GBF_HORSHOOT, 0,
	SpectrumGetZipName, SpecUnsquadronRomInfo, SpecUnsquadronRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Winter Games (128K)

static struct BurnRomInfo SpecWintergamesRomDesc[] = {
	{ "Winter Games (1986)(U.S. Gold).tap", 119630, 0x5ab9f21c, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecWintergames, SpecWintergames, Spec128)
STD_ROM_FN(SpecWintergames)

struct BurnDriver BurnSpecWintergames = {
	"spec_wintergames", NULL, "spec_spec128", NULL, "1986",
	"Winter Games (128K)\0", NULL, "U.S. Gold", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 4, HARDWARE_SPECTRUM, GBF_SPORTSMISC, 0,
	SpectrumGetZipName, SpecWintergamesRomInfo, SpecWintergamesRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// World Games (128K)

static struct BurnRomInfo SpecWorldgamesRomDesc[] = {
	{ "World Games (1987)(U.S. Gold).tap", 224518, 0xab9ccddb, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecWorldgames, SpecWorldgames, Spec128)
STD_ROM_FN(SpecWorldgames)

struct BurnDriver BurnSpecWorldgames = {
	"spec_worldgames", NULL, "spec_spec128", NULL, "1987",
	"World Games (128K)\0", NULL, "U.S. Gold", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 4, HARDWARE_SPECTRUM, GBF_SPORTSMISC, 0,
	SpectrumGetZipName, SpecWorldgamesRomInfo, SpecWorldgamesRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Xecutor - Remix (128K)

static struct BurnRomInfo SpecXecutorremixRomDesc[] = {
	{ "Xecutor - Remix (1987)(ACE Software).z80", 77548, 0x97ef7538, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecXecutorremix, SpecXecutorremix, Spec128)
STD_ROM_FN(SpecXecutorremix)

struct BurnDriver BurnSpecXecutorremix = {
	"spec_xecutorremix", NULL, "spec_spec128", NULL, "1987",
	"Xecutor - Remix (128K)\0", NULL, "ACE Software", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 2, HARDWARE_SPECTRUM, GBF_VERSHOOT, 0,
	SpectrumGetZipName, SpecXecutorremixRomInfo, SpecXecutorremixRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// 100 - Stolnik (128K)

static struct BurnRomInfo Spec100stolnikRomDesc[] = {
	{ "100 - Stolnik (1995)(Power Of Sound).z80", 43905, 0x8de30bdc, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Spec100stolnik, Spec100stolnik, Spec128)
STD_ROM_FN(Spec100stolnik)

struct BurnDriver BurnSpec100stolnik = {
	"spec_100stolnik", NULL, "spec_spec128", NULL, "1995",
	"100 - Stolnik (128K)\0", NULL, "Power Of Sound", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_PUZZLE, 0,
	SpectrumGetZipName, Spec100stolnikRomInfo, Spec100stolnikRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// 10th Frame (48K)

static struct BurnRomInfo Spec10thframeRomDesc[] = {
	{ "10th Frame (1987)(U.S. Gold).tap", 20061, 0x4a6d1860, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Spec10thframe, Spec10thframe, Spectrum)
STD_ROM_FN(Spec10thframe)

struct BurnDriver BurnSpec10thframe = {
	"spec_10thframe", NULL, "spec_spectrum", NULL, "1987",
	"10th Frame (48K)\0", NULL, "U.S. Gold", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 8, HARDWARE_SPECTRUM, GBF_SPORTSMISC, 0,
	SpectrumGetZipName, Spec10thframeRomInfo, Spec10thframeRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// 4 Soccer Simulators - 11-a-Side Soccer (128K)

static struct BurnRomInfo Spec4soccersims1RomDesc[] = {
	{ "4 Soccer Simulators - 11-a-Side Soccer (1989)(Codemasters).z80", 44128, 0x7409c0ec, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Spec4soccersims1, Spec4soccersims1, Spec128)
STD_ROM_FN(Spec4soccersims1)

struct BurnDriver BurnSpec4soccersims1 = {
	"spec_4soccersims1", NULL, "spec_spec128", NULL, "1989",
	"4 Soccer Simulators - 11-a-Side Soccer (128K)\0", NULL, "Codemasters", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 4, HARDWARE_SPECTRUM, GBF_SPORTSFOOTBALL, 0,
	SpectrumGetZipName, Spec4soccersims1RomInfo, Spec4soccersims1RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// 4 Soccer Simulators - Indoor Soccer (128K)

static struct BurnRomInfo Spec4soccersims2RomDesc[] = {
	{ "4 Soccer Simulators - Indoor Soccer (1989)(Codemasters).z80", 43850, 0xfa35acd5, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Spec4soccersims2, Spec4soccersims2, Spec128)
STD_ROM_FN(Spec4soccersims2)

struct BurnDriver BurnSpec4soccersims2 = {
	"spec_4soccersims2", NULL, "spec_spec128", NULL, "1989",
	"4 Soccer Simulators - Indoor Soccer (128K)\0", NULL, "Codemasters", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 4, HARDWARE_SPECTRUM, GBF_SPORTSFOOTBALL, 0,
	SpectrumGetZipName, Spec4soccersims2RomInfo, Spec4soccersims2RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// 4 Soccer Simulators - Soccer Skills (128K)

static struct BurnRomInfo Spec4soccersims3RomDesc[] = {
	{ "4 Soccer Simulators - Soccer Skills (1989)(Codemasters).z80", 45681, 0x387466c5, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Spec4soccersims3, Spec4soccersims3, Spec128)
STD_ROM_FN(Spec4soccersims3)

struct BurnDriver BurnSpec4soccersims3 = {
	"spec_4soccersims3", NULL, "spec_spec128", NULL, "1989",
	"4 Soccer Simulators - Soccer Skills (128K)\0", NULL, "Codemasters", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 2, HARDWARE_SPECTRUM, GBF_SPORTSFOOTBALL, 0,
	SpectrumGetZipName, Spec4soccersims3RomInfo, Spec4soccersims3RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// 4 Soccer Simulators - Street Soccer (128K)

static struct BurnRomInfo Spec4soccersims4RomDesc[] = {
	{ "4 Soccer Simulators - Street Soccer (1989)(Codemasters).z80", 45164, 0xb2e7be10, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Spec4soccersims4, Spec4soccersims4, Spec128)
STD_ROM_FN(Spec4soccersims4)

struct BurnDriver BurnSpec4soccersims4 = {
	"spec_4soccersims4", NULL, "spec_spec128", NULL, "1989",
	"4 Soccer Simulators - Street Soccer (128K)\0", NULL, "Codemasters", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 4, HARDWARE_SPECTRUM, GBF_SPORTSFOOTBALL, 0,
	SpectrumGetZipName, Spec4soccersims4RomInfo, Spec4soccersims4RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// 911 TS - Tiger Shark (48K)

static struct BurnRomInfo Spec911tsRomDesc[] = {
	{ "911 TS (1985)(Elite Systems).tap", 47921, 0x303db330, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Spec911ts, Spec911ts, Spectrum)
STD_ROM_FN(Spec911ts)

struct BurnDriver BurnSpec911ts = {
	"spec_911ts", NULL, "spec_spectrum", NULL, "1985",
	"911 TS - Tiger Shark (48K)\0", "Z, X, O, and P to play", "Elite Systems", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_RACING, 0,
	SpectrumGetZipName, Spec911tsRomInfo, Spec911tsRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Advanced Pinball Simulator (128K)

static struct BurnRomInfo SpecAdvancedpinballsimRomDesc[] = {
	{ "Advanced Pinball Simulator (1990)(Codemasters).tap", 51760, 0xfca7d79b, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecAdvancedpinballsim, SpecAdvancedpinballsim, Spec128)
STD_ROM_FN(SpecAdvancedpinballsim)

struct BurnDriver BurnSpecAdvancedpinballsim = {
	"spec_advancedpinballsim", NULL, "spec_spec128", NULL, "1990",
	"Advanced Pinball Simulator (128K)\0", NULL, "Codemasters", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 3, HARDWARE_SPECTRUM, GBF_PINBALL, 0,
	SpectrumGetZipName, SpecAdvancedpinballsimRomInfo, SpecAdvancedpinballsimRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Afterburner (128K)

static struct BurnRomInfo SpecAfterburnerRomDesc[] = {
	{ "Afterburner (1989)(Activision).tap", 102813, 0xb40696fb, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecAfterburner, SpecAfterburner, Spec128)
STD_ROM_FN(SpecAfterburner)

struct BurnDriver BurnSpecAfterburner = {
	"spec_afterburner", NULL, "spec_spec128", NULL, "1989",
	"Afterburner (128K)\0", NULL, "Activision", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_SHOOT, 0,
	SpectrumGetZipName, SpecAfterburnerRomInfo, SpecAfterburnerRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Alien 8 (48K)

static struct BurnRomInfo SpecAlien8RomDesc[] = {
	{ "Alien 8 (1985)(Ultimate Play The Game).tap", 47264, 0x043d8dae, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecAlien8, SpecAlien8, Spectrum)
STD_ROM_FN(SpecAlien8)

struct BurnDriver BurnSpecAlien8 = {
	"spec_alien8", NULL, "spec_spectrum", NULL, "1985",
	"Alien 8 (48K)\0", NULL, "Ultimate Play The Game", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_ADV, 0,
	SpectrumGetZipName, SpecAlien8RomInfo, SpecAlien8RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Arcade Flight Simulator (128K)

static struct BurnRomInfo SpecArcadeflightsimRomDesc[] = {
	{ "Arcade Flight Simulator (1989)(Codemasters).tap", 48526, 0x834a6586, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecArcadeflightsim, SpecArcadeflightsim, Spec128)
STD_ROM_FN(SpecArcadeflightsim)

struct BurnDriver BurnSpecArcadeflightsim = {
	"spec_arcadeflightsim", NULL, "spec_spec128", NULL, "1989",
	"Arcade Flight Simulator (128K)\0", NULL, "Codemasters", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 2, HARDWARE_SPECTRUM, GBF_VERSHOOT, 0,
	SpectrumGetZipName, SpecArcadeflightsimRomInfo, SpecArcadeflightsimRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Blade Warrior (48K)

static struct BurnRomInfo SpecBladewarriorRomDesc[] = {
	{ "Blade Warrior (1988)(Codemasters).tap", 41844, 0xd8bff571, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecBladewarrior, SpecBladewarrior, Spectrum)
STD_ROM_FN(SpecBladewarrior)

struct BurnDriver BurnSpecBladewarrior = {
	"spec_bladewarrior", NULL, "spec_spectrum", NULL, "1988",
	"Blade Warrior (48K)\0", "Select 'Sinclair Stick' to use controller", "Codemasters", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecBladewarriorRomInfo, SpecBladewarriorRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecIntf2DIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Bounty Hunter (128K)

static struct BurnRomInfo SpecBountyhunterRomDesc[] = {
	{ "Bounty Hunter (1989)(Codemasters).tap", 47005, 0x91c9c3ba, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecBountyhunter, SpecBountyhunter, Spec128)
STD_ROM_FN(SpecBountyhunter)

struct BurnDriver BurnSpecBountyhunter = {
	"spec_bountyhunter", NULL, "spec_spec128", NULL, "1989",
	"Bounty Hunter (128K)\0", NULL, "Codemasters", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_RUNGUN, 0,
	SpectrumGetZipName, SpecBountyhunterRomInfo, SpecBountyhunterRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecQAOPSpaceDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Jackal (48K)

static struct BurnRomInfo SpecJackalRomDesc[] = {
	{ "Jackal (1986)(Konami).tap", 26861, 0x954fae85, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecJackal, SpecJackal, Spectrum)
STD_ROM_FN(SpecJackal)

struct BurnDriver BurnSpecJackal = {
	"spec_jackal", NULL, "spec_spectrum", NULL, "1986",
	"Jackal (48K)\0", NULL, "Konami", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 2, HARDWARE_SPECTRUM, GBF_RUNGUN, 0,
	SpectrumGetZipName, SpecJackalRomInfo, SpecJackalRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Laser Squad (48K)

static struct BurnRomInfo SpecLasersquadRomDesc[] = {
	{ "Laser Squad 48K (1988)(Blade Software).tap", 156755, 0x6a065f74, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecLasersquad, SpecLasersquad, Spectrum)
STD_ROM_FN(SpecLasersquad)

struct BurnDriver BurnSpecLasersquad = {
	"spec_lasersquad", NULL, "spec_spectrum", NULL, "1988",
	"Laser Squad (48K)\0", NULL, "Blade Software", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 2, HARDWARE_SPECTRUM, GBF_STRATEGY, 0,
	SpectrumGetZipName, SpecLasersquadRomInfo, SpecLasersquadRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Road Race (48K)

static struct BurnRomInfo SpecRoadraceRomDesc[] = {
	{ "Road Race (1987)(Your Sinclair).tap", 46100, 0xa1daf3de, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecRoadrace, SpecRoadrace, Spectrum)
STD_ROM_FN(SpecRoadrace)

struct BurnDriver BurnSpecRoadrace = {
	"spec_roadrace", NULL, "spec_spectrum", NULL, "1987",
	"Road Race (48K)\0", NULL, "Your Sinclair", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_RACING, 0,
	SpectrumGetZipName, SpecRoadraceRomInfo, SpecRoadraceRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Rod-Land (128K)

static struct BurnRomInfo SpecRodlandRomDesc[] = {
	{ "Rod-Land (1991)(Storm).tap", 117150, 0xcd3ecd05, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecRodland, SpecRodland, Spec128)
STD_ROM_FN(SpecRodland)

struct BurnDriver BurnSpecRodland = {
	"spec_rodland", NULL, "spec_spec128", NULL, "1991",
	"Rod-Land (128K)\0", NULL, "Storm", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecRodlandRomInfo, SpecRodlandRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Shadow Of The Beast (128K)

static struct BurnRomInfo SpecShadowofthebeastRomDesc[] = {
	{ "Shadow Of The Beast (1990)(Gremlin Graphics).z80", 114104, 0x0507010d, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecShadowofthebeast, SpecShadowofthebeast, Spec128)
STD_ROM_FN(SpecShadowofthebeast)

struct BurnDriver BurnSpecShadowofthebeast = {
	"spec_shadowofthebeast", NULL, "spec_spec128", NULL, "1990",
	"Shadow Of The Beast (128K)\0", NULL, "Gremlin Graphics", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_RUNGUN | GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecShadowofthebeastRomInfo, SpecShadowofthebeastRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecQAOPSpaceDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Space Crusade (128K)

static struct BurnRomInfo SpecSpacecrusadeRomDesc[] = {
	{ "Space Crusade (1992)(Gremlin Graphics).tap", 70110, 0xa46b2859, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecSpacecrusade, SpecSpacecrusade, Spec128)
STD_ROM_FN(SpecSpacecrusade)

struct BurnDriver BurnSpecSpacecrusade = {
	"spec_spacecrusade", NULL, "spec_spec128", NULL, "1992",
	"Space Crusade (128K)\0", NULL, "Gremlin Graphics", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_STRATEGY, 0,
	SpectrumGetZipName, SpecSpacecrusadeRomInfo, SpecSpacecrusadeRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Super Car Trans Am - American Turbo King (128K)

static struct BurnRomInfo SpecSupercartransamRomDesc[] = {
	{ "Super Car Trans Am - American Turbo King (1989)(Virgin Mastertronic).tap", 43374, 0xeb2361c5, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecSupercartransam, SpecSupercartransam, Spec128)
STD_ROM_FN(SpecSupercartransam)

struct BurnDriver BurnSpecSupercartransam = {
	"spec_supercartransam", NULL, "spec_spec128", NULL, "1989",
	"Super Car Trans Am - American Turbo King (128K)\0", NULL, "Virgin Mastertronic", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_RUNGUN, 0,
	SpectrumGetZipName, SpecSupercartransamRomInfo, SpecSupercartransamRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Sword Slayer (128K)

static struct BurnRomInfo SpecSwordslayerRomDesc[] = {
	{ "Sword Slayer (1988)(Players).tap", 102652, 0x0a989566, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecSwordslayer, SpecSwordslayer, Spec128)
STD_ROM_FN(SpecSwordslayer)

struct BurnDriver BurnSpecSwordslayer = {
	"spec_swordslayer", NULL, "spec_spec128", NULL, "1988",
	"Sword Slayer (128K)\0", NULL, "Players Software", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 2, HARDWARE_SPECTRUM, GBF_VSFIGHT, 0,
	SpectrumGetZipName, SpecSwordslayerRomInfo, SpecSwordslayerRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// 1st Division Manager (128K)

static struct BurnRomInfo Spec1stdivmanagerRomDesc[] = {
	{ "1st Division Manager (1991)(Codemasters).z80", 47446, 0x57fed4fa, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Spec1stdivmanager, Spec1stdivmanager, Spec128)
STD_ROM_FN(Spec1stdivmanager)

struct BurnDriver BurnSpec1stdivmanager = {
	"spec_1stdivmanager", NULL, "spec_spec128", NULL, "1991",
	"1st Division Manager (128K)\0", NULL, "Codemasters", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_STRATEGY, 0,
	SpectrumGetZipName, Spec1stdivmanagerRomInfo, Spec1stdivmanagerRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// 750cc Grand Prix (128K)

static struct BurnRomInfo Spec750ccRomDesc[] = {
	{ "750cc Grand Prix (1991)(Code Masters).tap", 82394, 0xc9073683, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Spec750cc, Spec750cc, Spec128)
STD_ROM_FN(Spec750cc)

struct BurnDriver BurnSpec750cc = {
	"spec_750cc", NULL, "spec_spec128", NULL, "1991",
	"750cc Grand Prix (128K)\0", NULL, "Code Masters", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_RACING, 0,
	SpectrumGetZipName, Spec750ccRomInfo, Spec750ccRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Billy the Kid (48K)

static struct BurnRomInfo SpecBillythekidRomDesc[] = {
	{ "Billy the Kid (1989)(Virgin Mastertronic).tap", 34510, 0xbc4e886c, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecBillythekid, SpecBillythekid, Spectrum)
STD_ROM_FN(SpecBillythekid)

struct BurnDriver BurnSpecBillythekid = {
	"spec_billythekid", NULL, "spec_spectrum", NULL, "1989",
	"Billy the Kid (48K)\0", "Press Space and Enter to select options", "Virgin Mastertronic", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_SHOOT, 0,
	SpectrumGetZipName, SpecBillythekidRomInfo, SpecBillythekidRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Brainache (48K)

static struct BurnRomInfo SpecBrainacheRomDesc[] = {
	{ "Brainache (1987)(Codemasters).tap", 49898, 0x28592736, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecBrainache, SpecBrainache, Spectrum)
STD_ROM_FN(SpecBrainache)

struct BurnDriver BurnSpecBrainache = {
	"spec_brainache", NULL, "spec_spectrum", NULL, "1987",
	"Brainache (48K)\0", NULL, "Codemasters", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 2, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecBrainacheRomInfo, SpecBrainacheRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Bronx Street Cop (128K)

static struct BurnRomInfo SpecBronxstcopRomDesc[] = {
	{ "Bronx Street Cop (1989)(Virgin Mastertronic).tap", 45318, 0x9884b2f8, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecBronxstcop, SpecBronxstcop, Spec128)
STD_ROM_FN(SpecBronxstcop)

struct BurnDriver BurnSpecBronxstcop = {
	"spec_bronxstcop", NULL, "spec_spec128", NULL, "1989",
	"Bronx Street Cop (128K)\0", NULL, "Virgin Mastertronic", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_SHOOT, 0,
	SpectrumGetZipName, SpecBronxstcopRomInfo, SpecBronxstcopRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// By Fair Means...or Foul (48K)

static struct BurnRomInfo SpecByfairmeansorfoulRomDesc[] = {
	{ "By Fair Means Or Foul (1989)(Superior).z80", 44540, 0x8f8c6d0e, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecByfairmeansorfoul, SpecByfairmeansorfoul, Spectrum)
STD_ROM_FN(SpecByfairmeansorfoul)

struct BurnDriver BurnSpecByfairmeansorfoul = {
	"spec_byfairmeansorfoul", NULL, "spec_spectrum", NULL, "1989",
	"By Fair Means...or Foul (48K)\0", NULL, "Superior", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 2, HARDWARE_SPECTRUM, GBF_VSFIGHT, 0,
	SpectrumGetZipName, SpecByfairmeansorfoulRomInfo, SpecByfairmeansorfoulRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Captain Dynamo (128K)

static struct BurnRomInfo SpecCaptaindynamoRomDesc[] = {
	{ "Captain Dynamo (1992)(Codemasters).tap", 68500, 0x62855d75, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecCaptaindynamo, SpecCaptaindynamo, Spec128)
STD_ROM_FN(SpecCaptaindynamo)

struct BurnDriver BurnSpecCaptaindynamo = {
	"spec_captaindynamo", NULL, "spec_spec128", NULL, "1992",
	"Captain Dynamo (128K)\0", NULL, "Codemasters", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecCaptaindynamoRomInfo, SpecCaptaindynamoRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Cue Boy (128K)

static struct BurnRomInfo SpecCueboyRomDesc[] = {
	{ "Cue Boy (1992)(Codemasters).tap", 54475, 0x2b625a7e, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecCueboy, SpecCueboy, Spec128)
STD_ROM_FN(SpecCueboy)

struct BurnDriver BurnSpecCueboy = {
	"spec_cueboy", NULL, "spec_spec128", NULL, "1992",
	"Cue Boy (128K)\0", NULL, "Codemasters", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 2, HARDWARE_SPECTRUM, GBF_SPORTSMISC, 0,
	SpectrumGetZipName, SpecCueboyRomInfo, SpecCueboyRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Death Stalker (128K)

static struct BurnRomInfo SpecDeathstalkerRomDesc[] = {
	{ "Death Stalker (1988)(Codemasters).tap", 50809, 0x95e38b12, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecDeathstalker, SpecDeathstalker, Spec128)
STD_ROM_FN(SpecDeathstalker)

struct BurnDriver BurnSpecDeathstalker = {
	"spec_deathstalker", NULL, "spec_spec128", NULL, "1988",
	"Death Stalker (128K)\0", NULL, "Codemasters", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_PLATFORM | GBF_ADV, 0,
	SpectrumGetZipName, SpecDeathstalkerRomInfo, SpecDeathstalkerRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Dizzy 3 & a Half (48K)

static struct BurnRomInfo SpecDizzy3andahalfRomDesc[] = {
	{ "Dizzy 3 & a Half (1988)(Codemasters).tap", 25241, 0x8a82c837, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecDizzy3andahalf, SpecDizzy3andahalf, Spectrum)
STD_ROM_FN(SpecDizzy3andahalf)

struct BurnDriver BurnSpecDizzy3andahalf = {
	"spec_dizzy3andahalf", NULL, "spec_spectrum", NULL, "1988",
	"Dizzy III (Three).. and a Half (48K)\0", NULL, "Codemasters", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_PLATFORM | GBF_ADV, 0,
	SpectrumGetZipName, SpecDizzy3andahalfRomInfo, SpecDizzy3andahalfRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// DJ Puff's Volcanic Capers (128K)

static struct BurnRomInfo SpecDjpuffRomDesc[] = {
	{ "DJ Puff (1992)(Codemasters).z80", 66937, 0x38340567, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecDjpuff, SpecDjpuff, Spec128)
STD_ROM_FN(SpecDjpuff)

struct BurnDriver BurnSpecDjpuff = {
	"spec_djpuff", NULL, "spec_spec128", NULL, "1992",
	"DJ Puff's Volcanic Capers (128K)\0", NULL, "Codemasters", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecDjpuffRomInfo, SpecDjpuffRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecQAOPSpaceDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// F-16 Fighting Falcon (128K)

static struct BurnRomInfo SpecF16falconRomDesc[] = {
	{ "F-16 Fighting Falcon (1990)(Virgin Mastertronic).tap", 39117, 0x2071bb7c, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecF16falcon, SpecF16falcon, Spec128)
STD_ROM_FN(SpecF16falcon)

struct BurnDriver BurnSpecF16falcon = {
	"spec_f16falcon", NULL, "spec_spec128", NULL, "1990",
	"F-16 Fighting Falcon (128K)\0", NULL, "Virgin Mastertronic", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_SHOOT, 0,
	SpectrumGetZipName, SpecF16falconRomInfo, SpecF16falconRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Fairlight - A Prelude (128K)

static struct BurnRomInfo SpecFairlight128RomDesc[] = {
	{ "Fairlight - A Prelude 128K (1985)(The Edge).z80", 70829, 0x5fdc3ff9, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecFairlight128, SpecFairlight128, Spec128)
STD_ROM_FN(SpecFairlight128)

struct BurnDriver BurnSpecFairlight128 = {
	"spec_fairlight128", NULL, "spec_spec128", NULL, "1985",
	"Fairlight - A Prelude (128K)\0", "Press '9' to set controller", "The Edge", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_ADV, 0,
	SpectrumGetZipName, SpecFairlight128RomInfo, SpecFairlight128RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Fairlight - A Prelude (48K)

static struct BurnRomInfo SpecFairlight48RomDesc[] = {
	{ "Fairlight - A Prelude 48K (1985)(The Edge).tap", 50258, 0x12342db8, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecFairlight48, SpecFairlight48, Spectrum)
STD_ROM_FN(SpecFairlight48)

struct BurnDriver BurnSpecFairlight48 = {
	"spec_fairlight48", "spec_fairlight128", "spec_spectrum", NULL, "1985",
	"Fairlight - A Prelude (48K)\0", "Press '9' to set controller", "The Edge", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE, 1, HARDWARE_SPECTRUM, GBF_ADV, 0,
	SpectrumGetZipName, SpecFairlight48RomInfo, SpecFairlight48RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Fairlight II (128K)

static struct BurnRomInfo SpecFairlight2RomDesc[] = {
	{ "Fairlight II 128K (1986)(The Edge).tap", 97674, 0x5fac6ab2, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecFairlight2, SpecFairlight2, Spec128)
STD_ROM_FN(SpecFairlight2)

struct BurnDriver BurnSpecFairlight2 = {
	"spec_fairlight2", NULL, "spec_spec128", NULL, "1986",
	"Fairlight II - A Trail of Darkness (128K)\0", "Press '9' to set controller", "The Edge", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_ADV, 0,
	SpectrumGetZipName, SpecFairlight2RomInfo, SpecFairlight2RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Fantastic Voyage

static struct BurnRomInfo SpecFantvoyageRomDesc[] = {
	{ "Fantastic Voyage (1984)(Quicksilva Ltd.).tap", 42032, 0x8c6d5727, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecFantvoyage, SpecFantvoyage, Spectrum)
STD_ROM_FN(SpecFantvoyage)

struct BurnDriver BurnSpecFantvoyage = {
	"spec_fantvoyage", NULL, "spec_spectrum", NULL, "1984",
	"Fantastic Voyage (48K)\0", NULL, "Quicksilva Ltd.", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_MAZE, 0,
	SpectrumGetZipName, SpecFantvoyageRomInfo, SpecFantvoyageRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Frankenstein Jnr (48K)

static struct BurnRomInfo SpecFrankensteinjrRomDesc[] = {
	{ "Frankenstein Jnr (1990)(Codemasters).tap", 43412, 0xf549d96d, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecFrankensteinjr, SpecFrankensteinjr, Spectrum)
STD_ROM_FN(SpecFrankensteinjr)

struct BurnDriver BurnSpecFrankensteinjr = {
	"spec_frankensteinjr", NULL, "spec_spectrum", NULL, "1990",
	"Frankenstein Jnr (48K)\0", NULL, "Codemasters", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_ADV, 0,
	SpectrumGetZipName, SpecFrankensteinjrRomInfo, SpecFrankensteinjrRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Freddy Hardest in South Manhattan (Spanish) (48K)

static struct BurnRomInfo SpecFreddyhardestsouthmanRomDesc[] = {
	{ "Freddy Hardest in South Manhattan (1989)(Dinamic Software).tap", 61774, 0xffbc6590, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecFreddyhardestsouthman, SpecFreddyhardestsouthman, Spectrum)
STD_ROM_FN(SpecFreddyhardestsouthman)

struct BurnDriver BurnSpecFreddyhardestsouthman = {
	"spec_freddyhardestsouthman", NULL, "spec_spectrum", NULL, "1989",
	"Freddy Hardest in South Manhattan (Spanish) (48K)\0", "Set 'S - Sinclair' to use controller", "Dinamic Software", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_SCRFIGHT, 0,
	SpectrumGetZipName, SpecFreddyhardestsouthmanRomInfo, SpecFreddyhardestsouthmanRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecIntf2DIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Fruit Machine Simulator - Cash Bash (48K)

static struct BurnRomInfo SpecFruitmachinesim1RomDesc[] = {
	{ "Fruit Machine Simulator - Cash Bash (1987)(Codemasters).tap", 47967, 0x0f58266f, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecFruitmachinesim1, SpecFruitmachinesim1, Spectrum)
STD_ROM_FN(SpecFruitmachinesim1)

struct BurnDriver BurnSpecFruitmachinesim1 = {
	"spec_fruitmachinesim1", NULL, "spec_spectrum", NULL, "1987",
	"Fruit Machine Simulator - Cash Bash (48K)\0", NULL, "Codemasters", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_CASINO, 0,
	SpectrumGetZipName, SpecFruitmachinesim1RomInfo, SpecFruitmachinesim1RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Fruit Machine Simulator 2 - Mega Trek (128K)

static struct BurnRomInfo SpecFruitmachinesim2RomDesc[] = {
	{ "Fruit Machine Simulator 2 - Mega Trek (1990)(Codemasters).tap", 49545, 0xbb3c551e, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecFruitmachinesim2, SpecFruitmachinesim2, Spec128)
STD_ROM_FN(SpecFruitmachinesim2)

struct BurnDriver BurnSpecFruitmachinesim2 = {
	"spec_fruitmachinesim2", NULL, "spec_spec128", NULL, "1990",
	"Fruit Machine Simulator 2 - Mega Trek (128K)\0", NULL, "Codemasters", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_CASINO, 0,
	SpectrumGetZipName, SpecFruitmachinesim2RomInfo, SpecFruitmachinesim2RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Ghost Hunters (48K)

static struct BurnRomInfo SpecGhosthuntersRomDesc[] = {
	{ "Ghost Hunters (1987)(Codemasters).tap", 48896, 0xdfa7ffcb, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecGhosthunters, SpecGhosthunters, Spectrum)
STD_ROM_FN(SpecGhosthunters)

struct BurnDriver BurnSpecGhosthunters = {
	"spec_ghosthunters", NULL, "spec_spectrum", NULL, "1987",
	"Ghost Hunters (48K)\0", NULL, "Codemasters", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecGhosthuntersRomInfo, SpecGhosthuntersRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Grand Prix Simulator (48K)

static struct BurnRomInfo SpecGpsimulator1RomDesc[] = {
	{ "Grand Prix Simulator (1987)(Codemasters).tap", 55033, 0x379b30e6, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecGpsimulator1, SpecGpsimulator1, Spec128)
STD_ROM_FN(SpecGpsimulator1)

struct BurnDriver BurnSpecGpsimulator1 = {
	"spec_gpsimulator1", NULL, "spec_spec128", NULL, "1987",
	"Grand Prix Simulator (48K)\0", NULL, "Codemasters", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 2, HARDWARE_SPECTRUM, GBF_RACING, 0,
	SpectrumGetZipName, SpecGpsimulator1RomInfo, SpecGpsimulator1RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Grand Prix Simulator 2 (128K)

static struct BurnRomInfo SpecGpsimii128RomDesc[] = {
	{ "Grand Prix Simulator 2 (1989)(Codemasters)(48K-128K).tap", 53178, 0x2e2014ef, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecGpsimii128, SpecGpsimii128, Spec128)
STD_ROM_FN(SpecGpsimii128)

struct BurnDriver BurnSpecGpsimii128 = {
	"spec_gpsimii128", NULL, "spec_spec128", NULL, "1989",
	"Grand Prix Simulator 2 (128K)\0", NULL, "Codemasters", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 3, HARDWARE_SPECTRUM, GBF_RACING, 0,
	SpectrumGetZipName, SpecGpsimii128RomInfo, SpecGpsimii128RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Grand Prix Simulator 2 (48K)

static struct BurnRomInfo SpecGpsimii48RomDesc[] = {
	{ "Grand Prix Simulator 2 (1989)(Codemasters)(48K-128K).tap", 53178, 0x2e2014ef, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecGpsimii48, SpecGpsimii48, Spectrum)
STD_ROM_FN(SpecGpsimii48)

struct BurnDriver BurnSpecGpsimii48 = {
	"spec_gpsimii48", "spec_gpsimii128", "spec_spectrum", NULL, "1989",
	"Grand Prix Simulator 2 (48K)\0", NULL, "Codemasters", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE, 3, HARDWARE_SPECTRUM, GBF_RACING, 0,
	SpectrumGetZipName, SpecGpsimii48RomInfo, SpecGpsimii48RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Grell and Falla (128K)

static struct BurnRomInfo SpecGrellandfallaRomDesc[] = {
	{ "Grell and Falla (1987)(Codemasters).z80", 92473, 0x8929d289, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecGrellandfalla, SpecGrellandfalla, Spec128)
STD_ROM_FN(SpecGrellandfalla)

struct BurnDriver BurnSpecGrellandfalla = {
	"spec_grellandfalla", NULL, "spec_spec128", NULL, "1992",
	"Grell and Falla (128K)\0", NULL, "Codemasters", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecGrellandfallaRomInfo, SpecGrellandfallaRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Hit Squad, The (48K)

static struct BurnRomInfo SpecHitsquadRomDesc[] = {
	{ "Hit Squad, The (1988)(Codemasters).z80", 40124, 0x8afa17ab, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecHitsquad, SpecHitsquad, Spectrum)
STD_ROM_FN(SpecHitsquad)

struct BurnDriver BurnSpecHitsquad = {
	"spec_hitsquad", NULL, "spec_spectrum", NULL, "1988",
	"Hit Squad, The (48K)\0", NULL, "Codemasters", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecHitsquadRomInfo, SpecHitsquadRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Hundra (48K)

static struct BurnRomInfo SpecHundraRomDesc[] = {
	{ "Hundra (1988)(Dinamic Software).tap", 48229, 0xeeb32b33, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecHundra, SpecHundra, Spec128)
STD_ROM_FN(SpecHundra)

struct BurnDriver BurnSpecHundra = {
	"spec_hundra", NULL, "spec_spec128", NULL, "1988",
	"Hundra (48K)\0", NULL, "Dinamic Software", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecHundraRomInfo, SpecHundraRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// International 3D Tennis (128K)

static struct BurnRomInfo SpecInt3dtennisRomDesc[] = {
	{ "International 3D Tennis (1990)(Palace).tap", 85714, 0x6d60e471, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecInt3dtennis, SpecInt3dtennis, Spec128)
STD_ROM_FN(SpecInt3dtennis)

struct BurnDriver BurnSpecInt3dtennis = {
	"spec_int3dtennis", NULL, "spec_spec128", NULL, "1990",
	"International 3D Tennis (128K)\0", NULL, "Palace Software", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 2, HARDWARE_SPECTRUM, GBF_SPORTSMISC, 0,
	SpectrumGetZipName, SpecInt3dtennisRomInfo, SpecInt3dtennisRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecIntf2DIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// International Rugby (48K)

static struct BurnRomInfo SpecIntrugbyRomDesc[] = {
	{ "International Rugby (1987)(Codemasters).tap", 40161, 0xbaf57b6c, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecIntrugby, SpecIntrugby, Spectrum)
STD_ROM_FN(SpecIntrugby)

struct BurnDriver BurnSpecIntrugby = {
	"spec_intrugby", NULL, "spec_spectrum", NULL, "1987",
	"International Rugby (48K)\0", NULL, "Codemasters", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 2, HARDWARE_SPECTRUM, GBF_SPORTSMISC, 0,
	SpectrumGetZipName, SpecIntrugbyRomInfo, SpecIntrugbyRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// International Speedway (128K)

static struct BurnRomInfo SpecIntspeedwayRomDesc[] = {
	{ "International Speedway (1988)(Silverbird).z80", 35832, 0x7d1e8f3a, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecIntspeedway, SpecIntspeedway, Spec128)
STD_ROM_FN(SpecIntspeedway)

struct BurnDriver BurnSpecIntspeedway = {
	"spec_intspeedway", NULL, "spec_spec128", NULL, "1988",
	"International Speedway (128K)\0", NULL, "Silverbird Software", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_RACING, 0,
	SpectrumGetZipName, SpecIntspeedwayRomInfo, SpecIntspeedwayRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Italian Supercar (128K)

static struct BurnRomInfo SpecItaliansupercarRomDesc[] = {
	{ "Italian Supercar (1990)(Codemasters).tap", 48649, 0x0e28c70e, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecItaliansupercar, SpecItaliansupercar, Spec128)
STD_ROM_FN(SpecItaliansupercar)

struct BurnDriver BurnSpecItaliansupercar = {
	"spec_italiansupercar", NULL, "spec_spec128", NULL, "1990",
	"Italian Supercar (128K)\0", NULL, "Codemasters", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_VERSHOOT, 0,
	SpectrumGetZipName, SpecItaliansupercarRomInfo, SpecItaliansupercarRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Jet Bike Simulator - Lakes Standard (48K)

static struct BurnRomInfo SpecJetbikesim1RomDesc[] = {
	{ "Jet Bike Simulator - Lakes Standard (1988)(Codemasters).z80", 37457, 0x4732d30c, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecJetbikesim1, SpecJetbikesim1, Spectrum)
STD_ROM_FN(SpecJetbikesim1)

struct BurnDriver BurnSpecJetbikesim1 = {
	"spec_jetbikesim1", NULL, "spec_spectrum", NULL, "1988",
	"Jet Bike Simulator - Lakes Standard (48K)\0", NULL, "Codemasters", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 2, HARDWARE_SPECTRUM, GBF_RACING, 0,
	SpectrumGetZipName, SpecJetbikesim1RomInfo, SpecJetbikesim1RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Jet Bike Simulator - Coasts Standard (48K)

static struct BurnRomInfo SpecJetbikesim2RomDesc[] = {
	{ "Jet Bike Simulator - Coasts Standard (1988)(Codemasters).z80", 37757, 0x2311bdb7, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecJetbikesim2, SpecJetbikesim2, Spectrum)
STD_ROM_FN(SpecJetbikesim2)

struct BurnDriver BurnSpecJetbikesim2 = {
	"spec_jetbikesim2", NULL, "spec_spectrum", NULL, "1988",
	"Jet Bike Simulator - Coasts Standard (48K)\0", NULL, "Codemasters", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 2, HARDWARE_SPECTRUM, GBF_RACING, 0,
	SpectrumGetZipName, SpecJetbikesim2RomInfo, SpecJetbikesim2RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Jet Bike Simulator - Docks Standard (48K)

static struct BurnRomInfo SpecJetbikesim3RomDesc[] = {
	{ "Jet Bike Simulator - Docks Standard (1988)(Codemasters).z80", 37266, 0x94a6c7d8, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecJetbikesim3, SpecJetbikesim3, Spectrum)
STD_ROM_FN(SpecJetbikesim3)

struct BurnDriver BurnSpecJetbikesim3 = {
	"spec_jetbikesim3", NULL, "spec_spectrum", NULL, "1988",
	"Jet Bike Simulator - Docks Standard (48K)\0", NULL, "Codemasters", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 2, HARDWARE_SPECTRUM, GBF_RACING, 0,
	SpectrumGetZipName, SpecJetbikesim3RomInfo, SpecJetbikesim3RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Jet Bike Simulator - Lakes Expert (48K)

static struct BurnRomInfo SpecJetbikesim4RomDesc[] = {
	{ "Jet Bike Simulator - Lakes Expert (1988)(Codemasters).z80", 37906, 0x3681dac9, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecJetbikesim4, SpecJetbikesim4, Spectrum)
STD_ROM_FN(SpecJetbikesim4)

struct BurnDriver BurnSpecJetbikesim4 = {
	"spec_jetbikesim4", "spec_jetbikesim1", "spec_spectrum", NULL, "1988",
	"Jet Bike Simulator - Lakes Expert (48K)\0", NULL, "Codemasters", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE, 2, HARDWARE_SPECTRUM, GBF_RACING, 0,
	SpectrumGetZipName, SpecJetbikesim4RomInfo, SpecJetbikesim4RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Jet Bike Simulator - Coasts Expert (48K)

static struct BurnRomInfo SpecJetbikesim5RomDesc[] = {
	{ "Jet Bike Simulator - Coasts Expert (1988)(Codemasters).z80", 38206, 0xc3d570e4, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecJetbikesim5, SpecJetbikesim5, Spectrum)
STD_ROM_FN(SpecJetbikesim5)

struct BurnDriver BurnSpecJetbikesim5 = {
	"spec_jetbikesim5", "spec_jetbikesim2", "spec_spectrum", NULL, "1988",
	"Jet Bike Simulator - Coasts Expert (48K)\0", NULL, "Codemasters", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE, 2, HARDWARE_SPECTRUM, GBF_RACING, 0,
	SpectrumGetZipName, SpecJetbikesim5RomInfo, SpecJetbikesim5RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Jet Bike Simulator - Docks Expert (48K)

static struct BurnRomInfo SpecJetbikesim6RomDesc[] = {
	{ "Jet Bike Simulator - Docks Expert (1988)(Codemasters).z80", 37673, 0x8e1e9ee5, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecJetbikesim6, SpecJetbikesim6, Spectrum)
STD_ROM_FN(SpecJetbikesim6)

struct BurnDriver BurnSpecJetbikesim6 = {
	"spec_jetbikesim6", "spec_jetbikesim3", "spec_spectrum", NULL, "1988",
	"Jet Bike Simulator - Docks Expert (48K)\0", NULL, "Codemasters", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE, 2, HARDWARE_SPECTRUM, GBF_RACING, 0,
	SpectrumGetZipName, SpecJetbikesim6RomInfo, SpecJetbikesim6RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Jungle Warfare (48K)

static struct BurnRomInfo SpecJunglewarfareRomDesc[] = {
	{ "Jungle Warfare (1989)(Codemasters).tap", 43631, 0xeb977230, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecJunglewarfare, SpecJunglewarfare, Spec128)
STD_ROM_FN(SpecJunglewarfare)

struct BurnDriver BurnSpecJunglewarfare = {
	"spec_junglewarfare", NULL, "spec_spec128", NULL, "1989",
	"Jungle Warfare (48K)\0", NULL, "Codemasters", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_SHOOT, 0,
	SpectrumGetZipName, SpecJunglewarfareRomInfo, SpecJunglewarfareRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Kamikaze (128K)

static struct BurnRomInfo SpecKamikazeRomDesc[] = {
	{ "Kamikaze (1991)(Codemasters).tap", 47767, 0xd35b3009, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecKamikaze, SpecKamikaze, Spec128)
STD_ROM_FN(SpecKamikaze)

struct BurnDriver BurnSpecKamikaze = {
	"spec_kamikaze", NULL, "spec_spec128", NULL, "1991",
	"Kamikaze (128K)\0", NULL, "Codemasters", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_HORSHOOT, 0,
	SpectrumGetZipName, SpecKamikazeRomInfo, SpecKamikazeRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// KGB Super Spy (48K)

static struct BurnRomInfo SpecKgbsuperspyRomDesc[] = {
	{ "KGB Super Spy (1989)(Codemasters).tap", 45049, 0xa879a856, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecKgbsuperspy, SpecKgbsuperspy, Spectrum)
STD_ROM_FN(SpecKgbsuperspy)

struct BurnDriver BurnSpecKgbsuperspy = {
	"spec_kgbsuperspy", NULL, "spec_spectrum", NULL, "1989",
	"KGB Super Spy (48K)\0", NULL, "Codemasters", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_HORSHOOT, 0,
	SpectrumGetZipName, SpecKgbsuperspyRomInfo, SpecKgbsuperspyRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Escape from the Planet of the Robot Monsters (128K)

static struct BurnRomInfo SpecEscplanetrobotmonRomDesc[] = {
	{ "Escape from the Planet of the Robot Monsters (1990)(Domark).tap", 111578, 0x9f95584c, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecEscplanetrobotmon, SpecEscplanetrobotmon, Spec128)
STD_ROM_FN(SpecEscplanetrobotmon)

struct BurnDriver BurnSpecEscplanetrobotmon = {
	"spec_escplanetrobotmon", NULL, "spec_spec128", NULL, "1990",
	"Escape from the Planet of the Robot Monsters (128K)\0", NULL, "Domark", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 2, HARDWARE_SPECTRUM, GBF_RUNGUN, 0,
	SpectrumGetZipName, SpecEscplanetrobotmonRomInfo, SpecEscplanetrobotmonRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecIntf2DIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Gauntlet II (128K)

static struct BurnRomInfo SpecGauntlet2RomDesc[] = {
	{ "Gauntlet II (1987)(U.S. Gold).tap", 135163, 0x400e869d, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecGauntlet2, SpecGauntlet2, Spec128)
STD_ROM_FN(SpecGauntlet2)

struct BurnDriver BurnSpecGauntlet2 = {
	"spec_gauntlet2", NULL, "spec_spec128", NULL, "1987",
	"Gauntlet II (128K)\0", "In menu use Cursor Keys 5-6-7-8 + SPACE", "U.S. Gold", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 2, HARDWARE_SPECTRUM, GBF_MAZE | GBF_RUNGUN, 0,
	SpectrumGetZipName, SpecGauntlet2RomInfo, SpecGauntlet2RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Last Vampire, The (48K)

static struct BurnRomInfo SpecLastvampireRomDesc[] = {
	{ "Last Vampire, The (1990)(Atlantis).tap", 50418, 0x9c48dd1a, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecLastvampire, SpecLastvampire, Spectrum)
STD_ROM_FN(SpecLastvampire)

struct BurnDriver BurnSpecLastvampire = {
	"spec_lastvampire", NULL, "spec_spectrum", NULL, "1990",
	"Last Vampire, The (48K)\0", NULL, "Atlantis Software", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_RUNGUN, 0,
	SpectrumGetZipName, SpecLastvampireRomInfo, SpecLastvampireRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Michel Futbol Master - Championship (Spanish) (48K)

static struct BurnRomInfo SpecMichelfutbolchampRomDesc[] = {
	{ "Michel Futbol Master (1989)(Dinamic).tap", 42768, 0x38824765, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecMichelfutbolchamp, SpecMichelfutbolchamp, Spectrum)
STD_ROM_FN(SpecMichelfutbolchamp)

struct BurnDriver BurnSpecMichelfutbolchamp = {
	"spec_michelfutbolchamp", NULL, "spec_spectrum", NULL, "1989",
	"Michel Futbol Master - Championship (Spanish) (48K)\0", NULL, "Dinamic Software", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 2, HARDWARE_SPECTRUM, GBF_SPORTSFOOTBALL, 0,
	SpectrumGetZipName, SpecMichelfutbolchampRomInfo, SpecMichelfutbolchampRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Michel Futbol Master - Super Skills (Spanish) (48K)

static struct BurnRomInfo SpecMichelfutbolskillsRomDesc[] = {
	{ "Michel Super Skills (1989)(Dinamic).tap", 43013, 0x035d9ad3, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecMichelfutbolskills, SpecMichelfutbolskills, Spectrum)
STD_ROM_FN(SpecMichelfutbolskills)

struct BurnDriver BurnSpecMichelfutbolskills = {
	"spec_michelfutbolskills", NULL, "spec_spectrum", NULL, "1989",
	"Michel Futbol Master - Super Skills (Spanish) (48K)\0", NULL, "Dinamic Software", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_SPORTSFOOTBALL, 0,
	SpectrumGetZipName, SpecMichelfutbolskillsRomInfo, SpecMichelfutbolskillsRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// NARC (128K)

static struct BurnRomInfo SpecNarcRomDesc[] = {
	{ "NARC (1990)(Ocean).tap", 356350, 0x3c9ccc9c, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecNarc, SpecNarc, Spec128)
STD_ROM_FN(SpecNarc)

struct BurnDriver BurnSpecNarc = {
	"spec_narc", NULL, "spec_spec128", NULL, "1990",
	"NARC (128K)\0", NULL, "Ocean", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 2, HARDWARE_SPECTRUM, GBF_RUNGUN, 0,
	SpectrumGetZipName, SpecNarcRomInfo, SpecNarcRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Simpsons - Bart vs. the Space Mutants, The (128K)

static struct BurnRomInfo SpecSimpsonsbartvsspacemutRomDesc[] = {
	{ "Simpsons - Bart vs. the Space Mutants, The (1991)(Ocean).tap", 351987, 0x382137b2, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecSimpsonsbartvsspacemut, SpecSimpsonsbartvsspacemut, Spec128)
STD_ROM_FN(SpecSimpsonsbartvsspacemut)

struct BurnDriver BurnSpecSimpsonsbartvsspacemut = {
	"spec_simpsonsbartvsspacemut", NULL, "spec_spec128", NULL, "1991",
	"Simpsons - Bart vs. the Space Mutants, The (128K)\0", "Press '0' to start the game", "Ocean", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecSimpsonsbartvsspacemutRomInfo, SpecSimpsonsbartvsspacemutRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecIntf2DIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Skateball (48K)

static struct BurnRomInfo SpecSkateballRomDesc[] = {
	{ "Skateball (1988)(Ubi Soft).tap", 49399, 0xe816c833, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecSkateball, SpecSkateball, Spectrum)
STD_ROM_FN(SpecSkateball)

struct BurnDriver BurnSpecSkateball = {
	"spec_skateball", NULL, "spec_spectrum", NULL, "1988",
	"Skateball (48K)\0", NULL, "Ubi Soft", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 2, HARDWARE_SPECTRUM, GBF_SPORTSMISC, 0,
	SpectrumGetZipName, SpecSkateballRomInfo, SpecSkateballRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Sly Spy - Secret Agent (128K)

static struct BurnRomInfo SpecSlyspyRomDesc[] = {
	{ "Sly Spy - Secret Agent (1990)(Ocean).tap", 170067, 0x1276744e, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecSlyspy, SpecSlyspy, Spec128)
STD_ROM_FN(SpecSlyspy)

struct BurnDriver BurnSpecSlyspy = {
	"spec_slyspy", NULL, "spec_spec128", NULL, "1990",
	"Sly Spy - Secret Agent (128K)\0", "Select 'Interface II' to use controller", "Ocean", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_ACTION | GBF_RUNGUN, 0,
	SpectrumGetZipName, SpecSlyspyRomInfo, SpecSlyspyRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecIntf2DIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Steg the Slug (128K)

static struct BurnRomInfo SpecStegRomDesc[] = {
	{ "Steg (1992)(Codemasters).tap", 51271, 0x15a4d49e, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecSteg, SpecSteg, Spec128)
STD_ROM_FN(SpecSteg)

struct BurnDriver BurnSpecSteg = {
	"spec_steg", NULL, "spec_spec128", NULL, "1992",
	"Steg the Slug (128K)\0", NULL, "Codemasters", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_PLATFORM | GBF_PUZZLE, 0,
	SpectrumGetZipName, SpecStegRomInfo, SpecStegRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Toobin' (128K)

static struct BurnRomInfo SpecToobinRomDesc[] = {
	{ "Toobin' (1989)(Domark).tap", 53121, 0xa2acce35, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecToobin, SpecToobin, Spec128)
STD_ROM_FN(SpecToobin)

struct BurnDriver BurnSpecToobin = {
	"spec_toobin", NULL, "spec_spec128", NULL, "1989",
	"Toobin' (128K)\0", NULL, "Domark", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 2, HARDWARE_SPECTRUM, GBF_ACTION, 0,
	SpectrumGetZipName, SpecToobinRomInfo, SpecToobinRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// V (Visitors) (48K)

static struct BurnRomInfo SpecVoceanRomDesc[] = {
	{ "V (1986)(Ocean).tap", 56344, 0x07f2dfd6, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecVocean, SpecVocean, Spectrum)
STD_ROM_FN(SpecVocean)

struct BurnDriver BurnSpecVocean = {
	"spec_vocean", NULL, "spec_spectrum", NULL, "1986",
	"V (Visitors) (48K)\0", NULL, "Ocean", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_ACTION | GBF_RUNGUN, 0,
	SpectrumGetZipName, SpecVoceanRomInfo, SpecVoceanRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Where Time Stood Still (128K)

static struct BurnRomInfo SpecWheretimestoodstillRomDesc[] = {
	{ "Where Time Stood Still (1988)(Ocean).tap", 110751, 0xfa89cab1, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecWheretimestoodstill, SpecWheretimestoodstill, Spec128)
STD_ROM_FN(SpecWheretimestoodstill)

struct BurnDriver BurnSpecWheretimestoodstill = {
	"spec_wheretimestoodstill", NULL, "spec_spec128", NULL, "1988",
	"Where Time Stood Still (128K)\0", NULL, "Ocean", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_ADV, 0,
	SpectrumGetZipName, SpecWheretimestoodstillRomInfo, SpecWheretimestoodstillRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Wizball (128K)

static struct BurnRomInfo SpecWizballRomDesc[] = {
	{ "Wizball (1987)(Ocean).tap", 34256, 0x70cf4de6, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecWizball, SpecWizball, Spec128)
STD_ROM_FN(SpecWizball)

struct BurnDriver BurnSpecWizball = {
	"spec_wizball", NULL, "spec_spec128", NULL, "1987",
	"Wizball (128K)\0", NULL, "Ocean", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_PLATFORM | GBF_HORSHOOT, 0,
	SpectrumGetZipName, SpecWizballRomInfo, SpecWizballRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Aaargh! (48K)

static struct BurnRomInfo SpecAaarghRomDesc[] = {
	{ "Aaargh! (1989)(Binary Design).tap", 96865, 0x11417dd7, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecAaargh, SpecAaargh, Spectrum)
STD_ROM_FN(SpecAaargh)

struct BurnDriver BurnSpecAaargh = {
	"spec_aaargh", NULL, "spec_spectrum", NULL, "1989",
	"Aaargh! (48K)\0", NULL, "Binary Design", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 2, HARDWARE_SPECTRUM, GBF_ACTION, 0,
	SpectrumGetZipName, SpecAaarghRomInfo, SpecAaarghRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Agent Orange (48K)

static struct BurnRomInfo SpecAgentorangeRomDesc[] = {
	{ "Agent Orange (1987)(Icon Design).tap", 43566, 0x112aedd9, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecAgentorange, SpecAgentorange, Spectrum)
STD_ROM_FN(SpecAgentorange)

struct BurnDriver BurnSpecAgentorange = {
	"spec_agentorange", NULL, "spec_spectrum", NULL, "1987",
	"Agent Orange (48K)\0", NULL, "Icon Design", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_SHOOT, 0,
	SpectrumGetZipName, SpecAgentorangeRomInfo, SpecAgentorangeRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Ah Diddums (48K)

static struct BurnRomInfo SpecAhdiddumsRomDesc[] = {
	{ "Ah Diddums (1983)(Imagine).tap", 8490, 0xe5b88c3b, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecAhdiddums, SpecAhdiddums, Spec128)
STD_ROM_FN(SpecAhdiddums)

struct BurnDriver BurnSpecAhdiddums = {
	"spec_ahdiddums", NULL, "spec_spec128", NULL, "1983",
	"Ah Diddums (48K)\0", NULL, "Imagine Software", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_ACTION, 0,
	SpectrumGetZipName, SpecAhdiddumsRomInfo, SpecAhdiddumsRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Alien Evolution (128K)

static struct BurnRomInfo SpecAlienevolutionRomDesc[] = {
	{ "Alien Evolution (1987)(Gremlin).tap", 49510, 0xa16b45f5, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecAlienevolution, SpecAlienevolution, Spec128)
STD_ROM_FN(SpecAlienevolution)

struct BurnDriver BurnSpecAlienevolution = {
	"spec_alienevolution", NULL, "spec_spec128", NULL, "1987",
	"Alien Evolution (128K)\0", NULL, "Gremlin Graphics", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_SHOOT, 0,
	SpectrumGetZipName, SpecAlienevolutionRomInfo, SpecAlienevolutionRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Aqua Plane (48K)

static struct BurnRomInfo SpecAquaplaneRomDesc[] = {
	{ "Aqua Plane (1983)(Quicksilva).tap", 13553, 0x3ef11315, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecAquaplane, SpecAquaplane, Spectrum)
STD_ROM_FN(SpecAquaplane)

struct BurnDriver BurnSpecAquaplane = {
	"spec_aquaplane", NULL, "spec_spectrum", NULL, "1983",
	"Aqua Plane (48K)\0", "Press ENTER to skip 'loading screen'", "Quicksilva", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_ACTION, 0,
	SpectrumGetZipName, SpecAquaplaneRomInfo, SpecAquaplaneRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Atom Ant (128K)

static struct BurnRomInfo SpecAtomantRomDesc[] = {
	{ "Atom Ant (1990)(Hi-Tec).tap", 61884, 0xbb656d10, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecAtomant, SpecAtomant, Spec128)
STD_ROM_FN(SpecAtomant)

struct BurnDriver BurnSpecAtomant = {
	"spec_atomant", NULL, "spec_spec128", NULL, "1990",
	"Atom Ant (128K)\0", NULL, "Hi-Tec Software", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecAtomantRomInfo, SpecAtomantRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// AutoCrash (Spanish) (128K)

static struct BurnRomInfo SpecAutocrashRomDesc[] = {
	{ "Autocrash (1991)(Zigurat).tap", 49550, 0x02fe192c, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecAutocrash, SpecAutocrash, Spec128)
STD_ROM_FN(SpecAutocrash)

struct BurnDriver BurnSpecAutocrash = {
	"spec_autocrash", NULL, "spec_spec128", NULL, "1991",
	"AutoCrash (Spanish) (128K)\0", NULL, "Zigurat Software", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_ACTION, 0,
	SpectrumGetZipName, SpecAutocrashRomInfo, SpecAutocrashRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// B.C. Bill (48K)

static struct BurnRomInfo SpecBcbillRomDesc[] = {
	{ "B.C. Bill (1984)(Imagine).tap", 30496, 0x61b2d9bf, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecBcbill, SpecBcbill, Spectrum)
STD_ROM_FN(SpecBcbill)

struct BurnDriver BurnSpecBcbill = {
	"spec_bcbill", NULL, "spec_spectrum", NULL, "1984",
	"B.C. Bill (48K)\0", NULL, "Imagine Software", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_ACTION, 0,
	SpectrumGetZipName, SpecBcbillRomInfo, SpecBcbillRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// BC's Quest for Tires (48K)

static struct BurnRomInfo SpecBcquesttiresRomDesc[] = {
	{ "BC's Quest for Tires (1985)(Software Projects).tap", 43450, 0xfcb53b87, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecBcquesttires, SpecBcquesttires, Spectrum)
STD_ROM_FN(SpecBcquesttires)

struct BurnDriver BurnSpecBcquesttires = {
	"spec_bcquesttires", NULL, "spec_spectrum", NULL, "1985",
	"BC's Quest for Tires (48K)\0", "Press 'P' for control options", "Software Projects", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 2, HARDWARE_SPECTRUM, GBF_ACTION, 0,
	SpectrumGetZipName, SpecBcquesttiresRomInfo, SpecBcquesttiresRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Blob the Cop (48K)

static struct BurnRomInfo SpecBlobthecopRomDesc[] = {
	{ "Blob the Cop (1989)(Players Software).tap", 42196, 0x09b3e306, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecBlobthecop, SpecBlobthecop, Spectrum)
STD_ROM_FN(SpecBlobthecop)

struct BurnDriver BurnSpecBlobthecop = {
	"spec_blobthecop", NULL, "spec_spectrum", NULL, "1989",
	"Blob the Cop (48K)\0", "ALT + SPACE for control options", "Players Software", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_RUNGUN, 0,
	SpectrumGetZipName, SpecBlobthecopRomInfo, SpecBlobthecopRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Bubbler (48K)

static struct BurnRomInfo SpecBubblerRomDesc[] = {
	{ "Bubbler (1987)(Ultimate Play The Game).tap", 41660, 0x8e3e4b5a, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecBubbler, SpecBubbler, Spectrum)
STD_ROM_FN(SpecBubbler)

struct BurnDriver BurnSpecBubbler = {
	"spec_bubbler", NULL, "spec_spectrum", NULL, "1987",
	"Bubbler (48K)\0", "Press '1' for P1 options and '2' for P2 options", "Ultimate Play The Game", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 2, HARDWARE_SPECTRUM, GBF_ACTION, 0,
	SpectrumGetZipName, SpecBubblerRomInfo, SpecBubblerRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Buggy Ranger (48K)

static struct BurnRomInfo SpecBuggyrangerRomDesc[] = {
	{ "Buggy Ranger (1990)(Dinamic).tap", 50964, 0xc7f9d095, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecBuggyranger, SpecBuggyranger, Spectrum)
STD_ROM_FN(SpecBuggyranger)

struct BurnDriver BurnSpecBuggyranger = {
	"spec_buggyranger", NULL, "spec_spectrum", NULL, "1990",
	"Buggy Ranger (48K)\0", NULL, "Dinamic Software", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_RUNGUN, 0,
	SpectrumGetZipName, SpecBuggyrangerRomInfo, SpecBuggyrangerRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Chain Reaction (128K)

static struct BurnRomInfo SpecChainreactionRomDesc[] = {
	{ "Chain Reaction (1987)(Durell).tap", 51862, 0x42c4cd17, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecChainreaction, SpecChainreaction, Spec128)
STD_ROM_FN(SpecChainreaction)

struct BurnDriver BurnSpecChainreaction = {
	"spec_chainreaction", NULL, "spec_spec128", NULL, "1987",
	"Chain Reaction (128K)\0", NULL, "Durell Software", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_RUNGUN | GBF_ADV, 0,
	SpectrumGetZipName, SpecChainreactionRomInfo, SpecChainreactionRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Chickin Chase (Il Galletto) (Italian) (48K)

static struct BurnRomInfo SpecChickinchaseRomDesc[] = {
	{ "Chickin Chase (1985)(Firebird).tap", 50446, 0x5762f239, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecChickinchase, SpecChickinchase, Spectrum)
STD_ROM_FN(SpecChickinchase)

struct BurnDriver BurnSpecChickinchase = {
	"spec_chickinchase", NULL, "spec_spectrum", NULL, "1985",
	"Il Galletto - Chickin Chase (Italian) (48K)\0", NULL, "Firebird Software", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_ACTION, 0,
	SpectrumGetZipName, SpecChickinchaseRomInfo, SpecChickinchaseRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Comando Quatro (48K)

static struct BurnRomInfo SpecComandoquatroRomDesc[] = {
	{ "Comando Quatro (1989)(Zigurat).tap", 49485, 0x11a8ebc0, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecComandoquatro, SpecComandoquatro, Spectrum)
STD_ROM_FN(SpecComandoquatro)

struct BurnDriver BurnSpecComandoquatro = {
	"spec_comandoquatro", NULL, "spec_spectrum", NULL, "1989",
	"Comando Quatro (48K)\0", NULL, "Zigurat Software", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_ACTION, 0,
	SpectrumGetZipName, SpecComandoquatroRomInfo, SpecComandoquatroRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Cosmic Wartoad (48K)

static struct BurnRomInfo SpecCosmicwartoadRomDesc[] = {
	{ "Cosmic Wartoad (1985)(Ocean).tap", 45221, 0x81e8ae65, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecCosmicwartoad, SpecCosmicwartoad, Spectrum)
STD_ROM_FN(SpecCosmicwartoad)

struct BurnDriver BurnSpecCosmicwartoad = {
	"spec_cosmicwartoad", NULL, "spec_spectrum", NULL, "1985",
	"Cosmic Wartoad (48K)\0", NULL, "Ocean", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_ACTION, 0,
	SpectrumGetZipName, SpecCosmicwartoadRomInfo, SpecCosmicwartoadRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Dark Star (48K)

static struct BurnRomInfo SpecDarkstarRomDesc[] = {
	{ "Dark Star (1984)(Design Design).tap", 45980, 0x38a11d60, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecDarkstar, SpecDarkstar, Spectrum)
STD_ROM_FN(SpecDarkstar)

struct BurnDriver BurnSpecDarkstar = {
	"spec_darkstar", NULL, "spec_spectrum", NULL, "1984",
	"Dark Star (48K)\0", NULL, "Design Design", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_SHOOT, 0,
	SpectrumGetZipName, SpecDarkstarRomInfo, SpecDarkstarRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Dea Tenebrarum (Spanish) (48K)

static struct BurnRomInfo SpecDeatenebrarumRomDesc[] = {
	{ "Dea Tenebrarum (1988)(System 4).tap", 48584, 0xef594c3b, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecDeatenebrarum, SpecDeatenebrarum, Spectrum)
STD_ROM_FN(SpecDeatenebrarum)

struct BurnDriver BurnSpecDeatenebrarum = {
	"spec_deatenebrarum", NULL, "spec_spectrum", NULL, "1988",
	"Dea Tenebrarum (Spanish) (48K)\0", NULL, "System 4", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecDeatenebrarumRomInfo, SpecDeatenebrarumRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Denizen (128K)

static struct BurnRomInfo SpecDenizenRomDesc[] = {
	{ "Denizen (1988)(Players).tap", 112972, 0x6ae6dff4, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecDenizen, SpecDenizen, Spec128)
STD_ROM_FN(SpecDenizen)

struct BurnDriver BurnSpecDenizen = {
	"spec_denizen", NULL, "spec_spec128", NULL, "1988",
	"Denizen (128K)\0", NULL, "Players Software", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_RUNGUN | GBF_MAZE, 0,
	SpectrumGetZipName, SpecDenizenRomInfo, SpecDenizenRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Disco Dan (48K)

static struct BurnRomInfo SpecDiscodanRomDesc[] = {
	{ "Disco Dan (1984)(Gem Software).tap", 19931, 0x8ba92db3, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecDiscodan, SpecDiscodan, Spectrum)
STD_ROM_FN(SpecDiscodan)

struct BurnDriver BurnSpecDiscodan = {
	"spec_discodan", NULL, "spec_spectrum", NULL, "1984",
	"Disco Dan (48K) \0", "Set '1. Cursor Joystick' to use controller", "Gem Software", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_ACTION, 0,
	SpectrumGetZipName, SpecDiscodanRomInfo, SpecDiscodanRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecCursorKeysDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Duet (48K)

static struct BurnRomInfo SpecDuetRomDesc[] = {
	{ "Duet (1987)(Elite).tap", 41164, 0x80308e30, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecDuet, SpecDuet, Spectrum)
STD_ROM_FN(SpecDuet)

struct BurnDriver BurnSpecDuet = {
	"spec_duet", NULL, "spec_spectrum", NULL, "1987",
	"Duet (48K)\0", "Q, A, E, R and Z to control player 1", "Elite Systems", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 2, HARDWARE_SPECTRUM, GBF_RUNGUN | GBF_MAZE, 0,
	SpectrumGetZipName, SpecDuetRomInfo, SpecDuetRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Eliminator (128K)

static struct BurnRomInfo SpecEliminatorRomDesc[] = {
	{ "Eliminator (1988)(Hewson).tap", 44964, 0x4e5643c0, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecEliminator, SpecEliminator, Spec128)
STD_ROM_FN(SpecEliminator)

struct BurnDriver BurnSpecEliminator = {
	"spec_eliminator", NULL, "spec_spec128", NULL, "1988",
	"Eliminator (128K)\0", NULL, "Hewson Consultants", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_SHOOT | GBF_RACING, 0,
	SpectrumGetZipName, SpecEliminatorRomInfo, SpecEliminatorRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Fearless Frank (48K)

static struct BurnRomInfo SpecFearlessfrankRomDesc[] = {
	{ "Fearless Frank (1984)(Visions).tap", 40614, 0x573889ab, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecFearlessfrank, SpecFearlessfrank, Spectrum)
STD_ROM_FN(SpecFearlessfrank)

struct BurnDriver BurnSpecFearlessfrank = {
	"spec_fearlessfrank", NULL, "spec_spectrum", NULL, "1984",
	"Fearless Frank (48K)\0", NULL, "Visions", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_ACTION, 0,
	SpectrumGetZipName, SpecFearlessfrankRomInfo, SpecFearlessfrankRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Foxx Fights Back (128K)

static struct BurnRomInfo SpecFoxxfightsbackRomDesc[] = {
	{ "Foxx Fights Back (1988)(Image Works).tap", 47395, 0x7c9ae4b5, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecFoxxfightsback, SpecFoxxfightsback, Spec128)
STD_ROM_FN(SpecFoxxfightsback)

struct BurnDriver BurnSpecFoxxfightsback = {
	"spec_foxxfightsback", NULL, "spec_spec128", NULL, "1988",
	"Foxx Fights Back (128K)\0", NULL, "Image Works", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_ACTION, 0,
	SpectrumGetZipName, SpecFoxxfightsbackRomInfo, SpecFoxxfightsbackRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Star Blade (48K)

static struct BurnRomInfo SpecStarbladeRomDesc[] = {
	{ "Star Blade (1995)(Barry Addams).z80", 38880, 0x96e096c6, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecStarblade, SpecStarblade, Spectrum)
STD_ROM_FN(SpecStarblade)

struct BurnDriver BurnSpecStarblade = {
	"spec_starblade", NULL, "spec_spectrum", NULL, "1995",
	"Star Blade (48K)\0", NULL, "Barry Addams", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_HORSHOOT, 0,
	SpectrumGetZipName, SpecStarbladeRomInfo, SpecStarbladeRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// UVE Atac (48K)

static struct BurnRomInfo SpecUveatacRomDesc[] = {
	{ "UVE Atac (1985)(MicroHobby).tap", 17979, 0x78f60aea, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecUveatac, SpecUveatac, Spectrum)
STD_ROM_FN(SpecUveatac)

struct BurnDriver BurnSpecUveatac = {
	"spec_uveatac", NULL, "spec_spectrum", NULL, "1985",
	"UVE Atac (48K)\0", NULL, "MicroHobby", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_ACTION, 0,
	SpectrumGetZipName, SpecUveatacRomInfo, SpecUveatacRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Zub (128K)

static struct BurnRomInfo SpecZubRomDesc[] = {
	{ "Zub (1986)(Mastertronic Added Dimension).tap", 62034, 0xe48404af, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecZub, SpecZub, Spec128)
STD_ROM_FN(SpecZub)

struct BurnDriver BurnSpecZub = {
	"spec_zub", NULL, "spec_spec128", NULL, "1986",
	"Zub (128K)\0", NULL, "Mastertronic Added Dimension", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecZubRomInfo, SpecZubRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Gunhead (128K)

static struct BurnRomInfo SpecGunheadRomDesc[] = {
	{ "Gunhead (1990)(Hellenic).tap", 47800, 0xd1c99fb1, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecGunhead, SpecGunhead, Spec128)
STD_ROM_FN(SpecGunhead)

struct BurnDriver BurnSpecGunhead = {
	"spec_gunhead", NULL, "spec_spec128", NULL, "1990",
	"Gunhead (128K)\0", NULL, "Hellenic", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_HORSHOOT, 0,
	SpectrumGetZipName, SpecGunheadRomInfo, SpecGunheadRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecQAOPMDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Hunter (128K)

static struct BurnRomInfo SpecHunterRomDesc[] = {
	{ "Hunter (1991)(Stuffduck).tap", 47590, 0xac3c997b, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecHunter, SpecHunter, Spec128)
STD_ROM_FN(SpecHunter)

struct BurnDriver BurnSpecHunter = {
	"spec_hunter", NULL, "spec_spec128", NULL, "1991",
	"Hunter (128K)\0", NULL, "Stuffduck", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_SHOOT, 0,
	SpectrumGetZipName, SpecHunterRomInfo, SpecHunterRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// MASK (48K-128K)

static struct BurnRomInfo SpecMaskRomDesc[] = {
	{ "Mask (1987)(Gremlin).z80", 92283, 0xcdd8c4b1, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecMask, SpecMask, Spec128)
STD_ROM_FN(SpecMask)

struct BurnDriver BurnSpecMask = {
	"spec_mask", NULL, "spec_spec128", NULL, "1987",
	"MASK (48K-128K)\0", NULL, "Gremlin Graphics", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_RUNGUN, 0,
	SpectrumGetZipName, SpecMaskRomInfo, SpecMaskRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Master Blaster (48K)

static struct BurnRomInfo SpecMasterblasterRomDesc[] = {
	{ "Master Blaster (1990)(Cybadyne).tap", 49304, 0x83f104ce, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecMasterblaster, SpecMasterblaster, Spectrum)
STD_ROM_FN(SpecMasterblaster)

struct BurnDriver BurnSpecMasterblaster = {
	"spec_masterblaster", NULL, "spec_spectrum", NULL, "1990",
	"Master Blaster (48K)\0", NULL, "Cybadyne", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_VERSHOOT, 0,
	SpectrumGetZipName, SpecMasterblasterRomInfo, SpecMasterblasterRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Mean Streak (128K)

static struct BurnRomInfo SpecMeanstreakRomDesc[] = {
	{ "Mean Streak (1987)(Mirrorsoft).tap", 66912, 0xeaa7cbc4, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecMeanstreak, SpecMeanstreak, Spec128)
STD_ROM_FN(SpecMeanstreak)

struct BurnDriver BurnSpecMeanstreak = {
	"spec_meanstreak", NULL, "spec_spec128", NULL, "1987",
	"Mean Streak (128K)\0", NULL, "Mirrorsoft", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_ACTION, 0,
	SpectrumGetZipName, SpecMeanstreakRomInfo, SpecMeanstreakRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecQAOPSpaceDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Mega Apocalypse (128K)

static struct BurnRomInfo SpecMegaapocalypseRomDesc[] = {
	{ "Mega Apocalypse (1988)(Martech).tap", 45336, 0x989130c0, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecMegaapocalypse, SpecMegaapocalypse, Spec128)
STD_ROM_FN(SpecMegaapocalypse)

struct BurnDriver BurnSpecMegaapocalypse = {
	"spec_megaapocalypse", NULL, "spec_spec128", NULL, "1988",
	"Mega Apocalypse (128K)\0", NULL, "Martech", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_SHOOT, 0,
	SpectrumGetZipName, SpecMegaapocalypseRomInfo, SpecMegaapocalypseRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Mega Phoenix (128K)

static struct BurnRomInfo SpecMegaphoenixRomDesc[] = {
	{ "Megaphoenix (1991)(Dinamic).tap", 43229, 0x81937ff0, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecMegaphoenix, SpecMegaphoenix, Spec128)
STD_ROM_FN(SpecMegaphoenix)

struct BurnDriver BurnSpecMegaphoenix = {
	"spec_megaphoenix", NULL, "spec_spec128", NULL, "1991",
	"Mega Phoenix (128K)\0", NULL, "Dinamic Software", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 2, HARDWARE_SPECTRUM, GBF_SHOOT, 0,
	SpectrumGetZipName, SpecMegaphoenixRomInfo, SpecMegaphoenixRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Moon Strike (48K)

static struct BurnRomInfo SpecMoonstrikeRomDesc[] = {
	{ "Moon Strike (1987)(Mirrorsoft).tap", 49633, 0xbd0e5cbd, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecMoonstrike, SpecMoonstrike, Spectrum)
STD_ROM_FN(SpecMoonstrike)

struct BurnDriver BurnSpecMoonstrike = {
	"spec_moonstrike", NULL, "spec_spectrum", NULL, "1987",
	"Moon Strike (48K)\0", "Press Enter to skip intro screen", "Mirrorsoft", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_VERSHOOT, 0,
	SpectrumGetZipName, SpecMoonstrikeRomInfo, SpecMoonstrikeRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Paratroopers (16K)

static struct BurnRomInfo SpecParatroopersRomDesc[] = {
	{ "Paratroopers (1983)(Rabbit).tap", 21732, 0xc19b3a5d, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecParatroopers, SpecParatroopers, Spectrum)
STD_ROM_FN(SpecParatroopers)

struct BurnDriver BurnSpecParatroopers = {
	"spec_paratroopers", NULL, "spec_spectrum", NULL, "1983",
	"Paratroopers (16K)\0", NULL, "Rabbit", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_SHOOT, 0,
	SpectrumGetZipName, SpecParatroopersRomInfo, SpecParatroopersRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Phileas Fogg's Balloon Battles (48K)

static struct BurnRomInfo SpecPhileasfoggRomDesc[] = {
	{ "Phileas Fogg's Balloon Battles (1991)(Zeppelin).tap", 48594, 0xc311768c, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecPhileasfogg, SpecPhileasfogg, Spectrum)
STD_ROM_FN(SpecPhileasfogg)

struct BurnDriver BurnSpecPhileasfogg = {
	"spec_phileasfogg", NULL, "spec_spectrum", NULL, "1991",
	"Phileas Fogg's Balloon Battles (48K)\0", NULL, "Zeppelin Games", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_RUNGUN, 0,
	SpectrumGetZipName, SpecPhileasfoggRomInfo, SpecPhileasfoggRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecIntf2DIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Quick Draw McGraw (128K)

static struct BurnRomInfo SpecQuickdrawRomDesc[] = {
	{ "Quick Draw McGraw (1991)(Hi-Tec).tap", 53434, 0x435650ba, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecQuickdraw, SpecQuickdraw, Spec128)
STD_ROM_FN(SpecQuickdraw)

struct BurnDriver BurnSpecQuickdraw = {
	"spec_quickdraw", NULL, "spec_spec128", NULL, "1991",
	"Quick Draw McGraw (128K)\0", NULL, "Hi-Tec Software", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecQuickdrawRomInfo, SpecQuickdrawRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Seahawk (48K)

static struct BurnRomInfo SpecSeahawkRomDesc[] = {
	{ "Seahawk (1990)(Atlantis Software).tap", 48448, 0x32c8705a, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecSeahawk, SpecSeahawk, Spectrum)
STD_ROM_FN(SpecSeahawk)

struct BurnDriver BurnSpecSeahawk = {
	"spec_seahawk", NULL, "spec_spectrum", NULL, "1990",
	"Seahawk (48K)\0", NULL, "Atlantis Software", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_HORSHOOT, 0,
	SpectrumGetZipName, SpecSeahawkRomInfo, SpecSeahawkRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecQAOPMDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Shadow Skimmer (48K)

static struct BurnRomInfo SpecShadowskimmerRomDesc[] = {
	{ "Shadow Skimmer (1987)(The Edge).z80", 41483, 0xc7d0a073, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecShadowskimmer, SpecShadowskimmer, Spectrum)
STD_ROM_FN(SpecShadowskimmer)

struct BurnDriver BurnSpecShadowskimmer = {
	"spec_shadowskimmer", NULL, "spec_spectrum", NULL, "1987",
	"Shadow Skimmer (48K)\0", NULL, "The Edge", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_SHOOT, 0,
	SpectrumGetZipName, SpecShadowskimmerRomInfo, SpecShadowskimmerRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Shark (128K)

static struct BurnRomInfo SpecSharkRomDesc[] = {
	{ "Shark (1989)(Players Premier).tap", 101962, 0x014ce429, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecShark, SpecShark, Spec128)
STD_ROM_FN(SpecShark)

struct BurnDriver BurnSpecShark = {
	"spec_shark", NULL, "spec_spec128", NULL, "1989",
	"Shark (128K)\0", NULL, "Players Premier", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_SHOOT, 0,
	SpectrumGetZipName, SpecSharkRomInfo, SpecSharkRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Star Firebirds (48K)

static struct BurnRomInfo SpecStarfirebirdsRomDesc[] = {
	{ "Star Firebirds (1985)(Firebird).z80", 34053, 0xe38dec78, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecStarfirebirds, SpecStarfirebirds, Spectrum)
STD_ROM_FN(SpecStarfirebirds)

struct BurnDriver BurnSpecStarfirebirds = {
	"spec_starfirebirds", NULL, "spec_spectrum", NULL, "1985",
	"Star Firebirds (48K)\0", NULL, "Firebird Software", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 2, HARDWARE_SPECTRUM, GBF_SHOOT, 0,
	SpectrumGetZipName, SpecStarfirebirdsRomInfo, SpecStarfirebirdsRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Tomcat (48K)

static struct BurnRomInfo SpecTomcatRomDesc[] = {
	{ "Tomcat (1989)(Players).tap", 41255, 0xef75de5c, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecTomcat, SpecTomcat, Spectrum)
STD_ROM_FN(SpecTomcat)

struct BurnDriver BurnSpecTomcat = {
	"spec_tomcat", NULL, "spec_spectrum", NULL, "1989",
	"Tomcat (48K)\0", NULL, "Players Software", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_VERSHOOT, 0,
	SpectrumGetZipName, SpecTomcatRomInfo, SpecTomcatRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// 3D-Tetris (48K)

static struct BurnRomInfo Spec3dtetrisRomDesc[] = {
	{ "3D-Tetris (1989)(Antic).z80", 19178, 0xe46b5e70, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Spec3dtetris, Spec3dtetris, Spectrum)
STD_ROM_FN(Spec3dtetris)

struct BurnDriver BurnSpec3dtetris = {
	"spec_3dtetris", NULL, "spec_spectrum", NULL, "1989",
	"3D-Tetris (48K)\0", NULL, "Antic", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_PUZZLE, 0,
	SpectrumGetZipName, Spec3dtetrisRomInfo, Spec3dtetrisRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// 5 in 1 Tetris (128K)

static struct BurnRomInfo Spec5in1tetrisRomDesc[] = {
	{ "5 in 1 Tetris (1995)(Mortal Kombat).z80", 28526, 0x996128b2, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Spec5in1tetris, Spec5in1tetris, Spec128)
STD_ROM_FN(Spec5in1tetris)

struct BurnDriver BurnSpec5in1tetris = {
	"spec_5in1tetris", NULL, "spec_spec128", NULL, "1995",
	"5 in 1 Tetris (128K)\0", NULL, "Mortal Kombat", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_PUZZLE, 0,
	SpectrumGetZipName, Spec5in1tetrisRomInfo, Spec5in1tetrisRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Ammytris (128K)

static struct BurnRomInfo SpecAmmytrisRomDesc[] = {
	{ "Ammytris (1995)(Navigator).tap", 19901, 0x24de1080, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecAmmytris, SpecAmmytris, Spec128)
STD_ROM_FN(SpecAmmytris)

struct BurnDriver BurnSpecAmmytris = {
	"spec_ammytris", NULL, "spec_spec128", NULL, "1995",
	"Ammytris (128K)\0", NULL, "Navigator", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_PUZZLE, 0,
	SpectrumGetZipName, SpecAmmytrisRomInfo, SpecAmmytrisRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecQAOPSpaceDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Miami Chase (128K)

static struct BurnRomInfo SpecMiamichaseRomDesc[] = {
	{ "Miami Chase (1991)(Codemasters).tap", 48671, 0x24cb897c, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecMiamichase, SpecMiamichase, Spec128)
STD_ROM_FN(SpecMiamichase)

struct BurnDriver BurnSpecMiamichase = {
	"spec_miamichase", NULL, "spec_spec128", NULL, "1991",
	"Miami Chase (128K)\0", "Use keys K, M and L to select options", "Codemasters", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_SHOOT | GBF_RACING, 0,
	SpectrumGetZipName, SpecMiamichaseRomInfo, SpecMiamichaseRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Mig 29 Soviet Fighter (128K)

static struct BurnRomInfo SpecMig29sovfighterRomDesc[] = {
	{ "Mig 29 Soviet Fighter (1989)(Codemasters).tap", 49442, 0x93a448fd, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecMig29sovfighter, SpecMig29sovfighter, Spec128)
STD_ROM_FN(SpecMig29sovfighter)

struct BurnDriver BurnSpecMig29sovfighter = {
	"spec_mig29sovfighter", NULL, "spec_spec128", NULL, "1989",
	"Mig 29 Soviet Fighter (128K)\0", NULL, "Codemasters", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_SHOOT, 0,
	SpectrumGetZipName, SpecMig29sovfighterRomInfo, SpecMig29sovfighterRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Mission Jupiter (48K)

static struct BurnRomInfo SpecMissionjupiterRomDesc[] = {
	{ "Mission Jupiter (1987)(Codemasters).tap", 45152, 0xf136a5b9, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecMissionjupiter, SpecMissionjupiter, Spectrum)
STD_ROM_FN(SpecMissionjupiter)

struct BurnDriver BurnSpecMissionjupiter = {
	"spec_missionjupiter", NULL, "spec_spectrum", NULL, "1987",
	"Mission Jupiter (48K)\0", NULL, "Codemasters", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 2, HARDWARE_SPECTRUM, GBF_ACTION, 0,
	SpectrumGetZipName, SpecMissionjupiterRomInfo, SpecMissionjupiterRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Monte Carlo Casino (128K)

static struct BurnRomInfo SpecMontecarlocasinoRomDesc[] = {
	{ "Monte Carlo Casino (1989)(Codemasters).tap", 41821, 0x8d459dc5, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecMontecarlocasino, SpecMontecarlocasino, Spec128)
STD_ROM_FN(SpecMontecarlocasino)

struct BurnDriver BurnSpecMontecarlocasino = {
	"spec_montecarlocasino", NULL, "spec_spec128", NULL, "1989",
	"Monte Carlo Casino (128K)\0", NULL, "Codemasters", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_CASINO, 0,
	SpectrumGetZipName, SpecMontecarlocasinoRomInfo, SpecMontecarlocasinoRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Monty on the Run (128K)

static struct BurnRomInfo SpecmontrunRomDesc[] = {
	{ "Monty on the Run (1985)(Gremlin Graphics)(128k).z80", 0x09f68, 0x330a393a, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specmontrun, Specmontrun, Spec128)
STD_ROM_FN(Specmontrun)

struct BurnDriver BurnSpecmontrun = {
	"spec_montrun", NULL, "spec_spec128", NULL, "1985",
	"Monty on the Run (128K)\0", NULL, "Gremlin Graphics", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecmontrunRomInfo, SpecmontrunRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Moon Cresta (48K)

static struct BurnRomInfo SpecmooncrstRomDesc[] = {
	{ "Moon Cresta (1985)(Incentive Software).z80", 0x0979b, 0xaf817ac8, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specmooncrst, Specmooncrst, Spectrum)
STD_ROM_FN(Specmooncrst)

struct BurnDriver BurnSpecmooncrst = {
	"spec_mooncrst", NULL, "spec_spectrum", NULL, "1985",
	"Moon Cresta (48K)\0", NULL, "Incentive Software", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 2, HARDWARE_SPECTRUM, GBF_SHOOT, 0,
	SpectrumGetZipName, SpecmooncrstRomInfo, SpecmooncrstRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Moon Cresta - Review (48K)

static struct BurnRomInfo SpecmoocrereRomDesc[] = {
	{ "Moon Cresta - Review (1985)(Incentive Software).z80", 0x0147f, 0x5db616b9, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specmoocrere, Specmoocrere, Spectrum)
STD_ROM_FN(Specmoocrere)

struct BurnDriver BurnSpecmoocrere = {
	"spec_moocrere", "spec_mooncrst", "spec_spectrum", NULL, "1985",
	"Moon Cresta - Review (48K)\0", NULL, "Incentive Software", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE, 1, HARDWARE_SPECTRUM, GBF_MISC, 0,
	SpectrumGetZipName, SpecmoocrereRomInfo, SpecmoocrereRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Moto Cross Simulator (128K)

static struct BurnRomInfo SpecMotocrosssimRomDesc[] = {
	{ "Moto Cross Simulator (1989)(Codemasters).tap", 51225, 0x7abd5f87, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecMotocrosssim, SpecMotocrosssim, Spec128)
STD_ROM_FN(SpecMotocrosssim)

struct BurnDriver BurnSpecMotocrosssim = {
	"spec_motocrosssim", NULL, "spec_spec128", NULL, "1989",
	"Moto Cross Simulator (128K)\0", NULL, "Codemasters", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_ACTION, 0,
	SpectrumGetZipName, SpecMotocrosssimRomInfo, SpecMotocrosssimRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Mountain Bike Simulator (128K)

static struct BurnRomInfo SpecMountainbikesimRomDesc[] = {
	{ "Mountain Bike Simulator (1991)(Codemasters).tap", 53006, 0xd2738989, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecMountainbikesim, SpecMountainbikesim, Spec128)
STD_ROM_FN(SpecMountainbikesim)

struct BurnDriver BurnSpecMountainbikesim = {
	"spec_mountainbikesim", NULL, "spec_spec128", NULL, "1991",
	"Mountain Bike Simulator (128K)\0", NULL, "Codemasters", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_ACTION, 0,
	SpectrumGetZipName, SpecMountainbikesimRomInfo, SpecMountainbikesimRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Murray Mouse Super Cop (48K)

static struct BurnRomInfo SpecMurraymousesupercopRomDesc[] = {
	{ "Murray Mouse Super Cop (1992)(Codemasters).tap", 32045, 0x67637ddf, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecMurraymousesupercop, SpecMurraymousesupercop, Spectrum)
STD_ROM_FN(SpecMurraymousesupercop)

struct BurnDriver BurnSpecMurraymousesupercop = {
	"spec_murraymousesupercop", NULL, "spec_spectrum", NULL, "1992",
	"Murray Mouse Super Cop (48K)\0", NULL, "Codemasters", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_PLATFORM | GBF_ADV, 0,
	SpectrumGetZipName, SpecMurraymousesupercopRomInfo, SpecMurraymousesupercopRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Necris Dome (48K)

static struct BurnRomInfo SpecNecrisdomeRomDesc[] = {
	{ "Necris Dome (1987)(Codemasters).tap", 48602, 0x731d02fb, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecNecrisdome, SpecNecrisdome, Spectrum)
STD_ROM_FN(SpecNecrisdome)

struct BurnDriver BurnSpecNecrisdome = {
	"spec_necrisdome", NULL, "spec_spectrum", NULL, "1987",
	"Necris Dome (48K)\0", NULL, "Codemasters", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_ADV, 0,
	SpectrumGetZipName, SpecNecrisdomeRomInfo, SpecNecrisdomeRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Ninja Massacre (128K)

static struct BurnRomInfo SpecNinjamassacreRomDesc[] = {
	{ "Ninja Massacre (1989)(Codemasters).tap", 59148, 0x2b28000a, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecNinjamassacre, SpecNinjamassacre, Spec128)
STD_ROM_FN(SpecNinjamassacre)

struct BurnDriver BurnSpecNinjamassacre = {
	"spec_ninjamassacre", NULL, "spec_spec128", NULL, "1989",
	"Ninja Massacre (128K)\0", NULL, "Codemasters", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 2, HARDWARE_SPECTRUM, GBF_RUNGUN | GBF_MAZE, 0,
	SpectrumGetZipName, SpecNinjamassacreRomInfo, SpecNinjamassacreRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Operation Gunship (128K)

static struct BurnRomInfo SpecOperationgunshipRomDesc[] = {
	{ "Operation Gunship (1989)(Codemasters).tap", 52455, 0xee1f687d, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecOperationgunship, SpecOperationgunship, Spec128)
STD_ROM_FN(SpecOperationgunship)

struct BurnDriver BurnSpecOperationgunship = {
	"spec_operationgunship", NULL, "spec_spec128", NULL, "1989",
	"Operation Gunship (128K)\0", NULL, "Codemasters", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_SHOOT, 0,
	SpectrumGetZipName, SpecOperationgunshipRomInfo, SpecOperationgunshipRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Paris To Dakar (128K)

static struct BurnRomInfo SpecParistodakarRomDesc[] = {
	{ "Paris To Dakar (1991)(Codemasters).tap", 73017, 0xcb4a5edc, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecParistodakar, SpecParistodakar, Spec128)
STD_ROM_FN(SpecParistodakar)

struct BurnDriver BurnSpecParistodakar = {
	"spec_paristodakar", NULL, "spec_spec128", NULL, "1991",
	"Paris To Dakar (128K)\0", NULL, "Codemasters", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_RACING, 0,
	SpectrumGetZipName, SpecParistodakarRomInfo, SpecParistodakarRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Prince Clumsy (48K)

static struct BurnRomInfo SpecPrinceclumsyRomDesc[] = {
	{ "Prince Clumsy (1990)(Codemasters).z80", 36725, 0xc5ab98f3, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecPrinceclumsy, SpecPrinceclumsy, Spectrum)
STD_ROM_FN(SpecPrinceclumsy)

struct BurnDriver BurnSpecPrinceclumsy = {
	"spec_princeclumsy", NULL, "spec_spectrum", NULL, "1990",
	"Prince Clumsy (48K)\0", NULL, "Codemasters", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_RUNGUN, 0,
	SpectrumGetZipName, SpecPrinceclumsyRomInfo, SpecPrinceclumsyRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Professional BMX Simulator - Expert (128K)

static struct BurnRomInfo SpecProbmxsimexpertRomDesc[] = {
	{ "Professional BMX Simulator - Expert (1988)(Codemasters).tap", 81052, 0x5f895fc1, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecProbmxsimexpert, SpecProbmxsimexpert, Spec128)
STD_ROM_FN(SpecProbmxsimexpert)

struct BurnDriver BurnSpecProbmxsimexpert = {
	"spec_probmxsimexpert", "spec_probmxsimstandard", "spec_spec128", NULL, "1988",
	"Professional BMX Simulator - Expert (128K)\0", NULL, "Codemasters", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE, 4, HARDWARE_SPECTRUM, GBF_RACING, 0,
	SpectrumGetZipName, SpecProbmxsimexpertRomInfo, SpecProbmxsimexpertRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Professional BMX Simulator - Standard (128K)

static struct BurnRomInfo SpecProbmxsimstandardRomDesc[] = {
	{ "Professional BMX Simulator - Standard (1988)(Codemasters).tap", 81052, 0x7468e20b, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecProbmxsimstandard, SpecProbmxsimstandard, Spec128)
STD_ROM_FN(SpecProbmxsimstandard)

struct BurnDriver BurnSpecProbmxsimstandard = {
	"spec_probmxsimstandard", NULL, "spec_spec128", NULL, "1988",
	"Professional BMX Simulator - Standard (128K)\0", NULL, "Codemasters", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 4, HARDWARE_SPECTRUM, GBF_RACING, 0,
	SpectrumGetZipName, SpecProbmxsimstandardRomInfo, SpecProbmxsimstandardRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Pro Golf Simulator (128K)

static struct BurnRomInfo SpecProgolfsimRomDesc[] = {
	{ "Pro Golf Simulator (1990)(Codemasters).tap", 48603, 0x6e5bed28, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecProgolfsim, SpecProgolfsim, Spec128)
STD_ROM_FN(SpecProgolfsim)

struct BurnDriver BurnSpecProgolfsim = {
	"spec_progolfsim", NULL, "spec_spec128", NULL, "1990",
	"Pro Golf Simulator (128K)\0", NULL, "Codemasters", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_SPORTSMISC, 0,
	SpectrumGetZipName, SpecProgolfsimRomInfo, SpecProgolfsimRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Pro PowerBoat Simulator (128K)

static struct BurnRomInfo SpecPropowerboatsimRomDesc[] = {
	{ "Pro PowerBoat Simulator (1989)(Codemasters).tap", 49394, 0x5627c3f7, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecPropowerboatsim, SpecPropowerboatsim, Spec128)
STD_ROM_FN(SpecPropowerboatsim)

struct BurnDriver BurnSpecPropowerboatsim = {
	"spec_propowerboatsim", NULL, "spec_spec128", NULL, "1989",
	"Pro PowerBoat Simulator (128K)\0", NULL, "Codemasters", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 2, HARDWARE_SPECTRUM, GBF_ACTION, 0,
	SpectrumGetZipName, SpecPropowerboatsimRomInfo, SpecPropowerboatsimRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Pro Skateboard Simulator (48K)

static struct BurnRomInfo SpecProskateboardsimRomDesc[] = {
	{ "Pro Skateboard Simulator 48K (1989)(Codemasters).tap", 56277, 0xf3a7d031, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecProskateboardsim, SpecProskateboardsim, Spectrum)
STD_ROM_FN(SpecProskateboardsim)

struct BurnDriver BurnSpecProskateboardsim = {
	"spec_proskateboardsim", NULL, "spec_spectrum", NULL, "1989",
	"Pro Skateboard Simulator (48K)\0", NULL, "Codemasters", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 2, HARDWARE_SPECTRUM, GBF_SPORTSMISC, 0,
	SpectrumGetZipName, SpecProskateboardsimRomInfo, SpecProskateboardsimRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Professional Ski Simulator (48K)

static struct BurnRomInfo SpecProskisimRomDesc[] = {
	{ "Professional Ski Simulator (1987)(Codemasters).tap", 48913, 0xce33d499, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecProskisim, SpecProskisim, Spectrum)
STD_ROM_FN(SpecProskisim)

struct BurnDriver BurnSpecProskisim = {
	"spec_proskisim", NULL, "spec_spectrum", NULL, "1987",
	"Professional Ski Simulator (48K)\0", NULL, "Codemasters", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 2, HARDWARE_SPECTRUM, GBF_ACTION, 0,
	SpectrumGetZipName, SpecProskisimRomInfo, SpecProskisimRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Professional Snooker Simulator (48K)

static struct BurnRomInfo SpecProsnookersimRomDesc[] = {
	{ "Professional Snooker Simulator (1987)(Codemasters).tap", 22922, 0xc8b22bca, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecProsnookersim, SpecProsnookersim, Spectrum)
STD_ROM_FN(SpecProsnookersim)

struct BurnDriver BurnSpecProsnookersim = {
	"spec_prosnookersim", NULL, "spec_spectrum", NULL, "1987",
	"Professional Snooker Simulator (48K)\0", "Q, W, E, SPACE and ENTER to play", "Codemasters", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 2, HARDWARE_SPECTRUM, GBF_SPORTSMISC, 0,
	SpectrumGetZipName, SpecProsnookersimRomInfo, SpecProsnookersimRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecCursorKeysDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Pro Tennis Simulator (128K)

static struct BurnRomInfo SpecProtennissimRomDesc[] = {
	{ "Pro Tennis Simulator (1990)(Codemasters).z80", 27039, 0x9e79b85c, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecProtennissim, SpecProtennissim, Spec128)
STD_ROM_FN(SpecProtennissim)

struct BurnDriver BurnSpecProtennissim = {
	"spec_protennissim", NULL, "spec_spec128", NULL, "1990",
	"Pro Tennis Simulator (128K)\0", NULL, "Codemasters", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 2, HARDWARE_SPECTRUM, GBF_SPORTSMISC, 0,
	SpectrumGetZipName, SpecProtennissimRomInfo, SpecProtennissimRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Pub Trivia (128K)

static struct BurnRomInfo SpecPubtriviaRomDesc[] = {
	{ "Pub Trivia (1989)(Codemasters).z80", 100743, 0xbafbf528, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecPubtrivia, SpecPubtrivia, Spec128)
STD_ROM_FN(SpecPubtrivia)

struct BurnDriver BurnSpecPubtrivia = {
	"spec_pubtrivia", NULL, "spec_spec128", NULL, "1989",
	"Pub Trivia (128K)\0", NULL, "Codemasters", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 4, HARDWARE_SPECTRUM, GBF_QUIZ, 0,
	SpectrumGetZipName, SpecPubtriviaRomInfo, SpecPubtriviaRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Race Against Time, The (48K)

static struct BurnRomInfo SpecRaceagainsttimeRomDesc[] = {
	{ "Race Against Time, The (1988)(Codemasters).tap", 48905, 0xeec9a042, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecRaceagainsttime, SpecRaceagainsttime, Spectrum)
STD_ROM_FN(SpecRaceagainsttime)

struct BurnDriver BurnSpecRaceagainsttime = {
	"spec_raceagainsttime", NULL, "spec_spectrum", NULL, "1988",
	"Race Against Time, The (48K)\0", NULL, "Codemasters", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecRaceagainsttimeRomInfo, SpecRaceagainsttimeRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecIntf2DIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Rally Cross Simulator (128K)

static struct BurnRomInfo SpecRallycrosssimRomDesc[] = {
	{ "Rally Cross Simulator (1990)(Codemasters).tap", 111041, 0xa3f861ee, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecRallycrosssim, SpecRallycrosssim, Spec128)
STD_ROM_FN(SpecRallycrosssim)

struct BurnDriver BurnSpecRallycrosssim = {
	"spec_rallycrosssim", NULL, "spec_spec128", NULL, "1990",
	"Rally Cross Simulator (128K)\0", NULL, "Codemasters", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_RACING, 0,
	SpectrumGetZipName, SpecRallycrosssimRomInfo, SpecRallycrosssimRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Robin Hood - Legend Quest (128K)

static struct BurnRomInfo SpecRobinhoodlegendquestRomDesc[] = {
	{ "Robin Hood - Legend Quest (1993)(Codemasters).tap", 79970, 0x548b209e, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecRobinhoodlegendquest, SpecRobinhoodlegendquest, Spec128)
STD_ROM_FN(SpecRobinhoodlegendquest)

struct BurnDriver BurnSpecRobinhoodlegendquest = {
	"spec_robinhoodlegendquest", NULL, "spec_spec128", NULL, "1993",
	"Robin Hood - Legend Quest (128K)\0", NULL, "Codemasters", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_PLATFORM | GBF_ADV, 0,
	SpectrumGetZipName, SpecRobinhoodlegendquestRomInfo, SpecRobinhoodlegendquestRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Rock Star Goes Bizarre (128K)

static struct BurnRomInfo SpecRockstargoesbizarreRomDesc[] = {
	{ "Rock Star Goes Bizarre (1989)(Codemasters).tap", 68929, 0x4dc0ff5a, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecRockstargoesbizarre, SpecRockstargoesbizarre, Spec128)
STD_ROM_FN(SpecRockstargoesbizarre)

struct BurnDriver BurnSpecRockstargoesbizarre = {
	"spec_rockstargoesbizarre", NULL, "spec_spec128", NULL, "1989",
	"Rock Star Goes Bizarre (128K)\0", NULL, "Codemasters", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_STRATEGY, 0,
	SpectrumGetZipName, SpecRockstargoesbizarreRomInfo, SpecRockstargoesbizarreRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Rogue Trooper (48K)

static struct BurnRomInfo SpecRoguetrooperRomDesc[] = {
	{ "Rogue Trooper (1986)(Piranha).tap", 55832, 0x6e1e885e, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecRoguetrooper, SpecRoguetrooper, Spectrum)
STD_ROM_FN(SpecRoguetrooper)

struct BurnDriver BurnSpecRoguetrooper = {
	"spec_roguetrooper", NULL, "spec_spectrum", NULL, "1986",
	"Rogue Trooper (48K)\0", NULL, "Piranha", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_RUNGUN | GBF_ADV, 0,
	SpectrumGetZipName, SpecRoguetrooperRomInfo, SpecRoguetrooperRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// S.A.S. Combat Simulator (48K)

static struct BurnRomInfo SpecSascombatsimRomDesc[] = {
	{ "S.A.S. Combat Simulator (1988)(Codemasters).tap", 42914, 0x6324df81, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecSascombatsim, SpecSascombatsim, Spectrum)
STD_ROM_FN(SpecSascombatsim)

struct BurnDriver BurnSpecSascombatsim = {
	"spec_sascombatsim", NULL, "spec_spectrum", NULL, "1988",
	"S.A.S. Combat Simulator (48K)\0", NULL, "Codemasters", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_RUNGUN, 0,
	SpectrumGetZipName, SpecSascombatsimRomInfo, SpecSascombatsimRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// S.A.S. Operation Thunderflash (48K)

static struct BurnRomInfo SpecSasoperationthunderflashRomDesc[] = {
	{ "S.A.S. Operation Thunderflash (1986)(Sparklers).tap", 48345, 0x7013a78b, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecSasoperationthunderflash, SpecSasoperationthunderflash, Spectrum)
STD_ROM_FN(SpecSasoperationthunderflash)

struct BurnDriver BurnSpecSasoperationthunderflash = {
	"spec_sasoperationthunderflash", NULL, "spec_spectrum", NULL, "1986",
	"S.A.S. Operation Thunderflash (48K)\0", NULL, "Sparklers", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_RUNGUN, 0,
	SpectrumGetZipName, SpecSasoperationthunderflashRomInfo, SpecSasoperationthunderflashRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Scooby and Scrappy-Doo (48K)

static struct BurnRomInfo SpecSsdooRomDesc[] = {
	{ "Scooby and Scrappy-Doo (1991)(Hi-Tec).tap", 47868, 0x5e04acd0, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecSsdoo, SpecSsdoo, Spectrum)
STD_ROM_FN(SpecSsdoo)

struct BurnDriver BurnSpecSsdoo = {
	"spec_ssdoo", NULL, "spec_spectrum", NULL, "1991",
	"Scooby and Scrappy-Doo (48K)\0", NULL, "Hi-Tec Software", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecSsdooRomInfo, SpecSsdooRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Sergeant Seymour Robotcop (128K)

static struct BurnRomInfo SpecSergeantseymourrobotcopRomDesc[] = {
	{ "Sergeant Seymour Robotcop (1992)(Codemasters).tap", 35548, 0x2ecf649d, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecSergeantseymourrobotcop, SpecSergeantseymourrobotcop, Spec128)
STD_ROM_FN(SpecSergeantseymourrobotcop)

struct BurnDriver BurnSpecSergeantseymourrobotcop = {
	"spec_sergeantseymourrobotcop", NULL, "spec_spec128", NULL, "1992",
	"Sergeant Seymour Robotcop (128K)\0", NULL, "Codemasters", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_ACTION, 0,
	SpectrumGetZipName, SpecSergeantseymourrobotcopRomInfo, SpecSergeantseymourrobotcopRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Sky High Stuntman (128K)

static struct BurnRomInfo SpecSkyhighstuntmanRomDesc[] = {
	{ "Sky High Stuntman (1991)(Codemasters).tap", 50525, 0x45b71f16, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecSkyhighstuntman, SpecSkyhighstuntman, Spec128)
STD_ROM_FN(SpecSkyhighstuntman)

struct BurnDriver BurnSpecSkyhighstuntman = {
	"spec_skyhighstuntman", NULL, "spec_spec128", NULL, "1991",
	"Sky High Stuntman (128K)\0", NULL, "Codemasters", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 2, HARDWARE_SPECTRUM, GBF_VERSHOOT, 0,
	SpectrumGetZipName, SpecSkyhighstuntmanRomInfo, SpecSkyhighstuntmanRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Soccer Pinball (128K)

static struct BurnRomInfo SpecSoccerpinballRomDesc[] = {
	{ "Soccer Pinball (1992)(Codemasters).tap", 61677, 0xc1c3e3b6, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecSoccerpinball, SpecSoccerpinball, Spec128)
STD_ROM_FN(SpecSoccerpinball)

struct BurnDriver BurnSpecSoccerpinball = {
	"spec_soccerpinball", NULL, "spec_spec128", NULL, "1992",
	"Soccer Pinball (128K)\0", "Z = left flipper, M = right flipper, Space = shoot", "Codemasters", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 2, HARDWARE_SPECTRUM, GBF_PINBALL, 0,
	SpectrumGetZipName, SpecSoccerpinballRomInfo, SpecSoccerpinballRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Spike in Transylvania (128K)

static struct BurnRomInfo SpecSpikeintransylvaniaRomDesc[] = {
	{ "Spike in Transylvania (1991)(Codemasters).tap", 54232, 0x2e3cf5ca, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecSpikeintransylvania, SpecSpikeintransylvania, Spec128)
STD_ROM_FN(SpecSpikeintransylvania)

struct BurnDriver BurnSpecSpikeintransylvania = {
	"spec_spikeintransylvania", NULL, "spec_spec128", NULL, "1991",
	"Spike in Transylvania (128K)\0", NULL, "Codemasters", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_ADV, 0,
	SpectrumGetZipName, SpecSpikeintransylvaniaRomInfo, SpecSpikeintransylvaniaRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Star Dragon (128K)

static struct BurnRomInfo SpecStardragonRomDesc[] = {
	{ "Star Dragon (1991)(Ultrasoft).tap", 34469, 0xf7a81a91, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecStardragon, SpecStardragon, Spec128)
STD_ROM_FN(SpecStardragon)

struct BurnDriver BurnSpecStardragon = {
	"spec_stardragon", NULL, "spec_spec128", NULL, "1991",
	"Star Dragon (128K)\0", NULL, "Ultrasoft", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_VERSHOOT, 0,
	SpectrumGetZipName, SpecStardragonRomInfo, SpecStardragonRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Star Paws (128K)

static struct BurnRomInfo SpecStarpawsRomDesc[] = {
	{ "Star Paws (1988)(Software Projects).z80", 50264, 0x14d2c355, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecStarpaws, SpecStarpaws, Spec128)
STD_ROM_FN(SpecStarpaws)

struct BurnDriver BurnSpecStarpaws = {
	"spec_starpaws", NULL, "spec_spec128", NULL, "1988",
	"Star Paws (128K)\0", NULL, "Software Projects", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_ACTION, 0,
	SpectrumGetZipName, SpecStarpawsRomInfo, SpecStarpawsRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Star Runner (48K)

static struct BurnRomInfo SpecStarrunnerRomDesc[] = {
	{ "Star Runner (1987)(Codemasters).tap", 56277, 0xfbde3432, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecStarrunner, SpecStarrunner, Spectrum)
STD_ROM_FN(SpecStarrunner)

struct BurnDriver BurnSpecStarrunner = {
	"spec_starrunner", NULL, "spec_spectrum", NULL, "1987",
	"Star Runner (48K)\0", NULL, "Codemasters", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_ACTION, 0,
	SpectrumGetZipName, SpecStarrunnerRomInfo, SpecStarrunnerRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Street Gang Football (48K)

static struct BurnRomInfo SpecStreetgangfootballRomDesc[] = {
	{ "Street Gang Football (1989)(Codemasters).tap", 43537, 0x580c88d2, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecStreetgangfootball, SpecStreetgangfootball, Spectrum)
STD_ROM_FN(SpecStreetgangfootball)

struct BurnDriver BurnSpecStreetgangfootball = {
	"spec_streetgangfootball", NULL, "spec_spectrum", NULL, "1989",
	"Street Gang Football (48K)\0", NULL, "Codemasters", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 2, HARDWARE_SPECTRUM, GBF_SPORTSFOOTBALL, 0,
	SpectrumGetZipName, SpecStreetgangfootballRomInfo, SpecStreetgangfootballRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Superbike Trans-Am (128K)

static struct BurnRomInfo SpecSuperbiketransamRomDesc[] = {
	{ "Superbike Trans-Am (1989)(Codemasters).z80", 40870, 0x28322d99, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecSuperbiketransam, SpecSuperbiketransam, Spec128)
STD_ROM_FN(SpecSuperbiketransam)

struct BurnDriver BurnSpecSuperbiketransam = {
	"spec_superbiketransam", NULL, "spec_spec128", NULL, "1989",
	"Superbike Trans-Am (128K)\0", NULL, "Codemasters", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_RACING, 0,
	SpectrumGetZipName, SpecSuperbiketransamRomInfo, SpecSuperbiketransamRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecQAOPSpaceDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Super Dragon Slayer (128K)

static struct BurnRomInfo SpecSuperdragonslayerRomDesc[] = {
	{ "Super Dragon Slayer (1989)(Codemasters).tap", 42607, 0x95581414, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecSuperdragonslayer, SpecSuperdragonslayer, Spec128)
STD_ROM_FN(SpecSuperdragonslayer)

struct BurnDriver BurnSpecSuperdragonslayer = {
	"spec_superdragonslayer", NULL, "spec_spec128", NULL, "1989",
	"Super Dragon Slayer (128K)\0", NULL, "Codemasters", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecSuperdragonslayerRomInfo, SpecSuperdragonslayerRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Super G-Man (48K)

static struct BurnRomInfo SpecSupergmanRomDesc[] = {
	{ "Super G-Man (1987)(Codemasters).tap", 44091, 0x4e83dd70, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecSupergman, SpecSupergman, Spectrum)
STD_ROM_FN(SpecSupergman)

struct BurnDriver BurnSpecSupergman = {
	"spec_supergman", NULL, "spec_spectrum", NULL, "1987",
	"Super G-Man (48K)\0", NULL, "Codemasters", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_HORSHOOT, 0,
	SpectrumGetZipName, SpecSupergmanRomInfo, SpecSupergmanRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Super Hero (48K)

static struct BurnRomInfo SpecSuperheroRomDesc[] = {
	{ "Super Hero (1988)(Codemasters).tap", 51573, 0x26d19439, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecSuperhero, SpecSuperhero, Spectrum)
STD_ROM_FN(SpecSuperhero)

struct BurnDriver BurnSpecSuperhero = {
	"spec_superhero", NULL, "spec_spectrum", NULL, "1988",
	"Super Hero (48K)\0", NULL, "Codemasters", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_ADV, 0,
	SpectrumGetZipName, SpecSuperheroRomInfo, SpecSuperheroRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Super Robin Hood (48K)

static struct BurnRomInfo SpecSuperrobinhoodRomDesc[] = {
	{ "Super Robin Hood (1987)(Codemasters).tap", 47900, 0x61d90998, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecSuperrobinhood, SpecSuperrobinhood, Spectrum)
STD_ROM_FN(SpecSuperrobinhood)

struct BurnDriver BurnSpecSuperrobinhood = {
	"spec_superrobinhood", NULL, "spec_spectrum", NULL, "1987",
	"Super Robin Hood (48K)\0", NULL, "Codemasters", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecSuperrobinhoodRomInfo, SpecSuperrobinhoodRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Super Scramble Simulator (128K)

static struct BurnRomInfo SpecSuperscramblesimRomDesc[] = {
	{ "Super Scramble Simulator (1989)(Gremlin Graphics).tap", 56996, 0x1454d1d7, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecSuperscramblesim, SpecSuperscramblesim, Spec128)
STD_ROM_FN(SpecSuperscramblesim)

struct BurnDriver BurnSpecSuperscramblesim = {
	"spec_superscramblesim", NULL, "spec_spec128", NULL, "1989",
	"Super Scramble Simulator (128K)\0", NULL, "Gremlin Graphics", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_RACING, 0,
	SpectrumGetZipName, SpecSuperscramblesimRomInfo, SpecSuperscramblesimRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Super Seymour Saves the Planet (128K)

static struct BurnRomInfo SpecSuperseymoursavestheplanetRomDesc[] = {
	{ "Super Seymour Saves the Planet (1991)(Codemasters).tap", 65869, 0xd422deed, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecSuperseymoursavestheplanet, SpecSuperseymoursavestheplanet, Spec128)
STD_ROM_FN(SpecSuperseymoursavestheplanet)

struct BurnDriver BurnSpecSuperseymoursavestheplanet = {
	"spec_superseymoursavestheplanet", NULL, "spec_spec128", NULL, "1991",
	"Super Seymour Saves the Planet (128K)\0", NULL, "Codemasters", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_ACTION, 0,
	SpectrumGetZipName, SpecSuperseymoursavestheplanetRomInfo, SpecSuperseymoursavestheplanetRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Super Stuntman (48K)

static struct BurnRomInfo SpecSuperstuntmanRomDesc[] = {
	{ "Super Stuntman (1988)(Codemasters).tap", 43549, 0x984b4f12, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecSuperstuntman, SpecSuperstuntman, Spectrum)
STD_ROM_FN(SpecSuperstuntman)

struct BurnDriver BurnSpecSuperstuntman = {
	"spec_superstuntman", NULL, "spec_spectrum", NULL, "1988",
	"Super Stuntman (48K)\0", NULL, "Codemasters", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_ACTION, 0,
	SpectrumGetZipName, SpecSuperstuntmanRomInfo, SpecSuperstuntmanRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Super Tank Simulator (128K)

static struct BurnRomInfo SpecSupertanksimRomDesc[] = {
	{ "Super Tank Simulator (1989)(Codemasters).tap", 44848, 0xd88ed48a, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecSupertanksim, SpecSupertanksim, Spec128)
STD_ROM_FN(SpecSupertanksim)

struct BurnDriver BurnSpecSupertanksim = {
	"spec_supertanksim", NULL, "spec_spec128", NULL, "1989",
	"Super Tank Simulator (128K)\0", NULL, "Codemasters", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_RUNGUN, 0,
	SpectrumGetZipName, SpecSupertanksimRomInfo, SpecSupertanksimRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Tarzan (128K)

static struct BurnRomInfo SpecTarzanRomDesc[] = {
	{ "Tarzan (1986)(Martech Games).tap", 56277, 0xe95f4ac1, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecTarzan, SpecTarzan, Spec128)
STD_ROM_FN(SpecTarzan)

struct BurnDriver BurnSpecTarzan = {
	"spec_tarzan", NULL, "spec_spec128", NULL, "1986",
	"Tarzan (128K)\0", NULL, "Martech Games", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_ADV, 0,
	SpectrumGetZipName, SpecTarzanRomInfo, SpecTarzanRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Tarzan Goes Ape (48K)

static struct BurnRomInfo SpecTarzangoesapeRomDesc[] = {
	{ "Tarzan Goes Ape (1991)(Codemasters).tap", 47470, 0x558dadb9, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecTarzangoesape, SpecTarzangoesape, Spectrum)
STD_ROM_FN(SpecTarzangoesape)

struct BurnDriver BurnSpecTarzangoesape = {
	"spec_tarzangoesape", NULL, "spec_spectrum", NULL, "1991",
	"Tarzan Goes Ape (48K)\0", NULL, "Codemasters", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecTarzangoesapeRomInfo, SpecTarzangoesapeRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Terra Cognita - The Hollow Moon (48K)

static struct BurnRomInfo SpecTerracognitaRomDesc[] = {
	{ "Terra Cognita - The Hollow Moon (1986)(Codemasters).tap", 39094, 0x52ef2dd2, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecTerracognita, SpecTerracognita, Spectrum)
STD_ROM_FN(SpecTerracognita)

struct BurnDriver BurnSpecTerracognita = {
	"spec_terracognita", NULL, "spec_spectrum", NULL, "1986",
	"Terra Cognita - The Hollow Moon (48K)\0", NULL, "Codemasters", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_VERSHOOT, 0,
	SpectrumGetZipName, SpecTerracognitaRomInfo, SpecTerracognitaRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Tilt (128K)

static struct BurnRomInfo SpecTiltRomDesc[] = {
	{ "Tilt (1990)(Codemasters).tap", 39893, 0xa03922c0, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecTilt, SpecTilt, Spec128)
STD_ROM_FN(SpecTilt)

struct BurnDriver BurnSpecTilt = {
	"spec_tilt", NULL, "spec_spec128", NULL, "1990",
	"Tilt (128K)\0", NULL, "Codemasters", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_ACTION, 0,
	SpectrumGetZipName, SpecTiltRomInfo, SpecTiltRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecIntf2DIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Tornado ECR (48K)

static struct BurnRomInfo SpecTornadoecrRomDesc[] = {
	{ "Tornado ECR (1991)(Codemasters).tap", 47225, 0x7fe902cb, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecTornadoecr, SpecTornadoecr, Spectrum)
STD_ROM_FN(SpecTornadoecr)

struct BurnDriver BurnSpecTornadoecr = {
	"spec_tornadoecr", NULL, "spec_spectrum", NULL, "1991",
	"Tornado ECR (48K)\0", NULL, "Codemasters", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_HORSHOOT, 0,
	SpectrumGetZipName, SpecTornadoecrRomInfo, SpecTornadoecrRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Transmuter (48K)

static struct BurnRomInfo SpecTransmuterRomDesc[] = {
	{ "Transmuter (1987)(Codemasters).tap", 48599, 0x626052a5, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecTransmuter, SpecTransmuter, Spectrum)
STD_ROM_FN(SpecTransmuter)

struct BurnDriver BurnSpecTransmuter = {
	"spec_transmuter", NULL, "spec_spectrum", NULL, "1987",
	"Transmuter (48K)\0", NULL, "Codemasters", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_HORSHOOT, 0,
	SpectrumGetZipName, SpecTransmuterRomInfo, SpecTransmuterRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Turbo Girl (48K)

static struct BurnRomInfo SpecTurbogirlRomDesc[] = {
	{ "Turbo Girl (1988)(Dinamic).tap", 48800, 0x358aaac4, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecTurbogirl, SpecTurbogirl, Spectrum)
STD_ROM_FN(SpecTurbogirl)

struct BurnDriver BurnSpecTurbogirl = {
	"spec_turbogirl", NULL, "spec_spectrum", NULL, "1988",
	"Turbo Girl (48K)\0", "Press SPACE to jump", "Dinamic Software", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_VERSHOOT, 0,
	SpectrumGetZipName, SpecTurbogirlRomInfo, SpecTurbogirlRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Turbo the Tortoise (128K)

static struct BurnRomInfo SpecTurbothetortoiseRomDesc[] = {
	{ "Turbo the Tortoise (1992)(Hi-Tec Software).tap", 74768, 0x55513db0, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecTurbothetortoise, SpecTurbothetortoise, Spec128)
STD_ROM_FN(SpecTurbothetortoise)

struct BurnDriver BurnSpecTurbothetortoise = {
	"spec_turbothetortoise", NULL, "spec_spec128", NULL, "1992",
	"Turbo the Tortoise (128K)\0", NULL, "Hi-Tec Software", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecTurbothetortoiseRomInfo, SpecTurbothetortoiseRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Twin Turbo V8 (128K)

static struct BurnRomInfo SpecTwinturbov8RomDesc[] = {
	{ "Twin Turbo V8 (1989)(Codemasters).tap", 49133, 0x98b8d8ff, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecTwinturbov8, SpecTwinturbov8, Spec128)
STD_ROM_FN(SpecTwinturbov8)

struct BurnDriver BurnSpecTwinturbov8 = {
	"spec_twinturbov8", NULL, "spec_spec128", NULL, "1989",
	"Twin Turbo V8 (128K)\0", NULL, "Codemasters", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_RACING, 0,
	SpectrumGetZipName, SpecTwinturbov8RomInfo, SpecTwinturbov8RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecQAOPSpaceDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// White Heat (48K)

static struct BurnRomInfo SpecWhiteheatRomDesc[] = {
	{ "White Heat (1987)(Codemasters).tap", 35611, 0x37f5048a, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecWhiteheat, SpecWhiteheat, Spectrum)
STD_ROM_FN(SpecWhiteheat)

struct BurnDriver BurnSpecWhiteheat = {
	"spec_whiteheat", NULL, "spec_spectrum", NULL, "1987",
	"White Heat (48K)\0", NULL, "Codemasters", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_SHOOT, 0,
	SpectrumGetZipName, SpecWhiteheatRomInfo, SpecWhiteheatRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Wizard Willy (128K)

static struct BurnRomInfo SpecWizardwillyRomDesc[] = {
	{ "Wizard Willy (1990)(Codemasters).tap", 45968, 0x42cd6c9d, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecWizardwilly, SpecWizardwilly, Spec128)
STD_ROM_FN(SpecWizardwilly)

struct BurnDriver BurnSpecWizardwilly = {
	"spec_wizardwilly", NULL, "spec_spec128", NULL, "1990",
	"Wizard Willy (128K)\0", NULL, "Codemasters", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecWizardwillyRomInfo, SpecWizardwillyRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Wrestling Superstars (128K)

static struct BurnRomInfo SpecWrestlingsuperstarsRomDesc[] = {
	{ "Wrestling Superstars (1993)(Codemasters).z80", 41173, 0x0b21ab8e, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecWrestlingsuperstars, SpecWrestlingsuperstars, Spec128)
STD_ROM_FN(SpecWrestlingsuperstars)

struct BurnDriver BurnSpecWrestlingsuperstars = {
	"spec_wrestlingsuperstars", NULL, "spec_spec128", NULL, "1992",
	"Wrestling Superstars (128K)\0", NULL, "Codemasters", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 2, HARDWARE_SPECTRUM, GBF_VSFIGHT, 0,
	SpectrumGetZipName, SpecWrestlingsuperstarsRomInfo, SpecWrestlingsuperstarsRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// S.T.U.N. Runner (128K)

static struct BurnRomInfo SpecStunrunRomDesc[] = {
	{ "S.T.U.N. Runner 128K (1990)(Domark).tap", 60868, 0x3aec4137, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecStunrun, SpecStunrun, Spec128)
STD_ROM_FN(SpecStunrun)

struct BurnDriver BurnSpecStunrun = {
	"spec_stunrun", NULL, "spec_spec128", NULL, "1990",
	"S.T.U.N. Runner (128K)\0", NULL, "Domark", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_RACING | GBF_SHOOT, 0,
	SpectrumGetZipName, SpecStunrunRomInfo, SpecStunrunRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Starglider (128K)

static struct BurnRomInfo SpecStarglider128kRomDesc[] = {
	{ "Starglider 128K (1986)(Rainbird).tap", 113280, 0x5403a025, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecStarglider128k, SpecStarglider128k, Spec128)
STD_ROM_FN(SpecStarglider128k)

struct BurnDriver BurnSpecStarglider128k = {
	"spec_starglider128k", NULL, "spec_spec128", NULL, "1986",
	"Starglider (128K)\0", NULL, "Rainbird", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_SHOOT | GBF_SIM, 0,
	SpectrumGetZipName, SpecStarglider128kRomInfo, SpecStarglider128kRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Starglider (48K)

static struct BurnRomInfo SpecStarglider48kRomDesc[] = {
	{ "Starglider 48K (1986)(Rainbird).tap", 49104, 0x48258fa3, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecStarglider48k, SpecStarglider48k, Spectrum)
STD_ROM_FN(SpecStarglider48k)

struct BurnDriver BurnSpecStarglider48k = {
	"spec_starglider48k", "spec_starglider128k", "spec_spectrum", NULL, "1986",
	"Starglider (48K)\0", NULL, "Rainbird", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE, 1, HARDWARE_SPECTRUM, GBF_SHOOT | GBF_SIM, 0,
	SpectrumGetZipName, SpecStarglider48kRomInfo, SpecStarglider48kRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Viz - The Computer Game (128K)

static struct BurnRomInfo SpecVizRomDesc[] = {
	{ "Viz - The Computer Game 128K (1991)(Virgin Games).tap", 167642, 0x005482f2, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecViz, SpecViz, Spec128)
STD_ROM_FN(SpecViz)

struct BurnDriver BurnSpecViz = {
	"spec_viz", NULL, "spec_spec128", NULL, "1991",
	"Viz - The Computer Game (128K)\0", NULL, "Virgin Games", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_MINIGAMES, 0,
	SpectrumGetZipName, SpecVizRomInfo, SpecVizRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Sacred Armour of Antiriad, The (48K)

static struct BurnRomInfo SpecAntiriadRomDesc[] = {
	{ "Sacred Armour of Antiriad, The (1986)(Palace).z80", 41012, 0x1abe0529, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecAntiriad, SpecAntiriad, Spectrum)
STD_ROM_FN(SpecAntiriad)

struct BurnDriver BurnSpecAntiriad = {
	"spec_antiriad", NULL, "spec_spectrum", NULL, "1986",
	"Sacred Armour of Antiriad, The (48K)\0", NULL, "Palace Software", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_PLATFORM | GBF_ADV, 0,
	SpectrumGetZipName, SpecAntiriadRomInfo, SpecAntiriadRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Impossible Mission (48K)

static struct BurnRomInfo SpecImpmissionRomDesc[] = {
	{ "Impossible Mission (1985)(U.S. Gold).tap", 54480, 0xf3cddbf9, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecImpmission, SpecImpmission, Spectrum)
STD_ROM_FN(SpecImpmission)

struct BurnDriver BurnSpecImpmission = {
	"spec_impmission", NULL, "spec_spectrum", NULL, "1985",
	"Impossible Mission (48K)\0", NULL, "U.S. Gold", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_PLATFORM | GBF_ADV, 0,
	SpectrumGetZipName, SpecImpmissionRomInfo, SpecImpmissionRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Spherical (48K)

static struct BurnRomInfo SpecSphericalRomDesc[] = {
	{ "Spherical 48K (1989)(Rainbow Arts).tap", 154622, 0xc03fc493, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecSpherical, SpecSpherical, Spectrum)
STD_ROM_FN(SpecSpherical)

struct BurnDriver BurnSpecSpherical = {
	"spec_spherical", NULL, "spec_spectrum", NULL, "1989",
	"Spherical (48K)\0", NULL, "Rainbow Arts", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 2, HARDWARE_SPECTRUM, GBF_PLATFORM | GBF_PUZZLE, 0,
	SpectrumGetZipName, SpecSphericalRomInfo, SpecSphericalRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Barmy Burgers (48K)

static struct BurnRomInfo SpecBarmyburgRomDesc[] = {
	{ "Barmy Burgers (1983)(Blaby Computer Games).z80", 28248, 0x138faacd, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecBarmyburg, SpecBarmyburg, Spectrum)
STD_ROM_FN(SpecBarmyburg)

struct BurnDriver BurnSpecBarmyburg = {
	"spec_barmyburg", NULL, "spec_spectrum", NULL, "1983",
	"Barmy Burgers (48K)\0", NULL, "Blaby Computer Games", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_ACTION, 0,
	SpectrumGetZipName, SpecBarmyburgRomInfo, SpecBarmyburgRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Sorcery (48K)

static struct BurnRomInfo SpecSorceryRomDesc[] = {
	{ "Sorcery (1984)(Virgin Games).tap", 35062, 0xac2adaae, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecSorcery, SpecSorcery, Spectrum)
STD_ROM_FN(SpecSorcery)

struct BurnDriver BurnSpecSorcery = {
	"spec_sorcery", "spec_sorceryplus", "spec_spectrum", NULL, "1984",
	"Sorcery (48K)\0", "Q: left - A: right - 0: jump", "Virgin Games", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecSorceryRomInfo, SpecSorceryRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Magic Carpet (16K)

static struct BurnRomInfo SpecMcarpetRomDesc[] = {
	{ "Magic Carpet 16K (1985)(Mastertronic).z80", 11221, 0x05e22cf1, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecMcarpet, SpecMcarpet, Spectrum)
STD_ROM_FN(SpecMcarpet)

struct BurnDriver BurnSpecMcarpet = {
	"spec_mcarpet", NULL, "spec_spectrum", NULL, "1985",
	"Magic Carpet (16K)\0", NULL, "Mastertronic", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_ACTION, 0,
	SpectrumGetZipName, SpecMcarpetRomInfo, SpecMcarpetRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecIntf2DIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Munsters, The (128K)

static struct BurnRomInfo SpecMunstersRomDesc[] = {
	{ "Munsters, The (1989)(Again Again).tap", 78341, 0x37451ece, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecMunsters, SpecMunsters, Spec128)
STD_ROM_FN(SpecMunsters)

struct BurnDriver BurnSpecMunsters = {
	"spec_munsters", NULL, "spec_spec128", NULL, "1989",
	"Munsters, The (128K)\0", NULL, "Again Again", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_RUNGUN | GBF_ADV, 0,
	SpectrumGetZipName, SpecMunstersRomInfo, SpecMunstersRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Invasion Force (16K)

static struct BurnRomInfo SpecInvforceRomDesc[] = {
	{ "Invasion Force 16K (1982)(Artic Computing).tap", 9216, 0x14bf4579, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecInvforce, SpecInvforce, Spectrum)
STD_ROM_FN(SpecInvforce)

struct BurnDriver BurnSpecInvforce = {
	"spec_invforce", NULL, "spec_spectrum", NULL, "1982",
	"Invasion Force (16K)\0", "Use A, S and F to play", "Artic Computing", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_SHOOT, 0,
	SpectrumGetZipName, SpecInvforceRomInfo, SpecInvforceRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// 007 - Licence to Kill (128K)

static struct BurnRomInfo SpeclicencetokillRomDesc[] = {
	{ "Licence to Kill 128K (1989)(Domark).tap", 48133, 0x44505823, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Speclicencetokill, Speclicencetokill, Spec128)
STD_ROM_FN(Speclicencetokill)

struct BurnDriver BurnSpeclicencetokill = {
	"spec_licencetokill", NULL, "spec_spec128", NULL, "1989",
	"007 - Licence to Kill (128K)\0", NULL, "Domark", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_VERSHOOT, 0,
	SpectrumGetZipName, SpeclicencetokillRomInfo, SpeclicencetokillRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// 007 - Live and Let Die (128K)

static struct BurnRomInfo Spec007liveandletdieRomDesc[] = {
	{ "007 - Live and Let Die 128K (1988)(Domark).tap", 62379, 0x5f21da6b, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Spec007liveandletdie, Spec007liveandletdie, Spec128)
STD_ROM_FN(Spec007liveandletdie)

struct BurnDriver BurnSpec007liveandletdie = {
	"spec_007liveandletdie", NULL, "spec_spec128", NULL, "1988",
	"007 - Live and Let Die (128K)\0", NULL, "Domark", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_SHOOT, 0,
	SpectrumGetZipName, Spec007liveandletdieRomInfo, Spec007liveandletdieRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// 007 - The Living Daylights (48K)

static struct BurnRomInfo SpecThelivingdaylightsRomDesc[] = {
	{ "The Living Daylights 48K (1987)(Domark).tap", 38808, 0x9945b8ca, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecThelivingdaylights, SpecThelivingdaylights, Spectrum)
STD_ROM_FN(SpecThelivingdaylights)

struct BurnDriver BurnSpecThelivingdaylights = {
	"spec_thelivingdaylights", NULL, "spec_spectrum", NULL, "1987",
	"007 - The Living Daylights (48K)\0", NULL, "Domark", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_ACTION | GBF_SHOOT, 0,
	SpectrumGetZipName, SpecThelivingdaylightsRomInfo, SpecThelivingdaylightsRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// 007 - The Spy Who Loved Me (128K)

static struct BurnRomInfo Spec007spywholovedmeRomDesc[] = {
	{ "007 - The Spy Who Loved Me 128K (1990)(Domark).tap", 129851, 0x629c140c, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Spec007spywholovedme, Spec007spywholovedme, Spec128)
STD_ROM_FN(Spec007spywholovedme)

struct BurnDriver BurnSpec007spywholovedme = {
	"spec_007spywholovedme", NULL, "spec_spec128", NULL, "1990",
	"007 - The Spy Who Loved Me (128K)\0", NULL, "Domark", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_RACING | GBF_VERSHOOT, 0,
	SpectrumGetZipName, Spec007spywholovedmeRomInfo, Spec007spywholovedmeRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// 19 Part 1 - Boot Camp (128K)

static struct BurnRomInfo Spec19bootcampRomDesc[] = {
	{ "19 Part 1 - Boot Camp 128K (1988)(Cascade Games).tap", 213402, 0xae022874, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Spec19bootcamp, Spec19bootcamp, Spec128)
STD_ROM_FN(Spec19bootcamp)

struct BurnDriver BurnSpec19bootcamp = {
	"spec_19bootcamp", NULL, "spec_spec128", NULL, "1988",
	"19 Part 1 - Boot Camp (128K)\0", NULL, "Cascade Games", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 4, HARDWARE_SPECTRUM, GBF_ACTION | GBF_SHOOT, 0,
	SpectrumGetZipName, Spec19bootcampRomInfo, Spec19bootcampRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// 1985 - The Day After (48K)

static struct BurnRomInfo Spec1985tdaRomDesc[] = {
	{ "1985 - The Day After (1985)(Mastertronic).tap", 32574, 0x2bee2cb2, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Spec1985tda, Spec1985tda, Spectrum)
STD_ROM_FN(Spec1985tda)

struct BurnDriver BurnSpec1985tda = {
	"spec_1985tda", NULL, "spec_spectrum", NULL, "1985",
	"1985 - The Day After (48K)\0", NULL, "Mastertronic", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_ACTION, 0,
	SpectrumGetZipName, Spec1985tdaRomInfo, Spec1985tdaRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// 2088 (48K)

static struct BurnRomInfo Spec2088RomDesc[] = {
	{ "2088 (1988)(Zeppelin Games).tap", 28306, 0xdecaeb6e, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Spec2088, Spec2088, Spectrum)
STD_ROM_FN(Spec2088)

struct BurnDriver BurnSpec2088 = {
	"spec_2088", NULL, "spec_spectrum", NULL, "1988",
	"2088 (48K)\0", NULL, "Zeppelin Games", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 2, HARDWARE_SPECTRUM, GBF_SHOOT, 0,
	SpectrumGetZipName, Spec2088RomInfo, Spec2088RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// 2112 AD (48K)

static struct BurnRomInfo Spec2112adRomDesc[] = {
	{ "2112 AD (1985)(Design Design).tap", 45602, 0xca5a76f1, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Spec2112ad, Spec2112ad, Spectrum)
STD_ROM_FN(Spec2112ad)

struct BurnDriver BurnSpec2112ad = {
	"spec_2112ad", NULL, "spec_spectrum", NULL, "1985",
	"2112 AD (48K)\0", "Q: up - A: down - N: left - M: right - 0: action", "Design Design", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_ADV, 0,
	SpectrumGetZipName, Spec2112adRomInfo, Spec2112adRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// 21 Erotic (Czech) (48K)

static struct BurnRomInfo Spec21eroticRomDesc[] = {
	{ "21 Erotic (1995)(JardaSoft).tap", 10336, 0x80cb68c0, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Spec21erotic, Spec21erotic, Spectrum)
STD_ROM_FN(Spec21erotic)

struct BurnDriver BurnSpec21erotic = {
	"spec_21erotic", NULL, "spec_spectrum", NULL, "1995",
	"21 Erotic (Czech) (48K)\0", "Adult game", "JardaSoft", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_CARD, 0,
	SpectrumGetZipName, Spec21eroticRomInfo, Spec21eroticRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// 3D Lunattack (48K)

static struct BurnRomInfo SpecLunattackRomDesc[] = {
	{ "3D Lunattack (1984)(Hewson Consultants).z80", 16399, 0x0b58cfda, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecLunattack, SpecLunattack, Spectrum)
STD_ROM_FN(SpecLunattack)

struct BurnDriver BurnSpecLunattack = {
	"spec_lunattack", NULL, "spec_spectrum", NULL, "1984",
	"3D Lunattack (48K)\0", NULL, "Hewson Consultants", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_SHOOT, 0,
	SpectrumGetZipName, SpecLunattackRomInfo, SpecLunattackRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecCursorKeysDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// 3D Maze of Gold (16K)

static struct BurnRomInfo Spec3dmazegoldRomDesc[] = {
	{ "3D Maze of Gold 16K (1982)(Gilsoft International).tap", 7885, 0xc2da7962, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Spec3dmazegold, Spec3dmazegold, Spec128)
STD_ROM_FN(Spec3dmazegold)

struct BurnDriver BurnSpec3dmazegold = {
	"spec_3dmazegold", NULL, "spec_spec128", NULL, "1982",
	"3D Maze of Gold (16K)\0", NULL, "Gilsoft International", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_MAZE, 0,
	SpectrumGetZipName, Spec3dmazegoldRomInfo, Spec3dmazegoldRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecCursorKeysDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// 3D Seiddab Attack (16K)

static struct BurnRomInfo Spec3dseiatkRomDesc[] = {
	{ "3D Seiddab Attack 16K (1984)(Hewson Consultants).tap", 8581, 0x22e97ea1, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Spec3dseiatk, Spec3dseiatk, Spectrum)
STD_ROM_FN(Spec3dseiatk)

struct BurnDriver BurnSpec3dseiatk = {
	"spec_3dseiatk", NULL, "spec_spectrum", NULL, "1984",
	"3D Seiddab Attack (16K)\0", NULL, "Hewson Consultants", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 2, HARDWARE_SPECTRUM, GBF_SHOOT, 0,
	SpectrumGetZipName, Spec3dseiatkRomInfo, Spec3dseiatkRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// 3D Snooker (48K)

static struct BurnRomInfo Spec3dsnookerRomDesc[] = {
	{ "3D Snooker (1990)(Players Premier).z80", 29049, 0x8dfff22d, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Spec3dsnooker, Spec3dsnooker, Spectrum)
STD_ROM_FN(Spec3dsnooker)

struct BurnDriver BurnSpec3dsnooker = {
	"spec_3dsnooker", NULL, "spec_spectrum", NULL, "1990",
	"3D Snooker (48K)\0", NULL, "Players Premier", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 2, HARDWARE_SPECTRUM, GBF_SPORTSMISC, 0,
	SpectrumGetZipName, Spec3dsnookerRomInfo, Spec3dsnookerRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// 3D Starfighter (48K)

static struct BurnRomInfo Spec3dstarfighterRomDesc[] = {
	{ "3D Starfighter (1988)(Codemasters).tap", 48891, 0x4f3f2fbe, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Spec3dstarfighter, Spec3dstarfighter, Spectrum)
STD_ROM_FN(Spec3dstarfighter)

struct BurnDriver BurnSpec3dstarfighter = {
	"spec_3dstarfighter", NULL, "spec_spectrum", NULL, "1988",
	"3D Starfighter (48K)\0", NULL, "Codemasters", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 2, HARDWARE_SPECTRUM, GBF_SHOOT, 0,
	SpectrumGetZipName, Spec3dstarfighterRomInfo, Spec3dstarfighterRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// 3D Starstrike (48K)

static struct BurnRomInfo Spec3dstarstrikeRomDesc[] = {
	{ "3D Starstrike (1984)(Realtime Games).z80", 45116, 0x8bcdf6a3, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Spec3dstarstrike, Spec3dstarstrike, Spectrum)
STD_ROM_FN(Spec3dstarstrike)

struct BurnDriver BurnSpec3dstarstrike = {
	"spec_3dstarstrike", NULL, "spec_spectrum", NULL, "1984",
	"3D Starstrike (48K)\0", NULL, "Realtime Games", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_SHOOT, 0,
	SpectrumGetZipName, Spec3dstarstrikeRomInfo, Spec3dstarstrikeRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// 3D-Tanx (16K)

static struct BurnRomInfo Spec3dtanxRomDesc[] = {
	{ "3D-Tanx 16K (1982)(DK'Tronics).tap", 16185, 0xbae4a458, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Spec3dtanx, Spec3dtanx, Spectrum)
STD_ROM_FN(Spec3dtanx)

struct BurnDriver BurnSpec3dtanx = {
	"spec_3dtanx", NULL, "spec_spectrum", NULL, "1982",
	"3D-Tanx (16K)\0", NULL, "DK'Tronics", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 6, HARDWARE_SPECTRUM, GBF_SHOOT, 0,
	SpectrumGetZipName, Spec3dtanxRomInfo, Spec3dtanxRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Abadia del Crimen, La (Spanish) (128K)

static struct BurnRomInfo SpecAbadcrimRomDesc[] = {
	{ "Abadia del Crimen, La 128K (1988)(Opera Soft).tap", 99182, 0x7bd1ab11, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecAbadcrim, SpecAbadcrim, Spec128)
STD_ROM_FN(SpecAbadcrim)

struct BurnDriver BurnSpecAbadcrim = {
	"spec_abadcrim", NULL, "spec_spec128", NULL, "1988",
	"Abadia del Crimen, La (Spanish) (128K)\0", "To use main character: A, K, L and SPACE", "Opera Soft", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_ADV, 0,
	SpectrumGetZipName, SpecAbadcrimRomInfo, SpecAbadcrimRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// ACE - Air Combat Emulator (48K)

static struct BurnRomInfo SpecAceRomDesc[] = {
	{ "ACE 48K (1986)(Cascade Games).tap", 46187, 0x8568a584, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecAce, SpecAce, Spectrum)
STD_ROM_FN(SpecAce)

struct BurnDriver BurnSpecAce = {
	"spec_ace", NULL, "spec_spectrum", NULL, "1986",
	"ACE - Air Combat Emulator (48K)\0", "LSHIFT-Z: throttle - U: undercarriage - M: map - J: eject", "Cascade Games", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_SIM, 0,
	SpectrumGetZipName, SpecAceRomInfo, SpecAceRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// ACE 2 - The Ultimate Head to Head Conflict (128K)

static struct BurnRomInfo SpecAce2RomDesc[] = {
	{ "ACE 2 The Ultimate Head to Head Conflict 128K (1987)(Cascade Games).tap", 52048, 0x4f850b86, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecAce2, SpecAce2, Spec128)
STD_ROM_FN(SpecAce2)

struct BurnDriver BurnSpecAce2 = {
	"spec_ace2", NULL, "spec_spec128", NULL, "1987",
	"ACE 2 - The Ultimate Head to Head Conflict (128K)\0", "D-S/K-J: throttle - W/O: select weapon - C/B: map", "Cascade Games", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 2, HARDWARE_SPECTRUM, GBF_SIM, 0,
	SpectrumGetZipName, SpecAce2RomInfo, SpecAce2RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecIntf2DIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// ACE 2088 (128K)

static struct BurnRomInfo SpecAce2088RomDesc[] = {
	{ "ACE 2088 128K (1989)(Cascade Games).tap", 55874, 0x4ebb46a9, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecAce2088, SpecAce2088, Spec128)
STD_ROM_FN(SpecAce2088)

struct BurnDriver BurnSpecAce2088 = {
	"spec_ace2088", NULL, "spec_spec128", NULL, "1989",
	"ACE 2088 (128K)\0", "SHIFT+J/K: speed up/down - L, R, T: change weapons - W: set display", "Cascade Games", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_SHOOT | GBF_SIM, 0,
	SpectrumGetZipName, SpecAce2088RomInfo, SpecAce2088RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Action Biker (48K)

static struct BurnRomInfo SpecActionbikerRomDesc[] = {
	{ "Action Biker 48K (1985)(Mastertronic).tap", 63742, 0x1a62419a, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecActionbiker, SpecActionbiker, Spectrum)
STD_ROM_FN(SpecActionbiker)

struct BurnDriver BurnSpecActionbiker = {
	"spec_actionbiker", NULL, "spec_spectrum", NULL, "1985",
	"Action Biker (48K)\0", NULL, "Mastertronic", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_ACTION, 0,
	SpectrumGetZipName, SpecActionbikerRomInfo, SpecActionbikerRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Action Force (48K)

static struct BurnRomInfo SpecActionforceRomDesc[] = {
	{ "Action Force (1987)(Virgin Games).tap", 47303, 0x3ed87428, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecActionforce, SpecActionforce, Spectrum)
STD_ROM_FN(SpecActionforce)

struct BurnDriver BurnSpecActionforce = {
	"spec_actionforce", NULL, "spec_spectrum", NULL, "1987",
	"Action Force (48K)\0", NULL, "Virgin Games", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_SHOOT, 0,
	SpectrumGetZipName, SpecActionforceRomInfo, SpecActionforceRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Action Force II (128K)

static struct BurnRomInfo Specactionforce2RomDesc[] = {
	{ "Action Force II (1988)(Virgin Games).tap", 46813, 0x3d4139b7, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specactionforce2, Specactionforce2, Spec128)
STD_ROM_FN(Specactionforce2)

struct BurnDriver BurnSpecactionforce2 = {
	"spec_actionforce2", NULL, "spec_spec128", NULL, "1988",
	"Action Force II (128K)\0", "On black screen press SPACE to start", "Virgin Games", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_SHOOT, 0,
	SpectrumGetZipName, Specactionforce2RomInfo, Specactionforce2RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecCursorKeysDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Action Reflex (48K)

static struct BurnRomInfo SpecactionreflexRomDesc[] = {
	{ "Action Reflex (1986)(Mirrorsoft).tap", 41597, 0xaada6209, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specactionreflex, Specactionreflex, Spectrum)
STD_ROM_FN(Specactionreflex)

struct BurnDriver BurnSpecactionreflex = {
	"spec_actionreflex", NULL, "spec_spectrum", NULL, "1986",
	"Action Reflex (48K)\0", NULL, "Mirrorsoft", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_ACTION, 0,
	SpectrumGetZipName, SpecactionreflexRomInfo, SpecactionreflexRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Ad Astra (48K)

static struct BurnRomInfo SpecAdastraRomDesc[] = {
	{ "Ad Astra (1984)(Gargoyle Games).tap", 45791, 0xb4ff89d7, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecAdastra, SpecAdastra, Spectrum)
STD_ROM_FN(SpecAdastra)

struct BurnDriver BurnSpecAdastra = {
	"spec_adastra", NULL, "spec_spectrum", NULL, "1984",
	"Ad Astra (48K)\0", NULL, "Gargoyle Games", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 2, HARDWARE_SPECTRUM, GBF_SHOOT, 0,
	SpectrumGetZipName, SpecAdastraRomInfo, SpecAdastraRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Advanced Soccer Simulator (48K)

static struct BurnRomInfo SpecAdvsocsimRomDesc[] = {
	{ "Advanced Soccer Simulator 48K (1989)(Mastertronic Plus).tap", 44780, 0x4a1eaa7b, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecAdvsocsim, SpecAdvsocsim, Spectrum)
STD_ROM_FN(SpecAdvsocsim)

struct BurnDriver BurnSpecAdvsocsim = {
	"spec_advsocsim", NULL, "spec_spectrum", NULL, "1989",
	"Advanced Soccer Simulator (48K)\0", NULL, "Mastertronic Plus", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 8, HARDWARE_SPECTRUM, GBF_SPORTSFOOTBALL | GBF_STRATEGY, 0,
	SpectrumGetZipName, SpecAdvsocsimRomInfo, SpecAdvsocsimRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// After the War - Part 1 (Spanish) (48K)

static struct BurnRomInfo Specafterthewar1RomDesc[] = {
	{ "After the War - Part 1 ES (1989)(Dinamic).tap", 47283, 0x74ef00c2, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specafterthewar1, Specafterthewar1, Spectrum)
STD_ROM_FN(Specafterthewar1)

struct BurnDriver BurnSpecafterthewar1 = {
	"spec_afterthewar1", NULL, "spec_spectrum", NULL, "1989",
	"After the War - Part 1 (Spanish) (48K)\0", NULL, "Dinamic Software", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_SCRFIGHT, 0,
	SpectrumGetZipName, Specafterthewar1RomInfo, Specafterthewar1RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// After the War - Part 2 (Spanish) (48K)

static struct BurnRomInfo Specafterthewar2RomDesc[] = {
	{ "After the War - Part 2 ES (1989)(Dinamic).tap", 47483, 0xd36c48ad, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specafterthewar2, Specafterthewar2, Spectrum)
STD_ROM_FN(Specafterthewar2)

struct BurnDriver BurnSpecafterthewar2 = {
	"spec_afterthewar2", "spec_afterthewar1", "spec_spectrum", NULL, "1989",
	"After the War - Part 2 (Spanish) (48K)\0", "Password: 94656981", "Dinamic Software", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE, 1, HARDWARE_SPECTRUM, GBF_RUNGUN, 0,
	SpectrumGetZipName, Specafterthewar2RomInfo, Specafterthewar2RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Agent X (48K)

static struct BurnRomInfo SpecAgentxRomDesc[] = {
	{ "Agent X 48K (1986)(Mastertronic).tap", 73557, 0xcfac7765, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecAgentx, SpecAgentx, Spectrum)
STD_ROM_FN(SpecAgentx)

struct BurnDriver BurnSpecAgentx = {
	"spec_agentx", NULL, "spec_spectrum", NULL, "1986",
	"Agent X (48K)\0", NULL, "Mastertronic", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_ACTION | GBF_HORSHOOT, 0,
	SpectrumGetZipName, SpecAgentxRomInfo, SpecAgentxRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Agent X II - The Mad Prof's Back! (128K)

static struct BurnRomInfo SpecAgentx2RomDesc[] = {
	{ "Agent X II - The Mad Prof's Back! 128K (1987)(Mastertronic).tap", 144693, 0x889a8e99, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecAgentx2, SpecAgentx2, Spec128)
STD_ROM_FN(SpecAgentx2)

struct BurnDriver BurnSpecAgentx2 = {
	"spec_agentx2", NULL, "spec_spec128", NULL, "1987",
	"Agent X II - The Mad Prof's Back! (128K)\0", NULL, "Mastertronic", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_HORSHOOT | GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecAgentx2RomInfo, SpecAgentx2RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Airbase Invader (48K)

static struct BurnRomInfo SpecAbinvaderRomDesc[] = {
	{ "Airbase Invader 48K (1984)(CP Software).tap", 27746, 0x2f0273cc, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecAbinvader, SpecAbinvader, Spectrum)
STD_ROM_FN(SpecAbinvader)

struct BurnDriver BurnSpecAbinvader = {
	"spec_abinvader", NULL, "spec_spectrum", NULL, "1984",
	"Airbase Invader (48K)\0", NULL, "CP Software", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_ACTION, 0,
	SpectrumGetZipName, SpecAbinvaderRomInfo, SpecAbinvaderRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Airwolf (48K)

static struct BurnRomInfo SpecairwolfRomDesc[] = {
	{ "Airwolf (1985)(Elite Systems).tap", 48924, 0x762ff36d, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specairwolf, Specairwolf, Spectrum)
STD_ROM_FN(Specairwolf)

struct BurnDriver BurnSpecairwolf = {
	"spec_airwolf", NULL, "spec_spectrum", NULL, "1984",
	"Airwolf (48K)\0", NULL, "Elite Systems", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_SHOOT, 0,
	SpectrumGetZipName, SpecairwolfRomInfo, SpecairwolfRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Airwolf II (48K)

static struct BurnRomInfo Specairwolf2RomDesc[] = {
	{ "Airwolf II (1987)(Hit-Pak).tap", 42265, 0x655b28d9, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specairwolf2, Specairwolf2, Spectrum)
STD_ROM_FN(Specairwolf2)

struct BurnDriver BurnSpecairwolf2 = {
	"spec_airwolf2", NULL, "spec_spectrum", NULL, "1987",
	"Airwolf II (48K)\0", NULL, "Hit-Pak", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_HORSHOOT, 0,
	SpectrumGetZipName, Specairwolf2RomInfo, Specairwolf2RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Aknadach (Czech) (48K)

static struct BurnRomInfo SpecAknadachczRomDesc[] = {
	{ "Aknadach CZ 48K (1990)(Proxima).z80", 40314, 0xe221fe1b, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecAknadachcz, SpecAknadachcz, Spectrum)
STD_ROM_FN(SpecAknadachcz)

struct BurnDriver BurnSpecAknadachcz = {
	"spec_aknadachcz", NULL, "spec_spectrum", NULL, "1990",
	"Aknadach (Czech) (48K)\0", NULL, "Proxima", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_PUZZLE, 0,
	SpectrumGetZipName, SpecAknadachczRomInfo, SpecAknadachczRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Aknadach (English) (48K) (Hack)

static struct BurnRomInfo SpecAknadachenRomDesc[] = {
	{ "Aknadach EN 48K (2011)(Softhouse).z80", 40286, 0xf369810f, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecAknadachen, SpecAknadachen, Spectrum)
STD_ROM_FN(SpecAknadachen)

struct BurnDriver BurnSpecAknadachen = {
	"spec_aknadachen", "spec_aknadachcz", "spec_spectrum", NULL, "2011",
	"Aknadach (English) (48K) (Hack)\0", NULL, "Softhouse", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE | BDF_HACK, 1, HARDWARE_SPECTRUM, GBF_PUZZLE, 0,
	SpectrumGetZipName, SpecAknadachenRomInfo, SpecAknadachenRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Alchemist (48K)

static struct BurnRomInfo SpecAlchemistRomDesc[] = {
	{ "Alchemist (1983)(Imagine).z80", 37807, 0xe6f0df00, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecAlchemist, SpecAlchemist, Spectrum)
STD_ROM_FN(SpecAlchemist)

struct BurnDriver BurnSpecAlchemist = {
	"spec_alchemist", NULL, "spec_spectrum", NULL, "1983",
	"Alchemist (48K)\0", NULL, "Imagine Software", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecAlchemistRomInfo, SpecAlchemistRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Alien Highway (48K)

static struct BurnRomInfo SpecAlienhwRomDesc[] = {
	{ "Alien Highway 48K (1986)(Vortex).tap", 48086, 0xf5a22737, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecAlienhw, SpecAlienhw, Spectrum)
STD_ROM_FN(SpecAlienhw)

struct BurnDriver BurnSpecAlienhw = {
	"spec_alienhw", NULL, "spec_spectrum", NULL, "1986",
	"Alien Highway (48K)\0", NULL, "Vortex Software", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_SHOOT, 0,
	SpectrumGetZipName, SpecAlienhwRomInfo, SpecAlienhwRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// All-American Basketball (48K)

static struct BurnRomInfo SpecAabasketRomDesc[] = {
	{ "All-American Basketball 48K (1992)(Zeppelin Games).tap", 47298, 0xa6000491, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecAabasket, SpecAabasket, Spectrum)
STD_ROM_FN(SpecAabasket)

struct BurnDriver BurnSpecAabasket = {
	"spec_aabasket", NULL, "spec_spectrum", NULL, "1992",
	"All-American Basketball (48K)\0", NULL, "Zeppelin Games", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 2, HARDWARE_SPECTRUM, GBF_SPORTSMISC, 0,
	SpectrumGetZipName, SpecAabasketRomInfo, SpecAabasketRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecIntf2DIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Amaurote (128K)

static struct BurnRomInfo SpecAmaurote128RomDesc[] = {
	{ "Amaurote 128K (1987)(Mastertronic Added Dimension).tap", 90861, 0x636c9988, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecAmaurote128, SpecAmaurote128, Spec128)
STD_ROM_FN(SpecAmaurote128)

struct BurnDriver BurnSpecAmaurote128 = {
	"spec_amaurote128", NULL, "spec_spec128", NULL, "1987",
	"Amaurote (128K)\0", "Q/P/A/L: move - Space: fire - Shift: radio menu", "Mastertronic Added Dimension", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_ACTION, 0,
	SpectrumGetZipName, SpecAmaurote128RomInfo, SpecAmaurote128RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Amaurote (48K)

static struct BurnRomInfo SpecAmaurote48RomDesc[] = {
	{ "Amaurote 48K (1987)(Mastertronic Added Dimension).tap", 46026, 0x0fe33d95, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecAmaurote48, SpecAmaurote48, Spectrum)
STD_ROM_FN(SpecAmaurote48)

struct BurnDriver BurnSpecAmaurote48 = {
	"spec_amaurote48", "spec_amaurote128", "spec_spectrum", NULL, "1987",
	"Amaurote (48K)\0", "Q/P/A/L: move - Space: fire - Shift: radio menu", "Mastertronic Added Dimension", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE, 1, HARDWARE_SPECTRUM, GBF_ACTION, 0,
	SpectrumGetZipName, SpecAmaurote48RomInfo, SpecAmaurote48RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// American 3D Pool (48K)

static struct BurnRomInfo SpecAm3dpoolRomDesc[] = {
	{ "American 3D Pool 48K (1992)(Zeppelin Games).tap", 29882, 0x03f6cba6, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecAm3dpool, SpecAm3dpool, Spectrum)
STD_ROM_FN(SpecAm3dpool)

struct BurnDriver BurnSpecAm3dpool = {
	"spec_am3dpool", NULL, "spec_spectrum", NULL, "1992",
	"American 3D Pool (48K)\0", NULL, "Zeppelin Games", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 2, HARDWARE_SPECTRUM, GBF_SPORTSMISC, 0,
	SpectrumGetZipName, SpecAm3dpoolRomInfo, SpecAm3dpoolRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecIntf2DIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// American Tag-Team Wrestling (48K)

static struct BurnRomInfo SpecAttwrestRomDesc[] = {
	{ "American Tag-Team Wrestling 48K (1992)(Zeppelin Games).z80", 38767, 0xafb4c0cc, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecAttwrest, SpecAttwrest, Spectrum)
STD_ROM_FN(SpecAttwrest)

struct BurnDriver BurnSpecAttwrest = {
	"spec_attwrest", NULL, "spec_spectrum", NULL, "1992",
	"American Tag-Team Wrestling (48K)\0", NULL, "Zeppelin Games", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 2, HARDWARE_SPECTRUM, GBF_VSFIGHT, 0,
	SpectrumGetZipName, SpecAttwrestRomInfo, SpecAttwrestRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Anarchy (48K)

static struct BurnRomInfo SpecAnarchyRomDesc[] = {
	{ "Anarchy 48K (1987)(Rack-It).tap", 39327, 0xa48348c3, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecAnarchy, SpecAnarchy, Spectrum)
STD_ROM_FN(SpecAnarchy)

struct BurnDriver BurnSpecAnarchy = {
	"spec_anarchy", NULL, "spec_spectrum", NULL, "1987",
	"Anarchy (48K)\0", NULL, "Rack-It - Hewson", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 2, HARDWARE_SPECTRUM, GBF_SHOOT | GBF_PUZZLE, 0,
	SpectrumGetZipName, SpecAnarchyRomInfo, SpecAnarchyRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Angle Ball (48K)

static struct BurnRomInfo SpecAngleballRomDesc[] = {
	{ "Angle Ball 48K (1987)(Mastertronic Added Dimension).tap", 25307, 0xbd31af2d, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecAngleball, SpecAngleball, Spectrum)
STD_ROM_FN(SpecAngleball)

struct BurnDriver BurnSpecAngleball = {
	"spec_angleball", NULL, "spec_spectrum", NULL, "1987",
	"Angle Ball (48K)\0", NULL, "Mastertronic Added Dimension", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 2, HARDWARE_SPECTRUM, GBF_SPORTSMISC, 0,
	SpectrumGetZipName, SpecAngleballRomInfo, SpecAngleballRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Ant Attack (48K)

static struct BurnRomInfo SpecAntattackRomDesc[] = {
	{ "Ant Attack 48K (1983)(Quicksilva).tap", 41020, 0x91cbb1dd, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecAntattack, SpecAntattack, Spectrum)
STD_ROM_FN(SpecAntattack)

struct BurnDriver BurnSpecAntattack = {
	"spec_antattack", NULL, "spec_spectrum", NULL, "1983",
	"Ant Attack (48K)\0", NULL, "Quicksilva", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_ACTION | GBF_MAZE, 0,
	SpectrumGetZipName, SpecAntattackRomInfo, SpecAntattackRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Aquasquad (48K)

static struct BurnRomInfo SpecAquasquadRomDesc[] = {
	{ "Aquasquad (1988)(Atlantis).tap", 55215, 0x8d07b136, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecAquasquad, SpecAquasquad, Spectrum)
STD_ROM_FN(SpecAquasquad)

struct BurnDriver BurnSpecAquasquad = {
	"spec_aquasquad", NULL, "spec_spectrum", NULL, "1988",
	"Aquasquad (48K)\0", NULL, "Atlantis Software", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_MAZE | GBF_SHOOT, 0,
	SpectrumGetZipName, SpecAquasquadRomInfo, SpecAquasquadRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Archon - The Light and the Dark (48K)

static struct BurnRomInfo SpecArchonRomDesc[] = {
	{ "Archon - The Light and the Dark 48K (1985)(Ariolasoft UK).tap", 50271, 0x97a9d660, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecArchon, SpecArchon, Spectrum)
STD_ROM_FN(SpecArchon)

struct BurnDriver BurnSpecArchon = {
	"spec_archon", NULL, "spec_spectrum", NULL, "1985",
	"Archon - The Light and the Dark (48K)\0", NULL, "Ariolasoft UK", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 2, HARDWARE_SPECTRUM, GBF_ACTION | GBF_BOARD, 0,
	SpectrumGetZipName, SpecArchonRomInfo, SpecArchonRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Armageddon (16K)

static struct BurnRomInfo SpecArmageddonRomDesc[] = {
	{ "Armageddon 16K (1983)(Ocean).tap", 13845, 0x398b3aa5, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecArmageddon, SpecArmageddon, Spectrum)
STD_ROM_FN(SpecArmageddon)

struct BurnDriver BurnSpecArmageddon = {
	"spec_armageddon", NULL, "spec_spectrum", NULL, "1983",
	"Armageddon (16K)\0", NULL, "Ocean", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_SHOOT, 0,
	SpectrumGetZipName, SpecArmageddonRomInfo, SpecArmageddonRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Artura (48K)

static struct BurnRomInfo SpecArturaRomDesc[] = {
	{ "Artura 48K (1988)(Gremlin Graphics).tap", 48556, 0xa7d3f227, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecArtura, SpecArtura, Spectrum)
STD_ROM_FN(SpecArtura)

struct BurnDriver BurnSpecArtura = {
	"spec_artura", NULL, "spec_spectrum", NULL, "1988",
	"Artura (48K)\0", NULL, "Gremlin Graphics", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_SCRFIGHT | GBF_ADV, 0,
	SpectrumGetZipName, SpecArturaRomInfo, SpecArturaRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Asterix and the Magic Cauldron (48K)

static struct BurnRomInfo SpecAsterixRomDesc[] = {
	{ "Asterix and the Magic Cauldron (1986)(Melbourne House).tap", 48507, 0x38447018, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecAsterix, SpecAsterix, Spectrum)
STD_ROM_FN(SpecAsterix)

struct BurnDriver BurnSpecAsterix = {
	"spec_asterix", NULL, "spec_spectrum", NULL, "1986",
	"Asterix and the Magic Cauldron (48K)\0", NULL, "Melbourne House", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_ADV, 0,
	SpectrumGetZipName, SpecAsterixRomInfo, SpecAsterixRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Astroball (128K)

static struct BurnRomInfo SpecAstroballRomDesc[] = {
	{ "Astroball 128K (1992)(Revelation Software).tap", 48342, 0x92699532, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecAstroball, SpecAstroball, Spec128)
STD_ROM_FN(SpecAstroball)

struct BurnDriver BurnSpecAstroball = {
	"spec_astroball", NULL, "spec_spec128", NULL, "1992",
	"Astroball (128K)\0", NULL, "Revelation Software", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_ACTION, 0,
	SpectrumGetZipName, SpecAstroballRomInfo, SpecAstroballRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Astro Blaster (16K)

static struct BurnRomInfo SpecAstroblastRomDesc[] = {
	{ "Astro Blaster 16K (1983)(Quicksilva).tap", 7664, 0x2db0e6e9, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecAstroblast, SpecAstroblast, Spectrum)
STD_ROM_FN(SpecAstroblast)

struct BurnDriver BurnSpecAstroblast = {
	"spec_astroblast", NULL, "spec_spectrum", NULL, "1983",
	"Astro Blaster (16K)\0", NULL, "Quicksilva", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_SHOOT, 0,
	SpectrumGetZipName, SpecAstroblastRomInfo, SpecAstroblastRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Astroclone (48K)

static struct BurnRomInfo SpecAstrocloneRomDesc[] = {
	{ "Astroclone 48K (1985)(Hewson Consultants).tap", 47815, 0x105bdd98, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecAstroclone, SpecAstroclone, Spectrum)
STD_ROM_FN(SpecAstroclone)

struct BurnDriver BurnSpecAstroclone = {
	"spec_astroclone", NULL, "spec_spectrum", NULL, "1985",
	"Astroclone (48K)\0", NULL, "Hewson Consultants", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_ACTION | GBF_ADV, 0,
	SpectrumGetZipName, SpecAstrocloneRomInfo, SpecAstrocloneRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Astronut (48K)

static struct BurnRomInfo SpecAstronutRomDesc[] = {
	{ "Astronut 48K (1984)(Software Projects).z80", 25056, 0x876f4184, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecAstronut, SpecAstronut, Spectrum)
STD_ROM_FN(SpecAstronut)

struct BurnDriver BurnSpecAstronut = {
	"spec_astronut", NULL, "spec_spectrum", NULL, "1984",
	"Astronut (48K)\0", "Press 'S' to start a game session", "Software Projects", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecAstronutRomInfo, SpecAstronutRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// A-Team, The - El Equipo A - Part 1 (Spanish) (128K)

static struct BurnRomInfo SpecAteamp1RomDesc[] = {
	{ "A-Team, the - El Equipo A ES 128K - Part1 (1988)(Zafiro Software).tap", 48242, 0x9acf83a6, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecAteamp1, SpecAteamp1, Spec128)
STD_ROM_FN(SpecAteamp1)

struct BurnDriver BurnSpecAteamp1 = {
	"spec_ateamp1", NULL, "spec_spec128", NULL, "1988",
	"A-Team, The - El Equipo A - Part 1 (Spanish) (128K)\0", "SPACE: smart bomb", "Zafiro Software Division", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_SHOOT, 0,
	SpectrumGetZipName, SpecAteamp1RomInfo, SpecAteamp1RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// A-Team, The - El Equipo A - Part 2 (Spanish) (128K)

static struct BurnRomInfo SpecAteamp2RomDesc[] = {
	{ "A-Team, the - El Equipo A ES 128K - Part2 (1988)(Zafiro Software).tap", 49137, 0x8e139fd1, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecAteamp2, SpecAteamp2, Spec128)
STD_ROM_FN(SpecAteamp2)

struct BurnDriver BurnSpecAteamp2 = {
	"spec_ateamp2", "spec_ateamp1", "spec_spec128", NULL, "1988",
	"A-Team, The - El Equipo A - Part 2 (Spanish) (128K)\0", "Password: WHJONDPEAQR", "Zafiro Software Division", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE, 1, HARDWARE_SPECTRUM, GBF_SHOOT, 0,
	SpectrumGetZipName, SpecAteamp2RomInfo, SpecAteamp2RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// ATF - Advanced Tactical Fighter (128K)

static struct BurnRomInfo SpecAtf128kRomDesc[] = {
	{ "ATF 128K (1988)(Digital Integration).z80", 43769, 0x23f0731e, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecAtf128k, SpecAtf128k, Spec128)
STD_ROM_FN(SpecAtf128k)

struct BurnDriver BurnSpecAtf128k = {
	"spec_atf128k", NULL, "spec_spec128", NULL, "1988",
	"ATF - Advanced Tactical Fighter (128K)\0", NULL, "Digital Integration", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_SIM | GBF_SHOOT, 0,
	SpectrumGetZipName, SpecAtf128kRomInfo, SpecAtf128kRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Atlantis (48K)

static struct BurnRomInfo SpecAtlantisRomDesc[] = {
	{ "Atlantis 48K (1985)(Anirog).tap", 27049, 0x460ac502, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecAtlantis, SpecAtlantis, Spectrum)
STD_ROM_FN(SpecAtlantis)

struct BurnDriver BurnSpecAtlantis = {
	"spec_atlantis", NULL, "spec_spectrum", NULL, "1985",
	"Atlantis (48K)\0", "Press 'B' to use controller and 'S' to start game", "Anirog", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_ACTION, 0,
	SpectrumGetZipName, SpecAtlantisRomInfo, SpecAtlantisRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Atomic Robo-Kid (Demo) (48K)

static struct BurnRomInfo SpecAtomrobokidRomDesc[] = {
	{ "Atomic Robo-Kid (demo) (1991)(Activision).z80", 44259, 0x2f0c3726, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecAtomrobokid, SpecAtomrobokid, Spectrum)
STD_ROM_FN(SpecAtomrobokid)

struct BurnDriver BurnSpecAtomrobokid = {
	"spec_atomrobokid", NULL, "spec_spectrum", NULL, "1991",
	"Atomic Robo-Kid (Demo) (48K)\0", "Unfinished - no sound", "Activision", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_HORSHOOT, 0,
	SpectrumGetZipName, SpecAtomrobokidRomInfo, SpecAtomrobokidRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Attack of the Killer Tomatoes (48K)

static struct BurnRomInfo SpecAktomatoesRomDesc[] = {
	{ "Attack of the Killer Tomatoes 48K (1986)(Global).tap", 38305, 0xd3d3b7aa, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecAktomatoes, SpecAktomatoes, Spectrum)
STD_ROM_FN(SpecAktomatoes)

struct BurnDriver BurnSpecAktomatoes = {
	"spec_aktomatoes", NULL, "spec_spectrum", NULL, "1986",
	"Attack of the Killer Tomatoes (48K)\0", NULL, "Global Software", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_ADV, 0,
	SpectrumGetZipName, SpecAktomatoesRomInfo, SpecAktomatoesRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Automania (48K)

static struct BurnRomInfo SpecAutomaniaRomDesc[] = {
	{ "Automania 48K (1984)(Mikro-Gen).tap", 39462, 0xc637b38a, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecAutomania, SpecAutomania, Spectrum)
STD_ROM_FN(SpecAutomania)

struct BurnDriver BurnSpecAutomania = {
	"spec_automania", NULL, "spec_spectrum", NULL, "1984",
	"Automania (48K)\0", NULL, "Mikro-Gen", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecAutomaniaRomInfo, SpecAutomaniaRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Aventura Espacial, La - Parte 1 - Explorando Mundos (Spanish) (48K)

static struct BurnRomInfo SpecAvespacial1RomDesc[] = {
	{ "Aventura Espacial, La - Explorando Mundos (1990)(Aventuras AD).z80", 45676, 0xc46932a5, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecAvespacial1, SpecAvespacial1, Spectrum)
STD_ROM_FN(SpecAvespacial1)

struct BurnDriver BurnSpecAvespacial1 = {
	"spec_avespacial1", NULL, "spec_spectrum", NULL, "1990",
	"Aventura Espacial, La - Explorando Mundos (Spanish) (48K)\0", NULL, "Aventuras A.D.", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_ADV, 0,
	SpectrumGetZipName, SpecAvespacial1RomInfo, SpecAvespacial1RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Aventura Espacial, La - Parte 2 - En los Dominios del Cerebelo (Spanish) (48K)

static struct BurnRomInfo SpecAvespacial2RomDesc[] = {
	{ "Aventura Espacial, La - En los Dominios del Cerebelo (1990)(Aventuras AD).z80", 40915, 0x06d2fd5c, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecAvespacial2, SpecAvespacial2, Spectrum)
STD_ROM_FN(SpecAvespacial2)

struct BurnDriver BurnSpecAvespacial2 = {
	"spec_avespacial2", "spec_avespacial1", "spec_spectrum", NULL, "1990",
	"Aventura Espacial, La - En los Dominios del Cerebelo (Spanish) (48K)\0", "PASSWORD: CANES VENATICI", "Aventuras A.D.", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE, 1, HARDWARE_SPECTRUM, GBF_ADV, 0,
	SpectrumGetZipName, SpecAvespacial2RomInfo, SpecAvespacial2RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Aventura Original - Parte 1 - La Busqueda (Spanish) (48K)

static struct BurnRomInfo SpecAvoriginal1RomDesc[] = {
	{ "Aventura Original - Parte 1 - La Busqueda (1989)(Aventuras AD).z80", 46750, 0xce246950, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecAvoriginal1, SpecAvoriginal1, Spectrum)
STD_ROM_FN(SpecAvoriginal1)

struct BurnDriver BurnSpecAvoriginal1 = {
	"spec_avoriginal1", NULL, "spec_spectrum", NULL, "1989",
	"Aventura Original - Parte 1 - La Busqueda (Spanish) (48K)\0", NULL, "Aventuras A.D.", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_ADV, 0,
	SpectrumGetZipName, SpecAvoriginal1RomInfo, SpecAvoriginal1RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Aventura Original - Parte 2 - El Encuentro (Spanish) (48K)

static struct BurnRomInfo SpecAvoriginal2RomDesc[] = {
	{ "Aventura Original - Parte 2 - El Encuentro (1989)(Aventuras AD).z80", 47212, 0x126a3c19, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecAvoriginal2, SpecAvoriginal2, Spectrum)
STD_ROM_FN(SpecAvoriginal2)

struct BurnDriver BurnSpecAvoriginal2 = {
	"spec_avoriginal2", "spec_avoriginal1", "spec_spectrum", NULL, "1989",
	"Aventura Original - Parte 2 - El Encuentro (Spanish) (48K)\0", "PASSWORD: TIMACUS", "Aventuras A.D.", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE, 1, HARDWARE_SPECTRUM, GBF_ADV, 0,
	SpectrumGetZipName, SpecAvoriginal2RomInfo, SpecAvoriginal2RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Averno (Spanish) (48K)

static struct BurnRomInfo SpecAvernoRomDesc[] = {
	{ "Averno ES 48K (1989)(P.J.).tap", 22871, 0xef4758f8, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecAverno, SpecAverno, Spectrum)
STD_ROM_FN(SpecAverno)

struct BurnDriver BurnSpecAverno = {
	"spec_averno", NULL, "spec_spectrum", NULL, "1989",
	"Averno (Spanish) (48K)\0", NULL, "P.J. Software", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_PUZZLE, 0,
	SpectrumGetZipName, SpecAvernoRomInfo, SpecAvernoRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecCursorKeysDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Babaliba (48K)

static struct BurnRomInfo SpecBabalibaRomDesc[] = {
	{ "Babaliba 48K (1984)(Dinamic Software).tap", 47064, 0xfe7fc680, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecBabaliba, SpecBabaliba, Spectrum)
STD_ROM_FN(SpecBabaliba)

struct BurnDriver BurnSpecBabaliba = {
	"spec_babaliba", NULL, "spec_spectrum", NULL, "1984",
	"Babaliba (48K)\0", NULL, "Dinamic Software", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_MAZE, 0,
	SpectrumGetZipName, SpecBabalibaRomInfo, SpecBabalibaRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Backgammon (48K)

static struct BurnRomInfo SpecBackgammonRomDesc[] = {
	{ "Backgammon 48K (1988)(CDS Microsystems).tap", 20113, 0xe501a833, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecBackgammon, SpecBackgammon, Spectrum)
STD_ROM_FN(SpecBackgammon)

struct BurnDriver BurnSpecBackgammon = {
	"spec_backgammon", NULL, "spec_spectrum", NULL, "1988",
	"Backgammon (48K)\0", NULL, "CDS Microsystems", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 2, HARDWARE_SPECTRUM, GBF_BOARD, 0,
	SpectrumGetZipName, SpecBackgammonRomInfo, SpecBackgammonRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Back to Skool (48K)

static struct BurnRomInfo SpecbackskooRomDesc[] = {
	{ "Back to Skool 48K (1985)(Microsphere).tap", 48013, 0xc8136fa9, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specbackskoo, Specbackskoo, Spectrum)
STD_ROM_FN(Specbackskoo)

struct BurnDriver BurnSpecbackskoo = {
	"spec_backskoo", NULL, "spec_spectrum", NULL, "1985",
	"Back to Skool (48K)\0", NULL, "Microsphere", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_ACTION | GBF_ADV, 0,
	SpectrumGetZipName, SpecbackskooRomInfo, SpecbackskooRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecCursorKeysDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Back to the Future (48K)

static struct BurnRomInfo SpecbackfutuRomDesc[] = {
	{ "Back to the Future 48K (1985)(Electric Dreams).tap", 50449, 0xf29bbae6, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specbackfutu, Specbackfutu, Spectrum)
STD_ROM_FN(Specbackfutu)

struct BurnDriver BurnSpecbackfutu = {
	"spec_backfutu", NULL, "spec_spectrum", NULL, "1985",
	"Back to the Future (48K)\0", NULL, "Electric Dreams", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_ADV, 0,
	SpectrumGetZipName, SpecbackfutuRomInfo, SpecbackfutuRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Back to the Future II (128K)

static struct BurnRomInfo Specbackfut2RomDesc[] = {
	{ "Back to the Future II (1990)(Image Works).tap", 0x39d95, 0x464a3359, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specbackfut2, Specbackfut2, Spec128)
STD_ROM_FN(Specbackfut2)

struct BurnDriver BurnSpecbackfut2 = {
	"spec_backfut2", NULL, "spec_spec128", NULL, "1990",
	"Back to the Future II (128K)\0", "Set '2 Sinclair' to use controller", "Image Works", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_ACTION, 0,
	SpectrumGetZipName, Specbackfut2RomInfo, Specbackfut2RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecIntf2DIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Back to the Future III (128K)

static struct BurnRomInfo Specbackfut3RomDesc[] = {
	{ "Back to the Future III (1991)(Image Works).tap", 0x3661a, 0x5de2c954, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specbackfut3, Specbackfut3, Spec128)
STD_ROM_FN(Specbackfut3)

struct BurnDriver BurnSpecbackfut3 = {
	"spec_backfut3", NULL, "spec_spec128", NULL, "1991",
	"Back to the Future III (128K)\0", NULL, "Image Works", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_ACTION, 0,
	SpectrumGetZipName, Specbackfut3RomInfo, Specbackfut3RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecIntf2DIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Badlands (128K)

static struct BurnRomInfo SpecbadlandsRomDesc[] = {
	{ "Badlands (1990)(Domark)(48K-128K).tap", 33672, 0x5df53959, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specbadlands, Specbadlands, Spec128)
STD_ROM_FN(Specbadlands)

struct BurnDriver BurnSpecbadlands = {
	"spec_badlands", NULL, "spec_spec128", NULL, "1990",
	"Badlands (128K)\0", NULL, "Domark", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 2, HARDWARE_SPECTRUM, GBF_RACING, 0,
	SpectrumGetZipName, SpecbadlandsRomInfo, SpecbadlandsRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecQAOPSpaceDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Ballbreaker (128K)

static struct BurnRomInfo SpecBallbreak128RomDesc[] = {
	{ "Ballbreaker 128K (1988)(CRL Group).z80", 38627, 0x2fca1d2e, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecBallbreak128, SpecBallbreak128, Spec128)
STD_ROM_FN(SpecBallbreak128)

struct BurnDriver BurnSpecBallbreak128 = {
	"spec_ballbreak128", NULL, "spec_spec128", NULL, "1988",
	"Ballbreaker (128K)\0", "A, Z and SPACE to select options", "CRL Group", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_BREAKOUT, 0,
	SpectrumGetZipName, SpecBallbreak128RomInfo, SpecBallbreak128RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Ballbreaker (48K)

static struct BurnRomInfo SpecBallbreak48RomDesc[] = {
	{ "Ballbreaker 48K (1988)(CRL Group).z80", 37090, 0x09eecf5a, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecBallbreak48, SpecBallbreak48, Spectrum)
STD_ROM_FN(SpecBallbreak48)

struct BurnDriver BurnSpecBallbreak48 = {
	"spec_ballbreak48", "spec_ballbreak128", "spec_spectrum", NULL, "1988",
	"Ballbreaker (48K)\0", "A, Z and SPACE to select options", "CRL Group", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE, 1, HARDWARE_SPECTRUM, GBF_BREAKOUT, 0,
	SpectrumGetZipName, SpecBallbreak48RomInfo, SpecBallbreak48RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Bangers and Mash (128K)

static struct BurnRomInfo SpecBangmashRomDesc[] = {
	{ "Bangers and Mash (1990)(Alternative).tap", 74560, 0xd5225cf2, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecBangmash, SpecBangmash, Spec128)
STD_ROM_FN(SpecBangmash)

struct BurnDriver BurnSpecBangmash = {
	"spec_bangmash", NULL, "spec_spec128", NULL, "1990",
	"Bangers and Mash (128K)\0", "Z, X, L and ENTER to move main character", "Alternative Software", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecBangmashRomInfo, SpecBangmashRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Barbarian: The Ultimate Warrior - Part 1 (48K)

static struct BurnRomInfo Specbarbpt1RomDesc[] = {
	{ "Barbarian - 2 Players 48K (1987)(Palace Software).tap", 56407, 0x18dd79c2, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specbarbpt1, Specbarbpt1, Spectrum)
STD_ROM_FN(Specbarbpt1)

struct BurnDriver BurnSpecbarbpt1 = {
	"spec_barbpt1", NULL, "spec_spectrum", NULL, "1987",
	"Barbarian: The Ultimate Warrior - Part 1 (48K)\0", "one and two player mode", "Palace Software", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 2, HARDWARE_SPECTRUM, GBF_VSFIGHT, 0,
	SpectrumGetZipName, Specbarbpt1RomInfo, Specbarbpt1RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Barbarian: The Ultimate Warrior - Part 2 (48K)

static struct BurnRomInfo Specbarbpt2RomDesc[] = {
	{ "Barbarian - 1 Player 48K (1987)(Palace Software).tap", 56407, 0xa99a7baf, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specbarbpt2, Specbarbpt2, Spectrum)
STD_ROM_FN(Specbarbpt2)

struct BurnDriver BurnSpecbarbpt2 = {
	"spec_barbpt2", "spec_barbpt1", "spec_spectrum", NULL, "1987",
	"Barbarian: The Ultimate Warrior - Part 2 (48K)\0", "only one player mode", "Palace Software", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE, 1, HARDWARE_SPECTRUM, GBF_VSFIGHT, 0,
	SpectrumGetZipName, Specbarbpt2RomInfo, Specbarbpt2RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Barbarian (Psygnosis) (48K)

static struct BurnRomInfo SpecbarbarianRomDesc[] = {
	{ "Barbarian 48K (1988)(Melbourne House).tap", 46467, 0x51bf72c1, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specbarbarian, Specbarbarian, Spectrum)
STD_ROM_FN(Specbarbarian)

struct BurnDriver BurnSpecbarbarian = {
	"spec_barbarian", NULL, "spec_spectrum", NULL, "1988",
	"Barbarian (Psygnosis) (48K)\0", NULL, "Melbourne House", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_SCRFIGHT | GBF_ADV, 0,
	SpectrumGetZipName, SpecbarbarianRomInfo, SpecbarbarianRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Barbarian II: The Dungeon of Drax (128K)

static struct BurnRomInfo Specbarbarn2RomDesc[] = {
	{ "Barbarian II - The Dungeon of Drax 128K (1988)(Palace Software).z80", 0x1ac6c, 0x2215c3b7, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specbarbarn2, Specbarbarn2, Spec128)
STD_ROM_FN(Specbarbarn2)

struct BurnDriver BurnSpecbarbarn2 = {
	"spec_barbarn2", NULL, "spec_spec128", NULL, "1988",
	"Barbarian II: The Dungeon of Drax (128K)\0", "Press 'N' to through doors/gates", "Palace Software", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_SCRFIGHT, 0,
	SpectrumGetZipName, Specbarbarn2RomInfo, Specbarbarn2RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Batman: The Caped Crusader - Part 1: A Bird in the Hand (48K)

static struct BurnRomInfo SpecbatmanccRomDesc[] = {
	{ "Batman - The Caped Crusader - A Bird in the Hand 48K (1988)(Ocean).tap", 48076, 0x3102cef1, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specbatmancc, Specbatmancc, Spectrum)
STD_ROM_FN(Specbatmancc)

struct BurnDriver BurnSpecbatmancc = {
	"spec_batmancc", NULL, "spec_spectrum", NULL, "1988",
	"Batman: The Caped Crusader - Part 1: A Bird in the Hand (48K)\0", NULL, "Ocean", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_ACTION | GBF_ADV, 0,
	SpectrumGetZipName, SpecbatmanccRomInfo, SpecbatmanccRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Batman: The Caped Crusader - Part 2: A Fete Worse than Death (48K)

static struct BurnRomInfo Specbatmancc2RomDesc[] = {
	{ "Batman - The Caped Crusader - A Fete Worse than Death 48K (1988)(Ocean).tap", 49178, 0x967e82ee, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specbatmancc2, Specbatmancc2, Spectrum)
STD_ROM_FN(Specbatmancc2)

struct BurnDriver BurnSpecbatmancc2 = {
	"spec_batmancc2", "spec_batmancc", "spec_spectrum", NULL, "1988",
	"Batman: The Caped Crusader - Part 2: A Fete Worse than Death (48K)\0", NULL, "Ocean", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE, 1, HARDWARE_SPECTRUM, GBF_ACTION | GBF_ADV, 0,
	SpectrumGetZipName, Specbatmancc2RomInfo, Specbatmancc2RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Batman: The Movie (128K)

static struct BurnRomInfo SpecbatmanmvRomDesc[] = {
	{ "Batman - The Movie 128K (1989)(Ocean).tap", 121528, 0x910b0bf4, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specbatmanmv, Specbatmanmv, Spec128)
STD_ROM_FN(Specbatmanmv)

struct BurnDriver BurnSpecbatmanmv = {
	"spec_batmanmv", NULL, "spec_spec128", NULL, "1989",
	"Batman: The Movie (128K)\0", NULL, "Ocean", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_PLATFORM | GBF_RUNGUN, 0,
	SpectrumGetZipName, SpecbatmanmvRomInfo, SpecbatmanmvRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Batman - The Puaj Edition (48K)

static struct BurnRomInfo SpecbatmanpeRomDesc[] = {
	{ "Batman - The Puaj Edition 48K (1989)(Micro House).z80", 0x086f6, 0x1ec85d88, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specbatmanpe, Specbatmanpe, Spectrum)
STD_ROM_FN(Specbatmanpe)

struct BurnDriver BurnSpecbatmanpe = {
	"spec_batmanpe", NULL, "spec_spectrum", NULL, "1989",
	"Batman - The Puaj Edition (48K)\0", NULL, "Micro House", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_MAZE | GBF_ADV, 0,
	SpectrumGetZipName, SpecbatmanpeRomInfo, SpecbatmanpeRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Batman (48K)

static struct BurnRomInfo Specbatman_48RomDesc[] = {
	{ "Batman 48K-128K (1986)(Ocean).tap", 47819, 0x898d7592, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specbatman_48, Specbatman_48, Spectrum)
STD_ROM_FN(Specbatman_48)

struct BurnDriver BurnSpecbatman_48 = {
	"spec_batman_48", "spec_batman", "spec_spectrum", NULL, "1986",
	"Batman (48K)\0", NULL, "Ocean", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE, 1, HARDWARE_SPECTRUM, GBF_ADV, 0,
	SpectrumGetZipName, Specbatman_48RomInfo, Specbatman_48RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Batman (128K)

static struct BurnRomInfo SpecbatmanRomDesc[] = {
	{ "Batman 48K-128K (1986)(Ocean).tap", 47819, 0x898d7592, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specbatman, Specbatman, Spec128)
STD_ROM_FN(Specbatman)

struct BurnDriver BurnSpecbatman = {
	"spec_batman", NULL, "spec_spec128", NULL, "1986",
	"Batman (128K)\0", NULL, "Ocean", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_ADV, 0,
	SpectrumGetZipName, SpecbatmanRomInfo, SpecbatmanRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Battle Ships (48K)

static struct BurnRomInfo SpecBattleshipsRomDesc[] = {
	{ "Battle Ships 48K (1987)(Hit-Pak).z80", 38283, 0xfb32ab1a, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecBattleships, SpecBattleships, Spectrum)
STD_ROM_FN(SpecBattleships)

struct BurnDriver BurnSpecBattleships = {
	"spec_battleships", NULL, "spec_spectrum", NULL, "1987",
	"Battle Ships (48K)\0", NULL, "Hit-Pak", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 2, HARDWARE_SPECTRUM, GBF_STRATEGY | GBF_BOARD, 0,
	SpectrumGetZipName, SpecBattleshipsRomInfo, SpecBattleshipsRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Battle Valley (128K)

static struct BurnRomInfo SpecBtlvalleyRomDesc[] = {
	{ "Battle Valley 128K (1988)(Rack-It).tap", 39021, 0x93b3ce85, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecBtlvalley, SpecBtlvalley, Spec128)
STD_ROM_FN(SpecBtlvalley)

struct BurnDriver BurnSpecBtlvalley = {
	"spec_btlvalley", NULL, "spec_spec128", NULL, "1988",
	"Battle Valley (128K)\0", "Double click on fire button to stop/reverse", "Rack-It - Hewson", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_HORSHOOT, 0,
	SpectrumGetZipName, SpecBtlvalleyRomInfo, SpecBtlvalleyRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecIntf2DIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Battlezone (48K)

static struct BurnRomInfo SpecBtlzoneRomDesc[] = {
	{ "Battlezone 48K (1984)(Quicksilva).tap", 36210, 0xaf085e02, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecBtlzone, SpecBtlzone, Spectrum)
STD_ROM_FN(SpecBtlzone)

struct BurnDriver BurnSpecBtlzone = {
	"spec_btlzone", NULL, "spec_spectrum", NULL, "1984",
	"Battlezone (48K)\0", NULL, "Quicksilva", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_SHOOT, 0,
	SpectrumGetZipName, SpecBtlzoneRomInfo, SpecBtlzoneRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Beach-Head (48K)

static struct BurnRomInfo SpecBheadRomDesc[] = {
	{ "Beach-Head 48K (1984)(U.S. Gold).tap", 48048, 0x429ecb91, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecBhead, SpecBhead, Spectrum)
STD_ROM_FN(SpecBhead)

struct BurnDriver BurnSpecBhead = {
	"spec_bhead", NULL, "spec_spectrum", NULL, "1984",
	"Beach-Head (48K)\0", NULL, "U.S. Gold", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 2, HARDWARE_SPECTRUM, GBF_SHOOT, 0,
	SpectrumGetZipName, SpecBheadRomInfo, SpecBheadRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Beach-Head II - The Dictator Strikes Back! (48K)

static struct BurnRomInfo SpecBhead2RomDesc[] = {
	{ "Beach-Head II - The Dictator Strikes Back! 48K (1985)(U.S. Gold).tap", 48857, 0xe977cfce, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecBhead2, SpecBhead2, Spectrum)
STD_ROM_FN(SpecBhead2)

struct BurnDriver BurnSpecBhead2 = {
	"spec_bhead2", NULL, "spec_spectrum", NULL, "1985",
	"Beach-Head II - The Dictator Strikes Back! (48K)\0", NULL, "U.S. Gold", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 2, HARDWARE_SPECTRUM, GBF_SHOOT, 0,
	SpectrumGetZipName, SpecBhead2RomInfo, SpecBhead2RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Beach Volley (128K)

static struct BurnRomInfo SpecBvolleyRomDesc[] = {
	{ "Beach Volley (1989)(Ocean).tap", 93507, 0x73056f7e, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecBvolley, SpecBvolley, Spec128)
STD_ROM_FN(SpecBvolley)

struct BurnDriver BurnSpecBvolley = {
	"spec_bvolley", NULL, "spec_spec128", NULL, "1989",
	"Beach Volley (128K)\0", "'Redefine Keys' to use controller", "Ocean", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 2, HARDWARE_SPECTRUM, GBF_SPORTSMISC, 0,
	SpectrumGetZipName, SpecBvolleyRomInfo, SpecBvolleyRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecCursorKeysDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Beaky and the Egg Snatchers (48K)

static struct BurnRomInfo SpecBeakyRomDesc[] = {
	{ "Beaky and the Egg Snatchers 48K (1984)(Fantasy Software).z80", 40787, 0xc145a40c, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecBeaky, SpecBeaky, Spectrum)
STD_ROM_FN(SpecBeaky)

struct BurnDriver BurnSpecBeaky = {
	"spec_beaky", NULL, "spec_spectrum", NULL, "1984",
	"Beaky and the Egg Snatchers (48K)\0", "Select '2 Kempston joystick' to use controller", "Fantasy Software", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecBeakyRomInfo, SpecBeakyRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Beamrider (48K)

static struct BurnRomInfo SpecBeamriderRomDesc[] = {
	{ "Beamrider 48K (1984)(Activision).tap", 36219, 0xc80c9aa7, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecBeamrider, SpecBeamrider, Spectrum)
STD_ROM_FN(SpecBeamrider)

struct BurnDriver BurnSpecBeamrider = {
	"spec_beamrider", NULL, "spec_spectrum", NULL, "1984",
	"Beamrider (48K)\0", NULL, "Activision", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 4, HARDWARE_SPECTRUM, GBF_SHOOT, 0,
	SpectrumGetZipName, SpecBeamriderRomInfo, SpecBeamriderRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Bedlam (128K)

static struct BurnRomInfo SpecBedlamRomDesc[] = {
	{ "Bedlam 128K (1988)(Go!).tap", 106920, 0x733eb4ae, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecBedlam, SpecBedlam, Spec128)
STD_ROM_FN(SpecBedlam)

struct BurnDriver BurnSpecBedlam = {
	"spec_bedlam", NULL, "spec_spec128", NULL, "1988",
	"Bedlam (128K)\0", NULL, "Go!", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_VERSHOOT, 0,
	SpectrumGetZipName, SpecBedlamRomInfo, SpecBedlamRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecIntf2DIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Benny Hill's Madcap Chase! (48K)

static struct BurnRomInfo SpecBennyhilRomDesc[] = {
	{ "Benny Hill's Madcap Chase! (1985)(DK'Tronics).tap", 33223, 0xe7a95cef, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecBennyhil, SpecBennyhil, Spectrum)
STD_ROM_FN(SpecBennyhil)

struct BurnDriver BurnSpecBennyhil = {
	"spec_bennyhil", NULL, "spec_spectrum", NULL, "1985",
	"Benny Hill's Madcap Chase! (48K)\0", NULL, "DK'Tronics", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_MISC | GBF_ADV, 0,
	SpectrumGetZipName, SpecBennyhilRomInfo, SpecBennyhilRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Bestial Warrior (Spanish) (48K)

static struct BurnRomInfo SpecBestialwrrRomDesc[] = {
	{ "Bestial Warrior 48K ES (1989)(Dinamic).tap", 55620, 0xf1fb3cb5, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecBestialwrr, SpecBestialwrr, Spectrum)
STD_ROM_FN(SpecBestialwrr)

struct BurnDriver BurnSpecBestialwrr = {
	"spec_bestialwrr", NULL, "spec_spectrum", NULL, "1989",
	"Bestial Warrior (Spanish) (48K)\0", NULL, "Dinamic Software", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_RUNGUN | GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecBestialwrrRomInfo, SpecBestialwrrRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Beverly Hills Cop (128K)

static struct BurnRomInfo SpecBhcopRomDesc[] = {
	{ "Beverly Hills Cop 128K (1990)(Tynesoft).tap", 117310, 0x2b8a9a2f, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecBhcop, SpecBhcop, Spec128)
STD_ROM_FN(SpecBhcop)

struct BurnDriver BurnSpecBhcop = {
	"spec_bhcop", NULL, "spec_spec128", NULL, "1990",
	"Beverly Hills Cop (128K)\0", NULL, "Tynesoft", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_ACTION | GBF_SHOOT, 0,
	SpectrumGetZipName, SpecBhcopRomInfo, SpecBhcopRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Bigfoot (48K)

static struct BurnRomInfo SpecBigfootRomDesc[] = {
	{ "Bigfoot (1988)(Codemasters).tap", 39134, 0x5f68a506, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecBigfoot, SpecBigfoot, Spectrum)
STD_ROM_FN(SpecBigfoot)

struct BurnDriver BurnSpecBigfoot = {
	"spec_bigfoot", NULL, "spec_spectrum", NULL, "1988",
	"Bigfoot (48K)\0", NULL, "Codemasters", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecBigfootRomInfo, SpecBigfootRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Big Nose's American Adventure (48K)

static struct BurnRomInfo SpecBignoseamericanadvRomDesc[] = {
	{ "Big Nose's American Adventure (1992)(Codemasters).tap", 47470, 0x64006917, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecBignoseamericanadv, SpecBignoseamericanadv, Spectrum)
STD_ROM_FN(SpecBignoseamericanadv)

struct BurnDriver BurnSpecBignoseamericanadv = {
	"spec_bignoseamericanadv", NULL, "spec_spectrum", NULL, "1992",
	"Big Nose's American Adventure (48K)\0", NULL, "Codemasters", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecBignoseamericanadvRomInfo, SpecBignoseamericanadvRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Big Trouble in Little China (48K)

static struct BurnRomInfo SpecBigtroubleinlittlechinaRomDesc[] = {
	{ "Big Trouble in Little China (1986)(Electric Dreams).tap", 51364, 0x5ba135d3, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecBigtroubleinlittlechina, SpecBigtroubleinlittlechina, Spectrum)
STD_ROM_FN(SpecBigtroubleinlittlechina)

struct BurnDriver BurnSpecBigtroubleinlittlechina = {
	"spec_bigtroubleinlittlechina", NULL, "spec_spectrum", NULL, "1986",
	"Big Trouble in Little China (48K)\0", NULL, "Electric Dreams", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_SCRFIGHT, 0,
	SpectrumGetZipName, SpecBigtroubleinlittlechinaRomInfo, SpecBigtroubleinlittlechinaRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Blacula (Prototype) (48K)

static struct BurnRomInfo SpecBlaculaRomDesc[] = {
	{ "Blacula 48K Prototype (199x)(unreleased).tap", 43419, 0x817288d8, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecBlacula, SpecBlacula, Spectrum)
STD_ROM_FN(SpecBlacula)

struct BurnDriver BurnSpecBlacula = {
	"spec_blacula", NULL, "spec_spectrum", NULL, "19??",
	"Blacula (Prototype) (48K)\0", NULL, "unknown", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_PROTOTYPE, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecBlaculaRomInfo, SpecBlaculaRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// BLAM (128K)

static struct BurnRomInfo SpecBlamRomDesc[] = {
	{ "BLAM (1992-2023)(Scattergood Studios).tap", 41609, 0x1c0219f8, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecBlam, SpecBlam, Spec128)
STD_ROM_FN(SpecBlam)

struct BurnDriver BurnSpecBlam = {
	"spec_blam", NULL, "spec_spec128", NULL, "1992-2023",
	"BLAM (128K)\0", NULL, "Scattergood Studios", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_HORSHOOT, 0,
	SpectrumGetZipName, SpecBlamRomInfo, SpecBlamRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Blasteroids (128K)

static struct BurnRomInfo SpecBlasteroidsRomDesc[] = {
	{ "Blasteroids 128K (1989)(Image Works).tap", 50430, 0x5cfaa878, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecBlasteroids, SpecBlasteroids, Spec128)
STD_ROM_FN(SpecBlasteroids)

struct BurnDriver BurnSpecBlasteroids = {
	"spec_blasteroids", NULL, "spec_spec128", NULL, "1989",
	"Blasteroids (128K)\0", NULL, "Image Works", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 2, HARDWARE_SPECTRUM, GBF_SHOOT, 0,
	SpectrumGetZipName, SpecBlasteroidsRomInfo, SpecBlasteroidsRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Blazing Thunder (48K)

static struct BurnRomInfo SpecBlazthundRomDesc[] = {
	{ "Blazing Thunder 48K (1990)(Hi-Tec).z80", 41048, 0x41afcb85, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecBlazthund, SpecBlazthund, Spectrum)
STD_ROM_FN(SpecBlazthund)

struct BurnDriver BurnSpecBlazthund = {
	"spec_blazthund", NULL, "spec_spectrum", NULL, "1990",
	"Blazing Thunder (48K)\0", NULL, "Hi-Tec Software", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_VERSHOOT, 0,
	SpectrumGetZipName, SpecBlazthundRomInfo, SpecBlazthundRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Block-Dizzy (48K)

static struct BurnRomInfo SpecBlkdizzyRomDesc[] = {
	{ "Block-Dizzy 48K (1992)(Your Sinclair).tap", 39286, 0x639b4fe8, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecBlkdizzy, SpecBlkdizzy, Spectrum)
STD_ROM_FN(SpecBlkdizzy)

struct BurnDriver BurnSpecBlkdizzy = {
	"spec_blkdizzy", NULL, "spec_spectrum", NULL, "1992",
	"Block-Dizzy (48K)\0", "Set Controls>>Sinclair to use controller", "Your Sinclair", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_PUZZLE, 0,
	SpectrumGetZipName, SpecBlkdizzyRomInfo, SpecBlkdizzyRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecIntf2DIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Bloody (Spanish) (48K)

static struct BurnRomInfo SpecBloodyRomDesc[] = {
	{ "Bloody ES (1987)(Genesis Soft).tap", 28033, 0xae89fdd6, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecBloody, SpecBloody, Spectrum)
STD_ROM_FN(SpecBloody)

struct BurnDriver BurnSpecBloody = {
	"spec_bloody", NULL, "spec_spectrum", NULL, "1987",
	"Bloody (Spanish) (48K)\0", NULL, "Genesis Soft", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecBloodyRomInfo, SpecBloodyRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Bloody Paws - Part 1 (Spanish) (128K)

static struct BurnRomInfo SpecBloodpaws1RomDesc[] = {
	{ "Bloody Paws ES Part1 (1990)(G.LL.).tap", 35835, 0x65d44dab, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecBloodpaws1, SpecBloodpaws1, Spec128)
STD_ROM_FN(SpecBloodpaws1)

struct BurnDriver BurnSpecBloodpaws1 = {
	"spec_bloodpaws1", NULL, "spec_spec128", NULL, "1990",
	"Bloody Paws - Part 1 (Spanish) (128K)\0", "Set 'Sinclair' to use controller", "G.LL. Software", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_SCRFIGHT, 0,
	SpectrumGetZipName, SpecBloodpaws1RomInfo, SpecBloodpaws1RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecIntf2DIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Bloody Paws - Part 2 (Spanish) (128K)

static struct BurnRomInfo SpecBloodpaws2RomDesc[] = {
	{ "Bloody Paws ES Part2 (1990)(G.LL.).tap", 37263, 0x97af0b3a, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecBloodpaws2, SpecBloodpaws2, Spec128)
STD_ROM_FN(SpecBloodpaws2)

struct BurnDriver BurnSpecBloodpaws2 = {
	"spec_bloodpaws2", "spec_bloodpaws1", "spec_spec128", NULL, "1990",
	"Bloody Paws - Part 2 (Spanish) (128K)\0", "Set 'Sinclair' to use controller", "G.LL. Software", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE, 1, HARDWARE_SPECTRUM, GBF_SCRFIGHT, 0,
	SpectrumGetZipName, SpecBloodpaws2RomInfo, SpecBloodpaws2RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecIntf2DIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Blue Max (48K)

static struct BurnRomInfo SpecBluemaxRomDesc[] = {
	{ "Blue Max 48K (1984)(U.S. Gold).tap", 42721, 0x13b6bc71, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecBluemax, SpecBluemax, Spectrum)
STD_ROM_FN(SpecBluemax)

struct BurnDriver BurnSpecBluemax = {
	"spec_bluemax", NULL, "spec_spectrum", NULL, "1984",
	"Blue Max (48K)\0", NULL, "U.S. Gold", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_SHOOT, 0,
	SpectrumGetZipName, SpecBluemaxRomInfo, SpecBluemaxRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// BMX Ninja (48K)

static struct BurnRomInfo SpecBmxninjaRomDesc[] = {
	{ "BMX Ninja 48K (1988)(Alternative).tap", 57376, 0x64ed0d8b, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecBmxninja, SpecBmxninja, Spectrum)
STD_ROM_FN(SpecBmxninja)

struct BurnDriver BurnSpecBmxninja = {
	"spec_bmxninja", NULL, "spec_spectrum", NULL, "1988",
	"BMX Ninja (48K)\0", NULL, "Alternative Software", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_VSFIGHT, 0,
	SpectrumGetZipName, SpecBmxninjaRomInfo, SpecBmxninjaRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// BMX Racers (48K)

static struct BurnRomInfo SpecBmxraceRomDesc[] = {
	{ "BMX Racers 48K (1984)(Mastertronic).tap", 45509, 0x40ec8744, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecBmxrace, SpecBmxrace, Spectrum)
STD_ROM_FN(SpecBmxrace)

struct BurnDriver BurnSpecBmxrace = {
	"spec_bmxrace", NULL, "spec_spectrum", NULL, "1984",
	"BMX Racers (48K)\0", NULL, "Mastertronic", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_ACTION, 0,
	SpectrumGetZipName, SpecBmxraceRomInfo, SpecBmxraceRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// BMX Simulator (48K)

static struct BurnRomInfo SpecBmxsimRomDesc[] = {
	{ "BMX Simulator 48K (1987)(Codemasters).z80", 43885, 0x61e994c0, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecBmxsim, SpecBmxsim, Spectrum)
STD_ROM_FN(SpecBmxsim)

struct BurnDriver BurnSpecBmxsim = {
	"spec_bmxsim", NULL, "spec_spectrum", NULL, "1987",
	"BMX Simulator (48K)\0", NULL, "Codemasters", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 2, HARDWARE_SPECTRUM, GBF_RACING, 0,
	SpectrumGetZipName, SpecBmxsimRomInfo, SpecBmxsimRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// BMX Simulator 2 - Dirt Biking (128K)

static struct BurnRomInfo SpecBmxsim21RomDesc[] = {
	{ "BMX Simulator 2 - Dirt Biking (1989)(Codemasters).z80", 48248, 0xab7d26d5, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecBmxsim21, SpecBmxsim21, Spec128)
STD_ROM_FN(SpecBmxsim21)

struct BurnDriver BurnSpecBmxsim21 = {
	"spec_bmxsim21", NULL, "spec_spec128", NULL, "1989",
	"BMX Simulator 2 - Dirt Biking (128K)\0", NULL, "Codemasters", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 4, HARDWARE_SPECTRUM, GBF_RACING, 0,
	SpectrumGetZipName, SpecBmxsim21RomInfo, SpecBmxsim21RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecCursorKeysDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// BMX Simulator 2 - Quarry Racing (128K)

static struct BurnRomInfo SpecBmxsim22RomDesc[] = {
	{ "BMX Simulator 2 - Quarry Racing (1989)(Codemasters).z80", 48362, 0xd75318bf, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecBmxsim22, SpecBmxsim22, Spec128)
STD_ROM_FN(SpecBmxsim22)

struct BurnDriver BurnSpecBmxsim22 = {
	"spec_bmxsim22", NULL, "spec_spec128", NULL, "1989",
	"BMX Simulator 2 - Quarry Racing (128K)\0", NULL, "Codemasters", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 4, HARDWARE_SPECTRUM, GBF_RACING, 0,
	SpectrumGetZipName, SpecBmxsim22RomInfo, SpecBmxsim22RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecCursorKeysDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Bomber (16K)

static struct BurnRomInfo SpecBomberRomDesc[] = {
	{ "Bomber 16K (1983)(Spectrum Computing).tap", 4346, 0x76e3760f, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecBomber, SpecBomber, Spectrum)
STD_ROM_FN(SpecBomber)

struct BurnDriver BurnSpecBomber = {
	"spec_bomber", NULL, "spec_spectrum", NULL, "1983",
	"Bomber (16K)\0", NULL, "Spectrum Computing", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_ACTION, 0,
	SpectrumGetZipName, SpecBomberRomInfo, SpecBomberRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Bomberun (48K)

static struct BurnRomInfo SpecBomberunRomDesc[] = {
	{ "Bomberun 48K (1984)(DH Software).tap", 18026, 0x5a32b070, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecBomberun, SpecBomberun, Spectrum)
STD_ROM_FN(SpecBomberun)

struct BurnDriver BurnSpecBomberun = {
	"spec_bomberun", NULL, "spec_spectrum", NULL, "1984",
	"Bomberun (48K)\0", NULL, "DH Software", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_ACTION, 0,
	SpectrumGetZipName, SpecBomberunRomInfo, SpecBomberunRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Bombfusion (48K)

static struct BurnRomInfo SpecBombfusionRomDesc[] = {
	{ "Bombfusion 48K (1989)(Virgin Mastertronic).tap", 49726, 0xb7fcbc3b, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecBombfusion, SpecBombfusion, Spectrum)
STD_ROM_FN(SpecBombfusion)

struct BurnDriver BurnSpecBombfusion = {
	"spec_bombfusion", NULL, "spec_spectrum", NULL, "1989",
	"Bombfusion (48K)\0", NULL, "Virgin Mastertronic", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecBombfusionRomInfo, SpecBombfusionRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Bombscare (48K)

static struct BurnRomInfo SpecBombscareRomDesc[] = {
	{ "Bombscare 48K (1986)(Firebird).tap", 43914, 0x3e70f6fa, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecBombscare, SpecBombscare, Spectrum)
STD_ROM_FN(SpecBombscare)

struct BurnDriver BurnSpecBombscare = {
	"spec_bombscare", NULL, "spec_spectrum", NULL, "1986",
	"Bombscare (48K)\0", NULL, "Firebird Software", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_ACTION, 0,
	SpectrumGetZipName, SpecBombscareRomInfo, SpecBombscareRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Bonanza Bros. (128K)

static struct BurnRomInfo SpecBonanzabrosRomDesc[] = {
	{ "Bonanza Bros. 128K (1992)(U.S. Gold).tap", 132823, 0xb99578d6, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecBonanzabros, SpecBonanzabros, Spec128)
STD_ROM_FN(SpecBonanzabros)

struct BurnDriver BurnSpecBonanzabros = {
	"spec_bonanzabros", NULL, "spec_spec128", NULL, "1992",
	"Bonanza Bros. (128K)\0", NULL, "U.S. Gold", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 2, HARDWARE_SPECTRUM, GBF_ACTION, 0,
	SpectrumGetZipName, SpecBonanzabrosRomInfo, SpecBonanzabrosRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Booty (48K)

static struct BurnRomInfo SpecBootyRomDesc[] = {
	{ "Booty 48K (1984)(Firebird).tap", 45734, 0xa387b4bf, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecBooty, SpecBooty, Spectrum)
STD_ROM_FN(SpecBooty)

struct BurnDriver BurnSpecBooty = {
	"spec_booty", NULL, "spec_spectrum", NULL, "1984",
	"Booty (48K)\0", NULL, "Firebird Software", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_ACTION, 0,
	SpectrumGetZipName, SpecBootyRomInfo, SpecBootyRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Bored of the Rings v2 - side A (48K)

static struct BurnRomInfo SpecBringsaRomDesc[] = {
	{ "Bored of the Rings v2 - side A (1985)(Delta 4).tap", 91867, 0x9361bca3, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecBringsa, SpecBringsa, Spectrum)
STD_ROM_FN(SpecBringsa)

struct BurnDriver BurnSpecBringsa = {
	"spec_bringsa", NULL, "spec_spectrum", NULL, "1985",
	"Bored of the Rings v2 - side A (48K)\0", NULL, "Delta 4", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_ADV, 0,
	SpectrumGetZipName, SpecBringsaRomInfo, SpecBringsaRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Bored of the Rings v2 - side B (48K)

static struct BurnRomInfo SpecBringsbRomDesc[] = {
	{ "Bored of the Rings v2 - side B (1985)(Delta 4).tap", 69048, 0x2f287c8a, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecBringsb, SpecBringsb, Spectrum)
STD_ROM_FN(SpecBringsb)

struct BurnDriver BurnSpecBringsb = {
	"spec_bringsb", "spec_bringsa", "spec_spectrum", NULL, "1985",
	"Bored of the Rings v2 - side B (48K)\0", "PSW: Work for money, spend money, spend for love, love for money", "Delta 4", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE, 1, HARDWARE_SPECTRUM, GBF_ADV, 0,
	SpectrumGetZipName, SpecBringsbRomInfo, SpecBringsbRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Bosconian '87 (128K)

static struct BurnRomInfo SpecBosconianRomDesc[] = {
	{ "Bosconian '87 (1987)(Mastertronic).tap", 46840, 0x3cc3e412, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecBosconian, SpecBosconian, Spec128)
STD_ROM_FN(SpecBosconian)

struct BurnDriver BurnSpecBosconian = {
	"spec_bosconian", NULL, "spec_spec128", NULL, "1987",
	"Bosconian '87 (128K)\0", NULL, "Mastertronic", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_SHOOT, 0,
	SpectrumGetZipName, SpecBosconianRomInfo, SpecBosconianRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Bounces (48K)

static struct BurnRomInfo SpecBouncesRomDesc[] = {
	{ "Bounces (1986)(Beyond).z80", 35649, 0xbd312347, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecBounces, SpecBounces, Spectrum)
STD_ROM_FN(SpecBounces)

struct BurnDriver BurnSpecBounces = {
	"spec_bounces", NULL, "spec_spectrum", NULL, "1986",
	"Bounces (48K)\0", NULL, "Beyond", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 2, HARDWARE_SPECTRUM, GBF_BALLPADDLE, 0,
	SpectrumGetZipName, SpecBouncesRomInfo, SpecBouncesRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Bouncing Berty (16K)

static struct BurnRomInfo SpecBouncbertyRomDesc[] = {
	{ "Bouncing Berty 16K (1984)(PowerSoftware).tap", 14436, 0xad0e32a7, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecBouncberty, SpecBouncberty, Spectrum)
STD_ROM_FN(SpecBouncberty)

struct BurnDriver BurnSpecBouncberty = {
	"spec_bouncberty", NULL, "spec_spectrum", NULL, "1984",
	"Bouncing Berty (16K)\0", "Use keys 'Q, A, P, L' to move the character", "PowerSoftware", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_ACTION, 0,
	SpectrumGetZipName, SpecBouncbertyRomInfo, SpecBouncbertyRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Bouncing Bomb (48K)

static struct BurnRomInfo SpecBbombRomDesc[] = {
	{ "Bouncing Bomb 48K (1986)(Phil Ruston).z80", 49216, 0x86f9c286, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecBbomb, SpecBbomb, Spectrum)
STD_ROM_FN(SpecBbomb)

struct BurnDriver BurnSpecBbomb = {
	"spec_bbomb", NULL, "spec_spectrum", NULL, "1986",
	"Bouncing Bomb (48K)\0", "Press a key to start, '9' to move left, '0' to move right", "Phil Ruston", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecBbombRomInfo, SpecBbombRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Bounty Bob Strikes Back! (48K)

static struct BurnRomInfo SpecBountybobRomDesc[] = {
	{ "Bounty Bob Strikes Back! 48K (1984)(U.S. Gold).z80", 45436, 0xde0b2094, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecBountybob, SpecBountybob, Spectrum)
STD_ROM_FN(SpecBountybob)

struct BurnDriver BurnSpecBountybob = {
	"spec_bountybob", NULL, "spec_spectrum", NULL, "1984",
	"Bounty Bob Strikes Back! (48K)\0", NULL, "U.S. Gold", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecBountybobRomInfo, SpecBountybobRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecQAOPMDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Brainstorm (48K)

static struct BurnRomInfo SpecBrainstormRomDesc[] = {
	{ "Brainstorm 48K (1985)(Bubble Bus).tap", 48514, 0xf52bcc89, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecBrainstorm, SpecBrainstorm, Spectrum)
STD_ROM_FN(SpecBrainstorm)

struct BurnDriver BurnSpecBrainstorm = {
	"spec_brainstorm", NULL, "spec_spectrum", NULL, "1985",
	"Brainstorm (48K)\0", NULL, "Bubble Bus Software", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_MAZE | GBF_SHOOT, 0,
	SpectrumGetZipName, SpecBrainstormRomInfo, SpecBrainstormRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// BraveStarr (48K)

static struct BurnRomInfo SpecBravestarrRomDesc[] = {
	{ "BraveStarr (1987)(Go!).tap", 47812, 0xe4da9cc1, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecBravestarr, SpecBravestarr, Spectrum)
STD_ROM_FN(SpecBravestarr)

struct BurnDriver BurnSpecBravestarr = {
	"spec_bravestarr", NULL, "spec_spectrum", NULL, "1987",
	"BraveStarr (48K)\0", NULL, "Go!", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_RUNGUN | GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecBravestarrRomInfo, SpecBravestarrRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Breakthru (48K)

static struct BurnRomInfo SpecBreakthruRomDesc[] = {
	{ "Breakthru 48K (1986)(U.S. Gold).tap", 54622, 0xcd98cbc8, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecBreakthru, SpecBreakthru, Spectrum)
STD_ROM_FN(SpecBreakthru)

struct BurnDriver BurnSpecBreakthru = {
	"spec_breakthru", NULL, "spec_spectrum", NULL, "1986",
	"Breakthru (48K)\0", "Press 'SPACE' to jump", "U.S. Gold", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_HORSHOOT, 0,
	SpectrumGetZipName, SpecBreakthruRomInfo, SpecBreakthruRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Buccaneer (48K)

static struct BurnRomInfo SpecBuccaneerRomDesc[] = {
	{ "Buccaneer 48K (1985)(Firebird).tap", 49414, 0x9fdacef2, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecBuccaneer, SpecBuccaneer, Spectrum)
STD_ROM_FN(SpecBuccaneer)

struct BurnDriver BurnSpecBuccaneer = {
	"spec_buccaneer", NULL, "spec_spectrum", NULL, "1985",
	"Buccaneer (48K)\0", NULL, "Firebird Software", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_HORSHOOT, 0,
	SpectrumGetZipName, SpecBuccaneerRomInfo, SpecBuccaneerRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Budokan - The Martial Spirit (48K)

static struct BurnRomInfo SpecBudokanRomDesc[] = {
	{ "Budokan - The Martial Spirit 48K (1991)(Dro Soft).tap", 110405, 0x7672dc9d, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecBudokan, SpecBudokan, Spectrum)
STD_ROM_FN(SpecBudokan)

struct BurnDriver BurnSpecBudokan = {
	"spec_budokan", NULL, "spec_spectrum", NULL, "1991",
	"Budokan - The Martial Spirit (48K)\0", "Select 'Teclado' to use controller", "Dro Soft", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_VSFIGHT, 0,
	SpectrumGetZipName, SpecBudokanRomInfo, SpecBudokanRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecQAOPSpaceDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Buffalo Bill's Wild West Show (128K)

static struct BurnRomInfo SpecBbwwsRomDesc[] = {
	{ "Buffalo Bill's Wild West Show 128K (1989)(Tynesoft).tap", 131092, 0xfc253b18, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecBbwws, SpecBbwws, Spec128)
STD_ROM_FN(SpecBbwws)

struct BurnDriver BurnSpecBbwws = {
	"spec_bbwws", NULL, "spec_spec128", NULL, "1989",
	"Buffalo Bill's Wild West Show (128K)\0", "Set 'Sinclair port 1' to use controller", "Tynesoft", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 4, HARDWARE_SPECTRUM, GBF_SPORTSMISC, 0,
	SpectrumGetZipName, SpecBbwwsRomInfo, SpecBbwwsRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecIntf2DIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Buggy Blast (48K)

static struct BurnRomInfo SpecBuggyblastRomDesc[] = {
	{ "Buggy Blast 48K (1985)(Firebird).tap", 48493, 0x23a6eda1, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecBuggyblast, SpecBuggyblast, Spectrum)
STD_ROM_FN(SpecBuggyblast)

struct BurnDriver BurnSpecBuggyblast = {
	"spec_buggyblast", NULL, "spec_spectrum", NULL, "1985",
	"Buggy Blast (48K)\0", "Set Cursor Keys 'Y' to use controller", "Firebird Software", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_SHOOT, 0,
	SpectrumGetZipName, SpecBuggyblastRomInfo, SpecBuggyblastRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecCursorKeysDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Buggy Boy (128K)

static struct BurnRomInfo SpecBuggyboyRomDesc[] = {
	{ "Buggy Boy 128K (1988)(Elite Systems).tap", 53512, 0x9f71d593, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecBuggyboy, SpecBuggyboy, Spec128)
STD_ROM_FN(SpecBuggyboy)

struct BurnDriver BurnSpecBuggyboy = {
	"spec_buggyboy", NULL, "spec_spec128", NULL, "1988",
	"Buggy Boy (128K)\0", "Use 'P', 'L' and SPACE to set options", "Elite Systems", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_RACING, 0,
	SpectrumGetZipName, SpecBuggyboyRomInfo, SpecBuggyboyRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Bumpy (48K)

static struct BurnRomInfo SpecBumpyRomDesc[] = {
	{ "Bumpy (1989)(Loriciels).tap", 45878, 0xc79bed54, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecBumpy, SpecBumpy, Spectrum)
STD_ROM_FN(SpecBumpy)

struct BurnDriver BurnSpecBumpy = {
	"spec_bumpy", NULL, "spec_spectrum", NULL, "1989",
	"Bumpy (48K)\0", NULL, "Loriciels", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_ACTION, 0,
	SpectrumGetZipName, SpecBumpyRomInfo, SpecBumpyRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Butcher Hill (128K)

static struct BurnRomInfo SpecButcherhillRomDesc[] = {
	{ "Butcher Hill 128K (1989)(Gremlin Graphics).tap", 143515, 0x20f2abbb, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecButcherhill, SpecButcherhill, Spec128)
STD_ROM_FN(SpecButcherhill)

struct BurnDriver BurnSpecButcherhill = {
	"spec_butcherhill", NULL, "spec_spec128", NULL, "1989",
	"Butcher Hill (128K)\0", NULL, "Gremlin Graphics", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_ACTION | GBF_SHOOT, 0,
	SpectrumGetZipName, SpecButcherhillRomInfo, SpecButcherhillRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecIntf2DIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Cannibal Island (48K)

static struct BurnRomInfo SpecCannislandRomDesc[] = {
	{ "Cannibal Island 48K (1986)(LiveWire Software).tap", 48016, 0x0a6738c4, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecCannisland, SpecCannisland, Spectrum)
STD_ROM_FN(SpecCannisland)

struct BurnDriver BurnSpecCannisland = {
	"spec_cannisland", NULL, "spec_spectrum", NULL, "1986",
	"Cannibal Island (48K)\0", NULL, "LiveWire Software", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_ACTION | GBF_ADV, 0,
	SpectrumGetZipName, SpecCannislandRomInfo, SpecCannislandRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecQAOPSpaceDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Capitan Sevilla (Spanish) (48K)

static struct BurnRomInfo SpecCaptsevillaRomDesc[] = {
	{ "Capitan Sevilla 48K ES (1988)(Dinamic).tap", 97238, 0x8f8c61e7, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecCaptsevilla, SpecCaptsevilla, Spectrum)
STD_ROM_FN(SpecCaptsevilla)

struct BurnDriver BurnSpecCaptsevilla = {
	"spec_captsevilla", NULL, "spec_spectrum", NULL, "1988",
	"Capitan Sevilla (Spanish) (48K)\0", "SPACE: change to Cap. Sevilla (if available)", "Dinamic Software", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecCaptsevillaRomInfo, SpecCaptsevillaRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Captain America Defies the Doom Tube (48K)

static struct BurnRomInfo SpecCaptamericaRomDesc[] = {
	{ "Captain America Defies the Doom Tube 48K (1988)(Go!).z80", 47363, 0x6cb2fdb2, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecCaptamerica, SpecCaptamerica, Spectrum)
STD_ROM_FN(SpecCaptamerica)

struct BurnDriver BurnSpecCaptamerica = {
	"spec_captamerica", NULL, "spec_spectrum", NULL, "1988",
	"Captain America Defies the Doom Tube (48K)\0", NULL, "Go!", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_ACTION | GBF_ADV, 0,
	SpectrumGetZipName, SpecCaptamericaRomInfo, SpecCaptamericaRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Captain Kelly (48K)

static struct BurnRomInfo SpecCapkellyRomDesc[] = {
	{ "Captain Kelly 48K (1986)(Quicksilva).tap", 48724, 0x1aa9f1b9, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecCapkelly, SpecCapkelly, Spectrum)
STD_ROM_FN(SpecCapkelly)

struct BurnDriver BurnSpecCapkelly = {
	"spec_capkelly", NULL, "spec_spectrum", NULL, "1986",
	"Captain Kelly (48K)\0", NULL, "Quicksilva", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_RUNGUN | GBF_MAZE, 0,
	SpectrumGetZipName, SpecCapkellyRomInfo, SpecCapkellyRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Captain Planet and the Planeteers (48K)

static struct BurnRomInfo SpecCaptplanetRomDesc[] = {
	{ "Captain Planet and the Planeteers 48K (1991)(Mindscape International).tap", 61255, 0x0e629f1d, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecCaptplanet, SpecCaptplanet, Spectrum)
STD_ROM_FN(SpecCaptplanet)

struct BurnDriver BurnSpecCaptplanet = {
	"spec_captplanet", NULL, "spec_spectrum", NULL, "1991",
	"Captain Planet and the Planeteers (48K)\0", NULL, "Mindscape International", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_RUNGUN, 0,
	SpectrumGetZipName, SpecCaptplanetRomInfo, SpecCaptplanetRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Carrier Command (128K)

static struct BurnRomInfo SpecCarriercmdRomDesc[] = {
	{ "Carrier Command 128K (1989)(Rainbird).tap", 122111, 0x041d3f43, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecCarriercmd, SpecCarriercmd, Spec128)
STD_ROM_FN(SpecCarriercmd)

struct BurnDriver BurnSpecCarriercmd = {
	"spec_carriercmd", NULL, "spec_spec128", NULL, "1989",
	"Carrier Command (128K)\0", NULL, "Rainbird", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_SIM, 0,
	SpectrumGetZipName, SpecCarriercmdRomInfo, SpecCarriercmdRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Castle Master (48K)

static struct BurnRomInfo SpecCastlemasterRomDesc[] = {
	{ "Castle Master 48K (1990)(Domark).tap", 50816, 0x63e17cd0, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecCastlemaster, SpecCastlemaster, Spectrum)
STD_ROM_FN(SpecCastlemaster)

struct BurnDriver BurnSpecCastlemaster = {
	"spec_castlemaster", NULL, "spec_spectrum", NULL, "1990",
	"Castle Master (48K)\0", NULL, "Domark", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_ADV, 0,
	SpectrumGetZipName, SpecCastlemasterRomInfo, SpecCastlemasterRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Catch 23 (48K)

static struct BurnRomInfo SpecCatch23RomDesc[] = {
	{ "Catch 23 48K (1987)(Martech Games).tap", 54581, 0xbef82e6f, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecCatch23, SpecCatch23, Spectrum)
STD_ROM_FN(SpecCatch23)

struct BurnDriver BurnSpecCatch23 = {
	"spec_catch23", NULL, "spec_spectrum", NULL, "1987",
	"Catch 23 (48K)\0", NULL, "Martech Games", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_SHOOT | GBF_ADV, 0,
	SpectrumGetZipName, SpecCatch23RomInfo, SpecCatch23RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Cavelon (48K)

static struct BurnRomInfo SpecCavelonRomDesc[] = {
	{ "Cavelon 48K (1984)(Ocean).tap", 33603, 0xdf2fa332, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecCavelon, SpecCavelon, Spectrum)
STD_ROM_FN(SpecCavelon)

struct BurnDriver BurnSpecCavelon = {
	"spec_cavelon", NULL, "spec_spectrum", NULL, "1984",
	"Cavelon (48K)\0", NULL, "Ocean", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_ACTION | GBF_MAZE, 0,
	SpectrumGetZipName, SpecCavelonRomInfo, SpecCavelonRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Cavemania (48K)

static struct BurnRomInfo SpecCavemaniaRomDesc[] = {
	{ "Cavemania 48K (1990)(Atlantis Software).tap", 48245, 0x156c837d, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecCavemania, SpecCavemania, Spectrum)
STD_ROM_FN(SpecCavemania)

struct BurnDriver BurnSpecCavemania = {
	"spec_cavemania", NULL, "spec_spectrum", NULL, "1990",
	"Cavemania (48K)\0", NULL, "Atlantis Software", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecCavemaniaRomInfo, SpecCavemaniaRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Centi-Bug (16K)

static struct BurnRomInfo SpecCentibugRomDesc[] = {
	{ "Centi-Bug 16K (1983)(DK'Tronics).z80", 9769, 0x8c90f40d, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecCentibug, SpecCentibug, Spectrum)
STD_ROM_FN(SpecCentibug)

struct BurnDriver BurnSpecCentibug = {
	"spec_centibug", NULL, "spec_spectrum", NULL, "1983",
	"Centi-Bug - aka Centipede (16K)\0", "Press 'K' to select Kempston joystick", "DK'Tronics", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_SHOOT, 0,
	SpectrumGetZipName, SpecCentibugRomInfo, SpecCentibugRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Cerius (48K)

static struct BurnRomInfo SpecCeriusRomDesc[] = {
	{ "Cerius 48K (1988)(Atlantis Software).tap", 56277, 0xb5a41733, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecCerius, SpecCerius, Spectrum)
STD_ROM_FN(SpecCerius)

struct BurnDriver BurnSpecCerius = {
	"spec_cerius", NULL, "spec_spectrum", NULL, "1988",
	"Cerius (48K)\0", NULL, "Atlantis Software", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_RUNGUN | GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecCeriusRomInfo, SpecCeriusRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Championship 3D Snooker (48K)

static struct BurnRomInfo SpecCh3dsnookerRomDesc[] = {
	{ "Championship 3D Snooker 48K (1992)(Zeppelin Games).tap", 29038, 0x2d955668, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecCh3dsnooker, SpecCh3dsnooker, Spectrum)
STD_ROM_FN(SpecCh3dsnooker)

struct BurnDriver BurnSpecCh3dsnooker = {
	"spec_ch3dsnooker", NULL, "spec_spectrum", NULL, "1992",
	"Championship 3D Snooker (48K)\0", NULL, "Zeppelin Games", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 2, HARDWARE_SPECTRUM, GBF_SPORTSMISC, 0,
	SpectrumGetZipName, SpecCh3dsnookerRomInfo, SpecCh3dsnookerRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecIntf2DIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Championship Run (48K)

static struct BurnRomInfo SpecChamprunRomDesc[] = {
	{ "Championship Run 48K (1991)(Impulze).tap", 46687, 0x08eeabd6, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecChamprun, SpecChamprun, Spectrum)
STD_ROM_FN(SpecChamprun)

struct BurnDriver BurnSpecChamprun = {
	"spec_champrun", NULL, "spec_spectrum", NULL, "1991",
	"Championship Run (48K)\0", NULL, "Impulze", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_RACING, 0,
	SpectrumGetZipName, SpecChamprunRomInfo, SpecChamprunRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecIntf2DIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Chaos - The Battle of Wizards (48K)

static struct BurnRomInfo SpecChaosRomDesc[] = {
	{ "Chaos - The Battle of Wizards 48K (1985)(Games Workshop).tap", 41640, 0xa696e01a, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecChaos, SpecChaos, Spectrum)
STD_ROM_FN(SpecChaos)

struct BurnDriver BurnSpecChaos = {
	"spec_chaos", NULL, "spec_spectrum", NULL, "1985",
	"Chaos - The Battle of Wizards (48K)\0", "Q, W, E, A, D, Z, X, C, to move cursor", "Games Workshop", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 8, HARDWARE_SPECTRUM, GBF_STRATEGY, 0,
	SpectrumGetZipName, SpecChaosRomInfo, SpecChaosRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Chevy Chase (128K)

static struct BurnRomInfo SpecChevychaseRomDesc[] = {
	{ "Chevy Chase 128K (1991)(Hi-Tec).z80", 85403, 0x6d88b22c, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecChevychase, SpecChevychase, Spec128)
STD_ROM_FN(SpecChevychase)

struct BurnDriver BurnSpecChevychase = {
	"spec_chevychase", NULL, "spec_spec128", NULL, "1991",
	"Chevy Chase (128K)\0", NULL, "Hi-Tec Software", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_RACING, 0,
	SpectrumGetZipName, SpecChevychaseRomInfo, SpecChevychaseRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecIntf2DIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Chichen Itza - Part 1 (Spanish) (48K)

static struct BurnRomInfo SpecChichenitza1RomDesc[] = {
	{ "Chichen Itza - Part 1 ES (1992)(Aventuras AD).tap", 48166, 0x14e54546, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecChichenitza1, SpecChichenitza1, Spectrum)
STD_ROM_FN(SpecChichenitza1)

struct BurnDriver BurnSpecChichenitza1 = {
	"spec_chichenitza1", NULL, "spec_spectrum", NULL, "1992",
	"Chichen Itza - Part 1 (Spanish) (48K)\0", NULL, "Aventuras A.D.", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_ADV, 0,
	SpectrumGetZipName, SpecChichenitza1RomInfo, SpecChichenitza1RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Chichen Itza - Part 2 (Spanish) (48K)

static struct BurnRomInfo SpecChichenitza2RomDesc[] = {
	{ "Chichen Itza - Part 2 ES (1992)(Aventuras AD).tap", 48166, 0x0a229edb, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecChichenitza2, SpecChichenitza2, Spectrum)
STD_ROM_FN(SpecChichenitza2)

struct BurnDriver BurnSpecChichenitza2 = {
	"spec_chichenitza2", "spec_chichenitza1", "spec_spectrum", NULL, "1992",
	"Chichen Itza - Part 2 (Spanish) (48K)\0", "Password: RICO PAPASITO", "Aventuras A.D.", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE, 1, HARDWARE_SPECTRUM, GBF_ADV, 0,
	SpectrumGetZipName, SpecChichenitza2RomInfo, SpecChichenitza2RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Chickin Chase (48K)

static struct BurnRomInfo SpecCknchaseRomDesc[] = {
	{ "Chickin Chase 48K (1985)(Firebird).z80", 39512, 0xbf5ace62, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecCknchase, SpecCknchase, Spectrum)
STD_ROM_FN(SpecCknchase)

struct BurnDriver BurnSpecCknchase = {
	"spec_cknchase", NULL, "spec_spectrum", NULL, "1985",
	"Chickin Chase (48K)\0", NULL, "Firebird Software", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_ACTION, 0,
	SpectrumGetZipName, SpecCknchaseRomInfo, SpecCknchaseRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecCursorKeysDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Chiller (48K)

static struct BurnRomInfo SpecChillerRomDesc[] = {
	{ "Chiller 48K (1985)(Mastertronic).tap", 23016, 0x8ade8ac6, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecChiller, SpecChiller, Spectrum)
STD_ROM_FN(SpecChiller)

struct BurnDriver BurnSpecChiller = {
	"spec_chiller", NULL, "spec_spectrum", NULL, "1985",
	"Chiller (48K)\0", NULL, "Mastertronic", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecChillerRomInfo, SpecChillerRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Chinese Juggler, The (48K)

static struct BurnRomInfo SpecChinjugglerRomDesc[] = {
	{ "Chinese Juggler, The 48K (1984)(Ocean).tap", 44449, 0xc5956b3f, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecChinjuggler, SpecChinjuggler, Spectrum)
STD_ROM_FN(SpecChinjuggler)

struct BurnDriver BurnSpecChinjuggler = {
	"spec_chinjuggler", NULL, "spec_spectrum", NULL, "1984",
	"Chinese Juggler, The (48K)\0", "Press 'J' to use controller", "Ocean", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_ACTION, 0,
	SpectrumGetZipName, SpecChinjugglerRomInfo, SpecChinjugglerRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Chip's Challenge (128K)

static struct BurnRomInfo SpecChipchalRomDesc[] = {
	{ "Chip's Challenge 128K (1990)(U.S. Gold)(128K).tap", 142333, 0xfcb9eb9a, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecChipchal, SpecChipchal, Spec128)
STD_ROM_FN(SpecChipchal)

struct BurnDriver BurnSpecChipchal = {
	"spec_chipchal", NULL, "spec_spec128", NULL, "1990",
	"Chip's Challenge (128K)\0", NULL, "U.S. Gold", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_PUZZLE, 0,
	SpectrumGetZipName, SpecChipchalRomInfo, SpecChipchalRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecIntf2DIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Chubby Gristle (128K)

static struct BurnRomInfo SpecChubbyRomDesc[] = {
	{ "Chubby Gristle 128K (1988)(Grandslam).tap", 52414, 0x46a7bdd3, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecChubby, SpecChubby, Spec128)
STD_ROM_FN(SpecChubby)

struct BurnDriver BurnSpecChubby = {
	"spec_chubby", NULL, "spec_spec128", NULL, "1988",
	"Chubby Gristle (128K)\0", NULL, "Grandslam Entertainments", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecChubbyRomInfo, SpecChubbyRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Circus Games (48K)

static struct BurnRomInfo SpecCircusgamesRomDesc[] = {
	{ "Circus Games 48K (1988)(Tynesoft).tap", 151897, 0x4d953275, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecCircusgames, SpecCircusgames, Spectrum)
STD_ROM_FN(SpecCircusgames)

struct BurnDriver BurnSpecCircusgames = {
	"spec_circusgames", NULL, "spec_spectrum", NULL, "1988",
	"Circus Games (48K)\0", NULL, "Tynesoft", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 2, HARDWARE_SPECTRUM, GBF_ACTION, 0,
	SpectrumGetZipName, SpecCircusgamesRomInfo, SpecCircusgamesRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecQAOPSpaceDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// City Connection (128K)

static struct BurnRomInfo SpecCityconn128RomDesc[] = {
	{ "City Connection 128K (1988-2023)(Manuel Lemos).tap", 112933, 0xba5fd65a, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecCityconn128, SpecCityconn128, Spec128)
STD_ROM_FN(SpecCityconn128)

struct BurnDriver BurnSpecCityconn128 = {
	"spec_cityconn128", NULL, "spec_spec128", NULL, "2023",
	"City Connection (128K)\0", NULL, "Manuel Lemos", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 2, HARDWARE_SPECTRUM, GBF_ACTION, 0,
	SpectrumGetZipName, SpecCityconn128RomInfo, SpecCityconn128RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// City Connection (48K)

static struct BurnRomInfo SpecCityconn48RomDesc[] = {
	{ "City Connection 48K (1988)(Manuel Lemos).z80", 41555, 0xcbd6d656, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecCityconn48, SpecCityconn48, Spectrum)
STD_ROM_FN(SpecCityconn48)

struct BurnDriver BurnSpecCityconn48 = {
	"spec_cityconn48", "spec_cityconn128", "spec_spectrum", NULL, "1988",
	"City Connection (48K)\0", NULL, "Manuel Lemos", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE, 2, HARDWARE_SPECTRUM, GBF_ACTION, 0,
	SpectrumGetZipName, SpecCityconn48RomInfo, SpecCityconn48RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Cobra (48K)

static struct BurnRomInfo SpecCobraRomDesc[] = {
	{ "Cobra 48K (1986)(Ocean).z80", 42218, 0x28a20fe7, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecCobra, SpecCobra, Spectrum)
STD_ROM_FN(SpecCobra)

struct BurnDriver BurnSpecCobra = {
	"spec_cobra", NULL, "spec_spectrum", NULL, "1986",
	"Cobra (48K)\0", NULL, "Ocean", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_PLATFORM | GBF_RUNGUN, 0,
	SpectrumGetZipName, SpecCobraRomInfo, SpecCobraRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Cobra's Arc (Spanish) (48K)

static struct BurnRomInfo SpecCobrarcRomDesc[] = {
	{ "Cobra's Arc ES (1986)(Dinamic).tap", 49106, 0xed2e3bb4, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecCobrarc, SpecCobrarc, Spectrum)
STD_ROM_FN(SpecCobrarc)

struct BurnDriver BurnSpecCobrarc = {
	"spec_cobrarc", NULL, "spec_spectrum", NULL, "1986",
	"Cobra's Arc (Spanish) (48K)\0", NULL, "Dinamic Software", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_ADV, 0,
	SpectrumGetZipName, SpecCobrarcRomInfo, SpecCobrarcRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Colossus 4 Chess (48K)

static struct BurnRomInfo SpecC4chessRomDesc[] = {
	{ "Colossus 4 Chess 48K (1986)(CDS Microsystems).tap", 43369, 0xe0f45f59, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecC4chess, SpecC4chess, Spectrum)
STD_ROM_FN(SpecC4chess)

struct BurnDriver BurnSpecC4chess = {
	"spec_c4chess", NULL, "spec_spectrum", NULL, "1986",
	"Colossus 4 Chess (48K)\0", NULL, "CDS Microsystems", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_BOARD | GBF_STRATEGY, 0,
	SpectrumGetZipName, SpecC4chessRomInfo, SpecC4chessRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Colour of Magic, The (48K)

static struct BurnRomInfo SpecColmagicRomDesc[] = {
	{ "Colour of Magic, The 48K (1986)(Piranha).tap", 138647, 0x43a002ab, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecColmagic, SpecColmagic, Spectrum)
STD_ROM_FN(SpecColmagic)

struct BurnDriver BurnSpecColmagic = {
	"spec_colmagic", NULL, "spec_spectrum", NULL, "1986",
	"Colour of Magic, The (48K)\0", NULL, "Piranha", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_ADV, 0,
	SpectrumGetZipName, SpecColmagicRomInfo, SpecColmagicRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Comando Tracer (Spanish) (48K-128K)

static struct BurnRomInfo SpecCotracerRomDesc[] = {
	{ "Comando Tracer (1989)(Dinamic).tap", 41721, 0xd394ef8c, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecCotracer, SpecCotracer, Spec128)
STD_ROM_FN(SpecCotracer)

struct BurnDriver BurnSpecCotracer = {
	"spec_cotracer", NULL, "spec_spec128", NULL, "1988",
	"Comando Tracer (Spanish) (48K-128K)\0", NULL, "Dinamic Software", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_HORSHOOT, 0,
	SpectrumGetZipName, SpecCotracerRomInfo, SpecCotracerRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Combat Zone (48K)

static struct BurnRomInfo SpecCombzoneRomDesc[] = {
	{ "Combat Zone 48K (1983)(Artic Computing).tap", 17181, 0x5296e736, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecCombzone, SpecCombzone, Spectrum)
STD_ROM_FN(SpecCombzone)

struct BurnDriver BurnSpecCombzone = {
	"spec_combzone", NULL, "spec_spectrum", NULL, "1983",
	"Combat Zone (48K)\0", NULL, "Artic Computing", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_SHOOT, 0,
	SpectrumGetZipName, SpecCombzoneRomInfo, SpecCombzoneRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Comet Encounter (48K)

static struct BurnRomInfo SpecCometencRomDesc[] = {
	{ "Comet Encounter 48K (1986)(LiveWire).tap", 42481, 0x0d5b1212, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecCometenc, SpecCometenc, Spectrum)
STD_ROM_FN(SpecCometenc)

struct BurnDriver BurnSpecCometenc = {
	"spec_cometenc", NULL, "spec_spectrum", NULL, "1986",
	"Comet Encounter (48K)\0", NULL, "LiveWire", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_ACTION, 0,
	SpectrumGetZipName, SpecCometencRomInfo, SpecCometencRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Confuzion (48K)

static struct BurnRomInfo SpecConfuzionRomDesc[] = {
	{ "Confuzion (1985)(Incentive).tap", 49343, 0x9d0f6c6c, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecConfuzion, SpecConfuzion, Spectrum)
STD_ROM_FN(SpecConfuzion)

struct BurnDriver BurnSpecConfuzion = {
	"spec_confuzion", NULL, "spec_spectrum", NULL, "1985",
	"Confuzion (48K)\0", "Press R to invert commands", "Incentive", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_PUZZLE, 0,
	SpectrumGetZipName, SpecConfuzionRomInfo, SpecConfuzionRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Con-Quest (48K)

static struct BurnRomInfo SpecConquestRomDesc[] = {
	{ "Con-Quest 48K (1986)(Mastertronic).tap", 48762, 0x77e76fe8, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecConquest, SpecConquest, Spectrum)
STD_ROM_FN(SpecConquest)

struct BurnDriver BurnSpecConquest = {
	"spec_conquest", NULL, "spec_spectrum", NULL, "1986",
	"Con-Quest (48K)\0", NULL, "Mastertronic Added Dimension", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_MAZE | GBF_ADV, 0,
	SpectrumGetZipName, SpecConquestRomInfo, SpecConquestRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Contact Sam Cruise (48K)

static struct BurnRomInfo SpecContscruiseRomDesc[] = {
	{ "Contact Sam Cruise (1986)(Microsphere).tap", 56267, 0x3175c3e3, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecContscruise, SpecContscruise, Spectrum)
STD_ROM_FN(SpecContscruise)

struct BurnDriver BurnSpecContscruise = {
	"spec_contscruise", NULL, "spec_spectrum", NULL, "1986",
	"Contact Sam Cruise (48K)\0", "After the game has started, press 'J' and 'K' to use controller", "Microsphere", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_ADV, 0,
	SpectrumGetZipName, SpecContscruiseRomInfo, SpecContscruiseRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Cop-Out (48K)

static struct BurnRomInfo SpecCopoutRomDesc[] = {
	{ "Cop-Out 48K (1986)(Mikro-Gen).tap", 49468, 0x27dd134e, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecCopout, SpecCopout, Spectrum)
STD_ROM_FN(SpecCopout)

struct BurnDriver BurnSpecCopout = {
	"spec_copout", NULL, "spec_spectrum", NULL, "1986",
	"Cop-Out (48K)\0", NULL, "Mikro-Gen", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_SHOOT, 0,
	SpectrumGetZipName, SpecCopoutRomInfo, SpecCopoutRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// C.O.R.E. (48K)

static struct BurnRomInfo SpecCoreRomDesc[] = {
	{ "C.O.R.E. (1986)(A'n'F Software).tap", 48919, 0x716221c7, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecCore, SpecCore, Spectrum)
STD_ROM_FN(SpecCore)

struct BurnDriver BurnSpecCore = {
	"spec_core", NULL, "spec_spectrum", NULL, "1986",
	"C.O.R.E. (48K)\0", NULL, "A'n'F Software", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_ACTION | GBF_ADV, 0,
	SpectrumGetZipName, SpecCoreRomInfo, SpecCoreRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Cosmic Sheriff (Spanish) (48K)

static struct BurnRomInfo SpecCsheriffRomDesc[] = {
	{ "Cosmic Sheriff 48K ES (1990)(Dinamic).tap", 43690, 0x3bee0827, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecCsheriff, SpecCsheriff, Spectrum)
STD_ROM_FN(SpecCsheriff)

struct BurnDriver BurnSpecCsheriff = {
	"spec_csheriff", NULL, "spec_spectrum", NULL, "1990",
	"Cosmic Sheriff (Spanish) (48K)\0", "ENTER: quit game, come back to options screen", "Dinamic Software", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_SHOOT, 0,
	SpectrumGetZipName, SpecCsheriffRomInfo, SpecCsheriffRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Costa Capers (48K)

static struct BurnRomInfo SpecCostacapersRomDesc[] = {
	{ "Costa Capers 48K (1985)(Firebird).z80", 42187, 0x66258bbc, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecCostacapers, SpecCostacapers, Spectrum)
STD_ROM_FN(SpecCostacapers)

struct BurnDriver BurnSpecCostacapers = {
	"spec_costacapers", NULL, "spec_spectrum", NULL, "1985",
	"Costa Capers (48K)\0", NULL, "Firebird Software", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecCostacapersRomInfo, SpecCostacapersRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Count Duckula II (128K)

static struct BurnRomInfo SpecCountduck2128RomDesc[] = {
	{ "Count Duckula 2 128K (1992)(Alternative).tap", 96837, 0xe5c979e1, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecCountduck2128, SpecCountduck2128, Spec128)
STD_ROM_FN(SpecCountduck2128)

struct BurnDriver BurnSpecCountduck2128 = {
	"spec_countduck2128", NULL, "spec_spec128", NULL, "1992",
	"Count Duckula II (128K)\0", NULL, "Alternative Software", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_VERSHOOT | GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecCountduck2128RomInfo, SpecCountduck2128RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecIntf2DIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Count Duckula II (48K)

static struct BurnRomInfo SpecCountduck248RomDesc[] = {
	{ "Count Duckula 2 48K (1992)(Alternative).tap", 47605, 0x7b53965d, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecCountduck248, SpecCountduck248, Spectrum)
STD_ROM_FN(SpecCountduck248)

struct BurnDriver BurnSpecCountduck248 = {
	"spec_countduck248", "spec_countduck2128", "spec_spectrum", NULL, "1992",
	"Count Duckula II (48K)\0", NULL, "Alternative Software", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecCountduck248RomInfo, SpecCountduck248RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecIntf2DIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Count Duckula in No Sax Please - We're Egyptian (48K)

static struct BurnRomInfo SpecCountduckulaRomDesc[] = {
	{ "Count Duckula in No Sax Please 48K (1989)(Alternative).tap", 48615, 0x2185ec3a, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecCountduckula, SpecCountduckula, Spectrum)
STD_ROM_FN(SpecCountduckula)

struct BurnDriver BurnSpecCountduckula = {
	"spec_countduckula", NULL, "spec_spectrum", NULL, "1989",
	"Count Duckula in No Sax Please - We're Egyptian (48K)\0", NULL, "Alternative Software", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecCountduckulaRomInfo, SpecCountduckulaRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Cowboy Shootout (16K)

static struct BurnRomInfo SpecCbshootoutRomDesc[] = {
	{ "Cowboy Shootout 16K (1983)(Micro Power).tap", 8879, 0x274ecc03, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecCbshootout, SpecCbshootout, Spectrum)
STD_ROM_FN(SpecCbshootout)

struct BurnDriver BurnSpecCbshootout = {
	"spec_cbshootout", NULL, "spec_spectrum", NULL, "1983",
	"Cowboy Shootout (16K)\0", NULL, "Micro Power", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 2, HARDWARE_SPECTRUM, GBF_SHOOT, 0,
	SpectrumGetZipName, SpecCbshootoutRomInfo, SpecCbshootoutRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Crack Down (128K)

static struct BurnRomInfo SpecCrackdownRomDesc[] = {
	{ "Crack Down 128K (1990)(U.S. Gold).tap", 189590, 0xc82c010e, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecCrackdown, SpecCrackdown, Spec128)
STD_ROM_FN(SpecCrackdown)

struct BurnDriver BurnSpecCrackdown = {
	"spec_crackdown", NULL, "spec_spec128", NULL, "1990",
	"Crack Down (128K)\0", NULL, "U.S. Gold", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 2, HARDWARE_SPECTRUM, GBF_ACTION, 0,
	SpectrumGetZipName, SpecCrackdownRomInfo, SpecCrackdownRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecIntf2DIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Crazy Cars (128K)

static struct BurnRomInfo SpecCrazycarsRomDesc[] = {
	{ "Crazy Cars (1988)(Titus).tap", 44388, 0xe65d2bd3, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecCrazycars, SpecCrazycars, Spec128)
STD_ROM_FN(SpecCrazycars)

struct BurnDriver BurnSpecCrazycars = {
	"spec_crazycars", NULL, "spec_spec128", NULL, "1988",
	"Crazy Cars (128K)\0", NULL, "Titus", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_RACING, 0,
	SpectrumGetZipName, SpecCrazycarsRomInfo, SpecCrazycarsRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Crazy Cars II (128K)

static struct BurnRomInfo SpecCrazycars2RomDesc[] = {
	{ "Crazy Cars II (1988)(Titus).tap", 41208, 0x0b4a0be1, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecCrazycars2, SpecCrazycars2, Spec128)
STD_ROM_FN(SpecCrazycars2)

struct BurnDriver BurnSpecCrazycars2 = {
	"spec_crazycars2", NULL, "spec_spec128", NULL, "1988",
	"Crazy Cars II (128K)\0", NULL, "Titus", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_RACING, 0,
	SpectrumGetZipName, SpecCrazycars2RomInfo, SpecCrazycars2RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Crazy House (48K)

static struct BurnRomInfo SpecCrazyhouseRomDesc[] = {
	{ "Crazy House (1990)(Bulldog).z80", 18580, 0x876275b0, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecCrazyhouse, SpecCrazyhouse, Spectrum)
STD_ROM_FN(SpecCrazyhouse)

struct BurnDriver BurnSpecCrazyhouse = {
	"spec_crazyhouse", NULL, "spec_spectrum", NULL, "1990",
	"Crazy House (48K)\0", "Set 'Cursor' to use controller", "Bulldog Software", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_ACTION, 0,
	SpectrumGetZipName, SpecCrazyhouseRomInfo, SpecCrazyhouseRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecCursorKeysDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Crime Busters Featuring Bennie the Burglar (48K)

static struct BurnRomInfo SpecCrimebustRomDesc[] = {
	{ "Crime Busters Featuring Bennie the Burglar (1988)(Players Software).tap", 88832, 0x6fa4d165, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecCrimebust, SpecCrimebust, Spectrum)
STD_ROM_FN(SpecCrimebust)

struct BurnDriver BurnSpecCrimebust = {
	"spec_crimebust", NULL, "spec_spectrum", NULL, "1988",
	"Crime Busters Featuring Bennie the Burglar (48K)\0", "'Q', 'W' and 'Space' to set options", "Players Software", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecCrimebustRomInfo, SpecCrimebustRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Critical Mass (48K)

static struct BurnRomInfo SpecCriticmassRomDesc[] = {
	{ "Critical Mass 48K (1985)(Durell).tap", 44720, 0xba99bc3e, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecCriticmass, SpecCriticmass, Spectrum)
STD_ROM_FN(SpecCriticmass)

struct BurnDriver BurnSpecCriticmass = {
	"spec_criticmass", NULL, "spec_spectrum", NULL, "1985",
	"Critical Mass (48K)\0", NULL, "Durell Software", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_SHOOT, 0,
	SpectrumGetZipName, SpecCriticmassRomInfo, SpecCriticmassRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Curse of Sherwood, The (48K)

static struct BurnRomInfo SpecSherwoodRomDesc[] = {
	{ "Curse of Sherwood, The (1987)(Mastertronic).z80", 39360, 0x72701ae3, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecSherwood, SpecSherwood, Spectrum)
STD_ROM_FN(SpecSherwood)

struct BurnDriver BurnSpecSherwood = {
	"spec_sherwood", NULL, "spec_spectrum", NULL, "1987",
	"Curse of Sherwood, The (48K)\0", NULL, "Mastertronic", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_ACTION | GBF_ADV, 0,
	SpectrumGetZipName, SpecSherwoodRomInfo, SpecSherwoodRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Cyberun (48K)

static struct BurnRomInfo SpecCyberunRomDesc[] = {
	{ "Cyberun 48K (1986)(Ultimate Play The Game).tap", 47506, 0x8ffbbf90, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecCyberun, SpecCyberun, Spectrum)
STD_ROM_FN(SpecCyberun)

struct BurnDriver BurnSpecCyberun = {
	"spec_cyberun", NULL, "spec_spectrum", NULL, "1986",
	"Cyberun (48K)\0", "Press '1' or '2' to set controls", "Ultimate Play The Game", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 2, HARDWARE_SPECTRUM, GBF_HORSHOOT, 0,
	SpectrumGetZipName, SpecCyberunRomInfo, SpecCyberunRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Cyclone (48K)

static struct BurnRomInfo SpecCycloneRomDesc[] = {
	{ "Cyclone 48K (1985)(Vortex).tap", 48533, 0x980d9d6e, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecCyclone, SpecCyclone, Spectrum)
STD_ROM_FN(SpecCyclone)

struct BurnDriver BurnSpecCyclone = {
	"spec_cyclone", NULL, "spec_spectrum", NULL, "1985",
	"Cyclone (48K)\0", NULL, "Vortex Software", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_ACTION, 0,
	SpectrumGetZipName, SpecCycloneRomInfo, SpecCycloneRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Cyrus II Chess (128K)

static struct BurnRomInfo SpecCyrus2RomDesc[] = {
	{ "Cyrus II Chess 128K (1986)(Alligata).tap", 39926, 0x2eee30a5, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecCyrus2, SpecCyrus2, Spec128)
STD_ROM_FN(SpecCyrus2)

struct BurnDriver BurnSpecCyrus2 = {
	"spec_cyrus2", NULL, "spec_spec128", NULL, "1986",
	"Cyrus II Chess (128K)\0", "SHIFT + 'cursor keys': move marker - ENTER: select a piece", "Alligata Software", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 2, HARDWARE_SPECTRUM, GBF_STRATEGY | GBF_BOARD, 0,
	SpectrumGetZipName, SpecCyrus2RomInfo, SpecCyrus2RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecCursorKeysDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Dalek Attack - Doctor Who (Trainer) (128K)

static struct BurnRomInfo SpecDalekattackRomDesc[] = {
	{ "Dalek Attack (128K) (1992)(Admiral)(Trainer).tap", 119370, 0x5b3cd868, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecDalekattack, SpecDalekattack, Spec128)
STD_ROM_FN(SpecDalekattack)

struct BurnDriver BurnSpecDalekattack = {
	"spec_dalekattack", NULL, "spec_spec128", NULL, "1993",
	"Dalek Attack - Doctor Who (Trainer) (128K)\0", NULL, "Admiral Software", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 2, HARDWARE_SPECTRUM, GBF_PLATFORM | GBF_ADV, 0,
	SpectrumGetZipName, SpecDalekattackRomInfo, SpecDalekattackRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Dam Busters, The (48K)

static struct BurnRomInfo SpecDambustersRomDesc[] = {
	{ "Dam Busters, The 48K (1985)(U.S. Gold).tap", 48993, 0xc43163e6, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecDambusters, SpecDambusters, Spectrum)
STD_ROM_FN(SpecDambusters)

struct BurnDriver BurnSpecDambusters = {
	"spec_dambusters", NULL, "spec_spectrum", NULL, "1985",
	"Dam Busters, The (48K)\0", NULL, "U.S. Gold", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_SIM, 0,
	SpectrumGetZipName, SpecDambustersRomInfo, SpecDambustersRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Danger Mouse in Double Trouble (48K)

static struct BurnRomInfo SpecDangmouseRomDesc[] = {
	{ "Danger Mouse in Double Trouble 48K (1984)(Creative Sparks).z80", 44510, 0x691c3c67, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecDangmouse, SpecDangmouse, Spectrum)
STD_ROM_FN(SpecDangmouse)

struct BurnDriver BurnSpecDangmouse = {
	"spec_dangmouse", NULL, "spec_spectrum", NULL, "1984",
	"Danger Mouse in Double Trouble (48K)\0", NULL, "Creative Sparks", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_ACTION, 0,
	SpectrumGetZipName, SpecDangmouseRomInfo, SpecDangmouseRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Dark Sceptre (48K)

static struct BurnRomInfo SpecDarksceptreRomDesc[] = {
	{ "Dark Sceptre 48K (1987)(Firebird).tap", 54587, 0x364d1c34, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecDarksceptre, SpecDarksceptre, Spectrum)
STD_ROM_FN(SpecDarksceptre)

struct BurnDriver BurnSpecDarksceptre = {
	"spec_darksceptre", NULL, "spec_spectrum", NULL, "1987",
	"Dark Sceptre (48K)\0", NULL, "Firebird Software", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_STRATEGY, 0,
	SpectrumGetZipName, SpecDarksceptreRomInfo, SpecDarksceptreRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Days of Thunder (128K)

static struct BurnRomInfo SpecDaysthunderRomDesc[] = {
	{ "Days of Thunder 128K (1990)(Mindscape International).tap", 95028, 0x954ba9b3, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecDaysthunder, SpecDaysthunder, Spec128)
STD_ROM_FN(SpecDaysthunder)

struct BurnDriver BurnSpecDaysthunder = {
	"spec_daysthunder", NULL, "spec_spec128", NULL, "1990",
	"Days of Thunder (128K)\0", NULL, "Mindscape International", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_RACING, 0,
	SpectrumGetZipName, SpecDaysthunderRomInfo, SpecDaysthunderRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Death Pit (48K)

static struct BurnRomInfo SpecDeathpitRomDesc[] = {
	{ "Death Pit 48K (1985)(Durell).tap", 47673, 0x9ee83a80, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecDeathpit, SpecDeathpit, Spectrum)
STD_ROM_FN(SpecDeathpit)

struct BurnDriver BurnSpecDeathpit = {
	"spec_deathpit", NULL, "spec_spectrum", NULL, "1985",
	"Death Pit (48K)\0", NULL, "Durell Software", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_ACTION, 0,
	SpectrumGetZipName, SpecDeathpitRomInfo, SpecDeathpitRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Deathscape (48K)

static struct BurnRomInfo SpecDeathscapeRomDesc[] = {
	{ "Deathscape 48K (1987)(Starlight Software).tap", 48009, 0x72540c22, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecDeathscape, SpecDeathscape, Spectrum)
STD_ROM_FN(SpecDeathscape)

struct BurnDriver BurnSpecDeathscape = {
	"spec_deathscape", NULL, "spec_spectrum", NULL, "1987",
	"Deathscape (48K)\0", NULL, "Starlight Software", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_SHOOT, 0,
	SpectrumGetZipName, SpecDeathscapeRomInfo, SpecDeathscapeRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Death Star Interceptor (48K)

static struct BurnRomInfo SpecDstarintRomDesc[] = {
	{ "Death Star Interceptor 48K (1985)(System 3).tap", 50982, 0xc1e535f8, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecDstarint, SpecDstarint, Spectrum)
STD_ROM_FN(SpecDstarint)

struct BurnDriver BurnSpecDstarint = {
	"spec_dstarint", NULL, "spec_spectrum", NULL, "1985",
	"Death Star Interceptor (48K)\0", NULL, "System 3", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_SHOOT, 0,
	SpectrumGetZipName, SpecDstarintRomInfo, SpecDstarintRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Deep, The (48K)

static struct BurnRomInfo SpecDeepRomDesc[] = {
	{ "Deep, The 48K (1988)(U.S. Gold).tap", 30931, 0xac4f747e, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecDeep, SpecDeep, Spectrum)
STD_ROM_FN(SpecDeep)

struct BurnDriver BurnSpecDeep = {
	"spec_deep", NULL, "spec_spectrum", NULL, "1988",
	"Deep, The (48K)\0", NULL, "U.S. Gold", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_HORSHOOT, 0,
	SpectrumGetZipName, SpecDeepRomInfo, SpecDeepRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Deep Strike (48K)

static struct BurnRomInfo SpecDeepstrikeRomDesc[] = {
	{ "Deep Strike 48K (1986)(Durell).tap", 44086, 0x9af55985, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecDeepstrike, SpecDeepstrike, Spectrum)
STD_ROM_FN(SpecDeepstrike)

struct BurnDriver BurnSpecDeepstrike = {
	"spec_deepstrike", NULL, "spec_spectrum", NULL, "1986",
	"Deep Strike (48K)\0", NULL, "Durell Software", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_SHOOT, 0,
	SpectrumGetZipName, SpecDeepstrikeRomInfo, SpecDeepstrikeRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Defcom (128K)

static struct BurnRomInfo SpecDefcomRomDesc[] = {
	{ "Defcom 128K (1986)(Quicksilva).tap", 55692, 0x0f2a7a1b, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecDefcom, SpecDefcom, Spec128)
STD_ROM_FN(SpecDefcom)

struct BurnDriver BurnSpecDefcom = {
	"spec_defcom", NULL, "spec_spec128", NULL, "1986",
	"Defcom (128K)\0", NULL, "Quicksilva", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_SHOOT, 0,
	SpectrumGetZipName, SpecDefcomRomInfo, SpecDefcomRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Defenders of the Earth (128K)

static struct BurnRomInfo SpecDefearthRomDesc[] = {
	{ "Defenders of the Earth 128K (1990)(Enigma Variations).z80", 92616, 0xd0283da9, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecDefearth, SpecDefearth, Spec128)
STD_ROM_FN(SpecDefearth)

struct BurnDriver BurnSpecDefearth = {
	"spec_defearth", NULL, "spec_spec128", NULL, "1990",
	"Defenders of the Earth (128K)\0", NULL, "Enigma Variations", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_RUNGUN | GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecDefearthRomInfo, SpecDefearthRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Delta Charge! (128K)

static struct BurnRomInfo SpecDeltaRomDesc[] = {
	{ "Delta Charge! 128K (1990)(Thalamus).tap", 56562, 0x3f0b7368, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecDelta, SpecDelta, Spec128)
STD_ROM_FN(SpecDelta)

struct BurnDriver BurnSpecDelta = {
	"spec_delta", NULL, "spec_spec128", NULL, "1990",
	"Delta Charge! (128K)\0", "Press 2 to set Kempston controller", "Thalamus - Twilight", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_HORSHOOT, 0,
	SpectrumGetZipName, SpecDeltaRomInfo, SpecDeltaRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Demonslair (48K)

static struct BurnRomInfo SpecDemonslairRomDesc[] = {
	{ "Demonslair 48K (1987)(Gadtek Games).tap", 42898, 0xcb570271, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecDemonslair, SpecDemonslair, Spectrum)
STD_ROM_FN(SpecDemonslair)

struct BurnDriver BurnSpecDemonslair = {
	"spec_demonslair", NULL, "spec_spectrum", NULL, "1987",
	"Demonslair (48K)\0", "Set 'd: sinclair.II' to use controller", "Gadtek Games", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecDemonslairRomInfo, SpecDemonslairRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecIntf2DIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Deviants (128K)

static struct BurnRomInfo SpecDeviantsRomDesc[] = {
	{ "Deviants 128K (1987)(Players Software).z80", 42765, 0x2d05732e, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecDeviants, SpecDeviants, Spec128)
STD_ROM_FN(SpecDeviants)

struct BurnDriver BurnSpecDeviants = {
	"spec_deviants", NULL, "spec_spec128", NULL, "1987",
	"Deviants (128K)\0", NULL, "Players Software", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecDeviantsRomInfo, SpecDeviantsRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Devil Diver (48K)

static struct BurnRomInfo SpecDevildiverRomDesc[] = {
	{ "Devil Diver 48K (1984)(DK'Tronics).tap", 16199, 0x400e8e44, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecDevildiver, SpecDevildiver, Spectrum)
STD_ROM_FN(SpecDevildiver)

struct BurnDriver BurnSpecDevildiver = {
	"spec_devildiver", NULL, "spec_spectrum", NULL, "1984",
	"Devil Diver (48K)\0", NULL, "DK'Tronics", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 2, HARDWARE_SPECTRUM, GBF_ACTION, 0,
	SpectrumGetZipName, SpecDevildiverRomInfo, SpecDevildiverRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Di-Lithium Lift (16K)

static struct BurnRomInfo SpecDilithiumRomDesc[] = {
	{ "Di-Lithium Lift 16K (1983)(Hewson Consultants).tap", 8878, 0xc04e50e2, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecDilithium, SpecDilithium, Spectrum)
STD_ROM_FN(SpecDilithium)

struct BurnDriver BurnSpecDilithium = {
	"spec_dilithium", NULL, "spec_spectrum", NULL, "1983",
	"Di-Lithium Lift (16K)\0", NULL, "Hewson Consultants", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_ACTION, 0,
	SpectrumGetZipName, SpecDilithiumRomInfo, SpecDilithiumRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Diosa de Cozumel, La (Spanish) (48K)

static struct BurnRomInfo SpecCozumelRomDesc[] = {
	{ "Diosa de Cozumel, La ES 48K (1990)(Aventuras AD).tap", 95528, 0x225d77d8, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecCozumel, SpecCozumel, Spectrum)
STD_ROM_FN(SpecCozumel)

struct BurnDriver BurnSpecCozumel = {
	"spec_cozumel", NULL, "spec_spectrum", NULL, "1990",
	"Diosa de Cozumel, La (Spanish) (48K)\0", NULL, "Aventuras A.D.", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_ADV, 0,
	SpectrumGetZipName, SpecCozumelRomInfo, SpecCozumelRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// D.N.A. Warrior (128K)

static struct BurnRomInfo SpecDnawarriorRomDesc[] = {
	{ "D.N.A. Warrior 128K (1989)(Cascade Games).tap", 52639, 0x86a9a591, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecDnawarrior, SpecDnawarrior, Spec128)
STD_ROM_FN(SpecDnawarrior)

struct BurnDriver BurnSpecDnawarrior = {
	"spec_dnawarrior", NULL, "spec_spec128", NULL, "1989",
	"D.N.A. Warrior (128K)\0", NULL, "Cascade Games", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_HORSHOOT, 0,
	SpectrumGetZipName, SpecDnawarriorRomInfo, SpecDnawarriorRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Doctor Doom's Revenge (48K)

static struct BurnRomInfo SpecDoomrevengeRomDesc[] = {
	{ "Doctor Doom's Revenge 48K (1989)(Empire).tap", 197516, 0x5f1b6cfc, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecDoomrevenge, SpecDoomrevenge, Spectrum)
STD_ROM_FN(SpecDoomrevenge)

struct BurnDriver BurnSpecDoomrevenge = {
	"spec_doomrevenge", NULL, "spec_spectrum", NULL, "1989",
	"Doctor Doom's Revenge (48K)\0", "The Amazing Spider-Man and Captain America in Dr. Doom's Revenge!","Empire", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_VSFIGHT, 0,
	SpectrumGetZipName, SpecDoomrevengeRomInfo, SpecDoomrevengeRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecIntf2DIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Dogsbody to the Rescue (48K)

static struct BurnRomInfo SpecDogsbodyRomDesc[] = {
	{ "Dogsbody to the Rescue 48K (1985)(Bug-Byte).tap", 49460, 0xaad92250, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecDogsbody, SpecDogsbody, Spectrum)
STD_ROM_FN(SpecDogsbody)

struct BurnDriver BurnSpecDogsbody = {
	"spec_dogsbody", NULL, "spec_spectrum", NULL, "1985",
	"Dogsbody to the Rescue (48K)\0", NULL, "Bug-Byte Software", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_MAZE, 0,
	SpectrumGetZipName, SpecDogsbodyRomInfo, SpecDogsbodyRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Don Quijote de la Mancha - Parte I (Spanish) (48K)

static struct BurnRomInfo SpecDonquijote1RomDesc[] = {
	{ "Don Quijote de la Mancha - Parte I ES 48K (1987)(Dinamic).tap", 49387, 0x1ac98966, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecDonquijote1, SpecDonquijote1, Spectrum)
STD_ROM_FN(SpecDonquijote1)

struct BurnDriver BurnSpecDonquijote1 = {
	"spec_donquijote1", NULL, "spec_spectrum", NULL, "1987",
	"Don Quijote de la Mancha - Parte I (Spanish) (48K)\0", NULL, "Dinamic Software", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_ADV, 0,
	SpectrumGetZipName, SpecDonquijote1RomInfo, SpecDonquijote1RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Don Quijote de la Mancha - Parte II (Spanish) (48K)

static struct BurnRomInfo SpecDonquijote2RomDesc[] = {
	{ "Don Quijote de la Mancha - Parte II ES 48K (1987)(Dinamic).tap", 49387, 0x042db349, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecDonquijote2, SpecDonquijote2, Spectrum)
STD_ROM_FN(SpecDonquijote2)

struct BurnDriver BurnSpecDonquijote2 = {
	"spec_donquijote2", "spec_donquijote1", "spec_spectrum", NULL, "1987",
	"Don Quijote de la Mancha - Parte II (Spanish) (48K)\0", "Password: EL INGENIOSO HIDALGO", "Dinamic Software", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE, 1, HARDWARE_SPECTRUM, GBF_ADV, 0,
	SpectrumGetZipName, SpecDonquijote2RomInfo, SpecDonquijote2RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Don't Panic! (16K)

static struct BurnRomInfo SpecDontpanicRomDesc[] = {
	{ "Don't Panic! 16K (1985)(Firebird).z80", 15293, 0x75879d60, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecDontpanic, SpecDontpanic, Spectrum)
STD_ROM_FN(SpecDontpanic)

struct BurnDriver BurnSpecDontpanic = {
	"spec_dontpanic", NULL, "spec_spectrum", NULL, "1985",
	"Don't Panic! (16K)\0", NULL, "Firebird Software", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 2, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecDontpanicRomInfo, SpecDontpanicRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Down to Earth (48K)

static struct BurnRomInfo SpecDown2earthRomDesc[] = {
	{ "Down to Earth 48K (1987)(Firebird).tap", 52928, 0x12f4183e, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecDown2earth, SpecDown2earth, Spectrum)
STD_ROM_FN(SpecDown2earth)

struct BurnDriver BurnSpecDown2earth = {
	"spec_down2earth", NULL, "spec_spectrum", NULL, "1987",
	"Down to Earth (48K)\0", NULL, "Firebird Software", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_ACTION | GBF_MAZE, 0,
	SpectrumGetZipName, SpecDown2earthRomInfo, SpecDown2earthRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Downtown (48K)

static struct BurnRomInfo SpecDowntownRomDesc[] = {
	{ "Downtown 48K (1991)(Atlantis Software).tap", 48188, 0xcfbbf2fb, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecDowntown, SpecDowntown, Spectrum)
STD_ROM_FN(SpecDowntown)

struct BurnDriver BurnSpecDowntown = {
	"spec_downtown", NULL, "spec_spectrum", NULL, "1991",
	"Downtown (48K)\0", NULL, "Atlantis Software", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_RUNGUN, 0,
	SpectrumGetZipName, SpecDowntownRomInfo, SpecDowntownRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Dragon's Lair (48K)

static struct BurnRomInfo SpecDragonslairRomDesc[] = {
	{ "Dragon's Lair 48K (1986)(Software Projects).tap", 64368, 0x0f532df7, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecDragonslair, SpecDragonslair, Spectrum)
STD_ROM_FN(SpecDragonslair)

struct BurnDriver BurnSpecDragonslair = {
	"spec_dragonslair", NULL, "spec_spectrum", NULL, "1986",
	"Dragon's Lair (48K)\0", "Press 'J' to set Joystick", "Software Projects", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_ACTION, 0,
	SpectrumGetZipName, SpecDragonslairRomInfo, SpecDragonslairRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Dragon's Lair II - Escape from Singe's Castle (128K)

static struct BurnRomInfo SpecDragonslair2RomDesc[] = {
	{ "Dragon's Lair II - Escape from Singe's Castle 128K (1987)(Software Projects).tap", 119432, 0xb2b86318, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecDragonslair2, SpecDragonslair2, Spec128)
STD_ROM_FN(SpecDragonslair2)

struct BurnDriver BurnSpecDragonslair2 = {
	"spec_dragonslair2", NULL, "spec_spec128", NULL, "1987",
	"Dragon's Lair II - Escape from Singe's Castle (128K)\0", NULL, "Software Projects", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_ACTION, 0,
	SpectrumGetZipName, SpecDragonslair2RomInfo, SpecDragonslair2RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Dragons of Flame (48K)

static struct BurnRomInfo SpecDragflameRomDesc[] = {
	{ "Dragons of Flame 48K (1990)(U.S. Gold).tap", 79158, 0xef1661a9, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecDragflame, SpecDragflame, Spectrum)
STD_ROM_FN(SpecDragflame)

struct BurnDriver BurnSpecDragflame = {
	"spec_dragflame", NULL, "spec_spectrum", NULL, "1990",
	"Dragons of Flame (48K)\0", NULL, "U.S. Gold", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_RPG, 0,
	SpectrumGetZipName, SpecDragflameRomInfo, SpecDragflameRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Dragon Spirit (128K)

static struct BurnRomInfo SpecDragonspiritRomDesc[] = {
	{ "Dragon Spirit (1989)(Domark).tap", 164427, 0xadc4c7cf, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecDragonspirit, SpecDragonspirit, Spec128)
STD_ROM_FN(SpecDragonspirit)

struct BurnDriver BurnSpecDragonspirit = {
	"spec_dragonspirit", NULL, "spec_spec128", NULL, "1989",
	"Dragon Spirit (128K)\0", NULL, "Domark", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_VERSHOOT, 0,
	SpectrumGetZipName, SpecDragonspiritRomInfo, SpecDragonspiritRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Draughts Genius (48K)

static struct BurnRomInfo SpecDrgeniusRomDesc[] = {
	{ "Draughts Genius 48K (1987)(Rack-It).tap", 48605, 0x3d91e93f, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecDrgenius, SpecDrgenius, Spectrum)
STD_ROM_FN(SpecDrgenius)

struct BurnDriver BurnSpecDrgenius = {
	"spec_drgenius", NULL, "spec_spectrum", NULL, "1987",
	"Draughts Genius (48K)\0", NULL, "Rack-It - Hewson", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 2, HARDWARE_SPECTRUM, GBF_BOARD, 0,
	SpectrumGetZipName, SpecDrgeniusRomInfo, SpecDrgeniusRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecQAOPSpaceDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Drazen Petrovic Basket (spanish) (128K)

static struct BurnRomInfo SpecDpbasketRomDesc[] = {
	{ "Drazen Petrovic Basket ES 128K (1989)(Topo Soft).tap", 64906, 0xf0d8a280, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecDpbasket, SpecDpbasket, Spec128)
STD_ROM_FN(SpecDpbasket)

struct BurnDriver BurnSpecDpbasket = {
	"spec_dpbasket", NULL, "spec_spec128", NULL, "1989",
	"Drazen Petrovic Basket (Spanish) (128K)\0", NULL, "Topo Soft", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 2, HARDWARE_SPECTRUM, GBF_SPORTSMISC, 0,
	SpectrumGetZipName, SpecDpbasketRomInfo, SpecDpbasketRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Driller Tanks (48K)

static struct BurnRomInfo SpecDrilltanksRomDesc[] = {
	{ "Driller Tanks 48K (1983)(Sinclair Research).tap", 11572, 0x9c56a3f6, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecDrilltanks, SpecDrilltanks, Spectrum)
STD_ROM_FN(SpecDrilltanks)

struct BurnDriver BurnSpecDrilltanks = {
	"spec_drilltanks", NULL, "spec_spectrum", NULL, "1983",
	"Driller Tanks (48K)\0", NULL, "Sinclair Research", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_ACTION | GBF_MAZE, 0,
	SpectrumGetZipName, SpecDrilltanksRomInfo, SpecDrilltanksRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecIntf2DIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Duck Out! (48K)

static struct BurnRomInfo SpecDuckoutRomDesc[] = {
	{ "Duck Out! 48K (1989)(Dro Soft).tap", 108898, 0x2d8955dc, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecDuckout, SpecDuckout, Spectrum)
STD_ROM_FN(SpecDuckout)

struct BurnDriver BurnSpecDuckout = {
	"spec_duckout", NULL, "spec_spectrum", NULL, "1989",
	"Duck Out! (48K)\0", NULL, "Dro Soft", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_SCRFIGHT, 0,
	SpectrumGetZipName, SpecDuckoutRomInfo, SpecDuckoutRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Dynamite Dux (128K)

static struct BurnRomInfo SpecDynamduxRomDesc[] = {
	{ "Dynamite Dux (1989)(Activision).tap", 150855, 0xc13fc9c0, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecDynamdux, SpecDynamdux, Spec128)
STD_ROM_FN(SpecDynamdux)

struct BurnDriver BurnSpecDynamdux = {
	"spec_dynamdux", NULL, "spec_spec128", NULL, "1989",
	"Dynamite Dux (128K)\0", "Press '2' to use controller", "Activision", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_SCRFIGHT, 0,
	SpectrumGetZipName, SpecDynamduxRomInfo, SpecDynamduxRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Dynasty Wars (48K)

static struct BurnRomInfo SpecDynawarsRomDesc[] = {
	{ "Dynasty Wars 48K (1990)(U.S. Gold).tap", 100253, 0xae3317af, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecDynawars, SpecDynawars, Spectrum)
STD_ROM_FN(SpecDynawars)

struct BurnDriver BurnSpecDynawars = {
	"spec_dynawars", NULL, "spec_spectrum", NULL, "1990",
	"Dynasty Wars (48K)\0", "Press '1' for one player mode or '2' for two player mode", "U.S. Gold", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 2, HARDWARE_SPECTRUM, GBF_SCRFIGHT, 0,
	SpectrumGetZipName, SpecDynawarsRomInfo, SpecDynawarsRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecIntf2DIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Dynatron Mission (48K)

static struct BurnRomInfo SpecDynamissionRomDesc[] = {
	{ "Dynatron Mission 48K (1987)(Mastertronic).tap", 48128, 0xa54ffa29, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecDynamission, SpecDynamission, Spectrum)
STD_ROM_FN(SpecDynamission)

struct BurnDriver BurnSpecDynamission = {
	"spec_dynamission", NULL, "spec_spectrum", NULL, "1987",
	"Dynatron Mission (48K)\0", NULL, "Mastertronic", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_PLATFORM | GBF_ADV, 0,
	SpectrumGetZipName, SpecDynamissionRomInfo, SpecDynamissionRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Egghead (48K)

static struct BurnRomInfo SpecEggheadRomDesc[] = {
	{ "Egghead (1989)(Crash).tap", 48076, 0x794d5953, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecEgghead, SpecEgghead, Spectrum)
STD_ROM_FN(SpecEgghead)

struct BurnDriver BurnSpecEgghead = {
	"spec_egghead", NULL, "spec_spectrum", NULL, "1989",
	"Egghead (48K)\0", "AY Sound supported", "Crash", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecEggheadRomInfo, SpecEggheadRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Egghead II - Egghead to the Rescue (48K)

static struct BurnRomInfo SpecEgghead2RomDesc[] = {
	{ "Egghead II - Egghead to the Rescue (1990)(Crash).tap", 46328, 0xb854984a, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecEgghead2, SpecEgghead2, Spectrum)
STD_ROM_FN(SpecEgghead2)

struct BurnDriver BurnSpecEgghead2 = {
	"spec_egghead2", NULL, "spec_spectrum", NULL, "1990",
	"Egghead II - Egghead to the Rescue (48K)\0", "AY Sound supported", "Crash", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecEgghead2RomInfo, SpecEgghead2RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Earthlight (128K)

static struct BurnRomInfo SpecEarthlightRomDesc[] = {
	{ "Earthlight 128K (1988)(Firebird).tap", 61597, 0x063d61f4, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecEarthlight, SpecEarthlight, Spec128)
STD_ROM_FN(SpecEarthlight)

struct BurnDriver BurnSpecEarthlight = {
	"spec_earthlight", NULL, "spec_spec128", NULL, "1988",
	"Earthlight (128K)\0", "Q: increase altitude - W: decrease altitude", "Firebird Software", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_HORSHOOT, 0,
	SpectrumGetZipName, SpecEarthlightRomInfo, SpecEarthlightRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Earth Shaker (48K)

static struct BurnRomInfo SpecEarthshakerRomDesc[] = {
	{ "Earth Shaker 48K (1990)(Your Sinclair).tap", 37729, 0x0c8c1e65, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecEarthshaker, SpecEarthshaker, Spectrum)
STD_ROM_FN(SpecEarthshaker)

struct BurnDriver BurnSpecEarthshaker = {
	"spec_earthshaker", NULL, "spec_spectrum", NULL, "1990",
	"Earth Shaker (48K)\0", NULL, "Your Sinclair", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_ACTION, 0,
	SpectrumGetZipName, SpecEarthshakerRomInfo, SpecEarthshakerRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Edd the Duck (48K)

static struct BurnRomInfo SpecEddduckRomDesc[] = {
	{ "Edd the Duck 48K (1990)(Impulze).z80", 29944, 0x720cac09, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecEddduck, SpecEddduck, Spectrum)
STD_ROM_FN(SpecEddduck)

struct BurnDriver BurnSpecEddduck = {
	"spec_eddduck", NULL, "spec_spectrum", NULL, "1990",
	"Edd the Duck (48K)\0", NULL, "Impulze", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecEddduckRomInfo, SpecEddduckRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecIntf2DIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Eidolon, The (128K)

static struct BurnRomInfo SpecEidolonRomDesc[] = {
	{ "Eidolon, The 128K (1986)(Activision - LucasfilmGames).tap", 107749, 0x00677105, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecEidolon, SpecEidolon, Spec128)
STD_ROM_FN(SpecEidolon)

struct BurnDriver BurnSpecEidolon = {
	"spec_eidolon", NULL, "spec_spec128", NULL, "1986",
	"Eidolon, The (128K)\0", NULL, "Activision - Lucasfilm Games", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_SHOOT | GBF_ADV, 0,
	SpectrumGetZipName, SpecEidolonRomInfo, SpecEidolonRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Elevator Action (128K)

static struct BurnRomInfo SpecElevactionRomDesc[] = {
	{ "Elevator Action 128K (1987)(Quicksilva).tap", 48481, 0x3000fcd9, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecElevaction, SpecElevaction, Spec128)
STD_ROM_FN(SpecElevaction)

struct BurnDriver BurnSpecElevaction = {
	"spec_elevaction", NULL, "spec_spec128", NULL, "1987",
	"Elevator Action (128K)\0", NULL, "Quicksilva", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecElevactionRomInfo, SpecElevactionRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Empire Strikes Back, The (48K)

static struct BurnRomInfo SpecStarwars2RomDesc[] = {
	{ "Empire Strikes Back, The 48K (1988)(Domark).z80", 40675, 0x2f091fd7, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecStarwars2, SpecStarwars2, Spectrum)
STD_ROM_FN(SpecStarwars2)

struct BurnDriver BurnSpecStarwars2 = {
	"spec_starwars2", NULL, "spec_spectrum", NULL, "1988",
	"Empire Strikes Back, The (48K)\0", NULL, "Domark", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_SHOOT, 0,
	SpectrumGetZipName, SpecStarwars2RomInfo, SpecStarwars2RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Enchanted (Spanish) (48K)

static struct BurnRomInfo SpecEnchantedRomDesc[] = {
	{ "Enchanted ES 48K (1989)(Positive).tap", 48705, 0x841628d9, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecEnchanted, SpecEnchanted, Spectrum)
STD_ROM_FN(SpecEnchanted)

struct BurnDriver BurnSpecEnchanted = {
	"spec_enchanted", NULL, "spec_spectrum", NULL, "1989",
	"Enchanted (Spanish) (48K)\0", "Q: left flipper - P: right flipper - SPACE: shoot ball", "Positive", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_PINBALL, 0,
	SpectrumGetZipName, SpecEnchantedRomInfo, SpecEnchantedRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Enigma Force (48K)

static struct BurnRomInfo SpecEnigmaforceRomDesc[] = {
	{ "Enigma Force 48K (1985)(Beyond Software).tap", 48362, 0xedf3d51b, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecEnigmaforce, SpecEnigmaforce, Spectrum)
STD_ROM_FN(SpecEnigmaforce)

struct BurnDriver BurnSpecEnigmaforce = {
	"spec_enigmaforce", NULL, "spec_spectrum", NULL, "1985",
	"Enigma Force (48K)\0", NULL, "Beyond Software", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_STRATEGY, 0,
	SpectrumGetZipName, SpecEnigmaforceRomInfo, SpecEnigmaforceRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Equinox (48K)

static struct BurnRomInfo SpecEquinoxRomDesc[] = {
	{ "Equinox 48K (1986)(Mikro-Gen).z80", 45770, 0xaaa2864d, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecEquinox, SpecEquinox, Spectrum)
STD_ROM_FN(SpecEquinox)

struct BurnDriver BurnSpecEquinox = {
	"spec_equinox", NULL, "spec_spectrum", NULL, "1986",
	"Equinox (48K)\0", NULL, "Mikro-Gen", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_SHOOT | GBF_MAZE, 0,
	SpectrumGetZipName, SpecEquinoxRomInfo, SpecEquinoxRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Eric and the Floaters (48K)

static struct BurnRomInfo SpecEricfloatersRomDesc[] = {
	{ "Eric And The Floaters 48K (1983)(Hudson Soft).tap", 8492, 0x0f7826de, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecEricfloaters, SpecEricfloaters, Spectrum)
STD_ROM_FN(SpecEricfloaters)

struct BurnDriver BurnSpecEricfloaters = {
	"spec_ericfloaters", NULL, "spec_spectrum", NULL, "1983",
	"Eric and the Floaters (48K)\0", NULL, "Hudson Soft", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_MAZE | GBF_ACTION, 0,
	SpectrumGetZipName, SpecEricfloatersRomInfo, SpecEricfloatersRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecIntf2DIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Escape From Alderon (48K)

static struct BurnRomInfo SpecEscalderonRomDesc[] = {
	{ "Escape From Alderon 48K (1986)(Troggerz Inc.).tap", 22008, 0x0cc57bee, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecEscalderon, SpecEscalderon, Spectrum)
STD_ROM_FN(SpecEscalderon)

struct BurnDriver BurnSpecEscalderon = {
	"spec_escalderon", NULL, "spec_spectrum", NULL, "1986",
	"Escape From Alderon (48K)\0", NULL, "Troggerz Inc.", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_ACTION, 0,
	SpectrumGetZipName, SpecEscalderonRomInfo, SpecEscalderonRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Eskimo Capers (16K)

static struct BurnRomInfo SpecEskicapersRomDesc[] = {
	{ "Eskimo Capers 16K (1984)(PowerSoftware).tap", 15335, 0xc875d845, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecEskicapers, SpecEskicapers, Spectrum)
STD_ROM_FN(SpecEskicapers)

struct BurnDriver BurnSpecEskicapers = {
	"spec_eskicapers", NULL, "spec_spectrum", NULL, "1984",
	"Eskimo Capers (16K)\0", NULL, "PowerSoftware", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_ACTION, 0,
	SpectrumGetZipName, SpecEskicapersRomInfo, SpecEskicapersRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Eskimo Eddie (48K)

static struct BurnRomInfo SpecEskeddieRomDesc[] = {
	{ "Eskimo Eddie 48K (1984)(Ocean).tap", 36318, 0x4b532294, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecEskeddie, SpecEskeddie, Spectrum)
STD_ROM_FN(SpecEskeddie)

struct BurnDriver BurnSpecEskeddie = {
	"spec_eskeddie", NULL, "spec_spectrum", NULL, "1984",
	"Eskimo Eddie (48K)\0", NULL, "Ocean", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_ACTION, 0,
	SpectrumGetZipName, SpecEskeddieRomInfo, SpecEskeddieRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Everyone's a Wally (48K)

static struct BurnRomInfo SpecEverywallyRomDesc[] = {
	{ "Everyone's a Wally 48K (1985)(Mikro-Gen).tap", 40178, 0x94e1bce9, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecEverywally, SpecEverywally, Spectrum)
STD_ROM_FN(SpecEverywally)

struct BurnDriver BurnSpecEverywally = {
	"spec_everywally", NULL, "spec_spectrum", NULL, "1985",
	"Everyone's a Wally (48K)\0", NULL, "Mikro-Gen", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_PLATFORM | GBF_ADV, 0,
	SpectrumGetZipName, SpecEverywallyRomInfo, SpecEverywallyRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Exploding Fist+ (48K)

static struct BurnRomInfo SpecFistplusRomDesc[] = {
	{ "Exploding Fist+ 48K (1988)(Firebird).tap", 52905, 0xd096a4be, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecFistplus, SpecFistplus, Spectrum)
STD_ROM_FN(SpecFistplus)

struct BurnDriver BurnSpecFistplus = {
	"spec_fistplus", NULL, "spec_spectrum", NULL, "1988",
	"Exploding Fist+ (48K)\0", NULL, "Firebird Software", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 3, HARDWARE_SPECTRUM, GBF_VSFIGHT, 0,
	SpectrumGetZipName, SpecFistplusRomInfo, SpecFistplusRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Exploding Wall (48K)

static struct BurnRomInfo SpecExplodwallRomDesc[] = {
	{ "Exploding Wall 48K (1989)(Byte Back).tap", 37401, 0xfe708ad6, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecExplodwall, SpecExplodwall, Spectrum)
STD_ROM_FN(SpecExplodwall)

struct BurnDriver BurnSpecExplodwall = {
	"spec_explodwall", NULL, "spec_spectrum", NULL, "1989",
	"Exploding Wall (48K)\0", NULL, "Byte Back", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_BREAKOUT, 0,
	SpectrumGetZipName, SpecExplodwallRomInfo, SpecExplodwallRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Express Raider (48K)

static struct BurnRomInfo SpecExpraiderRomDesc[] = {
	{ "Express Raider 48K (1987)(U.S. Gold).z80", 42604, 0x78fa11ff, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecExpraider, SpecExpraider, Spectrum)
STD_ROM_FN(SpecExpraider)

struct BurnDriver BurnSpecExpraider = {
	"spec_expraider", NULL, "spec_spectrum", NULL, "1987",
	"Express Raider (48K)\0", NULL, "U.S. Gold", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_SCRFIGHT, 0,
	SpectrumGetZipName, SpecExpraiderRomInfo, SpecExpraiderRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Exterminator (128K)

static struct BurnRomInfo SpecExterminatorRomDesc[] = {
	{ "Exterminator 128K (1991)(Audiogenic).tap", 129464, 0xdfeed3ed, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecExterminator, SpecExterminator, Spec128)
STD_ROM_FN(SpecExterminator)

struct BurnDriver BurnSpecExterminator = {
	"spec_exterminator", NULL, "spec_spec128", NULL, "1991",
	"Exterminator (128K)\0", NULL, "Audiogenic", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 2, HARDWARE_SPECTRUM, GBF_ACTION, 0,
	SpectrumGetZipName, SpecExterminatorRomInfo, SpecExterminatorRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecIntf2SwapDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Exterminator (16K)

static struct BurnRomInfo SpecExterminat16kRomDesc[] = {
	{ "Exterminator 16K (1983)(Silversoft).tap", 11858, 0x6e8e8f5d, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecExterminat16k, SpecExterminat16k, Spectrum)
STD_ROM_FN(SpecExterminat16k)

struct BurnDriver BurnSpecExterminat16k = {
	"spec_exterminat16k", NULL, "spec_spectrum", NULL, "1983",
	"Exterminator (16K)\0", "Press 'C' to options, 'S' to start game", "Silversoft", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 2, HARDWARE_SPECTRUM, GBF_ACTION | GBF_SHOOT, 0,
	SpectrumGetZipName, SpecExterminat16kRomInfo, SpecExterminat16kRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Extreme (48K)

static struct BurnRomInfo SpecExtremeRomDesc[] = {
	{ "Extreme 48K (1991)(Digital Integration).tap", 54612, 0xdc41f6f2, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecExtreme, SpecExtreme, Spectrum)
STD_ROM_FN(SpecExtreme)

struct BurnDriver BurnSpecExtreme = {
	"spec_extreme", NULL, "spec_spectrum", NULL, "1991",
	"Extreme (48K)\0", NULL, "Digital Integration", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_SHOOT, 0,
	SpectrumGetZipName, SpecExtremeRomInfo, SpecExtremeRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// F-16 Combat Pilot (128K)

static struct BurnRomInfo SpecF16pilotRomDesc[] = {
	{ "F-16 Combat Pilot 128K (1991)(Digital Integration).tap", 102967, 0xe6ac9aee, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecF16pilot, SpecF16pilot, Spec128)
STD_ROM_FN(SpecF16pilot)

struct BurnDriver BurnSpecF16pilot = {
	"spec_f16pilot", NULL, "spec_spec128", NULL, "1991",
	"F-16 Combat Pilot (128K)\0", NULL, "Digital Integration", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_SIM, 0,
	SpectrumGetZipName, SpecF16pilotRomInfo, SpecF16pilotRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Fallen Angel (48K)

static struct BurnRomInfo SpecFallenangRomDesc[] = {
	{ "Fallen Angel 48K (1989)(Screen 7).tap", 82545, 0x0067c07c, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecFallenang, SpecFallenang, Spectrum)
STD_ROM_FN(SpecFallenang)

struct BurnDriver BurnSpecFallenang = {
	"spec_fallenang", NULL, "spec_spectrum", NULL, "1989",
	"Fallen Angel (48K)\0", "Select '2) Sinclair SJS' to use controller", "Screen 7", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_SCRFIGHT, 0,
	SpectrumGetZipName, SpecFallenangRomInfo, SpecFallenangRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecIntf2DIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Fat Worm Blows a Sparky (48K)

static struct BurnRomInfo SpecFatwormRomDesc[] = {
	{ "Fat Worm Blows a Sparky 48K (1986)(Durell Software).tap", 49617, 0xe2012e24, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecFatworm, SpecFatworm, Spectrum)
STD_ROM_FN(SpecFatworm)

struct BurnDriver BurnSpecFatworm = {
	"spec_fatworm", NULL, "spec_spectrum", NULL, "1986",
	"Fat Worm Blows a Sparky (48K)\0", NULL, "Durell Software", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_ACTION, 0,
	SpectrumGetZipName, SpecFatwormRomInfo, SpecFatwormRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Fiendish Freddy's Big Top o'Fun (128K)

static struct BurnRomInfo SpecFiendfreddyRomDesc[] = {
	{ "Fiendish Freddy's Big Top o'Fun 128K (1990)(Mindscape International).tap", 464247, 0x2214888c, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecFiendfreddy, SpecFiendfreddy, Spec128)
STD_ROM_FN(SpecFiendfreddy)

struct BurnDriver BurnSpecFiendfreddy = {
	"spec_fiendfreddy", NULL, "spec_spec128", NULL, "1990",
	"Fiendish Freddy's Big Top o'Fun (128K)\0", NULL, "Mindscape International", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 5, HARDWARE_SPECTRUM, GBF_ACTION, 0,
	SpectrumGetZipName, SpecFiendfreddyRomInfo, SpecFiendfreddyRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecIntf2DIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Fighter Bomber (48K)

static struct BurnRomInfo SpecFightbombRomDesc[] = {
	{ "Fighter Bomber 48K (1990)(Activision).tap", 163083, 0xa32a5d92, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecFightbomb, SpecFightbomb, Spectrum)
STD_ROM_FN(SpecFightbomb)

struct BurnDriver BurnSpecFightbomb = {
	"spec_fightbomb", NULL, "spec_spectrum", NULL, "1990",
	"Fighter Bomber (48K)\0", NULL, "Activision", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_SIM, 0,
	SpectrumGetZipName, SpecFightbombRomInfo, SpecFightbombRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Final Assault (128K)

static struct BurnRomInfo SpecFinalassaultRomDesc[] = {
	{ "Final Assault 128K (1988)(U.S. Gold).z80", 82875, 0xbd8bec7a, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecFinalassault, SpecFinalassault, Spec128)
STD_ROM_FN(SpecFinalassault)

struct BurnDriver BurnSpecFinalassault = {
	"spec_finalassault", NULL, "spec_spec128", NULL, "1988",
	"Final Assault (128K)\0", NULL, "U.S. Gold", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_SPORTSMISC, 0,
	SpectrumGetZipName, SpecFinalassaultRomInfo, SpecFinalassaultRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecIntf2DIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Fire and Forget (128K)

static struct BurnRomInfo SpecFireforgetRomDesc[] = {
	{ "Fire and Forget (1988)(Titus).tap", 46724, 0x7685ab3a, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecFireforget, SpecFireforget, Spec128)
STD_ROM_FN(SpecFireforget)

struct BurnDriver BurnSpecFireforget = {
	"spec_fireforget", NULL, "spec_spec128", NULL, "1988",
	"Fire and Forget (128K)\0", NULL, "Titus", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_SHOOT, 0,
	SpectrumGetZipName, SpecFireforgetRomInfo, SpecFireforgetRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Fireman Sam - The Hero Next Door (48K)

static struct BurnRomInfo SpecFiremansamRomDesc[] = {
	{ "Fireman Sam - The Hero Next Door 48K (1991)(Alternative).tap", 87975, 0xb813c820, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecFiremansam, SpecFiremansam, Spectrum)
STD_ROM_FN(SpecFiremansam)

struct BurnDriver BurnSpecFiremansam = {
	"spec_firemansam", NULL, "spec_spectrum", NULL, "1991",
	"Fireman Sam - The Hero Next Door (48K)\0", NULL, "Alternative Software", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_ACTION | GBF_ADV, 0,
	SpectrumGetZipName, SpecFiremansamRomInfo, SpecFiremansamRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecQAOPSpaceDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// FireTrap (48K)

static struct BurnRomInfo SpecFiretrapRomDesc[] = {
	{ "FireTrap 48K (1987)(Electric Dreams).z80", 32097, 0x50c2787b, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecFiretrap, SpecFiretrap, Spectrum)
STD_ROM_FN(SpecFiretrap)

struct BurnDriver BurnSpecFiretrap = {
	"spec_firetrap", NULL, "spec_spectrum", NULL, "1987",
	"FireTrap (48K)\0", NULL, "Electric Dreams", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_ACTION, 0,
	SpectrumGetZipName, SpecFiretrapRomInfo, SpecFiretrapRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Fist II - The Legend Continues (48K)

static struct BurnRomInfo SpecFist2RomDesc[] = {
	{ "Fist II - The Legend Continues 48K (1986)(Melbourne House).tap", 47879, 0x41d1677f, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecFist2, SpecFist2, Spectrum)
STD_ROM_FN(SpecFist2)

struct BurnDriver BurnSpecFist2 = {
	"spec_fist2", NULL, "spec_spectrum", NULL, "1986",
	"Fist II - The Legend Continues (48K)\0", "Press any key to choose options and ENTER to select one", "Melbourne House", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_SCRFIGHT, 0,
	SpectrumGetZipName, SpecFist2RomInfo, SpecFist2RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Flash Gordon (48K)

static struct BurnRomInfo SpecFlashgordonRomDesc[] = {
	{ "Flash Gordon (1987)(Mastertronic Added Dimension).tap", 76147, 0x6fd40ff2, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecFlashgordon, SpecFlashgordon, Spectrum)
STD_ROM_FN(SpecFlashgordon)

struct BurnDriver BurnSpecFlashgordon = {
	"spec_flashgordon", NULL, "spec_spectrum", NULL, "1987",
	"Flash Gordon (48K)\0", NULL, "Mastertronic Added Dimension", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_ACTION | GBF_ADV, 0,
	SpectrumGetZipName, SpecFlashgordonRomInfo, SpecFlashgordonRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Flintstones, the (128K)

static struct BurnRomInfo SpecFlintstonesRomDesc[] = {
	{ "Flintstones, the (1988)(Grandslam).tap", 76902, 0xba15a0de, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecFlintstones, SpecFlintstones, Spec128)
STD_ROM_FN(SpecFlintstones)

struct BurnDriver BurnSpecFlintstones = {
	"spec_flintstones", NULL, "spec_spec128", NULL, "1988",
	"Flintstones, the (128K)\0", NULL, "Grandslam Entertainments", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_ADV | GBF_MINIGAMES, 0,
	SpectrumGetZipName, SpecFlintstonesRomInfo, SpecFlintstonesRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Flunky (48K)

static struct BurnRomInfo SpecFlunkyRomDesc[] = {
	{ "Flunky 48K (1987)(Piranha).tap", 43939, 0x5f8ab2e3, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecFlunky, SpecFlunky, Spectrum)
STD_ROM_FN(SpecFlunky)

struct BurnDriver BurnSpecFlunky = {
	"spec_flunky", NULL, "spec_spectrum", NULL, "1987",
	"Flunky (48K)\0", "In the Menu Room, use the 'bar' to cycle and the 'buzzer' to set options", "Piranha", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_ADV, 0,
	SpectrumGetZipName, SpecFlunkyRomInfo, SpecFlunkyRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Frank N Stein (48K)

static struct BurnRomInfo SpecFranknsteinRomDesc[] = {
	{ "Frank N Stein 48K (1984)(PSS).tap", 47860, 0x3ff60912, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecFranknstein, SpecFranknstein, Spectrum)
STD_ROM_FN(SpecFranknstein)

struct BurnDriver BurnSpecFranknstein = {
	"spec_franknstein", NULL, "spec_spectrum", NULL, "1984",
	"Frank N Stein (48K)\0", NULL, "PSS", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecFranknsteinRomInfo, SpecFranknsteinRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Frenzy (16K)

static struct BurnRomInfo SpecFrenzyRomDesc[] = {
	{ "Frenzy 16K (1982)(Quicksilva).z80", 7770, 0x434c69e1, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecFrenzy, SpecFrenzy, Spectrum)
STD_ROM_FN(SpecFrenzy)

struct BurnDriver BurnSpecFrenzy = {
	"spec_frenzy", NULL, "spec_spectrum", NULL, "1982",
	"Frenzy (16K)\0", "To play properly, invert 'up' and 'down' inputs", "Quicksilva", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_ACTION, 0,
	SpectrumGetZipName, SpecFrenzyRomInfo, SpecFrenzyRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecIntf2DIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Friday the 13th (48K)

static struct BurnRomInfo SpecFriday13thRomDesc[] = {
	{ "Friday the 13th 48K (1986)(Domark).tap", 44032, 0xc8fd8929, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecFriday13th, SpecFriday13th, Spectrum)
STD_ROM_FN(SpecFriday13th)

struct BurnDriver BurnSpecFriday13th = {
	"spec_friday13th", NULL, "spec_spectrum", NULL, "1986",
	"Friday the 13th (48K)\0", NULL, "Domark", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_ACTION | GBF_ADV, 0,
	SpectrumGetZipName, SpecFriday13thRomInfo, SpecFriday13thRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Frightmare (128K)

static struct BurnRomInfo SpecFrightmareRomDesc[] = {
	{ "Frightmare 128K (1988)(Cascade Games).tap", 59861, 0xa5ba5804, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecFrightmare, SpecFrightmare, Spec128)
STD_ROM_FN(SpecFrightmare)

struct BurnDriver BurnSpecFrightmare = {
	"spec_frightmare", NULL, "spec_spec128", NULL, "1988",
	"Frightmare (128K)\0", NULL, "Cascade Games", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecFrightmareRomInfo, SpecFrightmareRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecIntf2DIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Frozen Penguin (48K)

static struct BurnRomInfo SpecFrozpenguinRomDesc[] = {
	{ "Frozen Penguin 48K (1984)(Silversoft).tap", 22962, 0x2e713fc8, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecFrozpenguin, SpecFrozpenguin, Spectrum)
STD_ROM_FN(SpecFrozpenguin)

struct BurnDriver BurnSpecFrozpenguin = {
	"spec_frozpenguin", NULL, "spec_spectrum", NULL, "1984",
	"Frozen Penguin (48K)\0", NULL, "Silversoft", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_ACTION, 0,
	SpectrumGetZipName, SpecFrozpenguinRomInfo, SpecFrozpenguinRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Fury, The (128K)

static struct BurnRomInfo SpecFuryRomDesc[] = {
	{ "Fury, The (1988)(Martech Games).tap", 47954, 0x7ecbb308, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecFury, SpecFury, Spec128)
STD_ROM_FN(SpecFury)

struct BurnDriver BurnSpecFury = {
	"spec_fury", NULL, "spec_spec128", NULL, "1988",
	"Fury, The (128K)\0", NULL, "Martech Games", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_ACTION, 0,
	SpectrumGetZipName, SpecFuryRomInfo, SpecFuryRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Future Bike Simulator (128K)

static struct BurnRomInfo SpecFutbikesimRomDesc[] = {
	{ "Future Bike Simulator (1990)(Hi-Tec).tap", 95192, 0x3ced8a2d, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecFutbikesim, SpecFutbikesim, Spec128)
STD_ROM_FN(SpecFutbikesim)

struct BurnDriver BurnSpecFutbikesim = {
	"spec_futbikesim", NULL, "spec_spec128", NULL, "1990",
	"Future Bike Simulator (128K)\0", NULL, "Hi-Tec Software", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_RACING | GBF_SHOOT, 0,
	SpectrumGetZipName, SpecFutbikesimRomInfo, SpecFutbikesimRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Future Knight (128K)

static struct BurnRomInfo SpecFutknightRomDesc[] = {
	{ "Future Knight (1986)(Gremlin Graphics).tap", 53471, 0xfc50df19, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecFutknight, SpecFutknight, Spec128)
STD_ROM_FN(SpecFutknight)

struct BurnDriver BurnSpecFutknight = {
	"spec_futknight", NULL, "spec_spec128", NULL, "1986",
	"Future Knight (128K)\0", NULL, "Gremlin Graphics", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecFutknightRomInfo, SpecFutknightRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Galaxian (48K)

static struct BurnRomInfo SpecgalaxianRomDesc[] = {
	{ "Galaxian 48K (1984)(Atarisoft).tap", 20304, 0x10276234, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specgalaxian, Specgalaxian, Spectrum)
STD_ROM_FN(Specgalaxian)

struct BurnDriver BurnSpecgalaxian = {
	"spec_galaxian", NULL, "spec_spectrum", NULL, "1984",
	"Galaxian (48K)\0", "Q: left - W: right - P: fire", "Atarisoft", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_SHOOT, 0,
	SpectrumGetZipName, SpecgalaxianRomInfo, SpecgalaxianRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Galivan - Cosmo Police (48K)

static struct BurnRomInfo SpecGalivanRomDesc[] = {
	{ "Galivan - Cosmo Police 48K (1986)(Ocean).tap", 49664, 0x6987e5d8, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecGalivan, SpecGalivan, Spectrum)
STD_ROM_FN(SpecGalivan)

struct BurnDriver BurnSpecGalivan = {
	"spec_galivan", NULL, "spec_spectrum", NULL, "1986",
	"Galivan - Cosmo Police (48K)\0", NULL, "Ocean", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 2, HARDWARE_SPECTRUM, GBF_RUNGUN | GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecGalivanRomInfo, SpecGalivanRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Game Over - Part 1 (48K)

static struct BurnRomInfo Specgameoverp1RomDesc[] = {
	{ "Game Over - Part 1 (1987)(Dinamic).tap", 40891, 0x9bc5e9b4, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specgameoverp1, Specgameoverp1, Spectrum)
STD_ROM_FN(Specgameoverp1)

struct BurnDriver BurnSpecgameoverp1 = {
	"spec_gameoverp1", NULL, "spec_spectrum", NULL, "1987",
	"Game Over - Part 1 (48K)\0", NULL, "Imagine & Dinamic Software", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_RUNGUN | GBF_PLATFORM, 0,
	SpectrumGetZipName, Specgameoverp1RomInfo, Specgameoverp1RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Game Over - Part 2 (48K)

static struct BurnRomInfo Specgameoverp2RomDesc[] = {
	{ "Game Over - Part 2 (1987)(Dinamic).tap", 41167, 0x2657f897, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specgameoverp2, Specgameoverp2, Spectrum)
STD_ROM_FN(Specgameoverp2)

struct BurnDriver BurnSpecgameoverp2 = {
	"spec_gameoverp2", "spec_gameoverp1", "spec_spectrum", NULL, "1987",
	"Game Over - Part 2 (48K)\0", "Password: 18024", "Imagine & Dinamic Software", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE, 1, HARDWARE_SPECTRUM, GBF_RUNGUN | GBF_PLATFORM, 0,
	SpectrumGetZipName, Specgameoverp2RomInfo, Specgameoverp2RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Garfield: Big, Fat, Hairy Deal (48K)

static struct BurnRomInfo SpecGarfieldRomDesc[] = {
	{ "Garfield - Big, Fat, Hairy Deal (1988)(The Edge).tap", 43785, 0x45c3ceb8, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecGarfield, SpecGarfield, Spectrum)
STD_ROM_FN(SpecGarfield)

struct BurnDriver BurnSpecGarfield = {
	"spec_garfield", NULL, "spec_spectrum", NULL, "1988",
	"Garfield: Big Fat Hairy Deal (48K)\0", NULL, "The Edge", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_ADV, 0,
	SpectrumGetZipName, SpecGarfieldRomInfo, SpecGarfieldRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Garfield: Winter's Tail (48K)

static struct BurnRomInfo SpecGarfield2RomDesc[] = {
	{ "Garfield - Winter's Tail 48K (1990)(The Edge).tap", 58470, 0x74f9f23e, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecGarfield2, SpecGarfield2, Spectrum)
STD_ROM_FN(SpecGarfield2)

struct BurnDriver BurnSpecGarfield2 = {
	"spec_garfield2", NULL, "spec_spectrum", NULL, "1990",
	"Garfield: Winter's Tail (48K)\0", NULL, "The Edge", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_ACTION | GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecGarfield2RomInfo, SpecGarfield2RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Gary Lineker's Hot-Shot! (48K)

static struct BurnRomInfo SpecglinhtRomDesc[] = {
	{ "Gary Lineker's Hot-Shot! (1988)(Gremlin Graphics Software).z80", 0x08a0a, 0x18e1d943, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specglinht, Specglinht, Spectrum)
STD_ROM_FN(Specglinht)

struct BurnDriver BurnSpecglinht = {
	"spec_glinht", NULL, "spec_spectrum", NULL, "1988",
	"Gary Lineker's Hot-Shot! (48K)\0", "Press 'Z' and 'X' to select and 'K' to change options", "Gremlin Graphics", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 2, HARDWARE_SPECTRUM, GBF_SPORTSMISC, 0,
	SpectrumGetZipName, SpecglinhtRomInfo, SpecglinhtRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Gary Lineker's Super Skills (128K)

static struct BurnRomInfo SpecglssRomDesc[] = {
	{ "Gary Lineker's Super Skills (1988)(Gremlin Graphics Software)[128K].z80", 0x12e6f, 0xfcb98fd1, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specglss, Specglss, Spec128)
STD_ROM_FN(Specglss)

struct BurnDriver BurnSpecglss = {
	"spec_glss", NULL, "spec_spec128", NULL, "1988",
	"Gary Lineker's Super Skills (128K)\0", NULL, "Gremlin Graphics", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 4, HARDWARE_SPECTRUM, GBF_SPORTSMISC, 0,
	SpectrumGetZipName, SpecglssRomInfo, SpecglssRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Gary Lineker's Super Star Soccer (48K)

static struct BurnRomInfo SpecglsssRomDesc[] = {
	{ "Gary Lineker's Super Star Soccer 48K (1987)(Gremlin Graphics).tap", 47904, 0x4eda2a1e, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specglsss, Specglsss, Spectrum)
STD_ROM_FN(Specglsss)

struct BurnDriver BurnSpecglsss = {
	"spec_glsss", NULL, "spec_spectrum", NULL, "1987",
	"Gary Lineker's Super Star Soccer (48K)\0", "Press '2' to use controller", "Gremlin Graphics", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_STRATEGY, 0,
	SpectrumGetZipName, SpecglsssRomInfo, SpecglsssRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Gatecrasher (48K)

static struct BurnRomInfo SpecGatecrashRomDesc[] = {
	{ "Gatecrasher 48K (1984)(Quicksilva).tap", 30466, 0x980c9911, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecGatecrash, SpecGatecrash, Spectrum)
STD_ROM_FN(SpecGatecrash)

struct BurnDriver BurnSpecGatecrash = {
	"spec_gatecrash", NULL, "spec_spectrum", NULL, "1984",
	"Gatecrasher (48K)\0", NULL, "Quicksilva", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_PUZZLE, 0,
	SpectrumGetZipName, SpecGatecrashRomInfo, SpecGatecrashRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Gemini Wing (128K)

static struct BurnRomInfo SpecGeminiwingRomDesc[] = {
	{ "Gemini Wing (1989)(Virgin Mastertronic).tap", 144667, 0x171e71ed, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecGeminiwing, SpecGeminiwing, Spec128)
STD_ROM_FN(SpecGeminiwing)

struct BurnDriver BurnSpecGeminiwing = {
	"spec_geminiwing", NULL, "spec_spec128", NULL, "1989",
	"Gemini Wing (128K)\0", NULL, "Virgin Mastertronic", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_VERSHOOT, 0,
	SpectrumGetZipName, SpecGeminiwingRomInfo, SpecGeminiwingRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Gerry the Germ Goes Body Poppin (48K)

static struct BurnRomInfo SpecGerrygermRomDesc[] = {
	{ "Gerry the Germ Goes Body Poppin (1985)(Firebird).tap", 49339, 0x4bf4b0b0, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecGerrygerm, SpecGerrygerm, Spectrum)
STD_ROM_FN(SpecGerrygerm)

struct BurnDriver BurnSpecGerrygerm = {
	"spec_gerrygerm", NULL, "spec_spectrum", NULL, "1985",
	"Gerry the Germ Goes Body Poppin (48K)\0", NULL, "Firebird Software", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_ACTION, 0,
	SpectrumGetZipName, SpecGerrygermRomInfo, SpecGerrygermRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Ghosts 'n' Goblins (48K)

static struct BurnRomInfo Specgng_48RomDesc[] = {
	{ "Ghosts 'n' Goblins (1986)(Elite Systems).z80", 0x0b805, 0xdc252529, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specgng_48, Specgng_48, Spectrum)
STD_ROM_FN(Specgng_48)

struct BurnDriver BurnSpecgng_48 = {
	"spec_gng_48", NULL, "spec_spectrum", NULL, "1986",
	"Ghosts 'n' Goblins (48K)\0", NULL, "Elite Systems", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_RUNGUN | GBF_PLATFORM, 0,
	SpectrumGetZipName, Specgng_48RomInfo, Specgng_48RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Ghost's Revenge (16K)

static struct BurnRomInfo SpecGhostrevRomDesc[] = {
	{ "Ghost's Revenge 16K (1983)(Micromania).tap", 15558, 0x4eb8d249, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecGhostrev, SpecGhostrev, Spectrum)
STD_ROM_FN(SpecGhostrev)

struct BurnDriver BurnSpecGhostrev = {
	"spec_ghostrev", NULL, "spec_spectrum", NULL, "1983",
	"Ghost's Revenge (16K)\0", NULL, "Micromania", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_MAZE | GBF_ACTION, 0,
	SpectrumGetZipName, SpecGhostrevRomInfo, SpecGhostrevRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Gift from the Gods (48K)

static struct BurnRomInfo SpecGiftgodsRomDesc[] = {
	{ "Gift from the Gods (1984)(Ocean).tap", 48083, 0xb6b38137, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecGiftgods, SpecGiftgods, Spectrum)
STD_ROM_FN(SpecGiftgods)

struct BurnDriver BurnSpecGiftgods = {
	"spec_giftgods", NULL, "spec_spectrum", NULL, "1984",
	"Gift from the Gods (48K)\0", NULL, "Ocean", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_ACTION | GBF_ADV, 0,
	SpectrumGetZipName, SpecGiftgodsRomInfo, SpecGiftgodsRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Gilbert - Escape from Drill (128K)

static struct BurnRomInfo SpecGilbert128RomDesc[] = {
	{ "Gilbert - Escape from Drill 128K (1989)(Again Again).tap", 69020, 0xa7022003, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecGilbert128, SpecGilbert128, Spec128)
STD_ROM_FN(SpecGilbert128)

struct BurnDriver BurnSpecGilbert128 = {
	"spec_gilbert128", NULL, "spec_spec128", NULL, "1989",
	"Gilbert - Escape from Drill (128K)\0", NULL, "Again Again", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_ACTION | GBF_ADV, 0,
	SpectrumGetZipName, SpecGilbert128RomInfo, SpecGilbert128RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Gilbert - Escape from Drill (48K)

static struct BurnRomInfo SpecGilbert48RomDesc[] = {
	{ "Gilbert - Escape from Drill 48K (1989)(Again Again).tap", 52455, 0x5b528a33, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecGilbert48, SpecGilbert48, Spectrum)
STD_ROM_FN(SpecGilbert48)

struct BurnDriver BurnSpecGilbert48 = {
	"spec_gilbert48", "spec_gilbert128", "spec_spectrum", NULL, "1989",
	"Gilbert - Escape from Drill (48K)\0", NULL, "Again Again", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE, 1, HARDWARE_SPECTRUM, GBF_ACTION | GBF_ADV, 0,
	SpectrumGetZipName, SpecGilbert48RomInfo, SpecGilbert48RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Gilligan's Gold (48K)

static struct BurnRomInfo SpecGillgoldRomDesc[] = {
	{ "Gilligan's Gold 48K (1984)(Ocean).tap", 45750, 0x8623c1a1, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecGillgold, SpecGillgold, Spectrum)
STD_ROM_FN(SpecGillgold)

struct BurnDriver BurnSpecGillgold = {
	"spec_gillgold", NULL, "spec_spectrum", NULL, "1984",
	"Gilligan's Gold (48K)\0", NULL, "Ocean", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_ACTION, 0,
	SpectrumGetZipName, SpecGillgoldRomInfo, SpecGillgoldRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Gladiator (128K)

static struct BurnRomInfo SpecGladtr128RomDesc[] = {
	{ "Gladiator 128K (1986)(Domark).tap", 70662, 0xca8e9327, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecGladtr128, SpecGladtr128, Spec128)
STD_ROM_FN(SpecGladtr128)

struct BurnDriver BurnSpecGladtr128 = {
	"spec_gladtr128", NULL, "spec_spec128", NULL, "1986",
	"Gladiator (128K)\0", NULL, "Domark", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 4, HARDWARE_SPECTRUM, GBF_VSFIGHT, 0,
	SpectrumGetZipName, SpecGladtr128RomInfo, SpecGladtr128RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Gladiator (48K)

static struct BurnRomInfo SpecGladtr48RomDesc[] = {
	{ "Gladiator 48K (1986)(Domark).tap", 45249, 0x902ff9d8, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecGladtr48, SpecGladtr48, Spectrum)
STD_ROM_FN(SpecGladtr48)

struct BurnDriver BurnSpecGladtr48 = {
	"spec_gladtr48", "spec_gladtr128", "spec_spectrum", NULL, "1986",
	"Gladiator (48K)\0", NULL, "Domark", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE, 2, HARDWARE_SPECTRUM, GBF_VSFIGHT, 0,
	SpectrumGetZipName, SpecGladtr48RomInfo, SpecGladtr48RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Glass (48K)

static struct BurnRomInfo SpecGlassRomDesc[] = {
	{ "Glass 48K (1985)(Quicksilva).tap", 44801, 0xc6e19b1c, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecGlass, SpecGlass, Spectrum)
STD_ROM_FN(SpecGlass)

struct BurnDriver BurnSpecGlass = {
	"spec_glass", NULL, "spec_spectrum", NULL, "1985",
	"Glass (48K)\0", NULL, "Quicksilva", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_SHOOT, 0,
	SpectrumGetZipName, SpecGlassRomInfo, SpecGlassRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Glug Glug (48K)

static struct BurnRomInfo SpecGlugglugRomDesc[] = {
	{ "Glug Glug 48K (1984)(CRL Ltd.).tap", 18843, 0x0a096413, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecGlugglug, SpecGlugglug, Spectrum)
STD_ROM_FN(SpecGlugglug)

struct BurnDriver BurnSpecGlugglug = {
	"spec_glugglug", NULL, "spec_spectrum", NULL, "1984",
	"Glug Glug (48K)\0", NULL, "CRL Ltd.", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 2, HARDWARE_SPECTRUM, GBF_ACTION, 0,
	SpectrumGetZipName, SpecGlugglugRomInfo, SpecGlugglugRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Gnasher (16K)

static struct BurnRomInfo SpecGnasherRomDesc[] = {
	{ "Gnasher 16K (1984)(Mastertronic).tap", 17027, 0x7c38b4ea, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecGnasher, SpecGnasher, Spectrum)
STD_ROM_FN(SpecGnasher)

struct BurnDriver BurnSpecGnasher = {
	"spec_gnasher", NULL, "spec_spectrum", NULL, "1984",
	"Gnasher (16K)\0", NULL, "Mastertronic", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_ACTION | GBF_MAZE, 0,
	SpectrumGetZipName, SpecGnasherRomInfo, SpecGnasherRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// GNONI (Spanish) (48K)

static struct BurnRomInfo SpecGnonimhRomDesc[] = {
	{ "Gnoni ES 48K (1988)(MicroHobby).tap", 17473, 0x14adb16f, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecGnonimh, SpecGnonimh, Spectrum)
STD_ROM_FN(SpecGnonimh)

struct BurnDriver BurnSpecGnonimh = {
	"spec_gnonimh", NULL, "spec_spectrum", NULL, "1988",
	"GNONI (Spanish) (48K)\0", "NB: when use 'Kempston Joys' the fire button crashes the game", "MicroHobby", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_ACTION, 0,
	SpectrumGetZipName, SpecGnonimhRomInfo, SpecGnonimhRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Golden Axe (128K)

static struct BurnRomInfo Specgoldenaxe1RomDesc[] = {
	{ "Golden Axe (1990)(Virgin Games).z80", 0x1b859, 0x3f32f82f, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specgoldenaxe1, Specgoldenaxe1, Spec128)
STD_ROM_FN(Specgoldenaxe1)

struct BurnDriver BurnSpecgoldenaxe1 = {
	"spec_goldenaxe1", NULL, "spec_spec128", NULL, "1990",
	"Golden Axe (128K)\0", NULL, "Virgin Games", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 2, HARDWARE_SPECTRUM, GBF_SCRFIGHT, 0,
	SpectrumGetZipName, Specgoldenaxe1RomInfo, Specgoldenaxe1RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Gonzzalezz - Part 1 (Spanish) (128K)

static struct BurnRomInfo SpecGonzzalezz1RomDesc[] = {
	{ "Gonzzalezz ES Part 1 (1989)(Opera Soft).tap", 48329, 0xf9309baa, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecGonzzalezz1, SpecGonzzalezz1, Spec128)
STD_ROM_FN(SpecGonzzalezz1)

struct BurnDriver BurnSpecGonzzalezz1 = {
	"spec_gonzzalezz1", NULL, "spec_spec128", NULL, "1989",
	"Gonzzalezz - Part 1 (Spanish) (128K)\0", NULL, "Opera Soft", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecGonzzalezz1RomInfo, SpecGonzzalezz1RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecQAOPSpaceDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Gonzzalezz - Part 2 (Spanish) (128K)

static struct BurnRomInfo SpecGonzzalezz2RomDesc[] = {
	{ "Gonzzalezz ES Part 2 (1989)(Opera Soft).tap", 48329, 0xd9041f4b, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecGonzzalezz2, SpecGonzzalezz2, Spec128)
STD_ROM_FN(SpecGonzzalezz2)

struct BurnDriver BurnSpecGonzzalezz2 = {
	"spec_gonzzalezz2", "spec_gonzzalezz1", "spec_spec128", NULL, "1989",
	"Gonzzalezz - Part 2 (Spanish) (128K)\0", NULL, "Opera Soft", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecGonzzalezz2RomInfo, SpecGonzzalezz2RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecQAOPSpaceDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Gothik (48K)

static struct BurnRomInfo SpecGothikRomDesc[] = {
	{ "Gothik 48K (1987)(Firebird).tap", 37854, 0x77774db3, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecGothik, SpecGothik, Spectrum)
STD_ROM_FN(SpecGothik)

struct BurnDriver BurnSpecGothik = {
	"spec_gothik", NULL, "spec_spectrum", NULL, "1987",
	"Gothik (48K)\0", NULL, "Firebird Software", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_MAZE | GBF_ACTION, 0,
	SpectrumGetZipName, SpecGothikRomInfo, SpecGothikRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecQAOPSpaceDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Grand Prix Circuit (128K)

static struct BurnRomInfo SpecGpcircuitRomDesc[] = {
	{ "Grand Prix Circuit 128K (1990)(Accolade).z80", 60323, 0x1203a59d, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecGpcircuit, SpecGpcircuit, Spec128)
STD_ROM_FN(SpecGpcircuit)

struct BurnDriver BurnSpecGpcircuit = {
	"spec_gpcircuit", NULL, "spec_spec128", NULL, "1990",
	"Grand Prix Circuit (128K)\0", NULL, "Accolade", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_RACING, 0,
	SpectrumGetZipName, SpecGpcircuitRomInfo, SpecGpcircuitRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecIntf2DIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Great Gurianos (48K)

static struct BurnRomInfo SpecGreatgurianRomDesc[] = {
	{ "Great Gurianos 48K (1991)(Hit-Pak).z80", 39089, 0x2a96de1d, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecGreatgurian, SpecGreatgurian, Spectrum)
STD_ROM_FN(SpecGreatgurian)

struct BurnDriver BurnSpecGreatgurian = {
	"spec_greatgurian", NULL, "spec_spectrum", NULL, "1991",
	"Great Gurianos (48K)\0", NULL, "Hit-Pak", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_SCRFIGHT, 0,
	SpectrumGetZipName, SpecGreatgurianRomInfo, SpecGreatgurianRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Gregory Loses His Clock (48K)

static struct BurnRomInfo SpecGreglhcRomDesc[] = {
	{ "Gregory Loses His Clock 48K (1989)(Mastertronic Plus).tap", 40109, 0x02af488a, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecGreglhc, SpecGreglhc, Spectrum)
STD_ROM_FN(SpecGreglhc)

struct BurnDriver BurnSpecGreglhc = {
	"spec_greglhc", NULL, "spec_spectrum", NULL, "1989",
	"Gregory Loses His Clock (48K)\0", NULL, "Mastertronic Plus", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_ADV, 0,
	SpectrumGetZipName, SpecGreglhcRomInfo, SpecGreglhcRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Gremlins - The Adventure (48K)

static struct BurnRomInfo SpecGremlinsadvRomDesc[] = {
	{ "Gremlins - The Adventure (1985)(Adventure International).z80", 45332, 0x7049389f, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecGremlinsadv, SpecGremlinsadv, Spectrum)
STD_ROM_FN(SpecGremlinsadv)

struct BurnDriver BurnSpecGremlinsadv = {
	"spec_gremlinsadv", NULL, "spec_spectrum", NULL, "1985",
	"Gremlins - The Adventure (48K)\0", NULL, "Adventure International", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_ADV, 0,
	SpectrumGetZipName, SpecGremlinsadvRomInfo, SpecGremlinsadvRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Gremlins 2 - The New Batch (48K)

static struct BurnRomInfo SpecGremlins2RomDesc[] = {
	{ "Gremlins 2 - The New Batch (1990)(Topo Soft - Elite Systems).tap", 0x2716c, 0xb1402dfe, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecGremlins2, SpecGremlins2, Spectrum)
STD_ROM_FN(SpecGremlins2)

struct BurnDriver BurnSpecGremlins2 = {
	"spec_gremlins2", NULL, "spec_spectrum", NULL, "1990",
	"Gremlins 2 - The New Batch (48K)\0", "In options menu use Q, A and Space", "Topo Soft - Elite Systems", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecGremlins2RomInfo, SpecGremlins2RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Gridrunner (16K)

static struct BurnRomInfo SpecGridrunnerRomDesc[] = {
	{ "Gridrunner 16K (1983)(Salamander Software).tap", 16231, 0xeb08f86a, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecGridrunner, SpecGridrunner, Spectrum)
STD_ROM_FN(SpecGridrunner)

struct BurnDriver BurnSpecGridrunner = {
	"spec_gridrunner", NULL, "spec_spectrum", NULL, "1983",
	"Gridrunner (16K)\0", NULL, "Salamander Software", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_SHOOT, 0,
	SpectrumGetZipName, SpecGridrunnerRomInfo, SpecGridrunnerRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecCursorKeysDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Guardian, The (48K)

static struct BurnRomInfo SpecGuardianRomDesc[] = {
	{ "Guardian, The 48K (1983)(PSS).tap", 19190, 0xd7fec385, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecGuardian, SpecGuardian, Spectrum)
STD_ROM_FN(SpecGuardian)

struct BurnDriver BurnSpecGuardian = {
	"spec_guardian", NULL, "spec_spectrum", NULL, "1983",
	"Guardian, The (48K)\0", NULL, "PSS", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_SHOOT, 0,
	SpectrumGetZipName, SpecGuardianRomInfo, SpecGuardianRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Gunship (128K)

static struct BurnRomInfo SpecgunshipRomDesc[] = {
	{ "Gunship (1987)(MicroProse Software).tap", 0x193ca, 0x01e4ed88, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specgunship, Specgunship, Spec128)
STD_ROM_FN(Specgunship)

struct BurnDriver BurnSpecgunship = {
	"spec_gunship", NULL, "spec_spec128", NULL, "1987",
	"Gunship (128K)\0", NULL, "MicroProse Software", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_SIM, 0,
	SpectrumGetZipName, SpecgunshipRomInfo, SpecgunshipRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Gunstar (48K)

static struct BurnRomInfo SpecGunstarRomDesc[] = {
	{ "Gunstar 48K (1987)(Firebird).tap", 47715, 0x23aedf6b, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecGunstar, SpecGunstar, Spectrum)
STD_ROM_FN(SpecGunstar)

struct BurnDriver BurnSpecGunstar = {
	"spec_gunstar", NULL, "spec_spectrum", NULL, "1987",
	"Gunstar (48K)\0", NULL, "Firebird Software", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_VERSHOOT, 0,
	SpectrumGetZipName, SpecGunstarRomInfo, SpecGunstarRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Gutz (48K)

static struct BurnRomInfo SpecGutzRomDesc[] = {
	{ "Gutz 48K (1988)(Ocean).tap", 43037, 0xad8a6a47, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecGutz, SpecGutz, Spectrum)
STD_ROM_FN(SpecGutz)

struct BurnDriver BurnSpecGutz = {
	"spec_gutz", NULL, "spec_spectrum", NULL, "1988",
	"Gutz (48K)\0", NULL, "Ocean", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_MAZE | GBF_RUNGUN, 0,
	SpectrumGetZipName, SpecGutzRomInfo, SpecGutzRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Hammerfist (48K)

static struct BurnRomInfo SpecHammerfistRomDesc[] = {
	{ "Hammerfist (1990)(Activision).tap", 188831, 0xc6ad835b, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecHammerfist, SpecHammerfist, Spectrum)
STD_ROM_FN(SpecHammerfist)

struct BurnDriver BurnSpecHammerfist = {
	"spec_hammerfist", NULL, "spec_spectrum", NULL, "1990",
	"Hammerfist (48K)\0", NULL, "Activision", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_PLATFORM | GBF_RUNGUN, 0,
	SpectrumGetZipName, SpecHammerfistRomInfo, SpecHammerfistRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Hard Cheese! (16K)

static struct BurnRomInfo SpecHardcheeseRomDesc[] = {
	{ "Hard Cheese! 16K (1983)(DK'Tronics).tap", 14704, 0xeee475b0, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecHardcheese, SpecHardcheese, Spectrum)
STD_ROM_FN(SpecHardcheese)

struct BurnDriver BurnSpecHardcheese = {
	"spec_hardcheese", NULL, "spec_spectrum", NULL, "1983",
	"Hard Cheese! (16K)\0", NULL, "DK'Tronics", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_ACTION, 0,
	SpectrumGetZipName, SpecHardcheeseRomInfo, SpecHardcheeseRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Havoc (128K)

static struct BurnRomInfo SpecHavocRomDesc[] = {
	{ "Havoc 128K (1990)(Players Premier).z80", 48092, 0x510c85ea, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecHavoc, SpecHavoc, Spec128)
STD_ROM_FN(SpecHavoc)

struct BurnDriver BurnSpecHavoc = {
	"spec_havoc", NULL, "spec_spec128", NULL, "1990",
	"Havoc (128K)\0", NULL, "Players Premier", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_VERSHOOT, 0,
	SpectrumGetZipName, SpecHavocRomInfo, SpecHavocRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Hawk Storm (128K)

static struct BurnRomInfo SpecHawkstormRomDesc[] = {
	{ "Hawk Storm 128K (1990)(Players).z80", 41034, 0xf38d5e3c, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecHawkstorm, SpecHawkstorm, Spec128)
STD_ROM_FN(SpecHawkstorm)

struct BurnDriver BurnSpecHawkstorm = {
	"spec_hawkstorm", NULL, "spec_spec128", NULL, "1990",
	"Hawk Storm (128K)\0", NULL, "Players Software", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_RUNGUN | GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecHawkstormRomInfo, SpecHawkstormRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Headbangers Heaven v2 (48K)

static struct BurnRomInfo SpecHbheavenRomDesc[] = {
	{ "Headbangers Heaven v2 48K (1983)(Llamasoft).tap", 19701, 0xc8697518, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecHbheaven, SpecHbheaven, Spectrum)
STD_ROM_FN(SpecHbheaven)

struct BurnDriver BurnSpecHbheaven = {
	"spec_hbheaven", NULL, "spec_spectrum", NULL, "1983",
	"Headbangers Heaven v2 (48K)\0", "N: move left, M: move right", "Llamasoft", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_ACTION, 0,
	SpectrumGetZipName, SpecHbheavenRomInfo, SpecHbheavenRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Heavy Metal (128K)

static struct BurnRomInfo SpecHeavymetalRomDesc[] = {
	{ "Heavy Metal 128K (1990)(U.S. Gold).tap", 86702, 0x67e11acf, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecHeavymetal, SpecHeavymetal, Spec128)
STD_ROM_FN(SpecHeavymetal)

struct BurnDriver BurnSpecHeavymetal = {
	"spec_heavymetal", NULL, "spec_spec128", NULL, "1990",
	"Heavy Metal (128K)\0", "W/S: speed up/down - E: select weapon - D: smoke", "U.S. Gold", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_SHOOT, 0,
	SpectrumGetZipName, SpecHeavymetalRomInfo, SpecHeavymetalRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Heavy on the Magick (48K)

static struct BurnRomInfo SpecHeavymagRomDesc[] = {
	{ "Heavy on the Magick 48K (1986)(Gargoyle Games).tap", 42787, 0x77ccf39c, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecHeavymag, SpecHeavymag, Spectrum)
STD_ROM_FN(SpecHeavymag)

struct BurnDriver BurnSpecHeavymag = {
	"spec_heavymag", NULL, "spec_spectrum", NULL, "1986",
	"Heavy on the Magick (48K)\0", NULL, "Gargoyle Games", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_ADV, 0,
	SpectrumGetZipName, SpecHeavymagRomInfo, SpecHeavymagRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Helter Skelter (128K)

static struct BurnRomInfo SpecHskelterRomDesc[] = {
	{ "Helter Skelter 128K (1990)(Audiogenic).tap", 70452, 0xeb7fdaa4, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecHskelter, SpecHskelter, Spec128)
STD_ROM_FN(SpecHskelter)

struct BurnDriver BurnSpecHskelter = {
	"spec_hskelter", NULL, "spec_spec128", NULL, "1990",
	"Helter Skelter (128K)\0", NULL, "Audiogenic", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 2, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecHskelterRomInfo, SpecHskelterRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Herbert's Dummy Run (48K)

static struct BurnRomInfo SpecHerbertdrRomDesc[] = {
	{ "Herbert's Dummy Run 48K (1985)(Mikro-Gen).tap", 49402, 0xa9b3b9c5, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecHerbertdr, SpecHerbertdr, Spectrum)
STD_ROM_FN(SpecHerbertdr)

struct BurnDriver BurnSpecHerbertdr = {
	"spec_herbertdr", NULL, "spec_spectrum", NULL, "1985",
	"Herbert's Dummy Run (48K)\0", NULL, "Mikro-Gen", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecHerbertdrRomInfo, SpecHerbertdrRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Hercules (48K)

static struct BurnRomInfo SpecHerculesRomDesc[] = {
	{ "Hercules 48K (1986)(Alpha-Omega).tap", 28621, 0x3b89d565, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecHercules, SpecHercules, Spectrum)
STD_ROM_FN(SpecHercules)

struct BurnDriver BurnSpecHercules = {
	"spec_hercules", NULL, "spec_spectrum", NULL, "1986",
	"Hercules (48K)\0", NULL, "Alpha-Omega", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecHerculesRomInfo, SpecHerculesRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Hercules - Slayer of the Damned (128K)

static struct BurnRomInfo SpecHercslayerRomDesc[] = {
	{ "Hercules - Slayer of the Damned (1988)(Gremlin Graphics).tap", 52958, 0xc6ca45e6, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecHercslayer, SpecHercslayer, Spec128)
STD_ROM_FN(SpecHercslayer)

struct BurnDriver BurnSpecHercslayer = {
	"spec_hercslayer", NULL, "spec_spec128", NULL, "1988",
	"Hercules - Slayer of the Damned (128K)\0", NULL, "Gremlin Graphics", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_VSFIGHT, 0,
	SpectrumGetZipName, SpecHercslayerRomInfo, SpecHercslayerRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Hero Quest (128K)

static struct BurnRomInfo SpecHeroquestRomDesc[] = {
	{ "Hero Quest 128K (1991)(Gremlin Graphics).tap", 111870, 0x76faec29, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecHeroquest, SpecHeroquest, Spec128)
STD_ROM_FN(SpecHeroquest)

struct BurnDriver BurnSpecHeroquest = {
	"spec_heroquest", NULL, "spec_spec128", NULL, "1991",
	"Hero Quest (128K)\0", NULL, "Gremlin Graphics", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_RPG, 0,
	SpectrumGetZipName, SpecHeroquestRomInfo, SpecHeroquestRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Hero Quest - Return of the Witch Lord (128K)

static struct BurnRomInfo SpecHeroquestwitchlordRomDesc[] = {
	{ "Hero Quest - Return of the Witch Lord 128K (1991)(Gremlin Graphics).tap", 131173, 0x65141180, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecHeroquestwitchlord, SpecHeroquestwitchlord, Spec128)
STD_ROM_FN(SpecHeroquestwitchlord)

struct BurnDriver BurnSpecHeroquestwitchlord = {
	"spec_heroquestwitchlord", NULL, "spec_spec128", NULL, "1991",
	"Hero Quest - Return of the Witch Lord (128K)\0", NULL, "Gremlin Graphics", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_RPG, 0,
	SpectrumGetZipName, SpecHeroquestwitchlordRomInfo, SpecHeroquestwitchlordRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// High Steel (128K)

static struct BurnRomInfo SpecHsteel128RomDesc[] = {
	{ "High Steel 128K (1989)(Screen 7).tap", 46853, 0x0461a6b8, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecHsteel128, SpecHsteel128, Spec128)
STD_ROM_FN(SpecHsteel128)

struct BurnDriver BurnSpecHsteel128 = {
	"spec_hsteel128", NULL, "spec_spec128", NULL, "1989",
	"High Steel (128K)\0", "Use SPACE and ENTER to choose and set options", "Screen 7", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_ACTION, 0,
	SpectrumGetZipName, SpecHsteel128RomInfo, SpecHsteel128RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// High Steel (48K)

static struct BurnRomInfo SpecHsteel48RomDesc[] = {
	{ "High Steel 48K (1989)(Screen 7).tap", 46598, 0x7afa1809, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecHsteel48, SpecHsteel48, Spectrum)
STD_ROM_FN(SpecHsteel48)

struct BurnDriver BurnSpecHsteel48 = {
	"spec_hsteel48", "spec_hsteel128", "spec_spectrum", NULL, "1989",
	"High Steel (48K)\0", "Use SPACE and ENTER to choose and set options", "Screen 7", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE, 1, HARDWARE_SPECTRUM, GBF_ACTION, 0,
	SpectrumGetZipName, SpecHsteel48RomInfo, SpecHsteel48RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Hobgoblin (48K)

static struct BurnRomInfo SpecHobgoblRomDesc[] = {
	{ "Hobgoblin 48K (1991)(Atlantis).tap", 31817, 0x244df5e4, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecHobgobl, SpecHobgobl, Spectrum)
STD_ROM_FN(SpecHobgobl)

struct BurnDriver BurnSpecHobgobl = {
	"spec_hobgobl", NULL, "spec_spectrum", NULL, "1991",
	"Hobgoblin (48K)\0", NULL, "Atlantis Software", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_RUNGUN | GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecHobgoblRomInfo, SpecHobgoblRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecQAOPMDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Hollow, The (48K)

static struct BurnRomInfo SpecHollowRomDesc[] = {
	{ "Hollow, The 48K (1985)(Gilsoft International).tap", 47876, 0xd2d36c3a, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecHollow, SpecHollow, Spectrum)
STD_ROM_FN(SpecHollow)

struct BurnDriver BurnSpecHollow = {
	"spec_hollow", NULL, "spec_spectrum", NULL, "1985",
	"Hollow, The (48K)\0", NULL, "Gilsoft International", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_ADV, 0,
	SpectrumGetZipName, SpecHollowRomInfo, SpecHollowRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Hong Kong Phooey (48K)

static struct BurnRomInfo SpecHkphooeyRomDesc[] = {
	{ "Hong Kong Phooey 48K (1990)(Hi-Tec).z80", 37410, 0x444a5064, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecHkphooey, SpecHkphooey, Spectrum)
STD_ROM_FN(SpecHkphooey)

struct BurnDriver BurnSpecHkphooey = {
	"spec_hkphooey", NULL, "spec_spectrum", NULL, "1990",
	"Hong Kong Phooey (48K)\0", NULL, "Hi-Tec Software", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecHkphooeyRomInfo, SpecHkphooeyRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Hotshot (128K)

static struct BurnRomInfo SpecHotshotRomDesc[] = {
	{ "Hotshot 128K (1988)(Addictive Games).tap", 30707, 0x4f9b3846, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecHotshot, SpecHotshot, Spec128)
STD_ROM_FN(SpecHotshot)

struct BurnDriver BurnSpecHotshot = {
	"spec_hotshot", NULL, "spec_spec128", NULL, "1988",
	"Hotshot (128K)\0", NULL, "Addictive Games", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 2, HARDWARE_SPECTRUM, GBF_ACTION | GBF_PUZZLE, 0,
	SpectrumGetZipName, SpecHotshotRomInfo, SpecHotshotRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Hunchback (48K)

static struct BurnRomInfo SpechunchbacRomDesc[] = {
	{ "Hunchback 48K (1984)(Ocean).tap", 43449, 0x3fd8f859, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Spechunchbac, Spechunchbac, Spectrum)
STD_ROM_FN(Spechunchbac)

struct BurnDriver BurnSpechunchbac = {
	"spec_hunchbac", NULL, "spec_spectrum", NULL, "1984",
	"Hunchback (48K)\0", NULL, "Ocean", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_ACTION, 0,
	SpectrumGetZipName, SpechunchbacRomInfo, SpechunchbacRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Hunchback II - Quasimodo's Revenge (48K)

static struct BurnRomInfo Spechunchbac2RomDesc[] = {
	{ "Hunchback II - Quasimodo's Revenge 48K (1984)(Ocean).tap", 49221, 0x65a9fcbe, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Spechunchbac2, Spechunchbac2, Spectrum)
STD_ROM_FN(Spechunchbac2)

struct BurnDriver BurnSpechunchbac2 = {
	"spec_hunchbac2", NULL, "spec_spectrum", NULL, "1985",
	"Hunchback II - Quasimodo's Revenge (48K)\0", NULL, "Ocean", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, Spechunchbac2RomInfo, Spechunchbac2RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};


// Hunchback: The Adventure (48K)

static struct BurnRomInfo SpecHunchadvRomDesc[] = {
	{ "Hunchback - The Adventure 48K (1986)(Ocean).tap", 116193, 0x63a4ccc9, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecHunchadv, SpecHunchadv, Spectrum)
STD_ROM_FN(SpecHunchadv)

struct BurnDriver BurnSpecHunchadv = {
	"spec_hunchadv", NULL, "spec_spectrum", NULL, "1986",
	"Hunchback: The Adventure (48K)\0", "NB: collect all items for next part, there is no way back!", "Ocean", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_ADV, 0,
	SpectrumGetZipName, SpecHunchadvRomInfo, SpecHunchadvRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Hunchback: The Adventure - Part 1 (48K)

static struct BurnRomInfo SpecHunchadvp1RomDesc[] = {
	{ "Hunchback - The Adventure - Part 1 48K (1986)(Ocean).tap", 40956, 0x812dc41b, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecHunchadvp1, SpecHunchadvp1, Spectrum)
STD_ROM_FN(SpecHunchadvp1)

struct BurnDriver BurnSpecHunchadvp1 = {
	"spec_hunchadvp1", "spec_hunchadv", "spec_spectrum", NULL, "1986",
	"Hunchback: The Adventure - Part 1 (48K)\0", "NB: the opening screen is corrupted, but the game is fully playable", "Ocean", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE, 1, HARDWARE_SPECTRUM, GBF_ADV, 0,
	SpectrumGetZipName, SpecHunchadvp1RomInfo, SpecHunchadvp1RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Hunchback: The Adventure - Part 2 (48K)

static struct BurnRomInfo SpecHunchadvp2RomDesc[] = {
	{ "Hunchback - The Adventure - Part 2 48K (1986)(Ocean).tap", 42701, 0x00d6dd37, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecHunchadvp2, SpecHunchadvp2, Spectrum)
STD_ROM_FN(SpecHunchadvp2)

struct BurnDriver BurnSpecHunchadvp2 = {
	"spec_hunchadvp2", "spec_hunchadv", "spec_spectrum", NULL, "1986",
	"Hunchback: The Adventure - Part 2 (48K)\0", "NB: the opening screen is corrupted, but the game is fully playable", "Ocean", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE, 1, HARDWARE_SPECTRUM, GBF_ADV, 0,
	SpectrumGetZipName, SpecHunchadvp2RomInfo, SpecHunchadvp2RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Hunchback: The Adventure - Part 3 (48K)

static struct BurnRomInfo SpecHunchadvp3RomDesc[] = {
	{ "Hunchback - The Adventure - Part 3 48K (1986)(Ocean).tap", 44533, 0x27c39c33, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecHunchadvp3, SpecHunchadvp3, Spectrum)
STD_ROM_FN(SpecHunchadvp3)

struct BurnDriver BurnSpecHunchadvp3 = {
	"spec_hunchadvp3", "spec_hunchadv", "spec_spectrum", NULL, "1986",
	"Hunchback: The Adventure - Part 3 (48K)\0", "NB: the opening screen is corrupted, but the game is fully playable", "Ocean", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE, 1, HARDWARE_SPECTRUM, GBF_ADV, 0,
	SpectrumGetZipName, SpecHunchadvp3RomInfo, SpecHunchadvp3RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Hunchy ~ Quasimodo (16K)

static struct BurnRomInfo SpecHunchyRomDesc[] = {
	{ "Hunchy - Quasimodo 16K (1983)(Your Computer).tap", 9560, 0x7c7f85ad, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecHunchy, SpecHunchy, Spectrum)
STD_ROM_FN(SpecHunchy)

struct BurnDriver BurnSpecHunchy = {
	"spec_hunchy", NULL, "spec_spectrum", NULL, "1983",
	"Hunchy ~ Quasimodo (16K)\0", NULL, "Your Computer", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_ACTION, 0,
	SpectrumGetZipName, SpecHunchyRomInfo, SpecHunchyRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Huxley Pig - Part 1 (48K)

static struct BurnRomInfo SpecHuxleyp1RomDesc[] = {
	{ "Huxley Pig part1 48K (1991)(Alternative).tap", 48133, 0x4d61bad5, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecHuxleyp1, SpecHuxleyp1, Spectrum)
STD_ROM_FN(SpecHuxleyp1)

struct BurnDriver BurnSpecHuxleyp1 = {
	"spec_huxleyp1", NULL, "spec_spectrum", NULL, "1991",
	"Huxley Pig - Part 1 (48K)\0", NULL, "Alternative Software", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_ADV, 0,
	SpectrumGetZipName, SpecHuxleyp1RomInfo, SpecHuxleyp1RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Huxley Pig - Part 2 (48K)

static struct BurnRomInfo SpecHuxleyp2RomDesc[] = {
	{ "Huxley Pig part2 48K (1991)(Alternative).tap", 48133, 0x2b75d8be, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecHuxleyp2, SpecHuxleyp2, Spectrum)
STD_ROM_FN(SpecHuxleyp2)

struct BurnDriver BurnSpecHuxleyp2 = {
	"spec_huxleyp2", "spec_huxleyp1", "spec_spectrum", NULL, "1991",
	"Huxley Pig - Part 2 (48K)\0", "Password: 5BJV7BYN", "Alternative Software", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE, 1, HARDWARE_SPECTRUM, GBF_ACTION | GBF_PUZZLE, 0,
	SpectrumGetZipName, SpecHuxleyp2RomInfo, SpecHuxleyp2RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Hydra (128K)

static struct BurnRomInfo SpecHydraRomDesc[] = {
	{ "Hydra 128K (1991)(Domark).tap", 133588, 0x78856d99, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecHydra, SpecHydra, Spec128)
STD_ROM_FN(SpecHydra)

struct BurnDriver BurnSpecHydra = {
	"spec_hydra", NULL, "spec_spec128", NULL, "1991",
	"Hydra (128K)\0", NULL, "Domark", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_SHOOT | GBF_RACING, 0,
	SpectrumGetZipName, SpecHydraRomInfo, SpecHydraRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecIntf2DIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Hysteria (48K)

static struct BurnRomInfo SpecHysteriaRomDesc[] = {
	{ "Hysteria 48K (1987)(Software Projects).tap", 47426, 0x70fcbb59, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecHysteria, SpecHysteria, Spectrum)
STD_ROM_FN(SpecHysteria)

struct BurnDriver BurnSpecHysteria = {
	"spec_hysteria", NULL, "spec_spectrum", NULL, "1987",
	"Hysteria (48K)\0", NULL, "Software Projects", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_RUNGUN, 0,
	SpectrumGetZipName, SpecHysteriaRomInfo, SpecHysteriaRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// I, Ball (48K)

static struct BurnRomInfo SpeciballRomDesc[] = {
	{ "I, Ball 48K (1987)(Firebird).tap", 53775, 0x4257b017, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Speciball, Speciball, Spectrum)
STD_ROM_FN(Speciball)

struct BurnDriver BurnSpeciball = {
	"spec_iball", NULL, "spec_spectrum", NULL, "1987",
	"I, Ball (48K)\0", "Press '3' to use controller", "Firebird Software", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_ACTION, 0,
	SpectrumGetZipName, SpeciballRomInfo, SpeciballRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecIntf2DIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// I, Ball II (48K)

static struct BurnRomInfo SpecIball2RomDesc[] = {
	{ "I, Ball II 48K (1987)(Firebird).tap", 40691, 0x8ddcdfc8, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecIball2, SpecIball2, Spectrum)
STD_ROM_FN(SpecIball2)

struct BurnDriver BurnSpecIball2 = {
	"spec_iball2", NULL, "spec_spectrum", NULL, "1987",
	"I, Ball II (48K)\0", "Press '3' to use controller", "Firebird Software", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_ACTION, 0,
	SpectrumGetZipName, SpecIball2RomInfo, SpecIball2RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecIntf2DIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Ice Temple, The (48K)

static struct BurnRomInfo SpecIcetempleRomDesc[] = {
	{ "Ice Temple, The 48K (1986)(Bubble Bus).tap", 45017, 0x269dc9de, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecIcetemple, SpecIcetemple, Spectrum)
STD_ROM_FN(SpecIcetemple)

struct BurnDriver BurnSpecIcetemple = {
	"spec_icetemple", NULL, "spec_spectrum", NULL, "1986",
	"Ice Temple, The (48K)\0", NULL, "Bubble Bus", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_SHOOT | GBF_ADV, 0,
	SpectrumGetZipName, SpecIcetempleRomInfo, SpecIcetempleRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Impossaball (48K)

static struct BurnRomInfo SpecImpossaballRomDesc[] = {
	{ "Impossaball 48K (1987)(Hewson Consultants).tap", 45993, 0x158525a4, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecImpossaball, SpecImpossaball, Spectrum)
STD_ROM_FN(SpecImpossaball)

struct BurnDriver BurnSpecImpossaball = {
	"spec_impossaball", NULL, "spec_spectrum", NULL, "1987",
	"Impossaball (48K)\0", NULL, "Hewson Consultants", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_ACTION, 0,
	SpectrumGetZipName, SpecImpossaballRomInfo, SpecImpossaballRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Impossible Mission II (48K)

static struct BurnRomInfo SpecImpmission2RomDesc[] = {
	{ "Impossible Mission II 48K (1988)(U.S. Gold).z80", 44256, 0xea95490a, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecImpmission2, SpecImpmission2, Spec128)
STD_ROM_FN(SpecImpmission2)

struct BurnDriver BurnSpecImpmission2 = {
	"spec_impmission2", NULL, "spec_spec128", NULL, "1988",
	"Impossible Mission II (48K)\0", NULL, "U.S. Gold", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_PLATFORM | GBF_ADV, 0,
	SpectrumGetZipName, SpecImpmission2RomInfo, SpecImpmission2RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Incredible Shrinking Fireman, The (48K)

static struct BurnRomInfo SpecincshrfRomDesc[] = {
	{ "Incredible Shrinking Fireman, The 48K (1986)(Mastertronic).tap", 46523, 0x1940b148, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specincshrf, Specincshrf, Spectrum)
STD_ROM_FN(Specincshrf)

struct BurnDriver BurnSpecincshrf = {
	"spec_incshrf", NULL, "spec_spectrum", NULL, "1986",
	"Incredible Shrinking Fireman, The (48K)\0", NULL, "Mastertronic", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecincshrfRomInfo, SpecincshrfRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Indiana Jones and the Fate of Atlantis (128K)

static struct BurnRomInfo SpecindyfateatlantisRomDesc[] = {
	{ "Indiana Jones and the Fate of Atlantis 128K (1992)(U.S. Gold).tap", 366350, 0xb6341899, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specindyfateatlantis, Specindyfateatlantis, Spec128)
STD_ROM_FN(Specindyfateatlantis)

struct BurnDriver BurnSpecindyfateatlantis = {
	"spec_indyfateatlantis", NULL, "spec_spec128", NULL, "1992",
	"Indiana Jones and the Fate of Atlantis (128K)\0", NULL, "U.S. Gold", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_ACTION | GBF_ADV, 0,
	SpectrumGetZipName, SpecindyfateatlantisRomInfo, SpecindyfateatlantisRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecIntf2DIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Indiana Jones and the Last Crusade (128K)

static struct BurnRomInfo SpecindylastcrusadeRomDesc[] = {
	{ "Indiana Jones and the Last Crusade 128K (1989)(U.S. Gold).tap", 145470, 0xe43fcead, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specindylastcrusade, Specindylastcrusade, Spec128)
STD_ROM_FN(Specindylastcrusade)

struct BurnDriver BurnSpecindylastcrusade = {
	"spec_indylastcrusade", NULL, "spec_spec128", NULL, "1989",
	"Indiana Jones and the Last Crusade (128K)\0", NULL, "U.S. Gold", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_PLATFORM | GBF_ADV, 0,
	SpectrumGetZipName, SpecindylastcrusadeRomInfo, SpecindylastcrusadeRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Indiana Jones and the Temple of Doom (48K)

static struct BurnRomInfo SpecindytempledoomRomDesc[] = {
	{ "Indiana Jones and the Temple of Doom 48K (1987)(U.S. Gold).tap", 76529, 0x2c9fa63e, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specindytempledoom, Specindytempledoom, Spectrum)
STD_ROM_FN(Specindytempledoom)

struct BurnDriver BurnSpecindytempledoom = {
	"spec_indytempledoom", NULL, "spec_spectrum", NULL, "1987",
	"Indiana Jones and the Temple of Doom (48K)\0", NULL, "U.S. Gold", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_ACTION, 0,
	SpectrumGetZipName, SpecindytempledoomRomInfo, SpecindytempledoomRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Indoor Soccer (48K)

static struct BurnRomInfo SpecindrsoccrRomDesc[] = {
	{ "Indoor Soccer 48K (1986)(Magnificent 7 Software).z80", 0x070ed, 0x85a1a21f, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specindrsoccr, Specindrsoccr, Spectrum)
STD_ROM_FN(Specindrsoccr)

struct BurnDriver BurnSpecindrsoccr = {
	"spec_indrsoccr", NULL, "spec_spectrum", NULL, "1986",
	"Indoor Soccer (48K)\0", NULL, "Magnificent 7 Software", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 2, HARDWARE_SPECTRUM, GBF_SPORTSFOOTBALL, 0,
	SpectrumGetZipName, SpecindrsoccrRomInfo, SpecindrsoccrRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Inspector Gadget and the Circus of Fear (48K)

static struct BurnRomInfo SpecInspgadgetRomDesc[] = {
	{ "Inspector Gadget and the Circus of Fear 48K (1987)(Melbourne House).tap", 44729, 0xc9dfdd10, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecInspgadget, SpecInspgadget, Spec128)
STD_ROM_FN(SpecInspgadget)

struct BurnDriver BurnSpecInspgadget = {
	"spec_inspgadget", NULL, "spec_spec128", NULL, "1987",
	"Inspector Gadget and the Circus of Fear (48K)\0", NULL, "Melbourne House", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_ACTION, 0,
	SpectrumGetZipName, SpecInspgadgetRomInfo, SpecInspgadgetRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Intensity (48K)

static struct BurnRomInfo SpecIntensityRomDesc[] = {
	{ "Intensity 48K (1988)(Firebird).tap", 45643, 0x77a12914, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecIntensity, SpecIntensity, Spectrum)
STD_ROM_FN(SpecIntensity)

struct BurnDriver BurnSpecIntensity = {
	"spec_intensity", NULL, "spec_spectrum", NULL, "1988",
	"Intensity (48K)\0", "Press 1: one player option - Press 2: two player option", "Firebird Software", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 2, HARDWARE_SPECTRUM, GBF_ACTION, 0,
	SpectrumGetZipName, SpecIntensityRomInfo, SpecIntensityRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecIntf2DIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// International Karate - Stage 1 (48K)

static struct BurnRomInfo SpecIntkarates1RomDesc[] = {
	{ "International Karate - Stage 1 48K (1985)(System 3).tap", 41266, 0xefa7860d, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecIntkarates1, SpecIntkarates1, Spectrum)
STD_ROM_FN(SpecIntkarates1)

struct BurnDriver BurnSpecIntkarates1 = {
	"spec_intkarates1", NULL, "spec_spectrum", NULL, "1985",
	"International Karate - Stage 1 (48K)\0", NULL, "System 3", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 2, HARDWARE_SPECTRUM, GBF_VSFIGHT, 0,
	SpectrumGetZipName, SpecIntkarates1RomInfo, SpecIntkarates1RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// International Karate - Stage 2 (48K)

static struct BurnRomInfo SpecIntkarates2RomDesc[] = {
	{ "International Karate - Stage 2 48K (1985)(System 3).tap", 47713, 0x89bfad75, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecIntkarates2, SpecIntkarates2, Spectrum)
STD_ROM_FN(SpecIntkarates2)

struct BurnDriver BurnSpecIntkarates2 = {
	"spec_intkarates2", "spec_intkarates1", "spec_spectrum", NULL, "1985",
	"International Karate - Stage 2 (48K)\0", NULL, "System 3", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE, 2, HARDWARE_SPECTRUM, GBF_VSFIGHT, 0,
	SpectrumGetZipName, SpecIntkarates2RomInfo, SpecIntkarates2RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// International Karate+ (128K)

static struct BurnRomInfo SpecintkarateplusRomDesc[] = {
	{ "International Karate+ 128K (1987)(System 3).tap", 0x0d173, 0x097593e6, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specintkarateplus, Specintkarateplus, Spec128)
STD_ROM_FN(Specintkarateplus)

struct BurnDriver BurnSpecintkarateplus = {
	"spec_intkarateplus", NULL, "spec_spec128", NULL, "1987",
	"International Karate+ (128K)\0", NULL, "System 3", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 2, HARDWARE_SPECTRUM, GBF_VSFIGHT, 0,
	SpectrumGetZipName, SpecintkarateplusRomInfo, SpecintkarateplusRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// International Kickboxing (128K)

static struct BurnRomInfo SpecIntkickboxRomDesc[] = {
	{ "International Kickboxing 128K (1993)(Your Sinclair).tap", 38591, 0xeefd494d, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecIntkickbox, SpecIntkickbox, Spec128)
STD_ROM_FN(SpecIntkickbox)

struct BurnDriver BurnSpecIntkickbox = {
	"spec_intkickbox", NULL, "spec_spec128", NULL, "1993",
	"International Kickboxing (128K)\0", "Use 'Space' and 'Enter' to choose and set options", "Your Sinclair", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 2, HARDWARE_SPECTRUM, GBF_VSFIGHT, 0,
	SpectrumGetZipName, SpecIntkickboxRomInfo, SpecIntkickboxRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// International Ninja Rabbits (128K)

static struct BurnRomInfo SpecIntninjarabRomDesc[] = {
	{ "International Ninja Rabbits 128K (1991)(Micro Value).tap", 87287, 0xf55bc727, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecIntninjarab, SpecIntninjarab, Spec128)
STD_ROM_FN(SpecIntninjarab)

struct BurnDriver BurnSpecIntninjarab = {
	"spec_intninjarab", NULL, "spec_spec128", NULL, "1991",
	"International Ninja Rabbits (128K)\0", NULL, "Micro Value", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_SCRFIGHT, 0,
	SpectrumGetZipName, SpecIntninjarabRomInfo, SpecIntninjarabRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Into the Eagle's Nest (48K-128K)

static struct BurnRomInfo SpecinteagnRomDesc[] = {
	{ "Into the Eagle's Nest (1987)(Pandora).tap", 40894, 0x289b4c8e, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specinteagn, Specinteagn, Spec128)
STD_ROM_FN(Specinteagn)

struct BurnDriver BurnSpecinteagn = {
	"spec_inteagn", NULL, "spec_spec128", NULL, "1987",
	"Into the Eagle's Nest (48K-128K)\0", NULL, "Pandora", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_MAZE | GBF_RUNGUN, 0,
	SpectrumGetZipName, SpecinteagnRomInfo, SpecinteagnRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Invaders (16K)

static struct BurnRomInfo SpecInvadersRomDesc[] = {
	{ "Invaders 16K (1983)(DK'Tronics).tap", 6795, 0xa6263eb9, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecInvaders, SpecInvaders, Spectrum)
STD_ROM_FN(SpecInvaders)

struct BurnDriver BurnSpecInvaders = {
	"spec_invaders", NULL, "spec_spectrum", NULL, "1983",
	"Invaders (16K)\0", NULL, "DK'Tronics", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_SHOOT, 0,
	SpectrumGetZipName, SpecInvadersRomInfo, SpecInvadersRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecCursorKeysDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Iron Lord (128K)

static struct BurnRomInfo SpecIronlordRomDesc[] = {
	{ "Iron Lord 128K (1989)(Ubi Soft).tap", 195062, 0xaaefd67c, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecIronlord, SpecIronlord, Spec128)
STD_ROM_FN(SpecIronlord)

struct BurnDriver BurnSpecIronlord = {
	"spec_ironlord", NULL, "spec_spec128", NULL, "1989",
	"Iron Lord (128K)\0", NULL, "Ubi Soft", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_ADV, 0,
	SpectrumGetZipName, SpecIronlordRomInfo, SpecIronlordRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecIntf2DIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Ivan 'Ironman' Stewart's Super Off Road Racer (128K)

static struct BurnRomInfo SpecironmanRomDesc[] = {
	{ "Ivan 'Ironman' Stewart's Super Off Road Racer 128K (1990)(Virgin Games).tap", 40051, 0x68cf1ab9, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specironman, Specironman, Spec128)
STD_ROM_FN(Specironman)

struct BurnDriver BurnSpecironman = {
	"spec_ironman", NULL, "spec_spec128", NULL, "1990",
	"Ivan 'Ironman' Stewart's Super Off Road Racer (128K)\0", "Press 6, 7 and ENTER to set the menu settings", "Virgin Games", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 2, HARDWARE_SPECTRUM, GBF_RACING, 0,
	SpectrumGetZipName, SpecironmanRomInfo, SpecironmanRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Jackson City (48K)

static struct BurnRomInfo SpecJacksoncityRomDesc[] = {
	{ "Jackson City 48K (1990)(Diabolic).tap", 42878, 0x320d1aec, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecJacksoncity, SpecJacksoncity, Spectrum)
STD_ROM_FN(SpecJacksoncity)

struct BurnDriver BurnSpecJacksoncity = {
	"spec_jacksoncity", NULL, "spec_spectrum", NULL, "1990",
	"Jackson City (48K)\0", NULL, "Diabolic", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_VERSHOOT, 0,
	SpectrumGetZipName, SpecJacksoncityRomInfo, SpecJacksoncityRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Jahangir Khan's World Championship Squash - Club Game (128K)

static struct BurnRomInfo SpecJksquashcgRomDesc[] = {
	{ "Jahangir Khan's World Championship Squash - Club Game 128K (1991)(Krisalis).tap", 102623, 0xf94e2be3, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecJksquashcg, SpecJksquashcg, Spec128)
STD_ROM_FN(SpecJksquashcg)

struct BurnDriver BurnSpecJksquashcg = {
	"spec_jksquashcg", NULL, "spec_spec128", NULL, "1991",
	"Jahangir Khan's World Championship Squash - Club Game (128K)\0", NULL, "Krisalis", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 2, HARDWARE_SPECTRUM, GBF_SPORTSMISC, 0,
	SpectrumGetZipName, SpecJksquashcgRomInfo, SpecJksquashcgRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecIntf2DIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Jahangir Khan's World Championship Squash - Tournament Game (128K)

static struct BurnRomInfo SpecJksquashtgRomDesc[] = {
	{ "Jahangir Khan's World Championship Squash - Tournament Game 128K (1991)(Krisalis).tap", 109108, 0xc5b34d11, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecJksquashtg, SpecJksquashtg, Spec128)
STD_ROM_FN(SpecJksquashtg)

struct BurnDriver BurnSpecJksquashtg = {
	"spec_jksquashtg", "spec_jksquashcg", "spec_spec128", NULL, "1991",
	"Jahangir Khan's World Championship Squash - Tournament Game (128K)\0", NULL, "Krisalis", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE, 2, HARDWARE_SPECTRUM, GBF_SPORTSMISC, 0,
	SpectrumGetZipName, SpecJksquashtgRomInfo, SpecJksquashtgRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecIntf2DIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Jail Break (48K)

static struct BurnRomInfo SpecJailbreakRomDesc[] = {
	{ "Jail Break 48K (1987)(Konami).z80", 44911, 0x9ee208aa, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecJailbreak, SpecJailbreak, Spectrum)
STD_ROM_FN(SpecJailbreak)

struct BurnDriver BurnSpecJailbreak = {
	"spec_jailbreak", NULL, "spec_spectrum", NULL, "1987",
	"Jail Break (48K)\0", NULL, "Konami", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_RUNGUN, 0,
	SpectrumGetZipName, SpecJailbreakRomInfo, SpecJailbreakRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Jasper! (48K)

static struct BurnRomInfo SpecJasperRomDesc[] = {
	{ "Jasper! 48K (1984)(Micromega).tap", 50132, 0xc2bd0b3e, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecJasper, SpecJasper, Spectrum)
STD_ROM_FN(SpecJasper)

struct BurnDriver BurnSpecJasper = {
	"spec_jasper", NULL, "spec_spectrum", NULL, "1984",
	"Jasper! (48K)\0", "Use A, S, O and L to move the main character", "Micromega", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_ACTION, 0,
	SpectrumGetZipName, SpecJasperRomInfo, SpecJasperRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Jaws (128K)

static struct BurnRomInfo SpecJawsRomDesc[] = {
	{ "Jaws 128K (1989)(Screen 7).tap", 70245, 0x5234e440, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecJaws, SpecJaws, Spec128)
STD_ROM_FN(SpecJaws)

struct BurnDriver BurnSpecJaws = {
	"spec_jaws", NULL, "spec_spec128", NULL, "1989",
	"Jaws (128K)\0", NULL, "Screen 7", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_SHOOT, 0,
	SpectrumGetZipName, SpecJawsRomInfo, SpecJawsRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Jawz (16K)

static struct BurnRomInfo SpecJawzRomDesc[] = {
	{ "Jawz 16K (1983)(Elfin Software).tap", 12530, 0x3fede27f, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecJawz, SpecJawz, Spectrum)
STD_ROM_FN(SpecJawz)

struct BurnDriver BurnSpecJawz = {
	"spec_jawz", NULL, "spec_spectrum", NULL, "1983",
	"Jawz (16K)\0", NULL, "Elfin Software", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 2, HARDWARE_SPECTRUM, GBF_SHOOT, 0,
	SpectrumGetZipName, SpecJawzRomInfo, SpecJawzRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Jester Quest (128K)

static struct BurnRomInfo SpecJesterquestRomDesc[] = {
	{ "Jester Quest 128K (1988)(Zenobi).tap", 126145, 0x67e918d2, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecJesterquest, SpecJesterquest, Spec128)
STD_ROM_FN(SpecJesterquest)

struct BurnDriver BurnSpecJesterquest = {
	"spec_jesterquest", NULL, "spec_spec128", NULL, "1988",
	"Jester Quest (128K)\0", NULL, "Zenobi Software", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_ADV, 0,
	SpectrumGetZipName, SpecJesterquestRomInfo, SpecJesterquestRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Jonny Quest in Doctor Zin's Underworld (128K)

static struct BurnRomInfo SpecJonnyquestRomDesc[] = {
	{ "Jonny Quest in Doctor Zin's Underworld 128K (1991)(Hi-Tec).tap", 54349, 0xd23f2c38, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecJonnyquest, SpecJonnyquest, Spec128)
STD_ROM_FN(SpecJonnyquest)

struct BurnDriver BurnSpecJonnyquest = {
	"spec_jonnyquest", NULL, "spec_spec128", NULL, "1991",
	"Jonny Quest in Doctor Zin's Underworld (128K)\0", NULL, "Hi-Tec Software", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecJonnyquestRomInfo, SpecJonnyquestRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Jumping Jack (48K)

static struct BurnRomInfo SpecJjackRomDesc[] = {
	{ "Jumping Jack (1983)(Imagine Software).tap", 11833, 0xc54ae2a8, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecJjack, SpecJjack, Spectrum)
STD_ROM_FN(SpecJjack)

struct BurnDriver BurnSpecJjack = {
	"spec_jjack", NULL, "spec_spectrum", NULL, "1983",
	"Jumping Jack (48K)\0", NULL, "Imagine Software", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_ACTION, 0,
	SpectrumGetZipName, SpecJjackRomInfo, SpecJjackRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecJJackDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Jungle Warrior (48K)

static struct BurnRomInfo SpecJwarriorRomDesc[] = {
	{ "Jungle Warrior 48K ES (1990)(Zigurat).tap", 46516, 0x7eaa77fe, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecJwarrior, SpecJwarrior, Spectrum)
STD_ROM_FN(SpecJwarrior)

struct BurnDriver BurnSpecJwarrior = {
	"spec_jwarrior", NULL, "spec_spectrum", NULL, "1990",
	"Jungle Warrior (Spanish) (48K)\0", NULL, "Zigurat Software", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_RUNGUN | GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecJwarriorRomInfo, SpecJwarriorRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Kamikaze (French) (16K)

static struct BurnRomInfo SpecKami16kRomDesc[] = {
	{ "Kamikaze 16K FR (1983)(Eyrolles).z80", 12096, 0x2baab69b, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecKami16k, SpecKami16k, Spectrum)
STD_ROM_FN(SpecKami16k)

struct BurnDriver BurnSpecKami16k = {
	"spec_kami16k", NULL, "spec_spectrum", NULL, "1983",
	"Kamikaze (French) (16K)\0", "L-Shift: left - Z: right - Space: fire", "Eyrolles", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_SHOOT, 0,
	SpectrumGetZipName, SpecKami16kRomInfo, SpecKami16kRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Kayleth (48K)

static struct BurnRomInfo SpecKaylethRomDesc[] = {
	{ "Kayleth 48K (1986)(U.S. Gold).tap", 48435, 0x5518a8e2, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecKayleth, SpecKayleth, Spectrum)
STD_ROM_FN(SpecKayleth)

struct BurnDriver BurnSpecKayleth = {
	"spec_kayleth", NULL, "spec_spectrum", NULL, "1986",
	"Kayleth (48K)\0", NULL, "U.S. Gold", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_ADV, 0,
	SpectrumGetZipName, SpecKaylethRomInfo, SpecKaylethRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Kenny Dalglish Soccer Manager (48K)

static struct BurnRomInfo SpecKdmanagerRomDesc[] = {
	{ "Kenny Dalglish Soccer Manager 48K (1989)(Cognito).tap", 54337, 0xaaa8bbb3, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecKdmanager, SpecKdmanager, Spectrum)
STD_ROM_FN(SpecKdmanager)

struct BurnDriver BurnSpecKdmanager = {
	"spec_kdmanager", NULL, "spec_spectrum", NULL, "1989",
	"Kenny Dalglish Soccer Manager (48K)\0", NULL, "Cognito", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_STRATEGY, 0,
	SpectrumGetZipName, SpecKdmanagerRomInfo, SpecKdmanagerRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Kenny Dalglish Soccer Match (128K)

static struct BurnRomInfo SpecKdmatchRomDesc[] = {
	{ "Kenny Dalglish Soccer Match 128K (1990)(Impressions).tap", 55717, 0xf9447e26, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecKdmatch, SpecKdmatch, Spec128)
STD_ROM_FN(SpecKdmatch)

struct BurnDriver BurnSpecKdmatch = {
	"spec_kdmatch", NULL, "spec_spec128", NULL, "1990",
	"Kenny Dalglish Soccer Match (128K)\0", NULL, "Impressions", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 2, HARDWARE_SPECTRUM, GBF_SPORTSFOOTBALL, 0,
	SpectrumGetZipName, SpecKdmatchRomInfo, SpecKdmatchRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Kinetik (48K)

static struct BurnRomInfo SpecKinetikRomDesc[] = {
	{ "Kinetik 48K (1987)(Firebird).z80", 43221, 0x39a23892, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecKinetik, SpecKinetik, Spectrum)
STD_ROM_FN(SpecKinetik)

struct BurnDriver BurnSpecKinetik = {
	"spec_kinetik", NULL, "spec_spectrum", NULL, "1987",
	"Kinetik (48K)\0", NULL, "Firebird Software", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_ACTION, 0,
	SpectrumGetZipName, SpecKinetikRomInfo, SpecKinetikRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// King Arthur's Quest (48K)

static struct BurnRomInfo SpecKaquestRomDesc[] = {
	{ "King Arthur's Quest 48K (1984)(Hill MacGibbon).tap", 51142, 0x461d1af1, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecKaquest, SpecKaquest, Spectrum)
STD_ROM_FN(SpecKaquest)

struct BurnDriver BurnSpecKaquest = {
	"spec_kaquest", NULL, "spec_spectrum", NULL, "1984",
	"King Arthur's Quest (48K)\0", "Use numeric keys to play", "Hill MacGibbon", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_ADV, 0,
	SpectrumGetZipName, SpecKaquestRomInfo, SpecKaquestRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// King Valley (128K)

static struct BurnRomInfo SpecKingvalleyRomDesc[] = {
	{ "King Valley 128K (1994)(WeByBelarus).tap", 39820, 0x1d1ebe53, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecKingvalley, SpecKingvalley, Spec128)
STD_ROM_FN(SpecKingvalley)

struct BurnDriver BurnSpecKingvalley = {
	"spec_kingvalley", NULL, "spec_spec128", NULL, "1994",
	"King Valley (128K)\0", NULL, "WE [BY] (Belarus)", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecKingvalleyRomInfo, SpecKingvalleyRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Klax (128K)

static struct BurnRomInfo SpecKlaxRomDesc[] = {
	{ "Klax 128K (1990)(Domark).tap", 138496, 0x591ded31, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecKlax, SpecKlax, Spec128)
STD_ROM_FN(SpecKlax)

struct BurnDriver BurnSpecKlax = {
	"spec_klax", NULL, "spec_spec128", NULL, "1990",
	"Klax (128K)\0", NULL, "Domark", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_PUZZLE, 0,
	SpectrumGetZipName, SpecKlaxRomInfo, SpecKlaxRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Knight Force (128K)

static struct BurnRomInfo SpecKnightforceRomDesc[] = {
	{ "Knight Force (1990)(Titus).tap", 82635, 0xf1ed8323, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecKnightforce, SpecKnightforce, Spec128)
STD_ROM_FN(SpecKnightforce)

struct BurnDriver BurnSpecKnightforce = {
	"spec_knightforce", NULL, "spec_spec128", NULL, "1990",
	"Knight Force (128K)\0", NULL, "Titus", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_ACTION, 0,
	SpectrumGetZipName, SpecKnightforceRomInfo, SpecKnightforceRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Knightmare (48K)

static struct BurnRomInfo SpecKnightmareRomDesc[] = {
	{ "Knightmare 48K (1987)(Activision).tap", 46267, 0x198f02d2, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecKnightmare, SpecKnightmare, Spectrum)
STD_ROM_FN(SpecKnightmare)

struct BurnDriver BurnSpecKnightmare = {
	"spec_knightmare", NULL, "spec_spectrum", NULL, "1987",
	"Knightmare (48K)\0", "Hold down 'fire button' to start game", "Activision", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_ADV, 0,
	SpectrumGetZipName, SpecKnightmareRomInfo, SpecKnightmareRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Knight Rider (48K)

static struct BurnRomInfo SpecKnightriderRomDesc[] = {
	{ "Knight Rider 48K (1986)(Ocean).tap", 49152, 0xdf4e4c85, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecKnightrider, SpecKnightrider, Spectrum)
STD_ROM_FN(SpecKnightrider)

struct BurnDriver BurnSpecKnightrider = {
	"spec_knightrider", NULL, "spec_spectrum", NULL, "1986",
	"Knight Rider (48K)\0", NULL, "Ocean", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_SHOOT | GBF_RACING, 0,
	SpectrumGetZipName, SpecKnightriderRomInfo, SpecKnightriderRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Knot in 3D (48K)

static struct BurnRomInfo SpecKnot3dRomDesc[] = {
	{ "Knot in 3D 48K (1983)(New Generation).tap", 23317, 0xad9d2b5f, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecKnot3d, SpecKnot3d, Spectrum)
STD_ROM_FN(SpecKnot3d)

struct BurnDriver BurnSpecKnot3d = {
	"spec_knot3d", NULL, "spec_spectrum", NULL, "1983",
	"Knot in 3D (48K)\0", "Select 'KEMPSTON joystick' to use controller", "New Generation", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_MAZE, 0,
	SpectrumGetZipName, SpecKnot3dRomInfo, SpecKnot3dRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Konami's Golf (48K)

static struct BurnRomInfo SpecKnmgolfRomDesc[] = {
	{ "Konami's Golf 48K (1986)(Imagine).tap", 43314, 0xb9a1109b, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecKnmgolf, SpecKnmgolf, Spectrum)
STD_ROM_FN(SpecKnmgolf)

struct BurnDriver BurnSpecKnmgolf = {
	"spec_knmgolf", NULL, "spec_spectrum", NULL, "1986",
	"Konami's Golf (48K)\0", NULL, "Imagine Software", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 2, HARDWARE_SPECTRUM, GBF_SPORTSMISC, 0,
	SpectrumGetZipName, SpecKnmgolfRomInfo, SpecKnmgolfRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Konami's Ping Pong (48K)

static struct BurnRomInfo SpecPingpongRomDesc[] = {
	{ "Konami's Ping Pong 48K (1986)(Imagine).tap", 39121, 0xd484a79a, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecPingpong, SpecPingpong, Spectrum)
STD_ROM_FN(SpecPingpong)

struct BurnDriver BurnSpecPingpong = {
	"spec_pingpong", NULL, "spec_spectrum", NULL, "1986",
	"Konami's Ping Pong (48K)\0", NULL, "Imagine Software", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 2, HARDWARE_SPECTRUM, GBF_SPORTSMISC, 0,
	SpectrumGetZipName, SpecPingpongRomInfo, SpecPingpongRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Konami's Tennis (48K)

static struct BurnRomInfo SpecKnmtennisRomDesc[] = {
	{ "Konami's Tennis 48K (1986)(Imagine).tap", 45644, 0xc78dd656, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecKnmtennis, SpecKnmtennis, Spectrum)
STD_ROM_FN(SpecKnmtennis)

struct BurnDriver BurnSpecKnmtennis = {
	"spec_knmtennis", NULL, "spec_spectrum", NULL, "1986",
	"Konami's Tennis (48K)\0", NULL, "Imagine Software", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 2, HARDWARE_SPECTRUM, GBF_SPORTSMISC, 0,
	SpectrumGetZipName, SpecKnmtennisRomInfo, SpecKnmtennisRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Kraal (48K)

static struct BurnRomInfo SpeckraalRomDesc[] = {
	{ "Kraal (1990)(Hewson Consultants).tap", 57614, 0x84276314, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Speckraal, Speckraal, Spectrum)
STD_ROM_FN(Speckraal)

struct BurnDriver BurnSpeckraal = {
	"spec_kraal", NULL, "spec_spectrum", NULL, "1990",
	"Kraal (48K)\0", "Select '1 Keyboard' to use controller", "Hewson Consultants", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_SHOOT, 0,
	SpectrumGetZipName, SpeckraalRomInfo, SpeckraalRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecQAOPSpaceDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Krakout (48K)

static struct BurnRomInfo SpecKrakoutRomDesc[] = {
	{ "Krakout 48K (1987)(Gremlin Graphics).tap", 49145, 0x71d2a0f6, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecKrakout, SpecKrakout, Spectrum)
STD_ROM_FN(SpecKrakout)

struct BurnDriver BurnSpecKrakout = {
	"spec_krakout", NULL, "spec_spectrum", NULL, "1987",
	"Krakout (48K)\0", NULL, "Gremlin Graphics", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_BREAKOUT, 0,
	SpectrumGetZipName, SpecKrakoutRomInfo, SpecKrakoutRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// LA Drugs Bust (128K)

static struct BurnRomInfo SpecLadbustRomDesc[] = {
	{ "LA Drugs Bust 128K (1990)(Players Software).tap", 50858, 0x20fbc4bd, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecLadbust, SpecLadbust, Spec128)
STD_ROM_FN(SpecLadbust)

struct BurnDriver BurnSpecLadbust = {
	"spec_ladbust", NULL, "spec_spec128", NULL, "1990",
	"Los Angeles Drugs Bust (128K)\0", NULL, "Players Software", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_SHOOT, 0,
	SpectrumGetZipName, SpecLadbustRomInfo, SpecLadbustRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Laser Zone (48K)

static struct BurnRomInfo SpecLaserzoneRomDesc[] = {
	{ "Laser Zone (1983)(Salamander Software).tap", 17800, 0x701e7685, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecLaserzone, SpecLaserzone, Spectrum)
STD_ROM_FN(SpecLaserzone)

struct BurnDriver BurnSpecLaserzone = {
	"spec_laserzone", NULL, "spec_spectrum", NULL, "1983",
	"Laser Zone (48K)\0", NULL, "Salamander Software", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 2, HARDWARE_SPECTRUM, GBF_SHOOT, 0,
	SpectrumGetZipName, SpecLaserzoneRomInfo, SpecLaserzoneRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Last Duel (48K)

static struct BurnRomInfo SpecLastduelRomDesc[] = {
	{ "Last Duel (1989)(Go!).tap", 171660, 0xca2ae42a, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecLastduel, SpecLastduel, Spectrum)
STD_ROM_FN(SpecLastduel)

struct BurnDriver BurnSpecLastduel = {
	"spec_lastduel", NULL, "spec_spectrum", NULL, "1989",
	"Last Duel (48K)\0", NULL, "Go!", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 2, HARDWARE_SPECTRUM, GBF_RACING | GBF_VERSHOOT, 0,
	SpectrumGetZipName, SpecLastduelRomInfo, SpecLastduelRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Last Mission, The (Spanish) (128K)

static struct BurnRomInfo SpecLastmissionRomDesc[] = {
	{ "Last Mission, The 128K ES (1987)(Opera Soft).tap", 45998, 0x2e93fbe9, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecLastmission, SpecLastmission, Spec128)
STD_ROM_FN(SpecLastmission)

struct BurnDriver BurnSpecLastmission = {
	"spec_lastmission", NULL, "spec_spec128", NULL, "1987",
	"Last Mission, The (Spanish) (128K)\0", NULL, "Opera Soft", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_SHOOT | GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecLastmissionRomInfo, SpecLastmissionRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecIntf2DIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Lazer Tag (48K)

static struct BurnRomInfo SpecLazertagRomDesc[] = {
	{ "Lazer Tag 48K (1988)(Go!).tap", 29696, 0xa88a379d, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecLazertag, SpecLazertag, Spectrum)
STD_ROM_FN(SpecLazertag)

struct BurnDriver BurnSpecLazertag = {
	"spec_lazertag", NULL, "spec_spectrum", NULL, "1988",
	"Lazer Tag (48K)\0", NULL, "Go!", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 2, HARDWARE_SPECTRUM, GBF_ACTION | GBF_SHOOT, 0,
	SpectrumGetZipName, SpecLazertagRomInfo, SpecLazertagRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Leader Board (48K)

static struct BurnRomInfo SpecLeaderboardRomDesc[] = {
	{ "Leader Board 48K (1987)(U.S. Gold).tap", 45712, 0x97794c06, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecLeaderboard, SpecLeaderboard, Spectrum)
STD_ROM_FN(SpecLeaderboard)

struct BurnDriver BurnSpecLeaderboard = {
	"spec_leaderboard", NULL, "spec_spectrum", NULL, "1987",
	"Leader Board (48K)\0", NULL, "U.S. Gold", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 4, HARDWARE_SPECTRUM, GBF_SPORTSMISC, 0,
	SpectrumGetZipName, SpecLeaderboardRomInfo, SpecLeaderboardRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Leapfrog (16K)

static struct BurnRomInfo SpecLeapfrogRomDesc[] = {
	{ "Leapfrog 16K (1983)(CDS Microsystems).tap", 14349, 0x03de02e5, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecLeapfrog, SpecLeapfrog, Spectrum)
STD_ROM_FN(SpecLeapfrog)

struct BurnDriver BurnSpecLeapfrog = {
	"spec_leapfrog", NULL, "spec_spectrum", NULL, "1983",
	"Leapfrog (16K)\0", "Press 'S' to start game", "CDS Microsystems", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_ACTION, 0,
	SpectrumGetZipName, SpecLeapfrogRomInfo, SpecLeapfrogRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecCursorKeysDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// LED Storm - Rally 2011 (128K)

static struct BurnRomInfo SpecLedstormRomDesc[] = {
	{ "LED Storm 128K (1988)(Go!).tap", 86247, 0xd20b9e67, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecLedstorm, SpecLedstorm, Spec128)
STD_ROM_FN(SpecLedstorm)

struct BurnDriver BurnSpecLedstorm = {
	"spec_ledstorm", NULL, "spec_spec128", NULL, "1988",
	"LED Storm - Rally 2011 (128K)\0", NULL, "Go!", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_ACTION | GBF_RACING, 0,
	SpectrumGetZipName, SpecLedstormRomInfo, SpecLedstormRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Legend of Kage, The (48K)

static struct BurnRomInfo SpecLegkageRomDesc[] = {
	{ "Legend of Kage, The 48K (1987)(Imagine).tap", 46959, 0x8433d421, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecLegkage, SpecLegkage, Spectrum)
STD_ROM_FN(SpecLegkage)

struct BurnDriver BurnSpecLegkage = {
	"spec_legkage", NULL, "spec_spectrum", NULL, "1987",
	"Legend of Kage, The (48K)\0", NULL, "Imagine Software", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 2, HARDWARE_SPECTRUM, GBF_ACTION | GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecLegkageRomInfo, SpecLegkageRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Legend of the Amazon Women (48K)

static struct BurnRomInfo SpecLegamawomRomDesc[] = {
	{ "Legend of the Amazon Women 48K (1986)(U.S. Gold).tap", 58363, 0xd5cc16d2, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecLegamawom, SpecLegamawom, Spectrum)
STD_ROM_FN(SpecLegamawom)

struct BurnDriver BurnSpecLegamawom = {
	"spec_legamawom", NULL, "spec_spectrum", NULL, "1986",
	"Legend of the Amazon Women (48K)\0", NULL, "U.S. Gold", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_SCRFIGHT, 0,
	SpectrumGetZipName, SpecLegamawomRomInfo, SpecLegamawomRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Lightforce (48K)

static struct BurnRomInfo SpecLightforceRomDesc[] = {
	{ "Lightforce 48K (1986)(Faster Than Light).tap", 35646, 0xf91ca147, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecLightforce, SpecLightforce, Spectrum)
STD_ROM_FN(SpecLightforce)

struct BurnDriver BurnSpecLightforce = {
	"spec_lightforce", NULL, "spec_spectrum", NULL, "1986",
	"Lightforce (48K)\0", NULL, "Faster Than Light", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_VERSHOOT, 0,
	SpectrumGetZipName, SpecLightforceRomInfo, SpecLightforceRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Little Puff in Dragonland (128K)

static struct BurnRomInfo SpecLittlepuffRomDesc[] = {
	{ "Little Puff in Dragonland (1990)(Codemasters).tap", 42827, 0x30a03963, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecLittlepuff, SpecLittlepuff, Spec128)
STD_ROM_FN(SpecLittlepuff)

struct BurnDriver BurnSpecLittlepuff = {
	"spec_littlepuff", NULL, "spec_spec128", NULL, "1990",
	"Little Puff in Dragonland (128K)\0", NULL, "Codemasters", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_PLATFORM | GBF_ADV, 0,
	SpectrumGetZipName, SpecLittlepuffRomInfo, SpecLittlepuffRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Loco (48K)

static struct BurnRomInfo SpecLocoRomDesc[] = {
	{ "Loco 48K (1986)(Alligata).z80", 29077, 0xed68d9d3, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecLoco, SpecLoco, Spectrum)
STD_ROM_FN(SpecLoco)

struct BurnDriver BurnSpecLoco = {
	"spec_loco", NULL, "spec_spectrum", NULL, "1986",
	"Loco (48K)\0", NULL, "Alligata Software", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_ACTION, 0,
	SpectrumGetZipName, SpecLocoRomInfo, SpecLocoRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Lode Runner - levels 1-150 (128K)

static struct BurnRomInfo Spec_loderunnerRomDesc[] = {
	{ "Lode Runner 128K (1986)(Software Projects).tap", 72752, 0x26929396, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Spec_loderunner, Spec_loderunner, Spec128)
STD_ROM_FN(Spec_loderunner)

struct BurnDriver BurnSpec_loderunner = {
	"spec_loderunner", NULL, "spec_spec128", NULL, "1986",
	"Lode Runner - levels 1-150 (128K)\0", "SHIFT+L: +level, SHIFT+M: +lives, SHIFT+A: abort, SHIFT+R: restart", "Software Projects", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, Spec_loderunnerRomInfo, Spec_loderunnerRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Lode Runner - Side A - levels 1-75 (48K)

static struct BurnRomInfo Spec_loderun48aRomDesc[] = {
	{ "Lode Runner - Side A 48K (1984)(Software Projects).tap", 48037, 0x7e7e4ec7, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Spec_loderun48a, Spec_loderun48a, Spectrum)
STD_ROM_FN(Spec_loderun48a)

struct BurnDriver BurnSpec_loderun48a = {
	"spec_loderun48a", "spec_loderunner", "spec_spectrum", NULL, "1984",
	"Lode Runner - Side A - levels 1-75 (48K)\0", "SHIFT+L: +level, SHIFT+M: +lives, SHIFT+A: abort, SHIFT+R: restart", "Software Projects", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, Spec_loderun48aRomInfo, Spec_loderun48aRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Lode Runner - Side B - levels 76-150 (48K)

static struct BurnRomInfo Spec_loderun48bRomDesc[] = {
	{ "Lode Runner - Side B 48K (1984)(Software Projects).tap", 48037, 0x1a98c993, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Spec_loderun48b, Spec_loderun48b, Spectrum)
STD_ROM_FN(Spec_loderun48b)

struct BurnDriver BurnSpec_loderun48b = {
	"spec_loderun48b", "spec_loderunner", "spec_spectrum", NULL, "1984",
	"Lode Runner - Side B - levels 76-150 (48K)\0", "SHIFT+L: +level, SHIFT+M: +lives, SHIFT+A: abort, SHIFT+R: restart", "Software Projects", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, Spec_loderun48bRomInfo, Spec_loderun48bRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Lone Wolf - The Mirror of Death (128K)

static struct BurnRomInfo SpecLonewolfRomDesc[] = {
	{ "Lone Wolf - The Mirror of Death 128K (1991)(Audiogenic).z80", 69519, 0xf598b404, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecLonewolf, SpecLonewolf, Spec128)
STD_ROM_FN(SpecLonewolf)

struct BurnDriver BurnSpecLonewolf = {
	"spec_lonewolf", NULL, "spec_spec128", NULL, "1990",
	"Lone Wolf - The Mirror of Death (128K)\0", NULL, "Audiogenic", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecLonewolfRomInfo, SpecLonewolfRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecIntf2DIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Loopz (48K)

static struct BurnRomInfo SpecLoopzRomDesc[] = {
	{ "Loopz 48K (1991)(Audiogenic).tap", 44809, 0x27fbfd1e, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecLoopz, SpecLoopz, Spectrum)
STD_ROM_FN(SpecLoopz)

struct BurnDriver BurnSpecLoopz = {
	"spec_loopz", NULL, "spec_spectrum", NULL, "1991",
	"Loopz (48K)\0", "SPACE: rotate the piece - ENTER: put the piece", "Audiogenic", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 2, HARDWARE_SPECTRUM, GBF_PUZZLE, 0,
	SpectrumGetZipName, SpecLoopzRomInfo, SpecLoopzRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecCursorKeysDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Lost Caves and the Tomb of Doom (128K)

static struct BurnRomInfo SpecLostcavesRomDesc[] = {
	{ "Lost Caves and the Tomb of Doom 128K (1989)(Players Premier).tap", 42515, 0x88113a26, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecLostcaves, SpecLostcaves, Spec128)
STD_ROM_FN(SpecLostcaves)

struct BurnDriver BurnSpecLostcaves = {
	"spec_lostcaves", NULL, "spec_spec128", NULL, "1989",
	"Lost Caves and the Tomb of Doom (128K)\0", NULL, "Players Premier", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_ACTION, 0,
	SpectrumGetZipName, SpecLostcavesRomInfo, SpecLostcavesRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Lotus Esprit Turbo Challenge (128K)

static struct BurnRomInfo SpeclotustcRomDesc[] = {
	{ "Lotus Esprit Turbo Challenge (1990)(Gremlin Graphics Software)(128k).z80", 0x159b6, 0xe2a3503d, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Speclotustc, Speclotustc, Spec128)
STD_ROM_FN(Speclotustc)

struct BurnDriver BurnSpeclotustc = {
	"spec_lotustc", NULL, "spec_spec128", NULL, "1990",
	"Lotus Esprit Turbo Challenge (128K)\0", NULL, "Gremlin Graphics", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 2, HARDWARE_SPECTRUM, GBF_RACING, 0,
	SpectrumGetZipName, SpeclotustcRomInfo, SpeclotustcRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecQAOPSpaceDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Luna Crabs (16K)

static struct BurnRomInfo SpecLunacrabsRomDesc[] = {
	{ "Luna Crabs 16K (1983)(Micromega).tap", 16574, 0xffd2fb0b, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecLunacrabs, SpecLunacrabs, Spectrum)
STD_ROM_FN(SpecLunacrabs)

struct BurnDriver BurnSpecLunacrabs = {
	"spec_lunacrabs", NULL, "spec_spectrum", NULL, "1983",
	"Luna Crabs (16K)\0", "1: left - 0: right - 9: fire", "Micromega", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_SHOOT, 0,
	SpectrumGetZipName, SpecLunacrabsRomInfo, SpecLunacrabsRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Lunar Jetman (48K)

static struct BurnRomInfo SpecljetmanRomDesc[] = {
	{ "Lunar Jetman (1983)(Ultimate Play The Game).z80", 0x08e6a, 0x914bc877, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specljetman, Specljetman, Spectrum)
STD_ROM_FN(Specljetman)

struct BurnDriver BurnSpecljetman = {
	"spec_ljetman", NULL, "spec_spectrum", NULL, "1983",
	"Lunar Jetman (48K)\0", NULL, "Ultimate Play The Game", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 2, HARDWARE_SPECTRUM, GBF_HORSHOOT, 0,
	SpectrumGetZipName, SpecljetmanRomInfo, SpecljetmanRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Lunar Rescue (48K)

static struct BurnRomInfo SpeclrescueRomDesc[] = {
	{ "Lunar Rescue (1984)(Lyversoft).z80", 0x046c2, 0x6aac917a, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Speclrescue, Speclrescue, Spectrum)
STD_ROM_FN(Speclrescue)

struct BurnDriver BurnSpeclrescue = {
	"spec_lrescue", NULL, "spec_spectrum", NULL, "1984",
	"Lunar Rescue (48K)\0", NULL, "Lyversoft", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_ACTION, 0,
	SpectrumGetZipName, SpeclrescueRomInfo, SpeclrescueRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Mach 3 (Spanish) (48K)

static struct BurnRomInfo SpecMach3RomDesc[] = {
	{ "Mach 3 ES 48K (1989)(Loricels).tap", 44944, 0xa7f9fe1b, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecMach3, SpecMach3, Spectrum)
STD_ROM_FN(SpecMach3)

struct BurnDriver BurnSpecMach3 = {
	"spec_mach3", NULL, "spec_spectrum", NULL, "1989",
	"Mach 3 (Spanish) (48K)\0", NULL, "Loricels - Proein Soft Line", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_SHOOT, 0,
	SpectrumGetZipName, SpecMach3RomInfo, SpecMach3RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Madballs (128K)

static struct BurnRomInfo SpecMadballsRomDesc[] = {
	{ "Madballs 128K (1988)(Ocean).tap", 47126, 0x08d83d83, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecMadballs, SpecMadballs, Spec128)
STD_ROM_FN(SpecMadballs)

struct BurnDriver BurnSpecMadballs = {
	"spec_madballs", NULL, "spec_spec128", NULL, "1988",
	"Madballs (128K)\0", NULL, "Ocean", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_ACTION, 0,
	SpectrumGetZipName, SpecMadballsRomInfo, SpecMadballsRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Mag Max - Robo Centurion (48K)

static struct BurnRomInfo SpecMagmaxRomDesc[] = {
	{ "Mag Max - Robo Centurion 48K (1987)(Imagine).tap", 48862, 0x11332148, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecMagmax, SpecMagmax, Spectrum)
STD_ROM_FN(SpecMagmax)

struct BurnDriver BurnSpecMagmax = {
	"spec_magmax", NULL, "spec_spectrum", NULL, "1987",
	"Mag Max - Robo Centurion (48K)\0", NULL, "Imagine Software", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_HORSHOOT, 0,
	SpectrumGetZipName, SpecMagmaxRomInfo, SpecMagmaxRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Magnetron (48K)

static struct BurnRomInfo SpecMagnetronRomDesc[] = {
	{ "Magnetron 48K (1988)(Firebird).tap", 48383, 0xf407317e, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecMagnetron, SpecMagnetron, Spectrum)
STD_ROM_FN(SpecMagnetron)

struct BurnDriver BurnSpecMagnetron = {
	"spec_magnetron", NULL, "spec_spectrum", NULL, "1988",
	"Magnetron (48K)\0", NULL, "Firebird Software", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_MAZE, 0,
	SpectrumGetZipName, SpecMagnetronRomInfo, SpecMagnetronRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Manollo - El Cavernicola (Spanish) (48K)

static struct BurnRomInfo SpecManolloRomDesc[] = {
	{ "Manollo - El Cavernicola ES 48K (1988)(Iber Software).tap", 39236, 0x594d3ab5, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecManollo, SpecManollo, Spectrum)
STD_ROM_FN(SpecManollo)

struct BurnDriver BurnSpecManollo = {
	"spec_manollo", NULL, "spec_spectrum", NULL, "1988",
	"Manollo - El Cavernicola (Spanish) (48K)\0", NULL, "Iber Software", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_PLATFORM | GBF_ADV, 0,
	SpectrumGetZipName, SpecManolloRomInfo, SpecManolloRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Martianoids (48K)

static struct BurnRomInfo SpecMartianoidsRomDesc[] = {
	{ "Martianoids 48K (1987)(Ultimate Play The Game).tap", 49438, 0xa1fe1d86, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecMartianoids, SpecMartianoids, Spectrum)
STD_ROM_FN(SpecMartianoids)

struct BurnDriver BurnSpecMartianoids = {
	"spec_martianoids", NULL, "spec_spectrum", NULL, "1987",
	"Martianoids (48K)\0", NULL, "Ultimate Play The Game", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_MAZE, 0,
	SpectrumGetZipName, SpecMartianoidsRomInfo, SpecMartianoidsRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// MASK II (128K)

static struct BurnRomInfo SpecMask2RomDesc[] = {
	{ "MASK II 128K (1987)(Gremlin Graphics).tap", 79774, 0x972b746c, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecMask2, SpecMask2, Spec128)
STD_ROM_FN(SpecMask2)

struct BurnDriver BurnSpecMask2 = {
	"spec_mask2", NULL, "spec_spec128", NULL, "1987",
	"MASK II (128K)\0", NULL, "Gremlin Graphics", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_ACTION | GBF_RUNGUN, 0,
	SpectrumGetZipName, SpecMask2RomInfo, SpecMask2RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// MASK III - VENOM Strikes Back (128K)

static struct BurnRomInfo SpecMask3RomDesc[] = {
	{ "MASK III - VENOM Strikes Back 128K (1988)(Gremlin Graphics).tap", 51714, 0xafcf673d, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecMask3, SpecMask3, Spec128)
STD_ROM_FN(SpecMask3)

struct BurnDriver BurnSpecMask3 = {
	"spec_mask3", NULL, "spec_spec128", NULL, "1988",
	"MASK III - VENOM Strikes Back (128K)\0", NULL, "Gremlin Graphics", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecMask3RomInfo, SpecMask3RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Masters of the Universe - The Super Adventure (48K)

static struct BurnRomInfo SpecMastersadvRomDesc[] = {
	{ "Masters of the Universe - The Super Adventure 48K (1987)(U.S. Gold).z80", 42460, 0x504d562f, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecMastersadv, SpecMastersadv, Spectrum)
STD_ROM_FN(SpecMastersadv)

struct BurnDriver BurnSpecMastersadv = {
	"spec_mastersadv", NULL, "spec_spectrum", NULL, "1987",
	"Masters of the Universe - The Super Adventure (48K)\0", NULL, "U.S. Gold", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_ADV, 0,
	SpectrumGetZipName, SpecMastersadvRomInfo, SpecMastersadvRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Masters of the Universe - The Arcade Game (48K)

static struct BurnRomInfo SpecMastersarcadeRomDesc[] = {
	{ "Masters of the Universe - The Arcade Game 48K (1987)(U.S. Gold).tap", 48552, 0x5bc7a995, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecMastersarcade, SpecMastersarcade, Spectrum)
STD_ROM_FN(SpecMastersarcade)

struct BurnDriver BurnSpecMastersarcade = {
	"spec_mastersarcade", NULL, "spec_spectrum", NULL, "1987",
	"Masters of the Universe - The Arcade Game (48K)\0", NULL, "U.S. Gold", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_ACTION, 0,
	SpectrumGetZipName, SpecMastersarcadeRomInfo, SpecMastersarcadeRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Masters of the Universe - The Movie (48K)

static struct BurnRomInfo SpecMastersmovieRomDesc[] = {
	{ "Masters of the Universe - The Movie 48K (1987)(Gremlin Graphics).tap", 44676, 0xd4f3892a, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecMastersmovie, SpecMastersmovie, Spectrum)
STD_ROM_FN(SpecMastersmovie)

struct BurnDriver BurnSpecMastersmovie = {
	"spec_mastersmovie", NULL, "spec_spectrum", NULL, "1987",
	"Masters of the Universe - The Movie (48K)\0", NULL, "Gremlin Graphics", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_RUNGUN, 0,
	SpectrumGetZipName, SpecMastersmovieRomInfo, SpecMastersmovieRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Masterword (Prototype) (48K)

static struct BurnRomInfo SpecMasterwordRomDesc[] = {
	{ "Masterword 48K (Proto)(1989)(Atlantis Software).tap", 49331, 0xc004b7d2, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecMasterword, SpecMasterword, Spectrum)
STD_ROM_FN(SpecMasterword)

struct BurnDriver BurnSpecMasterword = {
	"spec_masterword", NULL, "spec_spectrum", NULL, "1989",
	"Masterword (Prototype) (48K)\0", NULL, "Atlantis Software", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_PROTOTYPE, 4, HARDWARE_SPECTRUM, GBF_PUZZLE, 0,
	SpectrumGetZipName, SpecMasterwordRomInfo, SpecMasterwordRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Mega-Corp - Parte 1 (Spanish) (48K)

static struct BurnRomInfo SpecMegacorp1RomDesc[] = {
	{ "Mega-Corp - Parte 1 ES (1987)(Dinamic).tap", 49246, 0x1fe3c489, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecMegacorp1, SpecMegacorp1, Spectrum)
STD_ROM_FN(SpecMegacorp1)

struct BurnDriver BurnSpecMegacorp1 = {
	"spec_megacorp1", NULL, "spec_spectrum", NULL, "1987",
	"Mega-Corp - Parte 1 (Spanish) (48K)\0", NULL, "Dinamic Software", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_ADV, 0,
	SpectrumGetZipName, SpecMegacorp1RomInfo, SpecMegacorp1RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Mega-Corp - Parte 2 (Spanish) (48K)

static struct BurnRomInfo SpecMegacorp2RomDesc[] = {
	{ "Mega-Corp - Parte 2 ES (1987)(Dinamic).tap", 49246, 0x0d83942c, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecMegacorp2, SpecMegacorp2, Spectrum)
STD_ROM_FN(SpecMegacorp2)

struct BurnDriver BurnSpecMegacorp2 = {
	"spec_megacorp2", "spec_megacorp1", "spec_spectrum", NULL, "1987",
	"Mega-Corp - Parte 2 (Spanish) (48K)\0", "Password: REBECA", "Dinamic Software", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE, 1, HARDWARE_SPECTRUM, GBF_ADV, 0,
	SpectrumGetZipName, SpecMegacorp2RomInfo, SpecMegacorp2RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Mermaid Madness (48K)

static struct BurnRomInfo SpecMermadnesRomDesc[] = {
	{ "Mermaid Madness 48K (1986)(Electric Dreams).tap", 47263, 0x3b981595, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecMermadnes, SpecMermadnes, Spectrum)
STD_ROM_FN(SpecMermadnes)

struct BurnDriver BurnSpecMermadnes = {
	"spec_mermadnes", NULL, "spec_spectrum", NULL, "1986",
	"Mermaid Madness (48K)\0", NULL, "Electric Dreams", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_MAZE | GBF_ADV, 0,
	SpectrumGetZipName, SpecMermadnesRomInfo, SpecMermadnesRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Metabolis (48K)

static struct BurnRomInfo SpecMetabolisRomDesc[] = {
	{ "Metabolis 48K (1985)(Gremlin Graphics).tap", 56590, 0xd65e3628, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecMetabolis, SpecMetabolis, Spectrum)
STD_ROM_FN(SpecMetabolis)

struct BurnDriver BurnSpecMetabolis = {
	"spec_metabolis", NULL, "spec_spectrum", NULL, "1985",
	"Metabolis (48K)\0", NULL, "Gremlin Graphics", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_MAZE | GBF_ACTION, 0,
	SpectrumGetZipName, SpecMetabolisRomInfo, SpecMetabolisRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Meteor Storm (16K)

static struct BurnRomInfo SpecMeteorstormRomDesc[] = {
	{ "Meteor Storm 16K (1982)(Quicksilva).tap", 7303, 0x7ba250bc, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecMeteorstorm, SpecMeteorstorm, Spectrum)
STD_ROM_FN(SpecMeteorstorm)

struct BurnDriver BurnSpecMeteorstorm = {
	"spec_meteorstorm", NULL, "spec_spectrum", NULL, "1982",
	"Meteor Storm (16K)\0", "Press 'S' to start a game session", "Quicksilva", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_SHOOT, 0,
	SpectrumGetZipName, SpecMeteorstormRomInfo, SpecMeteorstormRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecIntf2DIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Metro-Cross (48K)

static struct BurnRomInfo SpecMetrocrossRomDesc[] = {
	{ "Metro-Cross 48K (1987)(U.S. Gold).tap", 44446, 0x6e1d27d3, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecMetrocross, SpecMetrocross, Spectrum)
STD_ROM_FN(SpecMetrocross)

struct BurnDriver BurnSpecMetrocross = {
	"spec_metrocross", NULL, "spec_spectrum", NULL, "1987",
	"Metro-Cross (48K)\0", NULL, "U.S. Gold", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 2, HARDWARE_SPECTRUM, GBF_ACTION, 0,
	SpectrumGetZipName, SpecMetrocrossRomInfo, SpecMetrocrossRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Miami Dice (48K)

static struct BurnRomInfo SpecMiamidiceRomDesc[] = {
	{ "Miami Dice 48K (1986)(Bug-Byte).tap", 49463, 0xfee92bf2, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecMiamidice, SpecMiamidice, Spectrum)
STD_ROM_FN(SpecMiamidice)

struct BurnDriver BurnSpecMiamidice = {
	"spec_miamidice", NULL, "spec_spectrum", NULL, "1986",
	"Miami Dice (48K)\0", "Betting keys - P1: 1, P2: Z, P3: M, P4: 0", "Bug-Byte Software", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 4, HARDWARE_SPECTRUM, GBF_CASINO, 0,
	SpectrumGetZipName, SpecMiamidiceRomInfo, SpecMiamidiceRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Mickey Mouse - The Computer Game (48K)

static struct BurnRomInfo SpecMickeymouseRomDesc[] = {
	{ "Mickey Mouse - The Computer Game 48K (1988)(Gremlin Graphics).z80", 43296, 0xaa993b54, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecMickeymouse, SpecMickeymouse, Spectrum)
STD_ROM_FN(SpecMickeymouse)

struct BurnDriver BurnSpecMickeymouse = {
	"spec_mickeymouse", NULL, "spec_spectrum", NULL, "1988",
	"Mickey Mouse - The Computer Game (48K)\0", "SPACE: change weapon", "Gremlin Graphics", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecMickeymouseRomInfo, SpecMickeymouseRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Micro Mouse Goes De-bugging (128K)

static struct BurnRomInfo SpecMicromouseRomDesc[] = {
	{ "Micro Mouse Goes De-bugging 128K (1989)(Mastertronic Plus).tap", 47636, 0x6a8a8635, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecMicromouse, SpecMicromouse, Spec128)
STD_ROM_FN(SpecMicromouse)

struct BurnDriver BurnSpecMicromouse = {
	"spec_micromouse", NULL, "spec_spec128", NULL, "1989",
	"Micro Mouse Goes De-bugging (128K)\0", NULL, "Mastertronic Plus", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_ACTION, 0,
	SpectrumGetZipName, SpecMicromouseRomInfo, SpecMicromouseRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Micronaut One (48K)

static struct BurnRomInfo SpecMicronautRomDesc[] = {
	{ "Micronaut One 48K (1987)(Nexus Productions).tap", 48156, 0xe8317652, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecMicronaut, SpecMicronaut, Spectrum)
STD_ROM_FN(SpecMicronaut)

struct BurnDriver BurnSpecMicronaut = {
	"spec_micronaut", NULL, "spec_spectrum", NULL, "1987",
	"Micronaut One (48K)\0", "S, Z, O, P and SPACE to set options in main menu", "Nexus Productions", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_SIM, 0,
	SpectrumGetZipName, SpecMicronautRomInfo, SpecMicronautRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// MicroProse Soccer - Six-a-Side Challenge 128K

static struct BurnRomInfo SpecMicrsocc6sdRomDesc[] = {
	{ "MicroProse Soccer - 6aside Challenge 128K (1989)(MicroProse).tap", 98198, 0x3c0a7349, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecMicrsocc6sd, SpecMicrsocc6sd, Spec128)
STD_ROM_FN(SpecMicrsocc6sd)

struct BurnDriver BurnSpecMicrsocc6sd = {
	"spec_micrsocc6sd", "spec_micrsoccint", "spec_spec128", NULL, "1989",
	"MicroProse Soccer - Six-a-Side Challenge (128K)\0", NULL, "MicroProse", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE, 2, HARDWARE_SPECTRUM, GBF_SPORTSFOOTBALL, 0,
	SpectrumGetZipName, SpecMicrsocc6sdRomInfo, SpecMicrsocc6sdRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// MicroProse Soccer - International Challenge (128K)

static struct BurnRomInfo SpecMicrsoccintRomDesc[] = {
	{ "MicroProse Soccer - Int. Challenge 128K (1989)(MicroProse).tap", 98198, 0x2d03b320, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecMicrsoccint, SpecMicrsoccint, Spec128)
STD_ROM_FN(SpecMicrsoccint)

struct BurnDriver BurnSpecMicrsoccint = {
	"spec_micrsoccint", NULL, "spec_spec128", NULL, "1989",
	"MicroProse Soccer - International Challenge (128K)\0", NULL, "MicroProse", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 2, HARDWARE_SPECTRUM, GBF_SPORTSFOOTBALL, 0,
	SpectrumGetZipName, SpecMicrsoccintRomInfo, SpecMicrsoccintRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// MicroProse Soccer (48K)

static struct BurnRomInfo SpecMicrsocc_48RomDesc[] = {
	{ "MicroProse Soccer 48K (1989)(MicroProse).tap", 36576, 0x68a260d1, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecMicrsocc_48, SpecMicrsocc_48, Spectrum)
STD_ROM_FN(SpecMicrsocc_48)

struct BurnDriver BurnSpecMicrsocc_48 = {
	"spec_micrsocc_48", "spec_micrsoccint", "spec_spectrum", NULL, "1989",
	"MicroProse Soccer (48K)\0", NULL, "MicroProse", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE, 2, HARDWARE_SPECTRUM, GBF_SPORTSFOOTBALL, 0,
	SpectrumGetZipName, SpecMicrsocc_48RomInfo, SpecMicrsocc_48RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Mindtrap (48K)

static struct BurnRomInfo SpecMindtrapRomDesc[] = {
	{ "Mindtrap 48K (1989)(Mastertronic).tap", 25758, 0x65e947b5, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecMindtrap, SpecMindtrap, Spectrum)
STD_ROM_FN(SpecMindtrap)

struct BurnDriver BurnSpecMindtrap = {
	"spec_mindtrap", NULL, "spec_spectrum", NULL, "1989",
	"Mindtrap (48K)\0", "Set '5. Sinclair II (R)' to use controller", "Mastertronic", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_PUZZLE, 0,
	SpectrumGetZipName, SpecMindtrapRomInfo, SpecMindtrapRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecIntf2DIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Mission Impossible (16K)

static struct BurnRomInfo SpecMissimp16kRomDesc[] = {
	{ "Mission Impossible 16K (1983)(Silversoft).tap", 10833, 0x420cc36f, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecMissimp16k, SpecMissimp16k, Spectrum)
STD_ROM_FN(SpecMissimp16k)

struct BurnDriver BurnSpecMissimp16k = {
	"spec_missimp16k", NULL, "spec_spectrum", NULL, "1983",
	"Mission Impossible (16K)\0", NULL, "Silversoft", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_ACTION, 0,
	SpectrumGetZipName, SpecMissimp16kRomInfo, SpecMissimp16kRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Mission Omega (48K)

static struct BurnRomInfo SpecMissomegaRomDesc[] = {
	{ "Mission Omega 48K (1983)(Pulsonic).tap", 34697, 0x7f5c1203, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecMissomega, SpecMissomega, Spectrum)
STD_ROM_FN(SpecMissomega)

struct BurnDriver BurnSpecMissomega = {
	"spec_missomega", NULL, "spec_spectrum", NULL, "1983",
	"Mission Omega (48K)\0", NULL, "Pulsonic", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 2, HARDWARE_SPECTRUM, GBF_ACTION, 0,
	SpectrumGetZipName, SpecMissomegaRomInfo, SpecMissomegaRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Monty Python's Flying Circus (128K)

static struct BurnRomInfo SpecMpflycircusRomDesc[] = {
	{ "Monty Python's Flying Circus 128K (1990)(Virgin Games).tap", 136802, 0x3a99fb2d, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecMpflycircus, SpecMpflycircus, Spec128)
STD_ROM_FN(SpecMpflycircus)

struct BurnDriver BurnSpecMpflycircus = {
	"spec_mpflycircus", NULL, "spec_spec128", NULL, "1990",
	"Monty Python's Flying Circus (128K)\0", NULL, "Virgin Games", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_ACTION | GBF_ADV, 0,
	SpectrumGetZipName, SpecMpflycircusRomInfo, SpecMpflycircusRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Moon Alert! (48K)

static struct BurnRomInfo SpecMoonalertRomDesc[] = {
	{ "Moon Alert! 48K (1984)(Ocean).tap", 47090, 0x01a888bf, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecMoonalert, SpecMoonalert, Spectrum)
STD_ROM_FN(SpecMoonalert)

struct BurnDriver BurnSpecMoonalert = {
	"spec_moonalert", NULL, "spec_spectrum", NULL, "1984",
	"Moon Alert! (48K)\0", NULL, "Ocean", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 2, HARDWARE_SPECTRUM, GBF_ACTION, 0,
	SpectrumGetZipName, SpecMoonalertRomInfo, SpecMoonalertRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecIntf2DIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Moonsweeper (48K)

static struct BurnRomInfo SpecMoonsweeperRomDesc[] = {
	{ "Moonsweeper 48K (1983)(CheetahSoft).z80", 26570, 0x71b11ba8, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecMoonsweeper, SpecMoonsweeper, Spectrum)
STD_ROM_FN(SpecMoonsweeper)

struct BurnDriver BurnSpecMoonsweeper = {
	"spec_moonsweeper", NULL, "spec_spectrum", NULL, "1983",
	"Moonsweeper (48K)\0", NULL, "CheetahSoft", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_SHOOT, 0,
	SpectrumGetZipName, SpecMoonsweeperRomInfo, SpecMoonsweeperRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Mot - Intro (Spanish) (128K)

static struct BurnRomInfo Specmot128introRomDesc[] = {
	{ "Mot - Intro ES 128K2a (1989)(Opera Soft).tap", 49127, 0x76c4cac9, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specmot128intro, Specmot128intro, Spec1282a)
STD_ROM_FN(Specmot128intro)

struct BurnDriver BurnSpecmot128intro = {
	"spec_mot128intro", NULL, "spec_spec1282a", NULL, "1989",
	"Mot - Intro (Spanish) (128K)\0", NULL, "Opera Soft", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_ACTION, 0,
	SpectrumGetZipName, Specmot128introRomInfo, Specmot128introRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecIntf2DIPInfo,
	Spec128KPlus2Init, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Mot - Part 1 (Spanish) (128K)

static struct BurnRomInfo Specmot128aRomDesc[] = {
	{ "Mot - Part 1 ES 128K2a (1989)(Opera Soft).tap", 49127, 0x0713edec, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specmot128a, Specmot128a, Spec1282a)
STD_ROM_FN(Specmot128a)

struct BurnDriver BurnSpecmot128a = {
	"spec_mot128a", "spec_mot128intro", "spec_spec1282a", NULL, "1989",
	"Mot - Part 1 (Spanish) (128K)\0", NULL, "Opera Soft", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE, 1, HARDWARE_SPECTRUM, GBF_ACTION, 0,
	SpectrumGetZipName, Specmot128aRomInfo, Specmot128aRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecIntf2DIPInfo,
	Spec128KPlus2Init, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Mot - Part 2 (Spanish) (128K)

static struct BurnRomInfo Specmot128bRomDesc[] = {
	{ "Mot - Part 2 ES 128K2a (1989)(Opera Soft).tap", 49127, 0xff0cb875, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specmot128b, Specmot128b, Spec1282a)
STD_ROM_FN(Specmot128b)

struct BurnDriver BurnSpecmot128b = {
	"spec_mot128b", "spec_mot128intro", "spec_spec1282a", NULL, "1989",
	"Mot - Part 2 (Spanish) (128K)\0", NULL, "Opera Soft", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE, 1, HARDWARE_SPECTRUM, GBF_ACTION, 0,
	SpectrumGetZipName, Specmot128bRomInfo, Specmot128bRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecIntf2DIPInfo,
	Spec128KPlus2Init, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Mot - Part 3 (Spanish) (128K)

static struct BurnRomInfo Specmot128cRomDesc[] = {
	{ "Mot - Part 3 ES 128K2a (1989)(Opera Soft).tap", 49127, 0x43ed24bc, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specmot128c, Specmot128c, Spec1282a)
STD_ROM_FN(Specmot128c)

struct BurnDriver BurnSpecmot128c = {
	"spec_mot128c", "spec_mot128intro", "spec_spec1282a", NULL, "1989",
	"Mot - Part 3 (Spanish) (128K)\0", NULL, "Opera Soft", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE, 1, HARDWARE_SPECTRUM, GBF_ACTION, 0,
	SpectrumGetZipName, Specmot128cRomInfo, Specmot128cRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecIntf2DIPInfo,
	Spec128KPlus2Init, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Motorbike Madness (48K)

static struct BurnRomInfo SpecMotomadnessRomDesc[] = {
	{ "Motorbike Madness 48K (1988)(Mastertronic).tap", 68303, 0x8346f6c5, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecMotomadness, SpecMotomadness, Spectrum)
STD_ROM_FN(SpecMotomadness)

struct BurnDriver BurnSpecMotomadness = {
	"spec_motomadness", NULL, "spec_spectrum", NULL, "1988",
	"Motorbike Madness (48K)\0", NULL, "Mastertronic", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_ACTION, 0,
	SpectrumGetZipName, SpecMotomadnessRomInfo, SpecMotomadnessRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// M.O.V.I.E. (48K)

static struct BurnRomInfo SpecMovieRomDesc[] = {
	{ "MOVIE 48K (1986)(Imagine).tap", 48594, 0x4f413f00, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecMovie, SpecMovie, Spectrum)
STD_ROM_FN(SpecMovie)

struct BurnDriver BurnSpecMovie = {
	"spec_movie", NULL, "spec_spectrum", NULL, "1986",
	"M.O.V.I.E. (48K)\0", "Press 'O' to change options", "Imagine Software", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_ADV, 0,
	SpectrumGetZipName, SpecMovieRomInfo, SpecMovieRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Mr. Heli (48K)

static struct BurnRomInfo SpecMrheliRomDesc[] = {
	{ "Mr Heli 48K (1989)(Firebird).tap", 126724, 0xbf869699, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecMrheli, SpecMrheli, Spectrum)
STD_ROM_FN(SpecMrheli)

struct BurnDriver BurnSpecMrheli = {
	"spec_mrheli", NULL, "spec_spectrum", NULL, "1989",
	"Mr. Heli (48K)\0", NULL, "Firebird Software", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_SHOOT, 0,
	SpectrumGetZipName, SpecMrheliRomInfo, SpecMrheliRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Mr. Weems and the She Vampires (48K)

static struct BurnRomInfo SpecMrweemsRomDesc[] = {
	{ "Mr. Weems and the She Vampires 48K (1987)(Piranha).tap", 54672, 0x4b04200a, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecMrweems, SpecMrweems, Spectrum)
STD_ROM_FN(SpecMrweems)

struct BurnDriver BurnSpecMrweems = {
	"spec_mrweems", NULL, "spec_spectrum", NULL, "1987",
	"Mr. Weems and the She Vampires (48K)\0", NULL, "Piranha", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_RUNGUN | GBF_MAZE, 0,
	SpectrumGetZipName, SpecMrweemsRomInfo, SpecMrweemsRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Muncher! (16K)

static struct BurnRomInfo SpecMuncher16RomDesc[] = {
	{ "Muncher! 16K (1982)(Silversoft).tap", 8192, 0x3e72dda7, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecMuncher16, SpecMuncher16, Spectrum)
STD_ROM_FN(SpecMuncher16)

struct BurnDriver BurnSpecMuncher16 = {
	"spec_muncher16", NULL, "spec_spectrum", NULL, "1982",
	"Muncher! (16K)\0", "S: start - Z: left - X: right - K: up - M: down", "Silversoft", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_ACTION | GBF_MAZE, 0,
	SpectrumGetZipName, SpecMuncher16RomInfo, SpecMuncher16RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Muncher Eats Chewits, The (128K)

static struct BurnRomInfo SpecMuncherRomDesc[] = {
	{ "Muncher Eats Chewits, The 128K (1988)(Gremlin Graphics).tap", 120973, 0x16db8cd7, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecMuncher, SpecMuncher, Spec128)
STD_ROM_FN(SpecMuncher)

struct BurnDriver BurnSpecMuncher = {
	"spec_muncher", NULL, "spec_spec128", NULL, "1988",
	"Muncher Eats Chewits, The (128K)\0", "Set 'Sinclair Joystick' to use controller", "Gremlin Graphics", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_ACTION, 0,
	SpectrumGetZipName, SpecMuncherRomInfo, SpecMuncherRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Mutant Fortress (128K)

static struct BurnRomInfo SpecMutfortressRomDesc[] = {
	{ "Mutant Fortress 128K (1989)(Players Premier).tap", 77979, 0xf82b3bcc, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecMutfortress, SpecMutfortress, Spec128)
STD_ROM_FN(SpecMutfortress)

struct BurnDriver BurnSpecMutfortress = {
	"spec_mutfortress", NULL, "spec_spec128", NULL, "1989",
	"Mutant Fortress (128K)\0", NULL, "Players Premier", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_RUNGUN | GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecMutfortressRomInfo, SpecMutfortressRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Mystery of Arkham Manor, The - Part 1 (48K)

static struct BurnRomInfo SpecMysteryarkham1RomDesc[] = {
	{ "Mystery of Arkham Manor, The - Part1 48K (1987)(Melbourne House).tap", 56277, 0x6bf7a6ea, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecMysteryarkham1, SpecMysteryarkham1, Spectrum)
STD_ROM_FN(SpecMysteryarkham1)

struct BurnDriver BurnSpecMysteryarkham1 = {
	"spec_mysteryarkham1", NULL, "spec_spectrum", NULL, "1987",
	"Mystery of Arkham Manor, The - Part 1 (48K)\0", "In-game menu - 6: down - 7: up - Space: select", "Melbourne House", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_ADV, 0,
	SpectrumGetZipName, SpecMysteryarkham1RomInfo, SpecMysteryarkham1RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Mystery of Arkham Manor, The - Part 2 (48K)

static struct BurnRomInfo SpecMysteryarkham2RomDesc[] = {
	{ "Mystery of Arkham Manor, The - Part2 48K (1987)(Melbourne House).tap", 49343, 0x85b9867c, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecMysteryarkham2, SpecMysteryarkham2, Spectrum)
STD_ROM_FN(SpecMysteryarkham2)

struct BurnDriver BurnSpecMysteryarkham2 = {
	"spec_mysteryarkham2", "spec_mysteryarkham1", "spec_spectrum", NULL, "1987",
	"Mystery of Arkham Manor, The - Part 2 (48K)\0", "In-game menu - 6: down - 7: up - Space: select", "Melbourne House", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE, 1, HARDWARE_SPECTRUM, GBF_ADV, 0,
	SpectrumGetZipName, SpecMysteryarkham2RomInfo, SpecMysteryarkham2RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Mystery of the Nile, The (English) (48K)

static struct BurnRomInfo SpecMysterynileRomDesc[] = {
	{ "Mystery of the Nile, The 48K (1987)(Firebird).tap", 50370, 0x3dad4b2a, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecMysterynile, SpecMysterynile, Spectrum)
STD_ROM_FN(SpecMysterynile)

struct BurnDriver BurnSpecMysterynile = {
	"spec_mysterynile", NULL, "spec_spectrum", NULL, "1987",
	"Mystery of the Nile, The (English) (48K)\0", NULL, "Firebird Software", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecMysterynileRomInfo, SpecMysterynileRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Misterio del Nilo, El (Spanish) (48K)

static struct BurnRomInfo SpecMisterioniloRomDesc[] = {
	{ "Misterio del Nilo, El 48K (1987)(Zigurat).tap", 50412, 0xfedd2fd4, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecMisterionilo, SpecMisterionilo, Spectrum)
STD_ROM_FN(SpecMisterionilo)

struct BurnDriver BurnSpecMisterionilo = {
	"spec_misterionilo", "spec_mysterynile", "spec_spectrum", NULL, "1987",
	"Misterio del Nilo, El (Spanish) (48K)\0", NULL, "Zigurat Software", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecMisterioniloRomInfo, SpecMisterioniloRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Mystical (128K)

static struct BurnRomInfo SpecMysticalRomDesc[] = {
	{ "Mystical 128K (1991)(Infogrames).tap", 260015, 0xf7df7363, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecMystical, SpecMystical, Spec128)
STD_ROM_FN(SpecMystical)

struct BurnDriver BurnSpecMystical = {
	"spec_mystical", NULL, "spec_spec128", NULL, "1991",
	"Mystical (128K)\0", NULL, "Infogrames", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 2, HARDWARE_SPECTRUM, GBF_VERSHOOT, 0,
	SpectrumGetZipName, SpecMysticalRomInfo, SpecMysticalRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Neighbours (48K)

static struct BurnRomInfo SpecNeighboursRomDesc[] = {
	{ "Neighbours 48K (1992)(Impulze).tap", 52388, 0x1a852016, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecNeighbours, SpecNeighbours, Spectrum)
STD_ROM_FN(SpecNeighbours)

struct BurnDriver BurnSpecNeighbours = {
	"spec_neighbours", NULL, "spec_spectrum", NULL, "1992",
	"Neighbours (48K)\0", "Set '3 SINCLAIR' to use controller", "Impulze", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_ACTION, 0,
	SpectrumGetZipName, SpecNeighboursRomInfo, SpecNeighboursRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecIntf2DIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Nemesis the Warlock (128K)

static struct BurnRomInfo SpecNwarlockRomDesc[] = {
	{ "Nemesis the Warlock 128K (1987)(Martech Games).tap", 70991, 0xe8b462f8, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecNwarlock, SpecNwarlock, Spec128)
STD_ROM_FN(SpecNwarlock)

struct BurnDriver BurnSpecNwarlock = {
	"spec_nwarlock", NULL, "spec_spec128", NULL, "1987",
	"Nemesis the Warlock (128K)\0", "at intro press Fire and Space to view 'credits+music' screen", "Martech Games", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_RUNGUN, 0,
	SpectrumGetZipName, SpecNwarlockRomInfo, SpecNwarlockRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// NeverEnding Story, The (128K)

static struct BurnRomInfo SpecNestoryRomDesc[] = {
	{ "NeverEnding Story, The 128K (1985)(Ocean).tap", 86648, 0xccc97acd, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecNestory, SpecNestory, Spec128)
STD_ROM_FN(SpecNestory)

struct BurnDriver BurnSpecNestory = {
	"spec_nestory", NULL, "spec_spec128", NULL, "1985",
	"NeverEnding Story, The (128K)\0", NULL, "Ocean", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_ADV, 0,
	SpectrumGetZipName, SpecNestoryRomInfo, SpecNestoryRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// NeverEnding Story II, The (48K)

static struct BurnRomInfo SpecNestory2RomDesc[] = {
	{ "Neverending Story II, The 48K (1990)(Linel).tap", 187826, 0x4d4ca801, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecNestory2, SpecNestory2, Spectrum)
STD_ROM_FN(SpecNestory2)

struct BurnDriver BurnSpecNestory2 = {
	"spec_nestory2", NULL, "spec_spectrum", NULL, "1990",
	"NeverEnding Story II, The (48K)\0", NULL, "Linel", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecNestory2RomInfo, SpecNestory2RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Nifty Lifty (48K)

static struct BurnRomInfo SpecNiftyliftyRomDesc[] = {
	{ "Nifty Lifty 48K (1984)(Visions Software Factory).tap", 21604, 0xe9333882, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecNiftylifty, SpecNiftylifty, Spectrum)
STD_ROM_FN(SpecNiftylifty)

struct BurnDriver BurnSpecNiftylifty = {
	"spec_niftylifty", NULL, "spec_spectrum", NULL, "1984",
	"Nifty Lifty (48K)\0", NULL, "Visions Software Factory", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_ACTION, 0,
	SpectrumGetZipName, SpecNiftyliftyRomInfo, SpecNiftyliftyRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Nigel Mansell's World Championship (128K)

static struct BurnRomInfo SpecMansellwcRomDesc[] = {
	{ "Nigel Mansell's World Championship 128K (1993)(Gremlin Graphics).tap", 224111, 0x36ee68e4, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecMansellwc, SpecMansellwc, Spec128)
STD_ROM_FN(SpecMansellwc)

struct BurnDriver BurnSpecMansellwc = {
	"spec_mansellwc", NULL, "spec_spec128", NULL, "1993",
	"Nigel Mansell's World Championship (128K)\0", NULL, "Gremlin Graphics", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_RACING, 0,
	SpectrumGetZipName, SpecMansellwcRomInfo, SpecMansellwcRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecIntf2DIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Nightmare Rally (48K)

static struct BurnRomInfo SpecNightrallyRomDesc[] = {
	{ "Nightmare Rally 48K (1986)(Ocean).tap", 49795, 0x69d1f958, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecNightrally, SpecNightrally, Spectrum)
STD_ROM_FN(SpecNightrally)

struct BurnDriver BurnSpecNightrally = {
	"spec_nightrally", NULL, "spec_spectrum", NULL, "1986",
	"Nightmare Rally (48K)\0", NULL, "Ocean", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 2, HARDWARE_SPECTRUM, GBF_RACING, 0,
	SpectrumGetZipName, SpecNightrallyRomInfo, SpecNightrallyRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Nightshade (48K)

static struct BurnRomInfo SpecNightshadeRomDesc[] = {
	{ "Nightshade 48K (1985)(Ultimate Play The Game).tap", 42456, 0xb421d1a8, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecNightshade, SpecNightshade, Spectrum)
STD_ROM_FN(SpecNightshade)

struct BurnDriver BurnSpecNightshade = {
	"spec_nightshade", NULL, "spec_spectrum", NULL, "1985",
	"Nightshade (48K)\0", NULL, "Ultimate Play The Game", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_ACTION | GBF_ADV, 0,
	SpectrumGetZipName, SpecNightshadeRomInfo, SpecNightshadeRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Ninja Commando (48K)

static struct BurnRomInfo SpecNinjacomRomDesc[] = {
	{ "Ninja Commando 48K (1989)(Zeppelin Games).tap", 40413, 0x60e678a2, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecNinjacom, SpecNinjacom, Spectrum)
STD_ROM_FN(SpecNinjacom)

struct BurnDriver BurnSpecNinjacom = {
	"spec_ninjacom", NULL, "spec_spectrum", NULL, "1989",
	"Ninja Commando (48K)\0", NULL, "Zeppelin Games", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecNinjacomRomInfo, SpecNinjacomRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecIntf2DIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Ninja Scooter Simulator (48K)

static struct BurnRomInfo SpecNinjascooterRomDesc[] = {
	{ "Ninja Scooter Simulator 48K (1987)(Sysoft).tap", 46884, 0xf9173adb, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecNinjascooter, SpecNinjascooter, Spectrum)
STD_ROM_FN(SpecNinjascooter)

struct BurnDriver BurnSpecNinjascooter = {
	"spec_ninjascooter", NULL, "spec_spectrum", NULL, "1987",
	"Ninja Scooter Simulator (48K)\0", NULL, "Sysoft", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 2, HARDWARE_SPECTRUM, GBF_ACTION, 0,
	SpectrumGetZipName, SpecNinjascooterRomInfo, SpecNinjascooterRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Ninja Spirit (128K)

static struct BurnRomInfo SpecNinjaspiritRomDesc[] = {
	{ "Ninja Spirit 128K (1990)(Activision).tap", 113048, 0x19b240f6, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecNinjaspirit, SpecNinjaspirit, Spec128)
STD_ROM_FN(SpecNinjaspirit)

struct BurnDriver BurnSpecNinjaspirit = {
	"spec_ninjaspirit", NULL, "spec_spec128", NULL, "1990",
	"Ninja Spirit (128K)\0", NULL, "Activision", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_SCRFIGHT, 0,
	SpectrumGetZipName, SpecNinjaspiritRomInfo, SpecNinjaspiritRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Ninja Warriors, The (128K)

static struct BurnRomInfo SpecNinjawarrRomDesc[] = {
	{ "Ninja Warriors, The 128K (1989)(Virgin Games).tap", 112187, 0xe870c241, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecNinjawarr, SpecNinjawarr, Spec128)
STD_ROM_FN(SpecNinjawarr)

struct BurnDriver BurnSpecNinjawarr = {
	"spec_ninjawarr", NULL, "spec_spec128", NULL, "1989",
	"Ninja Warriors, The (128K)\0", NULL, "Virgin Games", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 2, HARDWARE_SPECTRUM, GBF_SCRFIGHT, 0,
	SpectrumGetZipName, SpecNinjawarrRomInfo, SpecNinjawarrRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Nonamed (48K)

static struct BurnRomInfo SpecNonamedRomDesc[] = {
	{ "Nonamed 48K (1987)(Dinamic Software).tap", 44233, 0xdc3f2124, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecNonamed, SpecNonamed, Spectrum)
STD_ROM_FN(SpecNonamed)

struct BurnDriver BurnSpecNonamed = {
	"spec_nonamed", NULL, "spec_spectrum", NULL, "1987",
	"Nonamed (48K)\0", NULL, "Dinamic Software", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecNonamedRomInfo, SpecNonamedRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Nonterraqueous (48K)

static struct BurnRomInfo SpecNaqueousRomDesc[] = {
	{ "Nonterraqueous 48K (1985)(Mastertronic).tap", 48487, 0x658f720a, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecNaqueous, SpecNaqueous, Spectrum)
STD_ROM_FN(SpecNaqueous)

struct BurnDriver BurnSpecNaqueous = {
	"spec_naqueous", NULL, "spec_spectrum", NULL, "1985",
	"Nonterraqueous (48K)\0", NULL, "Mastertronic", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_ACTION, 0,
	SpectrumGetZipName, SpecNaqueousRomInfo, SpecNaqueousRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Nuclear Bowls (48K)

static struct BurnRomInfo SpecNubowlsRomDesc[] = {
	{ "Nuclear Bowls 48K (1986)(Zigurat).tap", 49570, 0x8a383a82, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecNubowls, SpecNubowls, Spectrum)
STD_ROM_FN(SpecNubowls)

struct BurnDriver BurnSpecNubowls = {
	"spec_nubowls", NULL, "spec_spectrum", NULL, "1986",
	"Nuclear Bowls (48K)\0", NULL, "Zigurat Software", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecNubowlsRomInfo, SpecNubowlsRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Official Father Christmas Game (48K)

static struct BurnRomInfo SpecOfchristmasRomDesc[] = {
	{ "Official Father Christmas Game 48K (1989)(Alternative Software).tap", 43745, 0x4a874155, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecOfchristmas, SpecOfchristmas, Spectrum)
STD_ROM_FN(SpecOfchristmas)

struct BurnDriver BurnSpecOfchristmas = {
	"spec_ofchristmas", NULL, "spec_spectrum", NULL, "1989",
	"Official Father Christmas Game (48K)\0", NULL, "Alternative Software", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_ACTION, 0,
	SpectrumGetZipName, SpecOfchristmasRomInfo, SpecOfchristmasRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Oh Mummy (16K)

static struct BurnRomInfo SpecOhmummyRomDesc[] = {
	{ "Oh Mummy 16K (1984)(Gem Software).tap", 11384, 0x575531e4, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecOhmummy, SpecOhmummy, Spectrum)
STD_ROM_FN(SpecOhmummy)

struct BurnDriver BurnSpecOhmummy = {
	"spec_ohmummy", NULL, "spec_spectrum", NULL, "1984-86",
	"Oh Mummy (16K)\0", "Default keys: A, Z, K, L", "Gem Software", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_MAZE, 0,
	SpectrumGetZipName, SpecOhmummyRomInfo, SpecOhmummyRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Oriental Games (128K)

static struct BurnRomInfo SpecOrgamesRomDesc[] = {
	{ "Oriental Games 128K (1990)(Micro Style).tap", 138887, 0x4e03b386, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecOrgames, SpecOrgames, Spec128)
STD_ROM_FN(SpecOrgames)

struct BurnDriver BurnSpecOrgames = {
	"spec_orgames", NULL, "spec_spec128", NULL, "1990",
	"Oriental Games (128K)\0", NULL, "Micro Style", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 2, HARDWARE_SPECTRUM, GBF_VSFIGHT, 0,
	SpectrumGetZipName, SpecOrgamesRomInfo, SpecOrgamesRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Outrun (128K)

static struct BurnRomInfo SpecOutrunRomDesc[] = {
	{ "Outrun 128K (1987)(U.S. Gold).tap", 0x1f5ca, 0x1c4dde6b, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecOutrun, SpecOutrun, Spec128)
STD_ROM_FN(SpecOutrun)

struct BurnDriver BurnSpecOutrun = {
	"spec_outrun", NULL, "spec_spec128", NULL, "1987",
	"Outrun (128K)\0", NULL, "U.S. Gold", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_RACING, 0,
	SpectrumGetZipName, SpecOutrunRomInfo, SpecOutrunRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Outrun Europa (128K)

static struct BurnRomInfo SpecOutruneuRomDesc[] = {
	{ "Outrun Europa 128K (1991)(U.S. Gold).tap", 0x495d0, 0x3240887e, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecOutruneu, SpecOutruneu, Spec128)
STD_ROM_FN(SpecOutruneu)

struct BurnDriver BurnSpecOutruneu = {
	"spec_outruneu", NULL, "spec_spec128", NULL, "1991",
	"Outrun Europa (128K)\0", NULL, "U.S. Gold", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_RACING, 0,
	SpectrumGetZipName, SpecOutruneuRomInfo, SpecOutruneuRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Overlander (128K)

static struct BurnRomInfo SpecOverlanderRomDesc[] = {
	{ "Overlander 128K (1988)(Elite Systems).tap", 58270, 0xf0e79999, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecOverlander, SpecOverlander, Spec128)
STD_ROM_FN(SpecOverlander)

struct BurnDriver BurnSpecOverlander = {
	"spec_overlander", NULL, "spec_spec128", NULL, "1988",
	"Overlander (128K)\0", NULL, "Elite Systems", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_RACING | GBF_SHOOT, 0,
	SpectrumGetZipName, SpecOverlanderRomInfo, SpecOverlanderRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Pac Hic (48K)

static struct BurnRomInfo SpecPachicRomDesc[] = {
	{ "Pac Hic (1986)(Daniel Isoba).tap", 45730, 0x42bb1258, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecPachic, SpecPachic, Spectrum)
STD_ROM_FN(SpecPachic)

struct BurnDriver BurnSpecPachic = {
	"spec_pachic", NULL, "spec_spectrum", NULL, "1986",
	"Pac Hic (48K)\0", NULL, "Daniel Isoba", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_MAZE | GBF_ACTION, 0,
	SpectrumGetZipName, SpecPachicRomInfo, SpecPachicRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Pakacuda (16K)

static struct BurnRomInfo SpecPakacudaRomDesc[] = {
	{ "Pakacuda 16K (1983)(Rabbit).tap", 15372, 0x789f0e0b, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecPakacuda, SpecPakacuda, Spectrum)
STD_ROM_FN(SpecPakacuda)

struct BurnDriver BurnSpecPakacuda = {
	"spec_pakacuda", NULL, "spec_spectrum", NULL, "1983",
	"Pakacuda (16K)\0", "Select 'N'ormal cursor keys to use controller", "Rabbit Software", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_MAZE | GBF_ACTION, 0,
	SpectrumGetZipName, SpecPakacudaRomInfo, SpecPakacudaRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecCursorKeysDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Paradroids (48K)

static struct BurnRomInfo SpecParadroidsRomDesc[] = {
	{ "Paradroids 48K (1984)(Mikro-Gen).tap", 12325, 0x6c990e7b, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecParadroids, SpecParadroids, Spectrum)
STD_ROM_FN(SpecParadroids)

struct BurnDriver BurnSpecParadroids = {
	"spec_paradroids", NULL, "spec_spectrum", NULL, "1984",
	"Paradroids (48K)\0", NULL, "Mikro-Gen", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_SHOOT, 0,
	SpectrumGetZipName, SpecParadroidsRomInfo, SpecParadroidsRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Pedro (48K)

static struct BurnRomInfo SpecPedroRomDesc[] = {
	{ "Pedro 48K (1984)(Imagine).tap", 27328, 0x4c759deb, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecPedro, SpecPedro, Spectrum)
STD_ROM_FN(SpecPedro)

struct BurnDriver BurnSpecPedro = {
	"spec_pedro", NULL, "spec_spectrum", NULL, "1984",
	"Pedro (48K)\0", NULL, "Imagine Software", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_ACTION, 0,
	SpectrumGetZipName, SpecPedroRomInfo, SpecPedroRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Penetrator (48K)

static struct BurnRomInfo SpecPenetratorRomDesc[] = {
	{ "Penetrator 48K (1983)(Melbourne House).tap", 40029, 0x7efe1706, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecPenetrator, SpecPenetrator, Spectrum)
STD_ROM_FN(SpecPenetrator)

struct BurnDriver BurnSpecPenetrator = {
	"spec_penetrator", NULL, "spec_spectrum", NULL, "1982-83",
	"Penetrator (48K)\0", NULL, "Melbourne House", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 2, HARDWARE_SPECTRUM, GBF_HORSHOOT, 0,
	SpectrumGetZipName, SpecPenetratorRomInfo, SpecPenetratorRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecQAOPSpaceDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Pentagram (48K)

static struct BurnRomInfo SpecPentagramRomDesc[] = {
	{ "Pentagram 48K (1986)(Ultimate).tap", 38524, 0x4d43b15a, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecPentagram, SpecPentagram, Spectrum)
STD_ROM_FN(SpecPentagram)

struct BurnDriver BurnSpecPentagram = {
	"spec_pentagram", NULL, "spec_spectrum", NULL, "1986",
	"Pentagram (48K)\0", NULL, "Ultimate", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_ADV, 0,
	SpectrumGetZipName, SpecPentagramRomInfo, SpecPentagramRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Percy the Potty Pigeon (48K)

static struct BurnRomInfo SpecPottypigeonRomDesc[] = {
	{ "Percy the Potty Pigeon 48K (1984)(Gremlin Graphics).tap", 49061, 0x243fb1ea, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecPottypigeon, SpecPottypigeon, Spectrum)
STD_ROM_FN(SpecPottypigeon)

struct BurnDriver BurnSpecPottypigeon = {
	"spec_pottypigeon", NULL, "spec_spectrum", NULL, "1984",
	"Percy the Potty Pigeon (48K)\0", NULL, "Gremlin Graphics", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_ACTION, 0,
	SpectrumGetZipName, SpecPottypigeonRomInfo, SpecPottypigeonRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Peter Pack Rat (128K)

static struct BurnRomInfo SpecPpratRomDesc[] = {
	{ "Peter Pack Rat 128K (1989)(Silverbird).tap", 53285, 0x5d318fcd, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecPprat, SpecPprat, Spec128)
STD_ROM_FN(SpecPprat)

struct BurnDriver BurnSpecPprat = {
	"spec_pprat", NULL, "spec_spec128", NULL, "1989",
	"Peter Pack Rat (128K)\0", NULL, "Silverbird Software", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 2, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecPpratRomInfo, SpecPpratRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Phantomas (48K)

static struct BurnRomInfo SpecPhantomasRomDesc[] = {
	{ "Phantomas 48K (1986)(Dinamic).tap", 48127, 0x34c8919a, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecPhantomas, SpecPhantomas, Spectrum)
STD_ROM_FN(SpecPhantomas)

struct BurnDriver BurnSpecPhantomas = {
	"spec_phantomas", NULL, "spec_spectrum", NULL, "1986",
	"Phantomas (48K)\0", "Press 6, 7 and ENTER to select control type", "Dinamic Software", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecPhantomasRomInfo, SpecPhantomasRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Phantomas 2 (48K)

static struct BurnRomInfo SpecPhantomas2RomDesc[] = {
	{ "Phantomas 2 48K (1986)(Dinamic).tap", 49300, 0x7e1fecc2, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecPhantomas2, SpecPhantomas2, Spectrum)
STD_ROM_FN(SpecPhantomas2)

struct BurnDriver BurnSpecPhantomas2 = {
	"spec_phantomas2", NULL, "spec_spectrum", NULL, "1986",
	"Phantomas 2 (48K)\0", "Press 6, 7 and ENTER to select control type", "Dinamic Software", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecPhantomas2RomInfo, SpecPhantomas2RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Phantom Club (48K)

static struct BurnRomInfo SpecPhantclubRomDesc[] = {
	{ "Phantom Club 48K (1988)(Ocean).tap", 51960, 0xda40ea75, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecPhantclub, SpecPhantclub, Spectrum)
STD_ROM_FN(SpecPhantclub)

struct BurnDriver BurnSpecPhantclub = {
	"spec_phantclub", NULL, "spec_spectrum", NULL, "1988",
	"Phantom Club (48K)\0", NULL, "Ocean", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_ACTION | GBF_ADV, 0,
	SpectrumGetZipName, SpecPhantclubRomInfo, SpecPhantclubRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Pheenix (16K)

static struct BurnRomInfo SpecPheenixRomDesc[] = {
	{ "Pheenix 16K (1983)(Megadodo).tap", 8551, 0x3c9549da, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecPheenix, SpecPheenix, Spectrum)
STD_ROM_FN(SpecPheenix)

struct BurnDriver BurnSpecPheenix = {
	"spec_pheenix", NULL, "spec_spectrum", NULL, "1983",
	"Pheenix (16K)\0", NULL, "Megadodo Software", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_VERSHOOT, 0,
	SpectrumGetZipName, SpecPheenixRomInfo, SpecPheenixRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Pinball (16K)

static struct BurnRomInfo SpecPinballRomDesc[] = {
	{ "Pinball 16K (1983)(Sagittarian).tap", 16187, 0x26267d3e, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecPinball, SpecPinball, Spectrum)
STD_ROM_FN(SpecPinball)

struct BurnDriver BurnSpecPinball = {
	"spec_pinball", NULL, "spec_spectrum", NULL, "1983",
	"Pinball (16K)\0", "Q: left flipper - P: right flipper - 0: launch the ball", "Sagittarian Software", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_PINBALL, 0,
	SpectrumGetZipName, SpecPinballRomInfo, SpecPinballRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Pink Panther (48K)

static struct BurnRomInfo SpecPinkpantherRomDesc[] = {
	{ "Pink Panther 48K (1988)(Gremlin Graphics).tap", 89454, 0x93b42f80, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecPinkpanther, SpecPinkpanther, Spectrum)
STD_ROM_FN(SpecPinkpanther)

struct BurnDriver BurnSpecPinkpanther = {
	"spec_pinkpanther", NULL, "spec_spectrum", NULL, "1988",
	"Pink Panther (48K)\0", NULL, "Gremlin Graphics", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_ACTION, 0,
	SpectrumGetZipName, SpecPinkpantherRomInfo, SpecPinkpantherRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecIntf2DIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Pitfall II - Lost Caverns (48K)

static struct BurnRomInfo SpecPifalliiRomDesc[] = {
	{ "Pitfall II - Lost Caverns 48K (1984)(Activision).tap", 27294, 0x9acc04e1, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecPifallii, SpecPifallii, Spectrum)
STD_ROM_FN(SpecPifallii)

struct BurnDriver BurnSpecPifallii = {
	"spec_pifallii", NULL, "spec_spectrum", NULL, "1984",
	"Pitfall II - Lost Caverns (48K)\0", "Press 'M' to open controller menu and 'Space' to stop music (during a game session)", "Activision", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecPifalliiRomInfo, SpecPifalliiRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Planet 10 (48K)

static struct BurnRomInfo SpecPlanet10RomDesc[] = {
	{ "Planet 10 48K (1989)(Mastertronic).tap", 48937, 0x4ae2d7a9, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecPlanet10, SpecPlanet10, Spectrum)
STD_ROM_FN(SpecPlanet10)

struct BurnDriver BurnSpecPlanet10 = {
	"spec_planet10", NULL, "spec_spectrum", NULL, "1989",
	"Planet 10 (48K)\0", NULL, "Mastertronic", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_MAZE, 0,
	SpectrumGetZipName, SpecPlanet10RomInfo, SpecPlanet10RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Planetoids (16K)

static struct BurnRomInfo SpecPlanetoidsRomDesc[] = {
	{ "Planetoids 16K (1982)(Sinclair Research).tap", 15635, 0x9c2a2474, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecPlanetoids, SpecPlanetoids, Spectrum)
STD_ROM_FN(SpecPlanetoids)

struct BurnDriver BurnSpecPlanetoids = {
	"spec_planetoids", NULL, "spec_spectrum", NULL, "1982",
	"Planetoids (16K)\0", NULL, "Sinclair Research", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_SHOOT, 0,
	SpectrumGetZipName, SpecPlanetoidsRomInfo, SpecPlanetoidsRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecIntf2SwapDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Pogo (48K)

static struct BurnRomInfo SpecPogoRomDesc[] = {
	{ "Pogo 48K (1984)(Ocean).tap", 42355, 0x858e042e, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecPogo, SpecPogo, Spectrum)
STD_ROM_FN(SpecPogo)

struct BurnDriver BurnSpecPogo = {
	"spec_pogo", NULL, "spec_spectrum", NULL, "1984",
	"Pogo (48K)\0", NULL, "Ocean", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_ACTION, 0,
	SpectrumGetZipName, SpecPogoRomInfo, SpecPogoRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Pole Position (C&VG) (48K)

static struct BurnRomInfo SpecPoleposcvgRomDesc[] = {
	{ "Pole Position (1984)(C&VG).tap", 11325, 0x049b1e06, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecPoleposcvg, SpecPoleposcvg, Spectrum)
STD_ROM_FN(SpecPoleposcvg)

struct BurnDriver BurnSpecPoleposcvg = {
	"spec_poleposcvg", NULL, "spec_spectrum", NULL, "1984",
	"Pole Position (C&VG) (48K)\0", NULL, "Computer & Video Games", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_ACTION, 0,
	SpectrumGetZipName, SpecPoleposcvgRomInfo, SpecPoleposcvgRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecCursorKeysDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Popeye (48K)

static struct BurnRomInfo SpecPopeyeRomDesc[] = {
	{ "Popeye (1985)(DK'Tronics).z80", 30572, 0x8582b9b7, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecPopeye, SpecPopeye, Spectrum)
STD_ROM_FN(SpecPopeye)

struct BurnDriver BurnSpecPopeye = {
	"spec_popeye", NULL, "spec_spectrum", NULL, "1985",
	"Popeye (48K)\0", NULL, "DK'Tronics", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecPopeyeRomInfo, SpecPopeyeRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Popeye 2 (48K)

static struct BurnRomInfo SpecPopeye2RomDesc[] = {
	{ "Popeye 2 48K (1991)(Alternative).tap", 26112, 0x13458bc7, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecPopeye2, SpecPopeye2, Spectrum)
STD_ROM_FN(SpecPopeye2)

struct BurnDriver BurnSpecPopeye2 = {
	"spec_popeye2", NULL, "spec_spectrum", NULL, "1991",
	"Popeye 2 (48K)\0", NULL, "Alternative Software", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecPopeye2RomInfo, SpecPopeye2RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecQAOPSpaceDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Popeye 3 - Wrestle Crazy (128K)

static struct BurnRomInfo SpecPopeye3RomDesc[] = {
	{ "Popeye 3 - Wrestle Crazy 128K (1992)(Alternative).tap", 124204, 0xaa3d43e3, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecPopeye3, SpecPopeye3, Spec128)
STD_ROM_FN(SpecPopeye3)

struct BurnDriver BurnSpecPopeye3 = {
	"spec_popeye3", NULL, "spec_spec128", NULL, "1992",
	"Popeye 3 - Wrestle Crazy (128K)\0", NULL, "Alternative Software", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 2, HARDWARE_SPECTRUM, GBF_VSFIGHT, 0,
	SpectrumGetZipName, SpecPopeye3RomInfo, SpecPopeye3RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Poseidon - Planet Eleven (48K)

static struct BurnRomInfo SpecPoseidonRomDesc[] = {
	{ "Poseidon - Planet Eleven 48K (1990)(Hi-Tec).z80", 35217, 0x73a2cdf2, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecPoseidon, SpecPoseidon, Spectrum)
STD_ROM_FN(SpecPoseidon)

struct BurnDriver BurnSpecPoseidon = {
	"spec_poseidon", NULL, "spec_spectrum", NULL, "1990",
	"Poseidon - Planet Eleven (48K)\0", NULL, "Hi-Tec Software", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_RUNGUN | GBF_ADV, 0,
	SpectrumGetZipName, SpecPoseidonRomInfo, SpecPoseidonRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecIntf2DIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Postman Pat - The Computer Game (48K)

static struct BurnRomInfo SpecPostmanRomDesc[] = {
	{ "Postman Pat - The Computer Game 48K (1989)(Alternative).z80", 43141, 0xc991a405, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecPostman, SpecPostman, Spectrum)
STD_ROM_FN(SpecPostman)

struct BurnDriver BurnSpecPostman = {
	"spec_postman", NULL, "spec_spectrum", NULL, "1989",
	"Postman Pat - The Computer Game (48K)\0", NULL, "Alternative Software", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_ADV, 0,
	SpectrumGetZipName, SpecPostmanRomInfo, SpecPostmanRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Post Mortem (Spanish) (48K)

static struct BurnRomInfo SpecPostmortemRomDesc[] = {
	{ "Post Mortem 48K ES (1988)(Iber Soft).z80", 39261, 0xfbdeb9b7, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecPostmortem, SpecPostmortem, Spectrum)
STD_ROM_FN(SpecPostmortem)

struct BurnDriver BurnSpecPostmortem = {
	"spec_postmortem", NULL, "spec_spectrum", NULL, "1987",
	"Post Mortem (Spanish) (48K)\0", NULL, "Iber Software", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_ADV, 0,
	SpectrumGetZipName, SpecPostmortemRomInfo, SpecPostmortemRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecQAOPSpaceDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Potsworth & Co. (128K)

static struct BurnRomInfo SpecPotsworthRomDesc[] = {
	{ "Potsworth & Co. 128K (1992)(Hi-Tec).tap", 130851, 0x23ac170e, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecPotsworth, SpecPotsworth, Spec128)
STD_ROM_FN(SpecPotsworth)

struct BurnDriver BurnSpecPotsworth = {
	"spec_potsworth", NULL, "spec_spec128", NULL, "1992",
	"Potsworth & Co. (128K)\0", NULL, "Hi-Tec Software", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecPotsworthRomInfo, SpecPotsworthRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Prince, The (48K)

static struct BurnRomInfo SpecPrinceRomDesc[] = {
	{ "Prince, The 48K (1984)(CCS).tap", 63922, 0x09952fb7, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecPrince, SpecPrince, Spectrum)
STD_ROM_FN(SpecPrince)

struct BurnDriver BurnSpecPrince = {
	"spec_prince", NULL, "spec_spectrum", NULL, "1984",
	"Prince, The (48K)\0", NULL, "CCS", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 4, HARDWARE_SPECTRUM, GBF_STRATEGY, 0,
	SpectrumGetZipName, SpecPrinceRomInfo, SpecPrinceRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Prince of Persia (Russian) (128K)

static struct BurnRomInfo SpecPrincepersiaRomDesc[] = {
	{ "Prince of Persia RU 128K (1996)(Magic Soft).tap", 630191, 0x07200508, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecPrincepersia, SpecPrincepersia, Spec128)
STD_ROM_FN(SpecPrincepersia)

struct BurnDriver BurnSpecPrincepersia = {
	"spec_princepersia", NULL, "spec_spec128", NULL, "1996",
	"Prince of Persia (Russian) (128K)\0", NULL, "Magic Soft", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_PLATFORM | GBF_ADV, 0,
	SpectrumGetZipName, SpecPrincepersiaRomInfo, SpecPrincepersiaRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecIntf2DIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Prison Riot (128K)

static struct BurnRomInfo SpecPrisriotRomDesc[] = {
	{ "Prison Riot 128K (1990)(Players Premier).tap", 64518, 0xb760f8bd, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecPrisriot, SpecPrisriot, Spec128)
STD_ROM_FN(SpecPrisriot)

struct BurnDriver BurnSpecPrisriot = {
	"spec_prisriot", NULL, "spec_spec128", NULL, "1990",
	"Prison Riot (128K)\0", NULL, "Players Premier", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_MAZE | GBF_RUNGUN, 0,
	SpectrumGetZipName, SpecPrisriotRomInfo, SpecPrisriotRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Psi Chess (48K)

static struct BurnRomInfo SpecPsichessRomDesc[] = {
	{ "Psi Chess (1986)(The Edge).tap", 49152, 0xdeca5027, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecPsichess, SpecPsichess, Spectrum)
STD_ROM_FN(SpecPsichess)

struct BurnDriver BurnSpecPsichess = {
	"spec_psichess", NULL, "spec_spectrum", NULL, "1986",
	"Psi Chess (48K)\0", "S+2/S+3: 2D/3D view - M+1: start game - J+2: use controller", "The Edge", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 2, HARDWARE_SPECTRUM, GBF_STRATEGY | GBF_BOARD, 0,
	SpectrumGetZipName, SpecPsichessRomInfo, SpecPsichessRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecIntf2DIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Pssst (16K)

static struct BurnRomInfo SpecPssstRomDesc[] = {
	{ "Pssst 16K (1983)(Ultimate Play The Game).tap", 15927, 0x5f5272d5, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecPssst, SpecPssst, Spectrum)
STD_ROM_FN(SpecPssst)

struct BurnDriver BurnSpecPssst = {
	"spec_pssst", NULL, "spec_spectrum", NULL, "1983",
	"Pssst (16K)\0", NULL, "Ultimate Play The Game", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 2, HARDWARE_SPECTRUM, GBF_ACTION, 0,
	SpectrumGetZipName, SpecPssstRomInfo, SpecPssstRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Psycho Soldier (48K)

static struct BurnRomInfo SpecPsychosoldRomDesc[] = {
	{ "Psycho Soldier 48K (1987)(Imagine).tap", 85352, 0x4c6cacaa, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecPsychosold, SpecPsychosold, Spectrum)
STD_ROM_FN(SpecPsychosold)

struct BurnDriver BurnSpecPsychosold = {
	"spec_psychosold", NULL, "spec_spectrum", NULL, "1987",
	"Psycho Soldier (48K)\0", NULL, "Imagine Software", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_RUNGUN | GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecPsychosoldRomInfo, SpecPsychosoldRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Puffy's Saga (128K)

static struct BurnRomInfo SpecPuffysagaRomDesc[] = {
	{ "Puffy's Saga 128K (1989)(Ubi Soft).tap", 112327, 0x393670c2, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecPuffysaga, SpecPuffysaga, Spec128)
STD_ROM_FN(SpecPuffysaga)

struct BurnDriver BurnSpecPuffysaga = {
	"spec_puffysaga", NULL, "spec_spec128", NULL, "1989",
	"Puffy's Saga (128K)\0", NULL, "Ubi Soft", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_RUNGUN | GBF_MAZE, 0,
	SpectrumGetZipName, SpecPuffysagaRomInfo, SpecPuffysagaRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecIntf2DIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Pulsoid (128K)

static struct BurnRomInfo SpecPulsoidsRomDesc[] = {
	{ "Pulsoids (1988)(Mastertronic).tap", 35011, 0x9d1fc1e3, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecPulsoids, SpecPulsoids, Spec128)
STD_ROM_FN(SpecPulsoids)

struct BurnDriver BurnSpecPulsoids = {
	"spec_pulsoids", NULL, "spec_spec128", NULL, "1988",
	"Pulsoid (128K)\0", NULL, "Mastertronic", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 2, HARDWARE_SPECTRUM, GBF_BREAKOUT, 0,
	SpectrumGetZipName, SpecPulsoidsRomInfo, SpecPulsoidsRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Punk Star (Spanish) (48K)

static struct BurnRomInfo SpecPunkstarRomDesc[] = {
	{ "Punk Star ES 48K (1988)(Genesis Soft).tap", 27136, 0xca188147, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecPunkstar, SpecPunkstar, Spectrum)
STD_ROM_FN(SpecPunkstar)

struct BurnDriver BurnSpecPunkstar = {
	"spec_punkstar", NULL, "spec_spectrum", NULL, "1988",
	"Punk Star (Spanish) (48K)\0", NULL, "Genesis Soft", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecPunkstarRomInfo, SpecPunkstarRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Puzzled! (48K)

static struct BurnRomInfo SpecPuzzledRomDesc[] = {
	{ "Puzzled! 48K (1989)(Guild, The).tap", 47979, 0x5a86daa5, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecPuzzled, SpecPuzzled, Spectrum)
STD_ROM_FN(SpecPuzzled)

struct BurnDriver BurnSpecPuzzled = {
	"spec_puzzled", NULL, "spec_spectrum", NULL, "1989",
	"Puzzled! (48K)\0", NULL, "Guild, The", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_ADV, 0,
	SpectrumGetZipName, SpecPuzzledRomInfo, SpecPuzzledRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Pyjamarama (48K)

static struct BurnRomInfo SpecPyjamaramaRomDesc[] = {
	{ "Pyjamarama 48K (1984)(Mikro-Gen).z80", 42069, 0x6801d962, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecPyjamarama, SpecPyjamarama, Spectrum)
STD_ROM_FN(SpecPyjamarama)

struct BurnDriver BurnSpecPyjamarama = {
	"spec_pyjamarama", NULL, "spec_spectrum", NULL, "1984",
	"Pyjamarama (48K)\0", NULL, "Mikro-Gen", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_PLATFORM | GBF_ADV, 0,
	SpectrumGetZipName, SpecPyjamaramaRomInfo, SpecPyjamaramaRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Pyracurse (48K)

static struct BurnRomInfo SpecPyracurseRomDesc[] = {
	{ "Pyracurse 48K (1986)(Hewson Consultants).tap", 47935, 0x0b498400, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecPyracurse, SpecPyracurse, Spectrum)
STD_ROM_FN(SpecPyracurse)

struct BurnDriver BurnSpecPyracurse = {
	"spec_pyracurse", NULL, "spec_spectrum", NULL, "1986",
	"Pyracurse (48K)\0", NULL, "Hewson Consultants", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_ACTION | GBF_ADV, 0,
	SpectrumGetZipName, SpecPyracurseRomInfo, SpecPyracurseRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Q10 Tank Buster (48K)

static struct BurnRomInfo SpecQ10tbRomDesc[] = {
	{ "Q10 Tank Buster 48K (1992)(Zeppelin Games).tap", 43523, 0x1fad3a24, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecQ10tb, SpecQ10tb, Spectrum)
STD_ROM_FN(SpecQ10tb)

struct BurnDriver BurnSpecQ10tb = {
	"spec_q10tb", NULL, "spec_spectrum", NULL, "1992",
	"Q10 Tank Buster (48K)\0", NULL, "Zeppelin Games", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_HORSHOOT, 0,
	SpectrumGetZipName, SpecQ10tbRomInfo, SpecQ10tbRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Rabbit Rabbit (16K)

static struct BurnRomInfo SpecRabbitrabbitRomDesc[] = {
	{ "Rabbit Rabbit 16K (1983)(ZX Computing).tap", 4937, 0xc741bf71, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecRabbitrabbit, SpecRabbitrabbit, Spectrum)
STD_ROM_FN(SpecRabbitrabbit)

struct BurnDriver BurnSpecRabbitrabbit = {
	"spec_rabbitrabbit", NULL, "spec_spectrum", NULL, "1983",
	"Rabbit Rabbit (16K)\0", NULL, "ZX Computing", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_MAZE, 0,
	SpectrumGetZipName, SpecRabbitrabbitRomInfo, SpecRabbitrabbitRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecCursorKeysDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Race, The (128K)

static struct BurnRomInfo SpecTheraceRomDesc[] = {
	{ "Race, The 128K (1990)(Players Premier).tap", 52939, 0x68b09f7c, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecTherace, SpecTherace, Spec128)
STD_ROM_FN(SpecTherace)

struct BurnDriver BurnSpecTherace = {
	"spec_therace", NULL, "spec_spec128", NULL, "1990",
	"Race, The (128K)\0", NULL, "Players Premier", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 2, HARDWARE_SPECTRUM, GBF_ACTION | GBF_RACING, 0,
	SpectrumGetZipName, SpecTheraceRomInfo, SpecTheraceRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Raid over Moscow (48K)

static struct BurnRomInfo SpecRaidmoscowRomDesc[] = {
	{ "Raid over Moscow 48K (1985)(U.S. Gold).tap", 49145, 0xe7b7fe42, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecRaidmoscow, SpecRaidmoscow, Spectrum)
STD_ROM_FN(SpecRaidmoscow)

struct BurnDriver BurnSpecRaidmoscow = {
	"spec_raidmoscow", NULL, "spec_spectrum", NULL, "1985",
	"Raid over Moscow (48K)\0", NULL, "U.S. Gold", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_SHOOT, 0,
	SpectrumGetZipName, SpecRaidmoscowRomInfo, SpecRaidmoscowRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Rally Cross (128K)

static struct BurnRomInfo SpecRallycrossRomDesc[] = {
	{ "Rally Cross 128K (1989)(Anco Software).tap", 81416, 0xe6e3cfb8, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecRallycross, SpecRallycross, Spec128)
STD_ROM_FN(SpecRallycross)

struct BurnDriver BurnSpecRallycross = {
	"spec_rallycross", NULL, "spec_spec128", NULL, "1989",
	"Rally Cross (128K)\0", NULL, "Anco Software", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 2, HARDWARE_SPECTRUM, GBF_RACING, 0,
	SpectrumGetZipName, SpecRallycrossRomInfo, SpecRallycrossRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Rambo - First Blood Part II (48K)

static struct BurnRomInfo SpecramboRomDesc[] = {
	{ "Rambo - First Blood Part II 48K (1985)(Ocean).tap", 49392, 0x769660f9, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specrambo, Specrambo, Spectrum)
STD_ROM_FN(Specrambo)

struct BurnDriver BurnSpecrambo = {
	"spec_rambo", NULL, "spec_spectrum", NULL, "1985",
	"Rambo - First Blood Part II (48K)\0", NULL, "Ocean", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 2, HARDWARE_SPECTRUM, GBF_RUNGUN, 0,
	SpectrumGetZipName, SpecramboRomInfo, SpecramboRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Rambo III (128K)

static struct BurnRomInfo Specrambo3RomDesc[] = {
	{ "Rambo III 128K (1988)(Ocean).tap", 95128, 0x63e49eee, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specrambo3, Specrambo3, Spec128)
STD_ROM_FN(Specrambo3)

struct BurnDriver BurnSpecrambo3 = {
	"spec_rambo3", NULL, "spec_spec128", NULL, "1988",
	"Rambo III (128K)\0", NULL, "Ocean", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_RUNGUN | GBF_ADV, 0,
	SpectrumGetZipName, Specrambo3RomInfo, Specrambo3RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Rampage (48K)

static struct BurnRomInfo SpecrampageRomDesc[] = {
	{ "Rampage 48K (1988)(Activision).z80", 0x094eb, 0x3735beaf, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specrampage, Specrampage, Spectrum)
STD_ROM_FN(Specrampage)

struct BurnDriver BurnSpecrampage = {
	"spec_rampage", NULL, "spec_spectrum", NULL, "1988",
	"Rampage (48K)\0", NULL, "Activision", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 3, HARDWARE_SPECTRUM, GBF_ACTION, 0,
	SpectrumGetZipName, SpecrampageRomInfo, SpecrampageRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Ramparts (48K)

static struct BurnRomInfo SpecRampartsRomDesc[] = {
	{ "Ramparts 48K (1988)(Go!).tap", 37824, 0xd6391cbe, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecRamparts, SpecRamparts, Spectrum)
STD_ROM_FN(SpecRamparts)

struct BurnDriver BurnSpecRamparts = {
	"spec_ramparts", NULL, "spec_spectrum", NULL, "1988",
	"Ramparts (48K)\0", NULL, "Go!", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 2, HARDWARE_SPECTRUM, GBF_ACTION, 0,
	SpectrumGetZipName, SpecRampartsRomInfo, SpecRampartsRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Rastan Saga (128K)

static struct BurnRomInfo SpecrastanRomDesc[] = {
	{ "Rastan Saga 128K (1988)(Imagine).tap", 190502, 0xf8c66b9c, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specrastan, Specrastan, Spec128)
STD_ROM_FN(Specrastan)

struct BurnDriver BurnSpecrastan = {
	"spec_rastan", NULL, "spec_spec128", NULL, "1988",
	"Rastan Saga (128K)\0", NULL, "Imagine Software", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_SCRFIGHT, 0,
	SpectrumGetZipName, SpecrastanRomInfo, SpecrastanRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Real Ghostbusters, The (128K)

static struct BurnRomInfo SpecRealgbRomDesc[] = {
	{ "Real Ghostbusters, The 128K (1989)(Activision).tap", 233398, 0xae6d542e, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecRealgb, SpecRealgb, Spec128)
STD_ROM_FN(SpecRealgb)

struct BurnDriver BurnSpecRealgb = {
	"spec_realgb", NULL, "spec_spec128", NULL, "1989",
	"Real Ghostbusters, The (128K)\0", NULL, "Activision", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 2, HARDWARE_SPECTRUM, GBF_RUNGUN, 0,
	SpectrumGetZipName, SpecRealgbRomInfo, SpecRealgbRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecIntf2DIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Reaper (128K)

static struct BurnRomInfo SpecReaperRomDesc[] = {
	{ "Reaper 128K (1990)(Ubi Soft).tap", 66597, 0xe2f63392, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecReaper, SpecReaper, Spec128)
STD_ROM_FN(SpecReaper)

struct BurnDriver BurnSpecReaper = {
	"spec_reaper", NULL, "spec_spec128", NULL, "1990",
	"Reaper (128K)\0", NULL, "Ubi Soft", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 2, HARDWARE_SPECTRUM, GBF_SCRFIGHT | GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecReaperRomInfo, SpecReaperRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Rebel (48K)

static struct BurnRomInfo SpecRebelRomDesc[] = {
	{ "Rebel 48K (1987)(Virgin Games).tap", 47276, 0xf7833b9c, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecRebel, SpecRebel, Spectrum)
STD_ROM_FN(SpecRebel)

struct BurnDriver BurnSpecRebel = {
	"spec_rebel", NULL, "spec_spectrum", NULL, "1987",
	"Rebel (48K)\0", NULL, "Virgin Games", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_ACTION, 0,
	SpectrumGetZipName, SpecRebelRomInfo, SpecRebelRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Rebelstar - 1 Player (48K)

static struct BurnRomInfo SpecRebstar1pRomDesc[] = {
	{ "Rebelstar - 1 Player 48K (1986)(Firebird).tap", 47940, 0xa709f199, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecRebstar1p, SpecRebstar1p, Spectrum)
STD_ROM_FN(SpecRebstar1p)

struct BurnDriver BurnSpecRebstar1p = {
	"spec_rebstar1p", NULL, "spec_spectrum", NULL, "1986",
	"Rebelstar - 1 Player (48K)\0", "Use keys W, A, D, X to move cursor", "Firebird Software", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_STRATEGY, 0,
	SpectrumGetZipName, SpecRebstar1pRomInfo, SpecRebstar1pRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Rebelstar - 2 Players (48K)

static struct BurnRomInfo SpecRebstar2pRomDesc[] = {
	{ "Rebelstar - 2 Players 48K (1986)(Firebird).tap", 47627, 0xe924d51f, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecRebstar2p, SpecRebstar2p, Spectrum)
STD_ROM_FN(SpecRebstar2p)

struct BurnDriver BurnSpecRebstar2p = {
	"spec_rebstar2p", "spec_rebstar1p", "spec_spectrum", NULL, "1986",
	"Rebelstar - 2 Players (48K)\0", "Use keys W, A, D, X to move cursor", "Firebird Software", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE, 2, HARDWARE_SPECTRUM, GBF_STRATEGY, 0,
	SpectrumGetZipName, SpecRebstar2pRomInfo, SpecRebstar2pRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Reckless Rufus (128K)

static struct BurnRomInfo SpecReckrufusRomDesc[] = {
	{ "Reckless Rufus 128K (1992)(Alternative).tap", 54968, 0x43172f9b, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecReckrufus, SpecReckrufus, Spec128)
STD_ROM_FN(SpecReckrufus)

struct BurnDriver BurnSpecReckrufus = {
	"spec_reckrufus", NULL, "spec_spec128", NULL, "1992",
	"Reckless Rufus (128K)\0", NULL, "Alternative Software", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_PUZZLE, 0,
	SpectrumGetZipName, SpecReckrufusRomInfo, SpecReckrufusRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Red Heat (48K)

static struct BurnRomInfo SpecRedheatRomDesc[] = {
	{ "Red Heat 48K (1989)(Ocean).tap", 119068, 0x4d1c484b, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecRedheat, SpecRedheat, Spectrum)
STD_ROM_FN(SpecRedheat)

struct BurnDriver BurnSpecRedheat = {
	"spec_redheat", NULL, "spec_spectrum", NULL, "1989",
	"Red Heat (48K)\0", NULL, "Ocean", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_SCRFIGHT, 0,
	SpectrumGetZipName, SpecRedheatRomInfo, SpecRedheatRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Rescate Atlantida (Spanish) (128K)

static struct BurnRomInfo SpecRescatlanRomDesc[] = {
	{ "Rescate Atlantida ES 128K (1989)(Dinamic).z80", 109724, 0xc437d0e7, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecRescatlan, SpecRescatlan, Spec128)
STD_ROM_FN(SpecRescatlan)

struct BurnDriver BurnSpecRescatlan = {
	"spec_rescatlan", NULL, "spec_spec128", NULL, "1989",
	"Rescate Atlantida (Spanish) (128K)\0", "Press SPACE to choose and ENTER to select options", "Dinamic Software", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_SHOOT, 0,
	SpectrumGetZipName, SpecRescatlanRomInfo, SpecRescatlanRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Rescate en el Golfo - Part 1 (128K)

static struct BurnRomInfo Specrescategolfo1RomDesc[] = {
	{ "Rescate en el Golfo (1990)(Opera Soft)(Part 1 of 2).z80", 34779, 0xd7add860, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specrescategolfo1, Specrescategolfo1, Spec128)
STD_ROM_FN(Specrescategolfo1)

struct BurnDriver BurnSpecrescategolfo1 = {
	"spec_rescategolfo1", NULL, "spec_spec128", NULL, "1990",
	"Rescate en el Golfo - Part 1 (128K)\0", NULL, "Opera Soft", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_SCRFIGHT, 0,
	SpectrumGetZipName, Specrescategolfo1RomInfo, Specrescategolfo1RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecQAOPSpaceDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Rescate en el Golfo - Part 2 (128K)

static struct BurnRomInfo Specrescategolfo2RomDesc[] = {
	{ "Rescate en el Golfo (1990)(Opera Soft)(Part 2 of 2).z80", 37620, 0x370d2085, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specrescategolfo2, Specrescategolfo2, Spec128)
STD_ROM_FN(Specrescategolfo2)

struct BurnDriver BurnSpecrescategolfo2 = {
	"spec_rescategolfo2", "spec_rescategolfo1", "spec_spec128", NULL, "1990",
	"Rescate en el Golfo - Part 2 (128K)\0", NULL, "Opera Soft", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE, 1, HARDWARE_SPECTRUM, GBF_SCRFIGHT, 0,
	SpectrumGetZipName, Specrescategolfo2RomInfo, Specrescategolfo2RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecQAOPSpaceDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Return of the Jedi (128K)

static struct BurnRomInfo SpecRetofjediRomDesc[] = {
	{ "Return of the Jedi 128K (1989)(Domark).tap", 37222, 0xdccfc93b, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecRetofjedi, SpecRetofjedi, Spec128)
STD_ROM_FN(SpecRetofjedi)

struct BurnDriver BurnSpecRetofjedi = {
	"spec_retofjedi", NULL, "spec_spec128", NULL, "1989",
	"Return of the Jedi (128K)\0", NULL, "Domark", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_SHOOT | GBF_ACTION, 0,
	SpectrumGetZipName, SpecRetofjediRomInfo, SpecRetofjediRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Reveal (48K)

static struct BurnRomInfo SpecRevealRomDesc[] = {
	{ "Reveal 48K (1988)(Mastertronic).tap", 26043, 0x9d350348, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecReveal, SpecReveal, Spectrum)
STD_ROM_FN(SpecReveal)

struct BurnDriver BurnSpecReveal = {
	"spec_reveal", NULL, "spec_spectrum", NULL, "1988",
	"Reveal (48K)\0", NULL, "Mastertronic", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_ACTION, 0,
	SpectrumGetZipName, SpecRevealRomInfo, SpecRevealRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Riding the Rapids (48K)

static struct BurnRomInfo SpecRidrapidsRomDesc[] = {
	{ "Riding the Rapids 48K (1987)(Players Software).tap", 47675, 0x940cabf6, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecRidrapids, SpecRidrapids, Spectrum)
STD_ROM_FN(SpecRidrapids)

struct BurnDriver BurnSpecRidrapids = {
	"spec_ridrapids", NULL, "spec_spectrum", NULL, "1987",
	"Riding the Rapids (48K)\0", NULL, "Players Software", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_ACTION, 0,
	SpectrumGetZipName, SpecRidrapidsRomInfo, SpecRidrapidsRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Rifle Range (16K)

static struct BurnRomInfo SpecRiflerangeRomDesc[] = {
	{ "Rifle Range 16K (1984)(Mastertronic).tap", 24834, 0xf94351b2, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecRiflerange, SpecRiflerange, Spectrum)
STD_ROM_FN(SpecRiflerange)

struct BurnDriver BurnSpecRiflerange = {
	"spec_riflerange", NULL, "spec_spectrum", NULL, "1984",
	"Rifle Range (16K)\0", NULL, "Mastertronic", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_SHOOT, 0,
	SpectrumGetZipName, SpecRiflerangeRomInfo, SpecRiflerangeRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecIntf2DIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Road Blasters (128K)

static struct BurnRomInfo SpecRblasterRomDesc[] = {
	{ "Road Blasters 128K (1988)(U.S. Gold).tap", 125428, 0x44e22515, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecRblaster, SpecRblaster, Spec128)
STD_ROM_FN(SpecRblaster)

struct BurnDriver BurnSpecRblaster = {
	"spec_rblaster", NULL, "spec_spec128", NULL, "1988",
	"Road Blasters (128K)\0", NULL, "U.S. Gold", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_RACING | GBF_SHOOT, 0,
	SpectrumGetZipName, SpecRblasterRomInfo, SpecRblasterRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Road Runner (48K)

static struct BurnRomInfo SpecRoadrunnerRomDesc[] = {
	{ "Road Runner 48K (1987)(U.S. Gold).z80", 45123, 0x4b781d7c, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecRoadrunner, SpecRoadrunner, Spectrum)
STD_ROM_FN(SpecRoadrunner)

struct BurnDriver BurnSpecRoadrunner = {
	"spec_roadrunner", NULL, "spec_spectrum", NULL, "1987",
	"Road Runner (48K)\0", NULL, "U.S. Gold", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_ACTION, 0,
	SpectrumGetZipName, SpecRoadrunnerRomInfo, SpecRoadrunnerRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Road Runner and Wile E. Coyote (48K)

static struct BurnRomInfo SpecRoadwile48RomDesc[] = {
	{ "Road Runner and Wile E. Coyote (1991)(Hi-Tec).tap", 50312, 0x766bd6a4, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecRoadwile48, SpecRoadwile48, Spectrum)
STD_ROM_FN(SpecRoadwile48)

struct BurnDriver BurnSpecRoadwile = {
	"spec_roadwile48", "spec_roadwile128", "spec_spectrum", NULL, "1991",
	"Road Runner and Wile E. Coyote (48K)\0", NULL, "Hi-Tec Software", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE, 1, HARDWARE_SPECTRUM, GBF_ACTION, 0,
	SpectrumGetZipName, SpecRoadwile48RomInfo, SpecRoadwile48RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Road Runner and Wile E. Coyote (128K)

static struct BurnRomInfo SpecRoadwile128RomDesc[] = {
	{ "Road Runner and Wile E. Coyote (1991)(Hi-Tec).tap", 50312, 0x766bd6a4, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecRoadwile128, SpecRoadwile128, Spec128)
STD_ROM_FN(SpecRoadwile128)

struct BurnDriver BurnSpecRoadwile128 = {
	"spec_roadwile128", NULL, "spec_spec128", NULL, "1991",
	"Road Runner and Wile E. Coyote (128K)\0", NULL, "Hi-Tec Software", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_ACTION, 0,
	SpectrumGetZipName, SpecRoadwile128RomInfo, SpecRoadwile128RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Rocco (48K)

static struct BurnRomInfo SpecRoccoRomDesc[] = {
	{ "Rocco 48K (1985)(Gremlin Graphics).z80", 38035, 0xdae7f761, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecRocco, SpecRocco, Spectrum)
STD_ROM_FN(SpecRocco)

struct BurnDriver BurnSpecRocco = {
	"spec_rocco", "spec_rocky", "spec_spectrum", NULL, "1985",
	"Rocco (48K)\0", NULL, "Gremlin Graphics", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE, 1, HARDWARE_SPECTRUM, GBF_VSFIGHT, 0,
	SpectrumGetZipName, SpecRoccoRomInfo, SpecRoccoRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Rocky (48K)

static struct BurnRomInfo SpecRockyRomDesc[] = {
	{ "Rocky 48K (1985)(Dinamic).z80", 38026, 0x79a0f6b2, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecRocky, SpecRocky, Spectrum)
STD_ROM_FN(SpecRocky)

struct BurnDriver BurnSpecRocky = {
	"spec_rocky", NULL, "spec_spectrum", NULL, "1985",
	"Rocky (48K)\0", NULL, "Dinamic Software", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_VSFIGHT, 0,
	SpectrumGetZipName, SpecRockyRomInfo, SpecRockyRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Rocky Horror Show, The (128K)

static struct BurnRomInfo SpecRhs128RomDesc[] = {
	{ "Rocky Horror Show, The 128K (1985)(CRL Group).tap", 81739, 0x011957b4, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecRhs128, SpecRhs128, Spec128)
STD_ROM_FN(SpecRhs128)

struct BurnDriver BurnSpecRhs128 = {
	"spec_rhs128", NULL, "spec_spec128", NULL, "1985",
	"Rocky Horror Show, The (128K)\0", "Press 'Space' to select and 'Enter' to set options", "CRL Group", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_ADV, 0,
	SpectrumGetZipName, SpecRhs128RomInfo, SpecRhs128RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Rocky Horror Show, The (48K)

static struct BurnRomInfo SpecRhs48RomDesc[] = {
	{ "Rocky Horror Show, The 48K (1985)(CRL Group).tap", 49355, 0x12082319, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecRhs48, SpecRhs48, Spectrum)
STD_ROM_FN(SpecRhs48)

struct BurnDriver BurnSpecRhs48 = {
	"spec_rhs48", "spec_rhs128", "spec_spectrum", NULL, "1985",
	"Rocky Horror Show, The (48K)\0", "Press 'Space' to select and 'Enter' to set options", "CRL Group", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE, 1, HARDWARE_SPECTRUM, GBF_ADV, 0,
	SpectrumGetZipName, SpecRhs48RomInfo, SpecRhs48RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Roland's Rat Race (48K)

static struct BurnRomInfo SpecRolandratRomDesc[] = {
	{ "Roland's Rat Race 48K (1985)(Ocean).tap", 44022, 0x44edc93e, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecRolandrat, SpecRolandrat, Spectrum)
STD_ROM_FN(SpecRolandrat)

struct BurnDriver BurnSpecRolandrat = {
	"spec_rolandrat", NULL, "spec_spectrum", NULL, "1985",
	"Roland's Rat Race (48K)\0", NULL, "Ocean", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_ACTION, 0,
	SpectrumGetZipName, SpecRolandratRomInfo, SpecRolandratRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Roller Coaster (48K)

static struct BurnRomInfo SpecRollercoastRomDesc[] = {
	{ "Roller Coaster 48K (1985)(Elite Systems).tap", 49343, 0xbd2b1a0f, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecRollercoast, SpecRollercoast, Spectrum)
STD_ROM_FN(SpecRollercoast)

struct BurnDriver BurnSpecRollercoast = {
	"spec_rollercoast", NULL, "spec_spectrum", NULL, "1985",
	"Roller Coaster (48K)\0", NULL, "Elite Systems", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecRollercoastRomInfo, SpecRollercoastRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// R-Type (128K)

static struct BurnRomInfo SpecrtypeRomDesc[] = {
	{ "R-Type 128K (1988)(Electric Dreams).tap", 0x206d7, 0xa03330a0, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specrtype, Specrtype, Spec128)
STD_ROM_FN(Specrtype)

struct BurnDriver BurnSpecrtype = {
	"spec_rtype", NULL, "spec_spec128", NULL, "1988",
	"R-Type (128K)\0", NULL, "Electric Dreams", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_HORSHOOT, 0,
	SpectrumGetZipName, SpecrtypeRomInfo, SpecrtypeRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Ruff and Reddy in the Space Adventure (128K)

static struct BurnRomInfo SpecRuffreddyRomDesc[] = {
	{ "Ruff and Reddy in the Space Adventure 128K (1990)(Hi-Tec).tap", 54708, 0xffe8fec1, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecRuffreddy, SpecRuffreddy, Spec128)
STD_ROM_FN(SpecRuffreddy)

struct BurnDriver BurnSpecRuffreddy = {
	"spec_ruffreddy", NULL, "spec_spec128", NULL, "1990",
	"Ruff and Reddy in the Space Adventure (128K)\0", NULL, "Hi-Tec Software", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecRuffreddyRomInfo, SpecRuffreddyRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Run the Gauntlet (128K)

static struct BurnRomInfo SpecrungauntRomDesc[] = {
	{ "Run the Gauntlet 128K (1989)(Ocean).z80", 0x1a162, 0xe2e3b97c, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specrungaunt, Specrungaunt, Spec128)
STD_ROM_FN(Specrungaunt)

struct BurnDriver BurnSpecrungaunt = {
	"spec_rungaunt", NULL, "spec_spec128", NULL, "1989",
	"Run the Gauntlet (128K)\0", "During the game press 'H' (Pause) then '2' to set controller", "Ocean", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 2, HARDWARE_SPECTRUM, GBF_RACING, 0,
	SpectrumGetZipName, SpecrungauntRomInfo, SpecrungauntRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Rygar (48K)

static struct BurnRomInfo SpecrygarRomDesc[] = {
	{ "Rygar - Legendary Warrior 48K (1987)(U.S. Gold).tap", 44813, 0xc64bbc03, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specrygar, Specrygar, Spectrum)
STD_ROM_FN(Specrygar)

struct BurnDriver BurnSpecrygar = {
	"spec_rygar", NULL, "spec_spectrum", NULL, "1987",
	"Rygar - Legendary Warrior (48K)\0", NULL, "U.S. Gold", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_RUNGUN | GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecrygarRomInfo, SpecrygarRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Sabotage (48K)

static struct BurnRomInfo SpecSabotageRomDesc[] = {
	{ "Sabotage 48K (1988)(Zeppelin Games).tap", 43408, 0xb83fb136, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecSabotage, SpecSabotage, Spectrum)
STD_ROM_FN(SpecSabotage)

struct BurnDriver BurnSpecSabotage = {
	"spec_sabotage", NULL, "spec_spectrum", NULL, "1988",
	"Sabotage (48K)\0", "Select icon 'S' to use controller", "Zeppelin Games", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_VERSHOOT, 0,
	SpectrumGetZipName, SpecSabotageRomInfo, SpecSabotageRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecIntf2DIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Sabrina (Spanish) (48K)

static struct BurnRomInfo SpecSabrinaRomDesc[] = {
	{ "Sabrina ES 48K (1989)(Iber Soft).z80", 35718, 0x04ffed4b, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecSabrina, SpecSabrina, Spectrum)
STD_ROM_FN(SpecSabrina)

struct BurnDriver BurnSpecSabrina = {
	"spec_sabrina", NULL, "spec_spectrum", NULL, "1989",
	"Sabrina (Spanish) (48K)\0", NULL, "Iber Software", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_SCRFIGHT, 0,
	SpectrumGetZipName, SpecSabrinaRomInfo, SpecSabrinaRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecQAOPMDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Samantha Fox Strip Poker (48K)

static struct BurnRomInfo SpecSamanthaspRomDesc[] = {
	{ "Samantha Fox Strip Poker 48K (1986)(Martech).tap", 48286, 0x06ac3b93, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecSamanthasp, SpecSamanthasp, Spectrum)
STD_ROM_FN(SpecSamanthasp)

struct BurnDriver BurnSpecSamanthasp = {
	"spec_samanthasp", NULL, "spec_spectrum", NULL, "1986",
	"Samantha Fox Strip Poker (48K)\0", "Adult game", "Martech Games", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_CARD, 0,
	SpectrumGetZipName, SpecSamanthaspRomInfo, SpecSamanthaspRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Samurai Warrior ~ Usagi Yojimbo (128K)

static struct BurnRomInfo SpecSamwarrior128kRomDesc[] = {
	{ "Samurai Warrior - Usagi Yojimbo 128K (1988)(Firebird).tap", 53473, 0x989e9ffd, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecSamwarrior128k, SpecSamwarrior128k, Spec128)
STD_ROM_FN(SpecSamwarrior128k)

struct BurnDriver BurnSpecSamwarrior128k = {
	"spec_samwarrior128k", NULL, "spec_spec128", NULL, "1988",
	"Samurai Warrior ~ Usagi Yojimbo (128K)\0", NULL, "Firebird Software", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_SCRFIGHT, 0,
	SpectrumGetZipName, SpecSamwarrior128kRomInfo, SpecSamwarrior128kRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Santa's Xmas Caper (48K)

static struct BurnRomInfo SpecSxmascaperRomDesc[] = {
	{ "Santa's Xmas Caper 48K (1990)(Zeppelin Games).tap", 47687, 0x5430dcc6, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecSxmascaper, SpecSxmascaper, Spectrum)
STD_ROM_FN(SpecSxmascaper)

struct BurnDriver BurnSpecSxmascaper = {
	"spec_sxmascaper", NULL, "spec_spectrum", NULL, "1990",
	"Santa's Xmas Caper (48K)\0", "Set '1 using sinclair' to use controller", "Zeppelin Games", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_HORSHOOT, 0,
	SpectrumGetZipName, SpecSxmascaperRomInfo, SpecSxmascaperRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecIntf2DIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Sanxion - The Spectrum Remix (128K)

static struct BurnRomInfo SpecSanxion128RomDesc[] = {
	{ "Sanxion - The Spectrum Remix (1989)(Thalamus)(128K).tap", 48009, 0x0cb4ad4d, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecSanxion128, SpecSanxion128, Spec128)
STD_ROM_FN(SpecSanxion128)

struct BurnDriver BurnSpecSanxion128 = {
	"spec_sanxion128", NULL, "spec_spec128", NULL, "1989",
	"Sanxion - The Spectrum Remix (128K)\0", NULL, "Thalamus", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_HORSHOOT, 0,
	SpectrumGetZipName, SpecSanxion128RomInfo, SpecSanxion128RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Sanxion - The Spectrum Remix (48K)

static struct BurnRomInfo SpecSanxion48RomDesc[] = {
	{ "Sanxion - The Spectrum Remix (1989)(Thalamus)(48K-128K).z80", 19615, 0x97a506fa, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecSanxion48, SpecSanxion48, Spectrum)
STD_ROM_FN(SpecSanxion48)

struct BurnDriver BurnSpecSanxion48 = {
	"spec_sanxion48", "spec_sanxion128", "spec_spectrum", NULL, "1989",
	"Sanxion - The Spectrum Remix (48K)\0", NULL, "Thalamus", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE, 1, HARDWARE_SPECTRUM, GBF_HORSHOOT, 0,
	SpectrumGetZipName, SpecSanxion48RomInfo, SpecSanxion48RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Scooby Doo in the Castle Mystery (48K)

static struct BurnRomInfo SpecScoobydooRomDesc[] = {
	{ "Scooby Doo in the Castle Mystery 48K (1986)(Elite Systems).tap", 46057, 0xf1436e40, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecScoobydoo, SpecScoobydoo, Spectrum)
STD_ROM_FN(SpecScoobydoo)

struct BurnDriver BurnSpecScoobydoo = {
	"spec_scoobydoo", NULL, "spec_spectrum", NULL, "1986",
	"Scooby Doo in the Castle Mystery (48K)\0", "Press 'J' to controller menu", "Elite Systems", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecScoobydooRomInfo, SpecScoobydooRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Score 3020 (Spanish) (48K)

static struct BurnRomInfo SpecScore3020RomDesc[] = {
	{ "Score 3020 ES 48K (1989)(Topo Soft).z80", 35177, 0x3f505d11, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecScore3020, SpecScore3020, Spectrum)
STD_ROM_FN(SpecScore3020)

struct BurnDriver BurnSpecScore3020 = {
	"spec_score3020", NULL, "spec_spectrum", NULL, "1989",
	"Score 3020 (Spanish) (48K)\0", NULL, "Topo Soft", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_PINBALL, 0,
	SpectrumGetZipName, SpecScore3020RomInfo, SpecScore3020RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Scuba Dive (48K)

static struct BurnRomInfo SpecScubadiveRomDesc[] = {
	{ "Scuba Dive (1983)(Durell).z80", 28778, 0xfc1543e0, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecScubadive, SpecScubadive, Spectrum)
STD_ROM_FN(SpecScubadive)

struct BurnDriver BurnSpecScubadive = {
	"spec_scubadive", NULL, "spec_spectrum", NULL, "1983",
	"Scuba Dive (48K)\0", NULL, "Durell Software", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_ACTION, 0,
	SpectrumGetZipName, SpecScubadiveRomInfo, SpecScubadiveRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecQAOPSpaceDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Senda Salvaje - Parte 1 (Spanish) (48K)

static struct BurnRomInfo SpecSendasalvaje1RomDesc[] = {
	{ "Senda Salvaje - Parte 1 ES 48K (1990)(Zigurat).tap", 48033, 0x62afcb2d, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecSendasalvaje1, SpecSendasalvaje1, Spectrum)
STD_ROM_FN(SpecSendasalvaje1)

struct BurnDriver BurnSpecSendasalvaje1 = {
	"spec_sendasalvaje1", NULL, "spec_spectrum", NULL, "1990",
	"Senda Salvaje - Parte 1 (Spanish) (48K)\0", NULL, "Zigurat Software", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_SCRFIGHT, 0,
	SpectrumGetZipName, SpecSendasalvaje1RomInfo, SpecSendasalvaje1RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Senda Salvaje - Parte 2 (Spanish) (48K)

static struct BurnRomInfo SpecSendasalvaje2RomDesc[] = {
	{ "Senda Salvaje - Parte 2 ES 48K (1990)(Zigurat).tap", 47701, 0xaccef9d1, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecSendasalvaje2, SpecSendasalvaje2, Spectrum)
STD_ROM_FN(SpecSendasalvaje2)

struct BurnDriver BurnSpecSendasalvaje2 = {
	"spec_sendasalvaje2", "spec_sendasalvaje1", "spec_spectrum", NULL, "1990",
	"Senda Salvaje - Parte 2 (Spanish) (48K)\0", "Password: PLATANON", "Zigurat Software", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE, 1, HARDWARE_SPECTRUM, GBF_SCRFIGHT, 0,
	SpectrumGetZipName, SpecSendasalvaje2RomInfo, SpecSendasalvaje2RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Sgrizam (Spanish) (48K)

static struct BurnRomInfo SpecSgrizamRomDesc[] = {
	{ "Sgrizam ES 48K (1985)(Dinamic).tap", 41246, 0xdfc7efc4, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecSgrizam, SpecSgrizam, Spectrum)
STD_ROM_FN(SpecSgrizam)

struct BurnDriver BurnSpecSgrizam = {
	"spec_sgrizam", NULL, "spec_spectrum", NULL, "1985",
	"Sgrizam (Spanish) (48K)\0", "Set '1 Teclado' to use controller", "Dinamic Software", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_ACTION, 0,
	SpectrumGetZipName, SpecSgrizamRomInfo, SpecSgrizamRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecQAOPSpaceDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Shadow of the Unicorn (128K, 'rebuilt' version)

static struct BurnRomInfo SpecShadowotuRomDesc[] = {
	{ "Shadow of the Unicorn 128K (1985)(Mikro-Gen).tap", 64656, 0xe88639c9, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecShadowotu, SpecShadowotu, Spec1282a)
STD_ROM_FN(SpecShadowotu)

struct BurnDriver BurnSpecShadowotu = {
	"spec_shadowotu", NULL, "spec_spec1282a", NULL, "1985",
	"Shadow of the Unicorn (128K, 'rebuilt' version)\0", "NB: don't go back to ROM menu/loader", "Mikro-Gen", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_ADV, 0,
	SpectrumGetZipName, SpecShadowotuRomInfo, SpecShadowotuRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KPlus2Init, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Shanghai Warriors (128K)

static struct BurnRomInfo SpecShanghaiwrsRomDesc[] = {
	{ "Shanghai Warriors 128K (1989)(Players Software).tap", 130361, 0x15961f1d, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecShanghaiwrs, SpecShanghaiwrs, Spec128)
STD_ROM_FN(SpecShanghaiwrs)

struct BurnDriver BurnSpecShanghaiwrs = {
	"spec_shanghaiwrs", NULL, "spec_spec128", NULL, "1989",
	"Shanghai Warriors (128K)\0", NULL, "Players Software", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_SCRFIGHT, 0,
	SpectrumGetZipName, SpecShanghaiwrsRomInfo, SpecShanghaiwrsRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Shao Lin's Road (48K)

static struct BurnRomInfo SpecShaolinroadRomDesc[] = {
	{ "Shao Lin's Road 48K (1986)(The Edge).tap", 43831, 0x100a17ad, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecShaolinroad, SpecShaolinroad, Spectrum)
STD_ROM_FN(SpecShaolinroad)

struct BurnDriver BurnSpecShaolinroad = {
	"spec_shaolinroad", NULL, "spec_spectrum", NULL, "1986",
	"Shao Lin's Road (48K)\0", NULL, "The Edge", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_ACTION, 0,
	SpectrumGetZipName, SpecShaolinroadRomInfo, SpecShaolinroadRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Sharkey's Moll (128K)

static struct BurnRomInfo SpecSkmollRomDesc[] = {
	{ "Sharkey's Moll 128K (1991)(Zeppelin Games).tap", 44068, 0xc831bf3f, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecSkmoll, SpecSkmoll, Spec128)
STD_ROM_FN(SpecSkmoll)

struct BurnDriver BurnSpecSkmoll = {
	"spec_skmoll", NULL, "spec_spec128", NULL, "1991",
	"Sharkey's Moll (128K)\0", NULL, "Zeppelin Games", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_SHOOT, 0,
	SpectrumGetZipName, SpecSkmollRomInfo, SpecSkmollRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecIntf2DIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Sherlock (48K)

static struct BurnRomInfo SpecSherlockRomDesc[] = {
	{ "Sherlock 48K (1984)(Melbourne House).z80", 43891, 0x91d7e211, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecSherlock, SpecSherlock, Spectrum)
STD_ROM_FN(SpecSherlock)

struct BurnDriver BurnSpecSherlock = {
	"spec_sherlock", NULL, "spec_spectrum", NULL, "1984",
	"Sherlock (48K)\0", NULL, "Melbourne House", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_ADV, 0,
	SpectrumGetZipName, SpecSherlockRomInfo, SpecSherlockRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Shoot-Out (128K)

static struct BurnRomInfo SpecShootoutRomDesc[] = {
	{ "Shoot-Out 128K (1989)(Martech Games).tap", 41276, 0x731942be, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecShootout, SpecShootout, Spec128)
STD_ROM_FN(SpecShootout)

struct BurnDriver BurnSpecShootout = {
	"spec_shootout", NULL, "spec_spec128", NULL, "1989",
	"Shoot-Out (128K)\0", NULL, "Martech Games", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_SHOOT, 0,
	SpectrumGetZipName, SpecShootoutRomInfo, SpecShootoutRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Short Circuit (128K)

static struct BurnRomInfo SpecShortcirc128RomDesc[] = {
	{ "Short Circuit 128K (1987)(Ocean).tap", 91424, 0xd080565f, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecShortcirc128, SpecShortcirc128, Spec128)
STD_ROM_FN(SpecShortcirc128)

struct BurnDriver BurnSpecShortcirc128 = {
	"spec_shortcirc128", NULL, "spec_spec128", NULL, "1987",
	"Short Circuit (128K)\0", NULL, "Ocean", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_ACTION | GBF_ADV, 0,
	SpectrumGetZipName, SpecShortcirc128RomInfo, SpecShortcirc128RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Short Circuit - Part 1 (48K)

static struct BurnRomInfo SpecShortcirc48p1RomDesc[] = {
	{ "Short Circuit - Part 1 48K (1987)(Ocean).tap", 44994, 0x19008124, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecShortcirc48p1, SpecShortcirc48p1, Spectrum)
STD_ROM_FN(SpecShortcirc48p1)

struct BurnDriver BurnSpecShortcirc48p1 = {
	"spec_shortcirc48p1", "spec_shortcirc128", "spec_spectrum", NULL, "1987",
	"Short Circuit - Part 1 (48K)\0", NULL, "Ocean", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE, 1, HARDWARE_SPECTRUM, GBF_ACTION | GBF_ADV, 0,
	SpectrumGetZipName, SpecShortcirc48p1RomInfo, SpecShortcirc48p1RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Short Circuit - Part 2 (48K)

static struct BurnRomInfo SpecShortcirc48p2RomDesc[] = {
	{ "Short Circuit - Part 2 48K (1987)(Ocean).tap", 44962, 0x100f3549, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecShortcirc48p2, SpecShortcirc48p2, Spectrum)
STD_ROM_FN(SpecShortcirc48p2)

struct BurnDriver BurnSpecShortcirc48p2 = {
	"spec_shortcirc48p2", "spec_shortcirc128", "spec_spectrum", NULL, "1987",
	"Short Circuit - Part 2 (48K)\0", NULL, "Ocean", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE, 1, HARDWARE_SPECTRUM, GBF_ACTION, 0,
	SpectrumGetZipName, SpecShortcirc48p2RomInfo, SpecShortcirc48p2RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Show Jumping (48K)

static struct BurnRomInfo SpecShowjumpRomDesc[] = {
	{ "Show Jumping 48K (1986)(Alligata).tap", 48330, 0x3b71fc97, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecShowjump, SpecShowjump, Spectrum)
STD_ROM_FN(SpecShowjump)

struct BurnDriver BurnSpecShowjump = {
	"spec_showjump", NULL, "spec_spectrum", NULL, "1986",
	"Show Jumping (48K)\0", "Press 'Enter' to skip loading screen", "Alligata Software", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 8, HARDWARE_SPECTRUM, GBF_SPORTSMISC, 0,
	SpectrumGetZipName, SpecShowjumpRomInfo, SpecShowjumpRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Shuriken (48K)

static struct BurnRomInfo SpecShurikenRomDesc[] = {
	{ "Shuriken 48K (1989)(Xplosive).tap", 48301, 0xcee18b6e, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecShuriken, SpecShuriken, Spectrum)
STD_ROM_FN(SpecShuriken)

struct BurnDriver BurnSpecShuriken = {
	"spec_shuriken", NULL, "spec_spectrum", NULL, "1989",
	"Shuriken (48K)\0", "Set '3 - Sinclair' to use contoller, press 'SPACE' for missiles", "Xplosive", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecShurikenRomInfo, SpecShurikenRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecIntf2SwapDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Side Arms (48K)

static struct BurnRomInfo SpecSidearmsRomDesc[] = {
	{ "Side Arms 48K (1988)(Go!).z80", 47259, 0xcb9fcf76, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecSidearms, SpecSidearms, Spectrum)
STD_ROM_FN(SpecSidearms)

struct BurnDriver BurnSpecSidearms = {
	"spec_sidearms", NULL, "spec_spectrum", NULL, "1988",
	"Side Arms (48K)\0", NULL, "Go!", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_HORSHOOT, 0,
	SpectrumGetZipName, SpecSidearmsRomInfo, SpecSidearmsRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Silent Service (48K)

static struct BurnRomInfo SpecSilentserviceRomDesc[] = {
	{ "Silent Service 48K (1986)(U.S. Gold).z80", 40554, 0x1822026f, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecSilentservice, SpecSilentservice, Spectrum)
STD_ROM_FN(SpecSilentservice)

struct BurnDriver BurnSpecSilentservice = {
	"spec_silentservice", NULL, "spec_spectrum", NULL, "1986",
	"Silent Service (48K)\0", NULL, "U.S. Gold", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_SIM, 0,
	SpectrumGetZipName, SpecSilentserviceRomInfo, SpecSilentserviceRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Sinbad and the Golden Ship - Part 1 (48K)

static struct BurnRomInfo SpecSinbadgs1RomDesc[] = {
	{ "Sinbad and the Golden Ship Part1 48K (1986)(Mastervision).tap", 55928, 0x3d445332, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecSinbadgs1, SpecSinbadgs1, Spectrum)
STD_ROM_FN(SpecSinbadgs1)

struct BurnDriver BurnSpecSinbadgs1 = {
	"spec_sinbadgs1", NULL, "spec_spectrum", NULL, "1986",
	"Sinbad and the Golden Ship - Part 1 (48K)\0", NULL, "Mastervision", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_ADV, 0,
	SpectrumGetZipName, SpecSinbadgs1RomInfo, SpecSinbadgs1RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Sinbad and the Golden Ship - Part 2 (48K)

static struct BurnRomInfo SpecSinbadgs2RomDesc[] = {
	{ "Sinbad and the Golden Ship Part2 48K (1986)(Mastervision).tap", 48955, 0xff490432, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecSinbadgs2, SpecSinbadgs2, Spectrum)
STD_ROM_FN(SpecSinbadgs2)

struct BurnDriver BurnSpecSinbadgs2 = {
	"spec_sinbadgs2", "spec_sinbadgs1", "spec_spectrum", NULL, "1986",
	"Sinbad and the Golden Ship - Part 2 (48K)\0", NULL, "Mastervision", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE, 1, HARDWARE_SPECTRUM, GBF_ADV, 0,
	SpectrumGetZipName, SpecSinbadgs2RomInfo, SpecSinbadgs2RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Sir Fred (48K)

static struct BurnRomInfo SpecsirfredRomDesc[] = {
	{ "Sir Fred (1986)(Mikro-Gen).tap", 49152, 0x4a4f8a17, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specsirfred, Specsirfred, Spectrum)
STD_ROM_FN(Specsirfred)

struct BurnDriver BurnSpecsirfred = {
	"spec_sirfred", NULL, "spec_spectrum", NULL, "1986",
	"Sir Fred (48K)\0", NULL, "Mikro-Gen", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecsirfredRomInfo, SpecsirfredRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Sir Lancelot (48K)

static struct BurnRomInfo SpecSirlancelotRomDesc[] = {
	{ "Sir Lancelot 48K (1984)(Melbourne House).tap", 16929, 0xd3b14ed5, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecSirlancelot, SpecSirlancelot, Spectrum)
STD_ROM_FN(SpecSirlancelot)

struct BurnDriver BurnSpecSirlancelot = {
	"spec_sirlancelot", NULL, "spec_spectrum", NULL, "1984",
	"Sir Lancelot (48K)\0", NULL, "Melbourne House", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecSirlancelotRomInfo, SpecSirlancelotRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Sirwood - Intro (128K)

static struct BurnRomInfo SpecsirwoodintroRomDesc[] = {
	{ "Sirwood - Intro (1990)(Opera Soft).tap", 38955, 0x4a37ad11, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specsirwoodintro, Specsirwoodintro, Spec128)
STD_ROM_FN(Specsirwoodintro)

struct BurnDriver BurnSpecsirwoodintro = {
	"spec_sirwoodintro", NULL, "spec_spec128", NULL, "1990",
	"Sirwood - Intro (128K)\0", NULL, "Opera Soft", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_ACTION, 0,
	SpectrumGetZipName, SpecsirwoodintroRomInfo, SpecsirwoodintroRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecQAOPSpaceDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Sirwood - Part 1 (128K)

static struct BurnRomInfo Specsirwoodg1RomDesc[] = {
	{ "Sirwood - Level 1 (1990)(Opera Soft).tap", 46340, 0xd75b2f05, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specsirwoodg1, Specsirwoodg1, Spec128)
STD_ROM_FN(Specsirwoodg1)

struct BurnDriver BurnSpecsirwoodg1 = {
	"spec_sirwoodg1", "spec_sirwoodintro", "spec_spec128", NULL, "1990",
	"Sirwood - Part 1 (128K)\0", "G: quit current game - M: pause on/off", "Opera Soft", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE, 1, HARDWARE_SPECTRUM, GBF_ACTION, 0,
	SpectrumGetZipName, Specsirwoodg1RomInfo, Specsirwoodg1RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecQAOPSpaceDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Sirwood - Part 2 (128K)

static struct BurnRomInfo Specsirwoodg2RomDesc[] = {
	{ "Sirwood - Level 2 (1990)(Opera Soft).tap", 47993, 0x8e0f2b4d, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specsirwoodg2, Specsirwoodg2, Spec128)
STD_ROM_FN(Specsirwoodg2)

struct BurnDriver BurnSpecsirwoodg2 = {
	"spec_sirwoodg2", "spec_sirwoodintro", "spec_spec128", NULL, "1990",
	"Sirwood - Part 2 (128K)\0", "G: quit current game - M: pause on/off", "Opera Soft", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE, 1, HARDWARE_SPECTRUM, GBF_ACTION, 0,
	SpectrumGetZipName, Specsirwoodg2RomInfo, Specsirwoodg2RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecQAOPSpaceDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Sirwood - Part 3 (128K)

static struct BurnRomInfo Specsirwoodg3RomDesc[] = {
	{ "Sirwood - Level 3 (1990)(Opera Soft).tap", 45506, 0xb58dc313, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specsirwoodg3, Specsirwoodg3, Spec128)
STD_ROM_FN(Specsirwoodg3)

struct BurnDriver BurnSpecsirwoodg3 = {
	"spec_sirwoodg3", "spec_sirwoodintro", "spec_spec128", NULL, "1990",
	"Sirwood - Part 3 (128K)\0", "G: quit current game - M: pause on/off", "Opera Soft", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE, 1, HARDWARE_SPECTRUM, GBF_ACTION, 0,
	SpectrumGetZipName, Specsirwoodg3RomInfo, Specsirwoodg3RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecQAOPSpaceDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Skate Crazy (128K)

static struct BurnRomInfo SpecSkatecrazyRomDesc[] = {
	{ "Skate Crazy 128K (1988)(Gremlin Graphics).tap", 127196, 0xefd46948, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecSkatecrazy, SpecSkatecrazy, Spec128)
STD_ROM_FN(SpecSkatecrazy)

struct BurnDriver BurnSpecSkatecrazy = {
	"spec_skatecrazy", NULL, "spec_spec128", NULL, "1988",
	"Skate Crazy (128K)\0", NULL, "Gremlin Graphics", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 2, HARDWARE_SPECTRUM, GBF_ACTION | GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecSkatecrazyRomInfo, SpecSkatecrazyRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Skate Crazy - Part 1 (48K-128K)

static struct BurnRomInfo SpecSkatecrazy1RomDesc[] = {
	{ "Skate Crazy - Part 1 (1988)(Gremlin Graphics).z80", 50655, 0x16add75c, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecSkatecrazy1, SpecSkatecrazy1, Spec128)
STD_ROM_FN(SpecSkatecrazy1)

struct BurnDriver BurnSpecSkatecrazy1 = {
	"spec_skatecrazy1", "spec_skatecrazy", "spec_spec128", NULL, "1988",
	"Skate Crazy - Part 1 (48K-128K)\0", "Z80 memory snapshot version", "Gremlin Graphics", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE, 2, HARDWARE_SPECTRUM, GBF_ACTION, 0,
	SpectrumGetZipName, SpecSkatecrazy1RomInfo, SpecSkatecrazy1RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Skate Crazy - Part 2 (48K)

static struct BurnRomInfo SpecSkatecrazy2RomDesc[] = {
	{ "Skate Crazy - Part 2 (1988)(Gremlin Graphics).z80", 35524, 0x82d5426e, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecSkatecrazy2, SpecSkatecrazy2, Spectrum)
STD_ROM_FN(SpecSkatecrazy2)

struct BurnDriver BurnSpecSkatecrazy2 = {
	"spec_skatecrazy2", "spec_skatecrazy", "spec_spectrum", NULL, "1988",
	"Skate Crazy - Part 2 (48K)\0", "Z80 memory snapshot version", "Gremlin Graphics", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE, 2, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecSkatecrazy2RomInfo, SpecSkatecrazy2RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Slaine (48K)

static struct BurnRomInfo SpecSlaineRomDesc[] = {
	{ "Slaine 48K (1987)(Martech Games).tap", 54524, 0x581c3541, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecSlaine, SpecSlaine, Spectrum)
STD_ROM_FN(SpecSlaine)

struct BurnDriver BurnSpecSlaine = {
	"spec_slaine", NULL, "spec_spectrum", NULL, "1987",
	"Slaine (48K)\0", NULL, "Martech Games", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_ADV, 0,
	SpectrumGetZipName, SpecSlaineRomInfo, SpecSlaineRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Snakes and Hazards (128K)

static struct BurnRomInfo SpecSnakeshazardsRomDesc[] = {
	{ "Snakes and Hazards 128K (1987)(Gremlin Graphics).tap", 51210, 0x6ff11e22, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecSnakeshazards, SpecSnakeshazards, Spec128)
STD_ROM_FN(SpecSnakeshazards)

struct BurnDriver BurnSpecSnakeshazards = {
	"spec_snakeshazards", NULL, "spec_spec128", NULL, "1987",
	"Snakes and Hazards (128K)\0", NULL, "Gremlin Graphics", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 4, HARDWARE_SPECTRUM, GBF_BOARD, 0,
	SpectrumGetZipName, SpecSnakeshazardsRomInfo, SpecSnakeshazardsRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Soldier of Light (48K)

static struct BurnRomInfo SpecSoldoflightRomDesc[] = {
	{ "Soldier of Light 48K (1988)(ACE).tap", 35934, 0xf417f933, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecSoldoflight, SpecSoldoflight, Spectrum)
STD_ROM_FN(SpecSoldoflight)

struct BurnDriver BurnSpecSoldoflight = {
	"spec_soldoflight", NULL, "spec_spectrum", NULL, "1988",
	"Soldier of Light (48K)\0", NULL, "ACE", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_RUNGUN | GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecSoldoflightRomInfo, SpecSoldoflightRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Sol Negro - Special Edition - Part 1 (Spanish) (48K)

static struct BurnRomInfo Specsolnegro1RomDesc[] = {
	{ "Sol Negro - Special Edition - Part 1 (1989)(Opera Soft).z80", 42525, 0x27b7e34f, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specsolnegro1, Specsolnegro1, Spectrum)
STD_ROM_FN(Specsolnegro1)

struct BurnDriver BurnSpecsolnegro1 = {
	"spec_solnegro1", NULL, "spec_spectrum", NULL, "1989",
	"Sol Negro - Special Edition - Part 1 (Spanish) (48K)\0", "AY Sound supported", "Opera Soft", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_RUNGUN, 0,
	SpectrumGetZipName, Specsolnegro1RomInfo, Specsolnegro1RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecQAOPSpaceDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Sol Negro - Special Edition - Part 2 (Spanish) (48K)

static struct BurnRomInfo Specsolnegro2RomDesc[] = {
	{ "Sol Negro - Special Edition - Part 2 (1989)(Opera Soft).z80", 37075, 0xaf912284, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specsolnegro2, Specsolnegro2, Spectrum)
STD_ROM_FN(Specsolnegro2)

struct BurnDriver BurnSpecsolnegro2 = {
	"spec_solnegro2", "spec_solnegro1", "spec_spectrum", NULL, "1989",
	"Sol Negro - Special Edition - Part 2 (Spanish) (48K)\0", "AY Sound supported", "Opera Soft", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE, 1, HARDWARE_SPECTRUM, GBF_RUNGUN, 0,
	SpectrumGetZipName, Specsolnegro2RomInfo, Specsolnegro2RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecQAOPSpaceDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Solomon's Key (48K)

static struct BurnRomInfo SpecsolomonRomDesc[] = {
	{ "Solomon's Key 48K (1987)(U.S. Gold).tap", 47085, 0xf8e955e0, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specsolomon, Specsolomon, Spectrum)
STD_ROM_FN(Specsolomon)

struct BurnDriver BurnSpecsolomon = {
	"spec_solomon", NULL, "spec_spectrum", NULL, "1987",
	"Solomon's Key (48K)\0", NULL, "U.S. Gold", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecsolomonRomInfo, SpecsolomonRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Sonic Boom (128K)

static struct BurnRomInfo SpecSonicboomRomDesc[] = {
	{ "Sonic Boom 128K (1990)(Activision).tap", 89195, 0x8fff430d, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecSonicboom, SpecSonicboom, Spec128)
STD_ROM_FN(SpecSonicboom)

struct BurnDriver BurnSpecSonicboom = {
	"spec_sonicboom", NULL, "spec_spec128", NULL, "1990",
	"Sonic Boom (128K) (HB)\0", NULL, "Activision", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_VERSHOOT, 0,
	SpectrumGetZipName, SpecSonicboomRomInfo, SpecSonicboomRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Sooty & Sweep (48K)

static struct BurnRomInfo SpecSootysweepRomDesc[] = {
	{ "Sooty & Sweep 48K (1989)(Alternative).tap", 40695, 0x88fe089c, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecSootysweep, SpecSootysweep, Spectrum)
STD_ROM_FN(SpecSootysweep)

struct BurnDriver BurnSpecSootysweep = {
	"spec_sootysweep", NULL, "spec_spectrum", NULL, "1990",
	"Sooty & Sweep (48K)\0", NULL, "Alternative Software", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 2, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecSootysweepRomInfo, SpecSootysweepRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Sorcery Plus (48K)

static struct BurnRomInfo SpecSorceryplusRomDesc[] = {
	{ "Sorcery Plus 48K (1986)(Virgin Games Ltd.).tap", 45930, 0xfc628bd2, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecSorceryplus, SpecSorceryplus, Spectrum)
STD_ROM_FN(SpecSorceryplus)

struct BurnDriver BurnSpecSorceryplus = {
	"spec_sorceryplus", NULL, "spec_spectrum", NULL, "1986",
	"Sorcery Plus (48K)\0", "Q: left - A: right - 0: jump", "Virgin Games", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecSorceryplusRomInfo, SpecSorceryplusRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Soul of a Robot - Nonterraqueous 2 (48K)

static struct BurnRomInfo SpecSoulrobotRomDesc[] = {
	{ "Soul of a Robot - Nonterraqueous 2 48K (1985)(Mastertronic).tap", 46080, 0xd158108d, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecSoulrobot, SpecSoulrobot, Spectrum)
STD_ROM_FN(SpecSoulrobot)

struct BurnDriver BurnSpecSoulrobot = {
	"spec_soulrobot", NULL, "spec_spectrum", NULL, "1985",
	"Soul of a Robot - Nonterraqueous 2 (48K)\0", NULL, "Mastertronic", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecSoulrobotRomInfo, SpecSoulrobotRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Souls of Darkon (48K)

static struct BurnRomInfo SpecSoulsdarkonRomDesc[] = {
	{ "Souls of Darkon (1985)(Taskset).tap", 48819, 0x7ad0d630, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecSoulsdarkon, SpecSoulsdarkon, Spectrum)
STD_ROM_FN(SpecSoulsdarkon)

struct BurnDriver BurnSpecSoulsdarkon = {
	"spec_soulsdarkon", NULL, "spec_spectrum", NULL, "1985",
	"Souls of Darkon (48K)\0", NULL, "Taskset", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_ADV, 0,
	SpectrumGetZipName, SpecSoulsdarkonRomInfo, SpecSoulsdarkonRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Space Gun (128K)

static struct BurnRomInfo SpecSpacegunRomDesc[] = {
	{ "Space Gun 128K (1992)(Ocean).tap", 155343, 0xd467691f, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecSpacegun, SpecSpacegun, Spec128)
STD_ROM_FN(SpecSpacegun)

struct BurnDriver BurnSpecSpacegun = {
	"spec_spacegun", NULL, "spec_spec128", NULL, "1992",
	"Space Gun (128K)\0", "Press 'K' during titles to use controller", "Ocean", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_SHOOT, 0,
	SpectrumGetZipName, SpecSpacegunRomInfo, SpecSpacegunRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Space Harrier (48K)

static struct BurnRomInfo SpecsharrierRomDesc[] = {
	{ "Space Harrier (1986)(Elite Systems).z80", 0x0b439, 0xd33b7f51, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specsharrier, Specsharrier, Spectrum)
STD_ROM_FN(Specsharrier)

struct BurnDriver BurnSpecsharrier = {
	"spec_sharrier", NULL, "spec_spectrum", NULL, "1986",
	"Space Harrier (48K)\0", NULL, "Elite Systems", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_SHOOT, 0,
	SpectrumGetZipName, SpecsharrierRomInfo, SpecsharrierRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Space Harrier II (128K)

static struct BurnRomInfo Specsharrir2RomDesc[] = {
	{ "Space Harrier II 128K (1990)(Grandslam).tap", 149856, 0x69d5d562, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specsharrir2, Specsharrir2, Spec128)
STD_ROM_FN(Specsharrir2)

struct BurnDriver BurnSpecsharrir2 = {
	"spec_sharrir2", NULL, "spec_spec128", NULL, "1990",
	"Space Harrier II (128K)\0", NULL, "Grandslam Entertainments", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_SHOOT, 0,
	SpectrumGetZipName, Specsharrir2RomInfo, Specsharrir2RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Space Warrior (48K)

static struct BurnRomInfo SpecSpacewarrRomDesc[] = {
	{ "Space Warrior 48K (1985)(Automata UK).tap", 26355, 0xf1bbd980, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecSpacewarr, SpecSpacewarr, Spectrum)
STD_ROM_FN(SpecSpacewarr)

struct BurnDriver BurnSpecSpacewarr = {
	"spec_spacewarr", NULL, "spec_spectrum", NULL, "1985",
	"Space Warrior (48K)\0", "Press 'J' to use controller", "Automata UK", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_SHOOT, 0,
	SpectrumGetZipName, SpecSpacewarrRomInfo, SpecSpacewarrRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Special Delivery (48K)

static struct BurnRomInfo SpecSpdeliveryRomDesc[] = {
	{ "Special Delivery 48K (1984)(Creative Sparks).tap", 27053, 0x83bbf568, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecSpdelivery, SpecSpdelivery, Spectrum)
STD_ROM_FN(SpecSpdelivery)

struct BurnDriver BurnSpecSpdelivery = {
	"spec_spdelivery", NULL, "spec_spectrum", NULL, "1984",
	"Special Delivery (48K)\0", NULL, "Creative Sparks", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_ACTION, 0,
	SpectrumGetZipName, SpecSpdeliveryRomInfo, SpecSpdeliveryRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Spectipede (16K)

static struct BurnRomInfo SpecSpectipedeRomDesc[] = {
	{ "Spectipede 16K (1983)(Mastertronic).tap", 16219, 0xda23e9dd, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecSpectipede, SpecSpectipede, Spectrum)
STD_ROM_FN(SpecSpectipede)

struct BurnDriver BurnSpecSpectipede = {
	"spec_spectipede", NULL, "spec_spectrum", NULL, "1983",
	"Spectipede (16K)\0", "Press 'P' to start a game session", "Mastertronic", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 2, HARDWARE_SPECTRUM, GBF_SHOOT, 0,
	SpectrumGetZipName, SpecSpectipedeRomInfo, SpecSpectipedeRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Spectrum Cross (16K)

static struct BurnRomInfo SpecSpecrossRomDesc[] = {
	{ "Spectrum Cross 16K (1983)(Your Computer).tap", 8210, 0xf16574c6, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecSpecross, SpecSpecross, Spectrum)
STD_ROM_FN(SpecSpecross)

struct BurnDriver BurnSpecSpecross = {
	"spec_specross", NULL, "spec_spectrum", NULL, "1983",
	"Spectrum Cross (16K)\0", "1: up - 9: left - 0: right", "Your Computer", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_ACTION, 0,
	SpectrumGetZipName, SpecSpecrossRomInfo, SpecSpecrossRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Spider's Web (16K)

static struct BurnRomInfo SpecSpidywebRomDesc[] = {
	{ "Spider's Web 16K (1983)(Thor Computer Software).tap", 15683, 0x86a0acc3, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecSpidyweb, SpecSpidyweb, Spectrum)
STD_ROM_FN(SpecSpidyweb)

struct BurnDriver BurnSpecSpidyweb = {
	"spec_spidyweb", NULL, "spec_spectrum", NULL, "1983",
	"Spider's Web (16K)\0", "6: reverse - 8: turn left - 9: turn right", "Thor Computer Software", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_ACTION, 0,
	SpectrumGetZipName, SpecSpidywebRomInfo, SpecSpidywebRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Spirits (Spanish) (48K)

static struct BurnRomInfo SpecSpiritsRomDesc[] = {
	{ "Spirits ES 48K (1987)(Topo Soft).tap", 49605, 0x901fbdaa, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecSpirits, SpecSpirits, Spectrum)
STD_ROM_FN(SpecSpirits)

struct BurnDriver BurnSpecSpirits = {
	"spec_spirits", NULL, "spec_spectrum", NULL, "1987",
	"Spirits (Spanish) (48K)\0", NULL, "Topo Soft", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_PLATFORM | GBF_ADV, 0,
	SpectrumGetZipName, SpecSpiritsRomInfo, SpecSpiritsRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecQAOPMDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Spirits - 30th anniversary (Spanish) (48K)

static struct BurnRomInfo SpecSpirits30thRomDesc[] = {
	{ "Spirits ES 30th anniversary 48K (1987-2017)(Topo Soft).tap", 57855, 0x521f32f4, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecSpirits30th, SpecSpirits30th, Spectrum)
STD_ROM_FN(SpecSpirits30th)

struct BurnDriver BurnSpecSpirits30th = {
	"spec_spirits30th", "spec_spirits", "spec_spectrum", NULL, "2017",
	"Spirits - 30th Anniversary (Spanish) (48K)\0", NULL, "Topo Soft", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE, 1, HARDWARE_SPECTRUM, GBF_PLATFORM | GBF_ADV, 0,
	SpectrumGetZipName, SpecSpirits30thRomInfo, SpecSpirits30thRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecQAOPMDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Splat! (48K)

static struct BurnRomInfo SpecSplatRomDesc[] = {
	{ "Splat! 48K (1983)(Incentive Software).tap", 49519, 0xec4bef12, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecSplat, SpecSplat, Spectrum)
STD_ROM_FN(SpecSplat)

struct BurnDriver BurnSpecSplat = {
	"spec_splat", NULL, "spec_spectrum", NULL, "1983",
	"Splat! (48K)\0", "Set '1 Keyboard Cursor Keys' to use controller", "Incentive Software", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_MAZE, 0,
	SpectrumGetZipName, SpecSplatRomInfo, SpecSplatRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecCursorKeysDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Spooked (48K)

static struct BurnRomInfo SpecSpookedRomDesc[] = {
	{ "Spooked 48K (1989)(Players Premier).tap", 40963, 0xb3b2d241, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecSpooked, SpecSpooked, Spectrum)
STD_ROM_FN(SpecSpooked)

struct BurnDriver BurnSpecSpooked = {
	"spec_spooked", NULL, "spec_spectrum", NULL, "1989",
	"Spooked (48K)\0", NULL, "Players Premier", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecSpookedRomInfo, SpecSpookedRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Spooky Castle (48K)

static struct BurnRomInfo SpecSpookycastleRomDesc[] = {
	{ "Spooky Castle 48K (1990)(Atlantis Software).z80", 38671, 0xfad004e5, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecSpookycastle, SpecSpookycastle, Spectrum)
STD_ROM_FN(SpecSpookycastle)

struct BurnDriver BurnSpecSpookycastle = {
	"spec_spookycastle", NULL, "spec_spectrum", NULL, "1990",
	"Spooky Castle (48K)\0", NULL, "Atlantis Software", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecSpookycastleRomInfo, SpecSpookycastleRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Stack Up (48K)

static struct BurnRomInfo SpecStackupRomDesc[] = {
	{ "Stack Up 48K (1991)(Zeppelin Games).tap", 43582, 0xf07a8de4, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecStackup, SpecStackup, Spectrum)
STD_ROM_FN(SpecStackup)

struct BurnDriver BurnSpecStackup = {
	"spec_stackup", NULL, "spec_spectrum", NULL, "1991",
	"Stack Up (48K)\0", NULL, "Zeppelin Games", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_PUZZLE, 0,
	SpectrumGetZipName, SpecStackupRomInfo, SpecStackupRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Star Bowls (48K)

static struct BurnRomInfo SpecStarbowlsRomDesc[] = {
	{ "Star Bowls 48K (1991)(Zigurat Software).tap", 47077, 0x10eb1995, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecStarbowls, SpecStarbowls, Spectrum)
STD_ROM_FN(SpecStarbowls)

struct BurnDriver BurnSpecStarbowls = {
	"spec_starbowls", NULL, "spec_spectrum", NULL, "1991",
	"Star Bowls (48K)\0", NULL, "Zigurat Software", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_RUNGUN, 0,
	SpectrumGetZipName, SpecStarbowlsRomInfo, SpecStarbowlsRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Stardust (48K)

static struct BurnRomInfo SpecStardustRomDesc[] = {
    { "Stardust 48K (1987)(Topo Soft).tap", 73651, 0xe6a65d7b, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecStardust, SpecStardust, Spectrum)
STD_ROM_FN(SpecStardust)

struct BurnDriver BurnSpecStardust = {
    "spec_stardust", NULL, "spec_spectrum", NULL, "1987",
    "Stardust (48K)\0", NULL, "Topo Soft", "ZX Spectrum",
    NULL, NULL, NULL, NULL,
    BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_VERSHOOT, 0,
    SpectrumGetZipName, SpecStardustRomInfo, SpecStardustRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecQAOPMDIPInfo,
    SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
    &SpecRecalc, 0x10, 288, 224, 4, 3
};

// Starglider 2 - The Egrons Strike Back (128K)

static struct BurnRomInfo SpecStarglider2RomDesc[] = {
	{ "Starglider 2 - The Egrons Strike Back 128K (1989)(Rainbird).tap", 95040, 0x667e1ddf, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecStarglider2, SpecStarglider2, Spec128)
STD_ROM_FN(SpecStarglider2)

struct BurnDriver BurnSpecStarglider2 = {
	"spec_starglider2", NULL, "spec_spec128", NULL, "1989",
	"Starglider 2 - The Egrons Strike Back (128K)\0", NULL, "Rainbird", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_SHOOT | GBF_SIM, 0,
	SpectrumGetZipName, SpecStarglider2RomInfo, SpecStarglider2RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Star Wars (48K)

static struct BurnRomInfo SpecStarwarsRomDesc[] = {
	{ "Star Wars 48K (1987)(Domark).tap", 0xa323, 0x14f2595f, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecStarwars, SpecStarwars, Spectrum)
STD_ROM_FN(SpecStarwars)

struct BurnDriver BurnSpecStarwars = {
	"spec_starwars", NULL, "spec_spectrum", NULL, "1987",
	"Star Wars (48K)\0", NULL, "Domark", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_SHOOT, 0,
	SpectrumGetZipName, SpecStarwarsRomInfo, SpecStarwarsRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Star Wars Droids (128K)

static struct BurnRomInfo SpecSwdroidsRomDesc[] = {
	{ "Star Wars Droids 128K (1988)(Mastertronic Added Dimension).tap", 40659, 0x6e6acfe4, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecSwdroids, SpecSwdroids, Spec128)
STD_ROM_FN(SpecSwdroids)

struct BurnDriver BurnSpecSwdroids = {
	"spec_swdroids", NULL, "spec_spec128", NULL, "1988",
	"Star Wars Droids (128K)\0", NULL, "Mastertronic Added Dimension", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_ACTION | GBF_ADV, 0,
	SpectrumGetZipName, SpecSwdroidsRomInfo, SpecSwdroidsRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Steel Eagle (128K)

static struct BurnRomInfo SpecSteeleagleRomDesc[] = {
	{ "Steel Eagle 128K (1990)(Players Premier).tap", 52632, 0x0d55bee6, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecSteeleagle, SpecSteeleagle, Spec128)
STD_ROM_FN(SpecSteeleagle)

struct BurnDriver BurnSpecSteeleagle = {
	"spec_steeleagle", NULL, "spec_spec128", NULL, "1990",
	"Steel Eagle (128K)\0", NULL, "Players Premier", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_HORSHOOT, 0,
	SpectrumGetZipName, SpecSteeleagleRomInfo, SpecSteeleagleRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Stifflip & Co. - Part 1 (English) (128K)

static struct BurnRomInfo SpecStifflip1enRomDesc[] = {
	{ "Stifflip and Co. - Part 1 EN 128K (1987)(Palace Software).tap", 56817, 0xc5a6eb1c, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecStifflip1en, SpecStifflip1en, Spec128)
STD_ROM_FN(SpecStifflip1en)

struct BurnDriver BurnSpecStifflip1en = {
	"spec_stifflip1en", NULL, "spec_spec128", NULL, "1987",
	"Stifflip & Co. - Part 1 (English) (128K)\0", NULL, "Palace Software", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_ADV, 0,
	SpectrumGetZipName, SpecStifflip1enRomInfo, SpecStifflip1enRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Stifflip & Co. - Part 1 (Spanish) (128K)

static struct BurnRomInfo SpecStifflip1esRomDesc[] = {
	{ "Stifflip and Co. - Part 1 ES 128K (1987)(Palace Software).tap", 56790, 0x53bc50dd, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecStifflip1es, SpecStifflip1es, Spec128)
STD_ROM_FN(SpecStifflip1es)

struct BurnDriver BurnSpecStifflip1es = {
	"spec_stifflip1es", "spec_stifflip1en", "spec_spec128", NULL, "1987",
	"Stifflip & Co. - Part 1 (Spanish) (128K)\0", NULL, "Palace Software", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE, 1, HARDWARE_SPECTRUM, GBF_ADV, 0,
	SpectrumGetZipName, SpecStifflip1esRomInfo, SpecStifflip1esRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Stifflip & Co. - Part 2 epilogue (English) (128K)

static struct BurnRomInfo SpecStifflip2enRomDesc[] = {
	{ "Stifflip and Co. - Part 2 epilogue EN 128K (1987)(Palace Software).tap", 56817, 0xf5798ea4, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecStifflip2en, SpecStifflip2en, Spec128)
STD_ROM_FN(SpecStifflip2en)

struct BurnDriver BurnSpecStifflip2en = {
	"spec_stifflip2en", "spec_stifflip1en", "spec_spec128", NULL, "1987",
	"Stifflip & Co. - Part 2 epilogue (English) (128K)\0", NULL, "Palace Software", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE, 1, HARDWARE_SPECTRUM, GBF_ADV, 0,
	SpectrumGetZipName, SpecStifflip2enRomInfo, SpecStifflip2enRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Stifflip & Co. - Part 2 epilogue (Spanish) (128K)

static struct BurnRomInfo SpecStifllip2esRomDesc[] = {
	{ "Stifflip and Co. - Part 2 epilogue ES 128K (1987)(Palace Software).tap", 56790, 0x15936e89, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecStifllip2es, SpecStifllip2es, Spec128)
STD_ROM_FN(SpecStifllip2es)

struct BurnDriver BurnSpecStifllip2es = {
	"spec_stifllip2es", "spec_stifflip1en", "spec_spec128", NULL, "1987",
	"Stifflip & Co. - Part 2 epilogue (Spanish) (128K)\0", NULL, "Palace Software", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE, 1, HARDWARE_SPECTRUM, GBF_ADV, 0,
	SpectrumGetZipName, SpecStifllip2esRomInfo, SpecStifllip2esRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Stomp (48K)

static struct BurnRomInfo SpecStompRomDesc[] = {
	{ "Stomp 48K (1985)(DK'Tronics).tap", 9669, 0x52926196, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecStomp, SpecStomp, Spectrum)
STD_ROM_FN(SpecStomp)

struct BurnDriver BurnSpecStomp = {
	"spec_stomp", NULL, "spec_spectrum", NULL, "1985",
	"Stomp (48K)\0", NULL, "DK'Tronics", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_ACTION, 0,
	SpectrumGetZipName, SpecStompRomInfo, SpecStompRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Stonkers (48K)

static struct BurnRomInfo SpecStonkersRomDesc[] = {
	{ "Stonkers 48K (1983)(Imagine).tap", 47132, 0x8e2bfa6f, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecStonkers, SpecStonkers, Spectrum)
STD_ROM_FN(SpecStonkers)

struct BurnDriver BurnSpecStonkers = {
	"spec_stonkers", NULL, "spec_spectrum", NULL, "1983",
	"Stonkers (48K)\0", NULL, "Imagine Software", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_STRATEGY, 0,
	SpectrumGetZipName, SpecStonkersRomInfo, SpecStonkersRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Street Hawk (48K)

static struct BurnRomInfo SpecStreethawkRomDesc[] = {
	{ "Street Hawk 48K (1986)(Ocean).tap", 47745, 0x37c623ad, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecStreethawk, SpecStreethawk, Spectrum)
STD_ROM_FN(SpecStreethawk)

struct BurnDriver BurnSpecStreethawk = {
	"spec_streethawk", NULL, "spec_spectrum", NULL, "1986",
	"Street Hawk (48K)\0", NULL, "Ocean", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_VERSHOOT, 0,
	SpectrumGetZipName, SpecStreethawkRomInfo, SpecStreethawkRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Street Sports Basketball (48K)

static struct BurnRomInfo SpecStsportbbRomDesc[] = {
	{ "Street Sports Basketball 48K (1988)(U.S. Gold).tap", 109322, 0x995f6a3a, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecStsportbb, SpecStsportbb, Spectrum)
STD_ROM_FN(SpecStsportbb)

struct BurnDriver BurnSpecStsportbb = {
	"spec_stsportbb", NULL, "spec_spectrum", NULL, "1988",
	"Street Sports Basketball (48K)\0", NULL, "U.S. Gold", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 2, HARDWARE_SPECTRUM, GBF_SPORTSMISC, 0,
	SpectrumGetZipName, SpecStsportbbRomInfo, SpecStsportbbRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Strontium Dog - The Killing (48K)

static struct BurnRomInfo SpecStrontiumdogRomDesc[] = {
	{ "Strontium Dog - The Killing 48K (1984)(Quicksilva).tap", 45575, 0xa4e892e9, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecStrontiumdog, SpecStrontiumdog, Spectrum)
STD_ROM_FN(SpecStrontiumdog)

struct BurnDriver BurnSpecStrontiumdog = {
	"spec_strontiumdog", NULL, "spec_spectrum", NULL, "1984",
	"Strontium Dog - The Killing (48K)\0", "Press '2' to use controller", "Quicksilva", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_RUNGUN | GBF_MAZE, 0,
	SpectrumGetZipName, SpecStrontiumdogRomInfo, SpecStrontiumdogRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Stryker - In the Crypts of Trogan (128K)

static struct BurnRomInfo SpecStrykercryptsoftroganRomDesc[] = {
	{ "Stryker - In the Crypts of Trogan (1992)(Codemasters).tap", 90060, 0xd24f539d, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecStrykercryptsoftrogan, SpecStrykercryptsoftrogan, Spec128)
STD_ROM_FN(SpecStrykercryptsoftrogan)

struct BurnDriver BurnSpecStrykercryptsoftrogan = {
	"spec_strykercryptsoftrogan", NULL, "spec_spec128", NULL, "1992",
	"Stryker - In the Crypts of Trogan (128K)\0", NULL, "Codemasters", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecStrykercryptsoftroganRomInfo, SpecStrykercryptsoftroganRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Stunt Man Seymour (128K)

static struct BurnRomInfo SpecStuntmanseymourRomDesc[] = {
	{ "Stunt Man Seymour (1992)(Codemasters).tap", 51574, 0x098c5ba8, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecStuntmanseymour, SpecStuntmanseymour, Spec128)
STD_ROM_FN(SpecStuntmanseymour)

struct BurnDriver BurnSpecStuntmanseymour = {
	"spec_stuntmanseymour", NULL, "spec_spec128", NULL, "1992",
	"Stunt Man Seymour (128K)\0", NULL, "Codemasters", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecStuntmanseymourRomInfo, SpecStuntmanseymourRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Sub Chase (16K)

static struct BurnRomInfo SpecSubchaseRomDesc[] = {
	{ "Sub Chase 16K (1983)(Gem Software).z80", 7701, 0x211cd8c0, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecSubchase, SpecSubchase, Spectrum)
STD_ROM_FN(SpecSubchase)

struct BurnDriver BurnSpecSubchase = {
	"spec_subchase", NULL, "spec_spectrum", NULL, "1983",
	"Sub Chase (16K)\0", "Q: left - P: right - keys from 1 to 9: use depth charges", "Gem Software", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_ACTION, 0,
	SpectrumGetZipName, SpecSubchaseRomInfo, SpecSubchaseRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Submariner (48K)

static struct BurnRomInfo SpecSubmarinerRomDesc[] = {
	{ "Submariner 48K (1988)(Atlantis).tap", 37754, 0xbdaf3289, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecSubmariner, SpecSubmariner, Spectrum)
STD_ROM_FN(SpecSubmariner)

struct BurnDriver BurnSpecSubmariner = {
	"spec_submariner", NULL, "spec_spectrum", NULL, "1988",
	"Submariner (48K)\0", NULL, "Atlantis Software", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_ACTION, 0,
	SpectrumGetZipName, SpecSubmarinerRomInfo, SpecSubmarinerRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Subterranean Nightmare (48K)

static struct BurnRomInfo SpecSubternightRomDesc[] = {
	{ "Subterranean Nightmare 48K (1986)(Americana Software).tap", 48606, 0xb8de9491, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecSubternight, SpecSubternight, Spectrum)
STD_ROM_FN(SpecSubternight)

struct BurnDriver BurnSpecSubternight = {
	"spec_subternight", NULL, "spec_spectrum", NULL, "1986",
	"Subterranean Nightmare (48K)\0", "Press '1' or '2' to start a game session", "Americana Software", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecSubternightRomInfo, SpecSubternightRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecIntf2DIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Super Cars (128K)

static struct BurnRomInfo SpecSupercarsRomDesc[] = {
	{ "Super Cars 128K (1990)(Gremlin Graphics).z80", 88269, 0x5a2d86ca, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecSupercars, SpecSupercars, Spec128)
STD_ROM_FN(SpecSupercars)

struct BurnDriver BurnSpecSupercars = {
	"spec_supercars", NULL, "spec_spec128", NULL, "1990",
	"Super Cars (128K)\0", NULL, "Gremlin Graphics", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_RACING, 0,
	SpectrumGetZipName, SpecSupercarsRomInfo, SpecSupercarsRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Super Cycle (48K)

static struct BurnRomInfo SpecSupercycleRomDesc[] = {
	{ "Super Cycle 48K (1987)(U.S. Gold).z80", 32852, 0x7aeedc00, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecSupercycle, SpecSupercycle, Spectrum)
STD_ROM_FN(SpecSupercycle)

struct BurnDriver BurnSpecSupercycle = {
	"spec_supercycle", NULL, "spec_spectrum", NULL, "1987",
	"Super Cycle (48K)\0", NULL, "U.S. Gold", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_RACING, 0,
	SpectrumGetZipName, SpecSupercycleRomInfo, SpecSupercycleRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Superman - The Man of Steel (48K)

static struct BurnRomInfo SpecSupermanRomDesc[] = {
	{ "Superman - The Man of Steel 48K (1989)(Tynesoft).tap", 300217, 0x578fc970, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecSuperman, SpecSuperman, Spectrum)
STD_ROM_FN(SpecSuperman)

struct BurnDriver BurnSpecSuperman = {
	"spec_superman", NULL, "spec_spectrum", NULL, "1989",
	"Superman - The Man of Steel (48K)\0", "Use '6', '7' and 'ENTER' to select options", "Tynesoft", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_SHOOT, 0,
	SpectrumGetZipName, SpecSupermanRomInfo, SpecSupermanRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Super Space Invaders (128K)

static struct BurnRomInfo SpecSsinvadersRomDesc[] = {
	{ "Super Space Invaders 128K (1991)(Domark).z80", 104637, 0x2a36ad52, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecSsinvaders, SpecSsinvaders, Spec128)
STD_ROM_FN(SpecSsinvaders)

struct BurnDriver BurnSpecSsinvaders = {
	"spec_ssinvaders", NULL, "spec_spec128", NULL, "1991",
	"Super Space Invaders (128K)\0", "In main menu: 'A' to move, 'Q' to select", "Domark", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 2, HARDWARE_SPECTRUM, GBF_SHOOT, 0,
	SpectrumGetZipName, SpecSsinvadersRomInfo, SpecSsinvadersRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// SuperTed - The Search for Spotty (48K)

static struct BurnRomInfo SpecSupertedRomDesc[] = {
	{ "SuperTed - The Search for Spotty 48K (1990)(Alternative).tap", 47934, 0xc857de12, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecSuperted, SpecSuperted, Spectrum)
STD_ROM_FN(SpecSuperted)

struct BurnDriver BurnSpecSuperted = {
	"spec_superted", NULL, "spec_spectrum", NULL, "1990",
	"SuperTed - The Search for Spotty (48K)\0", NULL, "Alternative Software", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecSupertedRomInfo, SpecSupertedRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Swords & Sorcery (48K)

static struct BurnRomInfo SpecSwordsnsorceryRomDesc[] = {
	{ "Swords & Sorcery 48K (1985)(PSS).tap", 48456, 0xee9f4073, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecSwordsnsorcery, SpecSwordsnsorcery, Spectrum)
STD_ROM_FN(SpecSwordsnsorcery)

struct BurnDriver BurnSpecSwordsnsorcery = {
	"spec_swordsnsorcery", NULL, "spec_spectrum", NULL, "1985",
	"Swords & Sorcery (48K)\0", "Press '1-2-3' to move character and '8-9-0' to use menu", "PSS", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_MAZE | GBF_RPG, 0,
	SpectrumGetZipName, SpecSwordsnsorceryRomInfo, SpecSwordsnsorceryRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Tapper (48K)

static struct BurnRomInfo SpecTapperRomDesc[] = {
	{ "Tapper 48K (1985)(U.S. Gold).tap", 48800, 0xdcc02c66, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecTapper, SpecTapper, Spectrum)
STD_ROM_FN(SpecTapper)

struct BurnDriver BurnSpecTapper = {
	"spec_tapper", NULL, "spec_spectrum", NULL, "1985",
	"Tapper (48K)\0", NULL, "U.S. Gold", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 2, HARDWARE_SPECTRUM, GBF_ACTION, 0,
	SpectrumGetZipName, SpecTapperRomInfo, SpecTapperRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Target (48K)

static struct BurnRomInfo SpecTargetRomDesc[] = {
	{ "Target 48K (1989)(Martech Games).tap", 26777, 0x5e01d8e8, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecTarget, SpecTarget, Spectrum)
STD_ROM_FN(SpecTarget)

struct BurnDriver BurnSpecTarget = {
	"spec_target", NULL, "spec_spectrum", NULL, "1989",
	"Target 48K\0", NULL, "Martech Games", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_ACTION, 0,
	SpectrumGetZipName, SpecTargetRomInfo, SpecTargetRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Tau-Ceti (48K)

static struct BurnRomInfo SpecTaucetiRomDesc[] = {
	{ "Tau-Ceti 48K (1985)(CRL Group).tap", 56807, 0x309522ae, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecTauceti, SpecTauceti, Spectrum)
STD_ROM_FN(SpecTauceti)

struct BurnDriver BurnSpecTauceti = {
	"spec_tauceti", "spec_taucetise", "spec_spectrum", NULL, "1985",
	"Tau-Ceti (48K)\0", "'SHIFT+SPACE' to begin, write 'HELP' to list commands", "CRL Group", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE, 1, HARDWARE_SPECTRUM, GBF_SHOOT | GBF_SIM, 0,
	SpectrumGetZipName, SpecTaucetiRomInfo, SpecTaucetiRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecQAOPSpaceDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Tau Ceti - The Special Edition (128K)

static struct BurnRomInfo SpecTaucetiseRomDesc[] = {
	{ "Tau Ceti - The Special Edition 128K (1987)(CRL Group).tap", 129527, 0x97369bcf, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecTaucetise, SpecTaucetise, Spec128)
STD_ROM_FN(SpecTaucetise)

struct BurnDriver BurnSpecTaucetise = {
	"spec_taucetise", NULL, "spec_spec128", NULL, "1987",
	"Tau Ceti - The Special Edition (128K)\0", "'SHIFT+SPACE' to begin, write 'HELP' to list commands", "CRL Group", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_SHOOT | GBF_SIM, 0,
	SpectrumGetZipName, SpecTaucetiseRomInfo, SpecTaucetiseRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecQAOPSpaceDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Techno-Cop (128K)

static struct BurnRomInfo SpecTechcop128RomDesc[] = {
	{ "Techno-Cop 128K (1988)(Gremlin Graphics).tap", 69984, 0x323627a4, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecTechcop128, SpecTechcop128, Spec128)
STD_ROM_FN(SpecTechcop128)

struct BurnDriver BurnSpecTechcop128 = {
	"spec_techcop128", NULL, "spec_spec128", NULL, "1988",
	"Techno-Cop (128K)\0", NULL, "Gremlin Graphics", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_SHOOT | GBF_SCRFIGHT, 0,
	SpectrumGetZipName, SpecTechcop128RomInfo, SpecTechcop128RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Techno-Cop (48K)

static struct BurnRomInfo SpecTechcop48RomDesc[] = {
	{ "Techno-Cop 48K (1988)(Gremlin Graphics).tap", 49374, 0x10fca563, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecTechcop48, SpecTechcop48, Spectrum)
STD_ROM_FN(SpecTechcop48)

struct BurnDriver BurnSpecTechcop48 = {
	"spec_techcop48", "spec_techcop128", "spec_spectrum", NULL, "1988",
	"Techno-Cop (48K)\0", NULL, "Gremlin Graphics", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE, 1, HARDWARE_SPECTRUM, GBF_SHOOT | GBF_SCRFIGHT, 0,
	SpectrumGetZipName, SpecTechcop48RomInfo, SpecTechcop48RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Tempest (48K)

static struct BurnRomInfo SpecTempestRomDesc[] = {
	{ "Tempest 48K (1986)(Electric Dreams).tap", 35221, 0x805e8742, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecTempest, SpecTempest, Spectrum)
STD_ROM_FN(SpecTempest)

struct BurnDriver BurnSpecTempest = {
	"spec_tempest", NULL, "spec_spectrum", NULL, "1986",
	"Tempest (48K)\0", NULL, "Electric Dreams", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 2, HARDWARE_SPECTRUM, GBF_SHOOT, 0,
	SpectrumGetZipName, SpecTempestRomInfo, SpecTempestRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Templos Sagrados, Los - Parte 1 (Spanish)

static struct BurnRomInfo SpecSagradosp1RomDesc[] = {
	{ "Templos Sagrados, Los - Parte 1 (1991)(Aventuras AD).tap", 45975, 0xcaebb204, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecSagradosp1, SpecSagradosp1, Spectrum)
STD_ROM_FN(SpecSagradosp1)

struct BurnDriver BurnSpecSagradosp1 = {
	"spec_sagradosp1", NULL, "spec_spectrum", NULL, "1991",
	"Templos Sagrados, Los - Parte 1 (Spanish)\0", NULL, "Aventuras A.D.", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_ADV, 0,
	SpectrumGetZipName, SpecSagradosp1RomInfo, SpecSagradosp1RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Templos Sagrados, Los - Parte 2 (Spanish)

static struct BurnRomInfo SpecSagradosp2RomDesc[] = {
	{ "Templos Sagrados, Los - Parte 2 (1991)(Aventuras AD).tap", 45936, 0x52e03e35, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecSagradosp2, SpecSagradosp2, Spectrum)
STD_ROM_FN(SpecSagradosp2)

struct BurnDriver BurnSpecSagradosp2 = {
	"spec_sagradosp2", "spec_sagradosp1", "spec_spectrum", NULL, "1991",
	"Templos Sagrados, Los - Parte 2 (Spanish)\0", "PASSWORD: TUR KOS BON", "Aventuras A.D.", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE, 1, HARDWARE_SPECTRUM, GBF_ADV, 0,
	SpectrumGetZipName, SpecSagradosp2RomInfo, SpecSagradosp2RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Terramex (128K)

static struct BurnRomInfo SpecTerramexRomDesc[] = {
	{ "Terramex 128K (1987)(Grandslam Entertainments).tap", 82251, 0xeaf717b6, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecTerramex, SpecTerramex, Spec128)
STD_ROM_FN(SpecTerramex)

struct BurnDriver BurnSpecTerramex = {
	"spec_terramex", NULL, "spec_spec128", NULL, "1987",
	"Terramex (128K)\0", NULL, "Grandslam Entertainments", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_PLATFORM | GBF_ADV, 0,
	SpectrumGetZipName, SpecTerramexRomInfo, SpecTerramexRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Terrorballs (48K)

static struct BurnRomInfo SpecTerrorballsRomDesc[] = {
	{ "Terrorballs 48K (1987)(Whammo Productions).tap", 42249, 0x86ae441b, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecTerrorballs, SpecTerrorballs, Spectrum)
STD_ROM_FN(SpecTerrorballs)

struct BurnDriver BurnSpecTerrorballs = {
	"spec_terrorballs", NULL, "spec_spectrum", NULL, "1987",
	"Terrorballs (48K)\0", NULL, "Whammo Productions", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_ACTION, 0,
	SpectrumGetZipName, SpecTerrorballsRomInfo, SpecTerrorballsRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Test Drive II - The Duel (128K)

static struct BurnRomInfo Spectestdrv2RomDesc[] = {
	{ "Test Drive II - The Duel (1989)(Accolade).z80", 0x0a8cd, 0x4047f7de, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Spectestdrv2, Spectestdrv2, Spec128)
STD_ROM_FN(Spectestdrv2)

struct BurnDriver BurnSpectestdrv2 = {
	"spec_testdrv2", NULL, "spec_spec128", NULL, "1989",
	"Test Drive II - The Duel (128K)\0", NULL, "Accolade", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_RACING, 0,
	SpectrumGetZipName, Spectestdrv2RomInfo, Spectestdrv2RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecIntf2DIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Thanatos (128K)

static struct BurnRomInfo SpecThanatos128RomDesc[] = {
	{ "Thanatos 128K (1986)(Durell).tap", 49266, 0x4ce5e54e, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecThanatos128, SpecThanatos128, Spec128)
STD_ROM_FN(SpecThanatos128)

struct BurnDriver BurnSpecThanatos128 = {
	"spec_thanatos128", NULL, "spec_spec128", NULL, "1986",
	"Thanatos (128K)\0", NULL, "Durell Software", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_HORSHOOT, 0,
	SpectrumGetZipName, SpecThanatos128RomInfo, SpecThanatos128RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Thanatos (48K)

static struct BurnRomInfo SpecThanatos48RomDesc[] = {
	{ "Thanatos 48K (1986)(Durell).tap", 43309, 0xefe49f3e, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecThanatos48, SpecThanatos48, Spectrum)
STD_ROM_FN(SpecThanatos48)

struct BurnDriver BurnSpecThanatos48 = {
	"spec_thanatos48", "spec_thanatos128", "spec_spectrum", NULL, "1986",
	"Thanatos (48K)\0", NULL, "Durell Software", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE, 1, HARDWARE_SPECTRUM, GBF_HORSHOOT, 0,
	SpectrumGetZipName, SpecThanatos48RomInfo, SpecThanatos48RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Throne of Fire (48K)

static struct BurnRomInfo SpecThronefireRomDesc[] = {
	{ "Throne of Fire 48K (1987)(Melbourne House).tap", 48101, 0xda030514, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecThronefire, SpecThronefire, Spectrum)
STD_ROM_FN(SpecThronefire)

struct BurnDriver BurnSpecThronefire = {
	"spec_thronefire", NULL, "spec_spectrum", NULL, "1987",
	"Throne of Fire (48K)\0", NULL, "Melbourne House", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 2, HARDWARE_SPECTRUM, GBF_ACTION | GBF_STRATEGY, 0,
	SpectrumGetZipName, SpecThronefireRomInfo, SpecThronefireRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Thundercats (128K)

static struct BurnRomInfo SpecthundcatsRomDesc[] = {
	{ "Thundercats (1987)(Elite Systems).tap", 94147, 0x76e19a9c, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specthundcats, Specthundcats, Spec128)
STD_ROM_FN(Specthundcats)

struct BurnDriver BurnSpecthundcats = {
	"spec_thundcats", NULL, "spec_spec128", NULL, "1987",
	"Thundercats (128K)\0", NULL, "Elite Systems", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_SCRFIGHT | GBF_HORSHOOT, 0,
	SpectrumGetZipName, SpecthundcatsRomInfo, SpecthundcatsRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Thunderbirds - Mission 1: Mine Menace (128K)

static struct BurnRomInfo SpecThundbirds1RomDesc[] = {
	{ "Thunderbirds - Part 1 (1989)(Grandslam Entertainments).tap", 52409, 0xa914189f, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecThundbirds1, SpecThundbirds1, Spec128)
STD_ROM_FN(SpecThundbirds1)

struct BurnDriver BurnSpecThundbirds1 = {
	"spec_thundbirds1", NULL, "spec_spec128", NULL, "1989",
	"Thunderbirds - Mission 1: Mine Menace (128K)\0", "Press SPACE to change character", "Grandslam Entertainments", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_ADV, 0,
	SpectrumGetZipName, SpecThundbirds1RomInfo, SpecThundbirds1RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Thunderbirds - Mission 2: Sub Crash (128K)

static struct BurnRomInfo SpecThundbirds2RomDesc[] = {
	{ "Thunderbirds - Part 2 (1989)(Grandslam Entertainments).tap", 52409, 0x0869b04e, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecThundbirds2, SpecThundbirds2, Spec128)
STD_ROM_FN(SpecThundbirds2)

struct BurnDriver BurnSpecThundbirds2 = {
	"spec_thundbirds2", "spec_thundbirds1", "spec_spec128", NULL, "1989",
	"Thunderbirds - Mission 2: Sub Crash (128K)\0", "Press SPACE to change character", "Grandslam Entertainments", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE, 1, HARDWARE_SPECTRUM, GBF_ADV, 0,
	SpectrumGetZipName, SpecThundbirds2RomInfo, SpecThundbirds2RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Thunderbirds - Mission 3: Bank Job (128K)

static struct BurnRomInfo SpecThundbirds3RomDesc[] = {
	{ "Thunderbirds - Part 3 (1989)(Grandslam Entertainments).tap", 52409, 0xb156bc82, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecThundbirds3, SpecThundbirds3, Spec128)
STD_ROM_FN(SpecThundbirds3)

struct BurnDriver BurnSpecThundbirds3 = {
	"spec_thundbirds3", "spec_thundbirds1", "spec_spec128", NULL, "1989",
	"Thunderbirds - Mission 3: Bank Job (128K)\0", "Press SPACE to change character", "Grandslam Entertainments", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE, 1, HARDWARE_SPECTRUM, GBF_ADV, 0,
	SpectrumGetZipName, SpecThundbirds3RomInfo, SpecThundbirds3RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Thunderbirds - Mission 4: Evil Hood (128K)

static struct BurnRomInfo SpecThundbirds4RomDesc[] = {
	{ "Thunderbirds - Part 4 (1989)(Grandslam Entertainments).tap", 52409, 0x9aafecf7, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecThundbirds4, SpecThundbirds4, Spec128)
STD_ROM_FN(SpecThundbirds4)

struct BurnDriver BurnSpecThundbirds4 = {
	"spec_thundbirds4", "spec_thundbirds1", "spec_spec128", NULL, "1989",
	"Thunderbirds - Mission 4: Evil Hood (128K)\0", "Press SPACE to change character", "Grandslam Entertainments", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE, 1, HARDWARE_SPECTRUM, GBF_ADV, 0,
	SpectrumGetZipName, SpecThundbirds4RomInfo, SpecThundbirds4RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Time Machine (48K)

static struct BurnRomInfo SpecTimemachineRomDesc[] = {
	{ "Time Machine 48K (1990)(Activision).z80", 41590, 0x5cf41e5c, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecTimemachine, SpecTimemachine, Spectrum)
STD_ROM_FN(SpecTimemachine)

struct BurnDriver BurnSpecTimemachine = {
	"spec_timemachine", NULL, "spec_spectrum", NULL, "1990",
	"Time Machine (48K)\0", NULL, "Activision", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_ADV, 0,
	SpectrumGetZipName, SpecTimemachineRomInfo, SpecTimemachineRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Tir Na Nog (48K)

static struct BurnRomInfo SpecTirnanogRomDesc[] = {
	{ "Tir Na Nog 48K (1984)(Gargoyle Games).tap", 46565, 0x8f5492ed, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecTirnanog, SpecTirnanog, Spectrum)
STD_ROM_FN(SpecTirnanog)

struct BurnDriver BurnSpecTirnanog = {
	"spec_tirnanog", NULL, "spec_spectrum", NULL, "1984",
	"Tir Na Nog (48K)\0", NULL, "Gargoyle Games", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_ADV, 0,
	SpectrumGetZipName, SpecTirnanogRomInfo, SpecTirnanogRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Titanic Blinky (48K)

static struct BurnRomInfo SpecTitblinkyRomDesc[] = {
	{ "Titanic Blinky 48K (1991)(Zeppelin Games).tap", 46442, 0xdff53244, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecTitblinky, SpecTitblinky, Spectrum)
STD_ROM_FN(SpecTitblinky)

struct BurnDriver BurnSpecTitblinky = {
	"spec_titblinky", NULL, "spec_spectrum", NULL, "1991",
	"Titanic Blinky (48K)\0", NULL, "Zeppelin Games", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecTitblinkyRomInfo, SpecTitblinkyRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Toad Runner (48K)

static struct BurnRomInfo SpecToadrunRomDesc[] = {
	{ "Toad Runner 48K (1986)(Ariolasoft).tap", 44113, 0x195edb9e, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecToadrun, SpecToadrun, Spectrum)
STD_ROM_FN(SpecToadrun)

struct BurnDriver BurnSpecToadrun = {
	"spec_toadrun", NULL, "spec_spectrum", NULL, "1986",
	"Toad Runner (48K)\0", NULL, "Ariolasoft", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_ACTION | GBF_ADV, 0,
	SpectrumGetZipName, SpecToadrunRomInfo, SpecToadrunRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Top Cat in Beverly Hills Cats (48K)

static struct BurnRomInfo SpecTopcatRomDesc[] = {
	{ "Top Cat in Beverly Hills Cats 48K (1991)(Hi-Tec).z80", 43855, 0x08eb7b04, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecTopcat, SpecTopcat, Spectrum)
STD_ROM_FN(SpecTopcat)

struct BurnDriver BurnSpecTopcat = {
	"spec_topcat", NULL, "spec_spectrum", NULL, "1991",
	"Top Cat in Beverly Hills Cats (48K) (HB)\0", NULL, "Hi-Tec Software", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_ADV, 0,
	SpectrumGetZipName, SpecTopcatRomInfo, SpecTopcatRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Total Eclipse (48K)

static struct BurnRomInfo SpecTeclipseRomDesc[] = {
	{ "Total Eclipse 48K (1988)(Incentive).tap", 50769, 0x1e312308, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecTeclipse, SpecTeclipse, Spectrum)
STD_ROM_FN(SpecTeclipse)

struct BurnDriver BurnSpecTeclipse = {
	"spec_teclipse", NULL, "spec_spectrum", NULL, "1988",
	"Total Eclipse (48K)\0", "P and L: look up and down - S: step size - SPACE: draw/replace gunsight", "Incentive", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_SHOOT | GBF_ADV, 0,
	SpectrumGetZipName, SpecTeclipseRomInfo, SpecTeclipseRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Total Eclipse 2 - The Sphinx Jinx (48K)

static struct BurnRomInfo SpecTeclipse2RomDesc[] = {
	{ "Total Eclipse 2 - The Sphinx Jinx 48K (1991)(Incentive).tap", 45514, 0x0c02705e, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecTeclipse2, SpecTeclipse2, Spectrum)
STD_ROM_FN(SpecTeclipse2)

struct BurnDriver BurnSpecTeclipse2 = {
	"spec_teclipse2", NULL, "spec_spectrum", NULL, "1991",
	"Total Eclipse 2 - The Sphinx Jinx (48K)\0", "P and L: look up and down - S: step size - SPACE: draw/replace gunsight", "Incentive", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_SHOOT | GBF_ADV, 0,
	SpectrumGetZipName, SpecTeclipse2RomInfo, SpecTeclipse2RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Toyota Celica GT Rally (128K)

static struct BurnRomInfo SpecCelicagtRomDesc[] = {
	{ "Toyota Celica GT Rally (1991)(Gremlin Graphics).tap", 97948, 0x153240dc, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecCelicagt, SpecCelicagt, Spec128)
STD_ROM_FN(SpecCelicagt)

struct BurnDriver BurnSpecCelicagt = {
	"spec_celicagt", NULL, "spec_spec128", NULL, "1991",
	"Toyota Celica GT Rally (128K)\0", NULL, "Gremlin Graphics", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_RACING, 0,
	SpectrumGetZipName, SpecCelicagtRomInfo, SpecCelicagtRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecIntf2DIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Train - Escape to Normandy, The (48K)

static struct BurnRomInfo SpecEscnormandyRomDesc[] = {
	{ "Train - Escape to Normandy, The 48K (1988)(Electronic Arts).z80", 44448, 0xccbf6d84, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecEscnormandy, SpecEscnormandy, Spectrum)
STD_ROM_FN(SpecEscnormandy)

struct BurnDriver BurnSpecEscnormandy = {
	"spec_escnormandy", NULL, "spec_spectrum", NULL, "1988",
	"Train - Escape to Normandy, The (48K)\0", NULL, "Electronic Arts", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_SIM | GBF_SHOOT, 0,
	SpectrumGetZipName, SpecEscnormandyRomInfo, SpecEscnormandyRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Tranz Am (16K)

static struct BurnRomInfo SpecTranzamRomDesc[] = {
	{ "Tranz Am (1983)(Ultimate Play The Game).z80", 14205, 0x00eaadb6, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecTranzam, SpecTranzam, Spectrum)
STD_ROM_FN(SpecTranzam)

struct BurnDriver BurnSpecTranzam = {
	"spec_tranzam", NULL, "spec_spectrum", NULL, "1983",
	"Tranz Am (16K)\0", NULL, "Ultimate Play The Game", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_RACING, 0,
	SpectrumGetZipName, SpecTranzamRomInfo, SpecTranzamRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Traxx (48K)

static struct BurnRomInfo SpecTraxxRomDesc[] = {
	{ "Traxx 48K (1983)(Salamander Software).tap", 18944, 0xc7221f97, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecTraxx, SpecTraxx, Spectrum)
STD_ROM_FN(SpecTraxx)

struct BurnDriver BurnSpecTraxx = {
	"spec_traxx", NULL, "spec_spectrum", NULL, "1983",
	"Traxx (48K)\0", NULL, "Salamander Software", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 2, HARDWARE_SPECTRUM, GBF_ACTION, 0,
	SpectrumGetZipName, SpecTraxxRomInfo, SpecTraxxRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecCursorKeysDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Treinador de Futebol (Portuguese) (48K)

static struct BurnRomInfo SpecTrfutebolRomDesc[] = {
	{ "Treinador de Futebol PT 48K (1983)(Microbyte Software).z80", 34576, 0x49dbc0b8, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecTrfutebol, SpecTrfutebol, Spectrum)
STD_ROM_FN(SpecTrfutebol)

struct BurnDriver BurnSpecTrfutebol = {
	"spec_trfutebol", NULL, "spec_spectrum", NULL, "1983",
	"Treinador de Futebol (Portuguese) (48K)\0", NULL, "Microbyte Software", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_STRATEGY, 0,
	SpectrumGetZipName, SpecTrfutebolRomInfo, SpecTrfutebolRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Troll (128K)

static struct BurnRomInfo SpecTrollRomDesc[] = {
	{ "Troll 128K (1988)(Palace Software).tap", 47794, 0xd3af2582, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecTroll, SpecTroll, Spec128)
STD_ROM_FN(SpecTroll)

struct BurnDriver BurnSpecTroll = {
	"spec_troll", NULL, "spec_spec128", NULL, "1988",
	"Troll (128K)\0", NULL, "Palace Software", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_ACTION, 0,
	SpectrumGetZipName, SpecTrollRomInfo, SpecTrollRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Trom (16K)

static struct BurnRomInfo SpecTromRomDesc[] = {
	{ "Trom 16K (1983)(DK'Tronics).z80", 10147, 0xfd8cc4f1, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecTrom, SpecTrom, Spectrum)
STD_ROM_FN(SpecTrom)

struct BurnDriver BurnSpecTrom = {
	"spec_trom", NULL, "spec_spectrum", NULL, "1983",
	"Trom (16K)\0", NULL, "DK'Tronics", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_ACTION, 0,
	SpectrumGetZipName, SpecTromRomInfo, SpecTromRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecQAOPSpaceDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Trouble Brewin (48K)

static struct BurnRomInfo SpecTroubrewinRomDesc[] = {
	{ "Trouble Brewin 48K (1984)(Silversoft).tap", 46240, 0x19c6677d, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecTroubrewin, SpecTroubrewin, Spectrum)
STD_ROM_FN(SpecTroubrewin)

struct BurnDriver BurnSpecTroubrewin = {
	"spec_troubrewin", NULL, "spec_spectrum", NULL, "1984",
	"Trouble Brewin (48K)\0", NULL, "Silversoft", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecTroubrewinRomInfo, SpecTroubrewinRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Tuareg (Spanish) (48K)

static struct BurnRomInfo SpecTuaregRomDesc[] = {
	{ "Tuareg ES 48K (1988)(Topo Soft).tap", 58215, 0x4fb0f564, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecTuareg, SpecTuareg, Spectrum)
STD_ROM_FN(SpecTuareg)

struct BurnDriver BurnSpecTuareg = {
	"spec_tuareg", NULL, "spec_spectrum", NULL, "1988",
	"Tuareg (Spanish) (48K)\0", NULL, "Topo Soft", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_ACTION | GBF_ADV, 0,
	SpectrumGetZipName, SpecTuaregRomInfo, SpecTuaregRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Turbo Boat Simulator (128K)

static struct BurnRomInfo SpecTurboboatsimRomDesc[] = {
	{ "Turbo Boat Simulator 128K (1988)(Silverbird).tap", 36697, 0xd7df8d1e, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecTurboboatsim, SpecTurboboatsim, Spec128)
STD_ROM_FN(SpecTurboboatsim)

struct BurnDriver BurnSpecTurboboatsim = {
	"spec_turboboatsim", NULL, "spec_spec128", NULL, "1988",
	"Turbo Boat Simulator (128K)\0", NULL, "Silverbird Software", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_HORSHOOT, 0,
	SpectrumGetZipName, SpecTurboboatsimRomInfo, SpecTurboboatsimRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Turbo Driver (16K)

static struct BurnRomInfo SpecTurbodriverRomDesc[] = {
	{ "Turbo Driver (1983)(B.O.S. Software).tap", 7006, 0x59b6a725, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecTurbodriver, SpecTurbodriver, Spectrum)
STD_ROM_FN(SpecTurbodriver)

struct BurnDriver BurnSpecTurbodriver = {
	"spec_turbodriver", NULL, "spec_spectrum", NULL, "1983",
	"Turbo Driver (16K)\0", NULL, "B.O.S. Software", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_RACING, 0,
	SpectrumGetZipName, SpecTurbodriverRomInfo, SpecTurbodriverRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Turbo Skate Fighter (48K)

static struct BurnRomInfo SpecTurboskateRomDesc[] = {
	{ "Turbo Skate Fighter 48K (1989)(Zeppelin Games).tap", 32414, 0x4c658b72, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecTurboskate, SpecTurboskate, Spectrum)
STD_ROM_FN(SpecTurboskate)

struct BurnDriver BurnSpecTurboskate = {
	"spec_turboskate", NULL, "spec_spectrum", NULL, "1989",
	"Turbo Skate Fighter (48K)\0", NULL, "Zeppelin Games", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_ACTION, 0,
	SpectrumGetZipName, SpecTurboskateRomInfo, SpecTurboskateRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Turbo the Tortoise - 30th Anniversary Edition (128K)

static struct BurnRomInfo SpecTurbothetortoise30thRomDesc[] = {
	{ "Turbo The Tortoise 30th Anniversary Edition (2022)(BiNMaN).tap", 74768, 0x0207ae31, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecTurbothetortoise30th, SpecTurbothetortoise30th, Spec128)
STD_ROM_FN(SpecTurbothetortoise30th)

struct BurnDriver BurnSpecTurbothetortoise30th = {
	"spec_turbothetortoise30th", "spec_turbothetortoise", "spec_spec128", NULL, "2022",
	"Turbo the Tortoise - 30th Anniversary Edition (128K)\0", NULL, "BiNMaN", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecTurbothetortoise30thRomInfo, SpecTurbothetortoise30thRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Tusker (128K)

static struct BurnRomInfo SpecTuskerRomDesc[] = {
	{ "Tusker 128K (1989)(System 3).tap", 135053, 0x4802c727, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecTusker, SpecTusker, Spec128)
STD_ROM_FN(SpecTusker)

struct BurnDriver BurnSpecTusker = {
	"spec_tusker", NULL, "spec_spec128", NULL, "1989",
	"Tusker (128K)\0", NULL, "System 3", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_ADV, 0,
	SpectrumGetZipName, SpecTuskerRomInfo, SpecTuskerRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Twin World (128K)

static struct BurnRomInfo SpecTwinworldRomDesc[] = {
	{ "Twin World 128K (1990)(Ubi Soft).tap", 357327, 0x1b14d5c6, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecTwinworld, SpecTwinworld, Spec128)
STD_ROM_FN(SpecTwinworld)

struct BurnDriver BurnSpecTwinworld = {
	"spec_twinworld", NULL, "spec_spec128", NULL, "1990",
	"Twin World (128K)\0", NULL, "Ubi Soft", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_PLATFORM | GBF_ADV, 0,
	SpectrumGetZipName, SpecTwinworldRomInfo, SpecTwinworldRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Twister (48K)

static struct BurnRomInfo SpecTwisterRomDesc[] = {
	{ "Twister 48K (1986)(System 3).tap", 25904, 0x3a54f2c8, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecTwister, SpecTwister, Spectrum)
STD_ROM_FN(SpecTwister)

struct BurnDriver BurnSpecTwister = {
	"spec_twister", NULL, "spec_spectrum", NULL, "1986",
	"Twister (48K)\0", NULL, "System 3", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_ACTION, 0,
	SpectrumGetZipName, SpecTwisterRomInfo, SpecTwisterRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Underground (Spanish) (48K)

static struct BurnRomInfo SpecUndergroundRomDesc[] = {
	{ "Underground ES 48K (1988)(System 4).tap", 49575, 0x0470e56c, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecUnderground, SpecUnderground, Spectrum)
STD_ROM_FN(SpecUnderground)

struct BurnDriver BurnSpecUnderground = {
	"spec_underground", NULL, "spec_spectrum", NULL, "1988",
	"Underground (Spanish) (48K)\0", NULL, "System 4", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecUndergroundRomInfo, SpecUndergroundRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Vectron (48K)

static struct BurnRomInfo SpecVectronRomDesc[] = {
	{ "Vectron 48K (1985)(Firebird).tap", 49343, 0xec6217eb, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecVectron, SpecVectron, Spectrum)
STD_ROM_FN(SpecVectron)

struct BurnDriver BurnSpecVectron = {
	"spec_vectron", NULL, "spec_spectrum", NULL, "1985",
	"Vectron (48K)\0", NULL, "Firebird Software", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_SHOOT, 0,
	SpectrumGetZipName, SpecVectronRomInfo, SpecVectronRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Vendetta (48K)

static struct BurnRomInfo SpecVendettaRomDesc[] = {
	{ "Vendetta 48K (1990)(System 3).tap", 165768, 0x8992478a, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecVendetta, SpecVendetta, Spectrum)
STD_ROM_FN(SpecVendetta)

struct BurnDriver BurnSpecVendetta = {
	"spec_vendetta", NULL, "spec_spectrum", NULL, "1990",
	"Vendetta (48K)\0", NULL, "System 3", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_ACTION | GBF_ADV, 0,
	SpectrumGetZipName, SpecVendettaRomInfo, SpecVendettaRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Victory Road - The Pathway to Fear (48K)

static struct BurnRomInfo SpecVictoryroadRomDesc[] = {
	{ "Victory Road - The Pathway to Fear 48K (1989)(Imagine).tap", 47710, 0x0613c95a, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecVictoryroad, SpecVictoryroad, Spectrum)
STD_ROM_FN(SpecVictoryroad)

struct BurnDriver BurnSpecVictoryroad = {
	"spec_victoryroad", NULL, "spec_spectrum", NULL, "1989",
	"Victory Road - The Pathway to Fear (48K)\0", NULL, "Imagine Software", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 2, HARDWARE_SPECTRUM, GBF_RUNGUN, 0,
	SpectrumGetZipName, SpecVictoryroadRomInfo, SpecVictoryroadRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Video Poker (48K)

static struct BurnRomInfo SpecVideopokerRomDesc[] = {
	{ "Video Poker 48K (1986)(Mastertronic).tap", 30085, 0xe67c72e3, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecVideopoker, SpecVideopoker, Spectrum)
STD_ROM_FN(SpecVideopoker)

struct BurnDriver BurnSpecVideopoker = {
	"spec_videopoker", NULL, "spec_spectrum", NULL, "1986",
	"Video Poker (48K)\0", NULL, "Mastertronic", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_CASINO, 0,
	SpectrumGetZipName, SpecVideopokerRomInfo, SpecVideopokerRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Virus (48K)

static struct BurnRomInfo SpecVirusRomDesc[] = {
	{ "Virus 48K (1988)(Firebird).tap", 42183, 0x7a2c590b, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecVirus, SpecVirus, Spectrum)
STD_ROM_FN(SpecVirus)

struct BurnDriver BurnSpecVirus = {
	"spec_virus", NULL, "spec_spectrum", NULL, "1988",
	"Virus (48K)\0", "Press 'R' to controller menu and ENTER to display in-game map", "Firebird Software", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_SHOOT, 0,
	SpectrumGetZipName, SpecVirusRomInfo, SpecVirusRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Vixen - Part 1 (128K)

static struct BurnRomInfo SpecVixen1RomDesc[] = {
	{ "Vixen - Part 1 (1988)(Martech Games).tap", 49543, 0xe5d1c4a9, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecVixen1, SpecVixen1, Spec128)
STD_ROM_FN(SpecVixen1)

struct BurnDriver BurnSpecVixen1 = {
	"spec_vixen1", NULL, "spec_spec128", NULL, "1988",
	"Vixen - Part 1 (128K)\0", NULL, "Martech Games", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_PLATFORM | GBF_RUNGUN, 0,
	SpectrumGetZipName, SpecVixen1RomInfo, SpecVixen1RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Vixen - Part 2 (128K)

static struct BurnRomInfo SpecVixen2RomDesc[] = {
	{ "Vixen - Part 2 (1988)(Martech Games).tap", 49547, 0xba54ec0b, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecVixen2, SpecVixen2, Spec128)
STD_ROM_FN(SpecVixen2)

struct BurnDriver BurnSpecVixen2 = {
	"spec_vixen2", "spec_vixen1", "spec_spec128", NULL, "1988",
	"Vixen - Part 2 (128K)\0", NULL, "Martech Games", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE, 1, HARDWARE_SPECTRUM, GBF_PLATFORM | GBF_RUNGUN, 0,
	SpectrumGetZipName, SpecVixen2RomInfo, SpecVixen2RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Vixen - Part 3 (128K)

static struct BurnRomInfo SpecVixen3RomDesc[] = {
	{ "Vixen - Part 3 (1988)(Martech Games).tap", 49547, 0x1d0845d9, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecVixen3, SpecVixen3, Spec128)
STD_ROM_FN(SpecVixen3)

struct BurnDriver BurnSpecVixen3 = {
	"spec_vixen3", "spec_vixen1", "spec_spec128", NULL, "1988",
	"Vixen - Part 3 (128K)\0", NULL, "Martech Games", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE, 1, HARDWARE_SPECTRUM, GBF_PLATFORM | GBF_RUNGUN, 0,
	SpectrumGetZipName, SpecVixen3RomInfo, SpecVixen3RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Wacky Darts (128K)

static struct BurnRomInfo SpecWackydartsRomDesc[] = {
	{ "Wacky Darts (1991)(Codemasters)(128K).z80", 58652, 0x6790db75, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecWackydarts, SpecWackydarts, Spec128)
STD_ROM_FN(SpecWackydarts)

struct BurnDriver BurnSpecWackydarts = {
	"spec_wackydarts", NULL, "spec_spec128", NULL, "1991",
	"Wacky Darts (128K)\0", "In options menu: keys '6', '7' and ENTER to choose and set", "Codemasters", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 8, HARDWARE_SPECTRUM, GBF_SPORTSMISC, 0,
	SpectrumGetZipName, SpecWackydartsRomInfo, SpecWackydartsRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecIntf2DIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// War Machine (128K)

static struct BurnRomInfo SpecWarmachineRomDesc[] = {
	{ "War Machine (1989)(Players Premier).tap", 54991, 0x7ac04bb1, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecWarmachine, SpecWarmachine, Spec128)
STD_ROM_FN(SpecWarmachine)

struct BurnDriver BurnSpecWarmachine = {
	"spec_warmachine", NULL, "spec_spec128", NULL, "1989",
	"War Machine (128K)\0", NULL, "Players Premier", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecWarmachineRomInfo, SpecWarmachineRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecIntf2DIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Wells & Fargo (Spanish) (48K)

static struct BurnRomInfo SpecWellsfargoRomDesc[] = {
	{ "Wells and Fargo ES 48K (1988)(Topo Soft).tap", 64570, 0x74b64095, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecWellsfargo, SpecWellsfargo, Spectrum)
STD_ROM_FN(SpecWellsfargo)

struct BurnDriver BurnSpecWellsfargo = {
	"spec_wellsfargo", NULL, "spec_spectrum", NULL, "1988",
	"Wells & Fargo (Spanish) (48K)\0", NULL, "Topo Soft", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 2, HARDWARE_SPECTRUM, GBF_SHOOT, 0,
	SpectrumGetZipName, SpecWellsfargoRomInfo, SpecWellsfargoRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Werewolves of London (48K)

static struct BurnRomInfo SpecWerewolvesRomDesc[] = {
	{ "Werewolves of London 48K (1989)(Mastertronic).tap", 47170, 0x8b9b9683, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecWerewolves, SpecWerewolves, Spectrum)
STD_ROM_FN(SpecWerewolves)

struct BurnDriver BurnSpecWerewolves = {
	"spec_werewolves", NULL, "spec_spectrum", NULL, "1989",
	"Werewolves of London (48K)\0", "O/P: select icon - H: pause - R: restart", "Mastertronic", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_ACTION | GBF_ADV, 0,
	SpectrumGetZipName, SpecWerewolvesRomInfo, SpecWerewolvesRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// West Bank (48K)

static struct BurnRomInfo SpecWestbankRomDesc[] = {
	{ "West Bank 48K (1985)(Gremlin Graphics).tap", 56654, 0xd211a5a9, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecWestbank, SpecWestbank, Spectrum)
STD_ROM_FN(SpecWestbank)

struct BurnDriver BurnSpecWestbank = {
	"spec_westbank", NULL, "spec_spectrum", NULL, "1985",
	"West Bank (48K)\0", "1-2-3 to shoot, O-P to move", "Gremlin Graphics", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_ACTION, 0,
	SpectrumGetZipName, SpecWestbankRomInfo, SpecWestbankRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Who Dares Wins II (48K)

static struct BurnRomInfo SpecWhodareswinsiiRomDesc[] = {
	{ "Who Dares Wins II 48K (1986)(Alligata).tap", 48631, 0x02dc6931, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecWhodareswinsii, SpecWhodareswinsii, Spectrum)
STD_ROM_FN(SpecWhodareswinsii)

struct BurnDriver BurnSpecWhodareswinsii = {
	"spec_whodareswinsii", NULL, "spec_spectrum", NULL, "1986",
	"Who Dares Wins II (48K)\0", NULL, "Alligata Software", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_RUNGUN, 0,
	SpectrumGetZipName, SpecWhodareswinsiiRomInfo, SpecWhodareswinsiiRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Wiking (Polish) (48K)

static struct BurnRomInfo SpecWiking48RomDesc[] = {
	{ "Wiking PL 48K (1987)(M.K.).tap", 16191, 0xb53a2ba2, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecWiking48, SpecWiking48, Spectrum)
STD_ROM_FN(SpecWiking48)

struct BurnDriver BurnSpecWiking48 = {
	"spec_wiking48", NULL, "spec_spectrum", NULL, "1987",
	"Wiking (Polish) (48K)\0", NULL, "M.K.", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecWiking48RomInfo, SpecWiking48RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecQAOPSpaceDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Wild West Hero (48K)

static struct BurnRomInfo SpecWwheroRomDesc[] = {
	{ "Wild West Hero 48K (1983)(Timescape).tap", 32638, 0xb709536d, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecWwhero, SpecWwhero, Spectrum)
STD_ROM_FN(SpecWwhero)

struct BurnDriver BurnSpecWwhero = {
	"spec_wwhero", NULL, "spec_spectrum", NULL, "1983",
	"Wild West Hero (48K)\0", NULL, "Timescape", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 2, HARDWARE_SPECTRUM, GBF_ACTION | GBF_SHOOT, 0,
	SpectrumGetZipName, SpecWwheroRomInfo, SpecWwheroRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Winged Avenger, The (16K)

static struct BurnRomInfo SpecWingaveRomDesc[] = {
	{ "Winged Avenger, The 16K (1982)(Work Force).tap", 18654, 0xf21cff61, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecWingave, SpecWingave, Spectrum)
STD_ROM_FN(SpecWingave)

struct BurnDriver BurnSpecWingave = {
	"spec_wingave", NULL, "spec_spectrum", NULL, "1982",
	"Winged Avenger, The (16K)\0", "MKI & MKII versions", "Work Force", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_SHOOT, 0,
	SpectrumGetZipName, SpecWingaveRomInfo, SpecWingaveRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Winged Warlords (16K)

static struct BurnRomInfo SpecWingwarRomDesc[] = {
	{ "Winged Warlords 16K (1983)(CDS Microsystems).tap", 6315, 0x1c86e32b, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecWingwar, SpecWingwar, Spectrum)
STD_ROM_FN(SpecWingwar)

struct BurnDriver BurnSpecWingwar = {
	"spec_wingwar", NULL, "spec_spectrum", NULL, "1983",
	"Winged Warlords (16K)\0", NULL, "CDS Microsystems", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 2, HARDWARE_SPECTRUM, GBF_ACTION, 0,
	SpectrumGetZipName, SpecWingwarRomInfo, SpecWingwarRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Winter Olympiad '88 (48K)

static struct BurnRomInfo SpecWinterolp88RomDesc[] = {
	{ "Winter Olympiad '88 48K (1987)(Tynesoft).tap", 226895, 0xa50b4143, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecWinterolp88, SpecWinterolp88, Spectrum)
STD_ROM_FN(SpecWinterolp88)

struct BurnDriver BurnSpecWinterolp88 = {
	"spec_winterolp88", NULL, "spec_spectrum", NULL, "1987",
	"Winter Olympiad '88 (48K)\0", NULL, "Tynesoft", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 9, HARDWARE_SPECTRUM, GBF_SPORTSMISC, 0,
	SpectrumGetZipName, SpecWinterolp88RomInfo, SpecWinterolp88RomName,
	NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Winter Sports (128K)

static struct BurnRomInfo SpecWintersp128RomDesc[] = {
	{ "Winter Sports 128K (1985)(Electric Dreams).tap", 117057, 0x4b785e14, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecWintersp128, SpecWintersp128, Spec128)
STD_ROM_FN(SpecWintersp128)

struct BurnDriver BurnSpecWintersp128 = {
	"spec_wintersp128", NULL, "spec_spec128", NULL, "1985",
	"Winter Sports (128K)\0", "Play events in random order - Press 'J' and after 'K' to use the controller", "Electric Dreams", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 2, HARDWARE_SPECTRUM, GBF_SPORTSMISC, 0,
	SpectrumGetZipName, SpecWintersp128RomInfo, SpecWintersp128RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Winter Sports (48K)

static struct BurnRomInfo SpecWintersp48RomDesc[] = {
	{ "Winter Sports 48K (1985)(Electric Dreams).tap", 245160, 0xb539f2c6, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecWintersp48, SpecWintersp48, Spectrum)
STD_ROM_FN(SpecWintersp48)

struct BurnDriver BurnSpecWintersp48 = {
	"spec_wintersp48", "spec_wintersp128", "spec_spectrum", NULL, "1985",
	"Winter Sports (48K)\0", "Play events in ascending order - Press 'J' and after 'K' to use the controller", "Electric Dreams", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE, 2, HARDWARE_SPECTRUM, GBF_SPORTSMISC, 0,
	SpectrumGetZipName, SpecWintersp48RomInfo, SpecWintersp48RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Witch's Cauldron, The (48K)

static struct BurnRomInfo SpecWitchcauldRomDesc[] = {
	{ "Witch's Cauldron, The 48K (1985)(Mikro-Gen).tap", 48322, 0xfab78469, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecWitchcauld, SpecWitchcauld, Spectrum)
STD_ROM_FN(SpecWitchcauld)

struct BurnDriver BurnSpecWitchcauld = {
	"spec_witchcauld", NULL, "spec_spectrum", NULL, "1985",
	"Witch's Cauldron, The (48K)\0", NULL, "Mikro-Gen", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_ADV, 0,
	SpectrumGetZipName, SpecWitchcauldRomInfo, SpecWitchcauldRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Wiz (48K)

static struct BurnRomInfo SpecWizv1RomDesc[] = {
	{ "Wiz 48K (1987)(Melbourne House).tap", 45956, 0xc77eef3e, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecWizv1, SpecWizv1, Spectrum)
STD_ROM_FN(SpecWizv1)

struct BurnDriver BurnSpecWizv1 = {
	"spec_wizv1", NULL, "spec_spectrum", NULL, "1987",
	"Wiz (48K)\0", NULL, "Melbourne House", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_ACTION, 0,
	SpectrumGetZipName, SpecWizv1RomInfo, SpecWizv1RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Wiz (Alt) (48K)

static struct BurnRomInfo SpecWizv2RomDesc[] = {
	{ "Wiz 48K (Alt)(1987)(Melbourne House).tap", 45979, 0xa1cb964b, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecWizv2, SpecWizv2, Spectrum)
STD_ROM_FN(SpecWizv2)

struct BurnDriver BurnSpecWizv2 = {
	"spec_wizv2", "spec_wizv1", "spec_spectrum", NULL, "1987",
	"Wiz (Alt) (48K)\0", NULL, "Melbourne House", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE, 1, HARDWARE_SPECTRUM, GBF_ACTION, 0,
	SpectrumGetZipName, SpecWizv2RomInfo, SpecWizv2RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// World Series Basketball (48K)

static struct BurnRomInfo SpecWsbasketRomDesc[] = {
	{ "World Series Basketball (1985)(Imagine).z80", 40058, 0x611090db, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecWsbasket, SpecWsbasket, Spectrum)
STD_ROM_FN(SpecWsbasket)

struct BurnDriver BurnSpecWsbasket = {
	"spec_wsbasket", NULL, "spec_spectrum", NULL, "1985",
	"World Series Basketball (48K)\0", NULL, "Imagine Software", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 2, HARDWARE_SPECTRUM, GBF_SPORTSMISC, 0,
	SpectrumGetZipName, SpecWsbasketRomInfo, SpecWsbasketRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Xadom (48K)

static struct BurnRomInfo SpecXadomRomDesc[] = {
	{ "Xadom 48K (1983)(Quicksilva).tap", 46890, 0x6c3120a6, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecXadom, SpecXadom, Spectrum)
STD_ROM_FN(SpecXadom)

struct BurnDriver BurnSpecXadom = {
	"spec_xadom", NULL, "spec_spectrum", NULL, "1983",
	"Xadom (48K)\0", NULL, "Quicksilva", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_ACTION | GBF_MAZE, 0,
	SpectrumGetZipName, SpecXadomRomInfo, SpecXadomRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecIntf2DIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Xavior (48K)

static struct BurnRomInfo SpecXaviorRomDesc[] = {
	{ "Xavior 48K (1984)(PSS).tap", 50074, 0x2c36bcdb, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecXavior, SpecXavior, Spectrum)
STD_ROM_FN(SpecXavior)

struct BurnDriver BurnSpecXavior = {
	"spec_xavior", NULL, "spec_spectrum", NULL, "1984",
	"Xavior (48K)\0", NULL, "PSS", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_ACTION, 0,
	SpectrumGetZipName, SpecXaviorRomInfo, SpecXaviorRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Xeno (48K)

static struct BurnRomInfo SpecXenoRomDesc[] = {
	{ "Xeno (1986)(A'n'F).z80", 37264, 0x4d10e27a, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecXeno, SpecXeno, Spectrum)
STD_ROM_FN(SpecXeno)

struct BurnDriver BurnSpecXeno = {
	"spec_xeno", NULL, "spec_spectrum", NULL, "1986",
	"Xeno (48K)\0", NULL, "A'n'F Software", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 2, HARDWARE_SPECTRUM, GBF_SPORTSMISC, 0,
	SpectrumGetZipName, SpecXenoRomInfo, SpecXenoRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Xeno II (16K)

static struct BurnRomInfo SpecXeno2RomDesc[] = {
	{ "Xeno II 16K (1983)(Anirog).z80", 23368, 0x0406cb03, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecXeno2, SpecXeno2, Spectrum)
STD_ROM_FN(SpecXeno2)

struct BurnDriver BurnSpecXeno2 = {
	"spec_xeno2", NULL, "spec_spectrum", NULL, "1983",
	"Xeno II (16K)\0", NULL, "Anirog", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_ACTION, 0,
	SpectrumGetZipName, SpecXeno2RomInfo, SpecXeno2RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Xenophobe (128K)

static struct BurnRomInfo SpecXenophobeRomDesc[] = {
	{ "Xenophobe 128K (1989)(Micro Style).tap", 213558, 0xb32da9b4, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecXenophobe, SpecXenophobe, Spec128)
STD_ROM_FN(SpecXenophobe)

struct BurnDriver BurnSpecXenophobe = {
	"spec_xenophobe", NULL, "spec_spec128", NULL, "1989",
	"Xenophobe (128K)\0", NULL, "Micro Style", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 2, HARDWARE_SPECTRUM, GBF_RUNGUN, 0,
	SpectrumGetZipName, SpecXenophobeRomInfo, SpecXenophobeRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecIntf2DIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Yabba Dabba Doo! - Flintstones (48K)

static struct BurnRomInfo SpecYabbadooRomDesc[] = {
	{ "Yabba Dabba Doo! 48K (1985)(Quicksilva).tap", 45016, 0xc13ebf67, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecYabbadoo, SpecYabbadoo, Spectrum)
STD_ROM_FN(SpecYabbadoo)

struct BurnDriver BurnSpecYabbadoo = {
	"spec_yabbadoo", NULL, "spec_spectrum", NULL, "1985",
	"Yabba Dabba Doo! - Flintstones (48K)\0", NULL, "Quicksilva", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_ACTION, 0,
	SpectrumGetZipName, SpecYabbadooRomInfo, SpecYabbadooRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Yenght - La Fuente de la Juventud (Spanish) (48K)

static struct BurnRomInfo SpecYenghtRomDesc[] = {
	{ "Yenght - La Fuente de la Juventud ES (1984)(Dinamic).tap", 45958, 0x36df229a, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecYenght, SpecYenght, Spectrum)
STD_ROM_FN(SpecYenght)

struct BurnDriver BurnSpecYenght = {
	"spec_yenght", NULL, "spec_spectrum", NULL, "1984",
	"Yenght - La Fuente de la Juventud (Spanish) (48K)\0", NULL, "Dinamic Software", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_ADV, 0,
	SpectrumGetZipName, SpecYenghtRomInfo, SpecYenghtRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Yeti (48K)

static struct BurnRomInfo SpecYetiRomDesc[] = {
	{ "Yeti 48K (1988)(Destiny).tap", 48009, 0x0536775c, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecYeti, SpecYeti, Spectrum)
STD_ROM_FN(SpecYeti)

struct BurnDriver BurnSpecYeti = {
	"spec_yeti", NULL, "spec_spectrum", NULL, "1988",
	"Yeti (48K)\0", NULL, "Destiny", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_RUNGUN, 0,
	SpectrumGetZipName, SpecYetiRomInfo, SpecYetiRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Yie Ar Kung-Fu (48K)

static struct BurnRomInfo Specyiarkufu_48RomDesc[] = {
	{ "Yie Ar Kung-Fu 48K (1985)(Imagine).tap", 49930, 0xb2ca9ba8, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specyiarkufu_48, Specyiarkufu_48, Spectrum)
STD_ROM_FN(Specyiarkufu_48)

struct BurnDriver BurnSpecyiarkufu_48 = {
	"spec_yiarkufu_48", "spec_yiarkufu", "spec_spectrum", NULL, "1985",
	"Yie Ar Kung-Fu (48K)\0", NULL, "Imagine Software", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE, 2, HARDWARE_SPECTRUM, GBF_VSFIGHT, 0,
	SpectrumGetZipName, Specyiarkufu_48RomInfo, Specyiarkufu_48RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Yie Ar Kung-Fu (128K)

static struct BurnRomInfo SpecyiarkufuRomDesc[] = {
	{ "Yie Ar Kung-Fu 128K (1985)(Imagine).z80", 0x157f7, 0xf7c52002, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specyiarkufu, Specyiarkufu, Spec128)
STD_ROM_FN(Specyiarkufu)

struct BurnDriver BurnSpecyiarkufu = {
	"spec_yiarkufu", NULL, "spec_spec128", NULL, "1986",
	"Yie Ar Kung-Fu (128K)\0", NULL, "Imagine Software", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 2, HARDWARE_SPECTRUM, GBF_VSFIGHT, 0,
	SpectrumGetZipName, SpecyiarkufuRomInfo, SpecyiarkufuRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Yie Ar Kung-Fu II (48K)

static struct BurnRomInfo Specyiarkuf2RomDesc[] = {
	{ "Yie Ar Kung-Fu II 48K (1986)(Imagine).tap", 41343, 0x8a149697, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specyiarkuf2, Specyiarkuf2, Spectrum)
STD_ROM_FN(Specyiarkuf2)

struct BurnDriver BurnSpecyiarkuf2 = {
	"spec_yiarkuf2", NULL, "spec_spectrum", NULL, "1986",
	"Yie Ar Kung-Fu II (48K)\0", NULL, "Imagine Software", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 2, HARDWARE_SPECTRUM, GBF_VSFIGHT, 0,
	SpectrumGetZipName, Specyiarkuf2RomInfo, Specyiarkuf2RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Yogi Bear (48K)

static struct BurnRomInfo SpecYogibearRomDesc[] = {
	{ "Yogi Bear 48K (1987)(Piranha).z80", 43670, 0x437fa194, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecYogibear, SpecYogibear, Spectrum)
STD_ROM_FN(SpecYogibear)

struct BurnDriver BurnSpecYogibear = {
	"spec_yogibear", NULL, "spec_spectrum", NULL, "1987",
	"Yogi Bear (48K)\0", NULL, "Piranha", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecYogibearRomInfo, SpecYogibearRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Yogi's Great Escape (48K)

static struct BurnRomInfo SpecYogigreatescapeRomDesc[] = {
	{ "Yogi's Great Escape 48K (1990)(Hi-Tec).tap", 54731, 0xd6afad15, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecYogigreatescape, SpecYogigreatescape, Spectrum)
STD_ROM_FN(SpecYogigreatescape)

struct BurnDriver BurnSpecYogigreatescape = {
	"spec_yogigreatescape", NULL, "spec_spectrum", NULL, "1990",
	"Yogi's Great Escape (48K)\0", NULL, "Hi-Tec Software", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecYogigreatescapeRomInfo, SpecYogigreatescapeRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Yogi Bear and Friends - The Greed Monster (128K)

static struct BurnRomInfo SpecYogigreedmonsterRomDesc[] = {
	{ "Yogi Bear and Friends - The Greed Monster 128K (1990)(Hi-Tec).tap", 56556, 0x601b7f1f, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecYogigreedmonster, SpecYogigreedmonster, Spec128)
STD_ROM_FN(SpecYogigreedmonster)

struct BurnDriver BurnSpecYogigreedmonster = {
	"spec_yogigreedmonster", NULL, "spec_spec128", NULL, "1990",
	"Yogi Bear and Friends - The Greed Monster (128K)\0", NULL, "Hi-Tec Software", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_ADV, 0,
	SpectrumGetZipName, SpecYogigreedmonsterRomInfo, SpecYogigreedmonsterRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Zanthrax - Race of The Future (48K)

static struct BurnRomInfo SpecZanthraxRomDesc[] = {
	{ "Zanthrax - Race of The Future 48K (1989)(WASP, Crash).tap", 48034, 0xe1ab01e7, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecZanthrax, SpecZanthrax, Spectrum)
STD_ROM_FN(SpecZanthrax)

struct BurnDriver BurnSpecZanthrax = {
	"spec_zanthrax", NULL, "spec_spectrum", NULL, "1989",
	"Zanthrax - Race of The Future (48K)\0", NULL, "WASP, Crash", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_VERSHOOT, 0,
	SpectrumGetZipName, SpecZanthraxRomInfo, SpecZanthraxRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Zenji (48K)

static struct BurnRomInfo SpecZenjiRomDesc[] = {
	{ "Zenji 48K (1984)(Activision).tap", 23232, 0x62e604e5, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecZenji, SpecZenji, Spectrum)
STD_ROM_FN(SpecZenji)

struct BurnDriver BurnSpecZenji = {
	"spec_zenji", NULL, "spec_spectrum", NULL, "1984",
	"Zenji (48K)\0", NULL, "Activision", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_PUZZLE, 0,
	SpectrumGetZipName, SpecZenjiRomInfo, SpecZenjiRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Zip-Zap (48K)

static struct BurnRomInfo SpecZipzapRomDesc[] = {
	{ "Zip-Zap 48K (1983)(Imagine).tap", 37207, 0xfcb30933, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecZipzap, SpecZipzap, Spectrum)
STD_ROM_FN(SpecZipzap)

struct BurnDriver BurnSpecZipzap = {
	"spec_zipzap", NULL, "spec_spectrum", NULL, "1983",
	"Zip-Zap (48K)\0", NULL, "Imagine Software", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 2, HARDWARE_SPECTRUM, GBF_ACTION, 0,
	SpectrumGetZipName, SpecZipzapRomInfo, SpecZipzapRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Zorro (48K)

static struct BurnRomInfo SpecZorroRomDesc[] = {
	{ "Zorro 48K (1986)(U.S. Gold).z80", 37618, 0x9f7b2443, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecZorro, SpecZorro, Spectrum)
STD_ROM_FN(SpecZorro)

struct BurnDriver BurnSpecZorro = {
	"spec_zorro", NULL, "spec_spectrum", NULL, "1986",
	"Zorro (48K)\0", "After options menu press ENTER to begin the game", "U.S. Gold - DataSoft", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecZorroRomInfo, SpecZorroRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// ZX Berserk (16K)

static struct BurnRomInfo SpecZxberserkRomDesc[] = {
	{ "ZX Berserk v3 16K (1982)(Jega).tap", 8304, 0xb2c83305, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecZxberserk, SpecZxberserk, Spectrum)
STD_ROM_FN(SpecZxberserk)

struct BurnDriver BurnSpecZxberserk = {
	"spec_zxberserk", NULL, "spec_spectrum", NULL, "1982",
	"ZX Berserk (16K)\0", "W: up - X: down - A: left - D: right - S: fire", "Jega", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_ACTION, 0,
	SpectrumGetZipName, SpecZxberserkRomInfo, SpecZxberserkRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Zybex (48K)

static struct BurnRomInfo SpeczybexRomDesc[] = {
	{ "Zybex 48K (1989)(Zeppelin Games).z80", 34544, 0xa79c6137, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Speczybex, Speczybex, Spectrum)
STD_ROM_FN(Speczybex)

struct BurnDriver BurnSpeczybex = {
	"spec_zybex", NULL, "spec_spectrum", NULL, "1989",
	"Zybex (48K)\0", NULL, "Zeppelin Games", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 2, HARDWARE_SPECTRUM, GBF_HORSHOOT, 0,
	SpectrumGetZipName, SpeczybexRomInfo, SpeczybexRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecIntf2DIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Zzoom (48K)

static struct BurnRomInfo SpecZzoomRomDesc[] = {
	{ "Zzoom 48K (1983)(Imagine).tap", 22601, 0x88263d9c, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecZzoom, SpecZzoom, Spectrum)
STD_ROM_FN(SpecZzoom)

struct BurnDriver BurnSpecZzoom = {
	"spec_zzoom", NULL, "spec_spectrum", NULL, "1983",
	"Zzoom (48K)\0", "Use 'Shift' and 'Enter' to set Joystick options", "Imagine Software", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING, 1, HARDWARE_SPECTRUM, GBF_SHOOT, 0,
	SpectrumGetZipName, SpecZzoomRomInfo, SpecZzoomRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};


// -------------------------------
// Homebrew/Hack/Aftermarket Games
// -------------------------------


// Terrahawks (128K) (HB)

static struct BurnRomInfo SpecTerrahawksRomDesc[] = {
	{ "Terrahawks (2014)(Gary James).tap", 24138, 0xd6f05303, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecTerrahawks, SpecTerrahawks, Spec128)
STD_ROM_FN(SpecTerrahawks)

struct BurnDriver BurnSpecTerrahawks = {
	"spec_terrahawks", NULL, "spec_spec128", NULL, "2014",
	"Terrahawks (128K) (HB)\0", NULL, "Gary James", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_SHOOT, 0,
	SpectrumGetZipName, SpecTerrahawksRomInfo, SpecTerrahawksRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Castle Capers (128K) (HB)

static struct BurnRomInfo SpecCastlecapersRomDesc[] = {
	{ "CastleCapers (2017)(Gabriele Amore).tap", 36751, 0xdb772c7c, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecCastlecapers, SpecCastlecapers, Spec128)
STD_ROM_FN(SpecCastlecapers)

struct BurnDriver BurnSpecCastlecapers = {
	"spec_castlecapers", NULL, "spec_spec128", NULL, "2017",
	"Castle Capers (128K) (HB)\0", NULL, "Gabriele Amore", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_ACTION, 0,
	SpectrumGetZipName, SpecCastlecapersRomInfo, SpecCastlecapersRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Dirty Dozer (128K) (HB)

static struct BurnRomInfo SpecDirtydozerRomDesc[] = {
	{ "Dirty Dozer (2019)(Miguetelo).tap", 41391, 0xaf06dd37, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecDirtydozer, SpecDirtydozer, Spec128)
STD_ROM_FN(SpecDirtydozer)

struct BurnDriver BurnSpecDirtydozer = {
	"spec_dirtydozer", NULL, "spec_spec128", NULL, "2019",
	"Dirty Dozer (128K) (HB)\0", NULL, "Miguetelo", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PUZZLE, 0,
	SpectrumGetZipName, SpecDirtydozerRomInfo, SpecDirtydozerRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// El Stompo (48K) (HB)

static struct BurnRomInfo SpecElstompoRomDesc[] = {
	{ "ElStompo (2014)(Stonechat Productions).tap", 44449, 0xb2370ea9, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecElstompo, SpecElstompo, Spectrum)
STD_ROM_FN(SpecElstompo)

struct BurnDriver BurnSpecElstompo = {
	"spec_elstompo", NULL, "spec_spectrum", NULL, "2014",
	"El Stompo (48K) (HB)\0", NULL, "Stonechat Productions", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_ACTION, 0,
	SpectrumGetZipName, SpecElstompoRomInfo, SpecElstompoRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecQAOPSpaceDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Gandalf Deluxe (128K) (HB)

static struct BurnRomInfo SpecGandalfRomDesc[] = {
	{ "Gandalf Deluxe (2018)(Speccy Nights).tap", 95019, 0x1f7f26c8, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecGandalf, SpecGandalf, Spec128)
STD_ROM_FN(SpecGandalf)

struct BurnDriver BurnSpecGandalf = {
	"spec_gandalf", NULL, "spec_spec128", NULL, "2018",
	"Gandalf Deluxe (128K) (HB)\0", NULL, "Speccy Nights", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecGandalfRomInfo, SpecGandalfRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Metal Man (Russian) (48K) (HB)

static struct BurnRomInfo SpecMetalmanRomDesc[] = {
	{ "Metal Man (2014)(Oleg Origin).tap", 48298, 0xef74d54b, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecMetalman, SpecMetalman, Spectrum)
STD_ROM_FN(SpecMetalman)

struct BurnDriver BurnSpecMetalman = {
	"spec_metalman", NULL, "spec_spectrum", NULL, "1997-2010",
	"Metal Man (Russian) (48K) (HB)\0", NULL, "Oleg Origin", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_RUNGUN | GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecMetalmanRomInfo, SpecMetalmanRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Ninja Gaiden Shadow Warriors (128K) (HB)

static struct BurnRomInfo SpecNinjagaidenRomDesc[] = {
	{ "Ninja Gaiden Shadow Warriors (2018)(Jerri, DaRkHoRaCe, Diver).tap", 103645, 0x44081e87, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecNinjagaiden, SpecNinjagaiden, Spec128)
STD_ROM_FN(SpecNinjagaiden)

struct BurnDriver BurnSpecNinjagaiden = {
	"spec_ninjagaiden", NULL, "spec_spec128", NULL, "2018",
	"Ninja Gaiden Shadow Warriors (128K) (HB)\0", NULL, "Jerri, DaRkHoRaCe, Diver", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_SCRFIGHT, 0,
	SpectrumGetZipName, SpecNinjagaidenRomInfo, SpecNinjagaidenRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Parachute (128K) (HB)

static struct BurnRomInfo SpecParachuteRomDesc[] = {
	{ "Parachute (2018)(Miguetelo).tap", 40791, 0xd23bd0ec, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecParachute, SpecParachute, Spec128)
STD_ROM_FN(SpecParachute)

struct BurnDriver BurnSpecParachute = {
	"spec_parachute", NULL, "spec_spec128", NULL, "2018",
	"Parachute (128K) (HB)\0", NULL, "Miguetelo", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_ACTION, 0,
	SpectrumGetZipName, SpecParachuteRomInfo, SpecParachuteRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Parsec (128K) (HB)

static struct BurnRomInfo SpecParsecRomDesc[] = {
	{ "Parsec (2020)(Martin Mangan).tap", 40874, 0xa61d638d, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecParsec, SpecParsec, Spec128)
STD_ROM_FN(SpecParsec)

struct BurnDriver BurnSpecParsec = {
	"spec_parsec", NULL, "spec_spec128", NULL, "2020",
	"Parsec (128K) (HB)\0", NULL, "Martin Mangan", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_HORSHOOT, 0,
	SpectrumGetZipName, SpecParsecRomInfo, SpecParsecRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Space Junk (128K) (HB)

static struct BurnRomInfo SpecSpacejunkRomDesc[] = {
	{ "Space Junk (2017)(Miguetelo).tap", 42761, 0xaa8930bf, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecSpacejunk, SpecSpacejunk, Spec128)
STD_ROM_FN(SpecSpacejunk)

struct BurnDriver BurnSpecSpacejunk = {
	"spec_spacejunk", NULL, "spec_spec128", NULL, "2017",
	"Space Junk (128K) (HB)\0", NULL, "Miguetelo", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_SHOOT, 0,
	SpectrumGetZipName, SpecSpacejunkRomInfo, SpecSpacejunkRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Squares (128K) (HB)

static struct BurnRomInfo SpecSquaresRomDesc[] = {
	{ "Squares (2017)(Kas29).tap", 16053, 0x1b924ae6, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecSquares, SpecSquares, Spec128)
STD_ROM_FN(SpecSquares)

struct BurnDriver BurnSpecSquares = {
	"spec_squares", NULL, "spec_spec128", NULL, "2017",
	"Squares (128K) (HB)\0", NULL, "Kas29", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_ACTION, 0,
	SpectrumGetZipName, SpecSquaresRomInfo, SpecSquaresRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Stormfinch (128K) (HB)

static struct BurnRomInfo SpecStormfinchRomDesc[] = {
	{ "Stormfinch (2015)(Stonechat).tap", 47695, 0xcc53ba30, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecStormfinch, SpecStormfinch, Spec128)
STD_ROM_FN(SpecStormfinch)

struct BurnDriver BurnSpecStormfinch = {
	"spec_stormfinch", NULL, "spec_spec128", NULL, "2015",
	"Stormfinch (128K) (HB)\0", NULL, "Stonechat", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_HORSHOOT, 0,
	SpectrumGetZipName, SpecStormfinchRomInfo, SpecStormfinchRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecQAOPSpaceDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Sword Of Ilana (128K) (HB)

static struct BurnRomInfo SpecSwordofilanaRomDesc[] = {
	{ "Sword Of Ilana (2017)(RetroWorks).tap", 212617, 0x448410cc, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecSwordofilana, SpecSwordofilana, Spec128)
STD_ROM_FN(SpecSwordofilana)

struct BurnDriver BurnSpecSwordofilana = {
	"spec_swordofilana", NULL, "spec_spec128", NULL, "2017",
	"Sword Of Ilana (128K) (HB)\0", NULL, "RetroWorks", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecSwordofilanaRomInfo, SpecSwordofilanaRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Lala Prologue (48K) (HB)

static struct BurnRomInfo SpeclalaRomDesc[] = {
	{ "Lala Prologue (2010)(Mojon Twins).tap", 32855, 0x547b6601, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Speclala, Speclala, Spectrum)
STD_ROM_FN(Speclala)

struct BurnDriver BurnSpeclala = {
	"spec_lala", NULL, "spec_spectrum", NULL, "2010",
	"Lala Prologue (48K) (HB)\0", NULL, "The Mojon Twins", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpeclalaRomInfo, SpeclalaRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecQAOPSpaceDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Sgt. Helmet Zero (128K) (HB)

static struct BurnRomInfo SpecsgthelmetRomDesc[] = {
	{ "Sgt. Helmet Zero (2009)(Mojon Twins).tap", 62412, 0xd182aa2e, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specsgthelmet, Specsgthelmet, Spec128)
STD_ROM_FN(Specsgthelmet)

struct BurnDriver BurnSpecsgthelmet = {
	"spec_sgthelmet", NULL, "spec_spec128", NULL, "2009",
	"Sgt. Helmet Zero (128K) (HB)\0", "Select '0' to prevent graphic glitches", "The Mojon Twins", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_RUNGUN, 0,
	SpectrumGetZipName, SpecsgthelmetRomInfo, SpecsgthelmetRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecQAOPSpaceDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Subaquatic Reloaded (128K) (HB)

static struct BurnRomInfo SpecsubacquaticRomDesc[] = {
	{ "Subaquatic Reloaded (2009)(Mojon Twins).tap", 55547, 0x45815dd3, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specsubacquatic, Specsubacquatic, Spec128)
STD_ROM_FN(Specsubacquatic)

struct BurnDriver BurnSpecsubacquatic = {
	"spec_subacquatic", NULL, "spec_spec128", NULL, "2010",
	"Subaquatic Reloaded (128K) (HB)\0", NULL, "The Mojon Twins", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_ACTION, 0,
	SpectrumGetZipName, SpecsubacquaticRomInfo, SpecsubacquaticRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// AstroCop (48K) (HB)

static struct BurnRomInfo SpecAstrocopRomDesc[] = {
	{ "AstroCop (2020)(MK1).tap", 41005, 0x2c1e9ca5, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecAstrocop, SpecAstrocop, Spectrum)
STD_ROM_FN(SpecAstrocop)

struct BurnDriver BurnSpecAstrocop = {
	"spec_astrocop", NULL, "spec_spectrum", NULL, "2020",
	"AstroCop (48K) (HB)\0", NULL, "Mananuk", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecAstrocopRomInfo, SpecAstrocopRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Herbert the Turbot (128K) (HB)

static struct BurnRomInfo SpecHerberttbRomDesc[] = {
	{ "Herbert the Turbot (2010)(Bob Smith).z80", 81123, 0x0a9c0799, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecHerberttb, SpecHerberttb, Spec128)
STD_ROM_FN(SpecHerberttb)

struct BurnDriver BurnSpecHerberttb = {
	"spec_herberttb", NULL, "spec_spec128", NULL, "2010",
	"Herbert the Turbot (128K) (HB)\0", NULL, "Bob Smith", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_ACTION | GBF_MAZE, 0,
	SpectrumGetZipName, SpecHerberttbRomInfo, SpecHerberttbRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Jet Pac RX (48K) (HB)

static struct BurnRomInfo SpecJetpacrxRomDesc[] = {
	{ "JetpacRX (2020)(highriser).tap", 22814, 0x2f0a86ff, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecJetpacrx, SpecJetpacrx, Spectrum)
STD_ROM_FN(SpecJetpacrx)

struct BurnDriver BurnSpecJetpacrx = {
	"spec_jetpacrx", "spec_jetpac", "spec_spectrum", NULL, "2020",
	"Jet Pac RX (48K) (HB)\0", "RX Mod of classic Jetpac", "highriser", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE | BDF_HOMEBREW | BDF_HACK, 2, HARDWARE_SPECTRUM, GBF_ACTION, 0,
	SpectrumGetZipName, SpecJetpacrxRomInfo, SpecJetpacrxRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Rygar Colour Enhancement Mod (48K) (Hack)

static struct BurnRomInfo SpecRygar2020RomDesc[] = {
	{ "Rygar 2020 (2020)(Ralf).tap", 35648, 0xc0af87aa, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecRygar2020, SpecRygar2020, Spectrum)
STD_ROM_FN(SpecRygar2020)

struct BurnDriver BurnSpecRygar2020 = {
	"spec_rygar2020", "spec_rygar", "spec_spectrum", NULL, "2019-20",
	"Rygar - Colour Enhancement Mod (48K) (Hack)\0", NULL, "Rafal Miazga", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE | BDF_HACK, 1, HARDWARE_SPECTRUM, GBF_RUNGUN | GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecRygar2020RomInfo, SpecRygar2020RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Bean Brothers (128K) (HB)

static struct BurnRomInfo SpecBeanbrosRomDesc[] = {
	{ "Bean Brothers (2018)(Stonechat Productions).tap", 55722, 0xaf989916, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecBeanbros, SpecBeanbros, Spec128)
STD_ROM_FN(SpecBeanbros)

struct BurnDriver BurnSpecBeanbros = {
	"spec_beanbros", NULL, "spec_spec128", NULL, "2018",
	"Bean Brothers (128K) (HB)\0", "Press 'down' to change character", "Stonechat Productions", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PLATFORM | GBF_PUZZLE, 0,
	SpectrumGetZipName, SpecBeanbrosRomInfo, SpecBeanbrosRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Castle Of Sorrow (48K) (HB)

static struct BurnRomInfo SpecCastleofsorrowRomDesc[] = {
	{ "Castle Of Sorrow (2018)(ZXMan48k).tap", 42259, 0x7750645b, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecCastleofsorrow, SpecCastleofsorrow, Spectrum)
STD_ROM_FN(SpecCastleofsorrow)

struct BurnDriver BurnSpecCastleofsorrow = {
	"spec_castleofsorrow", NULL, "spec_spectrum", NULL, "2018",
	"Castle Of Sorrow (48K) (HB)\0", NULL, "ZXMan48k", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecCastleofsorrowRomInfo, SpecCastleofsorrowRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Impossabubble (HB)

static struct BurnRomInfo SpecImpossabubbleRomDesc[] = {
	{ "Impossabubble (2018)(Dave Clarke).tap", 46080, 0x7cd3626a, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecImpossabubble, SpecImpossabubble, Spec128)
STD_ROM_FN(SpecImpossabubble)

struct BurnDriver BurnSpecImpossabubble = {
	"spec_impossabubble", NULL, "spec_spec128", NULL, "2018",
	"Impossabubble (HB)\0", NULL, "Dave Clarke", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecImpossabubbleRomInfo, SpecImpossabubbleRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Mister Kung Fu (128K) (HB, v1.3)

static struct BurnRomInfo SpecMrkungfuRomDesc[] = {
	{ "Mister Kung Fu v1.3 (2018-19)(Uprising Games).tap", 44141, 0x484a3e01, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecMrkungfu, SpecMrkungfu, Spec128)
STD_ROM_FN(SpecMrkungfu)

struct BurnDriver BurnSpecMrkungfu = {
	"spec_mrkungfu", NULL, "spec_spec128", NULL, "2018-19",
	"Mister Kung Fu (128K) (HB, v1.3)\0", NULL, "Uprising Games", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_SCRFIGHT, 0,
	SpectrumGetZipName, SpecMrkungfuRomInfo, SpecMrkungfuRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// OctuKitty (128K) (HB)

static struct BurnRomInfo SpecOctukttyRomDesc[] = {
	{ "OctuKitty (128K) (2018)(UltraNarwhal).tap", 43800, 0x28b574a6, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecOctuktty, SpecOctuktty, Spec128)
STD_ROM_FN(SpecOctuktty)

struct BurnDriver BurnSpecOctuktty = {
	"spec_octuktty", NULL, "spec_spec128", NULL, "2018",
	"OctuKitty (128K) (HB)\0", NULL, "UltraNarwhal", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PLATFORM | GBF_PUZZLE, 0,
	SpectrumGetZipName, SpecOctukttyRomInfo, SpecOctukttyRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Pooper Scooper (128K) (HB)

static struct BurnRomInfo SpecPooperscooperRomDesc[] = {
	{ "Pooper Scooper (2018)(The Death Squad).tap", 43438, 0x7bfc504c, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecPooperscooper, SpecPooperscooper, Spec128)
STD_ROM_FN(SpecPooperscooper)

struct BurnDriver BurnSpecPooperscooper = {
	"spec_pooperscooper", NULL, "spec_spec128", NULL, "2018",
	"Pooper Scooper (128K) (HB)\0", NULL, "The Death Squad", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_ACTION, 0,
	SpectrumGetZipName, SpecPooperscooperRomInfo, SpecPooperscooperRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecIntf2DIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Booty the remake (128K) (HB, v1.2)

static struct BurnRomInfo SpecBootyrmkRomDesc[] = {
	{ "Booty the remake v1.2 (2019)(salvaKantero).tap", 65706, 0xf6a0702a, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecBootyrmk, SpecBootyrmk, Spec128)
STD_ROM_FN(SpecBootyrmk)

struct BurnDriver BurnSpecBootyrmk = {
	"spec_bootyrmk", NULL, "spec_spec128", NULL, "2019",
	"Booty the remake (128K) (HB, v1.2)\0", NULL, "salvaKantero", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_ACTION, 0,
	SpectrumGetZipName, SpecBootyrmkRomInfo, SpecBootyrmkRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Booty the remake - Black Edition (128K) (HB, v1.2)

static struct BurnRomInfo SpecBootyrmkbeRomDesc[] = {
	{ "Booty the remake BE v1.2 (2019)(salvaKantero).tap", 65203, 0x08ae97d1, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecBootyrmkbe, SpecBootyrmkbe, Spec128)
STD_ROM_FN(SpecBootyrmkbe)

struct BurnDriver BurnSpecBootyrmkbe = {
	"spec_bootyrmkbe", "spec_bootyrmk", "spec_spec128", NULL, "2019",
	"Booty the remake - Black Edition (128K) (HB, v1.2)\0", NULL, "salvaKantero", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_ACTION, 0,
	SpectrumGetZipName, SpecBootyrmkbeRomInfo, SpecBootyrmkbeRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Bruce Lee RX (48K) (HB)

static struct BurnRomInfo SpecBruceleerxRomDesc[] = {
	{ "Bruce Lee RX (2019)(highriser).tap", 28795, 0x9dd8a157, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecBruceleerx, SpecBruceleerx, Spectrum)
STD_ROM_FN(SpecBruceleerx)

struct BurnDriver BurnSpecBruceleerx = {
	"spec_bruceleerx", NULL, "spec_spectrum", NULL, "2019",
	"Bruce Lee RX (48K) (HB)\0", NULL, "highriser", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 2, HARDWARE_SPECTRUM, GBF_ACTION, 0,
	SpectrumGetZipName, SpecBruceleerxRomInfo, SpecBruceleerxRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Car Wars (128K) (HB)

static struct BurnRomInfo SpecCarwarsRomDesc[] = {
	{ "Car Wars (2017)(Salva Cantero).tap", 73019, 0x6a59ae25, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecCarwars, SpecCarwars, Spec128)
STD_ROM_FN(SpecCarwars)

struct BurnDriver BurnSpecCarwars = {
	"spec_carwars", NULL, "spec_spec128", NULL, "2017",
	"Car Wars (128K) (HB)\0", NULL, "Salva Cantero", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_MAZE | GBF_ACTION, 0,
	SpectrumGetZipName, SpecCarwarsRomInfo, SpecCarwarsRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Corey Coolbrew (128K) (HB)

static struct BurnRomInfo SpecCoreycbRomDesc[] = {
	{ "Corey Coolbrew (2020)(Blue Bilby).tap", 33670, 0x6f1f52e4, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecCoreycb, SpecCoreycb, Spec128)
STD_ROM_FN(SpecCoreycb)

struct BurnDriver BurnSpecCoreycb = {
	"spec_coreycb", NULL, "spec_spec128", NULL, "2020",
	"Corey Coolbrew (128K) (HB)\0", NULL, "Blue Bilby", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_ACTION, 0,
	SpectrumGetZipName, SpecCoreycbRomInfo, SpecCoreycbRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// The Curse of Trasmoz (128K) (HB)

static struct BurnRomInfo SpecCursetrasmozRomDesc[] = {
	{ "The Curse of Trasmoz (2020)(VolcanoBytes).tap", 48706, 0x8649a0b2, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecCursetrasmoz, SpecCursetrasmoz, Spec128)
STD_ROM_FN(SpecCursetrasmoz)

struct BurnDriver BurnSpecCursetrasmoz = {
	"spec_cursetrasmoz", NULL, "spec_spec128", NULL, "2020",
	"The Curse of Trasmoz (128K) (HB)\0", NULL, "VolcanoBytes", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecCursetrasmozRomInfo, SpecCursetrasmozRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Hyperkill (128K) (HB)

static struct BurnRomInfo SpecHyperkillRomDesc[] = {
	{ "Hyperkill (2017)(Mat Recardo).tap", 42502, 0x382ad799, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecHyperkill, SpecHyperkill, Spec128)
STD_ROM_FN(SpecHyperkill)

struct BurnDriver BurnSpecHyperkill = {
	"spec_hyperkill", NULL, "spec_spec128", NULL, "2017",
	"Hyperkill (128K) (HB)\0", NULL, "Mat Recardo", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_SHOOT, 0,
	SpectrumGetZipName, SpecHyperkillRomInfo, SpecHyperkillRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Jetpack Jock (128K) (HB)

static struct BurnRomInfo SpecJetpackjockRomDesc[] = {
	{ "Jetpack Jock (2020)(Gaz Marshall).tap", 24820, 0x028217db, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecJetpackjock, SpecJetpackjock, Spec128)
STD_ROM_FN(SpecJetpackjock)

struct BurnDriver BurnSpecJetpackjock = {
	"spec_jetpackjock", NULL, "spec_spec128", NULL, "2020",
	"Jetpack Jock (128K) (HB)\0", NULL, "Gaz Marshall", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_ACTION, 0,
	SpectrumGetZipName, SpecJetpackjockRomInfo, SpecJetpackjockRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Mr. Do! (128K) (HB)

static struct BurnRomInfo SpecMrdoRomDesc[] = {
	{ "Mr. Do! (2019)(Adrian Singh).tap", 59653, 0xe3983c0e, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecMrdo, SpecMrdo, Spec128)
STD_ROM_FN(SpecMrdo)

struct BurnDriver BurnSpecMrdo = {
	"spec_mrdo", NULL, "spec_spec128", NULL, "2019",
	"Mr. Do! (128K) (HB)\0", "Arcade conversion", "Adrian Singh", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_ACTION, 0,
	SpectrumGetZipName, SpecMrdoRomInfo, SpecMrdoRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Perils of Willy (128K) (HB)

static struct BurnRomInfo SpecPerilsofwillyRomDesc[] = {
	{ "Perils of Willy (2020)(highriser).tap", 26968, 0x19e22f49, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecPerilsofwilly, SpecPerilsofwilly, Spec128)
STD_ROM_FN(SpecPerilsofwilly)

struct BurnDriver BurnSpecPerilsofwilly = {
	"spec_perilsofwilly", NULL, "spec_spec128", NULL, "2020",
	"Perils of Willy (128K) (HB)\0", NULL, "highriser", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecPerilsofwillyRomInfo, SpecPerilsofwillyRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Rat-A-Tat (128K) (HB)

static struct BurnRomInfo SpecRatatatRomDesc[] = {
	{ "Rat-A-Tat (2020)(JoeSoft).tap", 43232, 0x2d675b93, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecRatatat, SpecRatatat, Spec128)
STD_ROM_FN(SpecRatatat)

struct BurnDriver BurnSpecRatatat = {
	"spec_ratatat", NULL, "spec_spec128", NULL, "2020",
	"Rat-A-Tat (128K) (HB)\0", NULL, "JoeSoft", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecRatatatRomInfo, SpecRatatatRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Baby Monkey Alba (128K) (HB)

static struct BurnRomInfo SpecBabymonkeyalbaRomDesc[] = {
	{ "Baby Monkey Alba (2017)(Javier Quero).tap", 43837, 0x50100dba, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecBabymonkeyalba, SpecBabymonkeyalba, Spec128)
STD_ROM_FN(SpecBabymonkeyalba)

struct BurnDriver BurnSpecBabymonkeyalba = {
	"spec_babymonkeyalba", NULL, "spec_spec128", NULL, "2017",
	"Baby Monkey Alba (128K) (HB)\0", NULL, "Javier Quero", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecBabymonkeyalbaRomInfo, SpecBabymonkeyalbaRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Biscuits In Hell (48K-128K) (HB)

static struct BurnRomInfo SpecBiscuitsihRomDesc[] = {
	{ "Biscuits In Hell (2017)(Monument Microgames).tap", 36588, 0x7fefbfc7, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecBiscuitsih, SpecBiscuitsih, Spec128)
STD_ROM_FN(SpecBiscuitsih)

struct BurnDriver BurnSpecBiscuitsih = {
	"spec_biscuitsih", NULL, "spec_spec128", NULL, "2017",
	"Biscuits In Hell (48K-128K) (HB)\0", NULL, "Monument Microgames", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_ACTION, 0,
	SpectrumGetZipName, SpecBiscuitsihRomInfo, SpecBiscuitsihRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Dingo (48K) (HB, v1.3)

static struct BurnRomInfo SpecDingoRomDesc[] = {
	{ "Dingo (2015)(Sokurah, Tardis).tap", 41567, 0xb22b60c3, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecDingo, SpecDingo, Spectrum)
STD_ROM_FN(SpecDingo)

struct BurnDriver BurnSpecDingo = {
	"spec_dingo", NULL, "spec_spectrum", NULL, "2015",
	"Dingo (48K) (HB, v1.3)\0", NULL, "Tardis Remakes", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_MAZE | GBF_ACTION, 0,
	SpectrumGetZipName, SpecDingoRomInfo, SpecDingoRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// FistRO Fighters (48K) (HB)

static struct BurnRomInfo SpecFistroRomDesc[] = {
	{ "FISTro Fighters (2015)(Alexhino).tap", 48506, 0x1a08f515, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecFistro, SpecFistro, Spectrum)
STD_ROM_FN(SpecFistro)

struct BurnDriver BurnSpecFistro = {
	"spec_fistro", NULL, "spec_spectrum", NULL, "2016",
	"FistRO Fighters (48K) (HB)\0", NULL, "Retrobytes Productions", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_VSFIGHT | GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecFistroRomInfo, SpecFistroRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecQAOPSpaceDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Jet Pack Bob (48K) (HB)

static struct BurnRomInfo SpecJetpackbobRomDesc[] = {
	{ "Jet Pack Bob (2017)(Douglas Bagnall).tap", 28945, 0xb2444927, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecJetpackbob, SpecJetpackbob, Spectrum)
STD_ROM_FN(SpecJetpackbob)

struct BurnDriver BurnSpecJetpackbob = {
	"spec_jetpackbob", NULL, "spec_spectrum", NULL, "2017",
	"Jet Pack Bob (48K) (HB)\0", "A clone of classic Jetpac", "Douglas Bagnall", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_ACTION, 0,
	SpectrumGetZipName, SpecJetpackbobRomInfo, SpecJetpackbobRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecQAOPSpaceDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Monkey J - The Treasure Of The Gold Temple (128K) (HB)

static struct BurnRomInfo SpecMonkeyjRomDesc[] = {
	{ "Monkey J - The Treasure Of The Gold Temple (2017)(Gabriele Amore).tap", 35456, 0xc326ebce, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecMonkeyj, SpecMonkeyj, Spec128)
STD_ROM_FN(SpecMonkeyj)

struct BurnDriver BurnSpecMonkeyj = {
	"spec_monkeyj", NULL, "spec_spec128", NULL, "2017",
	"Monkey J - The Treasure Of The Gold Temple (128K) (HB)\0", NULL, "Gabriele Amore", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_ACTION, 0,
	SpectrumGetZipName, SpecMonkeyjRomInfo, SpecMonkeyjRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Omelettes For Breakfast (128K) (HB)

static struct BurnRomInfo SpecOmelettesRomDesc[] = {
	{ "Omelettes For Breakfast (2017)(Gabriele Amore).tap", 35499, 0xd1706d0b, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecOmelettes, SpecOmelettes, Spec128)
STD_ROM_FN(SpecOmelettes)

struct BurnDriver BurnSpecOmelettes = {
	"spec_omelettes", NULL, "spec_spec128", NULL, "2017",
	"Omelettes For Breakfast (128K) (HB)\0", NULL, "Gabriele Amore", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_ACTION, 0,
	SpectrumGetZipName, SpecOmelettesRomInfo, SpecOmelettesRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Phaeton (48K) (HB)

static struct BurnRomInfo SpecPhaetonRomDesc[] = {
	{ "Phaeton (2010)(Rafat Niazga).tap", 34036, 0x2e9f7f1f, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecPhaeton, SpecPhaeton, Spectrum)
STD_ROM_FN(SpecPhaeton)

struct BurnDriver BurnSpecPhaeton = {
	"spec_phaeton", NULL, "spec_spectrum", NULL, "2010",
	"Phaeton (48K) (HB)\0", NULL, "Rafat Niazga", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_MAZE | GBF_ACTION, 0,
	SpectrumGetZipName, SpecPhaetonRomInfo, SpecPhaetonRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Rabbit in Nightmareland (128K) (HB)

static struct BurnRomInfo SpecRabbitinnRomDesc[] = {
	{ "Rabbit in Nightmareland (2015)(Javier Fopiani).tap", 69594, 0xa0123467, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecRabbitinn, SpecRabbitinn, Spec128)
STD_ROM_FN(SpecRabbitinn)

struct BurnDriver BurnSpecRabbitinn = {
	"spec_rabbitinn", NULL, "spec_spec128", NULL, "2015",
	"Rabbit in Nightmareland (128K) (HB)\0", NULL, "Javier Fopiani", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecRabbitinnRomInfo, SpecRabbitinnRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Retroforce (128K) (HB)

static struct BurnRomInfo SpecRetroforceRomDesc[] = {
	{ "Retroforce (2017)(Climacus & KgMcNeil).tap", 64323, 0xc1c7cf3e, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecRetroforce, SpecRetroforce, Spec128)
STD_ROM_FN(SpecRetroforce)

struct BurnDriver BurnSpecRetroforce = {
	"spec_retroforce", NULL, "spec_spec128", NULL, "2017",
	"Retroforce (128K) (HB)\0", NULL, "Climacus & KgMcNeil", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_VERSHOOT, 0,
	SpectrumGetZipName, SpecRetroforceRomInfo, SpecRetroforceRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Springbot - Mars Attack! (48K) (HB, v1.3)

static struct BurnRomInfo SpecSpringbotRomDesc[] = {
	{ "Springbot - Mars Attack! (2020)(Andy Farrell).tap", 44477, 0x5345e0ec, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecSpringbot, SpecSpringbot, Spectrum)
STD_ROM_FN(SpecSpringbot)

struct BurnDriver BurnSpecSpringbot = {
	"spec_springbot", NULL, "spec_spectrum", NULL, "2020",
	"Springbot - Mars Attack! (48K) (HB, v1.3)\0", NULL, "Andy Farrell", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecSpringbotRomInfo, SpecSpringbotRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// UFO (48K) (HB)

static struct BurnRomInfo SpecUfoRomDesc[] = {
	{ "UFO (2020)(F.J. Urbaneja).tap", 33701, 0x44cb0dbb, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecUfo, SpecUfo, Spectrum)
STD_ROM_FN(SpecUfo)

struct BurnDriver BurnSpecUfo = {
	"spec_ufo", NULL, "spec_spectrum", NULL, "2020",
	"UFO (48K) (HB)\0", NULL, "F.J. Urbaneja", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecUfoRomInfo, SpecUfoRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Los Amores De Brunilda (English) (128K) (HB, v1.4)

static struct BurnRomInfo SpecBrunildaRomDesc[] = {
	{ "Los Amores De Brunilda (2013)(RetroWorks).tap", 85072, 0xbfa8a647, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecBrunilda, SpecBrunilda, Spec128)
STD_ROM_FN(SpecBrunilda)

struct BurnDriver BurnSpecBrunilda = {
	"spec_brunilda", NULL, "spec_spec128", NULL, "2013",
	"Los Amores De Brunilda (English) (128K) (HB, v1.4)\0", NULL, "RetroWorks", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_RPG, 0,
	SpectrumGetZipName, SpecBrunildaRomInfo, SpecBrunildaRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Captain Drexx (128K) (HB)

static struct BurnRomInfo SpecCaptaindrexxRomDesc[] = {
	{ "Captain Drexx (2014)(Vladimir Burenko).tap", 38766, 0x1c7be669, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecCaptaindrexx, SpecCaptaindrexx, Spec128)
STD_ROM_FN(SpecCaptaindrexx)

struct BurnDriver BurnSpecCaptaindrexx = {
	"spec_captaindrexx", NULL, "spec_spec128", NULL, "2014",
	"Captain Drexx (128K) (HB)\0", "Tower Defence subgenre", "Vladimir Burenko", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_STRATEGY, 0,
	SpectrumGetZipName, SpecCaptaindrexxRomInfo, SpecCaptaindrexxRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecIntf2DIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Complica DX (128K) (HB)

static struct BurnRomInfo SpecComplicadxRomDesc[] = {
	{ "Complica DX (2015)(Einar Saukas).tap", 48389, 0x4a72648f, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecComplicadx, SpecComplicadx, Spec128)
STD_ROM_FN(SpecComplicadx)

struct BurnDriver BurnSpecComplicadx = {
	"spec_complicadx", NULL, "spec_spec128", NULL, "2015",
	"Complica DX (128K) (HB)\0", "Use SPACE and ENTER in menu and 1-2-3-4 keys in game", "Einar Saukas", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 2, HARDWARE_SPECTRUM, GBF_PUZZLE, 0,
	SpectrumGetZipName, SpecComplicadxRomInfo, SpecComplicadxRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Snake Escape (128K) (HB)

static struct BurnRomInfo SpecSnakeescapeRomDesc[] = {
	{ "Snake Escape (2016)(Einar Saukas).tap", 39093, 0x870cc8cd, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecSnakeescape, SpecSnakeescape, Spec128)
STD_ROM_FN(SpecSnakeescape)

struct BurnDriver BurnSpecSnakeescape = {
	"spec_snakeescape", NULL, "spec_spec128", NULL, "2016",
	"Snake Escape (128K) (HB)\0", NULL, "Einar Saukas", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_ACTION, 0,
	SpectrumGetZipName, SpecSnakeescapeRomInfo, SpecSnakeescapeRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// splATTR (128K) (HB)

static struct BurnRomInfo SpecSplattrRomDesc[] = {
	{ "splATTR (2008)(Bob Smith).tap", 87068, 0xc1c7b410, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecSplattr, SpecSplattr, Spec128)
STD_ROM_FN(SpecSplattr)

struct BurnDriver BurnSpecSplattr = {
	"spec_splattr", NULL, "spec_spec128", NULL, "2008",
	"splATTR (128K) (HB)\0", NULL, "Bob Smith", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_ACTION, 0,
	SpectrumGetZipName, SpecSplattrRomInfo, SpecSplattrRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Sun Bucket (128K) (HB)

static struct BurnRomInfo SpecSunbucketRomDesc[] = {
	{ "Sun Bucket (2014)(Stonechat).tap", 44159, 0xd9761fa3, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecSunbucket, SpecSunbucket, Spec128)
STD_ROM_FN(SpecSunbucket)

struct BurnDriver BurnSpecSunbucket = {
	"spec_sunbucket", NULL, "spec_spec128", NULL, "2014",
	"Sun Bucket (128K) (HB)\0", NULL, "Stonechat", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PUZZLE | GBF_ACTION, 0,
	SpectrumGetZipName, SpecSunbucketRomInfo, SpecSunbucketRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecQAOPSpaceDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// The Dark (48K) (HB)

static struct BurnRomInfo SpecThedarkRomDesc[] = {
	{ "The Dark (1997)(Oleg Origin).tap", 42940, 0x8b93b94c, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecThedark, SpecThedark, Spectrum)
STD_ROM_FN(SpecThedark)

struct BurnDriver BurnSpecThedark = {
	"spec_thedark", NULL, "spec_spectrum", NULL, "1997",
	"The Dark (48K) (HB)\0", NULL, "Oleg Origin", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_MAZE | GBF_ADV, 0,
	SpectrumGetZipName, SpecThedarkRomInfo, SpecThedarkRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// HELL YEAH! (48K) (HB)

static struct BurnRomInfo SpecHellyeahRomDesc[] = {
	{ "HELL YEAH! (2020)(Andy Precious).tap", 47848, 0x064685ab, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecHellyeah, SpecHellyeah, Spectrum)
STD_ROM_FN(SpecHellyeah)

struct BurnDriver BurnSpecHellyeah = {
	"spec_hellyeah", NULL, "spec_spectrum", NULL, "2020",
	"HELL YEAH! (48K) (HB)\0", NULL, "Andy Precious", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_RUNGUN, 0,
	SpectrumGetZipName, SpecHellyeahRomInfo, SpecHellyeahRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// AtomiCat (48K) (HB)

static struct BurnRomInfo SpecAtomicatRomDesc[] = {
	{ "AtomiCat (2020)(Poe Games).tap", 40363, 0xe4f56a9d, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecAtomicat, SpecAtomicat, Spectrum)
STD_ROM_FN(SpecAtomicat)

struct BurnDriver BurnSpecAtomicat = {
	"spec_atomicat", NULL, "spec_spectrum", NULL, "2020",
	"AtomiCat (48K) (HB)\0", NULL, "Poe Games", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecAtomicatRomInfo, SpecAtomicatRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Bat Boy (128K) (HB)

static struct BurnRomInfo SpecBatboyRomDesc[] = {
	{ "Bat Boy (2020)(Antonio Perez).tap", 55621, 0x354abdb7, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecBatboy, SpecBatboy, Spec128)
STD_ROM_FN(SpecBatboy)

struct BurnDriver BurnSpecBatboy = {
	"spec_batboy", NULL, "spec_spec128", NULL, "2020",
	"Bat Boy (128K) (HB)\0", NULL, "Pat Morita Team", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecBatboyRomInfo, SpecBatboyRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Bonnie and Clyde (128K) (HB)

static struct BurnRomInfo SpecBonnieclydeRomDesc[] = {
	{ "Bonnie and Clyde (1986)(Zosya).tap", 47115, 0xf075ba62, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecBonnieclyde, SpecBonnieclyde, Spec128)
STD_ROM_FN(SpecBonnieclyde)

struct BurnDriver BurnSpecBonnieclyde = {
	"spec_bonnieclyde", NULL, "spec_spec128", NULL, "2020",
	"Bonnie and Clyde (128K) (HB)\0", NULL, "Zosya Entertainment", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecBonnieclydeRomInfo, SpecBonnieclydeRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Patrick Paddle (48K) (HB)

static struct BurnRomInfo SpecPatrickpaddleRomDesc[] = {
	{ "Patrick Paddle (2020)(PROSM Software).tap", 13883, 0x952f6855, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecPatrickpaddle, SpecPatrickpaddle, Spectrum)
STD_ROM_FN(SpecPatrickpaddle)

struct BurnDriver BurnSpecPatrickpaddle = {
	"spec_patrickpaddle", NULL, "spec_spectrum", NULL, "2020",
	"Patrick Paddle (48K) (HB)\0", NULL, "PROSM Software", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_ACTION, 0,
	SpectrumGetZipName, SpecPatrickpaddleRomInfo, SpecPatrickpaddleRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Trinidad (128K) (HB)

static struct BurnRomInfo SpecTrinidadRomDesc[] = {
	{ "Trinidad (2020)(Polybius).tap", 43020, 0x4879e658, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecTrinidad, SpecTrinidad, Spec128)
STD_ROM_FN(SpecTrinidad)

struct BurnDriver BurnSpecTrinidad = {
	"spec_trinidad", NULL, "spec_spec128", NULL, "2020",
	"Trinidad (128K) (HB)\0", NULL, "Polybius", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_RUNGUN | GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecTrinidadRomInfo, SpecTrinidadRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Albert The Wolf (48K) (HB)

static struct BurnRomInfo SpecAlbertwolfRomDesc[] = {
	{ "Albert The Wolf (2014)(Gabriele Amore).tap", 33984, 0xae388913, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecAlbertwolf, SpecAlbertwolf, Spectrum)
STD_ROM_FN(SpecAlbertwolf)

struct BurnDriver BurnSpecAlbertwolf = {
	"spec_albertwolf", NULL, "spec_spectrum", NULL, "2014",
	"Albert The Wolf (48K) (HB)\0", NULL, "Gabriele Amore", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecAlbertwolfRomInfo, SpecAlbertwolfRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Bandito (48K) (HB)

static struct BurnRomInfo SpecBanditoRomDesc[] = {
	{ "Bandito (2020)(Andy McDermott).tap", 34018, 0x3591be21, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecBandito, SpecBandito, Spectrum)
STD_ROM_FN(SpecBandito)

struct BurnDriver BurnSpecBandito = {
	"spec_bandito", NULL, "spec_spectrum", NULL, "2020",
	"Bandito (48K) (HB)\0", NULL, "Andy McDermott", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_RUNGUN, 0,
	SpectrumGetZipName, SpecBanditoRomInfo, SpecBanditoRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecQAOPSpaceDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Blizzard's Rift (128K) (HB)

static struct BurnRomInfo SpecBlizzardsriftRomDesc[] = {
	{ "Blizzard's Rift (2007)(Jonathan Cauldwell).tap", 66068, 0x884eef54, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecBlizzardsrift, SpecBlizzardsrift, Spec128)
STD_ROM_FN(SpecBlizzardsrift)

struct BurnDriver BurnSpecBlizzardsrift = {
	"spec_blizzardsrift", NULL, "spec_spec128", NULL, "2007",
	"Blizzard's Rift (128K) (HB)\0", NULL, "Jonathan Cauldwell", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_MINIGAMES, 0,
	SpectrumGetZipName, SpecBlizzardsriftRomInfo, SpecBlizzardsriftRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Buzzsaw+ (48K) (HB)

static struct BurnRomInfo SpecBuzzsawRomDesc[] = {
	{ "Buzzsaw+ (2011)(Jason J. Railton).tap", 42988, 0xf066f06d, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecBuzzsaw, SpecBuzzsaw, Spectrum)
STD_ROM_FN(SpecBuzzsaw)

struct BurnDriver BurnSpecBuzzsaw = {
	"spec_buzzsaw", NULL, "spec_spectrum", NULL, "2011",
	"Buzzsaw+ (48K) (HB)\0", NULL, "Jason J. Railton", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PUZZLE, 0,
	SpectrumGetZipName, SpecBuzzsawRomInfo, SpecBuzzsawRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// C is for Crabby (48K) (HB)

static struct BurnRomInfo SpecCcrabbyRomDesc[] = {
	{ "C is for Crabby (2013)(Gabriel Amore).tap", 41821, 0x490a07d9, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecCcrabby, SpecCcrabby, Spectrum)
STD_ROM_FN(SpecCcrabby)

struct BurnDriver BurnSpecCcrabby = {
	"spec_ccrabby", NULL, "spec_spectrum", NULL, "2013",
	"C is for Crabby (48K) (HB)\0", NULL, "Gabriele Amore", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_ACTION, 0,
	SpectrumGetZipName, SpecCcrabbyRomInfo, SpecCcrabbyRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecQAOPSpaceDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Castlevania - Spectral Interlude (English) (128K) (HB)

static struct BurnRomInfo SpeccastlevaniaRomDesc[] = {
	{ "Castlevania - Spectral Interlude (2015)(Rewind Team).tap", 0x1edba, 0xc100bb38, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Speccastlevania, Speccastlevania, Spec128)
STD_ROM_FN(Speccastlevania)

struct BurnDriver BurnSpeccastlevania = {
	"spec_castlevania", NULL, "spec_spec128", NULL, "2015",
	"Castlevania - Spectral Interlude (English) (128K) (HB)\0", NULL, "Rewind Team", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_RUNGUN | GBF_PLATFORM, 0,
	SpectrumGetZipName, SpeccastlevaniaRomInfo, SpeccastlevaniaRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Castlevania - Spectral Interlude (Russian) (128K) (HB)

static struct BurnRomInfo SpeccastlevanrusRomDesc[] = {
	{ "Castlevania - Spectral Interlude (2015)(Rewind Team)(Rus).tap", 0x1f04e, 0x45561b70, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Speccastlevanrus, Speccastlevanrus, Spec128)
STD_ROM_FN(Speccastlevanrus)

struct BurnDriver BurnSpeccastlevanrus = {
	"spec_castlevanrus", "spec_castlevania", "spec_spec128", NULL, "2015",
	"Castlevania - Spectral Interlude (Russian) (128K) (HB)\0", NULL, "Rewind Team", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_RUNGUN | GBF_PLATFORM, 0,
	SpectrumGetZipName, SpeccastlevanrusRomInfo, SpeccastlevanrusRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Chip Rescue (Russian) (128K) (HB)

static struct BurnRomInfo SpecChiprescueRomDesc[] = {
	{ "Chip Rescue (2019)(romancha).tap", 19111, 0x917b33c6, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecChiprescue, SpecChiprescue, Spec128)
STD_ROM_FN(SpecChiprescue)

struct BurnDriver BurnSpecChiprescue = {
	"spec_chiprescue", NULL, "spec_spec128", NULL, "2019",
	"Chip Rescue (Russian) (128K) (HB)\0", "Press 0 to start", "Romancha", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecChiprescueRomInfo, SpecChiprescueRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecIntf2DIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Chopper Drop (128K) (HB, v1.1)

static struct BurnRomInfo SpecChopperdropRomDesc[] = {
	{ "Chopper Drop (2011)(Paul Jenkinson).tap", 31591, 0x8c4eb3fe, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecChopperdrop, SpecChopperdrop, Spec128)
STD_ROM_FN(SpecChopperdrop)

struct BurnDriver BurnSpecChopperdrop = {
	"spec_chopperdrop", NULL, "spec_spec128", NULL, "2011",
	"Chopper Drop (128K) (HB, v1.1)\0", NULL, "Paul Jenkinson", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_ACTION, 0,
	SpectrumGetZipName, SpecChopperdropRomInfo, SpecChopperdropRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecQAOPSpaceDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Droid Buster (128K) (HB)

static struct BurnRomInfo SpecDroidbusterRomDesc[] = {
	{ "Droid Buster (2019)(Ariel Endaraues).tap", 45669, 0x4c635bbe, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecDroidbuster, SpecDroidbuster, Spec128)
STD_ROM_FN(SpecDroidbuster)

struct BurnDriver BurnSpecDroidbuster = {
	"spec_droidbuster", NULL, "spec_spec128", NULL, "2019",
	"Droid Buster (128K) (HB)\0", NULL, "Ariel Endaraues", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_ACTION, 0,
	SpectrumGetZipName, SpecDroidbusterRomInfo, SpecDroidbusterRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Fireman (48K) (HB)

static struct BurnRomInfo SpecFiremanRomDesc[] = {
	{ "Fireman (2008)(Miguel Angel Rodriguez Jodar).tap", 18452, 0xf7853e31, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecFireman, SpecFireman, Spectrum)
STD_ROM_FN(SpecFireman)

struct BurnDriver BurnSpecFireman = {
	"spec_fireman", NULL, "spec_spectrum", NULL, "2008",
	"Fireman (48K) (HB)\0", NULL, "Miguel Angel Rodriguez Jodar", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_ACTION, 0,
	SpectrumGetZipName, SpecFiremanRomInfo, SpecFiremanRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Lost Tapes of Albion, The (48K) (HB)

static struct BurnRomInfo SpecLosttapesRomDesc[] = {
	{ "The Lost Tapes of Albion (2012)(Stonechat Productions).tap", 25472, 0x0f3caf6e, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecLosttapes, SpecLosttapes, Spectrum)
STD_ROM_FN(SpecLosttapes)

struct BurnDriver BurnSpecLosttapes = {
	"spec_losttapes", NULL, "spec_spectrum", NULL, "2012",
	"Lost Tapes of Albion, The (48K) (HB)\0", NULL, "Stonechat Productions", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_ACTION, 0,
	SpectrumGetZipName, SpecLosttapesRomInfo, SpecLosttapesRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecQAOPSpaceDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Space Monsters meet The Hardy (128K) (HB)

static struct BurnRomInfo SpecThehardyRomDesc[] = {
	{ "Space Monsters meet The Hardy (2019)(Mayhem).tap", 87134, 0x96c02acc, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecThehardy, SpecThehardy, Spec128)
STD_ROM_FN(SpecThehardy)

struct BurnDriver BurnSpecThehardy = {
	"spec_thehardy", NULL, "spec_spec128", NULL, "2019",
	"Space Monsters meet The Hardy (128K) (HB)\0", NULL, "Mayhem", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecThehardyRomInfo, SpecThehardyRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Trooper Point 5 (128K) (HB)

static struct BurnRomInfo SpecTrooperpoint5RomDesc[] = {
	{ "Trooper Point 5 (2010)(The Bog Brothers).tap", 38475, 0xd17b61d0, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecTrooperpoint5, SpecTrooperpoint5, Spec128)
STD_ROM_FN(SpecTrooperpoint5)

struct BurnDriver BurnSpecTrooperpoint5 = {
	"spec_trooperpoint5", NULL, "spec_spec128", NULL, "2010",
	"Trooper Point 5 (128K) (HB)\0", NULL, "The Bog Brothers", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_ACTION, 0,
	SpectrumGetZipName, SpecTrooperpoint5RomInfo, SpecTrooperpoint5RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecIntf2DIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Generacio Digital (128K) (HB)

static struct BurnRomInfo SpecGenerdigitalRomDesc[] = {
	{ "Generacio Digital (2019)(Catalunya Radio).tap", 39117, 0x76ec9f7d, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecGenerdigital, SpecGenerdigital, Spec128)
STD_ROM_FN(SpecGenerdigital)

struct BurnDriver BurnSpecGenerdigital = {
	"spec_generdigital", NULL, "spec_spec128", NULL, "2019",
	"Generacio Digital (128K) (HB)\0", NULL, "Catalunya Radio", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_MAZE | GBF_ACTION, 0,
	SpectrumGetZipName, SpecGenerdigitalRomInfo, SpecGenerdigitalRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Space Journey (48K) (HB)

static struct BurnRomInfo SpecSpacejourneyRomDesc[] = {
	{ "Space Journey (2020)(Roman Cikryt).tap", 46896, 0xb55f0534, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecSpacejourney, SpecSpacejourney, Spec128)
STD_ROM_FN(SpecSpacejourney)

struct BurnDriver BurnSpecSpacejourney = {
	"spec_spacejourney", NULL, "spec_spec128", NULL, "2020",
	"Space Journey (48K) (HB)\0", NULL, "Roman Cikryt", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_HORSHOOT, 0,
	SpectrumGetZipName, SpecSpacejourneyRomInfo, SpecSpacejourneyRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecQAOPMDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Farmer Jack in Harvest Havoc (128K) (HB)

static struct BurnRomInfo SpecFjhhRomDesc[] = {
	{ "Farmer Jack in Harvest Havoc (2006)(Bob Smith).tap", 39223, 0x936f2345, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecFjhh, SpecFjhh, Spec128)
STD_ROM_FN(SpecFjhh)

struct BurnDriver BurnSpecFjhh = {
	"spec_fjhh", NULL, "spec_spec128", NULL, "2006",
	"Farmer Jack in Harvest Havoc (128K) (HB)\0", NULL, "Bob Smith", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_ACTION, 0,
	SpectrumGetZipName, SpecFjhhRomInfo, SpecFjhhRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Farmer Jack and the Hedge Monkeys (128K) (HB)

static struct BurnRomInfo SpecFjhmRomDesc[] = {
	{ "Farmer Jack and the Hedge Monkeys (2008)(Bob Smith).tap", 48405, 0xde828b58, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecFjhm, SpecFjhm, Spec128)
STD_ROM_FN(SpecFjhm)

struct BurnDriver BurnSpecFjhm = {
	"spec_fjhm", NULL, "spec_spec128", NULL, "2008",
	"Farmer Jack and the Hedge Monkeys (128K) (HB)\0", NULL, "Bob Smith", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_MAZE | GBF_ACTION, 0,
	SpectrumGetZipName, SpecFjhmRomInfo, SpecFjhmRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Jilly's Farm Volume 1 - SokoBAArn (48K) (HB, v1.01)

static struct BurnRomInfo SpecJillysfarm1RomDesc[] = {
	{ "Jilly's Farm Volume 1 - SokoBAArn (2018)(Bob Smith).tap", 47687, 0x52c86488, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecJillysfarm1, SpecJillysfarm1, Spectrum)
STD_ROM_FN(SpecJillysfarm1)

struct BurnDriver BurnSpecJillysfarm1 = {
	"spec_jillysfarm1", NULL, "spec_spectrum", NULL, "2018",
	"Jilly's Farm Volume 1 - SokoBAArn (48K) (HB, v1.01)\0", NULL, "Bob Smith", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_ACTION | GBF_PUZZLE, 0,
	SpectrumGetZipName, SpecJillysfarm1RomInfo, SpecJillysfarm1RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Night Stalker ZX (48K) (HB)

static struct BurnRomInfo SpecNightstalkerRomDesc[] = {
	{ "Night Stalker ZX (2018)(AMCgames).tap", 34047, 0x6c112118, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecNightstalker, SpecNightstalker, Spectrum)
STD_ROM_FN(SpecNightstalker)

struct BurnDriver BurnSpecNightstalker = {
	"spec_nightstalker", NULL, "spec_spectrum", NULL, "2018",
	"Night Stalker ZX (48K) (HB)\0", NULL, "AMCgames", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_MAZE | GBF_ACTION, 0,
	SpectrumGetZipName, SpecNightstalkerRomInfo, SpecNightstalkerRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Ninja Gardening-Simulator (128K) (HB)

static struct BurnRomInfo SpecNinjagardensimRomDesc[] = {
	{ "Ninja Gardening-Simulator (2019)(Andrew Bunker).tap", 45008, 0x80d96a46, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecNinjagardensim, SpecNinjagardensim, Spec128)
STD_ROM_FN(SpecNinjagardensim)

struct BurnDriver BurnSpecNinjagardensim = {
	"spec_ninjagardensim", NULL, "spec_spec128", NULL, "2019",
	"Ninja Gardening-Simulator (128K) (HB)\0", NULL, "Andrew Bunker", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_ACTION, 0,
	SpectrumGetZipName, SpecNinjagardensimRomInfo, SpecNinjagardensimRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Red Planet (English) (48K) (HB)

static struct BurnRomInfo SpecRedplanetRomDesc[] = {
	{ "Red Planet (2015)(salvakantero).tap", 42978, 0x6b8baac0, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecRedplanet, SpecRedplanet, Spectrum)
STD_ROM_FN(SpecRedplanet)

struct BurnDriver BurnSpecRedplanet = {
	"spec_redplanet", NULL, "spec_spectrum", NULL, "2015",
	"Red Planet (English) (48K) (HB)\0", NULL, "Salva Cantero", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecRedplanetRomInfo, SpecRedplanetRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Sc0tb0t (48K) (HB)

static struct BurnRomInfo SpecSc0tb0tRomDesc[] = {
	{ "Sc0tb0t (2020)(Dougie mcg).tap", 37704, 0x8848666f, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecSc0tb0t, SpecSc0tb0t, Spectrum)
STD_ROM_FN(SpecSc0tb0t)

struct BurnDriver BurnSpecSc0tb0t = {
	"spec_sc0tb0t", NULL, "spec_spectrum", NULL, "2020",
	"Sc0tb0t (48K) (HB)\0", NULL, "Dougie mcg", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecSc0tb0tRomInfo, SpecSc0tb0tRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Stranded 2.5 (128K) (HB)

static struct BurnRomInfo SpecStranded25RomDesc[] = {
	{ "Stranded 2.5 (2007)(Bob Smith).tap", 38037, 0x7b9f1cec, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecStranded25, SpecStranded25, Spec128)
STD_ROM_FN(SpecStranded25)

struct BurnDriver BurnSpecStranded25 = {
	"spec_stranded25", NULL, "spec_spec128", NULL, "2007",
	"Stranded 2.5 (128K) (HB)\0", NULL, "Cronosoft", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PUZZLE, 0,
	SpectrumGetZipName, SpecStranded25RomInfo, SpecStranded25RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// X=Y=Z (48K) (HB)

static struct BurnRomInfo SpecXyzRomDesc[] = {
	{ "X=Y=Z (2014)(Bob Smith).tap", 18555, 0x44da8f4e, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecXyz, SpecXyz, Spectrum)
STD_ROM_FN(SpecXyz)

struct BurnDriver BurnSpecXyz = {
	"spec_xyz", NULL, "spec_spectrum", NULL, "2014",
	"X=Y=Z (48K) (HB)\0", NULL, "Bob Smith", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PUZZLE, 0,
	SpectrumGetZipName, SpecXyzRomInfo, SpecXyzRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Bullet Storm (128K) (HB)

static struct BurnRomInfo SpecBulletstormRomDesc[] = {
	{ "Bullet Storm (2020)(VolatiL).tap", 48736, 0x16a8ccf9, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecBulletstorm, SpecBulletstorm, Spec128)
STD_ROM_FN(SpecBulletstorm)

struct BurnDriver BurnSpecBulletstorm = {
	"spec_bulletstorm", NULL, "spec_spec128", NULL, "2020",
	"Bullet Storm (128K) (HB)\0", NULL, "VolatiL", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_HORSHOOT, 0,
	SpectrumGetZipName, SpecBulletstormRomInfo, SpecBulletstormRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecQAOPSpaceDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Captain Gofer (Russian) (128K) (HB)

static struct BurnRomInfo SpecCaptaingoferRomDesc[] = {
	{ "Captain Gofer (2020)(Dwa83).z80", 27917, 0xf843ec65, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecCaptaingofer, SpecCaptaingofer, Spec128)
STD_ROM_FN(SpecCaptaingofer)

struct BurnDriver BurnSpecCaptaingofer = {
	"spec_captaingofer", NULL, "spec_spec128", NULL, "2020",
	"Captain Gofer (Russian) (128K) (HB)\0", NULL, "Dwa83", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_RUNGUN, 0,
	SpectrumGetZipName, SpecCaptaingoferRomInfo, SpecCaptaingoferRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Code-112 (128K) (HB)

static struct BurnRomInfo SpecCode112RomDesc[] = {
	{ "Code-112 (2020)(PCNONOGames).tap", 47864, 0xd3a6349d, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecCode112, SpecCode112, Spec128)
STD_ROM_FN(SpecCode112)

struct BurnDriver BurnSpecCode112 = {
	"spec_code112", NULL, "spec_spec128", NULL, "2020",
	"Code-112 (128K) (HB)\0", NULL, "PC NONO Games", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_ACTION | GBF_PUZZLE, 0,
	SpectrumGetZipName, SpecCode112RomInfo, SpecCode112RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Cosmic Payback (128K) (HB)

static struct BurnRomInfo SpecCosmicpaybackRomDesc[] = {
	{ "Cosmic Payback (2020)(John Connolly).tap", 58570, 0xc91aa72f, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecCosmicpayback, SpecCosmicpayback, Spec128)
STD_ROM_FN(SpecCosmicpayback)

struct BurnDriver BurnSpecCosmicpayback = {
	"spec_cosmicpayback", NULL, "spec_spec128", NULL, "2020",
	"Cosmic Payback (128K) (HB)\0", NULL, "John Connolly", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING| BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_ACTION, 0,
	SpectrumGetZipName, SpecCosmicpaybackRomInfo, SpecCosmicpaybackRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Cygnus - Alpha (128K) (HB)

static struct BurnRomInfo SpecCygnusalphaRomDesc[] = {
	{ "Cygnus - Alpha (2020)(ILFORD).z80", 50933, 0x2096d6d4, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecCygnusalpha, SpecCygnusalpha, Spec128)
STD_ROM_FN(SpecCygnusalpha)

struct BurnDriver BurnSpecCygnusalpha = {
	"spec_cygnusalpha", NULL, "spec_spec128", NULL, "2020",
	"Cygnus - Alpha (128K) (HB)\0", NULL, "Steve A.J. Broad", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_SHOOT | GBF_SIM, 0,
	SpectrumGetZipName, SpecCygnusalphaRomInfo, SpecCygnusalphaRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Enigmatik (48K) (HB)

static struct BurnRomInfo SpecEnignimatikRomDesc[] = {
	{ "Enigmatik (2020)(IADVD).tap", 45841, 0xc02dcd01, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecEnignimatik, SpecEnignimatik, Spectrum)
STD_ROM_FN(SpecEnignimatik)

struct BurnDriver BurnSpecEnignimatik = {
	"spec_enignimatik", NULL, "spec_spectrum", NULL, "2020",
	"Enigmatik (48K) (HB)\0", NULL, "iadvd - Molomazo", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PUZZLE | GBF_QUIZ, 0,
	SpectrumGetZipName, SpecEnignimatikRomInfo, SpecEnignimatikRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecQAOPSpaceDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Marsmare - Alienation (128K) (HB)

static struct BurnRomInfo SpecMarsmarealienationRomDesc[] = {
	{ "Marsmare - Alienation (2020)(Drunk Fly).tap", 115080, 0x906f7098, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecMarsmarealienation, SpecMarsmarealienation, Spec128)
STD_ROM_FN(SpecMarsmarealienation)

struct BurnDriver BurnSpecMarsmarealienation = {
	"spec_marsmarealienation", NULL, "spec_spec128", NULL, "2020",
	"Marsmare - Alienation (128K) (HB)\0", NULL, "Drunk Fly", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING| BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecMarsmarealienationRomInfo, SpecMarsmarealienationRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Red Raid - The Beginning - Side A (48K) (HB)

static struct BurnRomInfo SpecRedraidbeg1RomDesc[] = {
	{ "Red Raid The Beginning - Side A (2020)(ZX Bitles).tap", 46030, 0xbb242173, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecRedraidbeg1, SpecRedraidbeg1, Spectrum)
STD_ROM_FN(SpecRedraidbeg1)

struct BurnDriver BurnSpecRedraidbeg1 = {
	"spec_redraidbeg1", NULL, "spec_spectrum", NULL, "2020",
	"Red Raid - The Beginning - Side A (48K) (HB)\0", NULL, "ZX Bitles", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PLATFORM | GBF_ADV, 0,
	SpectrumGetZipName, SpecRedraidbeg1RomInfo, SpecRedraidbeg1RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecIntf2SwapDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Red Raid - The Beginning - Side B (48K) (HB)

static struct BurnRomInfo SpecRedraidbeg2RomDesc[] = {
	{ "Red Raid The Beginning - Side B (2020)(ZX Bitles).tap", 47114, 0xfa5b7f8d, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecRedraidbeg2, SpecRedraidbeg2, Spectrum)
STD_ROM_FN(SpecRedraidbeg2)

struct BurnDriver BurnSpecRedraidbeg2 = {
	"spec_redraidbeg2", "spec_redraidbeg1", "spec_spectrum", NULL, "2020",
	"Red Raid - The Beginning - Side B (48K) (HB)\0", NULL, "ZX Bitles", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PLATFORM | GBF_ADV, 0,
	SpectrumGetZipName, SpecRedraidbeg2RomInfo, SpecRedraidbeg2RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecIntf2SwapDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Sixsixsixsix (Russian) (48K) (HB)

static struct BurnRomInfo SpecSixsixsixsixRomDesc[] = {
	{ "Sixsixsixsix (2020)(Anhot Studio).tap", 42560, 0xaaae62a6, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecSixsixsixsix, SpecSixsixsixsix, Spectrum)
STD_ROM_FN(SpecSixsixsixsix)

struct BurnDriver BurnSpecSixsixsixsix = {
	"spec_sixsixsixsix", NULL, "spec_spectrum", NULL, "2020",
	"Sixsixsixsix (Russian) (48K) (HB)\0", NULL, "Anhot Studio", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_MISC, 0,
	SpectrumGetZipName, SpecSixsixsixsixRomInfo, SpecSixsixsixsixRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Techno Alice (Russian) (48K) (HB)

static struct BurnRomInfo SpecTechnoaliceRomDesc[] = {
	{ "Techno Alice (2020)(ALKO).tap", 44226, 0x6e631df4, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecTechnoalice, SpecTechnoalice, Spectrum)
STD_ROM_FN(SpecTechnoalice)

struct BurnDriver BurnSpecTechnoalice = {
	"spec_technoalice", NULL, "spec_spectrum", NULL, "2020",
	"Techno Alice (Russian) (48K) (HB)\0", NULL, "ALKO", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING| BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecTechnoaliceRomInfo, SpecTechnoaliceRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Yoyo's Great Adventure (128K) (HB)

static struct BurnRomInfo SpecYoyogreatadventureRomDesc[] = {
	{ "Yoyo's Great Adventure (2020)(Rafal Miazga).tap", 48287, 0xa4f5b9b9, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecYoyogreatadventure, SpecYoyogreatadventure, Spec128)
STD_ROM_FN(SpecYoyogreatadventure)

struct BurnDriver BurnSpecYoyogreatadventure = {
	"spec_yoyogreatadventure", NULL, "spec_spec128", NULL, "2020",
	"Yoyo's Great Adventure (128K) (HB)\0", NULL, "Rafal Miazga", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PLATFORM | GBF_ADV, 0,
	SpectrumGetZipName, SpecYoyogreatadventureRomInfo, SpecYoyogreatadventureRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Dizzy VIII - Wonderful Dizzy (128K) (HB)

static struct BurnRomInfo SpecDizzy8RomDesc[] = {
	{ "Dizzy VIII - Wonderful Dizzy (2020)(Team Yolkfolk).tap", 116323, 0x0f8b91f8, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecDizzy8, SpecDizzy8, Spec128)
STD_ROM_FN(SpecDizzy8)

struct BurnDriver BurnSpecDizzy8 = {
	"spec_dizzy8", NULL, "spec_spec128", NULL, "2020",
	"Dizzy VIII - Wonderful Dizzy (128K) (HB)\0", NULL, "Team Yolkfolk", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PLATFORM | GBF_ADV, 0,
	SpectrumGetZipName, SpecDizzy8RomInfo, SpecDizzy8RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Mousetrap (48K) (HB)

static struct BurnRomInfo SpecMousetrapRomDesc[] = {
	{ "Mousetrap (2020)(Chris Maling).tap", 10185, 0x170bd6b3, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecMousetrap, SpecMousetrap, Spec128)
STD_ROM_FN(SpecMousetrap)

struct BurnDriver BurnSpecMousetrap = {
	"spec_mousetrap", NULL, "spec_spec128", NULL, "2020",
	"Mousetrap (48K) (HB)\0", NULL, "Chris Maling", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_ACTION, 0,
	SpectrumGetZipName, SpecMousetrapRomInfo, SpecMousetrapRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecQAOPSpaceDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Roger the Pangolin in 2020 Knurled Tour (48K) (HB)

static struct BurnRomInfo SpecRogerpangolinRomDesc[] = {
	{ "Roger the Pangolin in 2020 Knurled Tour (2020)(Joefish).tap", 42586, 0x94fea62c, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecRogerpangolin, SpecRogerpangolin, Spectrum)
STD_ROM_FN(SpecRogerpangolin)

struct BurnDriver BurnSpecRogerpangolin = {
	"spec_rogerpangolin", NULL, "spec_spectrum", NULL, "2020",
	"Roger the Pangolin in 2020 Knurled Tour (48K) (HB)\0", NULL, "Joefish", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_ACTION, 0,
	SpectrumGetZipName, SpecRogerpangolinRomInfo, SpecRogerpangolinRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// A Very Sheepy Christmas (128K) (HB)

static struct BurnRomInfo SpecVerysheepyxmasRomDesc[] = {
	{ "A Very Sheepy Christmas (2020)(Quantum Sheep).tap", 27498, 0x94185496, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecVerysheepyxmas, SpecVerysheepyxmas, Spec128)
STD_ROM_FN(SpecVerysheepyxmas)

struct BurnDriver BurnSpecVerysheepyxmas = {
	"spec_verysheepyxmas", NULL, "spec_spec128", NULL, "2020",
	"A Very Sheepy Christmas (128K) (HB)\0", NULL, "Quantum Sheep", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_ACTION, 0,
	SpectrumGetZipName, SpecVerysheepyxmasRomInfo, SpecVerysheepyxmasRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Dark Redux, The (English) (48K) (HB)

static struct BurnRomInfo SpecDarkreduxRomDesc[] = {
	{ "Dark Redux, The (2021)(Zosya Entertainment).tap", 46873, 0x34f3deef, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecDarkredux, SpecDarkredux, Spectrum)
STD_ROM_FN(SpecDarkredux)

struct BurnDriver BurnSpecDarkredux = {
	"spec_darkredux", NULL, "spec_spectrum", NULL, "2021",
	"Dark Redux, The (English) (48K) (HB)\0", "Hold 'P' at the logo screen to skip the intro", "Zosya Entertainment", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_MAZE | GBF_RUNGUN, 0,
	SpectrumGetZipName, SpecDarkreduxRomInfo, SpecDarkreduxRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Krpat (Czech) (48K) (HB)

static struct BurnRomInfo SpecKrpatRomDesc[] = {
	{ "Krpat (2020)(Bizard).tap", 28477, 0xd4b9f078, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecKrpat, SpecKrpat, Spectrum)
STD_ROM_FN(SpecKrpat)

struct BurnDriver BurnSpecKrpat = {
	"spec_krpat", NULL, "spec_spectrum", NULL, "2020",
	"Krpat (Czech) (48K) (HB)\0", NULL, "Bizard", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecKrpatRomInfo, SpecKrpatRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Red Raid - The Sinking - Side A (48K) (HB)

static struct BurnRomInfo SpecRedraidsink1RomDesc[] = {
	{ "Red Raid The Sinking - Side A (2020)(ZX Bitles).tap", 46211, 0x095fab77, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecRedraidsink1, SpecRedraidsink1, Spectrum)
STD_ROM_FN(SpecRedraidsink1)

struct BurnDriver BurnSpecRedraidsink1 = {
	"spec_redraidsink1", NULL, "spec_spectrum", NULL, "2020",
	"Red Raid - The Sinking - Side A (48K) (HB)\0", NULL, "ZX Bitles", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PLATFORM | GBF_ADV, 0,
	SpectrumGetZipName, SpecRedraidsink1RomInfo, SpecRedraidsink1RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecIntf2SwapDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Red Raid - The Sinking - Side B (48K) (HB)

static struct BurnRomInfo SpecRedraidsink2RomDesc[] = {
	{ "Red Raid The Sinking - Side B (2020)(ZX Bitles).tap", 46408, 0x9067d221, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecRedraidsink2, SpecRedraidsink2, Spectrum)
STD_ROM_FN(SpecRedraidsink2)

struct BurnDriver BurnSpecRedraidsink2 = {
	"spec_redraidsink2", "spec_redraidsink1", "spec_spectrum", NULL, "2020",
	"Red Raid - The Sinking - Side B (48K) (HB)\0", NULL, "ZX Bitles", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PLATFORM | GBF_ADV, 0,
	SpectrumGetZipName, SpecRedraidsink2RomInfo, SpecRedraidsink2RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecIntf2SwapDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Sea Dragon (48K-128K) (HB)

static struct BurnRomInfo SpecseadragonRomDesc[] = {
	{ "Sea Dragon (2010)(Andrew Zhiglov)(128k).z80", 0x13b53, 0xf9fe097c, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specseadragon, Specseadragon, Spec128)
STD_ROM_FN(Specseadragon)

struct BurnDriver BurnSpecseadragon = {
	"spec_seadragon", NULL, "spec_spec128", NULL, "2010",
	"Sea Dragon (48K-128K) (HB)\0", NULL, "Andrew Zhiglov", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_HORSHOOT, 0,
	SpectrumGetZipName, SpecseadragonRomInfo, SpecseadragonRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Dark Transit (48K) (HB)

static struct BurnRomInfo SpecDarktransitRomDesc[] = {
	{ "Dark Transit (2021)(Goliat eXperience Games).z80", 131151, 0x6dd4e36d, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecDarktransit, SpecDarktransit, Spec128)
STD_ROM_FN(SpecDarktransit)

struct BurnDriver BurnSpecDarktransit = {
	"spec_darktransit", NULL, "spec_spec128", NULL, "2021",
	"Dark Transit (48K) (HB)\0", NULL, "Goliat eXperience Games", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecDarktransitRomInfo, SpecDarktransitRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecQAOPMDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Yanga Plus (128K) (HB)

static struct BurnRomInfo SpecYangaplusRomDesc[] = {
	{ "Yanga Plus (2021)(Serdjuk).tap", 55855, 0x4c6d1b08, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecYangaplus, SpecYangaplus, Spec128)
STD_ROM_FN(SpecYangaplus)

struct BurnDriver BurnSpecYangaplus = {
	"spec_yangaplus", NULL, "spec_spec128", NULL, "2021",
	"Yanga Plus (128K) (HB)\0", NULL, "Serdjuk", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PUZZLE, 0,
	SpectrumGetZipName, SpecYangaplusRomInfo, SpecYangaplusRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// AntEater (48K) (HB)

static struct BurnRomInfo SpecAnteaterRomDesc[] = {
	{ "AntEater (2021)(Defb).tap", 44819, 0x31e117c5, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecAnteater, SpecAnteater, Spec128)
STD_ROM_FN(SpecAnteater)

struct BurnDriver BurnSpecAnteater = {
	"spec_anteater", NULL, "spec_spec128", NULL, "2021",
	"AntEater (48K) (HB)\0", NULL, "DEFB Studio", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_ACTION, 0,
	SpectrumGetZipName, SpecAnteaterRomInfo, SpecAnteaterRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecQAOPSpaceDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Dizzy 1 Extended (48K) (HB)

static struct BurnRomInfo SpecDizzy1extRomDesc[] = {
	{ "Dizzy 1 Extended (2021)(Titus).tap", 38104, 0xd4393e5e, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecDizzy1ext, SpecDizzy1ext, Spectrum)
STD_ROM_FN(SpecDizzy1ext)

struct BurnDriver BurnSpecDizzy1ext = {
	"spec_dizzy1ext", NULL, "spec_spectrum", NULL, "2021",
	"Dizzy 1 Extended (48K) (HB)\0", NULL, "Dr. Titus", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PLATFORM | GBF_ADV, 0,
	SpectrumGetZipName, SpecDizzy1extRomInfo, SpecDizzy1extRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Dizzy 1 Extended (Russian) (48K) (HB)

static struct BurnRomInfo SpecDizzy1extrusRomDesc[] = {
	{ "Dizzy 1 Extended (2021)(Titus)(Rus).tap", 38214, 0x9df57a5a, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecDizzy1extrus, SpecDizzy1extrus, Spectrum)
STD_ROM_FN(SpecDizzy1extrus)

struct BurnDriver BurnSpecDizzy1extrus = {
	"spec_dizzy1extrus", "spec_dizzy1ext", "spec_spectrum", NULL, "2021",
	"Dizzy 1 Extended (Russian) (48K) (HB)\0", NULL, "Dr. Titus", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PLATFORM | GBF_ADV, 0,
	SpectrumGetZipName, SpecDizzy1extrusRomInfo, SpecDizzy1extrusRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Crystal Kingdom Dizzy - Remake (128K) (HB)

static struct BurnRomInfo SpecDizzy7remakeRomDesc[] = {
	{ "Crystal Kindom Dizzy - Remake (2017)(Codemasters).tap", 95390, 0x65df9186, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecDizzy7remake, SpecDizzy7remake, Spec128)
STD_ROM_FN(SpecDizzy7remake)

struct BurnDriver BurnSpecDizzy7remake = {
	"spec_dizzy7remake", NULL, "spec_spec128", NULL, "2017",
	"Crystal Kingdom Dizzy - Remake (128K) (HB)\0", NULL, "Codemasters", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PLATFORM | GBF_ADV, 0,
	SpectrumGetZipName, SpecDizzy7remakeRomInfo, SpecDizzy7remakeRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Lost Cavern (128K) (HB)

static struct BurnRomInfo SpecLostcavernRomDesc[] = {
	{ "Lost Cavern (2021)(Isaiasdiaz).tap", 46610, 0x987ec991, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecLostcavern, SpecLostcavern, Spec128)
STD_ROM_FN(SpecLostcavern)

struct BurnDriver BurnSpecLostcavern = {
	"spec_lostcavern", NULL, "spec_spec128", NULL, "2021",
	"Lost Cavern (128K) (HB)\0", NULL, "Isaiasdiaz", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING| BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecLostcavernRomInfo, SpecLostcavernRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Lost Cavern - Easy (128K) (HB)

static struct BurnRomInfo SpecLostcaverneasyRomDesc[] = {
	{ "Lost Cavern - Easy (2021)(Isaiasdiaz).tap", 46585, 0x37e3379b, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecLostcaverneasy, SpecLostcaverneasy, Spec128)
STD_ROM_FN(SpecLostcaverneasy)

struct BurnDriver BurnSpecLostcaverneasy = {
	"spec_lostcaverneasy", "spec_lostcavern", "spec_spec128", NULL, "2021",
	"Lost Cavern - Easy (128K) (HB)\0", NULL, "Isaiasdiaz", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecLostcaverneasyRomInfo, SpecLostcaverneasyRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Space Thunder (128K) (HB)
// https://isaiasdiaz.itch.io/space-thunder

static struct BurnRomInfo SpecSpacethunderRomDesc[] = {
	{ "Space Thunder (2021)(Isaiasdiaz).tap", 47073, 0xa11f0963, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecSpacethunder, SpecSpacethunder, Spec128)
STD_ROM_FN(SpecSpacethunder)

struct BurnDriver BurnSpecSpacethunder = {
	"spec_spacethunder", NULL, "spec_spec128", NULL, "2021",
	"Space Thunder (128K) (HB)\0", NULL, "Isaias Diaz Toledano", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_SHOOT, 0,
	SpectrumGetZipName, SpecSpacethunderRomInfo, SpecSpacethunderRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Ninja Poison - Part 1 (128K) (HB)

static struct BurnRomInfo SpecNinjapoison1RomDesc[] = {
	{ "Ninja Poison - Part 1 (2021)(TokuSoft).tap", 45684, 0x2cd6d37d, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecNinjapoison1, SpecNinjapoison1, Spec128)
STD_ROM_FN(SpecNinjapoison1)

struct BurnDriver BurnSpecNinjapoison1 = {
	"spec_ninjapoison1", NULL, "spec_spec128", NULL, "2021",
	"Ninja Poison - Part 1 (128K) (HB)\0", NULL, "TokuSoft", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecNinjapoison1RomInfo, SpecNinjapoison1RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Ninja Poison - Part 2 (128K) (HB)

static struct BurnRomInfo SpecNinjapoison2RomDesc[] = {
	{ "Ninja Poison - Part 2 (2021)(TokuSoft).tap", 48076, 0xe4cce9b9, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecNinjapoison2, SpecNinjapoison2, Spec128)
STD_ROM_FN(SpecNinjapoison2)

struct BurnDriver BurnSpecNinjapoison2 = {
	"spec_ninjapoison2", "spec_ninjapoison1", "spec_spec128", NULL, "2021",
	"Ninja Poison - Part 2 (128K) (HB)\0", NULL, "TokuSoft", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecNinjapoison2RomInfo, SpecNinjapoison2RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Nixy and the Seeds of Doom (128K) (HB)

static struct BurnRomInfo SpecnixyseedsofdRomDesc[] = {
	{ "Nixy and the Seeds of Doom 128K (2019)(Bubblesoft).tap", 48635, 0x3649c61d, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specnixyseedsofd, Specnixyseedsofd, Spec128)
STD_ROM_FN(Specnixyseedsofd)

struct BurnDriver BurnSpecnixyseedsofd = {
	"spec_nixyseedsofd", NULL, "spec_spec128", NULL, "2019",
	"Nixy and the Seeds of Doom (128K) (HB)\0", NULL, "Bubblesoft", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecnixyseedsofdRomInfo, SpecnixyseedsofdRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Nixy the Glade Sprite (128K) (HB)

static struct BurnRomInfo SpecnixyglaspriRomDesc[] = {
	{ "Nixy the Glade Sprite 128K (2018)(Bubblesoft).tap", 52957, 0x63257fff, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specnixyglaspri, Specnixyglaspri, Spec128)
STD_ROM_FN(Specnixyglaspri)

struct BurnDriver BurnSpecnixyglaspri = {
	"spec_nixyglaspri", NULL, "spec_spec128", NULL, "2018",
	"Nixy the Glade Sprite (128K) (HB)\0", "Press SPACE to choose and ENTER to set options", "Bubblesoft", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecnixyglaspriRomInfo, SpecnixyglaspriRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Organic Matter (48K) (HB)

static struct BurnRomInfo SpecOrganicmatterRomDesc[] = {
	{ "Organic Matter (2021)(Chentzilla).tap", 38849, 0xc4798ebb, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecOrganicmatter, SpecOrganicmatter, Spectrum)
STD_ROM_FN(SpecOrganicmatter)

struct BurnDriver BurnSpecOrganicmatter = {
	"spec_organicmatter", NULL, "spec_spectrum", NULL, "2021",
	"Organic Matter (48K) (HB)\0", NULL, "Chentzilla", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecOrganicmatterRomInfo, SpecOrganicmatterRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecIntf2DIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Maritrini Freelance Monster Slayer (128K) (HB)

static struct BurnRomInfo SpecmaritriniRomDesc[] = {
	{ "Maritrini Freelance MonsterSlayer (2011)(Mojon Twins).tap", 91550, 0xb2a347b9, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specmaritrini, Specmaritrini, Spec128)
STD_ROM_FN(Specmaritrini)

struct BurnDriver BurnSpecmaritrini = {
	"spec_maritrini", NULL, "spec_spec128", NULL, "2011",
	"Maritrini Freelance Monster Slayer (128K) (HB)\0", NULL, "The Mojon Twins", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_RUNGUN | GBF_MAZE, 0,
	SpectrumGetZipName, SpecmaritriniRomInfo, SpecmaritriniRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Fase Bonus The Game (48K) (HB)

static struct BurnRomInfo SpecFasebonusRomDesc[] = {
	{ "Fase Bonus The Game (2017)(Mojon Twins).tap", 39076, 0x493f3bc8, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecFasebonus, SpecFasebonus, Spectrum)
STD_ROM_FN(SpecFasebonus)

struct BurnDriver BurnSpecFasebonus = {
	"spec_fasebonus", NULL, "spec_spectrum", NULL, "2017",
	"Fase Bonus The Game (48K) (HB)\0", NULL, "The Mojon Twins", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecFasebonusRomInfo, SpecFasebonusRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Dungeons of Gomilandia (128K) (HB)

static struct BurnRomInfo SpecGomilandiaRomDesc[] = {
	{ "Dungeons of Gomilandia (2020)(RetroWorks).z80", 38766, 0x39d5e947, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecGomilandia, SpecGomilandia, Spec128)
STD_ROM_FN(SpecGomilandia)

struct BurnDriver BurnSpecGomilandia = {
	"spec_gomilandia", NULL, "spec_spec128", NULL, "2020",
	"Dungeons of Gomilandia (128K) (HB)\0", NULL, "RetroWorks", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PUZZLE, 0,
	SpectrumGetZipName, SpecGomilandiaRomInfo, SpecGomilandiaRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// MagicAble (48K) (HB)

static struct BurnRomInfo SpecMagicableRomDesc[] = {
	{ "MagicAble (2020)(Packobilly).tap", 44795, 0x4ca69d44, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecMagicable, SpecMagicable, Spec128)
STD_ROM_FN(SpecMagicable)

struct BurnDriver BurnSpecMagicable = {
	"spec_magicable", NULL, "spec_spec128", NULL, "2020",
	"MagicAble (48K) (HB)\0", "Set keyboard to use controller", "Packobilly", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecMagicableRomInfo, SpecMagicableRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecQAOPSpaceDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Mire Mare (48K) (HB, v1.09)

static struct BurnRomInfo SpecMiremareRomDesc[] = {
	{ "Mire Mare v1.09 (2020)(Luca Bordoni).tap", 47329, 0x91db2eb2, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecMiremare, SpecMiremare, Spectrum)
STD_ROM_FN(SpecMiremare)

struct BurnDriver BurnSpecMiremare = {
	"spec_miremare", NULL, "spec_spectrum", NULL, "2020",
	"Mire Mare (48K) (HB, v1.09)\0", NULL, "Luca Bordoni", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_ACTION | GBF_MAZE, 0,
	SpectrumGetZipName, SpecMiremareRomInfo, SpecMiremareRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Ramire the Vampire Rewamp - Part 1 (48K) (HB)

static struct BurnRomInfo SpecRamirer1RomDesc[] = {
	{ "Ramire the Vampire Rewamp Part1 (2016)(Mojon Twins).tap", 44159, 0x69c64f56, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecRamirer1, SpecRamirer1, Spectrum)
STD_ROM_FN(SpecRamirer1)

struct BurnDriver BurnSpecRamirer1 = {
	"spec_ramirer1", NULL, "spec_spectrum", NULL, "2016",
	"Ramire the Vampire Rewamp - Part 1 (48K) (HB)\0", NULL, "The Mojon Twins", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PLATFORM | GBF_ADV, 0,
	SpectrumGetZipName, SpecRamirer1RomInfo, SpecRamirer1RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Ramire the Vampire Rewamp - Part 2 (48K) (HB)

static struct BurnRomInfo SpecRamirer2RomDesc[] = {
	{ "Ramire the Vampire Rewamp Part2 (2016)(Mojon Twins).tap", 44509, 0x3ce3ae4c, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecRamirer2, SpecRamirer2, Spectrum)
STD_ROM_FN(SpecRamirer2)

struct BurnDriver BurnSpecRamirer2 = {
	"spec_ramirer2", "spec_ramirer1", "spec_spectrum", NULL, "2016",
	"Ramire the Vampire Rewamp - Part 2 (48K) (HB)\0", NULL, "The Mojon Twins", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PLATFORM | GBF_ADV, 0,
	SpectrumGetZipName, SpecRamirer2RomInfo, SpecRamirer2RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Ramire the Vampire Rewamp - Part 3 (48K) (HB)

static struct BurnRomInfo SpecRamirer3RomDesc[] = {
	{ "Ramire the Vampire Rewamp Part3 (2016)(Mojon Twins).tap", 44938, 0xa65f4290, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecRamirer3, SpecRamirer3, Spectrum)
STD_ROM_FN(SpecRamirer3)

struct BurnDriver BurnSpecRamirer3 = {
	"spec_ramirer3", "spec_ramirer1", "spec_spectrum", NULL, "2016",
	"Ramire the Vampire Rewamp - Part 3 (48K) (HB)\0", NULL, "The Mojon Twins", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PLATFORM | GBF_ADV, 0,
	SpectrumGetZipName, SpecRamirer3RomInfo, SpecRamirer3RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Ramiro el Vampiro Rewamp Parte 1 (48K) (HB)

static struct BurnRomInfo SpecRamiror1RomDesc[] = {
	{ "Ramiro el Vampiro Rewamp Part1 (2016)(Mojon Twins).tap", 44399, 0x9d9e0971, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecRamiror1, SpecRamiror1, Spectrum)
STD_ROM_FN(SpecRamiror1)

struct BurnDriver BurnSpecRamiror1 = {
	"spec_ramiror1", NULL, "spec_spectrum", NULL, "2016",
	"Ramiro el Vampiro Rewamp - Parte 1 (48K) (HB)\0", NULL, "The Mojon Twins", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PLATFORM | GBF_ADV, 0,
	SpectrumGetZipName, SpecRamiror1RomInfo, SpecRamiror1RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Ramiro el Vampiro Rewamp Parte 2 (48K) (HB)

static struct BurnRomInfo SpecRamiror2RomDesc[] = {
	{ "Ramiro el Vampiro Rewamp Part2 (2016)(Mojon Twins).tap", 44601, 0x01ed38e9, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecRamiror2, SpecRamiror2, Spectrum)
STD_ROM_FN(SpecRamiror2)

struct BurnDriver BurnSpecRamiror2 = {
	"spec_ramiror2", "spec_ramiror1", "spec_spectrum", NULL, "2016",
	"Ramiro el Vampiro Rewamp - Parte 2 (48K) (HB)\0", NULL, "The Mojon Twins", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PLATFORM | GBF_ADV, 0,
	SpectrumGetZipName, SpecRamiror2RomInfo, SpecRamiror2RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Ramiro el Vampiro Rewamp Parte 3 (48K) (HB)

static struct BurnRomInfo SpecRamiror3RomDesc[] = {
	{ "Ramiro el Vampiro Rewamp Part3 (2016)(Mojon Twins).tap", 45083, 0xec94c119, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecRamiror3, SpecRamiror3, Spectrum)
STD_ROM_FN(SpecRamiror3)

struct BurnDriver BurnSpecRamiror3 = {
	"spec_ramiror3", "spec_ramiror1", "spec_spectrum", NULL, "2016",
	"Ramiro el Vampiro Rewamp - Parte 3 (48K) (HB)\0", NULL, "The Mojon Twins", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PLATFORM | GBF_ADV, 0,
	SpectrumGetZipName, SpecRamiror3RomInfo, SpecRamiror3RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Castaway (48K) (HB)

static struct BurnRomInfo SpecCastawayRomDesc[] = {
	{ "Castaway 48K (2016)(usebox.net).tap", 29470, 0xa0fccc30, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecCastaway, SpecCastaway, Spectrum)
STD_ROM_FN(SpecCastaway)

struct BurnDriver BurnSpecCastaway = {
	"spec_castaway", NULL, "spec_spectrum", NULL, "2016",
	"Castaway (48K) (HB)\0", NULL, "usebox.net", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecCastawayRomInfo, SpecCastawayRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Crimbo: A Gloop Troops Tale (128K) (HB)

static struct BurnRomInfo SpecCrimboRomDesc[] = {
	{ "Crimbo 128K (2010)(Little Shop of Pixels).tap", 44707, 0x8b519cdf, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecCrimbo, SpecCrimbo, Spec128)
STD_ROM_FN(SpecCrimbo)

struct BurnDriver BurnSpecCrimbo = {
	"spec_crimbo", NULL, "spec_spec128", NULL, "2010",
	"Crimbo: A Gloop Troops Tale (128K) (HB)\0", NULL, "Little Shop of Pixels", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecCrimboRomInfo, SpecCrimboRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Mabus Mania (English) (128K) (HB)

static struct BurnRomInfo SpecMabusenRomDesc[] = {
	{ "Mabus Mania EN (2021)(Hicks).tap", 44176, 0x94ae9080, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecMabusen, SpecMabusen, Spec128)
STD_ROM_FN(SpecMabusen)

struct BurnDriver BurnSpecMabusen = {
	"spec_mabusen", NULL, "spec_spec128", NULL, "2021",
	"Mabus Mania (English) (128K) (HB)\0", "English-language version", "Hicks", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecMabusenRomInfo, SpecMabusenRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Mabus Mania (Spanish) (128K) (HB)

static struct BurnRomInfo SpecMabusesRomDesc[] = {
	{ "Mabus Mania ES (2021)(Hicks).tap", 44200, 0x3d70ab7b, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecMabuses, SpecMabuses, Spec128)
STD_ROM_FN(SpecMabuses)

struct BurnDriver BurnSpecMabuses = {
	"spec_mabuses", "spec_mabusen", "spec_spec128", NULL, "2021",
	"Mabus Mania (Spanish) (128K) (HB)\0", "Spanish-language version", "Hicks", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecMabusesRomInfo, SpecMabusesRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Metamorphosis (128K) (HB)

static struct BurnRomInfo SpecMetamorphosisRomDesc[] = {
	{ "Metamorphosis (2021)(ITNL-team).tap", 49194, 0xd22fb315, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecMetamorphosis, SpecMetamorphosis, Spec128)
STD_ROM_FN(SpecMetamorphosis)

struct BurnDriver BurnSpecMetamorphosis = {
	"spec_metamorphosis", NULL, "spec_spec128", NULL, "2021",
	"Metamorphosis (128K) (HB)\0", NULL, "ITNL-team", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecMetamorphosisRomInfo, SpecMetamorphosisRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Rodman v1.2 (128K) (HB)

static struct BurnRomInfo SpecRodmanRomDesc[] = {
	{ "Rodman v1.2 (2018)(TFW8b).tap", 21795, 0xc7ba4663, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecRodman, SpecRodman, Spec128)
STD_ROM_FN(SpecRodman)

struct BurnDriver BurnSpecRodman = {
	"spec_rodman", NULL, "spec_spec128", NULL, "2018",
	"Rodman v1.2 (128K) (HB)\0", NULL, "The Future Was 8bit", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_MAZE | GBF_ACTION, 0,
	SpectrumGetZipName, SpecRodmanRomInfo, SpecRodmanRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Shuttlebug (48K) (HB)

static struct BurnRomInfo SpecShuttlebugRomDesc[] = {
	{ "Shuttlebug 48K (2012)( Stonechat).tap", 32148, 0xe7f63317, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecShuttlebug, SpecShuttlebug, Spectrum)
STD_ROM_FN(SpecShuttlebug)

struct BurnDriver BurnSpecShuttlebug = {
	"spec_shuttlebug", NULL, "spec_spectrum", NULL, "2012",
	"Shuttlebug (48K) (HB)\0", NULL, " Stonechat", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_MAZE | GBF_ACTION, 0,
	SpectrumGetZipName, SpecShuttlebugRomInfo, SpecShuttlebugRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecQAOPSpaceDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Yumiko in the haunted mansion (128K) (HB)

static struct BurnRomInfo SpecYumikohmRomDesc[] = {
	{ "Yumiko in the haunted mansion (2012)(Fun Forge).tap", 30728, 0x2d83ffea, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecYumikohm, SpecYumikohm, Spec128)
STD_ROM_FN(SpecYumikohm)

struct BurnDriver BurnSpecYumikohm = {
	"spec_yumikohm", NULL, "spec_spec128", NULL, "2012",
	"Yumiko in the haunted mansion (128K) (HB)\0", NULL, "Fun Forge", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_MAZE | GBF_ADV, 0,
	SpectrumGetZipName, SpecYumikohmRomInfo, SpecYumikohmRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Astro Phobos (128K) (HB, v1.3)

static struct BurnRomInfo SpecAstrophobosRomDesc[] = {
	{ "Astro Phobos v1.3 (2021)(goliat).z80", 131151, 0x1acd61a3, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecAstrophobos, SpecAstrophobos, Spec128)
STD_ROM_FN(SpecAstrophobos)

struct BurnDriver BurnSpecAstrophobos = {
	"spec_astrophobos", NULL, "spec_spec128", NULL, "2021",
	"Astro Phobos (128K) (HB, v1.3)\0", NULL, "Goliat", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecAstrophobosRomInfo, SpecAstrophobosRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Cadaverion (English) (48K) (HB)

static struct BurnRomInfo SpecCadaverionenRomDesc[] = {
	{ "Cadaverion EN (2013)(Mojon Twins).tap", 40573, 0xce3d8b7f, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecCadaverionen, SpecCadaverionen, Spectrum)
STD_ROM_FN(SpecCadaverionen)

struct BurnDriver BurnSpecCadaverionen = {
	"spec_cadaverionen", NULL, "spec_spectrum", NULL, "2013",
	"Cadaverion (English) (48K) (HB)\0", NULL, "The Mojon Twins", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_ADV | GBF_PUZZLE, 0,
	SpectrumGetZipName, SpecCadaverionenRomInfo, SpecCadaverionenRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Cadaverion (Spanish) (48K) (HB)

static struct BurnRomInfo SpecCadaverionesRomDesc[] = {
	{ "Cadaverion ES (2013)(Mojon Twins).tap", 40581, 0x53481358, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecCadaveriones, SpecCadaveriones, Spectrum)
STD_ROM_FN(SpecCadaveriones)

struct BurnDriver BurnSpecCadaveriones = {
	"spec_cadaveriones", "spec_cadaverionen", "spec_spectrum", NULL, "2013",
	"Cadaverion (Spanish) (48K) (HB)\0", NULL, "The Mojon Twins", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_ADV | GBF_PUZZLE, 0,
	SpectrumGetZipName, SpecCadaverionesRomInfo, SpecCadaverionesRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Castle Of Dr Game (48K) (HB)

static struct BurnRomInfo SpecCodgRomDesc[] = {
	{ "Castle Of Dr Game (2020)(Paul Jenkinson).tap", 38366, 0x5063f4b5, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecCodg, SpecCodg, Spectrum)
STD_ROM_FN(SpecCodg)

struct BurnDriver BurnSpecCodg = {
	"spec_codg", NULL, "spec_spectrum", NULL, "2020",
	"Castle Of Dr Game (48K) (HB)\0", NULL, "Paul Jenkinson", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecCodgRomInfo, SpecCodgRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Charm, The (128K) (HB)

static struct BurnRomInfo SpecCharmRomDesc[] = {
	{ "The Charm (2014)(RetroWorks).z80", 100884, 0xe56ecd56, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecCharm, SpecCharm, Spec128)
STD_ROM_FN(SpecCharm)

struct BurnDriver BurnSpecCharm = {
	"spec_charm", NULL, "spec_spec128", NULL, "2014",
	"Charm, The (128K) (HB)\0", NULL, "RetroWorks", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecCharmRomInfo, SpecCharmRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Che-Man (48K) (HB)

static struct BurnRomInfo SpecChemanRomDesc[] = {
	{ "Che-Man (2019)(Mojon Twins).tap", 21346, 0xab6e3d27, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecCheman, SpecCheman, Spectrum)
STD_ROM_FN(SpecCheman)

struct BurnDriver BurnSpecCheman = {
	"spec_cheman", NULL, "spec_spectrum", NULL, "2019",
	"Che-Man (48K) (HB)\0", NULL, "The Mojon Twins", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecChemanRomInfo, SpecChemanRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Deep Core Raider (128K) (HB, v1.1)

static struct BurnRomInfo SpecDcraiderRomDesc[] = {
	{ "Deep Core Raider v1.1 (2016)(Paul Jenkinson).tap", 39312, 0x3a2bf4cb, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecDcraider, SpecDcraider, Spec128)
STD_ROM_FN(SpecDcraider)

struct BurnDriver BurnSpecDcraider = {
	"spec_dcraider", NULL, "spec_spec128", NULL, "2016",
	"Deep Core Raider (128K) (HB, v1.1)\0", NULL, "Paul Jenkinson", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_ACTION, 0,
	SpectrumGetZipName, SpecDcraiderRomInfo, SpecDcraiderRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Dogmole Tuppowski - The New Adventures (English) (128K) (HB)

static struct BurnRomInfo SpecDogadvenRomDesc[] = {
	{ "Dogmole Tuppowski - The New Adventures (2014)(Mojon Twins).tap", 53730, 0x23af5af2, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecDogadven, SpecDogadven, Spec128)
STD_ROM_FN(SpecDogadven)

struct BurnDriver BurnSpecDogadven = {
	"spec_dogadven", NULL, "spec_spec128", NULL, "2014",
	"Dogmole Tuppowski - The New Adventures (English) (128K) (HB)\0", NULL, "The Mojon Twins", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecDogadvenRomInfo, SpecDogadvenRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Dogmole Tuppowski - Las Nuevas Aventuras (Spanish) (128K) (HB)

static struct BurnRomInfo SpecDogadvesRomDesc[] = {
	{ "Dogmole Tuppowski - Las Nuevas Aventuras (2014)(Mojon Twins).tap", 53802, 0xd599d2db, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecDogadves, SpecDogadves, Spec128)
STD_ROM_FN(SpecDogadves)

struct BurnDriver BurnSpecDogadves = {
	"spec_dogadves", "spec_dogadven", "spec_spec128", NULL, "2014",
	"Dogmole Tuppowski - Las Nuevas Aventuras (Spanish) (128K) (HB)\0", NULL, "The Mojon Twins", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecDogadvesRomInfo, SpecDogadvesRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Factory Daze (128K) (HB)

static struct BurnRomInfo SpecFactdazeRomDesc[] = {
	{ "Factory Daze 128K (2009)(Bob Smith).tap", 59071, 0x73225faf, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecFactdaze, SpecFactdaze, Spec128)
STD_ROM_FN(SpecFactdaze)

struct BurnDriver BurnSpecFactdaze = {
	"spec_factdaze", NULL, "spec_spec128", NULL, "2009",
	"Factory Daze (128K) (HB)\0", NULL, "Bob Smith", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PUZZLE, 0,
	SpectrumGetZipName, SpecFactdazeRomInfo, SpecFactdazeRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// King's Valley (128K) (HB)

static struct BurnRomInfo SpecKingsvalleyRomDesc[] = {
	{ "King's Valley (2009)(RetroWorks).tap", 22083, 0xa9d5816a, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecKingsvalley, SpecKingsvalley, Spec128)
STD_ROM_FN(SpecKingsvalley)

struct BurnDriver BurnSpecKingsvalley = {
	"spec_kingsvalley", NULL, "spec_spec128", NULL, "2009",
	"King's Valley (128K) (HB)\0", NULL, "RetroWorks", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecKingsvalleyRomInfo, SpecKingsvalleyRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecQAOPSpaceDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Majikazo (48K) (HB)

static struct BurnRomInfo SpecMajikazoRomDesc[] = {
	{ "Majikazo (2012)(RetroWorks).tap", 41678, 0xc121cf7a, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecMajikazo, SpecMajikazo, Spectrum)
STD_ROM_FN(SpecMajikazo)

struct BurnDriver BurnSpecMajikazo = {
	"spec_majikazo", NULL, "spec_spectrum", NULL, "2012",
	"Majikazo (48K) (HB)\0", "Convertion of Lemonize's MSX version", "RetroWorks", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_ACTION, 0,
	SpectrumGetZipName, SpecMajikazoRomInfo, SpecMajikazoRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Ninja twins Going to Zedeaks (128K) (HB)

static struct BurnRomInfo SpecNinjatwinsRomDesc[] = {
	{ "Ninja twins Going to Zedeaks 128K (2014)(SAM Style).tap", 19337, 0x91a6dbd8, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecNinjatwins, SpecNinjatwins, Spec128)
STD_ROM_FN(SpecNinjatwins)

struct BurnDriver BurnSpecNinjatwins = {
	"spec_ninjatwins", NULL, "spec_spec128", NULL, "2014",
	"Ninja twins Going to Zedeaks (128K) (HB)\0", NULL, "SAM Style", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PUZZLE, 0,
	SpectrumGetZipName, SpecNinjatwinsRomInfo, SpecNinjatwinsRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecQAOPSpaceDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Nohzdyve (48K) (HB)

static struct BurnRomInfo SpecNohzdyveRomDesc[] = {
	{ "Nohzdyve (2018)(Tuckersoft).tap", 13797, 0xee509081, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecNohzdyve, SpecNohzdyve, Spectrum)
STD_ROM_FN(SpecNohzdyve)

struct BurnDriver BurnSpecNohzdyve = {
	"spec_nohzdyve", NULL, "spec_spectrum", NULL, "2018",
	"Nohzdyve (48K) (HB)\0", NULL, "Tuckersoft", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_ACTION, 0,
	SpectrumGetZipName, SpecNohzdyveRomInfo, SpecNohzdyveRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecQAOPSpaceDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Oddi The Viking (128K) (HB)

static struct BurnRomInfo SpecOddi128kRomDesc[] = {
	{ "Oddi The Viking 128K (2010)(Digital Brains).z80", 39355, 0x9a22b6ef, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecOddi128k, SpecOddi128k, Spec128)
STD_ROM_FN(SpecOddi128k)

struct BurnDriver BurnSpecOddi128k = {
	"spec_oddi128k", NULL, "spec_spec128", NULL, "2010",
	"Oddi The Viking (128K) (HB)\0", NULL, "Digital Brains", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PUZZLE, 0,
	SpectrumGetZipName, SpecOddi128kRomInfo, SpecOddi128kRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Oddi The Viking (48K) (HB)

static struct BurnRomInfo SpecOddi48kRomDesc[] = {
	{ "Oddi The Viking 48K (2010)(Digital Brains).z80", 31576, 0xc436d54c, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecOddi48k, SpecOddi48k, Spectrum)
STD_ROM_FN(SpecOddi48k)

struct BurnDriver BurnSpecOddi48k = {
	"spec_oddi48k", "spec_oddi128k", "spec_spectrum", NULL, "2010",
	"Oddi The Viking (48K) (HB)\0", NULL, "Digital Brains", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PUZZLE, 0,
	SpectrumGetZipName, SpecOddi48kRomInfo, SpecOddi48kRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Rabbit in Wonderland (128K) (HB)

static struct BurnRomInfo SpecRabbitinw128kRomDesc[] = {
	{ "Rabbit in Wonderland 128K (2015)(Javier Fopiani).tap", 69090, 0xe6d51e0d, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecRabbitinw128k, SpecRabbitinw128k, Spec128)
STD_ROM_FN(SpecRabbitinw128k)

struct BurnDriver BurnSpecRabbitinw128k = {
	"spec_rabbitinw128k", NULL, "spec_spec128", NULL, "2015",
	"Rabbit in Wonderland (128K) (HB)\0", NULL, "Javier Fopiani", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecRabbitinw128kRomInfo, SpecRabbitinw128kRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Rabbit in Wonderland (48K) (HB)

static struct BurnRomInfo SpecRabbitinw48kRomDesc[] = {
	{ "Rabbit in Wonderland 48K (2015)(Javier Fopiani).tap", 44047, 0x523420f3, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecRabbitinw48k, SpecRabbitinw48k, Spectrum)
STD_ROM_FN(SpecRabbitinw48k)

struct BurnDriver BurnSpecRabbitinw48k = {
	"spec_rabbitinw48k", "spec_rabbitinw128k", "spec_spectrum", NULL, "2015",
	"Rabbit in Wonderland (48K) (HB)\0", NULL, "Javier Fopiani", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecRabbitinw48kRomInfo, SpecRabbitinw48kRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Ragnablock (128K) (HB)

static struct BurnRomInfo SpecRagnablockRomDesc[] = {
	{ "Ragnablock (2006)(Computer Emuzone).z80", 31321, 0xd23221ec, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecRagnablock, SpecRagnablock, Spec128)
STD_ROM_FN(SpecRagnablock)

struct BurnDriver BurnSpecRagnablock = {
	"spec_ragnablock", NULL, "spec_spec128", NULL, "2006",
	"Ragnablock (128K) (HB)\0", NULL, "Computer Emuzone", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_BREAKOUT, 0,
	SpectrumGetZipName, SpecRagnablockRomInfo, SpecRagnablockRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Zombo (48K) (HB)

static struct BurnRomInfo SpecZomboRomDesc[] = {
	{ "Zombo (2015)(Monsterbytes).z80", 38721, 0xa6ff8b75, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecZombo, SpecZombo, Spectrum)
STD_ROM_FN(SpecZombo)

struct BurnDriver BurnSpecZombo = {
	"spec_zombo", NULL, "spec_spectrum", NULL, "2015",
	"Zombo (48K) (HB)\0", NULL, "Monsterbytes", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_MAZE | GBF_ADV, 0,
	SpectrumGetZipName, SpecZomboRomInfo, SpecZomboRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Code Zero (128K) (HB)

static struct BurnRomInfo SpecCodezeroRomDesc[] = {
	{ "Code Zero (2017)(Paul Jenkinson).tap", 39361, 0xeea64720, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecCodezero, SpecCodezero, Spec128)
STD_ROM_FN(SpecCodezero)

struct BurnDriver BurnSpecCodezero = {
	"spec_codezero", NULL, "spec_spec128", NULL, "2017",
	"Code Zero (128K) (HB)\0", NULL, "Paul Jenkinson", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecCodezeroRomInfo, SpecCodezeroRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Ishido - The Way of Stones (48K) (HB)

static struct BurnRomInfo SpecIshidoRomDesc[] = {
	{ "Ishido - The Way of Stones (2015)(mZX).tap", 15859, 0xedd612c3, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecIshido, SpecIshido, Spectrum)
STD_ROM_FN(SpecIshido)

struct BurnDriver BurnSpecIshido = {
	"spec_ishido", NULL, "spec_spectrum", NULL, "2015",
	"Ishido - The Way of Stones (48K) (HB)\0", NULL, "mZX", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PUZZLE, 0,
	SpectrumGetZipName, SpecIshidoRomInfo, SpecIshidoRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecQAOPSpaceDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Knightmare 2 ZX (48K) (HB)

static struct BurnRomInfo SpecKnightzxRomDesc[] = {
	{ "Knightmare 2 ZX (2015)(Climacus).tap", 54725, 0xc0b2638a, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecKnightzx, SpecKnightzx, Spectrum)
STD_ROM_FN(SpecKnightzx)

struct BurnDriver BurnSpecKnightzx = {
	"spec_knightzx", NULL, "spec_spectrum", NULL, "2015",
	"Knightmare 2 ZX (48K) (HB)\0", NULL, "Climacus", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecKnightzxRomInfo, SpecKnightzxRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Pac-Land Recoloured (128K) (Hack)

static struct BurnRomInfo SpecPaclandrcRomDesc[] = {
	{ "Pacland 128K Recoloured (2014)(Rafal Miazga).tap", 29706, 0xacc409c3, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecPaclandrc, SpecPaclandrc, Spec128)
STD_ROM_FN(SpecPaclandrc)

struct BurnDriver BurnSpecPaclandrc = {
	"spec_paclandrc", "spec_pacland", "spec_spec128", NULL, "2014",
	"Pac-Land Recoloured (128K) (Hack)\0", "Colour enhancement mod", "Rafal Miazga", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE | BDF_HACK, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecPaclandrcRomInfo, SpecPaclandrcRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Rod-Land Recoloured (128K) (Hack)

static struct BurnRomInfo SpecRodlandrcRomDesc[] = {
	{ "Rodland Recoloured (2012)(Rafal Miazga).tap", 93721, 0xa028893b, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecRodlandrc, SpecRodlandrc, Spec128)
STD_ROM_FN(SpecRodlandrc)

struct BurnDriver BurnSpecRodlandrc = {
	"spec_rodlandrc", "spec_rodland", "spec_spec128", NULL, "2012",
	"Rod-Land Recoloured (128K) (Hack)\0", "Colour enhancement mod", "Rafal Miazga", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE | BDF_HACK, 2, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecRodlandrcRomInfo, SpecRodlandrcRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Spherical Remastered (128K) (Graphics Hack)

static struct BurnRomInfo SpecSpherical128rmRomDesc[] = {
	{ "Spherical 128K Remastered (2015)(Rafal Miazga).tap", 171049, 0xc5ae07e6, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecSpherical128rm, SpecSpherical128rm, Spec128)
STD_ROM_FN(SpecSpherical128rm)

struct BurnDriver BurnSpecSpherical128rm = {
	"spec_spherical128rm", "spec_spherical", "spec_spec128", NULL, "2015",
	"Spherical Remastered (128K) (Graphics Hack)\0", "Graphics enhancement mod", "Rafal Miazga", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE | BDF_HACK, 2, HARDWARE_SPECTRUM, GBF_PLATFORM | GBF_PUZZLE, 0,
	SpectrumGetZipName, SpecSpherical128rmRomInfo, SpecSpherical128rmRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Three Octopuses (128K) (HB)

static struct BurnRomInfo Spec3octopusesRomDesc[] = {
	{ "Three Octopuses (2017)(kas29).tap", 25990, 0x0f80336f, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Spec3octopuses, Spec3octopuses, Spec128)
STD_ROM_FN(Spec3octopuses)

struct BurnDriver BurnSpec3octopuses = {
	"spec_3octopuses", NULL, "spec_spec128", NULL, "2017",
	"Three Octopuses (128K) (HB)\0", NULL, "kas29", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_ACTION, 0,
	SpectrumGetZipName, Spec3octopusesRomInfo, Spec3octopusesRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Androide (48K) (HB)

static struct BurnRomInfo SpecAndroideRomDesc[] = {
	{ "Androide (2014)(PixelSoft).tap", 42884, 0xa0cce6d2, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecAndroide, SpecAndroide, Spectrum)
STD_ROM_FN(SpecAndroide)

struct BurnDriver BurnSpecAndroide = {
	"spec_androide", NULL, "spec_spectrum", NULL, "2014",
	"Androide (48K) (HB)\0", NULL, "PixelSoft", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecAndroideRomInfo, SpecAndroideRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Archeomania (128K) (HB)

static struct BurnRomInfo SpecArcheomaniaRomDesc[] = {
	{ "Archeomania (2014)(Rafal Miazga).tap", 26567, 0x15340561, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecArcheomania, SpecArcheomania, Spec128)
STD_ROM_FN(SpecArcheomania)

struct BurnDriver BurnSpecArcheomania = {
	"spec_archeomania", NULL, "spec_spec128", NULL, "2014",
	"Archeomania (128K) (HB)\0", NULL, "Rafal Miazga", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PUZZLE, 0,
	SpectrumGetZipName, SpecArcheomaniaRomInfo, SpecArcheomaniaRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Dragonfire ZX (48K) (HB)

static struct BurnRomInfo SpecDragfirezxRomDesc[] = {
	{ "Dragonfire ZX (2018)(Luca Bordoni).tap", 27062, 0x0dc70f48, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecDragfirezx, SpecDragfirezx, Spectrum)
STD_ROM_FN(SpecDragfirezx)

struct BurnDriver BurnSpecDragfirezx = {
	"spec_dragfirezx", NULL, "spec_spectrum", NULL, "2018",
	"Dragonfire ZX (48K) (HB)\0", "A raw adaptation of the Atari VCS-2600 version", "Luca Bordoni", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_ACTION, 0,
	SpectrumGetZipName, SpecDragfirezxRomInfo, SpecDragfirezxRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Mr. Heli Remastered (128K) (Graphics Hack)

static struct BurnRomInfo SpecMrheli128rmRomDesc[] = {
	{ "Mr Heli 128K Remastered (2014)(Rafal Miazga).tap", 56737, 0x5beeb928, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecMrheli128rm, SpecMrheli128rm, Spec128)
STD_ROM_FN(SpecMrheli128rm)

struct BurnDriver BurnSpecMrheli128rm = {
	"spec_mrheli128rm", "spec_mrheli", "spec_spec128", NULL, "2014",
	"Mr. Heli Remastered (128K) (Graphics Hack)\0", "Trainer version", "Rafal Miazga", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE | BDF_HACK, 1, HARDWARE_SPECTRUM, GBF_SHOOT, 0,
	SpectrumGetZipName, SpecMrheli128rmRomInfo, SpecMrheli128rmRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Coloristic (128K) (HB)

static struct BurnRomInfo SpecColoristicRomDesc[] = {
	{ "Coloristic (2020)(Martin Borik).tap", 16436, 0x1c824bfb, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecColoristic, SpecColoristic, Spec128)
STD_ROM_FN(SpecColoristic)

struct BurnDriver BurnSpecColoristic = {
	"spec_coloristic", NULL, "spec_spec128", NULL, "2020",
	"Coloristic (128K) (HB)\0", NULL, "Martin Borik", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PUZZLE, 0,
	SpectrumGetZipName, SpecColoristicRomInfo, SpecColoristicRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Karlos und Schatze der Azteken (128K) (HB)

static struct BurnRomInfo SpecKarlosRomDesc[] = {
	{ "Karlos und Schatze der Azteken (2010)(Perspective Group).tap", 36908, 0x53482edd, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecKarlos, SpecKarlos, Spec128)
STD_ROM_FN(SpecKarlos)

struct BurnDriver BurnSpecKarlos = {
	"spec_karlos", NULL, "spec_spec128", NULL, "2010",
	"Karlos und Schatze der Azteken (128K) (HB)\0", NULL, "Perspective Group", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_ACTION | GBF_PUZZLE, 0,
	SpectrumGetZipName, SpecKarlosRomInfo, SpecKarlosRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecQAOPSpaceDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Lirus 128K (HB)

static struct BurnRomInfo SpecLirus128RomDesc[] = {
	{ "Lirus 128K (2015)(RetroSouls).tap", 69035, 0x1d9067bd, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecLirus128, SpecLirus128, Spec128)
STD_ROM_FN(SpecLirus128)

struct BurnDriver BurnSpecLirus128 = {
	"spec_lirus128", NULL, "spec_spec128", NULL, "2015",
	"Lirus (128K) (HB)\0", NULL, "RetroSouls", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_SHOOT, 0,
	SpectrumGetZipName, SpecLirus128RomInfo, SpecLirus128RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecIntf2DIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Lirus 48K (HB)

static struct BurnRomInfo SpecLirus48RomDesc[] = {
	{ "Lirus 48K (2015)(RetroSouls).tap", 23785, 0x6700e4cb, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecLirus48, SpecLirus48, Spectrum)
STD_ROM_FN(SpecLirus48)

struct BurnDriver BurnSpecLirus48 = {
	"spec_lirus48", "spec_lirus128", "spec_spectrum", NULL, "2015",
	"Lirus (48K) (HB)\0", NULL, "RetroSouls", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_SHOOT, 0,
	SpectrumGetZipName, SpecLirus48RomInfo, SpecLirus48RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecIntf2DIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Los Amores De Brunilda (Spanish) (128K) (HB, v1.4)

static struct BurnRomInfo SpecBrunildaesRomDesc[] = {
	{ "Los Amores De Brunilda ES (2013)(RetroWorks).tap", 96314, 0x6f8e464d, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecBrunildaes, SpecBrunildaes, Spec128)
STD_ROM_FN(SpecBrunildaes)

struct BurnDriver BurnSpecBrunildaes = {
	"spec_brunildaes", "spec_brunilda", "spec_spec128", NULL, "2013",
	"Los Amores De Brunilda (Spanish) (128K) (HB, v1.4)\0", NULL, "RetroWorks", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_RPG, 0,
	SpectrumGetZipName, SpecBrunildaesRomInfo, SpecBrunildaesRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Los Amores De Brunilda (Italian) (128K) (HB, v1.4)

static struct BurnRomInfo SpecBrunildaitRomDesc[] = {
	{ "Los Amores De Brunilda IT (2013)(RetroWorks).tap", 96385, 0xe234e388, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecBrunildait, SpecBrunildait, Spec128)
STD_ROM_FN(SpecBrunildait)

struct BurnDriver BurnSpecBrunildait = {
	"spec_brunildait", "spec_brunilda", "spec_spec128", NULL, "2013",
	"Los Amores De Brunilda (Italian) (128K) (HB, v1.4)\0", NULL, "RetroWorks", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_RPG, 0,
	SpectrumGetZipName, SpecBrunildaitRomInfo, SpecBrunildaitRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Los Amores De Brunilda (Russian) (128K) (HB, v1.4)

static struct BurnRomInfo SpecBrunildaruRomDesc[] = {
	{ "Los Amores De Brunilda RU (2013)(RetroWorks).tap", 96011, 0x0b49795c, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecBrunildaru, SpecBrunildaru, Spec128)
STD_ROM_FN(SpecBrunildaru)

struct BurnDriver BurnSpecBrunildaru = {
	"spec_brunildaru", "spec_brunilda", "spec_spec128", NULL, "2013",
	"Los Amores De Brunilda (Russian) (128K) (HB, v1.4)\0", NULL, "RetroWorks", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_RPG, 0,
	SpectrumGetZipName, SpecBrunildaruRomInfo, SpecBrunildaruRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Conspiralove (48K) (HB)

static struct BurnRomInfo SpecConspiraloveRomDesc[] = {
	{ "Conspiralove (2014)(Martian Devel).tap", 37173, 0x63d4d801, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecConspiralove, SpecConspiralove, Spectrum)
STD_ROM_FN(SpecConspiralove)

struct BurnDriver BurnSpecConspiralove = {
	"spec_conspiralove", NULL, "spec_spectrum", NULL, "2014",
	"Conspiralove (48K) (HB)\0", NULL, "Martian Devel", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecConspiraloveRomInfo, SpecConspiraloveRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Fantasy Zone - Escape from the Pyramid 128K (HB)

static struct BurnRomInfo SpecFntzone2RomDesc[] = {
	{ "Fantasy Zone - Escape from the Pyramid 128K (2016)(GreenWebSevilla).tap", 46356, 0x313f31f1, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecFntzone2, SpecFntzone2, Spec128)
STD_ROM_FN(SpecFntzone2)

struct BurnDriver BurnSpecFntzone2 = {
	"spec_fntzone2", NULL, "spec_spec128", NULL, "2016",
	"Fantasy Zone - Escape from the Pyramid (128K) (HB)\0", NULL, "GreenWebSevilla", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PLATFORM | GBF_SHOOT, 0,
	SpectrumGetZipName, SpecFntzone2RomInfo, SpecFntzone2RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Project Re.Ve.La.Tion (48K) (HB)

static struct BurnRomInfo SpecPrjrevRomDesc[] = {
	{ "Project Re.Ve.La.Tion (2021)(Retrobits).tap", 26553, 0x6056f1e6, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecPrjrev, SpecPrjrev, Spectrum)
STD_ROM_FN(SpecPrjrev)

struct BurnDriver BurnSpecPrjrev = {
	"spec_prjrev", NULL, "spec_spectrum", NULL, "2021",
	"Project Re.Ve.La.Tion (48K) (HB)\0", NULL, "Retrobits", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_ACTION, 0,
	SpectrumGetZipName, SpecPrjrevRomInfo, SpecPrjrevRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Renegade II - Target Renegade 128K Re-Imaginated (Hack)

static struct BurnRomInfo SpecRenegad2rcRomDesc[] = {
	{ "Renegade II - Target Renegade Re-Imaginated (2015)(Rafal Miazga).tap", 112085, 0x9929f978, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecRenegad2rc, SpecRenegad2rc, Spec128)
STD_ROM_FN(SpecRenegad2rc)

struct BurnDriver BurnSpecRenegad2rc = {
	"spec_renegad2rc", "spec_renegad2", "spec_spec128", NULL, "2015",
	"Renegade II - Target Renegade Re-Imaginated (128K) (Hack)\0", NULL, "Rafal Miazga", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE | BDF_HACK, 2, HARDWARE_SPECTRUM, GBF_SCRFIGHT, 0,
	SpectrumGetZipName, SpecRenegad2rcRomInfo, SpecRenegad2rcRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Renegade III 128K Re-Coloured (HB)

static struct BurnRomInfo SpecRenegad3rcRomDesc[] = {
	{ "Renegade III 128K Re-Coloured (2015)(Rafal Miazga).tap", 80464, 0x1eba3cb9, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecRenegad3rc, SpecRenegad3rc, Spec128)
STD_ROM_FN(SpecRenegad3rc)

struct BurnDriver BurnSpecRenegad3rc = {
	"spec_renegad3rc", "spec_renegad3", "spec_spec128", NULL, "2015",
	"Renegade III Re-Coloured (128K) (Hack)\0", NULL, "Rafal Miazga", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE | BDF_HACK, 1, HARDWARE_SPECTRUM, GBF_SCRFIGHT, 0,
	SpectrumGetZipName, SpecRenegad3rcRomInfo, SpecRenegad3rcRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Road Trippin' (48K) (HB)

static struct BurnRomInfo SpecRoadtripRomDesc[] = {
	{ "Road Trippin' (2020)(Joesoft).tap", 43691, 0x32b9471f, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecRoadtrip, SpecRoadtrip, Spectrum)
STD_ROM_FN(SpecRoadtrip)

struct BurnDriver BurnSpecRoadtrip = {
	"spec_roadtrip", "spec_roadtrip2", "spec_spectrum", NULL, "2020",
	"Road Trippin' (48K) (HB)\0", NULL, "Joesoft", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_MAZE | GBF_ACTION, 0,
	SpectrumGetZipName, SpecRoadtripRomInfo, SpecRoadtripRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Road Trippin' v2 (128K) (HB)

static struct BurnRomInfo SpecRoadtrip2RomDesc[] = {
	{ "Road Trippin' v2 (2020)(Joesoft).tap", 41414, 0xa5e4f1dc, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecRoadtrip2, SpecRoadtrip2, Spec128)
STD_ROM_FN(SpecRoadtrip2)

struct BurnDriver BurnSpecRoadtrip2 = {
	"spec_roadtrip2", NULL, "spec_spec128", NULL, "2020",
	"Road Trippin' (128K) (HB, v2)\0", NULL, "Joesoft", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_MAZE | GBF_ACTION, 0,
	SpectrumGetZipName, SpecRoadtrip2RomInfo, SpecRoadtrip2RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Run! (128K) (HB)

static struct BurnRomInfo SpecRunRomDesc[] = {
	{ "Run! (2020)(Roman Gikryt).tap", 72853, 0x4691c976, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecRun, SpecRun, Spec128)
STD_ROM_FN(SpecRun)

struct BurnDriver BurnSpecRun = {
	"spec_run", NULL, "spec_spec128", NULL, "2020",
	"Run! (128K) (HB)\0", NULL, "Roman Gikryt", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_ACTION, 0,
	SpectrumGetZipName, SpecRunRomInfo, SpecRunRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Tank Battle (128K) (HB)

static struct BurnRomInfo SpecTankbtleRomDesc[] = {
	{ "Tank battle (2020)(kas29).tap", 44955, 0x2d909b3c, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecTankbtle, SpecTankbtle, Spec128)
STD_ROM_FN(SpecTankbtle)

struct BurnDriver BurnSpecTankbtle = {
	"spec_tankbtle", NULL, "spec_spec128", NULL, "2020",
	"Tank Battle (128K) (HB)\0", NULL, "kas29", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 2, HARDWARE_SPECTRUM, GBF_ACTION, 0,
	SpectrumGetZipName, SpecTankbtleRomInfo, SpecTankbtleRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Zabij Ducha (Polish) (128K) (HB)

static struct BurnRomInfo SpecZabijdkRomDesc[] = {
	{ "Zabij Ducha (2017)(Rafal Miazga).tap", 23267, 0x4d360f64, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecZabijdk, SpecZabijdk, Spec128)
STD_ROM_FN(SpecZabijdk)

struct BurnDriver BurnSpecZabijdk = {
	"spec_zabijdk", NULL, "spec_spec128", NULL, "2017",
	"Zabij Ducha (Polish) (128K) (HB)\0", NULL, "Rafal Miazga", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_ACTION, 0,
	SpectrumGetZipName, SpecZabijdkRomInfo, SpecZabijdkRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Azzurro 8bit Jam (128K) (HB, v1.1)

static struct BurnRomInfo SpecA8bitjamRomDesc[] = {
	{ "Azzurro 8bit Jam v1.1 (2011)(Relevo).z80", 40029, 0x2a477e2f, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecA8bitjam, SpecA8bitjam, Spec128)
STD_ROM_FN(SpecA8bitjam)

struct BurnDriver BurnSpecA8bitjam = {
	"spec_a8bitjam", NULL, "spec_spec128", NULL, "2011",
	"Azzurro 8bit Jam (128K) (HB, v1.1)\0", NULL, "RELEVO", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_ACTION, 0,
	SpectrumGetZipName, SpecA8bitjamRomInfo, SpecA8bitjamRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Bouncing Bomb Redux (48K) (HB)

static struct BurnRomInfo SpecBbreduxRomDesc[] = {
	{ "Bouncing Bomb Redux (2012)(Retroleum).tap", 41717, 0xc04a9067, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecBbredux, SpecBbredux, Spectrum)
STD_ROM_FN(SpecBbredux)

struct BurnDriver BurnSpecBbredux = {
	"spec_bbredux", NULL, "spec_spectrum", NULL, "2012",
	"Bouncing Bomb Redux (48K) (HB)\0", NULL, "Retroleum", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecBbreduxRomInfo, SpecBbreduxRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Black Horse (128K) (HB)

static struct BurnRomInfo SpecBlackhorseRomDesc[] = {
	{ "Black Horse (2009)(Digital Brains).z80", 34122, 0x9e72c587, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecBlackhorse, SpecBlackhorse, Spec128)
STD_ROM_FN(SpecBlackhorse)

struct BurnDriver BurnSpecBlackhorse = {
	"spec_blackhorse", NULL, "spec_spec128", NULL, "2009",
	"Black Horse (128K) (HB)\0", NULL, "Digital Brains", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PUZZLE, 0,
	SpectrumGetZipName, SpecBlackhorseRomInfo, SpecBlackhorseRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecIntf2DIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Box Reloaded (48K) (HB)

static struct BurnRomInfo SpecBoxreloadRomDesc[] = {
	{ "Box Reloaded (2010)(Beiker Soft).z80", 43830, 0x2c4eda33, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecBoxreload, SpecBoxreload, Spectrum)
STD_ROM_FN(SpecBoxreload)

struct BurnDriver BurnSpecBoxreload = {
	"spec_boxreload", NULL, "spec_spectrum", NULL, "2010",
	"Box Reloaded (48K) (HB)\0", NULL, "Beiker Soft", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PUZZLE, 0,
	SpectrumGetZipName, SpecBoxreloadRomInfo, SpecBoxreloadRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecQAOPSpaceDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Dominetris (48K) (HB)

static struct BurnRomInfo SpecDominetrisRomDesc[] = {
	{ "Dominetris (2005)(Cronosoft).z80", 19842, 0x0c6a56e9, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecDominetris, SpecDominetris, Spectrum)
STD_ROM_FN(SpecDominetris)

struct BurnDriver BurnSpecDominetris = {
	"spec_dominetris", NULL, "spec_spectrum", NULL, "2005",
	"Dominetris (48K) (HB)\0", NULL, "Cronosoft", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PUZZLE, 0,
	SpectrumGetZipName, SpecDominetrisRomInfo, SpecDominetrisRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Invasion of the Zombie Monsters (128K) (HB)

static struct BurnRomInfo SpecInvzombieRomDesc[] = {
	{ "Invasion of the Zombie Monsters (2010)(Relevo).z80", 40094, 0x98e45a93, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecInvzombie, SpecInvzombie, Spec128)
STD_ROM_FN(SpecInvzombie)

struct BurnDriver BurnSpecInvzombie = {
	"spec_invzombie", NULL, "spec_spec128", NULL, "2010",
	"Invasion of the Zombie Monsters (128K) (HB)\0", NULL, "RELEVO", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_RUNGUN | GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecInvzombieRomInfo, SpecInvzombieRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Sploids (128K) (HB)

static struct BurnRomInfo SpecSploidsRomDesc[] = {
	{ "Sploids (2019)(Stonechat).tap", 47960, 0x04ec461b, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecSploids, SpecSploids, Spec128)
STD_ROM_FN(SpecSploids)

struct BurnDriver BurnSpecSploids = {
	"spec_sploids", NULL, "spec_spec128", NULL, "2019",
	"Sploids (128K) (HB)\0", NULL, "Stonechat", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PUZZLE, 0,
	SpectrumGetZipName, SpecSploidsRomInfo, SpecSploidsRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecQAOPSpaceDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Kyd Cadet (48K) (HB, v1.2)

static struct BurnRomInfo SpecKydcadetRomDesc[] = {
	{ "Kyd Cadet v1.2 48K (2010)(Paul Jenkinson).tap", 36329, 0x47f024fa, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecKydcadet, SpecKydcadet, Spectrum)
STD_ROM_FN(SpecKydcadet)

struct BurnDriver BurnSpecKydcadet = {
	"spec_kydcadet", NULL, "spec_spectrum", NULL, "2010",
	"Kyd Cadet (48K) (HB, v1.2)\0", "A: left - S: right - SPACE: jump", "Paul Jenkinson", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecKydcadetRomInfo, SpecKydcadetRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Lost In My Spectrum (128K) (HB)

static struct BurnRomInfo SpecLims128RomDesc[] = {
	{ "Lost In My Spectrum 128K (2021)(ZankleSoft).tap", 60314, 0x29458414, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecLims128, SpecLims128, Spec128)
STD_ROM_FN(SpecLims128)

struct BurnDriver BurnSpecLims128 = {
	"spec_lims128", NULL, "spec_spec128", NULL, "2021",
	"Lost In My Spectrum (128K) (HB)\0", NULL, "ZankleSoft", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecLims128RomInfo, SpecLims128RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Lost In My Spectrum (48K) (HB)

static struct BurnRomInfo SpecLims48RomDesc[] = {
	{ "Lost In My Spectrum 48K (2021)(ZankleSoft).tap", 52090, 0xf716e44e, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecLims48, SpecLims48, Spectrum)
STD_ROM_FN(SpecLims48)

struct BurnDriver BurnSpecLims48 = {
	"spec_lims48", "spec_lims128", "spec_spectrum", NULL, "2021",
	"Lost In My Spectrum (48K) (HB)\0", NULL, "ZankleSoft", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecLims48RomInfo, SpecLims48RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Mag the Magician (48K) (HB)

static struct BurnRomInfo SpecMagmagRomDesc[] = {
	{ "Mag the Magician (2017)(Radasten).tap", 23284, 0xdc4c5ad5, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecMagmag, SpecMagmag, Spectrum)
STD_ROM_FN(SpecMagmag)

struct BurnDriver BurnSpecMagmag = {
	"spec_magmag", NULL, "spec_spectrum", NULL, "2017",
	"Mag the Magician (48K) (HB)\0", NULL, "Radasten", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_ADV, 0,
	SpectrumGetZipName, SpecMagmagRomInfo, SpecMagmagRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecQAOPSpaceDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Pitman (48K) (HB)

static struct BurnRomInfo SpecPitmanRomDesc[] = {
	{ "Pitman (2021)(Under4Mhz).tap", 20266, 0x8295e93d, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecPitman, SpecPitman, Spectrum)
STD_ROM_FN(SpecPitman)

struct BurnDriver BurnSpecPitman = {
	"spec_pitman", NULL, "spec_spectrum", NULL, "2021",
	"Pitman (48K) (HB)\0", NULL, "Under4Mhz", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PUZZLE, 0,
	SpectrumGetZipName, SpecPitmanRomInfo, SpecPitmanRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Retroinvaders (128K) (HB)

static struct BurnRomInfo SpecRetroinv128RomDesc[] = {
	{ "Retroinvaders 128K (2011)(Climacus).tap", 80004, 0x1e3cf16c, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecRetroinv128, SpecRetroinv128, Spec128)
STD_ROM_FN(SpecRetroinv128)

struct BurnDriver BurnSpecRetroinv128 = {
	"spec_retroinv128", NULL, "spec_spec128", NULL, "2011",
	"Retroinvaders (128K) (HB)\0", NULL, "Climacus", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_SHOOT, 0,
	SpectrumGetZipName, SpecRetroinv128RomInfo, SpecRetroinv128RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecQAOPSpaceDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Jungle Queen (128K) (HB)

static struct BurnRomInfo SpecJunglequeenRomDesc[] = {
	{ "Jungle Queen (2020)(Gabriele Amore).tap", 38168, 0x46eaa69d, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecJunglequeen, SpecJunglequeen, Spec128)
STD_ROM_FN(SpecJunglequeen)

struct BurnDriver BurnSpecJunglequeen = {
	"spec_junglequeen", NULL, "spec_spec128", NULL, "2020",
	"Jungle Queen (128K) (HB)\0", NULL, "Gabriele Amore", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_ACTION, 0,
	SpectrumGetZipName, SpecJunglequeenRomInfo, SpecJunglequeenRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecQAOPMDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Just A Gal (48K) (HB)

static struct BurnRomInfo SpecJustagalRomDesc[] = {
	{ "Just A Gal (2019)(Zosya Entertainment).tap", 47039, 0x70b2bfdc, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecJustagal, SpecJustagal, Spectrum)
STD_ROM_FN(SpecJustagal)

struct BurnDriver BurnSpecJustagal = {
	"spec_justagal", NULL, "spec_spectrum", NULL, "2019",
	"Just A Gal (48K) (HB)\0", NULL, "Zosya Entertainment", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_RACING, 0,
	SpectrumGetZipName, SpecJustagalRomInfo, SpecJustagalRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Safecracker (48K) (HB)

static struct BurnRomInfo SpecSafecrackerRomDesc[] = {
	{ "Safecracker (2010)(Rafal Miazga).tap", 24134, 0xbb92aeb8, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecSafecracker, SpecSafecracker, Spectrum)
STD_ROM_FN(SpecSafecracker)

struct BurnDriver BurnSpecSafecracker = {
	"spec_safecracker", NULL, "spec_spectrum", NULL, "2010",
	"Safecracker (48K) (HB)\0", NULL, "Rafal Miazga", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PUZZLE, 0,
	SpectrumGetZipName, SpecSafecrackerRomInfo, SpecSafecrackerRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Sector (128K) (HB)

static struct BurnRomInfo SpecSectorRomDesc[] = {
	{ "Sector (2013)(RetroSouls).z80", 27939, 0x103cfccf, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecSector, SpecSector, Spec128)
STD_ROM_FN(SpecSector)

struct BurnDriver BurnSpecSector = {
	"spec_sector", NULL, "spec_spec128", NULL, "2013",
	"Sector (128K) (HB)\0", NULL, "RetroSouls", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_ACTION, 0,
	SpectrumGetZipName, SpecSectorRomInfo, SpecSectorRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Seto Taisho Vs Yokai (128K) (HB)

static struct BurnRomInfo SpecSetaishoRomDesc[] = {
	{ "Seto Taisho Vs Yokai (2016)(Monument Microgames).tap", 83481, 0x4a0550ce, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecSetaisho, SpecSetaisho, Spec128)
STD_ROM_FN(SpecSetaisho)

struct BurnDriver BurnSpecSetaisho = {
	"spec_setaisho", NULL, "spec_spec128", NULL, "2016",
	"Seto Taisho Vs Yokai (128K) (HB)\0", NULL, "Monument Microgames", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecSetaishoRomInfo, SpecSetaishoRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Diver - Mystery of the Deep (128K) (HB)

static struct BurnRomInfo SpecDiverRomDesc[] = {
	{ "Diver - Mystery of the Deep 128K (2004)(HorrorSoft).tap", 40385, 0x4b38e049, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecDiver, SpecDiver, Spec128)
STD_ROM_FN(SpecDiver)

struct BurnDriver BurnSpecDiver = {
	"spec_diver", NULL, "spec_spec128", NULL, "2004",
	"Diver - Mystery of the Deep (128K) (HB)\0", "6, 7 and Enter to choose and set options", "HorrorSoft", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_ACTION, 0,
	SpectrumGetZipName, SpecDiverRomInfo, SpecDiverRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecQAOPSpaceDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Doom Pit (128K) (HB)

static struct BurnRomInfo SpecDoompitRomDesc[] = {
	{ "Doom Pit 128K (2017)(Monument Microgames).tap", 63208, 0xed3fc3ca, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecDoompit, SpecDoompit, Spec128)
STD_ROM_FN(SpecDoompit)

struct BurnDriver BurnSpecDoompit = {
	"spec_doompit", NULL, "spec_spec128", NULL, "2017",
	"Doom Pit (128K) (HB)\0", "Press SPACE to choose and ENTER to set options", "Monument Microgames", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecDoompitRomInfo, SpecDoompitRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Frogger 2009 (128K) (HB)

static struct BurnRomInfo SpecFrogger2009RomDesc[] = {
	{ "Frogger 2009 (2009)(Deanysoft).tap", 36850, 0xcee5cee5, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecFrogger2009, SpecFrogger2009, Spec128)
STD_ROM_FN(SpecFrogger2009)

struct BurnDriver BurnSpecFrogger2009 = {
	"spec_frogger2009", NULL, "spec_spec128", NULL, "2009",
	"Frogger (128K) (HB)\0", NULL, "Deanysoft", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_ACTION, 0,
	SpectrumGetZipName, SpecFrogger2009RomInfo, SpecFrogger2009RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Heroes Rescue (128K) (HB)

static struct BurnRomInfo SpecHerorescueRomDesc[] = {
	{ "Heroes Rescue (2016)(Defecto Digital Studios).tap", 35792, 0x70ccdcda, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecHerorescue, SpecHerorescue, Spec128)
STD_ROM_FN(SpecHerorescue)

struct BurnDriver BurnSpecHerorescue = {
	"spec_herorescue", NULL, "spec_spec128", NULL, "2016",
	"Heroes Rescue (128K) (HB)\0", "Press '2' to use controller", "Defecto Digital Studios", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecHerorescueRomInfo, SpecHerorescueRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Homebrew (48K) (HB)

static struct BurnRomInfo SpecHomebrewRomDesc[] = {
	{ "Homebrew 48K (2009)(Jonathan Cauldwell).tap", 39137, 0x6e962533, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecHomebrew, SpecHomebrew, Spectrum)
STD_ROM_FN(SpecHomebrew)

struct BurnDriver BurnSpecHomebrew = {
	"spec_homebrew", NULL, "spec_spectrum", NULL, "2009",
	"Homebrew (48K) (HB)\0", NULL, "Jonathan Cauldwell", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_SHOOT, 0,
	SpectrumGetZipName, SpecHomebrewRomInfo, SpecHomebrewRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Joe Blob (128K) (HB, v2)

static struct BurnRomInfo SpecJoeblobRomDesc[] = {
	{ "Joe Blob v2 (2019)(Stonechat Productions).tap", 46630, 0x05f21de8, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecJoeblob, SpecJoeblob, Spec128)
STD_ROM_FN(SpecJoeblob)

struct BurnDriver BurnSpecJoeblob = {
	"spec_joeblob", NULL, "spec_spec128", NULL, "2019",
	"Joe Blob (128K) (HB, v2)\0", NULL, "Stonechat Productions", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_MAZE | GBF_PUZZLE, 0,
	SpectrumGetZipName, SpecJoeblobRomInfo, SpecJoeblobRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Nanako in Classic Japanese Monster Castle (48K) (HB)

static struct BurnRomInfo SpecNanakoRomDesc[] = {
	{ "Nanako in Classic Japanese Monster Castle 48K (2007)(Computer Emuzone).z80", 34742, 0x89eca399, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecNanako, SpecNanako, Spectrum)
STD_ROM_FN(SpecNanako)

struct BurnDriver BurnSpecNanako = {
	"spec_nanako", NULL, "spec_spectrum", NULL, "2007",
	"Nanako in Classic Japanese Monster Castle (48K) (HB)\0", NULL, "Computer Emuzone", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PUZZLE | GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecNanakoRomInfo, SpecNanakoRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecQAOPSpaceDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Pre-ZU (128K) (HB)

static struct BurnRomInfo SpecPrezuRomDesc[] = {
	{ "Pre-ZU (2019)(Vyacheslav Tretyak).tap", 22635, 0xaa7a7291, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecPrezu, SpecPrezu, Spec128)
STD_ROM_FN(SpecPrezu)

struct BurnDriver BurnSpecPrezu = {
	"spec_prezu", NULL, "spec_spec128", NULL, "2019",
	"Pre-ZU (128K) (HB)\0", NULL, "Vyacheslav Tretyak", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PUZZLE | GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecPrezuRomInfo, SpecPrezuRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecQAOPMDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// ZX Destroyer (Spanish) (48K) (HB)

static struct BurnRomInfo SpecZxdestroyerRomDesc[] = {
	{ "ZX Destroyer (2014)(Retrobytes Productions).tap", 44241, 0x50bca6d2, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecZxdestroyer, SpecZxdestroyer, Spectrum)
STD_ROM_FN(SpecZxdestroyer)

struct BurnDriver BurnSpecZxdestroyer = {
	"spec_zxdestroyer", NULL, "spec_spectrum", NULL, "2014",
	"ZX Destroyer (Spanish) (48K) (HB)\0", NULL, "Retrobytes Productions", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_SHOOT, 0,
	SpectrumGetZipName, SpecZxdestroyerRomInfo, SpecZxdestroyerRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecQAOPSpaceDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Coza De La Poza, La (Spanish) (128K) (HB)

static struct BurnRomInfo SpecCozapozaRomDesc[] = {
	{ "La Coza De La Poza (2021)(Furillo Productions).tap", 120638, 0x315bbb22, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecCozapoza, SpecCozapoza, Spec128)
STD_ROM_FN(SpecCozapoza)

struct BurnDriver BurnSpecCozapoza = {
	"spec_cozapoza", "spec_doompond", "spec_spec128", NULL, "2021",
	"Coza De La Poza, La (Spanish) (128K) (HB)\0", NULL, "Furillo Productions", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_ADV, 0,
	SpectrumGetZipName, SpecCozapozaRomInfo, SpecCozapozaRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Doom of the Pond, The (English) (128K) (HB)

static struct BurnRomInfo SpecDoompondRomDesc[] = {
	{ "The Doom of the Pond (2021)(Furillo Productions).tap", 121742, 0x09a1aac6, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecDoompond, SpecDoompond, Spec128)
STD_ROM_FN(SpecDoompond)

struct BurnDriver BurnSpecDoompond = {
	"spec_doompond", NULL, "spec_spec128", NULL, "2021",
	"Doom of the Pond, The (English) (128K) (HB)\0", NULL, "Furillo Productions", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_ADV, 0,
	SpectrumGetZipName, SpecDoompondRomInfo, SpecDoompondRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Moon Ranger (128K) (HB)

static struct BurnRomInfo SpecMoonrangerRomDesc[] = {
	{ "Moon Ranger (2020)(Gabriele Amore).tap", 29940, 0xd33c199a, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecMoonranger, SpecMoonranger, Spec128)
STD_ROM_FN(SpecMoonranger)

struct BurnDriver BurnSpecMoonranger = {
	"spec_moonranger", NULL, "spec_spec128", NULL, "2020",
	"Moon Ranger (128K) (HB)\0", NULL, "Gabriele Amore", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_ACTION, 0,
	SpectrumGetZipName, SpecMoonrangerRomInfo, SpecMoonrangerRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecQAOPMDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Pataslocas (128K) (HB)

static struct BurnRomInfo SpecPataslocasRomDesc[] = {
	{ "Pataslocas (2020)(Beyker Soft).tap", 48869, 0x242d2c7c, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecPataslocas, SpecPataslocas, Spec128)
STD_ROM_FN(SpecPataslocas)

struct BurnDriver BurnSpecPataslocas = {
	"spec_pataslocas", NULL, "spec_spec128", NULL, "2020",
	"Pataslocas (128K) (HB)\0", NULL, "Beyker Soft", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_ACTION | GBF_ADV, 0,
	SpectrumGetZipName, SpecPataslocasRomInfo, SpecPataslocasRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecQAOPSpaceDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Baldy ZX (48K) (HB)

static struct BurnRomInfo SpecBaldyzxRomDesc[] = {
	{ "Baldy ZX 48K (2015)(Paul Jenkinson).tap", 35494, 0xa7a3b233, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecBaldyzx, SpecBaldyzx, Spectrum)
STD_ROM_FN(SpecBaldyzx)

struct BurnDriver BurnSpecBaldyzx = {
	"spec_baldyzx", NULL, "spec_spectrum", NULL, "2015",
	"Baldy ZX (48K) (HB)\0", NULL, "Paul Jenkinson", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_ACTION, 0,
	SpectrumGetZipName, SpecBaldyzxRomInfo, SpecBaldyzxRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Future Looter (48K)

static struct BurnRomInfo SpecFuturelooterRomDesc[] = {
	{ "Future Looter 48K (2011)(Timmy).tap", 46338, 0x01962866, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecFuturelooter, SpecFuturelooter, Spectrum)
STD_ROM_FN(SpecFuturelooter)

struct BurnDriver BurnSpecFuturelooter = {
	"spec_futurelooter", NULL, "spec_spectrum", NULL, "2011",
	"Future Looter (48K) (HB)\0", NULL, "Timmy", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_SHOOT, 0,
	SpectrumGetZipName, SpecFuturelooterRomInfo, SpecFuturelooterRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Metal Man Remixed (English) (48K) (HB)

static struct BurnRomInfo SpecMetalmanrmxRomDesc[] = {
	{ "Metal Man Remixed 48K EN (2015)(Origin, Oleg).tap", 126688, 0x0f6c6bb5, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecMetalmanrmx, SpecMetalmanrmx, Spectrum)
STD_ROM_FN(SpecMetalmanrmx)

struct BurnDriver BurnSpecMetalmanrmx = {
	"spec_metalmanrmx", NULL, "spec_spectrum", NULL, "2015",
	"Metal Man Remixed (English) (48K) (HB)\0", NULL, "Oleg Origin", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_RUNGUN | GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecMetalmanrmxRomInfo, SpecMetalmanrmxRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Metal Man Remixed (Russian) (48K) (HB)

static struct BurnRomInfo SpecMetalmanrmxruRomDesc[] = {
	{ "Metal Man Remixed 48K RU (2015)(Origin, Oleg).tap", 126688, 0xbeac0298, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecMetalmanrmxru, SpecMetalmanrmxru, Spectrum)
STD_ROM_FN(SpecMetalmanrmxru)

struct BurnDriver BurnSpecMetalmanrmxru = {
	"spec_metalmanrmxru", "spec_metalmanrmx", "spec_spectrum", NULL, "2015",
	"Metal Man Remixed (Russian) (48K) (HB)\0", NULL, "Oleg Origin", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_RUNGUN | GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecMetalmanrmxruRomInfo, SpecMetalmanrmxruRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Hobbit, The (128K Edition) (HB)

static struct BurnRomInfo SpecHobbit128RomDesc[] = {
	{ "Hobbit, The 128K Edition (2015)(World of Spectrum).tap", 128019, 0x95cc8713, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecHobbit128, SpecHobbit128, Spec128)
STD_ROM_FN(SpecHobbit128)

struct BurnDriver BurnSpecHobbit128 = {
	"spec_hobbit128", NULL, "spec_spec128", NULL, "2015",
	"Hobbit, The (128K Edition) (HB)\0", NULL, "World of Spectrum", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_ADV, 0,
	SpectrumGetZipName, SpecHobbit128RomInfo, SpecHobbit128RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Paleto Jones (48K) (HB)

static struct BurnRomInfo SpecPaletojonesRomDesc[] = {
	{ "Paletojones 48K (2021)(Mananuk).tap", 44486, 0x9c9ec285, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecPaletojones, SpecPaletojones, Spectrum)
STD_ROM_FN(SpecPaletojones)

struct BurnDriver BurnSpecPaletojones = {
	"spec_paletojones", NULL, "spec_spectrum", NULL, "2021",
	"Paleto Jones (48K) (HB)\0", NULL, "Mananuk", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecPaletojonesRomInfo, SpecPaletojonesRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Pillars (128K) (HB)

static struct BurnRomInfo SpecPillarsRomDesc[] = {
	{ "Pillars 128K (2021)(Under4Mhz).tap", 19619, 0x8a6dd7c2, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecPillars, SpecPillars, Spec128)
STD_ROM_FN(SpecPillars)

struct BurnDriver BurnSpecPillars = {
	"spec_pillars", NULL, "spec_spec128", NULL, "2021",
	"Pillars (128K) (HB)\0", NULL, "Under4Mhz", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PUZZLE, 0,
	SpectrumGetZipName, SpecPillarsRomInfo, SpecPillarsRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Stranded (48K) (HB)

static struct BurnRomInfo SpecStrandedRomDesc[] = {
	{ "Stranded 48K (2005)(Cronosoft).tap", 33181, 0x1f36f27e, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecStranded, SpecStranded, Spectrum)
STD_ROM_FN(SpecStranded)

struct BurnDriver BurnSpecStranded = {
	"spec_stranded", NULL, "spec_spectrum", NULL, "2005",
	"Stranded (48K) (HB)\0", NULL, "Cronosoft", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PUZZLE, 0,
	SpectrumGetZipName, SpecStrandedRomInfo, SpecStrandedRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Stripping Penelope (128K) (HB)

static struct BurnRomInfo SpecStripenelopeRomDesc[] = {
	{ "Stripping Penelope (2021)(Jaime Grilo).tap", 44551, 0x578a4f32, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecStripenelope, SpecStripenelope, Spec128)
STD_ROM_FN(SpecStripenelope)

struct BurnDriver BurnSpecStripenelope = {
	"spec_stripenelope", NULL, "spec_spec128", NULL, "2021",
	"Stripping Penelope (128K) (HB)\0", NULL, " Jaime Grilo", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecStripenelopeRomInfo, SpecStripenelopeRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecQAOPSpaceDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Nightmare on Halloween (48K) (HB)

static struct BurnRomInfo SpecNighthallowenRomDesc[] = {
	{ "Nightmare on Halloween 48K (2013)(Radastan).tap", 39246, 0x06b32e79, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecNighthallowen, SpecNighthallowen, Spectrum)
STD_ROM_FN(SpecNighthallowen)

struct BurnDriver BurnSpecNighthallowen = {
	"spec_nighthallowen", NULL, "spec_spectrum", NULL, "2013",
	"Nightmare on Halloween (48K) (HB)\0", NULL, "Radastan", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecNighthallowenRomInfo, SpecNighthallowenRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Skurff (48K) (HB)

static struct BurnRomInfo SpecSkurffRomDesc[] = {
	{ "Skurff 48K (2013)(Stonechat Production).tap", 37171, 0x596f4c0c, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecSkurff, SpecSkurff, Spec128)
STD_ROM_FN(SpecSkurff)

struct BurnDriver BurnSpecSkurff = {
	"spec_skurff", NULL, "spec_spec128", NULL, "2013",
	"Skurff (48K) (HB)\0", NULL, "Stonechat Production", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecSkurffRomInfo, SpecSkurffRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecQAOPSpaceDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Vade Retro (128K) (HB)

static struct BurnRomInfo SpecVaderetroRomDesc[] = {
	{ "Vade Retro (2015)(RetroWorks).tap", 41204, 0x6252d3c9, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecVaderetro, SpecVaderetro, Spec128)
STD_ROM_FN(SpecVaderetro)

struct BurnDriver BurnSpecVaderetro = {
	"spec_vaderetro", NULL, "spec_spec128", NULL, "2015",
	"Vade Retro (128K) (HB)\0", "Press any key to start", "RetroWorks", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PLATFORM | GBF_ADV, 0,
	SpectrumGetZipName, SpecVaderetroRomInfo, SpecVaderetroRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecIntf2DIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Fucking Mili (English) (48K) (HB)

static struct BurnRomInfo SpecFkngmiliRomDesc[] = {
	{ "Fucking Mili (2020)(EJVG).tap", 39535, 0x04778248, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecFkngmili, SpecFkngmili, Spectrum)
STD_ROM_FN(SpecFkngmili)

struct BurnDriver BurnSpecFkngmili = {
	"spec_fkngmili", NULL, "spec_spectrum", NULL, "2020",
	"Fucking Mili (English) (48K) (HB)\0", NULL, "EJVG", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecFkngmiliRomInfo, SpecFkngmiliRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Pacman the Curse of the Slimers (48K) (HB, v1.1)

static struct BurnRomInfo SpecPmslimersRomDesc[] = {
	{ "Pacman the Curse of the Slimers v1.1 (2020)(DEFB Studio).z80", 25176, 0xe554c91e, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecPmslimers, SpecPmslimers, Spectrum)
STD_ROM_FN(SpecPmslimers)

struct BurnDriver BurnSpecPmslimers = {
	"spec_pmslimers", NULL, "spec_spectrum", NULL, "2020",
	"Pacman the Curse of the Slimers (48K) (HB, v1.1)\0", NULL, "DEFB Studio", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_MAZE | GBF_ACTION, 0,
	SpectrumGetZipName, SpecPmslimersRomInfo, SpecPmslimersRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecQAOPSpaceDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Puta Mili (Spanish) (48K) (HB)

static struct BurnRomInfo SpecPutamiliRomDesc[] = {
	{ "Puta Mili (2020)(EJVG).tap", 39542, 0xe1014a13, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecPutamili, SpecPutamili, Spectrum)
STD_ROM_FN(SpecPutamili)

struct BurnDriver BurnSpecPutamili = {
	"spec_putamili", "spec_fkngmili", "spec_spectrum", NULL, "2020",
	"Puta Mili (Spanish) (48K) (HB)\0", NULL, "EJVG", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecPutamiliRomInfo, SpecPutamiliRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Tardigrade (48K) (HB)

static struct BurnRomInfo SpecTardigradeRomDesc[] = {
	{ "Tardigrade (2021)(Dave Hughes).tap", 15763, 0x579ce6ba, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecTardigrade, SpecTardigrade, Spectrum)
STD_ROM_FN(SpecTardigrade)

struct BurnDriver BurnSpecTardigrade = {
	"spec_tardigrade", NULL, "spec_spectrum", NULL, "2021",
	"Tardigrade (48K) (HB)\0", NULL, "Dave Hughes", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_MAZE | GBF_ACTION, 0,
	SpectrumGetZipName, SpecTardigradeRomInfo, SpecTardigradeRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Humans, The (128K) (HB)

static struct BurnRomInfo SpecHumansRomDesc[] = {
	{ "Humans, The (2021)(Gabriele Amore).tap", 53285, 0xceb5401e, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecHumans, SpecHumans, Spec128)
STD_ROM_FN(SpecHumans)

struct BurnDriver BurnSpecHumans = {
	"spec_humans", NULL, "spec_spec128", NULL, "2021",
	"Humans, The (128K) (HB)\0", NULL, "Gabriele Amore", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PUZZLE, 0,
	SpectrumGetZipName, SpecHumansRomInfo, SpecHumansRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecQAOPMDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Dark Redux, The (Russian) (48K) (HB)

static struct BurnRomInfo SpecDarkreduxruRomDesc[] = {
	{ "Dark Redux, The RU (2021)(Zosya ent.).tap", 46873, 0xb78e53c3, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecDarkreduxru, SpecDarkreduxru, Spectrum)
STD_ROM_FN(SpecDarkreduxru)

struct BurnDriver BurnSpecDarkreduxru = {
	"spec_darkreduxru", "spec_darkredux", "spec_spectrum", NULL, "2021",
	"Dark Redux, The (Russian) (48K) (HB)\0", "Hold 'P' at the logo screen to skip the intro", "Zosya Entertainment", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_MAZE | GBF_RUNGUN, 0,
	SpectrumGetZipName, SpecDarkreduxruRomInfo, SpecDarkreduxruRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Esta en la Caja (Spanish) (48K) (HB)

static struct BurnRomInfo SpecEstacajaRomDesc[] = {
	{ "Esta en la Caja ES 48K (2009)(RELEVO).tap", 34823, 0x30e4b16b, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecEstacaja, SpecEstacaja, Spectrum)
STD_ROM_FN(SpecEstacaja)

struct BurnDriver BurnSpecEstacaja = {
	"spec_estacaja", NULL, "spec_spectrum", NULL, "2009",
	"Esta en la Caja (Spanish) (48K) (HB)\0", NULL, "RELEVO", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_ADV, 0,
	SpectrumGetZipName, SpecEstacajaRomInfo, SpecEstacajaRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Project Vaelius (128K) (HB)

static struct BurnRomInfo SpecPvaeliusRomDesc[] = {
	{ "Project Vaelius 128K (2021)(Toku Soft).tap", 43218, 0x4eae4683, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecPvaelius, SpecPvaelius, Spec128)
STD_ROM_FN(SpecPvaelius)

struct BurnDriver BurnSpecPvaelius = {
	"spec_pvaelius", NULL, "spec_spec128", NULL, "2021",
	"Project Vaelius (128K) (HB)\0", NULL, "Toku Soft", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_RUNGUN, 0,
	SpectrumGetZipName, SpecPvaeliusRomInfo, SpecPvaeliusRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Sokoban (Spanish) (128K) (HB)

static struct BurnRomInfo SpecSokobanRomDesc[] = {
	{ "Sokoban ES 128K (2006)(Compiler).tap", 48579, 0x5f38c1ce, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecSokoban, SpecSokoban, Spec128)
STD_ROM_FN(SpecSokoban)

struct BurnDriver BurnSpecSokoban = {
	"spec_sokoban", NULL, "spec_spec128", NULL, "2006",
	"Sokoban (Spanish) (128K) (HB)\0", "Press 'E' for in-game menu", "Compiler", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PUZZLE, 0,
	SpectrumGetZipName, SpecSokobanRomInfo, SpecSokobanRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Tiki Taca (48K) (HB)

static struct BurnRomInfo SpecTikitacaRomDesc[] = {
	{ "Tiki Taca 48K (2019)(Climacus).tap", 42129, 0xd8eb796b, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecTikitaca, SpecTikitaca, Spectrum)
STD_ROM_FN(SpecTikitaca)

struct BurnDriver BurnSpecTikitaca = {
	"spec_tikitaca", NULL, "spec_spectrum", NULL, "2019",
	"Tiki Taca (48K) (HB)\0", NULL, "Climacus", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_MAZE | GBF_ADV, 0,
	SpectrumGetZipName, SpecTikitacaRomInfo, SpecTikitacaRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Super 48K Box (128K)

static struct BurnRomInfo SpecSuper48kboxRomDesc[] = {
	{ "Super 48K Box 128K (2015)(Vanbsoftware).tap", 40975, 0xb2f7b936, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecSuper48kbox, SpecSuper48kbox, Spec128)
STD_ROM_FN(SpecSuper48kbox)

struct BurnDriver BurnSpecSuper48kbox = {
	"spec_super48kbox", NULL, "spec_spec128", NULL, "2015",
	"Super 48K Box (128K) (HB)\0", NULL, "Vanbsoftware", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_ACTION, 0,
	SpectrumGetZipName, SpecSuper48kboxRomInfo, SpecSuper48kboxRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// 7.7.7.7. (128K) (HB)

static struct BurnRomInfo Spec7777RomDesc[] = {
	{ "7.7.7.7. 128K (2023)(Anhot studio).tap", 48254, 0x957a3c62, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Spec7777, Spec7777, Spec128)
STD_ROM_FN(Spec7777)

struct BurnDriver BurnSpec7777 = {
	"spec_7777", NULL, "spec_spec128", NULL, "2023",
	"7.7.7.7. (128K) (HB)\0", NULL, "Anhot studio", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, Spec7777RomInfo, Spec7777RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecIntf2SwapDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// 8-Bit Wars, The (48K) (HB)

static struct BurnRomInfo Spec8bitwarsRomDesc[] = {
	{ "8-Bit Wars, The 48K (2022)(Cronosoft).tap", 44740, 0x76adb352, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Spec8bitwars, Spec8bitwars, Spectrum)
STD_ROM_FN(Spec8bitwars)

struct BurnDriver BurnSpec8bitwars = {
	"spec_8bitwars", NULL, "spec_spectrum", NULL, "2022",
	"8-Bit Wars, The (48K) (HB)\0", NULL, "Cronosoft", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, Spec8bitwarsRomInfo, Spec8bitwarsRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// 8 Channels (48K) (HB, Tech-Demo)

static struct BurnRomInfo Spec8chsRomDesc[] = {
	{ "8 Channels 48K (2011)(Mister Beep).tap", 20279, 0x53d9a4a6, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Spec8chs, Spec8chs, Spectrum)
STD_ROM_FN(Spec8chs)

struct BurnDriver BurnSpec8chs = {
	"spec_8chs", NULL, "spec_spectrum", NULL, "2011",
	"8 Channels (48K) (HB, Tech-Demo)\0", NULL, "Mister Beep", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW | BDF_DEMO, 1, HARDWARE_SPECTRUM, GBF_MISC, 0,
	SpectrumGetZipName, Spec8chsRomInfo, Spec8chsRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// 70908 (128K) (HB, Tech-Demo)

static struct BurnRomInfo Spec70908RomDesc[] = {
	{ "70908 128K (2019)(Scopex).tap", 84661, 0xe2268f42, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Spec70908, Spec70908, Spec128)
STD_ROM_FN(Spec70908)

struct BurnDriver BurnSpec70908 = {
	"spec_70908", NULL, "spec_spec128", NULL, "2019",
	"70908 (128K) (HB, Tech-Demo)\0", NULL, "Scopex", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW | BDF_DEMO, 1, HARDWARE_SPECTRUM, GBF_MISC, 0,
	SpectrumGetZipName, Spec70908RomInfo, Spec70908RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Acolyte (48K) (HB)

static struct BurnRomInfo SpecAcolyteRomDesc[] = {
	{ "Acolyte 48K (2009)(Video Images).tap", 54512, 0x94cd2150, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecAcolyte, SpecAcolyte, Spectrum)
STD_ROM_FN(SpecAcolyte)

struct BurnDriver BurnSpecAcolyte = {
	"spec_acolyte", NULL, "spec_spectrum", NULL, "2009",
	"Acolyte (48K) (HB)\0", "AY Sound supported", "Video Images", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_MAZE | GBF_ADV, 0,
	SpectrumGetZipName, SpecAcolyteRomInfo, SpecAcolyteRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Acolyte 2 - The Quest for Celeste (128K) (HB)

static struct BurnRomInfo SpecAcolyte2RomDesc[] = {
	{ "Acolyte 2 - The Quest for Celeste 128K (2021)(Deanysoft).tap", 101851, 0xc120c549, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecAcolyte2, SpecAcolyte2, Spec128)
STD_ROM_FN(SpecAcolyte2)

struct BurnDriver BurnSpecAcolyte2 = {
	"spec_acolyte2", NULL, "spec_spec128", NULL, "2021",
	"Acolyte 2 - The Quest for Celeste (128K) (HB)\0", NULL, "Deanysoft", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_ACTION | GBF_ADV, 0,
	SpectrumGetZipName, SpecAcolyte2RomInfo, SpecAcolyte2RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Add Astra (128K) (HB, Tech-Demo)

static struct BurnRomInfo SpecAddastraRomDesc[] = {
	{ "Add Astra 128K (2023)(Joker, Arise Design).tap", 578887, 0x4f2b48f7, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecAddastra, SpecAddastra, Spec128)
STD_ROM_FN(SpecAddastra)

struct BurnDriver BurnSpecAddastra = {
	"spec_addastra", NULL, "spec_spec128", NULL, "2023",
	"Add Astra 128K (128K) (HB, Tech-Demo)\0", NULL, "Joker, Arise Design", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW | BDF_DEMO, 1, HARDWARE_SPECTRUM, GBF_MISC, 0,
	SpectrumGetZipName, SpecAddastraRomInfo, SpecAddastraRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Ad Lunam Plus (English) (128K) (HB, v1.1)

static struct BurnRomInfo SpecAdlunamplusRomDesc[] = {
	{ "Ad Lunam Plus EN 128K (2020)(Alessandro Grussu).tap", 83541, 0x41988944, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecAdlunamplus, SpecAdlunamplus, Spec128)
STD_ROM_FN(SpecAdlunamplus)

struct BurnDriver BurnSpecAdlunamplus = {
	"spec_adlunamplus", NULL, "spec_spec128", NULL, "2020",
	"Ad Lunam Plus (English) (128K) (HB, v1.1)\0", NULL, "Alessandro Grussu", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_STRATEGY, 0,
	SpectrumGetZipName, SpecAdlunamplusRomInfo, SpecAdlunamplusRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Ad Lunam Plus (Spanish) (128K) (HB, v1.1)

static struct BurnRomInfo SpecAdlunamplusesRomDesc[] = {
	{ "Ad Lunam Plus ES 128K (2020)(Alessandro Grussu).tap", 83712, 0x1d812053, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecAdlunampluses, SpecAdlunampluses, Spec128)
STD_ROM_FN(SpecAdlunampluses)

struct BurnDriver BurnSpecAdlunampluses = {
	"spec_adlunampluses", "spec_adlunamplus", "spec_spec128", NULL, "2020",
	"Ad Lunam Plus (Spanish) (128K) (HB, v1.1)\0", NULL, "Alessandro Grussu", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_STRATEGY, 0,
	SpectrumGetZipName, SpecAdlunamplusesRomInfo, SpecAdlunamplusesRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Ad Lunam Plus (Italian) (128K) (HB, v1.1)

static struct BurnRomInfo SpecAdlunamplusitRomDesc[] = {
	{ "Ad Lunam Plus IT 128K (2020)(Alessandro Grussu).tap", 83768, 0xc27e30a2, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecAdlunamplusit, SpecAdlunamplusit, Spec128)
STD_ROM_FN(SpecAdlunamplusit)

struct BurnDriver BurnSpecAdlunamplusit = {
	"spec_adlunamplusit", "spec_adlunamplus", "spec_spec128", NULL, "2020",
	"Ad Lunam Plus (Italian) (128K) (HB, v1.1)\0", NULL, "Alessandro Grussu", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_STRATEGY, 0,
	SpectrumGetZipName, SpecAdlunamplusitRomInfo, SpecAdlunamplusitRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// ADVENTURES Continue - Side A (128K) (HB)

static struct BurnRomInfo SpecAdvcontaRomDesc[] = {
	{ "ADVENTURES Continue - Side A 128K (2021)(PC NONO Games).tap", 47271, 0xbbe196d4, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecAdvconta, SpecAdvconta, Spec128)
STD_ROM_FN(SpecAdvconta)

struct BurnDriver BurnSpecAdvconta = {
	"spec_advconta", NULL, "spec_spec128", NULL, "2021",
	"ADVENTURES Continue - Side A (128K) (HB)\0", NULL, "PC NONO Games", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PUZZLE, 0,
	SpectrumGetZipName, SpecAdvcontaRomInfo, SpecAdvcontaRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// ADVENTURES Continue - Side B (128K) (HB)

static struct BurnRomInfo SpecAdvcontbRomDesc[] = {
	{ "ADVENTURES Continue - Side B 128K (2021)(PC NONO Games).tap", 47965, 0xdb75b5f9, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecAdvcontb, SpecAdvcontb, Spec128)
STD_ROM_FN(SpecAdvcontb)

struct BurnDriver BurnSpecAdvcontb = {
	"spec_advcontb", "spec_advconta", "spec_spec128", NULL, "2021",
	"ADVENTURES Continue - Side B (128K) (HB)\0", NULL, "PC NONO Games", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PUZZLE, 0,
	SpectrumGetZipName, SpecAdvcontbRomInfo, SpecAdvcontbRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Adventures of Jane Jelly, The - The Treasure of Zedin (128K) (HB)

static struct BurnRomInfo SpecAdvsjaneRomDesc[] = {
	{ "Adventures of Jane Jelly, The - The Treasure of Zedin (2017)(Jaime Grilo).tap", 42992, 0x7079b65b, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecAdvsjane, SpecAdvsjane, Spec128)
STD_ROM_FN(SpecAdvsjane)

struct BurnDriver BurnSpecAdvsjane = {
	"spec_advsjane", NULL, "spec_spec128", NULL, "2017",
	"Adventures of Jane Jelly, The: the Treasure of Zedin (128K) (HB)\0", NULL, "Jaime Grilo", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecAdvsjaneRomInfo, SpecAdvsjaneRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Adventures of Jane Jelly 2, The - The Treasure of Hotmarmalade (128K) (HB)

static struct BurnRomInfo SpecAdvsjane2RomDesc[] = {
	{ "Adventures of Jane Jelly 2, The - The Treasure of Hotmarmalade (2017)(Jaime Grilo).tap", 43600, 0x8df0aa08, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecAdvsjane2, SpecAdvsjane2, Spec128)
STD_ROM_FN(SpecAdvsjane2)

struct BurnDriver BurnSpecAdvsjane2 = {
	"spec_advsjane2", NULL, "spec_spec128", NULL, "2017",
	"Adventures of Jane Jelly 2, The: the Treasure of Hotmarmalade (128K) (HB)\0", NULL, "Jaime Grilo", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecAdvsjane2RomInfo, SpecAdvsjane2RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Adventures of Jane Jelly 3, The - The Egg Diamond (128K) (HB)

static struct BurnRomInfo SpecAdvsjane3RomDesc[] = {
	{ "Adventures of Jane Jelly 3, The - The Egg Diamond (2018)(Jaime Grilo).tap", 46363, 0xa93a3d40, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecAdvsjane3, SpecAdvsjane3, Spec128)
STD_ROM_FN(SpecAdvsjane3)

struct BurnDriver BurnSpecAdvsjane3 = {
	"spec_advsjane3", NULL, "spec_spec128", NULL, "2018",
	"Adventures of Jane Jelly 3, The: the Egg Diamond (128K) (HB)\0", NULL, "Jaime Grilo", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecAdvsjane3RomInfo, SpecAdvsjane3RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Adventures of Kildu the Goblin, The - The New Home (48K) (HB)

static struct BurnRomInfo SpecAdvskilduRomDesc[] = {
	{ "Adventures of Kildu the Goblin, The - The New Home 48K (2024)(NOCKMANN Studio).tap", 16578, 0x2da12bcd, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecAdvskildu, SpecAdvskildu, Spectrum)
STD_ROM_FN(SpecAdvskildu)

struct BurnDriver BurnSpecAdvskildu = {
	"spec_advskildu", NULL, "spec_spectrum", NULL, "2024",
	"Adventures of Kildu the Goblin, The - The New Home (48K) (HB)\0", NULL, "NICKMANN Studio", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_ADV, 0,
	SpectrumGetZipName, SpecAdvskilduRomInfo, SpecAdvskilduRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Aeon 1 - Arrival at the Planet (128K) (HB)

static struct BurnRomInfo SpecAeon1RomDesc[] = {
	{ "Aeon 1 - Arrival at the Planet 128K (2018)(Sunteam).tap", 49959, 0x9ff27533, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecAeon1, SpecAeon1, Spec128)
STD_ROM_FN(SpecAeon1)

struct BurnDriver BurnSpecAeon1 = {
	"spec_aeon1", NULL, "spec_spec128", NULL, "2018",
	"Aeon 1 - Arrival at the Planet (128K) (HB)\0", "Game Part 1", "Sunteam", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecAeon1RomInfo, SpecAeon1RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Aeon 2 - Surveying for Crystals (128K) (HB)

static struct BurnRomInfo SpecAeon2RomDesc[] = {
	{ "Aeon 2 - Surveying for Crystals 128K (2018)(Sunteam).tap", 49584, 0x561d00ad, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecAeon2, SpecAeon2, Spec128)
STD_ROM_FN(SpecAeon2)

struct BurnDriver BurnSpecAeon2 = {
	"spec_aeon2", "spec_aeon1", "spec_spec128", NULL, "2018",
	"Aeon 2 - Surveying for Crystals (128K) (HB)\0", "Game Part 2", "Sunteam", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_ACTION, 0,
	SpectrumGetZipName, SpecAeon2RomInfo, SpecAeon2RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Aeon 3 - Communications (128K) (HB)

static struct BurnRomInfo SpecAeon3RomDesc[] = {
	{ "Aeon 3 - Communications 128K (2018)(Sunteam).tap", 49661, 0x883022fd, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecAeon3, SpecAeon3, Spec128)
STD_ROM_FN(SpecAeon3)

struct BurnDriver BurnSpecAeon3 = {
	"spec_aeon3", "spec_aeon1", "spec_spec128", NULL, "2018",
	"Aeon 3 - Communications (128K) (HB)\0", "Game Part 3", "Sunteam", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_ACTION, 0,
	SpectrumGetZipName, SpecAeon3RomInfo, SpecAeon3RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Aeon 4 - Escape from a Dead World (128K) (HB)

static struct BurnRomInfo SpecAeon4RomDesc[] = {
	{ "Aeon 4 - Escape from a Dead World 128K (2018)(Sunteam).tap", 51698, 0x305dd007, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecAeon4, SpecAeon4, Spec128)
STD_ROM_FN(SpecAeon4)

struct BurnDriver BurnSpecAeon4 = {
	"spec_aeon4", "spec_aeon1", "spec_spec128", NULL, "2018",
	"Aeon 4 - Escape from a Dead World (128K) (HB)\0", "Game Part 4", "Sunteam", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecAeon4RomInfo, SpecAeon4RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Aeon 5 - The End (HB)

static struct BurnRomInfo SpecAeon5RomDesc[] = {
	{ "Aeon 5 - The End (2018)(Sunteam).tap", 22913, 0xfd834c20, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecAeon5, SpecAeon5, Spec128)
STD_ROM_FN(SpecAeon5)

struct BurnDriver BurnSpecAeon5 = {
	"spec_aeon5", "spec_aeon1", "spec_spec128", NULL, "2018",
	"Aeon 5 - The End (HB)\0", "Ending screen", "Sunteam", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_MISC, 0,
	SpectrumGetZipName, SpecAeon5RomInfo, SpecAeon5RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Aerial (48K) (HB)

static struct BurnRomInfo SpecAerialRomDesc[] = {
	{ "Aerial 48K-AY (2021)(Inufuto).tap", 11486, 0x6a837856, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecAerial, SpecAerial, Spectrum)
STD_ROM_FN(SpecAerial)

struct BurnDriver BurnSpecAerial = {
	"spec_aerial", NULL, "spec_spectrum", NULL, "2021",
	"Aerial (48K) (HB)\0", "AY Sound supported", "Inufuto", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_HORSHOOT, 0,
	SpectrumGetZipName, SpecAerialRomInfo, SpecAerialRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Agatha Adventure (128K) (HB)

static struct BurnRomInfo SpecAgathadvRomDesc[] = {
	{ "Agatha Adventure 128K (2021)(Daniel Isoba).tap", 45114, 0x86776da5, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecAgathadv, SpecAgathadv, Spec128)
STD_ROM_FN(SpecAgathadv)

struct BurnDriver BurnSpecAgathadv = {
	"spec_agathadv", NULL, "spec_spec128", NULL, "2021",
	"Agatha Adventure (128K) (HB)\0", NULL, "Daniel Isoba", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecAgathadvRomInfo, SpecAgathadvRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Agatha Adventure (48K) (HB)

static struct BurnRomInfo SpecAgathadv48RomDesc[] = {
	{ "Agatha Adventure 48K (2021)(Daniel Isoba).tap", 43861, 0x5af05ceb, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecAgathadv48, SpecAgathadv48, Spectrum)
STD_ROM_FN(SpecAgathadv48)

struct BurnDriver BurnSpecAgathadv48 = {
	"spec_agathadv48", "spec_agathadv", "spec_spectrum", NULL, "2021",
	"Agatha Adventure (48K) (HB)\0", NULL, "Daniel Isoba", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecAgathadv48RomInfo, SpecAgathadv48RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Agatha Adventure 'hard-mode' (128K) (HB)

static struct BurnRomInfo SpecAgathadvhRomDesc[] = {
	{ "Agatha Adventure - hard-mode 128K (2021)(Daniel Isoba).tap", 44651, 0x56a5f727, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecAgathadvh, SpecAgathadvh, Spec128)
STD_ROM_FN(SpecAgathadvh)

struct BurnDriver BurnSpecAgathadvh = {
	"spec_agathadvh", "spec_agathadv", "spec_spec128", NULL, "2021",
	"Agatha Adventure 'hard-mode' (128K) (HB)\0", NULL, "Daniel Isoba", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecAgathadvhRomInfo, SpecAgathadvhRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Agent Blue - Daytime Missions (128K) (HB)

static struct BurnRomInfo SpecAgtbluedmRomDesc[] = {
	{ "Agent Blue - Daytime Missions 128K (2020)(Jaime Grilo).tap", 48248, 0xcb11c919, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecAgtbluedm, SpecAgtbluedm, Spec128)
STD_ROM_FN(SpecAgtbluedm)

struct BurnDriver BurnSpecAgtbluedm = {
	"spec_agtbluedm", NULL, "spec_spec128", NULL, "2020",
	"Agent Blue - Daytime Missions (128K) (HB)\0", NULL, "Jaime Grilo", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecAgtbluedmRomInfo, SpecAgtbluedmRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecQAOPSpaceDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Agent Blue - Night Missions (128K) (HB)

static struct BurnRomInfo SpecAgtbluenmRomDesc[] = {
	{ "Agent Blue - Night Missions 128K (2020)(Jaime Grilo).tap", 48005, 0x513a96d6, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecAgtbluenm, SpecAgtbluenm, Spec128)
STD_ROM_FN(SpecAgtbluenm)

struct BurnDriver BurnSpecAgtbluenm = {
	"spec_agtbluenm", "spec_agtbluedm", "spec_spec128", NULL, "2020",
	"Agent Blue - Night Missions (128K) (HB)\0", NULL, "Jaime Grilo", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecAgtbluenmRomInfo, SpecAgtbluenmRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecQAOPSpaceDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Akane (English) (128K) (HB)

static struct BurnRomInfo SpecAkaneRomDesc[] = {
	{ "Akane EN 128K (2021)(Hicks).tap", 46316, 0xa7a5dcf1, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecAkane, SpecAkane, Spec128)
STD_ROM_FN(SpecAkane)

struct BurnDriver BurnSpecAkane = {
	"spec_akane", NULL, "spec_spec128", NULL, "2021",
	"Akane (English) (128K) (HB)\0", NULL, "Hicks", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecAkaneRomInfo, SpecAkaneRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Akane (Spanish) (128K) (HB)

static struct BurnRomInfo SpecAkenespRomDesc[] = {
	{ "Akane ES 128K (2021)(Hicks).tap", 46353, 0x3138f6ad, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecAkenesp, SpecAkenesp, Spec128)
STD_ROM_FN(SpecAkenesp)

struct BurnDriver BurnSpecAkenesp = {
	"spec_akenesp", "spec_akane", "spec_spec128", NULL, "2021",
	"Akane (Spanish) (128K) (HB)\0", NULL, "Hicks", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecAkenespRomInfo, SpecAkenespRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Alien Astro Frenzy (48K) (HB)

static struct BurnRomInfo SpecAafRomDesc[] = {
	{ "Alien Astro Frenzy 48K (2021)(Vintage Software).z80", 27028, 0x4c32aff8, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecAaf, SpecAaf, Spectrum)
STD_ROM_FN(SpecAaf)

struct BurnDriver BurnSpecAaf = {
	"spec_aaf", NULL, "spec_spectrum", NULL, "2021",
	"Alien Astro Frenzy (48K) (HB)\0", NULL, "Vintage Software Systems", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_VERSHOOT, 0,
	SpectrumGetZipName, SpecAafRomInfo, SpecAafRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Alien Girl (128K) (HB)

static struct BurnRomInfo SpecAliengirl128RomDesc[] = {
	{ "Alien Girl 128K (2020)(Javier Fopiani).tap", 91068, 0xb99c427c, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecAliengirl128, SpecAliengirl128, Spec128)
STD_ROM_FN(SpecAliengirl128)

struct BurnDriver BurnSpecAliengirl128 = {
	"spec_aliengirl128", NULL, "spec_spec128", NULL, "2020",
	"Alien Girl (128K) (HB)\0", NULL, "Javier Fopiani", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_RUNGUN | GBF_MAZE, 0,
	SpectrumGetZipName, SpecAliengirl128RomInfo, SpecAliengirl128RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Alien Girl - Skirmish Edition (128K) (HB)

static struct BurnRomInfo SpecAliengirl128seRomDesc[] = {
	{ "Alien Girl - Skirmish Edition 128K (2021)(Javier Fopiani).tap", 78046, 0x3c3afed1, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecAliengirl128se, SpecAliengirl128se, Spec128)
STD_ROM_FN(SpecAliengirl128se)

struct BurnDriver BurnSpecAliengirl128se = {
	"spec_aliengirl128se", "spec_aliengirl128", "spec_spec128", NULL, "2021",
	"Alien Girl - Skirmish Edition (128K) (HB)\0", NULL, "Javier Fopiani", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_MAZE | GBF_RUNGUN, 0,
	SpectrumGetZipName, SpecAliengirl128seRomInfo, SpecAliengirl128seRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Alien Girl (English) (48K) (HB)

static struct BurnRomInfo SpecAliengirl48enRomDesc[] = {
	{ "Alien Girl 48K EN (2020)(Javier Fopiani).tap", 44475, 0x4e1c2f21, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecAliengirl48en, SpecAliengirl48en, Spectrum)
STD_ROM_FN(SpecAliengirl48en)

struct BurnDriver BurnSpecAliengirl48en = {
	"spec_aliengirl48en", "spec_aliengirl128", "spec_spectrum", NULL, "2020",
	"Alien Girl (English) (48K) (HB)\0", NULL, "Javier Fopiani", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_RUNGUN | GBF_MAZE, 0,
	SpectrumGetZipName, SpecAliengirl48enRomInfo, SpecAliengirl48enRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Alien Girl (Spanish) (48K) (HB)

static struct BurnRomInfo SpecAliengirl48esRomDesc[] = {
	{ "Alien Girl 48K ES (2020)(Javier Fopiani).tap", 44505, 0x1f883dba, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecAliengirl48es, SpecAliengirl48es, Spectrum)
STD_ROM_FN(SpecAliengirl48es)

struct BurnDriver BurnSpecAliengirl48es = {
	"spec_aliengirl48es", "spec_aliengirl128", "spec_spectrum", NULL, "2020",
	"Alien Girl (Spanish) (48K) (HB)\0", NULL, "Javier Fopiani", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_RUNGUN | GBF_MAZE, 0,
	SpectrumGetZipName, SpecAliengirl48esRomInfo, SpecAliengirl48esRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Aliens Neoplasma (English) (128K) (HB)

static struct BurnRomInfo SpecAliensneoRomDesc[] = {
	{ "Aliens-Neoplasma EN 128K (2019)(SaNchez).tap", 64233, 0x6cf1a406, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecAliensneo, SpecAliensneo, Spec128)
STD_ROM_FN(SpecAliensneo)

struct BurnDriver BurnSpecAliensneo = {
	"spec_aliensneo", NULL, "spec_spec128", NULL, "2019",
	"Aliens Neoplasma (English) (128K) (HB)\0", NULL, "SaNchez", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_RUNGUN | GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecAliensneoRomInfo, SpecAliensneoRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Aliens Neoplasma (Spanish) (128K) (HB)

static struct BurnRomInfo SpecAliensneoesRomDesc[] = {
	{ "Aliens-Neoplasma ES 128K (2019)(SaNchez).tap", 64212, 0x58a0d65f, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecAliensneoes, SpecAliensneoes, Spec128)
STD_ROM_FN(SpecAliensneoes)

struct BurnDriver BurnSpecAliensneoes = {
	"spec_aliensneoes", "spec_aliensneo", "spec_spec128", NULL, "2019",
	"Aliens Neoplasma (Spanish) (128K) (HB)\0", NULL, "SaNchez", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_RUNGUN | GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecAliensneoesRomInfo, SpecAliensneoesRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Aliens Neoplasma (Russian) (128K) (HB)

static struct BurnRomInfo SpecAliensneoruRomDesc[] = {
	{ "Aliens-Neoplasma RU 128K (2019)(SaNchez).tap", 64460, 0x84175b8b, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecAliensneoru, SpecAliensneoru, Spec128)
STD_ROM_FN(SpecAliensneoru)

struct BurnDriver BurnSpecAliensneoru = {
	"spec_aliensneoru", "spec_aliensneo", "spec_spec128", NULL, "2019",
	"Aliens Neoplasma (Russian) (128K) (HB)\0", NULL, "SaNchez", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_RUNGUN | GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecAliensneoruRomInfo, SpecAliensneoruRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Aliens Neoplasma II (English) (128K) (HB, v1.6)

static struct BurnRomInfo SpecAliensneo2enRomDesc[] = {
	{ "Aliens Neoplasma II v1.6 EN 128K (2024)(Sinc LAIR).tap", 157127, 0xcbf57237, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecAliensneo2en, SpecAliensneo2en, Spec128)
STD_ROM_FN(SpecAliensneo2en)

struct BurnDriver BurnSpecAliensneo2en = {
	"spec_aliensneo2en", NULL, "spec_spec128", NULL, "2024",
	"Aliens Neoplasma II (English) (128K) (HB, v1.6)\0", "Default control keys: W, A, S, D, B, N, M, P", "Sinc LAIR", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_RUNGUN | GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecAliensneo2enRomInfo, SpecAliensneo2enRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Aliens Neoplasma II (Spanish) (128K) (HB, v1.6)

static struct BurnRomInfo SpecAliensneo2esRomDesc[] = {
	{ "Aliens Neoplasma II v1.6 ES 128K (2024)(Sinc LAIR).tap", 157750, 0xe7acbc67, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecAliensneo2es, SpecAliensneo2es, Spec128)
STD_ROM_FN(SpecAliensneo2es)

struct BurnDriver BurnSpecAliensneo2es = {
	"spec_aliensneo2es", "spec_aliensneo2en", "spec_spec128", NULL, "2024",
	"Aliens Neoplasma II (Spanish) (128K) (HB, v1.6)\0", "Default control keys: W, A, S, D, B, N, M, P", "Sinc LAIR", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_RUNGUN | GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecAliensneo2esRomInfo, SpecAliensneo2esRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Aliens Neoplasma II (Portuguese) (128K) (HB, v1.6)

static struct BurnRomInfo SpecAliensneo2ptRomDesc[] = {
	{ "Aliens Neoplasma II v1.6 PT 128K (2024)(Sinc LAIR).tap", 157563, 0xc55ccc4b, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecAliensneo2pt, SpecAliensneo2pt, Spec128)
STD_ROM_FN(SpecAliensneo2pt)

struct BurnDriver BurnSpecAliensneo2pt = {
	"spec_aliensneo2pt", "spec_aliensneo2en", "spec_spec128", NULL, "2024",
	"Aliens Neoplasma II (Portuguese) (128K) (HB, v1.6)\0", "Default control keys: W, A, S, D, B, N, M, P", "Sinc LAIR", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_RUNGUN | GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecAliensneo2ptRomInfo, SpecAliensneo2ptRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Aliens Neoplasma II (Russian) (128K) (HB, v1.6)

static struct BurnRomInfo SpecAliensneo2ruRomDesc[] = {
	{ "Aliens Neoplasma II v1.6 RU 128K (2024)(Sinc LAIR).tap", 157969, 0xabc3b739, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecAliensneo2ru, SpecAliensneo2ru, Spec128)
STD_ROM_FN(SpecAliensneo2ru)

struct BurnDriver BurnSpecAliensneo2ru = {
	"spec_aliensneo2ru", "spec_aliensneo2en", "spec_spec128", NULL, "2024",
	"Aliens Neoplasma II (Russian) (128K) (HB, v1.6)\0", "Default control keys: W, A, S, D, B, N, M, P", "Sinc LAIR", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_RUNGUN | GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecAliensneo2ruRomInfo, SpecAliensneo2ruRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Alien Slaphead (128K) (HB, v2)

static struct BurnRomInfo SpecAlienshRomDesc[] = {
	{ "Alien Slaphead v2 128K (2022)(Joesoft).tap", 45449, 0x3194ad0d, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecAliensh, SpecAliensh, Spec128)
STD_ROM_FN(SpecAliensh)

struct BurnDriver BurnSpecAliensh = {
	"spec_aliensh", NULL, "spec_spec128", NULL, "2022",
	"Alien Slaphead (128K) (HB, v2)\0", NULL, "Joesoft", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PLATFORM | GBF_QUIZ, 0,
	SpectrumGetZipName, SpecAlienshRomInfo, SpecAlienshRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// All Hallows Rise of Pumpkin (128K) (HB)

static struct BurnRomInfo SpechlwpumpkinRomDesc[] = {
	{ "All Hallows Rise of Pumpkin 128K (2018)(Rucksack Games).tap", 46999, 0x76ecb3a0, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Spechlwpumpkin, Spechlwpumpkin, Spec128)
STD_ROM_FN(Spechlwpumpkin)

struct BurnDriver BurnSpechlwpumpkin = {
	"spec_hlwpumpkin", NULL, "spec_spec128", NULL, "2018",
	"All Hallows Rise of Pumpkin (128K) (HB)\0", NULL, "Rucksack Games", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpechlwpumpkinRomInfo, SpechlwpumpkinRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// All Present and Correct (48K) (HB)

static struct BurnRomInfo SpecAllpresentRomDesc[] = {
	{ "All Present and Correct 48K (2009)(Bob Smith).tap", 47984, 0x244d67d6, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecAllpresent, SpecAllpresent, Spectrum)
STD_ROM_FN(SpecAllpresent)

struct BurnDriver BurnSpecAllpresent = {
	"spec_allpresent", NULL, "spec_spectrum", NULL, "2009",
	"All Present and Correct (48K) (HB)\0", NULL, "Bob Smith", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PUZZLE, 0,
	SpectrumGetZipName, SpecAllpresentRomInfo, SpecAllpresentRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Alter Ego (48K) (HB)

static struct BurnRomInfo SpecalteregoRomDesc[] = {
	{ "Alter Ego 48K (2011)(RetroSouls).tap", 38926, 0x855c13dc, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specalterego, Specalterego, Spectrum)
STD_ROM_FN(Specalterego)

struct BurnDriver BurnSpecalterego = {
	"spec_alterego", NULL, "spec_spectrum", NULL, "2011",
	"Alter Ego (48K) (HB)\0", NULL, "RetroSouls", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PUZZLE, 0,
	SpectrumGetZipName, SpecalteregoRomInfo, SpecalteregoRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecIntf2DIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Amazing Rocketeer, The (48K) (HB)

static struct BurnRomInfo SpecAmazRockRomDesc[] = {
	{ "Amazing Rocketeer, The 48K (2007)(John Fotou).tap", 91791, 0x246b37d3, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecAmazRock, SpecAmazRock, Spectrum)
STD_ROM_FN(SpecAmazRock)

struct BurnDriver BurnSpecAmazRock = {
	"spec_amazrock", NULL, "spec_spectrum", NULL, "2007",
	"Amazing Rocketeer, The (48K) (HB)\0", "Y: fire -  X: exit Game - ,[comma]: next level - CTRL: map", "John Fotou", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecAmazRockRomInfo, SpecAmazRockRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Amidar (Spanish) (48K) (HB)

static struct BurnRomInfo SpecAmidarRomDesc[] = {
	{ "Amidar 48K (2022)(Salvador Camacho).tap", 26664, 0x7dda9ac5, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecAmidar, SpecAmidar, Spectrum)
STD_ROM_FN(SpecAmidar)

struct BurnDriver BurnSpecAmidar = {
	"spec_amidar", NULL, "spec_spectrum", NULL, "2022",
	"Amidar (Spanish) (48K) (HB)\0", NULL, "Salvador Camacho", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_ACTION, 0,
	SpectrumGetZipName, SpecAmidarRomInfo, SpecAmidarRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecQAOPMDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Angelita - Battle Mocita (48K) (HB)

static struct BurnRomInfo SpecAngelitaRomDesc[] = {
	{ "Angelita - Battle Mocita 48K (2020)(Nekokapi).tap", 38495, 0x5ccd938b, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecAngelita, SpecAngelita, Spectrum)
STD_ROM_FN(SpecAngelita)

struct BurnDriver BurnSpecAngelita = {
	"spec_angelita", NULL, "spec_spectrum", NULL, "2020",
	"Angelita - Battle Mocita (48K) (HB)\0", NULL, "Nekokapi", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecAngelitaRomInfo, SpecAngelitaRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Angels (128K) (HB)

static struct BurnRomInfo SpecAngelsRomDesc[] = {
	{ "Angels 128K (2021)(Zosya Ent.).tap", 85647, 0x6e928666, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecAngels, SpecAngels, Spec128)
STD_ROM_FN(SpecAngels)

struct BurnDriver BurnSpecAngels = {
	"spec_angels", NULL, "spec_spec128", NULL, "2021",
	"Angels (128K) (HB)\0", NULL, "Zosya Entertainment", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 2, HARDWARE_SPECTRUM, GBF_SCRFIGHT | GBF_VERSHOOT, 0,
	SpectrumGetZipName, SpecAngelsRomInfo, SpecAngelsRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Angry Birds - Opposition (48K) (HB)

static struct BurnRomInfo SpecAngrybirdsvsRomDesc[] = {
	{ "Angry Birds - Opposition (2016)(kas29).tap", 40061, 0x6f8373d9, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecAngrybirdsvs, SpecAngrybirdsvs, Spectrum)
STD_ROM_FN(SpecAngrybirdsvs)

struct BurnDriver BurnSpecAngrybirdsvs = {
	"spec_angrybirdsvs", NULL, "spec_spectrum", NULL, "2016",
	"Angry Birds - Opposition (48K) (HB)\0", NULL, "kas29", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_SHOOT, 0,
	SpectrumGetZipName, SpecAngrybirdsvsRomInfo, SpecAngrybirdsvsRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// AntiAir (48K) (HB)

static struct BurnRomInfo SpecAntiairRomDesc[] = {
	{ "AntiAir 48K-AY (2024)(Inufuto).tap", 7569, 0x548e4b80, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecAntiair, SpecAntiair, Spectrum)
STD_ROM_FN(SpecAntiair)

struct BurnDriver BurnSpecAntiair = {
	"spec_antiair", NULL, "spec_spectrum", NULL, "2024",
	"AntiAir (48K) (HB)\0", "AY Sound supported", "Inufuto", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_SHOOT, 0,
	SpectrumGetZipName, SpecAntiairRomInfo, SpecAntiairRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Antiquity Jones (48K) (HB)

static struct BurnRomInfo SpecAntyjonesRomDesc[] = {
	{ "Antiquity Jones 48K (2012)(Paul Jenkinson).tap", 37612, 0x74f2526d, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecAntyjones, SpecAntyjones, Spectrum)
STD_ROM_FN(SpecAntyjones)

struct BurnDriver BurnSpecAntyjones = {
	"spec_antyjones", NULL, "spec_spectrum", NULL, "2012",
	"Antiquity Jones (48K) (HB)\0", NULL, "Paul Jenkinson", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecAntyjonesRomInfo, SpecAntyjonesRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecIntf2DIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Apulija-13 (128K) (HB, v2.01)

static struct BurnRomInfo SpecApulija13RomDesc[] = {
	{ "Apulija-13 (2013)(Alessandro Grussu).tap", 28214, 0x356db043, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecApulija13, SpecApulija13, Spec128)
STD_ROM_FN(SpecApulija13)

struct BurnDriver BurnSpecApulija13 = {
	"spec_apulija13", NULL, "spec_spec128", NULL, "2018",
	"Apulija-13 (128K) (HB, v2.01)\0", NULL, "Alessandro Grussu", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_RUNGUN | GBF_ADV, 0,
	SpectrumGetZipName, SpecApulija13RomInfo, SpecApulija13RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Aquanoids (48K) (HB)

static struct BurnRomInfo SpecAquanoidsRomDesc[] = {
	{ "Aquanoids (2015)(Neil Parsons).tap", 38587, 0x6ead57d1, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecAquanoids, SpecAquanoids, Spectrum)
STD_ROM_FN(SpecAquanoids)

struct BurnDriver BurnSpecAquanoids = {
	"spec_aquanoids", NULL, "spec_spectrum", NULL, "2015",
	"Aquanoids (48K) (HB)\0", NULL, "Neil Parsons", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecAquanoidsRomInfo, SpecAquanoidsRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Area 51 - Roswell Incident (English) (48K) (HB)

static struct BurnRomInfo SpecArea51rieRomDesc[] = {
	{ "Area 51 - Roswell Incident EN 48K (2023)(Furillo Productions).tap", 39669, 0x2f51d9b7, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecArea51rie, SpecArea51rie, Spectrum)
STD_ROM_FN(SpecArea51rie)

struct BurnDriver BurnSpecArea51rie = {
	"spec_area51rie", NULL, "spec_spectrum", NULL, "2023",
	"Area 51 - Roswell Incident (English) (48K) (HB)\0", NULL, "Furillo Productions", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_ADV, 0,
	SpectrumGetZipName, SpecArea51rieRomInfo, SpecArea51rieRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecIntf2DIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Area 51 - Roswell Incident (Spanish) (48K) (HB)

static struct BurnRomInfo SpecArea51risRomDesc[] = {
	{ "Area 51 - Roswell Incident ES 48K (2023)(Furillo Productions).tap", 39708, 0x234c6cae, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecArea51ris, SpecArea51ris, Spectrum)
STD_ROM_FN(SpecArea51ris)

struct BurnDriver BurnSpecArea51ris = {
	"spec_area51ris", "spec_area51rie", "spec_spectrum", NULL, "2023",
	"Area 51 - Roswell Incident (Spanish) (48K) (HB)\0", NULL, "Furillo Productions", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_ADV, 0,
	SpectrumGetZipName, SpecArea51risRomInfo, SpecArea51risRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecIntf2DIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Area 51 - Roswell Incident (English) (128K) (HB)

static struct BurnRomInfo SpecArea51rie128RomDesc[] = {
	{ "Area 51 - Roswell Incident EN 128K (2024)(Furillo Productions).tap", 53546, 0xf0f39715, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecArea51rie128, SpecArea51rie128, Spec128)
STD_ROM_FN(SpecArea51rie128)

struct BurnDriver BurnSpecArea51rie128 = {
	"spec_area51rie128", NULL, "spec_spec128", NULL, "2024",
	"Area 51 - Roswell Incident (English) (128K) (HB)\0", NULL, "Furillo Productions", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_ADV, 0,
	SpectrumGetZipName, SpecArea51rie128RomInfo, SpecArea51rie128RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Area 51 - Roswell Incident (Spanish) (128K) (HB)

static struct BurnRomInfo SpecArea51ris128RomDesc[] = {
	{ "Area 51 - Roswell Incident ES 128K (2024)(Furillo Productions).tap", 55556, 0xc0ef665b, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecArea51ris128, SpecArea51ris128, Spec128)
STD_ROM_FN(SpecArea51ris128)

struct BurnDriver BurnSpecArea51ris128 = {
	"spec_area51ris128", "spec_area51rie128", "spec_spec128", NULL, "2024",
	"Area 51 - Roswell Incident (Spanish) (128K) (HB)\0", NULL, "Furillo Productions", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_ADV, 0,
	SpectrumGetZipName, SpecArea51ris128RomInfo, SpecArea51ris128RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Arlo (English) (128K) (HB)

static struct BurnRomInfo SpecArloeRomDesc[] = {
	{ "Arlo EN 128K (2024)(Packobilly).tap", 49621, 0xfff44368, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecArloe, SpecArloe, Spec128)
STD_ROM_FN(SpecArloe)

struct BurnDriver BurnSpecArloe = {
	"spec_arloe", NULL, "spec_spec128", NULL, "2024",
	"Arlo (English) (128K) (HB)\0", NULL, "Packobilly", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecArloeRomInfo, SpecArloeRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Arlo (Spanish) (128K) (HB)

static struct BurnRomInfo SpecArlosRomDesc[] = {
	{ "Arlo ES 128K (2024)(Packobilly).tap", 49620, 0x6b6de6c5, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecArlos, SpecArlos, Spec128)
STD_ROM_FN(SpecArlos)

struct BurnDriver BurnSpecArlos = {
	"spec_arlos", "spec_arloe", "spec_spec128", NULL, "2024",
	"Arlo (Spanish) (128K) (HB)\0", NULL, "Packobilly", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecArlosRomInfo, SpecArlosRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Art of HelpComputer0, The (128K) (HB, Tech-Demo)

static struct BurnRomInfo SpecArthc0RomDesc[] = {
	{ "Art of HelpComputer0, The 128K (2023)(Ate Bit).tap", 79925, 0xb2a94379, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecArthc0, SpecArthc0, Spec128)
STD_ROM_FN(SpecArthc0)

struct BurnDriver BurnSpecArthc0 = {
	"spec_arthc0", NULL, "spec_spec128", NULL, "2023",
	"Art of HelpComputer0, The (128K) (HB, Tech-Demo)\0", NULL, "Ate Bit", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW | BDF_DEMO, 1, HARDWARE_SPECTRUM, GBF_MISC, 0,
	SpectrumGetZipName, SpecArthc0RomInfo, SpecArthc0RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Ascend (48K) (HB)

static struct BurnRomInfo SpecAscendRomDesc[] = {
	{ "Ascend (2022)(Inufuto).tap", 9277, 0x37e43835, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecAscend, SpecAscend, Spectrum)
STD_ROM_FN(SpecAscend)

struct BurnDriver BurnSpecAscend = {
	"spec_ascend", NULL, "spec_spectrum", NULL, "2022",
	"Ascend (48K) (HB)\0", "AY Sound supported", "Inufuto", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_ACTION, 0,
	SpectrumGetZipName, SpecAscendRomInfo, SpecAscendRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Asteroids RX (128K) (HB)

static struct BurnRomInfo SpecAsteroidsrxRomDesc[] = {
	{ "Asteroids RX (2021)(highriser).tap", 43917, 0xe94e7cea, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecAsteroidsrx, SpecAsteroidsrx, Spec128)
STD_ROM_FN(SpecAsteroidsrx)

struct BurnDriver BurnSpecAsteroidsrx = {
	"spec_asteroidsrx", NULL, "spec_spec128", NULL, "2021",
	"Asteroids RX (128K) (HB)\0", NULL, "highriser", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_SHOOT, 0,
	SpectrumGetZipName, SpecAsteroidsrxRomInfo, SpecAsteroidsrxRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecQAOPSpaceDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Astronaut Labyrinth (128K) (HB)

static struct BurnRomInfo SpecAstrolabyRomDesc[] = {
	{ "Astronaut Labyrinth (2018)(Jaime Grilo).tap", 44602, 0x984c2c8a, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecAstrolaby, SpecAstrolaby, Spec128)
STD_ROM_FN(SpecAstrolaby)

struct BurnDriver BurnSpecAstrolaby = {
	"spec_astrolaby", NULL, "spec_spec128", NULL, "2018",
	"Astronaut Labyrinth (128K) (HB)\0", NULL, "Jaime Grilo", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecAstrolabyRomInfo, SpecAstrolabyRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Astrosmash! ZX (48K) (HB)

static struct BurnRomInfo SpecAstrosmashRomDesc[] = {
	{ "Astrosmash! ZX (2018)(AMCgames).tap", 23087, 0xb3fc0c89, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecAstrosmash, SpecAstrosmash, Spectrum)
STD_ROM_FN(SpecAstrosmash)

struct BurnDriver BurnSpecAstrosmash = {
	"spec_astrosmash", NULL, "spec_spectrum", NULL, "2018",
	"Astrosmash! ZX (48K) (HB)\0", "Converted from Intellivision version", "AMCgames", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_VERSHOOT, 0,
	SpectrumGetZipName, SpecAstrosmashRomInfo, SpecAstrosmashRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Atlantis ZX (48K) (HB)

static struct BurnRomInfo SpecAtlantiszxRomDesc[] = {
	{ "Atlantis ZX 48K (2019)(Luca Bordoni).tap", 24799, 0x9f7ef414, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecAtlantiszx, SpecAtlantiszx, Spectrum)
STD_ROM_FN(SpecAtlantiszx)

struct BurnDriver BurnSpecAtlantiszx = {
	"spec_atlantiszx", NULL, "spec_spectrum", NULL, "2019",
	"Atlantis ZX (48K) (HB)\0", NULL, "Luca Bordoni", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_SHOOT, 0,
	SpectrumGetZipName, SpecAtlantiszxRomInfo, SpecAtlantiszxRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Attributica (128K) (HB, Tech-Demo)

static struct BurnRomInfo SpecAttributicaRomDesc[] = {
	{ "Attributica 128K (2022)(Speccy.pl).tap", 214271, 0xcc5eee16, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecAttributica, SpecAttributica, Spec128)
STD_ROM_FN(SpecAttributica)

struct BurnDriver BurnSpecAttributica = {
	"spec_attributica", NULL, "spec_spec128", NULL, "2022",
	"Attributica (128K) (HB, Tech-Demo)\0", NULL, "Speccy.pl", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW | BDF_DEMO, 1, HARDWARE_SPECTRUM, GBF_MISC, 0,
	SpectrumGetZipName, SpecAttributicaRomInfo, SpecAttributicaRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Automated Cave Explorer (48K)

static struct BurnRomInfo SpecAutocavexplRomDesc[] = {
	{ "Automated Cave Explorer 48K (2019)(Borisov, Alexei).tap", 37827, 0xdaaa298f, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecAutocavexpl, SpecAutocavexpl, Spectrum)
STD_ROM_FN(SpecAutocavexpl)

struct BurnDriver BurnSpecAutocavexpl = {
	"spec_autocavexpl", NULL, "spec_spectrum", NULL, "2019",
	"Automated Cave Explorer (48K) (HB)\0", NULL, "Alexei Borisov", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_ACTION, 0,
	SpectrumGetZipName, SpecAutocavexplRomInfo, SpecAutocavexplRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Aztec (48K) (HB, pre-release)

static struct BurnRomInfo SpecAztecRomDesc[] = {
	{ "Aztec 48K (2021)(Rui Martins).tap", 20586, 0x143faf4b, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecAztec, SpecAztec, Spectrum)
STD_ROM_FN(SpecAztec)

struct BurnDriver BurnSpecAztec = {
	"spec_aztec", NULL, "spec_spectrum", NULL, "2021",
	"Aztec (48K) (HB, pre-release)\0", NULL, "Rui Martins", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PUZZLE, 0,
	SpectrumGetZipName, SpecAztecRomInfo, SpecAztecRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecQAOPSpaceDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Baby-man Vs Man-baby (48K) (HB)

static struct BurnRomInfo SpecBabymanRomDesc[] = {
	{ "Baby-man Vs Man-baby (2023)(PuttyCAD, Tin Soldiers).tap", 41891, 0x7562acb5, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecBabyman, SpecBabyman, Spectrum)
STD_ROM_FN(SpecBabyman)

struct BurnDriver BurnSpecBabyman = {
	"spec_babyman", NULL, "spec_spectrum", NULL, "2023",
	"Baby-man Vs Man-baby (48K) (HB)\0", NULL, "PuttyCAD, Tin Soldiers", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_SHOOT, 0,
	SpectrumGetZipName, SpecBabymanRomInfo, SpecBabymanRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Baby-man Vs Nappy Bird (48K) (HB, v1.01)

static struct BurnRomInfo SpecBabyman2RomDesc[] = {
	{ "Baby-man Vs Nappy Bird 48K v1.01 (2023)(PuttyCAD, Tin Soldiers).tap", 23108, 0x70cf01bf, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecBabyman2, SpecBabyman2, Spectrum)
STD_ROM_FN(SpecBabyman2)

struct BurnDriver BurnSpecBabyman2 = {
	"spec_babyman2", NULL, "spec_spectrum", NULL, "2023",
	"Baby-man Vs Nappy Bird (48K) (HB, v1.01)\0", NULL, "PuttyCAD, Tin Soldiers", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_ACTION, 0,
	SpectrumGetZipName, SpecBabyman2RomInfo, SpecBabyman2RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Baffo Jones (48K) (HB)

static struct BurnRomInfo SpecBaffojonesRomDesc[] = {
	{ "Baffo Jones 48K (2015)(Gabriele Amore).tap", 34142, 0x2edb29b5, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecBaffojones, SpecBaffojones, Spectrum)
STD_ROM_FN(SpecBaffojones)

struct BurnDriver BurnSpecBaffojones = {
	"spec_baffojones", NULL, "spec_spectrum", NULL, "2015",
	"Baffo Jones (48K) (HB)\0", NULL, "Gabriele Amore", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecBaffojonesRomInfo, SpecBaffojonesRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Banger Management (128K) (HB)

static struct BurnRomInfo SpecBangermanRomDesc[] = {
	{ "Banger Management 128K (2023)(Jonathan Cauldwell).tap", 32373, 0xd9e1049e, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecBangerman, SpecBangerman, Spec128)
STD_ROM_FN(SpecBangerman)

struct BurnDriver BurnSpecBangerman = {
	"spec_bangerman", NULL, "spec_spec128", NULL, "2023",
	"Banger Management (128K) (HB)\0", NULL, "Jonathan Cauldwell", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_ACTION | GBF_PUZZLE, 0,
	SpectrumGetZipName, SpecBangermanRomInfo, SpecBangermanRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Bark, The (48K) (HB)

static struct BurnRomInfo SpecBarkRomDesc[] = {
	{ "Bark, The 48K (2023)(Zosya Entertainment).tap", 35166, 0x075f4b7b, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecBark, SpecBark, Spectrum)
STD_ROM_FN(SpecBark)

struct BurnDriver BurnSpecBark = {
	"spec_bark", NULL, "spec_spectrum", NULL, "2023",
	"Bark, The (48K) (HB)\0", NULL, "Zosya Entertainment", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_MAZE | GBF_RUNGUN, 0,
	SpectrumGetZipName, SpecBarkRomInfo, SpecBarkRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Basilisko de Roco, El (128K) (HB)

static struct BurnRomInfo SpecBasiroco128RomDesc[] = {
	{ "Basilisko de Roco, El 128K (2024)(Retrosotano).tap", 53569, 0x1c3b947e, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecBasiroco128, SpecBasiroco128, Spec128)
STD_ROM_FN(SpecBasiroco128)

struct BurnDriver BurnSpecBasiroco128 = {
	"spec_basiroco128", NULL, "spec_spec128", NULL, "2024",
	"Basilisko de Roco, El (128K) (HB)\0", NULL, "Retrosotano", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecBasiroco128RomInfo, SpecBasiroco128RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Basilisko de Roco, El (48K) (HB)

static struct BurnRomInfo SpecBasiroco48RomDesc[] = {
	{ "Basilisko de Roco, El 48K (2024)(Retrosotano).tap", 53126, 0x99baf2c2, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecBasiroco48, SpecBasiroco48, Spectrum)
STD_ROM_FN(SpecBasiroco48)

struct BurnDriver BurnSpecBasiroco48 = {
	"spec_basiroco48", "spec_basiroco128", "spec_spectrum", NULL, "2024",
	"Basilisko de Roco, El (48K) (HB)\0", NULL, "Retrosotano", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecBasiroco48RomInfo, SpecBasiroco48RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Basilisko de Roco 2, El (128K) (HB)

static struct BurnRomInfo SpecBasilisk2128RomDesc[] = {
	{ "Basilisko de Roco 2, El 128K (2024)(Retrosotano).tap", 53844, 0xb8695d6b, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecBasilisk2128, SpecBasilisk2128, Spec128)
STD_ROM_FN(SpecBasilisk2128)

struct BurnDriver BurnSpecBasilisk2128 = {
	"spec_basilisk2128", NULL, "spec_spec128", NULL, "2024",
	"Basilisko de Roco 2, El (128K) (HB)\0", NULL, "Retrosotano", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecBasilisk2128RomInfo, SpecBasilisk2128RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Basilisko de Roco 2, El (48K) (HB)

static struct BurnRomInfo SpecBasilisk248RomDesc[] = {
	{ "Basilisko de Roco 2, El 48K (2024)(Retrosotano).tap", 53371, 0x137cc20d, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecBasilisk248, SpecBasilisk248, Spectrum)
STD_ROM_FN(SpecBasilisk248)

struct BurnDriver BurnSpecBasilisk248 = {
	"spec_basilisk248", "spec_basilisk2128", "spec_spectrum", NULL, "2024",
	"Basilisko de Roco 2, El (48K) (HB)\0", NULL, "Retrosotano", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecBasilisk248RomInfo, SpecBasilisk248RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Bastet (48K) (HB)

static struct BurnRomInfo SpecBastetRomDesc[] = {
	{ "Bastet 48K-AY (2024)(ZXdeveloper).tap", 44066, 0xb7debd82, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecBastet, SpecBastet, Spectrum)
STD_ROM_FN(SpecBastet)

struct BurnDriver BurnSpecBastet = {
	"spec_bastet", NULL, "spec_spectrum", NULL, "2024",
	"Bastet (48K) (HB)\0", "AY Sound supported", "ZXdeveloper", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecBastetRomInfo, SpecBastetRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// BatManolo (English) (48K) (HB, v1.1)

static struct BurnRomInfo SpecBatmanoloeRomDesc[] = {
	{ "BatManolo EN 48K v1.1 (2023)(Nahell Tresal).tap", 68188, 0x474350a7, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecBatmanoloe, SpecBatmanoloe, Spectrum)
STD_ROM_FN(SpecBatmanoloe)

struct BurnDriver BurnSpecBatmanoloe = {
	"spec_batmanoloe", NULL, "spec_spectrum", NULL, "2023",
	"BatManolo (English) (48K) (HB, v1.1)\0", NULL, "Nahell Tresal", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecBatmanoloeRomInfo, SpecBatmanoloeRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecQAOPSpaceDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// BatManolo (Spanish) (48K) (HB, v1.1)

static struct BurnRomInfo SpecBatmanolosRomDesc[] = {
	{ "BatManolo ES v1.1 48K (2023)(Nahell Tresal).tap", 68854, 0x3f92d604, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecBatmanolos, SpecBatmanolos, Spectrum)
STD_ROM_FN(SpecBatmanolos)

struct BurnDriver BurnSpecBatmanolos = {
	"spec_batmanolos", "spec_batmanoloe", "spec_spectrum", NULL, "2023",
	"BatManolo (Spanish) (48K) (HB, v1.1)\0", NULL, "Nahell Tresal", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecBatmanolosRomInfo, SpecBatmanolosRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecQAOPSpaceDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Battle City (48K) (HB)

static struct BurnRomInfo SpecBattlecityRomDesc[] = {
	{ "Battle City 48K (2021)(menyiques).tap", 36024, 0xf3b29343, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecBattlecity, SpecBattlecity, Spectrum)
STD_ROM_FN(SpecBattlecity)

struct BurnDriver BurnSpecBattlecity = {
	"spec_battlecity", NULL, "spec_spectrum", NULL, "2021",
	"Battle City (48K) (HB)\0", NULL, "Menyiques Soft", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_ACTION | GBF_SHOOT, 0,
	SpectrumGetZipName, SpecBattlecityRomInfo, SpecBattlecityRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecQAOPSpaceDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Battle Grid (48K) (HB)

static struct BurnRomInfo SpecBattlegridRomDesc[] = {
	{ "Battle Grid 48K (2022)(El Dr. Gusman).tap", 47062, 0xd9b40afa, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecBattlegrid, SpecBattlegrid, Spectrum)
STD_ROM_FN(SpecBattlegrid)

struct BurnDriver BurnSpecBattlegrid = {
	"spec_battlegrid", NULL, "spec_spectrum", NULL, "2022",
	"Battle Grid (48K) (HB)\0", "Press 'I' to show the instructions", "El Dr. Gusman", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_STRATEGY, 0,
	SpectrumGetZipName, SpecBattlegridRomInfo, SpecBattlegridRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Battle Tank 3D and some Crazy Aliens (128K) (HB)

static struct BurnRomInfo SpecBtltank3dRomDesc[] = {
	{ "Battle Tank 3D and some Crazy Aliens 128K (2014)(Gabriele Amore).tap", 20678, 0x69d1cf0a, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecBtltank3d, SpecBtltank3d, Spec128)
STD_ROM_FN(SpecBtltank3d)

struct BurnDriver BurnSpecBtltank3d = {
	"spec_btltank3d", NULL, "spec_spec128", NULL, "2014",
	"Battle Tank 3D and some Crazy Aliens (128K) (HB)\0", NULL, "Gabriele Amore", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_SHOOT, 0,
	SpectrumGetZipName, SpecBtltank3dRomInfo, SpecBtltank3dRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Beams of Light (48K) (HB)

static struct BurnRomInfo SpecBeamslightRomDesc[] = {
	{ "Beams of Light 48K (2017)(Shiny).tap", 19916, 0x866477f9, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecBeamslight, SpecBeamslight, Spectrum)
STD_ROM_FN(SpecBeamslight)

struct BurnDriver BurnSpecBeamslight = {
	"spec_beamslight", NULL, "spec_spectrum", NULL, "2017",
	"Beams of Light (48K) (HB)\0", "AY Sound supported", "Shiny", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PUZZLE, 0,
	SpectrumGetZipName, SpecBeamslightRomInfo, SpecBeamslightRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecCursorKeysDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Beastie Feastie 128K2a

static struct BurnRomInfo SpecBfeastieRomDesc[] = {
	{ "Beastie Feastie 128K2a (2005)(Beyker Soft).tap", 84411, 0xf1c07740, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecBfeastie, SpecBfeastie, Spec1282a)
STD_ROM_FN(SpecBfeastie)

struct BurnDriver BurnSpecBfeastie = {
	"spec_bfeastie", NULL, "spec_spec1282a", NULL, "2005",
	"Beastie Feastie (128K) (HB)\0", NULL, "Beyker Soft", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_ACTION, 0,
	SpectrumGetZipName, SpecBfeastieRomInfo, SpecBfeastieRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecQAOPSpaceDIPInfo,
	Spec128KPlus2Init, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Beethoven's Revenge (128K) (HB)

static struct BurnRomInfo SpecBeethrev128RomDesc[] = {
	{ "Beethoven's Revenge 128K (2022)(Sebastian Braunert).tap", 82557, 0xdae78c19, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecBeethrev128, SpecBeethrev128, Spec128)
STD_ROM_FN(SpecBeethrev128)

struct BurnDriver BurnSpecBeethrev128 = {
	"spec_beethrev128", NULL, "spec_spec128", NULL, "2022",
	"Beethoven's Revenge (128K) (HB)\0", NULL, "Sebastian Braunert", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_MAZE | GBF_ACTION, 0,
	SpectrumGetZipName, SpecBeethrev128RomInfo, SpecBeethrev128RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Beethoven's Revenge (48K) (HB)

static struct BurnRomInfo SpecBeethrev48RomDesc[] = {
	{ "Beethoven's Revenge 48K (2022)(Sebastian Braunert).tap", 59634, 0x184eaa29, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecBeethrev48, SpecBeethrev48, Spectrum)
STD_ROM_FN(SpecBeethrev48)

struct BurnDriver BurnSpecBeethrev48 = {
	"spec_beethrev48", "spec_beethrev128", "spec_spectrum", NULL, "2022",
	"Beethoven's Revenge (48K) (HB)\0", NULL, "Sebastian Braunert", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_MAZE | GBF_ACTION, 0,
	SpectrumGetZipName, SpecBeethrev48RomInfo, SpecBeethrev48RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Benny Hill's Madcap Chase! (128K) (Hack)

static struct BurnRomInfo SpecBennyhil128RomDesc[] = {
	{ "Benny Hill's Madcap Chase! 128K (2017)(AGDLabs).z80", 36822, 0x0c432e6c, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecBennyhil128, SpecBennyhil128, Spec128)
STD_ROM_FN(SpecBennyhil128)

struct BurnDriver BurnSpecBennyhil128 = {
	"spec_bennyhil128", "spec_bennyhil", "spec_spec128", NULL, "2017",
	"Benny Hill's Madcap Chase! (128K) (Hack)\0", "AY-PSG sound improvement", "AGDLabs", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE | BDF_HACK, 1, HARDWARE_SPECTRUM, GBF_MISC | GBF_ADV, 0,
	SpectrumGetZipName, SpecBennyhil128RomInfo, SpecBennyhil128RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Bigotudo, El (128K) (HB)

static struct BurnRomInfo SpecBigotudoRomDesc[] = {
	{ "Bigotudo, El 128K (2024)(Red Triangle).tap", 55771, 0x17e546ce, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecBigotudo, SpecBigotudo, Spec128)
STD_ROM_FN(SpecBigotudo)

struct BurnDriver BurnSpecBigotudo = {
	"spec_bigotudo", NULL, "spec_spec128", NULL, "2024",
	"Bigotudo, El (128K) (HB)\0", "S+O/S+P: select an item from inventory - ENTER: using an item", "Red Triangle", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecBigotudoRomInfo, SpecBigotudoRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Billie Night (English) (48K) (HB, v1.1)

static struct BurnRomInfo SpecBilnightenRomDesc[] = {
	{ "Billie Night EN v1.1 48K (2023)(Sequentia Soft).tap", 37685, 0xacd59adc, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecBilnighten, SpecBilnighten, Spectrum)
STD_ROM_FN(SpecBilnighten)

struct BurnDriver BurnSpecBilnighten = {
	"spec_bilnighten", NULL, "spec_spectrum", NULL, "2023",
	"Billie Night (English) (48K) (HB, v1.1)\0", NULL, "Sequentia Soft", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_ADV, 0,
	SpectrumGetZipName, SpecBilnightenRomInfo, SpecBilnightenRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecQAOPSpaceDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Billie Night (Spanish) (48K) (HB, v1.1)

static struct BurnRomInfo SpecBilnightesRomDesc[] = {
	{ "Billie Night ES v1.1 48K (2023)(Sequentia Soft).tap", 37685, 0x07b58694, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecBilnightes, SpecBilnightes, Spectrum)
STD_ROM_FN(SpecBilnightes)

struct BurnDriver BurnSpecBilnightes = {
	"spec_bilnightes", "spec_bilnighten", "spec_spectrum", NULL, "2023",
	"Billie Night (Spanish) (48K) (HB, v1.1)\0", NULL, "Sequentia Soft", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_ADV, 0,
	SpectrumGetZipName, SpecBilnightesRomInfo, SpecBilnightesRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecQAOPSpaceDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Binary Battle (48K) (HB)

static struct BurnRomInfo SpecBinbattleRomDesc[] = {
	{ "Binary Battle 48K (2024)(Haplo).tap", 23691, 0xb4d1bb45, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecBinbattle, SpecBinbattle, Spectrum)
STD_ROM_FN(SpecBinbattle)

struct BurnDriver BurnSpecBinbattle = {
	"spec_binbattle", NULL, "spec_spectrum", NULL, "2024",
	"Binary Battle (48K) (HB)\0", NULL, "Haplo", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PUZZLE, 0,
	SpectrumGetZipName, SpecBinbattleRomInfo, SpecBinbattleRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Binary Land (128K) (HB, v1.7)

static struct BurnRomInfo SpecBinlandRomDesc[] = {
	{ "Binary Land v1.7 128K (2020)(Joflof).z80", 45797, 0x1ce4134e, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecBinland, SpecBinland, Spec128)
STD_ROM_FN(SpecBinland)

struct BurnDriver BurnSpecBinland = {
	"spec_binland", NULL, "spec_spec128", NULL, "2020",
	"Binary Land (128K) (HB, v1.7)\0", NULL, "Joflof", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_MAZE | GBF_ACTION, 0,
	SpectrumGetZipName, SpecBinlandRomInfo, SpecBinlandRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Binary Land (16K) (HB, v1.7)

static struct BurnRomInfo SpecBinland16RomDesc[] = {
	{ "Binary Land v1.7 16K (2020)(Joflof).z80", 14855, 0xb4523bc7, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecBinland16, SpecBinland16, Spectrum)
STD_ROM_FN(SpecBinland16)

struct BurnDriver BurnSpecBinland16 = {
	"spec_binland16", NULL, "spec_spectrum", NULL, "2020",
	"Binary Land (16K) (HB, v1.7)\0", NULL, "Joflof", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_MAZE | GBF_ACTION, 0,
	SpectrumGetZipName, SpecBinland16RomInfo, SpecBinland16RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Biniax 2.0 (128K) (HB)

static struct BurnRomInfo SpecBiniaxRomDesc[] = {
	{ "Biniax 2.0 128K (2009)(Ubhres Productions).tap", 79889, 0x8af58c78, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecBiniax, SpecBiniax, Spec128)
STD_ROM_FN(SpecBiniax)

struct BurnDriver BurnSpecBiniax = {
	"spec_biniax", NULL, "spec_spec128", NULL, "2009",
	"Biniax 2.0 (128K) (HB)\0", NULL, "The Mojon Twins", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PUZZLE, 0,
	SpectrumGetZipName, SpecBiniaxRomInfo, SpecBiniaxRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Birdy Cantabile (48K) (HB)

static struct BurnRomInfo SpecBirdycantRomDesc[] = {
	{ "Birdy Cantabile 48K (2019)(iadvd).tap", 43648, 0x6271b643, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecBirdycant, SpecBirdycant, Spectrum)
STD_ROM_FN(SpecBirdycant)

struct BurnDriver BurnSpecBirdycant = {
	"spec_birdycant", NULL, "spec_spectrum", NULL, "2019",
	"Birdy Cantabile (48K) (HB)\0", NULL, "iadvd", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecBirdycantRomInfo, SpecBirdycantRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Black & White (128K) (HB, v1.1)

static struct BurnRomInfo SpecbnwRomDesc[] = {
	{ "Black & White v1.1 128K (2020)(Pat Morita Team).tap", 72706, 0x6629deb1, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specbnw, Specbnw, Spec128)
STD_ROM_FN(Specbnw)

struct BurnDriver BurnSpecbnw = {
	"spec_bnw", NULL, "spec_spec128", NULL, "2020",
	"Black & White (128K) (HB, v1.1)\0", "Use SPACE and ENTER to set options", "Pat Morita Team", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecbnwRomInfo, SpecbnwRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Black Sea - Treasure Hunters (128K) (HB)

static struct BurnRomInfo SpecBlackseaRomDesc[] = {
	{ "Black Sea - Treasure Hunters 128K (2021)(Mananuk).tap", 45089, 0x00ad4735, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecBlacksea, SpecBlacksea, Spec128)
STD_ROM_FN(SpecBlacksea)

struct BurnDriver BurnSpecBlacksea = {
	"spec_blacksea", NULL, "spec_spec128", NULL, "2021",
	"Black Sea - Treasure Hunters (128K) (HB)\0", NULL, "Mananuk", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_ACTION, 0,
	SpectrumGetZipName, SpecBlackseaRomInfo, SpecBlackseaRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Black Star (48K) (HB)

static struct BurnRomInfo SpecblackstarRomDesc[] = {
	{ "Black Star 48K (2015)(usebox.net).tap", 26349, 0xfd6cd75d, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specblackstar, Specblackstar, Spectrum)
STD_ROM_FN(Specblackstar)

struct BurnDriver BurnSpecblackstar = {
	"spec_blackstar", NULL, "spec_spectrum", NULL, "2015",
	"Black Star (48K) (HB)\0", NULL, "usebox.net", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_SHOOT, 0,
	SpectrumGetZipName, SpecblackstarRomInfo, SpecblackstarRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Blimpgeddon (48K) (HB, v1.03)

static struct BurnRomInfo SpecBlimpgeddonRomDesc[] = {
	{ "Blimpgeddon v1.03 48K (2019)(Narwhal).tap", 40791, 0x52db8723, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecBlimpgeddon, SpecBlimpgeddon, Spectrum)
STD_ROM_FN(SpecBlimpgeddon)

struct BurnDriver BurnSpecBlimpgeddon = {
	"spec_blimpgeddon", NULL, "spec_spectrum", NULL, "2017-19",
	"Blimpgeddon (48K) (HB, v1.03)\0", NULL, "Narwhal", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecBlimpgeddonRomInfo, SpecBlimpgeddonRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// BlockZ (128K) (HB)

static struct BurnRomInfo SpecblockzRomDesc[] = {
	{ "BlockZ 128K (2021)(Raymond Russell).tap", 24913, 0x44b94d5e, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specblockz, Specblockz, Spec128)
STD_ROM_FN(Specblockz)

struct BurnDriver BurnSpecblockz = {
	"spec_blockz", NULL, "spec_spec128", NULL, "2020",
	"BlockZ (128K) (HB)\0", NULL, "Raymond Russell", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PUZZLE, 0,
	SpectrumGetZipName, SpecblockzRomInfo, SpecblockzRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecQAOPMDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Bloktris (128K) (HB)

static struct BurnRomInfo SpecBloktris128RomDesc[] = {
	{ "Bloktris 128K (2023)(Under4Mhz).tap", 33725, 0xe3b57e30, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecBloktris128, SpecBloktris128, Spec128)
STD_ROM_FN(SpecBloktris128)

struct BurnDriver BurnSpecBloktris128 = {
	"spec_bloktris128", NULL, "spec_spec128", NULL, "2023",
	"Bloktris (128K) (HB)\0", NULL, "Under4Mhz", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PUZZLE, 0,
	SpectrumGetZipName, SpecBloktris128RomInfo, SpecBloktris128RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Bloktris (48K) (HB)

static struct BurnRomInfo SpecBloktris48RomDesc[] = {
	{ "Bloktris 48K (2023)(Under4Mhz).tap", 33787, 0xa5c03f14, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecBloktris48, SpecBloktris48, Spectrum)
STD_ROM_FN(SpecBloktris48)

struct BurnDriver BurnSpecBloktris48 = {
	"spec_bloktris48", "spec_bloktris128", "spec_spectrum", NULL, "2023",
	"Bloktris (48K) (HB)\0", NULL, "Under4Mhz", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PUZZLE, 0,
	SpectrumGetZipName, SpecBloktris48RomInfo, SpecBloktris48RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Blond, The (English) (128K) (HB)

static struct BurnRomInfo SpecBlond128enRomDesc[] = {
	{ "Blond, The EN 128K (2024)(Tom Martin).tap", 46904, 0xcdbcc874, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecBlond128en, SpecBlond128en, Spec128)
STD_ROM_FN(SpecBlond128en)

struct BurnDriver BurnSpecBlond128en = {
	"spec_blond128en", NULL, "spec_spec128", NULL, "2024",
	"Blond, The (English) (128K) (HB)\0", "Press SPACE to switch weapon: gun/dynamite", "Tom Martin", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_SHOOT, 0,
	SpectrumGetZipName, SpecBlond128enRomInfo, SpecBlond128enRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Blond, The (Spanish) (128K) (HB)

static struct BurnRomInfo SpecBlond128esRomDesc[] = {
	{ "Blond, The ES 128K (2024)(Tom Martin).tap", 46878, 0x10325f75, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecBlond128es, SpecBlond128es, Spec128)
STD_ROM_FN(SpecBlond128es)

struct BurnDriver BurnSpecBlond128es = {
	"spec_blond128es", NULL, "spec_spec128", NULL, "2024",
	"Blond, The (Spanish) (128K) (HB)\0", "Press SPACE to switch weapon: gun/dynamite", "Tom Martin", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_SHOOT, 0,
	SpectrumGetZipName, SpecBlond128esRomInfo, SpecBlond128esRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Blond, The (English) (48K) (HB)

static struct BurnRomInfo SpecBlond48enRomDesc[] = {
	{ "Blond, The EN 48K (2024)(Tom Martin).tap", 45806, 0xce0826dd, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecBlond48en, SpecBlond48en, Spectrum)
STD_ROM_FN(SpecBlond48en)

struct BurnDriver BurnSpecBlond48en = {
	"spec_blond48en", "spec_blond128en", "spec_spectrum", NULL, "2024",
	"Blond, The (English) (48K) (HB)\0", "Press SPACE to switch weapon: gun/dynamite", "Tom Martin", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_SHOOT, 0,
	SpectrumGetZipName, SpecBlond48enRomInfo, SpecBlond48enRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Blond, The (Spanish) (48K) (HB)

static struct BurnRomInfo SpecBlond48esRomDesc[] = {
	{ "Blond, The ES 48K (2024)(Tom Martin).tap", 45782, 0x5f7a0986, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecBlond48es, SpecBlond48es, Spectrum)
STD_ROM_FN(SpecBlond48es)

struct BurnDriver BurnSpecBlond48es = {
	"spec_blond48es", "spec_blond128es", "spec_spectrum", NULL, "2024",
	"Blond, The (Spanish) (48K) (HB)\0", "Press SPACE to switch weapon: gun/dynamite", "Tom Martin", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_SHOOT, 0,
	SpectrumGetZipName, SpecBlond48esRomInfo, SpecBlond48esRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Bobby Carrot (128K) (HB)

static struct BurnRomInfo SpecBobbycarrotRomDesc[] = {
	{ "Bobby Carrot (2018)(Diver, Quiet, Kyv, Zorba).tap", 34125, 0xaf5919ca, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecBobbycarrot, SpecBobbycarrot, Spec128)
STD_ROM_FN(SpecBobbycarrot)

struct BurnDriver BurnSpecBobbycarrot = {
	"spec_bobbycarrot", NULL, "spec_spec128", NULL, "2018",
	"Bobby Carrot (128K) (HB)\0", NULL, "Diver, Quiet, Kyv, Zorba", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_ACTION, 0,
	SpectrumGetZipName, SpecBobbycarrotRomInfo, SpecBobbycarrotRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Bomb Bomb Buster (128K) (HB)

static struct BurnRomInfo SpecBbbusterRomDesc[] = {
	{ "Bomb Bomb Buster 128K (2022)(Packobilly).tap", 56278, 0x041ddb4c, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecBbbuster, SpecBbbuster, Spec128)
STD_ROM_FN(SpecBbbuster)

struct BurnDriver BurnSpecBbbuster = {
	"spec_bbbuster", NULL, "spec_spec128", NULL, "2022",
	"Bomb Bomb Buster (128K) (HB)\0", NULL, "Packobilly", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecBbbusterRomInfo, SpecBbbusterRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Bomber Bot (128K) (HB)

static struct BurnRomInfo SpecBombbotRomDesc[] = {
	{ "Bomber Bot 128K (2023)(Gabriele Amore).tap", 56471, 0x37d131f3, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecBombbot, SpecBombbot, Spec128)
STD_ROM_FN(SpecBombbot)

struct BurnDriver BurnSpecBombbot = {
	"spec_bombbot", NULL, "spec_spec128", NULL, "2023",
	"Bomber Bot (128K) (HB)\0", NULL, "Gabriele Amore", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_ACTION, 0,
	SpectrumGetZipName, SpecBombbotRomInfo, SpecBombbotRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Bomber Pengo (128K) (HB)

static struct BurnRomInfo SpecBombpengoRomDesc[] = {
	{ "Bomber Pengo 128K (2023)(Gabriele Amore).tap", 47773, 0x6755ffe2, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecBombpengo, SpecBombpengo, Spec128)
STD_ROM_FN(SpecBombpengo)

struct BurnDriver BurnSpecBombpengo = {
	"spec_bombpengo", NULL, "spec_spec128", NULL, "2023",
	"Bomber Pengo (128K) (HB)\0", NULL, "Gabriele Amore", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecBombpengoRomInfo, SpecBombpengoRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecQAOPMDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Bomb Sweeper (48K) (HB)

static struct BurnRomInfo SpecBombsweepRomDesc[] = {
	{ "Bomb Sweeper 48K (2023)(Menyiques).tap", 46421, 0xb51f4118, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecBombsweep, SpecBombsweep, Spectrum)
STD_ROM_FN(SpecBombsweep)

struct BurnDriver BurnSpecBombsweep = {
	"spec_bombsweep", NULL, "spec_spectrum", NULL, "2023",
	"Bomb Sweeper (48K) (HB)\0", NULL, "Menyiques", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_MAZE, 0,
	SpectrumGetZipName, SpecBombsweepRomInfo, SpecBombsweepRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecQAOPSpaceDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Bootee (48K) (HB)

static struct BurnRomInfo SpecBooteeRomDesc[] = {
	{ "Bootee (2013)(The Mojon Twins).tap", 39147, 0xb4e06a95, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecBootee, SpecBootee, Spectrum)
STD_ROM_FN(SpecBootee)

struct BurnDriver BurnSpecBootee = {
	"spec_bootee", NULL, "spec_spectrum", NULL, "2013",
	"Bootee (48K) (HB)\0", NULL, "The Mojon Twins", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecBooteeRomInfo, SpecBooteeRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Boriel Runner (48K) (HB, v1.2)

static struct BurnRomInfo SpecBorielrunRomDesc[] = {
	{ "Boriel Runner v1.2 48K (2023)(briefer).tap", 30406, 0xddede219, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecBorielrun, SpecBorielrun, Spectrum)
STD_ROM_FN(SpecBorielrun)

struct BurnDriver BurnSpecBorielrun = {
	"spec_borielrun", NULL, "spec_spectrum", NULL, "2023",
	"Boriel Runner (48K) (HB, v1.2)\0", NULL, "briefer", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_ACTION, 0,
	SpectrumGetZipName, SpecBorielrunRomInfo, SpecBorielrunRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecQAOPSpaceDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// BoxeS (128K) (HB)

static struct BurnRomInfo SpecBoxes128RomDesc[] = {
	{ "BoxeS 128K (2017)(kas29).tap", 44491, 0xbba223cb, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecBoxes128, SpecBoxes128, Spec128)
STD_ROM_FN(SpecBoxes128)

struct BurnDriver BurnSpecBoxes128 = {
	"spec_boxes128", NULL, "spec_spec128", NULL, "2017",
	"BoxeS (128K) (HB)\0", NULL, "kas29", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PUZZLE, 0,
	SpectrumGetZipName, SpecBoxes128RomInfo, SpecBoxes128RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// BoxeS (48K) (HB)

static struct BurnRomInfo SpecBoxes48RomDesc[] = {
	{ "BoxeS 48K (2017)(kas29).tap", 41163, 0xa41a34a0, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecBoxes48, SpecBoxes48, Spectrum)
STD_ROM_FN(SpecBoxes48)

struct BurnDriver BurnSpecBoxes48 = {
	"spec_boxes48", "spec_boxes128", "spec_spectrum", NULL, "2017",
	"BoxeS (48K) (HB)\0", NULL, "kas29", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PUZZLE, 0,
	SpectrumGetZipName, SpecBoxes48RomInfo, SpecBoxes48RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Break Space (128K) (HB, Tech-Demo)

static struct BurnRomInfo SpecBreakspaceRomDesc[] = {
	{ "Break Space 128K (2016)(thesuper).tap", 93902, 0xb9212b04, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecBreakspace, SpecBreakspace, Spec128)
STD_ROM_FN(SpecBreakspace)

struct BurnDriver BurnSpecBreakspace = {
	"spec_breakspace", NULL, "spec_spec128", NULL, "2016",
	"Break Space (128K) (HB, Tech-Demo)\0", NULL, "thesuper", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW | BDF_DEMO, 1, HARDWARE_SPECTRUM, GBF_MISC, 0,
	SpectrumGetZipName, SpecBreakspaceRomInfo, SpecBreakspaceRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Brick Rick Graveyard Shift (128K) (HB, v1.03)

static struct BurnRomInfo SpecbrickrickRomDesc[] = {
	{ "Brick Rick Graveyard Shift v1.03 (2021)(usebox.net).tap", 31751, 0x5cea272c, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specbrickrick, Specbrickrick, Spec128)
STD_ROM_FN(Specbrickrick)

struct BurnDriver BurnSpecbrickrick = {
	"spec_brickrick", NULL, "spec_spec128", NULL, "2021",
	"Brick Rick Graveyard Shift (128K) (HB, v1.03)\0", NULL, "usebox.net", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecbrickrickRomInfo, SpecbrickrickRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Bruxolico - Side A (128K) (HB)

static struct BurnRomInfo SpecBruxolicoaRomDesc[] = {
	{ "Bruxolico - Side A 128K (2023)(Amaweks).tap", 42030, 0x8241a244, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecBruxolicoa, SpecBruxolicoa, Spec128)
STD_ROM_FN(SpecBruxolicoa)

struct BurnDriver BurnSpecBruxolicoa = {
	"spec_bruxolicoa", NULL, "spec_spec128", NULL, "2023",
	"Bruxolico - Side A (128K) (HB)\0", NULL, "Amaweks", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecBruxolicoaRomInfo, SpecBruxolicoaRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Bruxolico - Side B (128K) (HB)

static struct BurnRomInfo SpecBruxolicobRomDesc[] = {
	{ "Bruxolico - Side B 128K (2023)(Amaweks).tap", 41955, 0x29332105, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecBruxolicob, SpecBruxolicob, Spec128)
STD_ROM_FN(SpecBruxolicob)

struct BurnDriver BurnSpecBruxolicob = {
	"spec_bruxolicob", "spec_bruxolicoa", "spec_spec128", NULL, "2023",
	"Bruxolico - Side B (128K) (HB)\0", NULL, "Amaweks", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_ACTION, 0,
	SpectrumGetZipName, SpecBruxolicobRomInfo, SpecBruxolicobRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Bruxolico - Side C (128K) (HB)

static struct BurnRomInfo SpecBruxolicocRomDesc[] = {
	{ "Bruxolico - Side C 128K (2023)(Amaweks).tap", 44079, 0x82c08c58, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecBruxolicoc, SpecBruxolicoc, Spec128)
STD_ROM_FN(SpecBruxolicoc)

struct BurnDriver BurnSpecBruxolicoc = {
	"spec_bruxolicoc", "spec_bruxolicoa", "spec_spec128", NULL, "2023",
	"Bruxolico - Side C (128K) (HB)\0", NULL, "Amaweks", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_HORSHOOT, 0,
	SpectrumGetZipName, SpecBruxolicocRomInfo, SpecBruxolicocRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// B-Squared (128K) (HB)

static struct BurnRomInfo SpecBsquaredRomDesc[] = {
	{ "B-Squared 128K (2017)(Paul Jenkinson).tap", 40283, 0x240ad9f7, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecBsquared, SpecBsquared, Spec128)
STD_ROM_FN(SpecBsquared)

struct BurnDriver BurnSpecBsquared = {
	"spec_bsquared", NULL, "spec_spec128", NULL, "2017",
	"B-Squared (128K) (HB)\0", NULL, "Paul Jenkinson", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecBsquaredRomInfo, SpecBsquaredRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Bubble Frenzy (48K) (HB)

static struct BurnRomInfo SpecBubfrenzyRomDesc[] = {
	{ "Bubble Frenzy 48K (2016)(Gabriele Amore).tap", 28888, 0xb601336e, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecBubfrenzy, SpecBubfrenzy, Spectrum)
STD_ROM_FN(SpecBubfrenzy)

struct BurnDriver BurnSpecBubfrenzy = {
	"spec_bubfrenzy", NULL, "spec_spectrum", NULL, "2016",
	"Bubble Frenzy (48K) (HB)\0", NULL, "Gabriele Amore", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_ACTION, 0,
	SpectrumGetZipName, SpecBubfrenzyRomInfo, SpecBubfrenzyRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Bubble Monkey Bros (128K) (HB)

static struct BurnRomInfo SpecBubmonkeyRomDesc[] = {
	{ "Bubble Monkey Bros 128K (2015)(Gabriele Amore).tap", 36566, 0x729dd4be, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecBubmonkey, SpecBubmonkey, Spec128)
STD_ROM_FN(SpecBubmonkey)

struct BurnDriver BurnSpecBubmonkey = {
	"spec_bubmonkey", NULL, "spec_spec128", NULL, "2015",
	"Bubble Monkey Bros (128K) (HB)\0", "Reconfigure dipswitches for 2 Players coop-mode", "Gabriele Amore", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 2, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecBubmonkeyRomInfo, SpecBubmonkeyRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecQAOPMDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Bubble Monkey Bros (bug fixed) (48K) (HB)

static struct BurnRomInfo SpecBubmonkeybfRomDesc[] = {
	{ "Bubble Monkey Bros - bug fixed 48K (2015)(Gabriele Amore).tap", 32418, 0xbd320030, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecBubmonkeybf, SpecBubmonkeybf, Spectrum)
STD_ROM_FN(SpecBubmonkeybf)

struct BurnDriver BurnSpecBubmonkeybf = {
	"spec_bubmonkeybf", "spec_bubmonkey", "spec_spectrum", NULL, "2015",
	"Bubble Monkey Bros (bug fixed) (48K) (HB)\0", "Bug fixed version - without AY music", "Gabriele Amore", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE | BDF_HOMEBREW, 2, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecBubmonkeybfRomInfo, SpecBubmonkeybfRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecQAOPMDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Bubu's Anteater (48K) (HB)

static struct BurnRomInfo SpecBanteaterRomDesc[] = {
	{ "Bubu's Anteater 48K (2021)(Bubu).tap", 21399, 0x81c38b76, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecBanteater, SpecBanteater, Spectrum)
STD_ROM_FN(SpecBanteater)

struct BurnDriver BurnSpecBanteater = {
	"spec_banteater", NULL, "spec_spectrum", NULL, "2021",
	"Bubu's Anteater (48K) (HB)\0", "1: start - 5: insert coins", "Bubu", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_ACTION, 0,
	SpectrumGetZipName, SpecBanteaterRomInfo, SpecBanteaterRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecQAOPSpaceDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Bufonada (English) (128K) (HB)

static struct BurnRomInfo SpecBufonadaeRomDesc[] = {
	{ "Bufonada EN 128K (2022)(Roolandoo).tap", 48242, 0x03516fbe, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecBufonadae, SpecBufonadae, Spec128)
STD_ROM_FN(SpecBufonadae)

struct BurnDriver BurnSpecBufonadae = {
	"spec_bufonadae", NULL, "spec_spec128", NULL, "2022",
	"Bufonada (English) (128K) (HB)\0", NULL, "Roolandoo", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PLATFORM | GBF_PUZZLE, 0,
	SpectrumGetZipName, SpecBufonadaeRomInfo, SpecBufonadaeRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Bufonada (Spanish) (128K) (HB)


static struct BurnRomInfo SpecBufonadasRomDesc[] = {
	{ "Bufonada ES 128K (2022)(Roolandoo).tap", 48242, 0x2b5c7742, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecBufonadas, SpecBufonadas, Spec128)
STD_ROM_FN(SpecBufonadas)

struct BurnDriver BurnSpecBufonadas = {
	"spec_bufonadas", "spec_bufonadae", "spec_spec128", NULL, "2022",
	"Bufonada (Spanish) (128K) (HB)\0", NULL, "Roolandoo", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PLATFORM | GBF_PUZZLE, 0,
	SpectrumGetZipName, SpecBufonadasRomInfo, SpecBufonadasRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Cacorm (48K) (HB)

static struct BurnRomInfo SpecCacormRomDesc[] = {
	{ "Cacorm 48K-AY (2022)(Inufuto).tap", 9317, 0xbac52078, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecCacorm, SpecCacorm, Spectrum)
STD_ROM_FN(SpecCacorm)

struct BurnDriver BurnSpecCacorm = {
	"spec_cacorm", NULL, "spec_spectrum", NULL, "2022",
	"Cacorm (48K) (HB)\0", "AY Sound supported", "Inufuto", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_ACTION | GBF_MAZE, 0,
	SpectrumGetZipName, SpecCacormRomInfo, SpecCacormRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Cannibal Island 2 (48K) (HB)

static struct BurnRomInfo SpecCannisland2RomDesc[] = {
	{ "Cannibal Island 2 48K (2023)(Pavero Software).tap", 40421, 0x4bc3ae8a, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecCannisland2, SpecCannisland2, Spectrum)
STD_ROM_FN(SpecCannisland2)

struct BurnDriver BurnSpecCannisland2 = {
	"spec_cannisland2", NULL, "spec_spectrum", NULL, "2023",
	"Cannibal Island 2 (48K) (HB)\0", NULL, "Pavero Software", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_MISC, 0,
	SpectrumGetZipName, SpecCannisland2RomInfo, SpecCannisland2RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecIntf2DIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Carga de Profundidade (48K)

static struct BurnRomInfo SpecCargaprofRomDesc[] = {
	{ "Carga de Profundidade 48K (2021)(Filipe Veiga).tap", 17157, 0xe4e8dbff, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecCargaprof, SpecCargaprof, Spectrum)
STD_ROM_FN(SpecCargaprof)

struct BurnDriver BurnSpecCargaprof = {
	"spec_cargaprof", NULL, "spec_spectrum", NULL, "2021",
	"Carga de Profundidade (48K) (HB)\0", NULL, "Filipe Veiga", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_ACTION, 0,
	SpectrumGetZipName, SpecCargaprofRomInfo, SpecCargaprofRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecQAOPSpaceDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Carlos Michelis - All in One (English/Russian/Spanish) (128K) (HB)

static struct BurnRomInfo Spec_CarlosmRomDesc[] = {
	{ "Carlos Michelis - All in One 128K (2012-18)(World XXI Soft Inc.).tap", 316868, 0xdf7cf901, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Spec_Carlosm, Spec_Carlosm, Spec128)
STD_ROM_FN(Spec_Carlosm)

struct BurnDriver BurnSpec_Carlosm = {
	"spec_carlosm", NULL, "spec_spec128", NULL, "2012-2018",
	"Carlos Michelis - All in One (English/Russian/Spanish) (128K) (HB)\0", "Main version + Expansion version", "World XXI Soft Inc.", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_RUNGUN, 0,
	SpectrumGetZipName, Spec_CarlosmRomInfo, Spec_CarlosmRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Castle Escape (128K) (HB)

static struct BurnRomInfo SpecCastlescapeRomDesc[] = {
	{ "Castle Escape 128K (2022)(IrataHack).tap", 42131, 0x447f175d, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecCastlescape, SpecCastlescape, Spec128)
STD_ROM_FN(SpecCastlescape)

struct BurnDriver BurnSpecCastlescape = {
	"spec_castlescape", NULL, "spec_spec128", NULL, "2022",
	"Castle Escape (128K) (HB)\0", NULL, "IrataHack", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecCastlescapeRomInfo, SpecCastlescapeRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Cattivik (128K) (HB)

static struct BurnRomInfo SpecCattivikRomDesc[] = {
	{ "Cattivik 128K (2013)(Gabriele Amore, Alessandro Grussu).tap", 65828, 0x92e705bb, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecCattivik, SpecCattivik, Spec128)
STD_ROM_FN(SpecCattivik)

struct BurnDriver BurnSpecCattivik = {
	"spec_cattivik", NULL, "spec_spec128", NULL, "2013",
	"Cattivik (128K) (HB)\0", NULL, "Gabriele Amore, Alessandro Grussu", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecCattivikRomInfo, SpecCattivikRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecQAOPSpaceDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Cattivik 2024 (128K) (HB)

static struct BurnRomInfo SpecCattivik24RomDesc[] = {
	{ "Cattivik 2024 128K (2024)(Gabriele Amore).tap", 52220, 0xdba277f8, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecCattivik24, SpecCattivik24, Spec128)
STD_ROM_FN(SpecCattivik24)

struct BurnDriver BurnSpecCattivik24 = {
	"spec_cattivik24", NULL, "spec_spec128", NULL, "2024",
	"Cattivik 2024 (128K) (HB)\0", NULL, "Gabriele Amore", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecCattivik24RomInfo, SpecCattivik24RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Cattivik Never Dies (128K) (HB)

static struct BurnRomInfo SpecCattivik2RomDesc[] = {
	{ "Cattivik Never Dies 128K (2017)(Gabriele Amore).tap", 43306, 0xaa780aaf, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecCattivik2, SpecCattivik2, Spec128)
STD_ROM_FN(SpecCattivik2)

struct BurnDriver BurnSpecCattivik2 = {
	"spec_cattivik2", NULL, "spec_spec128", NULL, "2017",
	"Cattivik Never Dies (128K) (HB)\0", NULL, "Gabriele Amore", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecCattivik2RomInfo, SpecCattivik2RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecQAOPMDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Cave Raider - The Cursed Treasure (128K) (HB)

static struct BurnRomInfo SpecCaveraidRomDesc[] = {
	{ "Cave Raider - The Cursed Treasure 128K (2018)(Bumfun Software).tap", 35128, 0x57718ae2, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecCaveraid, SpecCaveraid, Spec128)
STD_ROM_FN(SpecCaveraid)

struct BurnDriver BurnSpecCaveraid = {
	"spec_caveraid", NULL, "spec_spec128", NULL, "2018",
	"Cave Raider - The Cursed Treasure (128K) (HB)\0", NULL, "Bumfun Software", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecCaveraidRomInfo, SpecCaveraidRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Caveted Against the Darkness (48K) (HB)

static struct BurnRomInfo SpecCavetedRomDesc[] = {
	{ "Caveted Against the Darkness 48K (2022)(Stonechat Productions).tap", 47774, 0x1fde01af, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecCaveted, SpecCaveted, Spectrum)
STD_ROM_FN(SpecCaveted)

struct BurnDriver BurnSpecCaveted = {
	"spec_caveted", NULL, "spec_spectrum", NULL, "2022",
	"Caveted Against the Darkness (48K) (HB)\0", NULL, "Stonechat Productions", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecCavetedRomInfo, SpecCavetedRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecQAOPMDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Cavit (48K) (HB)

static struct BurnRomInfo SpecCavitRomDesc[] = {
	{ "Cavit 48K-AY (2021)(Inufuto).tap", 11732, 0x1e7f06b4, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecCavit, SpecCavit, Spectrum)
STD_ROM_FN(SpecCavit)

struct BurnDriver BurnSpecCavit = {
	"spec_cavit", NULL, "spec_spectrum", NULL, "2021",
	"Cavit (48K) (HB)\0", "AY Sound supported", "Inufuto", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_ACTION, 0,
	SpectrumGetZipName, SpecCavitRomInfo, SpecCavitRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Cheesy Chase Moritz and the Mildewed Moon (128K) (HB)

static struct BurnRomInfo SpecCcmmmRomDesc[] = {
	{ "Cheesy Chase Moritz and the Mildewed Moon 128K (2019)(Sebastian Braunert).tap", 68545, 0xf0e827c2, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecCcmmm, SpecCcmmm, Spec128)
STD_ROM_FN(SpecCcmmm)

struct BurnDriver BurnSpecCcmmm = {
	"spec_ccmmm", NULL, "spec_spec128", NULL, "2019",
	"Cheesy Chase Moritz and the Mildewed Moon (128K) (HB)\0", NULL, "Sebastian Braunert", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_ACTION, 0,
	SpectrumGetZipName, SpecCcmmmRomInfo, SpecCcmmmRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecQAOPSpaceDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Cesare The Somnambule (128K) (HB)

static struct BurnRomInfo SpecCesaretsRomDesc[] = {
	{ "Cesare The Somnambule 128K (2024)(Team Moritz).tap", 123918, 0xb4c3289e, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecCesarets, SpecCesarets, Spec128)
STD_ROM_FN(SpecCesarets)

struct BurnDriver BurnSpecCesarets = {
	"spec_cesarets", NULL, "spec_spec128", NULL, "2024",
	"Cesare The Somnambule (128K) (HB)\0", NULL, "Team Moritz", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_ACTION | GBF_ADV, 0,
	SpectrumGetZipName, SpecCesaretsRomInfo, SpecCesaretsRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Chemi Chaos (128K) (HB)

static struct BurnRomInfo SpecChemichaosRomDesc[] = {
	{ "Chemi Chaos 128K (2023)(g0blinish).tap", 9393, 0x8905d3a7, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecChemichaos, SpecChemichaos, Spec128)
STD_ROM_FN(SpecChemichaos)

struct BurnDriver BurnSpecChemichaos = {
	"spec_chemichaos", NULL, "spec_spec128", NULL, "2023",
	"Chemi Chaos (128K) (HB)\0", "R: reset - S: skip - O: left - P: right - Enter: fire", "g0blinish", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PUZZLE, 0,
	SpectrumGetZipName, SpecChemichaosRomInfo, SpecChemichaosRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Cherils Perils (48K) (HB)

static struct BurnRomInfo SpecCherilsRomDesc[] = {
	{ "Cherils Perils 48K (2010)(The Mojon Twins).z80", 38505, 0xbf144935, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecCherils, SpecCherils, Spectrum)
STD_ROM_FN(SpecCherils)

struct BurnDriver BurnSpecCherils = {
	"spec_cherils", NULL, "spec_spectrum", NULL, "2010",
	"Cherils Perils (48K) (HB)\0", NULL, "The Mojon Twins", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecCherilsRomInfo, SpecCherilsRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Cheril the Goddess (48K) (HB, v2)

static struct BurnRomInfo SpeccherilgodRomDesc[] = {
	{ "Cheril The Goddess V2 (2011)(Mojon Twins).tap", 43545, 0x94cc3db8, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Speccherilgod, Speccherilgod, Spectrum)
STD_ROM_FN(Speccherilgod)

struct BurnDriver BurnSpeccherilgod = {
	"spec_cherilgod", NULL, "spec_spectrum", NULL, "2011",
	"Cheril the Goddess (48K) (HB, v2)\0", NULL, "The Mojon Twins", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpeccherilgodRomInfo, SpeccherilgodRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Chez Maxime ZX (French) (128K) (HB)

static struct BurnRomInfo SpecChezmaxRomDesc[] = {
	{ "Chez Maxime ZX FR 128K (2022)(highriser).tap", 48799, 0xab6b980b, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecChezmax, SpecChezmax, Spec128)
STD_ROM_FN(SpecChezmax)

struct BurnDriver BurnSpecChezmax = {
	"spec_chezmax", NULL, "spec_spec128", NULL, "2022",
	"Chez Maxime ZX (French) (128K) (HB)\0", NULL, "highriser", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_ACTION, 0,
	SpectrumGetZipName, SpecChezmaxRomInfo, SpecChezmaxRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Chicks & Bricks (48K) (HB)

static struct BurnRomInfo SpecChknbrkRomDesc[] = {
	{ "Chicks & Bricks 48K (2007)(Rafal Miazga).tap", 35427, 0x9dbfac6a, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecChknbrk, SpecChknbrk, Spectrum)
STD_ROM_FN(SpecChknbrk)

struct BurnDriver BurnSpecChknbrk = {
	"spec_chknbrk", NULL, "spec_spectrum", NULL, "2007",
	"Chicks & Bricks (48K) (HB)\0", NULL, "Rafal Miazga", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_BREAKOUT, 0,
	SpectrumGetZipName, SpecChknbrkRomInfo, SpecChknbrkRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Chispy (English) (128K) (HB)

static struct BurnRomInfo SpecChispyeRomDesc[] = {
	{ "Chispy EN 128K (2022)(xavisan).tap", 43965, 0x93b9fa2a, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecChispye, SpecChispye, Spec128)
STD_ROM_FN(SpecChispye)

struct BurnDriver BurnSpecChispye = {
	"spec_chispye", NULL, "spec_spec128", NULL, "2022",
	"Chispy (English) (128K) (HB)\0", NULL, "xavisan", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecChispyeRomInfo, SpecChispyeRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Chispy (Spanish) (128K) (HB)

static struct BurnRomInfo SpecChispysRomDesc[] = {
	{ "Chispy ES 128K (2022)(xavisan).tap", 43995, 0xc2c964b0, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecChispys, SpecChispys, Spec128)
STD_ROM_FN(SpecChispys)

struct BurnDriver BurnSpecChispys = {
	"spec_chispys", "spec_chispye", "spec_spec128", NULL, "2022",
	"Chispy (Spanish) (128K) (HB)\0", NULL, "xavisan", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecChispysRomInfo, SpecChispysRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Chopper Defence (48K) (HB, v1.2)

static struct BurnRomInfo SpecChopdefRomDesc[] = {
	{ "Chopper Defence v1.2 48K (2022)(Psychic Parrot).tap", 29454, 0x1a77fcba, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecChopdef, SpecChopdef, Spectrum)
STD_ROM_FN(SpecChopdef)

struct BurnDriver BurnSpecChopdef = {
	"spec_chopdef", NULL, "spec_spectrum", NULL, "2022",
	"Chopper Defence (48K) (HB, v1.2)\0", NULL, "Psychic Parrot", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_ACTION, 0,
	SpectrumGetZipName, SpecChopdefRomInfo, SpecChopdefRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Christmas Robin Simulator (48K) (HB)

static struct BurnRomInfo SpecChristmasrsRomDesc[] = {
	{ "Christmas Robin Simulator 48K (2022)(saucerbrain).tap", 22536, 0xcf6e38cd, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecChristmasrs, SpecChristmasrs, Spectrum)
STD_ROM_FN(SpecChristmasrs)

struct BurnDriver BurnSpecChristmasrs = {
	"spec_christmasrs", NULL, "spec_spectrum", NULL, "2022",
	"Christmas Robin Simulator (48K) (HB)\0", NULL, "saucerbrain", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_ACTION, 0,
	SpectrumGetZipName, SpecChristmasrsRomInfo, SpecChristmasrsRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecQAOPSpaceDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// ChristMaze (48K) (HB)

static struct BurnRomInfo SpecChristmazeRomDesc[] = {
	{ "ChristMaze 48K (2016)(Stonechat Productions).tap", 32692, 0x3bc84a53, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecChristmaze, SpecChristmaze, Spectrum)
STD_ROM_FN(SpecChristmaze)

struct BurnDriver BurnSpecChristmaze = {
	"spec_christmaze", NULL, "spec_spectrum", NULL, "2016",
	"ChristMaze (48K) (HB)\0", NULL, "Stonechat Productions", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_ACTION | GBF_MAZE, 0,
	SpectrumGetZipName, SpecChristmazeRomInfo, SpecChristmazeRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecQAOPSpaceDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Chromanoids (128K) (HB, v1.1)

static struct BurnRomInfo SpecChromanoidsRomDesc[] = {
	{ "Chromanoids v1.1 128K (2023)(Ariel Endaraues).tap", 43903, 0x1999bbdd, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecChromanoids, SpecChromanoids, Spec128)
STD_ROM_FN(SpecChromanoids)

struct BurnDriver BurnSpecChromanoids = {
	"spec_chromanoids", NULL, "spec_spec128", NULL, "2023",
	"Chromanoids (128K) (HB, v1.1)\0", NULL, "Ariel Endaraues", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_ACTION | GBF_MAZE, 0,
	SpectrumGetZipName, SpecChromanoidsRomInfo, SpecChromanoidsRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Circuitry (48K) (HB, v1.1)

static struct BurnRomInfo SpeccircuitryRomDesc[] = {
	{ "Circuitry v1.1 48K (2017)(Rucksack Games).tap", 31538, 0xc97061d1, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Speccircuitry, Speccircuitry, Spectrum)
STD_ROM_FN(Speccircuitry)

struct BurnDriver BurnSpeccircuitry = {
	"spec_circuitry", NULL, "spec_spectrum", NULL, "2017",
	"Circuitry (48K) (HB, v1.1)\0", NULL, "Rucksack Games", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpeccircuitryRomInfo, SpeccircuitryRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Clone Zoo (48K) (HB, v1.1)

static struct BurnRomInfo SpecClonezooRomDesc[] = {
	{ "Clone Zoo 48K (2023)(PuttyCAD).tap", 34871, 0xf4d23c8f, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecClonezoo, SpecClonezoo, Spectrum)
STD_ROM_FN(SpecClonezoo)

struct BurnDriver BurnSpecClonezoo = {
	"spec_clonezoo", NULL, "spec_spectrum", NULL, "2023",
	"Clone Zoo (48K) (HB, v1.1)\0", NULL, "PuttyCAD", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PUZZLE, 0,
	SpectrumGetZipName, SpecClonezooRomInfo, SpecClonezooRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Cocaine Bear (128K) (HB)

static struct BurnRomInfo SpecCocabearRomDesc[] = {
	{ "Cocaine Bear 128K (2023)(Azimov).tap", 27332, 0x8c4a6eac, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecCocabear, SpecCocabear, Spec128)
STD_ROM_FN(SpecCocabear)

struct BurnDriver BurnSpecCocabear = {
	"spec_cocabear", NULL, "spec_spec128", NULL, "2023",
	"Cocaine Bear (128K) (HB)\0", NULL, "Azimov", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_ACTION, 0,
	SpectrumGetZipName, SpecCocabearRomInfo, SpecCocabearRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Cocoa and the Time Machine (English) (128K) (HB)

static struct BurnRomInfo SpecCocoatmRomDesc[] = {
	{ "Cocoa and the Time Machine EN 128K (2020)(Minilop).tap", 46020, 0xbba37879, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecCocoatm, SpecCocoatm, Spec128)
STD_ROM_FN(SpecCocoatm)

struct BurnDriver BurnSpecCocoatm = {
	"spec_cocoatm", NULL, "spec_spec128", NULL, "2020",
	"Cocoa and the Time Machine (English) (128K) (HB)\0", NULL, "Minilop", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecCocoatmRomInfo, SpecCocoatmRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Cocoa and the Time Machine (Portuguese) (48K) (HB)

static struct BurnRomInfo SpecCocoaptRomDesc[] = {
	{ "Cocoa and the Time Machine PT 48K (2020)(Minilop).tap", 39130, 0x66dce3ad, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecCocoapt, SpecCocoapt, Spectrum)
STD_ROM_FN(SpecCocoapt)

struct BurnDriver BurnSpecCocoapt = {
	"spec_cocoapt", "spec_cocoatm", "spec_spectrum", NULL, "2020",
	"Cocoa and the Time Machine (Portuguese) (48K) (HB)\0", NULL, "Minilop", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecCocoaptRomInfo, SpecCocoaptRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Cocoa and the Time Machine (Spanish) (128K) (HB)

static struct BurnRomInfo SpecCocoaspRomDesc[] = {
	{ "Cocoa and the Time Machine ES 128K (2020)(Minilop).tap", 46033, 0xdd44856a, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecCocoasp, SpecCocoasp, Spec128)
STD_ROM_FN(SpecCocoasp)

struct BurnDriver BurnSpecCocoasp = {
	"spec_cocoasp", "spec_cocoatm", "spec_spec128", NULL, "2020",
	"Cocoa and the Time Machine (Spanish) (128K) (HB)\0", NULL, "Minilop", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecCocoaspRomInfo, SpecCocoaspRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Cocoa 2, 24 Hour Parsley People - Episode 1 (English) (128K) (HB)

static struct BurnRomInfo SpecCocoa2e1RomDesc[] = {
	{ "Cocoa 2 Twenty Four Hour Parsley People EP1 EN 128K (2020)(Minilop).tap", 49168, 0x091c933d, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecCocoa2e1, SpecCocoa2e1, Spec128)
STD_ROM_FN(SpecCocoa2e1)

struct BurnDriver BurnSpecCocoa2e1 = {
	"spec_cocoa2e1", NULL, "spec_spec128", NULL, "2020",
	"Cocoa 2, 24 Hour Parsley People - Episode 1 (English) (128K) (HB)\0", "Supporter Edition", "Minilop", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecCocoa2e1RomInfo, SpecCocoa2e1RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Cocoa 2, 24 Hour Parsley People - Episode 1 (Portuguese) (128K) (HB)

static struct BurnRomInfo SpecCocoa2e1ptRomDesc[] = {
	{ "Cocoa 2 Twenty Four Hour Parsley People EP1 PT 128K (2020)(Minilop).tap", 49168, 0x12e1984a, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecCocoa2e1pt, SpecCocoa2e1pt, Spec128)
STD_ROM_FN(SpecCocoa2e1pt)

struct BurnDriver BurnSpecCocoa2e1pt = {
	"spec_cocoa2e1pt", "spec_cocoa2e1", "spec_spec128", NULL, "2020",
	"Cocoa 2, 24 Hour Parsley People - Episode 1 (Portuguese) (128K) (HB)\0", "Supporter Edition", "Minilop", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecCocoa2e1ptRomInfo, SpecCocoa2e1ptRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Cocoa 2, 24 Hour Parsley People - Episode 1 (Spanish) (128K) (HB)

static struct BurnRomInfo SpecCocoa2e1spRomDesc[] = {
	{ "Cocoa 2 Twenty Four Hour Parsley People EP1 ES 128K (2020)(Minilop).tap", 49168, 0x98b01da0, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecCocoa2e1sp, SpecCocoa2e1sp, Spec128)
STD_ROM_FN(SpecCocoa2e1sp)

struct BurnDriver BurnSpecCocoa2e1sp = {
	"spec_cocoa2e1sp", "spec_cocoa2e1", "spec_spec128", NULL, "2020",
	"Cocoa 2, 24 Hour Parsley People - Episode 1 (Spanish) (128K) (HB)\0", "Supporter Edition", "Minilop", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecCocoa2e1spRomInfo, SpecCocoa2e1spRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Cocoa 2, 24 Hour Parsley People - Episode 2 (English) (128K) (HB)

static struct BurnRomInfo SpecCocoa2e2RomDesc[] = {
	{ "Cocoa 2 Twenty Four Hour Parsley People EP2 EN 128K (2020)(Minilop).tap", 49204, 0xcc0c836f, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecCocoa2e2, SpecCocoa2e2, Spec128)
STD_ROM_FN(SpecCocoa2e2)

struct BurnDriver BurnSpecCocoa2e2 = {
	"spec_cocoa2e2", NULL, "spec_spec128", NULL, "2020",
	"Cocoa 2, 24 Hour Parsley People - Episode 2 (English) (128K) (HB)\0", "Supporter Edition", "Minilop", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecCocoa2e2RomInfo, SpecCocoa2e2RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Cocoa 2, 24 Hour Parsley People - Episode 2 (Portuguese) (128K) (HB)

static struct BurnRomInfo SpecCocoa2e2ptRomDesc[] = {
	{ "Cocoa 2 Twenty Four Hour Parsley People EP2 PT 128K (2020)(Minilop).tap", 49204, 0x66efddd9, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecCocoa2e2pt, SpecCocoa2e2pt, Spec128)
STD_ROM_FN(SpecCocoa2e2pt)

struct BurnDriver BurnSpecCocoa2e2pt = {
	"spec_cocoa2e2pt", "spec_cocoa2e2", "spec_spec128", NULL, "2020",
	"Cocoa 2, 24 Hour Parsley People - Episode 2 (Portuguese) (128K) (HB)\0", "Supporter Edition", "Minilop", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecCocoa2e2ptRomInfo, SpecCocoa2e2ptRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Cocoa 2, 24 Hour Parsley People - Episode 2 (Spanish) (128K) (HB)

static struct BurnRomInfo SpecCocoa2e2spRomDesc[] = {
	{ "Cocoa 2 Twenty Four Hour Parsley People EP2 ES 128K (2020)(Minilop).tap", 49204, 0xacf1b30d, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecCocoa2e2sp, SpecCocoa2e2sp, Spec128)
STD_ROM_FN(SpecCocoa2e2sp)

struct BurnDriver BurnSpecCocoa2e2sp = {
	"spec_cocoa2e2sp", "spec_cocoa2e2", "spec_spec128", NULL, "2020",
	"Cocoa 2, 24 Hour Parsley People - Episode 2 (Spanish) (128K) (HB)\0", "Supporter Edition", "Minilop", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecCocoa2e2spRomInfo, SpecCocoa2e2spRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Cocoa 2, 24 Hour Parsley People - Episode 3 (English) (128K) (HB)

static struct BurnRomInfo SpecCocoa2e3RomDesc[] = {
	{ "Cocoa 2 Twenty Four Hour Parsley People EP3 EN 128K (2020)(Minilop).tap", 49200, 0x28595082, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecCocoa2e3, SpecCocoa2e3, Spec128)
STD_ROM_FN(SpecCocoa2e3)

struct BurnDriver BurnSpecCocoa2e3 = {
	"spec_cocoa2e3", NULL, "spec_spec128", NULL, "2020",
	"Cocoa 2, 24 Hour Parsley People - Episode 3 (English) (128K) (HB)\0", "Supporter Edition", "Minilop", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecCocoa2e3RomInfo, SpecCocoa2e3RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Cocoa 2, 24 Hour Parsley People - Episode 3 (Portuguese) (128K) (HB)

static struct BurnRomInfo SpecCocoa2e3ptRomDesc[] = {
	{ "Cocoa 2 Twenty Four Hour Parsley People EP3 PT 128K (2020)(Minilop).tap", 49200, 0xc135051e, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecCocoa2e3pt, SpecCocoa2e3pt, Spec128)
STD_ROM_FN(SpecCocoa2e3pt)

struct BurnDriver BurnSpecCocoa2e3pt = {
	"spec_cocoa2e3pt", "spec_cocoa2e3", "spec_spec128", NULL, "2020",
	"Cocoa 2, 24 Hour Parsley People - Episode 3 (Portuguese) (128K) (HB)\0", "Supporter Edition", "Minilop", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecCocoa2e3ptRomInfo, SpecCocoa2e3ptRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Cocoa 2, 24 Hour Parsley People - Episode 3 (Spanish) (128K) (HB)

static struct BurnRomInfo SpecCocoa2e3spRomDesc[] = {
	{ "Cocoa 2 Twenty Four Hour Parsley People EP3 ES 128K (2020)(Minilop).tap", 49200, 0x80202e0a, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecCocoa2e3sp, SpecCocoa2e3sp, Spec128)
STD_ROM_FN(SpecCocoa2e3sp)

struct BurnDriver BurnSpecCocoa2e3sp = {
	"spec_cocoa2e3sp", "spec_cocoa2e3", "spec_spec128", NULL, "2020",
	"Cocoa 2, 24 Hour Parsley People - Episode 3 (Spanish) (128K) (HB)\0", "Supporter Edition", "Minilop", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecCocoa2e3spRomInfo, SpecCocoa2e3spRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// COINZ are mine (128K) (HB)

static struct BurnRomInfo SpecCoinzRomDesc[] = {
	{ "COINZ are mine 128K (2023)(Drunk Fly).tap", 31418, 0x1c0f1177, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecCoinz, SpecCoinz, Spec128)
STD_ROM_FN(SpecCoinz)

struct BurnDriver BurnSpecCoinz = {
	"spec_coinz", NULL, "spec_spec128", NULL, "2023",
	"COINZ are mine (128K) (HB)\0", NULL, "Drunk Fly", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 2, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecCoinzRomInfo, SpecCoinzRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Colin: The Cleaner - Turbo Edition 48K (48K) (HB, v2.0)

static struct BurnRomInfo SpecColinteRomDesc[] = {
	{ "Colin - The Cleaner - Turbo Edition v2.0 48K (2024)(RetroTechtive).tap", 20570, 0xa7f853fb, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecColinte, SpecColinte, Spectrum)
STD_ROM_FN(SpecColinte)

struct BurnDriver BurnSpecColinte = {
	"spec_colinte", NULL, "spec_spectrum", NULL, "2024",
	"Colin: The Cleaner - Turbo Edition (48K) (HB, v2.0)\0", "Press 'S' to use controller", "RetroTechtive", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecColinteRomInfo, SpecColinteRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Coloco (48K) (HB)

static struct BurnRomInfo SpeccolocoRomDesc[] = {
	{ "Coloco 48K (2020)(The Mojon Twins).tap", 46855, 0xdd07fa17, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Speccoloco, Speccoloco, Spectrum)
STD_ROM_FN(Speccoloco)

struct BurnDriver BurnSpeccoloco = {
	"spec_coloco", NULL, "spec_spectrum", NULL, "2020",
	"Coloco (48K) (HB)\0", NULL, "The Mojon Twins", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpeccolocoRomInfo, SpeccolocoRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Colonos ZX (48K) (HB)

static struct BurnRomInfo SpecColonoszxRomDesc[] = {
	{ "Colonos ZX 48K (2020)(Jose Manuel Gris).tap", 35144, 0xb5bac9fa, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecColonoszx, SpecColonoszx, Spec128)
STD_ROM_FN(SpecColonoszx)

struct BurnDriver BurnSpecColonoszx = {
	"spec_colonoszx", NULL, "spec_spec128", NULL, "2020",
	"Colonos ZX (48K) (HB)\0", NULL, "Jose Manuel Gris", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecColonoszxRomInfo, SpecColonoszxRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecQAOPMDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Colonos III - Part 1 (English) (128K) (HB)

static struct BurnRomInfo SpecColonos3e1RomDesc[] = {
	{ "Colonos III Part1 EN 128K (2022)(Jose Manuel Gris).tap", 40239, 0xd479a589, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecColonos3e1, SpecColonos3e1, Spec128)
STD_ROM_FN(SpecColonos3e1)

struct BurnDriver BurnSpecColonos3e1 = {
	"spec_colonos3e1", NULL, "spec_spec128", NULL, "2022",
	"Colonos III - Part 1 (English) (128K) (HB)\0", NULL, "Jose Manuel Gris", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_ACTION | GBF_ADV, 0,
	SpectrumGetZipName, SpecColonos3e1RomInfo, SpecColonos3e1RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Colonos III - Part 2 (English) (128K) (HB)

static struct BurnRomInfo SpecColonos3e2RomDesc[] = {
	{ "Colonos III Part2 EN 128K (2022)(Jose Manuel Gris).tap", 37801, 0x9d287b2d, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecColonos3e2, SpecColonos3e2, Spec128)
STD_ROM_FN(SpecColonos3e2)

struct BurnDriver BurnSpecColonos3e2 = {
	"spec_colonos3e2", "spec_colonos3e1", "spec_spec128", NULL, "2022",
	"Colonos III - Part 2 (English) (128K) (HB)\0", NULL, "Jose Manuel Gris", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PLATFORM | GBF_ADV, 0,
	SpectrumGetZipName, SpecColonos3e2RomInfo, SpecColonos3e2RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Colonos III - Part 3 (English) (128K) (HB)

static struct BurnRomInfo SpecColonos3e3RomDesc[] = {
	{ "Colonos III Part3 EN 128K (2022)(Jose Manuel Gris).tap", 37320, 0x5fec4b64, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecColonos3e3, SpecColonos3e3, Spec128)
STD_ROM_FN(SpecColonos3e3)

struct BurnDriver BurnSpecColonos3e3 = {
	"spec_colonos3e3", "spec_colonos3e1", "spec_spec128", NULL, "2022",
	"Colonos III - Part 3 (English) (128K) (HB)\0", NULL, "Jose Manuel Gris", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PLATFORM | GBF_ADV, 0,
	SpectrumGetZipName, SpecColonos3e3RomInfo, SpecColonos3e3RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Colonos III - Parte 1 (Spanish) (128K) (HB)

static struct BurnRomInfo SpecColonos3s1RomDesc[] = {
	{ "Colonos III Parte1 ES 128K (2022)(Jose Manuel Gris).tap", 40344, 0x9faec737, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecColonos3s1, SpecColonos3s1, Spec128)
STD_ROM_FN(SpecColonos3s1)

struct BurnDriver BurnSpecColonos3s1 = {
	"spec_colonos3s1", NULL, "spec_spec128", NULL, "2022",
	"Colonos III - Parte 1 (Spanish) (128K) (HB)\0", NULL, "Jose Manuel Gris", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_ACTION | GBF_ADV, 0,
	SpectrumGetZipName, SpecColonos3s1RomInfo, SpecColonos3s1RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Colonos III - Parte 2 (Spanish) (128K) (HB)

static struct BurnRomInfo SpecColonos3s2RomDesc[] = {
	{ "Colonos III Parte2 ES 128K (2022)(Jose Manuel Gris).tap", 37808, 0x3f4195f8, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecColonos3s2, SpecColonos3s2, Spec128)
STD_ROM_FN(SpecColonos3s2)

struct BurnDriver BurnSpecColonos3s2 = {
	"spec_colonos3s2", "spec_colonos3s1", "spec_spec128", NULL, "2022",
	"Colonos III - Parte 2 (Spanish) (128K) (HB)\0", NULL, "Jose Manuel Gris", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PLATFORM | GBF_ADV, 0,
	SpectrumGetZipName, SpecColonos3s2RomInfo, SpecColonos3s2RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Colonos III - Parte 3 (Spanish) (128K) (HB)

static struct BurnRomInfo SpecColonos3s3RomDesc[] = {
	{ "Colonos III Parte3 ES 128K (2022)(Jose Manuel Gris).tap", 37325, 0x9f90edc5, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecColonos3s3, SpecColonos3s3, Spec128)
STD_ROM_FN(SpecColonos3s3)

struct BurnDriver BurnSpecColonos3s3 = {
	"spec_colonos3s3", "spec_colonos3s1", "spec_spec128", NULL, "2022",
	"Colonos III - Parte 3 (Spanish) (128K) (HB)\0", NULL, "Jose Manuel Gris", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PLATFORM | GBF_ADV, 0,
	SpectrumGetZipName, SpecColonos3s3RomInfo, SpecColonos3s3RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Comeme el Chip (128K) (HB)

static struct BurnRomInfo SpecComechipRomDesc[] = {
	{ "Comeme el Chip 128K (2019)(Beyker Soft).tap", 43332, 0x1ef8fe2b, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecComechip, SpecComechip, Spec128)
STD_ROM_FN(SpecComechip)

struct BurnDriver BurnSpecComechip = {
	"spec_comechip", NULL, "spec_spec128", NULL, "2019",
	"Comeme el Chip (128K) (HB)\0", NULL, "Beyker Soft", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PUZZLE, 0,
	SpectrumGetZipName, SpecComechipRomInfo, SpecComechipRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecQAOPSpaceDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Corona Encantada, La (Spanish) (128K) (HB)

static struct BurnRomInfo SpecCoronaencRomDesc[] = {
	{ "Corona Encantada, La 128K (2009)(RELEVO).tap", 47371, 0xe8b765aa, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecCoronaenc, SpecCoronaenc, Spec128)
STD_ROM_FN(SpecCoronaenc)

struct BurnDriver BurnSpecCoronaenc = {
	"spec_coronaenc", NULL, "spec_spec128", NULL, "2009",
	"Corona Encantada, La (Spanish) (128K) (HB)\0", NULL, "Karoshi Corp. - RELEVO", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PLATFORM | GBF_ADV, 0,
	SpectrumGetZipName, SpecCoronaencRomInfo, SpecCoronaencRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Cosmic Lander (48K) (HB)

static struct BurnRomInfo SpecCosmiclanRomDesc[] = {
	{ "Cosmic Lander 48K (2023)(RetroTeam).tap", 31318, 0x959db9b6, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecCosmiclan, SpecCosmiclan, Spectrum)
STD_ROM_FN(SpecCosmiclan)

struct BurnDriver BurnSpecCosmiclan = {
	"spec_cosmiclan", NULL, "spec_spectrum", NULL, "2023",
	"Cosmic Lander (48K) (HB)\0", NULL, "RetroTeam", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_ACTION, 0,
	SpectrumGetZipName, SpecCosmiclanRomInfo, SpecCosmiclanRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Cosmic Space Attack (48K) (HB)

static struct BurnRomInfo SpecCsaRomDesc[] = {
	{ "Cosmic Space Attack (2021)(Vintage Software).z80", 26629, 0x90ba6366, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecCsa, SpecCsa, Spectrum)
STD_ROM_FN(SpecCsa)

struct BurnDriver BurnSpecCsa = {
	"spec_csa", NULL, "spec_spectrum", NULL, "2021",
	"Cosmic Space Attack (48K) (HB)\0", NULL, "Vintage Software Systems", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 2, HARDWARE_SPECTRUM, GBF_VERSHOOT, 0,
	SpectrumGetZipName, SpecCsaRomInfo, SpecCsaRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Cousin Horace - Intro (48K) (HB)

static struct BurnRomInfo SpecChorace0RomDesc[] = {
	{ "Cousin Horace - Intro (2014)(Alessandro Grussu).tap", 10958, 0x9d6cab93, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecChorace0, SpecChorace0, Spectrum)
STD_ROM_FN(SpecChorace0)

struct BurnDriver BurnSpecChorace0 = {
	"spec_chorace0", NULL, "spec_spectrum", NULL, "2014",
	"Cousin Horace - Intro (48K) (HB)\0", NULL, "Alessandro Grussu", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_MISC, 0,
	SpectrumGetZipName, SpecChorace0RomInfo, SpecChorace0RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Cousin Horace - Chapter 1 (48K) (HB)

static struct BurnRomInfo SpecChorace1RomDesc[] = {
	{ "Cousin Horace - Chapter1 (2014)(Alessandro Grussu).tap", 20522, 0xabfe7bec, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecChorace1, SpecChorace1, Spectrum)
STD_ROM_FN(SpecChorace1)

struct BurnDriver BurnSpecChorace1 = {
	"spec_chorace1", "spec_chorace0", "spec_spectrum", NULL, "2014",
	"Cousin Horace - Chapter 1 (48K) (HB)\0", NULL, "Alessandro Grussu", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecChorace1RomInfo, SpecChorace1RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Cousin Horace - Chapter 2 (48K) (HB)

static struct BurnRomInfo SpecChorace2RomDesc[] = {
	{ "Cousin Horace - Chapter2 (2014)(Alessandro Grussu).tap", 18707, 0x82267f74, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecChorace2, SpecChorace2, Spectrum)
STD_ROM_FN(SpecChorace2)

struct BurnDriver BurnSpecChorace2 = {
	"spec_chorace2", "spec_chorace0", "spec_spectrum", NULL, "2014",
	"Cousin Horace - Chapter 2 (48K) (HB)\0", "Access Code: 54590173", "Alessandro Grussu", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_HORSHOOT, 0,
	SpectrumGetZipName, SpecChorace2RomInfo, SpecChorace2RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecIntf2DIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Cousin Horace - Chapter 3 (48K) (HB)

static struct BurnRomInfo SpecChorace3RomDesc[] = {
	{ "Cousin Horace - Chapter3 (2014)(Alessandro Grussu).tap", 29438, 0xa8c9d4c2, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecChorace3, SpecChorace3, Spectrum)
STD_ROM_FN(SpecChorace3)

struct BurnDriver BurnSpecChorace3 = {
	"spec_chorace3", "spec_chorace0", "spec_spectrum", NULL, "2014",
	"Cousin Horace - Chapter 3 (48K) (HB)\0", "Access Code: 65731498", "Alessandro Grussu", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_ADV, 0,
	SpectrumGetZipName, SpecChorace3RomInfo, SpecChorace3RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Cousin Horace - Chapter 4 (48K) (HB)

static struct BurnRomInfo SpecChorace4RomDesc[] = {
	{ "Cousin Horace - Chapter4 (2014)(Alessandro Grussu).tap", 22794, 0x029c0c5f, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecChorace4, SpecChorace4, Spectrum)
STD_ROM_FN(SpecChorace4)

struct BurnDriver BurnSpecChorace4 = {
	"spec_chorace4", "spec_chorace0", "spec_spectrum", NULL, "2014",
	"Cousin Horace - Chapter 4 (48K) (HB)\0", "Access Code: 29402763", "Alessandro Grussu", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_MAZE, 0,
	SpectrumGetZipName, SpecChorace4RomInfo, SpecChorace4RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Cousin Horace - Chapter 5 (48K) (HB)

static struct BurnRomInfo SpecChorace5RomDesc[] = {
	{ "Cousin Horace - Chapter5 (2014)(Alessandro Grussu).tap", 24521, 0xf4e700c0, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecChorace5, SpecChorace5, Spectrum)
STD_ROM_FN(SpecChorace5)

struct BurnDriver BurnSpecChorace5 = {
	"spec_chorace5", "spec_chorace0", "spec_spectrum", NULL, "2014",
	"Cousin Horace - Chapter 5 (48K) (HB)\0", "Access Code: 04758812", "Alessandro Grussu", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PUZZLE, 0,
	SpectrumGetZipName, SpecChorace5RomInfo, SpecChorace5RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Crazy Blaster (128K) (HB)

static struct BurnRomInfo SpeccblasterRomDesc[] = {
	{ "Crazy Blaster (2019)(TFW8b).tap", 32951, 0x6797d22c, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Speccblaster, Speccblaster, Spec128)
STD_ROM_FN(Speccblaster)

struct BurnDriver BurnSpeccblaster = {
	"spec_cblaster", NULL, "spec_spec128", NULL, "2019",
	"Crazy Blaster (128K) (HB)\0", NULL, "The Future Was 8bit", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_ACTION, 0,
	SpectrumGetZipName, SpeccblasterRomInfo, SpeccblasterRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Crazy Christmas (48K) (HB)

static struct BurnRomInfo SpecCrazychristmasRomDesc[] = {
	{ "Crazy Christmas 48K-AY v1.0 (2023)(Fransouls).tap", 46648, 0x647cff0e, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecCrazychristmas, SpecCrazychristmas, Spectrum)
STD_ROM_FN(SpecCrazychristmas)

struct BurnDriver BurnSpecCrazychristmas = {
	"spec_crazychristmas", NULL, "spec_spectrum", NULL, "2023",
	"Crazy Christmas (48K) (HB)\0", "AY Sound supported - Set '3.Sinclair' to use controller", "Fransouls", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecCrazychristmasRomInfo, SpecCrazychristmasRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecIntf2DIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Crazy Forest (128K) (HB)

static struct BurnRomInfo SpecCrazyforestRomDesc[] = {
	{ "Crazy Forest 128K (2023)(Daniel Isoba).tap", 57178, 0x52a1514c, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecCrazyforest, SpecCrazyforest, Spec128)
STD_ROM_FN(SpecCrazyforest)

struct BurnDriver BurnSpecCrazyforest = {
	"spec_crazyforest", NULL, "spec_spec128", NULL, "2023",
	"Crazy Forest (128K) (HB)\0", NULL, "Daniel Isoba", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecCrazyforestRomInfo, SpecCrazyforestRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Cray-5 (128K) (HB)

static struct BurnRomInfo SpecCray5RomDesc[] = {
	{ "Cray-5 128K (2011)(RetroWorks).tap", 53875, 0xa1467bf5, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecCray5, SpecCray5, Spec128)
STD_ROM_FN(SpecCray5)

struct BurnDriver BurnSpecCray5 = {
	"spec_cray5", NULL, "spec_spec128", NULL, "2011",
	"Cray-5 (128K) (HB)\0", NULL, "RetroWorks", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_RUNGUN, 0,
	SpectrumGetZipName, SpecCray5RomInfo, SpecCray5RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Crimenes Ilustrados - La Escena (48K) (HB)

static struct BurnRomInfo SpecCrimesaRomDesc[] = {
	{ "Crimenes Ilustrados - La Escena 48K (2022)(BasicGames).tap", 38892, 0xa46bf600, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecCrimesa, SpecCrimesa, Spectrum)
STD_ROM_FN(SpecCrimesa)

struct BurnDriver BurnSpecCrimesa = {
	"spec_crimesa", NULL, "spec_spectrum", NULL, "2022",
	"Crimenes Ilustrados - La Escena (48K) (HB)\0", NULL, "BasicGames", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_ADV, 0,
	SpectrumGetZipName, SpecCrimesaRomInfo, SpecCrimesaRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Crimenes Ilustrados - La Solucion (48K) (HB)

static struct BurnRomInfo SpecCrimesbRomDesc[] = {
	{ "Crimenes Ilustrados - La Solucion 48K (2022)(BasicGames).tap", 48147, 0x8d86b600, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecCrimesb, SpecCrimesb, Spectrum)
STD_ROM_FN(SpecCrimesb)

struct BurnDriver BurnSpecCrimesb = {
	"spec_crimesb", "spec_crimesa", "spec_spectrum", NULL, "2022",
	"Crimenes Ilustrados - La Solucion (48K) (HB)\0", NULL, "BasicGames", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_ADV, 0,
	SpectrumGetZipName, SpecCrimesbRomInfo, SpecCrimesbRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Cronopios Y Famas (German) (128K) (HB)

static struct BurnRomInfo SpecCronopiosdeRomDesc[] = {
	{ "Cronopios Y Famas DE (2021)(Zankle Soft).tap", 42169, 0x91be72a8, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecCronopiosde, SpecCronopiosde, Spec128)
STD_ROM_FN(SpecCronopiosde)

struct BurnDriver BurnSpecCronopiosde = {
	"spec_cronopiosde", "spec_cronopiosen", "spec_spec128", NULL, "2021",
	"Cronopios Y Famas (German) (128K) (HB)\0", NULL, "Zankle Soft", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_ADV, 0,
	SpectrumGetZipName, SpecCronopiosdeRomInfo, SpecCronopiosdeRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Cronopios Y Famas (English) (128K) (HB)

static struct BurnRomInfo SpecCronopiosenRomDesc[] = {
	{ "Cronopios Y Famas EN (2021)(Zankle Soft).tap", 42169, 0xc64db30e, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecCronopiosen, SpecCronopiosen, Spec128)
STD_ROM_FN(SpecCronopiosen)

struct BurnDriver BurnSpecCronopiosen = {
	"spec_cronopiosen", NULL, "spec_spec128", NULL, "2021",
	"Cronopios Y Famas (English) (128K) (HB)\0", NULL, "Zankle Soft", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_ADV, 0,
	SpectrumGetZipName, SpecCronopiosenRomInfo, SpecCronopiosenRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Cronopios Y Famas (Spanish) (128K) (HB)

static struct BurnRomInfo SpecCronopiosesRomDesc[] = {
	{ "Cronopios Y Famas ES (2021)(Zankle Soft).tap", 42169, 0x306e3003, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecCronopioses, SpecCronopioses, Spec128)
STD_ROM_FN(SpecCronopioses)

struct BurnDriver BurnSpecCronopioses = {
	"spec_cronopioses", "spec_cronopiosen", "spec_spec128", NULL, "2021",
	"Cronopios Y Famas (Spanish) (128K) (HB)\0", NULL, "Zankle Soft", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_ADV, 0,
	SpectrumGetZipName, SpecCronopiosesRomInfo, SpecCronopiosesRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Cronopios Y Famas (French) (128K) (HB)

static struct BurnRomInfo SpecCronopiosfrRomDesc[] = {
	{ "Cronopios Y Famas FR (2021)(Zankle Soft).tap", 42169, 0x61fb2ea9, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecCronopiosfr, SpecCronopiosfr, Spec128)
STD_ROM_FN(SpecCronopiosfr)

struct BurnDriver BurnSpecCronopiosfr = {
	"spec_cronopiosfr", "spec_cronopiosen", "spec_spec128", NULL, "2021",
	"Cronopios Y Famas (French) (128K) (HB)\0", NULL, "Zankle Soft", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_ADV, 0,
	SpectrumGetZipName, SpecCronopiosfrRomInfo, SpecCronopiosfrRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Cronopios Y Famas (Italian) (128K) (HB)

static struct BurnRomInfo SpecCronopiositRomDesc[] = {
	{ "Cronopios Y Famas IT (2021)(Zankle Soft).tap", 42169, 0xa28f595b, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecCronopiosit, SpecCronopiosit, Spec128)
STD_ROM_FN(SpecCronopiosit)

struct BurnDriver BurnSpecCronopiosit = {
	"spec_cronopiosit", "spec_cronopiosen", "spec_spec128", NULL, "2021",
	"Cronopios Y Famas (Italian) (128K) (HB)\0", NULL, "Zankle Soft", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_ADV, 0,
	SpectrumGetZipName, SpecCronopiositRomInfo, SpecCronopiositRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Cronopios Y Famas (Portuguese) (128K) (HB)

static struct BurnRomInfo SpecCronopiosptRomDesc[] = {
	{ "Cronopios Y Famas PT (2021)(Zankle Soft).tap", 42169, 0x11902f8d, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecCronopiospt, SpecCronopiospt, Spec128)
STD_ROM_FN(SpecCronopiospt)

struct BurnDriver BurnSpecCronopiospt = {
	"spec_cronopiospt", "spec_cronopiosen", "spec_spec128", NULL, "2021",
	"Cronopios Y Famas (Portuguese) (128K) (HB)\0", NULL, "Zankle Soft", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_ADV, 0,
	SpectrumGetZipName, SpecCronopiosptRomInfo, SpecCronopiosptRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Cubic Experience, The (128K) (HB, v1.0)

static struct BurnRomInfo SpecCubicexpRomDesc[] = {
	{ "Cubic Experience, The v1.0 (2022)(Toku Soft).tap", 37903, 0x366b62f0, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecCubicexp, SpecCubicexp, Spec128)
STD_ROM_FN(SpecCubicexp)

struct BurnDriver BurnSpecCubicexp = {
	"spec_cubicexp", NULL, "spec_spec128", NULL, "2022",
	"Cubic Experience, The (128K) (HB, v1.0)\0", "Press '2' to use controller", "Toku Soft", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_ACTION | GBF_PUZZLE, 0,
	SpectrumGetZipName, SpecCubicexpRomInfo, SpecCubicexpRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Cubic Experience, The - V.E. (128K) (HB, v1.0)

static struct BurnRomInfo SpecCubicexpveRomDesc[] = {
	{ "Cubic Experience, The VE v1.0 (2022)(Toku Soft).tap", 25874, 0x02582bdf, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecCubicexpve, SpecCubicexpve, Spec128)
STD_ROM_FN(SpecCubicexpve)

struct BurnDriver BurnSpecCubicexpve = {
	"spec_cubicexpve", "spec_cubicexp", "spec_spec128", NULL, "2022",
	"Cubic Experience, The - V.E. (128K) (HB, v1.0)\0", "Visual Experince: the interactive tutorial of the game" , "Toku Soft", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_ACTION | GBF_PUZZLE, 0,
	SpectrumGetZipName, SpecCubicexpveRomInfo, SpecCubicexpveRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Cult, The (English) (128K) (HB)

static struct BurnRomInfo SpecCulteRomDesc[] = {
	{ "Cult, The EN 128K (2023)(Tartessos Games).tap", 64177, 0xcea1ec75, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecCulte, SpecCulte, Spec128)
STD_ROM_FN(SpecCulte)

struct BurnDriver BurnSpecCulte = {
	"spec_culte", NULL, "spec_spec128", NULL, "2023",
	"Cult, The (English) (128K) (HB)\0", NULL, "Tartessos Games", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_RUNGUN | GBF_MAZE, 0,
	SpectrumGetZipName, SpecCulteRomInfo, SpecCulteRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Culto, El (Spanish) (128K) (HB)

static struct BurnRomInfo SpecCultsRomDesc[] = {
	{ "Culto, El ES 128K (2023)(Tartessos Games).tap", 64379, 0xc2fe8ae7, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecCults, SpecCults, Spec128)
STD_ROM_FN(SpecCults)

struct BurnDriver BurnSpecCults = {
	"spec_cults", "spec_culte", "spec_spec128", NULL, "2023",
	"Culto, El (Spanish) (128K) (HB)\0", NULL, "Tartessos Games", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_RUNGUN | GBF_MAZE, 0,
	SpectrumGetZipName, SpecCultsRomInfo, SpecCultsRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Cursed Castle (48K) (HB)

static struct BurnRomInfo SpecCursedcastleRomDesc[] = {
	{ "Cursed Castle 48K (2023)(Fransouls).tap", 47687, 0x15819503, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecCursedcastle, SpecCursedcastle, Spectrum)
STD_ROM_FN(SpecCursedcastle)

struct BurnDriver BurnSpecCursedcastle = {
	"spec_cursedcastle", NULL, "spec_spectrum", NULL, "2023",
	"Cursed Castle (48K) (HB)\0", "Set '3.Sinclair' to use controller", "Fransouls", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecCursedcastleRomInfo, SpecCursedcastleRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecIntf2DIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Cursed Castle 2 - Part 1 (English) (128K) (HB, v1.1)

static struct BurnRomInfo SpecCursedca2p1enRomDesc[] = {
	{ "Cursed Castle 2 - Part 1 v1.1 EN 128K (2024)(Fransouls).tap", 110644, 0x4d5ad0a4, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecCursedca2p1en, SpecCursedca2p1en, Spec128)
STD_ROM_FN(SpecCursedca2p1en)

struct BurnDriver BurnSpecCursedca2p1en = {
	"spec_cursedca2p1en", NULL, "spec_spec128", NULL, "2024",
	"Cursed Castle 2 - Part 1 (English) (128K) (HB, v1.1)\0", NULL, "Fransouls", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecCursedca2p1enRomInfo, SpecCursedca2p1enRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Cursed Castle 2 - Part 2 (English) (128K) (HB, v1.1)

static struct BurnRomInfo SpecCursedca2p2enRomDesc[] = {
	{ "Cursed Castle 2 - Part 2 v1.1 EN 128K (2024)(Fransouls).tap", 104803, 0x36244e1a, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecCursedca2p2en, SpecCursedca2p2en, Spec128)
STD_ROM_FN(SpecCursedca2p2en)

struct BurnDriver BurnSpecCursedca2p2en = {
	"spec_cursedca2p2en", "spec_cursedca2p1en", "spec_spec128", NULL, "2024",
	"Cursed Castle 2 - Part 2 (English) (128K) (HB, v1.1)\0", "Password: AXE777", "Fransouls", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecCursedca2p2enRomInfo, SpecCursedca2p2enRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Cursed Castle 2 - Part 1 (Spanish) (128K) (HB, v1.1)

static struct BurnRomInfo SpecCursedca2p1esRomDesc[] = {
	{ "Cursed Castle 2 - Part 1 v1.1 ES 128K (2024)(Fransouls).tap", 110708, 0x993ba4a1, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecCursedca2p1es, SpecCursedca2p1es, Spec128)
STD_ROM_FN(SpecCursedca2p1es)

struct BurnDriver BurnSpecCursedca2p1es = {
	"spec_cursedca2p1es", NULL, "spec_spec128", NULL, "2024",
	"Cursed Castle 2 - Part 1 (Spanish) (128K) (HB, v1.1)\0", NULL, "Fransouls", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecCursedca2p1esRomInfo, SpecCursedca2p1esRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Cursed Castle 2 - Part 2 (Spanish) (128K) (HB, v1.1)

static struct BurnRomInfo SpecCursedca2p2esRomDesc[] = {
	{ "Cursed Castle 2 - Part 2 v1.1 ES 128K (2024)(Fransouls).tap", 104809, 0xef7fb1ef, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecCursedca2p2es, SpecCursedca2p2es, Spec128)
STD_ROM_FN(SpecCursedca2p2es)

struct BurnDriver BurnSpecCursedca2p2es = {
	"spec_cursedca2p2es", "spec_cursedca2p1es", "spec_spec128", NULL, "2024",
	"Cursed Castle 2 - Part 2 (Spanish) (128K) (HB, v1.1)\0", "Password: AXE777", "Fransouls", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecCursedca2p2esRomInfo, SpecCursedca2p2esRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Cursed Castle DX - Archer (128K) (HB)

static struct BurnRomInfo SpecCursedcastdxaRomDesc[] = {
	{ "Cursed Castle DX - Archer 128K (2024)(Fransouls).tap", 52302, 0x1601fcd2, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecCursedcastdxa, SpecCursedcastdxa, Spec128)
STD_ROM_FN(SpecCursedcastdxa)

struct BurnDriver BurnSpecCursedcastdxa = {
	"spec_cursedcastdxa", NULL, "spec_spec128", NULL, "2024",
	"Cursed Castle DX - Archer (128K) (HB)\0", NULL, "Fransouls", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecCursedcastdxaRomInfo, SpecCursedcastdxaRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Cursed Castle DX - Black & White (128K) (HB)

static struct BurnRomInfo SpecCursedcastdxbwRomDesc[] = {
	{ "Cursed Castle DX - Black & White 128K (2024)(Fransouls).tap", 52288, 0x10d6936e, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecCursedcastdxbw, SpecCursedcastdxbw, Spec128)
STD_ROM_FN(SpecCursedcastdxbw)

struct BurnDriver BurnSpecCursedcastdxbw = {
	"spec_cursedcastdxbw", "spec_cursedcastdxa", "spec_spec128", NULL, "2024",
	"Cursed Castle DX - Black & White (128K) (HB)\0", NULL, "Fransouls", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecCursedcastdxbwRomInfo, SpecCursedcastdxbwRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Cursed Castle DX - Executer (128K) (HB)

static struct BurnRomInfo SpecCursedcastdxeRomDesc[] = {
	{ "Cursed Castle DX - Executer 128K (2024)(Fransouls).tap", 52440, 0xf03a5b6c, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecCursedcastdxe, SpecCursedcastdxe, Spec128)
STD_ROM_FN(SpecCursedcastdxe)

struct BurnDriver BurnSpecCursedcastdxe = {
	"spec_cursedcastdxe", "spec_cursedcastdxa", "spec_spec128", NULL, "2024",
	"Cursed Castle DX - Executer (128K) (HB)\0", NULL, "Fransouls", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecCursedcastdxeRomInfo, SpecCursedcastdxeRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Cursed Castle DX - Maaad (128K) (HB)

static struct BurnRomInfo SpecCursedcastdxmRomDesc[] = {
	{ "Cursed Castle DX - Maaad 128K (2024)(Fransouls).tap", 52302, 0xd5630cec, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecCursedcastdxm, SpecCursedcastdxm, Spec128)
STD_ROM_FN(SpecCursedcastdxm)

struct BurnDriver BurnSpecCursedcastdxm = {
	"spec_cursedcastdxm", "spec_cursedcastdxa", "spec_spec128", NULL, "2024",
	"Cursed Castle DX - Maaad (128K) (HB)\0", NULL, "Fransouls", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecCursedcastdxmRomInfo, SpecCursedcastdxmRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Cursed Castle DX - Skeleton (128K) (HB)

static struct BurnRomInfo SpecCursedcastdxsRomDesc[] = {
	{ "Cursed Castle DX - Skeleton 128K (2024)(Fransouls).tap", 52440, 0x888645ca, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecCursedcastdxs, SpecCursedcastdxs, Spec128)
STD_ROM_FN(SpecCursedcastdxs)

struct BurnDriver BurnSpecCursedcastdxs = {
	"spec_cursedcastdxs", "spec_cursedcastdxa", "spec_spec128", NULL, "2024",
	"Cursed Castle DX - Skeleton (128K) (HB)\0", NULL, "Fransouls", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecCursedcastdxsRomInfo, SpecCursedcastdxsRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Cursed Demons of Wallachia (English) (48K) (HB)

static struct BurnRomInfo SpecCursedemoneRomDesc[] = {
	{ "Cursed Demons Of Wallachia EN 48K (2023)(Furillo Productions).tap", 44991, 0xb03297f7, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecCursedemone, SpecCursedemone, Spectrum)
STD_ROM_FN(SpecCursedemone)

struct BurnDriver BurnSpecCursedemone = {
	"spec_cursedemone", NULL, "spec_spectrum", NULL, "2023",
	"Cursed Demons of Wallachia (English) (48K) (HB)\0", NULL, "Furillo Productions", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecCursedemoneRomInfo, SpecCursedemoneRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Cursed Demons of Wallachia (Portuguese) (48K) (HB)

static struct BurnRomInfo SpecCursedemonpRomDesc[] = {
	{ "Cursed Demons Of Wallachia PT 48K (2023)(Furillo Productions).tap", 44986, 0x7dc4605c, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecCursedemonp, SpecCursedemonp, Spectrum)
STD_ROM_FN(SpecCursedemonp)

struct BurnDriver BurnSpecCursedemonp = {
	"spec_cursedemonp", "spec_cursedemone", "spec_spectrum", NULL, "2023",
	"Cursed Demons of Wallachia (Portuguese) (48K) (HB)\0", NULL, "Furillo Productions", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecCursedemonpRomInfo, SpecCursedemonpRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Cursed Demons of Wallachia (Spanish) (48K) (HB)

static struct BurnRomInfo SpecCursedemonsRomDesc[] = {
	{ "Cursed Demons Of Wallachia ES 48K (2023)(Furillo Productions).tap", 45019, 0xa17dfc94, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecCursedemons, SpecCursedemons, Spectrum)
STD_ROM_FN(SpecCursedemons)

struct BurnDriver BurnSpecCursedemons = {
	"spec_cursedemons", "spec_cursedemone", "spec_spectrum", NULL, "2023",
	"Cursed Demons of Wallachia (Spanish) (48K) (HB)\0", NULL, "Furillo Productions", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecCursedemonsRomInfo, SpecCursedemonsRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Cybermania (48K) (HB)

static struct BurnRomInfo SpecCybermaniaRomDesc[] = {
	{ "Cybermania 48K (2016)(Payndz).tap", 29277, 0xcae44982, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecCybermania, SpecCybermania, Spectrum)
STD_ROM_FN(SpecCybermania)

struct BurnDriver BurnSpecCybermania = {
	"spec_cybermania", NULL, "spec_spectrum", NULL, "2016",
	"Cybermania (48K) (HB)\0", NULL, "Payndz", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_MAZE | GBF_ACTION, 0,
	SpectrumGetZipName, SpecCybermaniaRomInfo, SpecCybermaniaRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecQAOPSpaceDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Cyclus - Part 1 (128K) (HB)

static struct BurnRomInfo SpecCyclus1RomDesc[] = {
	{ "Cyclus Part 1 (2021)(Miguetelo).tap", 44736, 0x36b8fe17, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecCyclus1, SpecCyclus1, Spec128)
STD_ROM_FN(SpecCyclus1)

struct BurnDriver BurnSpecCyclus1 = {
	"spec_cyclus1", NULL, "spec_spec128", NULL, "2021",
	"Cyclus - Part 1 (128K) (HB)\0", NULL, "Miguetelo", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PUZZLE, 0,
	SpectrumGetZipName, SpecCyclus1RomInfo, SpecCyclus1RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Cyclus - Part 2 (128K) (HB)

static struct BurnRomInfo SpecCyclus2RomDesc[] = {
	{ "Cyclus Part 2 (2021)(Miguetelo).tap", 44007, 0x534ecbe7, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecCyclus2, SpecCyclus2, Spec128)
STD_ROM_FN(SpecCyclus2)

struct BurnDriver BurnSpecCyclus2 = {
	"spec_cyclus2", "spec_cyclus1", "spec_spec128", NULL, "2021",
	"Cyclus - Part 2 (128K) (HB)\0", NULL, "Miguetelo", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_ACTION, 0,
	SpectrumGetZipName, SpecCyclus2RomInfo, SpecCyclus2RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Danterrifik (128K) (HB, v1.1)

static struct BurnRomInfo SpecdanterrifikRomDesc[] = {
	{ "Danterrifik v1.1 128K (2020)(Toku Soft).tap", 49067, 0x6a088600, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specdanterrifik, Specdanterrifik, Spec128)
STD_ROM_FN(Specdanterrifik)

struct BurnDriver BurnSpecdanterrifik = {
	"spec_danterrifik", NULL, "spec_spec128", NULL, "2020",
	"Danterrifik (128K) (HB, v1.1)\0", NULL, "Toku Soft", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecdanterrifikRomInfo, SpecdanterrifikRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Danterrifik II - The Nightmare Continues (128K) (HB)

static struct BurnRomInfo Specdanterrifik2RomDesc[] = {
	{ "Danterrifik 2 128K (2020)(Toku Soft).tap", 49982, 0x478c7adf, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specdanterrifik2, Specdanterrifik2, Spec128)
STD_ROM_FN(Specdanterrifik2)

struct BurnDriver BurnSpecdanterrifik2 = {
	"spec_danterrifik2", NULL, "spec_spec128", NULL, "2020",
	"Danterrifik II - The Nightmare Continues (128K) (HB)\0", NULL, "Toku Soft", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, Specdanterrifik2RomInfo, Specdanterrifik2RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Danterrifik III - The Dark Age of Grotesque (128K) (HB)

static struct BurnRomInfo Specdanterrifik3RomDesc[] = {
	{ "Danterrifik 3 128K (2021)(TokuSoft).tap", 46898, 0x9041c4da, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specdanterrifik3, Specdanterrifik3, Spec128)
STD_ROM_FN(Specdanterrifik3)

struct BurnDriver BurnSpecdanterrifik3 = {
	"spec_danterrifik3", NULL, "spec_spec128", NULL, "2021",
	"Danterrifik III - The Dark Age of Grotesque (128K) (HB)\0", NULL, "Toku Soft", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, Specdanterrifik3RomInfo, Specdanterrifik3RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Danterrifik IV - A New Beginning (128K)

static struct BurnRomInfo SpecDanterrifik4RomDesc[] = {
	{ "Danterrifik IV A New Beginning 128K (2023)(Tokusoft).tap", 36977, 0x33a71a90, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecDanterrifik4, SpecDanterrifik4, Spec128)
STD_ROM_FN(SpecDanterrifik4)

struct BurnDriver BurnSpecDanterrifik4 = {
	"spec_danterrifik4", NULL, "spec_spec128", NULL, "2023",
	"Danterrifik IV - A New Beginning (128K) (HB)\0", NULL, "Toku Soft", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecDanterrifik4RomInfo, SpecDanterrifik4RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Dark, The - Lost Pages (128K) (HB)

static struct BurnRomInfo SpecDarklostpagesRomDesc[] = {
	{ "Dark, The - Lost Pages 128K (2021)(Zosya Entertainment).tap", 55037, 0x4ee940ee, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecDarklostpages, SpecDarklostpages, Spec128)
STD_ROM_FN(SpecDarklostpages)

struct BurnDriver BurnSpecDarklostpages = {
	"spec_darklostpages", NULL, "spec_spec128", NULL, "2021",
	"Dark, The - Lost Pages (128K) (HB)\0", NULL, "Zosya Entertainment", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_MAZE | GBF_RUNGUN, 0,
	SpectrumGetZipName, SpecDarklostpagesRomInfo, SpecDarklostpagesRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Dark Castle (128K) (HB)

static struct BurnRomInfo SpecDarkcastleRomDesc[] = {
	{ "Dark Castle 128K (2016)(kas29).tap", 24948, 0x828a865c, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecDarkcastle, SpecDarkcastle, Spec128)
STD_ROM_FN(SpecDarkcastle)

struct BurnDriver BurnSpecDarkcastle = {
	"spec_darkcastle", NULL, "spec_spec128", NULL, "2016",
	"Dark Castle (128K) (HB)\0", NULL, "kas29", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_MAZE, 0,
	SpectrumGetZipName, SpecDarkcastleRomInfo, SpecDarkcastleRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Dark Knight, The (English) (128K) (HB)

static struct BurnRomInfo SpecDarknighteRomDesc[] = {
	{ "Dark Knight, The EN 128K (2024)(Mananuk).tap", 64353, 0xbc00c3e7, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecDarknighte, SpecDarknighte, Spec128)
STD_ROM_FN(SpecDarknighte)

struct BurnDriver BurnSpecDarknighte = {
	"spec_darknighte", NULL, "spec_spec128", NULL, "2024",
	"Dark Knight, The (English) (128K) (HB)\0", NULL, "Mananuk", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecDarknighteRomInfo, SpecDarknighteRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Dark Knight, The (Spanish) (128K) (HB)

static struct BurnRomInfo SpecDarknightsRomDesc[] = {
	{ "Dark Knight, The ES 128K (2024)(Mananuk).tap", 64229, 0x8db09e89, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecDarknights, SpecDarknights, Spec128)
STD_ROM_FN(SpecDarknights)

struct BurnDriver BurnSpecDarknights = {
	"spec_darknights", "spec_darknighte", "spec_spec128", NULL, "2024",
	"Dark Knight, The (Spanish) (128K) (HB)\0", NULL, "Mananuk", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecDarknightsRomInfo, SpecDarknightsRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Dark Veil, The (English) (48K) (HB)

static struct BurnRomInfo SpecDarkveileRomDesc[] = {
	{ "Dark Veil, The EN 48K (2023)(Furillo Productions).tap", 37894, 0x2087c681, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecDarkveile, SpecDarkveile, Spectrum)
STD_ROM_FN(SpecDarkveile)

struct BurnDriver BurnSpecDarkveile = {
	"spec_darkveile", NULL, "spec_spectrum", NULL, "2023",
	"Dark Veil, The (English) (48K) (HB)\0", NULL, "Furillo Productions", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_ADV, 0,
	SpectrumGetZipName, SpecDarkveileRomInfo, SpecDarkveileRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecIntf2DIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Velo Oscuro, El (Spanish) (48K) (HB)

static struct BurnRomInfo SpecDarkveilsRomDesc[] = {
	{ "Dark Veil, The ES 48K (2023)(Furillo Productions).tap", 39208, 0x76e0b33b, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecDarkveils, SpecDarkveils, Spectrum)
STD_ROM_FN(SpecDarkveils)

struct BurnDriver BurnSpecDarkveils = {
	"spec_darkveils", "spec_darkveile", "spec_spectrum", NULL, "2023",
	"Velo Oscuro, El (Spanish) (48K) (HB)\0", NULL, "Furillo Productions", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_ADV, 0,
	SpectrumGetZipName, SpecDarkveilsRomInfo, SpecDarkveilsRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecIntf2DIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Dave Infuriators (48K) (HB)

static struct BurnRomInfo SpecDaveinfsRomDesc[] = {
	{ "Dave Infuriators 48K (2012)(Black Jet).tap", 43043, 0x78ffb0c5, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecDaveinfs, SpecDaveinfs, Spectrum)
STD_ROM_FN(SpecDaveinfs)

struct BurnDriver BurnSpecDaveinfs = {
	"spec_daveinfs", NULL, "spec_spectrum", NULL, "2012",
	"Dave Infuriators (48K) (HB)\0", "M: reverse - SPACE: jump - Q: quit", "Black Jet", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecDaveinfsRomInfo, SpecDaveinfsRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Dead Flesh Boy (128K) (HB)

static struct BurnRomInfo SpecDfboyRomDesc[] = {
	{ "Dead Flesh Boy 128K (2015)(VANB).tap", 40669, 0x3375d2c9, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecDfboy, SpecDfboy, Spec128)
STD_ROM_FN(SpecDfboy)

struct BurnDriver BurnSpecDfboy = {
	"spec_dfboy", NULL, "spec_spec128", NULL, "2015",
	"Dead Flesh Boy (128K) (HB)\0", NULL, "VANB", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecDfboyRomInfo, SpecDfboyRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Dead Space (48K) (HB)

static struct BurnRomInfo SpecDeadspaceRomDesc[] = {
	{ "Dead Space 48K (2021)(Radastan).tap", 21695, 0xb7fb4c48, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecDeadspace, SpecDeadspace, Spectrum)
STD_ROM_FN(SpecDeadspace)

struct BurnDriver BurnSpecDeadspace = {
	"spec_deadspace", NULL, "spec_spectrum", NULL, "2021",
	"Dead Space (48K) (HB)\0", NULL, "Radastan", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_ACTION, 0,
	SpectrumGetZipName, SpecDeadspaceRomInfo, SpecDeadspaceRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecQAOPSpaceDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Dead Zone - Check Point version (English) (128K) (HB)

static struct BurnRomInfo SpecDzone1enRomDesc[] = {
	{ "Dead Zone - Check Point version EN 128K (2019)(PC NONO Games).tap", 45639, 0x34d62528, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecDzone1en, SpecDzone1en, Spec128)
STD_ROM_FN(SpecDzone1en)

struct BurnDriver BurnSpecDzone1en = {
	"spec_dzone1en", NULL, "spec_spec128", NULL, "2019",
	"Dead Zone - Check Point version (English) (128K) (HB)\0", NULL, "PC NONO Games", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_SHOOT, 0,
	SpectrumGetZipName, SpecDzone1enRomInfo, SpecDzone1enRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Dead Zone  - Punto de Control version (Spanish) (128K) (HB)

static struct BurnRomInfo SpecDzone1esRomDesc[] = {
	{ "Dead Zone  - Punto de Control version ES 128K (2019)(PC NONO Games).tap", 45754, 0xb06a5bff, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecDzone1es, SpecDzone1es, Spec128)
STD_ROM_FN(SpecDzone1es)

struct BurnDriver BurnSpecDzone1es = {
	"spec_dzone1es", NULL, "spec_spec128", NULL, "2019",
	"Dead Zone  - Punto de Control version (Spanish) (128K) (HB)\0", NULL, "PC NONO Games", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_SHOOT, 0,
	SpectrumGetZipName, SpecDzone1esRomInfo, SpecDzone1esRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Dead Zone - Survival version (English) (128K) (HB)

static struct BurnRomInfo SpecDzone2enRomDesc[] = {
	{ "Dead Zone - Survival version EN 128K (2019)(PC NONO Games).tap", 45564, 0xde6ef5a4, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecDzone2en, SpecDzone2en, Spec128)
STD_ROM_FN(SpecDzone2en)

struct BurnDriver BurnSpecDzone2en = {
	"spec_dzone2en", "spec_dzone1en", "spec_spec128", NULL, "2019",
	"Dead Zone - Survival version (English) (128K) (HB)\0", NULL, "PC NONO Games", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_SHOOT, 0,
	SpectrumGetZipName, SpecDzone2enRomInfo, SpecDzone2enRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Dead Zone  - Supervivencia version (Spanish) (128K) (HB)

static struct BurnRomInfo SpecDzone2esRomDesc[] = {
	{ "Dead Zone  - Supervivencia version ES 128K (2019)(PC NONO Games).tap", 45681, 0xaf486c59, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecDzone2es, SpecDzone2es, Spec128)
STD_ROM_FN(SpecDzone2es)

struct BurnDriver BurnSpecDzone2es = {
	"spec_dzone2es", "spec_dzone1es", "spec_spec128", NULL, "2019",
	"Dead Zone  - Supervivencia version (Spanish) (128K) (HB)\0", NULL, "PC NONO Games", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_SHOOT, 0,
	SpectrumGetZipName, SpecDzone2esRomInfo, SpecDzone2esRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Defender (48K) (HB)

static struct BurnRomInfo SpecDefenderRomDesc[] = {
	{ "Defender 48K (2024)(rth).tap", 19363, 0xc310b86e, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecDefender, SpecDefender, Spectrum)
STD_ROM_FN(SpecDefender)

struct BurnDriver BurnSpecDefender = {
	"spec_defender", NULL, "spec_spectrum", NULL, "2024",
	"Defender (48K) (HB)\0", NULL, "rth", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_SHOOT, 0,
	SpectrumGetZipName, SpecDefenderRomInfo, SpecDefenderRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecQAOPSpaceDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Deva (English) (48K) (HB)

static struct BurnRomInfo SpecDevaenRomDesc[] = {
	{ "Deva 48K EN (2022)(Furillo Productions).tap", 41902, 0x131b4d56, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecDevaen, SpecDevaen, Spectrum)
STD_ROM_FN(SpecDevaen)

struct BurnDriver BurnSpecDevaen = {
	"spec_devaen", NULL, "spec_spectrum", NULL, "2022",
	"Deva (English) (48K) (HB)\0", NULL, "Furillo Productions", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PUZZLE, 0,
	SpectrumGetZipName, SpecDevaenRomInfo, SpecDevaenRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Deva - Hard (English) (48K) (HB)

static struct BurnRomInfo SpecDevaenhRomDesc[] = {
	{ "Deva 48K EN - Hard (2022)(Furillo Productions).tap", 41861, 0x655039a6, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecDevaenh, SpecDevaenh, Spectrum)
STD_ROM_FN(SpecDevaenh)

struct BurnDriver BurnSpecDevaenh = {
	"spec_devaenh", "spec_devaen", "spec_spectrum", NULL, "2022",
	"Deva - Hard (English) (48K) (HB)\0", NULL, "Furillo Productions", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PUZZLE, 0,
	SpectrumGetZipName, SpecDevaenhRomInfo, SpecDevaenhRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Deva (Spanish) (48K) (HB)

static struct BurnRomInfo SpecDevaspRomDesc[] = {
	{ "Deva 48K SP (2022)(Furillo Productions).tap", 41565, 0xcdf42de4, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecDevasp, SpecDevasp, Spectrum)
STD_ROM_FN(SpecDevasp)

struct BurnDriver BurnSpecDevasp = {
	"spec_devasp", "spec_devaen", "spec_spectrum", NULL, "2022",
	"Deva (Spanish) (48K) (HB)\0", NULL, "Furillo Productions", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PUZZLE, 0,
	SpectrumGetZipName, SpecDevaspRomInfo, SpecDevaspRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Deva - Hard (Spanish) (48K) (HB)

static struct BurnRomInfo SpecDevasphRomDesc[] = {
	{ "Deva 48K SP - Hard (2022)(Furillo Productions).tap", 41524, 0x62b9fd11, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecDevasph, SpecDevasph, Spectrum)
STD_ROM_FN(SpecDevasph)

struct BurnDriver BurnSpecDevasph = {
	"spec_devasph", "spec_devaen", "spec_spectrum", NULL, "2022",
	"Deva - Hard (Spanish) (48K) (HB)\0", NULL, "Furillo Productions", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PUZZLE, 0,
	SpectrumGetZipName, SpecDevasphRomInfo, SpecDevasphRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Devil's Quest (Spanish) (128K) (HB)

static struct BurnRomInfo SpecDevquestRomDesc[] = {
	{ "Devil's Quest ES (2020)(OCTOCOM).tap", 116175, 0xbc9565af, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecDevquest, SpecDevquest, Spec128)
STD_ROM_FN(SpecDevquest)

struct BurnDriver BurnSpecDevquest = {
	"spec_devquest", NULL, "spec_spec128", NULL, "2020",
	"Devil's Quest (Spanish) (128K) (HB)\0", NULL, "OCTOCOM", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_MAZE | GBF_ACTION, 0,
	SpectrumGetZipName, SpecDevquestRomInfo, SpecDevquestRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Devwill Too ZX (English) (128K) (HB)

static struct BurnRomInfo SpecDwtoozxenRomDesc[] = {
	{ "Devwill Too ZX 128K EN (2020)(Amaweks).tap", 47275, 0xc7bde450, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecDwtoozxen, SpecDwtoozxen, Spec128)
STD_ROM_FN(SpecDwtoozxen)

struct BurnDriver BurnSpecDwtoozxen = {
	"spec_dwtoozxen", NULL, "spec_spec128", NULL, "2020",
	"Devwill Too ZX (English) (128K) (HB)\0", NULL, "Amaweks", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecDwtoozxenRomInfo, SpecDwtoozxenRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Devwill Too ZX (Spanish) (128K) (HB)

static struct BurnRomInfo SpecDwtoozxesRomDesc[] = {
	{ "Devwill Too ZX 128K ES (2020)(Amaweks).tap", 47275, 0xc3283ba1, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecDwtoozxes, SpecDwtoozxes, Spec128)
STD_ROM_FN(SpecDwtoozxes)

struct BurnDriver BurnSpecDwtoozxes = {
	"spec_dwtoozxes", "spec_dwtoozxen", "spec_spec128", NULL, "2020",
	"Devwill Too ZX (Spanish) (128K) (HB)\0", NULL, "Amaweks", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecDwtoozxesRomInfo, SpecDwtoozxesRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Devwill Too ZX (Portuguese) (128K) (HB)

static struct BurnRomInfo SpecDwtoozxptRomDesc[] = {
	{ "Devwill Too ZX 128K PT (2020)(Amaweks).tap", 47275, 0xd8b4d123, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecDwtoozxpt, SpecDwtoozxpt, Spec128)
STD_ROM_FN(SpecDwtoozxpt)

struct BurnDriver BurnSpecDwtoozxpt = {
	"spec_dwtoozxpt", "spec_dwtoozxen", "spec_spec128", NULL, "2020",
	"Devwill Too ZX (Portuguese) (128K) (HB)\0", NULL, "Amaweks", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecDwtoozxptRomInfo, SpecDwtoozxptRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Devwill Too ZX (English) (128K) (HB, v2.0)

static struct BurnRomInfo SpecDwtoozxenv2RomDesc[] = {
	{ "Devwill Too ZX v2.0 128K EN (2023)(Amaweks).tap", 45098, 0x809ce1b8, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecDwtoozxenv2, SpecDwtoozxenv2, Spec128)
STD_ROM_FN(SpecDwtoozxenv2)

struct BurnDriver BurnSpecDwtoozxenv2 = {
	"spec_dwtoozxenv2", NULL, "spec_spec128", NULL, "2023",
	"Devwill Too ZX (English) (128K) (HB, v2.0)\0", NULL, "Amaweks", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecDwtoozxenv2RomInfo, SpecDwtoozxenv2RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Devwill Too ZX (Spanish) (128K) (HB, v2.0)

static struct BurnRomInfo SpecDwtoozxesv2RomDesc[] = {
	{ "Devwill Too ZX v2.0 128K ES (2023)(Amaweks).tap", 45098, 0x545dbc02, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecDwtoozxesv2, SpecDwtoozxesv2, Spec128)
STD_ROM_FN(SpecDwtoozxesv2)

struct BurnDriver BurnSpecDwtoozxesv2 = {
	"spec_dwtoozxesv2", "spec_dwtoozxenv2", "spec_spec128", NULL, "2023",
	"Devwill Too ZX (Spanish) (128K) (HB, v2.0)\0", NULL, "Amaweks", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecDwtoozxesv2RomInfo, SpecDwtoozxesv2RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Devwill Too ZX (Portuguese) (128K) (HB, v2.0)

static struct BurnRomInfo SpecDwtoozxptv2RomDesc[] = {
	{ "Devwill Too ZX v2.0 128K PT (2023)(Amaweks).tap", 45098, 0xa0e1f9df, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecDwtoozxptv2, SpecDwtoozxptv2, Spec128)
STD_ROM_FN(SpecDwtoozxptv2)

struct BurnDriver BurnSpecDwtoozxptv2 = {
	"spec_dwtoozxptv2", "spec_dwtoozxenv2", "spec_spec128", NULL, "2023",
	"Devwill Too ZX (Portuguese) (128K) (HB, v2.0)\0", NULL, "Amaweks", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecDwtoozxptv2RomInfo, SpecDwtoozxptv2RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Diana Cave Adventure (128K) (HB)

static struct BurnRomInfo SpecDcadv128RomDesc[] = {
	{ "Diana Cave Adventure 128K (2023)(Oniric Factor).tap", 42314, 0xfbec71ec, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecDcadv128, SpecDcadv128, Spec128)
STD_ROM_FN(SpecDcadv128)

struct BurnDriver BurnSpecDcadv128 = {
	"spec_dcadv128", NULL, "spec_spec128", NULL, "2023",
	"Diana Cave Adventure (128K) (HB)\0", NULL, "Oniric Factor", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecDcadv128RomInfo, SpecDcadv128RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Diana Cave Adventure (48K) (HB)

static struct BurnRomInfo SpecDcadv48RomDesc[] = {
	{ "Diana Cave Adventure 48K (2023)(Oniric Factor).tap", 34957, 0x3bd013ca, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecDcadv48, SpecDcadv48, Spectrum)
STD_ROM_FN(SpecDcadv48)

struct BurnDriver BurnSpecDcadv48 = {
	"spec_dcadv48", "spec_dcadv128", "spec_spectrum", NULL, "2023",
	"Diana Cave Adventure (48K) (HB)\0", NULL, "Oniric Factor", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecDcadv48RomInfo, SpecDcadv48RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Dig Dug Doug (128K) (HB)

static struct BurnRomInfo SpecDigdugdougRomDesc[] = {
	{ "Dig Dug Doug 128K (2022)(Gabriele Amore).tap", 48202, 0x008a28be, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecDigdugdoug, SpecDigdugdoug, Spec128)
STD_ROM_FN(SpecDigdugdoug)

struct BurnDriver BurnSpecDigdugdoug = {
	"spec_digdugdoug", NULL, "spec_spec128", NULL, "2022",
	"Dig Dug Doug (128K) (HB)\0", NULL, "Gabriele Amore", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_ACTION, 0,
	SpectrumGetZipName, SpecDigdugdougRomInfo, SpecDigdugdougRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecQAOPMDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Dizzy II: Treasure Island - Classic Edition 2023 (English) (128K) (Hack, v1.4)

static struct BurnRomInfo SpecDizzy2ce23enRomDesc[] = {
	{ "Dizzy II - Treasure Island - Classic Edition 2023 EN v1.4 128K (2023)(Dr.Titus).tap", 43272, 0x31d0a7e0, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecDizzy2ce23en, SpecDizzy2ce23en, Spec128)
STD_ROM_FN(SpecDizzy2ce23en)

struct BurnDriver BurnSpecDizzy2ce23en = {
	"spec_dizzy2ce23en", NULL, "spec_spec128", NULL, "2023",
	"Dizzy II: Treasure Island - Classic Edition 2023 (English) (128K) (Hack, v1.4)\0", NULL, "Dr.Titus", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HACK, 1, HARDWARE_SPECTRUM, GBF_PLATFORM | GBF_ADV, 0,
	SpectrumGetZipName, SpecDizzy2ce23enRomInfo, SpecDizzy2ce23enRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Dizzy II: Treasure Island - Classic Edition 2023 (Russian) (128K) (Hack, v1.4)

static struct BurnRomInfo SpecDizzy2ce23ruRomDesc[] = {
	{ "Dizzy II - Treasure Island - Classic Edition 2023 RU v1.4 128K (2023)(Dr.Titus).tap", 43317, 0xf3edd6b4, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecDizzy2ce23ru, SpecDizzy2ce23ru, Spec128)
STD_ROM_FN(SpecDizzy2ce23ru)

struct BurnDriver BurnSpecDizzy2ce23ru = {
	"spec_dizzy2ce23ru", "spec_dizzy2ce23en", "spec_spec128", NULL, "2023",
	"Dizzy II: Treasure Island - Classic Edition 2023 (Russian) (128K) (Hack, v1.4)\0", NULL, "Dr.Titus", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE | BDF_HACK, 1, HARDWARE_SPECTRUM, GBF_PLATFORM | GBF_ADV, 0,
	SpectrumGetZipName, SpecDizzy2ce23ruRomInfo, SpecDizzy2ce23ruRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Dizzy II: Treasure Island - Extended Edition 2023 (English) (128K) (Hack, v1.0d)

static struct BurnRomInfo SpecDizzy2ex23eRomDesc[] = {
	{ "Dizzy 2 - Treasure Island Ext2023 128K ENG v1.0d (2023)(Dr.Titus).tap", 44702, 0xd9f6f36f, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecDizzy2ex23e, SpecDizzy2ex23e, Spec128)
STD_ROM_FN(SpecDizzy2ex23e)

struct BurnDriver BurnSpecDizzy2ex23e = {
	"spec_dizzy2ex23e", NULL, "spec_spec128", NULL, "2023",
	"Dizzy II: Treasure Island - Extended Edition 2023 (English) (128K) (Hack, v1.0d)\0", NULL, "Dr.Titus", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HACK, 1, HARDWARE_SPECTRUM, GBF_ACTION | GBF_ADV, 0,
	SpectrumGetZipName, SpecDizzy2ex23eRomInfo, SpecDizzy2ex23eRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Dizzy II: Treasure Island - Extended Edition 2023 (Russian) (128K) (Hack, v1.0d)

static struct BurnRomInfo SpecDizzy2ex23rRomDesc[] = {
	{ "Dizzy 2 - Treasure Island Ext2023 128K RUS v1.0d (2023)(Dr.Titus).tap", 44695, 0xdccca5a7, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecDizzy2ex23r, SpecDizzy2ex23r, Spec128)
STD_ROM_FN(SpecDizzy2ex23r)

struct BurnDriver BurnSpecDizzy2es23r = {
	"spec_dizzy2ex23r", "spec_dizzy2ex23e", "spec_spec128", NULL, "2023",
	"Dizzy II: Treasure Island - Extended Edition 2023 (Russian) (128K) (Hack, v1.0d)\0", NULL, "Dr.Titus", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE | BDF_HACK, 1, HARDWARE_SPECTRUM, GBF_ACTION | GBF_ADV, 0,
	SpectrumGetZipName, SpecDizzy2ex23rRomInfo, SpecDizzy2ex23rRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Dizzy III: Fantasy World Dizzy - Classic Edition 2023 (English) (128K) (Hack)

static struct BurnRomInfo SpecDizzy3ce23RomDesc[] = {
	{ "Fantasy World Dizzy - 2023 Classic Edition ENG (2023)(Dr. Titus).tap", 50676, 0x9c46058f, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecDizzy3ce23, SpecDizzy3ce23, Spec128)
STD_ROM_FN(SpecDizzy3ce23)

struct BurnDriver BurnSpecDizzy3ce23 = {
	"spec_dizzy3ce23", NULL, "spec_spec128", NULL, "2023",
	"Dizzy III: Fantasy World Dizzy - Classic Edition 2023 (English) (128K) (Hack)\0", NULL, "Dr. Titus", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HACK, 1, HARDWARE_SPECTRUM, GBF_PLATFORM | GBF_ADV, 0,
	SpectrumGetZipName, SpecDizzy3ce23RomInfo, SpecDizzy3ce23RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Dizzy III: Fantasy World Dizzy - Classic Edition 2023 (Russian) (128K) (Hack)

static struct BurnRomInfo SpecDizzy3ce23ruRomDesc[] = {
	{ "Fantasy World Dizzy - 2023 Classic Edition RUS (2023)(Dr. Titus).tap", 50986, 0x5fc99016, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecDizzy3ce23ru, SpecDizzy3ce23ru, Spec128)
STD_ROM_FN(SpecDizzy3ce23ru)

struct BurnDriver BurnSpecDizzy3ce23ru = {
	"spec_dizzy3ce23ru", "spec_dizzy3ce23", "spec_spec128", NULL, "2023",
	"Dizzy III: Fantasy World Dizzy - Classic Edition 2023 (Russian) (128K) (Hack)\0", NULL, "Dr. Titus", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE| BDF_HACK, 1, HARDWARE_SPECTRUM, GBF_PLATFORM | GBF_ADV, 0,
	SpectrumGetZipName, SpecDizzy3ce23ruRomInfo, SpecDizzy3ce23ruRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Dizzy III: Fantasy World Dizzy - Extended Edition 2023 (English) (128K) (Hack)

static struct BurnRomInfo SpecDizzy3ee23enRomDesc[] = {
	{ "Dizzy III - Fantasy World Dizzy - Extended Edition 2023 EN 128K (2023)(Dr. Titus).tap", 51677, 0xbe462222, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecDizzy3ee23en, SpecDizzy3ee23en, Spec128)
STD_ROM_FN(SpecDizzy3ee23en)

struct BurnDriver BurnSpecDizzy3ee23en = {
	"spec_dizzy3ee23en", NULL, "spec_spec128", NULL, "2023",
	"Dizzy III: Fantasy World Dizzy - Extended Edition 2023 (English) (128K) (Hack)\0", NULL, "Dr. Titus", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HACK, 1, HARDWARE_SPECTRUM, GBF_PLATFORM | GBF_ADV, 0,
	SpectrumGetZipName, SpecDizzy3ee23enRomInfo, SpecDizzy3ee23enRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Dizzy III: Fantasy World Dizzy - Extended Edition 2023 (Russian) (128K) (Hack)

static struct BurnRomInfo SpecDizzy3ee23ruRomDesc[] = {
	{ "Dizzy III - Fantasy World Dizzy - Extended Edition 2023 RU 128K (2023)(Dr. Titus).tap", 52008, 0x71804a6c, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecDizzy3ee23ru, SpecDizzy3ee23ru, Spec128)
STD_ROM_FN(SpecDizzy3ee23ru)

struct BurnDriver BurnSpecDizzy3ee23ru = {
	"spec_dizzy3ee23ru", "spec_dizzy3ee23en", "spec_spec128", NULL, "2023",
	"Dizzy III: Fantasy World Dizzy - Extended Edition 2023 (Russian) (128K) (Hack)\0", NULL, "Dr. Titus", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE | BDF_HACK, 1, HARDWARE_SPECTRUM, GBF_PLATFORM | GBF_ADV, 0,
	SpectrumGetZipName, SpecDizzy3ee23ruRomInfo, SpecDizzy3ee23ruRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Dizzy IV: Magicland Dizzy - Classic Edition 2024 (English) (128K) (Hack, v1.1)

static struct BurnRomInfo SpecDizzy4ce24enRomDesc[] = {
	{ "Dizzy IV - Magicland Dizzy - Classic Edition 2024 EN v1.1 128K (2024)(Dr. Titus).tap", 48340, 0x949a2f47, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecDizzy4ce24en, SpecDizzy4ce24en, Spec128)
STD_ROM_FN(SpecDizzy4ce24en)

struct BurnDriver BurnSpecDizzy4ce24en = {
	"spec_dizzy4ce24en", NULL, "spec_spec128", NULL, "2024",
	"Dizzy IV: Magicland Dizzy - Classic Edition 2024 (English) (128K) (Hack, v1.1)\0", NULL, "Dr. Titus", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HACK, 1, HARDWARE_SPECTRUM, GBF_PLATFORM | GBF_ADV, 0,
	SpectrumGetZipName, SpecDizzy4ce24enRomInfo, SpecDizzy4ce24enRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Dizzy IV: Magicland Dizzy - Classic Edition 2024 (Russian) (128K) (Hack, v1.1)

static struct BurnRomInfo SpecDizzy4ce24ruRomDesc[] = {
	{ "Dizzy IV - Magicland Dizzy - Classic Edition 2024 RU v1.1 128K (2024)(Dr. Titus).tap", 48417, 0x7b4167da, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecDizzy4ce24ru, SpecDizzy4ce24ru, Spec128)
STD_ROM_FN(SpecDizzy4ce24ru)

struct BurnDriver BurnSpecDizzy4ce24ru = {
	"spec_dizzy4ce24ru", "spec_dizzy4ce24en", "spec_spec128", NULL, "2024",
	"Dizzy IV: Magicland Dizzy - Classic Edition 2024 (Russian) (128K) (Hack, v1.1)\0", NULL, "Dr. Titus", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE | BDF_HACK, 1, HARDWARE_SPECTRUM, GBF_PLATFORM | GBF_ADV, 0,
	SpectrumGetZipName, SpecDizzy4ce24ruRomInfo, SpecDizzy4ce24ruRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// DJ Puff's Volcanic Capers Recoloured (128K) (Hack)

static struct BurnRomInfo SpecDjpuff23RomDesc[] = {
	{ "DJ Puff's Volcanic Capers 128K (Re-coloured MOD)(2023)(Rafal Miazga).tap", 58200, 0xd26dc482, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecDjpuff23, SpecDjpuff23, Spec128)
STD_ROM_FN(SpecDjpuff23)

struct BurnDriver BurnSpecDjpuff23 = {
	"spec_djpuff23", "spec_djpuff", "spec_spec128", NULL, "2023",
	"DJ Puff's Volcanic Capers Recoloured (128K) (Hack)\0", NULL, "Rafal Miazga", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE | BDF_HACK, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecDjpuff23RomInfo, SpecDjpuff23RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecQAOPSpaceDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Doctor Adventures (Spanish) (48K) (HB)

static struct BurnRomInfo SpecDocadvRomDesc[] = {
	{ "Doctor Adventures ES 48K (2015)(errecepe).tap", 43037, 0xe74af4dc, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecDocadv, SpecDocadv, Spectrum)
STD_ROM_FN(SpecDocadv)

struct BurnDriver BurnSpecDocadv = {
	"spec_docadv", NULL, "spec_spectrum", NULL, "2015",
	"Doctor Adventures (Spanish) (48K) (HB)\0", NULL, "errecepe", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecDocadvRomInfo, SpecDocadvRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Dogmole Tuppowski (48K) (HB)

static struct BurnRomInfo SpecDogmoleRomDesc[] = {
	{ "Dogmole Tuppowski 48K (2013)(The Mojon Twins).tap", 39379, 0xa9167984, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecDogmole, SpecDogmole, Spectrum)
STD_ROM_FN(SpecDogmole)

struct BurnDriver BurnSpecDogmole = {
	"spec_dogmole", NULL, "spec_spectrum", NULL, "2013",
	"Dogmole Tuppowski (48K) (HB)\0", "VS. the Mesmerizing Meemaid From Miskatonic", "The Mojon Twins", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecDogmoleRomInfo, SpecDogmoleRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Donkey Kong 3 (48K) (HB, v1.1)

static struct BurnRomInfo SpecDkong3RomDesc[] = {
	{ "Donkey Kong 3 v1.1 (2022)(je7ebel).tap", 39512, 0xef597223, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecDkong3, SpecDkong3, Spectrum)
STD_ROM_FN(SpecDkong3)

struct BurnDriver BurnSpecDkong3 = {
	"spec_dkong3", NULL, "spec_spectrum", NULL, "2022",
	"Donkey Kong 3 (48K) (HB, v1.1)\0", NULL, "je7ebel", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 2, HARDWARE_SPECTRUM, GBF_ACTION, 0,
	SpectrumGetZipName, SpecDkong3RomInfo, SpecDkong3RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecIntf2DIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Donkey Kong Jr. ZX (128K) (HB)

static struct BurnRomInfo SpecDkingjrzxRomDesc[] = {
	{ "Donkey Kong Jr. ZX 128K (2016)(Gabriele Amore).tap", 33914, 0xec09aa59, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecDkingjrzx, SpecDkingjrzx, Spec128)
STD_ROM_FN(SpecDkingjrzx)

struct BurnDriver BurnSpecDkingjrzx = {
	"spec_dkingjrzx", NULL, "spec_spec128", NULL, "2016",
	"Donkey Kong Jr. ZX (128K) (HB)\0", NULL, "Gabriele Amore", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_ACTION, 0,
	SpectrumGetZipName, SpecDkingjrzxRomInfo, SpecDkingjrzxRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Donkey Kong Reloaded (48K) (HB)

static struct BurnRomInfo SpecDkreloadRomDesc[] = {
	{ "Donkey Kong Reloaded (2013)(Gabriele Amore).tap", 40960, 0x8000a624, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecDkreload, SpecDkreload, Spec128)
STD_ROM_FN(SpecDkreload)

struct BurnDriver BurnSpecDkreload = {
	"spec_dkreload", NULL, "spec_spec128", NULL, "2013",
	"Donkey Kong Reloaded (48K) (HB)\0", NULL, "Gabriele Amore", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecDkreloadRomInfo, SpecDkreloadRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecQAOPSpaceDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Don Quixote (16K) (HB)

static struct BurnRomInfo SpecDonquixoteRomDesc[] = {
	{ "Don Quixote 16K (2022)(Zosya Entertainment).tap", 12842, 0xa341a789, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecDonquixote, SpecDonquixote, Spectrum)
STD_ROM_FN(SpecDonquixote)

struct BurnDriver BurnSpecDonquixote = {
	"spec_donquixote", NULL, "spec_spectrum", NULL, "2022",
	"Don Quixote (16K) (HB)\0", NULL, "Zosya Entertainment", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecDonquixoteRomInfo, SpecDonquixoteRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Donum (English) (128K) (HB)

static struct BurnRomInfo SpecDonumeRomDesc[] = {
	{ "Donum (English)(2022)(Bitfans).tap", 115891, 0x1cb940e7, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecDonume, SpecDonume, Spec128)
STD_ROM_FN(SpecDonume)

struct BurnDriver BurnSpecDonume = {
	"spec_donume", NULL, "spec_spec128", NULL, "2022",
	"Donum (English) (128K) (HB)\0", NULL, "Bitfans", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_ADV, 0,
	SpectrumGetZipName, SpecDonumeRomInfo, SpecDonumeRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Donum (Spanish) (128K) (HB)

static struct BurnRomInfo SpecDonumsRomDesc[] = {
	{ "Donum (Spain)(2022)(Bitfans).tap", 116754, 0x5dae657a, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecDonums, SpecDonums, Spec128)
STD_ROM_FN(SpecDonums)

struct BurnDriver BurnSpecDonums = {
	"spec_donums", "spec_donume", "spec_spec128", NULL, "2022",
	"Donum (Spanish) (128K) (HB)\0", NULL, "Bitfans", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_ADV, 0,
	SpectrumGetZipName, SpecDonumsRomInfo, SpecDonumsRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// DoubleBubble (128K) (HB)

static struct BurnRomInfo SpecDbubble128kRomDesc[] = {
	{ "DoubleBubble 128K (2016)(Miguetelo).tap", 36664, 0x3c929681, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecDbubble128k, SpecDbubble128k, Spec128)
STD_ROM_FN(SpecDbubble128k)

struct BurnDriver BurnSpecDbubble128k = {
	"spec_dbubble128k", NULL, "spec_spec128", NULL, "2016",
	"DoubleBubble (128K) (HB)\0", NULL, "Miguetelo", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_ACTION, 0,
	SpectrumGetZipName, SpecDbubble128kRomInfo, SpecDbubble128kRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecQAOPSpaceDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// DoubleBubble (48K) (HB)

static struct BurnRomInfo SpecDbubble48kRomDesc[] = {
	{ "DoubleBubble 48K (2016)(Miguetelo).tap", 33169, 0x07a0f183, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecDbubble48k, SpecDbubble48k, Spec128)
STD_ROM_FN(SpecDbubble48k)

struct BurnDriver BurnSpecDbubble48k = {
	"spec_dbubble48k", "spec_dbubble128k", "spec_spec128", NULL, "2016",
	"DoubleBubble (48K) (HB)\0", NULL, "Miguetelo", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_ACTION, 0,
	SpectrumGetZipName, SpecDbubble48kRomInfo, SpecDbubble48kRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecQAOPSpaceDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Down the pipe (128K) (HB)

static struct BurnRomInfo Specdownpipe128RomDesc[] = {
	{ "Down the pipe 128K (2021)(DF Design).tap", 58634, 0x6ff07621, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specdownpipe128, Specdownpipe128, Spec128)
STD_ROM_FN(Specdownpipe128)

struct BurnDriver BurnSpecdownpipe128 = {
	"spec_downpipe128", NULL, "spec_spec128", NULL, "2021",
	"Down the pipe (128K) (HB)\0", NULL, "DF Design", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_VERSHOOT, 0,
	SpectrumGetZipName, Specdownpipe128RomInfo, Specdownpipe128RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecIntf2DIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Down the pipe (48K) (HB)

static struct BurnRomInfo Specdownpipe48RomDesc[] = {
	{ "Down the pipe 48K (2021)(DF Design).tap", 46991, 0x71ce5665, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specdownpipe48, Specdownpipe48, Spectrum)
STD_ROM_FN(Specdownpipe48)

struct BurnDriver BurnSpecdownpipe48 = {
	"spec_downpipe48", "spec_downpipe128", "spec_spectrum", NULL, "2021",
	"Down the pipe (48K) (HB)\0", NULL, "DF Design", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_VERSHOOT, 0,
	SpectrumGetZipName, Specdownpipe48RomInfo, Specdownpipe48RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecIntf2DIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Down the pipe Reflushed (48K) (HB)

static struct BurnRomInfo SpecDownpiperefRomDesc[] = {
	{ "Down the pipe Reflushed (2023)(DF Design)(Crash covertape).tap", 47160, 0x1a679367, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecDownpiperef, SpecDownpiperef, Spectrum)
STD_ROM_FN(SpecDownpiperef)

struct BurnDriver BurnSpecDownpiperef = {
	"spec_downpiperef", "spec_downpipe128", "spec_spectrum", NULL, "2023",
	"Down the pipe Reflushed (48K) (HB)\0", NULL, "DF Design", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_VERSHOOT, 0,
	SpectrumGetZipName, SpecDownpiperefRomInfo, SpecDownpiperefRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecIntf2DIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Dr.Acula (48K) (HB)

static struct BurnRomInfo SpecDraculaRomDesc[] = {
	{ "Dr.Acula 48K (2024)(Colin Stewart).tap", 46666, 0xc14a4558, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecDracula, SpecDracula, Spectrum)
STD_ROM_FN(SpecDracula)

struct BurnDriver BurnSpecDracula = {
	"spec_dracula", NULL, "spec_spectrum", NULL, "2024",
	"Dr.Acula (48K) (HB)\0", NULL, "Colin Stewart", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecDraculaRomInfo, SpecDraculaRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Dreamwalker - Alter Ego 2 (128K) (HB, v1.1)

static struct BurnRomInfo SpecdrmwalkerRomDesc[] = {
	{ "Dreamwalker128k v1.1 (2014)(RetroSouls).tap", 92060, 0x89db756e, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specdrmwalker, Specdrmwalker, Spec128)
STD_ROM_FN(Specdrmwalker)

struct BurnDriver BurnSpecdrmwalker = {
	"spec_drmwalker", NULL, "spec_spec128", NULL, "2014",
	"Dreamwalker - Alter Ego 2 (128K) (HB, v1.1)\0", NULL, "RetroSouls", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PUZZLE, 0,
	SpectrumGetZipName, SpecdrmwalkerRomInfo, SpecdrmwalkerRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecQAOPMDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Drift! (128K) (HB)

static struct BurnRomInfo SpecDriftRomDesc[] = {
	{ "Drift! (2019)(Zosya).tap", 128215, 0x35e3f19b, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecDrift, SpecDrift, Spec128)
STD_ROM_FN(SpecDrift)

struct BurnDriver BurnSpecDrift = {
	"spec_drift", NULL, "spec_spec128", NULL, "2019",
	"Drift! (128K) (HB)\0", "Designed esclusively for 128K system", "Zosya Entertainment", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_RACING, 0,
	SpectrumGetZipName, SpecDriftRomInfo, SpecDriftRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Drift! 2K22 (128K) (HB)

static struct BurnRomInfo SpecDrift2k22RomDesc[] = {
	{ "Drift! 2K22 (2022)(Zosya Ent.).tap", 82851, 0x8a50263e, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecDrift2k22, SpecDrift2k22, Spec128)
STD_ROM_FN(SpecDrift2k22)

struct BurnDriver BurnSpecDrift2k22 = {
	"spec_drift2k22", NULL, "spec_spec128", NULL, "2022",
	"Drift! 2K22 (128K) (HB)\0", NULL, "Zosya Entertainment", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_RACING, 0,
	SpectrumGetZipName, SpecDrift2k22RomInfo, SpecDrift2k22RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Duckstroma (128K) (HB, v1.03)

static struct BurnRomInfo SpecduckstromaRomDesc[] = {
	{ "Duckstroma v1.03 128K (2020)(UltraNarwhal).tap", 31560, 0x6979fcd5, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specduckstroma, Specduckstroma, Spec128)
STD_ROM_FN(Specduckstroma)

struct BurnDriver BurnSpecduckstroma = {
	"spec_duckstroma", NULL, "spec_spec128", NULL, "2020-23",
	"Duckstroma (128K) (HB, v1.03)\0", "Wait a second while unpacking", "UltraNarwhal", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecduckstromaRomInfo, SpecduckstromaRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Duckstroma pt.2 (128K) (HB, v1.01)

static struct BurnRomInfo SpecDuckstromapt2RomDesc[] = {
	{ "Duckstroma pt.2 v1.01 128K (2023)(UltraNarwhal).tap", 30758, 0x7e14b59e, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecDuckstromapt2, SpecDuckstromapt2, Spec128)
STD_ROM_FN(SpecDuckstromapt2)

struct BurnDriver BurnSpecDuckstromapt2 = {
	"spec_duckstromapt2", NULL, "spec_spec128", NULL, "2023",
	"Duckstroma pt.2 (128K) (HB, v1.01)\0", "Wait a second while unpacking", "UltraNarwhal", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecDuckstromapt2RomInfo, SpecDuckstromapt2RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// DuckTales Family Comes First! (English) (48K) (HB)

static struct BurnRomInfo SpecDtfcfenRomDesc[] = {
	{ "DuckTales Family Comes First! EN 48K (2023)(Team N10).tap", 81805, 0xd72cd925, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecDtfcfen, SpecDtfcfen, Spectrum)
STD_ROM_FN(SpecDtfcfen)

struct BurnDriver BurnSpecDtfcfen = {
	"spec_dtfcfen", NULL, "spec_spectrum", NULL, "2023",
	"DuckTales Family Comes First! (English) (48K) (HB)\0", NULL, "Team N10", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecDtfcfenRomInfo, SpecDtfcfenRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// DuckTales Family Comes First! (Ukrainian) (48K) (HB)

static struct BurnRomInfo SpecDtfcfuaRomDesc[] = {
	{ "DuckTales Family Comes First! UA 48K (2023)(Team N10).tap", 81727, 0xa4049cde, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecDtfcfua, SpecDtfcfua, Spectrum)
STD_ROM_FN(SpecDtfcfua)

struct BurnDriver BurnSpecDtfcfua = {
	"spec_dtfcfua", "spec_dtfcfen", "spec_spectrum", NULL, "2023",
	"DuckTales Family Comes First! (Ukrainian) (48K) (HB)\0", NULL, "Team N10", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecDtfcfuaRomInfo, SpecDtfcfuaRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// DuckTales ZX (English) (48K) (HB)

static struct BurnRomInfo SpecDucktalesenRomDesc[] = {
	{ "DuckTales ZX EN (2021)(Team N10).tap", 50407, 0x576e90d2, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecDucktalesen, SpecDucktalesen, Spectrum)
STD_ROM_FN(SpecDucktalesen)

struct BurnDriver BurnSpecDucktalesen = {
	"spec_ducktalesen", NULL, "spec_spectrum", NULL, "2021",
	"DuckTales ZX (English) (48K) (HB)\0", NULL, "Team N10", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecDucktalesenRomInfo, SpecDucktalesenRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// DuckTales ZX (Ukrainian) (48K) (HB)

static struct BurnRomInfo SpecDucktalesuaRomDesc[] = {
	{ "DuckTales ZX UA (2021)(Team N10).tap", 50407, 0x61e559ca, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecDucktalesua, SpecDucktalesua, Spectrum)
STD_ROM_FN(SpecDucktalesua)

struct BurnDriver BurnSpecDucktalesua = {
	"spec_ducktalesua", "spec_ducktalesen", "spec_spectrum", NULL, "2021",
	"DuckTales ZX (Ukrainian) (48K) (HB)\0", NULL, "Team N10", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW | BDF_CLONE, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecDucktalesuaRomInfo, SpecDucktalesuaRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Dungeons & Dragons Computer Fantasy Game (128K) (HB)

static struct BurnRomInfo SpecDndcfgRomDesc[] = {
	{ "Dungeons & Dragons Computer Fantasy Game 128K (2024)(El Dr. Gusman).tap", 69627, 0x79e04513, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecDndcfg, SpecDndcfg, Spec128)
STD_ROM_FN(SpecDndcfg)

struct BurnDriver BurnSpecDndcfg = {
	"spec_dndcfg", NULL, "spec_spec128", NULL, "2024",
	"Dungeons & Dragons Computer Fantasy Game (128K) (HB)\0", "Q: Cursor - W: Attack - E: Move", "El Dr. Gusman", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_ACTION, 0,
	SpectrumGetZipName, SpecDndcfgRomInfo, SpecDndcfgRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Dungeon Raiders (48K) (HB)

static struct BurnRomInfo SpecDungeonraidersRomDesc[] = {
	{ "Dungeon Raiders (2018)(Payndz).tap", 33635, 0x9bfd967b, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecDungeonraiders, SpecDungeonraiders, Spectrum)
STD_ROM_FN(SpecDungeonraiders)

struct BurnDriver BurnSpecDungeonraiders = {
	"spec_dungeonraiders", NULL, "spec_spectrum", NULL, "2018",
	"Dungeon Raiders (48K) (HB)\0", NULL, "Payndz", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_ACTION, 0,
	SpectrumGetZipName, SpecDungeonraidersRomInfo, SpecDungeonraidersRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecQAOPSpaceDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// D'Veel'Ng (48K) (HB)

static struct BurnRomInfo SpecDveelngRomDesc[] = {
	{ "D'Veel'Ng (2013)(The Mojon Twins).tap", 40424, 0xeeb6c372, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecDveelng, SpecDveelng, Spectrum)
STD_ROM_FN(SpecDveelng)

struct BurnDriver BurnSpecDveelng = {
	"spec_dveelng", NULL, "spec_spectrum", NULL, "2013",
	"D'Veel'Ng (48K) (HB)\0", NULL, "The Mojon Twins", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_ACTION, 0,
	SpectrumGetZipName, SpecDveelngRomInfo, SpecDveelngRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Eclipsia Part 1 - Beginning (48K) (HB)

static struct BurnRomInfo SpecEclipsiap1RomDesc[] = {
	{ "Eclipsia part 1 - Beginning 48K (2023)(ICreateworlds).tap", 48336, 0xfe2d5769, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecEclipsiap1, SpecEclipsiap1, Spectrum)
STD_ROM_FN(SpecEclipsiap1)

struct BurnDriver BurnSpecEclipsiap1 = {
	"spec_eclipsiap1", NULL, "spec_spectrum", NULL, "2023",
	"Eclipsia Part 1 - Beginning (48K) (HB)\0", NULL, "ICreateworlds", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_ADV, 0,
	SpectrumGetZipName, SpecEclipsiap1RomInfo, SpecEclipsiap1RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Eclipsia Part 2 - Awaken (48K) (HB)

static struct BurnRomInfo SpecEclipsiap2RomDesc[] = {
	{ "Eclipsia part 2 - Awaken 48K (2023)(ICreateworlds).tap", 48557, 0xea18f33b, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecEclipsiap2, SpecEclipsiap2, Spectrum)
STD_ROM_FN(SpecEclipsiap2)

struct BurnDriver BurnSpecEclipsiap2 = {
	"spec_eclipsiap2", "spec_eclipsiap1", "spec_spectrum", NULL, "2023",
	"Eclipsia part 2 - Awaken (48K) (HB)\0", NULL, "ICreateworlds", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_ADV, 0,
	SpectrumGetZipName, SpecEclipsiap2RomInfo, SpecEclipsiap2RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Eclipsia Part 3 - Starlost (48K) (HB)

static struct BurnRomInfo SpecEclipsiap3RomDesc[] = {
	{ "Eclipsia part 3 - Starlost 48K (2023)(ICreateworlds).tap", 48560, 0xbb923bfc, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecEclipsiap3, SpecEclipsiap3, Spectrum)
STD_ROM_FN(SpecEclipsiap3)

struct BurnDriver BurnSpecEclipsiap3 = {
	"spec_eclipsiap3", "spec_eclipsiap1", "spec_spectrum", NULL, "2023",
	"Eclipsia Part 3 - Starlost (48K) (HB)\0", NULL, "ICreateworlds", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_ADV, 0,
	SpectrumGetZipName, SpecEclipsiap3RomInfo, SpecEclipsiap3RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Egghead 3 - Egghead in Space (48K) (HB)

static struct BurnRomInfo SpecEgghead3RomDesc[] = {
	{ "Egghead 3 - Egghead in Space (2002)(Jonathan Cauldwell).tap", 44595, 0xee2d725c, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecEgghead3, SpecEgghead3, Spectrum)
STD_ROM_FN(SpecEgghead3)

struct BurnDriver BurnSpecEgghead3 = {
	"spec_egghead3", NULL, "spec_spectrum", NULL, "2002",
	"Egghead 3 - Egghead in Space (48K) (HB)\0", NULL, "Jonathan Cauldwell", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecEgghead3RomInfo, SpecEgghead3RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Egghead 4 - Egghead Entertains (128K) (HB)

static struct BurnRomInfo SpecEgghead4RomDesc[] = {
	{ "Egghead 4 - Egghead Entertains 128K (2006)(Jonathan Cauldwell).tap", 47432, 0x7b0e6452, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecEgghead4, SpecEgghead4, Spec128)
STD_ROM_FN(SpecEgghead4)

struct BurnDriver BurnSpecEgghead4 = {
	"spec_egghead4", NULL, "spec_spec128", NULL, "2006",
	"Egghead 4 - Egghead Entertains (128K) (HB)\0", NULL, "Jonathan Cauldwell", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecEgghead4RomInfo, SpecEgghead4RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Egghead 5 - Egghead Round the Med (128K) (HB)

static struct BurnRomInfo SpecEgghead5RomDesc[] = {
	{ "Egghead 5 - Egghead Round the Med 128K (2007)(Jonathan Cauldwell).tap", 64228, 0xda3aa38b, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecEgghead5, SpecEgghead5, Spec128)
STD_ROM_FN(SpecEgghead5)

struct BurnDriver BurnSpecEgghead5 = {
	"spec_egghead5", NULL, "spec_spec128", NULL, "2007",
	"Egghead 5 - Egghead Round the Med (128K) (HB)\0", NULL, "Jonathan Cauldwell", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecEgghead5RomInfo, SpecEgghead5RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Eggsterminator, The (128K) (HB)

static struct BurnRomInfo SpecEggsterminatorRomDesc[] = {
	{ "The Eggsterminator (2018)(The Death Squad).tap", 48022, 0x6960fff0, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecEggsterminator, SpecEggsterminator, Spec128)
STD_ROM_FN(SpecEggsterminator)

struct BurnDriver BurnSpecEggsterminator = {
	"spec_eggsterminator", NULL, "spec_spec128", NULL, "2018",
	"Eggsterminator, The (128K) (HB)\0", NULL, "The Death Squad", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_ACTION, 0,
	SpectrumGetZipName, SpecEggsterminatorRomInfo, SpecEggsterminatorRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecIntf2DIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// El hombre que susurraba a las balas (48K) (HB)

static struct BurnRomInfo SpecElhombreRomDesc[] = {
	{ "El hombre que susurraba a las balas 48K (2013)(Radastan).tap", 43394, 0xd41acad0, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecElhombre, SpecElhombre, Spectrum)
STD_ROM_FN(SpecElhombre)

struct BurnDriver BurnSpecElhombre = {
	"spec_elhombre", NULL, "spec_spectrum", NULL, "2013",
	"Hombre que susurraba a las balas, El (48K) (HB)\0", NULL, "Radastan", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_ACTION | GBF_ADV, 0,
	SpectrumGetZipName, SpecElhombreRomInfo, SpecElhombreRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Elixir Vitae (128K) (HB)

static struct BurnRomInfo SpecElixirvitaeRomDesc[] = {
	{ "Elixir Vitae (2013)(Rafal Miazga).tap", 19567, 0x68c69860, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecElixirvitae, SpecElixirvitae, Spec128)
STD_ROM_FN(SpecElixirvitae)

struct BurnDriver BurnSpecElixirvitae = {
	"spec_elixirvitae", NULL, "spec_spec128", NULL, "2013",
	"Elixir Vitae (128K) (HB)\0", NULL, "Rafal Miazga", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PUZZLE, 0,
	SpectrumGetZipName, SpecElixirvitaeRomInfo, SpecElixirvitaeRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Elon M. with Jetpack (128K) (HB)

static struct BurnRomInfo SpecElonjetpackRomDesc[] = {
	{ "Elon M. with Jetpack 128K (2019)(Matra).tap", 46403, 0xbc78e913, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecElonjetpack, SpecElonjetpack, Spec128)
STD_ROM_FN(SpecElonjetpack)

struct BurnDriver BurnSpecElonjetpack = {
	"spec_elonjetpack", NULL, "spec_spec128", NULL, "2018-19",
	"Elon M. with Jetpack (128K) (HB)\0", NULL, "Rafael Vico Costa - Matra", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_ACTION, 0,
	SpectrumGetZipName, SpecElonjetpackRomInfo, SpecElonjetpackRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Endless Forms Most Beautiful (48K) (HB)

static struct BurnRomInfo SpecefmbRomDesc[] = {
	{ "Endless Forms Most Beautiful (2012)(Stonechat Productions).tap", 31843, 0x4d866f79, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specefmb, Specefmb, Spectrum)
STD_ROM_FN(Specefmb)

struct BurnDriver BurnSpecefmb = {
	"spec_efmb", NULL, "spec_spectrum", NULL, "2012",
	"Endless Forms Most Beautiful (48K) (HB)\0", NULL, "Stonechat Productions", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 2, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecefmbRomInfo, SpecefmbRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecQAOPSpaceDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Escalera, La (English) (48K) (HB)

static struct BurnRomInfo SpecEscaleraeRomDesc[] = {
	{ "Escalera, La EN 48K (2024)(Fitosoft).tap", 39788, 0x2e6a5f01, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecEscalerae, SpecEscalerae, Spectrum)
STD_ROM_FN(SpecEscalerae)

struct BurnDriver BurnSpecEscalerae = {
	"spec_escalerae", NULL, "spec_spectrum", NULL, "2024",
	"Escalera, La (English) (48K) (HB)\0", NULL, "Fitosoft", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 4, HARDWARE_SPECTRUM, GBF_BOARD, 0,
	SpectrumGetZipName, SpecEscaleraeRomInfo, SpecEscaleraeRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Escalera, La (Portuguese) (48K) (HB)

static struct BurnRomInfo SpecEscalerapRomDesc[] = {
	{ "Escalera, La PT 48K (2024)(Fitosoft).tap", 39833, 0xd93a0fc4, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecEscalerap, SpecEscalerap, Spectrum)
STD_ROM_FN(SpecEscalerap)

struct BurnDriver BurnSpecEscalerap = {
	"spec_escalerap", "spec_escalerae", "spec_spectrum", NULL, "2024",
	"Escalera, La (Portuguese) (48K) (HB)\0", NULL, "Fitosoft", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE | BDF_HOMEBREW, 4, HARDWARE_SPECTRUM, GBF_BOARD, 0,
	SpectrumGetZipName, SpecEscalerapRomInfo, SpecEscalerapRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Escalera, La (Spanish) (48K) (HB)

static struct BurnRomInfo SpecEscalerasRomDesc[] = {
	{ "Escalera, La ES 48K (2024)(Fitosoft).tap", 39839, 0xaf366258, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecEscaleras, SpecEscaleras, Spectrum)
STD_ROM_FN(SpecEscaleras)

struct BurnDriver BurnSpecEscaleras = {
	"spec_escaleras", "spec_escalerae", "spec_spectrum", NULL, "2024",
	"Escalera, La (Spanish) (48K) (HB)\0", NULL, "Fitosoft", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE | BDF_HOMEBREW, 4, HARDWARE_SPECTRUM, GBF_BOARD, 0,
	SpectrumGetZipName, SpecEscalerasRomInfo, SpecEscalerasRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Escape from Asylum (48K) (HB)

static struct BurnRomInfo SpecEscasylumRomDesc[] = {
	{ "Escape from Asylum 48K (2013)(Dantoine Retro Studios).tap", 21176, 0x38d1bd63, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecEscasylum, SpecEscasylum, Spectrum)
STD_ROM_FN(SpecEscasylum)

struct BurnDriver BurnSpecEscasylum = {
	"spec_escasylum", NULL, "spec_spectrum", NULL, "2013",
	"Escape from Asylum (48K) (HB)\0", NULL, "Dantoine Retro Studios", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_ACTION | GBF_RPG, 0,
	SpectrumGetZipName, SpecEscasylumRomInfo, SpecEscasylumRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Escape From Colony 8 (48K) (HB)

static struct BurnRomInfo SpecEsccolonyRomDesc[] = {
	{ "Escape From Colony 8 48K (2014)(usebox.net).tap", 33045, 0xb55aad43, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecEsccolony, SpecEsccolony, Spectrum)
STD_ROM_FN(SpecEsccolony)

struct BurnDriver BurnSpecEsccolony = {
	"spec_esccolony", NULL, "spec_spectrum", NULL, "2014",
	"Escape From Colony 8 (48K) (HB)\0", NULL, "usebox.net", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_RUNGUN | GBF_MAZE, 0,
	SpectrumGetZipName, SpecEsccolonyRomInfo, SpecEsccolonyRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Escape from M.O.N.J.A.S. (English) (48K) (HB)

static struct BurnRomInfo SpecEscapemonjasenRomDesc[] = {
	{ "Escape from M.O.N.J.A.S. EN (2021)(Rastersoft).tap", 48592, 0x4a03ef89, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecEscapemonjasen, SpecEscapemonjasen, Spectrum)
STD_ROM_FN(SpecEscapemonjasen)

struct BurnDriver BurnSpecEscapemonjasen = {
	"spec_escapemonjasen", NULL, "spec_spectrum", NULL, "2021",
	"Escape from M.O.N.J.A.S. (English) (48K) (HB)\0", NULL, "Rastersoft", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_ADV, 0,
	SpectrumGetZipName, SpecEscapemonjasenRomInfo, SpecEscapemonjasenRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Escape from M.O.N.J.A.S. (Spanish) (48K) (HB)

static struct BurnRomInfo SpecEscapemonjasesRomDesc[] = {
	{ "Escape from M.O.N.J.A.S. ES (2021)(Rastersoft).tap", 48592, 0x6ef28d8a, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecEscapemonjases, SpecEscapemonjases, Spectrum)
STD_ROM_FN(SpecEscapemonjases)

struct BurnDriver BurnSpecEscapemonjases = {
	"spec_escapemonjases", "spec_escapemonjasen", "spec_spectrum", NULL, "2021",
	"Escape from M.O.N.J.A.S. (Spanish) (48K) (HB)\0", NULL, "Rastersoft", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_ADV, 0,
	SpectrumGetZipName, SpecEscapemonjasesRomInfo, SpecEscapemonjasesRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Esta en el Pantano (Spanish) (48K) (HB)

static struct BurnRomInfo SpecEstapantanoRomDesc[] = {
	{ "Esta en el Pantano ES 48K (2009)(RELEVO).tap", 36408, 0x5e84ce2f, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecEstapantano, SpecEstapantano, Spectrum)
STD_ROM_FN(SpecEstapantano)

struct BurnDriver BurnSpecEstapantano = {
	"spec_estapantano", NULL, "spec_spectrum", NULL, "2009",
	"Esta en el Pantano (Spanish) (48K) (HB)\0", NULL, "RELEVO", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_ADV, 0,
	SpectrumGetZipName, SpecEstapantanoRomInfo, SpecEstapantanoRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Eugene - Lord of the Bathroom (48K) (HB)

static struct BurnRomInfo SpecmminrelbRomDesc[] = {
	{ "Manic Miner - Eugene - Lord of the Bathroom (1999)(Manic Miner Technologies).z80", 0x07792, 0x3062e7d8, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specmminrelb, Specmminrelb, Spectrum)
STD_ROM_FN(Specmminrelb)

struct BurnDriver BurnSpecmminrelb = {
	"spec_mminrelb", NULL, "spec_spectrum", NULL, "1999",
	"Eugene - Lord of the Bathroom (48K) (HB)\0", NULL, "Manic Miner Technologies", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecmminrelbRomInfo, SpecmminrelbRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Exodron (48K) (HB)

static struct BurnRomInfo SpecExodronRomDesc[] = {
	{ "Exodron 48K (2024)(PixelSoft).tap", 43090, 0x6859f910, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecExodron, SpecExodron, Spectrum)
STD_ROM_FN(SpecExodron)

struct BurnDriver BurnSpecExodron = {
	"spec_exodron", NULL, "spec_spectrum", NULL, "2024",
	"Exodron (48K) (HB)\0", NULL, "PixelSoft", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecExodronRomInfo, SpecExodronRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// EXO-TICAL (English) (128K) (HB)

static struct BurnRomInfo SpecExoticalenRomDesc[] = {
	{ "EXO-TICAL 128K EN (2023)(Roolandoo).tap", 50368, 0x4a942675, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecExoticalen, SpecExoticalen, Spec128)
STD_ROM_FN(SpecExoticalen)

struct BurnDriver BurnSpecExoticalen = {
	"spec_exoticalen", NULL, "spec_spec128", NULL, "2023",
	"EXO-TICAL (English) (128K) (HB)\0", NULL, "Roolandoo", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_SHOOT, 0,
	SpectrumGetZipName, SpecExoticalenRomInfo, SpecExoticalenRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// EXO-TICAL (Spanish) (128K) (HB)

static struct BurnRomInfo SpecExoticalesRomDesc[] = {
	{ "EXO-TICAL 128K ES (2023)(Roolandoo).tap", 50624, 0xb5a3b89f, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecExoticales, SpecExoticales, Spec128)
STD_ROM_FN(SpecExoticales)

struct BurnDriver BurnSpecExoticales = {
	"spec_exoticales", "spec_exoticalen", "spec_spec128", NULL, "2023",
	"EXO-TICAL (Spanish) (128K) (HB)\0", NULL, "Roolandoo", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_SHOOT, 0,
	SpectrumGetZipName, SpecExoticalesRomInfo, SpecExoticalesRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Explorer (48K) (HB)

static struct BurnRomInfo SpecExplorerRomDesc[] = {
	{ "Explorer 48K (2015)(Luca Bordoni).tap", 31320, 0x1b3a7a81, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecExplorer, SpecExplorer, Spectrum)
STD_ROM_FN(SpecExplorer)

struct BurnDriver BurnSpecExplorer = {
	"spec_explorer", NULL, "spec_spectrum", NULL, "2015",
	"Explorer (48K) (HB)\0", NULL, "Luca Bordoni", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_VERSHOOT, 0,
	SpectrumGetZipName, SpecExplorerRomInfo, SpecExplorerRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecCursorKeysDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Explorer 2 (48K) (HB)

static struct BurnRomInfo SpecExplorer2RomDesc[] = {
	{ "Explorer 2 48K (2018)(Luca Bordoni).tap", 47170, 0x59c05e82, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecExplorer2, SpecExplorer2, Spectrum)
STD_ROM_FN(SpecExplorer2)

struct BurnDriver BurnSpecExplorer2 = {
	"spec_explorer2", NULL, "spec_spectrum", NULL, "2018",
	"Explorer 2 (48K) (HB)\0", NULL, "Luca Bordoni", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_SHOOT, 0,
	SpectrumGetZipName, SpecExplorer2RomInfo, SpecExplorer2RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Extruder (48K) (HB)

static struct BurnRomInfo SpecExtruderRomDesc[] = {
	{ "Extruder 48K (2018)(Rui Martins).tap", 28574, 0x289733c7, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecExtruder, SpecExtruder, Spectrum)
STD_ROM_FN(SpecExtruder)

struct BurnDriver BurnSpecExtruder = {
	"spec_extruder", NULL, "spec_spectrum", NULL, "2018",
	"Extruder (48K) (HB)\0", "Press 1 or 2 for one/two player mode, 3 or 4 for set options", "Rui Martins", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 2, HARDWARE_SPECTRUM, GBF_PUZZLE, 0,
	SpectrumGetZipName, SpecExtruderRomInfo, SpecExtruderRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Farmer Jack - Treasure Trove (128K) (HB)

static struct BurnRomInfo SpecFjttRomDesc[] = {
	{ "Farmer Jack - Treasure Trove 128K (2008)(Bob Smith).tap", 48403, 0x01f1d44f, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecFjtt, SpecFjtt, Spec128)
STD_ROM_FN(SpecFjtt)

struct BurnDriver BurnSpecFjtt = {
	"spec_fjtt", NULL, "spec_spec128", NULL, "2008",
	"Farmer Jack - Treasure Trove (128K) (HB)\0", NULL, "Bob Smith", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_ACTION | GBF_MAZE, 0,
	SpectrumGetZipName, SpecFjttRomInfo, SpecFjttRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Federation Underwater (English) (128K) (HB)

static struct BurnRomInfo SpecFederuwenRomDesc[] = {
	{ "Federation Underwater EN 128K (2022)(Furillo Productions).tap", 117594, 0x1b111c3f, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecFederuwen, SpecFederuwen, Spec128)
STD_ROM_FN(SpecFederuwen)

struct BurnDriver BurnSpecFederuwen = {
	"spec_federuwen", NULL, "spec_spec128", NULL, "2022",
	"Federation Underwater (English) (128K) (HB)\0", NULL, "Furillo Productions", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_ADV, 0,
	SpectrumGetZipName, SpecFederuwenRomInfo, SpecFederuwenRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Federation Underwater (Spanish) (128K) (HB)

static struct BurnRomInfo SpecFederuwesRomDesc[] = {
	{ "Federation Underwater ES 128K (2022)(Furillo Productions).tap", 117591, 0x21048ce6, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecFederuwes, SpecFederuwes, Spec128)
STD_ROM_FN(SpecFederuwes)

struct BurnDriver BurnSpecFederuwes = {
	"spec_federuwes", "spec_federuwen", "spec_spec128", NULL, "2022",
	"Federation Underwater (Spanish) (128K) (HB)\0", NULL, "Furillo Productions", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_ADV, 0,
	SpectrumGetZipName, SpecFederuwesRomInfo, SpecFederuwesRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Federation Z (English) (48K) (HB)

static struct BurnRomInfo SpecFedzenRomDesc[] = {
	{ "Federation Z EN 48K (2020)(Furillo Productions).tap", 44748, 0xfd72043c, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecFedzen, SpecFedzen, Spectrum)
STD_ROM_FN(SpecFedzen)

struct BurnDriver BurnSpecFedzen = {
	"spec_fedzen", NULL, "spec_spectrum", NULL, "2020",
	"Federation Z (English) (48K) (HB)\0", NULL, "Furillo Productions", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_ADV, 0,
	SpectrumGetZipName, SpecFedzenRomInfo, SpecFedzenRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Federation Z (Spanish) (48K) (HB)

static struct BurnRomInfo SpecFedzesRomDesc[] = {
	{ "Federation Z ES 48K (2020)(Furillo Productions).tap", 44747, 0x10e79d9d, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecFedzes, SpecFedzes, Spectrum)
STD_ROM_FN(SpecFedzes)

struct BurnDriver BurnSpecFedzes = {
	"spec_fedzes", "spec_fedzen", "spec_spectrum", NULL, "2020",
	"Federation Z (Spanish) (48K) (HB)\0", NULL, "Furillo Productions", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_ADV, 0,
	SpectrumGetZipName, SpecFedzesRomInfo, SpecFedzesRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// FilloMania (128K) (HB)

static struct BurnRomInfo SpecFillomaniaRomDesc[] = {
	{ "FilloMania 128K (2018)(g0blinish).tap", 23139, 0x1aee9684, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecFillomania, SpecFillomania, Spec128)
STD_ROM_FN(SpecFillomania)

struct BurnDriver BurnSpecFillomania = {
	"spec_fillomania", NULL, "spec_spec128", NULL, "2018",
	"FilloMania (128K) (HB)\0", NULL, "g0blinish", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PUZZLE, 0,
	SpectrumGetZipName, SpecFillomaniaRomInfo, SpecFillomaniaRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecCursorKeysDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Fitris (128K) (HB)

static struct BurnRomInfo SpecFitrisRomDesc[] = {
	{ "Fitris 128K (2023)(FitoSoft).tap", 39006, 0x6e656131, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecFitris, SpecFitris, Spec128)
STD_ROM_FN(SpecFitris)

struct BurnDriver BurnSpecFitris = {
	"spec_fitris", NULL, "spec_spec128", NULL, "2023",
	"Fitris (128K) (HB)\0", "'S': sound on/off - 'M': music on/off", "Fitosoft", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PUZZLE, 0,
	SpectrumGetZipName, SpecFitrisRomInfo, SpecFitrisRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecQAOPSpaceDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Flood (16K) (HB)

static struct BurnRomInfo SpecFloodRomDesc[] = {
	{ "Flood 16K (2024)(Jonathan Cauldwell).tap", 10333, 0x99e443bd, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecFlood, SpecFlood, Spectrum)
STD_ROM_FN(SpecFlood)

struct BurnDriver BurnSpecFlood = {
	"spec_flood", NULL, "spec_spectrum", NULL, "2024",
	"Flood (16K) (HB)\0", NULL, "Jonathan Cauldwell", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_HORSHOOT, 0,
	SpectrumGetZipName, SpecFloodRomInfo, SpecFloodRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Flype (48K) (HB)

static struct BurnRomInfo SpecFlypeRomDesc[] = {
	{ "Flype 48K (2015)(Repixel8).tap", 41719, 0x2b9a90f2, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecFlype, SpecFlype, Spectrum)
STD_ROM_FN(SpecFlype)

struct BurnDriver BurnSpecFlype = {
	"spec_flype", NULL, "spec_spectrum", NULL, "2015",
	"Flype (48K) (HB)\0", NULL, "Repixel8", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_ACTION, 0,
	SpectrumGetZipName, SpecFlypeRomInfo, SpecFlypeRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Foggy's Quest (128K) (HB)

static struct BurnRomInfo SpecfoggyquestRomDesc[] = {
	{ "Foggy's Quest 128K (2017)(Rucksack Games).tap", 43528, 0x0d624a1c, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specfoggyquest, Specfoggyquest, Spec128)
STD_ROM_FN(Specfoggyquest)

struct BurnDriver BurnSpecfoggyquest = {
	"spec_foggyquest", NULL, "spec_spec128", NULL, "2017",
	"Foggy's Quest (128K) (HB)\0", NULL, "Rucksack Games", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecfoggyquestRomInfo, SpecfoggyquestRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Forest Escape - A Knight's Quest (128K) (HB)

static struct BurnRomInfo SpecForestescapeRomDesc[] = {
	{ "Forest Escape - A Knight's Quest 128K (2023)(IrataHack).tap", 37329, 0xd183be0c, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecForestescape, SpecForestescape, Spec128)
STD_ROM_FN(SpecForestescape)

struct BurnDriver BurnSpecForestescape = {
	"spec_forestescape", NULL, "spec_spec128", NULL, "2023",
	"Forest Escape - A Knight's Quest (128K) (HB)\0", NULL, "IrataHack", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecForestescapeRomInfo, SpecForestescapeRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Forest Raider Cherry (48K) (HB)

static struct BurnRomInfo SpecFrcherryRomDesc[] = {
	{ "Forest Raider Cherry 48K (2010)(Timmy).tap", 34113, 0x62d75d3e, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecFrcherry, SpecFrcherry, Spectrum)
STD_ROM_FN(SpecFrcherry)

struct BurnDriver BurnSpecFrcherry = {
	"spec_frcherry", NULL, "spec_spectrum", NULL, "2010",
	"Forest Raider Cherry (48K) (HB)\0", NULL, "Timmy", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_ACTION, 0,
	SpectrumGetZipName, SpecFrcherryRomInfo, SpecFrcherryRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Forg Wants Love (128K) (HB)

static struct BurnRomInfo SpecForgRomDesc[] = {
	{ "Forg Wants Love 128K (2021)(Bee Bush Games).tap", 26752, 0xd63afeb3, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecForg, SpecForg, Spec128)
STD_ROM_FN(SpecForg)

struct BurnDriver BurnSpecForg = {
	"spec_forg", NULL, "spec_spec128", NULL, "2021",
	"Forg Wants Love (128K) (HB)\0", NULL, "Bee Bush Games", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_ACTION, 0,
	SpectrumGetZipName, SpecForgRomInfo, SpecForgRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Forward to the Past (48K) (HB)

static struct BurnRomInfo SpecForward2pastRomDesc[] = {
	{ "Forward to the Past 48K (2021)(Sloanysoft).tap", 31362, 0xe6b298c4, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecForward2past, SpecForward2past, Spectrum)
STD_ROM_FN(SpecForward2past)

struct BurnDriver BurnSpecForward2past = {
	"spec_forward2past", NULL, "spec_spectrum", NULL, "2021",
	"Forward to the Past (48K) (HB)\0", NULL, "Sloanysoft", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_ACTION, 0,
	SpectrumGetZipName, SpecForward2pastRomInfo, SpecForward2pastRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Frank N Stein Re-booted (48K) (HB)

static struct BurnRomInfo SpecFranknsteinrbtRomDesc[] = {
	{ "Frank N Stein Re-booted 48K (2011)(Colin Stewart).tap", 45250, 0x00dec00d, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecFranknsteinrbt, SpecFranknsteinrbt, Spectrum)
STD_ROM_FN(SpecFranknsteinrbt)

struct BurnDriver BurnSpecFranknsteinrbt = {
	"spec_franknsteinrbt", NULL, "spec_spectrum", NULL, "2011",
	"Frank N Stein Re-booted (48K) (HB)\0", NULL, "Colin Stewart", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecFranknsteinrbtRomInfo, SpecFranknsteinrbtRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Frantic Pengo (128K) (HB)

static struct BurnRomInfo Specfp128kRomDesc[] = {
	{ "Frantic Pengo 128K (2020)(Gabriele Amore).tap", 45927, 0x66c3790a, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specfp128k, Specfp128k, Spec128)
STD_ROM_FN(Specfp128k)

struct BurnDriver BurnSpecfp128k = {
	"spec_fp128k", NULL, "spec_spec128", NULL, "2020",
	"Frantic Pengo (128K) (HB)\0", NULL, "Gabriele Amore", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_ACTION, 0,
	SpectrumGetZipName, Specfp128kRomInfo, Specfp128kRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecQAOPMDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Frantic Pengo 2 (128K) (HB)

static struct BurnRomInfo SpecFpengo2RomDesc[] = {
	{ "Frantic Pengo 2 128K (2022)(Gabriele Amore).tap", 55197, 0x91f7e148, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecFpengo2, SpecFpengo2, Spec128)
STD_ROM_FN(SpecFpengo2)

struct BurnDriver BurnSpecFpengo2 = {
	"spec_fpengo2", NULL, "spec_spec128", NULL, "2022",
	"Frantic Pengo 2 (128K) (HB)\0", NULL, "Gabriele Amore", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_ACTION | GBF_MAZE, 0,
	SpectrumGetZipName, SpecFpengo2RomInfo, SpecFpengo2RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecQAOPMDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Fray Simon (English) (48K) (HB)

static struct BurnRomInfo SpecFraysimoneRomDesc[] = {
	{ "Fray Simon EN 48K (2022)(OI-3).tap", 49623, 0x8bf8e15b, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecFraysimone, SpecFraysimone, Spectrum)
STD_ROM_FN(SpecFraysimone)

struct BurnDriver BurnSpecFraysimone = {
	"spec_fraysimone", NULL, "spec_spectrum", NULL, "2022",
	"Fray Simon (English) (48K) (HB)\0", "Z: change obj - Z+Up: take obj - Z+Down: leave obj - 1: enter caves/gates", "OI-3", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PLATFORM | GBF_ADV, 0,
	SpectrumGetZipName, SpecFraysimoneRomInfo, SpecFraysimoneRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecQAOPSpaceDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Fray Simon (Spanish) (48K) (HB)

static struct BurnRomInfo SpecFraysimonsRomDesc[] = {
	{ "Fray Simon ES 48K (2022)(OI-3).tap", 49602, 0xc7841247, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecFraysimons, SpecFraysimons, Spectrum)
STD_ROM_FN(SpecFraysimons)

struct BurnDriver BurnSpecFraysimons = {
	"spec_fraysimons", "spec_fraysimone", "spec_spectrum", NULL, "2022",
	"Fray Simon (Spanish) (48K) (HB)\0", "Z: change obj - Z+Up: take obj - Z+Down: leave obj - 1: enter caves/gates", "OI-3", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PLATFORM | GBF_ADV, 0,
	SpectrumGetZipName, SpecFraysimonsRomInfo, SpecFraysimonsRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecQAOPSpaceDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Fundamentally Loathsome (48K) (HB)

static struct BurnRomInfo SpecFundloathRomDesc[] = {
	{ "Fundamentally Loathsome 48K (2011)(The Mojon Twins).tap", 46052, 0xc47920b6, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecFundloath, SpecFundloath, Spectrum)
STD_ROM_FN(SpecFundloath)

struct BurnDriver BurnSpecFundloath = {
	"spec_fundloath", NULL, "spec_spectrum", NULL, "2011",
	"Fundamentally Loathsome (48K) (HB)\0", NULL, "The Mojon Twins", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecFundloathRomInfo, SpecFundloathRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecQAOPSpaceDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Funky Fungus Reloaded (German) (128K) (HB)

static struct BurnRomInfo SpecFunfungusdeRomDesc[] = {
	{ "Funky Fungus Reloaded DE 128K (2019-2023)(Zankle Soft).tap", 96207, 0xc7072dda, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecFunfungusde, SpecFunfungusde, Spec128)
STD_ROM_FN(SpecFunfungusde)

struct BurnDriver BurnSpecFunfungusde = {
	"spec_funfungusde", "spec_funfungusen", "spec_spec128", NULL, "2019-2023",
	"Funky Fungus Reloaded (German) (128K) (HB)\0", "Press 'Space' to select and 'Enter' to set options", "Zankle Soft", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW | BDF_CLONE, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecFunfungusdeRomInfo, SpecFunfungusdeRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Funky Fungus Reloaded (English) (128K) (HB)

static struct BurnRomInfo SpecFunfungusenRomDesc[] = {
	{ "Funky Fungus Reloaded EN 128K (2019-2023)(Zankle Soft).tap", 96170, 0xa8a15992, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecFunfungusen, SpecFunfungusen, Spec128)
STD_ROM_FN(SpecFunfungusen)

struct BurnDriver BurnSpecFunfungusen = {
	"spec_funfungusen", NULL, "spec_spec128", NULL, "2019-2023",
	"Funky Fungus Reloaded (English) (128K) (HB)\0", "Press 'Space' to select and 'Enter' to set options", "Zankle Soft", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecFunfungusenRomInfo, SpecFunfungusenRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Funky Fungus Reloaded (Spanish) (128K) (HB)

static struct BurnRomInfo SpecFunfungusesRomDesc[] = {
	{ "Funky Fungus Reloaded ES 128K (2019-2023)(Zankle Soft).tap", 96223, 0x6002d9da, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecFunfunguses, SpecFunfunguses, Spec128)
STD_ROM_FN(SpecFunfunguses)

struct BurnDriver BurnSpecFunfunguses = {
	"spec_funfunguses", "spec_funfungusen", "spec_spec128", NULL, "2019-2023",
	"Funky Fungus Reloaded (Spanish) (128K) (HB)\0", "Press 'Space' to select and 'Enter' to set options", "Zankle Soft", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW | BDF_CLONE, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecFunfungusesRomInfo, SpecFunfungusesRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Funky Fungus Reloaded (French) (128K) (HB)

static struct BurnRomInfo SpecFunfungusfrRomDesc[] = {
	{ "Funky Fungus Reloaded FR 128K (2019-2023)(Zankle Soft).tap", 80892, 0xd453cf6c, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecFunfungusfr, SpecFunfungusfr, Spec128)
STD_ROM_FN(SpecFunfungusfr)

struct BurnDriver BurnSpecFunfungusfr = {
	"spec_funfungusfr", "spec_funfungusen", "spec_spec128", NULL, "2019-2023",
	"Funky Fungus Reloaded (French) (128K) (HB)\0", "Press 'Space' to select and 'Enter' to set options", "Zankle Soft", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW | BDF_CLONE, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecFunfungusfrRomInfo, SpecFunfungusfrRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Funky Fungus Reloaded (Italian) (128K) (HB)

static struct BurnRomInfo SpecFunfungusitRomDesc[] = {
	{ "Funky Fungus Reloaded IT 128K (2019-2023)(Zankle Soft).tap", 96181, 0x0b0e7184, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecFunfungusit, SpecFunfungusit, Spec128)
STD_ROM_FN(SpecFunfungusit)

struct BurnDriver BurnSpecFunfungusit = {
	"spec_funfungusit", "spec_funfungusen", "spec_spec128", NULL, "2019-2023",
	"Funky Fungus Reloaded (Italian) (128K) (HB)\0", "Press 'Space' to select and 'Enter' to set options", "Zankle Soft", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW | BDF_CLONE, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecFunfungusitRomInfo, SpecFunfungusitRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Funky Fungus Reloaded (Portuguese) (128K) (HB)

static struct BurnRomInfo SpecFunfungusptRomDesc[] = {
	{ "Funky Fungus Reloaded PT 128K (2019-2023)(Zankle Soft).tap", 96244, 0x1d23636e, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecFunfunguspt, SpecFunfunguspt, Spec128)
STD_ROM_FN(SpecFunfunguspt)

struct BurnDriver BurnSpecFunfunguspt = {
	"spec_funfunguspt", "spec_funfungusen", "spec_spec128", NULL, "2019-2023",
	"Funky Fungus Reloaded (Portuguese) (128K) (HB)\0", "Press 'Space' to select and 'Enter' to set options", "Zankle Soft", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW | BDF_CLONE, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecFunfungusptRomInfo, SpecFunfungusptRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Gamex (48K) (HB)

static struct BurnRomInfo SpecGamexRomDesc[] = {
	{ "Gamex 48K (2006)(Cronosoft).tap", 46680, 0xd996c4fc, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecGamex, SpecGamex, Spectrum)
STD_ROM_FN(SpecGamex)

struct BurnDriver BurnSpecGamex = {
	"spec_gamex", NULL, "spec_spectrum", NULL, "2006",
	"Gamex (48K) (HB)\0", "2: up, W: down, 9: left, 0: right, M: fire, Z: buy/sell, H: hold", "Cronosoft", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_ACTION, 0,
	SpectrumGetZipName, SpecGamexRomInfo, SpecGamexRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Ganimedes (128K) (HB, v1.01)

static struct BurnRomInfo SpecGanimedesRomDesc[] = {
	{ "Ganimedes v1.01 128K (2023)(Nextric).tap", 59199, 0xc58082f3, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecGanimedes, SpecGanimedes, Spec128)
STD_ROM_FN(SpecGanimedes)

struct BurnDriver BurnSpecGanimedes = {
	"spec_ganimedes", NULL, "spec_spec128", NULL, "2023",
	"Ganimedes (128K) (HB, v1.01)\0", NULL, "Nextric", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecGanimedesRomInfo, SpecGanimedesRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Gem Chaser (48K) (HB)

static struct BurnRomInfo SpecGemchaserRomDesc[] = {
	{ "Gem Chaser 48K (2013)(Electric Wolf).tap", 37361, 0x67791e1d, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecGemchaser, SpecGemchaser, Spectrum)
STD_ROM_FN(SpecGemchaser)

struct BurnDriver BurnSpecGemchaser = {
	"spec_gemchaser", NULL, "spec_spectrum", NULL, "2013",
	"Gem Chaser (48K) (HB)\0", NULL, "Electric Wolf", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PUZZLE, 0,
	SpectrumGetZipName, SpecGemchaserRomInfo, SpecGemchaserRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Gem Chaser 2 (48K) (HB)

static struct BurnRomInfo SpecGemchaser2RomDesc[] = {
	{ "Gem Chaser 2 48K (2013)(Electric Wolf).tap", 45479, 0x3ab12186, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecGemchaser2, SpecGemchaser2, Spectrum)
STD_ROM_FN(SpecGemchaser2)

struct BurnDriver BurnSpecGemchaser2 = {
	"spec_gemchaser2", NULL, "spec_spectrum", NULL, "2013",
	"Gem Chaser 2 (48K) (HB)\0", NULL, "Electric Wolf", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PUZZLE, 0,
	SpectrumGetZipName, SpecGemchaser2RomInfo, SpecGemchaser2RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Genesis dawn of a new day (128K) (HB)

static struct BurnRomInfo SpecGenesisRomDesc[] = {
	{ "Genesis dawn of a new day 128K (2010)(RetroWorks).tap", 63025, 0x3a5fb7db, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecGenesis, SpecGenesis, Spec128)
STD_ROM_FN(SpecGenesis)

struct BurnDriver BurnSpecGenesis = {
	"spec_genesis", NULL, "spec_spec128", NULL, "2010",
	"Genesis dawn of a new day (128K) (HB)\0", NULL, "RetroWorks", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_HORSHOOT, 0,
	SpectrumGetZipName, SpecGenesisRomInfo, SpecGenesisRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Get Out of Mars (128K) (HB, v1.1)

static struct BurnRomInfo SpecGetmarsRomDesc[] = {
	{ "Get Out of Mars v1.1 128K (2021)(Noentiendo).tap", 69762, 0xc699ac83, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecGetmars, SpecGetmars, Spec128)
STD_ROM_FN(SpecGetmars)

struct BurnDriver BurnSpecGetmars = {
	"spec_getmars", NULL, "spec_spec128", NULL, "2021",
	"Get Out of Mars (128K) (HB, v1.1)\0", NULL, "Noentiendo", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecGetmarsRomInfo, SpecGetmarsRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Gherbert Groundhog in Save the Date (48K) (HB)

static struct BurnRomInfo SpecGherbgroundRomDesc[] = {
	{ "Gherbert Groundhog in Save the Date 48K (2022)(PuttyCAD & Psychic Parrot).tap", 45411, 0x13f11e02, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecGherbground, SpecGherbground, Spectrum)
STD_ROM_FN(SpecGherbground)

struct BurnDriver BurnSpecGherbground = {
	"spec_gherbground", NULL, "spec_spectrum", NULL, "2022",
	"Gherbert Groundhog in Save the Date (48K) (HB)\0", NULL, "PuttyCAD & Psychic Parrot", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecGherbgroundRomInfo, SpecGherbgroundRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Ghost Castle (48K) (HB)

static struct BurnRomInfo SpecGcastleRomDesc[] = {
	{ "Ghost Castle 48K (2010)(CodenameV).tap", 24178, 0xfd1d14b9, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecGcastle, SpecGcastle, Spectrum)
STD_ROM_FN(SpecGcastle)

struct BurnDriver BurnSpecGcastle = {
	"spec_gcastle", NULL, "spec_spectrum", NULL, "2010",
	"Ghost Castle (48K) (HB)\0", NULL, "CodenameV", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_ACTION | GBF_PUZZLE, 0,
	SpectrumGetZipName, SpecGcastleRomInfo, SpecGcastleRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Ghost Castle 2 (48K) (HB)

static struct BurnRomInfo SpecGcastle2RomDesc[] = {
	{ "Ghost Castle 2 48K (2010)(The Bog Brothers).tap", 34385, 0x0476faff, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecGcastle2, SpecGcastle2, Spectrum)
STD_ROM_FN(SpecGcastle2)

struct BurnDriver BurnSpecGcastle2 = {
	"spec_gcastle2", NULL, "spec_spectrum", NULL, "2010",
	"Ghost Castle 2 (48K) (HB)\0", NULL, "The Bog Brothers", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_ACTION | GBF_ADV, 0,
	SpectrumGetZipName, SpecGcastle2RomInfo, SpecGcastle2RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Ghost Castle 2 - Special Edition (48K) (HB)

static struct BurnRomInfo SpecGcastle2seRomDesc[] = {
	{ "Ghost Castle 2 - Special Edition 48K (2011)(Cronosoft).tap", 37978, 0x6437856c, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecGcastle2se, SpecGcastle2se, Spectrum)
STD_ROM_FN(SpecGcastle2se)

struct BurnDriver BurnSpecGcastle2se = {
	"spec_gcastle2se", NULL, "spec_spectrum", NULL, "2011",
	"Ghost Castle 2 - Special Edition (48K) (HB)\0", NULL, "Cronosoft", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_ACTION | GBF_ADV, 0,
	SpectrumGetZipName, SpecGcastle2seRomInfo, SpecGcastle2seRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// ghoSte's Grunge (128K) (HB)

static struct BurnRomInfo SpecGhosteRomDesc[] = {
	{ "ghoSte's Grunge 128K (2022)(Buck).tap", 38696, 0x95860a70, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecGhoste, SpecGhoste, Spec128)
STD_ROM_FN(SpecGhoste)

struct BurnDriver BurnSpecGhoste = {
	"spec_ghoste", NULL, "spec_spec128", NULL, "2022",
	"ghoSte's Grunge (128K) (HB)\0", NULL, "Buck", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecGhosteRomInfo, SpecGhosteRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecCursorKeysDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Ghostly Capers (128K) (HB)

static struct BurnRomInfo SpecGhostlycapersRomDesc[] = {
	{ "Ghostly Capers 128K (2023)(Chopz).tap", 47437, 0x60003b1e, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecGhostlycapers, SpecGhostlycapers, Spec128)
STD_ROM_FN(SpecGhostlycapers)

struct BurnDriver BurnSpecGhostlycapers = {
	"spec_ghostlycapers", NULL, "spec_spec128", NULL, "2023",
	"Ghostly Capers (128K) (HB)\0", NULL, "Chopz", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecGhostlycapersRomInfo, SpecGhostlycapersRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Gilligans Mine (128K) (HB, beta v1.04.1)

static struct BurnRomInfo SpecGilliminebRomDesc[] = {
	{ "Gilligans Mine - beta v1.04.1 128K (2024)(Clebin Games).tap", 63187, 0xddf095c9, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecGillimineb, SpecGillimineb, Spec128)
STD_ROM_FN(SpecGillimineb)

struct BurnDriver BurnSpecGillimineb = {
	"spec_gillimineb", NULL, "spec_spec128", NULL, "2024",
	"Gilligans Mine (128K) (HB, beta v1.04.1)\0", NULL, "Clebin Games", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecGilliminebRomInfo, SpecGilliminebRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Gimme Bright (48K) (HB, v2.0)

static struct BurnRomInfo SpecGimmebrightRomDesc[] = {
	{ "Gimme Bright v2.0 (2011)(Climacus).tap", 31391, 0xd58d7107, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecGimmebright, SpecGimmebright, Spec128)
STD_ROM_FN(SpecGimmebright)

struct BurnDriver BurnSpecGimmebright = {
	"spec_gimmebright", NULL, "spec_spec128", NULL, "2011",
	"Gimme Bright (48K) (HB, v2.0)\0", NULL, "Climacus", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecGimmebrightRomInfo, SpecGimmebrightRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecQAOPSpaceDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// GlaZX (128K) (HB)

static struct BurnRomInfo SpecGlazx128RomDesc[] = {
	{ "GlaZX 128K (2019)(EugenyN).tap", 33322, 0x0160767e, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecGlazx128, SpecGlazx128, Spec128)
STD_ROM_FN(SpecGlazx128)

struct BurnDriver BurnSpecGlazx128 = {
	"spec_glazx128", NULL, "spec_spec128", NULL, "2019",
	"GlaZX (128K) (HB)\0", "2 players mode only", "EugenyN", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 2, HARDWARE_SPECTRUM, GBF_ACTION, 0,
	SpectrumGetZipName, SpecGlazx128RomInfo, SpecGlazx128RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// GlaZX (48K) (HB)

static struct BurnRomInfo SpecGlazx48RomDesc[] = {
	{ "GlaZX 48K (2019)(EugenyN).tap", 33710, 0x262b66f7, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecGlazx48, SpecGlazx48, Spectrum)
STD_ROM_FN(SpecGlazx48)

struct BurnDriver BurnSpecGlazx48 = {
	"spec_glazx48", "spec_glazx128", "spec_spectrum", NULL, "2019",
	"GlaZX (48K) (HB)\0", "2 players mode only", "EugenyN", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE | BDF_HOMEBREW, 2, HARDWARE_SPECTRUM, GBF_ACTION, 0,
	SpectrumGetZipName, SpecGlazx48RomInfo, SpecGlazx48RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Globus (Spanish) (48K) (HB)

static struct BurnRomInfo SpecGlobusRomDesc[] = {
	{ "Globus 48K (2017)(Sanchis, Paco Lafuente).tap", 48419, 0xb9eed88e, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecGlobus, SpecGlobus, Spectrum)
STD_ROM_FN(SpecGlobus)

struct BurnDriver BurnSpecGlobus = {
	"spec_globus", NULL, "spec_spectrum", NULL, "1987-2017",
	"Globus (Spanish) (48K) (HB)\0", "Unreleased game from 1987", "Paco Lafuente Sanchis", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_MAZE | GBF_ACTION, 0,
	SpectrumGetZipName, SpecGlobusRomInfo, SpecGlobusRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecQAOPMDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Gloop Troops (48K) (HB)

static struct BurnRomInfo SpecGlooptroopRomDesc[] = {
	{ "Gloop Troops 48K (2011)(Little Shop of Pixels).tap", 45168, 0xee3f924f, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecGlooptroop, SpecGlooptroop, Spectrum)
STD_ROM_FN(SpecGlooptroop)

struct BurnDriver BurnSpecGlooptroop = {
	"spec_glooptroop", NULL, "spec_spectrum", NULL, "2011",
	"Gloop Troops (48K) (HB)\0", "X: abort current game", "Little Shop of Pixels", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 2, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecGlooptroopRomInfo, SpecGlooptroopRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Gloop Troops - New Maps (48K) (HB)

static struct BurnRomInfo SpecGlooptroopnmRomDesc[] = {
	{ "Gloop Troops - New Maps 48K (2012)(Little Shop of Pixels).tap", 47449, 0x4babbad2, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecGlooptroopnm, SpecGlooptroopnm, Spectrum)
STD_ROM_FN(SpecGlooptroopnm)

struct BurnDriver BurnSpecGlooptroopnm = {
	"spec_glooptroopnm", "spec_glooptroop", "spec_spectrum", NULL, "2012",
	"Gloop Troops - New Maps (48K) (HB)\0", "X: abort current game", "Little Shop of Pixels", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE | BDF_HOMEBREW, 2, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecGlooptroopnmRomInfo, SpecGlooptroopnmRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Gluf (RetroSouls) (128K) (HB)

static struct BurnRomInfo SpecglufRomDesc[] = {
	{ "Gluf 128K (2019)(RetroSouls).tap",  34669, 0x52be5093, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specgluf, Specgluf, Spec128)
STD_ROM_FN(Specgluf)

struct BurnDriver BurnSpecgluf = {
	"spec_gluf", NULL, "spec_spec128", NULL, "2019",
	"GLUF (128K) (HB)\0", NULL, "RetroSouls", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PUZZLE, 0,
	SpectrumGetZipName, SpecglufRomInfo, SpecglufRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// GNONI 2020 (Spanish) (48K) (HB, v1.2)

static struct BurnRomInfo SpecGnoniRomDesc[] = {
	{ "GNONI v1.2 ES 48K (2020)(Azimov).tap", 26212, 0xbeb0553a, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecGnoni, SpecGnoni, Spectrum)
STD_ROM_FN(SpecGnoni)

struct BurnDriver BurnSpecGnoni = {
	"spec_gnoni", NULL, "spec_spectrum", NULL, "2020",
	"GNONI 2020 (Spanish) (48K) (HB, v1.2)\0", NULL, "Azimov", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_MAZE | GBF_ACTION, 0,
	SpectrumGetZipName, SpecGnoniRomInfo, SpecGnoniRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Goblin, The (English) (128K) (HB)

static struct BurnRomInfo SpecGoblinenRomDesc[] = {
	{ "Goblin, The EN 128K (2022)(Hicks).tap", 46960, 0xaaddbdc4, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecGoblinen, SpecGoblinen, Spec128)
STD_ROM_FN(SpecGoblinen)

struct BurnDriver BurnSpecGoblinen = {
	"spec_goblinen", NULL, "spec_spec128", NULL, "2022",
	"Goblin, The (English) (128K) (HB)\0", NULL, "Hicks", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecGoblinenRomInfo, SpecGoblinenRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Goblin, The (Spanish) (128K) (HB)

static struct BurnRomInfo SpecGoblinesRomDesc[] = {
	{ "Goblin, The ES 128K (2022)(Hicks).tap", 46999, 0x80e04163, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecGoblines, SpecGoblines, Spec128)
STD_ROM_FN(SpecGoblines)

struct BurnDriver BurnSpecGoblines = {
	"spec_goblines", "spec_goblinen", "spec_spec128", NULL, "2022",
	"Goblin, The (Spanish) (128K) (HB)\0", NULL, "Hicks", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecGoblinesRomInfo, SpecGoblinesRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Godkiller v2 (New Timeline Edition) (128K) (HB)

static struct BurnRomInfo SpecGodkillerRomDesc[] = {
	{ "Godkiller v2 128K (2019)(APSIS).tap", 57216, 0xba14a5b9, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecGodkiller, SpecGodkiller, Spec128)
STD_ROM_FN(SpecGodkiller)

struct BurnDriver BurnSpecGodkiller = {
	"spec_godkiller", NULL, "spec_spec128", NULL, "2019",
	"Godkiller - New Timeline Edition (128K) (HB, v2)\0", NULL, "APSIS", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_RUNGUN, 0,
	SpectrumGetZipName, SpecGodkillerRomInfo, SpecGodkillerRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Godkiller 2 Exile v2 (New Timeline Edition) (128K) (HB)

static struct BurnRomInfo SpecGodkiller2RomDesc[] = {
	{ "Godkiller 2 Exile v2 128K (2020)(APSIS).tap", 59462, 0xb5002734, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecGodkiller2, SpecGodkiller2, Spec128)
STD_ROM_FN(SpecGodkiller2)

struct BurnDriver BurnSpecGodkiller2 = {
	"spec_godkiller2", NULL, "spec_spec128", NULL, "2020",
	"Godkiller 2 Exile - New Timeline Edition (128K) (HB, v2)\0", NULL, "APSIS", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_RUNGUN, 0,
	SpectrumGetZipName, SpecGodkiller2RomInfo, SpecGodkiller2RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Godkiller 2 Exile intro v2 (New Timeline Edition) (128K) (HB)

static struct BurnRomInfo SpecGodkiller2introRomDesc[] = {
	{ "Godkiller 2 Exile intro v2 128K (2020)(APSIS).tap", 42380, 0x0fde1ae6, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecGodkiller2intro, SpecGodkiller2intro, Spec128)
STD_ROM_FN(SpecGodkiller2intro)

struct BurnDriver BurnSpecGodkiller2intro = {
	"spec_godkiller2intro", "spec_godkiller2", "spec_spec128", NULL, "2020",
	"Godkiller 2 Exile - New Timeline Edition - INTRO (128K) (HB, v2)\0", NULL, "APSIS", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_RUNGUN, 0,
	SpectrumGetZipName, SpecGodkiller2introRomInfo, SpecGodkiller2introRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Godkiller intro v2 (New Timeline Edition) (128K) (HB)

static struct BurnRomInfo SpecGodkillerintroRomDesc[] = {
	{ "Godkiller intro v2 128K (2019)(APSIS).tap", 35443, 0xa1957fa3, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecGodkillerintro, SpecGodkillerintro, Spec128)
STD_ROM_FN(SpecGodkillerintro)

struct BurnDriver BurnSpecGodkillerintro = {
	"spec_godkillerintro", "spec_godkiller", "spec_spec128", NULL, "2019",
	"Godkiller - New Timeline Edition - INTRO (128K) (HB, v2)\0", NULL, "APSIS", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_RUNGUN, 0,
	SpectrumGetZipName, SpecGodkillerintroRomInfo, SpecGodkillerintroRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Gommy Defensor Medieval (English) (128K) (HB)

static struct BurnRomInfo SpecGommyenRomDesc[] = {
	{ "Gommy Defensor Medieval EN 128K (2009)(RetroWorks).tap", 43940, 0xc2c41f87, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecGommyen, SpecGommyen, Spec128)
STD_ROM_FN(SpecGommyen)

struct BurnDriver BurnSpecGommyen = {
	"spec_gommyen", NULL, "spec_spec128", NULL, "2009",
	"Gommy Defensor Medieval (English) (128K) (HB)\0", NULL, "RetroWorks", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_ACTION, 0,
	SpectrumGetZipName, SpecGommyenRomInfo, SpecGommyenRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecQAOPSpaceDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Gommy Defensor Medieval (Spanish) (128K) (HB)

static struct BurnRomInfo SpecGommyesRomDesc[] = {
	{ "Gommy Defensor Medieval ES 128K (2009)(RetroWorks).tap", 43943, 0x53a9e2bf, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecGommyes, SpecGommyes, Spec128)
STD_ROM_FN(SpecGommyes)

struct BurnDriver BurnSpecGommyes = {
	"spec_gommyes", "spec_gommyen", "spec_spec128", NULL, "2009",
	"Gommy Defensor Medieval (Spanish) (128K) (HB)\0", NULL, "RetroWorks", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_ACTION, 0,
	SpectrumGetZipName, SpecGommyesRomInfo, SpecGommyesRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecQAOPSpaceDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Goobbler (48K) (HB)

static struct BurnRomInfo SpecGoobblerRomDesc[] = {
	{ "Goobbler 48K-AY (2024)(Kraptol).tap", 45480, 0xb150f069, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecGoobbler, SpecGoobbler, Spectrum)
STD_ROM_FN(SpecGoobbler)

struct BurnDriver BurnSpecGoobbler = {
	"spec_goobbler", NULL, "spec_spectrum", NULL, "2024",
	"Goobbler (48K) (HB)\0", "AY sound supported", "Kraptol", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecGoobblerRomInfo, SpecGoobblerRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Good Night Kanga (128K) (HB)

static struct BurnRomInfo SpecGnkangaRomDesc[] = {
	{ "Good Night Kanga 128K (2016)(Gabriele Amore - Factor 6).tap", 27329, 0x3e2bacae, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecGnkanga, SpecGnkanga, Spec128)
STD_ROM_FN(SpecGnkanga)

struct BurnDriver BurnSpecGnkanga = {
	"spec_gnkanga", NULL, "spec_spec128", NULL, "2016",
	"Good Night Kanga (128K) (HB)\0", NULL, "Gabriele Amore - Factor 6", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_ACTION, 0,
	SpectrumGetZipName, SpecGnkangaRomInfo, SpecGnkangaRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Goody Recoloured (128K) (Hack)

static struct BurnRomInfo SpecgoodyrecolourRomDesc[] = {
	{ "Goody Recoloured 128K (1987-2012)(Rafal Miazga).tap", 0x0dbb2, 0x8b02e960, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specgoodyrecolour, Specgoodyrecolour, Spec128)
STD_ROM_FN(Specgoodyrecolour)

struct BurnDriver BurnSpecgoodyrecolour = {
	"spec_goodyrecolour", "spec_goodyes", "spec_spec128", NULL, "2012",
	"Goody Recoloured (128K) (Hack)\0", NULL, "Rafal Miazga", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE | BDF_HACK, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecgoodyrecolourRomInfo, SpecgoodyrecolourRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecQAOPSpaceDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// GraviBots (RetroSouls) (128K) (HB)

static struct BurnRomInfo SpecgravibotsRomDesc[] = {
	{ "GraviBots 128K (2014)(RetroSouls).z80",  29436, 0x375bfa1c, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specgravibots, Specgravibots, Spec128)
STD_ROM_FN(Specgravibots)

struct BurnDriver BurnSpecgravibots = {
	"spec_gravibots", NULL, "spec_spec128", NULL, "2014",
	"GraviBots (128K) (HB)\0", "Press 'Space' and 'Enter' to select options", "RetroSouls", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_ACTION, 0,
	SpectrumGetZipName, SpecgravibotsRomInfo, SpecgravibotsRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Grazers (48K) (HB)

static struct BurnRomInfo SpecGrazersRomDesc[] = {
	{ "Grazers 48K (2024)(Snauts).tap", 16925, 0x867debf1, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecGrazers, SpecGrazers, Spectrum)
STD_ROM_FN(SpecGrazers)

struct BurnDriver BurnSpecGrazers = {
	"spec_grazers", NULL, "spec_spectrum", NULL, "2024",
	"Grazers (48K) (HB)\0", NULL, "Snauts", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_ACTION, 0,
	SpectrumGetZipName, SpecGrazersRomInfo, SpecGrazersRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecQAOPSpaceDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// GVolcano (128K) (HB)

static struct BurnRomInfo SpecGvolcanoRomDesc[] = {
	{ "GVolcano 128K (2022)(xavisan).tap", 39024, 0x0b6fe4c5, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecGvolcano, SpecGvolcano, Spec128)
STD_ROM_FN(SpecGvolcano)

struct BurnDriver BurnSpecGvolcano = {
	"spec_gvolcano", NULL, "spec_spec128", NULL, "2022",
	"GVolcano (128K) (HB)\0", NULL, "xavisan", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecGvolcanoRomInfo, SpecGvolcanoRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// H.E.R.O. Returns (128K) (HB)

static struct BurnRomInfo SpecHeroret128RomDesc[] = {
	{ "H.E.R.O. Returns 128K (2020)(Gusivision).tap", 72143, 0x61365fea, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecHeroret128, SpecHeroret128, Spec128)
STD_ROM_FN(SpecHeroret128)

struct BurnDriver BurnSpecHeroret128 = {
	"spec_heroret128", NULL, "spec_spec128", NULL, "2020",
	"H.E.R.O. Returns (128K) (HB)\0", NULL, "Gusivision", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecHeroret128RomInfo, SpecHeroret128RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecQAOPSpaceDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// H.E.R.O. Returns (48K) (HB)

static struct BurnRomInfo SpecHeroret48RomDesc[] = {
	{ "H.E.R.O. Returns 48K (2020)(Gusivision).tap", 47067, 0xba38117a, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecHeroret48, SpecHeroret48, Spectrum)
STD_ROM_FN(SpecHeroret48)

struct BurnDriver BurnSpecHeroret48 = {
	"spec_heroret48", "spec_heroret128", "spec_spectrum", NULL, "2020",
	"H.E.R.O. Returns (48K) (HB)\0", NULL, "Gusivision", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecHeroret48RomInfo, SpecHeroret48RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecQAOPSpaceDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Hairy Fly, The (128K) (HB)

static struct BurnRomInfo SpecHairyflyRomDesc[] = {
	{ "Hairy Fly, The 128K (2023)(Chopz).tap", 48184, 0xb1ef3fc2, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecHairyfly, SpecHairyfly, Spec128)
STD_ROM_FN(SpecHairyfly)

struct BurnDriver BurnSpecHairyfly = {
	"spec_hairyfly", NULL, "spec_spec128", NULL, "2023",
	"Hairy Fly, The (128K) (HB)\0", NULL, "Chopz", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecHairyflyRomInfo, SpecHairyflyRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Hakkenkast - Dungeon 1 (128K) (HB)

static struct BurnRomInfo SpecHakken128aRomDesc[] = {
	{ "Hakkenkast - Dungeon 1 128K (2022)(Minilop).tap", 49643, 0xbec43b92, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecHakken128a, SpecHakken128a, Spec128)
STD_ROM_FN(SpecHakken128a)

struct BurnDriver BurnSpecHakken128a = {
	"spec_hakken128a", NULL, "spec_spec128", NULL, "2022",
	"Hakkenkast - Dungeon 1 (128K) (HB)\0", "Use Q, A and Space to set options", "Minilop", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_MAZE | GBF_ACTION, 0,
	SpectrumGetZipName, SpecHakken128aRomInfo, SpecHakken128aRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Hakkenkast - Dungeon 1 (48K) (HB)

static struct BurnRomInfo SpecHakken48aRomDesc[] = {
	{ "Hakkenkast - Dungeon 1 48K (2022)(Minilop).tap", 46005, 0x083b55be, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecHakken48a, SpecHakken48a, Spectrum)
STD_ROM_FN(SpecHakken48a)

struct BurnDriver BurnSpecHakken48a = {
	"spec_hakken48a", "spec_hakken128a", "spec_spectrum", NULL, "2022",
	"Hakkenkast - Dungeon 1 (48K) (HB)\0", "Use Q, A and Space to set options", "Minilop", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_MAZE | GBF_ACTION, 0,
	SpectrumGetZipName, SpecHakken48aRomInfo, SpecHakken48aRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Hakkenkast - Dungeon 2 (128K) (HB)

static struct BurnRomInfo SpecHakken128bRomDesc[] = {
	{ "Hakkenkast - Dungeon 2 128K (2022)(Minilop).tap", 49420, 0xaf1c0ee4, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecHakken128b, SpecHakken128b, Spec128)
STD_ROM_FN(SpecHakken128b)

struct BurnDriver BurnSpecHakken128b = {
	"spec_hakken128b", NULL, "spec_spec128", NULL, "2022",
	"Hakkenkast - Dungeon 2 (128K) (HB)\0", "Use Q, A and Space to set options", "Minilop", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_MAZE | GBF_ACTION, 0,
	SpectrumGetZipName, SpecHakken128bRomInfo, SpecHakken128bRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Hakkenkast - Dungeon 2 (48K) (HB)

static struct BurnRomInfo SpecHakken48bRomDesc[] = {
	{ "Hakkenkast - Dungeon 2 48K (2022)(Minilop).tap", 46126, 0xa2fc72df, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecHakken48b, SpecHakken48b, Spectrum)
STD_ROM_FN(SpecHakken48b)

struct BurnDriver BurnSpecHakken48b = {
	"spec_hakken48b", "spec_hakken128b", "spec_spectrum", NULL, "2022",
	"Hakkenkast - Dungeon 2 (48K) (HB)\0", "Use Q, A and Space to set options", "Minilop", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_MAZE | GBF_ACTION, 0,
	SpectrumGetZipName, SpecHakken48bRomInfo, SpecHakken48bRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};


// Hakkenkast - Dungeon 3 (128K) (HB)

static struct BurnRomInfo SpecHakken128cRomDesc[] = {
	{ "Hakkenkast - Dungeon 3 128K (2022)(Minilop).tap", 49845, 0x2595e9dc, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecHakken128c, SpecHakken128c, Spec128)
STD_ROM_FN(SpecHakken128c)

struct BurnDriver BurnSpecHakken128c = {
	"spec_hakken128c", NULL, "spec_spec128", NULL, "2022",
	"Hakkenkast - Dungeon 3 (128K) (HB)\0", "Use Q, A and Space to set options", "Minilop", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_MAZE | GBF_ACTION, 0,
	SpectrumGetZipName, SpecHakken128cRomInfo, SpecHakken128cRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Hakkenkast - Dungeon 3 (48K) (HB)

static struct BurnRomInfo SpecHakken48cRomDesc[] = {
	{ "Hakkenkast - Dungeon 3 48K (2022)(Minilop).tap", 46019, 0x614c37f6, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecHakken48c, SpecHakken48c, Spectrum)
STD_ROM_FN(SpecHakken48c)

struct BurnDriver BurnSpecHakken48c = {
	"spec_hakken48c", "spec_hakken128c", "spec_spectrum", NULL, "2022",
	"Hakkenkast - Dungeon 3 (48K) (HB)\0", "Use Q, A and Space to set options", "Minilop", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_MAZE | GBF_ACTION, 0,
	SpectrumGetZipName, SpecHakken48cRomInfo, SpecHakken48cRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Hakkenkast - Dungeon 4 (128K) (HB)

static struct BurnRomInfo SpecHakken128dRomDesc[] = {
	{ "Hakkenkast - Dungeon 4 128K (2022)(Minilop).tap", 49395, 0x5f2756d0, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecHakken128d, SpecHakken128d, Spec128)
STD_ROM_FN(SpecHakken128d)

struct BurnDriver BurnSpecHakken128d = {
	"spec_hakken128d", NULL, "spec_spec128", NULL, "2022",
	"Hakkenkast - Dungeon 4 (128K) (HB)\0", "Use Q, A and Space to set options", "Minilop", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_MAZE | GBF_ACTION, 0,
	SpectrumGetZipName, SpecHakken128dRomInfo, SpecHakken128dRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Hakkenkast - Dungeon 4 (48K) (HB)

static struct BurnRomInfo SpecHakken48dRomDesc[] = {
	{ "Hakkenkast - Dungeon 4 48K (2022)(Minilop).tap", 46072, 0x3ad9cea9, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecHakken48d, SpecHakken48d, Spectrum)
STD_ROM_FN(SpecHakken48d)

struct BurnDriver BurnSpecHakken48d = {
	"spec_hakken48d", "spec_hakken128d", "spec_spectrum", NULL, "2022",
	"Hakkenkast - Dungeon 4 (48K) (HB)\0", "Use Q, A and Space to set options", "Minilop", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_MAZE | GBF_ACTION, 0,
	SpectrumGetZipName, SpecHakken48dRomInfo, SpecHakken48dRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Hallowed Knight (48K) (HB, v1.4)

static struct BurnRomInfo SpecHknightRomDesc[] = {
	{ "Hallowed Knight v1.4 48K (2021)(EJVG).tap", 46199, 0x489c76cc, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecHknight, SpecHknight, Spectrum)
STD_ROM_FN(SpecHknight)

struct BurnDriver BurnSpecHknight = {
	"spec_hknight", NULL, "spec_spectrum", NULL, "2021",
	"Hallowed Knight (48K) (HB, v1.4)\0", NULL, "EJVG", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecHknightRomInfo, SpecHknightRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Haoken (English) (128K) (HB)

static struct BurnRomInfo SpecHaokeneRomDesc[] = {
	{ "Haoken EN 128K (2023)(Hicks).tap", 50063, 0xee57e531, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecHaokene, SpecHaokene, Spec128)
STD_ROM_FN(SpecHaokene)

struct BurnDriver BurnSpecHaokene = {
	"spec_haokene", NULL, "spec_spec128", NULL, "2023",
	"Haoken (English) (128K) (HB)\0", NULL, "Hicks", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecHaokeneRomInfo, SpecHaokeneRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Haoken (Spanish) (128K) (HB)

static struct BurnRomInfo SpecHaokensRomDesc[] = {
	{ "Haoken ES 128K (2023)(Hicks).tap", 50024, 0xd30c2af6, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecHaokens, SpecHaokens, Spec128)
STD_ROM_FN(SpecHaokens)

struct BurnDriver BurnSpecHaokens = {
	"spec_haokens", "spec_haokene", "spec_spec128", NULL, "2023",
	"Haoken (Spanish) (128K) (HB)\0", NULL, "Hicks", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecHaokensRomInfo, SpecHaokensRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Harbinger 2: The Void - Tale Intro (128K) (HB)

static struct BurnRomInfo SpecHarb2introRomDesc[] = {
	{ "Harbinger 2 The Void, Intro 128K (2018)(APSIS).tap", 42380, 0x07570650, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecHarb2intro, SpecHarb2intro, Spec128)
STD_ROM_FN(SpecHarb2intro)

struct BurnDriver BurnSpecHarb2intro = {
	"spec_harb2intro", "spec_harb2p1", "spec_spec128", NULL, "2018",
	"Harbinger 2: The Void - Tale Intro (128K) (HB)\0", NULL, "APSIS", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_MISC, 0,
	SpectrumGetZipName, SpecHarb2introRomInfo, SpecHarb2introRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Harbinger 2: The Void - Part-1 (128K) (HB)

static struct BurnRomInfo SpecHarb2p1RomDesc[] = {
	{ "Harbinger 2 The Void, Part-1 128K (2018)(APSIS).tap", 65335, 0xae30b46e, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecHarb2p1, SpecHarb2p1, Spec128)
STD_ROM_FN(SpecHarb2p1)

struct BurnDriver BurnSpecHarb2p1 = {
	"spec_harb2p1", NULL, "spec_spec128", NULL, "2018",
	"Harbinger 2: The Void - Part-1 (128K) (HB)\0", NULL, "APSIS", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecHarb2p1RomInfo, SpecHarb2p1RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecQAOPSpaceDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Harbinger 2: The Void - Part-2 (128K) (HB)

static struct BurnRomInfo SpecHarb2p2RomDesc[] = {
	{ "Harbinger 2 The Void, Part-2 128K (2018)(APSIS).tap", 66496, 0xffc5c242, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecHarb2p2, SpecHarb2p2, Spec128)
STD_ROM_FN(SpecHarb2p2)

struct BurnDriver BurnSpecHarb2p2 = {
	"spec_harb2p2", "spec_harb2p1", "spec_spec128", NULL, "2018",
	"Harbinger 2: The Void - Part-2 (128K) (HB)\0", NULL, "APSIS", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecHarb2p2RomInfo, SpecHarb2p2RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecQAOPSpaceDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Harbinger 2: The Void - Part-3 (128K) (HB)

static struct BurnRomInfo SpecHarb2p3RomDesc[] = {
	{ "Harbinger 2 The Void, Part-3 128K (2018)(APSIS).tap", 65665, 0x4e9caca3, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecHarb2p3, SpecHarb2p3, Spec128)
STD_ROM_FN(SpecHarb2p3)

struct BurnDriver BurnSpecHarb2p3 = {
	"spec_harb2p3", "spec_harb2p1", "spec_spec128", NULL, "2018",
	"Harbinger 2: The Void - Part-3 (128K) (HB)\0", NULL, "APSIS", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecHarb2p3RomInfo, SpecHarb2p3RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecQAOPSpaceDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Harbinger: Convergence - Part-1 (128K) (HB)

static struct BurnRomInfo SpecHarbp1RomDesc[] = {
	{ "Harbinger Convergence Part-1 128K (2016)(APSIS).tap", 52742, 0x3f7e8009, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecHarbp1, SpecHarbp1, Spec128)
STD_ROM_FN(SpecHarbp1)

struct BurnDriver BurnSpecHarbp1 = {
	"spec_harbp1", NULL, "spec_spec128", NULL, "2016",
	"Harbinger: Convergence - Part-1 (128K) (HB)\0", NULL, "APSIS", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecHarbp1RomInfo, SpecHarbp1RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecQAOPSpaceDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Harbinger: Convergence - Part-2 (128K) (HB)

static struct BurnRomInfo SpecHarbp2RomDesc[] = {
	{ "Harbinger Convergence Part-2 128K (2016)(APSIS).tap", 55208, 0xc0920a99, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecHarbp2, SpecHarbp2, Spec128)
STD_ROM_FN(SpecHarbp2)

struct BurnDriver BurnSpecHarbp2 = {
	"spec_harbp2", "spec_harbp1", "spec_spec128", NULL, "2016",
	"Harbinger: Convergence - Part-2 (128K) (HB)\0", NULL, "APSIS", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecHarbp2RomInfo, SpecHarbp2RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecQAOPSpaceDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Haunted House (48K) (HB)

static struct BurnRomInfo SpecHaunthouseRomDesc[] = {
	{ "Haunted House 48K (2023)(bwganblack).tap", 48529, 0x1c6689c9, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecHaunthouse, SpecHaunthouse, Spectrum)
STD_ROM_FN(SpecHaunthouse)

struct BurnDriver BurnSpecHaunthouse = {
	"spec_haunthouse", NULL, "spec_spectrum", NULL, "2023",
	"Haunted House (48K) (HB)\0", NULL, "Philip Perrin", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_ADV, 0,
	SpectrumGetZipName, SpecHaunthouseRomInfo, SpecHaunthouseRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Heart Stealer (48K) (HB)

static struct BurnRomInfo SpecHeartstRomDesc[] = {
	{ "Heart Stealer 48K (2010)(Timmy).tap", 34975, 0xf6b33c0f, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecHeartst, SpecHeartst, Spectrum)
STD_ROM_FN(SpecHeartst)

struct BurnDriver BurnSpecHeartst = {
	"spec_heartst", NULL, "spec_spectrum", NULL, "2010",
	"Heart Stealer (48K) (HB)\0", NULL, "Timmy", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecHeartstRomInfo, SpecHeartstRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Heart Stealer - One Key Version (48K) (HB)

static struct BurnRomInfo SpecHeartst1kRomDesc[] = {
	{ "Heart Stealer 1-key-version 48K (2012)(Timmy).tap", 28037, 0xd8397ba2, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecHeartst1k, SpecHeartst1k, Spectrum)
STD_ROM_FN(SpecHeartst1k)

struct BurnDriver BurnSpecHeartst1k = {
	"spec_heartst1k", "spec_heartst", "spec_spectrum", NULL, "2012",
	"Heart Stealer - One Key Version (48K) (HB)\0", NULL, "Timmy", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecHeartst1kRomInfo, SpecHeartst1kRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Heart Stealer 2 (48K) (HB)

static struct BurnRomInfo SpecHeartst2RomDesc[] = {
	{ "Heart Stealer 2 48K (2013)(Timmy).tap", 33154, 0x9c01ef20, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecHeartst2, SpecHeartst2, Spectrum)
STD_ROM_FN(SpecHeartst2)

struct BurnDriver BurnSpecHeartst2 = {
	"spec_heartst2", NULL, "spec_spectrum", NULL, "2013",
	"Heart Stealer 2 (48K) (HB)\0", NULL, "Timmy", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecHeartst2RomInfo, SpecHeartst2RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Hell Tank - Easy Mode (128K) (HB)

static struct BurnRomInfo SpecHelltankeRomDesc[] = {
	{ "Hell Tank 128K - Easy (2024)(fransouls).tap", 56854, 0x33dc72ea, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecHelltanke, SpecHelltanke, Spec128)
STD_ROM_FN(SpecHelltanke)

struct BurnDriver BurnSpecHelltanke = {
	"spec_helltanke", NULL, "spec_spec128", NULL, "2024",
	"Hell Tank - Easy Mode (128K) (HB)\0", NULL, "fransouls", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_SHOOT, 0,
	SpectrumGetZipName, SpecHelltankeRomInfo, SpecHelltankeRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Hell Tank - Hard Mode (128K) (HB)

static struct BurnRomInfo SpecHelltankhRomDesc[] = {
	{ "Hell Tank 128K - Hard (2024)(fransouls).tap", 56919, 0x31514705, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecHelltankh, SpecHelltankh, Spec128)
STD_ROM_FN(SpecHelltankh)

struct BurnDriver BurnSpecHelltankh = {
	"spec_helltankh", "spec_helltanke", "spec_spec128", NULL, "2024",
	"Hell Tank - Hard Mode (128K) (HB)\0", NULL, "fransouls", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_SHOOT, 0,
	SpectrumGetZipName, SpecHelltankhRomInfo, SpecHelltankhRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Heritage (48K) (HB)

static struct BurnRomInfo SpecHeritageRomDesc[] = {
	{ "Heritage 48K (2009)(Rafal Miazga).tap", 48004, 0x14c9f1e0, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecHeritage, SpecHeritage, Spectrum)
STD_ROM_FN(SpecHeritage)

struct BurnDriver BurnSpecHeritage = {
	"spec_heritage", NULL, "spec_spectrum", NULL, "2009",
	"Heritage (48K) (HB)\0", NULL, "Rafal Miazga", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_ADV, 0,
	SpectrumGetZipName, SpecHeritageRomInfo, SpecHeritageRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Hoarder, The (English) (48K) (HB)

static struct BurnRomInfo SpecHoardereRomDesc[] = {
	{ "Hoarder, The EN 48K (2020)(iadvd).tap", 41367, 0x30a0539a, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecHoardere, SpecHoardere, Spectrum)
STD_ROM_FN(SpecHoardere)

struct BurnDriver BurnSpecHoardere = {
	"spec_hoardere", NULL, "spec_spectrum", NULL, "2020",
	"Hoarder, The (English) (48K) (HB)\0", NULL, "iadvd", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecHoardereRomInfo, SpecHoardereRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Hoarder, The (Spanish) (48K) (HB)

static struct BurnRomInfo SpecHoardersRomDesc[] = {
	{ "Hoarder, The ES 48K (2020)(iadvd).tap", 41374, 0x1c7cee3d, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecHoarders, SpecHoarders, Spectrum)
STD_ROM_FN(SpecHoarders)

struct BurnDriver BurnSpecHoarders = {
	"spec_hoarders", "spec_hoardere", "spec_spectrum", NULL, "2020",
	"Hoarder, The (Spanish) (48K) (HB)\0", NULL, "iadvd", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecHoardersRomInfo, SpecHoardersRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Hopman (48K) (HB)

static struct BurnRomInfo SpecHopmanRomDesc[] = {
	{ "Hopman 48K-AY (2023)(Inufuto).tap", 9385, 0xb5984e40, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecHopman, SpecHopman, Spectrum)
STD_ROM_FN(SpecHopman)

struct BurnDriver BurnSpecHopman = {
	"spec_hopman", NULL, "spec_spectrum", NULL, "2023",
	"Hopman (48K) (HB)\0", "AY Sound supported", "Inufuto", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecHopmanRomInfo, SpecHopmanRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Hop'n'Frog (128K) (HB)

static struct BurnRomInfo SpecHopnfrogRomDesc[] = {
	{ "Hop'n'Frog 128K (2015)(Gabriele Amore).tap", 23739, 0x7d55a6f4, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecHopnfrog, SpecHopnfrog, Spec128)
STD_ROM_FN(SpecHopnfrog)

struct BurnDriver BurnSpecHopnfrog = {
	"spec_hopnfrog", NULL, "spec_spec128", NULL, "2015",
	"Hop'n'Frog (128K) (HB)\0", NULL, "Gabriele Amore", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_ACTION, 0,
	SpectrumGetZipName, SpecHopnfrogRomInfo, SpecHopnfrogRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Horace and the Robots (128K) (HB)

static struct BurnRomInfo SpecHoracerobotsRomDesc[] = {
	{ "Horace and the Robots 128K (2017)(Steve Snake).tap", 25517, 0x05fd13e2, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecHoracerobots, SpecHoracerobots, Spec128)
STD_ROM_FN(SpecHoracerobots)

struct BurnDriver BurnSpecHoracerobots = {
	"spec_horacerobots", NULL, "spec_spec128", NULL, "2017",
	"Horace and the Robots (128K) (HB)\0", NULL, "Steve Snake", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_ACTION, 0,
	SpectrumGetZipName, SpecHoracerobotsRomInfo, SpecHoracerobotsRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecQAOPSpaceDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Hunt Buck - Nuclear Defence (Russian) (128K) (HB)

static struct BurnRomInfo SpecHuntbuckRomDesc[] = {
	{ "Hunt Buck - Nuclear Defence RU 128K (2019)(Red Triangle).tap", 45254, 0x42bf6f68, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecHuntbuck, SpecHuntbuck, Spec128)
STD_ROM_FN(SpecHuntbuck)

struct BurnDriver BurnSpecHuntbuck = {
	"spec_huntbuck", NULL, "spec_spec128", NULL, "2019",
	"Hunt Buck - Nuclear Defence (Russian) (128K) (HB)\0", NULL, "Red Triangle", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecHuntbuckRomInfo, SpecHuntbuckRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecQAOPSpaceDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Ice Slider Z (48K) (HB)

static struct BurnRomInfo SpecIceslidzRomDesc[] = {
	{ "Ice Slider Z 48K (2015)(EgoTrip).tap", 25840, 0x810f2cae, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecIceslidz, SpecIceslidz, Spectrum)
STD_ROM_FN(SpecIceslidz)

struct BurnDriver BurnSpecIceslidz = {
	"spec_iceslidz", NULL, "spec_spectrum", NULL, "2015",
	"Ice Slider Z (48K) (HB)\0", NULL, "EgoTrip", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_ACTION, 0,
	SpectrumGetZipName, SpecIceslidzRomInfo, SpecIceslidzRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecQAOPSpaceDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// iLogicAll (English) (128K) (HB)

static struct BurnRomInfo SpecIlogicenRomDesc[] = {
	{ "iLogicAll EN 128K (2008)(Computer Emuzone).z80", 61966, 0x58d2502a, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecIlogicen, SpecIlogicen, Spec128)
STD_ROM_FN(SpecIlogicen)

struct BurnDriver BurnSpecIlogicen = {
	"spec_ilogicen", NULL, "spec_spec128", NULL, "2008",
	"iLogicAll (English) (128K) (HB)\0", NULL, "Computer Emuzone", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PUZZLE, 0,
	SpectrumGetZipName, SpecIlogicenRomInfo, SpecIlogicenRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// iLogicAll (Spanish) (128K) (HB)

static struct BurnRomInfo SpecIlogicesRomDesc[] = {
	{ "iLogicAll ES 128K (2008)(Computer Emuzone).z80", 62023, 0x2568e7ea, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecIlogices, SpecIlogices, Spec128)
STD_ROM_FN(SpecIlogices)

struct BurnDriver BurnSpecIlogices = {
	"spec_ilogices", "spec_ilogicen", "spec_spec128", NULL, "2008",
	"iLogicAll (Spanish) (128K) (HB)\0", NULL, "Computer Emuzone", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PUZZLE, 0,
	SpectrumGetZipName, SpecIlogicesRomInfo, SpecIlogicesRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Impetus (48K) (HB)

static struct BurnRomInfo SpecImpetusRomDesc[] = {
	{ "Impetus 48K-AY (2021)(Inufuto).tap", 16231, 0x059b3699, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecImpetus, SpecImpetus, Spectrum)
STD_ROM_FN(SpecImpetus)

struct BurnDriver BurnSpecImpetus = {
	"spec_impetus", NULL, "spec_spectrum", NULL, "2021",
	"Impetus (48K) (HB)\0", "AY Sound supported", "Inufuto", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_VERSHOOT, 0,
	SpectrumGetZipName, SpecImpetusRomInfo, SpecImpetusRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Impossamal (128K) (HB)

static struct BurnRomInfo SpecImpossamal128RomDesc[] = {
	{ "Impossamal 128K (2023)(The Mojon Twins).tap", 48978, 0xb30c4dce, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecImpossamal128, SpecImpossamal128, Spec128)
STD_ROM_FN(SpecImpossamal128)

struct BurnDriver BurnSpecImpossamal128 = {
	"spec_impossamal128", NULL, "spec_spec128", NULL, "2023",
	"Impossamal (128K) (HB)\0", NULL, "The Mojon Twins", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_ACTION, 0,
	SpectrumGetZipName, SpecImpossamal128RomInfo, SpecImpossamal128RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Impossamal (48K) (HB)

static struct BurnRomInfo SpecImpossamal48RomDesc[] = {
	{ "Impossamal 48K (2023)(The Mojon Twins).tap", 41958, 0xda1579b8, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecImpossamal48, SpecImpossamal48, Spectrum)
STD_ROM_FN(SpecImpossamal48)

struct BurnDriver BurnSpecImpossamal48 = {
	"spec_impossamal48", "spec_impossamal128", "spec_spectrum", NULL, "2023",
	"Impossamal (48K) (HB)\0", NULL, "The Mojon Twins", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_ACTION, 0,
	SpectrumGetZipName, SpecImpossamal48RomInfo, SpecImpossamal48RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Incredible Shrinking Professor, The (128K) (HB, v1.1)

static struct BurnRomInfo SpecincprofRomDesc[] = {
	{ "Incredible Shrinking Professor, The 128K (2017)(Rucksack Games).tap", 44116, 0x03e3de19, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specincprof, Specincprof, Spec128)
STD_ROM_FN(Specincprof)

struct BurnDriver BurnSpecincprof = {
	"spec_incprof", NULL, "spec_spec128", NULL, "2017",
	"Incredible Shrinking Professor, The (128K) (HB, v1.1)\0", NULL, "Rucksack Games", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecincprofRomInfo, SpecincprofRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Innsmouth (English) (48K) (HB, v1.1)

static struct BurnRomInfo SpecInnsmouthenRomDesc[] = {
	{ "Innsmouth EN v1.1 48K (2024)(Bitfans).tap", 84868, 0xc775e949, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecInnsmouthen, SpecInnsmouthen, Spectrum)
STD_ROM_FN(SpecInnsmouthen)

struct BurnDriver BurnSpecInnsmouthen = {
	"spec_innsmouthen", NULL, "spec_spectrum", NULL, "2024",
	"Innsmouth (English) (48K) (HB, v1.1)\0", NULL, "Bitfans", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_ADV, 0,
	SpectrumGetZipName, SpecInnsmouthenRomInfo, SpecInnsmouthenRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Innsmouth (Spanish) (48K) (HB, v1.1)

static struct BurnRomInfo SpecInnsmouthesRomDesc[] = {
	{ "Innsmouth ES v1.1 48K (2024)(Bitfans).tap", 85751, 0x8c77ef4e, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecInnsmouthes, SpecInnsmouthes, Spectrum)
STD_ROM_FN(SpecInnsmouthes)

struct BurnDriver BurnSpecInnsmouthes = {
	"spec_innsmouthes", "spec_innsmouthen", "spec_spectrum", NULL, "2024",
	"Innsmouth (Spanish) (48K) (HB, v1.1)\0", NULL, "Bitfans", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_ADV, 0,
	SpectrumGetZipName, SpecInnsmouthesRomInfo, SpecInnsmouthesRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// In the Walls of Eryx (128K) (HB)

static struct BurnRomInfo SpecWallseryxRomDesc[] = {
	{ "In the Walls of Eryx (2016)(Monster's Legs Productions).tap", 39585, 0xe201c77c, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecWallseryx, SpecWallseryx, Spec128)
STD_ROM_FN(SpecWallseryx)

struct BurnDriver BurnSpecWallseryx = {
	"spec_wallseryx", NULL, "spec_spec128", NULL, "2016",
	"In the Walls of Eryx (128K) (HB)\0", NULL, "Monster's Legs Productions", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_MAZE | GBF_RUNGUN, 0,
	SpectrumGetZipName, SpecWallseryxRomInfo, SpecWallseryxRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// InvAGDers (48K) (HB)

static struct BurnRomInfo SpecInvagdRomDesc[] = {
	{ "InvAGDers 48K (2017)(Allan Turvey).tap", 38279, 0xd887b747, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecInvagd, SpecInvagd, Spectrum)
STD_ROM_FN(SpecInvagd)

struct BurnDriver BurnSpecInvagd = {
	"spec_invagd", NULL, "spec_spectrum", NULL, "2017",
	"InvAGDers (48K) (HB)\0", NULL, "Allan Turvey", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_SHOOT, 0,
	SpectrumGetZipName, SpecInvagdRomInfo, SpecInvagdRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecQAOPMDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Iron Sphere (48K) (HB)

static struct BurnRomInfo SpecIronsphereRomDesc[] = {
	{ "Iron Sphere 48K (2006)(Cronosoft).tap", 41960, 0x271afa7c, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecIronsphere, SpecIronsphere, Spectrum)
STD_ROM_FN(SpecIronsphere)

struct BurnDriver BurnSpecIronsphere = {
	"spec_ironsphere", NULL, "spec_spectrum", NULL, "2006",
	"Iron Sphere (48K) (HB)\0", NULL, "Cronosoft", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_ACTION, 0,
	SpectrumGetZipName, SpecIronsphereRomInfo, SpecIronsphereRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Isa Brave (128K) (HB)

static struct BurnRomInfo SpecIsabraveRomDesc[] = {
	{ "Isa Brave 128K (2023)(Tom Martin).tap", 50274, 0x32f917e4, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecIsabrave, SpecIsabrave, Spec128)
STD_ROM_FN(SpecIsabrave)

struct BurnDriver BurnSpecIsabrave = {
	"spec_isabrave", NULL, "spec_spec128", NULL, "2023",
	"Isa Brave (128K) (HB)\0", NULL, "Tom Martin", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecIsabraveRomInfo, SpecIsabraveRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Italia 1944 - Part 1 (German) (48K) (HB)

static struct BurnRomInfo SpecIt1944de1RomDesc[] = {
	{ "Italia 1944 - DE - 1 (2021)(Zankle Soft).tap", 24615, 0xd7e57dbb, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecIt1944de1, SpecIt1944de1, Spectrum)
STD_ROM_FN(SpecIt1944de1)

struct BurnDriver BurnSpecIt1944de1 = {
	"spec_it1944de1", NULL, "spec_spectrum", NULL, "2021",
	"Italia 1944 - Part 1 (German) (48K) (HB)\0", NULL, "Zankle Soft", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_ADV | GBF_STRATEGY, 0,
	SpectrumGetZipName, SpecIt1944de1RomInfo, SpecIt1944de1RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Italia 1944 - Part 2 (German) (48K) (HB)

static struct BurnRomInfo SpecIt1944de2RomDesc[] = {
	{ "Italia 1944 - DE - 2 (2021)(Zankle Soft).tap", 30415, 0xa70c5a81, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecIt1944de2, SpecIt1944de2, Spectrum)
STD_ROM_FN(SpecIt1944de2)

struct BurnDriver BurnSpecIt1944de2 = {
	"spec_it1944de2", "spec_it1944de1", "spec_spectrum", NULL, "2021",
	"Italia 1944 - Part 2 (German) (48K) (HB)\0", NULL, "Zankle Soft", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_ADV | GBF_STRATEGY, 0,
	SpectrumGetZipName, SpecIt1944de2RomInfo, SpecIt1944de2RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Italia 1944 - Part 3 (German) (48K) (HB)

static struct BurnRomInfo SpecIt1944de3RomDesc[] = {
	{ "Italia 1944 - DE - 3 (2021)(Zankle Soft).tap", 18589, 0x29fb67bc, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecIt1944de3, SpecIt1944de3, Spectrum)
STD_ROM_FN(SpecIt1944de3)

struct BurnDriver BurnSpecIt1944de3 = {
	"spec_it1944de3", "spec_it1944de1", "spec_spectrum", NULL, "2021",
	"Italia 1944 - Part 3 (German) (48K) (HB)\0", NULL, "Zankle Soft", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_ADV | GBF_STRATEGY, 0,
	SpectrumGetZipName, SpecIt1944de3RomInfo, SpecIt1944de3RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Italia 1944 - Part 1 (English) (48K) (HB)

static struct BurnRomInfo SpecIt1944en1RomDesc[] = {
	{ "Italia 1944 - EN - 1 (2021)(Zankle Soft).tap", 24615, 0x8dff1e08, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecIt1944en1, SpecIt1944en1, Spectrum)
STD_ROM_FN(SpecIt1944en1)

struct BurnDriver BurnSpecIt1944en1 = {
	"spec_it1944en1", NULL, "spec_spectrum", NULL, "2021",
	"Italia 1944 - Part 1 (English) (48K) (HB)\0", NULL, "Zankle Soft", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_ADV | GBF_STRATEGY, 0,
	SpectrumGetZipName, SpecIt1944en1RomInfo, SpecIt1944en1RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Italia 1944 - Part 2 (English) (48K) (HB)

static struct BurnRomInfo SpecIt1944en2RomDesc[] = {
	{ "Italia 1944 - EN - 2 (2021)(Zankle Soft).tap", 30415, 0x071b0ee0, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecIt1944en2, SpecIt1944en2, Spectrum)
STD_ROM_FN(SpecIt1944en2)

struct BurnDriver BurnSpecIt1944en2 = {
	"spec_it1944en2", "spec_it1944en1", "spec_spectrum", NULL, "2021",
	"Italia 1944 - Part 2 (English) (48K) (HB)\0", NULL, "Zankle Soft", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_ADV | GBF_STRATEGY, 0,
	SpectrumGetZipName, SpecIt1944en2RomInfo, SpecIt1944en2RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Italia 1944 - Part 3 (English) (48K) (HB)

static struct BurnRomInfo SpecIt1944en3RomDesc[] = {
	{ "Italia 1944 - EN - 3 (2021)(Zankle Soft).tap", 18512, 0x5e3962d4, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecIt1944en3, SpecIt1944en3, Spectrum)
STD_ROM_FN(SpecIt1944en3)

struct BurnDriver BurnSpecIt1944en3 = {
	"spec_it1944en3", "spec_it1944en1", "spec_spectrum", NULL, "2021",
	"Italia 1944 - Part 3 (English) (48K) (HB)\0", NULL, "Zankle Soft", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_ADV | GBF_STRATEGY, 0,
	SpectrumGetZipName, SpecIt1944en3RomInfo, SpecIt1944en3RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Italia 1944 - Part 1 (Spanish) (48K) (HB)

static struct BurnRomInfo SpecIt1944es1RomDesc[] = {
	{ "Italia 1944 - ES - 1 (2021)(Zankle Soft).tap", 24615, 0xa8e4e7e6, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecIt1944es1, SpecIt1944es1, Spectrum)
STD_ROM_FN(SpecIt1944es1)

struct BurnDriver BurnSpecIt1944es1 = {
	"spec_it1944es1", NULL, "spec_spectrum", NULL, "2021",
	"Italia 1944 - Part 1 (Spanish) (48K) (HB)\0", NULL, "Zankle Soft", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_ADV | GBF_STRATEGY, 0,
	SpectrumGetZipName, SpecIt1944es1RomInfo, SpecIt1944es1RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Italia 1944 - Part 2 (Spanish) (48K) (HB)

static struct BurnRomInfo SpecIt1944es2RomDesc[] = {
	{ "Italia 1944 - ES - 2 (2021)(Zankle Soft).tap", 30415, 0xb02b00f4, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecIt1944es2, SpecIt1944es2, Spectrum)
STD_ROM_FN(SpecIt1944es2)

struct BurnDriver BurnSpecIt1944es2 = {
	"spec_it1944es2", "spec_it1944es1", "spec_spectrum", NULL, "2021",
	"Italia 1944 - Part 2 (Spanish) (48K) (HB)\0", NULL, "Zankle Soft", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_ADV | GBF_STRATEGY, 0,
	SpectrumGetZipName, SpecIt1944es2RomInfo, SpecIt1944es2RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Italia 1944 - Part 3 (Spanish) (48K) (HB)

static struct BurnRomInfo SpecIt1944es3RomDesc[] = {
	{ "Italia 1944 - ES - 3 (2021)(Zankle Soft).tap", 18580, 0xed7119d7, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecIt1944es3, SpecIt1944es3, Spectrum)
STD_ROM_FN(SpecIt1944es3)

struct BurnDriver BurnSpecIt1944es3 = {
	"spec_it1944es3", "spec_it1944es1", "spec_spectrum", NULL, "2021",
	"Italia 1944 - Part 3 (Spanish) (48K) (HB)\0", NULL, "Zankle Soft", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_ADV | GBF_STRATEGY, 0,
	SpectrumGetZipName, SpecIt1944es3RomInfo, SpecIt1944es3RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Italia 1944 - Part 1 (Italian) (48K) (HB)

static struct BurnRomInfo SpecIt1944it1RomDesc[] = {
	{ "Italia 1944 - IT - 1 (2021)(Zankle Soft).tap", 24615, 0x9cfc36d8, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecIt1944it1, SpecIt1944it1, Spectrum)
STD_ROM_FN(SpecIt1944it1)

struct BurnDriver BurnSpecIt1944it1 = {
	"spec_it1944it1", NULL, "spec_spectrum", NULL, "2021",
	"Italia 1944 - Part 1 (Italian) (48K) (HB)\0", NULL, "Zankle Soft", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_ADV | GBF_STRATEGY, 0,
	SpectrumGetZipName, SpecIt1944it1RomInfo, SpecIt1944it1RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Italia 1944 - Part 2 (Italian) (48K) (HB)

static struct BurnRomInfo SpecIt1944it2RomDesc[] = {
	{ "Italia 1944 - IT - 2 (2021)(Zankle Soft).tap", 30415, 0x8f81d1d5, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecIt1944it2, SpecIt1944it2, Spectrum)
STD_ROM_FN(SpecIt1944it2)

struct BurnDriver BurnSpecIt1944it2 = {
	"spec_it1944it2", "spec_it1944it1", "spec_spectrum", NULL, "2021",
	"Italia 1944 - Part 2 (Italian) (48K) (HB)\0", NULL, "Zankle Soft", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_ADV | GBF_STRATEGY, 0,
	SpectrumGetZipName, SpecIt1944it2RomInfo, SpecIt1944it2RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Italia 1944 - Part 3 (Italian) (48K) (HB)

static struct BurnRomInfo SpecIt1944it3RomDesc[] = {
	{ "Italia 1944 - IT - 3 (2021)(Zankle Soft).tap", 18567, 0x6dd65bbb, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecIt1944it3, SpecIt1944it3, Spectrum)
STD_ROM_FN(SpecIt1944it3)

struct BurnDriver BurnSpecIt1944it3 = {
	"spec_it1944it3", "spec_it1944it1", "spec_spectrum", NULL, "2021",
	"Italia 1944 - Part 3 (Italian) (48K) (HB)\0", NULL, "Zankle Soft", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_ADV | GBF_STRATEGY, 0,
	SpectrumGetZipName, SpecIt1944it3RomInfo, SpecIt1944it3RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Italia 1944 - Part 1 (Portuguese) (48K) (HB)

static struct BurnRomInfo SpecIt1944pt1RomDesc[] = {
	{ "Italia 1944 - PT - 1 (2021)(Zankle Soft).tap", 24615, 0xe667c757, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecIt1944pt1, SpecIt1944pt1, Spectrum)
STD_ROM_FN(SpecIt1944pt1)

struct BurnDriver BurnSpecIt1944pt1 = {
	"spec_it1944pt1", NULL, "spec_spectrum", NULL, "2021",
	"Italia 1944 - Part 1 (Portuguese) (48K) (HB)\0", NULL, "Zankle Soft", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_ADV | GBF_STRATEGY, 0,
	SpectrumGetZipName, SpecIt1944pt1RomInfo, SpecIt1944pt1RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Italia 1944 - Part 2 (Portuguese) (48K) (HB)

static struct BurnRomInfo SpecIt1944pt2RomDesc[] = {
	{ "Italia 1944 - PT - 2 (2021)(Zankle Soft).tap", 30415, 0x307d7033, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecIt1944pt2, SpecIt1944pt2, Spectrum)
STD_ROM_FN(SpecIt1944pt2)

struct BurnDriver BurnSpecIt1944pt2 = {
	"spec_it1944pt2", "spec_it1944pt1", "spec_spectrum", NULL, "2021",
	"Italia 1944 - Part 2 (Portuguese) (48K) (HB)\0", NULL, "Zankle Soft", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_ADV | GBF_STRATEGY, 0,
	SpectrumGetZipName, SpecIt1944pt2RomInfo, SpecIt1944pt2RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Italia 1944 - Part 3 (Portuguese) (48K) (HB)

static struct BurnRomInfo SpecIt1944pt3RomDesc[] = {
	{ "Italia 1944 - PT - 3 (2021)(Zankle Soft).tap", 18550, 0x06746558, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecIt1944pt3, SpecIt1944pt3, Spectrum)
STD_ROM_FN(SpecIt1944pt3)

struct BurnDriver BurnSpecIt1944pt3 = {
	"spec_it1944pt3", "spec_it1944pt1", "spec_spectrum", NULL, "2021",
	"Italia 1944 - Part 3 (Portuguese) (48K) (HB)\0", NULL, "Zankle Soft", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_ADV | GBF_STRATEGY, 0,
	SpectrumGetZipName, SpecIt1944pt3RomInfo, SpecIt1944pt3RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Italia 1944 - Part 1 (Russian) (48K) (HB)

static struct BurnRomInfo SpecIt1944ru1RomDesc[] = {
	{ "Italia 1944 - RU - 1 (2021)(Zankle Soft).tap", 24872, 0xd19cae77, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecIt1944ru1, SpecIt1944ru1, Spectrum)
STD_ROM_FN(SpecIt1944ru1)

struct BurnDriver BurnSpecIt1944ru1 = {
	"spec_it1944ru1", NULL, "spec_spectrum", NULL, "2021",
	"Italia 1944 - Part 1 (Russian) (48K) (HB)\0", NULL, "Zankle Soft", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_ADV | GBF_STRATEGY, 0,
	SpectrumGetZipName, SpecIt1944ru1RomInfo, SpecIt1944ru1RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Italia 1944 - Part 2 (Russian) (48K) (HB)

static struct BurnRomInfo SpecIt1944ru2RomDesc[] = {
	{ "Italia 1944 - RU - 2 (2021)(Zankle Soft).tap", 30415, 0xb228c967, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecIt1944ru2, SpecIt1944ru2, Spectrum)
STD_ROM_FN(SpecIt1944ru2)

struct BurnDriver BurnSpecIt1944ru2 = {
	"spec_it1944ru2", "spec_it1944ru1", "spec_spectrum", NULL, "2021",
	"Italia 1944 - Part 2 (Russian) (48K) (HB)\0", NULL, "Zankle Soft", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_ADV | GBF_STRATEGY, 0,
	SpectrumGetZipName, SpecIt1944ru2RomInfo, SpecIt1944ru2RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Italia 1944 - Part 3 (Russian) (48K) (HB)

static struct BurnRomInfo SpecIt1944ru3RomDesc[] = {
	{ "Italia 1944 - RU - 3 (2021)(Zankle Soft).tap", 18486, 0xa05231de, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecIt1944ru3, SpecIt1944ru3, Spectrum)
STD_ROM_FN(SpecIt1944ru3)

struct BurnDriver BurnSpecIt1944ru3 = {
	"spec_it1944ru3", "spec_it1944ru1", "spec_spectrum", NULL, "2021",
	"Italia 1944 - Part 3 (Russian) (48K) (HB)\0", NULL, "Zankle Soft", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_ADV | GBF_STRATEGY, 0,
	SpectrumGetZipName, SpecIt1944ru3RomInfo, SpecIt1944ru3RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Izzy Wizzy (48K) (HB)

static struct BurnRomInfo SpecIzzywizzyRomDesc[] = {
	{ "Izzy Wizzy 48K (2007)(Cronosoft).tap", 45323, 0x6b081dca, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecIzzywizzy, SpecIzzywizzy, Spectrum)
STD_ROM_FN(SpecIzzywizzy)

struct BurnDriver BurnSpecIzzywizzy = {
	"spec_izzywizzy", NULL, "spec_spectrum", NULL, "2007",
	"Izzy Wizzy (48K) (HB)\0", NULL, "Cronosoft", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecIzzywizzyRomInfo, SpecIzzywizzyRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Jack Dragon (128K) (HB)

static struct BurnRomInfo SpecJdragon128RomDesc[] = {
	{ "Jack Dragon 128K (2023)(PsychicParrot).tap", 31657, 0x2940167d, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecJdragon128, SpecJdragon128, Spec128)
STD_ROM_FN(SpecJdragon128)

struct BurnDriver BurnSpecJdragon128 = {
	"spec_jdragon128", NULL, "spec_spec128", NULL, "2023",
	"Jack Dragon (128K) (HB)\0", NULL, "Psychic Parrot", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecJdragon128RomInfo, SpecJdragon128RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Jack Dragon (48K) (HB)

static struct BurnRomInfo SpecJdragon48RomDesc[] = {
	{ "Jack Dragon 48K (2023)(PsychicParrot).tap", 31418, 0x553da903, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecJdragon48, SpecJdragon48, Spectrum)
STD_ROM_FN(SpecJdragon48)

struct BurnDriver BurnSpecJdragon48 = {
	"spec_jdragon48", "spec_jdragon128", "spec_spectrum", NULL, "2023",
	"Jack Dragon (48K) (HB)\0", NULL, "Psychic Parrot", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecJdragon48RomInfo, SpecJdragon48RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Jam! (48K) (HB)

static struct BurnRomInfo SpecJamRomDesc[] = {
	{ "Jam! 48K (2023)(Teknamic Software).tap", 12492, 0x9be21edc, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecJam, SpecJam, Spectrum)
STD_ROM_FN(SpecJam)

struct BurnDriver BurnSpecJam = {
	"spec_jam", NULL, "spec_spectrum", NULL, "2023",
	"Jam! (48K) (HB)\0", NULL, "Teknamic Software", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PUZZLE, 0,
	SpectrumGetZipName, SpecJamRomInfo, SpecJamRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Janky Joe in Retro Hell (128K) (HB)

static struct BurnRomInfo SpecJankyjoeRomDesc[] = {
	{ "Janky Joe in Retro Hell 128K (2023)(Sloanysoft).tap", 43848, 0x247dddd0, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecJankyjoe, SpecJankyjoe, Spec128)
STD_ROM_FN(SpecJankyjoe)

struct BurnDriver BurnSpecJankyjoe = {
	"spec_jankyjoe", NULL, "spec_spec128", NULL, "2023",
	"Janky Joe in Retro Hell (128K) (HB)\0", NULL, "Sloanysoft", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecJankyjoeRomInfo, SpecJankyjoeRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Janosik (128K) (HB)

static struct BurnRomInfo SpecJanosikRomDesc[] = {
	{ "Janosik 128K (2013)(Rafal Miazga, Alex Heather).tap", 44357, 0x365cc37f, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecJanosik, SpecJanosik, Spec128)
STD_ROM_FN(SpecJanosik)

struct BurnDriver BurnSpecJanosik = {
	"spec_janosik", NULL, "spec_spec128", NULL, "2013",
	"Janosik (128K) (HB)\0", NULL, "Rafal Miazga, Alex Heather", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecJanosikRomInfo, SpecJanosikRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Jeff & the Blue Notes (128K) (HB)

static struct BurnRomInfo SpecJeffnotesRomDesc[] = {
	{ "Jeff & The Blue Notes 128K (2023)(Deanysoft).tap", 44152, 0x2484175f, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecJeffnotes, SpecJeffnotes, Spec128)
STD_ROM_FN(SpecJeffnotes)

struct BurnDriver BurnSpecJeffnotes = {
	"spec_jeffnotes", NULL, "spec_spec128", NULL, "2023",
	"Jeff & the Blue Notes (128K) (HB)\0", NULL, "Deanysoft", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecJeffnotesRomInfo, SpecJeffnotesRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Jet Paco (48K) (HB)

static struct BurnRomInfo SpecJetpacoRomDesc[] = {
	{ "Jet Paco (2013)(The Mojon Twins).tap", 39655, 0x566a9c20, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecJetpaco, SpecJetpaco, Spectrum)
STD_ROM_FN(SpecJetpaco)

struct BurnDriver BurnSpecJetpaco = {
	"spec_jetpaco", NULL, "spec_spectrum", NULL, "2013",
	"Jet Paco (48K) (HB)\0", NULL, "The Mojon Twins", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecJetpacoRomInfo, SpecJetpacoRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// JetSBOT (128K) (HB)

static struct BurnRomInfo SpecJetsbotRomDesc[] = {
	{ "JetSBOT 128K (2023)(Xavisan).tap", 49853, 0xc217d7a3, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecJetsbot, SpecJetsbot, Spec128)
STD_ROM_FN(SpecJetsbot)

struct BurnDriver BurnSpecJetsbot = {
	"spec_jetsbot", NULL, "spec_spec128", NULL, "2023",
	"JetSBOT (128K) (HB)\0", NULL, "Xavisan", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecJetsbotRomInfo, SpecJetsbotRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Jet Set Willy 128 (128K) (HB)

static struct BurnRomInfo SpecJswilly128RomDesc[] = {
	{ "Jet Set Willy 128K (1996)(John Elliott).tap", 118898, 0x1ec78ad5, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecJswilly128, SpecJswilly128, Spec128)
STD_ROM_FN(SpecJswilly128)

struct BurnDriver BurnSpecJswilly128 = {
	"spec_jswilly128", NULL, "spec_spec128", NULL, "1996",
	"Jet Set Willy 128 (128K) (HB)\0", NULL, "John Elliott", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecJswilly128RomInfo, SpecJswilly128RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Jet Set Willy - Role Reversal (48K) (HB)

static struct BurnRomInfo SpecJetsetwrrRomDesc[] = {
	{ "Jet Set Willy - Role Reversal 48K (2023)(Arcane Design).tap", 37243, 0xa15673a3, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecJetsetwrr, SpecJetsetwrr, Spectrum)
STD_ROM_FN(SpecJetsetwrr)

struct BurnDriver BurnSpecJetsetwrr = {
	"spec_jetsetwrr", NULL, "spec_spectrum", NULL, "2023",
	"Jet Set Willy - Role Reversal (48K) (HB)\0", NULL, "Arcane Design", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecJetsetwrrRomInfo, SpecJetsetwrrRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Jet Set Steamboat Willie (128K) (HB)

static struct BurnRomInfo SpecJetsetswRomDesc[] = {
	{ "Jet Set Steamboat Willie 128K (2024)(Sloanysoft).tap", 47655, 0x0e27d045, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecJetsetsw, SpecJetsetsw, Spec128)
STD_ROM_FN(SpecJetsetsw)

struct BurnDriver BurnSpecJetsetsw = {
	"spec_jetsetsw", NULL, "spec_spec128", NULL, "2024",
	"Jet Set Steamboat Willie (128K) (HB)\0", NULL, "Sloanysoft", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecJetsetswRomInfo, SpecJetsetswRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Jinj (128K) (HB)

static struct BurnRomInfo SpecJinjRomDesc[] = {
	{ "JinJ (2008)(Computer Emuzone).z80", 35854, 0xcab3bfaf, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecJinj, SpecJinj, Spec128)
STD_ROM_FN(SpecJinj)

struct BurnDriver BurnSpecJinj = {
	"spec_jinj", NULL, "spec_spec128", NULL, "2008",
	"JinJ (128K) (HB)\0", NULL, "Computer Emuzone", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_MAZE | GBF_RUNGUN, 0,
	SpectrumGetZipName, SpecJinjRomInfo, SpecJinjRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecIntf2DIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Jinj 2 - Belmonte's Revenge (128K) (HB)

static struct BurnRomInfo SpecJinj2RomDesc[] = {
	{ "Jinj 2 - Belmonte's Revenge (2012)(RetroWorks).tap", 47956, 0x068ec259, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecJinj2, SpecJinj2, Spec128)
STD_ROM_FN(SpecJinj2)

struct BurnDriver BurnSpecJinj2 = {
	"spec_jinj2", NULL, "spec_spec128", NULL, "2012",
	"Jinj 2 - Belmonte's Revenge (128K) (HB)\0", NULL, "RetroWorks", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_MAZE | GBF_RUNGUN, 0,
	SpectrumGetZipName, SpecJinj2RomInfo, SpecJinj2RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecIntf2DIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Johnny the Ghost (128K) (HB)

static struct BurnRomInfo SpecJohnnyghostRomDesc[] = {
	{ "Johnny the Ghost 128K (2023)(Harald & Marcus Gitzel).tap", 39448, 0x22359f59, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecJohnnyghost, SpecJohnnyghost, Spec128)
STD_ROM_FN(SpecJohnnyghost)

struct BurnDriver BurnSpecJohnnyghost = {
	"spec_johnnyghost", NULL, "spec_spec128", NULL, "2023",
	"Johnny the Ghost (128K) (HB)\0", "R: restart level - S: music on/off", "Harald & Marcus Gitzel", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PUZZLE, 0,
	SpectrumGetZipName, SpecJohnnyghostRomInfo, SpecJohnnyghostRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Jonathan - Trick or Treat (48K) (HB)

static struct BurnRomInfo SpecJonathanRomDesc[] = {
	{ "Jonathan - Trick or Treat 48K (2022)(ZXAMAZE).tap", 37147, 0x4b1299bf, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecJonathan, SpecJonathan, Spectrum)
STD_ROM_FN(SpecJonathan)

struct BurnDriver BurnSpecJonathan = {
	"spec_jonathan", NULL, "spec_spectrum", NULL, "2022",
	"Jonathan - Trick or Treat (48K) (HB)\0", NULL, "ZXAMAZE", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecJonathanRomInfo, SpecJonathanRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Jubbles (128K) (HB)

static struct BurnRomInfo SpecJubblesRomDesc[] = {
	{ "Jubbles 128K (2017)(Jonathan Cauldwell).tap", 33886, 0x75b86b88, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecJubbles, SpecJubbles, Spec128)
STD_ROM_FN(SpecJubbles)

struct BurnDriver BurnSpecJubbles = {
	"spec_jubbles", NULL, "spec_spec128", NULL, "2017",
	"Jubbles (128K) (HB)\0", "Press '2' to use controller", "Jonathan Cauldwell", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_ACTION, 0,
	SpectrumGetZipName, SpecJubblesRomInfo, SpecJubblesRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Jumpin' Jupiter (48K) (HB)

static struct BurnRomInfo SpecjumpjupiterRomDesc[] = {
	{ "Jumpin' Jupiter (2020)(Quantum Sheep).tap", 40858, 0x34a1f400, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specjumpjupiter, Specjumpjupiter, Spectrum)
STD_ROM_FN(Specjumpjupiter)

struct BurnDriver BurnSpecjumpjupiter = {
	"spec_jumpjupiter", NULL, "spec_spectrum", NULL, "2020",
	"Jumpin' Jupiter (48K) (HB)\0", NULL, "Quantum Sheep", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecjumpjupiterRomInfo, SpecjumpjupiterRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecQAOPMDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Justin (128K) (HB)

static struct BurnRomInfo SpecJustinRomDesc[] = {
	{ "Justin (2007)(CNGSoft).tap", 23501, 0xf5d58915, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecJustin, SpecJustin, Spec128)
STD_ROM_FN(SpecJustin)

struct BurnDriver BurnSpecJustin = {
	"spec_justin", NULL, "spec_spec128", NULL, "2007",
	"Justin (128K) (HB)\0", NULL, "CNGSoft", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecJustinRomInfo, SpecJustinRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Klondike Solitaire (128K) (HB, v1.04)

static struct BurnRomInfo SpecKlondike128RomDesc[] = {
	{ "Klondike Solitaire v1.04 128K (2023)(Under4Mhz).tap", 27682, 0xd446444e, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecKlondike128, SpecKlondike128, Spec128)
STD_ROM_FN(SpecKlondike128)

struct BurnDriver BurnSpecKlondike128 = {
	"spec_klondike128", NULL, "spec_spec128", NULL, "2023",
	"Klondike Solitaire (128K) (HB, v1.04)\0", NULL, "Under4Mhz", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_CARD, 0,
	SpectrumGetZipName, SpecKlondike128RomInfo, SpecKlondike128RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecQAOPSpaceDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Klondike Solitaire (48K) (HB, v1.04)

static struct BurnRomInfo SpecKlondike48RomDesc[] = {
	{ "Klondike Solitaire v1.04 48K (2023)(Under4Mhz).tap", 27278, 0x848b4208, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecKlondike48, SpecKlondike48, Spectrum)
STD_ROM_FN(SpecKlondike48)

struct BurnDriver BurnSpecKlondike48 = {
	"spec_klondike48", "spec_klondike128", "spec_spectrum", NULL, "2023",
	"Klondike Solitaire (48K) (HB, v1.04)\0", NULL, "Under4Mhz", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_CARD, 0,
	SpectrumGetZipName, SpecKlondike48RomInfo, SpecKlondike48RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecQAOPSpaceDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Klondike Solitaire - first release (128K) (HB)

static struct BurnRomInfo SpecKsolitaireRomDesc[] = {
	{ "Klondike Solitaire (2021)(Under4Mhz).tap", 20200, 0x5f351c11, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecKsolitaire, SpecKsolitaire, Spec128)
STD_ROM_FN(SpecKsolitaire)

struct BurnDriver BurnSpecKsolitaire = {
	"spec_ksolitaire", "spec_klondike128", "spec_spec128", NULL, "2021",
	"Klondike Solitaire - first release (128K) (HB)\0", NULL, "Under4Mhz", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_CARD, 0,
	SpectrumGetZipName, SpecKsolitaireRomInfo, SpecKsolitaireRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecQAOPSpaceDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Knights (48K) (HB)

static struct BurnRomInfo SpecKnightsRomDesc[] = {
	{ "Knights 48K (2018)(Darryl Sloan).tap", 32396, 0x54bf7e44, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecKnights, SpecKnights, Spectrum)
STD_ROM_FN(SpecKnights)

struct BurnDriver BurnSpecKnights = {
	"spec_knights", NULL, "spec_spectrum", NULL, "2018",
	"Knights (48K) (HB)\0", NULL, "Darryl Sloan", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PUZZLE, 0,
	SpectrumGetZipName, SpecKnightsRomInfo, SpecKnightsRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecQAOPSpaceDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Knights & Demons DX (128K) (HB)

static struct BurnRomInfo SpecKnddxRomDesc[] = {
	{ "Knights & Demons DX (2013)(Kabuto Factory).tap", 34350, 0x069c604c, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecKnddx, SpecKnddx, Spec128)
STD_ROM_FN(SpecKnddx)

struct BurnDriver BurnSpecKnddx = {
	"spec_knddx", NULL, "spec_spec128", NULL, "2013",
	"Knights & Demons DX (128K) (HB)\0", NULL, "Kabuto Factory", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PUZZLE, 0,
	SpectrumGetZipName, SpecKnddxRomInfo, SpecKnddxRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecQAOPMDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Kuatris (128K) (HB)

static struct BurnRomInfo SpecKuatrisRomDesc[] = {
	{ "Kuatris 128K (2022)(Mr Rancio).tap", 47935, 0x8c7c0245, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecKuatris, SpecKuatris, Spec128)
STD_ROM_FN(SpecKuatris)

struct BurnDriver BurnSpecKuatris = {
	"spec_kuatris", NULL, "spec_spec128", NULL, "2022",
	"Kuatris (128K) (HB)\0", NULL, "Mr. Rancio", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PUZZLE, 0,
	SpectrumGetZipName, SpecKuatrisRomInfo, SpecKuatrisRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecQAOPSpaceDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Kyd Cadet II - The Rescue of Pobbleflu (48K) (HB)

static struct BurnRomInfo SpecKydcadet2RomDesc[] = {
	{ "Kyd Cadet II - The Rescue of Pobbleflu (2010)(Paul Jenkinson).tap", 36690, 0x48cf3f8c, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecKydcadet2, SpecKydcadet2, Spectrum)
STD_ROM_FN(SpecKydcadet2)

struct BurnDriver BurnSpecKydcadet2 = {
	"spec_kydcadet2", NULL, "spec_spectrum", NULL, "2010",
	"Kyd Cadet II - The Rescue of Pobbleflu (48K) (HB)\0", "A: left - S: right - SPACE: jump", "Paul Jenkinson", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecKydcadet2RomInfo, SpecKydcadet2RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Kyd Cadet 3 - The Eyeburx Plee (48K) (HB)

static struct BurnRomInfo SpecKydcadet3RomDesc[] = {
	{ "Kyd Cadet 3 - The Eyeburx Plee (2014)(Paul Jenkinson).tap", 36487, 0xf15e37a7, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecKydcadet3, SpecKydcadet3, Spectrum)
STD_ROM_FN(SpecKydcadet3)

struct BurnDriver BurnSpecKydcadet3 = {
	"spec_kydcadet3", NULL, "spec_spectrum", NULL, "2014",
	"Kyd Cadet 3 - The Eyeburx Plee (48K) (HB)\0", NULL, "Paul Jenkinson", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecKydcadet3RomInfo, SpecKydcadet3RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// L'Abbaye des Morts (128K) (HB)

static struct BurnRomInfo SpeclabbayeRomDesc[] = {
	{ "L'Abbaye des Morts (2014)(Darkhorace).z80", 58110, 0xdec96ae7, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Speclabbaye, Speclabbaye, Spec128)
STD_ROM_FN(Speclabbaye)

struct BurnDriver BurnSpeclabbaye = {
	"spec_labbaye", NULL, "spec_spec128", NULL, "2014",
	"L'Abbaye des Morts (128K) (HB)\0", "Port of original game by Locomalito and Gryzor87", "Darkhorace", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PLATFORM | GBF_ADV, 0,
	SpectrumGetZipName, SpeclabbayeRomInfo, SpeclabbayeRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Laetitia - Part 1 (128K) (HB)

static struct BurnRomInfo SpecLaetitiap1RomDesc[] = {
	{ "Laetitia Part1 128K (2019)(Jaime Grilo).tap", 48451, 0x3a7f3003, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecLaetitiap1, SpecLaetitiap1, Spec128)
STD_ROM_FN(SpecLaetitiap1)

struct BurnDriver BurnSpecLaetitiap1 = {
	"spec_laetitiap1", NULL, "spec_spec128", NULL, "2019",
	"Laetitia - Part 1 (128K) (HB)\0", NULL, "Jaime Grilo", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_ACTION | GBF_ADV, 0,
	SpectrumGetZipName, SpecLaetitiap1RomInfo, SpecLaetitiap1RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Laetitia - Part 2 (128K) (HB)

static struct BurnRomInfo SpecLaetitiap2RomDesc[] = {
	{ "Laetitia Part2 128K (2019)(Jaime Grilo).tap", 47973, 0x8b919d99, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecLaetitiap2, SpecLaetitiap2, Spec128)
STD_ROM_FN(SpecLaetitiap2)

struct BurnDriver BurnSpecLaetitiap2 = {
	"spec_laetitiap2", "spec_laetitiap1", "spec_spec128", NULL, "2019",
	"Laetitia - Part 2 (128K) (HB)\0", NULL, "Jaime Grilo", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_ACTION | GBF_ADV, 0,
	SpectrumGetZipName, SpecLaetitiap2RomInfo, SpecLaetitiap2RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Laetitia - Part 3 (128K) (HB)

static struct BurnRomInfo SpecLaetitiap3RomDesc[] = {
	{ "Laetitia Part3 128K (2019)(Jaime Grilo).tap", 47954, 0x222b918d, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecLaetitiap3, SpecLaetitiap3, Spec128)
STD_ROM_FN(SpecLaetitiap3)

struct BurnDriver BurnSpecLaetitiap3 = {
	"spec_laetitiap3", "spec_laetitiap1", "spec_spec128", NULL, "2019",
	"Laetitia - Part 3 (128K) (HB)\0", NULL, "Jaime Grilo", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_ACTION | GBF_ADV, 0,
	SpectrumGetZipName, SpecLaetitiap3RomInfo, SpecLaetitiap3RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Laetitia - Part 4 (128K) (HB)

static struct BurnRomInfo SpecLaetitiap4RomDesc[] = {
	{ "Laetitia Part4 128K (2019)(Jaime Grilo).tap", 48657, 0x46c87322, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecLaetitiap4, SpecLaetitiap4, Spec128)
STD_ROM_FN(SpecLaetitiap4)

struct BurnDriver BurnSpecLaetitiap4 = {
	"spec_laetitiap4", "spec_laetitiap1", "spec_spec128", NULL, "2019",
	"Laetitia - Part 4 (128K) (HB)\0", NULL, "Jaime Grilo", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_ACTION | GBF_ADV, 0,
	SpectrumGetZipName, SpecLaetitiap4RomInfo, SpecLaetitiap4RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Lambs to the Slaughter (128K) (HB)

static struct BurnRomInfo SpecLambsRomDesc[] = {
	{ "Lambs to the Slaughter 128K (2024)(The Death Squad).tap", 47988, 0x24388902, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecLambs, SpecLambs, Spec128)
STD_ROM_FN(SpecLambs)

struct BurnDriver BurnSpecLambs = {
	"spec_lambs", NULL, "spec_spec128", NULL, "2024",
	"Lambs to the Slaughter (128K) (HB)\0", NULL, "The Death Squad", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_RUNGUN, 0,
	SpectrumGetZipName, SpecLambsRomInfo, SpecLambsRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecQAOPSpaceDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Land of Mire Mare (48K) (HB, v1.06)

static struct BurnRomInfo SpecLandmmRomDesc[] = {
	{ "Land of Mire Mare v1.06 48K (2014)(Luca Bordoni).tap", 48486, 0x2b377777, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecLandmm, SpecLandmm, Spectrum)
STD_ROM_FN(SpecLandmm)

struct BurnDriver BurnSpecLandmm = {
	"spec_landmm", "spec_landmmse", "spec_spectrum", NULL, "2014",
	"Land of Mire Mare (48K) (HB, v1.06)\0", NULL, "Luca Bordoni", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_ACTION | GBF_ADV, 0,
	SpectrumGetZipName, SpecLandmmRomInfo, SpecLandmmRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Land of Mire Mare - Special Edition (48K) (HB, v1.06)

static struct BurnRomInfo SpecLandmmseRomDesc[] = {
	{ "Land of Mire Mare v1.06 - Special Edition 48K (2014)(Luca Bordoni).tap", 48486, 0xf95c3439, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecLandmmse, SpecLandmmse, Spectrum)
STD_ROM_FN(SpecLandmmse)

struct BurnDriver BurnSpecLandmmse = {
	"spec_landmmse", NULL, "spec_spectrum", NULL, "2014",
	"Land of Mire Mare - Special Edition (48K) (HB, v1.06)\0", NULL, "Luca Bordoni", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_ACTION | GBF_ADV, 0,
	SpectrumGetZipName, SpecLandmmseRomInfo, SpecLandmmseRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// La Oca (English) (48K) (HB, v1.1)

static struct BurnRomInfo SpecLaocaenRomDesc[] = {
	{ "La Oca EN v1.1 48K (2024)(Fitosoft).tap", 38626, 0xedbd154e, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecLaocaen, SpecLaocaen, Spectrum)
STD_ROM_FN(SpecLaocaen)

struct BurnDriver BurnSpecLaocaen = {
	"spec_laocaen", NULL, "spec_spectrum", NULL, "2024",
	"La Oca (English) (48K) (HB, v1.1)\0", NULL, "Fitosoft", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 4, HARDWARE_SPECTRUM, GBF_BOARD, 0,
	SpectrumGetZipName, SpecLaocaenRomInfo, SpecLaocaenRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// La Oca (Spanish) (48K) (HB, v1.1)

static struct BurnRomInfo SpecLaocaesRomDesc[] = {
	{ "La Oca ES v1.1 48K (2024)(Fitosoft).tap", 38702, 0xc7a6d4c2, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecLaocaes, SpecLaocaes, Spectrum)
STD_ROM_FN(SpecLaocaes)

struct BurnDriver BurnSpecLaocaes = {
	"spec_laocaes", "spec_laocaen", "spec_spectrum", NULL, "2024",
	"La Oca (Spanish) (48K) (HB, v1.1)\0", NULL, "Fitosoft", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE | BDF_HOMEBREW, 4, HARDWARE_SPECTRUM, GBF_BOARD, 0,
	SpectrumGetZipName, SpecLaocaesRomInfo, SpecLaocaesRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// La Oca (Portuguese) (48K) (HB, v1.1)

static struct BurnRomInfo SpecLaocaptRomDesc[] = {
	{ "La Oca PT v1.1 48K (2024)(Fitosoft).tap", 38691, 0x86904775, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecLaocapt, SpecLaocapt, Spectrum)
STD_ROM_FN(SpecLaocapt)

struct BurnDriver BurnSpecLaocapt = {
	"spec_laocapt", "spec_laocaen", "spec_spectrum", NULL, "2024",
	"La Oca (Portuguese) (48K) (HB, v1.1)\0", NULL, "Fitosoft", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE | BDF_HOMEBREW, 4, HARDWARE_SPECTRUM, GBF_BOARD, 0,
	SpectrumGetZipName, SpecLaocaptRomInfo, SpecLaocaptRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Last Escape, The (128K) (HB)

static struct BurnRomInfo SpecLastescapeRomDesc[] = {
	{ "Last Escape (2021)(Ariel Endaraues).tap", 60031, 0x3ec22c35, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecLastescape, SpecLastescape, Spec128)
STD_ROM_FN(SpecLastescape)

struct BurnDriver BurnSpecLastescape = {
	"spec_lastescape", NULL, "spec_spec128", NULL, "2021",
	"Last Escape, The (128K) (HB)\0", NULL, "Poe Games", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PLATFORM | GBF_ADV, 0,
	SpectrumGetZipName, SpecLastescapeRomInfo, SpecLastescapeRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Last Train to Tranz-Central - Part 1 (128K) (HB)

static struct BurnRomInfo SpecLasttrain1RomDesc[] = {
	{ "Last Train to Tranz-Central Part1 128k (2020)(Quantum Sheep).tap", 44216, 0x5782d9b4, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecLasttrain1, SpecLasttrain1, Spec128)
STD_ROM_FN(SpecLasttrain1)

struct BurnDriver BurnSpecLasttrain1 = {
	"spec_lasttrain1", NULL, "spec_spec128", NULL, "2020",
	"Last Train to Tranz-Central - Part 1 (128K) (HB)\0", NULL, "Quantum Sheep", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecLasttrain1RomInfo, SpecLasttrain1RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Last Train to Tranz-Central - Part 2 (128K) (HB)

static struct BurnRomInfo SpecLasttrain2RomDesc[] = {
	{ "Last Train to Tranz-Central Part2 128k (2020)(Quantum Sheep).tap", 45932, 0x8a07ee28, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecLasttrain2, SpecLasttrain2, Spec128)
STD_ROM_FN(SpecLasttrain2)

struct BurnDriver BurnSpecLasttrain2 = {
	"spec_lasttrain2", "spec_lasttrain1", "spec_spec128", NULL, "2020",
	"Last Train to Tranz-Central - Part 2 (128K) (HB)\0", NULL, "Quantum Sheep", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecLasttrain2RomInfo, SpecLasttrain2RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Lava (16K) (HB)

static struct BurnRomInfo SpecLava16kRomDesc[] = {
	{ "Lava 16K (2021)(Zosya Ent.).tap", 8366, 0xa307f634, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecLava16k, SpecLava16k, Spectrum)
STD_ROM_FN(SpecLava16k)

struct BurnDriver BurnSpecLava16k = {
	"spec_lava16k", NULL, "spec_spectrum", NULL, "2021",
	"Lava - Speed up version (16K) (HB)\0", NULL, "Zosya Entertainment", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PLATFORM | GBF_PUZZLE, 0,
	SpectrumGetZipName, SpecLava16kRomInfo, SpecLava16kRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Lava Birds, The (128K) (HB)

static struct BurnRomInfo SpecLavabirdsRomDesc[] = {
	{ "Lava Birds, The 128K (2022)(Quantum Sheep).tap", 37340, 0x4ff095cf, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecLavabirds, SpecLavabirds, Spec128)
STD_ROM_FN(SpecLavabirds)

struct BurnDriver BurnSpecLavabirds = {
	"spec_lavabirds", NULL, "spec_spec128", NULL, "2022",
	"Lava Birds, The (128K) (HB)\0", NULL, "Quantum Sheep", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_ACTION, 0,
	SpectrumGetZipName, SpecLavabirdsRomInfo, SpecLavabirdsRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Left Behind - Escape from Mars (48K) (HB)

static struct BurnRomInfo SpecLeftbehindRomDesc[] = {
	{ "Left Behind - Escape from Mars 48K (2017)(Dave Hughes).tap", 18863, 0x2b7ff02e, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecLeftbehind, SpecLeftbehind, Spectrum)
STD_ROM_FN(SpecLeftbehind)

struct BurnDriver BurnSpecLeftbehind = {
	"spec_leftbehind", NULL, "spec_spectrum", NULL, "2017",
	"Left Behind - Escape from Mars (48K) (HB)\0", "O: left - P: right - Z: jump", "Dave Hughes", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecLeftbehindRomInfo, SpecLeftbehindRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Legend of the Frog Prince, The (128K) (HB)

static struct BurnRomInfo SpecFrogprinceRomDesc[] = {
	{ "Legend of the Frog Prince, The (2019)(Gabriele Amore).tap", 47015, 0x3eac1de4, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecFrogprince, SpecFrogprince, Spec128)
STD_ROM_FN(SpecFrogprince)

struct BurnDriver BurnSpecFrogprince = {
	"spec_frogprince", NULL, "spec_spec128", NULL, "2019",
	"Legend of the Frog Prince, The (128K) (HB)\0", NULL, "Gabriele Amore", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecFrogprinceRomInfo, SpecFrogprinceRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecQAOPMDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Legend of Traxtor (48K) (HB)

static struct BurnRomInfo SpectraxtorRomDesc[] = {
	{ "Legend of Traxtor (2014)(usebox.net).tap", 23986, 0xef7dc08c, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Spectraxtor, Spectraxtor, Spectrum)
STD_ROM_FN(Spectraxtor)

struct BurnDriver BurnSpectraxtor = {
	"spec_traxtor", NULL, "spec_spectrum", NULL, "2014",
	"Legend of Traxtor (48K) (HB)\0", NULL, "usebox.net", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PUZZLE, 0,
	SpectrumGetZipName, SpectraxtorRomInfo, SpectraxtorRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Lil' Ninja (48K) (HB)

static struct BurnRomInfo SpecLilninjaRomDesc[] = {
	{ "Lil' Ninja 48K (2023)(Joesoft).tap", 40609, 0xde3a35f2, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecLilninja, SpecLilninja, Spectrum)
STD_ROM_FN(SpecLilninja)

struct BurnDriver BurnSpecLilninja = {
	"spec_lilninja", NULL, "spec_spectrum", NULL, "2023",
	"Lil' Ninja (48K) (HB)\0", NULL, "Joesoft", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecLilninjaRomInfo, SpecLilninjaRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Liquid War (48K) (HB)

static struct BurnRomInfo SpecLiquidwarRomDesc[] = {
	{ "Liquid War (2020)(Fitosoft).tap", 33821, 0x3892d96f, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecLiquidwar, SpecLiquidwar, Spectrum)
STD_ROM_FN(SpecLiquidwar)

struct BurnDriver BurnSpecLiquidwar = {
	"spec_liquidwar", NULL, "spec_spectrum", NULL, "2020",
	"Liquid War (48K) (HB)\0", NULL, "Fitosoft", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_SHOOT, 0,
	SpectrumGetZipName, SpecLiquidwarRomInfo, SpecLiquidwarRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Lockdown Town ZX (48K) (HB)

static struct BurnRomInfo SpecLockdownzxRomDesc[] = {
	{ "Lockdown Town ZX (2022)(PuttyCAD, Tin Soldiers).tap", 30174, 0x2ee0a21e, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecLockdownzx, SpecLockdownzx, Spectrum)
STD_ROM_FN(SpecLockdownzx)

struct BurnDriver BurnSpecLockdownzx = {
	"spec_lockdownzx", NULL, "spec_spectrum", NULL, "2022",
	"Lockdown Town ZX (48K) (HB)\0", NULL, "PuttyCAD, Tin Soldiers", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_MAZE | GBF_ACTION, 0,
	SpectrumGetZipName, SpecLockdownzxRomInfo, SpecLockdownzxRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Lockdown Town ZXMas (48K) (HB)

static struct BurnRomInfo SpecLockdownzxmasRomDesc[] = {
	{ "Lockdown Town ZXMas (2022)(PuttyCAD, Tin Soldiers).tap", 30591, 0x607f00ef, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecLockdownzxmas, SpecLockdownzxmas, Spectrum)
STD_ROM_FN(SpecLockdownzxmas)

struct BurnDriver BurnSpecLockdownzxmas = {
	"spec_lockdownzxmas", "spec_lockdownzx", "spec_spectrum", NULL, "2022",
	"Lockdown Town ZXMas (48K) (HB)\0", NULL, "PuttyCAD, Tin Soldiers", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_MAZE | GBF_ACTION, 0,
	SpectrumGetZipName, SpecLockdownzxmasRomInfo, SpecLockdownzxmasRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Lockdown Town ZXMas - Crash Exclusive (48K) (HB, v1.01)

static struct BurnRomInfo SpecLockdownzxmasceRomDesc[] = {
	{ "Lockdown Town ZXMAS - Crash Exclusive v1.01 48K (2023)(PuttyCAD).tap", 32868, 0x8cfbb3e5, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecLockdownzxmasce, SpecLockdownzxmasce, Spectrum)
STD_ROM_FN(SpecLockdownzxmasce)

struct BurnDriver BurnSpecLockdownzxmasce = {
	"spec_lockdownzxmasce", "spec_lockdownzx", "spec_spectrum", NULL, "2023",
	"Lockdown Town ZXMas - Crash Exclusive (48K) (HB, v1.01)\0", NULL, "PuttyCAD, Tin Soldiers", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_MAZE | GBF_ACTION, 0,
	SpectrumGetZipName, SpecLockdownzxmasceRomInfo, SpecLockdownzxmasceRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Lockdown Town ZXWeen (48K) (HB)

static struct BurnRomInfo SpecLockdownzxweenRomDesc[] = {
	{ "Lockdown Town ZXWeen (2022)(PuttyCAD, Tin Soldiers).tap", 30441, 0x4d6a99cf, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecLockdownzxween, SpecLockdownzxween, Spectrum)
STD_ROM_FN(SpecLockdownzxween)

struct BurnDriver BurnSpecLockdownzxween = {
	"spec_lockdownzxween", "spec_lockdownzx", "spec_spectrum", NULL, "2022",
	"Lockdown Town ZXWeen (48K) (HB)\0", NULL, "PuttyCAD, Tin Soldiers", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_MAZE | GBF_ACTION, 0,
	SpectrumGetZipName, SpecLockdownzxweenRomInfo, SpecLockdownzxweenRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Lode Ranger (48K) (Hack)

static struct BurnRomInfo Spec_loderangerRomDesc[] = {
	{ "Lode Ranger 48K (1992)(Fantadrom)(Hack).tap", 48038, 0x4e990388, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Spec_loderanger, Spec_loderanger, Spectrum)
STD_ROM_FN(Spec_loderanger)

struct BurnDriver BurnSpec_loderanger = {
	"spec_loderanger", "spec_loderunner", "spec_spectrum", NULL, "1992",
	"Lode Ranger (48K) (Hack)\0", NULL, "Fantadrom", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE | BDF_HACK, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, Spec_loderangerRomInfo, Spec_loderangerRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Lode Runner 3 (48K) (Hack, Image)

static struct BurnRomInfo Spec_loderun3imRomDesc[] = {
	{ "Lode Runner 3 48K-AY (1996)(Image)(Hack).tap", 48038, 0x699aa740, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Spec_loderun3im, Spec_loderun3im, Spectrum)
STD_ROM_FN(Spec_loderun3im)

struct BurnDriver BurnSpec_loderun3im = {
	"spec_loderun3im", "spec_loderunner", "spec_spectrum", NULL, "1996",
	"Lode Runner 3 (48K) (Hack, Image)\0", "AY Sound supported", "Image", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE | BDF_HACK, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, Spec_loderun3imRomInfo, Spec_loderun3imRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Lode Runner 3 (48K) (Hack, Ma$ter)

static struct BurnRomInfo Spec_loderun3maRomDesc[] = {
	{ "Lode Runner 3 48K (1999)(Ma$ter)(Hack).tap", 48038, 0xdbf95098, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Spec_loderun3ma, Spec_loderun3ma, Spectrum)
STD_ROM_FN(Spec_loderun3ma)

struct BurnDriver BurnSpec_loderun3ma = {
	"spec_loderun3ma", "spec_loderunner", "spec_spectrum", NULL, "1999",
	"Lode Runner 3 (48K) (Hack, Ma$ter)\0", NULL, "Ma$ter", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE | BDF_HACK, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, Spec_loderun3maRomInfo, Spec_loderun3maRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Lode Runner 3 (Russian) (48K) (Hack, Biotech)

static struct BurnRomInfo Spec_loderun3biRomDesc[] = {
	{ "Lode Runner 3 48K (RU)(1995)(Biotech Software Inc.)(Hack).tap", 48038, 0x18cb6710, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Spec_loderun3bi, Spec_loderun3bi, Spectrum)
STD_ROM_FN(Spec_loderun3bi)

struct BurnDriver BurnSpec_loderun3bi = {
	"spec_loderun3bi", "spec_loderunner", "spec_spectrum", NULL, "1995",
	"Lode Runner 3 (Russian) (48K) (Hack, Biotech)\0", NULL, "Biotech Software", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE | BDF_HACK, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, Spec_loderun3biRomInfo, Spec_loderun3biRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Lode Runner 3 - At Hell to Paradise (48K) (Hack)

static struct BurnRomInfo Spec_loderun3heRomDesc[] = {
	{ "Lode Runner 3 - At Hell to Paradise 48K (2013)(Hungry Software)(Hack).tap", 48038, 0x5f5af05d, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Spec_loderun3he, Spec_loderun3he, Spectrum)
STD_ROM_FN(Spec_loderun3he)

struct BurnDriver BurnSpec_loderun3he = {
	"spec_loderun3he", "spec_loderunner", "spec_spectrum", NULL, "2013",
	"Lode Runner 3 - At Hell to Paradise (48K) (Hack)\0", NULL, "Hungry Software", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE | BDF_HACK, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, Spec_loderun3heRomInfo, Spec_loderun3heRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Lode Runner 3 (48K) (Hack, Rasoft)

static struct BurnRomInfo Spec_loderun3raRomDesc[] = {
	{ "Lode Runner 3 48K (2015)(Rasoft)(Hack).tap", 48038, 0xcc4c8f4b, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Spec_loderun3ra, Spec_loderun3ra, Spectrum)
STD_ROM_FN(Spec_loderun3ra)

struct BurnDriver BurnSpec_loderun3ra = {
	"spec_loderun3ra", "spec_loderunner", "spec_spectrum", NULL, "2015",
	"Lode Runner 3 (48K) (Hack, Rasoft)\0", NULL, "Rasoft", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE | BDF_HACK, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, Spec_loderun3raRomInfo, Spec_loderun3raRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Lode Runner 3 (Russian) (48K) (Hack, Fikus-Pikus)

static struct BurnRomInfo Spec_loderun3fiRomDesc[] = {
	{ "Lode Runner 3 48K (RU)(1995)(Fikus-Pikus)(Hack).tap", 48038, 0xd5b5852e, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Spec_loderun3fi, Spec_loderun3fi, Spectrum)
STD_ROM_FN(Spec_loderun3fi)

struct BurnDriver BurnSpec_loderun3fi = {
	"spec_loderun3fi", "spec_loderunner", "spec_spectrum", NULL, "1995",
	"Lode Runner 3 (Russian) (48K) (Hack, Fikus-Pikus)\0", "After 'Start Game' press 'SPACE' twice", "Fikus-Pikus", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE | BDF_HACK, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, Spec_loderun3fiRomInfo, Spec_loderun3fiRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Lode Runner 4 (48K) (Hack, Technospace)

static struct BurnRomInfo Spec_loderun4tsRomDesc[] = {
	{ "Lode Runner 4 48K (1997)(Technospace Software)(Hack).tap", 48038, 0x31d1cafc, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Spec_loderun4ts, Spec_loderun4ts, Spectrum)
STD_ROM_FN(Spec_loderun4ts)

struct BurnDriver BurnSpec_loderun4ts = {
	"spec_loderun4ts", "spec_loderunner", "spec_spectrum", NULL, "1997",
	"Lode Runner 4 (48K) (Hack, Technospace)\0", NULL, "Technospace Software", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE | BDF_HACK, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, Spec_loderun4tsRomInfo, Spec_loderun4tsRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Lode Runner 4 (128K) (Hack, DPS Group)

static struct BurnRomInfo Spec_loderun4dgRomDesc[] = {
	{ "Lode Runner 4 128K (1998)(DPS Group)(Hack).tap", 64508, 0x0ee8a641, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Spec_loderun4dg, Spec_loderun4dg, Spec128)
STD_ROM_FN(Spec_loderun4dg)

struct BurnDriver BurnSpec_loderun4dg = {
	"spec_loderun4dg", "spec_loderunner", "spec_spec128", NULL, "1998",
	"Lode Runner 4 (128K) (Hack, DPS Group)\0", NULL, "DPS Group", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE | BDF_HACK, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, Spec_loderun4dgRomInfo, Spec_loderun4dgRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Lost Treasures of Tulum, The (English) (128K) (HB)

static struct BurnRomInfo SpeclostulumenRomDesc[] = {
	{ "Lost Treasures of Tulum, The EN 128K (2020)(RetroWorks).tap", 75999, 0xc64c38ad, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Speclostulumen, Speclostulumen, Spec128)
STD_ROM_FN(Speclostulumen)

struct BurnDriver BurnSpeclostulumen = {
	"spec_lostulumen", NULL, "spec_spec128", NULL, "2020",
	"Lost Treasures of Tulum, The (English) (128K) (HB)\0", NULL, "RetroWorks", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PLATFORM | GBF_ADV, 0,
	SpectrumGetZipName, SpeclostulumenRomInfo, SpeclostulumenRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecIntf2DIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Lost Treasures of Tulum, The (Spanish) (128K) (HB)

static struct BurnRomInfo SpeclostulumesRomDesc[] = {
	{ "Lost Treasures of Tulum, The ES 128K (2020)(RetroWorks).tap", 75999, 0xc4087b0e, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Speclostulumes, Speclostulumes, Spec128)
STD_ROM_FN(Speclostulumes)

struct BurnDriver BurnSpeclostulumes = {
	"spec_lostulumes", "spec_lostulumen", "spec_spec128", NULL, "2020",
	"Lost Treasures of Tulum, The (Spanish) (128K) (HB)\0", NULL, "RetroWorks", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PLATFORM | GBF_ADV, 0,
	SpectrumGetZipName, SpeclostulumesRomInfo, SpeclostulumesRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecIntf2DIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Loxley (128K) (HB, v1.1)

static struct BurnRomInfo SpecLoxleyRomDesc[] = {
	{ "Loxley 128K v1.1 (2022)(World XXI Soft).tap", 144187, 0x89ef517a, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecLoxley, SpecLoxley, Spec128)
STD_ROM_FN(SpecLoxley)

struct BurnDriver BurnSpecLoxley = {
	"spec_loxley", NULL, "spec_spec128", NULL, "2022",
	"Loxley (128K) (HB, v1.1)\0", NULL, "World XXI Soft", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PLATFORM | GBF_ADV, 0,
	SpectrumGetZipName, SpecLoxleyRomInfo, SpecLoxleyRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Lprint Runaway (48K) (HB)

static struct BurnRomInfo SpecLprintrunRomDesc[] = {
	{ "Lprint Runaway 48K (2023)(ignacobo).tap", 43514, 0xa3785c33, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecLprintrun, SpecLprintrun, Spectrum)
STD_ROM_FN(SpecLprintrun)

struct BurnDriver BurnSpecLprintrun = {
	"spec_lprintrun", NULL, "spec_spectrum", NULL, "2023",
	"Lprint Runaway (48K) (HB)\0", NULL, "ignacobo", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_ACTION, 0,
	SpectrumGetZipName, SpecLprintrunRomInfo, SpecLprintrunRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecQAOPMDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// LumASCII (48K) (HB)

static struct BurnRomInfo SpecLumasciiRomDesc[] = {
	{ "LumASCII (2012)(Bob's Stuff).tap", 47917, 0xf29c2da7, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecLumascii, SpecLumascii, Spectrum)
STD_ROM_FN(SpecLumascii)

struct BurnDriver BurnSpecLumascii = {
	"spec_lumascii", NULL, "spec_spectrum", NULL, "2012",
	"LumASCII (48K) (HB)\0", NULL, "Bob's Stuff", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_HORSHOOT, 0,
	SpectrumGetZipName, SpecLumasciiRomInfo, SpecLumasciiRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Lunar Rescue RX (128K) (HB)

static struct BurnRomInfo SpecLrescuerxRomDesc[] = {
	{ "Lunar Rescue RX 128K (2023)(highriser).tap", 25699, 0x83c5df7e, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecLrescuerx, SpecLrescuerx, Spec128)
STD_ROM_FN(SpecLrescuerx)

struct BurnDriver BurnSpecLrescuerx = {
	"spec_lrescuerx", NULL, "spec_spec128", NULL, "2023",
	"Lunar Rescue RX (128K) (HB)\0", NULL, "highriser", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_ACTION, 0,
	SpectrumGetZipName, SpecLrescuerxRomInfo, SpecLrescuerxRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// M348 (English) (128K) (HB)

static struct BurnRomInfo SpecM348en128RomDesc[] = {
	{ "M348 EN 128K (2024)(Tartessos Games).tap", 57714, 0xf3a08128, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecM348en128, SpecM348en128, Spec128)
STD_ROM_FN(SpecM348en128)

struct BurnDriver BurnSpecM348en128 = {
	"spec_m348en128", NULL, "spec_spec128", NULL, "2024",
	"M348 (English) (128K) (HB)\0", NULL, "Tartessos Games", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecM348en128RomInfo, SpecM348en128RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// M348 (English) (48K) (HB)

static struct BurnRomInfo SpecM348en48RomDesc[] = {
	{ "M348 EN 48K (2024)(Tartessos Games).tap", 57361, 0x20af0941, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecM348en48, SpecM348en48, Spectrum)
STD_ROM_FN(SpecM348en48)

struct BurnDriver BurnSpecM348en48 = {
	"spec_m348en48", "spec_m348en128", "spec_spectrum", NULL, "2024",
	"M348 (English) (48K) (HB)\0", NULL, "Tartessos Games", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecM348en48RomInfo, SpecM348en48RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// M348 (Spanish) (128K) (HB)

static struct BurnRomInfo SpecM348es128RomDesc[] = {
	{ "M348 ES 128K (2024)(Tartessos Games).tap", 57774, 0x8e59ecf8, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecM348es128, SpecM348es128, Spec128)
STD_ROM_FN(SpecM348es128)

struct BurnDriver BurnSpecM348es128 = {
	"spec_m348es128", "spec_m348en128", "spec_spec128", NULL, "2024",
	"M348 (Spanish) (128K) (HB)\0", NULL, "Tartessos Games", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecM348es128RomInfo, SpecM348es128RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// M348 (Spanish) (48K) (HB)

static struct BurnRomInfo SpecM348es48RomDesc[] = {
	{ "M348 ES 48K (2024)(Tartessos Games).tap", 57420, 0xb30826a2, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecM348es48, SpecM348es48, Spectrum)
STD_ROM_FN(SpecM348es48)

struct BurnDriver BurnSpecM348es48 = {
	"spec_m348es48", "spec_m348en128", "spec_spectrum", NULL, "2024",
	"M348 (Spanish) (48K) (HB)\0", NULL, "Tartessos Games", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecM348es48RomInfo, SpecM348es48RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Mabus Mania Deluxe (English) (128K) (HB)

static struct BurnRomInfo SpecMabusdxenRomDesc[] = {
	{ "Mabus Mania Deluxe EN (2022)(Hicks).tap", 45631, 0xd08040f8, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecMabusdxen, SpecMabusdxen, Spec128)
STD_ROM_FN(SpecMabusdxen)

struct BurnDriver BurnSpecMabusdxen = {
	"spec_mabusdxen", NULL, "spec_spec128", NULL, "2022",
	"Mabus Mania Deluxe (English) (128K) (HB)\0", NULL, "Hicks", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecMabusdxenRomInfo, SpecMabusdxenRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Mabus Mania Deluxe (Spanish) (128K) (HB)

static struct BurnRomInfo SpecMabusdxesRomDesc[] = {
	{ "Mabus Mania Deluxe ES (2022)(Hicks).tap", 45631, 0x7c9ceb00, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecMabusdxes, SpecMabusdxes, Spec128)
STD_ROM_FN(SpecMabusdxes)

struct BurnDriver BurnSpecMabusdxes = {
	"spec_mabusdxes", "spec_mabusdxen", "spec_spec128", NULL, "2022",
	"Mabus Mania Deluxe (Spanish) (128K) (HB)\0", NULL, "Hicks", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecMabusdxesRomInfo, SpecMabusdxesRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Magenta Jim (128K) (HB)

static struct BurnRomInfo SpecMagjim128RomDesc[] = {
	{ "Magenta Jim 128K (2020)(Gaz Marshall).tap", 23797, 0x1fee648d, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecMagjim128, SpecMagjim128, Spec128)
STD_ROM_FN(SpecMagjim128)

struct BurnDriver BurnSpecMagjim128 = {
	"spec_magjim128", NULL, "spec_spec128", NULL, "2020",
	"Magenta Jim (128K) (HB)\0", NULL, "Gaz Marshall", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_MAZE | GBF_ACTION, 0,
	SpectrumGetZipName, SpecMagjim128RomInfo, SpecMagjim128RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Magenta Jim (48K) (HB)

static struct BurnRomInfo SpecMagjim48RomDesc[] = {
	{ "Magenta Jim 48K (2020)(Gaz Marshall).tap", 27972, 0x08215496, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecMagjim48, SpecMagjim48, Spectrum)
STD_ROM_FN(SpecMagjim48)

struct BurnDriver BurnSpecMagjim48 = {
	"spec_magjim48", "spec_magjim128", "spec_spectrum", NULL, "2020",
	"Magenta Jim (48K) (HB)\0", NULL, "Gaz Marshall", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_MAZE | GBF_ACTION, 0,
	SpectrumGetZipName, SpecMagjim48RomInfo, SpecMagjim48RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Mage Rage (128K) (HB)

static struct BurnRomInfo SpecMrage128RomDesc[] = {
	{ "Mage Rage 128K (2020)(Joesoft).tap", 45368, 0x7561c99e, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecMrage128, SpecMrage128, Spec128)
STD_ROM_FN(SpecMrage128)

struct BurnDriver BurnSpecMrage128 = {
	"spec_mrage128", NULL, "spec_spec128", NULL, "2020",
	"Mage Rage (128K) (HB)\0", NULL, "Joesoft", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_ACTION | GBF_MAZE, 0,
	SpectrumGetZipName, SpecMrage128RomInfo, SpecMrage128RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Mage Rage (48K) (HB)

static struct BurnRomInfo SpecMrage48RomDesc[] = {
	{ "Mage Rage 48K (2019)(Joesoft).tap", 40082, 0x685ad69b, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecMrage48, SpecMrage48, Spectrum)
STD_ROM_FN(SpecMrage48)

struct BurnDriver BurnSpecMrage48 = {
	"spec_mrage48", "spec_mrage128", "spec_spectrum", NULL, "2019",
	"Mage Rage (48K) (HB)\0", NULL, "Joesoft", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_ACTION | GBF_MAZE, 0,
	SpectrumGetZipName, SpecMrage48RomInfo, SpecMrage48RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Magical Tower Adventure - 2019 Version (48K) (HB)

static struct BurnRomInfo SpecMagtowerRomDesc[] = {
	{ "Magical Tower Adventure 2019 Version (2019)(Timmy).tap", 29119, 0x31340284, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecMagtower, SpecMagtower, Spectrum)
STD_ROM_FN(SpecMagtower)

struct BurnDriver BurnSpecMagtower = {
	"spec_magtower", NULL, "spec_spectrum", NULL, "2019",
	"Magical Tower Adventure - 2019 Version (48K) (HB)\0", NULL, "Timmy", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_MAZE | GBF_PUZZLE, 0,
	SpectrumGetZipName, SpecMagtowerRomInfo, SpecMagtowerRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Magical Tower Adventure - Mini Version

static struct BurnRomInfo SpecMagtoweroldRomDesc[] = {
	{ "Magical Tower Adventure Mini Version (2016)(Timmy).tap", 30281, 0x92b80407, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecMagtowerold, SpecMagtowerold, Spectrum)
STD_ROM_FN(SpecMagtowerold)

struct BurnDriver BurnSpecMagtowerold = {
	"spec_magtowerold", "spec_magtower", "spec_spectrum", NULL, "2016",
	"Magical Tower Adventure - Mini Version (48K) (HB)\0", NULL, "Timmy", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_MAZE | GBF_PUZZLE, 0,
	SpectrumGetZipName, SpecMagtoweroldRomInfo, SpecMagtoweroldRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Magic Tower (48K) (HB)

static struct BurnRomInfo SpecMagictowerRomDesc[] = {
	{ "Magic Tower 48K (2024)(Seagal).tap", 46839, 0x7e5cce9b, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecMagictower, SpecMagictower, Spectrum)
STD_ROM_FN(SpecMagictower)

struct BurnDriver BurnSpecMagictower = {
	"spec_magictower", NULL, "spec_spectrum", NULL, "2024",
	"Magic Tower (48K) (HB)\0", NULL, "Seagal", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecMagictowerRomInfo, SpecMagictowerRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Mahjong Solitaire (128K) (HB, v1.16)

static struct BurnRomInfo SpecMahjong128RomDesc[] = {
	{ "Mahjong Solitaire v1.16 128K (2023)(Under4Mhz).tap", 23482, 0x9688ae8b, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecMahjong128, SpecMahjong128, Spec128)
STD_ROM_FN(SpecMahjong128)

struct BurnDriver BurnSpecMahjong128 = {
	"spec_mahjong128", NULL, "spec_spec128", NULL, "2023",
	"Mahjong Solitaire (128K) (HB, v1.16)\0", NULL, "Under4Mhz", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_MAHJONG | GBF_PUZZLE, 0,
	SpectrumGetZipName, SpecMahjong128RomInfo, SpecMahjong128RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecQAOPSpaceDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Mahjong Solitaire (48K) (HB, v1.16)

static struct BurnRomInfo SpecMahjong48RomDesc[] = {
	{ "Mahjong Solitaire v1.16 48K (2023)(Under4Mhz).tap", 22574, 0x0657a886, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecMahjong48, SpecMahjong48, Spectrum)
STD_ROM_FN(SpecMahjong48)

struct BurnDriver BurnSpecMahjong48 = {
	"spec_mahjong48", "spec_mahjong128", "spec_spectrum", NULL, "2023",
	"Mahjong Solitaire (48K) (HB, v1.16)\0", NULL, "Under4Mhz", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_MAHJONG | GBF_PUZZLE, 0,
	SpectrumGetZipName, SpecMahjong48RomInfo, SpecMahjong48RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecQAOPSpaceDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Mahjong Solitaire - first release (128K) (HB)

static struct BurnRomInfo SpecMsolitaireRomDesc[] = {
	{ "Mahjong Solitaire 128K (2021)(Under4Mhz).tap", 20934, 0xc3a7afed, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecMsolitaire, SpecMsolitaire, Spec128)
STD_ROM_FN(SpecMsolitaire)

struct BurnDriver BurnSpecMsolitaire = {
	"spec_msolitaire", "spec_mahjong128", "spec_spec128", NULL, "2021",
	"Mahjong Solitaire - first release (128K) (HB)\0", NULL, "Under4Mhz", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PUZZLE, 0,
	SpectrumGetZipName, SpecMsolitaireRomInfo, SpecMsolitaireRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecQAOPSpaceDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Main Course - The Retro Cut (48K) (HB)

static struct BurnRomInfo SpecMaincourseRomDesc[] = {
	{ "Main Course - The Retro Cut 48K (2020)(Quantum Sheep).tap", 47151, 0xb64ed2bf, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecMaincourse, SpecMaincourse, Spectrum)
STD_ROM_FN(SpecMaincourse)

struct BurnDriver BurnSpecMaincourse = {
	"spec_maincourse", NULL, "spec_spectrum", NULL, "2020",
	"Main Course - The Retro Cut (48K) (HB)\0", NULL, "Quantum Sheep", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_ADV, 0,
	SpectrumGetZipName, SpecMaincourseRomInfo, SpecMaincourseRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Malignant Gore, The (128K) (HB)

static struct BurnRomInfo SpecMaligoreRomDesc[] = {
	{ "Malignant Gore, The (2019)(Sanctum Gamers).tap", 41830, 0xe4db2507, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecMaligore, SpecMaligore, Spec128)
STD_ROM_FN(SpecMaligore)

struct BurnDriver BurnSpecMaligore = {
	"spec_maligore", NULL, "spec_spec128", NULL, "2019",
	"Malignant Gore, The (128K) (HB)\0", "Press 'R' to restart level", "Sanctum Gamers", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PUZZLE, 0,
	SpectrumGetZipName, SpecMaligoreRomInfo, SpecMaligoreRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecQAOPMDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Mandarin II, The (128K) (HB)

static struct BurnRomInfo SpecMandarinii128RomDesc[] = {
	{ "Mandarin II, The 128K (2023)(Mananuk).tap", 53594, 0x6929ccd8, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecMandarinii128, SpecMandarinii128, Spec128)
STD_ROM_FN(SpecMandarinii128)

struct BurnDriver BurnSpecMandarinii128 = {
	"spec_mandarinii128", NULL, "spec_spec128", NULL, "2023",
	"Mandarin II, The (128K) (HB)\0", NULL, "Mananuk", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PLATFORM | GBF_HORSHOOT, 0,
	SpectrumGetZipName, SpecMandarinii128RomInfo, SpecMandarinii128RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Mandarin II, The (48K) (HB)

static struct BurnRomInfo SpecMandarinii48RomDesc[] = {
	{ "Mandarin II, The 48K (2023)(Mananuk).tap", 46001, 0x45817831, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecMandarinii48, SpecMandarinii48, Spectrum)
STD_ROM_FN(SpecMandarinii48)

struct BurnDriver BurnSpecMandarinii48 = {
	"spec_mandarinii48", "spec_mandarinii128", "spec_spectrum", NULL, "2023",
	"Mandarin II, The (48K) (HB)\0", NULL, "Mananuk", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PLATFORM | GBF_HORSHOOT, 0,
	SpectrumGetZipName, SpecMandarinii48RomInfo, SpecMandarinii48RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Mandarino, El (English) (48K) (HB)

static struct BurnRomInfo SpecMandarinoRomDesc[] = {
	{ "Mandarino, El 48K (2021)(Mananuk).tap", 44308, 0x9d9b9c94, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecMandarino, SpecMandarino, Spectrum)
STD_ROM_FN(SpecMandarino)

struct BurnDriver BurnSpecMandarino = {
	"spec_mandarino", NULL, "spec_spectrum", NULL, "2021",
	"Mandarino, El (English) (48K) (HB)\0", NULL, "Mananuk", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecMandarinoRomInfo, SpecMandarinoRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Mandarino, El (Spanish) (48K) (HB)

static struct BurnRomInfo SpecMandarinosRomDesc[] = {
	{ "Mandarino, El ES 48K (2021)(Mananuk).tap", 44317, 0xe0e44898, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecMandarinos, SpecMandarinos, Spectrum)
STD_ROM_FN(SpecMandarinos)

struct BurnDriver BurnSpecMandarinos = {
	"spec_mandarinos", "spec_mandarino", "spec_spectrum", NULL, "2021",
	"Mandarino, El (Spanish) (48K) (HB)\0", NULL, "Mananuk", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecMandarinosRomInfo, SpecMandarinosRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Manic Jet Set Willy (48K) (HB)

static struct BurnRomInfo SpecManicjswRomDesc[] = {
	{ "Manic Jet Set Willy 48K (2023)(Norman Sword).tap", 35618, 0x57f5b6de, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecManicjsw, SpecManicjsw, Spectrum)
STD_ROM_FN(SpecManicjsw)

struct BurnDriver BurnSpecManicjsw = {
	"spec_manicjsw", NULL, "spec_spectrum", NULL, "2023",
	"Manic Jet Set Willy (48K) (HB)\0", NULL, "Norman Sword", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecManicjswRomInfo, SpecManicjswRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Man with the Golden Joystick, The (128K) (HB, v1.2)

static struct BurnRomInfo SpecMangoldjoyRomDesc[] = {
	{ "Man with the Golden Joystick, The v1.2 (2022)(Sloanysoft).tap", 46063, 0xb12c7a1a, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecMangoldjoy, SpecMangoldjoy, Spec128)
STD_ROM_FN(SpecMangoldjoy)

struct BurnDriver BurnSpecMangoldjoy = {
	"spec_mangoldjoy", NULL, "spec_spec128", NULL, "2022",
	"Man with the Golden Joystick, The (128K) (HB, v1.2)\0", NULL, "Sloanysoft", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecMangoldjoyRomInfo, SpecMangoldjoyRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Match3 (128K) (HB)

static struct BurnRomInfo SpecMatch3RomDesc[] = {
	{ "Match3 128K (2023)(g0blinish).tap", 10210, 0x80bf78e4, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecMatch3, SpecMatch3, Spec128)
STD_ROM_FN(SpecMatch3)

struct BurnDriver BurnSpecMatch3 = {
	"spec_match3", NULL, "spec_spec128", NULL, "2023",
	"Match3 (128K) (HB)\0", "Set '3. Control : 67890' to use controller", "g0blinish", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PUZZLE, 0,
	SpectrumGetZipName, SpecMatch3RomInfo, SpecMatch3RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecIntf2DIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Mariano the Dragon in Capers in Cityland (48K) (HB)

static struct BurnRomInfo SpecMarianodrgRomDesc[] = {
	{ "Mariano the Dragon in Capers in Cityland 48K (2008)(Computer Emuzone).tap", 39238, 0x1ef4a310, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecMarianodrg, SpecMarianodrg, Spectrum)
STD_ROM_FN(SpecMarianodrg)

struct BurnDriver BurnSpecMarianodrg = {
	"spec_marianodrg", NULL, "spec_spectrum", NULL, "2008",
	"Mariano the Dragon in Capers in Cityland (48K) (HB)\0", NULL, "Computer Emuzone", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecMarianodrgRomInfo, SpecMarianodrgRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecQAOPSpaceDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Maria vs. Some Bastards (48K) (HB)

static struct BurnRomInfo SpecMvssbRomDesc[] = {
	{ "Maria vs. Some Bastards 48K (2019)(Manic Miner Technologies).tap", 78543, 0x4fdabb78, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecMvssb, SpecMvssb, Spectrum)
STD_ROM_FN(SpecMvssb)

struct BurnDriver BurnSpecMvssb = {
	"spec_mvssb", NULL, "spec_spectrum", NULL, "2019",
	"Maria vs. Some Bastards (48K) (HB)\0", "Q: left - P: right - Z-M: jump", "Manic Miner Technologies", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecMvssbRomInfo, SpecMvssbRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Mario Bros Game & Watch (48K) (HB)

static struct BurnRomInfo SpecMbgwRomDesc[] = {
	{ "Mario Bros G&W 48K (2023)(Menyiques).tap", 33825, 0xc0b8832f, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecMbgw, SpecMbgw, Spectrum)
STD_ROM_FN(SpecMbgw)

struct BurnDriver BurnSpecMbgw = {
	"spec_mbgw", NULL, "spec_spectrum", NULL, "2023",
	"Mario Bros Game & Watch (48K) (HB)\0", NULL, "Menyiques", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_ACTION, 0,
	SpectrumGetZipName, SpecMbgwRomInfo, SpecMbgwRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Maritrini Freelance Monster Slayer - Adventure (Spanish) (48K) (HB)

static struct BurnRomInfo SpecMaritriadvRomDesc[] = {
	{ "Maritrini Freelance Monster Slayer Adventure ES 48K (2012)(The Mojon Twins).tap", 40996, 0xfc0e6a74, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecMaritriadv, SpecMaritriadv, Spectrum)
STD_ROM_FN(SpecMaritriadv)

struct BurnDriver BurnSpecMaritriadv = {
	"spec_maritriadv", NULL, "spec_spectrum", NULL, "2012",
	"Maritrini Freelance Monster Slayer - Adventure (Spanish) (48K) (HB)\0", NULL, "The Mojon Twins, The", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_ACTION | GBF_ADV, 0,
	SpectrumGetZipName, SpecMaritriadvRomInfo, SpecMaritriadvRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecQAOPSpaceDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Marlow ZX - Episode 1 (128K) (HB)

static struct BurnRomInfo SpecMarlowep1RomDesc[] = {
	{ "Marlow ZX - Episode 1 128K (2024)(Amaweks).tap", 65745, 0x703e655b, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecMarlowep1, SpecMarlowep1, Spec128)
STD_ROM_FN(SpecMarlowep1)

struct BurnDriver BurnSpecMarlowep1 = {
	"spec_marlowep1", NULL, "spec_spec128", NULL, "2024",
	"Marlow ZX - Episode 1 (128K) (HB)\0", NULL, "Amaweks", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecMarlowep1RomInfo, SpecMarlowep1RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Marlow ZX - Episode 2 (128K) (HB)

static struct BurnRomInfo SpecMarlowep2RomDesc[] = {
	{ "Marlow ZX - Episode 2 128K (2024)(Amaweks).tap", 68372, 0x309e237b, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecMarlowep2, SpecMarlowep2, Spec128)
STD_ROM_FN(SpecMarlowep2)

struct BurnDriver BurnSpecMarlowep2 = {
	"spec_marlowep2", "spec_marlowep1", "spec_spec128", NULL, "2024",
	"Marlow ZX - Episode 2 (128K) (HB)\0", NULL, "Amaweks", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecMarlowep2RomInfo, SpecMarlowep2RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Max Stone (English) (128K) (HB)

static struct BurnRomInfo SpecMaxstone128enRomDesc[] = {
	{ "Max Stone EN 128K (2024)(Flopping).tap", 57430, 0xb62757a7, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecMaxstone128en, SpecMaxstone128en, Spec128)
STD_ROM_FN(SpecMaxstone128en)

struct BurnDriver BurnSpecMaxstone128en = {
	"spec_maxstone128en", NULL, "spec_spec128", NULL, "2024",
	"Max Stone (English) (128K) (HB)\0", NULL, "Flopping", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecMaxstone128enRomInfo, SpecMaxstone128enRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Max Stone (English) (48K) (HB)

static struct BurnRomInfo SpecMaxstone48enRomDesc[] = {
	{ "Max Stone EN 48K (2024)(Flopping).tap", 57130, 0xab6380e9, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecMaxstone48en, SpecMaxstone48en, Spec128)
STD_ROM_FN(SpecMaxstone48en)

struct BurnDriver BurnSpecMaxstone48en = {
	"spec_maxstone48en", "spec_maxstone128en", "spec_spec128", NULL, "2024",
	"Max Stone (English) (48K) (HB)\0", NULL, "Flopping", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecMaxstone48enRomInfo, SpecMaxstone48enRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Max Stone (Spanish) (128K) (HB)

static struct BurnRomInfo SpecMaxstone128esRomDesc[] = {
	{ "Max Stone ES 128K (2024)(Flopping).tap", 57394, 0xb882cd4b, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecMaxstone128es, SpecMaxstone128es, Spec128)
STD_ROM_FN(SpecMaxstone128es)

struct BurnDriver BurnSpecMaxstone128es = {
	"spec_maxstone128es", NULL, "spec_spec128", NULL, "2024",
	"Max Stone (Spanish) (128K) (HB)\0", NULL, "Flopping", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecMaxstone128esRomInfo, SpecMaxstone128esRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Max Stone (Spanish) (48K) (HB)

static struct BurnRomInfo SpecMaxstone48esRomDesc[] = {
	{ "Max Stone ES 48K (2024)(Flopping).tap", 57094, 0x9a150977, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecMaxstone48es, SpecMaxstone48es, Spectrum)
STD_ROM_FN(SpecMaxstone48es)

struct BurnDriver BurnSpecMaxstone48es = {
	"spec_maxstone48es", "spec_maxstone128es", "spec_spectrum", NULL, "2024",
	"Max Stone (Spanish) (48K) (HB)\0", NULL, "Flopping", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecMaxstone48esRomInfo, SpecMaxstone48esRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Mazania - Mage Rage Part II (48K) (HB)

static struct BurnRomInfo SpecMazaniaRomDesc[] = {
	{ "Mazania - Mage Rage Part II 48K (2022)(JoeSoft).tap", 40396, 0xf1648263, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecMazania, SpecMazania, Spectrum)
STD_ROM_FN(SpecMazania)

struct BurnDriver BurnSpecMazania = {
	"spec_mazania", NULL, "spec_spectrum", NULL, "2022",
	"Mazania - Mage Rage Part II (48K) (HB)\0", NULL, "JoeSoft", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_ACTION | GBF_MAZE, 0,
	SpectrumGetZipName, SpecMazaniaRomInfo, SpecMazaniaRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Maze Death Rally-X (48K) (HB, v1.6)

static struct BurnRomInfo SpecMdrallyxRomDesc[] = {
	{ "Maze Death Rally-X 48K (2018)(Tom Dalby).tap", 26219, 0xe8c301b0, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecMdrallyx, SpecMdrallyx, Spectrum)
STD_ROM_FN(SpecMdrallyx)

struct BurnDriver BurnSpecMdrallyx = {
	"spec_mdrallyx", NULL, "spec_spectrum", NULL, "2018",
	"Maze Death Rally-X (48K) (HB, v1.6)\0", NULL, "Tom Dalby", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_ACTION, 0,
	SpectrumGetZipName, SpecMdrallyxRomInfo, SpecMdrallyxRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Mazy (48K) (HB)

static struct BurnRomInfo SpecMazyRomDesc[] = {
	{ "Mazy (2021)(Inufuto).tap", 7646, 0x2a38c609, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecMazy, SpecMazy, Spectrum)
STD_ROM_FN(SpecMazy)

struct BurnDriver BurnSpecMazy = {
	"spec_mazy", NULL, "spec_spectrum", NULL, "2021",
	"Mazy (48K) (HB)\0", "AY Sound supported", "Inufuto", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_MAZE | GBF_ACTION, 0,
	SpectrumGetZipName, SpecMazyRomInfo, SpecMazyRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Meanie Golf (48K) (HB)

static struct BurnRomInfo SpecMeaniegolfRomDesc[] = {
	{ "Meanie Golf 48K (2023)(PsychicParrot).tap", 37359, 0x58d9e059, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecMeaniegolf, SpecMeaniegolf, Spectrum)
STD_ROM_FN(SpecMeaniegolf)

struct BurnDriver BurnSpecMeaniegolf = {
	"spec_meaniegolf", NULL, "spec_spectrum", NULL, "2023",
	"Meanie Golf (48K) (HB)\0", NULL, "Psychic Parrot", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 2, HARDWARE_SPECTRUM, GBF_ACTION | GBF_SPORTSMISC, 0,
	SpectrumGetZipName, SpecMeaniegolfRomInfo, SpecMeaniegolfRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Meanie Golf - Side B (48K) (HB)

static struct BurnRomInfo SpecMeaniegolfbRomDesc[] = {
	{ "Meanie Golf - Side B 48K (2023)(PsychicParrot).tap", 39351, 0x21a61a8f, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecMeaniegolfb, SpecMeaniegolfb, Spectrum)
STD_ROM_FN(SpecMeaniegolfb)

struct BurnDriver BurnSpecMeaniegolfb = {
	"spec_meaniegolfb", "spec_meaniegolf", "spec_spectrum", NULL, "2023",
	"Meanie Golf - Side B (48K) (HB)\0", NULL, "Psychic Parrot", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE | BDF_HOMEBREW, 2, HARDWARE_SPECTRUM, GBF_ACTION | GBF_SPORTSMISC, 0,
	SpectrumGetZipName, SpecMeaniegolfbRomInfo, SpecMeaniegolfbRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Mechwars Arena (48K) (HB)

static struct BurnRomInfo SpecMwarenaRomDesc[] = {
	{ "Mechwars Arena (2022)(ZX Bitles).tap", 48091, 0x8a8ebce4, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecMwarena, SpecMwarena, Spectrum)
STD_ROM_FN(SpecMwarena)

struct BurnDriver BurnSpecMwarena = {
	"spec_mwarena", NULL, "spec_spectrum", NULL, "2022",
	"Mechwars Arena (48K) (HB)\0", NULL, "ZX Bitles", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 2, HARDWARE_SPECTRUM, GBF_ACTION, 0,
	SpectrumGetZipName, SpecMwarenaRomInfo, SpecMwarenaRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecIntf2SwapDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Mechwars Centipede (48K) (HB)

static struct BurnRomInfo SpecMwcentipRomDesc[] = {
	{ "Mechwars Centipede 48K (2021)(ZX Bitles).tap", 47448, 0x1bca4c7d, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecMwcentip, SpecMwcentip, Spectrum)
STD_ROM_FN(SpecMwcentip)

struct BurnDriver BurnSpecMwcentip = {
	"spec_mwcentip", NULL, "spec_spectrum", NULL, "2021",
	"Mechwars Centipede (48K) (HB)\0", NULL, "ZX Bitles", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_ACTION | GBF_SHOOT, 0,
	SpectrumGetZipName, SpecMwcentipRomInfo, SpecMwcentipRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecIntf2SwapDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Mega Manic Mulholland (48K) (HB, v2.0)

static struct BurnRomInfo SpecMmmv2RomDesc[] = {
	{ "Mega Manic Mulholland v2.0 48K (2021)(Sloanysoft).tap", 41809, 0xbd87a5b1, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecMmmv2, SpecMmmv2, Spectrum)
STD_ROM_FN(SpecMmmv2)

struct BurnDriver BurnSpecMmmv2 = {
	"spec_mmmv2", NULL, "spec_spectrum", NULL, "2021",
	"Mega Manic Mulholland (48K) (HB, v2.0)\0", NULL, "Sloanysoft", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecMmmv2RomInfo, SpecMmmv2RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Mecki (128K) (HB)

static struct BurnRomInfo SpecMeckiRomDesc[] = {
	{ "Mecki 128K (2022)(Sebastian Braunert).tap", 39887, 0xe872c62d, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecMecki, SpecMecki, Spec128)
STD_ROM_FN(SpecMecki)

struct BurnDriver BurnSpecMecki = {
	"spec_mecki", NULL, "spec_spec128", NULL, "2022",
	"Mecki (128K) (HB)\0", NULL, "Sebastian Braunert", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_HORSHOOT, 0,
	SpectrumGetZipName, SpecMeckiRomInfo, SpecMeckiRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecIntf2DIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// MERGED! (128K) (HB, v1.0.1)

static struct BurnRomInfo SpecMergedRomDesc[] = {
	{ "MERGED! v1.0.1 128K (2022)(Dimon).tap", 42454, 0x03f33f0a, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecMerged, SpecMerged, Spec128)
STD_ROM_FN(SpecMerged)

struct BurnDriver BurnSpecMerged = {
	"spec_merged", NULL, "spec_spec128", NULL, "2022",
	"MERGED! (128K) (HB, v1.0.1)\0", NULL, "Dimon", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PUZZLE, 0,
	SpectrumGetZipName, SpecMergedRomInfo, SpecMergedRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecIntf2DIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Metal Brain (128K) (HB)

static struct BurnRomInfo SpecMetalbrain128RomDesc[] = {
	{ "Metal Brain 128K (2024)(Rod_Mad).tap", 52066, 0xa1985b2c, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecMetalbrain128, SpecMetalbrain128, Spec128)
STD_ROM_FN(SpecMetalbrain128)

struct BurnDriver BurnSpecMetalbrain128 = {
	"spec_metalbrain128", NULL, "spec_spec128", NULL, "2024",
	"Metal Brain (128K) (HB)\0", NULL, "Rod_Mad", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecMetalbrain128RomInfo, SpecMetalbrain128RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Metal Brain (48K) (HB)

static struct BurnRomInfo SpecMetalbrain48RomDesc[] = {
	{ "Metal Brain 48K (2024)(RodMad).tap", 51459, 0x450392ad, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecMetalbrain48, SpecMetalbrain48, Spectrum)
STD_ROM_FN(SpecMetalbrain48)

struct BurnDriver BurnSpecMetalbrain48 = {
	"spec_metalbrain48", "spec_metalbrain128", "spec_spectrum", NULL, "2024",
	"Metal Brain (48K) (HB)\0", NULL, "Rod_Mad", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecMetalbrain48RomInfo, SpecMetalbrain48RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Mickey and Donald Game & Watch (48K) (HB)

static struct BurnRomInfo SpecMickeydongwRomDesc[] = {
	{ "Mickey and Donald G&W 48K (2023)(Menyiques).tap", 37646, 0x93b6eb9d, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecMickeydongw, SpecMickeydongw, Spectrum)
STD_ROM_FN(SpecMickeydongw)

struct BurnDriver BurnSpecMickeydongw = {
	"spec_mickeydongw", NULL, "spec_spectrum", NULL, "2023",
	"Mickey and Donald Game & Watch (48K) (HB)\0", NULL, "Menyiques", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_ACTION, 0,
	SpectrumGetZipName, SpecMickeydongwRomInfo, SpecMickeydongwRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecQAOPSpaceDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Micro INC (48K) (HB)

static struct BurnRomInfo SpecMicroincRomDesc[] = {
	{ "Micro INC 48K (2014)(usebox.net).tap", 22573, 0x480427d0, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecMicroinc, SpecMicroinc, Spectrum)
STD_ROM_FN(SpecMicroinc)

struct BurnDriver BurnSpecMicroinc = {
	"spec_microinc", NULL, "spec_spectrum", NULL, "2014",
	"Micro INC (48K) (HB)\0", NULL, "usebox.net", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PUZZLE, 0,
	SpectrumGetZipName, SpecMicroincRomInfo, SpecMicroincRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Micro Profanation (48K) (HB, v1.1)

static struct BurnRomInfo SpecMprofanationRomDesc[] = {
	{ "Micro Profanation 48K v1.1 (2022)(Ivan Gomez Marquez).z80", 6912, 0x067f74cd, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecMprofanation, SpecMprofanation, Spectrum)
STD_ROM_FN(SpecMprofanation)

struct BurnDriver BurnSpecMprofanation = {
	"spec_mprofanation", NULL, "spec_spectrum", NULL, "2022",
	"Micro Profanation (48K) (HB, v1.1)\0", "O: left - P: right - Z: long jump - X: short jump", "Ivan Gomez Marquez", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecMprofanationRomInfo, SpecMprofanationRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Mighty Final Fight (128K) (HB, v1.2)

static struct BurnRomInfo SpecMightyffRomDesc[] = {
	{ "Mighty Final Fight v1.2 128K (2023)(Sinc LAIR).tap", 302058, 0xe2a30cd8, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecMightyff, SpecMightyff, Spec128)
STD_ROM_FN(SpecMightyff)

struct BurnDriver BurnSpecMightyff = {
	"spec_mightyff", NULL, "spec_spec128", NULL, "2018-2023",
	"Mighty Final Fight (128K) (HB, v1.2)\0", "Q, A and SPACE to select options in title screen", "Sinc LAIR, SaNchez", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_SCRFIGHT, 0,
	SpectrumGetZipName, SpecMightyffRomInfo, SpecMightyffRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Mijadore vase, The (English) (128K) (HB)

static struct BurnRomInfo SpecMijavaseeRomDesc[] = {
	{ "Mijadore vase, The EN 128K (2023)(EdD).tap", 50242, 0xe846116a, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecMijavasee, SpecMijavasee, Spec128)
STD_ROM_FN(SpecMijavasee)

struct BurnDriver BurnSpecMijavasee = {
	"spec_mijavasee", NULL, "spec_spec128", NULL, "2023",
	"Mijadore vase, The (English) (128K) (HB)\0", NULL, "EdD", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecMijavaseeRomInfo, SpecMijavaseeRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Mijadore vase, The (Spanish) (128K) (HB)

static struct BurnRomInfo SpecMijavasesRomDesc[] = {
	{ "Mijadore vase, The ES 128K (2023)(EdD).tap", 50243, 0xfcc46b25, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecMijavases, SpecMijavases, Spec128)
STD_ROM_FN(SpecMijavases)

struct BurnDriver BurnSpecMijavases = {
	"spec_mijavases", "spec_mijavasee", "spec_spec128", NULL, "2023",
	"Mijadore vase, The (Spanish) (128K) (HB)\0", NULL, "EdD", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecMijavasesRomInfo, SpecMijavasesRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Mike, The Guitar (128K) (HB)

static struct BurnRomInfo SpecMiketheguitarRomDesc[] = {
	{ "Mike, The Guitar 128K (2018)(Sebastian Braunert).tap", 62258, 0xdcaeb665, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecMiketheguitar, SpecMiketheguitar, Spec128)
STD_ROM_FN(SpecMiketheguitar)

struct BurnDriver BurnSpecMiketheguitar = {
	"spec_miketheguitar", NULL, "spec_spec128", NULL, "2018",
	"Mike, The Guitar (128K) (HB)\0", NULL, "Sebastian Braunert", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_ACTION, 0,
	SpectrumGetZipName, SpecMiketheguitarRomInfo, SpecMiketheguitarRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Mike the Guitar - The Shooter (128K) (HB)

static struct BurnRomInfo SpecMiketheguitar2RomDesc[] = {
	{ "Mike the Guitar - The Shooter (2019)(Sebastian Braunert).tap", 42463, 0x02644948, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecMiketheguitar2, SpecMiketheguitar2, Spec128)
STD_ROM_FN(SpecMiketheguitar2)

struct BurnDriver BurnSpecMiketheguitar2 = {
	"spec_miketheguitar2", NULL, "spec_spec128", NULL, "2019",
	"Mike the Guitar - The Shooter (128K) (HB)\0", NULL, "Sebastian Braunert", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_HORSHOOT, 0,
	SpectrumGetZipName, SpecMiketheguitar2RomInfo, SpecMiketheguitar2RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecQAOPMDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Min! (128K) (HB)

static struct BurnRomInfo SpecMin128RomDesc[] = {
	{ "Min! 128K (2024)(Zebra Games).tap", 40788, 0x284058a3, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecMin128, SpecMin128, Spec128)
STD_ROM_FN(SpecMin128)

struct BurnDriver BurnSpecMin128 = {
	"spec_min128", NULL, "spec_spec128", NULL, "2024",
	"Min! (128K) (HB)\0", "Default Keys: W, S, A, D, M", "Zebra Games", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PUZZLE, 0,
	SpectrumGetZipName, SpecMin128RomInfo, SpecMin128RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecQAOPSpaceDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Misco Jones - Raiders of the lost Vah-Ka (48K) (HB)

static struct BurnRomInfo SpecMiscojonsRomDesc[] = {
	{ "Misco Jones - Raiders of the lost Vah-Ka 48K (2013)(Radastan).tap", 38349, 0xff59438c, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecMiscojons, SpecMiscojons, Spectrum)
STD_ROM_FN(SpecMiscojons)

struct BurnDriver BurnSpecMiscojons = {
	"spec_miscojons", NULL, "spec_spectrum", NULL, "2013",
	"Misco Jones - Raiders of the lost Vah-Ka (48K) (HB)\0", NULL, "Radastan", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecMiscojonsRomInfo, SpecMiscojonsRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Misco Jones - En busca de la Vah-Ka perdida (48K) (HB)

static struct BurnRomInfo SpecMiscojoneRomDesc[] = {
	{ "Misco Jones - En busca de la Vah-Ka perdida 48K (2013)(Radastan).tap", 38471, 0xf635c825, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecMiscojone, SpecMiscojone, Spectrum)
STD_ROM_FN(SpecMiscojone)

struct BurnDriver BurnSpecMiscojone = {
	"spec_miscojone", "spec_miscojons", "spec_spectrum", NULL, "2013",
	"Misco Jones - En busca de la Vah-Ka perdida (48K) (HB)\0", NULL, "Radastan", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecMiscojoneRomInfo, SpecMiscojoneRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Misifu la Gatita (128K) (HB)

static struct BurnRomInfo SpecMisifuRomDesc[] = {
	{ "Misifu la Gatita 128K (2019)(Jordi).tap", 47525, 0x2c13468b, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecMisifu, SpecMisifu, Spec128)
STD_ROM_FN(SpecMisifu)

struct BurnDriver BurnSpecMisifu = {
	"spec_misifu", NULL, "spec_spec128", NULL, "2019",
	"Misifu la Gatita (128K) (HB)\0", NULL, "Jordi", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_ACTION, 0,
	SpectrumGetZipName, SpecMisifuRomInfo, SpecMisifuRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Misifu Remeow Color (128K) (HB, v1.4.1)

static struct BurnRomInfo SpecmisifucolorRomDesc[] = {
	{ "misifu_COLOR(2021)(JSM Soft).tap",  73610, 0x387dd30a, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specmisifucolor, Specmisifucolor, Spec128)
STD_ROM_FN(Specmisifucolor)

struct BurnDriver BurnSpecmisifucolor = {
	"spec_misifucolor", NULL, "spec_spec128", NULL, "2021",
	"Misifu Remeow Color (128K) (HB, v1.4.1)\0", "Alley Cat demake for ZX Spectrum", "JSM Soft", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_ACTION, 0,
	SpectrumGetZipName, SpecmisifucolorRomInfo, SpecmisifucolorRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3

};

// Misifu Remeow CGA (128K) (HB, v1.4.1)

static struct BurnRomInfo SpecmisifucgaRomDesc[] = {
	{ "misifu_CGA(2021)(JSM Soft).tap", 73469, 0x84550b57, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specmisifucga, Specmisifucga, Spec128)
STD_ROM_FN(Specmisifucga)

struct BurnDriver BurnSpecmisifucga = {
	"spec_misifucga", "spec_misifucolor", "spec_spec128", NULL, "2021",
	"Misifu Remeow CGA (128K) (HB, v1.4.1)\0", "Alley Cat demake for ZX Spectrum", "JSM Soft", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_ACTION, 0,
	SpectrumGetZipName, SpecmisifucgaRomInfo, SpecmisifucgaRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Misterio (English) (48K) (HB)

static struct BurnRomInfo SpecMisterienRomDesc[] = {
	{ "Misterio EN 48K (2023)(FitoSoft).tap", 46476, 0x0b2bc46c, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecMisterien, SpecMisterien, Spectrum)
STD_ROM_FN(SpecMisterien)

struct BurnDriver BurnSpecMisterien = {
	"spec_misterien", NULL, "spec_spectrum", NULL, "2023",
	"Misterio (English) (48K) (HB)\0", NULL, "Fitosoft", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 4, HARDWARE_SPECTRUM, GBF_BOARD, 0,
	SpectrumGetZipName, SpecMisterienRomInfo, SpecMisterienRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Misterio (Spanish) (48K) (HB)

static struct BurnRomInfo SpecMisteriesRomDesc[] = {
	{ "Misterio ES 48K (2023)(FitoSoft).tap", 46639, 0xd6709e91, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecMisteries, SpecMisteries, Spectrum)
STD_ROM_FN(SpecMisteries)

struct BurnDriver BurnSpecMisteries = {
	"spec_misteries", "spec_misterien", "spec_spectrum", NULL, "2023",
	"Misterio (Spanish) (48K) (HB)\0", NULL, "Fitosoft", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE | BDF_HOMEBREW, 4, HARDWARE_SPECTRUM, GBF_BOARD, 0,
	SpectrumGetZipName, SpecMisteriesRomInfo, SpecMisteriesRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Misterio (Portuguese) (48K) (HB)

static struct BurnRomInfo SpecMisteriptRomDesc[] = {
	{ "Misterio PT 48K (2023)(FitoSoft).tap", 46577, 0xf2b13a75, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecMisteript, SpecMisteript, Spectrum)
STD_ROM_FN(SpecMisteript)

struct BurnDriver BurnSpecMisteript = {
	"spec_misteript", "spec_misterien", "spec_spectrum", NULL, "2023",
	"Misterio (Portuguese) (48K) (HB)\0", NULL, "Fitosoft", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE | BDF_HOMEBREW, 4, HARDWARE_SPECTRUM, GBF_BOARD, 0,
	SpectrumGetZipName, SpecMisteriptRomInfo, SpecMisteriptRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Mochilo ZX (48K) (HB)

static struct BurnRomInfo SpecMochilozxRomDesc[] = {
	{ "Mochilo ZX 48K (2024)(rcougil).tap", 50961, 0xa2a2b641, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecMochilozx, SpecMochilozx, Spectrum)
STD_ROM_FN(SpecMochilozx)

struct BurnDriver BurnSpecMochilozx = {
	"spec_mochilozx", NULL, "spec_spectrum", NULL, "2024",
	"Mochilo ZX (48K) (HB)\0", NULL, "rcougil", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecMochilozxRomInfo, SpecMochilozxRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Moderate Retribution (128K) (HB)

static struct BurnRomInfo SpecModerateretributionRomDesc[] = {
	{ "Moderate Retribution (2013)(Stonechat).tap", 30951, 0x29efa9e5, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecModerateretribution, SpecModerateretribution, Spec128)
STD_ROM_FN(SpecModerateretribution)

struct BurnDriver BurnSpecModerateretribution = {
	"spec_moderateretribution", NULL, "spec_spec128", NULL, "2013",
	"Moderate Retribution (128K) (HB)\0", NULL, "Stonechat", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_HORSHOOT, 0,
	SpectrumGetZipName, SpecModerateretributionRomInfo, SpecModerateretributionRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecQAOPSpaceDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Mojoni - kid (English) (128K) (HB, v1.1)

static struct BurnRomInfo SpecMojoniekRomDesc[] = {
	{ "Mojoni - kid EN v1.1 (2022)(Isaias Diaz).tap", 47022, 0xb83dc99a, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecMojoniek, SpecMojoniek, Spec128)
STD_ROM_FN(SpecMojoniek)

struct BurnDriver BurnSpecMojoniek = {
	"spec_mojoniek", NULL, "spec_spec128", NULL, "2022",
	"Mojoni - kid (English) (128K) (HB, v1.1)\0", NULL, "Isaias Diaz", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecMojoniekRomInfo, SpecMojoniekRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Mojoni - men (English) (128K) (HB, v1.1)

static struct BurnRomInfo SpecMojoniemRomDesc[] = {
	{ "Mojoni - men EN v1.1 (2022)(Isaias Diaz).tap", 47153, 0x078b037e, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecMojoniem, SpecMojoniem, Spec128)
STD_ROM_FN(SpecMojoniem)

struct BurnDriver BurnSpecMojoniem = {
	"spec_mojoniem", "spec_mojoniek", "spec_spec128", NULL, "2022",
	"Mojoni - men (English) (128K) (HB, v1.1)\0", NULL, "Isaias Diaz", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecMojoniemRomInfo, SpecMojoniemRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Mojoni- macho (Spanish) (128K) (HB, v1.1)

static struct BurnRomInfo SpecMojonismRomDesc[] = {
	{ "Mojoni- macho SP v1.1 (2022)(Isaias Diaz).tap", 47161, 0x4c4a424a, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecMojonism, SpecMojonism, Spec128)
STD_ROM_FN(SpecMojonism)

struct BurnDriver BurnSpecMojonism = {
	"spec_mojonism", "spec_mojoniek", "spec_spec128", NULL, "2022",
	"Mojoni- macho (Spanish) (128K) (HB, v1.1)\0", NULL, "Isaias Diaz", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecMojonismRomInfo, SpecMojonismRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Mojoni- nene (Spanish) (128K) (HB, v1.1)

static struct BurnRomInfo SpecMojonisnRomDesc[] = {
	{ "Mojoni- nene SP v1.1 (2022)(Isaias Diaz).tap", 47024, 0x2a1cfd5b, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecMojonisn, SpecMojonisn, Spec128)
STD_ROM_FN(SpecMojonisn)

struct BurnDriver BurnSpecMojonisn = {
	"spec_mojonisn", "spec_mojoniek", "spec_spec128", NULL, "2022",
	"Mojoni- nene (Spanish) (128K) (HB, v1.1)\0", NULL, "Isaias Diaz", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecMojonisnRomInfo, SpecMojonisnRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Money Mouse in Full Barn House - Ep. 1 (48K) (HB)

static struct BurnRomInfo SpecMoneymouse1RomDesc[] = {
	{ "Money Mouse in Full Barn House - Ep 1 48K (2024)(KTB Productions).tap", 73790, 0x26193881, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecMoneymouse1, SpecMoneymouse1, Spectrum)
STD_ROM_FN(SpecMoneymouse1)

struct BurnDriver BurnSpecMoneymouse1 = {
	"spec_moneymouse1", NULL, "spec_spectrum", NULL, "2024",
	"Money Mouse in Full Barn House - Ep. 1 (48K) (HB)\0", NULL, "KTB Productions", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_MAZE, 0,
	SpectrumGetZipName, SpecMoneymouse1RomInfo, SpecMoneymouse1RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecQAOPMDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Money Mouse in Full Barn House - Ep. 2 (48K) (HB)

static struct BurnRomInfo SpecMoneymouse2RomDesc[] = {
	{ "Money Mouse in Full Barn House - Ep 2 48K (2024)(KTB Productions).tap", 64409, 0x14ffff53, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecMoneymouse2, SpecMoneymouse2, Spectrum)
STD_ROM_FN(SpecMoneymouse2)

struct BurnDriver BurnSpecMoneymouse2 = {
	"spec_moneymouse2", "spec_moneymouse1", "spec_spectrum", NULL, "2024",
	"Money Mouse in Full Barn House - Ep. 2 (48K) (HB)\0", NULL, "KTB Productions", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_MAZE, 0,
	SpectrumGetZipName, SpecMoneymouse2RomInfo, SpecMoneymouse2RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecQAOPMDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Monty Mole And The Temple Of Lost Souls (128K) (HB)

static struct BurnRomInfo SpecMmlostsoulsRomDesc[] = {
	{ "Monty Mole And The Temple Of Lost Souls (2017)(Bubblesoft).tap", 44714, 0xa1b4bf06, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecMmlostsouls, SpecMmlostsouls, Spec128)
STD_ROM_FN(SpecMmlostsouls)

struct BurnDriver BurnSpecMmlostsouls = {
	"spec_mmlostsouls", NULL, "spec_spec128", NULL, "2017",
	"Monty Mole And The Temple Of Lost Souls (128K) (HB)\0", NULL, "Bubblesoft", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecMmlostsoulsRomInfo, SpecMmlostsoulsRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Monty's Honey Run (128K) (HB)

static struct BurnRomInfo SpecMontyhoneyRomDesc[] = {
	{ "Monty's Honey Run 128K (2017)(Bubblesoft).tap", 44457, 0xaddb28f2, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecMontyhoney, SpecMontyhoney, Spec128)
STD_ROM_FN(SpecMontyhoney)

struct BurnDriver BurnSpecMontyhoney = {
	"spec_montyhoney", NULL, "spec_spec128", NULL, "2017",
	"Monty's Honey Run (128K) (HB)\0", NULL, "Bubblesoft", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecMontyhoneyRomInfo, SpecMontyhoneyRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Moon and the Pirates (English) (48K) (HB)

static struct BurnRomInfo SpecMoonpiratenRomDesc[] = {
	{ "Moon and the Pirates EN 48K (2019)(Furillo Productions).tap", 44496, 0xae6e18e3, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecMoonpiraten, SpecMoonpiraten, Spectrum)
STD_ROM_FN(SpecMoonpiraten)

struct BurnDriver BurnSpecMoonpiraten = {
	"spec_moonpiraten", NULL, "spec_spectrum", NULL, "2019",
	"Moon and the Pirates (English) (48K) (HB)\0", NULL, "Furillo Productions", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_ADV, 0,
	SpectrumGetZipName, SpecMoonpiratenRomInfo, SpecMoonpiratenRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Moon and the Pirates (Spanish) (48K) (HB)

static struct BurnRomInfo SpecMoonpiratesRomDesc[] = {
	{ "Moon and the Pirates ES 48K (2019)(Furillo Productions).tap", 44493, 0x57323cf4, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecMoonpirates, SpecMoonpirates, Spectrum)
STD_ROM_FN(SpecMoonpirates)

struct BurnDriver BurnSpecMoonpirates = {
	"spec_moonpirates", "spec_moonpiraten", "spec_spectrum", NULL, "2019",
	"Moon and the Pirates (Spanish) (48K) (HB)\0", NULL, "Furillo Productions", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_ADV, 0,
	SpectrumGetZipName, SpecMoonpiratesRomInfo, SpecMoonpiratesRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Moon Ranger (128K) (HB, v2)

static struct BurnRomInfo SpecMoonrangerv2RomDesc[] = {
	{ "Moon Ranger v2 128K (2021)(Gabriele Amore).tap", 45527, 0xebb5c2a1, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecMoonrangerv2, SpecMoonrangerv2, Spec128)
STD_ROM_FN(SpecMoonrangerv2)

struct BurnDriver BurnSpecMoonrangerv2 = {
	"spec_moonrangerv2", NULL, "spec_spec128", NULL, "2021",
	"Moon Ranger (128K) (HB, v2)\0", NULL, "Gabriele Amore", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_ACTION, 0,
	SpectrumGetZipName, SpecMoonrangerv2RomInfo, SpecMoonrangerv2RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecQAOPMDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// More Tea Vicar? (128K) (HB)

static struct BurnRomInfo SpecMoreteavicarRomDesc[] = {
	{ "More Tea Vicar 128K (2012)(Cronosoft).tap", 60372, 0xcb451c56, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecMoreteavicar, SpecMoreteavicar, Spec128)
STD_ROM_FN(SpecMoreteavicar)

struct BurnDriver BurnSpecMoreteavicar = {
	"spec_moreteavicar", NULL, "spec_spec128", NULL, "2012",
	"More Tea Vicar? (128K) (HB)\0", NULL, "Cronosoft", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_HORSHOOT, 0,
	SpectrumGetZipName, SpecMoreteavicarRomInfo, SpecMoreteavicarRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Moritz (48K) (HB)

static struct BurnRomInfo SpecMoritzRomDesc[] = {
	{ "Moritz 48K (2017)(Sebastian Braunert).z80", 39114, 0xf0f22000, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecMoritz, SpecMoritz, Spectrum)
STD_ROM_FN(SpecMoritz)

struct BurnDriver BurnSpecMoritz = {
	"spec_moritz", NULL, "spec_spectrum", NULL, "2017",
	"Moritz (48K) (HB)\0", NULL, "Sebastian Braunert", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecMoritzRomInfo, SpecMoritzRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Moritz on the Autobahn (128K)

static struct BurnRomInfo SpecMoritzautobathRomDesc[] = {
	{ "Moritz on the Autobahn 128K (2020)(Sebastian Braunert).tap", 64072, 0xde66f32f, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecMoritzautobath, SpecMoritzautobath, Spec128)
STD_ROM_FN(SpecMoritzautobath)

struct BurnDriver BurnSpecMoritzautobath = {
	"spec_moritzautobath", NULL, "spec_spec128", NULL, "2020",
	"Moritz on the Autobahn (128K) (HB)\0", NULL, "Sebastian Braunert", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_ACTION, 0,
	SpectrumGetZipName, SpecMoritzautobathRomInfo, SpecMoritzautobathRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecQAOPSpaceDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Moritz the Striker (48K) (HB)

static struct BurnRomInfo SpecMoritzstrikerRomDesc[] = {
	{ "Moritz the Striker 48K (2020)(Sebastian Braunert).tap", 60560, 0xb1402da9, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecMoritzstriker, SpecMoritzstriker, Spectrum)
STD_ROM_FN(SpecMoritzstriker)

struct BurnDriver BurnSpecMoritzstriker = {
	"spec_moritzstriker", NULL, "spec_spectrum", NULL, "2020",
	"Moritz the Striker (48K) (HB)\0", NULL, "Sebastian Braunert", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecMoritzstrikerRomInfo, SpecMoritzstrikerRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Moritz to the Moon (48K) (HB)

static struct BurnRomInfo SpecMoritzmoonRomDesc[] = {
	{ "Moritz to the Moon 48K (2019)(Sebastian Braunert).tap", 42447, 0x04bfae60, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecMoritzmoon, SpecMoritzmoon, Spectrum)
STD_ROM_FN(SpecMoritzmoon)

struct BurnDriver BurnSpecMoritzmoon = {
	"spec_moritzmoon", NULL, "spec_spectrum", NULL, "2019",
	"Moritz to the Moon (48K) (HB)\0", NULL, "Sebastian Braunert", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_VERSHOOT, 0,
	SpectrumGetZipName, SpecMoritzmoonRomInfo, SpecMoritzmoonRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecIntf2DIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Moritz Strikes Back (48K)

static struct BurnRomInfo SpecMoritzstkbckRomDesc[] = {
	{ "Moritz Strikes Back 48K (2019)(Sebastian Braunert).tap", 44313, 0xe4b7b51f, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecMoritzstkbck, SpecMoritzstkbck, Spectrum)
STD_ROM_FN(SpecMoritzstkbck)

struct BurnDriver BurnSpecMoritzstkbck = {
	"spec_moritzstkbck", NULL, "spec_spectrum", NULL, "2019",
	"Moritz Strikes Back (48K) (HB)\0", NULL, "Sebastian Braunert", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_ACTION, 0,
	SpectrumGetZipName, SpecMoritzstkbckRomInfo, SpecMoritzstkbckRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecIntf2DIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Mowy Adventures - Beer Quest (128K) (HB)

static struct BurnRomInfo SpecMowyadv128RomDesc[] = {
	{ "Mowy Adventures - Beer Quest 128K (2021)(Mowy Entertainment).tap", 60776, 0x9db853c6, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecMowyadv128, SpecMowyadv128, Spec128)
STD_ROM_FN(SpecMowyadv128)

struct BurnDriver BurnSpecMowyadv128 = {
	"spec_mowyadv128", NULL, "spec_spec128", NULL, "2021",
	"Mowy Adventures - Beer Quest (128K) (HB)\0", NULL, "Mowy Entertainment", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecMowyadv128RomInfo, SpecMowyadv128RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Mowy Adventures - Beer Quest (48K) (HB)

static struct BurnRomInfo SpecMowyadv48RomDesc[] = {
	{ "Mowy Adventures - Beer Quest 48K (2021)(Mowy Entertainment).tap", 40708, 0x48654617, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecMowyadv48, SpecMowyadv48, Spectrum)
STD_ROM_FN(SpecMowyadv48)

struct BurnDriver BurnSpecMowyadv48 = {
	"spec_mowyadv48", "spec_mowyadv128", "spec_spectrum", NULL, "2021",
	"Mowy Adventures - Beer Quest (48K) (HB)\0", NULL, "Mowy Entertainment", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecMowyadv48RomInfo, SpecMowyadv48RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Hair Raising Adventures of Mr Hair, The (128K) (HB)

static struct BurnRomInfo SpecMrhairRomDesc[] = {
	{ "Hair Raising Adventures of Mr Hair, The (2019)(Lee Stevenson).tap", 46747, 0x240d58bc, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecMrhair, SpecMrhair, Spec128)
STD_ROM_FN(SpecMrhair)

struct BurnDriver BurnSpecMrhair = {
	"spec_mrhair", "spec_mrhairiv", "spec_spec128", NULL, "2019",
	"Hair Raising Adventures of Mr Hair, The (128K) (HB)\0", NULL, "Lee Stevenson", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecMrhairRomInfo, SpecMrhairRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Hair Raising Adventures of Mr Hair, The - Imp. version (128K) (HB)

static struct BurnRomInfo SpecMrhairivRomDesc[] = {
	{ "Hair Raising Adventures of Mr Hair, The - Imp. version 128K (2024)(MicroChops).tap", 47133, 0xe8c66770, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecMrhairiv, SpecMrhairiv, Spec128)
STD_ROM_FN(SpecMrhairiv)

struct BurnDriver BurnSpecMrhairiv = {
	"spec_mrhairiv", NULL, "spec_spec128", NULL, "2020-24",
	"Hair Raising Adventures of Mr Hair, The - Imp. version (128K) (HB)\0", NULL, "MicroChops", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecMrhairivRomInfo, SpecMrhairivRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Mr Hair & The Fly (128K) (HB)

static struct BurnRomInfo SpecMrhair2RomDesc[] = {
	{ "Mr Hair & THE FLY (2021)(BitmapSoft).tap", 48426, 0xa78da9b2, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecMrhair2, SpecMrhair2, Spec128)
STD_ROM_FN(SpecMrhair2)

struct BurnDriver BurnSpecMrhair2 = {
	"spec_mrhair2", NULL, "spec_spec128", NULL, "2021",
	"Mr Hair & The Fly (128K) (HB)\0", NULL, "Bitmap Soft", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecMrhair2RomInfo, SpecMrhair2RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Mr Hair and the Kitty Katakombs (128K) (HB)

static struct BurnRomInfo SpecMrhairkkRomDesc[] = {
	{ "Mr Hair and the Kitty Katakombs (2022)(Chopz).tap", 48063, 0xcf8e1e19, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecMrhairkk, SpecMrhairkk, Spec128)
STD_ROM_FN(SpecMrhairkk)

struct BurnDriver BurnSpecMrhairkk = {
	"spec_mrhairkk", NULL, "spec_spec128", NULL, "2022",
	"Mr Hair and the Kitty Katakombs (128K) (HB)\0", NULL, "Chopz", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecMrhairkkRomInfo, SpecMrhairkkRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Mr Hair's - A Very Hairy RemiX (128K) (HB)

static struct BurnRomInfo SpecMrhair3RomDesc[] = {
	{ "Mr Hairs A Very Hairy RemiX 128K (2022)(Chopz).tap", 47052, 0x055ed60b, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecMrhair3, SpecMrhair3, Spec128)
STD_ROM_FN(SpecMrhair3)

struct BurnDriver BurnSpecMrhair3 = {
	"spec_mrhair3", NULL, "spec_spec128", NULL, "2022",
	"Mr Hair's - A Very Hairy RemiX (128K) (HB)\0", NULL, "Chopz", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecMrhair3RomInfo, SpecMrhair3RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Ms. Nam-Pac (128K) (HB, v0.0.5)

static struct BurnRomInfo SpecMsnampacRomDesc[] = {
	{ "Ms Nam-Pac v0.0.5 128K (2022)(Jordi).tap", 33045, 0x70c69fdc, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecMsnampac, SpecMsnampac, Spec128)
STD_ROM_FN(SpecMsnampac)

struct BurnDriver BurnSpecMsnampac = {
	"spec_msnampac", NULL, "spec_spec128", NULL, "2022",
	"Ms. Nam-Pac (128K) (HB, v0.0.5)\0", NULL, "Jordi", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_ACTION | GBF_MAZE, 0,
	SpectrumGetZipName, SpecMsnampacRomInfo, SpecMsnampacRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// MultiDude (128K) (HB)

static struct BurnRomInfo SpecmultidudeRomDesc[] = {
	{ "MultiDude128k (2014)(RetroSouls).tap",  47634, 0x0892872f, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specmultidude, Specmultidude, Spec128)
STD_ROM_FN(Specmultidude)

struct BurnDriver BurnSpecmultidude = {
	"spec_multidude", NULL, "spec_spec128", NULL, "2014",
	"MultiDude (128K) (HB)\0", NULL, "RetroSouls", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PUZZLE, 0,
	SpectrumGetZipName, SpecmultidudeRomInfo, SpecmultidudeRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecQAOPSpaceDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Mutant Mushrooms (128K) (HB)

static struct BurnRomInfo SpecMmushroomsRomDesc[] = {
	{ "Mutant Mushrooms 128K (2024)(The Death Squad).tap", 47778, 0xeed300c9, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecMmushrooms, SpecMmushrooms, Spec128)
STD_ROM_FN(SpecMmushrooms)

struct BurnDriver BurnSpecMmushrooms = {
	"spec_mmushrooms", NULL, "spec_spec128", NULL, "2024",
	"Mutant Mushrooms (128K) (HB)\0", NULL, "The Death Squad", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_ACTION, 0,
	SpectrumGetZipName, SpecMmushroomsRomInfo, SpecMmushroomsRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecQAOPSpaceDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Mushrom Soup (128K) (HB)

static struct BurnRomInfo SpecMushromsoupRomDesc[] = {
	{ "Mushrom Soup 128K (2024)(Ottersoft).tap", 45638, 0x8c755a93, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecMushromsoup, SpecMushromsoup, Spec128)
STD_ROM_FN(SpecMushromsoup)

struct BurnDriver BurnSpecMushromsoup = {
	"spec_mushromsoup", NULL, "spec_spec128", NULL, "2024",
	"Mushrom Soup (128K) (HB)\0", NULL, "Ottersoft", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecMushromsoupRomInfo, SpecMushromsoupRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Mystery of Markham Manor, The (128K) (HB)

static struct BurnRomInfo SpecMmmanor128RomDesc[] = {
	{ "Mystery of Markham Manor, The 128K (2022)(Taskmaster Software).tap", 113688, 0x763a7416, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecMmmanor128, SpecMmmanor128, Spec128)
STD_ROM_FN(SpecMmmanor128)

struct BurnDriver BurnSpecMmmanor128 = {
	"spec_mmmanor128", NULL, "spec_spec128", NULL, "2022",
	"Mystery of Markham Manor, The (128K) (HB)\0", NULL, "Taskmaster Software", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_ADV, 0,
	SpectrumGetZipName, SpecMmmanor128RomInfo, SpecMmmanor128RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Mystery of Markham Manor, The (48K) (HB)

static struct BurnRomInfo SpecMmmanor48RomDesc[] = {
	{ "Mystery of Markham Manor, The 48K (2022)(Taskmaster Software).tap", 44391, 0xcbd1d7da, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecMmmanor48, SpecMmmanor48, Spectrum)
STD_ROM_FN(SpecMmmanor48)

struct BurnDriver BurnSpecMmmanor48 = {
	"spec_mmmanor48", "spec_mmmanor128", "spec_spectrum", NULL, "2022",
	"Mystery of Markham Manor, The (48K) (HB)\0", NULL, "Taskmaster Software", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_ADV, 0,
	SpectrumGetZipName, SpecMmmanor48RomInfo, SpecMmmanor48RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Nanako Descends to Hell (128K) (HB)

static struct BurnRomInfo SpecNanakohellRomDesc[] = {
	{ "Nanako Descends to Hell 128K (2009)(Ubhres Productions).tap", 55992, 0x8b88287c, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecNanakohell, SpecNanakohell, Spec128)
STD_ROM_FN(SpecNanakohell)

struct BurnDriver BurnSpecNanakohell = {
	"spec_nanakohell", NULL, "spec_spec128", NULL, "2009",
	"Nanako Descends to Hell (128K) (HB)\0", NULL, "The Mojon Twins - UBHRES Productions", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_ACTION, 0,
	SpectrumGetZipName, SpecNanakohellRomInfo, SpecNanakohellRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Neadeital (128K) (HB)

static struct BurnRomInfo SpecNeadeitalRomDesc[] = {
	{ "Neadeital 128K (2020)(Heavy Spectrum Limited).tap", 69024, 0xca053eda, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecNeadeital, SpecNeadeital, Spec128)
STD_ROM_FN(SpecNeadeital)

struct BurnDriver BurnSpecNeadeital = {
	"spec_neadeital", NULL, "spec_spec128", NULL, "2020",
	"Neadeital 128K (HB)\0", NULL, "Heavy Spectrum Limited", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_ADV, 0,
	SpectrumGetZipName, SpecNeadeitalRomInfo, SpecNeadeitalRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecQAOPSpaceDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Nelo & Quqo and the Last Butifarreisson (English) (128K) (HB)

static struct BurnRomInfo SpecNeloquqoenRomDesc[] = {
	{ "Nelo & Quqo and the Last Butifarreisson EN 128K (2010)(RetroWorks).tap", 48550, 0x0721a309, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecNeloquqoen, SpecNeloquqoen, Spec128)
STD_ROM_FN(SpecNeloquqoen)

struct BurnDriver BurnSpecNeloquqoen = {
	"spec_neloquqoen", NULL, "spec_spec128", NULL, "2010",
	"Nelo & Quqo and the Last Butifarreisson (English) (128K) (HB)\0", NULL, "RetroWorks", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_ADV, 0,
	SpectrumGetZipName, SpecNeloquqoenRomInfo, SpecNeloquqoenRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecQAOPSpaceDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Nelo & Quqo and the Last Butifarreisson (Spanish) (128K) (HB)

static struct BurnRomInfo SpecNeloquqoesRomDesc[] = {
	{ "Nelo & Quqo and the Last Butifarreisson ES 128K (2010)(RetroWorks).tap", 48550, 0xac593149, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecNeloquqoes, SpecNeloquqoes, Spec128)
STD_ROM_FN(SpecNeloquqoes)

struct BurnDriver BurnSpecNeloquqoes = {
	"spec_neloquqoes", "spec_neloquqoen", "spec_spec128", NULL, "2010",
	"Nelo & Quqo and the Last Butifarreisson (Spanish) (128K) (HB)\0", NULL, "RetroWorks", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_ADV, 0,
	SpectrumGetZipName, SpecNeloquqoesRomInfo, SpecNeloquqoesRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecQAOPSpaceDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Neuras (48K) (HB)

static struct BurnRomInfo SpecNeurasRomDesc[] = {
	{ "Neuras 48K-AY (2021)(Inufuto).tap", 10055, 0x71ed6d45, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecNeuras, SpecNeuras, Spectrum)
STD_ROM_FN(SpecNeuras)

struct BurnDriver BurnSpecNeuras = {
	"spec_neuras", NULL, "spec_spectrum", NULL, "2021",
	"Neuras (48K) (HB)\0", "AY Sound supported", "Inufuto", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_MAZE | GBF_ACTION, 0,
	SpectrumGetZipName, SpecNeurasRomInfo, SpecNeurasRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Niko and the Magic Orb (128K) (HB)

static struct BurnRomInfo SpecNikotmo128RomDesc[] = {
	{ "Niko and the Magic Orb 128K (2024)(BTCO).tap", 52571, 0x0080e61f, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecNikotmo128, SpecNikotmo128, Spec128)
STD_ROM_FN(SpecNikotmo128)

struct BurnDriver BurnSpecNikotmo128 = {
	"spec_nikotmo128", NULL, "spec_spec128", NULL, "2024",
	"Niko and the Magic Orb (128K) (HB)\0", NULL, "BTCO", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecNikotmo128RomInfo, SpecNikotmo128RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Niko and the Magic Orb (48K) (HB)

static struct BurnRomInfo SpecNikotmo48RomDesc[] = {
	{ "Niko and the Magic Orb 48K (2024)(BTCO).tap", 52316, 0xd922530b, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecNikotmo48, SpecNikotmo48, Spectrum)
STD_ROM_FN(SpecNikotmo48)

struct BurnDriver BurnSpecNikotmo48 = {
	"spec_nikotmo48", "spec_nikotmo128", "spec_spectrum", NULL, "2024",
	"Niko and the Magic Orb (48K) (HB)\0", NULL, "BTCO", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecNikotmo48RomInfo, SpecNikotmo48RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Ninja Carnage (English) (128K) (HB)

static struct BurnRomInfo SpecNinjacarnagenRomDesc[] = {
	{ "Ninja Carnage EN 128K (2021)(Resistance).tap", 237825, 0x2f6c9195, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecNinjacarnagen, SpecNinjacarnagen, Spec128)
STD_ROM_FN(SpecNinjacarnagen)

struct BurnDriver BurnSpecNinjacarnagen = {
	"spec_ninjacarnagen", NULL, "spec_spec128", NULL, "2021",
	"Ninja Carnage (English) (128K) (HB)\0", NULL, "Resistance", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_ADV, 0,
	SpectrumGetZipName, SpecNinjacarnagenRomInfo, SpecNinjacarnagenRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Ninja Carnage (Spanish) (128K) (HB)

static struct BurnRomInfo SpecNinjacarnagesRomDesc[] = {
	{ "Ninja Carnage ES 128K (2021)(Resistance).tap", 237993, 0x9e930e3f, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecNinjacarnages, SpecNinjacarnages, Spec128)
STD_ROM_FN(SpecNinjacarnages)

struct BurnDriver BurnSpecNinjacarnages = {
	"spec_ninjacarnages", "spec_ninjacarnagen", "spec_spec128", NULL, "2021",
	"Ninja Carnage (Spanish) (128K) (HB)\0", NULL, "Resistance", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_ADV, 0,
	SpectrumGetZipName, SpecNinjacarnagesRomInfo, SpecNinjacarnagesRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Ninja Carnage (French) (128K) (HB)

static struct BurnRomInfo SpecNinjacarnagfrRomDesc[] = {
	{ "Ninja Carnage FR 128K (2021)(Resistance).tap", 237835, 0xd5c829fb, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecNinjacarnagfr, SpecNinjacarnagfr, Spec128)
STD_ROM_FN(SpecNinjacarnagfr)

struct BurnDriver BurnSpecNinjacarnagfr = {
	"spec_ninjacarnagfr", "spec_ninjacarnagen", "spec_spec128", NULL, "2021",
	"Ninja Carnage (French) (128K) (HB)\0", NULL, "Resistance", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_ADV, 0,
	SpectrumGetZipName, SpecNinjacarnagfrRomInfo, SpecNinjacarnagfrRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Ninja Carnage (German) (128K) (HB)

static struct BurnRomInfo SpecNinjacarnaggeRomDesc[] = {
	{ "Ninja Carnage DE 128K (2021)(Resistance).tap", 238054, 0x6fea53d8, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecNinjacarnagge, SpecNinjacarnagge, Spec128)
STD_ROM_FN(SpecNinjacarnagge)

struct BurnDriver BurnSpecNinjacarnagge = {
	"spec_ninjacarnagge", "spec_ninjacarnagen", "spec_spec128", NULL, "2021",
	"Ninja Carnage (German) (128K) (HB)\0", NULL, "Resistance", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_ADV, 0,
	SpectrumGetZipName, SpecNinjacarnaggeRomInfo, SpecNinjacarnaggeRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Ninja Carnage (Italian) (128K) (HB)

static struct BurnRomInfo SpecNinjacarnagitRomDesc[] = {
	{ "Ninja Carnage IT 128K (2021)(Resistance).tap", 238053, 0xd06b1569, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecNinjacarnagit, SpecNinjacarnagit, Spec128)
STD_ROM_FN(SpecNinjacarnagit)

struct BurnDriver BurnSpecNinjacarnagit = {
	"spec_ninjacarnagit", "spec_ninjacarnagen", "spec_spec128", NULL, "2021",
	"Ninja Carnage (Italian) (128K) (HB)\0", NULL, "Resistance", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_ADV, 0,
	SpectrumGetZipName, SpecNinjacarnagitRomInfo, SpecNinjacarnagitRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Ninjajar (128K) (HB, v1.1)

static struct BurnRomInfo SpecninjajarRomDesc[] = {
	{ "Ninjajar v1.1 128K (2014)(The Mojon Twins).tap", 122675, 0x617db06c, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specninjajar, Specninjajar, Spec128)
STD_ROM_FN(Specninjajar)

struct BurnDriver BurnSpecninjajar = {
	"spec_ninjajar", NULL, "spec_spec128", NULL, "2014",
	"Ninjajar (128K) (HB, v1.1)\0", NULL, "The Mojon Twins", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecninjajarRomInfo, SpecninjajarRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Ninjakul in the AUIC Temple (128K) (HB)

static struct BurnRomInfo SpecNinjakulRomDesc[] = {
	{ "Ninjakul in the AUIC Temple 128K (2018)(Pat Morita Team).tap", 60380, 0x787a7dbe, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecNinjakul, SpecNinjakul, Spec128)
STD_ROM_FN(SpecNinjakul)

struct BurnDriver BurnSpecNinjakul = {
	"spec_ninjakul", NULL, "spec_spec128", NULL, "2018",
	"Ninjakul in the AUIC Temple (128K) (HB)\0", NULL, "Pat Morita Team", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecNinjakulRomInfo, SpecNinjakulRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Ninjakul 2 The Last Ninja (128K) (HB)

static struct BurnRomInfo SpecNinjakul2RomDesc[] = {
	{ "Ninjakul 2 The Last Ninja 128K (2019)(Pat Morita Team).tap", 102791, 0xbeccf8b7, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecNinjakul2, SpecNinjakul2, Spec128)
STD_ROM_FN(SpecNinjakul2)

struct BurnDriver BurnSpecNinjakul2 = {
	"spec_ninjakul2", NULL, "spec_spec128", NULL, "2019",
	"Ninjakul 2 The Last Ninja (128K) (HB)\0", "Redefine keys to use controller", "Pat Morita Team", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecNinjakul2RomInfo, SpecNinjakul2RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecCursorKeysDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Noon30 Shovel Duck (48K) (HB, v1.1)

static struct BurnRomInfo SpecNoon30sdRomDesc[] = {
	{ "Noon30 Shovel Duck v1.1 48K (2023)(KTB Productions).tap", 46210, 0xbefa63c4, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecNoon30sd, SpecNoon30sd, Spectrum)
STD_ROM_FN(SpecNoon30sd)

struct BurnDriver BurnSpecNoon30sd = {
	"spec_noon30sd", NULL, "spec_spectrum", NULL, "2023",
	"Noon30 Shovel Duck (48K) (HB, v1.1)\0", NULL, "KTB Productions", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecNoon30sdRomInfo, SpecNoon30sdRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Nosy (English) (128K) (HB)

static struct BurnRomInfo SpecNosyenRomDesc[] = {
	{ "Nosy EN 128K (2022)(Javier Fopiani).tap", 125924, 0x97a842a0, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecNosyen, SpecNosyen, Spec1282a)
STD_ROM_FN(SpecNosyen)

struct BurnDriver BurnSpecNosyen = {
	"spec_nosyen", NULL, "spec_spec1282a", NULL, "2022",
	"Nosy (English) (128K) (HB)\0", NULL, "Javier Fopiani", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_ACTION | GBF_ADV, 0,
	SpectrumGetZipName, SpecNosyenRomInfo, SpecNosyenRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KPlus2Init, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Nosy (Spanish) (128K) (HB)

static struct BurnRomInfo SpecNosyesRomDesc[] = {
	{ "Nosy ES 128K (2022)(Javier Fopiani).tap", 125815, 0xc9fe4df5, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecNosyes, SpecNosyes, Spec1282a)
STD_ROM_FN(SpecNosyes)

struct BurnDriver BurnSpecNosyes = {
	"spec_nosyes", "spec_nosyen", "spec_spec1282a", NULL, "2022",
	"Nosy (Spanish) (128K) (HB)\0", NULL, "Javier Fopiani", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_ACTION | GBF_ADV, 0,
	SpectrumGetZipName, SpecNosyesRomInfo, SpecNosyesRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KPlus2Init, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Nosy - first version (48K) (HB)

static struct BurnRomInfo SpecNosyRomDesc[] = {
	{ "Nosy 48K (2020)(Javier Fopiani).tap", 43998, 0xefb0331a, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecNosy, SpecNosy, Spectrum)
STD_ROM_FN(SpecNosy)

struct BurnDriver BurnSpecNosy = {
	"spec_nosy", "spec_nosyen", "spec_spectrum", NULL, "2020",
	"Nosy - first version (48K) (HB)\0", NULL, "Javier Fopiani", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_ACTION | GBF_ADV, 0,
	SpectrumGetZipName, SpecNosyRomInfo, SpecNosyRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Nothing (128K) (HB)

static struct BurnRomInfo SpecNothingRomDesc[] = {
	{ "Nothing v1.0 128K (2021)(Gogin).tap", 46605, 0x11d0b7e2, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecNothing, SpecNothing, Spec128)
STD_ROM_FN(SpecNothing)

struct BurnDriver BurnSpecNothing = {
	"spec_nothing", NULL, "spec_spec128", NULL, "2021",
	"Nothing (128K) (HB)\0", NULL, "Gogin", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecNothingRomInfo, SpecNothingRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Oceano (128K) (HB)

static struct BurnRomInfo SpecOceanoRomDesc[] = {
	{ "Oceano 128K (2015)(EgoTrip).tap", 24434, 0x1ef0dccd, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecOceano, SpecOceano, Spec128)
STD_ROM_FN(SpecOceano)

struct BurnDriver BurnSpecOceano = {
	"spec_oceano", NULL, "spec_spec128", NULL, "2015",
	"Oceano (128K) (HB)\0", NULL, "EgoTrip", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_MAZE, 0,
	SpectrumGetZipName, SpecOceanoRomInfo, SpecOceanoRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecQAOPSpaceDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Old Tower (128K) (HB)

static struct BurnRomInfo SpecoldtowerRomDesc[] = {
	{ "Old Tower 128K (2018)(RetroSouls).tap",  97764, 0x837df93f, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specoldtower, Specoldtower, Spec128)
STD_ROM_FN(Specoldtower)

struct BurnDriver BurnSpecoldtower = {
	"spec_oldtower", NULL, "spec_spec128", NULL, "2019",
	"Old Tower (128K) (HB)\0", NULL, "RetroSouls", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_ACTION | GBF_PUZZLE, 0,
	SpectrumGetZipName, SpecoldtowerRomInfo, SpecoldtowerRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Ooze (128K) (HB)

static struct BurnRomInfo SpecOozeRomDesc[] = {
	{ "Ooze 128K (2017)(Bubblesoft).tap", 49790, 0x188d3e4e, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecOoze, SpecOoze, Spec128)
STD_ROM_FN(SpecOoze)

struct BurnDriver BurnSpecOoze = {
	"spec_ooze", NULL, "spec_spec128", NULL, "2017",
	"Ooze (128K) (HB)\0", NULL, "Bubblesoft", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecOozeRomInfo, SpecOozeRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Operation: Labyrinth Fall (128K) (HB)

static struct BurnRomInfo SpecOplabfallRomDesc[] = {
	{ "Operation - Labyrinth Fall 128K (2018)(Bumfun Software).tap", 36787, 0x22169ded, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecOplabfall, SpecOplabfall, Spec128)
STD_ROM_FN(SpecOplabfall)

struct BurnDriver BurnSpecOplabfall = {
	"spec_oplabfall", NULL, "spec_spec128", NULL, "2018",
	"Operation: Labyrinth Fall (128K) (HB)\0", NULL, "Bumfun Software", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecOplabfallRomInfo, SpecOplabfallRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KPlus2Init, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// O-Puzz Attack (48K) (HB)

static struct BurnRomInfo SpecOpuzzRomDesc[] = {
	{ "O-Puzz Attack 48K (2018)(Oblo).tap", 29543, 0xebadd8c9, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecOpuzz, SpecOpuzz, Spectrum)
STD_ROM_FN(SpecOpuzz)

struct BurnDriver BurnSpecOpuzz = {
	"spec_opuzz", NULL, "spec_spectrum", NULL, "2018",
	"O-Puzz Attack (48K) (HB)\0", NULL, "Oblo", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PUZZLE, 0,
	SpectrumGetZipName, SpecOpuzzRomInfo, SpecOpuzzRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecIntf2DIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Order of Sleeping Dragon, The (128K) (HB)

static struct BurnRomInfo SpecOrderslpdragRomDesc[] = {
	{ "Order of Sleeping Dragon, The (2019)(Evgeniya Zapolnova).tap", 125004, 0x5652c985, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecOrderslpdrag, SpecOrderslpdrag, Spec128)
STD_ROM_FN(SpecOrderslpdrag)

struct BurnDriver BurnSpecOrderslpdrag = {
	"spec_orderslpdrag", NULL, "spec_spec128", NULL, "2019",
	"Order of Sleeping Dragon, The (128K) (HB)\0", "Redifine controls to use 'Sinclair 2' joystick", "Evgeniya Zapolnova", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_ACTION | GBF_RPG, 0,
	SpectrumGetZipName, SpecOrderslpdragRomInfo, SpecOrderslpdragRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecIntf2DIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Orvol Voon (English) (48K) (HB)

static struct BurnRomInfo SpecOrvolenRomDesc[] = {
	{ "Orvol Voon EN 48K (2023)(The Mojon Twins).tap", 35818, 0x57c016ed, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecOrvolen, SpecOrvolen, Spectrum)
STD_ROM_FN(SpecOrvolen)

struct BurnDriver BurnSpecOrvolen = {
	"spec_orvolen", NULL, "spec_spectrum", NULL, "2023",
	"Orvol Voon (English) (48K) (HB)\0", NULL, "The Mojon Twins", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecOrvolenRomInfo, SpecOrvolenRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Orvol Voon (Spanish) (48K) (HB)

static struct BurnRomInfo SpecOrvolesRomDesc[] = {
	{ "Orvol Voon ES 48K (2023)(The Mojon Twins).tap", 35808, 0xfe3d978f, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecOrvoles, SpecOrvoles, Spectrum)
STD_ROM_FN(SpecOrvoles)

struct BurnDriver BurnSpecOrvoles = {
	"spec_orvoles", "spec_orvolen", "spec_spectrum", NULL, "2023",
	"Orvol Voon (Spanish) (48K) (HB)\0", NULL, "The Mojon Twins", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecOrvolesRomInfo, SpecOrvolesRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Oure - Dawn Of Hope (48K) (HB)

static struct BurnRomInfo SpecOureRomDesc[] = {
	{ "Oure - Dawn Of Hope 48K (2018)(Heavy Spectrum Limited).tap", 48603, 0xe9047023, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecOure, SpecOure, Spectrum)
STD_ROM_FN(SpecOure)

struct BurnDriver BurnSpecOure = {
	"spec_oure", NULL, "spec_spectrum", NULL, "2018",
	"Oure - Dawn Of Hope (48K) (HB)\0", NULL, "Heavy Spectrum Limited", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_ACTION, 0,
	SpectrumGetZipName, SpecOureRomInfo, SpecOureRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Pac Hic III (128K) (HB)

static struct BurnRomInfo SpecPachic3RomDesc[] = {
	{ "Pac Hic III 128K (2023)(Daniel Isoba).tap", 45683, 0xe6573dbd, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecPachic3, SpecPachic3, Spec128)
STD_ROM_FN(SpecPachic3)

struct BurnDriver BurnSpecPachic3 = {
	"spec_pachic3", NULL, "spec_spec128", NULL, "2023",
	"Pac Hic III (128K) (HB)\0", NULL, "Daniel Isoba", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_MAZE | GBF_ACTION, 0,
	SpectrumGetZipName, SpecPachic3RomInfo, SpecPachic3RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Pac Hic Rehab (128K) (HB)

static struct BurnRomInfo SpecPhrehabRomDesc[] = {
	{ "Pac - Hic Rehab 128k (2022)(Daniel Isoba).tap", 30767, 0x170729dc, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecPhrehab, SpecPhrehab, Spec128)
STD_ROM_FN(SpecPhrehab)

struct BurnDriver BurnSpecPhrehab = {
	"spec_phrehab", NULL, "spec_spec128", NULL, "2022",
	"Pac Hic Rehab (128K) (HB)\0", NULL, "Daniel Isoba", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_MAZE | GBF_ACTION, 0,
	SpectrumGetZipName, SpecPhrehabRomInfo, SpecPhrehabRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Pac-Man Arcade (48K) (HB, v1.3)

static struct BurnRomInfo SpecPmarcadeRomDesc[] = {
	{ "Pac-Man Arcade v1.3 48K (2024)(Marco Leal).tap", 25139, 0x42f9204e, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecPmarcade, SpecPmarcade, Spectrum)
STD_ROM_FN(SpecPmarcade)

struct BurnDriver BurnSpecPmarcade = {
	"spec_pmarcade", NULL, "spec_spectrum", NULL, "2024",
	"Pac-Man Arcade (48K) (HB, v1.3)\0", NULL, "Marco Leal", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 2, HARDWARE_SPECTRUM, GBF_MAZE | GBF_ACTION, 0,
	SpectrumGetZipName, SpecPmarcadeRomInfo, SpecPmarcadeRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Pac-Man Begins (48K) (HB)

static struct BurnRomInfo SpecPacbeginsRomDesc[] = {
	{ "Pac-Man Begins 48K (2024)(Highriser).tap", 12981, 0x4ce5c738, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecPacbegins, SpecPacbegins, Spectrum)
STD_ROM_FN(SpecPacbegins)

struct BurnDriver BurnSpecPacbegins = {
	"spec_pacbegins", NULL, "spec_spectrum", NULL, "2024",
	"Pac-Man Begins (48K) (HB)\0", "One button game", "Highriser", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_ACTION, 0,
	SpectrumGetZipName, SpecPacbeginsRomInfo, SpecPacbeginsRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Pac-Man RX (48K) (HB)

static struct BurnRomInfo SpecPacmanrxRomDesc[] = {
	{ "Pac-Man RX 48K-AY (2024)(Midnight Brew).tap", 47454, 0x99972823, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecPacmanrx, SpecPacmanrx, Spectrum)
STD_ROM_FN(SpecPacmanrx)

struct BurnDriver BurnSpecPacmanrx = {
	"spec_pacmanrx", NULL, "spec_spectrum", NULL, "2024",
	"Pac-Man RX (48K) (HB)\0", "AY Sound supported", "Midnight Brew", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_ACTION | GBF_MAZE, 0,
	SpectrumGetZipName, SpecPacmanrxRomInfo, SpecPacmanrxRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// PACO-MAN (48K) (HB)

static struct BurnRomInfo SpecPacomanRomDesc[] = {
	{ "PACO-MAN 48K (2024)(Tiranus Soft).tap", 33970, 0x08da3ab4, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecPacoman, SpecPacoman, Spectrum)
STD_ROM_FN(SpecPacoman)

struct BurnDriver BurnSpecPacoman = {
	"spec_pacoman", "spec_pacomanv2", "spec_spectrum", NULL, "2024",
	"PACO-MAN (48K) (HB)\0", NULL, "Tiranus Soft", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecPacomanRomInfo, SpecPacomanRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// PACO-MAN (128K) (HB)

static struct BurnRomInfo SpecPacomanv2RomDesc[] = {
	{ "PACO-MAN 128K (2024)(Tiranus Soft).tap", 41864, 0x89aa3909, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecPacomanv2, SpecPacomanv2, Spec128)
STD_ROM_FN(SpecPacomanv2)

struct BurnDriver BurnSpecPacomanv2 = {
	"spec_pacomanv2", NULL, "spec_spec128", NULL, "2024",
	"PACO-MAN (128K) (HB)\0", NULL, "Tiranus Soft", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecPacomanv2RomInfo, SpecPacomanv2RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// PaintBall-2048 - 1 Player Version (128K) (HB)

static struct BurnRomInfo SpecPb2048p1RomDesc[] = {
	{ "PaintBall-2048 - 1 Player (2019)(PC NONO Games).tap", 45736, 0xf256bac3, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecPb2048p1, SpecPb2048p1, Spec128)
STD_ROM_FN(SpecPb2048p1)

struct BurnDriver BurnSpecPb2048p1 = {
	"spec_pb2048p1", NULL, "spec_spec128", NULL, "2019",
	"PaintBall-2048 - 1 Player Version (128K) (HB)\0", NULL, "PC NONO Games", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_ACTION, 0,
	SpectrumGetZipName, SpecPb2048p1RomInfo, SpecPb2048p1RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// PaintBall-2048 - 2 Players Version (128K) (HB)

static struct BurnRomInfo SpecPb2048p2RomDesc[] = {
	{ "PaintBall-2048 - 2 Players (2019)(PC NONO Games).tap", 46282, 0x4478a7d8, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecPb2048p2, SpecPb2048p2, Spec128)
STD_ROM_FN(SpecPb2048p2)

struct BurnDriver BurnSpecPb2048p2 = {
	"spec_pb2048p2", "spec_pb2048p1", "spec_spec128", NULL, "2019",
	"PaintBall-2048 - 2 Players Version (128K) (HB)\0", NULL, "PC NONO Games", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE | BDF_HOMEBREW, 2, HARDWARE_SPECTRUM, GBF_ACTION, 0,
	SpectrumGetZipName, SpecPb2048p2RomInfo, SpecPb2048p2RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Paleto Jones and Bartolo's Flute (English) (48K) (HB)

static struct BurnRomInfo SpecPaletoj2enRomDesc[] = {
	{ "Paleto Jones and Bartolo's Flute EN 48K (2022)(Mananuk).tap", 41476, 0xad013c6c, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecPaletoj2en, SpecPaletoj2en, Spectrum)
STD_ROM_FN(SpecPaletoj2en)

struct BurnDriver BurnSpecPaletoj2en = {
	"spec_paletoj2en", NULL, "spec_spectrum", NULL, "2022",
	"Paleto Jones and Bartolo's Flute (English) (48K) (HB)\0", NULL, "Mananuk", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecPaletoj2enRomInfo, SpecPaletoj2enRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Paleto Jones y la flauta de Bartolo (Spanish) (48K) (HB)

static struct BurnRomInfo SpecPaletoj2spRomDesc[] = {
	{ "Paleto Jones y la flauta de Bartolo SP 48K (2022)(Mananuk).tap", 41498, 0xb2be31f3, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecPaletoj2sp, SpecPaletoj2sp, Spectrum)
STD_ROM_FN(SpecPaletoj2sp)

struct BurnDriver BurnSpecPaletoj2sp = {
	"spec_paletoj2sp", "spec_paletoj2en", "spec_spectrum", NULL, "2022",
	"Paleto Jones y la flauta de Bartolo (Spanish) (48K) (HB)\0", NULL, "Mananuk", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecPaletoj2spRomInfo, SpecPaletoj2spRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Papyrus (48K) (HB)

static struct BurnRomInfo SpecPapyrusRomDesc[] = {
	{ "Papyrus 48K (2020)(F.J. Urbaneja).tap", 29120, 0xf85af5a2, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecPapyrus, SpecPapyrus, Spectrum)
STD_ROM_FN(SpecPapyrus)

struct BurnDriver BurnSpecPapyrus = {
	"spec_papyrus", NULL, "spec_spectrum", NULL, "2020",
	"Papyrus (48K) (HB)\0", NULL, "F.J. Urbaneja", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecPapyrusRomInfo, SpecPapyrusRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecQAOPSpaceDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// ParaZXland (48K) (HB)

static struct BurnRomInfo SpecParazxlandRomDesc[] = {
	{ "ParaZXland 48K (2023)(Massimiliano Arca).tap", 41886, 0xaeac50ff, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecParazxland, SpecParazxland, Spectrum)
STD_ROM_FN(SpecParazxland)

struct BurnDriver BurnSpecParazxland = {
	"spec_parazxland", NULL, "spec_spectrum", NULL, "2023",
	"ParaZXland (48K) (HB)\0", NULL, "Massimiliano Arca", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_ACTION, 0,
	SpectrumGetZipName, SpecParazxlandRomInfo, SpecParazxlandRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Partyscroller (128K) (HB, Tech-Demo)

static struct BurnRomInfo SpecPartyscrollRomDesc[] = {
	{ "Partyscroller 128K (2023)(Darklite-Offence).tap", 72537, 0xfc40ad69, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecPartyscroll, SpecPartyscroll, Spec128)
STD_ROM_FN(SpecPartyscroll)

struct BurnDriver BurnSpecPartyscroll = {
	"spec_partyscroll", NULL, "spec_spec128", NULL, "2023",
	"Partyscroller (128K) (HB, Tech-Demo)\0", NULL, "Darklite/Offence", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW | BDF_DEMO, 1, HARDWARE_SPECTRUM, GBF_MISC, 0,
	SpectrumGetZipName, SpecPartyscrollRomInfo, SpecPartyscrollRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Pegged (48K) (HB, v1.02)

static struct BurnRomInfo SpecPeggedRomDesc[] = {
	{ "Pegged v1.02 48K (2023)(Under4Mhz).tap", 21268, 0x69591ee9, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecPegged, SpecPegged, Spectrum)
STD_ROM_FN(SpecPegged)

struct BurnDriver BurnSpecPegged = {
	"spec_pegged", NULL, "spec_spectrum", NULL, "2023",
	"Pegged (48K) (HB, v1.02)\0", NULL, "Under4Mhz", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PUZZLE, 0,
	SpectrumGetZipName, SpecPeggedRomInfo, SpecPeggedRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecQAOPSpaceDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Pengo mini (128K) (HB)

static struct BurnRomInfo SpecPengominiRomDesc[] = {
	{ "Pengo mini 128K (2023)(Gabriele Amore).tap", 56867, 0xba471a9c, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecPengomini, SpecPengomini, Spec128)
STD_ROM_FN(SpecPengomini)

struct BurnDriver BurnSpecPengomini = {
	"spec_pengomini", NULL, "spec_spec128", NULL, "2023",
	"Pengo mini (128K) (HB)\0", NULL, "Gabriele Amore", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_ACTION, 0,
	SpectrumGetZipName, SpecPengominiRomInfo, SpecPengominiRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Penguin Attack (128K) (HB, v1.1)

static struct BurnRomInfo SpecPengatk128RomDesc[] = {
	{ "Penguin Attack v1.1 128K (2024)(Pat Morita Team).tap", 56487, 0xb4b5626a, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecPengatk128, SpecPengatk128, Spec128)
STD_ROM_FN(SpecPengatk128)

struct BurnDriver BurnSpecPengatk128 = {
	"spec_pengatk128", NULL, "spec_spec128", NULL, "2024",
	"Penguin Attack (128K) (HB, v1.1)\0", NULL, "Pat Morita Team", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_ACTION, 0,
	SpectrumGetZipName, SpecPengatk128RomInfo, SpecPengatk128RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Penguin Attack (48K) (HB, v1.0)

static struct BurnRomInfo SpecPengatk48RomDesc[] = {
	{ "Penguin Attack v1.0 48K (2024)(Pat Morita Team).tap", 44423, 0x455ddf3d, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecPengatk48, SpecPengatk48, Spectrum)
STD_ROM_FN(SpecPengatk48)

struct BurnDriver BurnSpecPengatk48 = {
	"spec_pengatk48", "spec_pengatk128", "spec_spectrum", NULL, "2024",
	"Penguin Attack (48K) (HB, v1.0)\0", NULL, "Pat Morita Team", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_ACTION, 0,
	SpectrumGetZipName, SpecPengatk48RomInfo, SpecPengatk48RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Pentacorn Quest (English) (128K) (HB)

static struct BurnRomInfo SpecPentacornenRomDesc[] = {
	{ "Pentacorn Quest EN 128K (2015)(NightWolf).tap", 82077, 0xeff1d9ee, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecPentacornen, SpecPentacornen, Spec128)
STD_ROM_FN(SpecPentacornen)

struct BurnDriver BurnSpecPentacornen = {
	"spec_pentacornen", NULL, "spec_spec128", NULL, "2015",
	"Pentacorn Quest (English) (128K) (HB)\0", NULL, "NightWolf", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecPentacornenRomInfo, SpecPentacornenRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Pentacorn Quest (Spanish) (128K) (HB)

static struct BurnRomInfo SpecPentacornesRomDesc[] = {
	{ "Pentacorn Quest ES 128K (2015)(NightWolf).tap", 82077, 0x8fe2a141, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecPentacornes, SpecPentacornes, Spec128)
STD_ROM_FN(SpecPentacornes)

struct BurnDriver BurnSpecPentacornes = {
	"spec_pentacornes", "spec_pentacornen", "spec_spec128", NULL, "2015",
	"Pentacorn Quest (Spanish) (128K) (HB)\0", NULL, "NightWolf", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecPentacornesRomInfo, SpecPentacornesRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Peplum (48K) (HB)

static struct BurnRomInfo SpecPeplumRomDesc[] = {
	{ "Peplum 48K (2019)(Sequentia Soft).tap", 35094, 0x2e96abff, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecPeplum, SpecPeplum, Spectrum)
STD_ROM_FN(SpecPeplum)

struct BurnDriver BurnSpecPeplum = {
	"spec_peplum", NULL, "spec_spectrum", NULL, "2019",
	"Peplum (48K) (HB)\0", NULL, "Sequentia Soft", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecPeplumRomInfo, SpecPeplumRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecQAOPMDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Percy Penguin (128K) (HB)

static struct BurnRomInfo SpecPercypenguinRomDesc[] = {
	{ "Percy Penguin (2019)(John Davies).tap", 47843, 0xd944ba7a, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecPercypenguin, SpecPercypenguin, Spec128)
STD_ROM_FN(SpecPercypenguin)

struct BurnDriver BurnSpecPercypenguin = {
	"spec_percypenguin", NULL, "spec_spec128", NULL, "2019",
	"Percy Penguin (128K) (HB)\0", NULL, "John Davies", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecPercypenguinRomInfo, SpecPercypenguinRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Pets vs Aliens Prologue (128K) (HB)

static struct BurnRomInfo SpecpvsaRomDesc[] = {
	{ "Pets vs Aliens Prologue 128K (2015)(Einar Saukas).tap", 42877, 0x3bb9a912, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specpvsa, Specpvsa, Spec128)
STD_ROM_FN(Specpvsa)

struct BurnDriver BurnSpecpvsa = {
	"spec_pvsa", NULL, "spec_spec128", NULL, "2015",
	"Pets vs Aliens Prologue (128K) (HB)\0", NULL, "Einar Saukas", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 2, HARDWARE_SPECTRUM, GBF_PUZZLE, 0,
	SpectrumGetZipName, SpecpvsaRomInfo, SpecpvsaRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Phantomas en el museo (Spanish) (48K)

static struct BurnRomInfo SpecPhantmuseoRomDesc[] = {
	{ "Phantomas en el museo ES (2012)(The Mojon Twins).tap", 39978, 0x9b9f7ea2, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecPhantmuseo, SpecPhantmuseo, Spectrum)
STD_ROM_FN(SpecPhantmuseo)

struct BurnDriver BurnSpecPhantmuseo = {
	"spec_phantmuseo", NULL, "spec_spectrum", NULL, "2012",
	"Phantomas en el museo (Spanish) (48K)\0", NULL, "The Mojon Twins", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecPhantmuseoRomInfo, SpecPhantmuseoRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecIntf2DIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Phantomas Tales - Marsport (128K) (HB)

static struct BurnRomInfo SpecPhantmarsport128RomDesc[] = {
	{ "Phantomas Tales - Marsport 128K (2009)(The Mojon Twins).tap", 54275, 0x3cc5e0d2, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecPhantmarsport128, SpecPhantmarsport128, Spec128)
STD_ROM_FN(SpecPhantmarsport128)

struct BurnDriver BurnSpecPhantmarsport128 = {
	"spec_phantmarsport128", NULL, "spec_spec128", NULL, "2009",
	"Phantomas Tales - Marsport (128K) (HB)\0", NULL, "The Mojon Twins", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecPhantmarsport128RomInfo, SpecPhantmarsport128RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Phantomas Tales - Marsport (48K) (HB)

static struct BurnRomInfo SpecPhantmarsport48RomDesc[] = {
	{ "Phantomas Tales - Marsport 48K (2009)(The Mojon Twins).tap", 40932, 0xfd1ba206, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecPhantmarsport48, SpecPhantmarsport48, Spectrum)
STD_ROM_FN(SpecPhantmarsport48)

struct BurnDriver BurnSpecPhantmarsport48 = {
	"spec_phantmarsport48", "spec_phantmarsport128", "spec_spectrum", NULL, "2009",
	"Phantomas Tales - Marsport (48K) (HB)\0", NULL, "The Mojon Twins", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecPhantmarsport48RomInfo, SpecPhantmarsport48RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Phantomas Tales - Severin Sewers (128K) (HB)

static struct BurnRomInfo SpecPhantsevsewRomDesc[] = {
	{ "Phantomas Tales - Severin Sewers 128K (2011-2024)(The Mojon Twins).tap", 31971, 0xbe4e175d, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecPhantsevsew, SpecPhantsevsew, Spec128)
STD_ROM_FN(SpecPhantsevsew)

struct BurnDriver BurnSpecPhantsevsew = {
	"spec_phantsevsew", NULL, "spec_spec128", NULL, "2011-2024",
	"Phantomas Tales - Severin Sewers (128K) (HB)\0", NULL, "The Mojon Twins - UBHRES Productions", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecPhantsevsewRomInfo, SpecPhantsevsewRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Pi-Dentity (128K) (HB)

static struct BurnRomInfo SpecPidentity128RomDesc[] = {
	{ "Pi-Dentity 128K (2020)(Sebastian Braunert).tap", 70659, 0xb6aed3dd, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecPidentity128, SpecPidentity128, Spec128)
STD_ROM_FN(SpecPidentity128)

struct BurnDriver BurnSpecPidentity128 = {
	"spec_pidentity128", NULL, "spec_spec128", NULL, "2020",
	"Pi-Dentity (128K) (HB)\0", NULL, "Sebastian Braunert", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_VERSHOOT, 0,
	SpectrumGetZipName, SpecPidentity128RomInfo, SpecPidentity128RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecIntf2DIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Pi-Dentity (48K) (HB)

static struct BurnRomInfo SpecPidentity48RomDesc[] = {
	{ "Pi-Dentity 48K (2020)(Sebastian Braunert).tap", 43464, 0xecdc9c1c, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecPidentity48, SpecPidentity48, Spectrum)
STD_ROM_FN(SpecPidentity48)

struct BurnDriver BurnSpecPidentity48 = {
	"spec_pidentity48", "spec_pidentity128", "spec_spectrum", NULL, "2020",
	"Pi-Dentity (48K) (HB)\0", NULL, "Sebastian Braunert", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_VERSHOOT, 0,
	SpectrumGetZipName, SpecPidentity48RomInfo, SpecPidentity48RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecIntf2DIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Pilot Attack (128K) (HB, v1.1)

static struct BurnRomInfo SpecpilotatkRomDesc[] = {
	{ "Pilot Attack v1.1 (2017)(TFW8b).tap", 30930, 0x8e0f7278, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specpilotatk, Specpilotatk, Spec128)
STD_ROM_FN(Specpilotatk)

struct BurnDriver BurnSpecpilotatk = {
	"spec_pilotatk", NULL, "spec_spec128", NULL, "2017",
	"Pilot Attack (128K) (HB, v1.1)\0", NULL, "The Future Was 8bit", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_HORSHOOT, 0,
	SpectrumGetZipName, SpecpilotatkRomInfo, SpecpilotatkRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Pink Pills - no timer last (128K) (HB)

static struct BurnRomInfo SpecPinkpillsaRomDesc[] = {
	{ "Pink Pills 128 K - no timer last (2021)(Sebastian Braunert).tap", 56284, 0x330b371c, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecPinkpillsa, SpecPinkpillsa, Spec128)
STD_ROM_FN(SpecPinkpillsa)

struct BurnDriver BurnSpecPinkpillsa = {
	"spec_pinkpillsa", NULL, "spec_spec128", NULL, "2021",
	"Pink Pills - no timer last (128K) (HB)\0", NULL, "Sebastian Braunert", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecPinkpillsaRomInfo, SpecPinkpillsaRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Pink Pills - timer last (128K) (HB)

static struct BurnRomInfo SpecPinkpillsbRomDesc[] = {
	{ "Pink Pills 128 K - timer last (2021)(Sebastian Braunert).tap", 55882, 0xa88fe5f4, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecPinkpillsb, SpecPinkpillsb, Spec128)
STD_ROM_FN(SpecPinkpillsb)

struct BurnDriver BurnSpecPinkpillsb = {
	"spec_pinkpillsb", "spec_pinkpillsa", "spec_spec128", NULL, "2021",
	"Pink Pills - timer last (128K) (HB)\0", NULL, "Sebastian Braunert", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecPinkpillsbRomInfo, SpecPinkpillsbRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Pirate Captain Creed (English) (48K) (HB)

static struct BurnRomInfo SpecPcaptcreedenRomDesc[] = {
	{ "Pirate Captain Creed EN 48K (2023)(Furillo Productions).tap", 44764, 0x58a61ae6, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecPcaptcreeden, SpecPcaptcreeden, Spectrum)
STD_ROM_FN(SpecPcaptcreeden)

struct BurnDriver BurnSpecPcaptcreeden = {
	"spec_pcaptcreeden", NULL, "spec_spectrum", NULL, "2023",
	"Pirate Captain Creed (English) (48K) (HB)\0", NULL, "Furillo Productions", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecPcaptcreedenRomInfo, SpecPcaptcreedenRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Pirate Captain Creed (Spanish) (48K) (HB)

static struct BurnRomInfo SpecPcaptcreedesRomDesc[] = {
	{ "Pirate Captain Creed ES 48K (2023)(Furillo Productions).tap", 44740, 0x00f568ca, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecPcaptcreedes, SpecPcaptcreedes, Spectrum)
STD_ROM_FN(SpecPcaptcreedes)

struct BurnDriver BurnSpecPcaptcreedes = {
	"spec_pcaptcreedes", "spec_pcaptcreeden", "spec_spectrum", NULL, "2023",
	"Pirate Captain Creed (Spanish) (48K) (HB)\0", NULL, "Furillo Productions", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecPcaptcreedesRomInfo, SpecPcaptcreedesRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Pirate Captain Creed (Japanese) (48K) (HB)

static struct BurnRomInfo SpecPcaptcreedjpRomDesc[] = {
	{ "Pirate Captain Creed JP 48K (2023)(Furillo Productions).tap", 44553, 0x736f88d8, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecPcaptcreedjp, SpecPcaptcreedjp, Spectrum)
STD_ROM_FN(SpecPcaptcreedjp)

struct BurnDriver BurnSpecPcaptcreedjp = {
	"spec_pcaptcreedjp", "spec_pcaptcreeden", "spec_spectrum", NULL, "2023",
	"Pirate Captain Creed (Japanese) (48K) (HB)\0", NULL, "Furillo Productions", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecPcaptcreedjpRomInfo, SpecPcaptcreedjpRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Plantonio (English) (48K) (HB)

static struct BurnRomInfo SpecPlantonenRomDesc[] = {
	{ "Plantonio EN 48K (2023)(Kiuso).tap", 30373, 0x45a17b10, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecPlantonen, SpecPlantonen, Spectrum)
STD_ROM_FN(SpecPlantonen)

struct BurnDriver BurnSpecPlantonen = {
	"spec_plantonen", NULL, "spec_spectrum", NULL, "2023",
	"Plantonio (English) (48K) (HB)\0", NULL, "Kiuso", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecPlantonenRomInfo, SpecPlantonenRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Plantonio (Spanish) (48K) (HB)

static struct BurnRomInfo SpecPlantonesRomDesc[] = {
	{ "Plantonio ES 48K (2023)(Kiuso).tap", 30422, 0xd7e7c178, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecPlantones, SpecPlantones, Spectrum)
STD_ROM_FN(SpecPlantones)

struct BurnDriver BurnSpecPlantones = {
	"spec_plantones", "spec_plantonen", "spec_spectrum", NULL, "2023",
	"Plantonio (Spanish) (48K) (HB)\0", NULL, "Kiuso", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecPlantonesRomInfo, SpecPlantonesRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Plantonio (Portuguese) (48K) (HB)

static struct BurnRomInfo SpecPlantonptRomDesc[] = {
	{ "Plantonio PT 48K (2023)(Kiuso).tap", 30398, 0xc92e7e34, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecPlantonpt, SpecPlantonpt, Spectrum)
STD_ROM_FN(SpecPlantonpt)

struct BurnDriver BurnSpecPlantonpt = {
	"spec_plantonpt", "spec_plantonen", "spec_spectrum", NULL, "2023",
	"Plantonio (Portuguese) (48K) (HB)\0", NULL, "Kiuso", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecPlantonptRomInfo, SpecPlantonptRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Power of Yaddith - King of Yaddith (48K) (HB)

static struct BurnRomInfo SpecPowyadkingRomDesc[] = {
	{ "Power of Yaddith - King 48K-AY (2024)(Zagalicos Games).tap", 34572, 0xe8814e07, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecPowyadking, SpecPowyadking, Spectrum)
STD_ROM_FN(SpecPowyadking)

struct BurnDriver BurnSpecPowyadking = {
	"spec_powyadking", NULL, "spec_spectrum", NULL, "2024",
	"Power of Yaddith - King of Yaddith (48K) (HB)\0", "AY Sound supported", "Zagalicos Games", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_RUNGUN, 0,
	SpectrumGetZipName, SpecPowyadkingRomInfo, SpecPowyadkingRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Power of Yaddith - Queen of Yaddith (48K) (HB)

static struct BurnRomInfo SpecPowyadqueenRomDesc[] = {
	{ "Power of Yaddith - Queen 48K-AY (2024)(Zagalicos Games).tap", 41057, 0x423863be, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecPowyadqueen, SpecPowyadqueen, Spectrum)
STD_ROM_FN(SpecPowyadqueen)

struct BurnDriver BurnSpecPowyadqueen = {
	"spec_powyadqueen", "spec_powyadking", "spec_spectrum", NULL, "2024",
	"Power of Yaddith - Queen of Yaddith (48K) (HB)\0", "AY Sound supported", "Zagalicos Games", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecPowyadqueenRomInfo, SpecPowyadqueenRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Prima Dere ZX (48K) (HB)

static struct BurnRomInfo SpecPrimaderezxRomDesc[] = {
	{ "Prima Dere ZX 48K (2022)(AppSir Games).tap", 40838, 0x4909cec0, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecPrimaderezx, SpecPrimaderezx, Spectrum)
STD_ROM_FN(SpecPrimaderezx)

struct BurnDriver BurnSpecPrimaderezx = {
	"spec_primaderezx", NULL, "spec_spectrum", NULL, "2022",
	"Prima Dere ZX (48K) (HB)\0", NULL, "AppSir Games", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecPrimaderezxRomInfo, SpecPrimaderezxRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Psyqo Santa (128K) (HB)

static struct BurnRomInfo SpecPsyqosantaRomDesc[] = {
	{ "Psyqo Santa 128K (2023)(IAMAI Creation).tap", 56501, 0x2dc3a9d9, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecPsyqosanta, SpecPsyqosanta, Spec128)
STD_ROM_FN(SpecPsyqosanta)

struct BurnDriver BurnSpecPsyqosanta = {
	"spec_psyqosanta", NULL, "spec_spec128", NULL, "2023",
	"Psyqo Santa (128K) (HB)\0", NULL, "IAMAI Creation", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_ACTION, 0,
	SpectrumGetZipName, SpecPsyqosantaRomInfo, SpecPsyqosantaRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// PTM (128K) (HB)

static struct BurnRomInfo SpecPtmRomDesc[] = {
	{ "PTM 128K (2021)(UltraNarwhal).tap", 43484, 0x5f02d900, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecPtm, SpecPtm, Spec128)
STD_ROM_FN(SpecPtm)

struct BurnDriver BurnSpecPtm = {
	"spec_ptm", NULL, "spec_spec128", NULL, "2021",
	"PTM (128K) (HB)\0", NULL, "UltraNarwhal", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_SHOOT | GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecPtmRomInfo, SpecPtmRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Pulzar (48K) (HB)

static struct BurnRomInfo SpecPulzarRomDesc[] = {
	{ "Pulzar 48K (2024)(Snauts).tap", 24593, 0x9ef195f4, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecPulzar, SpecPulzar, Spectrum)
STD_ROM_FN(SpecPulzar)

struct BurnDriver BurnSpecPulzar = {
	"spec_pulzar", NULL, "spec_spectrum", NULL, "2024",
	"Pulzar (48K) (HB)\0", NULL, "Snauts", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_ACTION, 0,
	SpectrumGetZipName, SpecPulzarRomInfo, SpecPulzarRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Pumpkin Poe (128K) (HB)

static struct BurnRomInfo SpecPumpkinpoeRomDesc[] = {
	{ "Pumpkin Poe (2020)(Ariel Endaraues).tap", 39685, 0x64d9129d, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecPumpkinpoe, SpecPumpkinpoe, Spec128)
STD_ROM_FN(SpecPumpkinpoe)

struct BurnDriver BurnSpecPumpkinpoe = {
	"spec_pumpkinpoe", NULL, "spec_spec128", NULL, "2020",
	"Pumpkin Poe (128K) (HB)\0", NULL, "Ariel Endaraues", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecPumpkinpoeRomInfo, SpecPumpkinpoeRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Qbox (128K) (HB)

static struct BurnRomInfo SpecQbox128RomDesc[] = {
	{ "Qbox 128K (2017)(Sergio Llata Pena).tap", 53804, 0xf8e131b2, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecQbox128, SpecQbox128, Spec128)
STD_ROM_FN(SpecQbox128)

struct BurnDriver BurnSpecQbox128 = {
	"spec_qbox128", NULL, "spec_spec128", NULL, "2017",
	"Qbox (128K) (HB)\0", NULL, "Sergio Llata Pena", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_MAZE, 0,
	SpectrumGetZipName, SpecQbox128RomInfo, SpecQbox128RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Qbox (48K) (HB)

static struct BurnRomInfo SpecQbox48RomDesc[] = {
	{ "Qbox 48K (2017)(Sergio Llata Pena).tap", 47711, 0x386bb09d, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecQbox48, SpecQbox48, Spectrum)
STD_ROM_FN(SpecQbox48)

struct BurnDriver BurnSpecQbox48 = {
	"spec_qbox48", "spec_qbox128", "spec_spectrum", NULL, "2017",
	"Qbox (48K) (HB)\0", NULL, "Sergio Llata Pena", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_MAZE, 0,
	SpectrumGetZipName, SpecQbox48RomInfo, SpecQbox48RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// QI (Spanish) (48K) (HB)

static struct BurnRomInfo SpecQi48kRomDesc[] = {
	{ "QI ES 48K (2022)(Memsoftware).tap", 42543, 0xa3144c9f, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecQi48k, SpecQi48k, Spectrum)
STD_ROM_FN(SpecQi48k)

struct BurnDriver BurnSpecQi48k = {
	"spec_qi48k", NULL, "spec_spectrum", NULL, "2022",
	"QI (Spanish) (48K) (HB)\0", "Press 'L' to place a 'directional arrow'", "Memsoftware", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PUZZLE, 0,
	SpectrumGetZipName, SpecQi48kRomInfo, SpecQi48kRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecQAOPSpaceDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Quadron (48K) (HB, v1.23)

static struct BurnRomInfo SpecQuadronRomDesc[] = {
	{ "Quadron v1.23 48K (2019)(Andrew Beale).tap", 48026, 0x1f761019, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecQuadron, SpecQuadron, Spectrum)
STD_ROM_FN(SpecQuadron)

struct BurnDriver BurnSpecQuadron = {
	"spec_quadron", NULL, "spec_spectrum", NULL, "2019",
	"Quadron (48K) (HB, v1.23)\0", NULL, "Andrew Beale", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_ACTION, 0,
	SpectrumGetZipName, SpecQuadronRomInfo, SpecQuadronRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Quest for Elements (48K) (HB)

static struct BurnRomInfo SpecQuestelementsRomDesc[] = {
	{ "Quest for Elements (2019)(Gooseberry code).tap", 26559, 0x7ee0a6d4, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecQuestelements, SpecQuestelements, Spec128)
STD_ROM_FN(SpecQuestelements)

struct BurnDriver BurnSpecQuestelements = {
	"spec_questelements", NULL, "spec_spec128", NULL, "2019",
	"Quest for Elements (48K) (HB)\0", NULL, "Gooseberry code", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PLATFORM | GBF_ADV, 0,
	SpectrumGetZipName, SpecQuestelementsRomInfo, SpecQuestelementsRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecQAOPSpaceDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Rabid Mower (128K) (HB)

static struct BurnRomInfo SpecRabidmowerRomDesc[] = {
	{ "Rabid Mower 128K (2015)(kas29).tap", 23329, 0xbfd089f6, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecRabidmower, SpecRabidmower, Spec128)
STD_ROM_FN(SpecRabidmower)

struct BurnDriver BurnSpecRabidmower = {
	"spec_rabidmower", NULL, "spec_spec128", NULL, "2015",
	"Rabid Mower (128K) (HB)\0", NULL, "kas29", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 2, HARDWARE_SPECTRUM, GBF_ACTION, 0,
	SpectrumGetZipName, SpecRabidmowerRomInfo, SpecRabidmowerRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Raiders of the Lost Ark (48K) (HB)

static struct BurnRomInfo SpecRaidlostarkRomDesc[] = {
	{ "Raiders of the Lost Ark 48K (2024)(Luca Bordoni).tap", 42158, 0x0e391d68, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecRaidlostark, SpecRaidlostark, Spectrum)
STD_ROM_FN(SpecRaidlostark)

struct BurnDriver BurnSpecRaidlostark = {
	"spec_raidlostark", NULL, "spec_spectrum", NULL, "2024",
	"Raiders of the Lost Ark (48K) (HB)\0", "The original Atari 2600 game conversion/adaptation", "Luca Bordoni", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_ACTION, 0,
	SpectrumGetZipName, SpecRaidlostarkRomInfo, SpecRaidlostarkRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Rallybug (48K) (HB)

static struct BurnRomInfo SpecRallybugRomDesc[] = {
	{ "Rallybug 48K (2008)(Jonathan Cauldwell).tap", 48587, 0x07913054, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecRallybug, SpecRallybug, Spectrum)
STD_ROM_FN(SpecRallybug)

struct BurnDriver BurnSpecRallybug = {
	"spec_rallybug", NULL, "spec_spectrum", NULL, "2008",
	"Rallybug (48K) (HB)\0", NULL, "Jonathan Cauldwell", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecRallybugRomInfo, SpecRallybugRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Ramiro el Vampiro III (English) (128K) (HB)

static struct BurnRomInfo SpecRamiro3enRomDesc[] = {
	{ "Ramiro El Vampiro III EN 128K (2022)(The Mojon Twins).tap", 42853, 0x55fc67f2, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecRamiro3en, SpecRamiro3en, Spec128)
STD_ROM_FN(SpecRamiro3en)

struct BurnDriver BurnSpecRamiro3en = {
	"spec_ramiro3en", NULL, "spec_spec128", NULL, "2022",
	"Ramiro el Vampiro III (English) (128K) (HB)\0", NULL, "The Mojon Twins", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecRamiro3enRomInfo, SpecRamiro3enRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Ramiro el Vampiro III (Spanish) (128K) (HB)

static struct BurnRomInfo SpecRamiro3esRomDesc[] = {
	{ "Ramiro El Vampiro III ES 128K (2022)(The Mojon Twins).tap", 42874, 0xba4b626e, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecRamiro3es, SpecRamiro3es, Spec128)
STD_ROM_FN(SpecRamiro3es)

struct BurnDriver BurnSpecRamiro3es = {
	"spec_ramiro3es", "spec_ramiro3en", "spec_spec128", NULL, "2022",
	"Ramiro el Vampiro III (Spanish) (128K) (HB)\0", NULL, "The Mojon Twins", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecRamiro3esRomInfo, SpecRamiro3esRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Ramiro el Vampiro IV and the Mystery of the Papyrus (128K) (HB)

static struct BurnRomInfo SpecRamiro4enRomDesc[] = {
	{ "Ramiro el vampiro IV EN 128K (2023)(The Mojon Twins).tap", 49265, 0x643cf039, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecRamiro4en, SpecRamiro4en, Spec128)
STD_ROM_FN(SpecRamiro4en)

struct BurnDriver BurnSpecRamiro4en = {
	"spec_ramiro4en", NULL, "spec_spec128", NULL, "2023",
	"Ramiro el Vampiro IV and the Mystery of the Papyrus (128K) (HB)\0", NULL, "The Mojon Twins", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PLATFORM | GBF_ADV, 0,
	SpectrumGetZipName, SpecRamiro4enRomInfo, SpecRamiro4enRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Ramiro el Vampiro IV en el Misterio del Papiro (128K) (HB)

static struct BurnRomInfo SpecRamiro4esRomDesc[] = {
	{ "Ramiro el vampiro IV ES 128K (2023)(The Mojon Twins).tap", 49233, 0x54c870f3, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecRamiro4es, SpecRamiro4es, Spec128)
STD_ROM_FN(SpecRamiro4es)

struct BurnDriver BurnSpecRamiro4es = {
	"spec_ramiro4es", "spec_ramiro4en", "spec_spec128", NULL, "2023",
	"Ramiro el Vampiro IV en el Misterio del Papiro (128K) (HB)\0", NULL, "The Mojon Twins", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PLATFORM | GBF_ADV, 0,
	SpectrumGetZipName, SpecRamiro4esRomInfo, SpecRamiro4esRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Ratos! (48K) (HB)

static struct BurnRomInfo SpecRatosRomDesc[] = {
	{ "Ratos! 48K (2020)(MemSoftware).tap", 33662, 0x08d7327a, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecRatos, SpecRatos, Spectrum)
STD_ROM_FN(SpecRatos)

struct BurnDriver BurnSpecRatos = {
	"spec_ratos", NULL, "spec_spectrum", NULL, "2020",
	"Ratos! (48K) (HB)\0", NULL, "MemSoftware", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_ACTION | GBF_PUZZLE, 0,
	SpectrumGetZipName, SpecRatosRomInfo, SpecRatosRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecQAOPSpaceDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Rays' Reprisal (48K) (HB)

static struct BurnRomInfo SpecRaysreprisalRomDesc[] = {
	{ "Rays' Reprisal 48K (2015)(Luca Bordoni).tap", 45218, 0x5101d8a1, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecRaysreprisal, SpecRaysreprisal, Spectrum)
STD_ROM_FN(SpecRaysreprisal)

struct BurnDriver BurnSpecRaysreprisal = {
	"spec_raysreprisal", NULL, "spec_spectrum", NULL, "2015",
	"Rays' Reprisal (48K) (HB)\0", NULL, "Luca Bordoni", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_SHOOT, 0,
	SpectrumGetZipName, SpecRaysreprisalRomInfo, SpecRaysreprisalRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Red Planet (Spanish) (48K) (HB)

static struct BurnRomInfo SpecRedplanetesRomDesc[] = {
	{ "Red Planet ES 48K (2015)(Salva Cantero).tap", 42986, 0x405e7e24, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecRedplanetes, SpecRedplanetes, Spectrum)
STD_ROM_FN(SpecRedplanetes)

struct BurnDriver BurnSpecRedplanetes = {
	"spec_redplanetes", "spec_redplanet", "spec_spectrum", NULL, "2015",
	"Red Planet (Spanish) (48K) (HB)\0", NULL, "Salva Cantero", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecRedplanetesRomInfo, SpecRedplanetesRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Red Raid The Infiltrating - Side A (48K) (HB)

static struct BurnRomInfo SpecRedraidinf1RomDesc[] = {
	{ "Red Raid The Infiltrating - Side A 48K (2022)(ZX Bitles).tap", 47597, 0xfa7ac305, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecRedraidinf1, SpecRedraidinf1, Spectrum)
STD_ROM_FN(SpecRedraidinf1)

struct BurnDriver BurnSpecRedraidinf1 = {
	"spec_redraidinf1", NULL, "spec_spectrum", NULL, "2022",
	"Red Raid The Infiltrating - Side A (48K) (HB)\0", NULL, "ZX Bitles", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PLATFORM | GBF_ADV, 0,
	SpectrumGetZipName, SpecRedraidinf1RomInfo, SpecRedraidinf1RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecIntf2SwapDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Red Raid The Infiltrating - Side B (48K) (HB)

static struct BurnRomInfo SpecRedraidinf2RomDesc[] = {
	{ "Red Raid The Infiltrating - Side B 48K (2022)(ZX Bitles).tap", 47554, 0x9524172b, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecRedraidinf2, SpecRedraidinf2, Spectrum)
STD_ROM_FN(SpecRedraidinf2)

struct BurnDriver BurnSpecRedraidinf2 = {
	"spec_redraidinf2", "spec_redraidinf1", "spec_spectrum", NULL, "2022",
	"Red Raid The Infiltrating - Side B (48K) (HB)\0", NULL, "ZX Bitles", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PLATFORM | GBF_ADV, 0,
	SpectrumGetZipName, SpecRedraidinf2RomInfo, SpecRedraidinf2RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecIntf2SwapDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Redshift (128K) (HB)

static struct BurnRomInfo SpecRedshiftRomDesc[] = {
	{ "Redshift 128K (2019)(Ariel Ruiz).tap", 79855, 0x2198fb31, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecRedshift, SpecRedshift, Spec128)
STD_ROM_FN(SpecRedshift)

struct BurnDriver BurnSpecRedshift = {
	"spec_redshift", NULL, "spec_spec128", NULL, "2019",
	"Redshift (128K) (HB)\0", NULL, "Ariel Ruiz", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_VERSHOOT, 0,
	SpectrumGetZipName, SpecRedshiftRomInfo, SpecRedshiftRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Regulus (48K) (HB)

static struct BurnRomInfo SpecRegulusRomDesc[] = {
	{ "Regulus 48K (2022)(Proxima Software).tap", 40638, 0xf40c4211, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecRegulus, SpecRegulus, Spectrum)
STD_ROM_FN(SpecRegulus)

struct BurnDriver BurnSpecRegulus = {
	"spec_regulus", NULL, "spec_spectrum", NULL, "2022",
	"Regulus (48K) (HB)\0", NULL, "Proxima Software", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_RUNGUN | GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecRegulusRomInfo, SpecRegulusRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Relic, The ~ Reliquia, La (English) (128K) (HB)

static struct BurnRomInfo SpecrelicRomDesc[] = {
	{ "Relic, The 128K (2020)(Roolandoo).tap",  46629, 0x183c5158, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specrelic, Specrelic, Spec128)
STD_ROM_FN(Specrelic)

struct BurnDriver BurnSpecrelic = {
	"spec_relic", NULL, "spec_spec128", NULL, "2020",
	"Reliquia, La (English) (128K) (HB)\0", NULL, "Angel Colaso", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecrelicRomInfo, SpecrelicRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Reliquia, La (Spanish) (128K) (HB)

static struct BurnRomInfo SpecLareliquiaRomDesc[] = {
	{ "Reliquia, La 128K (2020)(Angel Colaso).tap", 46885, 0x8164dbff, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecLareliquia, SpecLareliquia, Spec128)
STD_ROM_FN(SpecLareliquia)

struct BurnDriver BurnSpecLareliquia = {
	"spec_lareliquia", "spec_relic", "spec_spec128", NULL, "2020",
	"Reliquia, La (Spanish) (128K) (HB)\0", NULL, "Angel Colaso", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW | BDF_CLONE, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecLareliquiaRomInfo, SpecLareliquiaRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Renegade Recoloured (128K) (Hack)

static struct BurnRomInfo SpecRenegade128rcRomDesc[] = {
	{ "Renegade Recoloured 128K (2013)(Rafal Miazga).tap", 59448, 0x12db0188, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecRenegade128rc, SpecRenegade128rc, Spec128)
STD_ROM_FN(SpecRenegade128rc)

struct BurnDriver BurnSpecRenegade128rc = {
	"spec_renegade128rc", "spec_renegade128", "spec_spec128", NULL, "2013",
	"Renegade Recoloured (128K) (Hack)\0", NULL, "Rafal Miazga", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE | BDF_HACK, 1, HARDWARE_SPECTRUM, GBF_SCRFIGHT, 0,
	SpectrumGetZipName, SpecRenegade128rcRomInfo, SpecRenegade128rcRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Renenzaiter (128K) (HB)

static struct BurnRomInfo SpecRenzait128RomDesc[] = {
	{ "Renenzaiter 128K (2024)(BTCO).tap", 53343, 0x875304f3, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecRenzait128, SpecRenzait128, Spec128)
STD_ROM_FN(SpecRenzait128)

struct BurnDriver BurnSpecRenzait128 = {
	"spec_renzait128", NULL, "spec_spec128", NULL, "2024",
	"Renenzaiter (128K) (HB)\0", NULL, "BTCO", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecRenzait128RomInfo, SpecRenzait128RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Renenzaiter (48K) (HB)

static struct BurnRomInfo SpecRenzait48RomDesc[] = {
	{ "Renenzaiter 48K (2024)(BTCO).tap", 53058, 0xaaf6ef2d, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecRenzait48, SpecRenzait48, Spectrum)
STD_ROM_FN(SpecRenzait48)

struct BurnDriver BurnSpecRenzait48 = {
	"spec_renzait48", "spec_renzait128", "spec_spectrum", NULL, "2024",
	"Renenzaiter (48K) (HB)\0", NULL, "BTCO", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecRenzait48RomInfo, SpecRenzait48RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Restless Andre (128K) (HB)

static struct BurnRomInfo SpecRestlessandreRomDesc[] = {
	{ "Restless Andre 128K (2019)(Jaime Grilo).tap", 48113, 0x0db7023a, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecRestlessandre, SpecRestlessandre, Spec128)
STD_ROM_FN(SpecRestlessandre)

struct BurnDriver BurnSpecRestlessandre = {
	"spec_restlessandre", NULL, "spec_spec128", NULL, "2019",
	"Restless Andre (128K) (HB)\0", NULL, "Jaime Grilo", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_MAZE | GBF_ACTION, 0,
	SpectrumGetZipName, SpecRestlessandreRomInfo, SpecRestlessandreRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Return of Traxtor (48K) (HB)

static struct BurnRomInfo Spectraxtor2RomDesc[] = {
	{ "Return of Traxtor 48K (2015)(usebox.net).tap", 27292, 0x6fac2cd6, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Spectraxtor2, Spectraxtor2, Spectrum)
STD_ROM_FN(Spectraxtor2)

struct BurnDriver BurnSpectraxtor2 = {
	"spec_traxtor2", NULL, "spec_spectrum", NULL, "2015",
	"Return of Traxtor (48K) (HB)\0", NULL, "usebox.net", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PUZZLE, 0,
	SpectrumGetZipName, Spectraxtor2RomInfo, Spectraxtor2RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Rescate En Marte (48K) (HB)

static struct BurnRomInfo SpecRescmarteRomDesc[] = {
	{ "Rescate En Marte 48K (2019)(Jose Manuel Gris).tap", 28007, 0xdcf24427, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecRescmarte, SpecRescmarte, Spectrum)
STD_ROM_FN(SpecRescmarte)

struct BurnDriver BurnSpecRescmarte = {
	"spec_rescmarte", NULL, "spec_spectrum", NULL, "2019",
	"Rescate En Marte (48K) (HB)\0", NULL, "Jose Manuel Gris", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_ACTION, 0,
	SpectrumGetZipName, SpecRescmarteRomInfo, SpecRescmarteRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecQAOPMDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Reverse Pong (48K) (HB)

static struct BurnRomInfo SpecRevpongRomDesc[] = {
	{ "Reverse Pong 48K (2020)(Miguetelo).tap", 26652, 0x6ecd4a6f, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecRevpong, SpecRevpong, Spectrum)
STD_ROM_FN(SpecRevpong)

struct BurnDriver BurnSpecRevpong = {
	"spec_revpong", NULL, "spec_spectrum", NULL, "2020",
	"Reverse Pong (48K) (HB)\0", NULL, "Miguetelo", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_BALLPADDLE, 0,
	SpectrumGetZipName, SpecRevpongRomInfo, SpecRevpongRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecQAOPSpaceDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// RINGO (128K) (HB)

static struct BurnRomInfo SpecRingoRomDesc[] = {
	{ "RINGO (2022)(RetroSouls).tap", 58931, 0x5d65b662, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecRingo, SpecRingo, Spec128)
STD_ROM_FN(SpecRingo)

struct BurnDriver BurnSpecRingo = {
	"spec_ringo", NULL, "spec_spec128", NULL, "2022",
	"RINGO (128K) (HB)\0", NULL, "RetroSouls", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PLATFORM | GBF_PUZZLE, 0,
	SpectrumGetZipName, SpecRingoRomInfo, SpecRingoRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Rival Gangs EXT (128K) (HB)

static struct BurnRomInfo SpecRivgangsextRomDesc[] = {
	{ "Rival Gangs EXT 128K (2022)(Presh).tap", 101721, 0x4bb86663, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecRivgangsext, SpecRivgangsext, Spec128)
STD_ROM_FN(SpecRivgangsext)

struct BurnDriver BurnSpecRivgangsext = {
	"spec_rivgangsext", NULL, "spec_spec128", NULL, "2022",
	"Rival Gangs EXT (128K) (HB)\0", NULL, "Presh", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_ACTION | GBF_ADV, 0,
	SpectrumGetZipName, SpecRivgangsextRomInfo, SpecRivgangsextRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Road Trippin' 2022 - Coast 2 Coast (128K) (HB)

static struct BurnRomInfo SpecRoadtripc2cRomDesc[] = {
	{ "Road Trippin' 2022 - Coast 2 Coast 128K (2022)(Joesoft).tap", 39520, 0x51e0c19e, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecRoadtripc2c, SpecRoadtripc2c, Spec128)
STD_ROM_FN(SpecRoadtripc2c)

struct BurnDriver BurnSpecRoadtripc2c = {
	"spec_roadtripc2c", NULL, "spec_spec128", NULL, "2022",
	"Road Trippin' 2022 - Coast 2 Coast (128K) (HB)\0", NULL, "Joesoft", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_ACTION, 0,
	SpectrumGetZipName, SpecRoadtripc2cRomInfo, SpecRoadtripc2cRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Robot 1 in... The Planet of Death (128K) (HB)

static struct BurnRomInfo SpecRobot1pdRomDesc[] = {
	{ "Robot 1 in... The Planet of Death 128K (2017)(Psytronik).tap", 43438, 0x352168fb, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecRobot1pd, SpecRobot1pd, Spec128)
STD_ROM_FN(SpecRobot1pd)

struct BurnDriver BurnSpecRobot1pd = {
	"spec_robot1pd", NULL, "spec_spec128", NULL, "2017",
	"Robot 1 in... The Planet of Death (128K) (HB)\0", NULL, "Psytronik", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecRobot1pdRomInfo, SpecRobot1pdRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Robots Rumble (128K) (HB)

static struct BurnRomInfo SpecrrumbleRomDesc[] = {
	{ "Robots Rumble (2018)(Miguetelo).tap", 39461, 0xc7c5dc79, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specrrumble, Specrrumble, Spec128)
STD_ROM_FN(Specrrumble)

struct BurnDriver BurnSpecrrumble = {
	"spec_rrumble", NULL, "spec_spec128", NULL, "2018",
	"Robots Rumble (128K) (HB)\0", NULL, "Miguetelo", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PUZZLE, 0,
	SpectrumGetZipName, SpecrrumbleRomInfo, SpecrrumbleRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Rogue (128K) (HB)

static struct BurnRomInfo SpecRogueRomDesc[] = {
	{ "Rogue 128K (2019)(Dmitry Krapivins).tap", 13718, 0xa9c91888, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecRogue, SpecRogue, Spec128)
STD_ROM_FN(SpecRogue)

struct BurnDriver BurnSpecRogue = {
	"spec_rogue", NULL, "spec_spec128", NULL, "2019",
	"Rogue (128K) (HB)\0", NULL, "Dmitry Krapivins", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_MAZE | GBF_RPG, 0,
	SpectrumGetZipName, SpecRogueRomInfo, SpecRogueRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecCursorKeysDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Rompetechos (English) (48K) (HB)

static struct BurnRomInfo SpecRompenRomDesc[] = {
	{ "Rompetechos EN (2016)(IvanBasic).tap", 48022, 0x44d14074, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecRompen, SpecRompen, Spectrum)
STD_ROM_FN(SpecRompen)

struct BurnDriver BurnSpecRompen = {
	"spec_rompen", NULL, "spec_spectrum", NULL, "2016",
	"Rompetechos (English) (48K) (HB)\0", NULL, "Ivan Gomez Marquez", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_ADV, 0,
	SpectrumGetZipName, SpecRompenRomInfo, SpecRompenRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecQAOPSpaceDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Rompetechos (Spanish) (48K) (HB)

static struct BurnRomInfo SpecRompesRomDesc[] = {
	{ "Rompetechos ES (2016)(IvanBasic).tap", 39786, 0x300392c7, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecRompes, SpecRompes, Spectrum)
STD_ROM_FN(SpecRompes)

struct BurnDriver BurnSpecRompes = {
	"spec_rompes", "spec_rompen", "spec_spectrum", NULL, "2016",
	"Rompetechos (Spanish) (48K) (HB)\0", NULL, "Ivan Gomez Marquez", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_ADV, 0,
	SpectrumGetZipName, SpecRompesRomInfo, SpecRompesRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecQAOPSpaceDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Rondure Sorcerer (48K) (HB)

static struct BurnRomInfo SpecRondureRomDesc[] = {
	{ "Rondure Sorcerer 48K (2023)(Jason Taylor).tap", 46528, 0x3444bd49, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecRondure, SpecRondure, Spectrum)
STD_ROM_FN(SpecRondure)

struct BurnDriver BurnSpecRondure = {
	"spec_rondure", NULL, "spec_spectrum", NULL, "2023",
	"Rondure Sorcerer 48K (HB)\0", NULL, "Jason Taylor", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_MAZE | GBF_RPG, 0,
	SpectrumGetZipName, SpecRondureRomInfo, SpecRondureRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecQAOPSpaceDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Roust (128K) (HB)

static struct BurnRomInfo SpecRoustRomDesc[] = {
	{ "Roust 128K (2018)(highriser).tap", 46458, 0xcf1fa81b, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecRoust, SpecRoust, Spec128)
STD_ROM_FN(SpecRoust)

struct BurnDriver BurnSpecRoust = {
	"spec_roust", NULL, "spec_spec128", NULL, "2018",
	"Roust (128K) (HB)\0", NULL, "highriser", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 2, HARDWARE_SPECTRUM, GBF_ACTION, 0,
	SpectrumGetZipName, SpecRoustRomInfo, SpecRoustRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// R.O.V.R. (128K) (HB)

static struct BurnRomInfo SpecRovrRomDesc[] = {
	{ "ROVR 128K (2018)(Paul Jenkinson).tap", 33973, 0x577fd144, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecRovr, SpecRovr, Spec128)
STD_ROM_FN(SpecRovr)

struct BurnDriver BurnSpecRovr = {
	"spec_rovr", NULL, "spec_spec128", NULL, "2018",
	"R.O.V.R. (128K) (HB)\0", "Remote Operated Vehicle Recovery", "Paul Jenkinson", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_ACTION, 0,
	SpectrumGetZipName, SpecRovrRomInfo, SpecRovrRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Rubicon (128K) (HB, v1.1)

static struct BurnRomInfo SpecrubiconRomDesc[] = {
	{ "Rubicon v1.1 128K (2018)(Rucksack Games).tap", 48224, 0x99ec517e, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specrubicon, Specrubicon, Spec128)
STD_ROM_FN(Specrubicon)

struct BurnDriver BurnSpecrubicon = {
	"spec_rubicon", NULL, "spec_spec128", NULL, "2018",
	"Rubicon (128K) (HB, v1.1)\0", NULL, "Rucksack Games", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_MAZE | GBF_ACTION, 0,
	SpectrumGetZipName, SpecrubiconRomInfo, SpecrubiconRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Rubinho Cucaracha (48K) (HB)

static struct BurnRomInfo SpecRubcucaRomDesc[] = {
	{ "Rubinho Cucaracha 48K (2022)(Zosya Ent.).tap", 46052, 0xf73a1a84, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecRubcuca, SpecRubcuca, Spectrum)
STD_ROM_FN(SpecRubcuca)

struct BurnDriver BurnSpecRubcuca = {
	"spec_rubcuca", NULL, "spec_spectrum", NULL, "2022",
	"Rubinho Cucaracha (48K) (HB)\0", NULL, "Zosya Entertainment", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_RACING, 0,
	SpectrumGetZipName, SpecRubcucaRomInfo, SpecRubcucaRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Ruptus (48K) (HB)

static struct BurnRomInfo SpecRuptusRomDesc[] = {
	{ "Ruptus (2021)(Inufuto).tap", 13000, 0xc818a61f, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecRuptus, SpecRuptus, Spectrum)
STD_ROM_FN(SpecRuptus)

struct BurnDriver BurnSpecRuptus = {
	"spec_ruptus", NULL, "spec_spectrum", NULL, "2021",
	"Ruptus (48K) (HB)\0", "AY Sound supported", "Inufuto", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_SHOOT, 0,
	SpectrumGetZipName, SpecRuptusRomInfo, SpecRuptusRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// S.I.P. - Special Intergalactic Painter (48K) (HB)

static struct BurnRomInfo SpecSipRomDesc[] = {
	{ "S.I.P. - Special Intergalactic Painter 48K (2014)(Gamemaster).tap", 35720, 0x564250e6, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecSip, SpecSip, Spectrum)
STD_ROM_FN(SpecSip)

struct BurnDriver BurnSpecSip = {
	"spec_sip", NULL, "spec_spectrum", NULL, "2014",
	"S.I.P. - Special Intergalactic Painter (48K) (HB)\0", NULL, "RetroSouls - Gamemaster", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_ACTION, 0,
	SpectrumGetZipName, SpecSipRomInfo, SpecSipRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Saboteur Deep Cover - Part 1 (128K) (HB)

static struct BurnRomInfo SpecSaboterdc1RomDesc[] = {
	{ "Saboteur Deep Cover - part 1 128K (2022)(Retro Fusion Annual 2022).tap", 103189, 0xaffc02c5, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecSaboterdc1, SpecSaboterdc1, Spec128)
STD_ROM_FN(SpecSaboterdc1)

struct BurnDriver BurnSpecSaboterdc1 = {
	"spec_saboterdc1", NULL, "spec_spec128", NULL, "2022",
	"Saboteur Deep Cover - Part 1 (128K) (HB)\0", NULL, "Retro Fusion Annual 2022", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_ADV, 0,
	SpectrumGetZipName, SpecSaboterdc1RomInfo, SpecSaboterdc1RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Saboteur Deep Cover - Part 2 (128K) (HB)

static struct BurnRomInfo SpecSaboterdc2RomDesc[] = {
	{ "Saboteur Deep Cover - part 2 128K (2022)(Retro Fusion Annual 2022).tap", 116482, 0x4723ffec, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecSaboterdc2, SpecSaboterdc2, Spec128)
STD_ROM_FN(SpecSaboterdc2)

struct BurnDriver BurnSpecSaboterdc2 = {
	"spec_saboterdc2", "spec_saboterdc1", "spec_spec128", NULL, "2022",
	"Saboteur Deep Cover - Part 2 (128K) (HB)\0", NULL, "Retro Fusion Annual 2022", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_ADV, 0,
	SpectrumGetZipName, SpecSaboterdc2RomInfo, SpecSaboterdc2RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Saboteur 2: Avenging Angel - Speed-up (128K) (Hack)

static struct BurnRomInfo SpecSabotur2speedRomDesc[] = {
	{ "Saboteur 2 - Avenging Angel - Speed-up 128K (2015)(Alex Rider).tap", 60290, 0x468098f8, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecSabotur2speed, SpecSabotur2speed, Spec128)
STD_ROM_FN(SpecSabotur2speed)

struct BurnDriver BurnSpecSabotur2speed = {
	"spec_sabotur2speed", "spec_sabotur2", "spec_spec128", NULL, "2015",
	"Saboteur 2: Avenging Angel - Speed-up (128K) (Hack)\0", NULL, "Alex Rider", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE | BDF_HACK, 1, HARDWARE_SPECTRUM, GBF_ACTION, 0,
	SpectrumGetZipName, SpecSabotur2speedRomInfo, SpecSabotur2speedRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Saboteur - Remastered (German) (48K) (HB)

static struct BurnRomInfo SpecSaboteur23deRomDesc[] = {
	{ "Saboteur - Remastered DE 48K (2023)(Teknamic).tap", 50522, 0x61f6f9e6, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecSaboteur23de, SpecSaboteur23de, Spectrum)
STD_ROM_FN(SpecSaboteur23de)

struct BurnDriver BurnSpecSaboteur23de = {
	"spec_saboteur23de", "spec_saboteur23en", "spec_spectrum", NULL, "2023",
	"Saboteur - Remastered (German) (48K) (HB)\0", "A: up - Z: down - N: left - M: right - SPACE: fire", "Teknamic", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_ACTION, 0,
	SpectrumGetZipName, SpecSaboteur23deRomInfo, SpecSaboteur23deRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Saboteur - Remastered (English) (48K) (HB)

static struct BurnRomInfo SpecSaboteur23enRomDesc[] = {
	{ "Saboteur - Remastered EN 48K (2023)(Teknamic).tap", 50857, 0xebfdc84b, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecSaboteur23en, SpecSaboteur23en, Spectrum)
STD_ROM_FN(SpecSaboteur23en)

struct BurnDriver BurnSpecSaboteur23en = {
	"spec_saboteur23en", NULL, "spec_spectrum", NULL, "2023",
	"Saboteur - Remastered (English) (48K) (HB)\0", "A: up - Z: down - N: left - M: right - SPACE: fire", "Teknamic", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_ACTION, 0,
	SpectrumGetZipName, SpecSaboteur23enRomInfo, SpecSaboteur23enRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Sam Boon - A Case of Murder (128K) (HB)

static struct BurnRomInfo SpecSambooncmRomDesc[] = {
	{ "Sam Boon - A Case of Murder 128K (2023)(Taskmaster Software).tap", 84818, 0x74af3999, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecSambooncm, SpecSambooncm, Spec128)
STD_ROM_FN(SpecSambooncm)

struct BurnDriver BurnSpecSambooncm = {
	"spec_sambooncm", NULL, "spec_spec128", NULL, "2023",
	"Sam Boon - A Case of Murder (128K) (HB)\0", NULL, "Taskmaster Software", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_ADV, 0,
	SpectrumGetZipName, SpecSambooncmRomInfo, SpecSambooncmRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Sam Boon - The Dragnet Case (128K) (HB)

static struct BurnRomInfo SpecSamboondcRomDesc[] = {
	{ "Sam Boon - The Dragnet Case 128K (2023)(Taskmaster Software).tap", 80368, 0x5198feda, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecSamboondc, SpecSamboondc, Spec128)
STD_ROM_FN(SpecSamboondc)

struct BurnDriver BurnSpecSamboondc = {
	"spec_samboondc", NULL, "spec_spec128", NULL, "2023",
	"Sam Boon - The Dragnet Case (128K) (HB)\0", NULL, "Taskmaster Software", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_ADV, 0,
	SpectrumGetZipName, SpecSamboondcRomInfo, SpecSamboondcRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Sam Mallard - the Case of the Missing Swan (English) (48K) (HB, v1.2)

static struct BurnRomInfo SpecSamallardenRomDesc[] = {
	{ "Sam Mallard EN v1.2 (2016-21)(Ersh).tap", 40161, 0xdd71d04f, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecSamallarden, SpecSamallarden, Spectrum)
STD_ROM_FN(SpecSamallarden)

struct BurnDriver BurnSpecSamallarden = {
	"spec_samallarden", NULL, "spec_spectrum", NULL, "2016-21",
	"Sam Mallard - the Case of the Missing Swan (English) (48K) (HB, v1.2)\0", NULL, "Ersh", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_ADV, 0,
	SpectrumGetZipName, SpecSamallardenRomInfo, SpecSamallardenRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Sam Mallard - el Caso del Cisne Desaparecido (Spanish) (48K) (HB, v1.2)

static struct BurnRomInfo SpecSamallardesRomDesc[] = {
	{ "Sam Mallard ES v1.2 (2016-21)(Ersh).tap", 40825, 0x1f33d48e, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecSamallardes, SpecSamallardes, Spectrum)
STD_ROM_FN(SpecSamallardes)

struct BurnDriver BurnSpecSamallardes = {
	"spec_samallardes", "spec_samallarden", "spec_spectrum", NULL, "2016-21",
	"Sam Mallard - el Caso del Cisne Desaparecido (Spanish) (48K) (HB, v1.2)\0", NULL, "Ersh", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_ADV, 0,
	SpectrumGetZipName, SpecSamallardesRomInfo, SpecSamallardesRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Sam Slade - Robo-Hunter (48K) (HB)

static struct BurnRomInfo SpecSamsladeRomDesc[] = {
	{ "Sam Slade - Robo-Hunter 48K (2022)(Monsterbytes).tap", 37981, 0xd5815f2f, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecSamslade, SpecSamslade, Spectrum)
STD_ROM_FN(SpecSamslade)

struct BurnDriver BurnSpecSamslade = {
	"spec_samslade", NULL, "spec_spectrum", NULL, "2022",
	"Sam Slade - Robo-Hunter (48K) (HB)\0", NULL, "Monsterbytes", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_ACTION | GBF_ADV, 0,
	SpectrumGetZipName, SpecSamsladeRomInfo, SpecSamsladeRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Santos' Christmas Adventure (48K) (HB)

static struct BurnRomInfo SpecSchristadvRomDesc[] = {
	{ "Santos' Christmas Adventure 48K-AY (2017)(Dave Hughes).tap", 43228, 0x24437e43, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecSchristadv, SpecSchristadv, Spec128)
STD_ROM_FN(SpecSchristadv)

struct BurnDriver BurnSpecSchristadv = {
	"spec_schristadv", NULL, "spec_spec128", NULL, "2017",
	"Santos' Christmas Adventure (48K) (HB)\0", "AY Sound supported", "Dave Hughes", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecSchristadvRomInfo, SpecSchristadvRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Savage Princess (48K) (HB)

static struct BurnRomInfo SpecSavageprincessRomDesc[] = {
	{ "Savage Princess 48K (2020)(BitmapSoft).tap", 41563, 0x6edbd594, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecSavageprincess, SpecSavageprincess, Spectrum)
STD_ROM_FN(SpecSavageprincess)

struct BurnDriver BurnSpecSavageprincess = {
	"spec_savageprincess", NULL, "spec_spectrum", NULL, "2020",
	"Savage Princess (48K) (HB)\0", NULL, "Bitmap Soft", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecSavageprincessRomInfo, SpecSavageprincessRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Savage Princess 2 (128K) (HB)

static struct BurnRomInfo SpecSavageprincess2RomDesc[] = {
	{ "Savage Princess 2 128K (2023)(Bitmap Soft).tap", 48393, 0xd3e5e547, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecSavageprincess2, SpecSavageprincess2, Spec128)
STD_ROM_FN(SpecSavageprincess2)

struct BurnDriver BurnSpecSavageprincess2 = {
	"spec_savageprincess2", NULL, "spec_spec128", NULL, "2023",
	"Savage Princess 2 (128K) (HB)\0", NULL, "Bitmap Soft", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecSavageprincess2RomInfo, SpecSavageprincess2RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Saving Kong - Crazy Kong City II (bgm Dizziness) (128K) (HB)

static struct BurnRomInfo SpecSavekongaRomDesc[] = {
	{ "Crazy Kong City - Saving Kong - Music Dizziness (2017)(Gabriele Amore).tap", 40684, 0x84860fed, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecSavekonga, SpecSavekonga, Spec128)
STD_ROM_FN(SpecSavekonga)

struct BurnDriver BurnSpecSavekonga = {
	"spec_savekonga", NULL, "spec_spec128", NULL, "2017",
	"Saving Kong - Crazy Kong City II (bgm Dizziness) (128K) (HB)\0", NULL, "Gabriele Amore", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_SHOOT, 0,
	SpectrumGetZipName, SpecSavekongaRomInfo, SpecSavekongaRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Saving Kong - Crazy Kong City II (bgm Massive Attack) (128K) (HB)

static struct BurnRomInfo SpecSavekongbRomDesc[] = {
	{ "Crazy Kong City - Saving Kong - Music Massive Attack (2017)(Gabriele Amore).tap", 39573, 0x3c553744, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecSavekongb, SpecSavekongb, Spec128)
STD_ROM_FN(SpecSavekongb)

struct BurnDriver BurnSpecSavekongb = {
	"spec_savekongb", "spec_savekonga", "spec_spec128", NULL, "2017",
	"Saving Kong - Crazy Kong City II (bgm Massive Attack) (128K) (HB)\0", NULL, "Gabriele Amore", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_SHOOT, 0,
	SpectrumGetZipName, SpecSavekongbRomInfo, SpecSavekongbRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// SBOT Part-1 (128K) (HB)

static struct BurnRomInfo SpecSbotp1RomDesc[] = {
	{ "SBOT Part1 128K (2023)(Xavisan).tap", 46761, 0x3377c7b7, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecSbotp1, SpecSbotp1, Spec128)
STD_ROM_FN(SpecSbotp1)

struct BurnDriver BurnSpecSbotp1 = {
	"spec_sbotp1", NULL, "spec_spec128", NULL, "2023",
	"SBOT Part-1 (128K) (HB)\0", NULL, "Xavisan", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecSbotp1RomInfo, SpecSbotp1RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// SBOT Part-2 (128K) (HB)

static struct BurnRomInfo SpecSbotp2RomDesc[] = {
	{ "SBOT Part2 128K (2023)(Xavisan).tap", 48067, 0xb2a76283, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecSbotp2, SpecSbotp2, Spec128)
STD_ROM_FN(SpecSbotp2)

struct BurnDriver BurnSpecSbotp2 = {
	"spec_sbotp2", "spec_sbotp1", "spec_spec128", NULL, "2023",
	"SBOT Part-2 (128K) (HB)\0", NULL, "Xavisan", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecSbotp2RomInfo, SpecSbotp2RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Scate Alice (128K) (HB)

static struct BurnRomInfo SpecSctaliceRomDesc[] = {
	{ "Scate Alice (2021)(alko-art).tap", 45253, 0x914a8f50, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecSctalice, SpecSctalice, Spec128)
STD_ROM_FN(SpecSctalice)

struct BurnDriver BurnSpecSctalice = {
	"spec_sctalice", NULL, "spec_spec128", NULL, "2021",
	"Scate Alice (128K) (HB)\0", NULL, "Alko-Art", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_ACTION, 0,
	SpectrumGetZipName, SpecSctaliceRomInfo, SpecSctaliceRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Secrets: Castle of Doom (48K) (HB)

static struct BurnRomInfo SpecSecretsRomDesc[] = {
	{ "Secrets - Castle of Doom 48K (1987-2024)(Wizbots Games).tap", 46289, 0x9d896188, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecSecrets, SpecSecrets, Spectrum)
STD_ROM_FN(SpecSecrets)

struct BurnDriver BurnSpecSecrets = {
	"spec_secrets", NULL, "spec_spectrum", NULL, "1987-2024",
	"Secrets: Castle of Doom (48K) (HB)\0", NULL, "Wizbots Games", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecSecretsRomInfo, SpecSecretsRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Sector Invasion (128K) (HB)

static struct BurnRomInfo SpecsectinvnRomDesc[] = {
	{ "Sector Invasion 128K (2014)(RetroSouls).tap", 38873, 0x275c56d5, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specsectinvn, Specsectinvn, Spec128)
STD_ROM_FN(Specsectinvn)

struct BurnDriver BurnSpecsectinvn = {
	"spec_sectinvn", NULL, "spec_spec128", NULL, "2014",
	"Sector Invasion (128K) (HB)\0", NULL, "RetroSouls", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_VERSHOOT, 0,
	SpectrumGetZipName, SpecsectinvnRomInfo, SpecsectinvnRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecQAOPSpaceDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Seraphima (English) (128K) (HB)

static struct BurnRomInfo SpecSeraphimaenRomDesc[] = {
	{ "Seraphima EN 128K (2023)(Zosya Entertainment).tap", 120381, 0xe03b415f, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecSeraphimaen, SpecSeraphimaen, Spec128)
STD_ROM_FN(SpecSeraphimaen)

struct BurnDriver BurnSpecSeraphimaen = {
	"spec_seraphimaen", NULL, "spec_spec128", NULL, "2023",
	"Seraphima (English) (128K) (HB)\0", NULL, "Zosya Entertainment", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_RUNGUN | GBF_ADV, 0,
	SpectrumGetZipName, SpecSeraphimaenRomInfo, SpecSeraphimaenRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Seraphima (Portuguese) (128K) (HB)

static struct BurnRomInfo SpecSeraphimaptRomDesc[] = {
	{ "Seraphima PT 128K (2023)(Zosya Entertainment).tap", 120493, 0x1f7bb40c, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecSeraphimapt, SpecSeraphimapt, Spec128)
STD_ROM_FN(SpecSeraphimapt)

struct BurnDriver BurnSpecSeraphimapt = {
	"spec_seraphimapt", "spec_seraphimaen", "spec_spec128", NULL, "2023",
	"Seraphima (Portuguese) (128K) (HB)\0", NULL, "Zosya Entertainment", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_RUNGUN | GBF_ADV, 0,
	SpectrumGetZipName, SpecSeraphimaptRomInfo, SpecSeraphimaptRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Seraphima (Russian) (128K) (HB)

static struct BurnRomInfo SpecSeraphimaruRomDesc[] = {
	{ "Seraphima RU 128K (2023)(Zosya Entertainment).tap", 120408, 0xf93bb1b5, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecSeraphimaru, SpecSeraphimaru, Spec128)
STD_ROM_FN(SpecSeraphimaru)

struct BurnDriver BurnSpecSeraphimaru = {
	"spec_seraphimaru", "spec_seraphimaen", "spec_spec128", NULL, "2023",
	"Seraphima (Russian) (128K) (HB)\0", NULL, "Zosya Entertainment", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_RUNGUN | GBF_ADV, 0,
	SpectrumGetZipName, SpecSeraphimaruRomInfo, SpecSeraphimaruRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Seraphima - First Edition (128K) (HB)

static struct BurnRomInfo SpecSeraphimaRomDesc[] = {
	{ "Seraphima 128K (2023)(Zosya Entertainment).tap", 119727, 0x70cd9fb0, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecSeraphima, SpecSeraphima, Spec128)
STD_ROM_FN(SpecSeraphima)

struct BurnDriver BurnSpecSeraphima = {
	"spec_seraphima", "spec_seraphimaen", "spec_spec128", NULL, "2023",
	"Seraphima - First Edition (128K) (HB)\0", "'Yandex Retro Games Battle' version", "Zosya Entertainment", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_RUNGUN | GBF_ADV, 0,
	SpectrumGetZipName, SpecSeraphimaRomInfo, SpecSeraphimaRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Sgt. Helmet - Training Day (English) (48K) (HB)

static struct BurnRomInfo SpecSgthtraindayenRomDesc[] = {
	{ "Sgt. Helmet - Training Day EN (2013)(The Mojon Twins).tap", 43229, 0xaed97757, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecSgthtraindayen, SpecSgthtraindayen, Spectrum)
STD_ROM_FN(SpecSgthtraindayen)

struct BurnDriver BurnSpecSgthtraindayen = {
	"spec_sgthtraindayen", NULL, "spec_spectrum", NULL, "2013",
	"Sgt. Helmet - Training Day (English) (48K) (HB)\0", NULL, "The Mojon Twins", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_RUNGUN | GBF_MAZE, 0,
	SpectrumGetZipName, SpecSgthtraindayenRomInfo, SpecSgthtraindayenRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Sgt. Helmet - Training Day (Spanish) (48K) (HB)

static struct BurnRomInfo SpecSgthtraindayesRomDesc[] = {
	{ "Sgt. Helmet - Training Day ES (2013)(The Mojon Twins).tap", 43285, 0x2de72a6c, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecSgthtraindayes, SpecSgthtraindayes, Spectrum)
STD_ROM_FN(SpecSgthtraindayes)

struct BurnDriver BurnSpecSgthtraindayes = {
	"spec_sgthtraindayes", "spec_sgthtraindayen", "spec_spectrum", NULL, "2013",
	"Sgt. Helmet - Training Day (Spanish) (48K) (HB)\0", NULL, "The Mojon Twins", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_RUNGUN | GBF_MAZE, 0,
	SpectrumGetZipName, SpecSgthtraindayesRomInfo, SpecSgthtraindayesRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Shape Shifter (48K) (HB)

static struct BurnRomInfo SpecShapeshiftRomDesc[] = {
	{ "Shape Shifter 48K (2014)(Cronosoft).tap", 48152, 0xcc277be0, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecShapeshift, SpecShapeshift, Spectrum)
STD_ROM_FN(SpecShapeshift)

struct BurnDriver BurnSpecShapeshift = {
	"spec_shapeshift", NULL, "spec_spectrum", NULL, "2014",
	"Shape Shifter (48K) (HB)\0", NULL, "Cronosoft", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PUZZLE, 0,
	SpectrumGetZipName, SpecShapeshiftRomInfo, SpecShapeshiftRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Sheepish (128K) (HB)

static struct BurnRomInfo SpecSheepishRomDesc[] = {
	{ "Sheepish 128K (2024)(Gabriele Amore).tap", 56786, 0x86b831eb, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecSheepish, SpecSheepish, Spec128)
STD_ROM_FN(SpecSheepish)

struct BurnDriver BurnSpecSheepish = {
	"spec_sheepish", NULL, "spec_spec128", NULL, "2024",
	"Sheepish (128K) (HB)\0", NULL, "Gabriele Amore", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecSheepishRomInfo, SpecSheepishRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Shovel Adventure ZX (128K) (HB, v1.2)

static struct BurnRomInfo SpecShoveladvRomDesc[] = {
	{ "Shovel Adventure ZX 1.2 (2021)(Pat Morita Team).tap", 62841, 0x75d4a6ee, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecShoveladv, SpecShoveladv, Spec128)
STD_ROM_FN(SpecShoveladv)

struct BurnDriver BurnSpecShoveladv = {
	"spec_shoveladv", NULL, "spec_spec128", NULL, "2021",
	"Shovel Adventure ZX (128K) (HB, v1.2)\0", NULL, "Pat Morita Team", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_MAZE | GBF_ACTION, 0,
	SpectrumGetZipName, SpecShoveladvRomInfo, SpecShoveladvRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Shovel Duck II - Dig Duck (48K) (HB)

static struct BurnRomInfo SpecSd2digduckRomDesc[] = {
	{ "Shovel Duck II - Dig Duck 48K (2024)(KTB Productions).tap", 49516, 0x88ad0612, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecSd2digduck, SpecSd2digduck, Spectrum)
STD_ROM_FN(SpecSd2digduck)

struct BurnDriver BurnSpecSd2digduck = {
	"spec_sd2digduck", NULL, "spec_spectrum", NULL, "2024",
	"Shovel Duck II - Dig Duck (48K) (HB)\0", NULL, "KTB Productions", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecSd2digduckRomInfo, SpecSd2digduckRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Shovel Duck II - A Minor Panic!  (48K) (HB)

static struct BurnRomInfo SpecSd2mipanicRomDesc[] = {
	{ "Shovel Duck II - A Minor Panic!  48K (2024)(KTB Productions).tap", 44856, 0xf44c9317, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecSd2mipanic, SpecSd2mipanic, Spectrum)
STD_ROM_FN(SpecSd2mipanic)

struct BurnDriver BurnSpecSd2mipanic = {
	"spec_sd2mipanic", "spec_sd2digduck", "spec_spectrum", NULL, "2024",
	"Shovel Duck II - A Minor Panic!  (48K) (HB)\0", NULL, "KTB Productions", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_ACTION, 0,
	SpectrumGetZipName, SpecSd2mipanicRomInfo, SpecSd2mipanicRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Shovel Duck II - A Mousive Panic! (48K) (HB)

static struct BurnRomInfo SpecSd2mopanicRomDesc[] = {
	{ "Shovel Duck II - A Mousive Panic! 48K (2024)(KTB Productions).tap", 43232, 0xfbcc3a8c, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecSd2mopanic, SpecSd2mopanic, Spectrum)
STD_ROM_FN(SpecSd2mopanic)

struct BurnDriver BurnSpecSd2mopanic = {
	"spec_sd2mopanic", "spec_sd2digduck", "spec_spectrum", NULL, "2024",
	"Shovel Duck II - A Mousive Panic! (48K) (HB)\0", NULL, "KTB Productions", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_ACTION, 0,
	SpectrumGetZipName, SpecSd2mopanicRomInfo, SpecSd2mopanicRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Sir Ababol (48K) (HB)

static struct BurnRomInfo SpecSirababolRomDesc[] = {
	{ "Sir Ababol 48K (2010)(The Mojon Twins).tap", 37260, 0xa5dfc1e9, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecSirababol, SpecSirababol, Spectrum)
STD_ROM_FN(SpecSirababol)

struct BurnDriver BurnSpecSirababol = {
	"spec_sirababol", "spec_sirababoldx", "spec_spectrum", NULL, "2010",
	"Sir Ababol (48K) (HB)\0", NULL, "The Mojon Twins", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecSirababolRomInfo, SpecSirababolRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Sir Ababol 2 (English) (128K) (HB)

static struct BurnRomInfo Specsirababol2RomDesc[] = {
	{ "Sir Ababol 2 EN 128K (2013)(The Mojon Twins).tap", 79038, 0x61a75108, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specsirababol2, Specsirababol2, Spec128)
STD_ROM_FN(Specsirababol2)

struct BurnDriver BurnSpecsirababol2 = {
	"spec_sirababol2", NULL, "spec_spec128", NULL, "2013",
	"Sir Ababol 2 (English) (128K) (HB)\0", NULL, "The Mojon Twins", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, Specsirababol2RomInfo, Specsirababol2RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Sir Ababol 2 (Spanish) (128K) (HB)

static struct BurnRomInfo SpecSirababol2esRomDesc[] = {
	{ "Sir Ababol 2 ES 128K (2013)(The Mojon Twins).tap", 79102, 0x480a2fcd, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecSirababol2es, SpecSirababol2es, Spec128)
STD_ROM_FN(SpecSirababol2es)

struct BurnDriver BurnSpecSirababol2es = {
	"spec_sirababol2es", "spec_sirababol2", "spec_spec128", NULL, "2013",
	"Sir Ababol 2 (Spanish) (128K) (HB)\0", NULL, "The Mojon Twins", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecSirababol2esRomInfo, SpecSirababol2esRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Sir Ababol DX (48K) (HB)

static struct BurnRomInfo SpecsirababoldxRomDesc[] = {
	{ "Sir Ababol DX 48K (2013)(The Mojon Twins).tap", 43469, 0x06705a38, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specsirababoldx, Specsirababoldx, Spectrum)
STD_ROM_FN(Specsirababoldx)

struct BurnDriver BurnSpecsirababoldx = {
	"spec_sirababoldx", NULL, "spec_spectrum", NULL, "2013",
	"Sir Ababol DX (48K) (HB)\0", NULL, "The Mojon Twins", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecsirababoldxRomInfo, SpecsirababoldxRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Slide-ZX UPG Edition (48K) (HB)

static struct BurnRomInfo SpecSlidezxRomDesc[] = {
	{ "Slide-ZX UPG Edition 48K (2023)(highriser).tap", 45682, 0x199ec73d, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecSlidezx, SpecSlidezx, Spectrum)
STD_ROM_FN(SpecSlidezx)

struct BurnDriver BurnSpecSlidezx = {
	"spec_slidezx", NULL, "spec_spectrum", NULL, "2023",
	"Slide-ZX UPG Edition (48K) (HB)\0", NULL, "highriser", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PUZZLE, 0,
	SpectrumGetZipName, SpecSlidezxRomInfo, SpecSlidezxRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Smelly Cat (English) (128K) (HB)

static struct BurnRomInfo SpecSmellycateRomDesc[] = {
	{ "Smelly Cat EN 128K (2022)(Flopping).tap", 48335, 0xfbd78a11, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecSmellycate, SpecSmellycate, Spec128)
STD_ROM_FN(SpecSmellycate)

struct BurnDriver BurnSpecSmellycate = {
	"spec_smellycate", NULL, "spec_spec128", NULL, "2022",
	"Smelly Cat (English) (128K) (HB)\0", NULL, "Flopping", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecSmellycateRomInfo, SpecSmellycateRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Smelly Cat (Italian) (128K) (HB)

static struct BurnRomInfo SpecSmellycatiRomDesc[] = {
	{ "Smelly Cat IT 128K (2022)(Flopping).tap", 48335, 0x3e07cc4f, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecSmellycati, SpecSmellycati, Spec128)
STD_ROM_FN(SpecSmellycati)

struct BurnDriver BurnSpecSmellycati = {
	"spec_smellycati", "spec_smellycate", "spec_spec128", NULL, "2022",
	"Smelly Cat (Italian) (128K) (HB)\0", NULL, "Flopping", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecSmellycatiRomInfo, SpecSmellycatiRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Smelly Cat (Portuguese) (128K) (HB)

static struct BurnRomInfo SpecSmellycatpRomDesc[] = {
	{ "Smelly Cat PT 128K (2022)(Flopping).tap", 48335, 0x7259714a, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecSmellycatp, SpecSmellycatp, Spec128)
STD_ROM_FN(SpecSmellycatp)

struct BurnDriver BurnSpecSmellycatp = {
	"spec_smellycatp", "spec_smellycate", "spec_spec128", NULL, "2022",
	"Smelly Cat (Portuguese) (128K) (HB)\0", NULL, "Flopping", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecSmellycatpRomInfo, SpecSmellycatpRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Smelly Cat (Spanish) (128K) (HB)

static struct BurnRomInfo SpecSmellycatsRomDesc[] = {
	{ "Smelly Cat ES 128K (2022)(Flopping).tap", 48335, 0x264ddb3a, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecSmellycats, SpecSmellycats, Spec128)
STD_ROM_FN(SpecSmellycats)

struct BurnDriver BurnSpecSmellycats = {
	"spec_smellycats", "spec_smellycate", "spec_spec128", NULL, "2022",
	"Smelly Cat (Spanish) (128K) (HB)\0", NULL, "Flopping", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecSmellycatsRomInfo, SpecSmellycatsRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Smudge Bad Moonee Rising (48K) (HB)

static struct BurnRomInfo SpecSbmrRomDesc[] = {
	{ "Smudge Bad Moonee Rising 48K (2022)(Clebin Games).tap", 32385, 0xdb530914, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecSbmr, SpecSbmr, Spectrum)
STD_ROM_FN(SpecSbmr)

struct BurnDriver BurnSpecSbmr = {
	"spec_sbmr", NULL, "spec_spectrum", NULL, "2022",
	"Smudge Bad Moonee Rising (48K) (HB)\0", NULL, "Clebin Games", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_ACTION, 0,
	SpectrumGetZipName, SpecSbmrRomInfo, SpecSbmrRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Snake (48K) (HB, v3)

static struct BurnRomInfo SpecSnakeRomDesc[] = {
	{ "Snake v3 48K (2022)(JBizzel).tap", 26586, 0x172abcaf, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecSnake, SpecSnake, Spectrum)
STD_ROM_FN(SpecSnake)

struct BurnDriver BurnSpecSnake = {
	"spec_snake", NULL, "spec_spectrum", NULL, "2021-22",
	"Snake (48K) (HB, v3)\0", "Redefine keys to use controller", "JBizzel", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_ACTION, 0,
	SpectrumGetZipName, SpecSnakeRomInfo, SpecSnakeRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecCursorKeysDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Snake Power (48K) (HB, v1.4)

static struct BurnRomInfo SpecSnakepowerRomDesc[] = {
	{ "Snake Power v1.4 48K (2023)(Hash6lron).tap", 37771, 0xf799a3ae, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecSnakepower, SpecSnakepower, Spectrum)
STD_ROM_FN(SpecSnakepower)

struct BurnDriver BurnSpecSnakepower = {
	"spec_snakepower", NULL, "spec_spectrum", NULL, "2023",
	"Snake Power (48K) (HB, v1.4)\0", NULL, "Hash6lron", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_ACTION, 0,
	SpectrumGetZipName, SpecSnakepowerRomInfo, SpecSnakepowerRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Snowed Under  (128K) (HB)

static struct BurnRomInfo SpecSnowund128RomDesc[] = {
	{ "Snowed Under 128K (2021)(Highrise).tap", 56461, 0xf90c37b5, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecSnowund128, SpecSnowund128, Spec128)
STD_ROM_FN(SpecSnowund128)

struct BurnDriver BurnSpecSnowund128 = {
	"spec_snowund128", NULL, "spec_spec128", NULL, "2021",
	"Snowed Under  (128K) (HB)\0", NULL, "highriser, Gabriele Amore", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecSnowund128RomInfo, SpecSnowund128RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Snowed Under (48K) (HB)

static struct BurnRomInfo SpecSnowund48RomDesc[] = {
	{ "Snowed Under 48K (2021)(Highrise).tap", 44322, 0x7e180aaa, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecSnowund48, SpecSnowund48, Spectrum)
STD_ROM_FN(SpecSnowund48)

struct BurnDriver BurnSpecSnowund48 = {
	"spec_snowund48", "spec_snowund128", "spec_spectrum", NULL, "2021",
	"Snowed Under (48K) (HB)\0", NULL, "highriser, Gabriele Amore", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecSnowund48RomInfo, SpecSnowund48RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// S.o.L.o. (English) (128K) (HB)

static struct BurnRomInfo SpecsoloenRomDesc[] = {
	{ "S.o.L.o EN 128K (2021)(Roolandoo).tap", 44386, 0x1db76b41, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specsoloen, Specsoloen, Spec128)
STD_ROM_FN(Specsoloen)

struct BurnDriver BurnSpecsoloen = {
	"spec_soloen", NULL, "spec_spec128", NULL, "2021",
	"S.o.L.o. (English) (128K) (HB)\0", NULL, "Roolandoo", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecsoloenRomInfo, SpecsoloenRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// S.o.L.o. (Spanish) (128K) (HB)

static struct BurnRomInfo SpecsoloesRomDesc[] = {
	{ "S.o.L.o ES 128K (2021)(Roolandoo).tap", 44386, 0x20ba7c97, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specsoloes, Specsoloes, Spec128)
STD_ROM_FN(Specsoloes)

struct BurnDriver BurnSpecsoloes = {
	"spec_soloes", "spec_soloen", "spec_spec128", NULL, "2021",
	"S.o.L.o. (Spanish) (128K) (HB)\0", NULL, "Roolandoo", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecsoloesRomInfo, SpecsoloesRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Somewhere in Hell (128K) (HB)

static struct BurnRomInfo SpecSomehellRomDesc[] = {
	{ "Somewhere in Hell 128K (2021)(Packobilly).tap", 51930, 0x76b01aff, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecSomehell, SpecSomehell, Spec128)
STD_ROM_FN(SpecSomehell)

struct BurnDriver BurnSpecSomehell = {
	"spec_somehell", NULL, "spec_spec128", NULL, "2021",
	"Somewhere in Hell (128K) (HB)\0", NULL, "Packobilly", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_ACTION, 0,
	SpectrumGetZipName, SpecSomehellRomInfo, SpecSomehellRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Sophia (128K) (HB)

static struct BurnRomInfo SpecsophiaRomDesc[] = {
	{ "Sophia (2017)(Alessandro Grussu).tap", 86193, 0x3681c3a0, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specsophia, Specsophia, Spec128)
STD_ROM_FN(Specsophia)

struct BurnDriver BurnSpecsophia = {
	"spec_sophia", NULL, "spec_spec128", NULL, "2017",
	"Sophia (128K) (HB)\0", "Use 'Space' and 'Enter' to choose options", "Alessandro Grussu", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecsophiaRomInfo, SpecsophiaRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Sophia II (128K) (HB)

static struct BurnRomInfo Specsophia2RomDesc[] = {
	{ "Sophia II (2019)(Alessandro Grussu).tap", 78009, 0xea86b2a1, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specsophia2, Specsophia2, Spec128)
STD_ROM_FN(Specsophia2)

struct BurnDriver BurnSpecsophia2 = {
	"spec_sophia2", NULL, "spec_spec128", NULL, "2019",
	"Sophia II (128K) (HB)\0", NULL, "Alessandro Grussu", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, Specsophia2RomInfo, Specsophia2RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Sorcerer Kid Adventure (128K) (HB)

static struct BurnRomInfo SpecSkidadvRomDesc[] = {
	{ "Sorcerer Kid Adventure 128K (2024)(vidaextraretro).tap", 40789, 0x35c93a20, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecSkidadv, SpecSkidadv, Spec128)
STD_ROM_FN(SpecSkidadv)

struct BurnDriver BurnSpecSkidadv = {
	"spec_skidadv", NULL, "spec_spec128", NULL, "2024",
	"Sorcerer Kid Adventure (128K) (HB)\0", NULL, "vidaextraretro", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecSkidadvRomInfo, SpecSkidadvRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Sorceress (48K) (HB)

static struct BurnRomInfo SpecSorceressRomDesc[] = {
	{ "Sorceress 48K (2017)(Payndz).tap", 32040, 0x4466ed3a, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecSorceress, SpecSorceress, Spectrum)
STD_ROM_FN(SpecSorceress)

struct BurnDriver BurnSpecSorceress = {
	"spec_sorceress", NULL, "spec_spectrum", NULL, "2017",
	"Sorceress (48K) (HB)\0", NULL, "Payndz", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_ACTION | GBF_MAZE, 0,
	SpectrumGetZipName, SpecSorceressRomInfo, SpecSorceressRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecQAOPSpaceDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Sorcery Island - First Part (English) (48K) (HB)

static struct BurnRomInfo SpecSorceryen1RomDesc[] = {
	{ "Sorcery Island Part1 EN 48K (2021)(Jose Manuel Gris).tap", 41574, 0x1578a4af, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecSorceryen1, SpecSorceryen1, Spectrum)
STD_ROM_FN(SpecSorceryen1)

struct BurnDriver BurnSpecSorceryen1 = {
	"spec_sorceryen1", NULL, "spec_spectrum", NULL, "2021",
	"Sorcery Island - First Part (English) (48K) (HB)\0", "Set 'keyboard' to use controller", "Jose Manuel Gris", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PLATFORM | GBF_ADV, 0,
	SpectrumGetZipName, SpecSorceryen1RomInfo, SpecSorceryen1RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecQAOPSpaceDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Sorcery Island - Second Part (English) (48K) (HB)

static struct BurnRomInfo SpecSorceryen2RomDesc[] = {
	{ "Sorcery Island Part2 EN 48K (2021)(Jose Manuel Gris).tap", 41887, 0xf1039856, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecSorceryen2, SpecSorceryen2, Spectrum)
STD_ROM_FN(SpecSorceryen2)

struct BurnDriver BurnSpecSorceryen2 = {
	"spec_sorceryen2", "spec_sorceryen1", "spec_spectrum", NULL, "2021",
	"Sorcery Island - Second Part (English) (48K) (HB)\0", "Set 'keyboard' to use controller", "Jose Manuel Gris", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PLATFORM | GBF_ADV, 0,
	SpectrumGetZipName, SpecSorceryen2RomInfo, SpecSorceryen2RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecQAOPSpaceDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Sorcery Island - First Part (Spanish) (48K) (HB)

static struct BurnRomInfo SpecSorceryes1RomDesc[] = {
	{ "Sorcery Island Part1 ES 48K (2021)(Jose Manuel Gris).tap", 41618, 0xa55dca8a, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecSorceryes1, SpecSorceryes1, Spectrum)
STD_ROM_FN(SpecSorceryes1)

struct BurnDriver BurnSpecSorceryes1 = {
	"spec_sorceryes1", NULL, "spec_spectrum", NULL, "2021",
	"Sorcery Island - First Part (Spanish) (48K) (HB)\0", "Set 'teclado' to use controller", "Jose Manuel Gris", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PLATFORM | GBF_ADV, 0,
	SpectrumGetZipName, SpecSorceryes1RomInfo, SpecSorceryes1RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecQAOPSpaceDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Sorcery Island - Second Part (Spanish) (48K) (HB)

static struct BurnRomInfo SpecSorceryes2RomDesc[] = {
	{ "Sorcery Island Part2 ES 48K (2021)(Jose Manuel Gris).tap", 41929, 0x4526a7fe, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecSorceryes2, SpecSorceryes2, Spectrum)
STD_ROM_FN(SpecSorceryes2)

struct BurnDriver BurnSpecSorceryes2 = {
	"spec_sorceryes2", "spec_sorceryes1", "spec_spectrum", NULL, "2021",
	"Sorcery Island - Second Part (Spanish) (48K) (HB)\0", "Set 'teclado' to use controller", "Jose Manuel Gris", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PLATFORM | GBF_ADV, 0,
	SpectrumGetZipName, SpecSorceryes2RomInfo, SpecSorceryes2RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecQAOPSpaceDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Souls (Spanish) (48K) (HB)

static struct BurnRomInfo SpecSoulsRomDesc[] = {
	{ "Souls ES (2013)(Retrobytes Productions).tap", 36310, 0xa6e182bb, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecSouls, SpecSouls, Spectrum)
STD_ROM_FN(SpecSouls)

struct BurnDriver BurnSpecSouls = {
	"spec_souls", "spec_soulsrmd", "spec_spectrum", NULL, "2013",
	"Souls (Spanish) (48K) (HB)\0", NULL, "Retrobytes Productions", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecSoulsRomInfo, SpecSoulsRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecQAOPSpaceDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Souls Remaster (Spanish) (48K) (HB)

static struct BurnRomInfo SpecSoulsrmdRomDesc[] = {
	{ "Souls Remaster ES (2013)(Retrobytes Productions).tap", 47594, 0x18fabeb3, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecSoulsrmd, SpecSoulsrmd, Spectrum)
STD_ROM_FN(SpecSoulsrmd)

struct BurnDriver BurnSpecSoulsrmd = {
	"spec_soulsrmd", NULL, "spec_spectrum", NULL, "2013",
	"Souls Remaster (Spanish) (48K) (HB)\0", NULL, "Retrobytes Productions", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecSoulsrmdRomInfo, SpecSoulsrmdRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecQAOPSpaceDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Space Racing (128K) (HB, v1.2)

static struct BurnRomInfo SpecSpracingRomDesc[] = {
	{ "Space Racing v1.2 128K (2021)(Voxel Tower).tap", 88616, 0x64e0ef40, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecSpracing, SpecSpracing, Spec128)
STD_ROM_FN(SpecSpracing)

struct BurnDriver BurnSpecSpracing = {
	"spec_spracing", NULL, "spec_spec128", NULL, "2021",
	"Space Racing (128K) (HB, v1.2)\0", NULL, "Voxel Tower", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_RACING, 0,
	SpectrumGetZipName, SpecSpracingRomInfo, SpecSpracingRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecQAOPSpaceDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Space Sludge (16K) (HB)

static struct BurnRomInfo SpecSpacesludgeRomDesc[] = {
	{ "Space Sludge 16K (2019)(Bumfun Software).tap", 8365, 0xdea551a7, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecSpacesludge, SpecSpacesludge, Spectrum)
STD_ROM_FN(SpecSpacesludge)

struct BurnDriver BurnSpecSpacesludge = {
	"spec_spacesludge", NULL, "spec_spectrum", NULL, "2019",
	"Space Sludge (16K) (HB)\0", NULL, "Bumfun Software", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_SHOOT, 0,
	SpectrumGetZipName, SpecSpacesludgeRomInfo, SpecSpacesludgeRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecCursorKeysDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Speccy Bros (48K) (HB)

static struct BurnRomInfo SpecSpeccybrosRomDesc[] = {
	{ "Speccy Bros 48K (2012)(Climacus).tap", 34325, 0x5dd530b2, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecSpeccybros, SpecSpeccybros, Spectrum)
STD_ROM_FN(SpecSpeccybros)

struct BurnDriver BurnSpecSpeccybros = {
	"spec_speccybros", NULL, "spec_spectrum", NULL, "2012",
	"Speccy Bros (48K) (HB)\0", NULL, "Climacus", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecSpeccybrosRomInfo, SpecSpeccybrosRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Speccy Quiz (English) (128K) (HB)

static struct BurnRomInfo SpecSpeccyqenRomDesc[] = {
	{ "Speccy Quiz 128K EN (2023)(Amebatron Software).tap", 44664, 0xef3cef0e, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecSpeccyqen, SpecSpeccyqen, Spec128)
STD_ROM_FN(SpecSpeccyqen)

struct BurnDriver BurnSpecSpeccyqen = {
	"spec_speccyqen", NULL, "spec_spec128", NULL, "2023",
	"Speccy Quiz (English) (128K) (HB)\0", NULL, "Amebatron Software", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PLATFORM | GBF_QUIZ, 0,
	SpectrumGetZipName, SpecSpeccyqenRomInfo, SpecSpeccyqenRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Speccy Quiz (Spanish) (128K) (HB)

static struct BurnRomInfo SpecSpeccyqesRomDesc[] = {
	{ "Speccy Quiz 128K ES (2022)(Amebatron Software).tap", 44628, 0x9c3eb564, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecSpeccyqes, SpecSpeccyqes, Spec128)
STD_ROM_FN(SpecSpeccyqes)

struct BurnDriver BurnSpecSpeccyqes = {
	"spec_speccyqes", "spec_speccyqen", "spec_spec128", NULL, "2022",
	"Speccy Quiz (Spanish) (128K) (HB)\0", NULL, "Amebatron Software", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PLATFORM | GBF_QUIZ, 0,
	SpectrumGetZipName, SpecSpeccyqesRomInfo, SpecSpeccyqesRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Speccy Soccer (128K) (HB)

static struct BurnRomInfo SpecSpeccysoccerRomDesc[] = {
	{ "Speccy Soccer 128K (2023)(Voxel Tower).tap", 106047, 0xa8eb8027, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecSpeccysoccer, SpecSpeccysoccer, Spec128)
STD_ROM_FN(SpecSpeccysoccer)

struct BurnDriver BurnSpecSpeccysoccer = {
	"spec_speccysoccer", NULL, "spec_spec128", NULL, "2023",
	"Speccy Soccer (128K) (HB)\0", NULL, "Voxel Tower", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 2, HARDWARE_SPECTRUM, GBF_SPORTSFOOTBALL, 0,
	SpectrumGetZipName, SpecSpeccysoccerRomInfo, SpecSpeccysoccerRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecIntf2DIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Speccies, The (48K) (HB, v1.2)

static struct BurnRomInfo SpecSpecciesRomDesc[] = {
	{ "Speccies, The v1.2 (2013)(Tardis Remakes).tap", 44236, 0x7b5ffc84, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecSpeccies, SpecSpeccies, Spectrum)
STD_ROM_FN(SpecSpeccies)

struct BurnDriver BurnSpecSpeccies = {
	"spec_speccies", NULL, "spec_spectrum", NULL, "2013",
	"Speccies, The (48K) (HB, v1.2)\0", NULL, "Tardis Remakes", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PUZZLE, 0,
	SpectrumGetZipName, SpecSpecciesRomInfo, SpecSpecciesRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecQAOPSpaceDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Speccies 2, The (16K) (HB, v1.1)

static struct BurnRomInfo SpecSpeccies2RomDesc[] = {
	{ "Speccies 2, The v1.1 (2013)(Tardis Remakes).tap", 15678, 0xeb239bb9, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecSpeccies2, SpecSpeccies2, Spectrum)
STD_ROM_FN(SpecSpeccies2)

struct BurnDriver BurnSpecSpeccies2 = {
	"spec_speccies2", NULL, "spec_spectrum", NULL, "2013",
	"Speccies 2, The (16K) (HB, v1.1)\0", NULL, "Tardis Remakes", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PUZZLE, 0,
	SpectrumGetZipName, SpecSpeccies2RomInfo, SpecSpeccies2RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecQAOPSpaceDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Spekku-man (48K) (HB)

static struct BurnRomInfo SpecSpekkumanRomDesc[] = {
	{ "Spekku-man 48K (2022)(Grongy, Art-top, AER).tap", 15062, 0x03510436, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecSpekkuman, SpecSpekkuman, Spectrum)
STD_ROM_FN(SpecSpekkuman)

struct BurnDriver BurnSpecSpekkuman = {
	"spec_spekkuman", NULL, "spec_spectrum", NULL, "2022",
	"Spekku-man (48K) (HB)\0", NULL, "Grongy, Art-top, AER", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_MAZE | GBF_ACTION, 0,
	SpectrumGetZipName, SpecSpekkumanRomInfo, SpecSpekkumanRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Spider Mami (Spanish) (128K) (HB)

static struct BurnRomInfo SpecSpidmamiRomDesc[] = {
	{ "Spider Mami ES 128K (2020)(Rafael Vico).tap", 46127, 0x05cc082e, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecSpidmami, SpecSpidmami, Spec128)
STD_ROM_FN(SpecSpidmami)

struct BurnDriver BurnSpecSpidmami = {
	"spec_spidmami", NULL, "spec_spec128", NULL, "2020",
	"Spider Mami (Spanish) (128K) (HB)\0", NULL, "Rafael Vico", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_ACTION, 0,
	SpectrumGetZipName, SpecSpidmamiRomInfo, SpecSpidmamiRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Spiker - Basic Training (128K) (HB)

static struct BurnRomInfo SpecSpikerbtRomDesc[] = {
	{ "Spiker - Basic Training 128K (2013)(Black Jet).tap", 45942, 0xc1190c90, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecSpikerbt, SpecSpikerbt, Spec128)
STD_ROM_FN(SpecSpikerbt)

struct BurnDriver BurnSpecSpikerbt = {
	"spec_spikerbt", NULL, "spec_spec128", NULL, "2013",
	"Spiker - Basic Training (128K) (HB)\0", NULL, "Black Jet", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_HORSHOOT, 0,
	SpectrumGetZipName, SpecSpikerbtRomInfo, SpecSpikerbtRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecIntf2DIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Sprouty (128K) (HB)

static struct BurnRomInfo SpecSproutyRomDesc[] = {
	{ "Sprouty 128K (2019)(Stonechat Productions).tap", 47357, 0x1fca6f57, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecSprouty, SpecSprouty, Spec128)
STD_ROM_FN(SpecSprouty)

struct BurnDriver BurnSpecSprouty = {
	"spec_sprouty", NULL, "spec_spec128", NULL, "2019",
	"Sprouty (128K) (HB)\0", NULL, "Stonechat Productions", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecSproutyRomInfo, SpecSproutyRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecQAOPSpaceDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// SQIJ'D (128K) (HB)

static struct BurnRomInfo SpecSqijdRomDesc[] = {
	{ "SQIJ'D 128K (2018)(Monument Microgames).tap", 42943, 0x494495e8, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecSqijd, SpecSqijd, Spec128)
STD_ROM_FN(SpecSqijd)

struct BurnDriver BurnSpecSqijd = {
	"spec_sqijd", NULL, "spec_spec128", NULL, "2018",
	"SQIJ'D (128K) (HB)\0", NULL, "Monument Microgames", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_ACTION | GBF_SHOOT, 0,
	SpectrumGetZipName, SpecSqijdRomInfo, SpecSqijdRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// SQIJ! ZX (48K) (HB)

static struct BurnRomInfo SpecSqijzxRomDesc[] = {
	{ "Sqij ZX (2018)(Psytronik Software).tap", 46020, 0xd0af1a22, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecSqijzx, SpecSqijzx, Spectrum)
STD_ROM_FN(SpecSqijzx)

struct BurnDriver BurnSpecSqijzx = {
	"spec_sqijzx", NULL, "spec_spectrum", NULL, "2018",
	"SQIJ! ZX (48K) (HB)\0", NULL, "Tardis Remakes", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_ACTION | GBF_SHOOT, 0,
	SpectrumGetZipName, SpecSqijzxRomInfo, SpecSqijzxRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// STARS - Gumi (128K) (HB)

static struct BurnRomInfo SpecStarsgumiRomDesc[] = {
	{ "STARS - Gumi 128K (2015)(kas29).tap", 23775, 0xeb39a1df, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecStarsgumi, SpecStarsgumi, Spec128)
STD_ROM_FN(SpecStarsgumi)

struct BurnDriver BurnSpecStarsgumi = {
	"spec_starsgumi", NULL, "spec_spec128", NULL, "2015",
	"STARS - Gumi (128K) (HB)\0", NULL, "kas29", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_ACTION, 0,
	SpectrumGetZipName, SpecStarsgumiRomInfo, SpecStarsgumiRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// STARS - Gumi 2 (128K) (HB)

static struct BurnRomInfo SpecStarsgumi2RomDesc[] = {
	{ "Stars - Gumi 2 (2022)(kas29).tap", 41242, 0x64e2c5f5, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecStarsgumi2, SpecStarsgumi2, Spec128)
STD_ROM_FN(SpecStarsgumi2)

struct BurnDriver BurnSpecStarsgumi2 = {
	"spec_starsgumi2", NULL, "spec_spec128", NULL, "2022",
	"STARS - Gumi 2 (128K) (HB)\0", NULL, "kas29", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecStarsgumi2RomInfo, SpecStarsgumi2RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecQAOPSpaceDIPInfo,
	Spec128KPlus2Init, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Star Warrior (48K) (HB, v2.0)

static struct BurnRomInfo SpecStarwarr2RomDesc[] = {
	{ "Star Warrior v2.0 48K (2021)(Cronosoft).tap", 40690, 0xb8921b6f, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecStarwarr2, SpecStarwarr2, Spectrum)
STD_ROM_FN(SpecStarwarr2)

struct BurnDriver BurnSpecStarwarr2 = {
	"spec_starwarr2", NULL, "spec_spectrum", NULL, "2021",
	"Star Warrior (48K) (HB, v2.0)\0", NULL, "Cronosoft", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 2, HARDWARE_SPECTRUM, GBF_SHOOT, 0,
	SpectrumGetZipName, SpecStarwarr2RomInfo, SpecStarwarr2RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecIntf2DIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Stercore (48K) (HB)

static struct BurnRomInfo SpecStercoreRomDesc[] = {
	{ "Stercore 48K (2018)(C64CD Studios).tap", 24174, 0x8b8a630b, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecStercore, SpecStercore, Spectrum)
STD_ROM_FN(SpecStercore)

struct BurnDriver BurnSpecStercore = {
	"spec_stercore", NULL, "spec_spectrum", NULL, "2018",
	"Stercore (48K) (HB)\0", NULL, "C64CD Studios", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_HORSHOOT, 0,
	SpectrumGetZipName, SpecStercoreRomInfo, SpecStercoreRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Stop Virus (128K) (HB)

static struct BurnRomInfo SpecStopvirusRomDesc[] = {
	{ "Stop Virus (2021)(PC NONO Games).tap", 41987, 0x2dd657f9, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecStopvirus, SpecStopvirus, Spec128)
STD_ROM_FN(SpecStopvirus)

struct BurnDriver BurnSpecStopvirus = {
	"spec_stopvirus", NULL, "spec_spec128", NULL, "2021",
	"Stop Virus (128K) (HB)\0", NULL, "PC NONO Games", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_BREAKOUT, 0,
	SpectrumGetZipName, SpecStopvirusRomInfo, SpecStopvirusRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Stop Virus (Axel F bgm) (128K) (HB)

static struct BurnRomInfo SpecStopvirusafRomDesc[] = {
	{ "Stop Virus (Axel F bgm) (2021)(PC NONO Games).tap", 41520, 0xc746e35d, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecStopvirusaf, SpecStopvirusaf, Spec128)
STD_ROM_FN(SpecStopvirusaf)

struct BurnDriver BurnSpecStopvirusaf = {
	"spec_stopvirusaf", "spec_stopvirus", "spec_spec128", NULL, "2021",
	"Stop Virus (Axel F bgm) (128K) (HB)\0", NULL, "PC NONO Games", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_BREAKOUT, 0,
	SpectrumGetZipName, SpecStopvirusafRomInfo, SpecStopvirusafRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Storm Over Valhalla (48K) (HB, v1.2)

static struct BurnRomInfo SpecStvalhallaRomDesc[] = {
	{ "Storm Over Valhalla v1.2 48K (2023)(Sequentia Soft).tap", 64600, 0xbc4731e6, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecStvalhalla, SpecStvalhalla, Spectrum)
STD_ROM_FN(SpecStvalhalla)

struct BurnDriver BurnSpecStvalhalla = {
	"spec_stvalhalla", NULL, "spec_spectrum", NULL, "2023",
	"Storm Over Valhalla (48K) (HB, v1.2)\0", NULL, "Sequentia Soft", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecStvalhallaRomInfo, SpecStvalhallaRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Storm Over Valhalla Epilogue (English) (48K) (HB, v1.2)

static struct BurnRomInfo SpecStvalhallaepeRomDesc[] = {
	{ "Storm Over Valhalla - Epilogue v1.2 EN (2023)(Sequentia Soft).tap", 25905, 0xc6ff8fdb, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecStvalhallaepe, SpecStvalhallaepe, Spectrum)
STD_ROM_FN(SpecStvalhallaepe)

struct BurnDriver BurnSpecStvalhallaepe = {
	"spec_stvalhallaepe", "spec_stvalhalla", "spec_spectrum", NULL, "2023",
	"Storm Over Valhalla Epilogue (English) (48K) (HB, v1.2)\0", NULL, "Sequentia Soft", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecStvalhallaepeRomInfo, SpecStvalhallaepeRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Storm Over Valhalla Epilogue (Spanish) (48K) (HB, v1.2)

static struct BurnRomInfo SpecStvalhallaepsRomDesc[] = {
	{ "Storm Over Valhalla - Epilogue v1.2 ES (2023)(Sequentia Soft).tap", 26127, 0x994943ed, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecStvalhallaeps, SpecStvalhallaeps, Spectrum)
STD_ROM_FN(SpecStvalhallaeps)

struct BurnDriver BurnSpecStvalhallaeps = {
	"spec_stvalhallaeps", "spec_stvalhalla", "spec_spectrum", NULL, "2023",
	"Storm Over Valhalla Epilogue (Spanish) (48K) (HB, v1.2)\0", NULL, "Sequentia Soft", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecStvalhallaepsRomInfo, SpecStvalhallaepsRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Submarine! Remake (48K) (HB, v1.5)

static struct BurnRomInfo SpecSubmarmkRomDesc[] = {
	{ "Submarine! Remake v1.5 (2022)(je7ebel).tap", 47428, 0xd1b19ffc, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecSubmarmk, SpecSubmarmk, Spectrum)
STD_ROM_FN(SpecSubmarmk)

struct BurnDriver BurnSpecSubmarmk = {
	"spec_submarmk", NULL, "spec_spectrum", NULL, "2022",
	"Submarine! Remake (48K) (HB, v1.5)\0", "B: scuba diver - S: sonar", "je7ebel", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_ACTION, 0,
	SpectrumGetZipName, SpecSubmarmkRomInfo, SpecSubmarmkRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecIntf2DIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Sudoku NY (48K) (HB)

static struct BurnRomInfo SpecSudokunyRomDesc[] = {
	{ "Sudoku NY 48K-AY (2023)(g0blinish).tap", 10599, 0x2d065d7e, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecSudokuny, SpecSudokuny, Spectrum)
STD_ROM_FN(SpecSudokuny)

struct BurnDriver BurnSpecSudokuny = {
	"spec_sudokuny", NULL, "spec_spectrum", NULL, "2023",
	"Sudoku NY (48K) (HB)\0", "AY Sound supported", "g0blinish", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PUZZLE, 0,
	SpectrumGetZipName, SpecSudokunyRomInfo, SpecSudokunyRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecCursorKeysDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Summer Santa 2022 (128K) (HB)

static struct BurnRomInfo SpecSummersantaRomDesc[] = {
	{ "Summer Santa 2022 128K (2022)(Paul Jenkinson).tap", 39765, 0xf44f3e0c, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecSummersanta, SpecSummersanta, Spec128)
STD_ROM_FN(SpecSummersanta)

struct BurnDriver BurnSpecSummersanta = {
	"spec_summersanta", NULL, "spec_spec128", NULL, "2022",
	"Summer Santa 2022 (128K) (HB)\0", NULL, "Paul Jenkinson", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecSummersantaRomInfo, SpecSummersantaRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Super 48K Bro. (48K) (HB)

static struct BurnRomInfo SpecSup48kbroRomDesc[] = {
	{ "Super 48k Bro. (2024)(Rothers).tap", 47966, 0xc378e0a2, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecSup48kbro, SpecSup48kbro, Spectrum)
STD_ROM_FN(SpecSup48kbro)

struct BurnDriver BurnSpecSup48kbro = {
	"spec_sup48kbro", NULL, "spec_spectrum", NULL, "2024",
	"Super 48K Bro. (48K) (HB)\0", NULL, "Rothers", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecSup48kbroRomInfo, SpecSup48kbroRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecQAOPMDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Super Alien Blast (48K) (HB)

static struct BurnRomInfo SpecSupalienblRomDesc[] = {
	{ "Super Alien Blast 48K (2021)(Vintage Software Systems).z80", 26442, 0xa4b43504, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecSupalienbl, SpecSupalienbl, Spectrum)
STD_ROM_FN(SpecSupalienbl)

struct BurnDriver BurnSpecSupalienbl = {
	"spec_supalienbl", NULL, "spec_spectrum", NULL, "2021",
	"Super Alien Blast (48K) (HB)\0", NULL, "Vintage Software Systems", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 2, HARDWARE_SPECTRUM, GBF_SHOOT, 0,
	SpectrumGetZipName, SpecSupalienblRomInfo, SpecSupalienblRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Super Bomberman 2 Remix (128K) (HB, v1.1)

static struct BurnRomInfo SpecSbman2rRomDesc[] = {
	{ "Super Bomberman 2 Remix v1.1 128K (2023)(ZX Evolution).tap", 55559, 0x2343539f, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecSbman2r, SpecSbman2r, Spec128)
STD_ROM_FN(SpecSbman2r)

struct BurnDriver BurnSpecSbman2r = {
	"spec_sbman2r", NULL, "spec_spec128", NULL, "2023",
	"Super Bomberman 2 Remix (128K) (HB, v1.1)\0", NULL, "ZX Evolution", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 2, HARDWARE_SPECTRUM, GBF_ACTION | GBF_MAZE, 0,
	SpectrumGetZipName, SpecSbman2rRomInfo, SpecSbman2rRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// SuperHair (128K) (HB)

static struct BurnRomInfo SpecSuperhair128RomDesc[] = {
	{ "SuperHair 128K (2021)(Bitmap Soft).tap", 65034, 0x252c0023, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecSuperhair128, SpecSuperhair128, Spec128)
STD_ROM_FN(SpecSuperhair128)

struct BurnDriver BurnSpecSuperhair128 = {
	"spec_superhair128", NULL, "spec_spec128", NULL, "2021",
	"SuperHair (128K) (HB)\0", NULL, "Bitmap Soft", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_HORSHOOT, 0,
	SpectrumGetZipName, SpecSuperhair128RomInfo, SpecSuperhair128RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecIntf2DIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// SuperHair (48K) (HB)

static struct BurnRomInfo SpecSuperhair48RomDesc[] = {
	{ "SuperHair 48K (2021)(Bitmap Soft).tap", 48216, 0x96c8b90a, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecSuperhair48, SpecSuperhair48, Spectrum)
STD_ROM_FN(SpecSuperhair48)

struct BurnDriver BurnSpecSuperhair48 = {
	"spec_superhair48", "spec_superhair128", "spec_spectrum", NULL, "2021",
	"SuperHair (48K) (HB)\0", NULL, "Bitmap Soft", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_HORSHOOT, 0,
	SpectrumGetZipName, SpecSuperhair48RomInfo, SpecSuperhair48RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecIntf2DIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Super Meteor Blast (48K) (HB)

static struct BurnRomInfo SpecSupmeteorblRomDesc[] = {
	{ "Super Meteor Blast 48K (2021)(Vintage Software Systems).z80", 26792, 0x9198c2de, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecSupmeteorbl, SpecSupmeteorbl, Spectrum)
STD_ROM_FN(SpecSupmeteorbl)

struct BurnDriver BurnSpecSupmeteorbl = {
	"spec_supmeteorbl", NULL, "spec_spectrum", NULL, "2021",
	"Super Meteor Blast (48K) (HB)\0", NULL, "Vintage Software Systems", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 2, HARDWARE_SPECTRUM, GBF_SHOOT, 0,
	SpectrumGetZipName, SpecSupmeteorblRomInfo, SpecSupmeteorblRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Super Moritz (128K) (HB)

static struct BurnRomInfo SpecSupmoritzRomDesc[] = {
	{ "Super Moritz 128K (2018)(Sebastian Braunert).tap", 51661, 0x62e77ef9, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecSupmoritz, SpecSupmoritz, Spec128)
STD_ROM_FN(SpecSupmoritz)

struct BurnDriver BurnSpecSupmoritz = {
	"spec_supmoritz", NULL, "spec_spec128", NULL, "2018",
	"Super Moritz (128K) (HB)\0", NULL, "Sebastian Braunert", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_SHOOT, 0,
	SpectrumGetZipName, SpecSupmoritzRomInfo, SpecSupmoritzRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecQAOPSpaceDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Super Moritz (128K) (HB, v2)

static struct BurnRomInfo SpecSupmoritzv2RomDesc[] = {
	{ "Super Moritz 128K updated version (2020)(Sebastian Braunert).tap", 70428, 0x6380316b, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecSupmoritzv2, SpecSupmoritzv2, Spec128)
STD_ROM_FN(SpecSupmoritzv2)

struct BurnDriver BurnSpecSupmoritzv2 = {
	"spec_supmoritzv2", "spec_supmoritz", "spec_spec128", NULL, "2020",
	"Super Moritz (128K) (HB, v2)\0", NULL, "Sebastian Braunert", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_SHOOT, 0,
	SpectrumGetZipName, SpecSupmoritzv2RomInfo, SpecSupmoritzv2RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecQAOPSpaceDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Swarm is Coming, The - Mission 1 (128K) (HB)

static struct BurnRomInfo SpecSwarmm1RomDesc[] = {
	{ "Swarm is Coming, The - Mission 1 128K (2021)(Minilop).tap", 45725, 0xe0b98f2b, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecSwarmm1, SpecSwarmm1, Spec128)
STD_ROM_FN(SpecSwarmm1)

struct BurnDriver BurnSpecSwarmm1 = {
	"spec_swarmm1", "spec_swarmtr", "spec_spec128", NULL, "2021",
	"Swarm is Coming, The - Mission 1 (128K) (HB)\0", "Use keys 1 and 2 to select weapon, 'I' to MAT-ZX Menu (if available)", "Minilop", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_RUNGUN | GBF_MAZE, 0,
	SpectrumGetZipName, SpecSwarmm1RomInfo, SpecSwarmm1RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Swarm is Coming, The - Mission 2 (128K) (HB)

static struct BurnRomInfo SpecSwarmm2RomDesc[] = {
	{ "Swarm is Coming, The - Mission 2 128K (2021)(Minilop).tap", 45768, 0x22d7278a, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecSwarmm2, SpecSwarmm2, Spec128)
STD_ROM_FN(SpecSwarmm2)

struct BurnDriver BurnSpecSwarmm2 = {
	"spec_swarmm2", "spec_swarmtr", "spec_spec128", NULL, "2021",
	"Swarm is Coming, The - Mission 2 (128K) (HB)\0", "Use keys 1 and 2 to select weapon, 'I' to MAT-ZX Menu (if available)", "Minilop", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_RUNGUN | GBF_MAZE, 0,
	SpectrumGetZipName, SpecSwarmm2RomInfo, SpecSwarmm2RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Swarm is Coming, The - Mission 3 (128K) (HB)

static struct BurnRomInfo SpecSwarmm3RomDesc[] = {
	{ "Swarm is Coming, The - Mission 3 128K (2021)(Minilop).tap", 43623, 0xc4be938d, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecSwarmm3, SpecSwarmm3, Spec128)
STD_ROM_FN(SpecSwarmm3)

struct BurnDriver BurnSpecSwarmm3 = {
	"spec_swarmm3", "spec_swarmtr", "spec_spec128", NULL, "2021",
	"Swarm is Coming, The - Mission 3 (128K) (HB)\0", "Use keys 1 and 2 to select weapon, 'I' to MAT-ZX Menu (if available)", "Minilop", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_RUNGUN | GBF_MAZE, 0,
	SpectrumGetZipName, SpecSwarmm3RomInfo, SpecSwarmm3RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Swarm is Coming, The - Mission 4 (128K) (HB)

static struct BurnRomInfo SpecSwarmm4RomDesc[] = {
	{ "Swarm is Coming, The - Mission 4 128K (2021)(Minilop).tap", 44572, 0xd6e6c041, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecSwarmm4, SpecSwarmm4, Spec128)
STD_ROM_FN(SpecSwarmm4)

struct BurnDriver BurnSpecSwarmm4 = {
	"spec_swarmm4", "spec_swarmtr", "spec_spec128", NULL, "2021",
	"Swarm is Coming, The - Mission 4 (128K) (HB)\0", "Use keys 1 and 2 to select weapon, 'I' to MAT-ZX Menu (if available)", "Minilop", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_RUNGUN | GBF_MAZE, 0,
	SpectrumGetZipName, SpecSwarmm4RomInfo, SpecSwarmm4RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Swarm is Coming, The - Training (128K) (HB)

static struct BurnRomInfo SpecSwarmtrRomDesc[] = {
	{ "Swarm is Coming, The - Training 128K (2021)(Minilop).tap", 39136, 0x38bafd6a, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecSwarmtr, SpecSwarmtr, Spec128)
STD_ROM_FN(SpecSwarmtr)

struct BurnDriver BurnSpecSwarmtr = {
	"spec_swarmtr", NULL, "spec_spec128", NULL, "2021",
	"Swarm is Coming, The - Training (128K) (HB)\0", "Use keys 1 and 2 to select weapon, 'I' to MAT-ZX Menu (if available)", "Minilop", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_RUNGUN | GBF_MAZE, 0,
	SpectrumGetZipName, SpecSwarmtrRomInfo, SpecSwarmtrRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// T.C.Q. ZX (128K) (HB)

static struct BurnRomInfo SpecTcqzxRomDesc[] = {
	{ "T.C.Q. ZX 128K (2022)(Amaweks).tap", 36372, 0xf74fa56e, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecTcqzx, SpecTcqzx, Spec128)
STD_ROM_FN(SpecTcqzx)

struct BurnDriver BurnSpecTcqzx = {
	"spec_tcqzx", NULL, "spec_spec128", NULL, "2022",
	"T.C.Q. ZX (128K) (HB)\0", NULL, "Amaweks", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PLATFORM | GBF_PUZZLE, 0,
	SpectrumGetZipName, SpecTcqzxRomInfo, SpecTcqzxRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Tailwind (128K) (HB, Tech-Demo)

static struct BurnRomInfo SpecTailwindRomDesc[] = {
	{ "Tailwind 128K (2016)(Gemba Boys).tap", 73481, 0x639c8f75, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecTailwind, SpecTailwind, Spec128)
STD_ROM_FN(SpecTailwind)

struct BurnDriver BurnSpecTailwind = {
	"spec_tailwind", NULL, "spec_spec128", NULL, "2016",
	"Tailwind (128K) (HB, Tech-Demo)\0", NULL, "Gemba Boys", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW | BDF_DEMO, 1, HARDWARE_SPECTRUM, GBF_MISC, 0,
	SpectrumGetZipName, SpecTailwindRomInfo, SpecTailwindRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Tales of Grupp, The (Spanish) (48K) (HB)

static struct BurnRomInfo SpecTalesgruppRomDesc[] = {
	{ "Tales of Grupp, The (2015)(Retrobytes Productions).tap", 47718, 0xefbe14c3, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecTalesgrupp, SpecTalesgrupp, Spectrum)
STD_ROM_FN(SpecTalesgrupp)

struct BurnDriver BurnSpecTalesgrupp = {
	"spec_talesgrupp", NULL, "spec_spectrum", NULL, "2015",
	"Tales of Grupp, The (Spanish) (48K) (HB)\0", NULL, "Retrobytes Productions", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_ADV, 0,
	SpectrumGetZipName, SpecTalesgruppRomInfo, SpecTalesgruppRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecQAOPSpaceDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Tapeworm (48K)

static struct BurnRomInfo SpecTapewormRomDesc[] = {
	{ "Tapeworm 48K (2023)(ZXKerl).tap", 17206, 0x55d0b8a6, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecTapeworm, SpecTapeworm, Spectrum)
STD_ROM_FN(SpecTapeworm)

struct BurnDriver BurnSpecTapeworm = {
	"spec_tapeworm", NULL, "spec_spectrum", NULL, "2023",
	"Tapeworm (48K) (HB)\0", NULL, "ZXKerl", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_ACTION, 0,
	SpectrumGetZipName, SpecTapewormRomInfo, SpecTapewormRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Tenebra (48K) (HB)

static struct BurnRomInfo SpecTenebraRomDesc[] = {
	{ "Tenebra 48K (2022)(Haplo).tap", 13084, 0x2f49b790, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecTenebra, SpecTenebra, Spectrum)
STD_ROM_FN(SpecTenebra)

struct BurnDriver BurnSpecTenebra = {
	"spec_tenebra", NULL, "spec_spectrum", NULL, "2022",
	"Tenebra (48K) (HB)\0", NULL, "Haplo", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_MAZE, 0,
	SpectrumGetZipName, SpecTenebraRomInfo, SpecTenebraRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Tenebra 2 (48K) (HB)

static struct BurnRomInfo SpecTenebra2RomDesc[] = {
	{ "Tenebra 2 48K (2023)(Haplo).tap", 21310, 0x150edd15, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecTenebra2, SpecTenebra2, Spectrum)
STD_ROM_FN(SpecTenebra2)

struct BurnDriver BurnSpecTenebra2 = {
	"spec_tenebra2", NULL, "spec_spectrum", NULL, "2023",
	"Tenebra 2 (48K) (HB)\0", NULL, "Haplo", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_MAZE, 0,
	SpectrumGetZipName, SpecTenebra2RomInfo, SpecTenebra2RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Tenebra Macabre (48K) (HB)

static struct BurnRomInfo SpectmacabreRomDesc[] = {
	{ "Tenebra Macabre 48K (2013)(The Mojon Twins).tap", 39490, 0xbfa58b2c, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Spectmacabre, Spectmacabre, Spectrum)
STD_ROM_FN(Spectmacabre)

struct BurnDriver BurnSpectmacabre = {
	"spec_tmacabre", NULL, "spec_spectrum", NULL, "2013",
	"Tenebra Macabre (48K) (HB)\0", NULL, "The Mojon Twins", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PLATFORM | GBF_PUZZLE, 0,
	SpectrumGetZipName, SpectmacabreRomInfo, SpectmacabreRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Teodoro Can't Fly (English) (48K) (HB)

static struct BurnRomInfo SpecTeodoroenRomDesc[] = {
	{ "Teodoro Can't Fly EN 48K (2010)(RetroWorks).tap", 45440, 0x39826d34, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecTeodoroen, SpecTeodoroen, Spectrum)
STD_ROM_FN(SpecTeodoroen)

struct BurnDriver BurnSpecTeodoroen = {
	"spec_teodoroen", NULL, "spec_spectrum", NULL, "2010",
	"Teodoro Can't Fly (English) (48K) (HB)\0", NULL, "RetroWorks", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_ADV, 0,
	SpectrumGetZipName, SpecTeodoroenRomInfo, SpecTeodoroenRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecQAOPSpaceDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Teodoro No Sabe Volar (48K) (HB)

static struct BurnRomInfo SpecTeodoroesRomDesc[] = {
	{ "Teodoro No Sabe Volar ES 48K (2010)(RetroWorks).tap", 45436, 0x20ea68ea, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecTeodoroes, SpecTeodoroes, Spectrum)
STD_ROM_FN(SpecTeodoroes)

struct BurnDriver BurnSpecTeodoroes = {
	"spec_teodoroes", "spec_teodoroen", "spec_spectrum", NULL, "2010",
	"Teodoro No Sabe Volar (Spanish) (48K) (HB)\0", NULL, "RetroWorks", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_ADV, 0,
	SpectrumGetZipName, SpecTeodoroesRomInfo, SpecTeodoroesRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecQAOPSpaceDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Teodoro No Sabe Volar (Russian) (48K) (HB)

static struct BurnRomInfo SpecTeodororuRomDesc[] = {
	{ "Teodoro No Sabe Volar RU 48K (2010)(RetroWorks).tap", 45440, 0xd4b96f36, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecTeodororu, SpecTeodororu, Spectrum)
STD_ROM_FN(SpecTeodororu)

struct BurnDriver BurnSpecTeodororu = {
	"spec_teodororu", "spec_teodoroen", "spec_spectrum", NULL, "2010",
	"Teodoro No Sabe Volar (Russian) (48K) (HB)\0", NULL, "RetroWorks", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_ADV, 0,
	SpectrumGetZipName, SpecTeodororuRomInfo, SpecTeodororuRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecQAOPSpaceDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Terrapins ZX (English) (128K) (HB)

static struct BurnRomInfo SpecTerrapinsenRomDesc[] = {
	{ "Terrapins ZX EN 128K (2017)(Highrise).tap", 48853, 0xb90d0afa, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecTerrapinsen, SpecTerrapinsen, Spec1282a)
STD_ROM_FN(SpecTerrapinsen)

struct BurnDriver BurnSpecTerrapinsen = {
	"spec_terrapinsen", NULL, "spec_spec1282a", NULL, "2017",
	"Terrapins ZX (English) (128K) (HB)\0", NULL, "highriser", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_ACTION | GBF_MAZE, 0,
	SpectrumGetZipName, SpecTerrapinsenRomInfo, SpecTerrapinsenRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KPlus2Init, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Terrapins ZX (Spanish) (128K) (HB)

static struct BurnRomInfo SpecTerrapinsesRomDesc[] = {
	{ "Terrapins ZX ES 128K (2017)(Highrise).tap", 48854, 0x16a8bd5f, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecTerrapinses, SpecTerrapinses, Spec1282a)
STD_ROM_FN(SpecTerrapinses)

struct BurnDriver BurnSpecTerrapinses = {
	"spec_terrapinses", "spec_terrapinsen", "spec_spec1282a", NULL, "2017",
	"Terrapins ZX (Spanish) (128K) (HB)\0", NULL, "highriser", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_ACTION | GBF_MAZE, 0,
	SpectrumGetZipName, SpecTerrapinsesRomInfo, SpecTerrapinsesRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KPlus2Init, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Terror House (48K)

static struct BurnRomInfo SpecTerrorhouseRomDesc[] = {
	{ "Terror House 48K (2023)(Menyiques).tap", 41827, 0x8667366d, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecTerrorhouse, SpecTerrorhouse, Spectrum)
STD_ROM_FN(SpecTerrorhouse)

struct BurnDriver BurnSpecTerrorhouse = {
	"spec_terrorhouse", NULL, "spec_spectrum", NULL, "2023",
	"Terror House (48K) (HB)\0", NULL, "Menyiques", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_ACTION, 0,
	SpectrumGetZipName, SpecTerrorhouseRomInfo, SpecTerrorhouseRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecQAOPSpaceDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Tetris - Championship Edition (48K-128K) (HB, v0.8)

static struct BurnRomInfo SpecTetrisceRomDesc[] = {
	{ "Tetris - Championship Edition (2020)(TheShich).tap", 47734, 0xa03fb265, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecTetrisce, SpecTetrisce, Spec128)
STD_ROM_FN(SpecTetrisce)

struct BurnDriver BurnSpecTetrisce = {
	"spec_tetrisce", NULL, "spec_spec128", NULL, "2020",
	"Tetris - Championship Edition (48K-128K) (HB, v0.8)\0", NULL, "The Shich", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 2, HARDWARE_SPECTRUM, GBF_PUZZLE, 0,
	SpectrumGetZipName, SpecTetrisceRomInfo, SpecTetrisceRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// That Sinking Feeling (128K) (HB)

static struct BurnRomInfo SpectsfeelingRomDesc[] = {
	{ "That Sinking Feeling 128K (2017)(Sunteam).tap", 34567, 0xe7822c33, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Spectsfeeling, Spectsfeeling, Spec128)
STD_ROM_FN(Spectsfeeling)

struct BurnDriver BurnSpectsfeeling = {
	"spec_tsfeeling", NULL, "spec_spec128", NULL, "2017",
	"That Sinking Feeling (128K) (HB)\0", NULL, "Sunteam", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpectsfeelingRomInfo, SpectsfeelingRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecQAOPSpaceDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Thieves School ~ Escuela de Ladrones - All in One (English/Spanish) (128K) (HB)

static struct BurnRomInfo Spec_thievesschoolRomDesc[] = {
	{ "Thieves School - All in One 128K (2008)(World XXI Soft Inc.).tap", 339068, 0x621a5647, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Spec_thievesschool, Spec_thievesschool, Spec128)
STD_ROM_FN(Spec_thievesschool)

struct BurnDriver BurnSpec_thievesschool = {
	"spec_thievesschool", NULL, "spec_spec128", NULL, "2008-2018",
	"Thieves School ~ Escuela de Ladrones - All in One (English/Spanish) (128K) (HB)\0", "Main Menu: set options with '6', '7' and 'Enter'", "World XXI Soft Inc.", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PLATFORM | GBF_ADV, 0,
	SpectrumGetZipName, Spec_thievesschoolRomInfo, Spec_thievesschoolRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Time Sniper (128K) (HB)

static struct BurnRomInfo SpecTimesniperRomDesc[] = {
	{ "Time Sniper 128K (2023)(Nextric).tap", 44158, 0x737a235f, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecTimesniper, SpecTimesniper, Spec128)
STD_ROM_FN(SpecTimesniper)

struct BurnDriver BurnSpecTimesniper = {
	"spec_timesniper", NULL, "spec_spec128", NULL, "2023",
	"Time Sniper (128K) (HB)\0", NULL, "Nextric", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_SHOOT, 0,
	SpectrumGetZipName, SpecTimesniperRomInfo, SpecTimesniperRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Tiny Crate (English) (128K) (HB)

static struct BurnRomInfo SpecTinycratenRomDesc[] = {
	{ "Tiny Crate EN 128K (2022)(PC NONO Games).tap", 39862, 0xc1f7ff6b, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecTinycraten, SpecTinycraten, Spec128)
STD_ROM_FN(SpecTinycraten)

struct BurnDriver BurnSpecTinycraten = {
	"spec_tinycraten", NULL, "spec_spec128", NULL, "2022",
	"Tiny Crate (English) (128K) (HB)\0", NULL, "PC NONO Games", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PUZZLE, 0,
	SpectrumGetZipName, SpecTinycratenRomInfo, SpecTinycratenRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Tiny Crate (Spanish) (128K) (HB)

static struct BurnRomInfo SpecTinycratesRomDesc[] = {
	{ "Tiny Crate ES 128K (2022)(PC NONO Games).tap", 39862, 0xdcab09bc, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecTinycrates, SpecTinycrates, Spec128)
STD_ROM_FN(SpecTinycrates)

struct BurnDriver BurnSpecTinycrates = {
	"spec_tinycrates", "spec_tinycraten", "spec_spec128", NULL, "2022",
	"Tiny Crate (Spanish) (128K) (HB)\0", NULL, "PC NONO Games", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PUZZLE, 0,
	SpectrumGetZipName, SpecTinycratesRomInfo, SpecTinycratesRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Tiny Dungeons (128K) (HB)

static struct BurnRomInfo SpecTinydungsRomDesc[] = {
	{ "Tiny Dungeons 128K (2022)(RetroSouls).tap", 81973, 0xa35f23f6, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecTinydungs, SpecTinydungs, Spec128)
STD_ROM_FN(SpecTinydungs)

struct BurnDriver BurnSpecTinydungs = {
	"spec_tinydungs", NULL, "spec_spec128", NULL, "2022",
	"Tiny Dungeons (128K) (HB)\0", "Press 'Z', 'X' and 'C' to switch characters", "RetroSouls", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_ACTION | GBF_RPG, 0,
	SpectrumGetZipName, SpecTinydungsRomInfo, SpecTinydungsRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Tiny Nightmares (English) (48K) (HB)

static struct BurnRomInfo SpecTinynighteRomDesc[] = {
	{ "Tiny Nightmares EN 48K (2023)(ejvg).tap", 40151, 0x1edd7448, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecTinynighte, SpecTinynighte, Spectrum)
STD_ROM_FN(SpecTinynighte)

struct BurnDriver BurnSpecTinynighte = {
	"spec_tinynighte", NULL, "spec_spectrum", NULL, "2023",
	"Tiny Nightmares (English) (48K) (HB)\0", NULL, "EJVG", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecTinynighteRomInfo, SpecTinynighteRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Tiny Nightmares (Spanish) (48K) (HB)

static struct BurnRomInfo SpecTinynightsRomDesc[] = {
	{ "Tiny Nightmares ES 48K (2023)(ejvg).tap", 40151, 0x12d6c318, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecTinynights, SpecTinynights, Spectrum)
STD_ROM_FN(SpecTinynights)

struct BurnDriver BurnSpecTinynights = {
	"spec_tinynights", "spec_tinynighte", "spec_spectrum", NULL, "2023",
	"Tiny Nightmares (Spanish) (48K) (HB)\0", NULL, "EJVG", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecTinynightsRomInfo, SpecTinynightsRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// TJ's Cavern Capers (128K) (HB)

static struct BurnRomInfo SpecTjccRomDesc[] = {
	{ "TJ Cavern Capers 128K (2021)(Gaz Marshall).tap", 33902, 0x2749ecc9, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecTjcc, SpecTjcc, Spec128)
STD_ROM_FN(SpecTjcc)

struct BurnDriver BurnSpecTjcc = {
	"spec_tjcc", NULL, "spec_spec128", NULL, "2021",
	"TJ's Cavern Capers (128K) (HB)\0", NULL, "Gaz Marshall", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecTjccRomInfo, SpecTjccRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Tobit (128K) (HB)

static struct BurnRomInfo SpecTobitRomDesc[] = {
	{ "Tobit 128K (2023)(Rafal Miazga).tap", 47074, 0x69b7065d, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecTobit, SpecTobit, Spec128)
STD_ROM_FN(SpecTobit)

struct BurnDriver BurnSpecTobit = {
	"spec_tobit", NULL, "spec_spec128", NULL, "2023",
	"Tobit (128K) (HB)\0", NULL, "Rafal Miazga", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PUZZLE, 0,
	SpectrumGetZipName, SpecTobitRomInfo, SpecTobitRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Tokimal (128K) (HB)

static struct BurnRomInfo SpecTokimalRomDesc[] = {
	{ "Tokimal 128K (2021)(Pat Morita Team).tap", 104649, 0x40373e06, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecTokimal, SpecTokimal, Spec128)
STD_ROM_FN(SpecTokimal)

struct BurnDriver BurnSpecTokimal = {
	"spec_tokimal", NULL, "spec_spec128", NULL, "2021",
	"Tokimal (128K) (HB)\0", NULL, "Pat Morita Team", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecTokimalRomInfo, SpecTokimalRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Toofy in Fan Land (48K) (HB)

static struct BurnRomInfo SpecToofyRomDesc[] = {
	{ "Toofy in Fan Land 48K (2012)(Paul Jenkinson).tap", 35803, 0xc3588e5a, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecToofy, SpecToofy, Spectrum)
STD_ROM_FN(SpecToofy)

struct BurnDriver BurnSpecToofy = {
	"spec_toofy", NULL, "spec_spectrum", NULL, "2012",
	"Toofy in Fan Land (48K) (HB)\0", NULL, "Paul Jenkinson", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecToofyRomInfo, SpecToofyRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Toofy's Nutty Nightmare (128K) (HB)

static struct BurnRomInfo SpecToofy3RomDesc[] = {
	{ "Toofy's Nutty Nightmare 128K (2020)(Paul Jenkinson).tap", 35807, 0xec5a53fd, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecToofy3, SpecToofy3, Spec128)
STD_ROM_FN(SpecToofy3)

struct BurnDriver BurnSpecToofy3 = {
	"spec_toofy3", NULL, "spec_spec128", NULL, "2020",
	"Toofy's Nutty Nightmare (128K) (HB)\0", NULL, "Paul Jenkinson", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecToofy3RomInfo, SpecToofy3RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Toofy's Winter Nuts (48K) (HB)

static struct BurnRomInfo SpecToofy2RomDesc[] = {
	{ "Toofy's Winter Nuts 48K (2013)(Paul Jenkinson).tap", 39747, 0xaea85a8d, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecToofy2, SpecToofy2, Spectrum)
STD_ROM_FN(SpecToofy2)

struct BurnDriver BurnSpecToofy2 = {
	"spec_toofy2", NULL, "spec_spectrum", NULL, "2013",
	"Toofy's Winter Nuts (48K) (HB)\0", NULL, "Paul Jenkinson", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecToofy2RomInfo, SpecToofy2RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Tourmaline (128K) (HB)

static struct BurnRomInfo SpectrmlineRomDesc[] = {
	{ "Tourmaline 128K (2016)(RetroSouls).tap",  46613, 0xf573c97f, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Spectrmline, Spectrmline, Spec128)
STD_ROM_FN(Spectrmline)

struct BurnDriver BurnSpectrmline = {
	"spec_trmline", NULL, "spec_spec128", NULL, "2016",
	"Tourmaline (128K) (HB)\0", NULL, "RetroSouls", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_ACTION, 0,
	SpectrumGetZipName, SpectrmlineRomInfo, SpectrmlineRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Tournament Arkanoid (48K) (Hack)

static struct BurnRomInfo SpecTourarkanoidRomDesc[] = {
	{ "Tournament Arkanoid 48K (2022)(Martyn Carrol).tap", 48367, 0x3790ec49, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecTourarkanoid, SpecTourarkanoid, Spectrum)
STD_ROM_FN(SpecTourarkanoid)

struct BurnDriver BurnSpecTourarkanoid = {
	"spec_tourarkanoid", NULL, "spec_spectrum", NULL, "2022",
	"Tournament Arkanoid (48K) (Hack)\0", "A mod of the classic game 'Arkanoid'", "Martyn Carrol", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HACK, 1, HARDWARE_SPECTRUM, GBF_BREAKOUT, 0,
	SpectrumGetZipName, SpecTourarkanoidRomInfo, SpecTourarkanoidRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecIntf2SwapDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Transylvanian Castle 2 - The Eye of Velnor (English) (48K) (HB, v1.06)

static struct BurnRomInfo SpecTc2enRomDesc[] = {
	{ "TC2 - The Eye of Velnor v1.06 EN 48K (2021)(Fitosoft).tap", 46907, 0xa72cb028, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecTc2en, SpecTc2en, Spectrum)
STD_ROM_FN(SpecTc2en)

struct BurnDriver BurnSpecTc2en = {
	"spec_tc2en", NULL, "spec_spectrum", NULL, "2021",
	"Transylvanian Castle 2 - The Eye of Velnor (English) (48K) (HB, v1.06)\0", NULL, "Fitosoft", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_MAZE | GBF_RPG, 0,
	SpectrumGetZipName, SpecTc2enRomInfo, SpecTc2enRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecQAOPSpaceDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Transylvanian Castle 2 - The Eye of Velnor (Spanish) (48K) (HB, v1.06)

static struct BurnRomInfo SpecTc2esRomDesc[] = {
	{ "TC2 - The Eye of Velnor v1.06 ES 48K (2021)(Fitosoft).tap", 47034, 0xfb98d48b, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecTc2es, SpecTc2es, Spectrum)
STD_ROM_FN(SpecTc2es)

struct BurnDriver BurnSpecTc2es = {
	"spec_tc2es", "spec_tc2en", "spec_spectrum", NULL, "2021",
	"Transylvanian Castle 2 - The Eye of Velnor (Spanish) (48K) (HB, v1.06)\0", NULL, "Fitosoft", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_MAZE | GBF_RPG, 0,
	SpectrumGetZipName, SpecTc2esRomInfo, SpecTc2esRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecQAOPSpaceDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Transylvanian Castle 2 - The Eye of Velnor (Portuguese) (48K) (HB, v1.06)

static struct BurnRomInfo SpecTc2ptRomDesc[] = {
	{ "TC2 - The Eye of Velnor v1.06 PT 48K (2021)(Fitosoft).tap", 47067, 0xa47da046, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecTc2pt, SpecTc2pt, Spectrum)
STD_ROM_FN(SpecTc2pt)

struct BurnDriver BurnSpecTc2pt = {
	"spec_tc2pt", "spec_tc2en", "spec_spectrum", NULL, "2021",
	"Transylvanian Castle 2 - The Eye of Velnor (Portuguese) (48K) (HB, v1.06)\0", NULL, "Fitosoft", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_MAZE | GBF_RPG, 0,
	SpectrumGetZipName, SpecTc2ptRomInfo, SpecTc2ptRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecQAOPSpaceDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Transylvanian Castle 3 - The Queen of Sorcery (English) (48K) (HB, v1.2)

static struct BurnRomInfo SpecTc3enRomDesc[] = {
	{ "Transylvanian Castle 3 EN 48K (2022)(Fitosoft).tap", 47171, 0xf820a20b, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecTc3en, SpecTc3en, Spectrum)
STD_ROM_FN(SpecTc3en)

struct BurnDriver BurnSpecTc3en = {
	"spec_tc3en", NULL, "spec_spectrum", NULL, "2022",
	"Transylvanian Castle 3 - The Queen of Sorcery (English) (48K) (HB, v1.2)\0", NULL, "Fitosoft", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_RPG | GBF_MAZE, 0,
	SpectrumGetZipName, SpecTc3enRomInfo, SpecTc3enRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecQAOPMDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Transylvanian Castle 3 - The Queen of Sorcery (Spanish) (48K) (HB, v1.2)

static struct BurnRomInfo SpecTc3esRomDesc[] = {
	{ "Transylvanian Castle 3 ES 48K (2022)(Fitosoft).tap", 47154, 0xa73d13c7, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecTc3es, SpecTc3es, Spectrum)
STD_ROM_FN(SpecTc3es)

struct BurnDriver BurnSpecTc3es = {
	"spec_tc3es", "spec_tc3en", "spec_spectrum", NULL, "2022",
	"Transylvanian Castle 3 - The Queen of Sorcery (Spanish) (48K) (HB, v1.2)\0", NULL, "Fitosoft", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_RPG | GBF_MAZE, 0,
	SpectrumGetZipName, SpecTc3esRomInfo, SpecTc3esRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecQAOPMDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Transylvanian Castle 3 - The Queen of Sorcery (Portuguese) (48K) (HB, v1.2)

static struct BurnRomInfo SpecTc3ptRomDesc[] = {
	{ "Transylvanian Castle 3 PT 48K (2022)(Fitosoft).tap", 47171, 0xfd8f0e09, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecTc3pt, SpecTc3pt, Spectrum)
STD_ROM_FN(SpecTc3pt)

struct BurnDriver BurnSpecTc3pt = {
	"spec_tc3pt", "spec_tc3en", "spec_spectrum", NULL, "2022",
	"Transylvanian Castle 3 - The Queen of Sorcery (Portuguese) (48K) (HB, v1.2)\0", NULL, "Fitosoft", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_RPG | GBF_MAZE, 0,
	SpectrumGetZipName, SpecTc3ptRomInfo, SpecTc3ptRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecQAOPMDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Transylvanian Castle Dx2 (English) (48K) (HB, v1.1)

static struct BurnRomInfo SpecTcdx2enRomDesc[] = {
	{ "Transylvanian Castle Dx2 v1.1 EN 48K (2023)(Fitosoft).tap", 45432, 0xc52fb2e0, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecTcdx2en, SpecTcdx2en, Spectrum)
STD_ROM_FN(SpecTcdx2en)

struct BurnDriver BurnSpecTcdx2en = {
	"spec_tcdx2en", NULL, "spec_spectrum", NULL, "2020-23",
	"Transylvanian Castle Dx2 (English) (48K) (HB, v1.1)\0", NULL, "Fitosoft", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_RPG, 0,
	SpectrumGetZipName, SpecTcdx2enRomInfo, SpecTcdx2enRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecQAOPSpaceDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Transylvanian Castle Dx2 (Spanish) (48K) (HB, v1.1)

static struct BurnRomInfo SpecTcdx2esRomDesc[] = {
	{ "Transylvanian Castle Dx2 v1.1 ES 48K (2023)(Fitosoft).tap", 45562, 0x72f59e69, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecTcdx2es, SpecTcdx2es, Spectrum)
STD_ROM_FN(SpecTcdx2es)

struct BurnDriver BurnSpecTcdx2es = {
	"spec_tcdx2es", "spec_tcdx2en", "spec_spectrum", NULL, "2020-23",
	"Transylvanian Castle Dx2 (Spanish) (48K) (HB, v1.1)\0", NULL, "Fitosoft", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_RPG, 0,
	SpectrumGetZipName, SpecTcdx2esRomInfo, SpecTcdx2esRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecQAOPSpaceDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Transylvanian Castle Dx2 (Portuguese) (48K) (HB, v1.1)

static struct BurnRomInfo SpecTcdx2ptRomDesc[] = {
	{ "Transylvanian Castle Dx2 v1.1 PT 48K (2023)(Fitosoft).tap", 45567, 0x3a7d88ad, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecTcdx2pt, SpecTcdx2pt, Spectrum)
STD_ROM_FN(SpecTcdx2pt)

struct BurnDriver BurnSpecTcdx2pt = {
	"spec_tcdx2pt", "spec_tcdx2en", "spec_spectrum", NULL, "2020-23",
	"Transylvanian Castle Dx2 (Portuguese) (48K) (HB, v1.1)\0", NULL, "Fitosoft", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_RPG, 0,
	SpectrumGetZipName, SpecTcdx2ptRomInfo, SpecTcdx2ptRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecQAOPSpaceDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// TRASHMAN - Crisis Time (English) (128K) (HB)

static struct BurnRomInfo SpecTrashmanenRomDesc[] = {
	{ "TRASHMAN - Crisis Time EN 128K (2020)(PC NONO).tap", 48911, 0x0b6d2b91, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecTrashmanen, SpecTrashmanen, Spec128)
STD_ROM_FN(SpecTrashmanen)

struct BurnDriver BurnSpecTrashmanen = {
	"spec_trashmanen", NULL, "spec_spec128", NULL, "2020",
	"TRASHMAN - Crisis Time (English) (128K) (HB)\0", NULL, "PC NONO Games", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_ACTION, 0,
	SpectrumGetZipName, SpecTrashmanenRomInfo, SpecTrashmanenRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// TRASHMAN - Crisis Time (Spanish) (128K) (HB)

static struct BurnRomInfo SpecTrashmanesRomDesc[] = {
	{ "TRASHMAN - Crisis Time ES 128K (2020)(PC NONO).tap", 48913, 0x69df80ca, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecTrashmanes, SpecTrashmanes, Spec128)
STD_ROM_FN(SpecTrashmanes)

struct BurnDriver BurnSpecTrashmanes = {
	"spec_trashmanes", "spec_trashmanen", "spec_spec128", NULL, "2020",
	"TRASHMAN - Crisis Time (Spanish) (128K) (HB)\0", NULL, "PC NONO Games", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_ACTION, 0,
	SpectrumGetZipName, SpecTrashmanesRomInfo, SpecTrashmanesRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Travel Through Time Volume 1 - Northern Lights (128K) (HB)

static struct BurnRomInfo SpecTttvol1RomDesc[] = {
	{ "Travel Through Time Volume 1 Northern Lights (2021)(Zosya Ent.).tap", 126588, 0xc792257f, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecTttvol1, SpecTttvol1, Spec128)
STD_ROM_FN(SpecTttvol1)

struct BurnDriver BurnSpecTttvol1 = {
	"spec_tttvol1", NULL, "spec_spec128", NULL, "2021",
	"Travel Through Time Volume 1 - Northern Lights (128K) (HB)\0", NULL, "Zosya Entertainment", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_RACING, 0,
	SpectrumGetZipName, SpecTttvol1RomInfo, SpecTttvol1RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Tuerkas, the White Cat (48K) (HB, v1.2)

static struct BurnRomInfo SpecTuerkasRomDesc[] = {
	{ "Tuerkas, the White Cat v1.2 48K (2023)(Retrobensoft).tap", 49005, 0x5fe3b3a4, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecTuerkas, SpecTuerkas, Spectrum)
STD_ROM_FN(SpecTuerkas)

struct BurnDriver BurnSpecTuerkas = {
	"spec_tuerkas", NULL, "spec_spectrum", NULL, "2023",
	"Tuerkas, the White Cat (48K) (HB, v1.2)\0", NULL, "Retrobensoft", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecTuerkasRomInfo, SpecTuerkasRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Tumult - Phase 1 (48K) (HB)

static struct BurnRomInfo SpecTumultph1RomDesc[] = {
	{ "Tumult - Phase 1 48K (2018)(Bumfun Games).tap", 35256, 0xd080978e, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecTumultph1, SpecTumultph1, Spectrum)
STD_ROM_FN(SpecTumultph1)

struct BurnDriver BurnSpecTumultph1 = {
	"spec_tumultph1", NULL, "spec_spectrum", NULL, "2018",
	"Tumult - Phase 1 (48K) (HB)\0", NULL, "BumFun Games", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_SHOOT, 0,
	SpectrumGetZipName, SpecTumultph1RomInfo, SpecTumultph1RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Turbomania (16K) (HB)

static struct BurnRomInfo SpecTurbomaniaRomDesc[] = {
	{ "Turbomania 16K (2005)(Jonathan Cauldwell).tap", 8669, 0x2632bccf, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecTurbomania, SpecTurbomania, Spectrum)
STD_ROM_FN(SpecTurbomania)

struct BurnDriver BurnSpecTurbomania = {
	"spec_turbomania", NULL, "spec_spectrum", NULL, "2005",
	"Turbomania (16K) (HB)\0", NULL, "Jonathan Cauldwell", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_ACTION | GBF_MAZE, 0,
	SpectrumGetZipName, SpecTurbomaniaRomInfo, SpecTurbomaniaRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Turdle (48K) (HB)

static struct BurnRomInfo SpecTurdleRomDesc[] = {
	{ "Turdle 48K (2023)(Megastyle).tap", 16188, 0x582949e5, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecTurdle, SpecTurdle, Spectrum)
STD_ROM_FN(SpecTurdle)

struct BurnDriver BurnSpecTurdle = {
	"spec_turdle", NULL, "spec_spectrum", NULL, "2023",
	"Turdle (48K) (HB)\0", NULL, "Megastyle", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PUZZLE, 0,
	SpectrumGetZipName, SpecTurdleRomInfo, SpecTurdleRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Tut-Tut (48K) (HB, v1.04)

static struct BurnRomInfo SpecTuttutRomDesc[] = {
	{ "Tut-Tut v1.04 48K (2020)(Cronosoft).tap", 25895, 0xede5bd5d, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecTuttut, SpecTuttut, Spectrum)
STD_ROM_FN(SpecTuttut)

struct BurnDriver BurnSpecTuttut = {
	"spec_tuttut", NULL, "spec_spectrum", NULL, "2020",
	"Tut-Tut (48K) (HB, v1.04)\0", "Improved version of ZX81 Tut-Tut", "Cronosoft", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_MAZE, 0,
	SpectrumGetZipName, SpecTuttutRomInfo, SpecTuttutRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Twinlight (128K) (HB)

static struct BurnRomInfo SpecTwinlightRomDesc[] = {
	{ "Twinlight 128K (2017)(Retrosouls).tap", 93039, 0x8d52e8f6, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecTwinlight, SpecTwinlight, Spec128)
STD_ROM_FN(SpecTwinlight)

struct BurnDriver BurnSpecTwinlight = {
	"spec_twinlight", NULL, "spec_spec128", NULL, "2017",
	"Twinlight (128K) (HB)\0", NULL, "RetroSouls", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PUZZLE, 0,
	SpectrumGetZipName, SpecTwinlightRomInfo, SpecTwinlightRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Undersea Colossus, The (128K) (HB)

static struct BurnRomInfo SpecUndseasolRomDesc[] = {
	{ "Undersea Colossus, The 128K (2024)(TokuSoft).tap", 60058, 0x6947495c, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecUndseasol, SpecUndseasol, Spec128)
STD_ROM_FN(SpecUndseasol)

struct BurnDriver BurnSpecUndseasol = {
	"spec_undseasol", NULL, "spec_spec128", NULL, "2024",
	"Undersea Colossus, The (128K) (HB)\0", NULL, "TokuSoft", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecUndseasolRomInfo, SpecUndseasolRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Unicorn, The (48K) (HB)

static struct BurnRomInfo SpecUnicornRomDesc[] = {
	{ "Unicorn, The 48K (2023)(UriS).tap", 28858, 0xd7d33b1e, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecUnicorn, SpecUnicorn, Spectrum)
STD_ROM_FN(SpecUnicorn)

struct BurnDriver BurnSpecUnicorn = {
	"spec_unicorn", NULL, "spec_spectrum", NULL, "2023",
	"Unicorn, The 48K (HB)\0", NULL, "UriS", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PUZZLE, 0,
	SpectrumGetZipName, SpecUnicornRomInfo, SpecUnicornRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecQAOPSpaceDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Uroboros (128K) (HB)

static struct BurnRomInfo SpecUroborosRomDesc[] = {
	{ "Uroboros 128K (2015)(Rafal Miazga).tap", 20267, 0x9d288091, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecUroboros, SpecUroboros, Spec128)
STD_ROM_FN(SpecUroboros)

struct BurnDriver BurnSpecUroboros = {
	"spec_uroboros", NULL, "spec_spec128", NULL, "2015",
	"Uroboros (128K) (HB)\0", NULL, "Rafal Miazga", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_ACTION, 0,
	SpectrumGetZipName, SpecUroborosRomInfo, SpecUroborosRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Urok (English) (128K) (HB)

static struct BurnRomInfo SpecUrokenRomDesc[] = {
	{ "Urok EN 128K (2024)(Tartessos Games).tap", 81964, 0x17df69f5, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecUroken, SpecUroken, Spec128)
STD_ROM_FN(SpecUroken)

struct BurnDriver BurnSpecUroken = {
	"spec_uroken", NULL, "spec_spec128", NULL, "2024",
	"Urok (English) (128K) (HB)\0", NULL, "Tartessos Games", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecUrokenRomInfo, SpecUrokenRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Urok (Spanish) (128K) (HB)

static struct BurnRomInfo SpecUrokesRomDesc[] = {
	{ "Urok ES 128K (2024)(Tartessos Games).tap", 82115, 0xf8349f90, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecUrokes, SpecUrokes, Spec128)
STD_ROM_FN(SpecUrokes)

struct BurnDriver BurnSpecUrokes = {
	"spec_urokes", "spec_uroken", "spec_spec128", NULL, "2024",
	"Urok (Spanish) (128K) (HB)\0", NULL, "Tartessos Games", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecUrokesRomInfo, SpecUrokesRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Uwol (128K) (HB)

static struct BurnRomInfo SpecuwolRomDesc[] = {
	{ "Uwol 128K (2010)(Mojon Twins).tap", 50254, 0x4fee82ec, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specuwol, Specuwol, Spec128)
STD_ROM_FN(Specuwol)

struct BurnDriver BurnSpecuwol = {
	"spec_uwol", NULL, "spec_spec128", NULL, "2009",
	"Uwol (128K) (HB)\0", NULL, "The Mojon Twins", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecuwolRomInfo, SpecuwolRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Vallation 128K Edition (128K) (HB)

static struct BurnRomInfo SpecVallation128RomDesc[] = {
	{ "Vallation 128K (2017)(Tardis Remakes).tap", 68513, 0xa30a4662, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecVallation128, SpecVallation128, Spec128)
STD_ROM_FN(SpecVallation128)

struct BurnDriver BurnSpecVallation128 = {
	"spec_vallation128", NULL, "spec_spec128", NULL, "2017",
	"Vallation 128K Edition (128K) (HB)\0", NULL, "Tardis Remakes", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_SHOOT, 0,
	SpectrumGetZipName, SpecVallation128RomInfo, SpecVallation128RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Vallation (48K) (HB, v1.1)

static struct BurnRomInfo SpecVallationRomDesc[] = {
	{ "Vallation v1.1 48K (2016)(Tardis Remakes).tap", 41179, 0xe67312ca, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecVallation, SpecVallation, Spectrum)
STD_ROM_FN(SpecVallation)

struct BurnDriver BurnSpecVallation = {
	"spec_vallation", "spec_vallation128", "spec_spectrum", NULL, "2016",
	"Vallation (48K) (HB, v1.1)\0", NULL, "Tardis Remakes", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_SHOOT, 0,
	SpectrumGetZipName, SpecVallationRomInfo, SpecVallationRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecQAOPSpaceDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Valley of Rains (128K) (HB)

static struct BurnRomInfo SpecvalleyofrainsRomDesc[] = {
	{ "Valley of Rains 128K (2019)(Zosya Entertainment).z80", 0x0b647, 0x7a79ca09, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specvalleyofrains, Specvalleyofrains, Spec128)
STD_ROM_FN(Specvalleyofrains)

struct BurnDriver BurnSpecvalleyofrains = {
	"spec_valleyofrains", NULL, "spec_spec128", NULL, "2019",
	"Valley of Rains (128K) (HB)\0", NULL, "Zosya Entertainment", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_RUNGUN, 0,
	SpectrumGetZipName, SpecvalleyofrainsRomInfo, SpecvalleyofrainsRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Vampire Vengeance (128K) (HB, v1.1)

static struct BurnRomInfo SpecvampvengRomDesc[] = {
	{ "Vampire Vengeance v1.1 128K (2020)(Poe Games).tap", 58059, 0x2796871d, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specvampveng, Specvampveng, Spec128)
STD_ROM_FN(Specvampveng)

struct BurnDriver BurnSpecvampveng = {
	"spec_vampveng", NULL, "spec_spec128", NULL, "2020",
	"Vampire Vengeance (128K) (HB, v1.1)\0", NULL, "Poe Games", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecvampvengRomInfo, SpecvampvengRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Vectornauts, The (128K) (HB)

static struct BurnRomInfo SpecVectornautsRomDesc[] = {
	{ "Vectornauts, The 128K (2018)(Dom Reardon).tap", 43878, 0x79bd68c7, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecVectornauts, SpecVectornauts, Spec128)
STD_ROM_FN(SpecVectornauts)

struct BurnDriver BurnSpecVectornauts = {
	"spec_vectornauts", NULL, "spec_spec128", NULL, "2018",
	"Vectornauts, The (128K) (HB)\0", NULL, "Dom Reardon", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecVectornautsRomInfo, SpecVectornautsRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecQAOPSpaceDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Vegetables Deluxe (48K) (HB)

static struct BurnRomInfo SpecVegetadxRomDesc[] = {
	{ "Vegetables Deluxe 48K-AY (2021)(Mike Richmond).tap", 40589, 0x4c7d2616, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecVegetadx, SpecVegetadx, Spectrum)
STD_ROM_FN(SpecVegetadx)

struct BurnDriver BurnSpecVegetadx = {
	"spec_vegetadx", NULL, "spec_spectrum", NULL, "2021",
	"Vegetables Deluxe (48K) (HB)\0", "AY Sound supported", "Mike Richmond", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PUZZLE, 0,
	SpectrumGetZipName, SpecVegetadxRomInfo, SpecVegetadxRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Vexed (128K) (HB, v1.08)

static struct BurnRomInfo SpecVexed128RomDesc[] = {
	{ "Vexed 128K v1.08 (2023)(Under4Mhz).tap", 18657, 0x44fd70ed, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecVexed128, SpecVexed128, Spec128)
STD_ROM_FN(SpecVexed128)

struct BurnDriver BurnSpecVexed128 = {
	"spec_vexed128", NULL, "spec_spec128", NULL, "2023",
	"Vexed (128K) (HB, v1.08)\0", "'R': quit game - 'X': restart level", "Under4Mhz", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PUZZLE, 0,
	SpectrumGetZipName, SpecVexed128RomInfo, SpecVexed128RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Vexed (48K) (HB, v1.08)

static struct BurnRomInfo SpecVexed48RomDesc[] = {
	{ "Vexed 48K v1.08 (2023)(Under4Mhz).tap", 17352, 0xada67981, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecVexed48, SpecVexed48, Spectrum)
STD_ROM_FN(SpecVexed48)

struct BurnDriver BurnSpecVexed48 = {
	"spec_vexed48", "spec_vexed128", "spec_spectrum", NULL, "2023",
	"Vexed (48K) (HB, v1.08)\0", "'R': quit game - 'X': restart level", "Under4Mhz", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PUZZLE, 0,
	SpectrumGetZipName, SpecVexed48RomInfo, SpecVexed48RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Virgil's Purgatory ZX (English) (128K) (HB)

static struct BurnRomInfo SpecVirgilsenRomDesc[] = {
	{ "Virgil's Purgatory ZX 128K EN (2021)(Amaweks).tap", 45165, 0xacf1905a, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecVirgilsen, SpecVirgilsen, Spec128)
STD_ROM_FN(SpecVirgilsen)

struct BurnDriver BurnSpecVirgilsen = {
	"spec_virgilsen", NULL, "spec_spec128", NULL, "2021",
	"Virgil's Purgatory ZX (English) (128K) (HB)\0", NULL, "Amaweks", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecVirgilsenRomInfo, SpecVirgilsenRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Virgil's Purgatory ZX (Portuguese) (128K) (HB)

static struct BurnRomInfo SpecVirgilsptRomDesc[] = {
	{ "Virgil's Purgatory ZX 128K PT (2021)(Amaweks).tap", 45165, 0x0d5942d0, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecVirgilspt, SpecVirgilspt, Spec128)
STD_ROM_FN(SpecVirgilspt)

struct BurnDriver BurnSpecVirgilspt = {
	"spec_virgilspt", "spec_virgilsen", "spec_spec128", NULL, "2021",
	"Virgil's Purgatory ZX (Portuguese) (128K) (HB)\0", NULL, "Amaweks", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW | BDF_CLONE, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecVirgilsptRomInfo, SpecVirgilsptRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Virgil's Purgatory 2.0 - Chapter 1 (English) (128K) (HB)

static struct BurnRomInfo SpecVirgils2p1enRomDesc[] = {
	{ "Virgil's Purgatory 2.0 - Chapter 1 EN (2023)(Amaweks).tap", 45477, 0x82c3fd88, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecVirgils2p1en, SpecVirgils2p1en, Spec128)
STD_ROM_FN(SpecVirgils2p1en)

struct BurnDriver BurnSpecVirgils2p1en = {
	"spec_virgils2p1en", NULL, "spec_spec128", NULL, "2023",
	"Virgil's Purgatory 2.0 - Chapter 1 (English) (128K) (HB)\0", NULL, "Amaweks", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecVirgils2p1enRomInfo, SpecVirgils2p1enRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// O Purgatorio de Virgilio 2.0 - Capitulo 1 (Portuguese) (128K) (HB)

static struct BurnRomInfo SpecVirgils2p1ptRomDesc[] = {
	{ "O Purgatorio de Virgilio 2.0 - Capitulo 1 PT (2023)(Amaweks).tap", 45538, 0xe61963ba, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecVirgils2p1pt, SpecVirgils2p1pt, Spec128)
STD_ROM_FN(SpecVirgils2p1pt)

struct BurnDriver BurnSpecVirgils2p1pt = {
	"spec_virgils2p1pt", "spec_virgils2p1en", "spec_spec128", NULL, "2023",
	"O Purgatorio de Virgilio 2.0 - Capitulo 1 (Portuguese) (128K) (HB)\0", NULL, "Amaweks", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecVirgils2p1ptRomInfo, SpecVirgils2p1ptRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Virgil's Purgatory 2.0 - Chapter 2 (English) (128K) (HB)

static struct BurnRomInfo SpecVirgils2p2enRomDesc[] = {
	{ "Virgil's Purgatory 2.0 - Chapter 2 EN (2023)(Amaweks).tap", 46411, 0xd4121d1c, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecVirgils2p2en, SpecVirgils2p2en, Spec128)
STD_ROM_FN(SpecVirgils2p2en)

struct BurnDriver BurnSpecVirgils2p2en = {
	"spec_virgils2p2en", "spec_virgils2p1en", "spec_spec128", NULL, "2023",
	"Virgil's Purgatory 2.0 - Chapter 2 (English) (128K) (HB)\0", NULL, "Amaweks", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecVirgils2p2enRomInfo, SpecVirgils2p2enRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// O Purgatorio de Virgilio 2.0 - Capitulo 2 (Portuguese) (128K) (HB)

static struct BurnRomInfo SpecVirgils2p2ptRomDesc[] = {
	{ "O Purgatorio de Virgilio 2.0 - Capitulo 2 PT (2023)(Amaweks).tap", 46473, 0x5d26605e, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecVirgils2p2pt, SpecVirgils2p2pt, Spec128)
STD_ROM_FN(SpecVirgils2p2pt)

struct BurnDriver BurnSpecVirgils2p2pt = {
	"spec_virgils2p2pt", "spec_virgils2p1en", "spec_spec128", NULL, "2023",
	"O Purgatorio de Virgilio 2.0 - Capitulo 2 (Portuguese) (128K) (HB)\0", NULL, "Amaweks", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecVirgils2p2ptRomInfo, SpecVirgils2p2ptRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Vixen 'Enhanced Version' (128K) (Hack)

static struct BurnRomInfo SpecVixenevRomDesc[] = {
	{ "Vixen Enhanced Version 128K (2024)(Rafal Miazga).tap", 42263, 0x4d018fae, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecVixenev, SpecVixenev, Spec128)
STD_ROM_FN(SpecVixenev)

struct BurnDriver BurnSpecVixenev = {
	"spec_vixenev", "spec_vixen1", "spec_spec128", NULL, "2024",
	"Vixen 'Enhanced Version' (128K) (Hack)\0", NULL, "Rafal Miazga", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE | BDF_HACK, 1, HARDWARE_SPECTRUM, GBF_RUNGUN | GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecVixenevRomInfo, SpecVixenevRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Void (128K) (HB, Tech-Demo)

static struct BurnRomInfo SpecVoidRomDesc[] = {
	{ "Void 128K (2020)(Invaders Group).tap", 80143, 0xa1fdfe03, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecVoid, SpecVoid, Spec128)
STD_ROM_FN(SpecVoid)

struct BurnDriver BurnSpecVoid = {
	"spec_void", NULL, "spec_spec128", NULL, "2020",
	"Void (128K) (HB, Tech-Demo)\0", NULL, "Invaders Group", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW | BDF_DEMO, 1, HARDWARE_SPECTRUM, GBF_MISC, 0,
	SpectrumGetZipName, SpecVoidRomInfo, SpecVoidRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Void Bringer, The (English) (128K) (HB)

static struct BurnRomInfo SpecVoidbringenRomDesc[] = {
	{ "Void Bringer, The EN 128K (2022)(Furillo Productions).tap", 76461, 0x601ae424, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecVoidbringen, SpecVoidbringen, Spec128)
STD_ROM_FN(SpecVoidbringen)

struct BurnDriver BurnSpecVoidbringen = {
	"spec_voidbringen", NULL, "spec_spec128", NULL, "2022",
	"Void Bringer, The (English) (128K) (HB)\0", "'O', 'P' and 'Space' to set options", "Furillo Productions", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 2, HARDWARE_SPECTRUM, GBF_ACTION | GBF_PUZZLE, 0,
	SpectrumGetZipName, SpecVoidbringenRomInfo, SpecVoidbringenRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Void Bringer, The (Spanish) (128K) (HB)

static struct BurnRomInfo SpecVoidbringesRomDesc[] = {
	{ "Void Bringer, The ES 128K (2022)(Furillo Productions).tap", 76461, 0xfdd4d848, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecVoidbringes, SpecVoidbringes, Spec128)
STD_ROM_FN(SpecVoidbringes)

struct BurnDriver BurnSpecVoidbringes = {
	"spec_voidbringes", "spec_voidbringen", "spec_spec128", NULL, "2022",
	"Void Bringer, The (Spanish) (128K) (HB)\0", "'O', 'P' and 'Space' to set options", "Furillo Productions", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE | BDF_HOMEBREW, 2, HARDWARE_SPECTRUM, GBF_ACTION | GBF_PUZZLE, 0,
	SpectrumGetZipName, SpecVoidbringesRomInfo, SpecVoidbringesRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Vradark's Sphere (128K) (HB)

static struct BurnRomInfo SpecVradarkRomDesc[] = {
	{ "Vradark's Sphere 128K (2018)(Sanchez).tap", 74713, 0x0ec63453, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecVradark, SpecVradark, Spec128)
STD_ROM_FN(SpecVradark)

struct BurnDriver BurnSpecVradark = {
	"spec_vradark", NULL, "spec_spec128", NULL, "2018",
	"Vradark's Sphere (128K) (HB)\0", NULL, "SaNchez", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_MAZE | GBF_ADV, 0,
	SpectrumGetZipName, SpecVradarkRomInfo, SpecVradarkRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecQAOPSpaceDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Waldo's Life (English) (128K) (HB)

static struct BurnRomInfo SpecWaldolifeRomDesc[] = {
	{ "Waldo's Life EN 128K (2022)(Daniel Isoba).tap", 45064, 0xa18f6513, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecWaldolife, SpecWaldolife, Spec128)
STD_ROM_FN(SpecWaldolife)

struct BurnDriver BurnSpecWaldolife = {
	"spec_waldolife", NULL, "spec_spec128", NULL, "2022",
	"Waldo's Life (English) (128K) (HB)\0", NULL, "Daniel Isoba", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecWaldolifeRomInfo, SpecWaldolifeRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Waldo's Life (Spanish) (128K) (HB)

static struct BurnRomInfo SpecWaldolifesRomDesc[] = {
	{ "Waldo's Life ES 128K (2022)(Daniel Isoba).tap", 45060, 0x62890e34, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecWaldolifes, SpecWaldolifes, Spec128)
STD_ROM_FN(SpecWaldolifes)

struct BurnDriver BurnSpecWaldolifes = {
	"spec_waldolifes", "spec_waldolife", "spec_spec128", NULL, "2022",
	"Waldo's Life (Spanish) (128K) (HB)\0", NULL, "Daniel Isoba", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecWaldolifesRomInfo, SpecWaldolifesRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Wanderers Chained In The Dark (128K) (HB)

static struct BurnRomInfo SpecWanderersRomDesc[] = {
	{ "Wanderers Chained In The Dark 128K (2014)(SAM Style).tap", 69383, 0x8c45a071, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecWanderers, SpecWanderers, Spec128)
STD_ROM_FN(SpecWanderers)

struct BurnDriver BurnSpecWanderers = {
	"spec_wanderers", NULL, "spec_spec128", NULL, "2014",
	"Wanderers Chained In The Dark (128K) (HB)\0", NULL, "SAM Style", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_RPG, 0,
	SpectrumGetZipName, SpecWanderersRomInfo, SpecWanderersRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecQAOPSpaceDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// WHB (128K) (HB)

static struct BurnRomInfo SpecWhbRomDesc[] = {
	{ "WHB 128K (2009)(Bob Smith).tap", 47750, 0xd55e0561, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecWhb, SpecWhb, Spec128)
STD_ROM_FN(SpecWhb)

struct BurnDriver BurnSpecWhb = {
	"spec_whb", NULL, "spec_spec128", NULL, "2009",
	"WHB (128K) (HB)\0", NULL, "Bob Smith", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PUZZLE, 0,
	SpectrumGetZipName, SpecWhbRomInfo, SpecWhbRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// White Jaguar (128K) (HB)

static struct BurnRomInfo SpecWhitejaguarRomDesc[] = {
	{ "White Jaguar 128K (2020)(Romancha).tap", 24324, 0xbb1edf8f, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecWhitejaguar, SpecWhitejaguar, Spec128)
STD_ROM_FN(SpecWhitejaguar)

struct BurnDriver BurnSpecWhitejaguar = {
	"spec_whitejaguar", "spec_whitejaguarzx", "spec_spec128", NULL, "2020",
	"White Jaguar (128K) (HB)\0", NULL, "Romancha", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecWhitejaguarRomInfo, SpecWhitejaguarRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// White Jaguar (128K) (HB, v1.3)

static struct BurnRomInfo SpecWhitejaguarzxRomDesc[] = {
	{ "White Jaguar v1.3 128K (2023)(ZX-Online).tap", 72533, 0xb82f465f, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecWhitejaguarzx, SpecWhitejaguarzx, Spec128)
STD_ROM_FN(SpecWhitejaguarzx)

struct BurnDriver BurnSpecWhitejaguarzx = {
	"spec_whitejaguarzx", NULL, "spec_spec128", NULL, "2023",
	"White Jaguar (128K) (HB, v1.3)\0", NULL, "ZX-Online - Romancha", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecWhitejaguarzxRomInfo, SpecWhitejaguarzxRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Wiking - Improved Version (Polish) (128K) (Hack)

static struct BurnRomInfo SpecWiking128RomDesc[] = {
	{ "Wiking PL 128K (1987-2015)(Slider, Tiboh).tap", 80986, 0xaeae7e18, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecWiking128, SpecWiking128, Spec128)
STD_ROM_FN(SpecWiking128)

struct BurnDriver BurnSpecWiking128 = {
	"spec_wiking128", NULL, "spec_spec128", NULL, "1987-2015",
	"Wiking - Improved Version (Polish) (128K) (Hack)\0", "SounDrive/Covox SFX unavailable with FBNeo", "Slider, Tiboh", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HACK, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecWiking128RomInfo, SpecWiking128RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// WiLLY iN ThE MiRRoRVeRSE (48K) (HB)

static struct BurnRomInfo SpecWitmRomDesc[] = {
	{ "WiLLY iN ThE MiRRoRVeRSE 48K (2023)(Carl Paterson).tap", 36897, 0xff8bb357, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecWitm, SpecWitm, Spectrum)
STD_ROM_FN(SpecWitm)

struct BurnDriver BurnSpecWitm = {
	"spec_witm", NULL, "spec_spectrum", NULL, "2023",
	"WiLLY iN ThE MiRRoRVeRSE (48K) (HB)\0", NULL, "Carl Paterson", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecWitmRomInfo, SpecWitmRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Willy The Wasp (48K-128K) (HB)

static struct BurnRomInfo SpecWwaspRomDesc[] = {
	{ "Willy The Wasp (2014)(The Death Squad).tap", 41193, 0x030e8442, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecWwasp, SpecWwasp, Spec128)
STD_ROM_FN(SpecWwasp)

struct BurnDriver BurnSpecWwasp = {
	"spec_wwasp", NULL, "spec_spec128", NULL, "2014",
	"Willy The Wasp (48K-128K) (HB)\0", NULL, "The Death Squad", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_ACTION, 0,
	SpectrumGetZipName, SpecWwaspRomInfo, SpecWwaspRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecIntf2DIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Willy The Wasp 2 (128K) (HB)

static struct BurnRomInfo SpecWwasp2RomDesc[] = {
	{ "Willy The Wasp 2 (2014)(The Death Squad).tap", 64604, 0x3e8b70fb, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecWwasp2, SpecWwasp2, Spec128)
STD_ROM_FN(SpecWwasp2)

struct BurnDriver BurnSpecWwasp2 = {
	"spec_wwasp2", NULL, "spec_spec128", NULL, "2014",
	"Willy The Wasp 2 (128K) (HB)\0", NULL, "The Death Squad", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_ACTION, 0,
	SpectrumGetZipName, SpecWwasp2RomInfo, SpecWwasp2RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecIntf2DIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Winter Wonder Worm (48K) (HB)

static struct BurnRomInfo SpecWwwormRomDesc[] = {
	{ "Winter Wonder Worm 48K (2022)(Bob Fossil).tap", 5353, 0x39d13cd1, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecWwworm, SpecWwworm, Spectrum)
STD_ROM_FN(SpecWwworm)

struct BurnDriver BurnSpecWwworm = {
	"spec_wwworm", NULL, "spec_spectrum", NULL, "2022",
	"Winter Wonder Worm (48K) (HB)\0", NULL, "Bob Fossil", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_ACTION, 0,
	SpectrumGetZipName, SpecWwwormRomInfo, SpecWwwormRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Witch, The (128K) (HB, v1.0.1)

static struct BurnRomInfo SpecthewitchRomDesc[] = {
	{ "Witch, The v1.0.1 128K (2020)(Serranito Online).tap",  48970, 0x4dacc7d8, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specthewitch, Specthewitch, Spec128)
STD_ROM_FN(Specthewitch)

struct BurnDriver BurnSpecthewitch = {
	"spec_thewitch", NULL, "spec_spec128", NULL, "2020",
	"Witch, The (128K) (HB, v1.0.1)\0", "The remake of classic game Cauldron", "Serranito Online", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_HORSHOOT | GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecthewitchRomInfo, SpecthewitchRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3

};

// Witch, The - Ending (English) (128K) (HB)

static struct BurnRomInfo SpecthewitchenRomDesc[] = {
	{ "Witch, The - Ending EN 128K (2020)(Serranito Online).tap", 36161, 0x1d18f64a, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specthewitchen, Specthewitchen, Spec128)
STD_ROM_FN(Specthewitchen)

struct BurnDriver BurnSpecthewitchen = {
	"spec_thewitchen", "spec_thewitch", "spec_spec128", NULL, "2020",
	"Witch, The - Ending (English) (128K) (HB)\0", "The remake of classic game Cauldron", "Serranito Online", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_HORSHOOT | GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecthewitchenRomInfo, SpecthewitchenRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecQAOPSpaceDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Witch, The - Ending (Spanish) (128K) (HB)

static struct BurnRomInfo SpecthewitchesRomDesc[] = {
	{ "Witch, The - Ending ES 128K (2020)(Serranito Online).tap", 36165, 0x3c916134, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specthewitches, Specthewitches, Spec128)
STD_ROM_FN(Specthewitches)

struct BurnDriver BurnSpecthewitches = {
	"spec_thewitches", "spec_thewitch", "spec_spec128", NULL, "2020",
	"Witch, The - Ending (Spanish) (128K) (HB)\0", "The remake of classic game Cauldron", "Serranito Online", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_HORSHOOT | GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecthewitchesRomInfo, SpecthewitchesRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecQAOPSpaceDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// World War Simulator, The - Part 1 (48K) (HB)

static struct BurnRomInfo SpecWwsp1RomDesc[] = {
	{ "World War Simulator, The - Part 1 EN 48K (2012)(Retrobytes Productions).tap", 36335, 0x893ed613, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecWwsp1, SpecWwsp1, Spectrum)
STD_ROM_FN(SpecWwsp1)

struct BurnDriver BurnSpecWwsp1 = {
	"spec_wwsp1", NULL, "spec_spectrum", NULL, "2012",
	"World War Simulator, The - Part 1 (48K) (HB)\0", NULL, "Retrobytes Productions", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_ADV, 0,
	SpectrumGetZipName, SpecWwsp1RomInfo, SpecWwsp1RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecQAOPSpaceDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// World War Simulator, The - Part 2 (Spanish) (48K) (HB)

static struct BurnRomInfo SpecWwsp2RomDesc[] = {
	{ "World War Simulator, The - Part 2 ES 48K (2018)(Retrobytes Productions).tap", 51254, 0xa6e457e1, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecWwsp2, SpecWwsp2, Spectrum)
STD_ROM_FN(SpecWwsp2)

struct BurnDriver BurnSpecWwsp2 = {
	"spec_wwsp2", NULL, "spec_spectrum", NULL, "2018",
	"World War Simulator, The - Part 2 (Spanish) (48K) (HB)\0", NULL, "Retrobytes Productions", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_MAZE | GBF_RUNGUN, 0,
	SpectrumGetZipName, SpecWwsp2RomInfo, SpecWwsp2RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecQAOPSpaceDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Wudang (48K) (HB)

static struct BurnRomInfo SpecWudangRomDesc[] = {
	{ "Wudang 48K (2020)(World XXI Soft).tap", 22457, 0x68b849e6, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecWudang, SpecWudang, Spectrum)
STD_ROM_FN(SpecWudang)

struct BurnDriver BurnSpecWudang = {
	"spec_wudang", NULL, "spec_spectrum", NULL, "2020",
	"Wudang (48K) (HB)\0", NULL, "World XXI Soft", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecWudangRomInfo, SpecWudangRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Wycheweald (128K) (HB)

static struct BurnRomInfo SpecWychewealdRomDesc[] = {
	{ "Wycheweald 128K (2023)(Red Zebra).tap", 60543, 0x2ab6e85c, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecWycheweald, SpecWycheweald, Spec128)
STD_ROM_FN(SpecWycheweald)

struct BurnDriver BurnSpecWycheweald = {
	"spec_wycheweald", NULL, "spec_spec128", NULL, "2023",
	"Wycheweald (128K) (HB)\0", NULL, "Red Zebra", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_ADV | GBF_RPG, 0,
	SpectrumGetZipName, SpecWychewealdRomInfo, SpecWychewealdRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Xantel Jones en la Tabla Verdosa (Spanish) (48K) (HB)

static struct BurnRomInfo SpecXanteljonesRomDesc[] = {
	{ "Xantel Jones en la Tabla Verdosa ES 48K (2024)(graelx).tap", 41461, 0x18157570, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecXanteljones, SpecXanteljones, Spectrum)
STD_ROM_FN(SpecXanteljones)

struct BurnDriver BurnSpecXanteljones = {
	"spec_xanteljones", NULL, "spec_spectrum", NULL, "2024",
	"Xantel Jones en la Tabla Verdosa (Spanish) (48K) (HB)\0", NULL, "graelx", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecXanteljonesRomInfo, SpecXanteljonesRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Xenoblast (128K) (HB)

static struct BurnRomInfo SpecXenoblast128RomDesc[] = {
	{ "Xenoblast 128K (2021)(Tony Kingsmill).tap", 53319, 0xf7412190, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecXenoblast128, SpecXenoblast128, Spec128)
STD_ROM_FN(SpecXenoblast128)

struct BurnDriver BurnSpecXenoblast128 = {
	"spec_xenoblast128", NULL, "spec_spec128", NULL, "2021",
	"Xenoblast (128K) (HB)\0", NULL, "Tony Kingsmill", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_VERSHOOT, 0,
	SpectrumGetZipName, SpecXenoblast128RomInfo, SpecXenoblast128RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecIntf2DIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Xenoblast (48K) (HB)

static struct BurnRomInfo SpecXenoblast48RomDesc[] = {
	{ "Xenoblast 48K (2021)(Tony Kingsmill).tap", 39743, 0xc2be176f, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecXenoblast48, SpecXenoblast48, Spectrum)
STD_ROM_FN(SpecXenoblast48)

struct BurnDriver BurnSpecXenoblast48 = {
	"spec_xenoblast48", "spec_xenoblast128", "spec_spectrum", NULL, "2021",
	"Xenoblast (48K) (HB)\0", NULL, "Tony Kingsmill", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_VERSHOOT, 0,
	SpectrumGetZipName, SpecXenoblast48RomInfo, SpecXenoblast48RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecIntf2DIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// YAKS - Yet Another Krappy Shooter (16K) (HB)

static struct BurnRomInfo SpecYaksRomDesc[] = {
	{ "YAKS - Yet Another Krappy Shooter 16K (2022)(Krappygamez).tap", 13963, 0x2ebe7e38, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecYaks, SpecYaks, Spectrum)
STD_ROM_FN(SpecYaks)

struct BurnDriver BurnSpecYaks = {
	"spec_yaks", NULL, "spec_spectrum", NULL, "2022",
	"YAKS - Yet Another Krappy Shooter (16K) (HB)\0", NULL, "Krappygamez", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_HORSHOOT, 0,
	SpectrumGetZipName, SpecYaksRomInfo, SpecYaksRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Yars Revenge ZX (48K) (HB)

static struct BurnRomInfo SpecYarsrevengeRomDesc[] = {
	{ "Yars Revenge ZX 48K (2015)(Monument Microgames).tap", 31227, 0x9d59babf, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecYarsrevenge, SpecYarsrevenge, Spectrum)
STD_ROM_FN(SpecYarsrevenge)

struct BurnDriver BurnSpecYarsrevenge = {
	"spec_yarsrevenge", NULL, "spec_spectrum", NULL, "2015",
	"Yars Revenge ZX (48K) (HB)\0", NULL, "Monument Microgames", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_SHOOT, 0,
	SpectrumGetZipName, SpecYarsrevengeRomInfo, SpecYarsrevengeRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Yazzie (128K) (HB)

static struct BurnRomInfo SpecyazzieRomDesc[] = {
	{ "Yazzie 128K (2019)(RetroSouls).tap",  70103, 0x68d21e49, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Specyazzie, Specyazzie, Spec128)
STD_ROM_FN(Specyazzie)

struct BurnDriver BurnSpecyazzie = {
	"spec_yazzie", NULL, "spec_spec128", NULL, "2019",
	"Yazzie (128K) (HB)\0", NULL, "RetroSouls", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PUZZLE, 0,
	SpectrumGetZipName, SpecyazzieRomInfo, SpecyazzieRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Yewdow (48K) (HB)

static struct BurnRomInfo SpecYewdowRomDesc[] = {
	{ "Yewdow 48K (2023)(Inufuto).tap", 9818, 0xfaf70151, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecYewdow, SpecYewdow, Spectrum)
STD_ROM_FN(SpecYewdow)

struct BurnDriver BurnSpecYewdow = {
	"spec_yewdow", NULL, "spec_spectrum", NULL, "2023",
	"Yewdow (48K) (HB)\0", "AY Sound supported", "Inufuto", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_MAZE | GBF_ACTION, 0,
	SpectrumGetZipName, SpecYewdowRomInfo, SpecYewdowRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Yokai Monk (128K) (HB, v1.8)

static struct BurnRomInfo SpecYokaimonkRomDesc[] = {
	{ "Yokai Monk v1.8 128K (2022)(Poe Games).tap", 43185, 0x8b1fc9fa, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecYokaimonk, SpecYokaimonk, Spec128)
STD_ROM_FN(SpecYokaimonk)

struct BurnDriver BurnSpecYokaimonk = {
	"spec_yokaimonk", NULL, "spec_spec128", NULL, "2022",
	"Yokai Monk (128K) (HB, v1.8)\0", NULL, "Poe Games", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecYokaimonkRomInfo, SpecYokaimonkRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Yopparai Sarariiman (English) (48K) (HB)

static struct BurnRomInfo SpecYopparaienRomDesc[] = {
	{ "Yopparai Sarariiman EN 48K (2021)(Furillo Productions).tap", 43450, 0x735f63c8, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecYopparaien, SpecYopparaien, Spectrum)
STD_ROM_FN(SpecYopparaien)

struct BurnDriver BurnSpecYopparaien = {
	"spec_yopparaien", NULL, "spec_spectrum", NULL, "2021",
	"Yopparai Sarariiman (English) (48K) (HB)\0", NULL, "Furillo Productions", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecYopparaienRomInfo, SpecYopparaienRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Yopparai Sarariiman (Spanish) (48K) (HB)

static struct BurnRomInfo SpecYopparaiesRomDesc[] = {
	{ "Yopparai Sarariiman ES 48K (2021)(Furillo Productions).tap", 43485, 0xf7c3f32b, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecYopparaies, SpecYopparaies, Spectrum)
STD_ROM_FN(SpecYopparaies)

struct BurnDriver BurnSpecYopparaies = {
	"spec_yopparaies", "spec_yopparaien", "spec_spectrum", NULL, "2021",
	"Yopparai Sarariiman (Spanish) (48K) (HB)\0", NULL, "Furillo Productions", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecYopparaiesRomInfo, SpecYopparaiesRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Yopparai Sarariiman (Japanese) (48K) (HB)

static struct BurnRomInfo SpecYopparaijpRomDesc[] = {
	{ "Yopparai Sarariiman JP 48K (2021)(Furillo Productions).tap", 43980, 0x6a551fc8, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecYopparaijp, SpecYopparaijp, Spectrum)
STD_ROM_FN(SpecYopparaijp)

struct BurnDriver BurnSpecYopparaijp = {
	"spec_yopparaijp", "spec_yopparaien", "spec_spectrum", NULL, "2021",
	"Yopparai Sarariiman (Japanese) (48K) (HB)\0", NULL, "Furillo Productions", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecYopparaijpRomInfo, SpecYopparaijpRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// YoYo's Revenge (48K) (HB)

static struct BurnRomInfo SpecYoyorevengeRomDesc[] = {
	{ "YoYo's Revenge 48K (2023)(J Bizzel).tap", 35245, 0x62a2ca40, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecYoyorevenge, SpecYoyorevenge, Spectrum)
STD_ROM_FN(SpecYoyorevenge)

struct BurnDriver BurnSpecYoyorevenge = {
	"spec_yoyorevenge", NULL, "spec_spectrum", NULL, "2023",
	"YoYo's Revenge (48K) (HB)\0", NULL, "J Bizzel", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecYoyorevengeRomInfo, SpecYoyorevengeRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecQAOPMDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Zedex Tanks (128K) (HB, v1.2)

static struct BurnRomInfo SpecZedextanksRomDesc[] = {
	{ "Zedex Tanks v1.2 128K (2023)(Epsilon Gamers).tap", 72623, 0x33f2e7b8, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecZedextanks, SpecZedextanks, Spec128)
STD_ROM_FN(SpecZedextanks)

struct BurnDriver BurnSpecZedextanks = {
	"spec_zedextanks", NULL, "spec_spec128", NULL, "2023",
	"Zedex Tanks (128K) (HB, v1.2)\0", NULL, "Epsilon Gamers", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 2, HARDWARE_SPECTRUM, GBF_SHOOT | GBF_MAZE, 0,
	SpectrumGetZipName, SpecZedextanksRomInfo, SpecZedextanksRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// ZEN (48K) (HB)

static struct BurnRomInfo SpecZenRomDesc[] = {
	{ "ZEN 48K (2014)(Einar Saukas).tap", 40412, 0x3a9f650e, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecZen, SpecZen, Spectrum)
STD_ROM_FN(SpecZen)

struct BurnDriver BurnSpecZen = {
	"spec_zen", NULL, "spec_spectrum", NULL, "2014",
	"ZEN (48K) (HB)\0", NULL, "Einar Saukas", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PUZZLE, 0,
	SpectrumGetZipName, SpecZenRomInfo, SpecZenRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// ZEN II (48K) (HB)

static struct BurnRomInfo SpecZeniiRomDesc[] = {
	{ "ZEN II 48K (2015)(Einar Saukas).tap", 35732, 0x3e1fb513, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecZenii, SpecZenii, Spectrum)
STD_ROM_FN(SpecZenii)

struct BurnDriver BurnSpecZenii = {
	"spec_zenii", NULL, "spec_spectrum", NULL, "2015",
	"ZEN II (48K) (HB)\0", NULL, "Einar Saukas", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PUZZLE, 0,
	SpectrumGetZipName, SpecZeniiRomInfo, SpecZeniiRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Ziona Quest - Training (128K) (HB)

static struct BurnRomInfo SpecZquesttRomDesc[] = {
	{ "Ziona Quest - Training 128K (2023)(Iamai).tap", 55749, 0xa614eac8, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecZquestt, SpecZquestt, Spec128)
STD_ROM_FN(SpecZquestt)

struct BurnDriver BurnSpecZquestt = {
	"spec_zquestt", NULL, "spec_spec128", NULL, "2023",
	"Ziona Quest - Training (128K) (HB)\0", NULL, "Iamai", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecZquesttRomInfo, SpecZquesttRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Ziona Quest - Game (128K) (HB)

static struct BurnRomInfo SpecZquestgRomDesc[] = {
	{ "Ziona Quest - Game 128K (2023)(Iamai).tap", 57048, 0x8ff580cd, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecZquestg, SpecZquestg, Spec128)
STD_ROM_FN(SpecZquestg)

struct BurnDriver BurnSpecZquestg = {
	"spec_zquestg", "spec_zquestt", "spec_spec128", NULL, "2023",
	"Ziona Quest - Game (128K) (HB)\0", NULL, "Iamai", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PLATFORM, 0,
	SpectrumGetZipName, SpecZquestgRomInfo, SpecZquestgRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// ZLALOX (48K) (HB)

static struct BurnRomInfo SpecZlaloxRomDesc[] = {
	{ "ZLALOX 48K (2024)(Snauts).tap", 10778, 0x0449b616, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecZlalox, SpecZlalox, Spectrum)
STD_ROM_FN(SpecZlalox)

struct BurnDriver BurnSpecZlalox = {
	"spec_zlalox", NULL, "spec_spectrum", NULL, "2024",
	"ZLALOX (48K) (HB)\0", NULL, "Snauts", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_ACTION, 0,
	SpectrumGetZipName, SpecZlaloxRomInfo, SpecZlaloxRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Zoinho (Spanish) (48K) (HB)

static struct BurnRomInfo SpecZoinhoRomDesc[] = {
	{ "Zoinho 48K (2021)(Bitnamic).tap", 29137, 0xa64d5e5a, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecZoinho, SpecZoinho, Spectrum)
STD_ROM_FN(SpecZoinho)

struct BurnDriver BurnSpecZoinho = {
	"spec_zoinho", NULL, "spec_spectrum", NULL, "2021",
	"Zoinho (Spanish) (48K) (HB)\0", NULL, "Bitnamic", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_ACTION | GBF_ADV, 0,
	SpectrumGetZipName, SpecZoinhoRomInfo, SpecZoinhoRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecIntf2DIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Zombie Calavera Prologue (48K) (HB)

static struct BurnRomInfo SpeczcalaveraRomDesc[] = {
	{ "Zombie Calavera Prologue 48K (2010)(Mojon Twins).tap", 37966, 0x9eaf2b79, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(Speczcalavera, Speczcalavera, Spectrum)
STD_ROM_FN(Speczcalavera)

struct BurnDriver BurnSpeczcalavera = {
	"spec_zcalavera", NULL, "spec_spectrum", NULL, "2010",
	"Zombie Calavera Prologue (48K) (HB)\0", NULL, "The Mojon Twins", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_RUNGUN | GBF_PLATFORM, 0,
	SpectrumGetZipName, SpeczcalaveraRomInfo, SpeczcalaveraRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Zombi Mall (English) (48K) (HB)

static struct BurnRomInfo SpecZombmalleRomDesc[] = {
	{ "Zombi Mall EN 48K (2022)(The Mojon Twins).tap", 44766, 0xc0591dbf, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecZombmalle, SpecZombmalle, Spectrum)
STD_ROM_FN(SpecZombmalle)

struct BurnDriver BurnSpecZombmalle = {
	"spec_zombmalle", NULL, "spec_spectrum", NULL, "2022",
	"Zombi Mall (English) (48K) (HB)\0", NULL, "The Mojon Twins", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_ACTION | GBF_ADV, 0,
	SpectrumGetZipName, SpecZombmalleRomInfo, SpecZombmalleRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Zombi Mall (Spanish) (48K) (HB)

static struct BurnRomInfo SpecZombmallsRomDesc[] = {
	{ "Zombi Mall ES 48K (2022)(The Mojon Twins).tap", 44764, 0x5deb94ed, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecZombmalls, SpecZombmalls, Spectrum)
STD_ROM_FN(SpecZombmalls)

struct BurnDriver BurnSpecZombmalls = {
	"spec_zombmalls", "spec_zombmalle", "spec_spectrum", NULL, "2022",
	"Zombi Mall (Spanish) (48K) (HB)\0", NULL, "The Mojon Twins", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_ACTION | GBF_ADV, 0,
	SpectrumGetZipName, SpecZombmallsRomInfo, SpecZombmallsRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Zombo's Christmas Capers (48K) (HB)

static struct BurnRomInfo SpecZomboccRomDesc[] = {
	{ "Zombo's Christmas Capers 48K (2016)(Monsterbytes).z80", 39448, 0xc3992bbe, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecZombocc, SpecZombocc, Spectrum)
STD_ROM_FN(SpecZombocc)

struct BurnDriver BurnSpecZombocc = {
	"spec_zombocc", NULL, "spec_spectrum", NULL, "2016",
	"Zombo's Christmas Capers (48K) (HB)\0", NULL, "Monsterbytes", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_MAZE | GBF_ADV, 0,
	SpectrumGetZipName, SpecZomboccRomInfo, SpecZomboccRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Zooming Secretary (128K) (HB)

static struct BurnRomInfo SpecZoomsecrRomDesc[] = {
	{ "Zooming Secretary 128K (2021)(PC NONO Games).tap", 47173, 0xd612501b, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecZoomsecr, SpecZoomsecr, Spec128)
STD_ROM_FN(SpecZoomsecr)

struct BurnDriver BurnSpecZoomsecr = {
	"spec_zoomsecr", NULL, "spec_spec128", NULL, "2021",
	"Zooming Secretary (128K) (HB)\0", NULL, "PC NONO Games", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_ACTION, 0,
	SpectrumGetZipName, SpecZoomsecrRomInfo, SpecZoomsecrRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KPlus2Init, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Arcade ZX Collection - Botanic (48K) (HB)

static struct BurnRomInfo SpecZxbotanicRomDesc[] = {
	{ "Arcade ZX Collection - Botanic 48K (2022)(Bubu).tap", 26024, 0xa4ac4eef, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecZxbotanic, SpecZxbotanic, Spectrum)
STD_ROM_FN(SpecZxbotanic)

struct BurnDriver BurnSpecZxbotanic = {
	"spec_zxbotanic", NULL, "spec_spectrum", NULL, "2022",
	"Arcade ZX Collection - Botanic (48K) (HB)\0", "Press '5' and then '1' to start game", "Bubu", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_ACTION | GBF_MAZE, 0,
	SpectrumGetZipName, SpecZxbotanicRomInfo, SpecZxbotanicRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecQAOPSpaceDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// ZX Enigma Machine (48K) (HB)

static struct BurnRomInfo SpecZxenigmaRomDesc[] = {
	{ "ZX Enigma Machine (2021)(Menyiques).tap", 28003, 0xc5bbd1e3, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecZxenigma, SpecZxenigma, Spectrum)
STD_ROM_FN(SpecZxenigma)

struct BurnDriver BurnSpecZxenigma = {
	"spec_zxenigma", NULL, "spec_spectrum", NULL, "2021",
	"ZX Enigma Machine (48K) (HB)\0", "Press ENTER for instructions", "Menyiques", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PUZZLE, 0,
	SpectrumGetZipName, SpecZxenigmaRomInfo, SpecZxenigmaRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// ZX Football Manager 2005 (128K) (HB)

static struct BurnRomInfo SpecZxfm2005RomDesc[] = {
	{ "ZX Football Manager 2005 (2004)(Cronosoft).tap", 80114, 0xec86d147, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecZxfm2005, SpecZxfm2005, Spec128)
STD_ROM_FN(SpecZxfm2005)

struct BurnDriver BurnSpecZxfm2005 = {
	"spec_zxfm2005", NULL, "spec_spec128", NULL, "2004",
	"ZX Football Manager 2005 (128K) (HB)\0", NULL, "Cronosoft", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_STRATEGY, 0,
	SpectrumGetZipName, SpecZxfm2005RomInfo, SpecZxfm2005RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// ZX Frogger (48K)

static struct BurnRomInfo SpecZxfroggerRomDesc[] = {
	{ "ZX Frogger 48K (2021)(ref).tap", 22766, 0x12907894, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecZxfrogger, SpecZxfrogger, Spectrum)
STD_ROM_FN(SpecZxfrogger)

struct BurnDriver BurnSpecZxfrogger = {
	"spec_zxfrogger", NULL, "spec_spectrum", NULL, "2021",
	"ZX Frogger (48K) (HB)\0", NULL, "ref", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_ACTION, 0,
	SpectrumGetZipName, SpecZxfroggerRomInfo, SpecZxfroggerRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecQAOPSpaceDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// ZX Larry (128K) (HB)

static struct BurnRomInfo SpecZxlarryRomDesc[] = {
	{ "ZX Larry 128K (2019)(Rafal Miazga).tap", 48364, 0xd4a981d5, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecZxlarry, SpecZxlarry, Spec128)
STD_ROM_FN(SpecZxlarry)

struct BurnDriver BurnSpecZxlarry = {
	"spec_zxlarry", NULL, "spec_spec128", NULL, "2019",
	"ZX Larry (128K) (HB)\0", NULL, "Rafal Miazga", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_ADV, 0,
	SpectrumGetZipName, SpecZxlarryRomInfo, SpecZxlarryRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	Spec128KInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// ZX Oil Panic Game & Watch (48K) (HB)

static struct BurnRomInfo SpecZxoilpgwRomDesc[] = {
	{ "ZX Oil Panic G&W 48K (2021)(Menyiques).tap", 24803, 0x5cc0012c, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecZxoilpgw, SpecZxoilpgw, Spectrum)
STD_ROM_FN(SpecZxoilpgw)

struct BurnDriver BurnSpecZxoilpgw = {
	"spec_zxoilpgw", "spec_zxoilpgwv2", "spec_spectrum", NULL, "2021",
	"ZX Oil Panic Game & Watch (48K) (HB)\0", NULL, "Menyiques", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_CLONE | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_ACTION, 0,
	SpectrumGetZipName, SpecZxoilpgwRomInfo, SpecZxoilpgwRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecQAOPSpaceDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// ZX Oil Panic Game & Watch (48K) (HB, v2)

static struct BurnRomInfo SpecZxoilpgwv2RomDesc[] = {
	{ "ZX Oil Panic Game & Watch v2 48K (2024)(Menyiques).tap", 31702, 0x7aceda1b, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecZxoilpgwv2, SpecZxoilpgwv2, Spectrum)
STD_ROM_FN(SpecZxoilpgwv2)

struct BurnDriver BurnSpecZxoilpgwv2 = {
	"spec_zxoilpgwv2", NULL, "spec_spectrum", NULL, "2024",
	"ZX Oil Panic Game & Watch (48K) (HB, v2)\0", NULL, "Menyiques", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_ACTION, 0,
	SpectrumGetZipName, SpecZxoilpgwv2RomInfo, SpecZxoilpgwv2RomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// ZX Parachute (16K) (HB)

static struct BurnRomInfo SpecZxparachuteRomDesc[] = {
	{ "ZX Parachute 16K (2020)(Carles Oriol).tap", 12897, 0xc82e385e, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecZxparachute, SpecZxparachute, Spectrum)
STD_ROM_FN(SpecZxparachute)

struct BurnDriver BurnSpecZxparachute = {
	"spec_zxparachute", NULL, "spec_spectrum", NULL, "2020",
	"ZX Parachute (16K) (HB)\0", "1 or 2: left - 7 or 8: right", "Carles Oriol", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_ACTION, 0,
	SpectrumGetZipName, SpecZxparachuteRomInfo, SpecZxparachuteRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// ZX Robots (48K) (HB)

static struct BurnRomInfo SpecZxrobotsRomDesc[] = {
	{ "ZX Robots 48K (2023)(Skult).tap", 37038, 0x94e4578d, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecZxrobots, SpecZxrobots, Spectrum)
STD_ROM_FN(SpecZxrobots)

struct BurnDriver BurnSpecZxrobots = {
	"spec_zxrobots", NULL, "spec_spectrum", NULL, "2023",
	"ZX Robots (48K) (HB)\0", NULL, "Skult", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_ACTION, 0,
	SpectrumGetZipName, SpecZxrobotsRomInfo, SpecZxrobotsRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// Arcade ZX Collection - Tetris (48K) (HB)

static struct BurnRomInfo SpecZxtetrisRomDesc[] = {
	{ "Arcade ZX Collection - Tetris 48K (2021)(Bubu).tap", 37111, 0xc1896fba, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecZxtetris, SpecZxtetris, Spectrum)
STD_ROM_FN(SpecZxtetris)

struct BurnDriver BurnSpecZxtetris = {
	"spec_zxtetris", NULL, "spec_spectrum", NULL, "2021",
	"Arcade ZX Collection - Tetris (48K) (HB)\0", "Press '5' to start game", "Bubu", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PUZZLE, 0,
	SpectrumGetZipName, SpecZxtetrisRomInfo, SpecZxtetrisRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecQAOPSpaceDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

// ZX Wordle (48K) (HB)

static struct BurnRomInfo SpecZxwordleRomDesc[] = {
	{ "ZX Wordle 48K (2022)(Adam Ainsworth).tap", 30945, 0xcab302a6, BRF_ESS | BRF_PRG },
};

STDROMPICKEXT(SpecZxwordle, SpecZxwordle, Spectrum)
STD_ROM_FN(SpecZxwordle)

struct BurnDriver BurnSpecZxwordle = {
	"spec_zxwordle", NULL, "spec_spectrum", NULL, "2022",
	"ZX Wordle (48K) (HB)\0", NULL, "Adam Ainsworth", "ZX Spectrum",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_HOMEBREW, 1, HARDWARE_SPECTRUM, GBF_PUZZLE, 0,
	SpectrumGetZipName, SpecZxwordleRomInfo, SpecZxwordleRomName, NULL, NULL, NULL, NULL, SpecInputInfo, SpecDIPInfo,
	SpecInit, SpecExit, SpecFrame, SpecDraw, SpecScan,
	&SpecRecalc, 0x10, 288, 224, 4, 3
};

